package org.processmining.common.frame;
 
import java.awt.Color;
import java.awt.Component;
import java.util.ArrayList;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.DefaultListModel;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollBar;
import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.ScrollPaneConstants;
import javax.swing.SwingConstants;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeCellRenderer;

import com.fluxicon.slickerbox.ui.SlickerScrollBarUI;

 
/**
 * @version 1.0
 * @author liangdong
 *
 */
public class CreateTreePanel extends JPanel {
    /**
     * 
     */
    private static final long serialVersionUID = 1L;   
    private JLabel listLabel = new JLabel("list values (number: 0)");
    private DefaultListModel<String> listModel = new DefaultListModel<String>();
    private JList<String> list  = new JList<String>(listModel);//control the content of the list panel
    private JScrollPane listScroller;//control the size of the list panel    
    private List<String> valueList = new ArrayList<String>();//abstract the content by the form of a list of string
    private String name = "list values";
    
	public static final Color COLOR_ENCLOSURE_BG = new Color(40, 40, 40);
	public static final Color COLOR_NON_FOCUS = new Color(70, 70, 70);
	public static final Color COLOR_LIST_BG = new Color(60, 60, 60);
	public static final Color COLOR_LIST_BG_LOWER = new Color(45, 45, 45);
	public static final Color COLOR_LIST_FG = new Color(180, 180, 180);
	public static final Color COLOR_LIST_SELECTION_BG = new Color(80, 0, 0);
	public static final Color COLOR_LIST_SELECTION_BG_LOWER = new Color(30, 10, 10);
	public static final Color COLOR_LIST_SELECTION_FG = new Color(240, 240, 240);
	
	private JTree mdtree;
	
	public JTree getJTree(){
		return mdtree;
	}
	
	/**
	 * 
	 * @param root
	 * @param treeName if null, hide the tree name label
	 */
    public CreateTreePanel(DefaultMutableTreeNode root, String treeName) {    		
		// Create metadata tree.
		mdtree = new JTree(root);		
		mdtree.setBackground(COLOR_ENCLOSURE_BG);
		mdtree.setForeground(Color.WHITE);
		
		DefaultTreeCellRenderer renderer = (DefaultTreeCellRenderer) mdtree
				.getCellRenderer();
		renderer.setBackground(COLOR_ENCLOSURE_BG);
		renderer
				.setBackgroundNonSelectionColor(COLOR_ENCLOSURE_BG);
		renderer
				.setBackgroundSelectionColor(COLOR_LIST_SELECTION_BG);
		renderer.setForeground(Color.WHITE);
		renderer.setTextNonSelectionColor(Color.WHITE);
		renderer.setTextSelectionColor(COLOR_LIST_SELECTION_FG);

		JScrollPane treePane= new JScrollPane(mdtree);
		treePane.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
		treePane.setOpaque(false);
		treePane.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);
		treePane.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		
		JScrollBar vBar = treePane.getVerticalScrollBar();
		vBar.setUI(new SlickerScrollBarUI(vBar,COLOR_ENCLOSURE_BG, new Color(160, 160, 160),COLOR_NON_FOCUS, 4, 12));
		
		JScrollBar hBar = treePane.getHorizontalScrollBar();
		hBar.setUI(new SlickerScrollBarUI(hBar,COLOR_ENCLOSURE_BG, new Color(160, 160, 160),COLOR_NON_FOCUS, 4, 12));
		
		JLabel treeNameLabel = new JLabel(treeName);
		treeNameLabel.setOpaque(false);
		treeNameLabel.setForeground(COLOR_LIST_SELECTION_FG);
		treeNameLabel.setFont(treeNameLabel.getFont().deriveFont(13f));
		treeNameLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
		treeNameLabel.setHorizontalAlignment(SwingConstants.CENTER);
		treeNameLabel.setHorizontalTextPosition(SwingConstants.CENTER);	
		
		
		this.removeAll();		
		this.setBackground(COLOR_ENCLOSURE_BG);
		this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		if(treeName != null){
			this.add(treeNameLabel);
		}	
		this.add(Box.createVerticalStrut(8));
		this.add(treePane);		
		this.revalidate();
		this.repaint();
		this.setBorder(BorderFactory.createRaisedBevelBorder());
    }
    
    /**
	 * 
	 * @param root
	 * @param treeName if null, hide the tree name label
	 */
    public void updateTreePanel(DefaultMutableTreeNode root, String treeName) {    		
		// Create metadata tree.
		mdtree = new JTree(root);
		mdtree.setBackground(COLOR_LIST_BG);
		mdtree.setForeground(Color.WHITE);
		
		DefaultTreeCellRenderer renderer = (DefaultTreeCellRenderer) mdtree
				.getCellRenderer();
		renderer.setBackground(COLOR_LIST_BG);
		renderer.setBackgroundNonSelectionColor(COLOR_LIST_BG);
		renderer.setBackgroundSelectionColor(COLOR_LIST_SELECTION_BG);
		renderer.setForeground(Color.WHITE);
		renderer.setTextNonSelectionColor(Color.WHITE);
		renderer.setTextSelectionColor(COLOR_LIST_SELECTION_FG);

//		instancesList.setBackground(COLOR_LIST_BG);
//		instancesList.setForeground(COLOR_LIST_FG);
//		instancesList.setSelectionBackground(COLOR_LIST_SELECTION_BG);
//		instancesList.setSelectionForeground(COLOR_LIST_SELECTION_FG);
//		instancesList.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
		
		JScrollPane treePane= new JScrollPane(mdtree);
		treePane.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
		treePane.setOpaque(false);
		treePane.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);
		treePane.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		
		JScrollBar vBar = treePane.getVerticalScrollBar();
		vBar.setUI(new SlickerScrollBarUI(vBar,COLOR_ENCLOSURE_BG, new Color(160, 160, 160),COLOR_NON_FOCUS, 4, 12));		
		JScrollBar hBar = treePane.getHorizontalScrollBar();
		hBar.setUI(new SlickerScrollBarUI(hBar,COLOR_ENCLOSURE_BG, new Color(160, 160, 160),COLOR_NON_FOCUS, 4, 12));
		
		JLabel treeNameLabel = new JLabel(treeName);
		treeNameLabel.setOpaque(false);
		treeNameLabel.setForeground(COLOR_LIST_SELECTION_FG);
		treeNameLabel.setFont(treeNameLabel.getFont().deriveFont(13f));
		treeNameLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
		treeNameLabel.setHorizontalAlignment(SwingConstants.CENTER);
		treeNameLabel.setHorizontalTextPosition(SwingConstants.CENTER);	
		
		
		this.removeAll();		
		this.setBackground(COLOR_ENCLOSURE_BG);
		this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		if(treeName != null){
			this.add(treeNameLabel);
		}	
		this.add(Box.createVerticalStrut(8));
		this.add(treePane);		
		this.revalidate();
		this.repaint();
		this.setBorder(BorderFactory.createRaisedBevelBorder());
    }
    
    public static void main(String[] args) {
    	DefaultMutableTreeNode root = new DefaultMutableTreeNode("Event complex");
    	CreateTreePanel treePanel = new CreateTreePanel(root, null);

//		// Create metadata tree.
//		final JTree mdtree = new JTree(root);		
//		mdtree.setBackground(COLOR_ENCLOSURE_BG);
//		mdtree.setForeground(Color.WHITE);
//		
//		DefaultTreeCellRenderer renderer = (DefaultTreeCellRenderer) mdtree
//				.getCellRenderer();
//		renderer.setBackground(COLOR_ENCLOSURE_BG);
//		renderer
//				.setBackgroundNonSelectionColor(COLOR_ENCLOSURE_BG);
//		renderer
//				.setBackgroundSelectionColor(COLOR_LIST_SELECTION_BG);
//		renderer.setForeground(Color.WHITE);
//		renderer.setTextNonSelectionColor(Color.WHITE);
//		renderer.setTextSelectionColor(COLOR_LIST_SELECTION_FG);
//
//		JScrollPane treePane= new JScrollPane(mdtree);
//		treePane.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
//		treePane.setOpaque(false);
//		treePane.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);
//		treePane.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
//		
//		JScrollBar vBar = treePane.getVerticalScrollBar();
//		vBar.setUI(new SlickerScrollBarUI(vBar,COLOR_ENCLOSURE_BG, new Color(160, 160, 160),COLOR_NON_FOCUS, 4, 12));
//		
//		JScrollBar hBar = treePane.getHorizontalScrollBar();
//		hBar.setUI(new SlickerScrollBarUI(hBar,COLOR_ENCLOSURE_BG, new Color(160, 160, 160),COLOR_NON_FOCUS, 4, 12));
//		
//		JPanel deviationPane = new JPanel();
//		deviationPane.removeAll();
//		
//		deviationPane.setBackground(COLOR_ENCLOSURE_BG);
//		deviationPane.setLayout(new BoxLayout(deviationPane, BoxLayout.Y_AXIS));
//		deviationPane.add(Box.createVerticalStrut(8));
//		deviationPane.add(treePane);
//		
//		deviationPane.revalidate();
//		deviationPane.repaint();
		
    	JFrame frame = new JFrame();
//		frame.add(treePane);
		
//		frame.getContentPane().add(treePane);
		frame.getContentPane().add(treePanel);
		frame.setVisible(true);
    }
}