package org.processmining.dolibarr.operationrobot.plugin;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Image;

import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;

import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.contexts.uitopia.annotations.Visualizer;
import org.processmining.dolibarr.operationrobot.panels.BuildClickInterface;
import org.processmining.dolibarr.operationrobot.panels.RunDolibarrPanel;
import org.processmining.framework.plugin.annotations.Plugin;

import com.fluxicon.slickerbox.components.IconVerticalTabbedPane;



public class PopulateDolibarrDatabasePlugin extends JPanel{
	
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;	
	
	protected BuildClickInterface simulationLogPanel;
	protected RunDolibarrPanel runDolibarrPanel;

	private final static String DASHBOARDICON = "clicklist.png";
	private final static String INSPECTORICON = "run.png";

	// icons
	protected Image dashboardIcon = new ImageIcon(PopulateDolibarrDatabasePlugin.class.getResource(DASHBOARDICON),DASHBOARDICON).getImage();
	protected Image inspectorIcon = new ImageIcon(PopulateDolibarrDatabasePlugin.class.getResource(INSPECTORICON),INSPECTORICON).getImage();


	@Plugin(
		name = "ERP Data Generator",
		parameterLabels = {},
		returnLabels = {"ERP Data Generator"},
		returnTypes = { PopulateDolibarrDatabaseObject.class },
		userAccessible = true,
		help = "Start up editor"
	)
	@UITopiaVariant(
		affiliation = UITopiaVariant.EHV,
		author = "Guangming Li",
		email = "g.li.3@tue.nl"
	)
	public PopulateDolibarrDatabaseObject populateDolibarrDatabase(UIPluginContext context) throws Exception{		
		context.log("ERP Data Generator starts");		
		populateDolibarrDatabasePanel(context);		       
		return new PopulateDolibarrDatabaseObject(this);
	}
	
	@Plugin(
			name = "ERP Data Generator Visualization", 
			parameterLabels = {"visualizer"},
			returnTypes = JComponent.class, 
			returnLabels = "ERP Data Generator Visualization", 
			help = "ERP Data Generator"
	)	
	@Visualizer
	public JComponent visualizePopulateDolibarrDatabase(UIPluginContext context, final PopulateDolibarrDatabaseObject input){
		System.out.println("ERP Data Generator visualizer");
		context.log("welcome ERP Data Generator visualizer");
		JPanel resultPanel = input.panel;
		return resultPanel;
	}
	
	public JComponent populateDolibarrDatabasePanel(UIPluginContext context) throws Exception {
		initialize(null);
		completeGui();
		return this;
	}
	JTextField selectedPath;
	JTextField selectedConfigurationFilePath;


 	
	protected void initialize(String path) {
		setBackground(new Color(40, 40, 40));
		setBorder(BorderFactory.createEmptyBorder());
		setLayout(new BorderLayout());
	}
		
	
	protected void completeGui() throws Exception {
		runDolibarrPanel = new RunDolibarrPanel(this);
		simulationLogPanel = new BuildClickInterface(this);

		IconVerticalTabbedPane iconTabs = new IconVerticalTabbedPane(new Color(230, 230, 230, 210), new Color(20, 20,
				20, 160));
//		iconTabs.addTab("Build Clicks", dashboardIcon, new JScrollPane(simulationLogPanel));
		iconTabs.addTab("Build Clicks", dashboardIcon, new JScrollPane(simulationLogPanel));
		iconTabs.addTab("Run System", inspectorIcon, new JScrollPane(runDolibarrPanel));    
		
		add(iconTabs, BorderLayout.CENTER);
		revalidate();
		repaint();
	}
	
    public RunDolibarrPanel getRunDolibarrPanel(){
		return runDolibarrPanel;
    }
    
    public BuildClickInterface getSimulationLogPanel(){
		return simulationLogPanel;
    }
	
    public static void main(String[] args) throws Exception{  
    		 
    	PopulateDolibarrDatabasePlugin plugin = new PopulateDolibarrDatabasePlugin();
    	JPanel ret = (JPanel) plugin.populateDolibarrDatabasePanel(null);
    	JFrame frame = new JFrame();
    	frame.getContentPane().add(ret);
    	frame.setExtendedState(javax.swing.JFrame.MAXIMIZED_BOTH);
    	frame.setVisible(true);
    	frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    }
}

