/**
 * Copyright (c) 2006-2012, JGraph Ltd */
package org.processmining.mxgraph.examples.swing;

import java.awt.Color;
import java.awt.Point;
import java.io.UnsupportedEncodingException;
import java.net.URL;
import java.text.NumberFormat;
import java.util.Iterator;
import java.util.List;

import javax.swing.ImageIcon;
import javax.swing.UIManager;

import org.processmining.mxgraph.examples.swing.editor.BasicGraphEditor;
import org.processmining.mxgraph.examples.swing.editor.EditorMenuBar;
import org.processmining.mxgraph.examples.swing.editor.EditorPalette;
import org.processmining.mxgraph.examples.swing.visualization.Style;
import org.processmining.mxgraph.io.mxCodec;
import org.processmining.mxgraph.model.mxCell;
import org.processmining.mxgraph.model.mxGeometry;
import org.processmining.mxgraph.model.mxICell;
import org.processmining.mxgraph.model.mxIGraphModel;
import org.processmining.mxgraph.swing.mxGraphComponent;
import org.processmining.mxgraph.swing.util.mxSwingConstants;
import org.processmining.mxgraph.util.mxConstants;
import org.processmining.mxgraph.util.mxPoint;
import org.processmining.mxgraph.util.mxResources;
import org.processmining.mxgraph.util.mxUtils;
import org.processmining.mxgraph.view.mxCellState;
import org.processmining.mxgraph.view.mxGraph;
import org.processmining.ocbc.discovery.panels.ConstraintInstancePanel;
import org.processmining.ocbc.discovery.panels.DistributionPanel;
import org.processmining.ocbc.models.OCBCModel;
import org.processmining.ocbc.models.impl.OCBCModelImpl;
import org.w3c.dom.Document;

public class GraphEditor extends BasicGraphEditor
{
	/**
	 * 
	 */
	private static final long serialVersionUID = -4601740824088314699L;

	/**
	 * Holds the shared number formatter.
	 * 
	 * @see NumberFormat#getInstance()
	 */
	public static final NumberFormat numberFormat = NumberFormat.getInstance();

	/**
	 * Holds the URL for the icon to be used as a handle for creating new
	 * connections. This is currently unused.
	 */
	public static URL url = null;


	//GraphEditor.class.getResource("/org/processmining/mxgraph/examples/swing/images/connector.gif");

	public GraphEditor()
	{
		this("mxGraph Editor", new CustomGraphComponent(new CustomGraph()), new OCBCModelImpl());
	}
	
	/**
	 * 
	 */
	public GraphEditor(String appTitle, mxGraphComponent component, OCBCModel model)
	{
		super(appTitle, component, model);
		final mxGraph graph = graphComponent.getGraph();
		
		
		System.out.println("\n graph activity relation number: " + graph.getActivityRelationList());
		
//		for(String name : model.getAllActivityNames()){
//			System.out.println("\n activity name: " + name);
//		}
		
		//create my styles
		Style.createStyle(graph);
		graph.setAllowDanglingEdges(false);
		graph.setSplitEnabled(false);	
		
		//if the graph editor is used to show an discovered model
		if(model != null && model.getModelSource() != null && model.getModelSource().equals(mxConstants.discovered)){
			System.out.println("model is existent");
			setOCBCModel(model);
			//disable the model to create target
			graphComponent.getConnectionHandler().setCreateTarget(false);
			Object[] nodeCells = graphComponent.getGraph().getChildCells(graphComponent.getGraph().getDefaultParent());
			System.out.println("nodeCells.length" + nodeCells.length);
			for(int i=0; i<nodeCells.length; i++){
				mxCell nodeCell = (mxCell) nodeCells[i];
				nodeCell.setConnectable(false);
	
			}
			//disable the model to connect
			ConstraintInstancePanel constraintInstancePanle = new ConstraintInstancePanel(10, 5, 0);
			DistributionPanel distributionPanel = new DistributionPanel(10, 5, 0, model, this);
//			distributionPanel.setMaximumSize(new Dimension(200,200));
			insertMyPalettePanel(mxConstants.Distribution, distributionPanel);
			insertMyPalettePanel(mxConstants.Instance, constraintInstancePanle);
		}
		else{
			System.out.println("model is empty");
			EditorPalette symbolsPalette = insertPalette(mxResources.get("symbols"));

			symbolsPalette
			.addTemplate(
					mxConstants.ACTIVITYCELL,
//					mxConstants.ACTIVITY,
					new ImageIcon(
							GraphEditor.class
									.getResource("/org/processmining/mxgraph/examples/swing/images/activity.png")),
					"roundImage;image=/org/processmining/mxgraph/examples/swing/images/activity.png",
					53, 40, mxConstants.ACTIVITYCELL);
			
			symbolsPalette
			.addTemplate(
					mxConstants.CLASSCELL,
//					"Class",
					new ImageIcon(
							GraphEditor.class
									.getResource("/org/processmining/mxgraph/examples/swing/images/class.png")),
					"roundImage;image=/org/processmining/mxgraph/examples/swing/images/class.png",
					53, 40, mxConstants.CLASSCELL);
		}
		
		//set default zoom scale
//		double scale = 1.0;
//		graphComponent.setCenterZoom(true);
//		graphComponent.zoomTo(scale, graphComponent
//				.isCenterZoom());
//
//        JScrollBar horizontal = graphComponent.getHorizontalScrollBar();
//
//        int panelWidth = horizontal.getMaximum();
//        System.out.println("panelWidth: " + panelWidth);
//        double barLengthHorizontal = (horizontal.getMaximum()*51)/100;
//
//        horizontal.setValue(200);
//        
//        System.out.println("horizontal.getMaximum(): " + horizontal.getMaximum());
//        System.out.println("panelWidth: " + panelWidth);
//        System.out.println("barLengthHorizontal: " + barLengthHorizontal);
// 
//        JScrollBar vertical = graphComponent.getVerticalScrollBar();
//        double panelHeight = vertical.getMaximum();
//        //when the scale is 31%, the length of the bar is the height of the panel;
//        //accordingly, when the scale is 100%, the length of the bar is the 31/100 height of the panel;
//        double barLength = (panelHeight*31)/100;
//        vertical.setValue((int)(panelHeight/2  - barLength/2));
//        System.out.println("vertical.getMaximum(): " + vertical.getMaximum());
//        System.out.println("panelHeight: " + panelHeight);
//        System.out.println("barLength: " + barLength);
//
//        horizontal.setValue(675);

	}

	/**
	 * 
	 */
	public static class CustomGraphComponent extends mxGraphComponent
	{

		/**
		 * 
		 */
		private static final long serialVersionUID = -6833603133512882012L;

		/**
		 * 
		 * @param graph
		 */
		public CustomGraphComponent(mxGraph graph)
		{
			super(graph);

			// Sets switches typically used in an editor
			setPageVisible(true);
			setGridVisible(true);
			setToolTips(true);
			getConnectionHandler().setCreateTarget(true);

			// Loads the defalt stylesheet from an external file
			mxCodec codec = new mxCodec();
			Document doc = mxUtils.loadDocument(GraphEditor.class.getResource(
					"/org/processmining/mxgraph/examples/swing/resources/default-style.xml")
					.toString());
			codec.decode(doc.getDocumentElement(), graph.getStylesheet());

			// Sets the background to white
			getViewport().setOpaque(true);
			getViewport().setBackground(Color.WHITE);
		}

		/**
		 * Overrides drop behaviour to set the cell style if the target
		 * is not a valid drop target and the cells are of the same
		 * type (eg. both vertices or both edges). 
		 */
		public Object[] importCells(Object[] cells, double dx, double dy,
				Object target, Point location)
		{
			if (target == null && cells.length == 1 && location != null)
			{
				target = getCellAt(location.x, location.y);

				if (target instanceof mxICell && cells[0] instanceof mxICell)
				{
					mxICell targetCell = (mxICell) target;
					mxICell dropCell = (mxICell) cells[0];

					if (targetCell.isVertex() == dropCell.isVertex()
							|| targetCell.isEdge() == dropCell.isEdge())
					{
						mxIGraphModel model = graph.getModel();
						model.setStyle(target, model.getStyle(cells[0]));
						graph.setSelectionCell(target);

						return null;
					}
				}
			}

			return super.importCells(cells, dx, dy, target, location);
		}

	}

	/**
	 * A graph that creates new edges from a given template edge.
	 */
	public static class CustomGraph extends mxGraph
	{
		/**
		 * Holds the edge to be used as a template for inserting new edges.
		 */
		protected Object edgeTemplate;

		/**
		 * Custom graph that defines the alternate edge style to be used when
		 * the middle control point of edges is double clicked (flipped).
		 */
		public CustomGraph()
		{
			setAlternateEdgeStyle("edgeStyle=mxEdgeStyle.ElbowConnector;elbow=vertical");
		}

		/**
		 * Sets the edge template to be used to inserting edges.
		 */
		public void setEdgeTemplate(Object template)
		{
			edgeTemplate = template;
		}

		/**
		 * Prints out some useful information about the cell in the tooltip.
		 */
		public String getToolTipForCell(Object cell)
		{
			String tip = "<html>";
			mxGeometry geo = getModel().getGeometry(cell);
			mxCellState state = getView().getState(cell);

			if (getModel().isEdge(cell))
			{
				tip += "points={";

				if (geo != null)
				{
					List<mxPoint> points = geo.getPoints();

					if (points != null)
					{
						Iterator<mxPoint> it = points.iterator();

						while (it.hasNext())
						{
							mxPoint point = it.next();
							tip += "[x=" + numberFormat.format(point.getX())
									+ ",y=" + numberFormat.format(point.getY())
									+ "],";
						}

						tip = tip.substring(0, tip.length() - 1);
					}
				}

				tip += "}<br>";
				tip += "absPoints={";

				if (state != null)
				{

					for (int i = 0; i < state.getAbsolutePointCount(); i++)
					{
						mxPoint point = state.getAbsolutePoint(i);
						tip += "[x=" + numberFormat.format(point.getX())
								+ ",y=" + numberFormat.format(point.getY())
								+ "],";
					}

					tip = tip.substring(0, tip.length() - 1);
				}

				tip += "}";
			}
			else
			{
				tip += "geo=[";

				if (geo != null)
				{
					tip += "x=" + numberFormat.format(geo.getX()) + ",y="
							+ numberFormat.format(geo.getY()) + ",width="
							+ numberFormat.format(geo.getWidth()) + ",height="
							+ numberFormat.format(geo.getHeight());
				}

				tip += "]<br>";
				tip += "state=[";

				if (state != null)
				{
					tip += "x=" + numberFormat.format(state.getX()) + ",y="
							+ numberFormat.format(state.getY()) + ",width="
							+ numberFormat.format(state.getWidth())
							+ ",height="
							+ numberFormat.format(state.getHeight());
				}

				tip += "]";
			}

			mxPoint trans = getView().getTranslate();

			tip += "<br>scale=" + numberFormat.format(getView().getScale())
					+ ", translate=[x=" + numberFormat.format(trans.getX())
					+ ",y=" + numberFormat.format(trans.getY()) + "]";
			tip += "</html>";

			return tip;
		}

		/**
		 * Overrides the method to use the currently selected edge template for
		 * new edges.
		 * 
		 * @param graph
		 * @param parent
		 * @param id
		 * @param value
		 * @param source
		 * @param target
		 * @param style
		 * @return
		 */
		public Object createEdge(Object parent, String id, Object value,
				Object source, Object target, String style)
		{
			if (edgeTemplate != null)
			{
				mxCell edge = (mxCell) cloneCells(new Object[] { edgeTemplate })[0];
				edge.setId(id);

				return edge;
			}

			return super.createEdge(parent, id, value, source, target, style);
		}
	}


	
	/**
	 * 
	 * @param args
	 * @throws UnsupportedEncodingException 
	 */
	public static void main(String[] args) throws UnsupportedEncodingException
	{
//		for (int c=1; c<256; c++) {
//		    System.out.println(c + ": " + Character.toString ((char)c));
//		}
//		System.out.println("\u2666");
//		String nullString = "";
//		if(nullString == null){
//			System.out.println("empty: " + nullString);
//		}
//		else{
//			System.out.println("not empty: " + nullString);
//		}
//		System.exit(1);
		
		byte[] utf8Bytes       = null; 
		String convertedString = null; 
		String unicodeString   = new String("\u25C7"); //"Help" 
		utf8Bytes       = unicodeString.getBytes("UTF8");   
		convertedString = new String(utf8Bytes,  "UTF8"); 
		System.out.println(convertedString); //same as the original string 
		
		System.out.println("*\u25C7" + "*");
		try
		{
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		}
		catch (Exception e1)
		{
			e1.printStackTrace();
		}

		mxSwingConstants.SHADOW_COLOR = Color.LIGHT_GRAY;
		mxConstants.W3C_SHADOWCOLOR = "#D3D3D3";

		GraphEditor editor = new GraphEditor();
		editor.createFrame(new EditorMenuBar(editor)).setVisible(true);
	}
}
