package org.processmining.mxgraph.examples.swing.editor;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Desktop;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.MouseEvent;
import java.io.File;
import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JLayer;
import javax.swing.JMenuBar;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTabbedPane;
import javax.swing.JToolBar;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.util.Pair;
import org.processmining.mxgraph.examples.swing.io.TransformBetweenJGraphXAndModel;
import org.processmining.mxgraph.examples.swing.listener.AddElementListener;
import org.processmining.mxgraph.examples.swing.listener.GraphComponentMouseListener;
import org.processmining.mxgraph.examples.swing.listener.GraphComponentMouseMotionListener;
import org.processmining.mxgraph.examples.swing.listener.GraphComponentMouseWheelListener;
import org.processmining.mxgraph.examples.swing.panels.AttributePanel;
import org.processmining.mxgraph.examples.swing.panels.CloseableTabbedPaneLayerUI;
import org.processmining.mxgraph.examples.swing.panels.FilterPanel;
import org.processmining.mxgraph.examples.swing.panels.GenerateInformationSystemFrame;
import org.processmining.mxgraph.examples.swing.visualization.Style;
import org.processmining.mxgraph.layout.mxCircleLayout;
import org.processmining.mxgraph.layout.mxCompactTreeLayout;
import org.processmining.mxgraph.layout.mxEdgeLabelLayout;
import org.processmining.mxgraph.layout.mxIGraphLayout;
import org.processmining.mxgraph.layout.mxOrganicLayout;
import org.processmining.mxgraph.layout.mxParallelEdgeLayout;
import org.processmining.mxgraph.layout.mxPartitionLayout;
import org.processmining.mxgraph.layout.mxStackLayout;
import org.processmining.mxgraph.layout.hierarchical.mxHierarchicalLayout;
import org.processmining.mxgraph.model.mxCell;
import org.processmining.mxgraph.model.mxGeometry;
import org.processmining.mxgraph.model.mxGraphModel.mxChildChange;
import org.processmining.mxgraph.model.mxGraphModel.mxGeometryChange;
import org.processmining.mxgraph.swing.mxGraphComponent;
import org.processmining.mxgraph.swing.mxGraphOutline;
import org.processmining.mxgraph.swing.handler.mxKeyboardHandler;
import org.processmining.mxgraph.swing.handler.mxRubberband;
import org.processmining.mxgraph.swing.util.mxMorphing;
import org.processmining.mxgraph.util.mxConstants;
import org.processmining.mxgraph.util.mxEvent;
import org.processmining.mxgraph.util.mxEventObject;
import org.processmining.mxgraph.util.mxEventSource.mxIEventListener;
import org.processmining.mxgraph.util.mxRectangle;
import org.processmining.mxgraph.util.mxResources;
import org.processmining.mxgraph.util.mxUndoManager;
import org.processmining.mxgraph.util.mxUndoableEdit;
import org.processmining.mxgraph.util.mxUndoableEdit.mxUndoableChange;
import org.processmining.mxgraph.view.mxGraph;
import org.processmining.ocbc.discovery.layout.ModelLayout;
import org.processmining.ocbc.discovery.panels.ButtonPanel;
import org.processmining.ocbc.discovery.panels.ConstraintInstancePanel;
import org.processmining.ocbc.discovery.panels.DistributionPanel;
import org.processmining.ocbc.discovery.panels.MetricsAlgorithmPanel;
import org.processmining.ocbc.discovery.panels.SliderPanel;
import org.processmining.ocbc.models.OCBCModel;
import org.processmining.ocbc.models.editor.plugins.OCBCEditorConstants;

public class BasicGraphEditor extends JPanel
{

	/**
	 * 
	 */
	private static final long serialVersionUID = -6561623072112577140L;

	protected static final String diamondSymbol = null;

	/**
	 * Adds required resources for i18n
	 */
	static
	{
		try
		{
			mxResources.add("org.processmining/mxgraph/examples/swing/resources/editor");
		}
		catch (Exception e)
		{
			// ignore
		}
	}

	/**
	 * save the model in the workspace
	 */
	protected PluginContext context;
	
	/**
	 * 
	 */
	protected mxGraphComponent graphComponent;

	protected JTabbedPane multipleGraphPanel;
	
	/**
	 * 
	 */
	protected mxGraphOutline graphOutline;

	/**
	 * 
	 */
	protected JTabbedPane libraryPane;

	/**
	 * 
	 */
	protected mxUndoManager undoManager;

	/**
	 * 
	 */
	protected String appTitle;

	/**
	 * 
	 */
	protected JLabel statusBar;

	/**
	 * 
	 */
	protected File currentFile;
	
	/**
	 * each graph component corresponds to one file
	 */
	protected Map<mxGraphComponent, File> graphComponentVSFileMap = new HashMap<mxGraphComponent, File>();

	/**
	 * Flag indicating whether the current graph has been modified 
	 */
	protected boolean modified = false;

	/**
	 * 
	 */
	protected mxRubberband rubberband;

	/**
	 * 
	 */
	protected mxKeyboardHandler keyboardHandler;
	
	protected EditorToolBar toolBar;
	
	protected OCBCModel model;
	
	public static final NumberFormat numberFormat = NumberFormat.getInstance();

	/**
	 * 
	 */
	protected mxIEventListener undoHandler = new mxIEventListener()
	{
		public void invoke(Object source, mxEventObject evt)
		{
			undoManager.undoableEditHappened((mxUndoableEdit) evt
					.getProperty("edit"));
		}
	};
    
    private FilterPanel relationTypePanel;
    private SliderPanel sliderPanel;
    private MetricsAlgorithmPanel metricsAlgorithmPanel;
    private ButtonPanel buttonPanel;
    private AttributePanel attributePanel;
	/**
	 * 
	 */
	protected mxIEventListener changeTracker = new mxIEventListener()
	{
		public void invoke(Object source, mxEventObject evt)
		{
			setModified(true);
			//change name in multiple graph panel						
			addModificationFlag();							
			//add source and target labels for the new created edge
			Object changes = evt.getProperties().get("changes");
            if (changes instanceof ArrayList){           	
                ArrayList list = (ArrayList) changes;
                //judge the chagne type
                for (Object o : list){
                    if (o instanceof mxGeometryChange){
                        Object value = ((mxCell) ((mxGeometryChange) o).getCell()).getValue();
                        System.out.println("mxGeometryChange");
                    }
                    else if (o instanceof mxChildChange)
                    {
                    	mxCell newCell = (mxCell) ((mxChildChange) o).getChild();
                    	//if the graph contain the cell and the cell is not imported
                    	AddElementListener listenerAction = new AddElementListener(graphComponent);
                    	listenerAction.transformCell(newCell);                
                    }
                }
           }
		}
	};

	/**
	 * get the attribute panel (which show the attributes of selected node or edge)
	 * @return
	 */
	public AttributePanel getAttributePanel(){
		return attributePanel;
	}
	
	/**
	 * 
	 */
	public BasicGraphEditor(String appTitle, mxGraphComponent component, OCBCModel model)
	{
		// Stores and updates the frame title
		this.appTitle = appTitle;
		this.model = model;
		// Stores a reference to the graph and creates the command history
		this.graphComponent = component;		
		
		
		multipleGraphPanel = new JTabbedPane();
		multipleGraphPanel.addTab("graph1*" + "     ", graphComponent);
		multipleGraphPanel.addChangeListener(new ChangeListener(){
			public void stateChanged(ChangeEvent arg0) {
				// TODO Auto-generated method stub
				System.out.println("Tab: " + multipleGraphPanel.getSelectedIndex());
				int selectedComponentIndex = multipleGraphPanel.getSelectedIndex();
				
				mxGraphComponent selectedComponent = (mxGraphComponent) multipleGraphPanel.getComponentAt(selectedComponentIndex);
				setGraphComponent(selectedComponent);
				setCurrentFile(graphComponentVSFileMap.get(selectedComponent));
				
//				selectedComponent.setSize(5000, 5000);
				graphOutline.setGraphComponent(selectedComponent);
				graphOutline.repaint();
				
				//update the value in the zoom combobox
		    	double scale = selectedComponent.getGraph().getView().getScale();
				getToolBar().getZoomBox().setSelectedItem((int) (100 * scale) + "%");
				updateEditor();
			}
		});
		
		JLayer<JTabbedPane> closeableTabbedPanel = new JLayer<JTabbedPane>(multipleGraphPanel, new CloseableTabbedPaneLayerUI());
		 
		final mxGraph graph = graphComponent.getGraph();
		undoManager = createUndoManager();

 			
		// Do not change the scale and translation after files have been loaded
		graph.setResetViewOnRootChange(false);

		// Updates the modified flag if the graph model changes
		graph.getModel().addListener(mxEvent.CHANGE, changeTracker);

		// Adds the command history to the model and view
		graph.getModel().addListener(mxEvent.UNDO, undoHandler);
		graph.getView().addListener(mxEvent.UNDO, undoHandler);

		// Keeps the selection in sync with the command history
		mxIEventListener undoHandler = new mxIEventListener()
		{
			public void invoke(Object source, mxEventObject evt)
			{
				List<mxUndoableChange> changes = ((mxUndoableEdit) evt
						.getProperty("edit")).getChanges();
				graph.setSelectionCells(graph
						.getSelectionCellsForChanges(changes));
			}
		};

		undoManager.addListener(mxEvent.UNDO, undoHandler);
		undoManager.addListener(mxEvent.REDO, undoHandler);      
		
		//creates the attribute component of classes	
		attributePanel = new AttributePanel(graphComponent);
		JScrollPane attributeJScrollPanel = new JScrollPane(attributePanel);
		

		// Creates the graph outline component
		graphOutline = new mxGraphOutline(graphComponent);
		
        JSplitPane leftBottom = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
        leftBottom.setTopComponent(attributeJScrollPanel);
        leftBottom.setBottomComponent(graphOutline);
        leftBottom.setResizeWeight(0.1);
        		
		// Creates the library pane that contains the tabs with the palettes
		libraryPane = new JTabbedPane();
//		libraryPane.setMaximumSize(new Dimension(200,200));
        JSplitPane leftPanel = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
//        leftPanel.setTopComponent(new JScrollPane(libraryPane));
        leftPanel.setTopComponent(libraryPane);
        leftPanel.setBottomComponent(leftBottom);
        leftPanel.setResizeWeight(0.3);
     
		// Creates the complete pane that contains the left split pane and
		// the graph component on the right side of the window
        JSplitPane completePanel = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
        completePanel.setLeftComponent(leftPanel);
        
        Object[] nodeCells = graphComponent.getGraph().getChildCells(graphComponent.getGraph().getDefaultParent());
        System.out.println("nodeCells.length in basic" + nodeCells.length);
        
        //in the discovery mode, add the filter panel
        if(model != null && model.getModelSource() != null && model.getModelSource().equals(mxConstants.discovered)){
        	relationTypePanel = new FilterPanel(graphComponent, this);
        	sliderPanel = new SliderPanel(this);
        	metricsAlgorithmPanel = new MetricsAlgorithmPanel(this);
        	buttonPanel = new ButtonPanel(this);
    	    
//    		JSplitPane filterPanel = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
//    		filterPanel.setTopComponent(buttonPanel);
//    		filterPanel.add(metricsAlgorithmPanel);
//    		filterPanel.setBottomComponent(sliderPanel);
//    		filterPanel.setResizeWeight(0.01);
    		
        	JPanel filterPanel = new JPanel();
        	filterPanel.setLayout(new BoxLayout(filterPanel, BoxLayout.Y_AXIS));
        	buttonPanel.setMaximumSize(new Dimension(1000,30));
        	metricsAlgorithmPanel.setMaximumSize(new Dimension(1000,30));
//        	sliderPanel.setMaximumSize(new Dimension(1000,30));
    		filterPanel.add(buttonPanel);
    		filterPanel.add(metricsAlgorithmPanel);
    		filterPanel.add(sliderPanel);
    		
    		JSplitPane operationPanel = new JSplitPane(JSplitPane.VERTICAL_SPLIT);   		
    		operationPanel.setTopComponent(new JScrollPane(relationTypePanel));
    		operationPanel.setBottomComponent(filterPanel);
    		operationPanel.setResizeWeight(0.2);
    		    
            JSplitPane rightPanel = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
            rightPanel.setLeftComponent(closeableTabbedPanel);
            rightPanel.setRightComponent(operationPanel);
            rightPanel.setResizeWeight(0.8);
            completePanel.setRightComponent(rightPanel);
        	completePanel.setResizeWeight(0.2);
        }
        else{
            completePanel.setRightComponent(closeableTabbedPanel);
            completePanel.setResizeWeight(0.02);
        }

        
		// Creates the status bar
		statusBar = createStatusBar();

		// Display some useful information about repaint events
		installRepaintListener();

		// Puts everything together
		setLayout(new BorderLayout());
		add(completePanel, BorderLayout.CENTER);
		add(statusBar, BorderLayout.SOUTH);
		installToolBar();

		// Installs rubberband selection and handling for some special
		// keystrokes such as F2, Control-C, -V, X, A etc.
		installHandlers();
		installListeners();
		updateTitle();
		attributePanel.disableAttributePanel();
		//set the default horizontal page count as 2
		graphComponent.setHorizontalPageCount(2);
	}
	
	protected  void addListenersToGraphComponent(mxGraphComponent graphComponent){
		final mxGraph graph = graphComponent.getGraph();
		System.out.println("enter add listeners");
		// Do not change the scale and translation after files have been loaded
		graph.setResetViewOnRootChange(false);

		// Updates the modified flag if the graph model changes
		graph.getModel().addListener(mxEvent.CHANGE, changeTracker);

		// Adds the command history to the model and view
		graph.getModel().addListener(mxEvent.UNDO, undoHandler);
		graph.getView().addListener(mxEvent.UNDO, undoHandler);

		// Keeps the selection in sync with the command history
		mxIEventListener undoHandler = new mxIEventListener()
		{
			public void invoke(Object source, mxEventObject evt)
			{
				List<mxUndoableChange> changes = ((mxUndoableEdit) evt
						.getProperty("edit")).getChanges();
				graph.setSelectionCells(graph
						.getSelectionCellsForChanges(changes));
			}
		};

		undoManager.addListener(mxEvent.UNDO, undoHandler);
		undoManager.addListener(mxEvent.REDO, undoHandler);

//		// Creates the graph outline component
//		graphOutline = new mxGraphOutline(graphComponent);




		// Display some useful information about repaint events
		installRepaintListener();


		// Installs rubberband selection and handling for some special
		// keystrokes such as F2, Control-C, -V, X, A etc.
		installHandlers();
		installListeners();
		updateTitle();
	}

	public FilterPanel getFilterPanel(){
		return relationTypePanel;
	}
	
	public SliderPanel getSliderPanel(){
		return sliderPanel;
	}
	
	public MetricsAlgorithmPanel getAlgorithmPanel(){
		return metricsAlgorithmPanel;
	}	 
	
	public ButtonPanel getButtonPanel(){
		return buttonPanel;
	}
	
	public EditorToolBar getToolBar(){
		return toolBar;
	}

	//when the graph model is modified, add the "*" symbol
	public void addModificationFlag(){
		int componentIndex = multipleGraphPanel.indexOfComponent(graphComponent);
		String pureName = multipleGraphPanel.getTitleAt(componentIndex);
		String modifedName = null;
		if(!pureName.contains("*")){
			//add "*" into the name to indicate the file is modified and unsaved
			modifedName = pureName.replace("     ", "");
			modifedName = modifedName + "*" + "     ";
			multipleGraphPanel.setTitleAt(componentIndex, modifedName);	
		}
	}
	
	public void updateEditor(){
    	graphComponent.getGraph().getModel().beginUpdate();
    	graphComponent.getGraph().getModel().endUpdate();
		this.updateUI();
	}
	
	//refresh the graph model, such as labels
	public void refreshGraph(){
		System.out.println("refresh");
		mxGraph graph = graphComponent.getGraph();	
		Object[] edgeCells = graph.getChildEdges(graph.getDefaultParent());
		Object[] nodeCells = graph.getChildVertices(graph.getDefaultParent());	
		for(int j=0; j<nodeCells.length; j++){
			mxCell childCell = (mxCell) nodeCells[j];					
			String type = childCell.getAttributes().get(mxConstants.TYPE);
			System.out.println("type" + type);
			if(type.equals(mxConstants.CLASSCELL)){	
				if(!childCell.getAttributes().containsKey(mxConstants.ROWID)){
					//assume the primary id is "rowid";could be changed in future
					childCell.getAttributes().put(mxConstants.ROWID, mxConstants.PK + "-" + "rowid");
				}
				if(!childCell.getAttributes().containsKey(mxConstants.TABLE)){
					//assume the primary id is "rowid";could be changed in future
					childCell.getAttributes().put(mxConstants.TABLE, "table" + "-" + "");
				}
			}
			else if(type.equals(mxConstants.ACTIVITYCELL)){	
				if(!childCell.getAttributes().containsKey(mxConstants.ACTIVITY)){
					//assume the primary id is "rowid";could be changed in future
					childCell.getAttributes().put(mxConstants.ACTIVITY, "column" + "-" + "name");
				}
				if(!childCell.getAttributes().containsKey(mxConstants.TABLE)){
					//assume the primary id is "rowid";could be changed in future
					childCell.getAttributes().put(mxConstants.TABLE, "table" + "-" + "");
				}
				
			}
		}
				
		for(int j=0; j<edgeCells.length; j++){
			mxCell childCell = (mxCell) edgeCells[j];					
			String type = childCell.getAttributes().get(mxConstants.TYPE);
			//update class relations
			if(type.equals(mxConstants.CLASSRELATION)){	
				System.out.println("update the label of class relations");
				//update the cardinality label of class relations
				mxCell sourceCellChildAlways = (mxCell) childCell.getChildAt(0);
				mxCell sourceCellChildEventually = (mxCell) childCell.getChildAt(1);
				mxCell targetCellChildAlways = (mxCell) childCell.getChildAt(2);
				mxCell targetCellChildEventually = (mxCell) childCell.getChildAt(3);
				String sourceAlwaysCardinality = childCell.getAttributes().get(mxConstants.SOURCEALWAYSCARDINALITY); 
				String sourceEventuallyCardinality = childCell.getAttributes().get(mxConstants.SOURCEEVENTUALLYCARDINALITY);
				String targetAlwaysCardinality = childCell.getAttributes().get(mxConstants.TARGETALWAYSCARDINALITY);
				String targetEventuallyCardinality = childCell.getAttributes().get(mxConstants.TARGETEVENTUALLYCARDINALITY);				
				sourceCellChildAlways.setValue(mxConstants.SQUARESYMBOL + sourceAlwaysCardinality);
				sourceCellChildEventually.setValue(mxConstants.DIAMONDSYMBOL + sourceEventuallyCardinality);
				targetCellChildAlways.setValue(mxConstants.SQUARESYMBOL + targetAlwaysCardinality);
				targetCellChildEventually.setValue(mxConstants.DIAMONDSYMBOL + targetEventuallyCardinality);
				
				//refresh the edge name in the class relationship type
				mxCell sourceCell = (mxCell) childCell.getSource();
				mxCell targetCell = (mxCell) childCell.getTarget();
				String edgeName = (String) childCell.getValue();
				childCell.getAttributes().put(mxConstants.RELATIONSHIPTYPE, sourceCell.getValue() + "-" + edgeName + "-" + targetCell.getValue());
				//refresh or add the fk attribute in the target class
				targetCell.getAttributes().put(edgeName, mxConstants.FK + "-" + sourceCell.getValue());	     															   		
        	}			
			else if(type.equals(mxConstants.AOCRELATION)){	
				System.out.println("update aoc relations");
				//refresh or add the activity attribute in the activity node and class node
				mxCell sourceCell = (mxCell) childCell.getSource();//activity node
				mxCell targetCell = (mxCell) childCell.getTarget();//class node
				mxCell sourceCellChildAlways = (mxCell) childCell.getChildAt(0);
				mxCell sourceCellChildEventually = (mxCell) childCell.getChildAt(1);
				mxCell targetCellChildAlways = (mxCell) childCell.getChildAt(2);
				String sourceAlwaysCardinality = childCell.getAttributes().get(mxConstants.SOURCEALWAYSCARDINALITY); 
				String sourceEventuallyCardinality = childCell.getAttributes().get(mxConstants.SOURCEEVENTUALLYCARDINALITY);
				String targetAlwaysCardinality = childCell.getAttributes().get(mxConstants.TARGETALWAYSCARDINALITY);

				sourceCellChildAlways.setValue(mxConstants.SQUARESYMBOL + sourceAlwaysCardinality);
				sourceCellChildEventually.setValue(mxConstants.DIAMONDSYMBOL + sourceEventuallyCardinality);
				targetCellChildAlways.setValue(targetAlwaysCardinality);
				
				String edgeName = (String) childCell.getValue();
				String activityName = (String) sourceCell.getValue();
				String activityTableName = sourceCell.getAttributes().get(mxConstants.TABLE);
				String classTableName = targetCell.getAttributes().get(mxConstants.TABLE);
//				//the edge name is empty; currently it indicate it is not a source aoc relation
//				if(edgeName == null || edgeName.equals("")){
//					
//				}
				System.out.println("update activity and class based on aoc relations");
				System.out.println("classTableName: " + classTableName);
				sourceCell.getAttributes().put(mxConstants.TABLE, classTableName + "-" + classTableName); 
				sourceCell.getAttributes().put(mxConstants.ACTIVITY, edgeName + "-" + activityName); 
				targetCell.getAttributes().put(edgeName, mxConstants.ACTIVITY + "-" + activityName); 	    						
			}
			graph.refresh();
		}
		
		
	}
	
	/**
	 * 
	 */
	protected mxUndoManager createUndoManager()
	{
		return new mxUndoManager();
	}

	/**
	 * 
	 */
	protected void installHandlers()
	{
		rubberband = new mxRubberband(graphComponent);
		keyboardHandler = new EditorKeyboardHandler(graphComponent);
	}

	/**
	 * 
	 */
	protected void installToolBar()
	{
		EditorToolBar editorToolBar = new EditorToolBar(this, JToolBar.HORIZONTAL);
		toolBar = editorToolBar;
		toolBar.getCardinalityTypeBox().setEnabled(false);
		toolBar.getCardinalitySourceAlwaysBox().setEnabled(false);
		toolBar.getCardinalitySourceEventuallyBox().setEnabled(false);
		toolBar.getCardinalityTargetAlwaysBox().setEnabled(false);
		toolBar.getCardinalityTargetEventuallyBox().setEnabled(false);
		add(editorToolBar, BorderLayout.NORTH);
	}

	/**
	 * 
	 */
	protected JLabel createStatusBar()
	{
		JLabel statusBar = new JLabel(mxResources.get("ready"));
		statusBar.setBorder(BorderFactory.createEmptyBorder(2, 4, 2, 4));

		return statusBar;
	}

	/**
	 * 
	 */
	protected void installRepaintListener()
	{
		graphComponent.getGraph().addListener(mxEvent.REPAINT,
				new mxIEventListener()
				{
					public void invoke(Object source, mxEventObject evt)
					{
						String buffer = (graphComponent.getTripleBuffer() != null) ? ""
								: " (unbuffered)";
						mxRectangle dirty = (mxRectangle) evt
								.getProperty("region");

						if (dirty == null)
						{
							status("Repaint all" + buffer);
						}
						else
						{
							status("Repaint: x=" + (int) (dirty.getX()) + " y="
									+ (int) (dirty.getY()) + " w="
									+ (int) (dirty.getWidth()) + " h="
									+ (int) (dirty.getHeight()) + buffer);
						}
					}
				});
	}

	/**
	 * 
	 */
	public EditorPalette insertPalette(String title)
	{
		final EditorPalette palette = new EditorPalette();
		final JScrollPane scrollPane = new JScrollPane(palette);
		scrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		scrollPane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
		libraryPane.add(title, scrollPane);

		// Updates the widths of the palettes if the container size changes
		libraryPane.addComponentListener(new ComponentAdapter()
		{
			/**
			 * 
			 */
			public void componentResized(ComponentEvent e)
			{
				int w = scrollPane.getWidth()
						- scrollPane.getVerticalScrollBar().getWidth();
				palette.setPreferredWidth(w);
			}

		});

		return palette;
	}

	public void insertMyPalettePanel(String title, Component panel)
	{
		libraryPane.add(title, panel);
	}


	/**
	 * 
	 */
	protected void showOutlinePopupMenu(MouseEvent e)
	{
		Point pt = SwingUtilities.convertPoint(e.getComponent(), e.getPoint(),
				graphComponent);
		JCheckBoxMenuItem item = new JCheckBoxMenuItem(
				mxResources.get("magnifyPage"));
		item.setSelected(graphOutline.isFitPage());

		item.addActionListener(new ActionListener()
		{
			/**
			 * 
			 */
			public void actionPerformed(ActionEvent e)
			{
				graphOutline.setFitPage(!graphOutline.isFitPage());
				graphOutline.repaint();
			}
		});

		JCheckBoxMenuItem item2 = new JCheckBoxMenuItem(
				mxResources.get("showLabels"));
		item2.setSelected(graphOutline.isDrawLabels());

		item2.addActionListener(new ActionListener()
		{
			/**
			 * 
			 */
			public void actionPerformed(ActionEvent e)
			{
				graphOutline.setDrawLabels(!graphOutline.isDrawLabels());
				graphOutline.repaint();
			}
		});

		JCheckBoxMenuItem item3 = new JCheckBoxMenuItem(
				mxResources.get("buffering"));
		item3.setSelected(graphOutline.isTripleBuffered());

		item3.addActionListener(new ActionListener()
		{
			/**
			 * 
			 */
			public void actionPerformed(ActionEvent e)
			{
				graphOutline.setTripleBuffered(!graphOutline.isTripleBuffered());
				graphOutline.repaint();
			}
		});

		JPopupMenu menu = new JPopupMenu();
		menu.add(item);
		menu.add(item2);
		menu.add(item3);
		menu.show(graphComponent, pt.x, pt.y);

		e.consume();
	}

	/**
	 * 
	 */
	public void showGraphPopupMenu(MouseEvent e)
	{
		Point pt = SwingUtilities.convertPoint(e.getComponent(), e.getPoint(),
				graphComponent);
		EditorPopupMenu menu = new EditorPopupMenu(BasicGraphEditor.this);
		System.out.println("show pop up menu");
		menu.show(graphComponent, pt.x, pt.y);

		e.consume();
	}

	/**
	 * 
	 */
	protected void installListeners()
	{
		//create the mouse wheel event listener
		GraphComponentMouseWheelListener wheelTracker = new GraphComponentMouseWheelListener(this);
		// Handles mouse wheel events in the outline and graph component
		graphOutline.addMouseWheelListener(wheelTracker);
		graphComponent.addMouseWheelListener(wheelTracker);
		

		// Installs the popup menu in the graph component
		GraphComponentMouseListener graphComponentMouseListener = new GraphComponentMouseListener(model, graphComponent);
		graphComponent.getGraphControl().addMouseListener(graphComponentMouseListener);
		graphComponentMouseListener.setModelEditor(this);

		// Installs a mouse motion listener to display the mouse location
		graphComponent.getGraphControl().addMouseMotionListener(new GraphComponentMouseMotionListener(this));	
	}

	public void disableAllBoxes(){
		toolBar.getEdgeStyleBox().setEnabled(false);
		toolBar.getCardinalityTypeBox().setEnabled(false);
		toolBar.getCardinalitySourceAlwaysBox().setEnabled(false);
		toolBar.getCardinalitySourceEventuallyBox().setEnabled(false);
		toolBar.getCardinalityTargetAlwaysBox().setEnabled(false);
		toolBar.getCardinalityTargetEventuallyBox().setEnabled(false);
	}
	
	/**
	 * highlight one cell with restoring all other cells
	 */
	public void highlightCell(mxCell selectedCell){
		String type = selectedCell.getAttributes().get(mxConstants.TYPE);
		if(selectedCell.isEdge()){		
			//highlight selected cell and all its child cells
			Style.addStyleElementToCell(selectedCell, mxConstants.STYLE_STROKECOLOR, "red");
			if(type.equals(mxConstants.CRELRTRELATION)){
				mxCell connectorActivityEdge = (mxCell) selectedCell.getSource();
				mxCell connectorClassEdge = (mxCell) selectedCell.getTarget();
				mxCell activityEdge = (mxCell) connectorActivityEdge.getParent();
				mxCell classEdge = (mxCell) connectorClassEdge.getParent();
				Style.addStyleElementToCell(activityEdge, mxConstants.STYLE_STROKECOLOR, "red");
				Style.addStyleElementToCell(classEdge, mxConstants.STYLE_STROKECOLOR, "red");
			}
			else if(type.equals(mxConstants.CRELOCRELATION)){
				mxCell connectorActivityEdge = (mxCell) selectedCell.getSource();
				mxCell activityEdge = (mxCell) connectorActivityEdge.getParent();
				Style.addStyleElementToCell((mxCell)selectedCell.getTarget(), mxConstants.STYLE_FONTCOLOR, "red");	
				Style.addStyleElementToCell(activityEdge, mxConstants.STYLE_STROKECOLOR, "red");
			}
			else if(type.equals(mxConstants.CLASSRELATION)){
				mxCell connectorCell = (mxCell) selectedCell.getChildAt(0);
				Object[] edgeCells = graphComponent.getGraph().getChildEdges(graphComponent.getGraph().getDefaultParent());
	    		for(int i=0; i<edgeCells.length; i++){
	    			mxCell edge = (mxCell) edgeCells[i];
	    			if(edge.getSource().equals(connectorCell) || edge.getTarget().equals(connectorCell)){
	    				Style.addStyleElementToCell(edge, mxConstants.STYLE_STROKECOLOR, "red");
	    			}
	    		}		   
    			Style.addStyleElementToCell((mxCell)selectedCell.getTarget(), mxConstants.STYLE_FONTCOLOR, "red");
    			Style.addStyleElementToCell((mxCell)selectedCell.getSource(), mxConstants.STYLE_FONTCOLOR, "red");	
			}
			else if(type.equals(mxConstants.ACTIVITYRELATION)){
				Style.addStyleElementToCell(selectedCell, mxConstants.STYLE_FONTCOLOR, "red");
				mxCell connectorCell = (mxCell) selectedCell.getChildAt(0);
				Object[] edgeCells = graphComponent.getGraph().getChildEdges(graphComponent.getGraph().getDefaultParent());
	    		for(int i=0; i<edgeCells.length; i++){
	    			mxCell edge = (mxCell) edgeCells[i];
	    			if(edge.getSource().equals(connectorCell) || edge.getTarget().equals(connectorCell)){
	    				Style.addStyleElementToCell(edge, mxConstants.STYLE_STROKECOLOR, "red");
	    			}				    			
	    		}	
    			Style.addStyleElementToCell((mxCell)selectedCell.getTarget(), mxConstants.STYLE_FONTCOLOR, "red");
    			Style.addStyleElementToCell((mxCell)selectedCell.getSource(), mxConstants.STYLE_FONTCOLOR, "red");	
	    		System.out.println("cardinalityType: " + selectedCell.getAttributes().get(mxConstants.CARDINALITYTYPE));
			}
			else if(type.equals(mxConstants.AOCRELATION)){
    			Style.addStyleElementToCell((mxCell)selectedCell.getTarget(), mxConstants.STYLE_FONTCOLOR, "red");
    			Style.addStyleElementToCell((mxCell)selectedCell.getSource(), mxConstants.STYLE_FONTCOLOR, "red");	
			}
			//highlight the child cells of the selected cell if possible
			Object[] childCells = graphComponent.getGraph().getChildCells(selectedCell);
			for(int i=0; i<childCells.length; i++){
				System.out.println("kfgz child cell");
				mxCell cilldCell = (mxCell) childCells[i];
				String childCellType = cilldCell.getAttributes().get(mxConstants.TYPE);
				if(childCellType.equals(mxConstants.LABLE)){
					cilldCell.setStyle(mxConstants.LABEL_FOCUSED);	
				}
				else if(childCellType.equals(mxConstants.CONNECTOR)){
					
				}
					    				
			}	
		}
		else{		
    		//highlight the name of the selected node and all linked edges to the selected node
    		Style.addStyleElementToCell(selectedCell, mxConstants.STYLE_FONTCOLOR, "red");
    		Object[] edgeCells = graphComponent.getGraph().getChildEdges(graphComponent.getGraph().getDefaultParent());
    		for(int i=0; i<edgeCells.length; i++){
    			mxCell edge = (mxCell) edgeCells[i];
    			if(edge.getSource().equals(selectedCell) || edge.getTarget().equals(selectedCell)){
    				Style.addStyleElementToCell(edge, mxConstants.STYLE_STROKECOLOR, "red");
    			}
    		}
		}
		graphComponent.getGraph().refresh();
		System.out.println("exit highlightCell");
	}
	
	/*
	 * update the Constraint Boxes (behavioral constraint type or cardinality) based on the selected cell
	 */
	public void updateConstraintBoxes(mxCell selectedCell){
		String type = selectedCell.getAttributes().get(mxConstants.TYPE);
		if(type.equals(mxConstants.ACTIVITYRELATION)){
			toolBar.getCardinalityTypeBox().setEnabled(true);
			toolBar.getCardinalityTypeBox().setSelectedItem(selectedCell.getAttributes().get(mxConstants.CARDINALITYTYPE));
		}
		else if(type.equals(mxConstants.CLASSRELATION)){
			toolBar.getCardinalitySourceAlwaysBox().setEnabled(true);                    		    			
			toolBar.getCardinalitySourceEventuallyBox().setEnabled(true);		    			
			toolBar.getCardinalityTargetAlwaysBox().setEnabled(true);		    			
			toolBar.getCardinalityTargetEventuallyBox().setEnabled(true);

			//update the selected items
			toolBar.getCardinalitySourceAlwaysBox().setSelectedItem(
        			selectedCell.getAttributes().get(mxConstants.SOURCEALWAYSCARDINALITY));	
			toolBar.getCardinalitySourceEventuallyBox().setSelectedItem(
        			selectedCell.getAttributes().get(mxConstants.SOURCEEVENTUALLYCARDINALITY));
			toolBar.getCardinalityTargetAlwaysBox().setSelectedItem(
        			selectedCell.getAttributes().get(mxConstants.TARGETALWAYSCARDINALITY));
			toolBar.getCardinalityTargetEventuallyBox().setSelectedItem(
        			selectedCell.getAttributes().get(mxConstants.TARGETEVENTUALLYCARDINALITY));
		}
		else if(type.equals(mxConstants.AOCRELATION)){
			toolBar.getCardinalitySourceAlwaysBox().setEnabled(true);
			toolBar.getCardinalitySourceEventuallyBox().setEnabled(true);
			toolBar.getCardinalityTargetAlwaysBox().setEnabled(true);
			
			//update the selected items
			toolBar.getCardinalitySourceAlwaysBox().setSelectedItem(
        			selectedCell.getAttributes().get(mxConstants.SOURCEALWAYSCARDINALITY));	
			toolBar.getCardinalitySourceEventuallyBox().setSelectedItem(
        			selectedCell.getAttributes().get(mxConstants.SOURCEEVENTUALLYCARDINALITY));
			toolBar.getCardinalityTargetAlwaysBox().setSelectedItem(
        			selectedCell.getAttributes().get(mxConstants.TARGETALWAYSCARDINALITY));
		}
		else if(type.equals(mxConstants.LABLE)){
			String cardinaltiyType = selectedCell.getAttributes().get(mxConstants.CARDINALITYPOSITION);
			mxCell parentCell = (mxCell) selectedCell.getParent();
			if(cardinaltiyType.equals(mxConstants.SOURCEALWAYSCARDINALITY)){
				toolBar.getCardinalitySourceAlwaysBox().setEnabled(true);
				//update the selected items
				toolBar.getCardinalitySourceAlwaysBox().setSelectedItem(
						parentCell.getAttributes().get(mxConstants.SOURCEALWAYSCARDINALITY));	
			}
			if(cardinaltiyType.equals(mxConstants.SOURCEEVENTUALLYCARDINALITY)){
				toolBar.getCardinalitySourceEventuallyBox().setEnabled(true);
				//update the selected items
				toolBar.getCardinalitySourceEventuallyBox().setSelectedItem(
						parentCell.getAttributes().get(mxConstants.SOURCEEVENTUALLYCARDINALITY));
			}
			else if(cardinaltiyType.equals(mxConstants.TARGETALWAYSCARDINALITY)){
				toolBar.getCardinalityTargetAlwaysBox().setEnabled(true);
				toolBar.getCardinalityTargetAlwaysBox().setSelectedItem(
						parentCell.getAttributes().get(mxConstants.TARGETALWAYSCARDINALITY));
			}
			else if(cardinaltiyType.equals(mxConstants.TARGETEVENTUALLYCARDINALITY)){
				toolBar.getCardinalityTargetEventuallyBox().setEnabled(true);
				toolBar.getCardinalityTargetEventuallyBox().setSelectedItem(
						parentCell.getAttributes().get(mxConstants.TARGETEVENTUALLYCARDINALITY));
			}	
		}
	}

	/*
	 * black all cells:edges, cardinalities and names of edges, names of nodes, connector nodes
	 */
	public void restoreAllCells(){
		Object[] childCells = graphComponent.getGraph().getChildCells(graphComponent.getGraph().getDefaultParent());
		for(int i=0; i<childCells.length; i++){
			mxCell childCell = (mxCell)childCells[i];
			String childCellType = childCell.getAttributes().get(mxConstants.TYPE);
			if(childCellType.equals(mxConstants.AOCRELATION) || childCellType.equals(mxConstants.CRELOCRELATION) || childCellType.equals(mxConstants.CRELRTRELATION)){
				String edgeColor = Style.getStyleElementValueFromCell(graphComponent.getGraph(), childCell, mxConstants.STYLE_STROKECOLOR);
    			System.out.println("in black edgeColor: " + edgeColor);
				Style.addStyleElementToCell((mxCell)childCells[i], mxConstants.STYLE_STROKECOLOR, "blue");
//				Style.addStyleElementToCell((mxCell)childCells[i], mxConstants.STYLE_STROKECOLOR, "black");
			}
			else{
				Style.addStyleElementToCell((mxCell)childCells[i], mxConstants.STYLE_STROKECOLOR, "black");
			}
			
			Style.addStyleElementToCell((mxCell)childCells[i], mxConstants.STYLE_FONTCOLOR, "black");
			Object[] labelCells = graphComponent.getGraph().getChildCells(childCells[i]);
			for(int j=0; j<labelCells.length; j++){
				mxCell labelCell = (mxCell) labelCells[j];
				String cellType = labelCell.getAttributes().get(mxConstants.TYPE);
				if(cellType != null){
					System.out.println("label style : " + labelCell.getStyle());
					Style.addStyleElementToCell(labelCell, mxConstants.STYLE_FONTCOLOR, "black");
				}				
			}						
		}
		graphComponent.getGraph().refresh();
	}
	
	//update the constraint instance  and distribution panel for the selected cell
	public void updateConstraintPanel(mxCell selectedCell){
		System.out.println("\n selectedCell in updateConstraintInstancePanel" + selectedCell.getId());
		ConstraintInstancePanel constraintInstancePanel;
		DistributionPanel distributionPanel;
		System.out.println("libraryPane.getTabCount()" + libraryPane.getTabCount());
		for(int i=0; i<libraryPane.getTabCount(); i++){
			String tabComponentName = libraryPane.getTitleAt(i);
			if(tabComponentName.equals(mxConstants.Instance)){
				constraintInstancePanel = (ConstraintInstancePanel) libraryPane.getComponentAt(i);
				constraintInstancePanel.updateConstraintInstances(selectedCell, model);
			}
//			else if(tabComponentName.equals(mxConstants.Distribution)){
//				distributionPanel = (DistributionPanel) libraryPane.getComponentAt(i);
//				distributionPanel.updateDistributionPanelBoxes(selectedCell, model);
//			}
		}
	}
	
	/**
	 * 
	 */
	public void setCurrentFile(File file)
	{
		File oldValue = currentFile;
		currentFile = file;

		firePropertyChange("currentFile", oldValue, file);

		if (oldValue != file)
		{
			updateTitle();
		}
	}

	/**
	 * 
	 */
	public File getCurrentFile()
	{
		return currentFile;
	}
	
	/**
	 * 
	 */
	public void setGraphComponentFile(mxGraphComponent graphComponent, File file)
	{
		File oldValue = graphComponentVSFileMap.get(graphComponent);
		if(oldValue == null){
			graphComponentVSFileMap.put(graphComponent, file);
		}
		else{
			graphComponentVSFileMap.remove(graphComponent);
			graphComponentVSFileMap.put(graphComponent, file);
		}
	}
	
	/**
	 * 
	 */
	public File getGraphComponentFile(mxGraphComponent graphComponent)
	{
		return graphComponentVSFileMap.get(graphComponent);
	}

	/**
	 * 
	 * @param modified
	 */
	public void setModified(boolean modified)
	{
		boolean oldValue = this.modified;
		this.modified = modified;

		firePropertyChange("modified", oldValue, modified);

		if (oldValue != modified)
		{
			updateTitle();
		}
	}

	/**
	 * 
	 * @return whether or not the current graph has been modified
	 */
	public boolean isModified()
	{
		return modified;
	}

	/**
	 * 
	 */
	public mxGraphComponent getGraphComponent()
	{
		return graphComponent;
	}

	/**
	 * 
	 */
	public void setGraphComponent(mxGraphComponent graphComponent)
	{
		this.graphComponent = graphComponent;
	}
	
	/**
	 * 
	 */
	public PluginContext getContext()
	{
		return context;
	}

	/**
	 * 
	 */
	public void setContext(PluginContext context)
	{
		this.context = context;
	}
	
	/**
	 * create a panel to hold multiple graph components
	 */
	public JTabbedPane getMultipleGraphPanel()
	{
		return multipleGraphPanel;
	}
	
	/**
	 * 
	 */
	public mxGraphOutline getGraphOutline()
	{
		return graphOutline;
	}
	
	/**
	 * 
	 */
	public JTabbedPane getLibraryPane()
	{
		return libraryPane;
	}

	/**
	 * 
	 */
	public mxUndoManager getUndoManager()
	{
		return undoManager;
	}

	/**
	 * 
	 * @param name
	 * @param action
	 * @return a new Action bound to the specified string name
	 */
	public Action bind(String name, final Action action)
	{
		return bind(name, action, null);
	}

	/**
	 * 
	 * @param name
	 * @param action
	 * @return a new Action bound to the specified string name and icon
	 */
	@SuppressWarnings("serial")
	public Action bind(String name, final Action action, String iconUrl)
	{
		AbstractAction newAction = new AbstractAction(name, (iconUrl != null) ? new ImageIcon(
				BasicGraphEditor.class.getResource(iconUrl)) : null)
		{
			public void actionPerformed(ActionEvent e)
			{
				action.actionPerformed(new ActionEvent(getGraphComponent(), e
						.getID(), e.getActionCommand()));
			}
		};
		
		newAction.putValue(Action.SHORT_DESCRIPTION, action.getValue(Action.SHORT_DESCRIPTION));
		
		return newAction;
	}

	/**
	 * 
	 * @param msg
	 */
	public void status(String msg)
	{
		statusBar.setText(msg);
	}

	/**
	 * 
	 */
	public void updateTitle()
	{
		JFrame frame = (JFrame) SwingUtilities.windowForComponent(this);

		if (frame != null)
		{
			String title = (currentFile != null) ? currentFile
					.getAbsolutePath() : mxResources.get("newDiagram");

			if (modified)
			{
				title += "*";
			}

			frame.setTitle(title + " - " + appTitle);
		}
	}

	/**
	 * generate and output an information system (interfaces realized as php files + MYSQL database) based on the designed OCBC model
	 */
	public void generateInformationSystem()
	{
		JFrame frame = (JFrame) SwingUtilities.windowForComponent(this);

		if (frame != null)
		{
			GenerateInformationSystemFrame informationSystem = new GenerateInformationSystemFrame(frame, graphComponent.getGraph());
			informationSystem.setModal(true);

			// Centers inside the application frame
			int x = frame.getX() + (frame.getWidth() - informationSystem.getWidth()) / 2;
			int y = frame.getY() + (frame.getHeight() - informationSystem.getHeight()) / 2;
			informationSystem.setLocation(x, y);

			// Shows the modal dialog and waits
			informationSystem.setVisible(true);
		}
	}
	
	/**
	 *  
	 */
	public void about()
	{
		JFrame frame = (JFrame) SwingUtilities.windowForComponent(this);

		if (frame != null)
		{
			EditorAboutFrame about = new EditorAboutFrame(frame);
			about.setModal(true);

			// Centers inside the application frame
			int x = frame.getX() + (frame.getWidth() - about.getWidth()) / 2;
			int y = frame.getY() + (frame.getHeight() - about.getHeight()) / 2;
			about.setLocation(x, y);

			// Shows the modal dialog and waits
			about.setVisible(true);
		}
	}

	/**
	 * pop up a message box to show the link to the manual
	 * @throws URISyntaxException 
	 */
	public void showManual(){		
		JFrame frame = (JFrame) SwingUtilities.windowForComponent(this);

		if (frame != null)
		{
			EditorManualFrame manual = new EditorManualFrame(frame);
			manual.setModal(true);

			// Centers inside the application frame
			int x = frame.getX() + (frame.getWidth() - manual.getWidth()) / 2;
			int y = frame.getY() + (frame.getHeight() - manual.getHeight()) / 2;
			manual.setLocation(x, y);

			// Shows the modal dialog and waits
			manual.setVisible(true);
		}
	}
	
	private static void open(URI uri) {
        if (Desktop.isDesktopSupported()) {      	
        	try {
        		Desktop.getDesktop().browse(uri);
        	} catch (IOException e) { /* TODO: error handling */ }
        } else { /* TODO: error handling */ }
    }
	
	
	/**
	 * 
	 */
	public void exit()
	{
		JFrame frame = (JFrame) SwingUtilities.windowForComponent(this);

		if (frame != null)
		{
			frame.dispose();
		}
	}

	/**
	 * 
	 */
	public void setLookAndFeel(String clazz)
	{
		JFrame frame = (JFrame) SwingUtilities.windowForComponent(this);

		if (frame != null)
		{
			try
			{
				UIManager.setLookAndFeel(clazz);
				SwingUtilities.updateComponentTreeUI(frame);

				// Needs to assign the key bindings again
				keyboardHandler = new EditorKeyboardHandler(graphComponent);
			}
			catch (Exception e1)
			{
				e1.printStackTrace();
			}
		}
	}

	/**
	 * 
	 */
	public JFrame createFrame(JMenuBar menuBar)
	{
		JFrame frame = new JFrame();
//		CustomGraphComponent graphComponent = new CustomGraphComponent(new CustomGraph());
//		frame.getContentPane().add(graphComponent);
		frame.getContentPane().add(this);		
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.setJMenuBar(menuBar);
//		frame.setSize(870, 640);
		frame.setSize(Toolkit.getDefaultToolkit().getScreenSize());

		// Updates the frame title
		updateTitle();

		return frame;
	}

	@SuppressWarnings("serial")
	public Action myGraphLayout(final String key, boolean animate)
	{
		return new AbstractAction(key)
		{
			public void actionPerformed(ActionEvent e)
			{
				System.out.println("show my layout");
				ModelLayout modelLayout = new ModelLayout(model);
				model = modelLayout.optimizeLayout();				
				Map<String, Pair<Double, Double>> nodeNameVSCoordinateMap = modelLayout.getNodeNameVSCoordinateMap();
					
				model.setModelVersion(OCBCEditorConstants.VERSION2_0);
				System.out.println("\n class node list size: " + model.getAllClasses().size());
				
				TransformBetweenJGraphXAndModel transformer = new TransformBetweenJGraphXAndModel();
				mxGraph graphNew = transformer.transformModelToJGraphX(model, nodeNameVSCoordinateMap);
				mxGraph graph = graphComponent.getGraph();
				graph.removeCells(graph.getChildEdges(graph.getDefaultParent()));
				graph.removeCells(graph.getChildVertices(graph.getDefaultParent()));
				graph.addCells(graphNew.getChildVertices(graphNew.getDefaultParent()));
				graph.addCells(graphNew.getChildEdges(graphNew.getDefaultParent()));
				graph.refresh();
			}
		};
	}
	
	/**
	 * Creates an action that executes the vertical hierarchical layout.
	 * 
	 * @param key Key to be used for getting the label from mxResources and also
	 * to create the layout instance for the commercial graph editor example.
	 * @return an action that executes the specified layout
	 */
	@SuppressWarnings("serial")
	public Action verticalHierarchicalGraphLayout(final String key, boolean animate)
	{
		mxGraph graph = graphComponent.getGraph();
		final mxHierarchicalLayout layout = new mxHierarchicalLayout(graph);
		return new AbstractAction(mxResources.get(key))
		{
			public void actionPerformed(ActionEvent e)
			{
				System.out.println("show verticalHierarchicalGraphLayout");
				final mxGraph graph = graphComponent.getGraph();
				Object cell = graph.getSelectionCell();

				if (cell == null
						|| graph.getModel().getChildCount(cell) == 0)
				{
					cell = graph.getDefaultParent();
				}

				graph.getModel().beginUpdate();
				try
				{
					long t0 = System.currentTimeMillis();
					Object[] childVertices = graph.getChildVertices(cell);
					Object[] childEdges = graph.getChildEdges(cell);
					
					List<Object> childCellList = new ArrayList<Object>();
					for(int i=0; i<childVertices.length; i++){
						childCellList.add(childVertices[i]);
					}
					for(int i=0; i<childEdges.length; i++){
						childCellList.add(childEdges[i]);
					}
					
					layout.execute(cell, childCellList);
					status("Layout: " + (System.currentTimeMillis() - t0)
							+ " ms");
				}
				finally
				{
					mxMorphing morph = new mxMorphing(graphComponent, 20,
							1.2, 20);

					morph.addListener(mxEvent.DONE, new mxIEventListener()
					{

						public void invoke(Object sender, mxEventObject evt)
						{
							graph.getModel().endUpdate();
						}

					});

					morph.startAnimation();
				}
				
				//add the "X" for non-activity-constraint
				Object[] childEdges = graph.getChildEdges(graph.getDefaultParent());
				for(int i=0; i<childEdges.length; i++){
					mxCell edgeCell = (mxCell) childEdges[i];
					String type = edgeCell.getAttributes().get(mxConstants.CARDINALITYTYPE);		
					if(type != null && type.contains("non")){
						mxGeometry geometry = new mxGeometry();
						geometry.setRelative(true);
						geometry.setPoints(edgeCell.getGeometry().getPoints());
						edgeCell.setGeometry(geometry);	
						edgeCell.setValue("x");				
					}
				}
				graph.refresh();
			}
		};
	}
	
	/**
	 * Creates an action that executes the horizontal hierarchical layout.
	 * 
	 * @param key Key to be used for getting the label from mxResources and also
	 * to create the layout instance for the commercial graph editor example.
	 * @return an action that executes the specified layout
	 */
	@SuppressWarnings("serial")
	public Action horizontalHierarchicalGraphLayout(final String key, boolean animate)
	{
		mxGraph graph = graphComponent.getGraph();
		final mxHierarchicalLayout layout = new mxHierarchicalLayout(graph, JLabel.WEST);
		return new AbstractAction(mxResources.get(key))
		{
			public void actionPerformed(ActionEvent e)
			{
				System.out.println("show horizontalHierarchical");
				final mxGraph graph = graphComponent.getGraph();
				Object cell = graph.getDefaultParent();

				graph.getModel().beginUpdate();
				try
				{
					long t0 = System.currentTimeMillis();
					Object[] childVertices = graph.getChildVertices(cell);
					Object[] childEdges = graph.getChildEdges(cell);
					
					List<Object> childCellList = new ArrayList<Object>();
					for(int i=0; i<childVertices.length; i++){
						childCellList.add(childVertices[i]);
					}
					for(int i=0; i<childEdges.length; i++){
						childCellList.add(childEdges[i]);
					}
					
					layout.execute(cell, childCellList);
					status("Layout: " + (System.currentTimeMillis() - t0)
							+ " ms");
				}
				finally
				{
//					mxMorphing morph = new mxMorphing(graphComponent, 20,
//							1.2, 20);
//
//					morph.addListener(mxEvent.DONE, new mxIEventListener()
//					{
//
//						public void invoke(Object sender, mxEventObject evt)
//						{
//							graph.getModel().endUpdate();
//						}
//
//					});
//
//					morph.startAnimation();
				}
				
				//add the "X" for non-activity-constraint
				Object[] childEdges = graph.getChildEdges(graph.getDefaultParent());
				for(int i=0; i<childEdges.length; i++){
					mxCell edgeCell = (mxCell) childEdges[i];
					String type = edgeCell.getAttributes().get(mxConstants.CARDINALITYTYPE);		
					if(type != null && type.contains("non")){
						mxGeometry geometry = new mxGeometry();
						geometry.setRelative(true);
						geometry.setPoints(edgeCell.getGeometry().getPoints());
						edgeCell.setGeometry(geometry);	
						edgeCell.setValue("x");
					}
				}

				System.out.println("graph.refresh() in horizontalHierarchicalGraphLayout");

				graph.refresh();



			}
		};
	}
	
	/**
	 * Creates an action that executes the specified layout.
	 * 
	 * @param key Key to be used for getting the label from mxResources and also
	 * to create the layout instance for the commercial graph editor example.
	 * @return an action that executes the specified layout
	 */
	@SuppressWarnings("serial")
	public Action graphLayout(final String key, boolean animate)
	{
		final mxIGraphLayout layout = createLayout(key, animate);

		if (layout != null)
		{
			return new AbstractAction(mxResources.get(key))
			{
				public void actionPerformed(ActionEvent e)
				{
					System.out.println("show graphLayout");
					final mxGraph graph = graphComponent.getGraph();
					Object cell = graph.getSelectionCell();

					if (cell == null
							|| graph.getModel().getChildCount(cell) == 0)
					{
						cell = graph.getDefaultParent();
					}

					graph.getModel().beginUpdate();
					try
					{
						long t0 = System.currentTimeMillis();
						layout.execute(cell);
						status("Layout: " + (System.currentTimeMillis() - t0)
								+ " ms");
					}
					finally
					{
						mxMorphing morph = new mxMorphing(graphComponent, 20,
								1.2, 20);

						morph.addListener(mxEvent.DONE, new mxIEventListener()
						{

							public void invoke(Object sender, mxEventObject evt)
							{
								graph.getModel().endUpdate();
							}

						});

						morph.startAnimation();
					}
					graph.refresh();
				}

			};
		}
		else
		{
			return new AbstractAction(mxResources.get(key))
			{

				public void actionPerformed(ActionEvent e)
				{
					JOptionPane.showMessageDialog(graphComponent,
							mxResources.get("noLayout"));
				}

			};
		}
	}

	/**
	 * Creates a layout instance for the given identifier.
	 */
	protected mxIGraphLayout createLayout(String ident, boolean animate)
	{
		mxIGraphLayout layout = null;

		if (ident != null)
		{
			mxGraph graph = graphComponent.getGraph();

			if (ident.equals("verticalHierarchical"))
			{
				layout = new mxHierarchicalLayout(graph);
			}
			else if (ident.equals("horizontalHierarchical"))
			{
				layout = new mxHierarchicalLayout(graph, JLabel.WEST);
			}
			else if (ident.equals("verticalTree"))
			{
				layout = new mxCompactTreeLayout(graph, false);
			}
			else if (ident.equals("horizontalTree"))
			{
				layout = new mxCompactTreeLayout(graph, true);
			}
			else if (ident.equals("parallelEdges"))
			{
				layout = new mxParallelEdgeLayout(graph);
			}
			else if (ident.equals("placeEdgeLabels"))
			{
				layout = new mxEdgeLabelLayout(graph);
			}
			else if (ident.equals("organicLayout"))
			{
				layout = new mxOrganicLayout(graph);
			}
			if (ident.equals("verticalPartition"))
			{
				layout = new mxPartitionLayout(graph, false)
				{
					/**
					 * Overrides the empty implementation to return the size of the
					 * graph control.
					 */
					public mxRectangle getContainerSize()
					{
						return graphComponent.getLayoutAreaSize();
					}
				};
			}
			else if (ident.equals("horizontalPartition"))
			{
				layout = new mxPartitionLayout(graph, true)
				{
					/**
					 * Overrides the empty implementation to return the size of the
					 * graph control.
					 */
					public mxRectangle getContainerSize()
					{
						return graphComponent.getLayoutAreaSize();
					}
				};
			}
			else if (ident.equals("verticalStack"))
			{
				layout = new mxStackLayout(graph, false)
				{
					/**
					 * Overrides the empty implementation to return the size of the
					 * graph control.
					 */
					public mxRectangle getContainerSize()
					{
						return graphComponent.getLayoutAreaSize();
					}
				};
			}
			else if (ident.equals("horizontalStack"))
			{
				layout = new mxStackLayout(graph, true)
				{
					/**
					 * Overrides the empty implementation to return the size of the
					 * graph control.
					 */
					public mxRectangle getContainerSize()
					{
						return graphComponent.getLayoutAreaSize();
					}
				};
			}
			else if (ident.equals("circleLayout"))
			{
				layout = new mxCircleLayout(graph);
			}
		}

		return layout;
	}

	public void setOCBCModel(OCBCModel model){
		this.model = model;;
	}
	
	public OCBCModel getOCBCModel(){
		return model;
	}
}
