package org.processmining.mxgraph.examples.swing.editor;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Desktop;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.Frame;
import java.awt.GradientPaint;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JEditorPane;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRootPane;
import javax.swing.KeyStroke;
import javax.swing.event.HyperlinkEvent;
import javax.swing.event.HyperlinkListener;

import org.processmining.mxgraph.view.mxGraph;

public class EditorAboutFrame extends JDialog
{

	/**
	 * 
	 */
	private static final long serialVersionUID = -3378029138434324390L;
	/**
	 * modified by Guangming Li on June 14,2017
	 */
	public EditorAboutFrame(Frame owner)
	{
		super(owner);
		setTitle("About OCBC Model Editor");
		setLayout(new BorderLayout());

		// Creates the gradient panel
		JPanel panel = new JPanel(new BorderLayout())
		{

			/**
			 * 
			 */
			private static final long serialVersionUID = -5062895855016210947L;

			/**
			 * 
			 */
			public void paintComponent(Graphics g)
			{
				super.paintComponent(g);

				// Paint gradient background
				Graphics2D g2d = (Graphics2D) g;
				g2d.setPaint(new GradientPaint(0, 0, Color.WHITE, getWidth(),
						0, getBackground()));
				g2d.fillRect(0, 0, getWidth(), getHeight());
			}

		};

		panel.setBorder(BorderFactory.createCompoundBorder(BorderFactory
				.createMatteBorder(0, 0, 1, 0, Color.GRAY), BorderFactory
				.createEmptyBorder(8, 8, 12, 8)));

		// Adds title
		JLabel titleLabel = new JLabel("About OCBC Model Editor");
		titleLabel.setFont(titleLabel.getFont().deriveFont(Font.BOLD));
		titleLabel.setBorder(BorderFactory.createEmptyBorder(4, 0, 0, 0));
		titleLabel.setOpaque(false);
		panel.add(titleLabel, BorderLayout.NORTH);

		// Adds introduction with a link to the OCBC website
		JEditorPane jep = new JEditorPane("text/html", "This software is used to edit and output an OCBC model. "
				+ "It refers to the mxGraph editor and is an open source software. "
				+ "The source codes are available in the OCBC package at <a href='https://svn.win.tue.nl/repos/prom/Packages/OCBC/'>"
			  	+ "https://svn.win.tue.nl/repos/prom/Packages/OCBC/</a>. "
				+ "For more information, please visit <a href='http://www.win.tue.nl/ocbc/'>"
			  	+ "http://www.win.tue.nl/ocbc/</a>.");
		jep.setEditable(false);
		jep.setOpaque(false);
		jep.addHyperlinkListener(new HyperlinkListener() {						
			public void hyperlinkUpdate(HyperlinkEvent hle) {				
			    if (HyperlinkEvent.EventType.ACTIVATED.equals(hle.getEventType())) {
			    	if (Desktop.isDesktopSupported()) {
			    		try {
			  	            try {
								Desktop.getDesktop().browse(new URI(hle.getURL().toString()));
							} catch (URISyntaxException e) {
								// TODO Auto-generated catch block
								e.printStackTrace();
							}
			  	          } catch (IOException e) { /* TODO: error handling */ }			  	        		    					  	          
			  	  	 }
			    }
			}
		});
		panel.add(jep, BorderLayout.CENTER);  		  
		getContentPane().add(panel, BorderLayout.NORTH);

		JPanel content = new JPanel();
		content.setLayout(new BoxLayout(content, BoxLayout.Y_AXIS));
		content.setBorder(BorderFactory.createEmptyBorder(12, 12, 12, 12));

		content.add(new JLabel("This Editor is based on JGraph X - The Swing Portion of mxGraph"));
		content.add(new JLabel(" "));

		content.add(new JLabel("Editor Version 2.0"));
		content.add(new JLabel("mxGraph Version " + mxGraph.VERSION));
		content.add(new JLabel("Copyright (C) 2017 by Guangming Li and JGraph Ltd."));
		content.add(new JLabel("Email: g.li.3@tue.nl"));
		content.add(new JLabel("All rights reserved."));
		content.add(new JLabel(" "));
		
		
		try
		{
			content.add(new JLabel("Operating System Name: "
					+ System.getProperty("os.name")));
			content.add(new JLabel("Operating System Version: "
					+ System.getProperty("os.version")));
			content.add(new JLabel(" "));

			content.add(new JLabel("Java Vendor: "
					+ System.getProperty("java.vendor", "undefined")));
			content.add(new JLabel("Java Version: "
					+ System.getProperty("java.version", "undefined")));
			content.add(new JLabel(" "));

			content.add(new JLabel("Total Memory: "
					+ Runtime.getRuntime().totalMemory()));
			content.add(new JLabel("Free Memory: "
					+ Runtime.getRuntime().freeMemory()));
		}
		catch (Exception e)
		{
			// ignore
		}

		getContentPane().add(content, BorderLayout.CENTER);

		JPanel buttonPanel = new JPanel(new FlowLayout(FlowLayout.RIGHT));
		buttonPanel.setBorder(BorderFactory.createCompoundBorder(BorderFactory
				.createMatteBorder(1, 0, 0, 0, Color.GRAY), BorderFactory
				.createEmptyBorder(16, 8, 8, 8)));
		getContentPane().add(buttonPanel, BorderLayout.SOUTH);

		// Adds OK button to close window
		JButton closeButton = new JButton("Close");
		closeButton.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				setVisible(false);
			}
		});

		buttonPanel.add(closeButton);

		// Sets default button for enter key
		getRootPane().setDefaultButton(closeButton);

		setResizable(false);
		setSize(500, 500);
	}

	
	/**
	 * Overrides {@link JDialog#createRootPane()} to return a root pane that
	 * hides the window when the user presses the ESCAPE key.O
	 */
	protected JRootPane createRootPane()
	{
		KeyStroke stroke = KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0);
		JRootPane rootPane = new JRootPane();
		rootPane.registerKeyboardAction(new ActionListener()
		{
			public void actionPerformed(ActionEvent actionEvent)
			{
				setVisible(false);
			}
		}, stroke, JComponent.WHEN_IN_FOCUSED_WINDOW);
		return rootPane;
	}

}
