/*
 * Copyright (c) 2001-2012, JGraph Ltd
 */
package org.processmining.mxgraph.examples.swing.editor;

import java.awt.Color;
import java.awt.Component;
import java.awt.Image;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import java.awt.print.PageFormat;
import java.awt.print.Paper;
import java.awt.print.PrinterException;
import java.awt.print.PrinterJob;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FilenameFilter;
import java.io.IOException;
import java.lang.reflect.Method;
import java.net.URLDecoder;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.swing.AbstractAction;
import javax.swing.ImageIcon;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JColorChooser;
import javax.swing.JComponent;
import javax.swing.JEditorPane;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JSplitPane;
import javax.swing.JTabbedPane;
import javax.swing.SwingUtilities;
import javax.swing.filechooser.FileFilter;
import javax.swing.text.html.HTML;
import javax.swing.text.html.HTMLDocument;
import javax.swing.text.html.HTMLEditorKit;

import org.processmining.common.file.SVG2PDF;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.mxgraph.analysis.mxDistanceCostFunction;
import org.processmining.mxgraph.analysis.mxGraphAnalysis;
import org.processmining.mxgraph.canvas.mxGraphics2DCanvas;
import org.processmining.mxgraph.canvas.mxICanvas;
import org.processmining.mxgraph.canvas.mxSvgCanvas;
import org.processmining.mxgraph.examples.swing.GraphEditor.CustomGraph;
import org.processmining.mxgraph.examples.swing.GraphEditor.CustomGraphComponent;
import org.processmining.mxgraph.examples.swing.io.TransformToOCBCSemanticModel;
import org.processmining.mxgraph.examples.swing.panels.GenerateInformationSystemFrame;
import org.processmining.mxgraph.examples.swing.visualization.Style;
import org.processmining.mxgraph.io.mxCodec;
import org.processmining.mxgraph.io.mxGdCodec;
import org.processmining.mxgraph.model.mxCell;
import org.processmining.mxgraph.model.mxGraphModel;
import org.processmining.mxgraph.model.mxIGraphModel;
import org.processmining.mxgraph.shape.mxStencilShape;
import org.processmining.mxgraph.swing.mxGraphComponent;
import org.processmining.mxgraph.swing.mxGraphOutline;
import org.processmining.mxgraph.swing.handler.mxConnectionHandler;
import org.processmining.mxgraph.swing.util.mxGraphActions;
import org.processmining.mxgraph.swing.view.mxCellEditor;
import org.processmining.mxgraph.util.mxCellRenderer;
import org.processmining.mxgraph.util.mxCellRenderer.CanvasFactory;
import org.processmining.mxgraph.util.mxConstants;
import org.processmining.mxgraph.util.mxDomUtils;
import org.processmining.mxgraph.util.mxResources;
import org.processmining.mxgraph.util.mxUtils;
import org.processmining.mxgraph.util.mxXmlUtils;
import org.processmining.mxgraph.util.png.mxPngEncodeParam;
import org.processmining.mxgraph.util.png.mxPngImageEncoder;
import org.processmining.mxgraph.util.png.mxPngTextDecoder;
import org.processmining.mxgraph.view.mxGraph;
import org.processmining.ocbc.models.OCBCModel;
import org.processmining.ocbc.models.editor.plugins.OCBCEditorConstants;
import org.w3c.dom.Document;
import org.w3c.dom.Node;

/**
 *
 */
public class EditorActions
{
	/**
	 * 
	 * @param e
	 * @return Returns the graph for the given action event.
	 */
	public static final BasicGraphEditor getEditor(ActionEvent e)
	{
		if (e.getSource() instanceof Component)
		{
			Component component = (Component) e.getSource();

			while (component != null
					&& !(component instanceof BasicGraphEditor))
			{
				component = component.getParent();
			}

			return (BasicGraphEditor) component;
		}

		return null;
	}

	
	/**
	 *
	 */
	@SuppressWarnings("serial")
	public static class ToggleRulersItem extends JCheckBoxMenuItem
	{
		/**
		 * 
		 */
		public ToggleRulersItem(final BasicGraphEditor editor, String name)
		{
			super(name);
			setSelected(editor.getGraphComponent().getColumnHeader() != null);

			addActionListener(new ActionListener()
			{
				/**
				 * 
				 */
				public void actionPerformed(ActionEvent e)
				{
					mxGraphComponent graphComponent = editor
							.getGraphComponent();

					if (graphComponent.getColumnHeader() != null)
					{
						graphComponent.setColumnHeader(null);
						graphComponent.setRowHeader(null);
					}
					else
					{
						graphComponent.setColumnHeaderView(new EditorRuler(
								graphComponent,
								EditorRuler.ORIENTATION_HORIZONTAL));
						graphComponent.setRowHeaderView(new EditorRuler(
								graphComponent,
								EditorRuler.ORIENTATION_VERTICAL));
					}
				}
			});
		}
	}

	/**
	 *
	 */
	@SuppressWarnings("serial")
	public static class ToggleGridItem extends JCheckBoxMenuItem
	{
		/**
		 * 
		 */
		public ToggleGridItem(final BasicGraphEditor editor, String name)
		{
			super(name);
			setSelected(true);

			addActionListener(new ActionListener()
			{
				/**
				 * 
				 */
				public void actionPerformed(ActionEvent e)
				{
//					mxGraphComponent graphComponent = editor
//							.getGraphComponent();
//					mxGraph graph = graphComponent.getGraph();
//					boolean enabled = !graph.isGridEnabled();
//
//					graph.setGridEnabled(enabled);
//					graphComponent.setGridVisible(enabled);
//					graphComponent.repaint();
//					setSelected(enabled);
					
					for(int i=0; i<editor.getMultipleGraphPanel().getComponentCount(); i++){
						mxGraphComponent graphComponent = (mxGraphComponent) editor.getMultipleGraphPanel().getComponentAt(i);
						mxGraph graph = graphComponent.getGraph();
						boolean enabled = !graph.isGridEnabled();

						graph.setGridEnabled(enabled);
						graphComponent.setGridVisible(enabled);
						graphComponent.repaint();
						setSelected(enabled);
					}					
				}
			});
		}
	}

	/**
	 *
	 */
	@SuppressWarnings("serial")
	public static class ToggleOutlineItem extends JCheckBoxMenuItem
	{
		/**
		 * 
		 */
		public ToggleOutlineItem(final BasicGraphEditor editor, String name)
		{
			super(name);
			setSelected(true);

			addActionListener(new ActionListener()
			{
				/**
				 * 
				 */
				public void actionPerformed(ActionEvent e)
				{
					final mxGraphOutline outline = editor.getGraphOutline();
					outline.setVisible(!outline.isVisible());
					outline.revalidate();

					SwingUtilities.invokeLater(new Runnable()
					{
						/*
						 * (non-Javadoc)
						 * @see java.lang.Runnable#run()
						 */
						public void run()
						{
							if (outline.getParent() instanceof JSplitPane)
							{
								if (outline.isVisible())
								{
									((JSplitPane) outline.getParent())
											.setDividerLocation(editor
													.getHeight() - 300);
									((JSplitPane) outline.getParent())
											.setDividerSize(6);
								}
								else
								{
									((JSplitPane) outline.getParent())
											.setDividerSize(0);
								}
							}
						}
					});
				}
			});
		}
	}

	/**
	 *
	 */
	@SuppressWarnings("serial")
	public static class ExitAction extends AbstractAction
	{
		/**
		 * 
		 */
		public void actionPerformed(ActionEvent e)
		{
			BasicGraphEditor editor = getEditor(e);

			if (editor != null)
			{
				editor.exit();
			}
		}
	}

	/**
	 *
	 */
	@SuppressWarnings("serial")
	public static class StylesheetAction extends AbstractAction
	{
		/**
		 * 
		 */
		protected String stylesheet;

		/**
		 * 
		 */
		public StylesheetAction(String stylesheet)
		{
			this.stylesheet = stylesheet;
		}

		/**
		 * 
		 */
		public void actionPerformed(ActionEvent e)
		{
			if (e.getSource() instanceof mxGraphComponent)
			{
				mxGraphComponent graphComponent = (mxGraphComponent) e
						.getSource();
				mxGraph graph = graphComponent.getGraph();
				mxCodec codec = new mxCodec();
				Document doc = mxUtils.loadDocument(EditorActions.class
						.getResource(stylesheet).toString());

				if (doc != null)
				{
					codec.decode(doc.getDocumentElement(),
							graph.getStylesheet());
					graph.refresh();
				}
			}
		}
	}

	/**
	 *
	 */
	@SuppressWarnings("serial")
	public static class ZoomPolicyAction extends AbstractAction
	{
		/**
		 * 
		 */
		protected int zoomPolicy;

		/**
		 * 
		 */
		public ZoomPolicyAction(int zoomPolicy)
		{
			this.zoomPolicy = zoomPolicy;
		}

		/**
		 * 
		 */
		public void actionPerformed(ActionEvent e)
		{
			if (e.getSource() instanceof mxGraphComponent)
			{
				mxGraphComponent graphComponent = (mxGraphComponent) e
						.getSource();
				graphComponent.setPageVisible(true);
				graphComponent.setZoomPolicy(zoomPolicy);
			}
		}
	}

	/**
	 *
	 */
	@SuppressWarnings("serial")
	public static class GridStyleAction extends AbstractAction
	{
		/**
		 * 
		 */
		protected int style;

		/**
		 * 
		 */
		public GridStyleAction(int style)
		{
			this.style = style;
		}

		/**
		 * 
		 */
		public void actionPerformed(ActionEvent e)
		{
			if (e.getSource() instanceof mxGraphComponent)
			{
				mxGraphComponent graphComponent = (mxGraphComponent) e
						.getSource();
				graphComponent.setGridStyle(style);
				graphComponent.repaint();
			}
		}
	}

	/**
	 *
	 */
	@SuppressWarnings("serial")
	public static class GridColorAction extends AbstractAction
	{
		/**
		 * 
		 */
		public void actionPerformed(ActionEvent e)
		{
			if (e.getSource() instanceof mxGraphComponent)
			{
				mxGraphComponent graphComponent = (mxGraphComponent) e
						.getSource();
				Color newColor = JColorChooser.showDialog(graphComponent,
						mxResources.get("gridColor"),
						graphComponent.getGridColor());

				if (newColor != null)
				{
					graphComponent.setGridColor(newColor);
					graphComponent.repaint();
				}
			}
		}
	}

	/**
	 *
	 */
	@SuppressWarnings("serial")
	public static class ScaleAction extends AbstractAction
	{
		/**
		 * 
		 */
		protected double scale;

		/**
		 * 
		 */
		public ScaleAction(double scale)
		{
			this.scale = scale;
		}

		/**
		 * 
		 */
		public void actionPerformed(ActionEvent e)
		{
			if (e.getSource() instanceof mxGraphComponent)
			{
				mxGraphComponent graphComponent = (mxGraphComponent) e
						.getSource();
				double scale = this.scale;

				if (scale == 0)
				{
					String value = (String) JOptionPane.showInputDialog(
							graphComponent, mxResources.get("value"),
							mxResources.get("scale") + " (%)",
							JOptionPane.PLAIN_MESSAGE, null, null, "");

					if (value != null)
					{
						scale = Double.parseDouble(value.replace("%", "")) / 100;
					}
				}

				if (scale > 0)
				{
					graphComponent.zoomTo(scale, graphComponent.isCenterZoom());
				}
			}
		}
	}

	/**
	 *
	 */
	@SuppressWarnings("serial")
	public static class PageSetupAction extends AbstractAction
	{
		/**
		 * 
		 */
		public void actionPerformed(ActionEvent e)
		{
			if (e.getSource() instanceof mxGraphComponent)
			{
				mxGraphComponent graphComponent = (mxGraphComponent) e
						.getSource();
				PrinterJob pj = PrinterJob.getPrinterJob();
				PageFormat format = pj.pageDialog(graphComponent
						.getPageFormat());

				if (format != null)
				{
					graphComponent.setPageFormat(format);
					graphComponent.zoomAndCenter();
				}
			}
		}
	}

	/**
	 *
	 */
	@SuppressWarnings("serial")
	public static class PrintAction extends AbstractAction
	{
		/**
		 * 
		 */
		public void actionPerformed(ActionEvent e)
		{
			if (e.getSource() instanceof mxGraphComponent)
			{
				mxGraphComponent graphComponent = (mxGraphComponent) e
						.getSource();
				PrinterJob pj = PrinterJob.getPrinterJob();

				if (pj.printDialog())
				{
					PageFormat pf = graphComponent.getPageFormat();
					Paper paper = new Paper();
					double margin = 36;
					paper.setImageableArea(margin, margin, paper.getWidth()
							- margin * 2, paper.getHeight() - margin * 2);
					pf.setPaper(paper);
					pj.setPrintable(graphComponent, pf);

					try
					{
						pj.print();
					}
					catch (PrinterException e2)
					{
						System.out.println(e2);
					}
				}
			}
		}
	}

	/**
	 *
	 */
	@SuppressWarnings("serial")
	public static class SaveAction extends AbstractAction
	{
		/**
		 * 
		 */
		protected boolean showDialog;

		/**
		 * 
		 */
		protected String lastDir = null;

		/**
		 * 
		 */
		public SaveAction(boolean showDialog)
		{
			this.showDialog = showDialog;
		}

		/**
		 * Saves XML+PNG format.
		 */
		protected void saveXmlPng(BasicGraphEditor editor, String filename,
				Color bg) throws IOException
		{
			mxGraphComponent graphComponent = editor.getGraphComponent();
			mxGraph graph = graphComponent.getGraph();

			// Creates the image for the PNG file
			BufferedImage image = mxCellRenderer.createBufferedImage(graph,
					null, 1, bg, graphComponent.isAntiAlias(), null,
					graphComponent.getCanvas());

			// Creates the URL-encoded XML data
			mxCodec codec = new mxCodec();
			String xml = URLEncoder.encode(
					mxXmlUtils.getXml(codec.encode(graph.getModel())), "UTF-8");
			mxPngEncodeParam param = mxPngEncodeParam
					.getDefaultEncodeParam(image);
			param.setCompressedText(new String[] { "mxGraphModel", xml });

			// Saves as a PNG file
			FileOutputStream outputStream = new FileOutputStream(new File(
					filename));
			try
			{
				mxPngImageEncoder encoder = new mxPngImageEncoder(outputStream,
						param);

				if (image != null)
				{
					encoder.encode(image);

					editor.setModified(false);
					editor.setCurrentFile(new File(filename));
				}
				else
				{
					JOptionPane.showMessageDialog(graphComponent,
							mxResources.get("noImageData"));
				}
			}
			finally
			{
				outputStream.close();
			}
		}

		/**
		 * 
		 */
		public void actionPerformed(ActionEvent e)
		{
			BasicGraphEditor editor = getEditor(e);
			if (editor != null)
			{
				mxGraphComponent graphComponent = editor.getGraphComponent();
//				mxGraphComponent graphComponent = (mxGraphComponent) editor.getMultipleGraphPanel().getSelectedComponent();
				mxGraph graph = graphComponent.getGraph();
				FileFilter selectedFilter = null;
				
				
				DefaultFileFilter oCBCModelFilter = new DefaultFileFilter(".ocbcmml",
						"OCBC Model File " + " (.ocbcmml)");

				String filename = null;
				String pureName = null;
				boolean dialogShown = false;

				
//				File currentFile = editor.getGraphComponentFile(graphComponent);
				if (showDialog || editor.getCurrentFile() == null)
				{
					String wd;
					System.out.println("e == null");
					if (lastDir != null)
					{
						wd = lastDir;
					}
					else if (editor.getCurrentFile() != null)
					{
						wd = editor.getCurrentFile().getParent();
					}
					else
					{
						wd = System.getProperty("user.dir");
					}

					JFileChooser fc = new JFileChooser(wd);

					
					
					// Adds the default file format					
					FileFilter defaultFilter = oCBCModelFilter;
										
					fc.addChoosableFileFilter(defaultFilter);					
					fc.addChoosableFileFilter( new DefaultFileFilter(".xml",
							"OCBC Table Model (.xml)"));
					
					// Adds special vector graphics formats
					fc.addChoosableFileFilter(new DefaultFileFilter(".svg",
							"SVG " + mxResources.get("file") + " (.svg)"));

					// Adds special vector graphics formats
					fc.addChoosableFileFilter(new DefaultFileFilter(".pdf",
							"PDF " + mxResources.get("file") + " (.pdf)"));

					// Adds filter that accepts all supported image formats
					fc.setFileFilter(defaultFilter);
					int rc = fc.showDialog(null, mxResources.get("save"));
					dialogShown = true;

					if (rc != JFileChooser.APPROVE_OPTION)
					{
						return;
					}
					else
					{
						lastDir = fc.getSelectedFile().getParent();
					}
					
					//get name
					pureName = fc.getSelectedFile().getName() + "     ";						
					
					filename = fc.getSelectedFile().getAbsolutePath();
					selectedFilter = fc.getFileFilter();

					if (selectedFilter instanceof DefaultFileFilter)
					{
						String ext = ((DefaultFileFilter) selectedFilter)
								.getExtension();

						if (!filename.toLowerCase().endsWith(ext))
						{
							filename += ext;
						}
					}

					if (new File(filename).exists()
							&& JOptionPane.showConfirmDialog(graphComponent,
									mxResources.get("overwriteExistingFile")) != JOptionPane.YES_OPTION)
					{
						return;
					}
				}
				else
				{
					filename = editor.getCurrentFile().getAbsolutePath();
				}

				 
				
//				String filename = "d:\\newOCBC.ocbcmml";
//				String pureName = null;
			
				
				try
				{
					String ext = filename.substring(filename.lastIndexOf('.') + 1);
				
					if(ext.equalsIgnoreCase("ocbcmml")){
						mxCodec codec = new mxCodec();
//						graph.getModel().get
						Node modelNode = codec.encode(graph.getModel());
						System.out.println("model node name: " + modelNode.getNodeName());
						System.out.println("getChildNodes().getLength(): " + modelNode.getChildNodes().getLength());
						
						for(int i=0; i< modelNode.getChildNodes().getLength(); i++){
							Node childNode = modelNode.getChildNodes().item(i);
							System.out.println("model childNode name: " + childNode.getNodeName());
							for(int j=0; j< childNode.getChildNodes().getLength(); j++){
								Node childNode2 = childNode.getChildNodes().item(j);
								System.out.println("model childNode2 name: " + childNode2.getNodeName());
								for(int n=0; n< childNode2.getChildNodes().getLength(); n++){
									Node childNode3 = childNode2.getChildNodes().item(n);
									System.out.println("model childNode3 name: " + childNode3.getNodeName());
									
								}
							}
						}
						String xml = mxXmlUtils.getXml(codec.encode(graph.getModel()));
						System.out.println("xml: " + xml);
						//add the ocbc version to the file
						String xmlPlusVersion = "<?" + OCBCEditorConstants.OCBCVERSION + "=\"" + OCBCEditorConstants.VERSION2_0+ "\"?>\n" + xml;
						mxUtils.writeFile(xmlPlusVersion, filename);

						editor.setModified(false);
						editor.setCurrentFile(new File(filename));
						
						//save the model in the workspace if the context is not null
						PluginContext context = editor.getContext();
						if(context != null){
							System.out.println("output to the platform");
							TransformToOCBCSemanticModel transformer = new TransformToOCBCSemanticModel();
							mxGraph graphX = editor.getGraphComponent().getGraph();
							transformer.transform(graphX);
							OCBCModel model = transformer.getModel();
							model.setGraphX(graphX);
							model.setModelVersion(OCBCEditorConstants.VERSION2_0);
							model.setModelString(xml);		
							String pureFileName = filename.substring(filename.lastIndexOf("\\")+1);
							context.getProvidedObjectManager().createProvidedObject(pureFileName,
					        		model, OCBCModel.class, context);
					        if (context instanceof UIPluginContext) {
					               UIPluginContext uiPluginContext = (UIPluginContext) context;
					               uiPluginContext.getGlobalContext().getResourceManager().getResourceForInstance(model)
					                             .setFavorite(true);
					        }	
					        
						}
						//change name in multiple graph panel						
						JTabbedPane multipleGraphPanel = editor.getMultipleGraphPanel();					
						int componentIndex = multipleGraphPanel.indexOfComponent(editor.getGraphComponent());
						if(pureName == null){
							pureName = multipleGraphPanel.getTitleAt(componentIndex);
						}
						if(pureName.contains("*")){
							//remove "*" into the name to indicate the file is modified and unsaved
							pureName = pureName.replace("*", "");
						}
						
						multipleGraphPanel.setTitleAt(componentIndex, pureName);												
						editor.setGraphComponentFile(editor.getGraphComponent(), editor.getCurrentFile());	
					}
					//output OCBC Table Model (configuration file for log generator)
					else if (ext.equalsIgnoreCase("xml")){						
						Map<String, String> tableVSClass = new HashMap<String, String>();
						Map<String, Map<String, String>> tableVSChildTableForeignKey = new HashMap<String, Map<String, String>>();
						Map<String, List<String>> tableVSFatherTables = new HashMap<String, List<String>>();
						Map<String, List<String>> tableColumnVSImpactedTableColumns = new HashMap<String, List<String>>();
						Map<String, List<String>> activityVSClasses = new HashMap<String, List<String>>();
						Map<String, Map<String, String>> classVSActivityColumnVSName = new HashMap<String, Map<String, String>>();
						
						Object[] cells = graph.getChildCells(graph.getDefaultParent());
						for(int i=0; i<cells.length; i++){
							System.out.println("***********index: " + i);
							mxCell cell = (mxCell) cells[i];
							String cellType = cell.getAttributes().get(mxConstants.TYPE);
							System.out.println("cellType: "  + cellType);
							if(cellType.equals(mxConstants.CLASSCELL)){
								String tableName = cell.getAttributes().get("table").split("-")[0];//e.g., table-necessary
								String className = (String) cell.getValue();
								String classNameSequence;
								if(tableVSClass.containsKey(tableName)){
									classNameSequence = tableVSClass.get(tableName);
									if(classNameSequence.equals("")){
										classNameSequence = className;
									}
									else{
										classNameSequence = classNameSequence + "-" + className;
									}
									
								}
								else{
									classNameSequence = className;
								}
								tableVSClass.put(tableName, classNameSequence);
							}
							else if(cellType.equals(mxConstants.CLASSRELATION)){
								mxCell cellSource = (mxCell) cell.getSource();
								mxCell cellTarget = (mxCell) cell.getTarget();
								String tableNameSource = cellSource.getAttributes().get("table").split("-")[0];//e.g., table-necessary
								String tableNameTarget = cellTarget.getAttributes().get("table").split("-")[0];
								String fk = (String) cell.getValue();
								System.out.println("tableNameSource: "  + tableNameSource);
								System.out.println("tableNameTarget: "  + tableNameTarget);
								System.out.println("fk: " + fk);
								if(!tableVSChildTableForeignKey.containsKey(tableNameSource)){
									tableVSChildTableForeignKey.put(tableNameSource, new HashMap<String, String>());
								}
								tableVSChildTableForeignKey.get(tableNameSource).put(tableNameTarget, fk);
								
								if(!tableVSFatherTables.containsKey(tableNameTarget)){
									tableVSFatherTables.put(tableNameTarget, new ArrayList<String>());
								}
								if(!tableVSFatherTables.get(tableNameTarget).contains(tableNameSource)){
									tableVSFatherTables.get(tableNameTarget).add(tableNameSource);
								}
								
							}
							else if(cellType.equals(mxConstants.AOCRELATION)){
								mxCell cellActivity = (mxCell) cell.getSource();
								mxCell cellClass = (mxCell) cell.getTarget();

								//add the new reference relation into the variables(create activity vs class table mapping)	
								String activityName = (String) cellActivity.getValue();
								String className = (String) cellClass.getValue();
								
								System.out.println("activityName: " + activityName);
								System.out.println("className: " + className);
								
								if(!activityVSClasses.containsKey(activityName)){
									activityVSClasses.put(activityName, new ArrayList<String>());
								}
								if(!activityVSClasses.get(activityName).contains(className)){
									activityVSClasses.get(activityName).add(className); 
								}				
								
								String tableNameActivity = cellActivity.getAttributes().get("table").split("-")[0];//e.g., table-necessary
								String tableNameClass = cellClass.getAttributes().get("table").split("-")[0];							
								String activityColumn = (String) cell.getValue();
								
								System.out.println("tableNameActivity: " + tableNameActivity);
								System.out.println("tableNameClass: " + tableNameClass);
								System.out.println("className: " + className);
								//mapping activity column to activity name
								if(tableNameActivity.equals(tableNameClass)){//indicate the table is the source table of activity
									System.out.println("source table");
									if(!classVSActivityColumnVSName.containsKey(className)){
										classVSActivityColumnVSName.put(className, new HashMap<String, String>());
									}
									classVSActivityColumnVSName.get(className).put(activityColumn, activityName);

								}
								
								String tableName = tableNameActivity;
								String activityField = activityColumn;
								String impactingTable = tableNameClass;
								String impactingField = activityColumn;
								String impactingIntermediateKey = null;
								System.out.println("tableName: " + tableName);
								System.out.println("activityField: " + activityField);
								System.out.println("impactingTable: " + impactingTable);
								System.out.println("impactingField: " + impactingField);
								//the activity impacting columns from the same table
								if(activityColumn != null && !activityColumn.equals("")){
									if(tableName.equals(impactingTable)){
										impactingIntermediateKey = "rowid";
									}
									else{
										//impacting columns from the child table
										if(tableVSFatherTables.containsKey(impactingTable) && tableVSFatherTables.get(impactingTable).contains(tableName)){
											System.out.println("impacting columns from the child table");
											impactingIntermediateKey = activityColumn;
										}
										//impacting columns from the father table
										else if(tableVSFatherTables.containsKey(tableName) && tableVSFatherTables.get(tableName).contains(impactingTable)){
											impactingIntermediateKey = activityColumn;
											System.out.println("impacting columns from the father table");
										}
									}
									String activityKey = tableName + "-" + activityField;
									String impactedValue = impactingTable + "-" + impactingField + "-" + impactingIntermediateKey;
									if(!tableColumnVSImpactedTableColumns.containsKey(activityKey)){
										tableColumnVSImpactedTableColumns.put(activityKey, new ArrayList<String>());
									}					
									if(!tableColumnVSImpactedTableColumns.get(activityKey).contains(impactedValue))
										tableColumnVSImpactedTableColumns.get(activityKey).add(impactedValue);
								}

							}
							

						}
					}
					
					else if (ext.equalsIgnoreCase("svg")){
						System.out.println("enter equalsIgnoreCase svg");
						mxSvgCanvas canvas = (mxSvgCanvas) mxCellRenderer
								.drawCells(graph, null, 1, null,
										new CanvasFactory()
										{
											public mxICanvas createCanvas(
													int width, int height)
											{
												mxSvgCanvas canvas = new mxSvgCanvas(
														mxDomUtils.createSvgDocument(
																width, height));
												canvas.setEmbedded(true);

												return canvas;
											}

										});

						mxUtils.writeFile(mxXmlUtils.getXml(canvas.getDocument()),
								filename);
					}
					//output pdf file added by Guangming on June 16,2017
					else if (ext.equalsIgnoreCase("pdf")){
						mxSvgCanvas canvas = (mxSvgCanvas) mxCellRenderer
								.drawCells(graph, null, 1, null,
										new CanvasFactory()
										{
											public mxICanvas createCanvas(
													int width, int height)
											{
												mxSvgCanvas canvas = new mxSvgCanvas(
														mxDomUtils.createSvgDocument(
																width, height));
												canvas.setEmbedded(true);

												return canvas;
											}

										});
						SVG2PDF converter= new SVG2PDF();
						File pdfFile=new File(filename);  
						converter.convertSvgString2PdfFile(mxXmlUtils.getXml(canvas.getDocument()),
								pdfFile);
//						mxUtils.writeFile(mxXmlUtils.getXml(canvas.getDocument()),
//								filename);
					}
					else if (ext.equalsIgnoreCase("mxe")
							|| ext.equalsIgnoreCase("xml") ){
						mxCodec codec = new mxCodec();
						String xml = mxXmlUtils.getXml(codec.encode(graph
								.getModel()));

						mxUtils.writeFile(xml, filename);

						editor.setModified(false);
						editor.setCurrentFile(new File(filename));
						
						//change name in multiple graph panel						
						JTabbedPane multipleGraphPanel = editor.getMultipleGraphPanel();					
						int componentIndex = multipleGraphPanel.indexOfComponent(editor.getGraphComponent());
						if(pureName == null){
							pureName = multipleGraphPanel.getTitleAt(componentIndex);
						}
						if(pureName.contains("*")){
							//remove "*" into the name to indicate the file is modified and unsaved
							pureName = pureName.replace("*", "");
						}
						
						multipleGraphPanel.setTitleAt(componentIndex, pureName);												
						editor.setGraphComponentFile(editor.getGraphComponent(), editor.getCurrentFile());														
					}
				}
				catch (Throwable ex)
				{
					ex.printStackTrace();
					JOptionPane.showMessageDialog(graphComponent,
							ex.toString(), mxResources.get("error"),
							JOptionPane.ERROR_MESSAGE);
				}
			}
		}
	}

	/**
	 *
	 */
	@SuppressWarnings("serial")
	public static class SelectShortestPathAction extends AbstractAction
	{
		/**
		 * 
		 */
		protected boolean directed;

		/**
		 * 
		 */
		public SelectShortestPathAction(boolean directed)
		{
			this.directed = directed;
		}

		/**
		 * 
		 */
		public void actionPerformed(ActionEvent e)
		{
			if (e.getSource() instanceof mxGraphComponent)
			{
				mxGraphComponent graphComponent = (mxGraphComponent) e
						.getSource();
				mxGraph graph = graphComponent.getGraph();
				mxIGraphModel model = graph.getModel();

				Object source = null;
				Object target = null;

				Object[] cells = graph.getSelectionCells();

				for (int i = 0; i < cells.length; i++)
				{
					if (model.isVertex(cells[i]))
					{
						if (source == null)
						{
							source = cells[i];
						}
						else if (target == null)
						{
							target = cells[i];
						}
					}

					if (source != null && target != null)
					{
						break;
					}
				}

				if (source != null && target != null)
				{
					int steps = graph.getChildEdges(graph.getDefaultParent()).length;
					Object[] path = mxGraphAnalysis.getInstance()
							.getShortestPath(graph, source, target,
									new mxDistanceCostFunction(), steps,
									directed);
					graph.setSelectionCells(path);
				}
				else
				{
					JOptionPane.showMessageDialog(graphComponent,
							mxResources.get("noSourceAndTargetSelected"));
				}
			}
		}
	}

	/**
	 *
	 */
	@SuppressWarnings("serial")
	public static class SelectSpanningTreeAction extends AbstractAction
	{
		/**
		 * 
		 */
		protected boolean directed;

		/**
		 * 
		 */
		public SelectSpanningTreeAction(boolean directed)
		{
			this.directed = directed;
		}

		/**
		 * 
		 */
		public void actionPerformed(ActionEvent e)
		{
			if (e.getSource() instanceof mxGraphComponent)
			{
				mxGraphComponent graphComponent = (mxGraphComponent) e
						.getSource();
				mxGraph graph = graphComponent.getGraph();
				mxIGraphModel model = graph.getModel();

				Object parent = graph.getDefaultParent();
				Object[] cells = graph.getSelectionCells();

				for (int i = 0; i < cells.length; i++)
				{
					if (model.getChildCount(cells[i]) > 0)
					{
						parent = cells[i];
						break;
					}
				}

				Object[] v = graph.getChildVertices(parent);
				Object[] mst = mxGraphAnalysis.getInstance()
						.getMinimumSpanningTree(graph, v,
								new mxDistanceCostFunction(), directed);
				graph.setSelectionCells(mst);
			}
		}
	}

	/**
	 *
	 */
	@SuppressWarnings("serial")
	public static class ToggleDirtyAction extends AbstractAction
	{
		/**
		 * 
		 */
		public void actionPerformed(ActionEvent e)
		{
			if (e.getSource() instanceof mxGraphComponent)
			{
				mxGraphComponent graphComponent = (mxGraphComponent) e
						.getSource();
				graphComponent.showDirtyRectangle = !graphComponent.showDirtyRectangle;
			}
		}

	}

	/**
	 *
	 */
	@SuppressWarnings("serial")
	public static class ToggleConnectModeAction extends AbstractAction
	{
		/**
		 * 
		 */
		public void actionPerformed(ActionEvent e)
		{
			if (e.getSource() instanceof mxGraphComponent)
			{
				mxGraphComponent graphComponent = (mxGraphComponent) e
						.getSource();
				mxConnectionHandler handler = graphComponent
						.getConnectionHandler();
				handler.setHandleEnabled(!handler.isHandleEnabled());
			}
		}
	}

	/**
	 *
	 */
	@SuppressWarnings("serial")
	public static class ToggleCreateTargetItem extends JCheckBoxMenuItem
	{
		/**
		 * 
		 */
		public ToggleCreateTargetItem(final BasicGraphEditor editor, String name)
		{
			super(name);
			setSelected(true);

			addActionListener(new ActionListener()
			{
				/**
				 * 
				 */
				public void actionPerformed(ActionEvent e)
				{
					mxGraphComponent graphComponent = editor
							.getGraphComponent();

					if (graphComponent != null)
					{
						mxConnectionHandler handler = graphComponent
								.getConnectionHandler();
						handler.setCreateTarget(!handler.isCreateTarget());
						setSelected(handler.isCreateTarget());
					}
				}
			});
		}
	}

	/**
	 *
	 */
	@SuppressWarnings("serial")
	public static class PromptPropertyAction extends AbstractAction
	{
		/**
		 * 
		 */
		protected Object target;

		/**
		 * 
		 */
		protected String fieldname, message;

		/**
		 * 
		 */
		public PromptPropertyAction(Object target, String message)
		{
			this(target, message, message);
		}

		/**
		 * 
		 */
		public PromptPropertyAction(Object target, String message,
				String fieldname)
		{
			this.target = target;
			this.message = message;
			this.fieldname = fieldname;
		}

		/**
		 * 
		 */
		public void actionPerformed(ActionEvent e)
		{
			if (e.getSource() instanceof Component)
			{
				try
				{
					Method getter = target.getClass().getMethod(
							"get" + fieldname);
					Object current = getter.invoke(target);

					// TODO: Support other atomic types
					if (current instanceof Integer)
					{
						Method setter = target.getClass().getMethod(
								"set" + fieldname, new Class[] { int.class });

						String value = (String) JOptionPane.showInputDialog(
								(Component) e.getSource(), "Value", message,
								JOptionPane.PLAIN_MESSAGE, null, null, current);

						if (value != null)
						{
							setter.invoke(target, Integer.parseInt(value));
						}
					}
				}
				catch (Exception ex)
				{
					ex.printStackTrace();
				}
			}

			// Repaints the graph component
			if (e.getSource() instanceof mxGraphComponent)
			{
				mxGraphComponent graphComponent = (mxGraphComponent) e
						.getSource();
				graphComponent.repaint();
			}
		}
	}

	/**
	 *
	 */
	@SuppressWarnings("serial")
	public static class TogglePropertyItem extends JCheckBoxMenuItem
	{
		/**
		 * 
		 */
		public TogglePropertyItem(Object target, String name, String fieldname)
		{
			this(target, name, fieldname, false);
		}

		/**
		 * 
		 */
		public TogglePropertyItem(Object target, String name, String fieldname,
				boolean refresh)
		{
			this(target, name, fieldname, refresh, null);
		}

		/**
		 * 
		 */
		public TogglePropertyItem(final Object target, String name,
				final String fieldname, final boolean refresh,
				ActionListener listener)
		{
			super(name);

			// Since action listeners are processed last to first we add the given
			// listener here which means it will be processed after the one below
			if (listener != null)
			{
				addActionListener(listener);
			}

			addActionListener(new ActionListener()
			{
				/**
				 * 
				 */
				public void actionPerformed(ActionEvent e)
				{
					execute(target, fieldname, refresh);
				}
			});

			PropertyChangeListener propertyChangeListener = new PropertyChangeListener()
			{

				/*
				 * (non-Javadoc)
				 * @see java.beans.PropertyChangeListener#propertyChange(java.beans.PropertyChangeEvent)
				 */
				public void propertyChange(PropertyChangeEvent evt)
				{
					if (evt.getPropertyName().equalsIgnoreCase(fieldname))
					{
						update(target, fieldname);
					}
				}
			};

			if (target instanceof mxGraphComponent)
			{
				((mxGraphComponent) target)
						.addPropertyChangeListener(propertyChangeListener);
			}
			else if (target instanceof mxGraph)
			{
				((mxGraph) target)
						.addPropertyChangeListener(propertyChangeListener);
			}

			update(target, fieldname);
		}

		/**
		 * 
		 */
		public void update(Object target, String fieldname)
		{
			if (target != null && fieldname != null)
			{
				try
				{
					Method getter = target.getClass().getMethod(
							"is" + fieldname);

					if (getter != null)
					{
						Object current = getter.invoke(target);

						if (current instanceof Boolean)
						{
							setSelected(((Boolean) current).booleanValue());
//							setSelected(true);
						}
					}
				}
				catch (Exception e)
				{
					// ignore
				}
			}
		}

		/**
		 * 
		 */
		public void execute(Object target, String fieldname, boolean refresh)
		{
			if (target != null && fieldname != null)
			{
				try
				{
					Method getter = target.getClass().getMethod(
							"is" + fieldname);
					Method setter = target.getClass().getMethod(
							"set" + fieldname, new Class[] { boolean.class });

					Object current = getter.invoke(target);

					if (current instanceof Boolean)
					{
						boolean value = !((Boolean) current).booleanValue();
						setter.invoke(target, value);
						setSelected(value);
					}

					if (refresh)
					{
//						mxGraph graph = null;
//
//						if (target instanceof mxGraph)
//						{
//							graph = (mxGraph) target;
//						}
//						else if (target instanceof mxGraphComponent)
//						{
//							graph = ((mxGraphComponent) target).getGraph();
//						}
//
//						graph.refresh();
					}
				}
				catch (Exception e)
				{
					// ignore
				}
			}
		}
	}

	/**
	 *
	 */
	@SuppressWarnings("serial")
	public static class HistoryAction extends AbstractAction
	{
		/**
		 * 
		 */
		protected boolean undo;

		/**
		 * 
		 */
		public HistoryAction(boolean undo)
		{
			this.undo = undo;
		}

		/**
		 * 
		 */
		public void actionPerformed(ActionEvent e)
		{
			BasicGraphEditor editor = getEditor(e);

			if (editor != null)
			{
				if (undo)
				{
					editor.getUndoManager().undo();
				}
				else
				{
					editor.getUndoManager().redo();
				}
			}
		}
	}

	/**
	 *
	 */
	@SuppressWarnings("serial")
	public static class FontStyleAction extends AbstractAction
	{
		/**
		 * 
		 */
		protected boolean bold;

		/**
		 * 
		 */
		public FontStyleAction(boolean bold)
		{
			this.bold = bold;
		}

		/**
		 * 
		 */
		public void actionPerformed(ActionEvent e)
		{
			if (e.getSource() instanceof mxGraphComponent)
			{
				mxGraphComponent graphComponent = (mxGraphComponent) e
						.getSource();
				Component editorComponent = null;

				if (graphComponent.getCellEditor() instanceof mxCellEditor)
				{
					editorComponent = ((mxCellEditor) graphComponent
							.getCellEditor()).getEditor();
				}

				if (editorComponent instanceof JEditorPane)
				{
					JEditorPane editorPane = (JEditorPane) editorComponent;
					int start = editorPane.getSelectionStart();
					int ende = editorPane.getSelectionEnd();
					String text = editorPane.getSelectedText();

					if (text == null)
					{
						text = "";
					}

					try
					{
						HTMLEditorKit editorKit = new HTMLEditorKit();
						HTMLDocument document = (HTMLDocument) editorPane
								.getDocument();
						document.remove(start, (ende - start));
						editorKit.insertHTML(document, start, ((bold) ? "<b>"
								: "<i>") + text + ((bold) ? "</b>" : "</i>"),
								0, 0, (bold) ? HTML.Tag.B : HTML.Tag.I);
					}
					catch (Exception ex)
					{
						ex.printStackTrace();
					}

					editorPane.requestFocus();
					editorPane.select(start, ende);
				}
				else
				{
					mxIGraphModel model = graphComponent.getGraph().getModel();
					model.beginUpdate();
					try
					{
						graphComponent.stopEditing(false);
						graphComponent.getGraph().toggleCellStyleFlags(
								mxConstants.STYLE_FONTSTYLE,
								(bold) ? mxConstants.FONT_BOLD
										: mxConstants.FONT_ITALIC);
					}
					finally
					{
						model.endUpdate();
					}
				}
			}
		}
	}

	/**
	 *
	 */
	@SuppressWarnings("serial")
	public static class WarningAction extends AbstractAction
	{
		/**
		 * 
		 */
		public void actionPerformed(ActionEvent e)
		{
			if (e.getSource() instanceof mxGraphComponent)
			{
				mxGraphComponent graphComponent = (mxGraphComponent) e
						.getSource();
				Object[] cells = graphComponent.getGraph().getSelectionCells();

				if (cells != null && cells.length > 0)
				{
					String warning = JOptionPane.showInputDialog(mxResources
							.get("enterWarningMessage"));

					for (int i = 0; i < cells.length; i++)
					{
						graphComponent.setCellWarning(cells[i], warning);
					}
				}
				else
				{
					JOptionPane.showMessageDialog(graphComponent,
							mxResources.get("noCellSelected"));
				}
			}
		}
	}

	/**
	 *
	 */
	@SuppressWarnings("serial")
	public static class NewAction extends AbstractAction
	{
		/**
		 * 
		 */
		private int tabIndex = 2;
		public void actionPerformed(ActionEvent e)
		{
			BasicGraphEditor editor = getEditor(e);
			
			
			if (editor != null)
			{		
				//record current file
				File currentFile = editor.getCurrentFile();
				mxGraphComponent graphComponent = editor.getGraphComponent();				
				editor.setGraphComponentFile(graphComponent, currentFile);
				
//				graphComponent.setPageVisible(false);
				//create new component
				mxGraphComponent newGraphComponent = new CustomGraphComponent(new CustomGraph());			
				mxGraph graph = newGraphComponent.getGraph();
				Style.createStyle(graph);
				graph.setAllowDanglingEdges(false);
				graph.setSplitEnabled(false);
				//add new created component into the multiple graph panel
				JTabbedPane multipleGraphPanel = editor.getMultipleGraphPanel();			
				multipleGraphPanel.add("graph" + tabIndex + "*" + "   ", newGraphComponent);

				tabIndex ++;
				
				//set the new created component as the selected one (and set the current as an existent file)
				multipleGraphPanel.setSelectedComponent(newGraphComponent);
				editor.setGraphComponent(newGraphComponent);
				editor.setCurrentFile(null);
				
				//set listeners to the graph(this should be after setting the new graph component as the current component)
				editor.addListenersToGraphComponent(newGraphComponent);
				
				//set the size of the canvas
				newGraphComponent.setHorizontalPageCount(2);
				
				//set default zoom scale
//				double scale = 1.0;
//				newGraphComponent.setCenterZoom(true);
//				newGraphComponent.zoomTo(scale, newGraphComponent
//						.isCenterZoom());
				
				//hide page breaks
				newGraphComponent.setPageBreaksVisible(graph.isPageBreaksEnabled());
				newGraphComponent.updateUI();
				newGraphComponent.refresh();
				
//				newGraphComponent.setCenterZoom(true);
//				newGraphComponent.zoom(1.5);
//				newGraphComponent.zoomTo(1.5,true);
				
			}
		}
	}

	/**
	 *
	 */
	@SuppressWarnings("serial")
	public static class ImportAction extends AbstractAction
	{
		/**
		 * 
		 */
		protected String lastDir;

		/**
		 * Loads and registers the shape as a new shape in mxGraphics2DCanvas and
		 * adds a new entry to use that shape in the specified palette
		 * @param palette The palette to add the shape to.
		 * @param nodeXml The raw XML of the shape
		 * @param path The path to the directory the shape exists in
		 * @return the string name of the shape
		 */
		public static String addStencilShape(EditorPalette palette,
				String nodeXml, String path)
		{

			// Some editors place a 3 byte BOM at the start of files
			// Ensure the first char is a "<"
			int lessthanIndex = nodeXml.indexOf("<");
			nodeXml = nodeXml.substring(lessthanIndex);
			mxStencilShape newShape = new mxStencilShape(nodeXml);
			String name = newShape.getName();
			ImageIcon icon = null;

			if (path != null)
			{
				String iconPath = path + newShape.getIconPath();
				icon = new ImageIcon(iconPath);
			}

			// Registers the shape in the canvas shape registry
			mxGraphics2DCanvas.putShape(name, newShape);

			if (palette != null && icon != null)
			{
				palette.addTemplate(name, icon, "shape=" + name, 80, 80, "");
			}

			return name;
		}

		/**
		 * 
		 */
		public void actionPerformed(ActionEvent e)
		{
			BasicGraphEditor editor = getEditor(e);

			if (editor != null)
			{
				String wd = (lastDir != null) ? lastDir : System
						.getProperty("user.dir");

				JFileChooser fc = new JFileChooser(wd);

				fc.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);

				// Adds file filter for Dia shape import
				fc.addChoosableFileFilter(new DefaultFileFilter(".shape",
						"Dia Shape " + mxResources.get("file") + " (.shape)"));

				int rc = fc.showDialog(null, mxResources.get("importStencil"));

				if (rc == JFileChooser.APPROVE_OPTION)
				{
					lastDir = fc.getSelectedFile().getParent();

					try
					{
						if (fc.getSelectedFile().isDirectory())
						{
							EditorPalette palette = editor.insertPalette(fc
									.getSelectedFile().getName());

							for (File f : fc.getSelectedFile().listFiles(
									new FilenameFilter()
									{
										public boolean accept(File dir,
												String name)
										{
											return name.toLowerCase().endsWith(
													".shape");
										}
									}))
							{
								String nodeXml = mxUtils.readFile(f
										.getAbsolutePath());
								addStencilShape(palette, nodeXml, f.getParent()
										+ File.separator);
							}

							JComponent scrollPane = (JComponent) palette
									.getParent().getParent();
							editor.getLibraryPane().setSelectedComponent(
									scrollPane);

							// FIXME: Need to update the size of the palette to force a layout
							// update. Re/in/validate of palette or parent does not work.
							//editor.getLibraryPane().revalidate();
						}
						else
						{
							String nodeXml = mxUtils.readFile(fc
									.getSelectedFile().getAbsolutePath());
							String name = addStencilShape(null, nodeXml, null);

							JOptionPane.showMessageDialog(editor, mxResources
									.get("stencilImported",
											new String[] { name }));
						}
					}
					catch (IOException e1)
					{
						e1.printStackTrace();
					}
				}
			}
		}
	}

	/**
	 *
	 */
	@SuppressWarnings("serial")
	public static class OpenAction extends AbstractAction
	{
		/**
		 * 
		 */
		protected String lastDir;

		/**
		 * 
		 */
		protected void resetEditor(BasicGraphEditor editor)
		{
			editor.setModified(false);
			editor.getUndoManager().clear();
			editor.getGraphComponent().zoomAndCenter();
		}

		/**
		 * Reads XML+PNG format.
		 */
		protected void openXmlPng(BasicGraphEditor editor, File file)
				throws IOException
		{
			Map<String, String> text = mxPngTextDecoder
					.decodeCompressedText(new FileInputStream(file));

			if (text != null)
			{
				String value = text.get("mxGraphModel");

				if (value != null)
				{
					Document document = mxXmlUtils.parseXml(URLDecoder.decode(
							value, "UTF-8"));
					mxCodec codec = new mxCodec(document);
					codec.decode(document.getDocumentElement(), editor
							.getGraphComponent().getGraph().getModel());
					editor.setCurrentFile(file);
					resetEditor(editor);

					return;
				}
			}

			JOptionPane.showMessageDialog(editor,
					mxResources.get("imageContainsNoDiagramData"));
		}

		/**
		 * @throws IOException
		 *
		 */
		protected void openGD(BasicGraphEditor editor, File file,
				String gdText)
		{
			mxGraph graph = editor.getGraphComponent().getGraph();

			// Replaces file extension with .mxe
			String filename = file.getName();
			filename = filename.substring(0, filename.length() - 4) + ".mxe";

			if (new File(filename).exists()
					&& JOptionPane.showConfirmDialog(editor,
							mxResources.get("overwriteExistingFile")) != JOptionPane.YES_OPTION)
			{
				return;
			}

			((mxGraphModel) graph.getModel()).clear();
			mxGdCodec.decode(gdText, graph);
			editor.getGraphComponent().zoomAndCenter();
			editor.setCurrentFile(new File(lastDir + "/" + filename));
		}

		/**
		 * 
		 */
		public void actionPerformed(ActionEvent e)
		{
			BasicGraphEditor editor = getEditor(e);

			if (editor != null)
			{
//				if (!editor.isModified()
//						|| JOptionPane.showConfirmDialog(editor,
//								mxResources.get("loseChanges")) == JOptionPane.YES_OPTION)
//				{
				if (true)
				{
					mxGraph graph = editor.getGraphComponent().getGraph();

					if (graph != null)
					{
						String wd = (lastDir != null) ? lastDir : System
								.getProperty("user.dir");

						JFileChooser fc = new JFileChooser(wd);

						// Adds file filter for supported file format
						DefaultFileFilter combinedFilter = new DefaultFileFilter(
								".mxe", mxResources.get("allSupportedFormats")
										+ " (.mxe, .png, .vdx)")
						{

							public boolean accept(File file)
							{
								String lcase = file.getName().toLowerCase();

								return super.accept(file)
										|| lcase.endsWith(".png")
										|| lcase.endsWith(".vdx");
							}
						};
						
						DefaultFileFilter defaultFilter = new DefaultFileFilter(".ocbcmml",
								"OCBC Model File " + " (.ocbcmml)");
						
						fc.addChoosableFileFilter(defaultFilter);
						
						fc.addChoosableFileFilter(combinedFilter);

						fc.addChoosableFileFilter(new DefaultFileFilter(".mxe",
								"mxGraph Editor " + mxResources.get("file")
										+ " (.mxe)"));
						fc.addChoosableFileFilter(new DefaultFileFilter(".png",
								"PNG+XML  " + mxResources.get("file")
										+ " (.png)"));

						// Adds file filter for VDX import
						fc.addChoosableFileFilter(new DefaultFileFilter(".vdx",
								"XML Drawing  " + mxResources.get("file")
										+ " (.vdx)"));

						// Adds file filter for GD import
						fc.addChoosableFileFilter(new DefaultFileFilter(".txt",
								"Graph Drawing  " + mxResources.get("file")
										+ " (.txt)"));

						fc.setFileFilter(defaultFilter);

						int rc = fc.showDialog(null,
								mxResources.get("openFile"));

						if (rc == JFileChooser.APPROVE_OPTION)
						{
							lastDir = fc.getSelectedFile().getParent();

							try
							{
								if (fc.getSelectedFile().getAbsolutePath()
										.toLowerCase().endsWith(".png"))
								{
									openXmlPng(editor, fc.getSelectedFile());
								}
								else if (fc.getSelectedFile().getAbsolutePath()
										.toLowerCase().endsWith(".txt"))
								{
									openGD(editor, fc.getSelectedFile(),
											mxUtils.readFile(fc
													.getSelectedFile()
													.getAbsolutePath()));
								}
								else if(fc.getSelectedFile().getAbsolutePath()
										.toLowerCase().endsWith(".ocbcmml"))
								{
									String fileString = mxUtils.readFile(fc
											.getSelectedFile()
											.getAbsolutePath());
									
									//get the version added by guangming
									String versionString = fileString.split("\n")[0];
									String versionName = versionString.split("\"")[1];
									System.out.println("versionName: " + versionName);
									//remove the version string added by guangming
									String removeVersion = fileString.substring(fileString.indexOf('\n')+1);
									System.out.println("model file: " + removeVersion);
									
									Document document = mxXmlUtils
											.parseXml(removeVersion);

									mxCodec codec = new mxCodec(document);
									codec.decode(
											document.getDocumentElement(),
											graph.getModel());
									graph.getAttributes().put(OCBCEditorConstants.OCBCVERSION, versionName);
									//do not change the input file
//									editor.setCurrentFile(fc
//											.getSelectedFile());

									resetEditor(editor);
									System.out.println("update the label after importing file");
									editor.refreshGraph();
								}
								else
								{
									Document document = mxXmlUtils
											.parseXml(mxUtils.readFile(fc
													.getSelectedFile()
													.getAbsolutePath()));

									mxCodec codec = new mxCodec(document);
									codec.decode(
											document.getDocumentElement(),
											graph.getModel());
									editor.setCurrentFile(fc
											.getSelectedFile());

									resetEditor(editor);
									
								}
							}
							catch (IOException ex)
							{
								ex.printStackTrace();
								JOptionPane.showMessageDialog(
										editor.getGraphComponent(),
										ex.toString(),
										mxResources.get("error"),
										JOptionPane.ERROR_MESSAGE);
							}
						}
					}
				}
			}
		}
	}

	/**
	 *
	 */
	@SuppressWarnings("serial")
	public static class ToggleAction extends AbstractAction
	{
		/**
		 * 
		 */
		protected String key;

		/**
		 * 
		 */
		protected boolean defaultValue;

		/**
		 * 
		 * @param key
		 */
		public ToggleAction(String key)
		{
			this(key, false);
		}

		/**
		 * 
		 * @param key
		 */
		public ToggleAction(String key, boolean defaultValue)
		{
			this.key = key;
			this.defaultValue = defaultValue;
		}

		/**
		 * 
		 */
		public void actionPerformed(ActionEvent e)
		{
			mxGraph graph = mxGraphActions.getGraph(e);

			if (graph != null)
			{
				graph.toggleCellStyles(key, defaultValue);
			}
		}
	}

	/**
	 *
	 */
	@SuppressWarnings("serial")
	public static class SetLabelPositionAction extends AbstractAction
	{
		/**
		 * 
		 */
		protected String labelPosition, alignment;

		/**
		 * 
		 * @param key
		 */
		public SetLabelPositionAction(String labelPosition, String alignment)
		{
			this.labelPosition = labelPosition;
			this.alignment = alignment;
		}

		/**
		 * 
		 */
		public void actionPerformed(ActionEvent e)
		{
			mxGraph graph = mxGraphActions.getGraph(e);

			if (graph != null && !graph.isSelectionEmpty())
			{
				graph.getModel().beginUpdate();
				try
				{
					// Checks the orientation of the alignment to use the correct constants
					if (labelPosition.equals(mxConstants.ALIGN_LEFT)
							|| labelPosition.equals(mxConstants.ALIGN_CENTER)
							|| labelPosition.equals(mxConstants.ALIGN_RIGHT))
					{
						graph.setCellStyles(mxConstants.STYLE_LABEL_POSITION,
								labelPosition);
						graph.setCellStyles(mxConstants.STYLE_ALIGN, alignment);
					}
					else
					{
						graph.setCellStyles(
								mxConstants.STYLE_VERTICAL_LABEL_POSITION,
								labelPosition);
						graph.setCellStyles(mxConstants.STYLE_VERTICAL_ALIGN,
								alignment);
					}
				}
				finally
				{
					graph.getModel().endUpdate();
				}
			}
		}
	}

	/**
	 *
	 */
	@SuppressWarnings("serial")
	public static class SetStyleAction extends AbstractAction
	{
		/**
		 * 
		 */
		protected String value;

		/**
		 * 
		 * @param key
		 */
		public SetStyleAction(String value)
		{
			this.value = value;
		}

		/**
		 * 
		 */
		public void actionPerformed(ActionEvent e)
		{
			mxGraph graph = mxGraphActions.getGraph(e);

			if (graph != null && !graph.isSelectionEmpty())
			{
				graph.setCellStyle(value);
			}
		}
	}

	/**
	 *
	 */
	@SuppressWarnings("serial")
	public static class KeyValueAction extends AbstractAction
	{
		/**
		 * 
		 */
		protected String key, value;

		/**
		 * 
		 * @param key
		 */
		public KeyValueAction(String key)
		{
			this(key, null);
		}

		/**
		 * 
		 * @param key
		 */
		public KeyValueAction(String key, String value)
		{
			this.key = key;
			this.value = value;
		}

		/**
		 * 
		 */
		public void actionPerformed(ActionEvent e)
		{
			mxGraph graph = mxGraphActions.getGraph(e);

			if (graph != null && !graph.isSelectionEmpty())
			{
				graph.setCellStyles(key, value);
			}
		}
	}

	/**
	 *
	 */
	@SuppressWarnings("serial")
	public static class PromptValueAction extends AbstractAction
	{
		/**
		 * 
		 */
		protected String key, message;

		/**
		 * 
		 * @param key
		 */
		public PromptValueAction(String key, String message)
		{
			this.key = key;
			this.message = message;
		}

		/**
		 * 
		 */
		public void actionPerformed(ActionEvent e)
		{
			if (e.getSource() instanceof Component)
			{
				mxGraph graph = mxGraphActions.getGraph(e);

				if (graph != null && !graph.isSelectionEmpty())
				{
					String value = (String) JOptionPane.showInputDialog(
							(Component) e.getSource(),
							mxResources.get("value"), message,
							JOptionPane.PLAIN_MESSAGE, null, null, "");

					if (value != null)
					{
						if (value.equals(mxConstants.NONE))
						{
							value = null;
						}

						graph.setCellStyles(key, value);
					}
				}
			}
		}
	}

	/**
	 *
	 */
	@SuppressWarnings("serial")
	public static class AlignCellsAction extends AbstractAction
	{
		/**
		 * 
		 */
		protected String align;

		/**
		 * 
		 * @param key
		 */
		public AlignCellsAction(String align)
		{
			this.align = align;
		}

		/**
		 * 
		 */
		public void actionPerformed(ActionEvent e)
		{
			mxGraph graph = mxGraphActions.getGraph(e);

			if (graph != null && !graph.isSelectionEmpty())
			{
				graph.alignCells(align);
			}
		}
	}

	/**
	 *
	 */
	@SuppressWarnings("serial")
	public static class AutosizeAction extends AbstractAction
	{
		/**
		 * 
		 */
		public void actionPerformed(ActionEvent e)
		{
			mxGraph graph = mxGraphActions.getGraph(e);

			if (graph != null && !graph.isSelectionEmpty())
			{
				Object[] cells = graph.getSelectionCells();
				mxIGraphModel model = graph.getModel();

				model.beginUpdate();
				try
				{
					for (int i = 0; i < cells.length; i++)
					{
						graph.updateCellSize(cells[i]);
					}
				}
				finally
				{
					model.endUpdate();
				}
			}
		}
	}

	/**
	 *
	 */
	@SuppressWarnings("serial")
	public static class ColorAction extends AbstractAction
	{
		/**
		 * 
		 */
		protected String name, key;

		/**
		 * 
		 * @param key
		 */
		public ColorAction(String name, String key)
		{
			this.name = name;
			this.key = key;
		}

		/**
		 * 
		 */
		public void actionPerformed(ActionEvent e)
		{
			if (e.getSource() instanceof mxGraphComponent)
			{
				mxGraphComponent graphComponent = (mxGraphComponent) e
						.getSource();
				mxGraph graph = graphComponent.getGraph();

				if (!graph.isSelectionEmpty())
				{
					Color newColor = JColorChooser.showDialog(graphComponent,
							name, null);

					if (newColor != null)
					{
						graph.setCellStyles(key, mxUtils.hexString(newColor));
					}
				}
			}
		}
	}

	/**
	 *
	 */
	@SuppressWarnings("serial")
	public static class BackgroundImageAction extends AbstractAction
	{
		/**
		 * 
		 */
		public void actionPerformed(ActionEvent e)
		{
			if (e.getSource() instanceof mxGraphComponent)
			{
				mxGraphComponent graphComponent = (mxGraphComponent) e
						.getSource();
				String value = (String) JOptionPane.showInputDialog(
						graphComponent, mxResources.get("backgroundImage"),
						"URL", JOptionPane.PLAIN_MESSAGE, null, null,
						"http://www.callatecs.com/images/background2.JPG");

				if (value != null)
				{
					if (value.length() == 0)
					{
						graphComponent.setBackgroundImage(null);
					}
					else
					{
						Image background = mxUtils.loadImage(value);
						// Incorrect URLs will result in no image.
						// TODO provide feedback that the URL is not correct
						if (background != null)
						{
							graphComponent.setBackgroundImage(new ImageIcon(
									background));
						}
					}

					// Forces a repaint of the outline
					graphComponent.getGraph().repaint();
				}
			}
		}
	}

	/**
	 *
	 */
	@SuppressWarnings("serial")
	public static class BackgroundAction extends AbstractAction
	{
		/**
		 * 
		 */
		public void actionPerformed(ActionEvent e)
		{
			if (e.getSource() instanceof mxGraphComponent)
			{
				mxGraphComponent graphComponent = (mxGraphComponent) e
						.getSource();
				Color newColor = JColorChooser.showDialog(graphComponent,
						mxResources.get("background"), null);

				if (newColor != null)
				{
					graphComponent.getViewport().setOpaque(true);
					graphComponent.getViewport().setBackground(newColor);
				}

				// Forces a repaint of the outline
				graphComponent.getGraph().repaint();
			}
		}
	}

	/**
	 *
	 */
	@SuppressWarnings("serial")
	public static class PageBackgroundAction extends AbstractAction
	{
		/**
		 * 
		 */
		public void actionPerformed(ActionEvent e)
		{
			if (e.getSource() instanceof mxGraphComponent)
			{
				mxGraphComponent graphComponent = (mxGraphComponent) e
						.getSource();
				Color newColor = JColorChooser.showDialog(graphComponent,
						mxResources.get("pageBackground"), null);

				if (newColor != null)
				{
					graphComponent.setPageBackgroundColor(newColor);
				}

				// Forces a repaint of the component
				graphComponent.repaint();
			}
		}
	}

	/**
	 *
	 */
	@SuppressWarnings("serial")
	public static class StyleAction extends AbstractAction
	{
		/**
		 * 
		 */
		public void actionPerformed(ActionEvent e)
		{
			if (e.getSource() instanceof mxGraphComponent)
			{
				mxGraphComponent graphComponent = (mxGraphComponent) e
						.getSource();
				mxGraph graph = graphComponent.getGraph();
				String initial = graph.getModel().getStyle(
						graph.getSelectionCell());
				String value = (String) JOptionPane.showInputDialog(
						graphComponent, mxResources.get("style"),
						mxResources.get("style"), JOptionPane.PLAIN_MESSAGE,
						null, null, initial);

				if (value != null)
				{
					graph.setCellStyle(value);
				}
			}
		}
	}
	
	/**
	 * add the item in the pop up menu for showing/hiding connectors
	 * (used to create crel edges)
	 */
	@SuppressWarnings("serial")
	public static class ToggleConnectorItem extends JCheckBoxMenuItem
	{
		/**
		 * 
		 */
		public ToggleConnectorItem(final BasicGraphEditor editor, String name)
		{
			super(name);
			mxGraph graph = editor.getGraphComponent().getGraph();	
			//show the connectors by default
//			graph.setConnectorEnabled(true);
			setSelected(graph.isConnectorEnabled());
//			showConnectors(editor);
			addActionListener(new ActionListener(){
				/**
				 * 
				 */
				public void actionPerformed(ActionEvent e){	
					mxGraph graph = editor.getGraphComponent().getGraph();		
					boolean enabled = graph.isConnectorEnabled();
					//hide the connector
					if(enabled){
						hideConnectors(editor);
					}
					//show the connector
					else{
						showConnectors(editor);
					}						
				}
			});
		}
		
		
		/**
		 * show the connector
		 * @param editor
		 */
		public void showConnectors(BasicGraphEditor editor){
			mxGraph graph = editor.getGraphComponent().getGraph();	
			Object[] edgeCells = graph.getChildEdges(graph.getDefaultParent());
			setSelected(true);
			graph.setConnectorEnabled(true);
			for(int j=0; j<edgeCells.length; j++){
				mxCell edgeCell = (mxCell) edgeCells[j];
				String edgeType = edgeCell.getAttributes().get(mxConstants.TYPE);
				System.out.println("cancel edge type: " + edgeCell);
				if(edgeType.equals(mxConstants.ACTIVITYRELATION) || edgeType.equals(mxConstants.CLASSRELATION)){
					Object[] nodeCells = graph.getChildVertices(edgeCell);
					for(int i=0; i<nodeCells.length; i++){
						mxCell nodeCell = (mxCell) nodeCells[i];
						String nodeType = nodeCell.getAttributes().get(mxConstants.TYPE);
						System.out.println("cancel node type: " + nodeType);
						if(nodeType != null && nodeType.equals(mxConstants.CONNECTOR)){
							System.out.println("cancel node connector");
							editor.getGraphComponent().getGraph().getModel().setStyle(nodeCell, mxConstants.CONNECTOR_FOCUSED);
							editor.getGraphComponent().getGraph().refresh();
						}
					}		
				}						
			}	
		}
		
		/**
		 * show the connector
		 * @param editor
		 */
		public void hideConnectors(BasicGraphEditor editor){
			mxGraph graph = editor.getGraphComponent().getGraph();	
			Object[] edgeCells = graph.getChildEdges(graph.getDefaultParent());
			setSelected(false);
			graph.setConnectorEnabled(false);
			for(int j=0; j<edgeCells.length; j++){
				mxCell edgeCell = (mxCell) edgeCells[j];
				String edgeType = edgeCell.getAttributes().get(mxConstants.TYPE);
				System.out.println("cancel edge type: " + edgeCell);
				if(edgeType.equals(mxConstants.ACTIVITYRELATION) || edgeType.equals(mxConstants.CLASSRELATION)){
					Object[] nodeCells = graph.getChildVertices(edgeCell);
					for(int i=0; i<nodeCells.length; i++){
						mxCell nodeCell = (mxCell) nodeCells[i];
						String nodeType = nodeCell.getAttributes().get(mxConstants.TYPE);
						System.out.println("cancel node type: " + nodeType);
						if(nodeType != null && nodeType.equals(mxConstants.CONNECTOR)){
							System.out.println("cancel node connector");
							editor.getGraphComponent().getGraph().getModel().setStyle(nodeCell, mxConstants.CONNECTOR_TRANSPARENT);
							editor.getGraphComponent().getGraph().refresh();
						}
					}		
				}						
			}				
		}
	}
	
	/**
	 * add the item in the pop up menu for checking OCBC models
	 */
	@SuppressWarnings("serial")
	public static class ToggleCheckItem extends JCheckBoxMenuItem
	{
		/**
		 * 
		 */
		public ToggleCheckItem(final BasicGraphEditor editor, String name)
		{
			super(name);
			mxGraph graph = editor.getGraphComponent().getGraph();	
			setSelected(graph.isCheckEnabled());

			addActionListener(new ActionListener()
			{
				/**
				 * 
				 */
				public void actionPerformed(ActionEvent e)
				{	
					mxGraph graph = editor.getGraphComponent().getGraph();	
					boolean enabled = graph.isCheckEnabled();
					Object[] edgeCells = graph.getChildEdges(graph.getDefaultParent());
					Object[] nodeCells = graph.getChildVertices(graph.getDefaultParent());
					
					//show the diagnosis information
					if(!enabled){					
						setSelected(true);
						graph.setCheckEnabled(true);
						System.out.println("enabled: " + enabled);
						//check if the activity nodes have aoc relation and highlight the labels of violating nodes
						for(int j=0; j<nodeCells.length; j++){
							mxCell nodeCell = (mxCell) nodeCells[j];
							String nodeType = nodeCell.getAttributes().get(mxConstants.TYPE);
							System.out.println("cancel edge type: " + nodeType);
							if(nodeType.equals(mxConstants.ACTIVITYCELL)){				
								int linkedAOCEdgeNumber = 0;
								for(int k=0; k<nodeCell.getEdgeCount(); k++){
									mxCell linkedEdgeCell = (mxCell) nodeCell.getEdgeAt(k);
									String linkEdgeType = linkedEdgeCell.getAttributes().get(mxConstants.TYPE);
									if(linkEdgeType.equals(mxConstants.AOCRELATION)){
										linkedAOCEdgeNumber ++;
									}
								}
								if(linkedAOCEdgeNumber == 0){
									Style.addStyleElementToCell(nodeCell, mxConstants.STYLE_FONTCOLOR, "red");
									editor.getGraphComponent().getGraph().refresh();
								}								
							}			
						}		
						//check if the activity edges have crel relation and highlight the violating connector
						for(int j=0; j<edgeCells.length; j++){
							mxCell edgeCell = (mxCell) edgeCells[j];
							String edgeType = edgeCell.getAttributes().get(mxConstants.TYPE);
							System.out.println("cancel edge type: " + edgeCell);
							if(edgeType.equals(mxConstants.ACTIVITYRELATION)){
								Object[] connectorCells = graph.getChildVertices(edgeCell);
								mxCell connectorCell = (mxCell) connectorCells[0];
								int linkedEdgeNumber = connectorCell.getEdgeCount();
								if(linkedEdgeNumber != 1){
									editor.getGraphComponent().getGraph().getModel().setStyle(connectorCell, mxConstants.CONNECTOR_VIOLATING);
									editor.getGraphComponent().getGraph().refresh();
								}								
							}			
						}					
					}
					//hide the diagnosis information
					else{
						setSelected(false);
						graph.setCheckEnabled(false);
						System.out.println("enabled: " + enabled);
						for(int j=0; j<edgeCells.length; j++){
							mxCell edgeCell = (mxCell) edgeCells[j];
							String edgeType = edgeCell.getAttributes().get(mxConstants.TYPE);
							System.out.println("cancel edge type: " + edgeCell);
							if(edgeType.equals(mxConstants.ACTIVITYRELATION)){
								Object[] childNodeCells = graph.getChildVertices(edgeCell);
								for(int i=0; i<childNodeCells.length; i++){
									mxCell nodeCell = (mxCell) childNodeCells[i];
									String nodeType = nodeCell.getAttributes().get(mxConstants.TYPE);
									System.out.println("cancel node type: " + nodeType);
									if(nodeType != null && nodeType.equals(mxConstants.CONNECTOR)){
										System.out.println("cancel node connector");
										editor.getGraphComponent().getGraph().getModel().setStyle(nodeCell, mxConstants.CONNECTOR_TRANSPARENT);
										editor.getGraphComponent().getGraph().refresh();
									}
								}		
							}						
						}	
						
						for(int j=0; j<nodeCells.length; j++){
							mxCell nodeCell = (mxCell) nodeCells[j];
							String nodeType = nodeCell.getAttributes().get(mxConstants.TYPE);
							System.out.println("cancel edge type: " + nodeType);
							if(nodeType.equals(mxConstants.ACTIVITYCELL)){				
								Style.addStyleElementToCell(nodeCell, mxConstants.STYLE_FONTCOLOR, "black");
								editor.getGraphComponent().getGraph().refresh();			
							}			
						}										
					}						
				}
			});
		}
	}

	/**
	 * add the item in the pop up menu for checking OCBC models
	 */
	@SuppressWarnings("serial")
	public static class RefreshModelAction extends AbstractAction
	{		
		public void actionPerformed(ActionEvent e)
		{
			BasicGraphEditor editor = getEditor(e);

			if (editor != null)
			{
				editor.refreshGraph();
			}
		}
	}
	
	/**
	 * add the item in the pop up menu for swapping source and target
	 */
	@SuppressWarnings("serial")
	public static class SwapSourceAndTargetAction extends AbstractAction
	{		
		public void actionPerformed(ActionEvent e)
		{
			BasicGraphEditor editor = getEditor(e);

			if (editor != null)
			{
				mxGraph graph = editor.getGraphComponent().getGraph();
				mxCell selectedCell = (mxCell) graph.getSelectionCell();
				
				if(selectedCell != null){
					String selectedCellType = selectedCell.getAttributes().get("type");
					System.out.println("type: " + selectedCellType);
					
					if(selectedCellType.equals(mxConstants.CLASSRELATION)){	
						System.out.println("swap class relation");
						mxCell sourceCell = (mxCell) selectedCell.getSource();
						mxCell targetCell = (mxCell) selectedCell.getTarget();
						selectedCell.setSource(targetCell);
						selectedCell.setTarget(sourceCell);
						editor.addModificationFlag();
						editor.refreshGraph();						
					}
					else if(selectedCellType.equals(mxConstants.ACTIVITYRELATION)){	
						System.out.println("swap activity relation");
						mxCell sourceCell = (mxCell) selectedCell.getSource();
						mxCell targetCell = (mxCell) selectedCell.getTarget();
						selectedCell.setSource(targetCell);
						selectedCell.setTarget(sourceCell);
						editor.addModificationFlag();
						editor.refreshGraph();						
					}
				}
			}
		}
	}
	
	/**
	 * set the count of pages on the horizontal and vertical perspective
	 */
	@SuppressWarnings("serial")
	public static class PageCountAction extends AbstractAction
	{
		/**
		 * 
		 */
		public void actionPerformed(ActionEvent e)
		{
			if (e.getSource() instanceof mxGraphComponent)
			{
				mxGraphComponent graphComponent = (mxGraphComponent) e
						.getSource();
				
				int horizontalCountCurrent = graphComponent.getHorizontalPageCount();
				int verticalCountCurrent = graphComponent.getVerticalPageCount();
				
				String horizontalValue =  (String) JOptionPane.showInputDialog(
						graphComponent, "horizontal page count",
						"horizontal page count", JOptionPane.PLAIN_MESSAGE,
						null, null , horizontalCountCurrent);
				String verticalValue  = (String) JOptionPane.showInputDialog(
						graphComponent, "vertical page count",
						"vertical page count", JOptionPane.PLAIN_MESSAGE,
						null, null , verticalCountCurrent);
				
				int horizontalCountInput =  Integer.parseInt(horizontalValue);
				int verticalCountInput =  Integer.parseInt(verticalValue);
				
				graphComponent.setHorizontalPageCount(horizontalCountInput);
				graphComponent.setVerticalPageCount(verticalCountInput);

			}
		}
	}
	
	/**
	 *
	 */
	@SuppressWarnings("serial")
	public static class ToggleBreaksItem extends JCheckBoxMenuItem
	{
		/**
		 * 
		 */
		public ToggleBreaksItem(final BasicGraphEditor editor, String name)
		{
			super(name);
			mxGraph graph = editor.getGraphComponent().getGraph();	
			setSelected(graph.isPageBreaksEnabled());
			editor.getGraphComponent().setPageBreaksVisible(graph.isPageBreaksEnabled());
			editor.getGraphComponent().updateUI();
			editor.getGraphComponent().refresh();
			
			addActionListener(new ActionListener()
			{
				/**
				 * 
				 */
				public void actionPerformed(ActionEvent e)
				{	
					mxGraph graph = editor.getGraphComponent().getGraph();	
					boolean enabled = graph.isPageBreaksEnabled();
	
					
					//show the page breaks
					if(!enabled){					
						setSelected(true);
						graph.setPageBreaksEnabled(true);
						System.out.println("ToggleBreaksItem enabled: " + enabled);
						editor.getGraphComponent().setPageBreaksVisible(true);
					
					}			
					else{
						setSelected(false);
						graph.setPageBreaksEnabled(false);
						System.out.println("ToggleBreaksItem enabled: " + enabled);
						editor.getGraphComponent().setPageBreaksVisible(false);
					}
					
					editor.getGraphComponent().updateUI();
					editor.getGraphComponent().refresh();
				}
			});
		}
	}
	
	/**
	 *
	 */
	@SuppressWarnings("serial")
	public static class ToggleShowingAOCRelation extends JCheckBoxMenuItem
	{
		/**
		 * 
		 */
		public ToggleShowingAOCRelation(final BasicGraphEditor editor, String name)
		{
			super(name);
			mxGraph graph = editor.getGraphComponent().getGraph();	
			setSelected(graph.isShowingAOCRelationEnabled());
			editor.getGraphComponent().updateUI();
			editor.getGraphComponent().refresh();			
			addActionListener(new ActionListener()
			{
				/**
				 * 
				 */
				public void actionPerformed(ActionEvent e)
				{	
					mxGraph graph = editor.getGraphComponent().getGraph();	
					boolean enabled = graph.isShowingAOCRelationEnabled();					
					//show aoc relations
					if(!enabled){					
						setSelected(true);
						graph.setShowingAOCRelationEnabled(true);
						System.out.println("ToggleShowingAOCRelation enabled: " + true);
						if(graph.isShowingCardinalityEnabled()){
							for(mxCell cell : graph.getAOCRelationListWithCardinalities()){
								graph.getModel().setStyle(cell, mxConstants.AOCRELATION);	
							}	
						}
						else{
							for(mxCell cell : graph.getAOCRelationListWithoutCardinalities()){
								graph.getModel().setStyle(cell, mxConstants.AOCRELATION);	
							}	
						}
					}			
					else{
						setSelected(false);
						graph.setShowingAOCRelationEnabled(false);
						System.out.println("ToggleShowingAOCRelation enabled: " + false);
						
						for(mxCell cell : graph.getAOCRelationListWithCardinalities()){
							graph.getModel().setStyle(cell, mxConstants.TRANSPARENTEDGE);	
						}						
					}					
					editor.getGraphComponent().updateUI();
					editor.getGraphComponent().refresh();
				}
			});
		}
	}
	
	/**
	 *
	 */
	@SuppressWarnings("serial")
	public static class ToggleShowingCrelRelation extends JCheckBoxMenuItem
	{
		/**
		 * 
		 */
		public ToggleShowingCrelRelation(final BasicGraphEditor editor, String name)
		{
			super(name);
			mxGraph graph = editor.getGraphComponent().getGraph();	
			setSelected(graph.isShowingCrelRelationEnabled());
			editor.getGraphComponent().updateUI();
			editor.getGraphComponent().refresh();			
			addActionListener(new ActionListener()
			{
				/**
				 * 
				 */
				public void actionPerformed(ActionEvent e)
				{	
					mxGraph graph = editor.getGraphComponent().getGraph();	
					boolean enabled = graph.isShowingCrelRelationEnabled();					
					//show crel relations
					if(!enabled){					
						setSelected(true);
						graph.setShowingCrelRelationEnabled(true);
						System.out.println("ToggleShowingCrelRelation enabled: " + true);
						for(mxCell cell : graph.getCrelRelationList()){
							mxCell activityEdgeConnector = (mxCell) cell.getSource();
							mxCell activityEdge = (mxCell) activityEdgeConnector.getParent();

							if(!graph.getModel().getStyle(activityEdge).equals(mxConstants.TRANSPARENTEDGE)){
								graph.getModel().setStyle(cell, mxConstants.CRELRELATION);	
							}							
						}						
					}			
					else{
						setSelected(false);
						graph.setShowingCrelRelationEnabled(false);
						System.out.println("ToggleShowingCrelRelation enabled: " + false);
						
						for(mxCell cell : graph.getCrelRelationList()){
							graph.getModel().setStyle(cell, mxConstants.TRANSPARENTEDGE);	
						}						
					}					
					editor.getGraphComponent().updateUI();
					editor.getGraphComponent().refresh();
				}
			});
		}
				
		/**
		 *
		 */
		@SuppressWarnings("serial")
		public static class ToggleShowingCardinalityRelation extends JCheckBoxMenuItem
		{
			/**
			 * 
			 */
			public ToggleShowingCardinalityRelation(final BasicGraphEditor editor, String name)
			{
				super(name);
				mxGraph graph = editor.getGraphComponent().getGraph();	
				setSelected(graph.isShowingCardinalityEnabled());
				editor.getGraphComponent().updateUI();
				editor.getGraphComponent().refresh();			
				addActionListener(new ActionListener()
				{
					/**
					 * 
					 */
					public void actionPerformed(ActionEvent e)
					{	
						mxGraph graph = editor.getGraphComponent().getGraph();	
						boolean enabled = graph.isShowingCardinalityEnabled();					
						//show cardinalitys
						if(!enabled){					
							setSelected(true);
							graph.setShowingCardinalityEnabled(true);
							System.out.println("ToggleShowingCardinality enabled: " + true);
							for(mxCell cell : graph.getCardinalities()){
								graph.getModel().setStyle(cell, mxConstants.CARDINALITY);	
							}						
						}			
						else{
							setSelected(false);
							graph.setShowingCardinalityEnabled(false);
							System.out.println("ToggleShowingCardinality enabled: " + false);
							
							for(mxCell cell : graph.getCardinalities()){
								graph.getModel().setStyle(cell, mxConstants.TRANSPARENTCARDINALITY);	
							}						
						}					
						editor.getGraphComponent().updateUI();
						editor.getGraphComponent().refresh();
					}
				});
			}
		}
				
		/**
		 *
		 */
		@SuppressWarnings("serial")
		public static class ToggleShowingEdgeName extends JCheckBoxMenuItem
		{
			/**
			 * 
			 */
			public ToggleShowingEdgeName(final BasicGraphEditor editor, String name)
			{
				super(name);
				mxGraph graph = editor.getGraphComponent().getGraph();	
				setSelected(graph.isShowingNameEnabled());
				editor.getGraphComponent().updateUI();
				editor.getGraphComponent().refresh();			
				addActionListener(new ActionListener()
				{
					/**
					 * 
					 */
					public void actionPerformed(ActionEvent e)
					{	
						mxGraph graph = editor.getGraphComponent().getGraph();	
						boolean enabled = graph.isShowingNameEnabled();					
						//show the edge names
						if(!enabled){					
							setSelected(true);
							graph.setShowingNameEnabled(true);
							System.out.println("ToggleShowingName enabled: " + true);
							if(graph.isShowingAOCRelationEnabled()){
								if(graph.isShowingCardinalityEnabled()){
									for(mxCell cell : graph.getAOCRelationListWithCardinalities()){
										graph.getModel().setStyle(cell, mxConstants.AOCRELATION);	
									}	
								}
								else{
									for(mxCell cell : graph.getAOCRelationListWithoutCardinalities()){
										graph.getModel().setStyle(cell, mxConstants.AOCRELATION);	
									}	
								}
							}

							if(graph.isShowingCardinalityEnabled()){
								for(mxCell cell : graph.getClassRelationListWithCardinalities()){
									graph.getModel().setStyle(cell, mxConstants.CLASSRELATION);	
								}	
							}
							else{
								for(mxCell cell : graph.getClassRelationListWithoutCardinalities()){
									graph.getModel().setStyle(cell, mxConstants.CLASSRELATION);	
								}	
							}

						}			
						else{
							setSelected(false);
							graph.setShowingNameEnabled(false);
							System.out.println("ToggleShowingName enabled: " + false);
							if(graph.isShowingAOCRelationEnabled()){
								for(mxCell cell : graph.getAOCRelationListWithCardinalities()){
									graph.getModel().setStyle(cell, mxConstants.TRANSPARENTAOCEDGENAME);	
								}	
							}
									
							for(mxCell cell : graph.getClassRelationListWithCardinalities()){
								graph.getModel().setStyle(cell, mxConstants.TRANSPARENTCLASSEDGENAME);	
							}
							if(graph.isShowingCardinalityEnabled()){
								for(mxCell cell : graph.getCardinalities()){
									graph.getModel().setStyle(cell, mxConstants.CARDINALITY);	
								}
							}

						}					
						editor.getGraphComponent().updateUI();
						editor.getGraphComponent().refresh();
					}
				});
			}
		}
	}

	
	/**
	 *
	 */
	@SuppressWarnings("serial")
	public static class ToggleShowingTransitiveRelation extends JCheckBoxMenuItem
	{
		/**
		 * 
		 */
		public ToggleShowingTransitiveRelation(final BasicGraphEditor editor, String name)
		{
			super(name);
			mxGraph graph = editor.getGraphComponent().getGraph();	
			setSelected(graph.isShowingAOCRelationEnabled());
			editor.getGraphComponent().updateUI();
			editor.getGraphComponent().refresh();			
			addActionListener(new ActionListener()
			{
				/**
				 * 
				 */
				public void actionPerformed(ActionEvent e)
				{	
					mxGraph graph = editor.getGraphComponent().getGraph();	
					boolean enabled = graph.isShowingTransitiveRelationEnabled();					
					//show transitive activity relations
					if(!enabled){					
						setSelected(true);
						graph.setShowingTransitiveRelationEnabled(true);
						System.out.println("ToggleShowingTransitiveRelation enabled: " + true);
						for(mxCell cell : graph.getTransitiveActivityRelationList()){
							System.out.println("set transitive relation as it should be");	
							String cardinalityType = cell.getAttributes().get(mxConstants.CARDINALITYTYPE);		
		    				mxCell connectorCell = (mxCell) cell.getChildAt(0);
		    				Object[] edgeCells = graph.getChildEdges(graph.getDefaultParent());
				    		for(int i=0; i<edgeCells.length; i++){
				    			mxCell edge = (mxCell) edgeCells[i];
				    			if(edge.getSource().equals(connectorCell) || edge.getTarget().equals(connectorCell)){
				    				graph.getModel().setStyle(edge, mxConstants.CRELRELATION);
				    			}				    			
				    		}
							graph.getModel().setStyle(cell, cardinalityType);	
						}	
					}			
					else{
						setSelected(false);
						graph.setShowingTransitiveRelationEnabled(false);
						System.out.println("ToggleShowingTransitiveRelation enabled: " + false);					
						for(mxCell cell : graph.getTransitiveActivityRelationList()){
							System.out.println("set transitive relation transparent");	
		    				mxCell connectorCell = (mxCell) cell.getChildAt(0);
		    				Object[] edgeCells = graph.getChildEdges(graph.getDefaultParent());
				    		for(int i=0; i<edgeCells.length; i++){
				    			mxCell edge = (mxCell) edgeCells[i];
				    			if(edge.getSource().equals(connectorCell) || edge.getTarget().equals(connectorCell)){
				    				graph.getModel().setStyle(edge, mxConstants.TRANSPARENTEDGE);
				    			}				    			
				    		}
				    		graph.getModel().setStyle(cell, mxConstants.TRANSPARENTEDGE);	
						}						
					}					
					editor.getGraphComponent().updateUI();
					editor.getGraphComponent().refresh();
				}
			});
		}
		
		/**
		 *
		 */
		public static class ToggleShowingNonRelation extends JCheckBoxMenuItem
		{
			/**
			 * 
			 */
			public ToggleShowingNonRelation(final BasicGraphEditor editor, String name)
			{
				super(name);
				mxGraph graph = editor.getGraphComponent().getGraph();	
				setSelected(graph.isShowingNonRelationEnabled());
				editor.getGraphComponent().updateUI();
				editor.getGraphComponent().refresh();			
				addActionListener(new ActionListener()
				{
					/**
					 * 
					 */
					public void actionPerformed(ActionEvent e)
					{	
						mxGraph graph = editor.getGraphComponent().getGraph();	
						boolean enabled = graph.isShowingNonRelationEnabled();					
						//show negative activity relations
						if(!enabled){					
							setSelected(true);
							graph.setShowingNonRelationEnabled(true);
							System.out.println("ToggleShowingNonRelation enabled: " + true);
							for(mxCell cell : graph.getNonActivityRelationList()){
								System.out.println("set Non relation as it should be");	
								String cardinalityType = cell.getAttributes().get(mxConstants.CARDINALITYTYPE);		
			    				mxCell connectorCell = (mxCell) cell.getChildAt(0);
			    				Object[] edgeCells = graph.getChildEdges(graph.getDefaultParent());
					    		for(int i=0; i<edgeCells.length; i++){
					    			mxCell edge = (mxCell) edgeCells[i];
					    			if(edge.getSource().equals(connectorCell) || edge.getTarget().equals(connectorCell)){
					    				graph.getModel().setStyle(edge, mxConstants.CRELRELATION);
					    			}				    			
					    		}
								graph.getModel().setStyle(cell, cardinalityType);	
							}	
						}			
						else{
							setSelected(false);
							graph.setShowingNonRelationEnabled(false);
							System.out.println("ToggleShowingNonRelation enabled: " + false);					
							for(mxCell cell : graph.getNonActivityRelationList()){
								System.out.println("set Non relation transparent");	
			    				mxCell connectorCell = (mxCell) cell.getChildAt(0);
			    				Object[] edgeCells = graph.getChildEdges(graph.getDefaultParent());
					    		for(int i=0; i<edgeCells.length; i++){
					    			mxCell edge = (mxCell) edgeCells[i];
					    			if(edge.getSource().equals(connectorCell) || edge.getTarget().equals(connectorCell)){
					    				graph.getModel().setStyle(edge, mxConstants.TRANSPARENTEDGE);
					    			}				    			
					    		}
					    		graph.getModel().setStyle(cell, mxConstants.TRANSPARENTEDGE);	
							}						
						}					
						editor.getGraphComponent().updateUI();
						editor.getGraphComponent().refresh();
					}
				});
			}
		}
	}

	
	/**
	 *
	 */
	@SuppressWarnings("serial")
	public static class ToggleShowingRelation extends JCheckBoxMenuItem
	{
		Object[] removedCells;
		/**
		 * 
		 */
		public ToggleShowingRelation(final BasicGraphEditor editor, String name, final String relationType)
		{
			super(name);
			mxGraph graph = editor.getGraphComponent().getGraph();	
			setSelected(graph.getRelationDisplayFlag(relationType));
			editor.getGraphComponent().updateUI();
			editor.getGraphComponent().refresh();			
			addActionListener(new ActionListener()
			{
				/**
				 * 
				 */
				public void actionPerformed(ActionEvent e)
				{	
					mxGraph graph = editor.getGraphComponent().getGraph();	
					boolean enabled = graph.getRelationDisplayFlag(relationType);					
					//show negative activity relations
					if(!enabled){					
						setSelected(true);
						graph.setRelationDisplayFlag(relationType, true);
						if(removedCells != null){
							graph.addCells(removedCells);
						}	
					}			
					else{
						setSelected(false);
						graph.setRelationDisplayFlag(relationType, false);				
						List<mxCell> removedCellList = new ArrayList<mxCell>();
						for(mxCell cell : graph.getActivityRelationList()){
							String cardinalityType = cell.getAttributes().get(mxConstants.CARDINALITYTYPE);	
							if(cardinalityType.equals(relationType)){
			    				mxCell connectorCell = (mxCell) cell.getChildAt(0);
			    				Object[] edgeCells = graph.getChildEdges(graph.getDefaultParent());
					    		for(int i=0; i<edgeCells.length; i++){
					    			mxCell edge = (mxCell) edgeCells[i];
					    			if(edge.getSource().equals(connectorCell) || edge.getTarget().equals(connectorCell)){
					    				removedCellList.add(edge);
					    			}				    			
					    		}
					    		removedCellList.add(cell);
							}
						}
						removedCells = removedCellList.toArray();
						graph.removeCells(removedCells);
					}					
					editor.getGraphComponent().updateUI();
					editor.getGraphComponent().refresh();
				}
			});
		}
	}
	
	/**
	 *
	 */
	@SuppressWarnings("serial")
	public static class ToggleShowingUnaryResponseRelation extends JCheckBoxMenuItem
	{
		/**
		 * 
		 */
		public ToggleShowingUnaryResponseRelation(final BasicGraphEditor editor, String name)
		{
			super(name);
			mxGraph graph = editor.getGraphComponent().getGraph();	
			setSelected(graph.isShowingUnaryResponseRelationEnabled());
			editor.getGraphComponent().updateUI();
			editor.getGraphComponent().refresh();			
			addActionListener(new ActionListener()
			{
				/**
				 * 
				 */
				public void actionPerformed(ActionEvent e)
				{	
					mxGraph graph = editor.getGraphComponent().getGraph();	
					boolean enabled = graph.isShowingUnaryResponseRelationEnabled();					
					if(!enabled){					
						setSelected(true);
						graph.setShowingUnaryResponseRelationEnabled(true);
						System.out.println("ToggleShowingUnaryResponseRelation enabled: " + true);
						for(mxCell cell : graph.getActivityRelationList()){
							System.out.println("set Unary Response relation as it should be");	
							String cardinalityType = cell.getAttributes().get(mxConstants.CARDINALITYTYPE);	
							if(cardinalityType.equals(mxConstants.UNARYRESPONSE)){
			    				mxCell connectorCell = (mxCell) cell.getChildAt(0);		    			
			    				Object[] edgeCells = graph.getChildEdges(graph.getDefaultParent());
					    		for(int i=0; i<edgeCells.length; i++){
					    			mxCell edge = (mxCell) edgeCells[i];
					    			if(edge.getSource().equals(connectorCell) || edge.getTarget().equals(connectorCell)){
					    				graph.getModel().setStyle(edge, mxConstants.CRELRELATION);
					    			}				    			
					    		}
								graph.getModel().setStyle(cell, cardinalityType);
							}
						}	
					}			
					else{
						setSelected(false);
						graph.setShowingUnaryResponseRelationEnabled(false);
						System.out.println("ToggleShowingUnaryResponseRelation enabled: " + false);					
						for(mxCell cell : graph.getActivityRelationList()){
							System.out.println("set Unary Response relation transparent");	
							String cardinalityType = cell.getAttributes().get(mxConstants.CARDINALITYTYPE);	
							if(cardinalityType.equals(mxConstants.UNARYRESPONSE)){
			    				mxCell connectorCell = (mxCell) cell.getChildAt(0);
			    				Object[] edgeCells = graph.getChildEdges(graph.getDefaultParent());
					    		for(int i=0; i<edgeCells.length; i++){
					    			mxCell edge = (mxCell) edgeCells[i];
					    			if(edge.getSource().equals(connectorCell) || edge.getTarget().equals(connectorCell)){
					    				graph.getModel().setStyle(edge, mxConstants.TRANSPARENTEDGE);
					    			}				    			
					    		}
					    		graph.getModel().setStyle(cell, mxConstants.TRANSPARENTEDGE);
							}
						}						
					}					
					editor.getGraphComponent().updateUI();
					editor.getGraphComponent().refresh();
				}
			});
		}
	}
	
	/**
	 *
	 */
	@SuppressWarnings("serial")
	public static class ToggleShowingNonResponseRelation extends JCheckBoxMenuItem
	{
		/**
		 * 
		 */
		public ToggleShowingNonResponseRelation(final BasicGraphEditor editor, String name)
		{
			super(name);
			mxGraph graph = editor.getGraphComponent().getGraph();	
			setSelected(graph.isShowingNonResponseRelationEnabled());
			editor.getGraphComponent().updateUI();
			editor.getGraphComponent().refresh();			
			addActionListener(new ActionListener()
			{
				/**
				 * 
				 */
				public void actionPerformed(ActionEvent e)
				{	
					mxGraph graph = editor.getGraphComponent().getGraph();	
					boolean enabled = graph.isShowingNonResponseRelationEnabled();					
					if(!enabled){					
						setSelected(true);
						graph.setShowingNonResponseRelationEnabled(true);
						System.out.println("ToggleShowingNonResponseRelation enabled: " + true);
						for(mxCell cell : graph.getActivityRelationList()){
							System.out.println("set Non Response relation as it should be");	
							String cardinalityType = cell.getAttributes().get(mxConstants.CARDINALITYTYPE);	
							if(cardinalityType.equals(mxConstants.NONRESPONSE)){
			    				mxCell connectorCell = (mxCell) cell.getChildAt(0);		    			
			    				Object[] edgeCells = graph.getChildEdges(graph.getDefaultParent());
					    		for(int i=0; i<edgeCells.length; i++){
					    			mxCell edge = (mxCell) edgeCells[i];
					    			if(edge.getSource().equals(connectorCell) || edge.getTarget().equals(connectorCell)){
					    				graph.getModel().setStyle(edge, mxConstants.CRELRELATION);
					    			}				    			
					    		}
								graph.getModel().setStyle(cell, cardinalityType);
							}
						}	
					}			
					else{
						setSelected(false);
						graph.setShowingNonResponseRelationEnabled(false);
						System.out.println("ToggleShowingNonResponseRelation enabled: " + false);					
						for(mxCell cell : graph.getActivityRelationList()){
							System.out.println("set Non Response relation transparent");	
							String cardinalityType = cell.getAttributes().get(mxConstants.CARDINALITYTYPE);	
							if(cardinalityType.equals(mxConstants.NONRESPONSE)){
			    				mxCell connectorCell = (mxCell) cell.getChildAt(0);
			    				Object[] edgeCells = graph.getChildEdges(graph.getDefaultParent());
					    		for(int i=0; i<edgeCells.length; i++){
					    			mxCell edge = (mxCell) edgeCells[i];
					    			if(edge.getSource().equals(connectorCell) || edge.getTarget().equals(connectorCell)){
					    				graph.getModel().setStyle(edge, mxConstants.TRANSPARENTEDGE);
					    			}				    			
					    		}
					    		graph.getModel().setStyle(cell, mxConstants.TRANSPARENTEDGE);
							}
						}						
					}					
					editor.getGraphComponent().updateUI();
					editor.getGraphComponent().refresh();
				}
			});
		}
	}
	
	/**
	 *
	 */
	@SuppressWarnings("serial")
	public static class ToggleShowingPrecedenceRelation extends JCheckBoxMenuItem
	{
		/**
		 * 
		 */
		public ToggleShowingPrecedenceRelation(final BasicGraphEditor editor, String name)
		{
			super(name);
			mxGraph graph = editor.getGraphComponent().getGraph();	
			setSelected(graph.isShowingPrecedenceRelationEnabled());
			editor.getGraphComponent().updateUI();
			editor.getGraphComponent().refresh();			
			addActionListener(new ActionListener()
			{
				/**
				 * 
				 */
				public void actionPerformed(ActionEvent e)
				{	
					mxGraph graph = editor.getGraphComponent().getGraph();	
					boolean enabled = graph.isShowingPrecedenceRelationEnabled();					
					if(!enabled){					
						setSelected(true);
						graph.setShowingPrecedenceRelationEnabled(true);
						System.out.println("ToggleShowingPrecedenceRelation enabled: " + true);
						for(mxCell cell : graph.getActivityRelationList()){
							System.out.println("set Precedence relation as it should be");	
							String cardinalityType = cell.getAttributes().get(mxConstants.CARDINALITYTYPE);	
							if(cardinalityType.equals(mxConstants.PRECEDENCE)){
			    				mxCell connectorCell = (mxCell) cell.getChildAt(0);		    			
			    				Object[] edgeCells = graph.getChildEdges(graph.getDefaultParent());
					    		for(int i=0; i<edgeCells.length; i++){
					    			mxCell edge = (mxCell) edgeCells[i];
					    			if(edge.getSource().equals(connectorCell) || edge.getTarget().equals(connectorCell)){
					    				graph.getModel().setStyle(edge, mxConstants.CRELRELATION);
					    			}				    			
					    		}
								graph.getModel().setStyle(cell, cardinalityType);
							}
						}	
					}			
					else{
						setSelected(false);
						graph.setShowingPrecedenceRelationEnabled(false);
						System.out.println("ToggleShowingPrecedenceRelation enabled: " + false);					
						for(mxCell cell : graph.getActivityRelationList()){
							System.out.println("set Precedence relation transparent");	
							String cardinalityType = cell.getAttributes().get(mxConstants.CARDINALITYTYPE);	
							if(cardinalityType.equals(mxConstants.PRECEDENCE)){
			    				mxCell connectorCell = (mxCell) cell.getChildAt(0);
			    				Object[] edgeCells = graph.getChildEdges(graph.getDefaultParent());
					    		for(int i=0; i<edgeCells.length; i++){
					    			mxCell edge = (mxCell) edgeCells[i];
					    			if(edge.getSource().equals(connectorCell) || edge.getTarget().equals(connectorCell)){
					    				graph.getModel().setStyle(edge, mxConstants.TRANSPARENTEDGE);
					    			}				    			
					    		}
					    		graph.getModel().setStyle(cell, mxConstants.TRANSPARENTEDGE);
							}
						}						
					}					
					editor.getGraphComponent().updateUI();
					editor.getGraphComponent().refresh();
				}
			});
		}
	}
	
	/**
	 *
	 */
	@SuppressWarnings("serial")
	public static class ToggleShowingUnaryPrecedenceRelation extends JCheckBoxMenuItem
	{
		/**
		 * 
		 */
		public ToggleShowingUnaryPrecedenceRelation(final BasicGraphEditor editor, String name)
		{
			super(name);
			mxGraph graph = editor.getGraphComponent().getGraph();	
			setSelected(graph.isShowingUnaryPrecedenceRelationEnabled());
			editor.getGraphComponent().updateUI();
			editor.getGraphComponent().refresh();			
			addActionListener(new ActionListener()
			{
				/**
				 * 
				 */
				public void actionPerformed(ActionEvent e)
				{	
					mxGraph graph = editor.getGraphComponent().getGraph();	
					boolean enabled = graph.isShowingUnaryPrecedenceRelationEnabled();					
					if(!enabled){					
						setSelected(true);
						graph.setShowingUnaryPrecedenceRelationEnabled(true);
						System.out.println("ToggleShowingUnaryPrecedenceRelation enabled: " + true);
						for(mxCell cell : graph.getActivityRelationList()){
							System.out.println("set Unary Precedence relation as it should be");	
							String cardinalityType = cell.getAttributes().get(mxConstants.CARDINALITYTYPE);	
							if(cardinalityType.equals(mxConstants.UNARYPRECEDENCE)){
			    				mxCell connectorCell = (mxCell) cell.getChildAt(0);		    			
			    				Object[] edgeCells = graph.getChildEdges(graph.getDefaultParent());
					    		for(int i=0; i<edgeCells.length; i++){
					    			mxCell edge = (mxCell) edgeCells[i];
					    			if(edge.getSource().equals(connectorCell) || edge.getTarget().equals(connectorCell)){
					    				graph.getModel().setStyle(edge, mxConstants.CRELRELATION);
					    			}				    			
					    		}
								graph.getModel().setStyle(cell, cardinalityType);
							}
						}	
					}			
					else{
						setSelected(false);
						graph.setShowingUnaryPrecedenceRelationEnabled(false);
						System.out.println("ToggleShowingUnaryPrecedenceRelation enabled: " + false);					
						for(mxCell cell : graph.getActivityRelationList()){
							System.out.println("set Unary Precedence relation transparent");	
							String cardinalityType = cell.getAttributes().get(mxConstants.CARDINALITYTYPE);	
							if(cardinalityType.equals(mxConstants.UNARYPRECEDENCE)){
			    				mxCell connectorCell = (mxCell) cell.getChildAt(0);
			    				Object[] edgeCells = graph.getChildEdges(graph.getDefaultParent());
					    		for(int i=0; i<edgeCells.length; i++){
					    			mxCell edge = (mxCell) edgeCells[i];
					    			if(edge.getSource().equals(connectorCell) || edge.getTarget().equals(connectorCell)){
					    				graph.getModel().setStyle(edge, mxConstants.TRANSPARENTEDGE);
					    			}				    			
					    		}
					    		graph.getModel().setStyle(cell, mxConstants.TRANSPARENTEDGE);
							}
						}						
					}					
					editor.getGraphComponent().updateUI();
					editor.getGraphComponent().refresh();
				}
			});
		}
	}
	
	/**
	 *
	 */
	@SuppressWarnings("serial")
	public static class ToggleShowingNonPrecedenceRelation extends JCheckBoxMenuItem
	{
		/**
		 * 
		 */
		public ToggleShowingNonPrecedenceRelation(final BasicGraphEditor editor, String name)
		{
			super(name);
			mxGraph graph = editor.getGraphComponent().getGraph();	
			setSelected(graph.isShowingNonPrecedenceRelationEnabled());
			editor.getGraphComponent().updateUI();
			editor.getGraphComponent().refresh();			
			addActionListener(new ActionListener()
			{
				/**
				 * 
				 */
				public void actionPerformed(ActionEvent e)
				{	
					mxGraph graph = editor.getGraphComponent().getGraph();	
					boolean enabled = graph.isShowingNonPrecedenceRelationEnabled();					
					if(!enabled){					
						setSelected(true);
						graph.setShowingNonPrecedenceRelationEnabled(true);
						System.out.println("ToggleShowingNonPrecedenceRelation enabled: " + true);
						for(mxCell cell : graph.getActivityRelationList()){
							System.out.println("set Non Precedence relation as it should be");	
							String cardinalityType = cell.getAttributes().get(mxConstants.CARDINALITYTYPE);	
							if(cardinalityType.equals(mxConstants.NONPRECEDENCE)){
			    				mxCell connectorCell = (mxCell) cell.getChildAt(0);		    			
			    				Object[] edgeCells = graph.getChildEdges(graph.getDefaultParent());
					    		for(int i=0; i<edgeCells.length; i++){
					    			mxCell edge = (mxCell) edgeCells[i];
					    			if(edge.getSource().equals(connectorCell) || edge.getTarget().equals(connectorCell)){
					    				graph.getModel().setStyle(edge, mxConstants.CRELRELATION);
					    			}				    			
					    		}
								graph.getModel().setStyle(cell, cardinalityType);
							}
						}	
					}			
					else{
						setSelected(false);
						graph.setShowingNonPrecedenceRelationEnabled(false);
						System.out.println("ToggleShowingNonPrecedenceRelation enabled: " + false);					
						for(mxCell cell : graph.getActivityRelationList()){
							System.out.println("set Non Precedence relation transparent");	
							String cardinalityType = cell.getAttributes().get(mxConstants.CARDINALITYTYPE);	
							if(cardinalityType.equals(mxConstants.NONPRECEDENCE)){
			    				mxCell connectorCell = (mxCell) cell.getChildAt(0);
			    				Object[] edgeCells = graph.getChildEdges(graph.getDefaultParent());
					    		for(int i=0; i<edgeCells.length; i++){
					    			mxCell edge = (mxCell) edgeCells[i];
					    			if(edge.getSource().equals(connectorCell) || edge.getTarget().equals(connectorCell)){
					    				graph.getModel().setStyle(edge, mxConstants.TRANSPARENTEDGE);
					    			}				    			
					    		}
					    		graph.getModel().setStyle(cell, mxConstants.TRANSPARENTEDGE);
							}
						}						
					}					
					editor.getGraphComponent().updateUI();
					editor.getGraphComponent().refresh();
				}
			});
		}
	}
	
	/**
	 *
	 */
	@SuppressWarnings("serial")
	public static class ToggleShowingCoExistenceRelation extends JCheckBoxMenuItem
	{
		/**
		 * 
		 */
		public ToggleShowingCoExistenceRelation(final BasicGraphEditor editor, String name)
		{
			super(name);
			mxGraph graph = editor.getGraphComponent().getGraph();	
			setSelected(graph.isShowingCoExistenceRelationEnabled());
			editor.getGraphComponent().updateUI();
			editor.getGraphComponent().refresh();			
			addActionListener(new ActionListener()
			{
				/**
				 * 
				 */
				public void actionPerformed(ActionEvent e)
				{	
					mxGraph graph = editor.getGraphComponent().getGraph();	
					boolean enabled = graph.isShowingCoExistenceRelationEnabled();					
					if(!enabled){					
						setSelected(true);
						graph.setShowingCoExistenceRelationEnabled(true);
						System.out.println("ToggleShowingCoExistenceRelation enabled: " + true);
						for(mxCell cell : graph.getActivityRelationList()){
							System.out.println("set CoExistence relation as it should be");	
							String cardinalityType = cell.getAttributes().get(mxConstants.CARDINALITYTYPE);	
							if(cardinalityType.equals(mxConstants.COEXISTENCE)){
			    				mxCell connectorCell = (mxCell) cell.getChildAt(0);		    			
			    				Object[] edgeCells = graph.getChildEdges(graph.getDefaultParent());
					    		for(int i=0; i<edgeCells.length; i++){
					    			mxCell edge = (mxCell) edgeCells[i];
					    			if(edge.getSource().equals(connectorCell) || edge.getTarget().equals(connectorCell)){
					    				graph.getModel().setStyle(edge, mxConstants.CRELRELATION);
					    			}				    			
					    		}
								graph.getModel().setStyle(cell, cardinalityType);
							}
						}	
					}			
					else{
						setSelected(false);
						graph.setShowingCoExistenceRelationEnabled(false);
						System.out.println("ToggleShowingCoExistenceRelation enabled: " + false);					
						for(mxCell cell : graph.getActivityRelationList()){
							System.out.println("set CoExistence relation transparent");	
							String cardinalityType = cell.getAttributes().get(mxConstants.CARDINALITYTYPE);	
							if(cardinalityType.equals(mxConstants.COEXISTENCE)){
			    				mxCell connectorCell = (mxCell) cell.getChildAt(0);
			    				Object[] edgeCells = graph.getChildEdges(graph.getDefaultParent());
					    		for(int i=0; i<edgeCells.length; i++){
					    			mxCell edge = (mxCell) edgeCells[i];
					    			if(edge.getSource().equals(connectorCell) || edge.getTarget().equals(connectorCell)){
					    				graph.getModel().setStyle(edge, mxConstants.TRANSPARENTEDGE);
					    			}				    			
					    		}
					    		graph.getModel().setStyle(cell, mxConstants.TRANSPARENTEDGE);
							}
						}						
					}					
					editor.getGraphComponent().updateUI();
					editor.getGraphComponent().refresh();
				}
			});
		}
	}
	
	/**
	 *
	 */
	@SuppressWarnings("serial")
	public static class ToggleShowingNonCoExistenceRelation extends JCheckBoxMenuItem
	{
		/**
		 * 
		 */
		public ToggleShowingNonCoExistenceRelation(final BasicGraphEditor editor, String name)
		{
			super(name);
			mxGraph graph = editor.getGraphComponent().getGraph();	
			setSelected(graph.isShowingNonCoExistenceRelationEnabled());
			editor.getGraphComponent().updateUI();
			editor.getGraphComponent().refresh();			
			addActionListener(new ActionListener()
			{
				/**
				 * 
				 */
				public void actionPerformed(ActionEvent e)
				{	
					mxGraph graph = editor.getGraphComponent().getGraph();	
					boolean enabled = graph.isShowingNonCoExistenceRelationEnabled();					
					if(!enabled){					
						setSelected(true);
						graph.setShowingNonCoExistenceRelationEnabled(true);
						System.out.println("ToggleShowingNonCoExistenceRelation enabled: " + true);
						for(mxCell cell : graph.getActivityRelationList()){
							System.out.println("set NonCoExistence relation as it should be");	
							String cardinalityType = cell.getAttributes().get(mxConstants.CARDINALITYTYPE);	
							if(cardinalityType.equals(mxConstants.NONCOEXISTENCE)){
			    				mxCell connectorCell = (mxCell) cell.getChildAt(0);		    			
			    				Object[] edgeCells = graph.getChildEdges(graph.getDefaultParent());
					    		for(int i=0; i<edgeCells.length; i++){
					    			mxCell edge = (mxCell) edgeCells[i];
					    			if(edge.getSource().equals(connectorCell) || edge.getTarget().equals(connectorCell)){
					    				graph.getModel().setStyle(edge, mxConstants.CRELRELATION);
					    			}				    			
					    		}
								graph.getModel().setStyle(cell, cardinalityType);
							}
						}	
					}			
					else{
						setSelected(false);
						graph.setShowingNonCoExistenceRelationEnabled(false);
						System.out.println("ToggleShowingNonCoExistenceRelation enabled: " + false);					
						for(mxCell cell : graph.getActivityRelationList()){
							System.out.println("set NonCoExistence relation transparent");	
							String cardinalityType = cell.getAttributes().get(mxConstants.CARDINALITYTYPE);	
							if(cardinalityType.equals(mxConstants.NONCOEXISTENCE)){
			    				mxCell connectorCell = (mxCell) cell.getChildAt(0);
			    				Object[] edgeCells = graph.getChildEdges(graph.getDefaultParent());
					    		for(int i=0; i<edgeCells.length; i++){
					    			mxCell edge = (mxCell) edgeCells[i];
					    			if(edge.getSource().equals(connectorCell) || edge.getTarget().equals(connectorCell)){
					    				graph.getModel().setStyle(edge, mxConstants.TRANSPARENTEDGE);
					    			}				    			
					    		}
					    		graph.getModel().setStyle(cell, mxConstants.TRANSPARENTEDGE);
							}
						}						
					}					
					editor.getGraphComponent().updateUI();
					editor.getGraphComponent().refresh();
				}
			});
		}
	}
	
	/**
	 *
	 */
	@SuppressWarnings("serial")
	public static class generateInformationSystem extends AbstractAction
	{
		
		mxGraph graph;
		BasicGraphEditor editor;
		public generateInformationSystem(mxGraph graph, BasicGraphEditor editor){
			this.graph = graph;
			this.editor = editor;
		}
		/**
		 * 
		 */
		public void actionPerformed(ActionEvent e)
		{	
			JFrame frame = (JFrame) SwingUtilities.windowForComponent(editor);

			if (frame != null)
			{
				GenerateInformationSystemFrame informationSystem = new GenerateInformationSystemFrame(frame, graph);
				informationSystem.setModal(true);

				// Centers inside the application frame
				int x = frame.getX() + (frame.getWidth() - informationSystem.getWidth()) / 2;
				int y = frame.getY() + (frame.getHeight() - informationSystem.getHeight()) / 2;
				informationSystem.setLocation(x, y);

				// Shows the modal dialog and waits
				informationSystem.setVisible(true);
			}
		}
	}
}
