package org.processmining.mxgraph.examples.swing.editor;

import javax.swing.JMenu;
import javax.swing.JPopupMenu;
import javax.swing.TransferHandler;

import org.processmining.mxgraph.examples.swing.editor.EditorActions.HistoryAction;
import org.processmining.mxgraph.examples.swing.editor.EditorActions.RefreshModelAction;
import org.processmining.mxgraph.examples.swing.editor.EditorActions.SwapSourceAndTargetAction;
import org.processmining.mxgraph.examples.swing.editor.EditorActions.ToggleCheckItem;
import org.processmining.mxgraph.examples.swing.editor.EditorActions.ToggleConnectorItem;
import org.processmining.mxgraph.model.mxCell;
import org.processmining.mxgraph.swing.util.mxGraphActions;
import org.processmining.mxgraph.util.mxConstants;
import org.processmining.mxgraph.util.mxResources;

public class EditorPopupMenu extends JPopupMenu
{

	/**
	 * 
	 */
	private static final long serialVersionUID = -3132749140550242191L;

	public EditorPopupMenu(BasicGraphEditor editor)
	{
		
		boolean selected = !editor.getGraphComponent().getGraph()
				.isSelectionEmpty();
		
		boolean classORActivityEdgeselected = false;
		if(selected){
			mxCell selectedCell = (mxCell) editor.getGraphComponent().getGraph().getSelectionCell();
			classORActivityEdgeselected = selectedCell.getAttributes().get(mxConstants.TYPE).equals(mxConstants.CLASSRELATION) ||
					selectedCell.getAttributes().get(mxConstants.TYPE).equals(mxConstants.ACTIVITYRELATION);
		}
				
		add(new ToggleConnectorItem(editor, "Connector"));
		
		addSeparator();
		
		add(new ToggleCheckItem(editor, "Check"));
		
		addSeparator();
		
		add(editor.bind("Refresh", new RefreshModelAction()));
		
		addSeparator();
		
		add(editor.bind("Swap S/T", new SwapSourceAndTargetAction())).setEnabled(classORActivityEdgeselected);;
		
		addSeparator();
		
		add(editor.bind(mxResources.get("undo"), new HistoryAction(true),
				"/org/processmining/mxgraph/examples/swing/images/undo.gif"));

		addSeparator();

		add(
				editor.bind(mxResources.get("cut"), TransferHandler
						.getCutAction(),
						"/org/processmining/mxgraph/examples/swing/images/cut.gif"))
				.setEnabled(selected);
		add(
				editor.bind(mxResources.get("copy"), TransferHandler
						.getCopyAction(),
						"/org/processmining/mxgraph/examples/swing/images/copy.gif"))
				.setEnabled(selected);
		add(editor.bind(mxResources.get("paste"), TransferHandler
				.getPasteAction(),
				"/org/processmining/mxgraph/examples/swing/images/paste.gif"));

		addSeparator();

		add(
				editor.bind(mxResources.get("delete"), mxGraphActions
						.getDeleteAction(),
						"/org/processmining/mxgraph/examples/swing/images/delete.gif"))
				.setEnabled(selected);

		addSeparator();

		// Creates the format menu
		JMenu menu = (JMenu) add(new JMenu(mxResources.get("format")));

		EditorMenuBar.populateFormatMenu(menu, editor);

//		// Creates the shape menu
//		menu = (JMenu) add(new JMenu(mxResources.get("shape")));
//
//		EditorMenuBar.populateShapeMenu(menu, editor);

		addSeparator();

//		add(
//				editor.bind(mxResources.get("edit"), mxGraphActions
//						.getEditAction())).setEnabled(selected);
//
//		addSeparator();

		add(editor.bind(mxResources.get("selectVertices"), mxGraphActions
				.getSelectVerticesAction()));
//		add(editor.bind(mxResources.get("selectEdges"), mxGraphActions
//				.getSelectEdgesAction()));

		addSeparator();

		add(editor.bind(mxResources.get("selectAll"), mxGraphActions
				.getSelectAllAction()));
	}

}
