package org.processmining.mxgraph.examples.swing.editor;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.JComboBox;
import javax.swing.JOptionPane;
import javax.swing.JToolBar;
import javax.swing.TransferHandler;

import org.processmining.mxgraph.examples.swing.editor.EditorActions.HistoryAction;
import org.processmining.mxgraph.examples.swing.editor.EditorActions.NewAction;
import org.processmining.mxgraph.examples.swing.editor.EditorActions.OpenAction;
import org.processmining.mxgraph.examples.swing.editor.EditorActions.PrintAction;
import org.processmining.mxgraph.examples.swing.editor.EditorActions.SaveAction;
import org.processmining.mxgraph.examples.swing.visualization.Style;
import org.processmining.mxgraph.model.mxCell;
import org.processmining.mxgraph.swing.mxGraphComponent;
import org.processmining.mxgraph.swing.util.mxGraphActions;
import org.processmining.mxgraph.util.mxConstants;
import org.processmining.mxgraph.util.mxEvent;
import org.processmining.mxgraph.util.mxEventObject;
import org.processmining.mxgraph.util.mxEventSource.mxIEventListener;
import org.processmining.mxgraph.util.mxResources;
import org.processmining.mxgraph.view.mxGraph;
import org.processmining.mxgraph.view.mxGraphView;

public class EditorToolBar extends JToolBar
{

	/**
	 * 
	 */
	private static final long serialVersionUID = -8015443128436394471L;

	/**
	 * 
	 * @param frame
	 * @param orientation
	 */
	private boolean ignoreZoomChange = false;

	private JComboBox<String> zoomCombo;
	private JComboBox<String> activityCombo;
	private JComboBox<String> sourceAlwaysCombo;
	private JComboBox<String> sourceEventuallyCombo;
	private JComboBox<String> targetAlwaysCombo;
	private JComboBox<String> targetEventuallyCombo;
	private JComboBox<String> edgeStyleCombo;
//	final JButton showConnectorButton;
//	final JButton checkButton;//used to check if the designed model is legal
//	final JButton refreshButton;//used to refresh the model
//	final JButton swapSTButton;//used to swap the source and target nodes
//	final JButton pageNumButton;//used to swap the source and target nodes
	
	public JComboBox<String> getZoomBox(){
		return zoomCombo;
	}
	
	public JComboBox<String> getCardinalityTypeBox(){
		return activityCombo;
	}
	
	public JComboBox<String> getCardinalitySourceAlwaysBox(){
		return sourceAlwaysCombo;
	}
	
	public JComboBox<String> getCardinalitySourceEventuallyBox(){
		return sourceEventuallyCombo;
	}
	
	public JComboBox<String> getCardinalityTargetAlwaysBox(){
		return targetAlwaysCombo;
	}
	
	public JComboBox<String> getCardinalityTargetEventuallyBox(){
		return targetEventuallyCombo;
	}
	
	public JComboBox<String> getEdgeStyleBox(){
		return edgeStyleCombo;
	}
	
	/**
	 * 
	 */
	public EditorToolBar(final BasicGraphEditor editor, int orientation)
	{
		super(orientation);
		setBorder(BorderFactory.createCompoundBorder(BorderFactory
				.createEmptyBorder(3, 3, 3, 3), getBorder()));
		setFloatable(false);

		add(editor.bind("New", new NewAction(),
				"/org/processmining/mxgraph/examples/swing/images/new.gif"));
		add(editor.bind("Open", new OpenAction(),
				"/org/processmining/mxgraph/examples/swing/images/open.gif"));
		add(editor.bind("Save", new SaveAction(false),
				"/org/processmining/mxgraph/examples/swing/images/save.gif"));

//		addSeparator();
//
//		add(editor.bind("Save to workspace", new SaveAction(false),
//				"/org/processmining/mxgraph/examples/swing/images/workspace1.png"));
		
		addSeparator();

		add(editor.bind("Print", new PrintAction(),
				"/org/processmining/mxgraph/examples/swing/images/print.gif"));

		addSeparator();

		add(editor.bind("Cut", TransferHandler.getCutAction(),
				"/org/processmining/mxgraph/examples/swing/images/cut.gif"));
		add(editor.bind("Copy", TransferHandler.getCopyAction(),
				"/org/processmining/mxgraph/examples/swing/images/copy.gif"));
		add(editor.bind("Paste", TransferHandler.getPasteAction(),
				"/org/processmining/mxgraph/examples/swing/images/paste.gif"));

		addSeparator();

		add(editor.bind("Delete", mxGraphActions.getDeleteAction(),
				"/org/processmining/mxgraph/examples/swing/images/delete.gif"));

		addSeparator();

		add(editor.bind("Undo", new HistoryAction(true),
				"/org/processmining/mxgraph/examples/swing/images/undo.gif"));
		add(editor.bind("Redo", new HistoryAction(false),
				"/org/processmining/mxgraph/examples/swing/images/redo.gif"));

		addSeparator();
//
//		// Gets the list of available fonts from the local graphics environment
//		// and adds some frequently used fonts at the beginning of the list
//		GraphicsEnvironment env = GraphicsEnvironment
//				.getLocalGraphicsEnvironment();
//		List<String> fonts = new ArrayList<String>();
//		fonts.addAll(Arrays.asList(new String[] { "Helvetica", "Verdana",
//				"Times New Roman", "Garamond", "Courier New", "-" }));
//		fonts.addAll(Arrays.asList(env.getAvailableFontFamilyNames()));
//
//		final JComboBox fontCombo = new JComboBox(fonts.toArray());
//		fontCombo.setEditable(true);
//		fontCombo.setMinimumSize(new Dimension(120, 0));
//		fontCombo.setPreferredSize(new Dimension(120, 0));
//		fontCombo.setMaximumSize(new Dimension(120, 100));
//		add(fontCombo);
//
//		fontCombo.addActionListener(new ActionListener()
//		{
//			/**
//			 * 
//			 */
//			public void actionPerformed(ActionEvent e)
//			{
//				String font = fontCombo.getSelectedItem().toString();
//
//				if (font != null && !font.equals("-"))
//				{
//					mxGraph graph = editor.getGraphComponent().getGraph();
//					graph.setCellStyles(mxConstants.STYLE_FONTFAMILY, font);
//				}
//			}
//		});
//
//		final JComboBox sizeCombo = new JComboBox(new Object[] { "6pt", "8pt",
//				"9pt", "10pt", "12pt", "14pt", "18pt", "24pt", "30pt", "36pt",
//				"48pt", "60pt" });
//		sizeCombo.setEditable(true);
//		sizeCombo.setMinimumSize(new Dimension(65, 0));
//		sizeCombo.setPreferredSize(new Dimension(65, 0));
//		sizeCombo.setMaximumSize(new Dimension(65, 100));
//		add(sizeCombo);
//
//		sizeCombo.addActionListener(new ActionListener()
//		{
//			/**
//			 * 
//			 */
//			public void actionPerformed(ActionEvent e)
//			{
//				mxGraph graph = editor.getGraphComponent().getGraph();
//				graph.setCellStyles(mxConstants.STYLE_FONTSIZE, sizeCombo
//						.getSelectedItem().toString().replace("pt", ""));
//			}
//		});
//
//		addSeparator();
//
//		add(editor.bind("Bold", new FontStyleAction(true),
//				"/org/processmining/mxgraph/examples/swing/images/bold.gif"));
//		add(editor.bind("Italic", new FontStyleAction(false),
//				"/org/processmining/mxgraph/examples/swing/images/italic.gif"));
//
//		addSeparator();
//
//		add(editor.bind("Left", new KeyValueAction(mxConstants.STYLE_ALIGN,
//				mxConstants.ALIGN_LEFT),
//				"/org/processmining/mxgraph/examples/swing/images/left.gif"));
//		add(editor.bind("Center", new KeyValueAction(mxConstants.STYLE_ALIGN,
//				mxConstants.ALIGN_CENTER),
//				"/org/processmining/mxgraph/examples/swing/images/center.gif"));
//		add(editor.bind("Right", new KeyValueAction(mxConstants.STYLE_ALIGN,
//				mxConstants.ALIGN_RIGHT),
//				"/org/processmining/mxgraph/examples/swing/images/right.gif"));
//
//		addSeparator();
//
//		add(editor.bind("Font", new ColorAction("Font",
//				mxConstants.STYLE_FONTCOLOR),
//				"/org/processmining/mxgraph/examples/swing/images/fontcolor.gif"));
//		add(editor.bind("Stroke", new ColorAction("Stroke",
//				mxConstants.STYLE_STROKECOLOR),
//				"/org/processmining/mxgraph/examples/swing/images/linecolor.gif"));
//		add(editor.bind("Fill", new ColorAction("Fill",
//				mxConstants.STYLE_FILLCOLOR),
//				"/org/processmining/mxgraph/examples/swing/images/fillcolor.gif"));
//
//		addSeparator();

		final mxGraphView view = editor.getGraphComponent().getGraph()
				.getView();
		zoomCombo = new JComboBox(new Object[] { "400%",
				"200%", "150%", "100%", "75%", "50%", mxResources.get("page"),
				mxResources.get("width"), mxResources.get("actualSize") });
		zoomCombo.setEditable(true);
		zoomCombo.setMinimumSize(new Dimension(75, 0));
		zoomCombo.setPreferredSize(new Dimension(75, 0));
		zoomCombo.setMaximumSize(new Dimension(75, 100));
		zoomCombo.setMaximumRowCount(9);
//		zoomCombo.setSelectedIndex(2);
		add(zoomCombo);

		// Sets the zoom in the zoom combo the current value
		mxIEventListener scaleTracker = new mxIEventListener()
		{
			/**
			 * 
			 */
			public void invoke(Object sender, mxEventObject evt)
			{
				ignoreZoomChange = true;

				try
				{
					zoomCombo.setSelectedItem((int) Math.round(100 * view
							.getScale())
							+ "%");

				}
				finally
				{
					ignoreZoomChange = false;
				}
			}
		};

		// Installs the scale tracker to update the value in the combo box
		// if the zoom is changed from outside the combo box
		view.getGraph().getView().addListener(mxEvent.SCALE, scaleTracker);
		view.getGraph().getView().addListener(mxEvent.SCALE_AND_TRANSLATE,
				scaleTracker);

		// Invokes once to sync with the actual zoom value
		scaleTracker.invoke(null, null);

		zoomCombo.addActionListener(new ActionListener()
		{
			/**
			 * 
			 */
			public void actionPerformed(ActionEvent e)
			{
				mxGraphComponent graphComponent = editor.getGraphComponent();

				// Zoomcombo is changed when the scale is changed in the diagram
				// but the change is ignored here
				if (!ignoreZoomChange)
				{
					String zoom = zoomCombo.getSelectedItem().toString();

					if (zoom.equals(mxResources.get("page")))
					{
						graphComponent.setPageVisible(true);
						graphComponent
								.setZoomPolicy(mxGraphComponent.ZOOM_POLICY_PAGE);
					}
					else if (zoom.equals(mxResources.get("width")))
					{
						graphComponent.setPageVisible(true);
						graphComponent
								.setZoomPolicy(mxGraphComponent.ZOOM_POLICY_WIDTH);
					}
					else if (zoom.equals(mxResources.get("actualSize")))
					{
						graphComponent.zoomActual();
					}
					else
					{
						try
						{
							zoom = zoom.replace("%", "");
							double scale = Math.min(16, Math.max(0.01,
									Double.parseDouble(zoom) / 100));
							graphComponent.zoomTo(scale, graphComponent
									.isCenterZoom());
						}
						catch (Exception ex)
						{
							JOptionPane.showMessageDialog(editor, ex
									.getMessage());
						}
					}
				}
			}
		});
		
		//add my tools
		addSeparator();
		
		//set the activity cardinality		
		activityCombo = new JComboBox<String>(new String[] { 
				mxConstants.UNARYRESPONSE, mxConstants.RESPONSE, mxConstants.PRECEDENCE, mxConstants.UNARYPRECEDENCE,
				mxConstants.RESPONSE_PRECEDENCE, mxConstants.UNARYRESPONSE_UNARYPRECEDENCE, mxConstants.NONRESPONSE_NONPRECEDENCE,
				mxConstants.COEXISTENCE, mxConstants.NONRESPONSE, mxConstants.NONPRECEDENCE, mxConstants.NONCOEXISTENCE, 
				mxConstants.COEXISTENCE_COEXISTENCE, mxConstants.NONCOEXISTENCE_NONCOEXISTENCE});
	
		activityCombo.setEditable(true);
		activityCombo.setMinimumSize(new Dimension(165, 0));
		activityCombo.setPreferredSize(new Dimension(165, 0));
		activityCombo.setMaximumSize(new Dimension(165, 100));
		add(activityCombo);

		activityCombo.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				mxGraph graph = editor.getGraphComponent().getGraph();
				String selectedItem = activityCombo.getSelectedItem().toString();
				mxCell selectedCell = (mxCell) graph.getSelectionCell();
				if(selectedCell != null){
					String selectedCellType = selectedCell.getAttributes().get("type");
					System.out.println("activity combo type: " + selectedCellType);
					System.out.println("activity combo select value: " + selectedItem);
					if(selectedCellType.equals(mxConstants.ACTIVITYRELATION)){						
						selectedCell.setValue(null);
						if(selectedItem.contains("non")){
							System.out.println("non");
							selectedCell.setValue("x");
						}
						
						String itemSetString = null;						
						for(int i=0; i<activityCombo.getItemCount(); i++){
							itemSetString = itemSetString + (String) activityCombo.getItemAt(i);						
						}
						
						String[] styleElements = selectedCell.getStyle().split(";");
						String newStyle = selectedItem + ";";;
						for(int i=0; i<styleElements.length; i++){
							String element = styleElements[i];
							if(!itemSetString.contains(element)){
								newStyle = newStyle + styleElements[i] + ";";
							}
							
						}
												
						selectedCell.setStyle(newStyle);						
						selectedCell.getAttributes().put(mxConstants.CARDINALITYTYPE, selectedItem);
						graph.refresh();
						System.out.println("activity combo new type: " + newStyle);		        		
		        	}
				}											
			}
		});

		addSeparator();
		
		int comboLength = 55;
		int edgeStyleComboLength = 125;
		//set the source always cardinality		
		sourceAlwaysCombo = new JComboBox<String>(new String[] { 
				"*", "1", "1..*", "0..1", "1..2", "2"});
	
		sourceAlwaysCombo.setEditable(true);
		sourceAlwaysCombo.setMinimumSize(new Dimension(comboLength, 0));
		sourceAlwaysCombo.setPreferredSize(new Dimension(comboLength, 0));
		sourceAlwaysCombo.setMaximumSize(new Dimension(comboLength, 100));
		add(sourceAlwaysCombo);

		sourceAlwaysCombo.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				mxGraph graph = editor.getGraphComponent().getGraph();
				String selectedItem = sourceAlwaysCombo.getSelectedItem().toString();
				System.out.println("select value: " + selectedItem);
				mxCell selectedCell = (mxCell) graph.getSelectionCell();
				
				if(selectedCell != null){
					String selectedCellType = selectedCell.getAttributes().get("type");
					System.out.println("type: " + selectedCellType);
					
					if(selectedCellType.equals(mxConstants.CLASSRELATION) || selectedCellType.equals(mxConstants.AOCRELATION)){						
						mxCell sourceCell = (mxCell) selectedCell.getChildAt(0);
						sourceCell.setValue(mxConstants.SQUARESYMBOL + selectedItem);
						selectedCell.getAttributes().put(mxConstants.SOURCEALWAYSCARDINALITY, selectedItem);
						graph.refresh();										   		
		        	}
					else if(selectedCellType.equals(mxConstants.LABLE)){	
						mxCell parentCell = (mxCell) selectedCell.getParent();	
						selectedCell.setValue(mxConstants.SQUARESYMBOL + selectedItem);
						parentCell.getAttributes().put(mxConstants.SOURCEALWAYSCARDINALITY, selectedItem);
						graph.refresh();	
					}
				}											
			}
		});

		addSeparator();
		
		//set the source eventually cardinality		
		sourceEventuallyCombo = new JComboBox<String>(new String[] { 
				"*", "1", "1..*", "0..1", "1..2", "2"});
	
		sourceEventuallyCombo.setEditable(true);
		sourceEventuallyCombo.setMinimumSize(new Dimension(comboLength, 0));
		sourceEventuallyCombo.setPreferredSize(new Dimension(comboLength, 0));
		sourceEventuallyCombo.setMaximumSize(new Dimension(comboLength, 100));
		add(sourceEventuallyCombo);

		sourceEventuallyCombo.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				mxGraph graph = editor.getGraphComponent().getGraph();
				String selectedItem = sourceEventuallyCombo.getSelectedItem().toString();
				System.out.println("select value: " + selectedItem);
				mxCell selectedCell = (mxCell) graph.getSelectionCell();
				
				if(selectedCell != null){
					String selectedCellType = selectedCell.getAttributes().get("type");
					System.out.println("type: " + selectedCellType);				
					if(selectedCellType.equals(mxConstants.CLASSRELATION) || selectedCellType.equals(mxConstants.AOCRELATION)){						
						mxCell sourceCell = (mxCell) selectedCell.getChildAt(1);						
						sourceCell.setValue(mxConstants.DIAMONDSYMBOL + selectedItem);
						selectedCell.getAttributes().put(mxConstants.SOURCEEVENTUALLYCARDINALITY, selectedItem);
						graph.refresh();								   		
		        	}

					else if(selectedCellType.equals(mxConstants.LABLE)){	
						mxCell parentCell = (mxCell) selectedCell.getParent();	
						selectedCell.setValue(mxConstants.DIAMONDSYMBOL + selectedItem);
						parentCell.getAttributes().put(mxConstants.SOURCEEVENTUALLYCARDINALITY, selectedItem);
						graph.refresh();	
					}
				}											
			}
		});

		addSeparator();
		
		//set the target always cardinality		
		targetAlwaysCombo = new JComboBox<String>(new String[] { 
				"*", "1", "1..*", "0..1", "1..2", "2"});
	
		targetAlwaysCombo.setEditable(true);
		targetAlwaysCombo.setMinimumSize(new Dimension(comboLength, 0));
		targetAlwaysCombo.setPreferredSize(new Dimension(comboLength, 0));
		targetAlwaysCombo.setMaximumSize(new Dimension(comboLength, 100));
		add(targetAlwaysCombo);

		targetAlwaysCombo.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				mxGraph graph = editor.getGraphComponent().getGraph();
				String selectedItem = targetAlwaysCombo.getSelectedItem().toString();
				System.out.println("select value: " + selectedItem);
				mxCell selectedCell = (mxCell) graph.getSelectionCell();
				
				if(selectedCell != null){
					String selectedCellType = selectedCell.getAttributes().get("type");
					System.out.println("type: " + selectedCellType);
					
					if(selectedCellType.equals(mxConstants.CLASSRELATION)){						
						mxCell targetCell = (mxCell) selectedCell.getChildAt(2);
						targetCell.setValue(mxConstants.SQUARESYMBOL + selectedItem);
						System.out.println("selectedItem kfgz: " + selectedItem);
						selectedCell.getAttributes().put(mxConstants.TARGETALWAYSCARDINALITY, selectedItem);								   		
		        	}
					else if(selectedCellType.equals(mxConstants.AOCRELATION)){						
						mxCell targetCell = (mxCell) selectedCell.getChildAt(2);
						targetCell.setValue(selectedItem);
						selectedCell.getAttributes().put(mxConstants.TARGETALWAYSCARDINALITY, selectedItem);								   		
		        	}
					else if(selectedCellType.equals(mxConstants.LABLE)){
						mxCell parentCell = (mxCell) selectedCell.getParent();	
						String parentCellType = parentCell.getAttributes().get("type");
						if(parentCellType.equals(mxConstants.CLASSRELATION)){
							selectedCell.setValue(mxConstants.SQUARESYMBOL + selectedItem);						
						}
						else if(parentCellType.equals(mxConstants.AOCRELATION)){
							selectedCell.setValue(selectedItem);						
						}
						parentCell.getAttributes().put(mxConstants.TARGETALWAYSCARDINALITY, selectedItem);			
					}
					graph.refresh();	
				}											
			}
		});

		addSeparator();
		
		
		//set the target eventually cardinality		
		targetEventuallyCombo = new JComboBox<String>(new String[] { 
				"*", "1", "1..*", "0..1", "1..2", "2"});
	
		targetEventuallyCombo.setEditable(true);
		targetEventuallyCombo.setMinimumSize(new Dimension(comboLength, 0));
		targetEventuallyCombo.setPreferredSize(new Dimension(comboLength, 0));
		targetEventuallyCombo.setMaximumSize(new Dimension(comboLength, 100));
		add(targetEventuallyCombo);

		targetEventuallyCombo.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				mxGraph graph = editor.getGraphComponent().getGraph();
				String selectedItem = targetEventuallyCombo.getSelectedItem().toString();

				System.out.println("select value: " + selectedItem);
				mxCell selectedCell = (mxCell) graph.getSelectionCell();
				
				if(selectedCell != null){
					String selectedCellType = selectedCell.getAttributes().get("type");
					System.out.println("type: " + selectedCellType);
					
					if(selectedCellType.equals(mxConstants.CLASSRELATION)){						
						mxCell targetCell = (mxCell) selectedCell.getChildAt(3);
						targetCell.setValue(mxConstants.DIAMONDSYMBOL + selectedItem);
						selectedCell.getAttributes().put(mxConstants.TARGETEVENTUALLYCARDINALITY, selectedItem);
						graph.refresh();									   		
		        	}
					else if(selectedCellType.equals(mxConstants.LABLE)){
						mxCell parentCell = (mxCell) selectedCell.getParent();	
						selectedCell.setValue(mxConstants.DIAMONDSYMBOL + selectedItem);
						parentCell.getAttributes().put(mxConstants.TARGETEVENTUALLYCARDINALITY, selectedItem);
						graph.refresh();
					}
				}											
			}
		});

		addSeparator();
		
		//set the edge style	
		edgeStyleCombo = new JComboBox<String>(new String[] { 
				mxConstants.SHAPE_LINE, mxConstants.EDGESTYLE_ELBOW, mxConstants.EDGESTYLE_SEGMENT, 
				mxConstants.EDGESTYLE_LOOP, mxConstants.EDGESTYLE_SIDETOSIDE, mxConstants.EDGESTYLE_ENTITY_RELATION,
				mxConstants.EDGESTYLE_TOPTOBOTTOM});
	
		edgeStyleCombo.setEditable(true);
		edgeStyleCombo.setEnabled(false);
		edgeStyleCombo.setMinimumSize(new Dimension(edgeStyleComboLength, 0));
		edgeStyleCombo.setPreferredSize(new Dimension(edgeStyleComboLength, 0));
		edgeStyleCombo.setMaximumSize(new Dimension(edgeStyleComboLength, 100));
		add(edgeStyleCombo);

		edgeStyleCombo.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				mxGraph graph = editor.getGraphComponent().getGraph();
				String selectedItem = edgeStyleCombo.getSelectedItem().toString();

				System.out.println("select edge style value: " + selectedItem);
				mxCell selectedCell = (mxCell) graph.getSelectionCell();
				
				if(selectedCell != null && selectedCell.isEdge()){
					Style.addStyleElementToCell(selectedCell, mxConstants.STYLE_EDGE, selectedItem);					
					graph.refresh();
				}
											
			}
		});

//		addSeparator();
		
//		showConnectorButton = new JButton("connetor");
//		showConnectorButton.setBorder(BorderFactory.createRaisedBevelBorder());
////		add(showConnectorButton);
//		showConnectorButton.addActionListener(new ActionListener()
//		{
//			public void actionPerformed(ActionEvent e)
//			{
//				mxGraph graph = editor.getGraphComponent().getGraph();				
//				Object[] edgeCells = graph.getChildEdges(graph.getDefaultParent());
//				
//				//hide the connector
//				if(showConnectorButton.getBorder().equals(BorderFactory.createLoweredBevelBorder())){
//					showConnectorButton.setBorder(BorderFactory.createRaisedBevelBorder());
//					for(int j=0; j<edgeCells.length; j++){
//						mxCell edgeCell = (mxCell) edgeCells[j];
//						String edgeType = edgeCell.getAttributes().get(mxConstants.TYPE);
//						System.out.println("cancel edge type: " + edgeCell);
//						if(edgeType.equals(mxConstants.ACTIVITYRELATION) || edgeType.equals(mxConstants.CLASSRELATION)){
//							Object[] nodeCells = graph.getChildVertices(edgeCell);
//							for(int i=0; i<nodeCells.length; i++){
//								mxCell nodeCell = (mxCell) nodeCells[i];
//								String nodeType = nodeCell.getAttributes().get(mxConstants.TYPE);
//								System.out.println("cancel node type: " + nodeType);
//								if(nodeType != null && nodeType.equals(mxConstants.CONNECTOR)){
//									System.out.println("cancel node connector");
//									editor.getGraphComponent().getGraph().getModel().setStyle(nodeCell, mxConstants.CONNECTOR_TRANSPARENT);
//									editor.getGraphComponent().getGraph().refresh();
//								}
//							}		
//						}						
//					}					
//				}
//				//show the connector
//				else{
//					showConnectorButton.setBorder(BorderFactory.createLoweredBevelBorder());
//					for(int j=0; j<edgeCells.length; j++){
//						mxCell edgeCell = (mxCell) edgeCells[j];
//						String edgeType = edgeCell.getAttributes().get(mxConstants.TYPE);
//						System.out.println("cancel edge type: " + edgeCell);
//						if(edgeType.equals(mxConstants.ACTIVITYRELATION) || edgeType.equals(mxConstants.CLASSRELATION)){
//							Object[] nodeCells = graph.getChildVertices(edgeCell);
//							for(int i=0; i<nodeCells.length; i++){
//								mxCell nodeCell = (mxCell) nodeCells[i];
//								String nodeType = nodeCell.getAttributes().get(mxConstants.TYPE);
//								System.out.println("cancel node type: " + nodeType);
//								if(nodeType != null && nodeType.equals(mxConstants.CONNECTOR)){
//									System.out.println("cancel node connector");
//									editor.getGraphComponent().getGraph().getModel().setStyle(nodeCell, mxConstants.CONNECTOR_FOCUSED);
//									editor.getGraphComponent().getGraph().refresh();
//								}
//							}		
//						}						
//					}	
//				}																				
//			}
//		});		
//		
////		addSeparator();		
//				
//		//check if the graph is legal (it has all elements an OCBC model requires)
//		checkButton = new JButton("check");
//		checkButton.setBorder(BorderFactory.createRaisedBevelBorder());
////		add(checkButton);
//		checkButton.addActionListener(new ActionListener()
//		{
//			public void actionPerformed(ActionEvent e)
//			{
//				mxGraph graph = editor.getGraphComponent().getGraph();				
//				Object[] edgeCells = graph.getChildEdges(graph.getDefaultParent());
//				Object[] nodeCells = graph.getChildVertices(graph.getDefaultParent());
//				
//				if(checkButton.getBorder().equals(BorderFactory.createRaisedBevelBorder())){					
//					checkButton.setBorder(BorderFactory.createLoweredBevelBorder());
//					//check if the activity nodes have aoc relation and highlight the labels of violating nodes
//					for(int j=0; j<nodeCells.length; j++){
//						mxCell nodeCell = (mxCell) nodeCells[j];
//						String nodeType = nodeCell.getAttributes().get(mxConstants.TYPE);
//						System.out.println("cancel edge type: " + nodeType);
//						if(nodeType.equals(mxConstants.ACTIVITYCELL)){				
//							int linkedAOCEdgeNumber = 0;
//							for(int k=0; k<nodeCell.getEdgeCount(); k++){
//								mxCell linkedEdgeCell = (mxCell) nodeCell.getEdgeAt(k);
//								String linkEdgeType = linkedEdgeCell.getAttributes().get(mxConstants.TYPE);
//								if(linkEdgeType.equals(mxConstants.AOCRELATION)){
//									linkedAOCEdgeNumber ++;
//								}
//							}
//							if(linkedAOCEdgeNumber == 0){
//								Style.addStyleElementToCell(nodeCell, mxConstants.STYLE_FONTCOLOR, "red");
//								editor.getGraphComponent().getGraph().refresh();
//							}								
//						}			
//					}		
//					//check if the activity edges have crel relation and highlight the violating connector
//					for(int j=0; j<edgeCells.length; j++){
//						mxCell edgeCell = (mxCell) edgeCells[j];
//						String edgeType = edgeCell.getAttributes().get(mxConstants.TYPE);
//						System.out.println("cancel edge type: " + edgeCell);
//						if(edgeType.equals(mxConstants.ACTIVITYRELATION)){
//							Object[] connectorCells = graph.getChildVertices(edgeCell);
//							mxCell connectorCell = (mxCell) connectorCells[0];
//							int linkedEdgeNumber = connectorCell.getEdgeCount();
//							if(linkedEdgeNumber != 1){
//								editor.getGraphComponent().getGraph().getModel().setStyle(connectorCell, mxConstants.CONNECTOR_VIOLATING);
//								editor.getGraphComponent().getGraph().refresh();
//							}								
//						}			
//					}					
//				}
//				//hide the connector
//				else{
//					checkButton.setBorder(BorderFactory.createRaisedBevelBorder());
//					for(int j=0; j<edgeCells.length; j++){
//						mxCell edgeCell = (mxCell) edgeCells[j];
//						String edgeType = edgeCell.getAttributes().get(mxConstants.TYPE);
//						System.out.println("cancel edge type: " + edgeCell);
//						if(edgeType.equals(mxConstants.ACTIVITYRELATION)){
//							Object[] childNodeCells = graph.getChildVertices(edgeCell);
//							for(int i=0; i<childNodeCells.length; i++){
//								mxCell nodeCell = (mxCell) childNodeCells[i];
//								String nodeType = nodeCell.getAttributes().get(mxConstants.TYPE);
//								System.out.println("cancel node type: " + nodeType);
//								if(nodeType != null && nodeType.equals(mxConstants.CONNECTOR)){
//									System.out.println("cancel node connector");
//									editor.getGraphComponent().getGraph().getModel().setStyle(nodeCell, mxConstants.CONNECTOR_TRANSPARENT);
//									editor.getGraphComponent().getGraph().refresh();
//								}
//							}		
//						}						
//					}	
//					
//					for(int j=0; j<nodeCells.length; j++){
//						mxCell nodeCell = (mxCell) nodeCells[j];
//						String nodeType = nodeCell.getAttributes().get(mxConstants.TYPE);
//						System.out.println("cancel edge type: " + nodeType);
//						if(nodeType.equals(mxConstants.ACTIVITYCELL)){				
//							Style.addStyleElementToCell(nodeCell, mxConstants.STYLE_FONTCOLOR, "black");
//							editor.getGraphComponent().getGraph().refresh();			
//						}			
//					}		
//				}																				
//			}
//		});					
//					
////		addSeparator();		
//		
//		//check if the graph is legal (it has all elements an OCBC model requires)
//		refreshButton = new JButton("refresh");
//		refreshButton.setBorder(BorderFactory.createRaisedBevelBorder());
////		add(refreshButton);
//		refreshButton.addActionListener(new ActionListener()
//		{
//			public void actionPerformed(ActionEvent e){
//				editor.refreshGraph();
//			}
//		});					
//		
//		
////		addSeparator();		
//		
//		//check if the graph is legal (it has all elements an OCBC model requires)
//		swapSTButton = new JButton("swap S/T");
//		swapSTButton.setBorder(BorderFactory.createRaisedBevelBorder());
////		add(swapSTButton);
//		swapSTButton.addActionListener(new ActionListener()
//		{
//			public void actionPerformed(ActionEvent e){
//				mxGraph graph = editor.getGraphComponent().getGraph();
//				mxCell selectedCell = (mxCell) graph.getSelectionCell();
//				
//				if(selectedCell != null){
//					String selectedCellType = selectedCell.getAttributes().get("type");
//					System.out.println("type: " + selectedCellType);
//					
//					if(selectedCellType.equals(mxConstants.CLASSRELATION)){	
//						System.out.println("swap");
//						mxCell sourceCell = (mxCell) selectedCell.getSource();
//						mxCell targetCell = (mxCell) selectedCell.getTarget();
//						selectedCell.setSource(targetCell);
//						selectedCell.setTarget(sourceCell);
//						editor.addModificationFlag();
//						editor.refreshGraph();						
//					}
//				}
//			}
//		});					
//		
////		addSeparator();		
//		
//		//check if the graph is legal (it has all elements an OCBC model requires)
//		pageNumButton = new JButton("page zoom");
//		pageNumButton.setBorder(BorderFactory.createRaisedBevelBorder());
////		add(pageNumButton);
//		pageNumButton.addActionListener(new ActionListener()
//		{
//			public void actionPerformed(ActionEvent e){
//				mxGraph graph = editor.getGraphComponent().getGraph();
//				mxCell selectedCell = (mxCell) graph.getSelectionCell();
////				int height = editor.getGraphComponent().getSize().height;
////				int width = editor.getGraphComponent().getSize().width;
////				System.out.println("height: " + height);
////				System.out.println("width: " + width);
////				Object v1 = graph.insertVertex(graph.getDefaultParent(), null, "Hello", width*2, height*2, 80,
////				30);
//				
//				System.out.println("getHorizontalPageCount(): " + editor.getGraphComponent().getHorizontalPageCount());
//				System.out.println("getVerticalPageCount: " + editor.getGraphComponent().getVerticalPageCount());
//				int horizontalCount = editor.getGraphComponent().getHorizontalPageCount();
//				int verticalCount = editor.getGraphComponent().getVerticalPageCount();
//				
//				editor.getGraphComponent().setHorizontalPageCount(horizontalCount+1);
//				editor.getGraphComponent().setVerticalPageCount(verticalCount+1);
//
//			}
//		});	
		
	}
}
