package org.processmining.mxgraph.examples.swing.io;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.processmining.framework.util.Pair;
import org.processmining.mxgraph.examples.swing.listener.AddElementListener;
import org.processmining.mxgraph.examples.swing.visualization.Style;
import org.processmining.mxgraph.model.mxCell;
import org.processmining.mxgraph.model.mxGeometry;
import org.processmining.mxgraph.util.mxConstants;
import org.processmining.mxgraph.view.mxGraph;
import org.processmining.ocbc.models.AOCEdge;
import org.processmining.ocbc.models.ActivityEdge;
import org.processmining.ocbc.models.ActivityNode;
import org.processmining.ocbc.models.ClassEdge;
import org.processmining.ocbc.models.ClassNode;
import org.processmining.ocbc.models.CrelOCEdge;
import org.processmining.ocbc.models.CrelRTEdge;
import org.processmining.ocbc.models.OCBCModel;
import org.processmining.ocbc.models.impl.OCBCModelImpl;
import org.processmining.ocbc.models.visualization.ModelLayoutBuilder;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;
public class TransformBetweenJGraphXAndModel {
	private OCBCModel model = new OCBCModelImpl("OCBC model");
    private ModelLayoutBuilder builder = new ModelLayoutBuilder(model);
    private  Map<String, mxCell> idVSCell = new HashMap<String, mxCell>();
    private  Map<mxCell, String> cellVSId = new HashMap<mxCell, String>();
    public static void main(String[] args) throws FileNotFoundException {
    	 
//        TransformToOCBCSemanticModel ajavademo = new TransformToOCBCSemanticModel();
//        System.out.println("\n" + ajavademo.model.getModelName());
//        System.out.println("\n" + ajavademo.model.getAllActivities().size());
//        FileInputStream is = new FileInputStream("E:\\understandableModel.ocml");
//        ajavademo.parse(is);
//        
//        System.out.println("\n" + ajavademo.model.getAllActivities().size());
//        
//        JFrame frame = new JFrame();  
//        frame.setSize(1000, 500);  
//        frame.setTitle("Activity Diagram");  
//        frame.getContentPane().add(new JScrollPane(ajavademo.model.getGraph()));  
//          
//         frame.setBounds(50, 50, 700, 800);  
//          
//         frame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);  
//         frame.setVisible(true);  
    }
    
    public void parse(InputStream input) {
        try {
            SAXParserFactory factory = SAXParserFactory.newInstance();
            SAXParser parser = factory.newSAXParser();
            DefaultHandler handler = new SAXHandler();
            parser.parse(input, handler);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
    
    public mxGraph transformModelToJGraphX(OCBCModel model, Map<String, Pair<Double, Double>> nodeNameVSCoordinateMap){
    	mxGraph graph = new mxGraph();
    	AddElementListener addElement = new AddElementListener(null);
    	
    	Style.createStyle(graph);
		graph.setAllowDanglingEdges(false);
		graph.setSplitEnabled(false);
		
	    double souceLabelPosition = -0.6;
	    double targetLabelPosition = 0.4;
	    double connectorSize = 5;
	    double connectorX = 0.2;
	    
    	List<ClassNode> classNodeList = model.getAllClasses();
    	for(ClassNode classNode : classNodeList){
    		String className = classNode.getNodeName();
    		String classNodeId = classNode.getID();
    		
    		Object parent = graph.getDefaultParent();
//    		double xCoordinate = nodeNameVSCoordinateMap.get(className).getFirst();
//    		double yCoordinate = nodeNameVSCoordinateMap.get(className).getSecond();
    		double xCoordinate = 0;
    		double yCoordinate = 0;
    		mxCell classCell = (mxCell) graph.insertVertex(parent, classNodeId, className, xCoordinate,
    				yCoordinate, mxConstants.CLASSCELLWIDTH, mxConstants.CLASSCELLHEIGHT, 
					"roundImage;image=/org/processmining/mxgraph/examples/swing/images/class.png");
//    		classCell.getAttributes().put("ElementId", classNodeId);
    		classCell.getAttributes().put(mxConstants.TYPE, mxConstants.CLASSCELL);                           		
    		classCell.getAttributes().put(mxConstants.TABLE, "table_name");
    		classCell.getAttributes().put(mxConstants.ROWID, mxConstants.PK);
    		idVSCell.put(className, classCell);
    		cellVSId.put(classCell, className);
    	}
    	//class relations
    	List<ClassEdge> classEdgeList = model.getAllClassRelations();
    	for(ClassEdge classEdge : classEdgeList){     
    		ClassNode sourceClassNode = classEdge.getSourceNode();
    		ClassNode targetClassNode = classEdge.getTargetNode();
    		String id = classEdge.getID();
    		
    		mxCell sourceClassCell = idVSCell.get(sourceClassNode.getNodeName());
    		mxCell targetClassCell = idVSCell.get(targetClassNode.getNodeName());
    		
    		System.out.println("class edge id when created:" + id);
			System.out.println("sourceClassCell name when created:" + sourceClassCell.getValue());
			System.out.println("targetClassCell name when created:" + targetClassCell.getValue());
			
        	Object parent = graph.getDefaultParent();
    		mxCell edgeCell = (mxCell) graph.insertEdge(parent, id, "", sourceClassCell, targetClassCell, "");
    		System.out.println("class cell id when created:" + edgeCell.getId());
    		idVSCell.put(id, edgeCell);
    		cellVSId.put(edgeCell, id);
    		String classRelationType = classEdge.getEdgeName();
    		String sourceClassName = classEdge.getSourceNode().getNodeName();
    		String targetClassName = classEdge.getTargetNode().getNodeName();
    		String sourceAlwaysCardinality = classEdge.getSourceAlwaysCardinality().toString();
   			String sourceEventuallyCardinality = classEdge.getSourceEventuallyCardinality().toString();
   			String targetAlwaysCardinality = classEdge.getTargetAlwaysCardinality().toString();
   			String targetEventuallyCardinality = classEdge.getTargetEventuallyCardinality().toString();   		
			
        	System.out.println("create class relation");
        	//class relation
        	addElement.addClassEdgeIntoGraph(edgeCell, graph, classRelationType, sourceClassName, targetClassName, sourceAlwaysCardinality, 
        			sourceEventuallyCardinality, targetAlwaysCardinality, targetEventuallyCardinality, false);
    	}
    	
    	//activity
    	List<ActivityNode> activityNodeList = model.getAllActivities();
    	for(ActivityNode activityNode : activityNodeList){
    		String activityName = activityNode.getNodeName();
    		String id = activityNode.getID();
    		
    		Object parent = graph.getDefaultParent();
//    		double xCoordinate = nodeNameVSCoordinateMap.get(activityName).getFirst();
//    		double yCoordinate = nodeNameVSCoordinateMap.get(activityName).getSecond();
    		double xCoordinate = 0;
    		double yCoordinate = 0;
    		mxCell activityCell = (mxCell) graph.insertVertex(parent, id, activityName, xCoordinate,
    				yCoordinate, mxConstants.ACTIVITYCELLWIDTH, mxConstants.ACTIVITYCELLHEIGHT, 
					"roundImage;image=/org/processmining/mxgraph/examples/swing/images/activity.png");
    		
    		activityCell.getAttributes().put(mxConstants.TYPE, mxConstants.ACTIVITYCELL);
    		activityCell.getAttributes().put(mxConstants.TABLE, "table_name");
    		activityCell.getAttributes().put(mxConstants.ACTIVITY, "column-name");
    		idVSCell.put(activityName, activityCell);
    		cellVSId.put(activityCell, activityName);
    	}
    	//aoc relations
    	List<AOCEdge> aOCEdgeList = model.getAllAOCRelations();
    	for(AOCEdge aOCEdge : aOCEdgeList){
    		String id = aOCEdge.getID();
    		ActivityNode activityNode = aOCEdge.getActivityNode();
    		ClassNode classNode = aOCEdge.getClassNode();
   			String sourceAlwaysCardinality = aOCEdge.getActivityAlwaysCardinality().toString();
   			String sourceEventuallyCardinality = aOCEdge.getActivityEventuallyCardinality().toString();
   			String targetAlwaysCardinality = aOCEdge.getClassCardinality().toString();
   			
    		mxCell activityCell = idVSCell.get(activityNode.getNodeName());
    		mxCell classCell = idVSCell.get(classNode.getNodeName());
    		
    		
    		Object parent = graph.getDefaultParent();
    		mxCell edgeCell = (mxCell) graph.insertEdge(parent, id, "", activityCell, classCell, "");
    		idVSCell.put(aOCEdge.getID(), edgeCell);
    		cellVSId.put(edgeCell, aOCEdge.getID());
    		
//    		edgeCell.getAttributes().put(mxConstants.TYPE, mxConstants.AOCRELATION);
//    		edgeCell.getAttributes().put(mxConstants.SOURCEALWAYSCARDINALITY, sourceAlwaysCardinality);
//    		edgeCell.getAttributes().put(mxConstants.SOURCEEVENTUALLYCARDINALITY, sourceEventuallyCardinality);
//    		edgeCell.getAttributes().put(mxConstants.TARGETALWAYSCARDINALITY, targetAlwaysCardinality);

        	
//        	System.out.println("create AOCRelation");    
//			System.out.println(mxConstants.DIAMONDSYMBOL);
//			                       		
//			mxGeometry geoSource = new mxGeometry(souceLabelPosition, souceLabelPosition, 0, 0);           		    		         		    			
// 			mxGeometry geoTarget = new mxGeometry(targetLabelPosition, targetLabelPosition, 0, 0);
//			geoTarget.setRelative(true); 
//			geoSource.setRelative(true);
//			
// 			mxCell sourceLabel;
// 			mxCell targetLabel;
// 			
//			sourceLabel = new mxCell(mxConstants.SQUARESYMBOL + sourceAlwaysCardinality + 
//					mxConstants.DIAMONDSYMBOL + sourceEventuallyCardinality, geoSource, mxConstants.LABEL_NORMAL);
//			targetLabel = new mxCell(targetAlwaysCardinality, geoTarget, mxConstants.LABEL_NORMAL);
//			sourceLabel.setVertex(true);
//			sourceLabel.getAttributes().put(mxConstants.CARDINALITYPOSITION, mxConstants.SOURCECARDINALITY);
//			targetLabel.setVertex(true);
//			targetLabel.getAttributes().put(mxConstants.CARDINALITYPOSITION, mxConstants.TARGETCARDINALITY);
//			//the first cell added into the graph is the label close to the activity side
//			graph.addCell(sourceLabel, edgeCell);
//			graph.addCell(targetLabel, edgeCell);
//
//   			sourceLabel.getAttributes().put(mxConstants.TYPE, mxConstants.LABLE);
//   			targetLabel.getAttributes().put(mxConstants.TYPE, mxConstants.LABLE);
//        	//AOC relation
//   			graph.getModel().setStyle(edgeCell, mxConstants.AOCRELATION);
//        	graph.getModel().getStyle(edgeCell);
//        	graph.refresh();
        	
        	addElement.addAOCEdgeIntoGraph(edgeCell, graph, activityCell, classCell, sourceAlwaysCardinality, 
        			sourceEventuallyCardinality, targetAlwaysCardinality);
    		

    	}

    	//add activity edges
       	List<ActivityEdge> activityEdgeList = model.getAllActivityRelations();
       	Map<String, String> combinedIdVSCardinalityType = new HashMap<String, String>();
       	Map<String, mxCell> combinedIdVSCell = new HashMap<String, mxCell>();
       	List<String> implicitEdgeIdList = new ArrayList<String>();//record the edge ids which are incorporated into existing edges
       	List<String> transitiveRelationIdList = model.getTransitiveRelationIdList();
       	System.out.println("transitiveRelationIdList.size: " + transitiveRelationIdList.size());	
    	for(ActivityEdge activityEdge : activityEdgeList){
    		//insert value (class relation type)	
//        	newCell.setValue("name");         
    		ActivityNode sourceNode = activityEdge.getReferenceNode();
    		ActivityNode targetNode = activityEdge.getTargetNode();
    		String cardinalityType = activityEdge.getCardinalityType();
    		String edgeId = activityEdge.getID();
    		

    		mxCell sourceCell = idVSCell.get(sourceNode.getNodeName());
    		mxCell targetCell = idVSCell.get(targetNode.getNodeName());
    		
    		String combinedId = sourceNode.getNodeName() + "-" + targetNode.getNodeName();
    		String combinedIdReverse = targetNode.getNodeName() + "-" + sourceNode.getNodeName();

    		boolean twinActivityEdgeFlag = false;
//    		if(combinedIdVSCardinalityType.containsKey(combinedIdReverse)){
//    			if((cardinalityType.equals(mxConstants.UNARYRESPONSE) && combinedIdVSCardinalityType.
//    					get(combinedIdReverse).equals(mxConstants.UNARYPRECEDENCE)) || 
//    					(cardinalityType.equals(mxConstants.UNARYPRECEDENCE) && combinedIdVSCardinalityType.
//    					get(combinedIdReverse).equals(mxConstants.UNARYRESPONSE))){
//    				twinActivityEdgeFlag = true;
//    			}
//    		}
//    		twinActivityEdgeFlag = false;
    		if(twinActivityEdgeFlag){
    			mxCell edge = combinedIdVSCell.get(combinedIdReverse);
				graph.getModel().setStyle(edge, mxConstants.UNARYRESPONSE_UNARYPRECEDENCE);
				edge.getAttributes().put(mxConstants.SECONDID, edgeId);//add the edge is as the second id of the cell
				edge.getAttributes().put(mxConstants.CARDINALITYTYPE, mxConstants.UNARYRESPONSE_UNARYPRECEDENCE);
				implicitEdgeIdList.add(edgeId);
    		}
    		else{
    			Object parent = graph.getDefaultParent();
        		mxCell edgeCell = (mxCell) graph.insertEdge(parent, edgeId, "", sourceCell, targetCell, "");
        		idVSCell.put(activityEdge.getID(), edgeCell);
        		cellVSId.put(edgeCell, activityEdge.getID());
        		
            	//insert attributes of type and cardinalities	
        		edgeCell.getAttributes().put(mxConstants.TYPE, mxConstants.ACTIVITYRELATION);
        		edgeCell.getAttributes().put(mxConstants.CARDINALITYTYPE, cardinalityType);
    			if(transitiveRelationIdList.contains(edgeId)){
    				System.out.println("add simplification type");	
    				edgeCell.getAttributes().put(mxConstants.simplificationType, mxConstants.transitive);
    				edgeCell.getAttributes().put(mxConstants.transitive, mxConstants.Yes);

    			}
    			
//    			Map<String, Map<String, Integer>> activityEdgeVSConstraintVSInstanceNumMap = model.getModelDiscovery()
//    					.getBehaviorModelDiscovery().getActivityEdgeVSConstraintVSInstanceNumMap();
//    			Map<String, Map<XOCEvent, List<XOCEvent>>> activityEdgeVSEventVSPrecedenceEventList = model.getModelDiscovery()
//    					.getBehaviorModelDiscovery().getActivityEdgeVSEventVSPrecedenceEventList();
    			
    			System.out.println("activityEdge.getID() in transform model into graphx" + activityEdge.getID());
//    			double satifiedInstanceNumOfOneSegment = activityEdgeVSConstraintVSInstanceNumMap
//    					.get(activityEdge.getID())
//    					.get(cardinalityType);
//    			double totalInstanceNumOfAllSegments = 0;
//    			for(String activityEdgeId : activityEdgeVSEventVSPrecedenceEventList.keySet()){
//    				totalInstanceNumOfAllSegments = totalInstanceNumOfAllSegments 
//    						+ activityEdgeVSEventVSPrecedenceEventList.get(activityEdgeId).size();
//    			}
//    			double totalInstanceNum = activityEdgeVSEventVSPrecedenceEventList.get(activityEdge.getID()).size();
//    			double totalEventNum = model.getModelSourceLog().getAllEvents().size();
//    			
//    			double confidence = satifiedInstanceNumOfOneSegment/totalInstanceNum;
//    			double support = satifiedInstanceNumOfOneSegment/totalEventNum;
    			
//    			edgeCell.getAttributes().put(mxConstants.confidence, confidence + "");
//    			edgeCell.getAttributes().put(mxConstants.support, support + "");
    			edgeCell.getAttributes().put(mxConstants.confidence, "0");
    			edgeCell.getAttributes().put(mxConstants.support, "0");
    			edgeCell.getAttributes().put(mxConstants.lift, "0");
        		//add the brother flag in the attributes
//        		Map<String, Map<String, Map<String, List<String>>>> refActivityVSTarActivityVSTypeVSConstraintList = model
//        				.getModelDiscovery().getBehaviorModelDiscovery().getRefActivityVSTarActivityVSTypeVSConstraintList();
//        		List<String> constraintList = refActivityVSTarActivityVSTypeVSConstraintList.get(sourceNode.getID())
//        				.get(targetNode.getID()).get(cardinalityType);
//        		if(constraintList.size() > 1){
//        			edgeCell.getAttributes().put(mxConstants.brotherConstraint, mxConstants.Yes);
//        		}
    			
//        		List<String> brotherRelationIdList = model.getModelDiscovery().getBehaviorModelDiscovery().getBrotherRelationIdList();
//        		if(brotherRelationIdList.contains(edgeId)){
//        			System.out.println("add brother constraint type");	
//    				edgeCell.getAttributes().put(mxConstants.brotherConstraint, mxConstants.Yes);
//        		}
        		
            	System.out.println("create activity relation");
            	System.out.println("activity relation type: " + cardinalityType);
            	//class relation
    			
     			//add the connector
     			mxGeometry middlePositon = new mxGeometry(connectorX, 0, connectorSize, connectorSize);
     			mxCell middleConnecor = new mxCell(" ", middlePositon, mxConstants.CONNECTOR_TRANSPARENT);
     			middleConnecor.getAttributes().put(mxConstants.TYPE, mxConstants.CONNECTOR);
     			middlePositon.setRelative(true);
     			middleConnecor.setVertex(true);            		    			             		    		                       		    			
     			graph.addCell(middleConnecor, edgeCell);
   			
     			System.out.println("cardinalityType when create activity edge: " + cardinalityType);
     			graph.getModel().setStyle(edgeCell, cardinalityType);
     			//add the "X" to the edge if the cardinalityType contains "non"
//     			if(!transitiveRelationIdList.contains(edgeId) && cardinalityType.contains("non")){
//     				System.out.println("contains");
//     				edgeCell.setValue("x");
//     				edgeCell.getAttributes().put(mxConstants.simplificationType, mxConstants.redundant);
//     			}
     			if(cardinalityType.contains("non")){
     				System.out.println("contains");
     				edgeCell.setValue("x");
     				edgeCell.getAttributes().put(mxConstants.simplificationType, mxConstants.redundant);
     				
     			}
     			combinedIdVSCardinalityType.put(combinedId, cardinalityType);
     			combinedIdVSCell.put(combinedId, edgeCell);
    		}
 			graph.refresh();

    	}
    	
    	//add crel edges
       	List<CrelOCEdge> crelOCEdgeList = model.getAllCrelOCRelations();
       	for(CrelOCEdge crelOCEdge : crelOCEdgeList){
       		String id = crelOCEdge.getID();
    		ActivityEdge activityEdge = crelOCEdge.getRelatedActivityRelation();
    		ClassNode classNode = crelOCEdge.getRelatedClass();
    		 		
    		if(!implicitEdgeIdList.contains(activityEdge.getID())){
    			mxCell activityEdgeCell = idVSCell.get(activityEdge.getID());
        		mxCell activityEdgeConnectorCell = (mxCell) activityEdgeCell.getChildAt(0);
        		mxCell classNodeCell = idVSCell.get(classNode.getNodeName());
        		
            	Object parent = graph.getDefaultParent();
        		mxCell edgeCell = (mxCell) graph.insertEdge(parent, id, "", activityEdgeConnectorCell, classNodeCell, "");
        		
            	edgeCell.getAttributes().put(mxConstants.TYPE, mxConstants.CRELOCRELATION); 
            	System.out.println("create connect oc relation");                              	
            	graph.getModel().setStyle(edgeCell, mxConstants.CRELRELATION);
    		}
       	}
       	
       	List<CrelRTEdge> crelRTEdgeList = model.getAllCrelRTRelations();
       	for(CrelRTEdge crelRTEdge : crelRTEdgeList){
    		String id = crelRTEdge.getID();
    		ActivityEdge activityEdge = crelRTEdge.getRelatedActivityRelation();
    		ClassEdge classEdge = crelRTEdge.getRelatedClassRelation();
    		
    		if(!implicitEdgeIdList.contains(activityEdge.getID())){
    			mxCell activityEdgeCell = idVSCell.get(activityEdge.getID());
        		mxCell activityEdgeConnectorCell = (mxCell) activityEdgeCell.getChildAt(0);
        		mxCell classEdgeCell = idVSCell.get(classEdge.getID());
        		mxCell classEdgeConnectorCell = (mxCell) classEdgeCell.getChildAt(0);
        		
            	Object parent = graph.getDefaultParent();
        		mxCell edgeCell = (mxCell) graph.insertEdge(parent, id, "", activityEdgeConnectorCell, classEdgeConnectorCell, "");
        		
            	edgeCell.getAttributes().put(mxConstants.TYPE, mxConstants.CRELRTRELATION); 
            	System.out.println("create connect rt relation");                              	
            	graph.getModel().setStyle(edgeCell, mxConstants.CRELRELATION);
    		}
       	}
    	graph.refresh();
    	model.setGraphX(graph);//add graphX to the model by Guangming at 2018-Jan-07
    	return graph;
      
    }
    
    public Map<String, mxCell> getIdVSCell(){
    	return idVSCell;
    }
    
    public Map<mxCell, String> getCellVSId(){
    	return cellVSId;
    }
    
    public void transform(mxGraph graph){
    	Object[] allVertexs = graph.getChildVertices(graph.getDefaultParent());
    	Object[] allEdges = graph.getChildEdges(graph.getDefaultParent());
    	for(int i=0; i<allVertexs.length; i++){
    		String displayName = ((mxCell)allVertexs[i]).getValue().toString();
    		String id = ((mxCell)allVertexs[i]).getId();
    		String type = ((mxCell)allVertexs[i]).getAttributes().get(mxConstants.TYPE);
    		double positionX = ((mxCell)allVertexs[i]).getGeometry().getX();
    		double positionY = ((mxCell)allVertexs[i]).getGeometry().getY();
    		double height = ((mxCell)allVertexs[i]).getGeometry().getHeight();
    		double width = ((mxCell)allVertexs[i]).getGeometry().getWidth();
			
			System.out.println("displayName: " + displayName);
			System.out.println("id: " + id);
			System.out.println("type: " + type);
			System.out.println("positionX: " + positionX);
			System.out.println("positionY: " + positionY);
			System.out.println("height: " + height);
			System.out.println("width: " + width);
			
			System.out.println("------------------");
								
            builder.addNode(type, displayName, id, positionX, positionY, height, width);
        }
    	
    	for(int i=0; i<allEdges.length; i++){
//			((mxCell)allEdges[i]).setValue("edge");
//			((mxCell)allEdges[i]).set
			System.out.println("value: " + ((mxCell)allEdges[i]).getValue());
			System.out.println("Id: " + ((mxCell)allEdges[i]).getId());
			System.out.println("getSource().getId(): " + ((mxCell)allEdges[i]).getSource().getId());
			System.out.println("getTarget().getId(): " + ((mxCell)allEdges[i]).getTarget().getId());
			mxCell currentCell = (mxCell) allEdges[i];
			String edgeType = currentCell.getAttributes().get(mxConstants.TYPE);
			
    		String id = currentCell.getId();


    		
    		
//    		double positionX = ((mxCell)allVertexs[i]).getGeometry().getX();
//    		double positionY = ((mxCell)allVertexs[i]).getGeometry().getY();
//    		double height = ((mxCell)allVertexs[i]).getGeometry().getHeight();
//    		double width = ((mxCell)allVertexs[i]).getGeometry().getWidth();
    		
    		
    		if(edgeType.equals(mxConstants.CLASSRELATION)) {
//    			String displayName = currentCell.getValue().toString();
    			String relationshipType = currentCell.getAttributes().get(mxConstants.RELATIONSHIPTYPE);
    			String sourceNodeId = currentCell.getSource().getId();
        		String targetNodeId = currentCell.getTarget().getId();
    			String sourceAlwaysCardinality = currentCell.getAttributes().get(mxConstants.SOURCEALWAYSCARDINALITY);
    			String sourceEventuallyCardinality = currentCell.getAttributes().get(mxConstants.SOURCEEVENTUALLYCARDINALITY);
    			String targetAlwaysCardinality = currentCell.getAttributes().get(mxConstants.TARGETALWAYSCARDINALITY);
    			String targetEventuallyCardinality = currentCell.getAttributes().get(mxConstants.TARGETEVENTUALLYCARDINALITY);
    			
//    			Cardinality sourceAlwaysCardinality = new Cardinality(nameValue.get("sourceAlwaysCardinality").toString());
//    			Cardinality sourceEventuallyCardinality = new Cardinality(nameValue.get("sourceEventuallyCardinality").toString());
//    			Cardinality targetAlwaysCardinality = new Cardinality(nameValue.get("targetAlwaysCardinality").toString());
//    			Cardinality targetEventuallyCardinality = new Cardinality(nameValue.get("targetEventuallyCardinality").toString());
    			builder.addClassEdge(id, relationshipType, sourceNodeId, 
    					sourceAlwaysCardinality, sourceEventuallyCardinality, targetNodeId, targetAlwaysCardinality,  targetEventuallyCardinality,
    				0, 0, 55, 55);
    			}
    		else if(edgeType.equals(mxConstants.ACTIVITYRELATION)) {
    			
    			String cardinalityType = currentCell.getAttributes().get(mxConstants.CARDINALITYTYPE);
    			String sourceNodeId = currentCell.getSource().getId();
        		String targetNodeId = currentCell.getTarget().getId();
    			String oldCardinalityType;
    			switch(cardinalityType)
    	        {
    	        	case mxConstants.RESPONSE: oldCardinalityType = "ActivityEdgeResponse";
    					break;
    				case mxConstants.UNARYRESPONSE: oldCardinalityType = "ActivityEdgeUnaryResponse";
    					break;
    				case mxConstants.NONRESPONSE: oldCardinalityType = "ActivityEdgeNonResponse";
    					break;
    				case mxConstants.PRECEDENCE: oldCardinalityType = "ActivityEdgePrecedence";      	
    					break;
    				case mxConstants.UNARYPRECEDENCE: oldCardinalityType = "ActivityEdgeUnaryPrecedence";      	
    					break;
    				case mxConstants.NONPRECEDENCE: oldCardinalityType = "ActivityEdgeNonPrecedence";        	
    					break;
    				case mxConstants.COEXISTENCE: oldCardinalityType = "ActivityEdgeCoExistence";        	
    					break;
    				case mxConstants.NONCOEXISTENCE: oldCardinalityType = "ActivityEdgeNonCoExistence";        	
    					break;
    				case mxConstants.RESPONSE_PRECEDENCE: oldCardinalityType = "ActivityEdgeResponse_ActivityEdgePrecedence";	        	
    					break;
    				case mxConstants.UNARYRESPONSE_UNARYPRECEDENCE: oldCardinalityType = "ActivityEdgeUnaryResponse_ActivityEdgeUnaryPrecedence";	        	
    					break;
    				case mxConstants.COEXISTENCE_COEXISTENCE: oldCardinalityType = "ActivityEdgeCoExistence_ActivityEdgeCoExistence";	        	
    					break;
    				case mxConstants.NONRESPONSE_NONPRECEDENCE: oldCardinalityType = "ActivityEdgeNonResponse_ActivityEdgeNonPrecedence";	        	
    					break;
    				case mxConstants.NONCOEXISTENCE_NONCOEXISTENCE: oldCardinalityType = "ActivityEdgeNonCoExistence_ActivityEdgeNonCoExistence";	        	
    					break;	

    				default: oldCardinalityType = "ActivityEdgeResponse";
    	        }
    			
    			builder.addActivityEdge(id, sourceNodeId, targetNodeId, oldCardinalityType, 0, 0);
    		}
    		else if(edgeType.equals(mxConstants.AOCRELATION)) {
    			String sourceAlwaysCardinality = currentCell.getAttributes().get(mxConstants.SOURCEALWAYSCARDINALITY);
    			String sourceEventuallyCardinality = currentCell.getAttributes().get(mxConstants.SOURCEEVENTUALLYCARDINALITY);
    			String targetAlwaysCardinality = currentCell.getAttributes().get(mxConstants.TARGETALWAYSCARDINALITY);
    			
    			mxCell startCell = (mxCell) currentCell.getSource();
    			String sourceId;
        		String targetId;
    			if(startCell.getAttributes().get(mxConstants.TYPE).equals(mxConstants.ACTIVITYCELL)){
    				sourceId = currentCell.getSource().getId();
            		targetId = currentCell.getTarget().getId();
    			}
    			else{
    				targetId = currentCell.getSource().getId();
    				sourceId = currentCell.getTarget().getId();
    			}
    			
    			builder.addAOCEdge(id, sourceId, 
    					sourceAlwaysCardinality, sourceEventuallyCardinality, targetId, targetAlwaysCardinality,  
        				0, 0);
    		}
    			
    		else if(edgeType.equals(mxConstants.CRELOCRELATION)) {    			
    			mxCell startCell = (mxCell) currentCell.getSource().getParent();
    			mxCell endCell = (mxCell) currentCell.getTarget().getParent();
    			String sourceId;
        		String targetId;
    			if(startCell.getAttributes().containsKey(mxConstants.TYPE) && 
    					startCell.getAttributes().get(mxConstants.TYPE).equals(mxConstants.ACTIVITYRELATION)){
    				sourceId = startCell.getId();
            		targetId = currentCell.getTarget().getId();
    			}
    			else{
    				targetId = currentCell.getSource().getId();
    				sourceId = endCell.getId();
    			}
    			System.out.println("id: " + id);
    			System.out.println("sourceId: " + sourceId);
    			System.out.println("targetId: " + targetId);
        		builder.addCrelOCEdge(id, sourceId, targetId,  0, 0);
    		}   			
			else if(edgeType.equals(mxConstants.CRELRTRELATION)) {
				mxCell startCell = (mxCell) currentCell.getSource().getParent();
				mxCell endCell = (mxCell) currentCell.getTarget().getParent();
    			String sourceId;
        		String targetId;
        		System.out.println("start type: " + startCell.getAttributes().get(mxConstants.TYPE));
        		System.out.println("end type: " + endCell.getAttributes().get(mxConstants.TYPE));
    			if(startCell.getAttributes().get(mxConstants.TYPE).equals(mxConstants.ACTIVITYRELATION)){
    				System.out.println("ACTIVITYRELATION");
    				sourceId = startCell.getId();
            		targetId = endCell.getId();
    			}
    			else{
    				System.out.println("not ACTIVITYRELATION");
    				targetId = startCell.getId();
    				sourceId = endCell.getId();
    			}
    			System.out.println("id: " + id);
    			System.out.println("sourceId: " + sourceId);
    			System.out.println("targetId: " + targetId);
    			builder.addCrelRTEdge(id, sourceId, targetId,  0, 0);
			}
    			
		}
    	

    	System.out.println("\n activity node size: " + model.getAllActivities().size());
    	for(ActivityNode activityNode : model.getAllActivities()){
        	System.out.println("\n activity id: " + activityNode.getID());
        	System.out.println("\n activity name: " + activityNode.getNodeName());
        }   
    	
    	System.out.println("\n class node size: " + model.getAllClasses().size());
    	for(ClassNode classNode : model.getAllClasses()){
        	System.out.println("\n class id: " + classNode.getID());
        	System.out.println("\n class name: " + classNode.getNodeName());
        }   
    	
    	System.out.println("\n activity relation size: " + model.getAllActivityRelations().size());
    	for(ActivityEdge activityEdge : model.getAllActivityRelations()){
        	System.out.println("\n activityEdge id: " + activityEdge.getID());
        	System.out.println("\n activityEdge type: " + activityEdge.getCardinalityType());
        }   
        
    	System.out.println("\n class relation size: " + model.getAllClassRelations().size());
    	for(ClassEdge edge : model.getAllClassRelations()){
        	System.out.println("\n edge id: " + edge.getID());
        	System.out.println("\n edge name: " + edge.getEdgeName());
        	System.out.println("\n source always name: " + edge.getSourceAlwaysCardinality().toString());
        	System.out.println("\n source eventually name: " + edge.getSourceEventuallyCardinality().toString());
        	System.out.println("\n target always name: " + edge.getTargetAlwaysCardinality().toString());
        	System.out.println("\n target eventually name: " + edge.getTargetEventuallyCardinality().toString());
        }  
    	
    	System.out.println("\n aoc relation size: " + model.getAllAOCRelations().size());
    	for(AOCEdge edge : model.getAllAOCRelations()){
        	System.out.println("\n edge id: " + edge.getID());
        	System.out.println("\n Activity always name: " + edge.getActivityAlwaysCardinality());
        	System.out.println("\n Activity eventually name: " + edge.getActivityEventuallyCardinality().toString());
        	System.out.println("\n class name: " + edge.getClassCardinality());
        } 
    	
    	System.out.println("\n crel rt relation size: " + model.getAllCrelRTRelations().size());
        for(CrelRTEdge edge : model.getAllCrelRTRelations()){
        	System.out.println("\n activityEdge id: " + edge.getID());
        	System.out.println("\n activityEdge id: " + edge.getRelatedActivityRelation().getID());
        	System.out.println("\n classEdge id: " + edge.getRelatedClassRelation().getID());
        }
        
        System.out.println("\n crel oc relation size: " + model.getAllCrelOCRelations().size());
        for(CrelOCEdge edge : model.getAllCrelOCRelations()){
        	System.out.println("\n activityEdge id: " + edge.getID());
        	System.out.println("\n activityEdge id: " + edge.getRelatedActivityRelation().getID());
        	System.out.println("\n class id: " + edge.getRelatedClass().getID());

        }

    }
    
    public OCBCModel getModel(){
    	return model;
    }
    
    class SAXHandler extends DefaultHandler {
        @Override
        public void startElement(String uri, String localName,
                                 String qName, Attributes attributes)
                throws SAXException {
            int attributeLength = attributes.getLength();
            
            if ("Node".equals(qName)) {
            	Map<String, String> nameValue = new HashMap<String, String>();
                for (int i = 0; i < attributeLength; i++) {                  
                    String name = attributes.getQName(i);
                    String value = attributes.getValue(i);
                    nameValue.put(name, value);
                }              
                builder.addNode(nameValue.get("type"), nameValue.get("name"), nameValue.get("ID"), 
                		Double.parseDouble(nameValue.get("positionX")), Double.parseDouble(nameValue.get("positionY")),
                		Double.parseDouble(nameValue.get("height")), Double.parseDouble(nameValue.get("width")));
            }
            
            if ("Edge".equals(qName)) {
            	Map<String, String> nameValue = new HashMap<String, String>();

                for (int i = 0; i < attributeLength; i++) {
                    String name = attributes.getQName(i);
                    String value = attributes.getValue(i);
                    nameValue.put(name, value);
                }

                String edgeType = nameValue.get("type");
  			               	
        		if(edgeType.equals("ClassEdge")) {
//        			Cardinality sourceAlwaysCardinality = new Cardinality(nameValue.get("sourceAlwaysCardinality").toString());
//        			Cardinality sourceEventuallyCardinality = new Cardinality(nameValue.get("sourceEventuallyCardinality").toString());
//        			Cardinality targetAlwaysCardinality = new Cardinality(nameValue.get("targetAlwaysCardinality").toString());
//        			Cardinality targetEventuallyCardinality = new Cardinality(nameValue.get("targetEventuallyCardinality").toString());
        			builder.addClassEdge(nameValue.get("ID"), nameValue.get("name"), nameValue.get("sourceNodeID"), 
        					nameValue.get("sourceAlwaysCardinality"), nameValue.get("sourceEventuallyCardinality"), 
        				nameValue.get("targetNodeID"), nameValue.get("targetAlwaysCardinality"),  
        				nameValue.get("targetEventuallyCardinality"),
        				Double.parseDouble(nameValue.get("transparentCellX")), Double.parseDouble(nameValue.get("transparentCellY")),
        				Double.parseDouble(nameValue.get("height")), Double.parseDouble(nameValue.get("width")));
        			}
        			else if(edgeType.equals("ActivityEdge")) 
        			builder.addActivityEdge(nameValue.get("ID"), nameValue.get("referenceNodeID"), 
        				 nameValue.get("targetNodeID"), nameValue.get("cardinalityType"),  
        				Double.parseDouble(nameValue.get("transparentCellX")), Double.parseDouble(nameValue.get("transparentCellY")));
        		else if(edgeType.equals("AOCEdge")) 
        			builder.addAOCEdge(nameValue.get("ID"), nameValue.get("sourceNodeID"), 
        				nameValue.get("sourceAlwaysCardinality"), nameValue.get("sourceEventuallyCardinality"), nameValue.get("targetNodeID"), nameValue.get("targetCardinality"),  
        				Double.parseDouble(nameValue.get("transparentCellX")), Double.parseDouble(nameValue.get("transparentCellY")));
        		else if(edgeType.equals("CrelOCEdge")) 
        			builder.addCrelOCEdge(nameValue.get("ID"), nameValue.get("activityRelationID"), nameValue.get("classNodeID"),  
        				Double.parseDouble(nameValue.get("transparentCellX")), Double.parseDouble(nameValue.get("transparentCellY")));
				else if(edgeType.equals("CrelRTEdge")) 
        			builder.addCrelRTEdge(nameValue.get("ID"), nameValue.get("activityRelationID"), nameValue.get("classRelationID"),  
        				Double.parseDouble(nameValue.get("transparentCellX")), Double.parseDouble(nameValue.get("transparentCellY")));
            }
        }
    }
}