package org.processmining.mxgraph.examples.swing.listener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import org.processmining.mxgraph.examples.swing.editor.BasicGraphEditor;
import org.processmining.mxgraph.examples.swing.visualization.Style;
import org.processmining.mxgraph.model.mxCell;
import org.processmining.mxgraph.swing.mxGraphComponent;
import org.processmining.mxgraph.util.mxConstants;
import org.processmining.ocbc.conformance.visualization.views.ResultModelView;
import org.processmining.ocbc.models.OCBCModel;

public class GraphComponentMouseListener extends MouseAdapter{

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	private mxGraphComponent graphComponent;
//	private BasicGraphEditor graphEditor;
    private double souceLabelPosition = -0.6;
    private double targetLabelPosition = 0.4;
    private double connectorSize = 5;
    private double connectorX = 0.2;
	
    private BasicGraphEditor modelEditor;
    private ResultModelView conformanceCheckingModelView;
    private OCBCModel model;
	public GraphComponentMouseListener(OCBCModel model, mxGraphComponent graphComponent) {
		this.model = model;
		this.graphComponent = graphComponent;
	}
	
	public void setModelEditor(BasicGraphEditor modelEditor){
		this.modelEditor = modelEditor;
		
	}
	
	public void setConformanceCheckingModelView(ResultModelView conformanceCheckingModelView){
		this.conformanceCheckingModelView = conformanceCheckingModelView;
	}
	
	/**
	 * 
	 */
	public void mousePressed(MouseEvent e)
	{
		// Handles context menu on the Mac where the trigger is on mousepressed
		System.out.println("mousePressed in GraphComponentMouseListener of org.processmining.mxgraph.examples.swing.listener");
		
		int dragStartX = e.getX();
		int dragStartY = e.getY();

		System.out.println("graphComponent dragStartX: " + dragStartX);
		System.out.println("graphComponent dragStartY: " + dragStartY);
	}

	/**
	 * 
	 */
	public void mouseReleased(MouseEvent e)
	{
		System.out.println("mouseReleased in GraphComponentMouseListener of org.processmining.mxgraph.examples.swing.listener");
		if(model.getModelEnvironment().equals(mxConstants.model_editor)){
			modelEditor.updateEditor();

			System.out.println("released in BasicGraphEditor");
			if (e.isPopupTrigger())
			{
				modelEditor.showGraphPopupMenu(e);
			}
		}

	}
	
	/**
	 * monitor the clicking event
	 */
    public void mouseClicked(MouseEvent e) {  
    	System.out.println("mouseClicked in GraphComponentMouseListener of org.processmining.mxgraph.examples.swing.listener");
    	String modelEnvironment = model.getModelEnvironment();
    	System.out.println("modelEnvironment: " + modelEnvironment);
        // TODO Auto-generated method stub  
    	mxCell selectedCell = (mxCell) graphComponent.getGraph().getSelectionCell();
    	if(modelEnvironment.equals(mxConstants.model_editor)){
    		System.out.println("in the model_editor mode");
    		modelEditor.restoreAllCells();
    		modelEditor.disableAllBoxes();
    		modelEditor.getAttributePanel().disableAttributePanel();    	
	    	if(selectedCell != null){	    		
	    		//highlight the selected cell
	    		modelEditor.highlightCell(selectedCell);	    		
	    		//update the style box    		
				String edgeStyle = Style.getStyleElementValueFromCell(graphComponent.getGraph(), selectedCell, mxConstants.STYLE_EDGE);
				if(edgeStyle != null && selectedCell.isEdge()){
					modelEditor.getToolBar().getEdgeStyleBox().setEnabled(true);
					modelEditor.getToolBar().getEdgeStyleBox().setSelectedItem(edgeStyle);
				}		    		
				//update the constraint boxes
				modelEditor.updateConstraintBoxes(selectedCell);
				//update the attribute panel
				modelEditor.getAttributePanel().updateAttributePanel(selectedCell);	
	    		//update the constraint instance and distribution panel for the selected cell
				modelEditor.updateConstraintPanel(selectedCell);
	    	}
	    	graphComponent.getGraph().refresh();	
    	}
    	else if(modelEnvironment.equals(mxConstants.conformance_checking_plugin)){
    		System.out.println("in the conformance_checking_plugin mode");
		}    
    }  
}