package org.processmining.mxgraph.examples.swing.panels;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JPanel;
import javax.swing.JRadioButton;

import org.processmining.mxgraph.examples.swing.editor.BasicGraphEditor;
import org.processmining.mxgraph.model.mxCell;
import org.processmining.mxgraph.swing.mxGraphComponent;
import org.processmining.mxgraph.util.mxConstants;
import org.processmining.mxgraph.view.mxGraph;
import org.processmining.ocbc.discovery.filter.SimplifyModel;
import org.processmining.ocbc.models.ActivityEdge;
import org.processmining.ocbc.models.OCBCModel;

import com.fluxicon.slickerbox.factory.SlickerFactory;

public class FilterPanel extends JPanel {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private SlickerFactory slicker = SlickerFactory.instance();
	private JRadioButton radioResponse = slicker.createRadioButton("response");
	private JRadioButton radioUnaryResponse = slicker.createRadioButton("unary response");
	private JRadioButton radioNonResponse = slicker.createRadioButton("non-response");
	private JRadioButton radioPrecedence = slicker.createRadioButton("precedence");
	private JRadioButton radioUnaryPrecedence = slicker.createRadioButton("unary precedence");
	private JRadioButton radioNonPrecedence = slicker.createRadioButton("non-precedence");
	private JRadioButton radioCoExistence = slicker.createRadioButton("co-existence");
	private JRadioButton radioNonCoExistence = slicker.createRadioButton("non-coexistence");
	
	private JRadioButton radioBrotherConstraint = slicker.createRadioButton("twin constraint");
	private JRadioButton radioFatherConstraint = slicker.createRadioButton("father constraint");
	private JRadioButton radioTransitiveConstraint = slicker.createRadioButton("transitive constraint");
	
	private String radioTypeGlobal;//used to access in the listener action
	private JRadioButton radioGlobal;//used to access in the listener action
	
	private Map<String, JRadioButton> behaviorConstraintTypeVSRadioButton = new HashMap<String, JRadioButton>();
	private Map<String, JRadioButton> behaviorConstraintNewTypeVSRadioButton = new HashMap<String, JRadioButton>();
	private Map<String, JRadioButton> activityNameVSRadioButton = new HashMap<String, JRadioButton>();
	
	private JButton selectAllButton = new JButton("select/cancel all");
	private JButton selectAllActivityButton = new JButton("select/cancel all");
	private JButton selectAllNewTypeButton = new JButton("select/cancel all");
	private boolean selectionFlag = true;
	private boolean selectionActivityFlag = true;
	private boolean selectionNewTypeFlag = true;
	private JCheckBox simplifiedBox = new JCheckBox("simplified");
	private mxGraphComponent graphComponent;
	private BasicGraphEditor graphEditor;
	private JPanel relationTypePanel;
	public static final Map<String, Object[]> behavioralConstraintTypeVSCellSet = new HashMap<String, Object[]>();
	
	public FilterPanel(final mxGraphComponent graphComponent, BasicGraphEditor graphEditor) {
	    super();
	    this.graphComponent = graphComponent;
	    this.graphEditor = graphEditor;
	    mxGraph graph = graphComponent.getGraph();
	    relationTypePanel = new JPanel();
	    relationTypePanel.setLayout(new GridBagLayout());
    
	    //initialize all radio as selected
	    radioResponse.setSelected(true);
	    radioUnaryResponse.setSelected(true);
	    radioNonResponse.setSelected(true);
	    radioPrecedence.setSelected(true);
	    radioUnaryPrecedence.setSelected(true);
	    radioNonPrecedence.setSelected(true);
	    radioCoExistence.setSelected(true);
	    radioNonCoExistence.setSelected(true);
	    
	    radioBrotherConstraint.setSelected(true);
	    radioFatherConstraint.setSelected(true);
	    radioTransitiveConstraint.setSelected(true);
	    
	    behaviorConstraintTypeVSRadioButton.put(mxConstants.RESPONSE, radioResponse);
	    behaviorConstraintTypeVSRadioButton.put(mxConstants.UNARYRESPONSE, radioUnaryResponse);
	    behaviorConstraintTypeVSRadioButton.put(mxConstants.NONRESPONSE, radioNonResponse);
	    behaviorConstraintTypeVSRadioButton.put(mxConstants.PRECEDENCE, radioPrecedence);
	    behaviorConstraintTypeVSRadioButton.put(mxConstants.UNARYPRECEDENCE, radioUnaryPrecedence);
	    behaviorConstraintTypeVSRadioButton.put(mxConstants.NONPRECEDENCE, radioNonPrecedence);
	    behaviorConstraintTypeVSRadioButton.put(mxConstants.COEXISTENCE, radioCoExistence);
	    behaviorConstraintTypeVSRadioButton.put(mxConstants.NONCOEXISTENCE, radioNonCoExistence);
	    
	    behaviorConstraintNewTypeVSRadioButton.put(mxConstants.twinConstraint, radioBrotherConstraint);
	    behaviorConstraintNewTypeVSRadioButton.put(mxConstants.fatherConstraint, radioFatherConstraint);
//	    behaviorConstraintNewTypeVSRadioButton.put("twin_constraint", radioBrotherConstraint);
//	    behaviorConstraintNewTypeVSRadioButton.put("implied_constraint", radioFatherConstraint);
//	    behaviorConstraintNewTypeVSRadioButton.put(mxConstants.transitive, radioTransitiveConstraint);	    
	    
		GridBagConstraints c = new GridBagConstraints();
		c = new GridBagConstraints();
		c.gridx = 0;
		c.gridy = 1;
		c.anchor = GridBagConstraints.LINE_START;
		relationTypePanel.add(radioUnaryResponse, c);
	
		c = new GridBagConstraints();
		c.gridx = 0;
		c.gridy = 2;
		c.anchor = GridBagConstraints.LINE_START;
		relationTypePanel.add(radioUnaryPrecedence, c);
		
		c = new GridBagConstraints();
		c.gridx = 0;
		c.gridy = 3;
		c.anchor = GridBagConstraints.LINE_START;
		relationTypePanel.add(radioResponse, c);
		
		c = new GridBagConstraints();
		c.gridx = 0;
		c.gridy = 4;
		c.anchor = GridBagConstraints.LINE_START;
		relationTypePanel.add(radioPrecedence, c);
		
		c = new GridBagConstraints();
		c.gridx = 0;
		c.gridy = 5;
		c.anchor = GridBagConstraints.LINE_START;
		relationTypePanel.add(radioCoExistence, c);

		c = new GridBagConstraints();
		c.gridx = 0;
		c.gridy = 6;
		c.anchor = GridBagConstraints.LINE_START;
		relationTypePanel.add(radioNonResponse, c);
		
		c = new GridBagConstraints();
		c.gridx = 0;
		c.gridy = 7;
		c.anchor = GridBagConstraints.LINE_START;
		relationTypePanel.add(radioNonPrecedence, c);
		
		c = new GridBagConstraints();
		c.gridx = 0;
		c.gridy = 8;
		c.anchor = GridBagConstraints.LINE_START;
		relationTypePanel.add(radioNonCoExistence, c);
		
		c = new GridBagConstraints();
		c.gridx = 0;
		c.gridy = 9;
		c.anchor = GridBagConstraints.LINE_START;	
		relationTypePanel.add(selectAllButton, c);
		
		c = new GridBagConstraints();
		c.gridx = 1;
		c.gridy = 1;
		c.anchor = GridBagConstraints.LINE_START;
		relationTypePanel.add(radioBrotherConstraint, c);
		
		c = new GridBagConstraints();
		c.gridx = 1;
		c.gridy = 2;
		c.anchor = GridBagConstraints.LINE_START;
		relationTypePanel.add(radioFatherConstraint, c);
		
		//hide the transitive button
//		c = new GridBagConstraints();
//		c.gridx = 1;
//		c.gridy = 3;
//		c.anchor = GridBagConstraints.LINE_START;
//		relationTypePanel.add(radioTransitiveConstraint, c);
		
		c = new GridBagConstraints();
		c.gridx = 1;
		c.gridy = 4;
		c.anchor = GridBagConstraints.LINE_START;
		relationTypePanel.add(selectAllNewTypeButton, c);		
		
		for(String type : behaviorConstraintTypeVSRadioButton.keySet()){
			JRadioButton radioButton = behaviorConstraintTypeVSRadioButton.get(type);
			radioTypeGlobal = type;
			radioGlobal = radioButton;
			radioGlobal.addItemListener(new ItemListener() {			
				public void itemStateChanged(ItemEvent e) {				
					System.out.println("itemStateChanged for filter behavioral constraint type");
					updateBehaviorConstraints();
				}
			});
		}		
		
		selectAllButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {	
				if(selectionFlag){
					selectionFlag = false;
				    radioResponse.setSelected(false);
				    radioUnaryResponse.setSelected(false);
				    radioNonResponse.setSelected(false);
				    radioPrecedence.setSelected(false);
				    radioUnaryPrecedence.setSelected(false);
				    radioNonPrecedence.setSelected(false);
				    radioCoExistence.setSelected(false);
				    radioNonCoExistence.setSelected(false);
				}
				else{
					selectionFlag = true;
				    radioResponse.setSelected(true);
				    radioUnaryResponse.setSelected(true);
				    radioNonResponse.setSelected(true);
				    radioPrecedence.setSelected(true);
				    radioUnaryPrecedence.setSelected(true);
				    radioNonPrecedence.setSelected(true);
				    radioCoExistence.setSelected(true);
				    radioNonCoExistence.setSelected(true);
				}
			}
		});

		for(String type : behaviorConstraintNewTypeVSRadioButton.keySet()){
			JRadioButton radioButton = behaviorConstraintNewTypeVSRadioButton.get(type);
			radioTypeGlobal = type;
			radioGlobal = radioButton;
			radioGlobal.addItemListener(new ItemListener() {			
				public void itemStateChanged(ItemEvent e) {				
					System.out.println("itemStateChanged");
					updateBehaviorConstraints();
				}
			});
		}	
		
		selectAllNewTypeButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {	
				if(selectionNewTypeFlag){
					selectionNewTypeFlag = false;
					for(String type : behaviorConstraintNewTypeVSRadioButton.keySet()){
						JRadioButton radioButton = behaviorConstraintNewTypeVSRadioButton.get(type);
						radioButton.setSelected(false);
					}
				}
				else{
					selectionNewTypeFlag = true;
					for(String type : behaviorConstraintNewTypeVSRadioButton.keySet()){
						JRadioButton radioButton = behaviorConstraintNewTypeVSRadioButton.get(type);
						radioButton.setSelected(true);
					}
				}
			}
		});
		
	    //add activity radios
    	Object[] allVertexs = graph.getChildVertices(graph.getDefaultParent());
    	int activityNum = 0;
    	for(int i=0; i<allVertexs.length; i++){
    		System.out.println("add actiity radios");
    		String displayName = ((mxCell)allVertexs[i]).getValue().toString();
    		String id = ((mxCell)allVertexs[i]).getId();
    		String type = ((mxCell)allVertexs[i]).getAttributes().get(mxConstants.TYPE);
    		System.out.println("type: " + type);
    		if(type.equals(mxConstants.ACTIVITYCELL)){
    			System.out.println("enter add actiity radios");
    			activityNum ++;
    			activityNameVSRadioButton.put(displayName, slicker.createRadioButton(displayName));
				c = new GridBagConstraints();
				c.gridx = 2;
				c.gridy = activityNum;
				c.anchor = GridBagConstraints.LINE_START;
				relationTypePanel.add(activityNameVSRadioButton.get(displayName), c);
				activityNameVSRadioButton.get(displayName).setSelected(true);
    		}
    		//add the select/cancel button
    		c = new GridBagConstraints();
			c.gridx = 2;
			c.gridy = activityNum + 1;
			c.anchor = GridBagConstraints.LINE_START;
    		relationTypePanel.add(selectAllActivityButton, c);
        }
    	
    	for(String type : activityNameVSRadioButton.keySet()){
			JRadioButton radioButton = activityNameVSRadioButton.get(type);
			radioTypeGlobal = type;
			radioGlobal = radioButton;
			radioGlobal.addItemListener(new ItemListener() {			
				public void itemStateChanged(ItemEvent e) {				
					System.out.println("itemStateChanged");
					updateBehaviorConstraints();
				}
			});
		}
    	
		selectAllActivityButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {	
				if(selectionActivityFlag){
					selectionActivityFlag = false;
					for(String type : activityNameVSRadioButton.keySet()){
						JRadioButton radioButton = activityNameVSRadioButton.get(type);
						radioButton.setSelected(false);
					}
				}
				else{
					selectionActivityFlag = true;
					for(String type : activityNameVSRadioButton.keySet()){
						JRadioButton radioButton = activityNameVSRadioButton.get(type);
						radioButton.setSelected(true);
					}
				}
			}
		});
		//update my new defined constraint type
		for(String type : behaviorConstraintNewTypeVSRadioButton.keySet()){
			JRadioButton radioButton = behaviorConstraintNewTypeVSRadioButton.get(type);
			updateBehaviorConstraintsByNewType(type, radioButton);
		}
		updateConstraintNum();
		
		this.add(relationTypePanel);
	}
	
	public void updateBehaviorConstraints(){
		//update the formal constraint type
		for(String type : behaviorConstraintTypeVSRadioButton.keySet()){
			JRadioButton radioButton = behaviorConstraintTypeVSRadioButton.get(type);
			updateBehaviorConstraintsByType(type, radioButton);
		}
		//update my new defined constraint type
		for(String type : behaviorConstraintNewTypeVSRadioButton.keySet()){
			JRadioButton radioButton = behaviorConstraintNewTypeVSRadioButton.get(type);
			updateBehaviorConstraintsByNewType(type, radioButton);
		}
		//update activity type
		for(String type : activityNameVSRadioButton.keySet()){
			JRadioButton radioButton = activityNameVSRadioButton.get(type);
			updateBehaviorConstraintByActivity(type, radioButton);
		}
		updateConstraintNum();
		//update the activity space of button panel for sliders
		graphEditor.getButtonPanel().updateShownActivityList();
		
	}
	
	public List<mxCell> filterBehaviorConstraintByType(String relationType){
		mxGraph graph = graphComponent.getGraph();
		System.out.println("filterBehaviorConstraintByType: " + relationType);
		List<mxCell> cellList = new ArrayList<mxCell>();
		for(mxCell cell : graph.getActivityRelationList()){
			String cardinalityType = cell.getAttributes().get(mxConstants.CARDINALITYTYPE);	
			System.out.println("cardinalityType: " + cardinalityType);
			if(cardinalityType.equals(relationType)){
				System.out.println("equal");
				mxCell connectorCell = (mxCell) cell.getChildAt(0);
				Object[] edgeCells = graph.getChildEdges(graph.getDefaultParent());
	    		for(int i=0; i<edgeCells.length; i++){
	    			mxCell edge = (mxCell) edgeCells[i];
	    			if(edge.getSource().equals(connectorCell) || edge.getTarget().equals(connectorCell)){
	    				cellList.add(edge);
	    			}				    			
	    		}
	    		cellList.add(cell);
			}
		}
		System.out.println("cellList size: " + cellList.size());
		return cellList;
	}
	
	public void updateBehaviorConstraintsByNewType(String relationType, JRadioButton button){	
		mxGraph graph = graphComponent.getGraph();
		Object[] removedCells = null;
		OCBCModel model = graphEditor.getOCBCModel();
		
		//update the father constraint flag		
		if(relationType.equals(mxConstants.fatherConstraint)){
//		if(relationType.equals("implied_constraint")){
			
			for(mxCell cell : graph.getActivityRelationList()){
				String cellId = cell.getId();
				ActivityEdge activityRelation = model.getActivityRelationById(cellId);
				//check if there exists a child relation
				for(mxCell cellChild : graph.getActivityRelationList()){
					String cellIdChild = cellChild.getId();
					ActivityEdge activityRelationChild = model.getActivityRelationById(cellIdChild);
					if(activityRelation != null && activityRelationChild != null && activityRelation.isFatherActivityRelationOf(activityRelationChild)){
						cell.getAttributes().put(mxConstants.fatherConstraint, mxConstants.Yes);
						break;
					}			
				}
			}
		}	
		
		//update the twin constraint flag
		else if(relationType.equals(mxConstants.twinConstraint)){
			for(int i=0; i<graph.getActivityRelationList().size(); i++){
				mxCell cell = graph.getActivityRelationList().get(i);
				String cellId = cell.getId();
				ActivityEdge activityRelation = model.getActivityRelationById(cellId);
				//check if there exists a child relation
				for(int j=i+1; j<graph.getActivityRelationList().size(); j++){
					mxCell cellChild = graph.getActivityRelationList().get(j);
					String cellIdChild = cellChild.getId();
					ActivityEdge activityRelationChild = model.getActivityRelationById(cellIdChild);
					if(activityRelation != null && activityRelationChild != null && activityRelation.isTwinActivityRelationOf(activityRelationChild)){
						cellChild.getAttributes().put(mxConstants.twinConstraint, mxConstants.Yes);
					}
				}								
			}
		}
		
		//update the transitive constraint flag
		else if(relationType.equals(mxConstants.transitive)){
			SimplifyModel reducer = new SimplifyModel(model);
			reducer.filterTransitiveActicityEdges();
			List<String> transitiveRelationIdList = model.getTransitiveRelationIdList();
			for(mxCell cell : graph.getActivityRelationList()){
				String cellId = cell.getId();
				System.out.println("get each cell");	
				if(transitiveRelationIdList.contains(cellId)){
					System.out.println("update transitive flag");	
					cell.getAttributes().put(mxConstants.transitive, mxConstants.Yes);
				}
			}						
		}
		
		
		if(button.isSelected()){	
//			updateBehaviorConstraints();
//			System.out.println("SELECTED");	
		}			
		else{
			System.out.println("DESELECTED");
			List<mxCell> removedCellList = new ArrayList<mxCell>();
			for(mxCell cell : graph.getActivityRelationList()){
//				String cardinalityType = cell.getAttributes().get(mxConstants.CARDINALITYTYPE);	
//				String brotherConstraintFlag = cell.getAttributes().get(mxConstants.brotherConstraint);	
				String constraintFlag = cell.getAttributes().get(relationType);	
				System.out.println("constraintFlag in deselected: " + constraintFlag);
				if(constraintFlag != null && constraintFlag.equals(mxConstants.Yes)){
    				mxCell connectorCell = (mxCell) cell.getChildAt(0);
    				Object[] edgeCells = graph.getChildEdges(graph.getDefaultParent());
		    		for(int i=0; i<edgeCells.length; i++){
		    			mxCell edge = (mxCell) edgeCells[i];
		    			if(edge.getSource().equals(connectorCell) || edge.getTarget().equals(connectorCell)){
		    				removedCellList.add(edge);
		    			}				    			
		    		}
		    		removedCellList.add(cell);
				}
			}
			removedCells = removedCellList.toArray();
			graph.removeCells(removedCells);
		}
		graphComponent.updateUI();
		graphComponent.refresh();
	}
	
	/**
	 * filter the behavioral constraints based on the type selection condition (checked or not) 
	 * @param relationType
	 * @param button
	 */
	public void updateBehaviorConstraintsByType(String relationType, JRadioButton button){	
		mxGraph graph = graphComponent.getGraph();
		Object[] satisfiedCells = null;
		if(!behavioralConstraintTypeVSCellSet.containsKey(relationType)){
			behavioralConstraintTypeVSCellSet.put(relationType, filterBehaviorConstraintByType(relationType).toArray());
		}
		System.out.println("updateBehaviorConstraintsByType: " + relationType);
		satisfiedCells = behavioralConstraintTypeVSCellSet.get(relationType);

		System.out.println("satisfiedCells length: " + satisfiedCells.length);
		if(button.isSelected()){	
			System.out.println("SELECTED");
			graph.setRelationDisplayFlag(relationType, true);
			if(satisfiedCells != null){
				graph.addCells(satisfiedCells);
				System.out.println("behavioralConstraintTypeVSCellSet add: " + relationType);
			}	
		}			
		else{			
			graph.removeCells(satisfiedCells);
			System.out.println("behavioralConstraintTypeVSCellSet remove: " + relationType);
		}					
		graphComponent.updateUI();
		graphComponent.refresh();
	}
	
	/**
	 * when updating the model (i.e., rediscover the model), the map of behavioralConstraintTypeVSCellSet
	 * should be clear
	 */
	public void clearConstraintTypeVSCellsMap(){
		behavioralConstraintTypeVSCellSet.clear();
	}
	
	
	public void updateBehaviorConstraintByActivity(String activity, JRadioButton button){	
		mxGraph graph = graphComponent.getGraph();
		System.out.println("filterBehaviorConstraintByActivity: " + activity);
		List<mxCell> removedCellList = new ArrayList<mxCell>();
		Object[] removedCells = null;
		if(!button.isSelected()){	
			for(mxCell cell : graph.getActivityRelationList()){
				String sourceActivityName = (String) cell.getSource().getValue();
				String targetActivityName = (String) cell.getTarget().getValue();
				if(sourceActivityName.equals(activity) || targetActivityName.equals(activity)){
					removedCellList.add(cell);
				}
			}
			removedCells = removedCellList.toArray();
			graph.removeCells(removedCells);
			graphComponent.updateUI();
			graphComponent.refresh();
		}					
	}
	
	/**
	 * update the the number of shown constraints in the graph for each type
	 */
	public void updateConstraintNum(){
		mxGraph graph = graphComponent.getGraph();
		for(String type : behaviorConstraintTypeVSRadioButton.keySet()){
			JRadioButton radioButton = behaviorConstraintTypeVSRadioButton.get(type);
			System.out.println("type in updateConstraintNum: " + type);
			int mun = 0;
			for(mxCell cell : graph.getActivityRelationList()){
				String cardinalityType = cell.getAttributes().get(mxConstants.CARDINALITYTYPE);	
				if(cardinalityType.equals(type)){
					mun ++;
				}				
			}
			radioButton.setText(type + ": " + mun);
			System.out.println("mun in updateConstraintNum: " + mun);
		}		
		
		for(String type : behaviorConstraintNewTypeVSRadioButton.keySet()){
			JRadioButton radioButton = behaviorConstraintNewTypeVSRadioButton.get(type);
			int mun = 0;
			for(mxCell cell : graph.getActivityRelationList()){
				String constraintFlag = cell.getAttributes().get(type);	
				System.out.println("constraintFlag in deselected in new type: " + constraintFlag);
				if(constraintFlag != null && constraintFlag.equals(mxConstants.Yes)){
					mun ++;
				}
			}
			radioButton.setText(type + ": " + mun);
			if(type.equals(mxConstants.fatherConstraint)){
				radioButton.setText("implied_constraint" + ": " + mun);
			}
			else if(type.equals(mxConstants.twinConstraint)){
				radioButton.setText("twin_constraint" + ": " + mun);
			}
			
		}
		
		for(String type : activityNameVSRadioButton.keySet()){
			JRadioButton radioButton = activityNameVSRadioButton.get(type);
			int mun = 0;
			for(mxCell cell : graph.getActivityRelationList()){
				String sourceActivityName = (String) cell.getSource().getValue();
				String targetActivityName = (String) cell.getTarget().getValue();
				if(sourceActivityName.equals(type) || targetActivityName.equals(type)){
					mun ++;
				}
			}
			radioButton.setText(type + ": " + mun);
		}	
	}
	
	/**
	 * initialize All Radio buttons As Selected;
	 * should be used after updating the model
	 */
	public void initializeAllRadioButtonsAsSelected(){
	    radioResponse.setSelected(true);
	    radioUnaryResponse.setSelected(true);
	    radioNonResponse.setSelected(true);
	    radioPrecedence.setSelected(true);
	    radioUnaryPrecedence.setSelected(true);
	    radioNonPrecedence.setSelected(true);
	    radioCoExistence.setSelected(true);
	    radioNonCoExistence.setSelected(true);
	    
	    for(String type : activityNameVSRadioButton.keySet()){
			JRadioButton radioButton = activityNameVSRadioButton.get(type);
			radioButton.setSelected(true);
	    }
	}
    public static void main(String[] args) {
    	
//        JFrame frame = new JFrame("Hello world");
//       
//        FilterPanel panel = new FilterPanel();
//       
//        frame.add(panel);
//
//        frame.setSize(600, 400);
//        frame.setVisible(true);
       
     }

}