package org.processmining.mxgraph.examples.swing.panels;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Desktop;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Frame;
import java.awt.GradientPaint;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JEditorPane;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JSplitPane;
import javax.swing.JTextField;
import javax.swing.event.HyperlinkEvent;
import javax.swing.event.HyperlinkListener;

import org.processmining.common.dialogue.IOFileDialogue;
import org.processmining.mxgraph.model.mxCell;
import org.processmining.mxgraph.util.mxConstants;
import org.processmining.mxgraph.view.mxGraph;
import org.processmining.ocbc.models.OCBCModel;

public class GenerateInformationSystemFrame extends JDialog
{

    private JComboBox<String> driverBox = new JComboBox<String>();
    private String schemaName = ""; //record the selected schema name, such as "onlineshopping"
    private JLabel driverLabel = new JLabel("    driver:    ");
    private JLabel hostNameLabel = new JLabel("hostname:");
    private JLabel portLabel = new JLabel("      port:     ");
    private JLabel userNameLabel = new JLabel("      user:    ");
    private JLabel passwordLabel = new JLabel("password:");
    private JLabel databaseLabel = new JLabel("database: ");
    
    private JTextField hostNameField = new JTextField();
    private JTextField portField = new JTextField();
    private JTextField userNameField = new JTextField();
    private JTextField passwordField = new JTextField();
    private JTextField databaseField = new JTextField();
    private JButton connectButton = new JButton("check server connection");
    private JButton generateDBButton = new JButton("generate database");
    private JButton generateIFButton = new JButton("generate interface");
    
//    private Connection conn = null;
    private Map<String, String> parameterNameVSValue = new HashMap<String, String>();
    private File lastFile;
    OCBCModel model;
    private mxGraph graph;
	/**
	 * 
	 */
	private static final long serialVersionUID = -3378029138434324390L;
	/**
	 * modified by Guangming Li on June 14,2017
	 */
	public GenerateInformationSystemFrame(Frame owner, mxGraph graph)
	{
		super(owner);
//		this.model = model;
		this.graph = graph;
		setTitle("Information System Generator");
		setLayout(new BorderLayout());

		// Creates the gradient panel
		JPanel panel = new JPanel(new BorderLayout())
		{

			/**
			 * 
			 */
			private static final long serialVersionUID = -5062895855016210947L;

			/**
			 * 
			 */
			public void paintComponent(Graphics g)
			{
				super.paintComponent(g);

				// Paint gradient background
				Graphics2D g2d = (Graphics2D) g;
				g2d.setPaint(new GradientPaint(0, 0, Color.WHITE, getWidth(),
						0, getBackground()));
				g2d.fillRect(0, 0, getWidth(), getHeight());
			}

		};

		panel.setBorder(BorderFactory.createCompoundBorder(BorderFactory
				.createMatteBorder(0, 0, 1, 0, Color.GRAY), BorderFactory
				.createEmptyBorder(8, 8, 12, 8)));

		// Adds title
//		JLabel titleLabel = new JLabel("About OCBC Model Editor");
//		titleLabel.setFont(titleLabel.getFont().deriveFont(Font.BOLD));
//		titleLabel.setBorder(BorderFactory.createEmptyBorder(4, 0, 0, 0));
//		titleLabel.setOpaque(false);
//		panel.add(titleLabel, BorderLayout.NORTH);

		// Adds introduction with a link to the OCBC website
		JEditorPane jep = new JEditorPane("text/html", "This interface generate an information sytem based on the "
				+ "designed OCBC model. More precisely, the class model is transformed into a database, "
				+ "the activities (with attributes) are transformed into buttons (with input fields) on interfaces, "
				+ "the constraints are transformed into the enabled conditions of buttons and the AOC relationships "
				+ "are transformed into SQL sentences in the function corresponding to the buttons. "
				+ "For more information, please visit <a href='http://www.win.tue.nl/ocbc/softwares/model_editor.html'>"
			  	+ "http://www.win.tue.nl/ocbc/softwares/model_editor.html</a>.");
	
		jep.setEditable(false);
		jep.setOpaque(false);
		jep.addHyperlinkListener(new HyperlinkListener() {						
			public void hyperlinkUpdate(HyperlinkEvent hle) {				
			    if (HyperlinkEvent.EventType.ACTIVATED.equals(hle.getEventType())) {
			    	if (Desktop.isDesktopSupported()) {
			    		try {
			  	            try {
								Desktop.getDesktop().browse(new URI(hle.getURL().toString()));
							} catch (URISyntaxException e) {
								// TODO Auto-generated catch block
								e.printStackTrace();
							}
			  	          } catch (IOException e) { /* TODO: error handling */ }			  	        		    					  	          
			  	  	 }
			    }
			}
		});
		panel.add(jep, BorderLayout.CENTER);  		  
		getContentPane().add(panel, BorderLayout.NORTH);

		JPanel content = new JPanel();


    	driverBox.addItem("jdbc:mysql");
    	driverBox.setEditable(true);
    	
        connectButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent arg0) {
            	try {
            		String driver = driverBox.getSelectedItem().toString();
            		String hostName = hostNameField.getText();
            		String port = portField.getText();
            		String userName = userNameField.getText();
            		String password = passwordField.getText();        		
            		Connection conn = connect(driver, hostName, port, userName, password);
					if(conn != null){
	                    Statement stmt = conn.createStatement();       
	                    stmt.close();
					}
					System.out.println("I'm in of try-catch block in Java.");
                } catch (Throwable e) {
                	System.out.println("I'm out of try-catch block in Java.");
                    e.printStackTrace();
                }
            }
        });

        hostNameField.setPreferredSize(new Dimension(70,30));
        portField.setPreferredSize(new Dimension(70,30));
        userNameField.setPreferredSize(new Dimension(70,30));
        passwordField.setPreferredSize(new Dimension(70,30));
        
        hostNameField.setText("localhost");
        portField.setText("3306");
        userNameField.setText("root");
        passwordField.setText("kfgz0874981211");
   
    	content.setLayout(new BoxLayout(content, BoxLayout.Y_AXIS)); 
    	JPanel driverPanel = new JPanel();
    	driverPanel.setLayout(new BoxLayout(driverPanel, BoxLayout.X_AXIS)); 
    	driverPanel.add(driverLabel);
    	driverPanel.add(driverBox);  
    	content.add(driverPanel);
    	
    	JPanel hostPanel = new JPanel();
    	hostPanel.setLayout(new BoxLayout(hostPanel, BoxLayout.X_AXIS)); 
    	hostPanel.add(hostNameLabel);
    	hostPanel.add(hostNameField);  
    	content.add(hostPanel);
    	
    	JPanel portPanel = new JPanel();
    	portPanel.setLayout(new BoxLayout(portPanel, BoxLayout.X_AXIS)); 
    	portPanel.add(portLabel);
    	portPanel.add(portField);  
    	content.add(portPanel);
    	
    	JPanel userPanel = new JPanel();
    	userPanel.setLayout(new BoxLayout(userPanel, BoxLayout.X_AXIS)); 
    	userPanel.add(userNameLabel);
    	userPanel.add(userNameField);  
    	content.add(userPanel);

    	JPanel passwordPanel = new JPanel();
    	passwordPanel.setLayout(new BoxLayout(passwordPanel, BoxLayout.X_AXIS)); 
    	passwordPanel.add(passwordLabel);
    	passwordPanel.add(passwordField);  
    	content.add(passwordPanel);
    	
    	JPanel databasePanel = new JPanel();
    	databasePanel.setLayout(new BoxLayout(databasePanel, BoxLayout.X_AXIS)); 
    	databasePanel.add(databaseLabel);
    	databasePanel.add(databaseField);  
    	content.add(databasePanel);
    	
    	
    	
        content.add(connectButton);
		
        JPanel generationPanel = new JPanel();
        generateDBButton.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				generateDatabase();
			}
		});
		
        generationPanel.add(generateDBButton);
        
        generateIFButton.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				setVisible(false);
			}
		});
		
        generationPanel.add(generateIFButton);
             
		JSplitPane centerPanel = new JSplitPane(JSplitPane.VERTICAL_SPLIT, content, generationPanel);
		centerPanel.setResizeWeight(0.4);
		
		getContentPane().add(centerPanel, BorderLayout.CENTER);

		JPanel buttonPanel = new JPanel(new FlowLayout(FlowLayout.RIGHT));
		buttonPanel.setBorder(BorderFactory.createCompoundBorder(BorderFactory
				.createMatteBorder(1, 0, 0, 0, Color.GRAY), BorderFactory
				.createEmptyBorder(16, 8, 8, 8)));
		getContentPane().add(buttonPanel, BorderLayout.SOUTH);

		// Adds OK button to close window
		JButton closeButton = new JButton("Close");
		closeButton.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				setVisible(false);
			}
		});

		buttonPanel.add(closeButton);

		// Sets default button for enter key
		getRootPane().setDefaultButton(closeButton);

		setResizable(false);
		setSize(500, 500);
	}

	/**
	 * connect to server; return the connection if succeed; return null if fail
	 * @param driver
	 * @param host
	 * @param port
	 * @param userName
	 * @param password
	 * @return
	 * @throws Exception
	 */
	public Connection connect(String driver, String host, String port, String userName, String password) throws Exception{
		 String url = driver + "://" + host + ":" + port + "?user=" + userName + "&password=" + password + "&useUnicode=true&characterEncoding=UTF8";	
		 Connection conn;
		 try {
			 Class.forName("com.mysql.jdbc.Driver").newInstance();
			 conn = DriverManager.getConnection(url);
			 System.out.println("succeed connect to server!");
			 JOptionPane.showMessageDialog(null, "Succeed to connect to the server!");
		 } catch (SQLException e) {	 
			 System.out.println("Fail to connect to the server!");
			 e.printStackTrace();
			 JOptionPane.showMessageDialog(null, "Fail to connect to the server! \n" 
						+ "Connection url is: " + url + ". \n"
						+ e.getMessage());			
			 return null;
		 }	           
		 return conn;
	}

	/**
	 * generate database based on the designed class model;
	 * the class is transformed as tables; the class attributes are transformed as corresponding columns;
	 * 
	 */
	public void generateDatabase(){
		Connection conn = null;
		String driver = driverBox.getSelectedItem().toString();
		String host = hostNameField.getText();
		String port = portField.getText();
		String userName = userNameField.getText();
		String password = passwordField.getText();
		String databaseName = databaseField.getText();
		
		//if the connection fail, return
		try {
			conn = connect(driver, host, port, userName, password);
			//event though there is exception, the following sentences are still executed
			if(conn == null){
				System.out.println("Skip the process of creating the database!");
				JOptionPane.showMessageDialog(null, "Skip the process of creating the database!");
				return;
			}			
		} catch (Throwable e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		//if the connection succeed, create the database
		Statement stmt;
		String sql = null;	
        try {		
			sql = "CREATE DATABASE " + databaseName;
	        System.out.println(sql);  
	        stmt = conn.createStatement();
	        stmt.executeUpdate(sql);
	        System.out.println("Database created successfully...");
        } catch (SQLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			JOptionPane.showMessageDialog(null, "Fail to create the database! \n" 
					+ "SQL sentence is: " + sql + ". \n"
					+ e.getMessage());			
			return;
		}
        
        //if the database is created, create tables (with primary key)
        for(mxCell classNode : graph.generateClassList()){      	
        	String className = (String) classNode.getValue();
        	System.out.println("className: " + className);
//        	boolean firstColumn = true;
//        	sql = "CREATE TABLE " + databaseName + "." + className + " (";
//        	for(String attributeName : classNode.getAttributes().keySet()){
//        		String complex = classNode.getAttributes().get(attributeName);
//        		System.out.println("complex: " + complex);
//        		if(complex.split("-").length == 2){
//	        		String category = complex.split("-")[0];
//	        		String type = complex.split("-")[1];
//	        		if(category.equals(mxConstants.column) || category.equals(mxConstants.PK) || category.equals(mxConstants.FK)){
//	        			if(!firstColumn){
//	        				sql = sql + ", ";		        				
//	        			}
//	        			firstColumn = false;
//	        			sql = sql + attributeName + " " + type;
//	        		}	        		
//        		}
//        	}
//        	sql = sql + ")";
//        	System.out.println("sql: " + sql);
        	
        	List<String> nameList = new ArrayList<String>();
        	List<String> valueTypeList = new ArrayList<String>();
        	classNode.geneateAttributesofCategory(mxConstants.PK, nameList, valueTypeList);
        	classNode.geneateAttributesofCategory(mxConstants.FK, nameList, valueTypeList);
        	classNode.geneateAttributesofCategory(mxConstants.column, nameList, valueTypeList);
//        	for(int i=0; i<nameList.size(); i++){
//        		if(!firstColumn){
//    				sql = sql + ", ";		        				
//    			}
//    			firstColumn = false;
//    			sql = sql + nameList.get(i) + " " + valueTypeList.get(i);
//        	}
        	
        	sql = generateTableCreationSQL(databaseName, className, nameList, valueTypeList, classNode.generateAttributeListofCategory(mxConstants.PK));
//        	nameList.clear();
//        	valueTypeList.clear();   	
//        	classNode.geneateAttributesofCategory(mxConstants.FK, nameList, valueTypeList);
//        	for(int i=0; i<nameList.size(); i++){
//        		if(!firstColumn){
//    				sql = sql + ", ";		        				
//    			}
//    			firstColumn = false;
//    			sql = sql + nameList.get(i) + " " + valueTypeList.get(i);
//        	}
//        		
//        	nameList.clear();
//        	valueTypeList.clear();   	
//        	classNode.geneateAttributesofCategory(mxConstants.column, nameList, valueTypeList);
//        	for(int i=0; i<nameList.size(); i++){
//        		if(!firstColumn){
//        			sql = sql + nameList.get(i) + " " + valueTypeList.get(i);
//        		}
//        	}
//
//        	sql = sql + ")";
        	System.out.println("sql: " + sql);
        	
        	try {
				stmt.executeUpdate(sql);
			} catch (SQLException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
				JOptionPane.showMessageDialog(null, "Fail to create the table: " + className + ". \n" 
						+ "SQL sentence is: " + sql + ". \n"
						+ e.getMessage());			
			}
        	
        	
        }
        
        
//        //After tables are created, create foreign keys 
//        for(mxCell classNode : graph.generateClassList()){      	
//        	String className = (String) classNode.getValue();
//        	System.out.println("className: " + className);
//        	boolean firstColumn = true;
//        	sql = "ALTER TABLE " + databaseName + "." + className + " (";
//        	for(String attributeName : classNode.getAttributes().keySet()){
//        		String complex = classNode.getAttributes().get(attributeName);
//        		System.out.println("complex: " + complex);
//        		if(complex.split("-").length == 2){
//	        		String category = complex.split("-")[0];
//	        		String type = complex.split("-")[1];
//	        		if(category.equals(mxConstants.column) || category.equals(mxConstants.PK) || category.equals(mxConstants.FK)){
//	        			if(!firstColumn){
//	        				sql = sql + ", ";		        				
//	        			}
//	        			firstColumn = false;
//	        			sql = sql + attributeName + " " + type;
//	        		}	        		
//        		}
//        	}
//        	sql = sql + ")";
//        	System.out.println("sql: " + sql);
//        	try {
//				stmt.executeUpdate(sql);
//			} catch (SQLException e) {
//				e.printStackTrace();
//				JOptionPane.showMessageDialog(null, "Fail to create the table: " + className + ". \n" 
//						+ "SQL sentence is: " + sql + ". \n"
//						+ e.getMessage());			
//			}
//        }
	}
	
	/**
	 * 
	 */
	public void generateInformationSystem(){
		System.out.println("generate logs");
		IOFileDialogue fileImporter = new IOFileDialogue();
		fileImporter.show("Information system interface (.php)", "php", JFileChooser.FILES_ONLY, "Output", lastFile);
		File file = fileImporter.getSelectedFile();
		if(file != null){			
			lastFile = file.getParentFile();
			String fileName = file.getAbsolutePath();				        
			//save xes log
			if(fileName != null){
//				XesXmlSerializer serializer = new XesXmlSerializer();
//				OutputStream outputLog = null;
//				try {
//					outputLog = new FileOutputStream("\\" + fileName + ".xes");
//				} catch (FileNotFoundException e1) {
//					// TODO Auto-generated catch block
//					e1.printStackTrace();
//				}
//
//	    		
//				try {
//					outputLog.close();
//				} catch (IOException e) {
//					// TODO Auto-generated catch block
//					e.printStackTrace();
//				}
			}
		}	
	}
	
	public String generateTableCreationSQL(String databaseName, String tableName, List<String> columnList, List<String> typeList, List<String> pkColumnList){
		String sql = "CREATE TABLE " + databaseName + "." + tableName + " (";
		boolean firstColumn = true;
    	for(int i=0; i<columnList.size(); i++){
    		if(!firstColumn){
				sql = sql + ", ";		        				
			}
			firstColumn = false;
			sql = sql + columnList.get(i) + " " + typeList.get(i);
    	}	
    	//add fk constraints
    	for(int i=0; i<pkColumnList.size(); i++){
			sql = sql + ", " + "PRIMARY KEY (" + pkColumnList.get(i) + ")";
    	}
    	sql = sql + ")";
    	System.out.println("sql: " + sql);
		return sql;
	}
	
	public static void main(String args[]){  
		try{  
		Class.forName("com.mysql.jdbc.Driver").newInstance();
//		Class.forName("com.mysql.cj.jdbc.Driver").newInstance();
		
		String url = "jdbc:mysql://localhost:3306?"
	            + "user=root&password=kfgz0874981211&useUnicode=true&characterEncoding=UTF8";
		Connection con=DriverManager.getConnection(  
				url);  
		//here sonoo is database name, root is username and password  
		Statement stmt=con.createStatement();  
		ResultSet rs=stmt.executeQuery("select * from dolibarrptp.llx_user");  
		while(rs.next())  
		System.out.println(rs.getInt(1)+"  "+rs.getString(2)+"  "+rs.getString(3));  
		con.close();  
		}catch(Exception e){ System.out.println(e);}  
		
	}
//	    MysqlDataSource dataSource = new MysqlDataSource();
//
//	    // Set dataSource Properties
//	    dataSource.setServerName("localhost");
//	    dataSource.setPortNumber(3306);
//	    dataSource.setDatabaseName("dolibarrptp");
//	    dataSource.setUser("root");
//	    dataSource.setPassword("kfgz0874981211");
//	    
//	    Connection conn = null;
//	    Statement stmt = null;
//	    ResultSet rs = null;
//	    try {
//
//	      // Get connection from DataSource
//	      conn = dataSource.getConnection();
//	      stmt = conn.createStatement();
//	      rs = stmt.executeQuery("SELECT version()");
//	      
//	      if (rs.next()) {
//	        System.out.println("Database Version : " + rs.getString(1));
//	      }
//	    } catch (SQLException e) {
//	      e.printStackTrace();
//	    } finally {
//	      try {
//	        if (rs != null) {
//	          rs.close();
//	        }
//	        if (stmt != null) {
//	          stmt.close();
//	        }
//	        if (conn != null) {
//	          conn.close();
//	        }
//	      } catch (Exception e) {
//	        e.printStackTrace();
//	      }
//	    }
//	  }
	 
}
