/**
 * Copyright (c) 2007-2012, JGraph Ltd
 */
package org.processmining.mxgraph.util;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Contains all global constants.
 */
public class mxConstants
{
	/**
	 * Defines the number of radians per degree.
	 */
	public static double RAD_PER_DEG = 0.0174532;

	/**
	 * Defines the number of degrees per radian.
	 */
	public static double DEG_PER_RAD = 57.2957795;

	/**
	 * Defines the minimum scale at which rounded polylines should be painted.
	 * Default is 0.05.
	 */
	public static double MIN_SCALE_FOR_ROUNDED_LINES = 0.05;

	/**
	 * Defines the portion of the cell which is to be used as a connectable
	 * region. Default is 0.3.
	 */
	public static double DEFAULT_HOTSPOT = 0.3;

	/**
	 * Defines the minimum size in pixels of the portion of the cell which is
	 * to be used as a connectable region. Default is 8.
	 */
	public static int MIN_HOTSPOT_SIZE = 8;

	/**
	 * Defines the maximum size in pixels of the portion of the cell which is
	 * to be used as a connectable region. Use 0 for no maximum. Default is 0.
	 */
	public static int MAX_HOTSPOT_SIZE = 0;

	/**
	 * Defines the SVG namespace.
	 */
	public static String NS_SVG = "http://www.w3.org/2000/svg";

	/**
	 * Defines the XHTML namespace.
	 */
	public static String NS_XHTML = "http://www.w3.org/1999/xhtml";

	/**
	 * Defines the XLink namespace.
	 */
	public static String NS_XLINK = "http://www.w3.org/1999/xlink";

	/**
	 * Comma separated list of default fonts for CSS properties.
	 * And the default font family value for new image export.
	 * Default is Arial, Helvetica.
	 */
	public static String DEFAULT_FONTFAMILIES = "Arial,Helvetica";
	
	/**
	 * Defines the font for cardinality including symbol "□"
	 * added by Guangming Li on June 16,2017
	 * The possible font candidates are "KaiTi", "FangSong", "NSimSum", "YouYuan" and "Gulim"
	 */
	public static String CARDINALITY_FONTFAMILY = "KaiTi";

	/**
	 * Defines the default font family. Default is "Dialog". (To be replaced
	 * with Font.DIALOG after EOL of Java 1.5.)
	 */
	public static String DEFAULT_FONTFAMILY = "Dialog";

	/**
	 * Defines the default font size. Default is 11.
	 */
	public static int DEFAULT_FONTSIZE = 11;

	/**
	 * Defines the default start size for swimlanes. Default is 40.
	 */
	public static int DEFAULT_STARTSIZE = 40;

	/**
	 * Default line height for text output. Default is 1.2. This is ignored for HTML in
	 * the current version of Java. See
	 * http://docs.oracle.com/javase/6/docs/api/index.html?javax/swing/text/html/CSS.html
	 */
	public static float LINE_HEIGHT = 1.2f;

	/**
	 * Specifies if absolute line heights should be used (px) in CSS. Default
	 * is false. Set this to true for backwards compatibility.
	 */
	public static boolean ABSOLUTE_LINE_HEIGHT = false;

	/**
	 * Specifies the line spacing. Default is 0.
	 */
	public static int LINESPACING = 0;

	/**
	 * Whether or not to split whole words when applying word wrapping in mxUtils.wordWrap.
	 */
	public static boolean SPLIT_WORDS = true;

	/**
	 * Defines the inset in absolute pixels between the label bounding box and
	 * the label text. Default is 3.
	 */
	public static int LABEL_INSET = 3;

	/**
	 * Multiplier to the width that is passed into the word wrapping calculation
	 * See mxUtils.wordWrap for details
	 */
	public static double LABEL_SCALE_BUFFER = 0.9;

	/**
	 * Defines the default marker size. Default is 6.
	 */
	public static int DEFAULT_MARKERSIZE = 6;

	/**
	 * Defines the default image size. Default is 24.
	 */
	public static int DEFAULT_IMAGESIZE = 24;

	/**
	 * Defines the default opacity for stencils shadows. Default is 1.
	 */
	public static int STENCIL_SHADOW_OPACITY = 1;

	/**
	 * Defines the default shadow color for stencils. Default is "gray".
	 */
	public static String STENCIL_SHADOWCOLOR = "gray";

	/**
	 * Defines the x-offset to be used for shadows. Default is 2.
	 */
	public static int SHADOW_OFFSETX = 2;

	/**
	 * Defines the y-offset to be used for shadows. Default is 3.
	 */
	public static int SHADOW_OFFSETY = 3;

	/**
	 * Defines the color to be used to draw shadows in W3C standards. Default
	 * is gray.
	 */
	public static String W3C_SHADOWCOLOR = "gray";

	/**
	 * Defines the transformation used to draw shadows in SVG.
	 */
	public static String SVG_SHADOWTRANSFORM = "translate(2 3)";

	/**
	 * Specifies the default dash pattern, 3 pixels solid, 3 pixels clear.
	 */
	public static float[] DEFAULT_DASHED_PATTERN = new float[] {3.0f , 3.0f};//problematic
	
	/**
	 * Specifies the crel edge dash pattern, 5 pixels solid, 5 pixels clear.
	 */
	public static String CRELRELATION_DASHED_PATTERN = "5.0,5.0";
	
	/**
	 * Specifies the AOC edge dash pattern, 5 pixels solid, 5 pixels clear.
	 */
	public static String AOCRELATION_DASHED_PATTERN = "1.0,1.0";
	

	/**
	 * Specifies the default distance at 1.0 scale that the label curve is 
	 * created from its base curve
	 */
	public static double DEFAULT_LABEL_BUFFER = 12.0;

	/**
	 * Defines the handle size. Default is 7.
	 */
	public static int HANDLE_SIZE = 7;

	/**
	 * Defines the handle size. Default is 4.
	 */
	public static int LABEL_HANDLE_SIZE = 4;

	/**
	 * Defines the default value for the connect handle. Default is false.
	 */
	public static boolean CONNECT_HANDLE_ENABLED = false;

	/**
	 * Defines the connect handle size. Default is 8.
	 */
	public static int CONNECT_HANDLE_SIZE = 8;

	/**
	 * Defines the length of the horizontal segment of an Entity Relation.
	 * This can be overridden using mxConstants.STYLE_SEGMENT style.
	 * Default is 30.
	 */
	public static int ENTITY_SEGMENT = 30;

	/**
	 * Defines the rounding factor for rounded rectangles in percent between
	 * 0 and 1. Values should be smaller than 0.5. Default is 0.15.
	 */
	public static double RECTANGLE_ROUNDING_FACTOR = 0.15;

	/**
	 * Defines the size of the arcs for rounded edges. Default is 10.
	 */
	public static double LINE_ARCSIZE = 10;

	/**
	 * Defines the spacing between the arrow shape and its terminals. Default
	 * is 10.
	 */
	public static int ARROW_SPACING = 10;

	/**
	 * Defines the width of the arrow shape. Default is 30.
	 */
	public static int ARROW_WIDTH = 30;

	/**
	 * Defines the size of the arrowhead in the arrow shape. Default is 30.
	 */
	public static int ARROW_SIZE = 30;

	/**
	 * Defines the value for none. Default is "none".
	 */
	public static String NONE = "none";

	/**
	 * Defines the key for the perimeter style.
	 * This is a function that defines the perimeter around a particular shape.
	 * Possible values are the functions defined in mxPerimeter that use the 
	 * <code>mxPerimeterFunction</code> interface. Alternatively, the constants
	 * in this class that start with <code>PERIMETER_</code> may be used to 
	 * access perimeter styles in <code>mxStyleRegistry</code>.
	 */
	public static String STYLE_PERIMETER = "perimeter";

	/**
	 * Defines the ID of the cell that should be used for computing the
	 * perimeter point of the source for an edge. This allows for graphically
	 * connecting to a cell while keeping the actual terminal of the edge.
	 */
	public static String STYLE_SOURCE_PORT = "sourcePort";

	/**
	 * Defines the ID of the cell that should be used for computing the
	 * perimeter point of the target for an edge. This allows for graphically
	 * connecting to a cell while keeping the actual terminal of the edge.
	 */
	public static String STYLE_TARGET_PORT = "targetPort";

	/**
	 * Defines the direction(s) that edges are allowed to connect to cells in.
	 * Possible values are <code>DIRECTION_NORTH, DIRECTION_SOUTH, 
	 * DIRECTION_EAST</code> and <code>DIRECTION_WEST</code>.
	 * 
	 */
	public static String STYLE_PORT_CONSTRAINT = "portConstraint";

	/**
	 * Defines the key for the opacity style. The type of the value is 
	 * <code>float</code> and the possible range is 0-100.
	 */
	public static String STYLE_OPACITY = "opacity";

	/**
	 * Defines the key for the fill opacity style. The type of the value is 
	 * <code>float</code> and the possible range is 0-100.
	 */
	public static String STYLE_FILL_OPACITY = "fillOpacity";

	/**
	 * Defines the key for the stroke opacity style. The type of the value is 
	 * <code>float</code> and the possible range is 0-100.
	 */
	public static String STYLE_STROKE_OPACITY = "strokeOpacity";

	/**
	 * Defines the key for the text opacity style. The type of the value is 
	 * <code>float</code> and the possible range is 0-100.
	 */
	public static String STYLE_TEXT_OPACITY = "textOpacity";

	/**
	 * Defines the key for the overflow style. Possible values are "visible",
	 * "hidden" and "fill". The default value is "visible". This value
	 * specifies how overlapping vertex labels are handles. A value of
	 * "visible" will show the complete label. A value of "hidden" will clip
	 * the label so that it does not overlap the vertex bounds. A value of
	 * "fill" will use the vertex bounds for the label.
	 * 
	 * @see org.processmining.mxgraph.view.mxGraph#isLabelClipped(Object)
	 */
	public static String STYLE_OVERFLOW = "overflow";

	/**
	* Defines if the connection points on either end of the edge should be
	* computed so that the edge is vertical or horizontal if possible and
	* if the point is not at a fixed location. Default is false. This is
	* used in mxGraph.isOrthogonal, which also returns true if the edgeStyle
	* of the edge is an elbow or entity.
	*/
	public static String STYLE_ORTHOGONAL = "orthogonal";

	/**
	* Defines the key for the horizontal relative coordinate connection point
	* of an edge with its source terminal.
	*/
	public static String STYLE_EXIT_X = "exitX";

	/**
	* Defines the key for the vertical relative coordinate connection point
	* of an edge with its source terminal.
	*/
	public static String STYLE_EXIT_Y = "exitY";

	/**
	* Defines if the perimeter should be used to find the exact entry point
	* along the perimeter of the source. Possible values are 0 (false) and
	* 1 (true). Default is 1 (true).
	*/
	public static String STYLE_EXIT_PERIMETER = "exitPerimeter";

	/**
	* Defines the key for the horizontal relative coordinate connection point
	* of an edge with its target terminal.
	*/
	public static String STYLE_ENTRY_X = "entryX";

	/**
	* Defines the key for the vertical relative coordinate connection point
	* of an edge with its target terminal.
	*/
	public static String STYLE_ENTRY_Y = "entryY";

	/**
	* Defines if the perimeter should be used to find the exact entry point
	* along the perimeter of the target. Possible values are 0 (false) and
	* 1 (true). Default is 1 (true).
	*/
	public static String STYLE_ENTRY_PERIMETER = "entryPerimeter";

	/**
	 * Defines the key for the white-space style. Possible values are "nowrap"
	 * and "wrap". The default value is "nowrap". This value specifies how
	 * white-space inside a HTML vertex label should be handled. A value of
	 * "nowrap" means the text will never wrap to the next line until a
	 * linefeed is encountered. A value of "wrap" means text will wrap when
	 * necessary.
	 */
	public static String STYLE_WHITE_SPACE = "whiteSpace";

	/**
	 * Defines the key for the rotation style. The type of the value is 
	 * <code>double</code> and the possible range is 0-360.
	 */
	public static String STYLE_ROTATION = "rotation";

	/**
	 * Defines the key for the fill color of the swimlane background.
	 * The value is a string expression supported by mxUtils.parseColor.
	 * 
	 * @see org.processmining.mxgraph.util.mxUtils#parseColor(String)
	 */
	public static String STYLE_SWIMLANE_FILLCOLOR = "swimlaneFillColor";

	/**
	 * Defines the key for the fillColor style. The value is a string
	 * expression supported by mxUtils.parseColor.
	 * 
	 * @see org.processmining.mxgraph.util.mxUtils#parseColor(String)
	 */
	public static String STYLE_FILLCOLOR = "fillColor";

	/**
	 * Defines the key for the gradientColor style. The value is a string
	 * expression supported by mxUtils.parseColor. This is ignored if no fill
	 * color is defined.
	 * 
	 * @see org.processmining.mxgraph.util.mxUtils#parseColor(String)
	 */
	public static String STYLE_GRADIENTCOLOR = "gradientColor";

	/**
	 * Defines the key for the gradient direction. Possible values are
	 * <code>DIRECTION_EAST</code>, <code>DIRECTION_WEST</code>,
	 * <code>DIRECTION_NORTH</code> and <code>DIRECTION_SOUTH</code>. Default
	 * is <code>DIRECTION_SOUTH</code>. Generally, and by default in mxGraph,
	 * gradient painting is done from the value of <code>STYLE_FILLCOLOR</code>
	 * to the value of <code>STYLE_GRADIENTCOLOR</code>. Taking the example of
	 * <code>DIRECTION_NORTH</code>, this means <code>STYLE_FILLCOLOR</code>
	 * color at the bottom of paint pattern and
	 * <code>STYLE_GRADIENTCOLOR</code> at top, with a gradient in-between.
	 */
	public static String STYLE_GRADIENT_DIRECTION = "gradientDirection";

	/**
	 * Defines the key for the strokeColor style. The value is a string
	 * expression supported by mxUtils.parseColor.
	 * 
	 * @see org.processmining.mxgraph.util.mxUtils#parseColor(String)
	 */
	public static String STYLE_STROKECOLOR = "strokeColor";

	/**
	 * Defines the key for the separatorColor style. The value is a string
	 * expression supported by mxUtils.parseColor. This style is only used
	 * for SHAPE_SWIMLANE shapes.
	 * 
	 * @see org.processmining.mxgraph.util.mxUtils#parseColor(String)
	 */
	public static String STYLE_SEPARATORCOLOR = "separatorColor";

	/**
	 * Defines the key for the strokeWidth style. The type of the value is
	 * <code>float</code> and the possible range is any non-negative value.
	 * The value reflects the stroke width in pixels.
	 */
	public static String STYLE_STROKEWIDTH = "strokeWidth";

	/**
	 * Defines the key for the align style. Possible values are
	 * <code>ALIGN_LEFT</code>, <code>ALIGN_CENTER</code> and
	 * <code>ALIGN_RIGHT</code>. This value defines how the lines of the label
	 * are horizontally aligned. <code>ALIGN_LEFT</code> mean label text lines
	 * are aligned to left of the label bounds, <code>ALIGN_RIGHT</code> to the
	 * right of the label bounds and <code>ALIGN_CENTER</code> means the
	 * center of the text lines are aligned in the center of the label bounds.
	 * Note this value doesn't affect the positioning of the overall label
	 * bounds relative to the vertex, to move the label bounds horizontally, use
	 * <code>STYLE_LABEL_POSITION</code>.
	 */
	public static String STYLE_ALIGN = "align";

	/**
	 * Defines the key for the verticalAlign style. Possible values are
	 * <code>ALIGN_TOP</code>, <code>ALIGN_MIDDLE</code> and
	 * <code>ALIGN_BOTTOM</code>. This value defines how the lines of the label
	 * are vertically aligned. <code>ALIGN_TOP</code> means the topmost label
	 * text line is aligned against the top of the label bounds,
	 * <code>ALIGN_BOTTOM</code> means the bottom-most label text line is
	 * aligned against the bottom of the label bounds and
	 * <code>ALIGN_MIDDLE</code> means there is equal spacing between the
	 * topmost text label line and the top of the label bounds and the
	 * bottom-most text label line and the bottom of the label bounds. Note
	 * this value doesn't affect the positioning of the overall label bounds
	 * relative to the vertex, to move the label bounds vertically, use
	 * <code>STYLE_VERTICAL_LABEL_POSITION</code>.
	 */
	public static String STYLE_VERTICAL_ALIGN = "verticalAlign";

	/**
	 * Defines the key for the horizontal label position of vertices. Possible
	 * values are <code>ALIGN_LEFT</code>, <code>ALIGN_CENTER</code> and
	 * <code>ALIGN_RIGHT</code>. Default is <code>ALIGN_CENTER</code>. The
	 * label align defines the position of the label relative to the cell.
	 * <code>ALIGN_LEFT</code> means the entire label bounds is placed
	 * completely just to the left of the vertex, <code>ALIGN_RIGHT</code>
	 * means adjust to the right and <code>ALIGN_CENTER</code> means the label
	 * bounds are vertically aligned with the bounds of the vertex. Note this
	 * value doesn't affect the positioning of label within the label bounds,
	 * to move the label horizontally within the label bounds, use
	 * <code>STYLE_ALIGN</code>.
	 */
	public static String STYLE_LABEL_POSITION = "labelPosition";

	/**
	 * Defines the key for the vertical label position of vertices. Possible
	 * values are <code>ALIGN_TOP</code>, <code>ALIGN_BOTTOM</code> and
	 * <code>ALIGN_MIDDLE</code>. Default is <code>ALIGN_MIDDLE</code>. The
	 * label align defines the position of the label relative to the cell.
	 * <code>ALIGN_TOP</code> means the entire label bounds is placed
	 * completely just on the top of the vertex, <code>ALIGN_BOTTOM</code>
	 * means adjust on the bottom and <code>ALIGN_MIDDLE</code> means the label
	 * bounds are horizontally aligned with the bounds of the vertex. Note
	 * this value doesn't affect the positioning of label within the label
	 * bounds, to move the label vertically within the label bounds, use
	 * <code>STYLE_VERTICAL_ALIGN</code>.
	 */
	public static String STYLE_VERTICAL_LABEL_POSITION = "verticalLabelPosition";

	/**
	 * Defines the key for the align style. Possible values are
	 * <code>ALIGN_LEFT</code>, <code>ALIGN_CENTER</code> and
	 * <code>ALIGN_RIGHT</code>. The value defines how any image in the vertex
	 * label is aligned horizontally within the label bounds of a SHAPE_LABEL
	 * shape.
	 */
	public static String STYLE_IMAGE_ALIGN = "imageAlign";

	/**
	 * Defines the key for the verticalAlign style. Possible values are
	 * <code>ALIGN_TOP</code>, <code>ALIGN_MIDDLE</code> and
	 * <code>ALIGN_BOTTOM</code>. The value defines how any image in the vertex
	 * label is aligned vertically within the label bounds of a SHAPE_LABEL
	 * shape.
	 */
	public static String STYLE_IMAGE_VERTICAL_ALIGN = "imageVerticalAlign";

	/**
	 * Defines the key for the glass style. Possible values are 0 (disabled) and
	 * 1(enabled). The default value is 0. This is used in mxLabel.
	 */
	public static String STYLE_GLASS = "glass";

	/**
	 * Defines the key for the image style. Possible values are any image URL,
	 * registered key in mxImageResources or short data URI as defined in
	 * mxImageBundle.
	 * The type of the value is <code>String</code>. This is the path to the
	 * image to image that is to be displayed within the label of a vertex. See
	 * mxGraphics2DCanvas.getImageForStyle, loadImage and setImageBasePath on
	 * how the image URL is resolved. Finally, mxUtils.loadImage is used for
	 * loading the image for a given value.
	 */
	public static String STYLE_IMAGE = "image";

	/**
	 * Defines the key for the imageWidth style. The type of this value is
	 * <code>int</code>, the value is the image width in pixels and must be
	 * greated than 0.
	 */
	public static String STYLE_IMAGE_WIDTH = "imageWidth";

	/**
	 * Defines the key for the imageHeight style The type of this value is
	 * <code>int</code>, the value is the image height in pixels and must be
	 * greater than 0.
	 */
	public static String STYLE_IMAGE_HEIGHT = "imageHeight";

	/**
	 * Defines the key for the image background color. This style is only used
	 * for image shapes. Possible values are all HTML color names or HEX codes.
	 */
	public static String STYLE_IMAGE_BACKGROUND = "imageBackground";

	/**
	 * Defines the key for the image border color. This style is only used for
	 * image shapes. Possible values are all HTML color names or HEX codes.
	 */
	public static String STYLE_IMAGE_BORDER = "imageBorder";

	/**
	 * Defines the key for the horizontal image flip. This style is only used
	 * in mxImageShape. Possible values are 0 and 1. Default is 0.
	 */
	public static String STYLE_IMAGE_FLIPH = "imageFlipH";

	/**
	 * Defines the key for the vertical image flip. This style is only used
	 * in mxImageShape. Possible values are 0 and 1. Default is 0.
	 */
	public static String STYLE_IMAGE_FLIPV = "imageFlipV";

	/**
	 * Defines the key for the horizontal stencil flip. This style is only used
	 * for <mxStencilShape>. Possible values are 0 and 1. Default is 0.
	 */
	public static String STYLE_STENCIL_FLIPH = "stencilFlipH";

	/**
	 * Defines the key for the vertical stencil flip. This style is only used
	 * for <mxStencilShape>. Possible values are 0 and 1. Default is 0.
	 */
	public static String STYLE_STENCIL_FLIPV = "stencilFlipV";


	/**
	 * Defines the key for the horizontal image flip. This style is only used
	 * in <mxImageShape>. Possible values are 0 and 1. Default is 0.
	 */
	public static String STYLE_FLIPH = "flipH";

	/**
	 * Variable: STYLE_FLIPV
	 * 
	 * Defines the key for the vertical flip. Possible values are 0 and 1.
	 * Default is 0.
	 */
	public static String STYLE_FLIPV = "flipV";

	/**
	 * Defines the key for the noLabel style. If this is
	 * true then no label is visible for a given cell.
	 * Possible values are true or false (1 or 0).
	 * Default is false.
	 */
	public static String STYLE_NOLABEL = "noLabel";

	/**
	 * Defines the key for the noEdgeStyle style. If this is
	 * true then no edge style is applied for a given edge.
	 * Possible values are true or false (1 or 0).
	 * Default is false.
	 */
	public static String STYLE_NOEDGESTYLE = "noEdgeStyle";

	/**
	 * Defines the key for the label background color. The value is a string
	 * expression supported by mxUtils.parseColor.
	 * 
	 * @see org.processmining.mxgraph.util.mxUtils#parseColor(String)
	 */
	public static String STYLE_LABEL_BACKGROUNDCOLOR = "labelBackgroundColor";

	/**
	 * Defines the key for the label border color. The value is a string
	 * expression supported by mxUtils.parseColor.
	 * 
	 * @see org.processmining.mxgraph.util.mxUtils#parseColor(String)
	 */
	public static String STYLE_LABEL_BORDERCOLOR = "labelBorderColor";

	/**
	 * Defines the key for the indicatorShape style.
	 * Possible values are any of the SHAPE_*
	 * constants.
	 */
	public static String STYLE_INDICATOR_SHAPE = "indicatorShape";

	/**
	 * Defines the key for the indicatorImage style.
	 * Possible values are any image URL, the type of the value is
	 * <code>String</code>.
	 */
	public static String STYLE_INDICATOR_IMAGE = "indicatorImage";

	/**
	 * Defines the key for the indicatorColor style. The value is a string
	 * expression supported by mxUtils.parseColor.
	 * 
	 * @see org.processmining.mxgraph.util.mxUtils#parseColor(String)
	 */
	public static String STYLE_INDICATOR_COLOR = "indicatorColor";

	/**
	 * Defines the key for the indicatorGradientColor style. The value is a
	 * string expression supported by mxUtils.parseColor. This style is only
	 * supported in SHAPE_LABEL shapes.
	 * 
	 * @see org.processmining.mxgraph.util.mxUtils#parseColor(String)
	 */
	public static String STYLE_INDICATOR_GRADIENTCOLOR = "indicatorGradientColor";

	/**
	 * Defines the key for the indicatorSpacing style (in px).
	 */
	public static String STYLE_INDICATOR_SPACING = "indicatorSpacing";

	/**
	 * Defines the key for the indicatorWidth style (in px).
	 */
	public static String STYLE_INDICATOR_WIDTH = "indicatorWidth";

	/**
	 * Defines the key for the indicatorHeight style (in px).
	 */
	public static String STYLE_INDICATOR_HEIGHT = "indicatorHeight";

	/**
	 * Defines the key for the shadow style. The type of the value is
	 * <code>boolean</code>. This style applies to vertices and arrow style
	 * edges.
	 */
	public static String STYLE_SHADOW = "shadow";

	/**
	 * Defines the key for the segment style. The type of this value is
	 * <code>float</code> and the value represents the size of the horizontal
	 * segment of the entity relation style. Default is ENTITY_SEGMENT.
	 */
	public static String STYLE_SEGMENT = "segment";

	/**
	 * Defines the key for the endArrow style.
	 * Possible values are all constants in this
	 * class that start with ARROW_. This style is
	 * supported in the <code>mxConnector</code> shape.
	 */
	public static String STYLE_ENDARROW = "endArrow";

	/**
	 * Defines the key for the startArrow style.
	 * Possible values are all constants in this
	 * class that start with ARROW_.
	 * See STYLE_ENDARROW.
	 * This style is supported in the mxConnector shape.
	 */
	public static String STYLE_STARTARROW = "startArrow";

	/**
	 * Defines the key for the endSize style. The type of this value is 
	 * <code>float</code> and the value represents the size of the end 
	 * marker in pixels.
	 */
	public static String STYLE_ENDSIZE = "endSize";

	/**
	 * Defines the key for the startSize style. The type of this value is
	 * <code>float</code> and the value represents the size of the start marker
	 * or the size of the swimlane title region depending on the shape it is
	 * used for.
	 */
	public static String STYLE_STARTSIZE = "startSize";

	/**
	 * Defines the key for the swimlaneLine style. This style specifies whether
	 * the line between the title regio of a swimlane should be visible. Use 0
	 * for hidden or 1 (default) for visible. Value is "swimlaneLine".
	 */
	public static String STYLE_SWIMLANE_LINE = "swimlaneLine";
	
	/**
	 * Defines the key for the endFill style. Use 0 for no fill or 1
	 * (default) for fill. (This style is only exported via <mxImageExport>.)
	 */
	public static String STYLE_ENDFILL = "endFill";

	/**
	 * Defines the key for the startFill style. Use 0 for no fill or 1
	 * (default) for fill. (This style is only exported via <mxImageExport>.)
	 */
	public static String STYLE_STARTFILL = "startFill";

	/**
	 * Defines the key for the dashed style. The type of this value is
	 * <code>boolean</code> and the value determines whether or not an edge or
	 * border is drawn with a dashed pattern along the line.
	 */
	public static String STYLE_DASHED = "dashed";

	/**
	 * Defines the key for the dashed pattern style. The type of this value
	 * is <code>float[]</code> and the value specifies the dashed pattern 
	 * to apply to edges drawn with this style. This style allows the user
	 * to specify a custom-defined dash pattern. This is done using a series
	 * of numbers. Dash styles are defined in terms of the length of the dash
	 * (the drawn part of the stroke) and the length of the space between the
	 * dashes. The lengths are relative to the line width: a length of "1" is
	 * equal to the line width.
	 */
	public static String STYLE_DASH_PATTERN = "dashPattern";

	/**
	 * Defines the key for the rounded style. The type of this value is
	 * <code>boolean</code>. For edges this determines whether or not joins
	 * between edges segments are smoothed to a rounded finish. For vertices
	 * that have the rectangle shape, this determines whether or not the
	 * rectangle is rounded.
	 */
	public static String STYLE_ROUNDED = "rounded";

	/**
	 * Defines the rounding factor for a rounded rectangle in percent (without
	 * the percent sign). Possible values are between 0 and 100. If this value
	 * is not specified then RECTANGLE_ROUNDING_FACTOR * 100 is used. For
	 * edges, this defines the absolute size of rounded corners in pixels. If
	 * this values is not specified then LINE_ARCSIZE is used.
	 * (This style is only exported via <mxImageExport>.) Value is "arcSize".
	 */
	public static String STYLE_ARCSIZE = "arcSize";

	/**
	 * Defines the key for the source perimeter spacing. The type of this value
	 * is <code>double</code>. This is the distance between the source
	 * connection point of an edge and the perimeter of the source vertex in
	 * pixels. This style only applies to edges.
	 */
	public static String STYLE_SOURCE_PERIMETER_SPACING = "sourcePerimeterSpacing";

	/**
	 * Defines the key for the target perimeter spacing. The type of this value
	 * is <code>double</code>. This is the distance between the target
	 * connection point of an edge and the perimeter of the target vertex in
	 * pixels. This style only applies to edges.
	 */
	public static String STYLE_TARGET_PERIMETER_SPACING = "targetPerimeterSpacing";

	/**
	 * Defines the key for the perimeter spacing. This is the distance between
	 * the connection point and the perimeter in pixels. When used in a vertex
	 * style, this applies to all incoming edges to floating ports (edges that
	 * terminate on the perimeter of the vertex). When used in an edge style,
	 * this spacing applies to the source and target separately, if they
	 * terminate in floating ports (on the perimeter of the vertex).
	 */
	public static String STYLE_PERIMETER_SPACING = "perimeterSpacing";

	/**
	 * Defines the key for the spacing. The value represents the spacing, in
	 * pixels, added to each side of a label in a vertex (style applies to
	 * vertices only).
	 */
	public static String STYLE_SPACING = "spacing";

	/**
	 * Defines the key for the spacingTop style. The value represents the
	 * spacing, in pixels, added to the top side of a label in a vertex (style
	 * applies to vertices only).
	 */
	public static String STYLE_SPACING_TOP = "spacingTop";

	/**
	 * Defines the key for the spacingLeft style. The value represents the
	 * spacing, in pixels, added to the left side of a label in a vertex (style
	 * applies to vertices only).
	 */
	public static String STYLE_SPACING_LEFT = "spacingLeft";

	/**
	 * Defines the key for the spacingBottom style The value represents the
	 * spacing, in pixels, added to the bottom side of a label in a vertex
	 * (style applies to vertices only).
	 */
	public static String STYLE_SPACING_BOTTOM = "spacingBottom";

	/**
	 * Defines the key for the spacingRight style The value represents the
	 * spacing, in pixels, added to the right side of a label in a vertex (style
	 * applies to vertices only).
	 */
	public static String STYLE_SPACING_RIGHT = "spacingRight";

	/**
	 * Defines the key for the horizontal style. Possible values are
	 * <code>true</code> or <code>false</code>. This value only applies to
	 * vertices. If the <code>STYLE_SHAPE</code> is <code>SHAPE_SWIMLANE</code>
	 * a value of <code>false</code> indicates that the swimlane should be drawn
	 * vertically, <code>true</code> indicates to draw it horizontally. If the
	 * shape style does not indicate that this vertex is a swimlane, this value
	 * affects only whether the label is drawn horizontally or vertically.
	 */
	public static String STYLE_HORIZONTAL = "horizontal";

	/**
	 * Defines the key for the direction style. The direction style is used to
	 * specify the direction of certain shapes (eg. <code>mxTriangle</code>).
	 * Possible values are <code>DIRECTION_EAST</code> (default),
	 * <code>DIRECTION_WEST</code>, <code>DIRECTION_NORTH</code> and
	 * <code>DIRECTION_SOUTH</code>. This value only applies to vertices.
	 */
	public static String STYLE_DIRECTION = "direction";

	/**
	 * Defines the key for the elbow style. Possible values are
	 * <code>ELBOW_HORIZONTAL</code> and <code>ELBOW_VERTICAL</code>. Default is
	 * <code>ELBOW_HORIZONTAL</code>. This defines how the three segment
	 * orthogonal edge style leaves its terminal vertices. The vertical style
	 * leaves the terminal vertices at the top and bottom sides.
	 */
	public static String STYLE_ELBOW = "elbow";

	/**
	 * Defines the key for the fontColor style. The value is type
	 * <code>String</code> and of the expression supported by
	 * mxUtils.parseColor.
	 * 
	 * @see org.processmining.mxgraph.util.mxUtils#parseColor(String)
	 */
	public static String STYLE_FONTCOLOR = "fontColor";

	/**
	 * Defines the key for the fontFamily style. Possible values are names such
	 * as Arial; Dialog; Verdana; Times New Roman. The value is of type
	 * <code>String</code>.
	 */
	public static String STYLE_FONTFAMILY = "fontFamily";

	/**
	 * Defines the key for the fontSize style (in points). The type of the value
	 * is <code>int</code>.
	 */
	public static String STYLE_FONTSIZE = "fontSize";

	/**
	 * Defines the key for the fontStyle style. Values may be any logical AND
	 * (sum) of FONT_BOLD, FONT_ITALIC and FONT_UNDERLINE. The type
	 * of the value is <code>int</code>.
	 */
	public static String STYLE_FONTSTYLE = "fontStyle";

	/**
	 * Defines the key for the autosize style. This specifies if a cell should be
	 * resized automatically if the value has changed. Possible values are 0 or 1.
	 * Default is 0. See mxGraph.isAutoSizeCell. This is normally combined with
	 * STYLE_RESIZABLE to disable manual sizing.
	 */
	public static String STYLE_AUTOSIZE = "autosize";

	/**
	 * Defines the key for the foldable style. This specifies if a cell is foldable
	 * using a folding icon. Possible values are 0 or 1. Default is 1. See
	 * mxGraph.isCellFoldable.
	 */
	public static String STYLE_FOLDABLE = "foldable";

	/**
	 * Defines the key for the editable style. This specifies if the value of
	 * a cell can be edited using the in-place editor. Possible values are 0 or
	 * 1. Default is 1. See mxGraph.isCellEditable.
	 */
	public static String STYLE_EDITABLE = "editable";

	/**
	 * Defines the key for the bendable style. This specifies if the control
	 * points of an edge can be moved. Possible values are 0 or 1. Default is
	 * 1. See mxGraph.isCellBendable.
	 */
	public static String STYLE_BENDABLE = "bendable";

	/**
	 * Defines the key for the movable style. This specifies if a cell can
	 * be moved. Possible values are 0 or 1. Default is 1. See
	 * mxGraph.isCellMovable.
	 */
	public static String STYLE_MOVABLE = "movable";

	/**
	 * Defines the key for the resizable style. This specifies if a cell can
	 * be resized. Possible values are 0 or 1. Default is 1. See
	 * mxGraph.isCellResizable.
	 */
	public static String STYLE_RESIZABLE = "resizable";

	/**
	 * Defines the key for the cloneable style. This specifies if a cell can
	 * be cloned. Possible values are 0 or 1. Default is 1. See
	 * mxGraph.isCellCloneable.
	 */
	public static String STYLE_CLONEABLE = "cloneable";

	/**
	 * Defines the key for the deletable style. This specifies if a cell can be
	 * deleted. Possible values are 0 or 1. Default is 1. See
	 * mxGraph.isCellDeletable.
	 */
	public static String STYLE_DELETABLE = "deletable";

	/**
	 * Defines the key for the shape style.
	 * Possible values are any of the SHAPE_*
	 * constants.
	 */
	public static String STYLE_SHAPE = "shape";

	/**
	 * Takes a function that creates points. Possible values are the
	 * functions defined in mxEdgeStyle.
	 */
	public static String STYLE_EDGE = "edgeStyle";
	
	/**
	 * Defines the key for the loop style. Possible values are the
	 * functions defined in mxEdgeStyle.
	 */
	public static String STYLE_LOOP = "loopStyle";

	/**
	 * Defines the key for the horizontal routing center. Possible values are
	 * between -0.5 and 0.5. This is the relative offset from the center used
	 * for connecting edges. The type of this value is <code>float</code>.
	 */
	public static String STYLE_ROUTING_CENTER_X = "routingCenterX";

	/**
	 * Defines the key for the vertical routing center. Possible values are
	 * between -0.5 and 0.5. This is the relative offset from the center used
	 * for connecting edges. The type of this value is <code>float</code>.
	 */
	public static String STYLE_ROUTING_CENTER_Y = "routingCenterY";

	/**
	 * FONT_BOLD
	 */
	public static final int FONT_BOLD = 1;

	/**
	 * FONT_ITALIC
	 */
	public static final int FONT_ITALIC = 2;

	/**
	 * FONT_UNDERLINE
	 */
	public static final int FONT_UNDERLINE = 4;

	/**
	 * SHAPE_RECTANGLE
	 */
	public static final String SHAPE_RECTANGLE = "rectangle";

	/**
	 * SHAPE_ELLIPSE
	 */
	public static final String SHAPE_ELLIPSE = "ellipse";

	/**
	 * SHAPE_DOUBLE_RECTANGLE
	 */
	public static final String SHAPE_DOUBLE_RECTANGLE = "doubleRectangle";

	/**
	 * SHAPE_DOUBLE_ELLIPSE
	 */
	public static final String SHAPE_DOUBLE_ELLIPSE = "doubleEllipse";

	/**
	 * SHAPE_RHOMBUS
	 */
	public static final String SHAPE_RHOMBUS = "rhombus";

	/**
	 * SHAPE_LINE
	 */
	public static final String SHAPE_LINE = "line";

	/**
	 * SHAPE_IMAGE
	 */
	public static final String SHAPE_IMAGE = "image";

	/**
	 * SHAPE_ARROW
	 */
	public static final String SHAPE_ARROW = "arrow";

	/**
	 * SHAPE_ARROW
	 */
	public static final String SHAPE_CURVE = "curve";

	/**
	 * SHAPE_LABEL
	 */
	public static final String SHAPE_LABEL = "label";

	/**
	 * SHAPE_CYLINDER
	 */
	public static final String SHAPE_CYLINDER = "cylinder";

	/**
	 * SHAPE_SWIMLANE
	 */
	public static final String SHAPE_SWIMLANE = "swimlane";

	/**
	 * SHAPE_CONNECTOR
	 */
	public static final String SHAPE_CONNECTOR = "connector";

	/**
	 * SHAPE_ACTOR
	 */
	public static final String SHAPE_ACTOR = "actor";

	/**
	 * SHAPE_CLOUD
	 */
	public static final String SHAPE_CLOUD = "cloud";

	/**
	 * SHAPE_TRIANGLE
	 */
	public static final String SHAPE_TRIANGLE = "triangle";

	/**
	 * SHAPE_HEXAGON
	 */
	public static final String SHAPE_HEXAGON = "hexagon";

	/**
	 * ARROW_CLASSIC
	 */
	public static final String ARROW_CLASSIC = "classic";

	/**
	 * ARROW_BLOCK
	 */
	public static final String ARROW_BLOCK = "block";
		
	/**
	 * ARROW_BLOCK_DOT-Guangming
	 */
	public static final String ARROW_BLOCK_DOT = "blockdot";
	
	/**
	 * ARROW_DOUBLE_BLOCK-Guangming
	 */
	public static final String ARROW_DOUBLE_BLOCK = "doubleblock";

	/**
	 * ARROW_DOUBLE_BLOCK_DOT-Guangming
	 */
	public static final String ARROW_DOUBLE_BLOCK_DOT = "doubleblockdot";
	
	/**
	 * ARROW_OPEN
	 */
	public static final String ARROW_OPEN = "open";

	/**
	 * ARROW_BLOCK
	 */
	public static final String ARROW_OVAL = "oval";

	/**
	 * ARROW_OPEN
	 */
	public static final String ARROW_DIAMOND = "diamond";

	/**
	 * ALIGN_LEFT
	 */
	public static final String ALIGN_LEFT = "left";

	/**
	 * ALIGN_CENTER
	 */
	public static final String ALIGN_CENTER = "center";

	/**
	 * ALIGN_RIGHT
	 */
	public static final String ALIGN_RIGHT = "right";

	/**
	 * ALIGN_TOP
	 */
	public static final String ALIGN_TOP = "top";

	/**
	 * ALIGN_MIDDLE
	 */
	public static final String ALIGN_MIDDLE = "middle";

	/**
	 * ALIGN_BOTTOM
	 */
	public static final String ALIGN_BOTTOM = "bottom";

	/**
	 * DIRECTION_NORTH
	 */
	public static final String DIRECTION_NORTH = "north";

	/**
	 * DIRECTION_SOUTH
	 */
	public static final String DIRECTION_SOUTH = "south";

	/**
	 * DIRECTION_EAST
	 */
	public static final String DIRECTION_EAST = "east";

	/**
	 * DIRECTION_WEST
	 */
	public static final String DIRECTION_WEST = "west";

	/**
	 * DIRECTION_MASK_NONE
	 */
	public static final int DIRECTION_MASK_NONE = 0x00;

	/**
	 * DIRECTION_MASK_WEST
	 */
	public static final int DIRECTION_MASK_WEST = 0x01;
	
	/**
	 * DIRECTION_MASK_NORTH
	 */
	public static final int DIRECTION_MASK_NORTH = 0x02;

	/**
	 * DIRECTION_MASK_SOUTH
	 */
	public static final int DIRECTION_MASK_SOUTH = 0x04;

	/**
	 * DIRECTION_MASK_EAST
	 */
	public static final int DIRECTION_MASK_EAST = 0x08;
	
	/**
	 * DIRECTION_MASK_EAST
	 */
	public static final int DIRECTION_MASK_ALL = 0x0F;

	/**
	 * ELBOW_VERTICAL
	 */
	public static final String ELBOW_VERTICAL = "vertical";

	/**
	 * ELBOW_HORIZONTAL
	 */
	public static final String ELBOW_HORIZONTAL = "horizontal";

	/**
	 * Name of the elbow edge style. Can be used as a string value
	 * for the STYLE_EDGE style.
	 */
	public static final String EDGESTYLE_ELBOW = "elbowEdgeStyle";

	/**
	 * Name of the entity relation edge style. Can be used as a string value
	 * for the STYLE_EDGE style.
	 */
	public static final String EDGESTYLE_ENTITY_RELATION = "entityRelationEdgeStyle";

	/**
	 * Name of the loop edge style. Can be used as a string value
	 * for the STYLE_EDGE style.
	 */
	public static final String EDGESTYLE_LOOP = "loopEdgeStyle";

	/**
	 * Name of the side to side edge style. Can be used as a string value
	 * for the STYLE_EDGE style.
	 */
	public static final String EDGESTYLE_SIDETOSIDE = "sideToSideEdgeStyle";

	/**
	 * Name of the top to bottom edge style. Can be used as a string value
	 * for the STYLE_EDGE style.
	 */
	public static final String EDGESTYLE_TOPTOBOTTOM = "topToBottomEdgeStyle";
	
	/**
	 * Name of the orthogonal edge style. Can be used as a string value for
	 * the STYLE_EDGE style.
	 */
	public static final String EDGESTYLE_ORTHOGONAL = "orthogonalEdgeStyle";

	/**
	 * Name of the generic segment edge style. Can be used as a string value
	 * for the STYLE_EDGE style.
	 */
	public static final String EDGESTYLE_SEGMENT = "segmentEdgeStyle";
 
	/**
	 * Name of the ellipse perimeter. Can be used as a string value
	 * for the STYLE_PERIMETER style.
	 */
	public static final String PERIMETER_ELLIPSE = "ellipsePerimeter";

	/**
	 * Name of the rectangle perimeter. Can be used as a string value
	 * for the STYLE_PERIMETER style.
	 */
	public static final String PERIMETER_RECTANGLE = "rectanglePerimeter";

	/**
	 * Name of the rhombus perimeter. Can be used as a string value
	 * for the STYLE_PERIMETER style.
	 */
	public static final String PERIMETER_RHOMBUS = "rhombusPerimeter";

	/**
	 * Name of the triangle perimeter. Can be used as a string value
	 * for the STYLE_PERIMETER style.
	 */
	public static final String PERIMETER_TRIANGLE = "trianglePerimeter";

	/**
	 * Name of the hexagon perimeter. Can be used as a string value
	 * for the STYLE_PERIMETER style.
	 */
	
	public static final String PERIMETER_HEXAGON  = "hexagonPerimeter";
		
	/**
	 * Name of attributes of OCBC models
	 */		
	public static final String source  = "source";//indicate the source of the model, such as "discovered" or "designed" or "imported"
	
	public static final String discovered  = "discovered";
	
	public static final String designed  = "designed";
	
	public static final String imported  = "imported";
	
	/**
	 * indicate the environment where the model is in, such as "conformance_checking_plugin" or "model_editor"
	 */
	
	public static final String model_environment  = "model_environment";
	
	public static final String conformance_checking_plugin  = "conformance_checking_plugin";
	
	public static final String model_editor  = "model_editor";
	
	/**
	 * Name of the discovery approach
	 */
	
	public static final String perfectFitnessApproach = "perfectFitnessApproach";
	
	public static final String balancedMetricsApproach = "customizedModelApproach";
	
	/**
	 * constants used in the model discovery plugin (balanced metrics)
	 */
	public static final String fitnessAlgorithmWithoutThreshold = "fitnessAlgorithmWithoutThreshold";
	
	public static final String fitnessAlgorithmWithNumThreshold = "fitnessAlgorithmWithNumThreshold";
	
	public static final String fitnessAlgorithmWithRatioThreshold = "fitnessAlgorithmWithRatioThreshold";
	
	public static final String fitnessAlgorithmHalfWithThreshold = "fitnessAlgorithmHalfWithThreshold";
	
	public static final String precisionAlgorithmWithNumThreshold = "precisionAlgorithmWithNumThreshold";
	
	public static final String precisionAlgorithmWithRatioThreshold = "precisionAlgorithmWithRatioThreshold";
	
	public static final String precisionAlgorithmByEntropy = "precisionAlgorithmByEntropy";
	
	public static final String generalizationAlgorithmBinomialDistribution= "generalizationAlgorithmBinomialDistribution";
	
	public static final String generalizationAlgorithmBetaDistribution = "generalizationAlgorithmBetaDistribution";
	
	/**
	 * Name of simplification types of OCBC models
	 */		
	public static final String simplificationType  = "simplificationType";//indicate the source of the model, such as "discovered" or "designed" or "imported"
	
	public static final String transitive  = "transitive";
	
	public static final String necessary  = "necessary";
	
	public static final String redundant  = "redundant";
	
	public static final String Yes  = "yes";
	
	public static final String No  = "no";
	
	public static final String brotherConstraint  = "brotherConstraint";//same: reference, target and crel reference; different cardinality types
	
	public static final String fatherConstraint  = "fatherConstraint";
	
	public static final String twinConstraint  = "twinConstraint";//same: reference, target and cardinality type; different crel references

	/**
	 * Name of metrics of activity edge cells
	 */	
	public static final String confidence  = "confidence";
	
	public static final String support  = "support";
	
	public static final String lift  = "lift";
	
	public static final String precision  = "precision";

	/**
	 * Name of attributes of cells
	 */		
	public static final String TYPE  = "type";//indicate the type of the cell(such as "Class", "ActivityRelation")
	
	public static final String CARDINALITY  = "Cardinality";//indicate a node is a cardinality node, used for showing/hide the cardinality
	
	public static final String CARDINALITYTYPE  = "CardinalityType";//indicate the cardinality type of activity relation (such as "response", "unary-precedence")
	
	public static final String correlationType  = "correlationType";//indicate the correlation type of activity relation (such as "ClassCell" or "ClassEdge")
	
	public static final String CARDINALITYPOSITION  = "CardinalityPosition";//Name: indicate the cardinality type of activity relation (such as "response", "unary-precedence")
	
	public static final String SOURCECARDINALITY  = "SourceCardinality";//Value: indicate the source cardinality of class and aoc relation	 used for CARDINALITYPOSITION
	
	public static final String TARGETCARDINALITY  = "TargetCardinality";//Value: indicate the Target cardinality of class and aoc relation used for CARDINALITYPOSITION
	
	public static final String RELATIONSHIPTYPE  = "RelationshipType";//indicate the class relationship type (such as "supplier_order-fk_commande-supplier-order_line")
	
	public static final String SOURCEALWAYSCARDINALITY  = "SourceAlwaysCardinality";//indicate the source always cardinality of class and aoc relation
	
	public static final String SOURCEEVENTUALLYCARDINALITY  = "SourceEventuallyCardinality";//indicate the source Eventually cardinality of class and aoc relation
	
	public static final String TARGETALWAYSCARDINALITY  = "TargetAlwaysCardinality";//indicate the Target always cardinality of class and aoc relation
	
	public static final String TARGETEVENTUALLYCARDINALITY  = "TargetEventuallyCardinality";//indicate the Target Eventually cardinality of class relation
	
	public static final String SECONDID  = "SecondId";//indicate the second of the cell(if the cell corresponds to multiple elements in the OCBC model)
	
	/**
	 * state of the cells-Guangming
	 */		
	public static final String CELLSTATE  = "CellState";
	
	public static final String IMPORTEDSTATE  = "InputState";//indicate the cell is imported
	
	public static final String CREATEDSTATE  = "CreatedState";//indicate the cell is created
	
	/**
	 * Name of the nodes-Guangming
	 */		
	public static final String CLASSCELL  = "ClassCell";
	
	public static final String ACTIVITYCELL  = "ActivityCell";	
	
	public static final String CONNECTOR  = "ConnectorCell";	
	
	public static final String LABLE = "LabelCell";		
	
	/**
	 * geometry of the nodes-Guangming
	 */		
	public static final double CLASSCELLHEIGHT  = 40.0;
	
	public static final double CLASSCELLWIDTH  = 53.0;
	
	public static final double ACTIVITYCELLHEIGHT  = 40.0;
	
	public static final double ACTIVITYCELLWIDTH  = 53.0;
	
	
	/**
	 * Name of the edges
	 */		
	public static final String AOCRELATION  = "AOCEdge";
	
	public static final String CLASSRELATION  = "ClassEdge";
	
	public static final String ACTIVITYRELATION  = "ActivityEdge";
	
	public static final String CRELRELATION  = "CrelEdge";
	
	public static final String CRELOCRELATION  = "CrelOCEdge";
	
	public static final String CRELRTRELATION  = "CrelRTEdge";

	/**
	 * Name of the activity cardinalities
	 */
	public static final String RESPONSE  = "response";
	
	public static final String UNARYRESPONSE  = "unary_response";
	
	public static final String PRECEDENCE  = "precedence";
	
	public static final String UNARYPRECEDENCE  = "unary_precedence";
	
	public static final String COEXISTENCE   = "co_existence";
	
	public static final String NONRESPONSE  = "non_response";
	
	public static final String NONPRECEDENCE  = "non_precedence";
	
	public static final String NONCOEXISTENCE  = "non_coexistence";
	
	public static final String NoConstraints  = "NoConstraints";//fake type to indicate there are no constraints
	
	/**
	 * Name of the combined activity cardinalities
	 */
	
	public static final String RESPONSE_PRECEDENCE  = "response-precedence";//arrow direction: from reference to target

	public static final String UNARYRESPONSE_UNARYPRECEDENCE = "unary_response-unary_precedence";
	
	public static final String NONRESPONSE_NONPRECEDENCE  = "non_response-non_precedence";
	
	public static final String COEXISTENCE_COEXISTENCE  = "co_existence-co_existence";
	
	public static final String NONCOEXISTENCE_NONCOEXISTENCE  = "non_coexistence-non_coexistence";
	
	/**
	 * Name of 9 variants (constituting the model state)
	 */
	
	public static final String beforeZeroAndAfterZero  = "beforeZeroAndAfterZero";

	public static final String beforeOneAndAfterZero = "beforeOneAndAfterZero";
	
	public static final String beforeMultipleAndAfterZero  = "beforeMultipleAndAfterZero";
	
	public static final String beforeZeroAndAfterOne  = "beforeZeroAndAfterOne";

	public static final String beforeOneAndAfterOne = "beforeOneAndAfterOne";
	
	public static final String beforeMultipleAndAfterOne  = "beforeMultipleAndAfterOne";
	
	public static final String beforeZeroAndAfterMultiple  = "beforeZeroAndAfterMultiple";

	public static final String beforeOneAndAfterMultiple = "beforeOneAndAfterMultiple";
	
	public static final String beforeMultipleAndAfterMultiple  = "beforeMultipleAndAfterMultiple";	
	
	/**
	 * Names of the nodes in the trees in instance panel in model discovery plugin
	 */
	public static final String source_object_list  = "source_object_list";	
	
	public static final String target_object_list  = "target_object_list";	
	
	/**
	 * Names of the nodes in the trees in conformance checking plugin
	 */
	public static final String conformance_checking_input_log  = "input: XOC log";
	
	public static final String event_list  = "event_list";	
	
	public static final String attribute_list  = "attribute_list";
	
	public static final String object_reference_list  = "object_reference_list";
	
	public static final String object_model_list  = "object_model_list";
	
	public static final String last_object_model  = "last_object_model";
	
	public static final String object_list  = "object_list";
	
	public static final String object_model  = "object_model";
	
	public static final String object_relation_list  = "object_relation_list";
	
	public static final String no_input = "no_input";
	
	public static final String conformance_checking_input_model = "input: OCBC model";
	
	public static final String class_model  = "class_model";
	
	public static final String class_list  = "class_list";
	
	public static final String class_node  = "class";
	
	public static final String class_relation_list  = "class_relation_list";
	
	public static final String class_relation  = "class_relation";
	
	public static final String source_cardinality  = "source_cardinality";
	
	public static final String target_cardinality  = "target_cardinality";
	
	public static final String behavioral_model  = "behavioral_model";
	
	public static final String activity_list  = "activity_list";
	
	public static final String AOC_relation_list  = "AOC_relation_list";
	
	public static final String AOC_relation  = "AOC_relation";
	
	public static final String behavioral_constraint_list  = "behavioral_constraint_list";
	
	public static final String behavioral_constraint  = "behavioral_constraint";
	
	public static final String reference_activity  = "reference_activity";
	
	public static final String target_activity  = "target_activity";
	
	public static final String cardinality_type  = "cardinality_type";
	
	public static final String correlation_type  = "correlation_type";
	
	public static final String conformance_checking_output = "output: deviations";
	
	/**
	 * Names of the nodes (object attributes shown in monotonicity panel) in the trees in conformance checking plugin
	 */
//	public static final String birthEventOrderNum  = "birth_moment";
//	
//	public static final String deathEventOrderNum  = "death_moment";
	
	
	public static final String lifecycle  = "lifecycle";
	
	public static final String classVersionList  = "classVersionList";
	
	public static final String lifecycle_attribute  = "lifecycle_attribute";
	
	public static final String startEventOrderNum  = "start_moment";
	
	public static final String endEventOrderNum  = "end_moment";
	
	public static final String classChangeFlag  = "class_change";
	
	public static final String disappearingObjectList  = "disappearing_object_list";
	
	public static final String classChangedObjectList  = "class_changing_object_list";
	
	/**
	 * Name of the symbols
	 */
	
//	public static final String DIAMONDSYMBOL = "\u25C7";
//	
//	public static final String SQUARESYMBOL = "\u25A1";
	
	public static final String DIAMONDSYMBOL = "◇";
	
	public static final String SQUARESYMBOL = "□";
	
	/**
	 * Name of the symbols
	 */
	
	public static final String 	CONNECTOR_FOCUSED = "connector_focused";
	
	public static final String CONNECTOR_TRANSPARENT = "connector_transparent";
	
	public static final String 	CONNECTOR_VIOLATING = "connector_violated";
	
	public static final String 	LABEL_FOCUSED = "Label_focused";
	
	public static final String LABEL_NORMAL = "Label_normal";
	
	public static final String STRAIGHTLINE  = "StraightLine";
	
	public static final String HIGHLIGHTEDGE  = "highlight_edge";
	
	/**
	 * Name of the views
	 */
	
	public static final String TRANSPARENTEDGE  = "transparent_edge";
	
	public static final String TRANSPARENTCLASSEDGENAME  = "transparent_class_edge_name";
	
	public static final String TRANSPARENTAOCEDGENAME  = "transparent_aoc_edge_name";
	
	public static final String TRANSPARENTCARDINALITY  = "transparent_cardinality";
	
	/**
	 * Name of panels in the model discovery UI
	 */
	public static final String Instance  = "Instance";
	
	public static final String Distribution  = "Distribution";
	/**
	 * Name of variable related database
	 */
	public static final String TABLE  = "table";//indicate the source table name
	
	public static final String ROWID  = "rowid";//indicate the id in table
	
	public static final String FK  = "foreign_key";//indicate the foreign key in table
	
	public static final String PK  = "primary_key";//indicate the primary key in table
	
	public static final String ACTIVITY  = "activity";//indicate an activity column in table
	
	public static final String CONSTRAINT  = "constraint";//indicate a column constraint of a class from one table
	
	public static final String column  = "column";//indicate common columns of a table
	
	public static final String mapping  = "mapping";//
	
	public static final String reference  = "reference";//indicate a fk refers to a pk
	
	/**
	 * value types of table column
	 */
	public static final String Int  = "int";
	
	public static final String Varchar  = "varchar(256)";
	
	public static final String Double  = "double";
	
	public static final String Datetime  = "datetime";
	
	public static final String Text  = "text";
	
	/**
	 * Name of the elements in Dolibarr
	 */	
	public static final String Login  = "Login";
	
	public static final String CommercialMenu  = "CommercialMenu";
	
	public static final String NewCustomerOrder  = "NewCustomerOrder";
	
	public static final String CreateCustomerOrderDraft  = "CreateCustomerOrderDraft";
	
	public static final String AddCustomerOrderLine  = "AddCustomerOrderLine";
	
	public static final String ValidateCustomerOrderDraft  = "ValidateCustomerOrderDraft";
		
	public static final String ValidateCustomerOrderYes  = "ValidateCustomerOrderYes";
	
	public static final String BuildObjectIdMapping  = "BuildObjectIdMapping";
	
	public static final String SearchObject  = "SearchObject";
	
	public static final String SearchSupplierOrder  = "SearchSupplierOrder";
	
	public static final String SearchShipment  = "SearchShipment";
	
	public static final String SearchContent  = "SearchContent"; //an attribute attached on the search operation
	
	public static final String ShipCustomerOrderProduct  = "ShipCustomerOrderProduct";
	
	public static final String NewCustomerOrderShipment  = "NewCustomerOrderShipment";
	
	public static final String CreateCustomerOrderShipment  = "CreateCustomerOrderShipment";
	
	public static final String ValidateCustomerOrderShipment  = "ValidateCustomerOrderShipment";
	
	public static final String ValidateCustomerOrderShipmentYes  = "ValidateCustomerOrderShipmentYes";
	
	public static final String CreateCustomerOrderInvoiceOnShipmentCard  = "CreateCustomerOrderInvoiceOnShipmentCard"; //on the shipment card
	
	public static final String CreateCustomerOrderInvoiceDraft  = "CreateCustomerOrderInvoiceDraft"; 
	
	public static final String RemoveAllCustomerInvoiceLines  = "RemoveAllCustomerInvoiceLines";//remove all invoice lines
	
	public static final String RemoveAllCustomerInvoiceLinesYes  = "RemoveAllCustomerInvoiceLinesYes";//remove all invoice lines
	
	public static final String ValidateCustomerOrderInvoiceDraft  = "ValidateCustomerOrderInvoiceDraft"; 
	
	public static final String ValidateCustomerOrderInvoiceDraftYes  = "ValidateCustomerOrderInvoiceDraftYes"; 
	
	public static final String DoPaymentOnCustomerOrderInvoiceCard  = "DoPaymentForCustomerOrderInvoice"; //on the invoice card
	
	public static final String InputAmoutForInvoiceWhenPaying  = "InputAmoutForInvoiceWhenPaying"; 
	
	public static final String PayCustomerOrderInvoice  = "PayCustomerOrderInvoice"; 
	
	public static final String ValidatePayment  = "ValidatePayment"; 
	
	/**
	 * Name of attributes of elements on the page for the edit click panel of the dolibarr robot plugin
	 */
	
	public static final String ElementType  = "element_type";//indicate the type of page element
	
	public static final String click  = "click";//used for element type
	
	public static final String clickFake  = "click_fake";//used for element type to indicate a fake click
	
	public static final String clickCheck  = "click_check";//used for element type to indicate a check box click
	
	public static final String attribute  = "attribute";//used for element type
	
	public static final String attributeMultiple  = "attribute_multiple";//used for element type to refer to multiple attributes
	
	public static final String hierarchy  = "hierarchy";//used for element type (could be the parent element of an attribute element)
	
	public static final String ElementName  = "element_name";//indicate the type of page element
	
	public static final String ElementId  = "id";
	
	public static final String TagName  = "tag_name";
	
	public static final String TagValue  = "tag_value";
	
	public static final String AttributeName  = "attribute_name";
	
	public static final String AttributeValue  = "attribute_value";
	
	public static final String ElementIndex  = "element_index";//if multiple elements are satisfied, the ElementIndex indicate choosing which one
	
	public static final String DefaultValue  = "default_value";//indicate the default value for an input
	
	public static final String Dependency  = "dependency";//indicate the dependent element of one element
	
	public static final String Direction  = "direction";//indicate the father element of one element
	
	public static final String nullValue  = "null";//indicate the null value
	
	public static final String child  = "child";//indicate the child direction
	
	public static final String parent  = "parent";//indicate the parent direction
	
	public static final String input  = "input";//indicate the tag name in the tag name box 
	
	public static final String form  = "form";//indicate the tag name in the tag name box 
	
	public static final String table  = "table";//indicate the tag name in the tag name box 
	
	public static final String textarea  = "textarea";//indicate the tag name in the tag name box 
	
	public static final String a  = "a";//indicate the tag name in the tag name box 
	
	public static final String tr  = "tr";//indicate the tag name in the tag name box 
	
	public static final String td  = "td";//indicate the tag name in the tag name box 
	
	public static final String span  = "span";//indicate the tag name in the tag name box 
	
	public static final String select  = "select";//indicate the tag name in the tag name box 
	
	public static final String id  = "id";//indicate the id in the attribute name box (a special attribute name)
	
	public static final String dynamic  = "dynamic";//indicate the value is dynamic (depending on a value in the log)
	
	public static final String live  = "live";//indicate the value is generated when the system is running (such as FA1702-0025)
	
	/**
	 * Name of attributes in the simulation log
	 */
	
	public static final String 	activity = "activity";//indicate the activity of one click, such as "create order", "add order line"
	
	public static final String 	clickPerson = "clickPerson";//keep the person who click a button on the Dolibarr system
	
	public static final String 	clickTime = "clickTime";//indicate the time when someone click a button on the Dolibarr system	
	
	public static final String 	orderId = "orderId";//indicate the order id in one object such as an order line
	
	public static final String 	orderLineId = "orderLineId";//indicate the order line id in one object
	
	public static final String 	productName = "productName";//indicate the product name	
	
	public static final String 	shipmentId = "shipmentId";//indicate the shipment id 
	
	public static final String 	shipmentLineId = "shipmentLineId";//indicate the shipment line id in one shipment
	
	public static final String 	invoiceId = "invoiceId";//indicate the invoice id 
	
	public static final String 	invoiceLineId = "invoiceLineId";//indicate the invoice line id 
	
	public static final String 	paymentId = "paymentId";//indicate the paymentId id 
	
	public static final String 	paymentLineId = "paymentLineId";//indicate the payment line id 
	
	public static final String 	amount = "amount";//indicate the amount for one payment line (corresponding to one invoice)
	
	public static final String 	warehouse = "warehouse";//indicate the order id in one object such as an order line
	
	public static final String 	invoiceLines = "invoiceLines";//indicate the invoice list in the invoice object	
	
	public static final String 	shipmentLines = "shipmentLines";//indicate the shipment line list in the invoice object	
	
	public static final String 	paymentLines = "paymentLines";//indicate the payment line list in the invoice object	
	
	public static final String 	createCustomerOrder = "create_customer_order";//indicate the activity of creating customer order
	
	public static final String 	orderDate = "orderDate";//indicate the date when an order is created
	
	public static final String 	orderYear = "orderYear";//indicate the date (year) when an order is created
	
	public static final String 	orderMonth = "orderMonth";//indicate the date (month) when an order is created
	
	public static final String 	orderDay = "orderDay";//indicate the date (day) when an order is created
	
	public static final String 	deliveryDate = "deliveryDate";//indicate the date when an order is delivered on schedule
	
	public static final String 	deliveryYear = "deliveryYear";//indicate the date (year) when an order is delivered on schedule
	
	public static final String 	deliveryMonth = "deliveryMonth";//indicate the date (month) when an order is delivered on schedule
	
	public static final String 	deliveryDay = "deliveryDay";//indicate the date (day) when an order is delivered on schedule
	
	public static final String 	deliveryHour = "deliveryHour";//indicate the date (month) when an order is delivered on schedule
	
	public static final String 	deliveryMinute = "deliveryMinute";//indicate the date (day) when an order is delivered on schedule
	
	public static final String 	addCustomerOrderLine = "add_customer_order_line";//indicate the activity of adding customer order line
	
	public static final String 	validateCustomerOrder = "validate_customer_order";//indicate the activity of validating customer order
	
	public static final String 	createCustomerShipmentPreparation = "create_customer_shipment_preparation";//indicate the preparation for creating customer shipment
	
	public static final String 	createCustomerShipment = "create_customer_shipment";//indicate the activity of creating customer shipment
	
	public static final String 	validateCustomerShipment = "validate_customer_shipment";//indicate the activity of validating customer shipment
	
	public static final String 	createCustomerInvoice = "create_customer_invoice";//indicate the activity of creating customer invoice
	
	public static final String 	createCustomerInvoiceForOneOrder = "create_customer_invoice_for_one_order";//indicate the invoice for one order
	
	public static final String 	invoiceDate = "invoiceDate";//indicate the date when an invoice is created
	
	public static final String 	invoiceYear = "invoiceYear";//indicate the date (year) when an invoice is created
	
	public static final String 	invoiceMonth = "invoiceMonth";//indicate the date (month) when an invoice is created
	
	public static final String 	invoiceDay = "invoiceDay";//indicate the date (day) when an invoice is created
	
	public static final String 	createCustomerInvoiceForMultipleOrders = "create_customer_invoice_for_multiple_orders";//indicate the invoice for multiple orders
	
	public static final String 	checkCustomerOrderForOneCustomer = "checkCustomerOrderForOneCustomer";//artificial click to check order
	
	public static final String 	generateInvoiceAfterCheckingOrders = "generateInvoiceAfterCheckingOrders";//artificial click to generate invoice after checking order
	
	public static final String 	removeCustomerInvoiceLine = "remove_customer_invoice_line";//artificial click to remove the invoice lines added by default
	
	public static final String 	addCustomerInvoiceLine = "add_customer_invoice_line";//add invoice line for an invoice
	
	public static final String 	validateCustomerInvoice = "validate_customer_invoice";//indicate the activity of validating customer invoice
	
	public static final String 	createCustomerShipmentForOneOrder = "create_customer_shipment_for_one_order";//indicate the shipment for one order
	
	public static final String 	createCustomerShipmentForMultipleOrders = "create_customer_shipment_for_multiple_orders";//indicate the shipment for multiple orders
	
	public static final String 	addCustomerShipmentLine = "add_customer_shipment_line";//add shipment line for an shipment
	
	public static final String 	addCustomerPaymentLine = "add_customer_payment_line";//add payment line for an payment (a amount for an invoice)
	
	public static final String 	createCustomerPaymentPreparation = "create_customer_payment_preparation";//indicate the preparation for creating customer payment
	
	public static final String 	createCustomerPayment = "create_customer_payment";//indicate the activity of creating customer payment
	
	public static final String 	paymentDate = "paymentDate";//indicate the date when an payment is created
	
	public static final String 	paymentYear = "paymentYear";//indicate the date (year) when an payment is created
	
	public static final String 	paymentMonth = "paymentMonth";//indicate the date (month) when an payment is created
	
	public static final String 	paymentDay = "paymentDay";//indicate the date (day) when an payment is created
	
	/**
	 * suplier order
	 */
	public static final String 	createSupplierOrder = "create_supplier_order";//indicate the activity of creating supplier order
	
	public static final String 	addSupplierOrderLine = "add_supplier_order_line";
	
	public static final String 	validateSupplierOrder = "validate_supplier_order";
	
	public static final String 	receiveSupplierOrder = "receive_supplier_order";
	
	public static final String 	makeSupplierOrder = "make_supplier_order";
	
	public static final String 	addGoodsLine = "add_goods_line";
	
	public static final String 	validateDispatchment = "validate_dispatchment";
	
	public static final String 	createSupplierInvoice = "create_supplier_invoice";
	
	public static final String 	create_supplier_invoice_preparation = "create_supplier_invoice_preparation";
	
	public static final String 	checkSupplierOrder = "check_supplier_order";//artificial click to check order
	
	public static final String 	create_supplier_invoice_draft = "create_supplier_invoice_draft";
	
	public static final String 	removeSupplierInvoiceLine = "remove_supplier_invoice_line";//artificial click to remove the invoice lines added by default
	
	public static final String 	addSupplierInvoiceLine = "add_supplier_invoice_line";//add invoice line for an invoice
	
	public static final String 	create_supplier_payment_preparation = "create_supplier_payment_preparation";
	
	public static final String 	add_supplier_payment_line = "add_supplier_payment_line";
	
	public static final String 	create_supplier_payment = "create_supplier_payment";
	
	/**
	 * type of objects in the dolibarr
	 */
	public static final String 	customerOrder= "customerOrder";
	
	public static final String 	customerInvoice= "customerInvoice";
	
	public static final String 	supplierOrder= "supplierOrder";
	
	public static final String 	supplierInvoice= "supplierInvoice";
	
	/**
	 * Name of parameters in the editing click panel which will be writen into the configuration file
	 */
	//record the features of one element on the page, the feature (such tag = "order" or null (content), id = 12 (special attribute), 
    //width = "100%" (common attributes), index=3 (the third element satisfying above features))
    //are used to identify the element: (1) use the id (2)if no id (=value), use the tag=content) 
    //(3)if no content, use the tag and attribute (=value) (4)use index except the first situation
	public static final String 	pageElementNameVSFeatureVSValue = "pageElementNameVSFeatureVSValue";//discarded
	
	public static final String 	clickVSElementVSFeatureVSValue = "clickVSElementVSFeatureVSValue";
	
    //record the attribute element names of clicks
	public static final String 	clickVSAttributeList = "clickVSAttributeList";	
    
    //record the matched attributes for each page element of each click of each activity in the simulation log
	public static final String 	activityVSClickVSElementVSAttribute = "activityVSClickVSElementVSAttribute";
	
    //record the matched click names of each activity in the simulation log
	public static final String 	activityVSClickList = "activityVSClickList";
	
	/*
	 * constants for the mapping for the click instance panel
	 */	
	public static final String 	clickInstanceList = "clickInstanceList";
	
	public static final String 	eventList = "eventList";	
	
	public static final String 	columnNameVSValueList = "columnNameVSValueList";
	
	public static final String 	clickInstanceVSElementNameVSValue = "clickInstanceVSElementNameVSValue";
		
	/*
	 * constants for the column name for the click instance panel
	 */
	public static final String 	simulation_log_event = "simulation_log_event";	
	
	public static final String 	dolibarr_click_instance = "dolibarr_click_instance";		

	public static final String 	click_instance = "click_instance";		
	
	public static final String 	event_attribute_name = "event_attribute_name";	
	
	public static final String 	click_element_name = "click_element_name";	
	
	public static final String 	value = "value";		
	
	public static final List<String> NECESSARYATTRIBUTELIST = Collections.unmodifiableList(
	    new ArrayList<String>() {
			private static final long serialVersionUID = 1L;{
	        add(TYPE);
	        add(TABLE);
	        add(ROWID);
	        add(ACTIVITY);
	        add(CARDINALITYTYPE);
	        add(SOURCEALWAYSCARDINALITY);
	        add(SOURCEEVENTUALLYCARDINALITY);
	        add(TARGETALWAYSCARDINALITY);
	        add(TARGETEVENTUALLYCARDINALITY);
			}
	    }
	);

	/**
	 * define a list containing all constraint variants, such as beforeZeroAndAfterZero, i.e., (0,0)
	 * the variants are sorted, the first element is (0,0)
	 */
	public static final List<String> constraintVariantList = Collections.unmodifiableList(
	    new ArrayList<String>() {
			private static final long serialVersionUID = 1L;{
				add(beforeZeroAndAfterZero);
	        	add(beforeOneAndAfterZero);
	        	add(beforeMultipleAndAfterZero);
	        	add(beforeZeroAndAfterOne);
	        	add(beforeOneAndAfterOne);
	        	add(beforeMultipleAndAfterOne);
	        	add(beforeZeroAndAfterMultiple);
	        	add(beforeOneAndAfterMultiple);
	        	add(beforeMultipleAndAfterMultiple);
			}
	    }
	);
	
	/**
	 * record the corresponding variant list for each constraint type
	 */
	public static final Map<String, List<String>> constraintTypeVSVariantList = Collections.unmodifiableMap(
	    new HashMap<String, List<String>>() {
			private static final long serialVersionUID = 1L;{			
		        put(NoConstraints, new ArrayList<String>(){
					private static final long serialVersionUID = 1L;
					{
			        	add(beforeZeroAndAfterZero);
			        	add(beforeOneAndAfterZero);
			        	add(beforeMultipleAndAfterZero);
			        	add(beforeZeroAndAfterOne);
			        	add(beforeOneAndAfterOne);
			        	add(beforeMultipleAndAfterOne);
			        	add(beforeZeroAndAfterMultiple);
			        	add(beforeOneAndAfterMultiple);
			        	add(beforeMultipleAndAfterMultiple);
					}
		        });
		        put(COEXISTENCE, new ArrayList<String>(){
					private static final long serialVersionUID = 1L;
					{
			        	add(beforeOneAndAfterZero);
			        	add(beforeMultipleAndAfterZero);
			        	add(beforeZeroAndAfterOne);
			        	add(beforeOneAndAfterOne);
			        	add(beforeMultipleAndAfterOne);
			        	add(beforeZeroAndAfterMultiple);
			        	add(beforeOneAndAfterMultiple);
			        	add(beforeMultipleAndAfterMultiple);
					}
		        });
		        put(RESPONSE, new ArrayList<String>(){
					private static final long serialVersionUID = 1L;
					{
			        	add(beforeZeroAndAfterOne);
			        	add(beforeOneAndAfterOne);
			        	add(beforeMultipleAndAfterOne);
			        	add(beforeZeroAndAfterMultiple);
			        	add(beforeOneAndAfterMultiple);
			        	add(beforeMultipleAndAfterMultiple);
					}
		        });
		        put(UNARYRESPONSE, new ArrayList<String>(){
					private static final long serialVersionUID = 1L;
					{
			        	add(beforeZeroAndAfterOne);
			        	add(beforeOneAndAfterOne);
			        	add(beforeMultipleAndAfterOne);
					}
		        });
		        put(NONRESPONSE, new ArrayList<String>(){
					private static final long serialVersionUID = 1L;
					{
			        	add(beforeZeroAndAfterZero);
			        	add(beforeOneAndAfterZero);
			        	add(beforeMultipleAndAfterZero);
					}
		        });
		        put(PRECEDENCE, new ArrayList<String>(){
					private static final long serialVersionUID = 1L;
					{
			        	add(beforeOneAndAfterZero);
			        	add(beforeMultipleAndAfterZero);
			        	add(beforeOneAndAfterOne);
			        	add(beforeMultipleAndAfterOne);
			        	add(beforeOneAndAfterMultiple);
			        	add(beforeMultipleAndAfterMultiple);
					}
		        });
		        put(UNARYPRECEDENCE, new ArrayList<String>(){
					private static final long serialVersionUID = 1L;
					{
			        	add(beforeOneAndAfterZero);
			        	add(beforeOneAndAfterOne);
			        	add(beforeOneAndAfterMultiple);
					}
		        });
		        put(NONPRECEDENCE, new ArrayList<String>(){
					private static final long serialVersionUID = 1L;
					{
			        	add(beforeZeroAndAfterZero);
			        	add(beforeZeroAndAfterOne);
			        	add(beforeZeroAndAfterMultiple);
					}
		        });
		        put(NONCOEXISTENCE, new ArrayList<String>(){
					private static final long serialVersionUID = 1L;
					{
						add(beforeZeroAndAfterZero);
					}
		        });
			}
	    }
	);
	
	public mxConstants(){

//		NECESSARYATTRIBUTELIST.add(TYPE);
		
//		public static final String   = "type";//indicate the type of the cell(such as "Class", "ActivityRelation")
//		
//		public static final String CARDINALITYTYPE  = "CardinalityType";//indicate the cardinality of activity relation
//		
//		public static final String SOURCEALWAYSCARDINALITY  = "SourceAlwaysCardinality";//indicate the source always cardinality of class and aoc relation
//		
//		public static final String SOURCEEVENTUALLYCARDINALITY  = "SourceEventuallyCardinality";//indicate the source Eventually cardinality of class and aoc relation
//		
//		public static final String TARGETALWAYSCARDINALITY  = "TargetAlwaysCardinality";//indicate the Target always cardinality of class and aoc relation
//		
//		public static final String TARGETEVENTUALLYCARDINALITY  = "TargetEventuallyCardinality";//indicate the Target Eventually cardinality of class relation
//		
	}
	
}
