package org.processmining.ocbc.conformance.checkers;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.processmining.mxgraph.util.mxConstants;
import org.processmining.ocbc.discovery.filter.FilterFunction;
import org.processmining.ocbc.log.model.XOCAttribute;
import org.processmining.ocbc.log.model.XOCEvent;
import org.processmining.ocbc.log.model.XOCLog;
import org.processmining.ocbc.log.model.XOCORelation;
import org.processmining.ocbc.log.model.XOCObject;
import org.processmining.ocbc.log.model.impl.XOCORelationImpl;
import org.processmining.ocbc.log.model.impl.xConstants;
import org.processmining.ocbc.models.ActivityEdge;
import org.processmining.ocbc.models.ActivityNode;
import org.processmining.ocbc.models.ClassEdge;
import org.processmining.ocbc.models.ClassNode;
import org.processmining.ocbc.models.OCBCModel;

public class BehavioralConstraintsChecking {
	private FilterFunction filter = new  FilterFunction();
	private Map<String, Map<String, List<String>>> activityEdgeVSEventVSPrecedenceEventList = new HashMap<String, Map<String, List<String>>>();
	private Map<String, Map<String, List<String>>> activityEdgeVSEventVSResponseEventList = new HashMap<String, Map<String, List<String>>>();
	
	private Map<ActivityEdge, Map<XOCEvent, List<XOCEvent>>> behavioralConstraintVSEventVSPrecedenceEventList = new HashMap<ActivityEdge, Map<XOCEvent, List<XOCEvent>>>();
	private Map<ActivityEdge, Map<XOCEvent, List<XOCEvent>>> behavioralConstraintVSEventVSResponseEventList = new HashMap<ActivityEdge, Map<XOCEvent, List<XOCEvent>>>();
	
	/**
	 * get the preceding corresponding events of deviating event for one behavioral constraint
	 * @return
	 */
	public Map<ActivityEdge, Map<XOCEvent, List<XOCEvent>>> getBehavioralConstraintVSEventVSPrecedenceEventListMap(){
		return behavioralConstraintVSEventVSPrecedenceEventList;
	}
	
	/**
	 * get the subsequent corresponding events of deviating event for one behavioral constraint
	 * @return
	 */
	public Map<ActivityEdge, Map<XOCEvent, List<XOCEvent>>> getBehavioralConstraintVSEventVSResponseEventListMap(){
		return behavioralConstraintVSEventVSResponseEventList;
	}
	
	public void checking(XOCLog log, OCBCModel model)
	{		
		List<ActivityEdge> activityRelations = model.getAllActivityRelations();

		System.out.println("output activity relation end **********");
		// for each constraint c in C
		for(ActivityEdge activityRelation : activityRelations){
			behavioralConstraintVSEventVSPrecedenceEventList.put(activityRelation, new HashMap<XOCEvent, List<XOCEvent>>());
			behavioralConstraintVSEventVSResponseEventList.put(activityRelation, new HashMap<XOCEvent, List<XOCEvent>>());
			String activityRelationId = activityRelation.getID();
			String correlationType = activityRelation.getCorrelationType();
			ActivityNode referenceNode = activityRelation.getReferenceNode();	
			Map<String, List<String>> eventVSPrecedenceEventList = new HashMap<String, List<String>>();
			Map<String, List<String>> eventVSResponseEventList = new HashMap<String, List<String>>(); 
			
//			System.out.println("activityRelationId: " + activityRelationId);
//			System.out.println("referenceActivityName: " + referenceActivityName);
			
			// for each reference event in E-filter(c.reference.activity)		
			for(XOCEvent eventReference : filterEvents(log.getAllEvents(), null, referenceNode, 5)){			
				String sourceEventId = eventReference.getAttributes().get(xConstants.ID).toString();
				List<XOCEvent> targetEventList;
				if(correlationType.equals(mxConstants.CLASSCELL)){
					targetEventList = computeTargetEventListByClass(activityRelation, log, eventReference);
				}
				else{
					targetEventList = computeTargetEventListByClassRelation(activityRelation, log, eventReference);
				}
				List<String> precedenceEventIdList = new ArrayList<String>();
				List<String> responseEventIdList = new ArrayList<String>();
				
				List<XOCEvent> precedenceEventList = new ArrayList<XOCEvent>();
				List<XOCEvent> responseEventList = new ArrayList<XOCEvent>();
//				System.out.println("sourceEventId: " + sourceEventId);
				
				for(XOCEvent targetEvent :targetEventList){
//					System.out.println("getRelatedEventList");
					String targetEventId = targetEvent.getAttributes().get(xConstants.ID).toString();
//					System.out.println("targetEventId: " + targetEventId);
					if(eventReference.getOrderNum() > targetEvent.getOrderNum()){
						if(!precedenceEventIdList.contains(targetEventId)){
							precedenceEventIdList.add(targetEventId);
						}		
						if(!precedenceEventList.contains(targetEvent)){
							precedenceEventList.add(targetEvent);
						}	
					}																							
					else{
						if(!responseEventIdList.contains(targetEventId)){
							responseEventIdList.add(targetEventId);		
						}	
						if(!responseEventList.contains(targetEvent)){
							responseEventList.add(targetEvent);		
						}
					}	
				}
				
				eventVSPrecedenceEventList.put(sourceEventId, precedenceEventIdList);
				eventVSResponseEventList.put(sourceEventId, responseEventIdList);
				
				if(!activityRelation.checkActivityEdgeCardinality(precedenceEventIdList.size(), responseEventIdList.size())){	
//					System.out.println("activity constraint violaed");
//					System.out.println("precedenceEventIdList.size(): " + precedenceEventIdList.size());
//					System.out.println("responseEventIdList.size(): " + responseEventIdList.size());

					behavioralConstraintVSEventVSPrecedenceEventList.get(activityRelation).put(eventReference, precedenceEventList);
					behavioralConstraintVSEventVSResponseEventList.get(activityRelation).put(eventReference, responseEventList);
				}
				
			}	
			activityEdgeVSEventVSPrecedenceEventList.put(activityRelationId, eventVSPrecedenceEventList);
			activityEdgeVSEventVSResponseEventList.put(activityRelationId, eventVSResponseEventList);	
		}
	}
	
	private boolean checkFutureEvent(ActivityEdge activityRelation, XOCLog log, XOCEvent eventReference){
		//initializing the futureEventExistenceFlag is false, if detecting one future event, the value is set to true
		boolean futureEventExistenceFlag = false;
		// for any ef in E, judge if exist such an ef
		for(XOCEvent eventF : log.getAllEvents()){	 
			//initializing the fallEventDotSatisfiedFlag is true, if detecting one is not satisfied, the value is set to false
			if(checkAllEventsAfterFutureEvent(activityRelation, log, eventF, eventReference)) {
				futureEventExistenceFlag = true;
				break;
			}
		}  
		return futureEventExistenceFlag;
	}
	
	private boolean checkAllEventsAfterFutureEvent(ActivityEdge activityRelation, XOCLog log, XOCEvent eventF, XOCEvent eventReference){
		boolean allEventDotSatisfiedFlag = true;
		// for any e' in (E-after or equal ef), if there is one e' such that the constraint is violated, let existFlag = false;
		for(XOCEvent eventDot : filterEvents(log.getAllEvents(), eventF, null, 2)){   			
			if(!checkOneEventsAfterFutureEvent(activityRelation, log, eventDot, eventReference)){
				allEventDotSatisfiedFlag = false;
				break;
			}
		}
		return allEventDotSatisfiedFlag;
	}
	
	private boolean checkOneEventsAfterFutureEvent(ActivityEdge activityRelation, XOCLog log, XOCEvent eventDot, XOCEvent eventReference){
		boolean oneEventDotSatisfiedFlag = true;
		List<XOCEvent> Etar = computeEtar(activityRelation, log, eventDot, eventReference);	
		
		if(!activityRelation.checkActivityEdgeCardinality(filterEvents(Etar, eventReference, null, 3).size(),
				  filterEvents(Etar, eventReference, null, 4).size())){		
			oneEventDotSatisfiedFlag = false;
		}
		return oneEventDotSatisfiedFlag;
	}
	
	private boolean checkLastEvents(ActivityEdge activityRelation, XOCLog log, XOCEvent eventLast, XOCEvent eventReference){
		boolean lastEventSatisfiedFlag = true;
		List<XOCEvent> Etar = computeEtar(activityRelation, log, eventLast, eventReference);	
		
		if(!activityRelation.checkActivityEdgeCardinality(filterEvents(Etar, eventReference, null, 3).size(),
				  filterEvents(Etar, eventReference, null, 4).size())){		
			lastEventSatisfiedFlag = false;
		}
		return lastEventSatisfiedFlag;
	}
	
	private List<XOCEvent> computeTargetEventListByClass(ActivityEdge activityRelation, XOCLog log, XOCEvent eventReference){
		List<XOCEvent> targetEventList = new ArrayList<XOCEvent>();
		ClassNode classNode = activityRelation.getCorrelationOCMedia();
		String className = classNode.getNodeName();
		String targetActivityName = activityRelation.getTargetNode().getNodeName();
		
		String eventId = eventReference.getAttributes().get(xConstants.ID).toString();
//		System.out.println("eventId: " + eventId);
//		System.out.println("className: " + className);
//		System.out.println("targetActivityName: " + targetActivityName);
		
		
		
//		for(XOCObject relatedObject : filter.filterObjects(eventReference.getRelatedObjects(), className)){		
		for(XOCObject relatedObject : filter.filterObjectsByClassAtEvent(eventReference.getRelatedObjects(), className,log.getLastEvent())){	
			for(XOCEvent targetEvent : filter.filterEvents(relatedObject.getRelatedEventList(), null, targetActivityName, 5)){
//				System.out.println("getRelatedEventList");
				targetEventList.add(targetEvent);
				String targetEventId = targetEvent.getAttributes().get(xConstants.ID).toString();
//				System.out.println("add(targetEvent): " + targetEventId);
			}
		}	
//		System.out.println("targetEventList.size(): " + targetEventList.size());
		
		return targetEventList;			
	}
	
	private List<XOCEvent> computeTargetEventListByClassRelation(ActivityEdge activityRelation, XOCLog log, XOCEvent eventReference){
		List<XOCEvent> targetEventList = new ArrayList<XOCEvent>();
		XOCEvent lastEvent = log.getLastEvent();
//		System.out.println("kfgz activityRelation.getRelatedCrelRTRelations().size(): " + activityRelation.getRelatedCrelRTRelations().size());
		ClassEdge classRelation = activityRelation.getCorrelationRTMedia();
		String classRelationName = classRelation.getEdgeName();
		String sourceClassName = classRelation.getSourceNode().getNodeName();
		String targetClassName = classRelation.getTargetNode().getNodeName();	
		String targetActivityName = activityRelation.getTargetNode().getNodeName();
		
		Map<String, XOCObject> idVSObjectMap = log.getIdVSObjectMap();
		//get all objects of the source class of the class relation
		for(XOCObject referenceObject : filter.filterObjectsByClassAtEvent(eventReference.getRelatedObjects(), sourceClassName, lastEvent)){ 
			//get all object relations of one object of source class
			for(XOCORelation relatedObjectRelation : referenceObject.getRelatedObjectRelationList()){ 
				String relationType = relatedObjectRelation.getAttributes().get("relation").toString();
				//if the relation type of the object relation satisfies and the object relation is alive at the last event
				if(relationType.equals(classRelationName) && relatedObjectRelation.isAliveAtEvent(lastEvent)){
					String relatedObjectId = relatedObjectRelation.getAttributes().get("targetobjectid").toString();
					XOCObject relatedObject = idVSObjectMap.get(relatedObjectId);
					XOCAttribute classXOCAttribute = relatedObject.getXOCAttributes().get(xConstants.CLASS);
					if(classXOCAttribute != null){
						String classValueAtLastEvent = classXOCAttribute.getValueVersion(lastEvent.getOrderNum());
						if(classValueAtLastEvent != null && classValueAtLastEvent.equals(targetClassName)){
							for(XOCEvent targetEvent : filter.filterEvents(relatedObject.getRelatedEventList(), null, targetActivityName, 5)){
//								System.out.println("getRelatedEventList");
								targetEventList.add(targetEvent);
							}
						}
					}
				}				
			}
		}	
		
		//get all objects of the target class of the class relation
		for(XOCObject referenceObject : filter.filterObjectsByClassAtEvent(eventReference.getRelatedObjects(), targetClassName, lastEvent)){ 
			//get all object relations of one object of source class
			for(XOCORelation relatedObjectRelation : referenceObject.getRelatedObjectRelationList()){ 			
				String relationType = relatedObjectRelation.getAttributes().get("relation").toString();
				//if the relation type of the object relation satisfies and the object relation is alive at the last event
				if(relationType.equals(classRelationName) && relatedObjectRelation.isAliveAtEvent(lastEvent)){
					String relatedObjectId = relatedObjectRelation.getAttributes().get("sourceobjectid").toString();
					XOCObject relatedObject = idVSObjectMap.get(relatedObjectId);
					XOCAttribute classXOCAttribute = relatedObject.getXOCAttributes().get(xConstants.CLASS);
					if(classXOCAttribute != null){
						String classValueAtLastEvent = classXOCAttribute.getValueVersion(lastEvent.getOrderNum());
						if(classValueAtLastEvent != null && classValueAtLastEvent.equals(sourceClassName)){
							for(XOCEvent targetEvent : filter.filterEvents(relatedObject.getRelatedEventList(), null, targetActivityName, 5)){
//								System.out.println("getRelatedEventList");
								targetEventList.add(targetEvent);
							}
						}
					}
				}				
			}
		}	
		
//		for(XOCObject sourceObject : filter.filterObjects(eventReference.getRelatedObjects(), targetClassName)){			
////			System.out.println("sourceObjectId: " + sourceObjectId);
//			for(String relatedObjectIdOfSourceObject : sourceObject.getRelatedObjectIdList()){ 
//				XOCObject relatedObjectOfSourceObject = idVSObjectMap.get(relatedObjectIdOfSourceObject);
////				System.out.println("target candidate objectId: " + objectId);
//				String objectClass = relatedObjectOfSourceObject.getAttributes().get(xConstants.CLASS).toString();
////				System.out.println("target candidate objectClass: " + objectClass);
//				if(objectClass.equals(sourceClassName)){
////					System.out.println("is target");
////					System.out.println("getRelatedEventList().size: " + relatedObjectOfSourceObject.getRelatedEventList().size());			
//					for(XOCEvent targetEvent : filter.filterEvents(relatedObjectOfSourceObject.getRelatedEventList(), null, targetActivityName, 5)){
////						System.out.println("getRelatedEventList");
//						targetEventList.add(targetEvent);
//					}
//				}					
//			}
//		}	
		
		return targetEventList;
	
	}
	
	private List<XOCEvent> computeEtar(ActivityEdge activityRelation, XOCLog log, XOCEvent eventDot, XOCEvent eventReference){
		List<XOCEvent> Etar = new ArrayList<XOCEvent>();
		//for each target event in E-filter(c.target.activity)---(compute Etar)
		for(XOCEvent eventTarget :  filterEvents(log.getAllEvents(), null, activityRelation.getTargetNode(), 5)){	
			//if crel(c)=oc in OC
			if(activityRelation.getRelatedCrelOCRelations().size() == 1){
				Etar.addAll(computeOCEtar(activityRelation, eventDot, eventReference, eventTarget));
			}
			//if crel(c)=r in RT
			else if(activityRelation.getRelatedCrelRTRelations().size() == 1){
				Etar.addAll(computeRTEtar(activityRelation, eventDot, eventReference, eventTarget));
			}
		}	
		return Etar;
	}
	
	private List<XOCEvent> computeOCEtar(ActivityEdge activityRelation, XOCEvent eventDot, XOCEvent eventReference, XOCEvent eventTarget){
		List<XOCEvent> Etar = new ArrayList<XOCEvent>();
		// for any o in modelObjects(e')-filter(oc)
		ClassNode classNode = activityRelation.getRelatedCrelOCRelations().get(0).getRelatedClass();
		for(XOCObject object : filterObjects(eventDot.getObjectModel().getObjectSet(), classNode)){	
			if(eventReference.relatedObjectsContain(object) && eventTarget.relatedObjectsContain(object)){
				Etar.add(eventTarget);
				break;
			}
		}
		return Etar;
	}
	
	private List<XOCEvent> computeRTEtar(ActivityEdge activityRelation, XOCEvent eventDot, XOCEvent eventReference, XOCEvent eventTarget){
		List<XOCEvent> Etar = new ArrayList<XOCEvent>();
		//get r
		ClassEdge classRelation = activityRelation.getRelatedCrelRTRelations().get(0).getRelatedClassRelation();					
		//for any o1 in modelObjects(e')
		for(XOCObject objectReference : eventDot.getObjectModel().getObjectSet()){	
			//if o1 is related to event reference
			if(eventReference.relatedObjectsContain(objectReference)){
				//for any o2 in modelObjects(e')
				for(XOCObject objectTarget : eventDot.getObjectModel().getObjectSet()){
					//if o2 is related to target reference
					if(eventTarget.relatedObjectsContain(objectTarget)){
						//create two object relations
						XOCORelation objectRelationA = new XOCORelationImpl(classRelation.getCorrespondingCell().getUserObject().toString(), objectReference, objectTarget);
						XOCORelation objectRelationB = new XOCORelationImpl(classRelation.getCorrespondingCell().getUserObject().toString(), objectTarget, objectReference);
						//for each object relation in Rel(e')
						for(XOCORelation objectRelation : eventDot.getObjectModel().getObjectRelationSet()){	
							//if objectRelation equal to objectRelationA or objectRelationB
							if(objectRelation.compareObjectRelation(objectRelationA) || objectRelation.compareObjectRelation(objectRelationB)){											
								if(!Etar.contains(eventTarget))
									Etar.add(eventTarget);
							}
						}
					}
				}		    									
			}
		}
		return Etar;
	}
	
	private static List<XOCEvent> filterEvents(List<XOCEvent> inputEvents, XOCEvent eventReference, 
											  ActivityNode activityNode, int filterType){
		List<XOCEvent> outputEvents = new ArrayList<XOCEvent>();
		switch(filterType)
		{
			case 1:
				for(XOCEvent eventCandidate : inputEvents){
					if(eventCandidate.getOrderNum() <= eventReference.getOrderNum())						
						outputEvents.add(eventCandidate);
				}			
				break;
			case 2:
				for(XOCEvent eventCandidate : inputEvents){
					if(eventCandidate.getOrderNum() >= eventReference.getOrderNum())
						outputEvents.add(eventCandidate);
				}				
				break;
			case 3:
				for(XOCEvent eventCandidate : inputEvents){
					
					if(eventCandidate.getOrderNum() < eventReference.getOrderNum())					
						outputEvents.add(eventCandidate);
					
				}			
				break;
			case 4:
				for(XOCEvent eventCandidate : inputEvents){
					if(eventCandidate.getOrderNum() > eventReference.getOrderNum())
						outputEvents.add(eventCandidate);
				}
				break;
			case 5:
				for(XOCEvent eventCandidate : inputEvents){
					if(eventCandidate.getAttributes().get("activity").toString().equals(activityNode.getNodeName()))
						outputEvents.add(eventCandidate);
				}
				break;
		}
		return outputEvents;
	}
	

	public static List<XOCObject> filterObjects(List<XOCObject> inputObjects, ClassNode classNodeReference){
		List<XOCObject> outputObjects = new ArrayList<XOCObject>();
		for(XOCObject objectCandidate: inputObjects){
			if(objectCandidate.getAttributes().get("class").toString().equals(classNodeReference.getNodeName()))
				outputObjects.add(objectCandidate);
		}
		return outputObjects;
	}

//	public static void main(String[] str){
//		BehavioralConstraintsChecking
//	}
}