package org.processmining.ocbc.conformance.checkers;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.processmining.common.structure.IntegerPair;
import org.processmining.ocbc.discovery.filter.FilterFunction;
import org.processmining.ocbc.log.model.XOCAttribute;
import org.processmining.ocbc.log.model.XOCEvent;
import org.processmining.ocbc.log.model.XOCLog;
import org.processmining.ocbc.log.model.XOCObject;
import org.processmining.ocbc.log.model.impl.xConstants;
import org.processmining.ocbc.models.AOCEdge;
import org.processmining.ocbc.models.ActivityNode;
import org.processmining.ocbc.models.Cardinality;
import org.processmining.ocbc.models.ClassNode;
import org.processmining.ocbc.models.OCBCModel;

public class EventNumberPerObjectChecking {
	private FilterFunction filter = new  FilterFunction();
	private List<AOCEdge> violatedAOCRelationList = new ArrayList<AOCEdge>();
//	private Map<AOCEdge, Map<XOCObject, List<XOCEvent>>> violatedAOCRelationVSObjectVSMomentList = new HashMap<AOCEdge, Map<XOCObject, List<XOCEvent>>>();	
	/**
	 * record deviating objects for each AOC relation;
	 * note that, each object has a list of periods, which means object versions of one object during these periods are deviating;
	 */
	private Map<AOCEdge, Map<XOCObject, List<IntegerPair>>> violatedAOCRelationAlwaysVSObjectVSPeriodList = new HashMap<AOCEdge, Map<XOCObject, List<IntegerPair>>>();	
	private Map<AOCEdge, Map<XOCObject, List<IntegerPair>>> violatedAOCRelationEventuallyVSObjectVSPeriodList = new HashMap<AOCEdge, Map<XOCObject, List<IntegerPair>>>();	
	
//	private Map<AOCEdge, List<XOCObject>> violatedAOCRelationVSObjectList = new HashMap<AOCEdge, List<XOCObject>>();

	
			
	public List<AOCEdge> getViolatedAOCRelationList(){
		return violatedAOCRelationList;
	}
	
//	public Map<AOCEdge, Map<XOCObject, List<XOCEvent>>> getViolatedAOCRelationVSObjectVSMomentListMap(){
//		return violatedAOCRelationVSObjectVSMomentList;
//	}
	
	/**
	 * get the violatedAOCRelationAlwaysVSObjectVSPeriodList which records deviating objects for each AOC relation;
	 * note that, each object has a list of periods, which means object versions of one object during these periods are deviating;
	 * @return
	 */
	public Map<AOCEdge, Map<XOCObject, List<IntegerPair>>> getViolatedAOCRelationAlwaysVSObjectVSPeriodListMap(){
		return violatedAOCRelationAlwaysVSObjectVSPeriodList;
	}
	
	/**
	 * get the violatedAOCRelationEventuallyVSObjectVSPeriodList which records deviating objects for each AOC relation;
	 * note that, each object has a list of periods, which means object versions of one object during these periods are deviating;
	 * @return
	 */
	public Map<AOCEdge, Map<XOCObject, List<IntegerPair>>> getViolatedAOCRelationEventuallyVSObjectVSPeriodListMap(){
		return violatedAOCRelationEventuallyVSObjectVSPeriodList;
	}
	
//	public Map<AOCEdge, List<XOCObject>> getViolatedAOCRelationVSObjectListMap(){
//		return violatedAOCRelationVSObjectList;
//	}
	
	public void checking(XOCLog log, OCBCModel model){
		List<AOCEdge> aOCRelations = model.getAllAOCRelations();
		Map<String, XOCEvent> idVSEventMap = log.getIdVSEventMap();
		XOCEvent lastEvent = log.getLastEvent();
		List<XOCObject> completeObjectList = lastEvent.getObjectModel().getObjectSet();
		

		
		//for any (a,oc) in AOC
		for(AOCEdge aOCEdge : aOCRelations){
			violatedAOCRelationList.add(aOCEdge);
//			violatedAOCRelationVSObjectVSMomentList.put(aOCEdge, new HashMap<XOCObject, List<XOCEvent>>());
			violatedAOCRelationAlwaysVSObjectVSPeriodList.put(aOCEdge, new HashMap<XOCObject, List<IntegerPair>>());
			violatedAOCRelationEventuallyVSObjectVSPeriodList.put(aOCEdge, new HashMap<XOCObject, List<IntegerPair>>());
//			violatedAOCRelationVSObjectList.put(aOCEdge, new ArrayList<XOCObject>());
			ActivityNode activityNode = aOCEdge.getActivityNode();
			ClassNode classNode = aOCEdge.getClassNode();
			
			String activityName = activityNode.getNodeName();
			String className = classNode.getNodeName();
			Cardinality activityAlwaysCardinality = aOCEdge.getActivityAlwaysCardinality();	
			Cardinality activityEventuallyCardinality = aOCEdge.getActivityEventuallyCardinality();
			
//			System.out.println("check the always cardinality");
			//check the always cardinality on the activity side	
			for(XOCObject object : filter.filterObjects(completeObjectList, className)){
//				violatedAOCRelationVSObjectVSMomentList.get(aOCEdge).put(object, new ArrayList<XOCEvent>());

				List<XOCEvent> relatedEventListForAOCRelation = filter.filterEvents(object.getRelatedEventList(), null, activityName, 5);
				List<XOCEvent> checkingMomentList = new ArrayList<XOCEvent>();
				//add checking moment into the list
				List<Integer> checkingEventOrderNumList = new ArrayList<Integer>();	
				
				//add checking moment into the list
				for(IntegerPair period : object.getLifecyclePeriodList()){
					int startValue = period.getFirstValue();
					checkingMomentList.add(log.get(startValue-1));
					checkingEventOrderNumList.add(startValue);
				}
				//event after the moment of object birth
				for(XOCEvent event : relatedEventListForAOCRelation){
					if(!checkingMomentList.contains(event)){
						checkingMomentList.add(event);	
						checkingEventOrderNumList.add(event.getOrderNum());
					}									
				}
				
				//create the deviating period list which violate the always cardinality on the activity side	
				List<IntegerPair> deviatingPeriodListAlways = createObjectDeviatingPeriodList(object, checkingEventOrderNumList, 
						relatedEventListForAOCRelation, activityAlwaysCardinality, xConstants.source_always_cardinality);
				//create the deviating period list which violate the eventually cardinality on the activity side	
				List<IntegerPair> deviatingPeriodListEventually = createObjectDeviatingPeriodList(object, checkingEventOrderNumList, 
						relatedEventListForAOCRelation, activityEventuallyCardinality, xConstants.source_eventually_cardinality);	
				
				//filter the deviating period list with class value period
				XOCAttribute classXOCAttribute = object.getXOCAttributes().get(xConstants.CLASS);
				List<IntegerPair> survivalPeriodList = classXOCAttribute.getSurvivalPeriodListOfValue(className);			
				List<IntegerPair> filteredDeviatingPeriodListAlways = filter.filterNonOverlappingPeriodParts(deviatingPeriodListAlways, survivalPeriodList, log.size());
				if(filteredDeviatingPeriodListAlways.size() > 0){
					violatedAOCRelationAlwaysVSObjectVSPeriodList.get(aOCEdge).put(object, filteredDeviatingPeriodListAlways);
				}			
				List<IntegerPair> filteredDeviatingPeriodListEventually = filter.filterNonOverlappingPeriodParts(deviatingPeriodListEventually, survivalPeriodList, log.size());
				if(filteredDeviatingPeriodListEventually.size() > 0){
					violatedAOCRelationEventuallyVSObjectVSPeriodList.get(aOCEdge).put(object, filteredDeviatingPeriodListEventually);
				}	
			}
			

			//check the eventually cardinality on the activity side	
//			for(XOCObject object : filter.filterObjectsByClassAtEvent(completeObjectList, className, log.getLastEvent())){
//				int constraintValue = filter.filterEvents(object.getRelatedEventList(), null, activityName, 5).size();
////				System.out.println("constraintValue: " + constraintValue);
//				//if the cardinality is violated, record the object
//				if(!activityEventuallyCardinality.checkCardinality(constraintValue)){
////					System.out.println("violated: " + constraintValue);
//					violatedAOCRelationVSObjectList.get(aOCEdge).add(object);
//				}						
//			}		
			
//			if(violatedAOCRelationVSObjectVSMomentList.get(aOCEdge).size() == 0 &&
//					violatedAOCRelationVSObjectList.get(aOCEdge).size() == 0){
//				violatedAOCRelationList.remove(aOCEdge);
//			}
			if(violatedAOCRelationAlwaysVSObjectVSPeriodList.get(aOCEdge).size() == 0 &&
					violatedAOCRelationEventuallyVSObjectVSPeriodList.get(aOCEdge).size() == 0){
				violatedAOCRelationList.remove(aOCEdge);
			}	
		}
	}
	
	/**
	 * check if an object version (at one checking moment, i.e., the object version's birth moment) 
	 * has the required number of corresponding events at each latter moment (after or equal to its birth moment);
	 * @param object
	 * @param objectVersionMoment
	 * @param checkingMomentList
	 * @param relatedEventList
	 * @param activityAlwaysCardinality
	 * @return
	 */
	public boolean isEventNumberCorrectAtLatterMomentsForObjectVersion(XOCObject object, int objectVersionMoment, List<Integer> checkingMomentList,
			List<XOCEvent> relatedEventList, Cardinality activityAlwaysCardinality){
		for(int checkingMoment : checkingMomentList){
			if(checkingMoment >= objectVersionMoment){
				int constraintValue = 0;
				for(XOCEvent event : relatedEventList){
					int relatedEventOrderNum = event.getOrderNum();	
					//if the event is alive at the checking moment
					if(relatedEventOrderNum <= checkingMoment){
						constraintValue ++;
					}	
				}
				//if the cardinality is violated some moment, return false
				if(!activityAlwaysCardinality.checkCardinality(constraintValue)){
					return false;
				}	
			}
		}
		//if the event number is correct at each checking moment, return true;
		return true;
	}
	
	/**
	 * check if an object version (at one checking moment, i.e., the object version's birth moment) 
	 * has the required number of corresponding events at the last moment (corresponding to the last event in a log);
	 * currently, we consider all event in relatedEventList are corresponding events (because all events are before or equal the last event);
	 * @param object
	 * @param objectVersionMoment
	 * @param checkingMomentList
	 * @param relatedEventList
	 * @param activityAlwaysCardinality
	 * @return
	 */
	public boolean isEventNumberCorrectAtLastMomentForObjectVersion(XOCObject object, int objectVersionMoment, List<Integer> checkingMomentList,
			List<XOCEvent> relatedEventList, Cardinality activityEventuallyCardinality){
		int constraintValue = relatedEventList.size();
		//if the cardinality is violated at the last moment, return false
		if(!activityEventuallyCardinality.checkCardinality(constraintValue)){
			return false;
		}	
		//if the event number is correct at the last moment, return true;
		else{
			return true;
		}
		
		
	}
	
	/**
	 * create the deviating period for an object based on the checking moment list, related event list and cardinality;
	 * @param object
	 * @param checkingMomentList
	 * @param relatedEventList
	 * @param activityAlwaysCardinality
	 * @return
	 */
	public List<IntegerPair> createObjectDeviatingPeriodList(XOCObject object, List<Integer> checkingMomentList, 
			List<XOCEvent> relatedEventList, Cardinality cardinality, String cardinalityType){
		List<IntegerPair> deviatingPeriodList = new ArrayList<IntegerPair>();
		
		//sort the list from small to large
		Collections.sort(checkingMomentList);
		boolean objectIsDeviatingFlag = false;
		//check one object version
		for(int checkingMoment : checkingMomentList){
			boolean objectVersionNormalFlag = false;
			if(cardinalityType.equals(xConstants.source_always_cardinality)){
				objectVersionNormalFlag = isEventNumberCorrectAtLatterMomentsForObjectVersion(object, checkingMoment, checkingMomentList,
						relatedEventList, cardinality);
			}
			else if(cardinalityType.equals(xConstants.source_eventually_cardinality)){
				objectVersionNormalFlag = isEventNumberCorrectAtLastMomentForObjectVersion(object, checkingMoment, checkingMomentList,
						relatedEventList, cardinality);
			}				
			//if the object version is deviating
			if(!objectVersionNormalFlag){
				//if the object is normal before the object version, i.e., the object change to deviating
				if(!objectIsDeviatingFlag){
					deviatingPeriodList.add(new IntegerPair(checkingMoment, null));
				}
				//if the object is deviating before the object version, do nothing
				else{
					
				}
				objectIsDeviatingFlag = true;
			}
			//if the object version is normal
			else{			
				//if the object is deviating before the object version, i.e., the object change to normal
				if(objectIsDeviatingFlag){
					deviatingPeriodList.get(deviatingPeriodList.size()-1).setSecondValue(checkingMoment-1);
				}
				//if the object is normal previous, do nothing
				else{
					
				}
				objectIsDeviatingFlag = false;
			}
		}
		return deviatingPeriodList;
	}
}