package org.processmining.ocbc.conformance.checkers;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.processmining.ocbc.log.model.XOCAttribute;
import org.processmining.ocbc.log.model.XOCEvent;
import org.processmining.ocbc.log.model.XOCLog;
import org.processmining.ocbc.log.model.XOCORelation;
import org.processmining.ocbc.log.model.XOCObject;
import org.processmining.ocbc.log.model.impl.xConstants;
import org.processmining.ocbc.models.Cardinality;
import org.processmining.ocbc.models.ClassEdge;
import org.processmining.ocbc.models.ClassNode;
import org.processmining.ocbc.models.OCBCModel;



public class FulfilmentChecking {
//	private Map<ClassEdge, Map<XOCObject, List<XOCObject>>> violatedSourceClassRelationVSObjectVSRelatedObjectList = new HashMap<ClassEdge, Map<XOCObject, List<XOCObject>>>();
//	private Map<ClassEdge, Map<XOCObject, List<XOCObject>>> violatedTargetClassRelationVSObjectVSRelatedObjectList = new HashMap<ClassEdge, Map<XOCObject, List<XOCObject>>>();
	 
	private List<ClassEdge> violatedClassRelationList = new ArrayList<ClassEdge>();
	//objects who violate the source always cardinality of class relations
	private Map<ClassEdge, List<XOCObject>> violatedSourceClassRelationVSObjectList = new HashMap<ClassEdge, List<XOCObject>>();
	private Map<ClassEdge, List<XOCObject>> violatedTargetClassRelationVSObjectList = new HashMap<ClassEdge, List<XOCObject>>();
	
	private XOCLog log;
	
//	public Map<ClassEdge, Map<XOCObject, List<XOCObject>>> getViolatedSourceClassRelationVSObjectVSRelatedObjectListMap(){
//		return violatedSourceClassRelationVSObjectVSRelatedObjectList;
//	}
	
//	public Map<ClassEdge, Map<XOCObject, List<XOCObject>>> getViolatedTargetClassRelationVSObjectVSRelatedObjectListMap(){
//		return violatedTargetClassRelationVSObjectVSRelatedObjectList;
//	}
	
	public Map<ClassEdge, List<XOCObject>> getViolatedSourceClassRelationVSObjectListMap(){
		return violatedSourceClassRelationVSObjectList;
	}
	
	public Map<ClassEdge, List<XOCObject>> getViolatedTargetClassRelationVSObjectListMap(){
		return violatedTargetClassRelationVSObjectList;
	}	
	
	public List<ClassEdge>  getViolatedClassRelationList(){
		return violatedClassRelationList;		
	}	

	public void checking(XOCLog log, OCBCModel model){
		this.log = log;
		XOCEvent lastEvent = log.getLastEvent();		
		List<ClassEdge> classRelations = model.getAllClassRelations();		
		
		for(ClassEdge classRelation : classRelations){
			String classRelationType = classRelation.getEdgeName();
//			System.out.println("classRelationType: " + classRelationType);
			
			violatedSourceClassRelationVSObjectList.put(classRelation, new ArrayList<XOCObject>());
			violatedTargetClassRelationVSObjectList.put(classRelation, new ArrayList<XOCObject>());
			
			ClassNode classEdgeSourceNode = classRelation.getSourceNode();
			ClassNode classEdgeTargetNode = classRelation.getTargetNode();
				
			String sourceClassName = classEdgeSourceNode.getNodeName();
			String targetClassName = classEdgeTargetNode.getNodeName();
			
			Cardinality sourceEventuallyCardinality = classRelation.getSourceEventuallyCardinality();	
			Cardinality targetEventuallyCardinality = classRelation.getTargetEventuallyCardinality();				
			
//			for(XOCObject object : lastEvent.getObjectModel().getObjectSet()){
			for(XOCObject object : log.getObjectList()){
				XOCAttribute classXOCAttribute = object.getXOCAttributes().get(xConstants.CLASS);
				//if the object once exists
				if(classXOCAttribute != null){
//					List<String> objectClassValueList = object.getXOCAttributes().get(xConstants.CLASS).getValueList();
					String classValueAtLastEvent = classXOCAttribute.getValueVersion(log.size());
					//only checking the alive objects at the last moment
					if(classValueAtLastEvent != null){
						boolean satisfyingFlag = false;
						if(classValueAtLastEvent.equals(targetClassName)){
							satisfyingFlag = checkIfESatisfyingConstraintForOneObject(object, classRelationType, sourceEventuallyCardinality);
							if(!satisfyingFlag){
	//							String objectId = object.getAttributes().get(xConstants.ID).toString();
	//							System.out.println("objectId in not satisfying: " + objectId);
								violatedSourceClassRelationVSObjectList.get(classRelation).add(object);
							}	
						}
						else if(classValueAtLastEvent.equals(sourceClassName)){
							satisfyingFlag = checkIfESatisfyingConstraintForOneObject(object, classRelationType, targetEventuallyCardinality);			
							if(!satisfyingFlag){
								violatedTargetClassRelationVSObjectList.get(classRelation).add(object);
							}	
						}	
					}
				}							
			}
			
			if(violatedSourceClassRelationVSObjectList.get(classRelation).size() > 0 ||
					violatedTargetClassRelationVSObjectList.get(classRelation).size() > 0){
				violatedClassRelationList.add(classRelation);
			}	
			if(violatedSourceClassRelationVSObjectList.get(classRelation).size() == 0){
				violatedSourceClassRelationVSObjectList.remove(classRelation);
			}
			if(violatedTargetClassRelationVSObjectList.get(classRelation).size() == 0){
				violatedTargetClassRelationVSObjectList.remove(classRelation);
			}
	
		}
	}
	
	/**
	 * check if one object eventually satisfies its corresponding constraint based on its related object relations
	 * @param object
	 * @param objectRelationType
	 * @param cardinality
	 * @return true if satisfying, false otherwise
	 */
	private boolean checkIfESatisfyingConstraintForOneObject(XOCObject object, String objectRelationType, Cardinality cardinality){
		boolean checkConstraintFlag = true;		
		int constraintValue = 0;
//		String objectId = object.getAttributes().get(xConstants.ID).toString();
//		System.out.println("objectId in checkIfESatisfyingConstraintForOneObject: " + objectId);
//		System.out.println("cardinality: " + cardinality.toString());
		
		for(XOCORelation objectRelation : object.getRelatedObjectRelationList()){
			String relationType = objectRelation.getRelationType();
//			XOCEvent birthEvent = objectRelation.getBirthEvent();
//			XOCEvent deathEvent = objectRelation.getDeathEvent();
			
//			System.out.println("objectRelation.getRelationType(): " + objectRelation.getRelationType());
//			System.out.println("objectRelationType: " + objectRelationType);
		
			//if the object relation is alive at the last event and satisfy the relation type
			if(objectRelation.isAliveAtEvent(log.getLastEvent()) && objectRelationType.equals(relationType)){
//				System.out.println("constraintValue equal: " + constraintValue);
				constraintValue ++;
			}						
		}
		
//		System.out.println("constraintValue: " + constraintValue);
		//if the cardinality is violated, record the object
		if(!cardinality.checkCardinality(constraintValue)){
			checkConstraintFlag = false;
		}
		return checkConstraintFlag;
	}
}