package org.processmining.ocbc.conformance.checkers;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.processmining.common.structure.IntegerPair;
import org.processmining.framework.util.Pair;
import org.processmining.ocbc.log.model.XOCAttribute;
import org.processmining.ocbc.log.model.XOCEvent;
import org.processmining.ocbc.log.model.XOCLog;
import org.processmining.ocbc.log.model.XOCORelation;
import org.processmining.ocbc.log.model.XOCObject;
import org.processmining.ocbc.log.model.XOCObjectModel;
import org.processmining.ocbc.log.model.impl.XOCORelationImpl;
import org.processmining.ocbc.log.model.impl.xConstants;
import org.processmining.ocbc.models.Cardinality;
import org.processmining.ocbc.models.ClassEdge;
import org.processmining.ocbc.models.ClassNode;
import org.processmining.ocbc.models.OCBCModel;



public class ObjectModelValidityChecking {
	private List<XOCEvent> deviatingEventList = new ArrayList<XOCEvent>();	//record events which have invalid object model
	private Map<XOCEvent, List<XOCORelation>>  eventVSDeviatingObjectRelationList = new HashMap<XOCEvent, List<XOCORelation>>();
	private Map<XOCORelation, List<String>> deviatingObjectRelationVSContentList = new HashMap<XOCORelation, List<String>>();
	//record the deviating object relations with its deviating contents at deviating moment (event);
	//in this way, it maybe takes a lot of memory;
	//in future, we can use Map<XOCORelation, Map<String, List<Pair<Integer, Integer>>>> relationVSContentVSDeviatingPeriodList
	private Map<XOCEvent, Map<XOCORelation, List<String>>>  eventVSDeviatingObjectRelationVSContentList = new HashMap<XOCEvent, Map<XOCORelation, List<String>>>();
	
	//record the deviating class relations with its deviating objects at deviating moment (event);
	//in this way, it maybe takes a lot of memory;
	//in future, we can use Map<XOCORelation, Map<String, List<Pair<Integer, Integer>>>> relationVSContentVSDeviatingPeriodList
	private Map<XOCEvent, Map<ClassEdge, List<XOCObject>>>  eventVSDeviatingClassRelationVSSourceObjectList = new HashMap<XOCEvent, Map<ClassEdge, List<XOCObject>>>();
	//record the deviating target objects in terms of the class relation
	private Map<XOCEvent, Map<ClassEdge, List<XOCObject>>>  eventVSDeviatingClassRelationVSTargetObjectList = new HashMap<XOCEvent, Map<ClassEdge, List<XOCObject>>>();
	
	private Map<XOCObject, List<Pair<Integer, Integer>>> deviatingObjectVSStartAndEndPairList = new HashMap<XOCObject, List<Pair<Integer, Integer>>>();	
	private Map<ClassEdge, Map<XOCObject, List<XOCObject>>> violatedSourceClassRelationVSObjectVSRelatedObjectList = new HashMap<ClassEdge, Map<XOCObject, List<XOCObject>>>();
	private Map<ClassEdge, Map<XOCObject, List<XOCObject>>> violatedTargetClassRelationVSObjectVSRelatedObjectList = new HashMap<ClassEdge, Map<XOCObject, List<XOCObject>>>();
	
	private Map<XOCEvent, List<ClassEdge>>  eventVSDeviatingClassRelationList = new HashMap<XOCEvent, List<ClassEdge>>();
	
	private Map<ClassEdge, List<Pair<Integer, Integer>>> violatedClassRelationVSStartAndEndPairList = new HashMap<ClassEdge, List<Pair<Integer, Integer>>>();	
	
	//objects who violate the source always cardinality of class relations
	private Map<ClassEdge, List<XOCObject>> violatedSourceClassRelationVSObjectList = new HashMap<ClassEdge, List<XOCObject>>();
	private Map<ClassEdge, List<XOCObject>> violatedTargetClassRelationVSObjectList = new HashMap<ClassEdge, List<XOCObject>>();
	
	private Map<ClassEdge, Map<XOCObject, List<Pair<Integer, Integer>>>> violatedSourceClassRelationVSObjectVSPeriodList = new HashMap<ClassEdge, Map<XOCObject, List<Pair<Integer, Integer>>>>();
	private Map<ClassEdge, Map<XOCObject, List<Pair<Integer, Integer>>>> violatedTargetClassRelationVSObjectVSPeriodList = new HashMap<ClassEdge, Map<XOCObject, List<Pair<Integer, Integer>>>>();
	
	private XOCLog log;
	
	public List<XOCEvent> getDeviatingEventList(){
		return deviatingEventList;
	}
	
	public Map<XOCEvent, List<XOCORelation>> getEventVSDeviatingObjectRelationListMap(){
		return eventVSDeviatingObjectRelationList;
	}
	
	public Map<XOCORelation, List<String>> getDeviatingObjectRelationVSContentListMap(){
		return deviatingObjectRelationVSContentList;
	}
	
	public Map<XOCEvent, Map<XOCORelation, List<String>>> getEventVSDeviatingObjectRelationVSContentListMap(){
		return eventVSDeviatingObjectRelationVSContentList;
	}	
	
	public Map<XOCObject, List<Pair<Integer, Integer>>> getDeviatingObjectVSStartAndEndPairListMap(){
		return deviatingObjectVSStartAndEndPairList;
	}
	
	public Map<ClassEdge, Map<XOCObject, List<XOCObject>>> getViolatedSourceClassRelationVSObjectVSRelatedObjectListMap(){
		return violatedSourceClassRelationVSObjectVSRelatedObjectList;
	}
	
	public Map<ClassEdge, Map<XOCObject, List<XOCObject>>> getViolatedTargetClassRelationVSObjectVSRelatedObjectListMap(){
		return violatedTargetClassRelationVSObjectVSRelatedObjectList;
	}
	
	public Map<ClassEdge, List<XOCObject>> getViolatedSourceClassRelationVSObjectListMap(){
		return violatedSourceClassRelationVSObjectList;
	}
	
	public Map<ClassEdge, List<XOCObject>> getViolatedTargetClassRelationVSObjectListMap(){
		return violatedTargetClassRelationVSObjectList;
	}
	
	public Map<ClassEdge, List<Pair<Integer, Integer>>>  getViolatedClassRelationVSStartAndEndPairListMap(){
		return violatedClassRelationVSStartAndEndPairList;
	}	
	
	public Map<XOCEvent, Map<ClassEdge, List<XOCObject>>>  getEventVSDeviatingClassRelationVSSourceObjectListMap(){
		return eventVSDeviatingClassRelationVSSourceObjectList;
	}	
	
	public Map<XOCEvent, Map<ClassEdge, List<XOCObject>>>  getEventVSDeviatingClassRelationVSTargetObjectListMap(){
		return eventVSDeviatingClassRelationVSTargetObjectList;
	}
	
	public void checking(XOCLog log, OCBCModel model){	
		this.log = log;
		Map<String, XOCEvent> idVSEventMap = log.getIdVSEventMap();
		List<ClassEdge> classRelations = model.getAllClassRelations();
		
		model.generateTypeVSClassRelationMap();
		Map<String, ClassEdge> typeVSClassRelationMap = model.getTypeVSClassRelationMap();
		
		//check if the object relations are valid
		for(XOCORelation objectRelation : log.getObjectRelationList()){
			deviatingObjectRelationVSContentList.put(objectRelation, new ArrayList<String>());
			String relationType = objectRelation.getRelationType();
			XOCObject sourceObject = objectRelation.getSourceObject();
			XOCObject targetObject = objectRelation.getTargetObject();
			System.out.println("objectRelation id: " + objectRelation.getAttributes().get(xConstants.ID).toString());
			System.out.println("sourceObject id: " + sourceObject.getAttributes().get(xConstants.ID).toString());
			System.out.println("targetObject id: " + targetObject.getAttributes().get(xConstants.ID).toString());		
			
			String sourceObjectId = sourceObject.getAttributes().get(xConstants.ID).toString();
			String targetObjectId  = targetObject.getAttributes().get(xConstants.ID).toString();		
			
			ClassEdge classRelation = typeVSClassRelationMap.get(relationType);
			if(classRelation != null){
				String sourceClassOfRelation = classRelation.getSourceNode().getNodeName();
				String targetClassOfRelation = classRelation.getTargetNode().getNodeName();
				
				//check all versions of source object
				List<String> sourceObjectClassValueList = sourceObject.getXOCAttributes().get(xConstants.CLASS).getValueList();
				for(String classValue : sourceObjectClassValueList){
					if(!sourceClassOfRelation.equals(classValue)){
						List<Pair<Integer, Integer>> valueSurvivalPeriodList = sourceObject.getXOCAttributes().get(xConstants.CLASS).getSurvivalPeriodListForValue(classValue);
						for(Pair<Integer, Integer> survivalPeriod : valueSurvivalPeriodList){
							int startEventIndex = survivalPeriod.getFirst()-1;
							int endEventIndex = survivalPeriod.getSecond() != null ? survivalPeriod.getSecond()-1 : log.size()-1;
							for(int i=startEventIndex; i<=endEventIndex; i++){
								XOCEvent deviatingEvent = log.get(i);
								if(!eventVSDeviatingObjectRelationVSContentList.containsKey(deviatingEvent)){
									eventVSDeviatingObjectRelationVSContentList.put(deviatingEvent, new HashMap<XOCORelation, List<String>>());	
								}			
								if(!eventVSDeviatingObjectRelationVSContentList.get(deviatingEvent).containsKey(objectRelation)){
									eventVSDeviatingObjectRelationVSContentList.get(deviatingEvent).put(objectRelation, new ArrayList<String>());
								}
								eventVSDeviatingObjectRelationVSContentList.get(deviatingEvent).get(objectRelation).add(xConstants.SOURCEOBJECTID + ": " + sourceObjectId);
							}
						}

					}
				}
				//check all versions of target object
				List<String> targetObjectClassValueList = targetObject.getXOCAttributes().get(xConstants.CLASS).getValueList();
				for(String classValue : targetObjectClassValueList){
					if(!targetClassOfRelation.equals(classValue)){
						List<Pair<Integer, Integer>> valueSurvivalPeriodList = targetObject.getXOCAttributes().get(xConstants.CLASS).getSurvivalPeriodListForValue(classValue);
						for(Pair<Integer, Integer> survivalPeriod : valueSurvivalPeriodList){
							int startEventIndex = survivalPeriod.getFirst()-1;
							int endEventIndex = survivalPeriod.getSecond() != null ? survivalPeriod.getSecond()-1 : log.size()-1;
							for(int i=startEventIndex; i<=endEventIndex; i++){
								XOCEvent deviatingEvent = log.get(i);
								if(!eventVSDeviatingObjectRelationVSContentList.containsKey(deviatingEvent)){
									eventVSDeviatingObjectRelationVSContentList.put(deviatingEvent, new HashMap<XOCORelation, List<String>>());	
								}			
								if(!eventVSDeviatingObjectRelationVSContentList.get(deviatingEvent).containsKey(objectRelation)){
									eventVSDeviatingObjectRelationVSContentList.get(deviatingEvent).put(objectRelation, new ArrayList<String>());
								}
								eventVSDeviatingObjectRelationVSContentList.get(deviatingEvent).get(objectRelation).add(xConstants.TARGETOBJECTID + ": " + targetObjectId);
							}
						}

					}
				}
			}
			//if the relation type is deviating (inexistent in the class model)
			else{
				System.out.println("the relation type is deviating, relationType: " + relationType);
				System.out.println("the relation type is deviating, id: " + objectRelation.getAttributes().get(xConstants.ID).toString());
				deviatingObjectRelationVSContentList.get(objectRelation).add(xConstants.relationship_type + ": " + relationType);
				for(IntegerPair period : objectRelation.getLifecyclePeriodList()){
					int startEventOrderNum = period.getFirstValue();
					int endEventOrderNum = period.getSecondValue() == null ? log.size() : period.getSecondValue();
					for(int i=startEventOrderNum; i<=endEventOrderNum; i++){
						XOCEvent deviatingEvent = log.get(i-1);
						if(!eventVSDeviatingObjectRelationVSContentList.containsKey(deviatingEvent)){
							eventVSDeviatingObjectRelationVSContentList.put(deviatingEvent, new HashMap<XOCORelation, List<String>>());	
						}					
						eventVSDeviatingObjectRelationVSContentList.get(deviatingEvent).put(objectRelation, new ArrayList<String>());
						eventVSDeviatingObjectRelationVSContentList.get(deviatingEvent).get(objectRelation).add(xConstants.relationship_type + ": " + relationType);
					}
				}
			}		
		}		
		
		//record the object (which violates the source cardinality of the class relation) and its related object list 
		for(ClassEdge classRelation : classRelations){
			violatedSourceClassRelationVSObjectList.put(classRelation, new ArrayList<XOCObject>());
			violatedTargetClassRelationVSObjectList.put(classRelation, new ArrayList<XOCObject>());
			violatedClassRelationVSStartAndEndPairList.put(classRelation, new ArrayList<Pair<Integer, Integer>>());
			
			violatedSourceClassRelationVSObjectVSPeriodList.put(classRelation, new HashMap<XOCObject, List<Pair<Integer, Integer>>>());
			violatedTargetClassRelationVSObjectVSPeriodList.put(classRelation, new HashMap<XOCObject, List<Pair<Integer, Integer>>>());
			
			ClassNode classEdgeSourceNode = classRelation.getSourceNode();
			ClassNode classEdgeTargetNode = classRelation.getTargetNode();
				
			String relationType = classRelation.getEdgeName();
			String sourceClassName = classEdgeSourceNode.getNodeName();
			String targetClassName = classEdgeTargetNode.getNodeName();
			
			Cardinality sourceAlwaysCardinality = classRelation.getSourceAlwaysCardinality();	
			Cardinality targetAlwaysCardinality = classRelation.getTargetAlwaysCardinality();				
			
			for(XOCObject object : log.getObjectList()){
				Cardinality alwaysCardinality = null;
				//if the object has the class attribute
				XOCAttribute classXOCAttribute = object.getXOCAttributes().get(xConstants.CLASS);
				if(classXOCAttribute != null){
					//check the "always" cardinality on the source side				
					List<String> objectClassValueList = classXOCAttribute.getValueList();
					for(String classValue : objectClassValueList){
						List<Pair<Integer, Integer>> valueSurvivalPeriodList = classXOCAttribute.getSurvivalPeriodListForValue(classValue);
						if(classValue.equals(targetClassName)){
							alwaysCardinality = sourceAlwaysCardinality;
							List<Pair<Integer, Integer>> deviatingStartAndEndPairList = checkIfASatisfyingConstraintForOneObject(object, 
									relationType, alwaysCardinality, idVSEventMap, valueSurvivalPeriodList);	
							if(deviatingStartAndEndPairList.size() > 0){
								violatedSourceClassRelationVSObjectList.get(classRelation).add(object);
								deviatingObjectVSStartAndEndPairList.put(object, deviatingStartAndEndPairList);
								violatedClassRelationVSStartAndEndPairList.get(classRelation).addAll(deviatingStartAndEndPairList);
								
								violatedSourceClassRelationVSObjectVSPeriodList.get(classRelation).put(object, new ArrayList<Pair<Integer, Integer>>());
								violatedSourceClassRelationVSObjectVSPeriodList.get(classRelation).get(object).addAll(deviatingStartAndEndPairList);
								
								for(Pair<Integer, Integer> period : deviatingStartAndEndPairList){
									int startValue = period.getFirst();
									int secondValue = period.getSecond();
									System.out.println("startValue in targetClassName: " + startValue);
									System.out.println("secondValue in targetClassName: " + secondValue);
									System.out.println("object id in targetClassName: " + object.getAttributes().get(xConstants.ID).toString());
									System.out.println("classRelation id in targetClassName: " + classRelation.getID());
									for(int eventOrderNum=startValue; eventOrderNum<=secondValue; eventOrderNum++){
										XOCEvent event = log.get(eventOrderNum-1);
										if(!eventVSDeviatingClassRelationVSTargetObjectList.containsKey(event)){
											eventVSDeviatingClassRelationVSTargetObjectList.put(event, new HashMap<ClassEdge, List<XOCObject>>());
										}
										if(!eventVSDeviatingClassRelationVSTargetObjectList.get(event).containsKey(classRelation)){
											eventVSDeviatingClassRelationVSTargetObjectList.get(event).put(classRelation, new ArrayList<XOCObject>());
										}
										//no need to detect if containing the object (should not contain)
										eventVSDeviatingClassRelationVSTargetObjectList.get(event).get(classRelation).add(object);
									}
									
								}
							}
						}
						else if(classValue.equals(sourceClassName)){
							alwaysCardinality = targetAlwaysCardinality;
							List<Pair<Integer, Integer>> deviatingStartAndEndPairList = checkIfASatisfyingConstraintForOneObject(object, 
									relationType, alwaysCardinality, idVSEventMap, valueSurvivalPeriodList);	
							if(deviatingStartAndEndPairList.size() > 0){
								violatedTargetClassRelationVSObjectList.get(classRelation).add(object);
								deviatingObjectVSStartAndEndPairList.put(object, deviatingStartAndEndPairList);
								violatedClassRelationVSStartAndEndPairList.get(classRelation).addAll(deviatingStartAndEndPairList);
								
								violatedTargetClassRelationVSObjectVSPeriodList.get(classRelation).put(object, new ArrayList<Pair<Integer, Integer>>());
								violatedTargetClassRelationVSObjectVSPeriodList.get(classRelation).get(object).addAll(deviatingStartAndEndPairList);	
								
								for(Pair<Integer, Integer> period : deviatingStartAndEndPairList){
									int startValue = period.getFirst();
									int secondValue = period.getSecond();
									System.out.println("startValue in sourceClassName: " + startValue);
									System.out.println("secondValue in sourceClassName: " + secondValue);
									System.out.println("object id in sourceClassName: " + object.getAttributes().get(xConstants.ID).toString());
									System.out.println("classRelation id in sourceClassName: " + classRelation.getID());
									for(int eventOrderNum=startValue; eventOrderNum<=secondValue; eventOrderNum++){
										XOCEvent event = log.get(eventOrderNum-1);
										if(!eventVSDeviatingClassRelationVSSourceObjectList.containsKey(event)){
											eventVSDeviatingClassRelationVSSourceObjectList.put(event, new HashMap<ClassEdge, List<XOCObject>>());
										}
										if(!eventVSDeviatingClassRelationVSSourceObjectList.get(event).containsKey(classRelation)){
											eventVSDeviatingClassRelationVSSourceObjectList.get(event).put(classRelation, new ArrayList<XOCObject>());
										}
										//no need to detect if containing the object (should not contain)
										eventVSDeviatingClassRelationVSSourceObjectList.get(event).get(classRelation).add(object);
									}
									
								}
							}	
						}
					}
				}							
			}
			if(violatedSourceClassRelationVSObjectList.get(classRelation).size() == 0){
				violatedSourceClassRelationVSObjectList.remove(classRelation);
			}
			if(violatedTargetClassRelationVSObjectList.get(classRelation).size() == 0){
				violatedTargetClassRelationVSObjectList.remove(classRelation);
			}
			if(violatedClassRelationVSStartAndEndPairList.get(classRelation).size() == 0){
				violatedClassRelationVSStartAndEndPairList.remove(classRelation);
			}
		}	
		//generate the deviation display result based on the conformance checking result
		generateDeviationDisplayResult(log);
	}
	
	/**
	 * generate the deviation display result based on the conformance checking result
	 * (1)deviatingEventList: record all deviating events;
	 * (2)eventVSDeviatingObjectRelationList: record all deviating object relations for each event;
	 */
	public void generateDeviationDisplayResult(XOCLog log){		
		Set<Integer> deviatingEventIndexSet = new HashSet<Integer>();

		for(XOCEvent event : eventVSDeviatingObjectRelationVSContentList.keySet()){
			deviatingEventIndexSet.add(event.getOrderNum());
		}
		
		
		
//		//generate the result for display: OM(event)->deviating object relation->...
//		for(XOCEvent event : log.getAllEvents()){
//			int eventIndex = event.getOrderNum();
//			eventVSDeviatingObjectRelationList.put(event, new ArrayList<XOCORelation>());	
//			for(XOCORelation objectRelation : deviatingObjectRelationVSContentList.keySet()){
//				int birthEventIndex = objectRelation.getBirthEvent().getOrderNum();
//				int deathEventIndex;
//				if(objectRelation.getDeathEvent() == null){
//					deathEventIndex = eventIndex + 1;
//				}
//				else{
//					deathEventIndex = objectRelation.getDeathEvent().getOrderNum();
//				}
//				if(eventIndex >= birthEventIndex && eventIndex < deathEventIndex){
//					eventVSDeviatingObjectRelationList.get(event).add(objectRelation);
//				}		
//			}
//			
//			//if the event is not deviating, remove it from the map
//			if(eventVSDeviatingObjectRelationList.get(event).size() == 0){
//				eventVSDeviatingObjectRelationList.remove(event);
//			}
//			else{
//				deviatingEventIndexSet.add(eventIndex);
//			}
//		}
		
		//adding deviating event in to the list if it has deviating objects
		List<Pair<Integer, Integer>> startAndEndPairListGlobal = new ArrayList<Pair<Integer, Integer>>();
		for(XOCObject object : deviatingObjectVSStartAndEndPairList.keySet()){
			for(Pair<Integer, Integer> startAndEndPair : deviatingObjectVSStartAndEndPairList.get(object)){
				int startIndex = startAndEndPair.getFirst();				
				int endIndex;
				if(startAndEndPair.getSecond() == null){
					endIndex = log.getAllEvents().size();
				}
				else{
					endIndex = startAndEndPair.getSecond();
				}
				
//				System.out.println("startIndex in deviatingObjectVSStartAndEndPairList.keySet(): " + startIndex);
//				System.out.println("endIndex in deviatingObjectVSStartAndEndPairList.keySet(): " + endIndex);
				
				Pair<Integer, Integer> containingStartPair = null;
				Pair<Integer, Integer> containingEndPair = null;
				//check the start and end of one object is contained in the global pair
				for(Pair<Integer, Integer> startAndEndPairGlobal : startAndEndPairListGlobal){
					int startGlobal = startAndEndPairGlobal.getFirst();				
					int endGlobal;
					if(startAndEndPairGlobal.getSecond() == null){
						endGlobal = log.getAllEvents().size();
					}
					else{
						endGlobal = startAndEndPair.getSecond();
					}
					//if the start is contained
					if(startGlobal <= startIndex && endGlobal >= startIndex){
						containingStartPair = startAndEndPairGlobal;
					}
					if(startGlobal <= endIndex && endGlobal >= endIndex){
						containingEndPair = startAndEndPairGlobal;
					}
					
					if(containingStartPair != null && containingEndPair != null){
						break;
					}
				}
				
				if(containingStartPair == null && containingEndPair == null){
					startAndEndPairListGlobal.add(startAndEndPair);
//					System.out.println("containingStartPair == null && containingEndPair == null");
				}
				else if(containingStartPair == null && containingEndPair != null){
					Pair<Integer, Integer> extendPair = new Pair<Integer, Integer>(startIndex, containingEndPair.getSecond());
					startAndEndPairListGlobal.remove(containingEndPair);
					startAndEndPairListGlobal.add(extendPair);
//					System.out.println("containingStartPair == null && containingEndPair != null");
				}
				else if(containingStartPair != null && containingEndPair == null){
					Pair<Integer, Integer> extendPair = new Pair<Integer, Integer>(containingStartPair.getFirst(), endIndex);
					startAndEndPairListGlobal.remove(containingStartPair);
					startAndEndPairListGlobal.add(extendPair);
//					System.out.println("containingStartPair != null && containingEndPair == null");
				}
				else if(containingStartPair != null && containingEndPair != null){
//					System.out.println("containingStartPair != null && containingEndPair != null");
					if(containingStartPair != containingEndPair){
						Pair<Integer, Integer> extendPair = new Pair<Integer, Integer>(containingStartPair.getFirst(), containingEndPair.getSecond());
						startAndEndPairListGlobal.remove(containingStartPair);
						startAndEndPairListGlobal.remove(containingEndPair);
						startAndEndPairListGlobal.add(extendPair);
//						System.out.println("two pairs");
					}		
				}			
			}
		}
		
		
		
		for(Pair<Integer, Integer> startAndEndPair : startAndEndPairListGlobal){
			int startIndex = startAndEndPair.getFirst();				
			int endIndex;
			if(startAndEndPair.getSecond() == null){
				endIndex = log.getAllEvents().size();
			}
			else{
				endIndex = startAndEndPair.getSecond();
			}
//			System.out.println("startIndex: " + startIndex);
//			System.out.println("endIndex: " + endIndex);
			
			for(int i=startIndex-1; i<endIndex; i++){
				XOCEvent event = log.getAllEvents().get(i);
				deviatingEventIndexSet.add(event.getOrderNum());				
			}
		}
		
		List<Integer> deviatingEventIndexList = new ArrayList<Integer>(deviatingEventIndexSet);
		Collections.sort(deviatingEventIndexList);
		for(int eventIndex : deviatingEventIndexList){
			deviatingEventList.add(log.getAllEvents().get(eventIndex-1));
		}

		
	}

	
	/**
	 * check if one object always satisfies its corresponding constraint based on its related object relations;
	 * note that one object could have multiple class values; valueSurvivalPeriodList indicates the periods when one
	 * class value satisfies requirements;
	 * return the deviating periods, i.e., a list of (startEventOrderNum,endEventOrderNum)
	 * @param object
	 * @param objectRelationType
	 * @param cardinality
	 * @param valueSurvivalPeriodList
	 * @return
	 */
	private List<Pair<Integer, Integer>> checkIfASatisfyingConstraintForOneObject(XOCObject object, String objectRelationType, 
			Cardinality cardinality, Map<String, XOCEvent> idVSEventMap, List<Pair<Integer, Integer>> valueSurvivalPeriodList){
		List<Pair<Integer, Integer>> deviatingStartAndEndPairList = new ArrayList<Pair<Integer, Integer>>();		
				
		System.out.println("object id in checkIfASatisfyingConstraintForOneObject: " + object.getAttributes().get(xConstants.ID).toString());	
		System.out.println("objectRelationType in checkIfASatisfyingConstraintForOneObject: " + objectRelationType);	
		//add checking moment into the list
		List<Integer> checkingEventOrderNumList = new ArrayList<Integer>();		
		for(IntegerPair period : object.getLifecyclePeriodList()){
			int startEventOrderNum = period.getFirstValue();
			checkingEventOrderNumList.add(startEventOrderNum);
			
//			System.out.println("startEventOrderNum in checkIfASatisfyingConstraintForOneObject: " + startEventOrderNum);	
		}
		for(XOCORelation objectRelation : object.getRelatedObjectRelationList()){
			String relationType = objectRelation.getRelationType();
			if(objectRelationType.equals(relationType)){					
				for(IntegerPair period : objectRelation.getLifecyclePeriodList()){
					Integer startEventOrderNum = period.getFirstValue();
					Integer endEventOrderNum = period.getSecondValue();

					if(!checkingEventOrderNumList.contains(startEventOrderNum)){
						checkingEventOrderNumList.add(startEventOrderNum);
//						System.out.println("startEventOrderNum in !checkingEventOrderNumList.contains(startEventOrderNum): " + startEventOrderNum);	
					}
					if(endEventOrderNum != null && !checkingEventOrderNumList.contains(endEventOrderNum) && endEventOrderNum<log.size()){
						checkingEventOrderNumList.add(endEventOrderNum+1);
//						System.out.println("startEventOrderNum in endEventOrderNum != null: " + startEventOrderNum);	
					} 				
				}
				
			}						
		}
		//sort the list from small to large
		Collections.sort(checkingEventOrderNumList);
		
		//check if the cardinality is satisfied at each moment	
		boolean deviatingFlag = false;
		Integer deviatingStartOrderNum = null;//record the event order num (1..*), the deviation appears after the event first time(in the object model corresponding to the event, there is deviation)
		Integer deviatingEndOrderNum = null;//record the event order num (1..*), the deviation appears after the event last time(in the object model corresponding to the event, there is the last deviation)
		for(int checkingMomentOrderNum : checkingEventOrderNumList){
//			System.out.println("checkingMomentOrderNum in for(int checkingMomentOrderNum : checkingEventOrderNumList){: " + checkingMomentOrderNum);	
			int constraintValue = 0;
			for(XOCORelation objectRelation : object.getRelatedObjectRelationList()){
				String relationType = objectRelation.getRelationType();
				if(objectRelationType.equals(relationType)){			
					//if the relation is alive at the checking moment
					if(objectRelation.isAliveAtEvent(log.get(checkingMomentOrderNum-1))){
						constraintValue ++;
					}
				}						
			}
			//if the cardinality is violated, record the object
			if(!cardinality.checkCardinality(constraintValue)){
				//at this moment, the object is deviating and is normal at last moment
//				System.out.println("checkingMomentOrderNum in !cardinality.checkCardinality(constraintValue): " + checkingMomentOrderNum);	
				if(!deviatingFlag){
					deviatingStartOrderNum = checkingMomentOrderNum;
				}
				deviatingFlag = true;
			}			
			else{
				//at this moment, the object is normal and is deviating at last moment
				if(deviatingFlag){
					deviatingEndOrderNum = checkingMomentOrderNum -1;
				}			
				deviatingFlag = false;
			}
			if(deviatingStartOrderNum != null && deviatingEndOrderNum != null){
				Pair<Integer, Integer> deviatingStartAndEndPair = new Pair<Integer, Integer>(deviatingStartOrderNum, deviatingEndOrderNum);
				deviatingStartAndEndPairList.add(deviatingStartAndEndPair);
				deviatingStartOrderNum = null;
				deviatingEndOrderNum = null;
			}			
		}
		//if the deviation exists (after the last checking moment) until the end of the log
		//set the ending event order num as the log.size, i.e., the order num of the last event
		if(deviatingFlag){
			deviatingEndOrderNum = log.size();
			Pair<Integer, Integer> deviatingStartAndEndPair = new Pair<Integer, Integer>(deviatingStartOrderNum, deviatingEndOrderNum);
			deviatingStartAndEndPairList.add(deviatingStartAndEndPair);
		}	
		
		//filter the moments out of the survival periods
		List<Pair<Integer, Integer>> filteredDeviatingStartAndEndPairList = new ArrayList<Pair<Integer, Integer>>();		
		for(Pair<Integer, Integer> deviatingStartAndEndPair : deviatingStartAndEndPairList){
			//identify the overlapping part
			int deviatingStart = deviatingStartAndEndPair.getFirst();
			int deviatingEnd = deviatingStartAndEndPair.getSecond() == null ? log.size() : deviatingStartAndEndPair.getSecond();
			for(Pair<Integer, Integer> valueSurvivalPeriod : valueSurvivalPeriodList){
				int survivalStart = valueSurvivalPeriod.getFirst();
				if(valueSurvivalPeriod.getSecond() == null){
					System.out.println("valueSurvivalPeriod.getSecond() == null");
				}
				int survivalEnd = valueSurvivalPeriod.getSecond() == null ? log.size() : valueSurvivalPeriod.getSecond();
				System.out.println("survivalStart in filter: " + survivalStart);
				System.out.println("survivalEnd in filter: " + survivalEnd);
				System.out.println("deviatingStart in filter: " + survivalStart);
				System.out.println("deviatingEnd in filter: " + deviatingEnd);
				
				int filteredPeriodStart = deviatingStart > survivalStart ? deviatingStart : survivalStart;
				int filteredPeriodEnd = deviatingEnd < survivalEnd ? deviatingEnd : survivalEnd;
				if(filteredPeriodStart <= filteredPeriodEnd){
					filteredDeviatingStartAndEndPairList.add(new Pair<Integer, Integer>(filteredPeriodStart, filteredPeriodEnd));
				}
			}

		}
		return filteredDeviatingStartAndEndPairList;
	}
	

	
	//|{o1 in obj(e`)|(r,o1,o2) in Rel}| in card. type=1: card=card(source,always); type=2: card=card(source,eventually);
	//type=3: card=card(target,always);type=4: card=card(target,eventually);
	private boolean checkConstraint(XOCObjectModel objectModel, XOCObject object, ClassEdge classRelation, int type){
		boolean checkConstraintFlag = true;
		int constraintValue = 0;
		Cardinality cardinality = new Cardinality();
		XOCORelation newObjectRelation = null;
		if(type == 1)
			cardinality = classRelation.getSourceAlwaysCardinality();		
		else if (type == 2)
			cardinality = classRelation.getSourceEventuallyCardinality();
		else if (type == 3)
			cardinality = classRelation.getTargetAlwaysCardinality();
		else if (type == 4)
			cardinality = classRelation.getTargetEventuallyCardinality();

		for(XOCObject modelObject : objectModel.getObjectSet()){	
			if(type == 1 || type == 2)
				newObjectRelation = new XOCORelationImpl(classRelation.getEdgeName(), modelObject, object);
			else if(type == 3 || type == 4)
				newObjectRelation = new XOCORelationImpl(classRelation.getEdgeName(), object, modelObject);
						//for each object relation in Rel(e')
			for(XOCORelation objectRelation : objectModel.getObjectRelationSet()){	
				//if objectRelation equal to objectRelationA or objectRelationB
				if(objectRelation.compareObjectRelation(newObjectRelation)){											
					constraintValue ++;
					break;
				}
			}
		}
		//if the cardinality is violated, record the object
		if(!cardinality.checkCardinality(constraintValue)){
			System.out.println("classRelation: " + classRelation.getEdgeName());
			System.out.println("type: " + type);
			System.out.println("constraintValue: " + constraintValue);
			checkConstraintFlag = false;
		}
		return checkConstraintFlag;
	}
}