package org.processmining.ocbc.conformance.checkers;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.processmining.framework.util.Pair;
import org.processmining.ocbc.log.model.XOCEvent;
import org.processmining.ocbc.log.model.XOCObject;
import org.processmining.ocbc.log.model.XOCObjectModel;

public class ObjectModelsContainmentCheckingResult {

	private List<XOCEvent> eventsOfConflictingObjectModels;		
	private Map<XOCEvent, List<XOCEvent>>  mapOfConflictingEventsVSEvent;//mapping event A on some events whose object models conflict with the event A's object model 	
	private Map<Pair<XOCObjectModel, XOCObjectModel>, List<XOCObject>> mapOfModelPairVSDifference; //record the objects which are include in the first object model but not in the second object model
	private Map<Pair<XOCEvent, XOCEvent>, List<XOCObject>> mapOfEventPairVSDifference; //record the objects which are include in the first object model but not in the second object model
	
	public ObjectModelsContainmentCheckingResult(){
		eventsOfConflictingObjectModels = new ArrayList<XOCEvent>();
		mapOfConflictingEventsVSEvent = new HashMap<XOCEvent, List<XOCEvent>>();
		mapOfModelPairVSDifference = new HashMap<Pair<XOCObjectModel, XOCObjectModel>, List<XOCObject>>();
		mapOfEventPairVSDifference = new HashMap<Pair<XOCEvent, XOCEvent>, List<XOCObject>>();
	}
			
	public List<XOCEvent> getEventsOfConflictingObjectModels(){
		return eventsOfConflictingObjectModels;
	}	
	
	public Map<XOCEvent, List<XOCEvent>> getMapOfConflictingEventsVSEvent(){
		return mapOfConflictingEventsVSEvent;
	}
	
	public Map<Pair<XOCObjectModel, XOCObjectModel>, List<XOCObject>> getMapOfModelPairVSDifference(){
		return mapOfModelPairVSDifference;
	}	
	
	public Map<Pair<XOCEvent, XOCEvent>, List<XOCObject>> getMapOfEventPairVSDifference(){
		return mapOfEventPairVSDifference;
	}	
	
	public void linkDifferenceWithEventPair(XOCEvent precedingEvent, XOCEvent subsequentEvent, List<XOCObject> modelDifference){				
		if(!eventsOfConflictingObjectModels.contains(precedingEvent)){
			eventsOfConflictingObjectModels.add(precedingEvent);	
			List<XOCEvent> events = new ArrayList<XOCEvent>();
			events.add(subsequentEvent);
			mapOfConflictingEventsVSEvent.put(precedingEvent, events);
		}
			
		if(!eventsOfConflictingObjectModels.contains(subsequentEvent)){
			eventsOfConflictingObjectModels.add(subsequentEvent);	
			List<XOCEvent> events = new ArrayList<XOCEvent>();
			events.add(precedingEvent);
			mapOfConflictingEventsVSEvent.put(subsequentEvent, events);
		}
		
		if(!mapOfConflictingEventsVSEvent.get(precedingEvent).contains(subsequentEvent))
			mapOfConflictingEventsVSEvent.get(precedingEvent).add(subsequentEvent);
		if(!mapOfConflictingEventsVSEvent.get(subsequentEvent).contains(precedingEvent))
			mapOfConflictingEventsVSEvent.get(subsequentEvent).add(precedingEvent);
		
		XOCObjectModel precedingModel = precedingEvent.getObjectModel();
		XOCObjectModel subsequentModel = subsequentEvent.getObjectModel();
		Pair<XOCObjectModel, XOCObjectModel> modelPair = new Pair<XOCObjectModel, XOCObjectModel>(precedingModel, subsequentModel);
		if(!mapOfModelPairVSDifference.containsKey(modelPair)){												
			mapOfModelPairVSDifference.put(modelPair, modelDifference);		
		}
		
		Pair<XOCEvent, XOCEvent> eventPair = new Pair<XOCEvent, XOCEvent>(precedingEvent, subsequentEvent);
		if(!mapOfEventPairVSDifference.containsKey(eventPair)){												
			mapOfEventPairVSDifference.put(eventPair, modelDifference);		
		}
	}
}
