package org.processmining.ocbc.conformance.checkers;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.processmining.ocbc.discovery.filter.FilterFunction;
import org.processmining.ocbc.log.model.XOCEvent;
import org.processmining.ocbc.log.model.XOCLog;
import org.processmining.ocbc.log.model.XOCObject;
import org.processmining.ocbc.models.AOCEdge;
import org.processmining.ocbc.models.ActivityNode;
import org.processmining.ocbc.models.Cardinality;
import org.processmining.ocbc.models.ClassNode;
import org.processmining.ocbc.models.OCBCModel;

public class ObjectNumberPerEventChecking {
	private FilterFunction filter = new  FilterFunction();
	private Map<AOCEdge, List<XOCEvent>> violatedAOCRelationVSEventList = new HashMap<AOCEdge, List<XOCEvent>>();
	private Map<XOCEvent, List<AOCEdge>> deviatingEventVSAOCRelationList = new HashMap<XOCEvent, List<AOCEdge>>();// record event and its violated eventually AOC relations
	
	public Map<XOCEvent, List<AOCEdge>> getDeviatingEventVSAOCRelationListMap(){
		return deviatingEventVSAOCRelationList;
	}
	
	public Map<AOCEdge, List<XOCEvent>> getViolatedAOCRelationVSEventListMap(){
		return violatedAOCRelationVSEventList;
	}
	
	public void checking(XOCLog log, OCBCModel model){
		List<AOCEdge> aOCRelations = model.getAllAOCRelations();
		//for any (a,oc) in AOC
		for(AOCEdge aOCEdge : aOCRelations){
			violatedAOCRelationVSEventList.put(aOCEdge, new ArrayList<XOCEvent>());
			ActivityNode activityNode = aOCEdge.getActivityNode();
			ClassNode classNode = aOCEdge.getClassNode();
			
			String activityName = activityNode.getNodeName();
			String className = classNode.getNodeName();
			

			Cardinality classCardinality = aOCEdge.getClassCardinality();
			//check the cardinality on the class side	
			for(XOCEvent event : filter.filterEvents(log.getAllEvents(), null, activityName, 5)){
				int constraintValue = 0;
//				for(XOCObject object : filter.filterObjects(event.getRelatedObjects(), className)){
				for(XOCObject object : filter.filterObjectsByClassAtEvent(event.getRelatedObjects(), className, event)){														
					//if the relation is alive at the checking moment
					if(object.isAliveAtEvent(event)){//can be removed to be faster
						constraintValue ++;
					}										
				}
				
				//if the cardinality is violated, record the event
				if(!classCardinality.checkCardinality(constraintValue)){
//					System.out.println("constraintValue: " + constraintValue);
					violatedAOCRelationVSEventList.get(aOCEdge).add(event);
					
					if(!deviatingEventVSAOCRelationList.containsKey(event)){
						deviatingEventVSAOCRelationList.put(event, new ArrayList<AOCEdge>());
					}
					if(!deviatingEventVSAOCRelationList.get(event).contains(aOCEdge)){
						deviatingEventVSAOCRelationList.get(event).add(aOCEdge);
					}
				}											
			}			
		}
	}
}