package org.processmining.ocbc.conformance.checkers;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.processmining.ocbc.log.model.XOCAttribute;
import org.processmining.ocbc.log.model.XOCEvent;
import org.processmining.ocbc.log.model.XOCLog;
import org.processmining.ocbc.log.model.XOCObject;
import org.processmining.ocbc.log.model.impl.xConstants;
import org.processmining.ocbc.models.AOCEdge;
import org.processmining.ocbc.models.OCBCModel;

public class ProperClassesChecking {
	private Map<XOCEvent, List<XOCObject>> eventVSObjectListOfUnrelatedClasses = new HashMap<XOCEvent, List<XOCObject>>();//record the objects wrongly referred to by an event
	
	/**
	 * record the classes and objects wrongly referred to by an event
	 */
	private Map<XOCEvent, Map<String, List<XOCObject>>> eventVSUnrelatedClassVSObjectList = new HashMap<XOCEvent, Map<String, List<XOCObject>>>();
	
	public Map<XOCEvent, List<XOCObject>> getEventVSObjectListOfUnrelatedClassesMap(){
		return eventVSObjectListOfUnrelatedClasses;
	}
	
	/**
	 * get the eventVSUnrelatedClassesVSObjectList which records the classes and objects wrongly referred to by an event
	 */
	public Map<XOCEvent, Map<String, List<XOCObject>>> getEventVSUnrelatedClassVSObjectListMap(){
		return eventVSUnrelatedClassVSObjectList;
	}
	
	public void checking(XOCLog log, OCBCModel model)
	{		
		for(XOCEvent event : log.getAllEvents()){		
			String activity = event.getAttributes().get(xConstants.ACTIVITY).toString();
			eventVSObjectListOfUnrelatedClasses.put(event, new ArrayList<XOCObject>());
			eventVSUnrelatedClassVSObjectList.put(event, new HashMap<String, List<XOCObject>>());
			//for each o of (e,o) in EO
			for(XOCObject relatedObject : event.getRelatedObjects()){		
				
//				String objectClass = relatedObject.getAttributes().get(xConstants.CLASS).toString();	
				XOCAttribute classXOCAttribute = relatedObject.getXOCAttributes().get(xConstants.CLASS);
				if(classXOCAttribute != null){
					String objectClass = classXOCAttribute.getValueVersion(event.getOrderNum());
					if(objectClass != null){
						//check if (act(e), class(o)) in AOC
						boolean aOCRelationsContainFlag = false;			
						for(AOCEdge aOCEdge : model.getAllAOCRelations()){
							if(aOCEdge.getActivityNode().getNodeName().equals(activity) 
							   &&  aOCEdge.getClassNode().getNodeName().equals(objectClass)){
								aOCRelationsContainFlag = true;
								break;
							}
						}					
						
						if(!aOCRelationsContainFlag) {					
							eventVSObjectListOfUnrelatedClasses.get(event).add(relatedObject);
							if(!eventVSUnrelatedClassVSObjectList.get(event).containsKey(objectClass)){
								eventVSUnrelatedClassVSObjectList.get(event).put(objectClass, new ArrayList<XOCObject>());
							}
							eventVSUnrelatedClassVSObjectList.get(event).get(objectClass).add(relatedObject);
						}
					}					
				}				
			}	
			if(eventVSObjectListOfUnrelatedClasses.get(event).size() == 0){
				eventVSObjectListOfUnrelatedClasses.remove(event);
			}
			if(eventVSUnrelatedClassVSObjectList.get(event).size() == 0){
				eventVSUnrelatedClassVSObjectList.remove(event);
			}
		}
	}  	
}