package org.processmining.ocbc.conformance.plugin;

import java.io.File;
import java.io.FileInputStream;

import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JPanel;

import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.contexts.uitopia.annotations.Visualizer;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.mxgraph.util.mxConstants;
import org.processmining.ocbc.conformance.visualization.views.ConformanceCheckingResultView;
import org.processmining.ocbc.log.io.XocXmlParser;
import org.processmining.ocbc.log.model.XOCLog;
import org.processmining.ocbc.models.OCBCModel;
import org.processmining.ocbc.models.io.ImportOCBCModel;

public class ConformanceCheckingPlugin {
	@Plugin(
			name = "OCBC conformance checking", 
			parameterLabels = {"XOCLog", "OCBC model"}, 
			returnTypes = ConformanceCheckingObject.class, 
			returnLabels = "OCBC Conformance Checking", 
			mostSignificantResult = 1, 
			help = "OCBC conformance checking"
	)
	
	@UITopiaVariant(
		affiliation = UITopiaVariant.EHV,
		author = "Guangming Li",
		email = "g.li.3@tue.nl"
	)	
	public ConformanceCheckingObject generateConformanceCheckingResult(UIPluginContext context, XOCLog log, OCBCModel model){
		
		context.log("OCBC conformance checking starts");
		
		ConformanceCheckingResultView modelPerspective = new ConformanceCheckingResultView();		       
		OCBCModel inputModel = (OCBCModel) model.clone();	
		inputModel.setModelEnvironment(mxConstants.conformance_checking_plugin);
//		OCBCModel inputModel = model;	
		JPanel checkingResult = (JPanel) modelPerspective.conformanceChecking(context, log, inputModel);
		return new ConformanceCheckingObject(checkingResult);
	}
		
	
	@Plugin(name = "OCBC conformance checking visualizer", 
			returnLabels = { "visualizer" }, 
			returnTypes = { JComponent.class }, 
			parameterLabels = {	"OCBC model" })
	@Visualizer
	public JComponent visualizeConformanceCheckingResult(UIPluginContext context, final ConformanceCheckingObject input){
		System.out.println("OCBC conformance checking result visualizer");
		context.log("welcome to OCBC visualizer");
		JPanel resultPanel = input.panel;
		return resultPanel;
	}
	
	public static void main(String[] args) throws Exception{  					
		FileInputStream input = new FileInputStream("D:\\otc.ocbcmml");
		ImportOCBCModel importer = new ImportOCBCModel();
		OCBCModel model = importer.generateOCBCModel(input);
		OCBCModel  inputModel = (OCBCModel) model.clone();
		inputModel.setModelEnvironment(mxConstants.conformance_checking_plugin);
		
		XocXmlParser XOCParser = new XocXmlParser();
//		XOCParser.parse(new File("D:\\deviating.xoc"));	
		XOCParser.parse(new File("D:\\log.xoc"));			
		XOCLog inputLog = XOCParser.getLog();
		
		System.out.println("create checker: ");
        ConformanceCheckingResultView modelPerspective = new ConformanceCheckingResultView();  
        System.out.println("************log information*******************");
		
		JComponent resultWithModel = modelPerspective.conformanceChecking(null, inputLog, inputModel);
		JFrame frame = new JFrame();
		frame.getContentPane().add(resultWithModel);
		frame.setSize(1800, 1000);
		frame.setVisible(true);	       
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    }
}
	

