package org.processmining.ocbc.conformance.plugin;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.List;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;

import com.fluxicon.slickerbox.components.SlickerButton;
import com.fluxicon.slickerbox.factory.SlickerFactory;

/**
 * The first panel of the ActiTraC-preferences-wizard.
 * 
 * @author Niels Lambrigts (niels.lambrigts@gmail.com)
 * @author Jochen De Weerdt (Jochen.DeWeerdt@econ.kuleuven.be)
 * @date 2011-08-01
 */
public class ParameterPanel extends JPanel {

	private static final long serialVersionUID = -5858631772351410585L;
	
	private JRadioButton radioValidityOfObjectModel;
	private JRadioButton radioFulfilment;
	private JRadioButton radioMonotonicity;	
	private JRadioButton radioActivityExistence ;
	private JRadioButton radioObjectExistence ;
	private JRadioButton radioProperClasses;
	private JRadioButton radioEventsPerObject;
	private JRadioButton radioObjectsPerEvent;
	private JRadioButton radioBehavioralConstraints;
	
//	private JButton selectAllButton = new JButton(" select/cancel all ");
	private JButton selectAllButton = new SlickerButton(" select/cancel all ");
	private List<JRadioButton> radioList;

	/**
	 * Creates the entire first panel of the settings.
	 */
	public ParameterPanel() {
		this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		this.setOpaque(false);
		this.add(Box.createVerticalStrut(10));
		this.add(createPerspectiveSelectionPanel());
//		this.add(Box.createVerticalStrut(10));
//		this.add(helpPane());
//		this.add(Box.createVerticalGlue());
	}
	
	/**
	 * Return the parameters that the current state of the panel represent.
	 */
	public boolean getValidityOfObjectModelSelectionFlag(){
		return radioValidityOfObjectModel.isSelected();
	}

	public boolean getFulfilmentSelectionFlag(){
		return radioFulfilment.isSelected();
	}
	
	public boolean getMonotonicitySelectionFlag(){
		return radioMonotonicity.isSelected();
	}
	
	public boolean getActivityExistenceSelectionFlag(){
		return radioActivityExistence.isSelected();
	}
	
	public boolean getObjectExistenceSelectionFlag(){
		return radioObjectExistence.isSelected();
	}
	
	public boolean getProperClassesSelectionFlag(){
		return radioProperClasses.isSelected();
	}

	public boolean getEventsPerObjectSelectionFlag(){
		return radioEventsPerObject.isSelected();
	}
	
	public boolean getObjectsPerEventSelectionFlag(){
		return radioObjectsPerEvent.isSelected();
	}
	
	public boolean getBehavioralConstraintsSelectionFlag(){
		return radioBehavioralConstraints.isSelected();
	}
	
	/**
	 * Create the "Remaining traces"-section.
	 */
	private JPanel helpPane() {
		SlickerFactory slicker = SlickerFactory.instance();
		JPanel panel = slicker.createRoundedPanel(15, Color.GRAY);
		panel.setLayout(new GridBagLayout());		
		
		JLabel title = slicker.createLabel("Introduction:");
		title.setFont(new java.awt.Font("Dialog", java.awt.Font.BOLD, 15));
		title.setForeground(new Color(40, 40, 40));
		
		JLabel state1 = slicker.createLabel("1. validity of object models: check if object models is valid for the class model;");
		state1.setFont(new java.awt.Font("Dialog", java.awt.Font.BOLD, 13));
		state1.setForeground(new Color(40, 40, 40));
		
		JLabel state2 = slicker.createLabel("2. fulfilment: check if object models is fulfilled for the class model;");
		state2.setFont(new java.awt.Font("Dialog", java.awt.Font.BOLD, 13));
		state2.setForeground(new Color(40, 40, 40));

		JLabel state3 = slicker.createLabel("3. monotonicity: check if objects disappear or change class in-between events;");
		state3.setFont(new java.awt.Font("Dialog", java.awt.Font.BOLD, 13));
		state3.setForeground(new Color(40, 40, 40));
		
		JLabel state4 = slicker.createLabel("4. activity existence: check if all activities referred to by an event exist in the behavioral model;");
		state4.setFont(new java.awt.Font("Dialog", java.awt.Font.BOLD, 13));
		state4.setForeground(new Color(40, 40, 40));
		
		JLabel state5 = slicker.createLabel("5. object existence: check if all objects referred to by an event exist in the object model when the event occurs;");
		state5.setFont(new java.awt.Font("Dialog", java.awt.Font.BOLD, 13));
		state5.setForeground(new Color(40, 40, 40));
		
		JLabel state6 = slicker.createLabel("6. proper classes: check if events refer to objects of unrelated classes;");
		state6.setFont(new java.awt.Font("Dialog", java.awt.Font.BOLD, 13));
		state6.setForeground(new Color(40, 40, 40));
		
		JLabel state7 = slicker.createLabel("7. right number of events per object: check if each object always/eventually has the required number of corresponding events;");
		state7.setFont(new java.awt.Font("Dialog", java.awt.Font.BOLD, 13));
		state7.setForeground(new Color(40, 40, 40));
		
		JLabel state8 = slicker.createLabel("8. right number of objects per event: check if each event has the required number of corresponding objects;");
		state8.setFont(new java.awt.Font("Dialog", java.awt.Font.BOLD, 13));
		state8.setForeground(new Color(40, 40, 40));
		
		JLabel state9 = slicker.createLabel("9. behavioral constraints: check if each reference event has the required number of target events.");
		state9.setFont(new java.awt.Font("Dialog", java.awt.Font.BOLD, 13));
		state9.setForeground(new Color(40, 40, 40));
		
		JLabel state10 = slicker.createLabel("For more information, please visit the website http://www.win.tue.nl/ocbc/.");
		state10.setFont(new java.awt.Font("Dialog", java.awt.Font.BOLD, 13));
		state10.setForeground(new Color(40, 40, 40));
		
		List<JLabel> stateList = new ArrayList<JLabel>();
		stateList.add(state1);
		stateList.add(state2);
		stateList.add(state3);
		stateList.add(state4);
		stateList.add(state5);
		stateList.add(state6);
		stateList.add(state7);
		stateList.add(state8);
		stateList.add(state9);
		stateList.add(state10);

		GridBagConstraints c = new GridBagConstraints();
		c.anchor = GridBagConstraints.LINE_START;
		c.gridx = 0;
		c.gridy = 0;
		c.weightx = 1.0;
		panel.add(title, c);

		for(int i=0; i<stateList.size(); i++){
			c = new GridBagConstraints();
			c.anchor = GridBagConstraints.LINE_START;
			c.gridx = 0;
			c.gridy = i+1;
			c.weightx = 1.0;
			panel.add(stateList.get(i), c);
		}
		
		panel.setMaximumSize(new Dimension(Integer.MAX_VALUE, panel
				.getPreferredSize().height));

		return panel;
	}

	/**
	 * Create the "Profile"-section.
	 */
	private JPanel createPerspectiveSelectionPanel() {
		SlickerFactory slicker = SlickerFactory.instance();
		JPanel stop_pane = slicker.createRoundedPanel(15, Color.GRAY);
		stop_pane.setLayout(new GridBagLayout());

		JLabel title = slicker.createLabel("Perspectives:");
		title.setFont(new java.awt.Font("Dialog", java.awt.Font.BOLD, 18));
		title.setForeground(new Color(40, 40, 40));
		
		radioValidityOfObjectModel = slicker.createRadioButton("validity of object models");
		radioFulfilment = slicker.createRadioButton("fulfilment");
		radioMonotonicity = slicker.createRadioButton("monotonicity");
		radioActivityExistence = slicker.createRadioButton("activity existence");
		radioObjectExistence = slicker.createRadioButton("object existence");
		radioProperClasses = slicker.createRadioButton("proper classes");
		radioEventsPerObject = slicker.createRadioButton("right number of events per object");
		radioObjectsPerEvent = slicker.createRadioButton("right number of objects per event");
		radioBehavioralConstraints = slicker.createRadioButton("behavioral constraints");
		
		radioList = new ArrayList<JRadioButton>();
		radioList.add(radioValidityOfObjectModel);
		radioList.add(radioFulfilment);
		radioList.add(radioMonotonicity);
		radioList.add(radioActivityExistence);
		radioList.add(radioObjectExistence);
		radioList.add(radioProperClasses);
		radioList.add(radioEventsPerObject);
		radioList.add(radioObjectsPerEvent);
		radioList.add(radioBehavioralConstraints);
		
		
		GridBagConstraints c = new GridBagConstraints();
		c.gridx = 0;
		c.gridy = 0;
		c.gridwidth = 2;
		c.anchor = GridBagConstraints.LINE_START;
		title.setMaximumSize(title.getPreferredSize());
		stop_pane.add(title, c);

		for(int i=0; i<radioList.size(); i++){
			c = new GridBagConstraints();
			c.gridx = 0;
			c.gridy = i+1;
			c.anchor = GridBagConstraints.LINE_START;
			stop_pane.add(radioList.get(i), c);
		}
		
		c = new GridBagConstraints();
		c.gridx = 0;
		c.gridy = 10;
		c.anchor = GridBagConstraints.LINE_START;
//		selectAllButton.setBackground(Color.GRAY);		
//		selectAllButton.setBorder(BorderFactory.createEtchedBorder());
		stop_pane.add(selectAllButton, c);
		selectAllButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {	
				int selectedRadioNumber = 0;
				for(int i=0; i<radioList.size(); i++){
					if(radioList.get(i).isSelected()){
						selectedRadioNumber ++;
					}
					
				}
				if(selectedRadioNumber < radioList.size()/2){
					for(int i=0; i<radioList.size(); i++){
						radioList.get(i).setSelected(true);
					}
				}
				else{
					for(int i=0; i<radioList.size(); i++){
						radioList.get(i).setSelected(false);
					}
				}
				
			}
		});
		
		
		
		stop_pane.setMaximumSize(new Dimension(Integer.MAX_VALUE, stop_pane
				.getPreferredSize().height));

		return stop_pane;
	}
	
    public static void main(String[] args) {
        JFrame frame = new JFrame("Hello world");
       
        ParameterPanel panel = new ParameterPanel();
       
        frame.add(panel);

       frame.setSize(600, 400);
        frame.setVisible(true);
       
     }

}