package org.processmining.ocbc.conformance.software;
 
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

import org.processmining.common.dialogue.IOFileDialogue;
import org.processmining.mxgraph.util.mxConstants;
import org.processmining.ocbc.conformance.plugin.ParameterPanel;
import org.processmining.ocbc.log.io.XocXmlParser;
import org.processmining.ocbc.log.model.XOCLog;
import org.processmining.ocbc.models.OCBCModel;
import org.processmining.ocbc.models.io.ImportOCBCModel;
 
/**
 * @version 1.0
 * @author liangdong
 *
 */
public class SoftwareParameterInterface extends JPanel{
 
    /**
     * 
     */
    private static final long serialVersionUID = 1L;

    private File lastLogFile;
    private File lastModelFile;
    private ParameterPanel parameterPanel = new ParameterPanel();
    private XOCLog inputLog;
    private OCBCModel  inputModel;
    private XocXmlParser XOCParser;
    private ConformanceCheckingSoftware softwarePanel;
    private JButton selectLogButton;	
    private JButton selectModelButton;
    private JButton checkConformanceButton;
	
    public SoftwareParameterInterface(ConformanceCheckingSoftware softwarePanel) { 	
    	this.softwarePanel = softwarePanel;
    	XOCParser = new XocXmlParser();
    	generateConformanceCheckingParameterPanel();
    }

    public ParameterPanel getParameterPanel(){
    	return parameterPanel;
    }

    
    public void generateConformanceCheckingParameterPanel(){	

		selectLogButton = new JButton("import log");	
		selectModelButton = new JButton("import model");
		checkConformanceButton = new JButton("check conformance");

		selectLogButton.setToolTipText("import XOC log");
		selectModelButton.setToolTipText("import OCBC model");

		
//		XOCParser.parse(new File("D:\\deviating.xoc"));	
		
		
		JPanel buttonPanel = new JPanel();
		buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.X_AXIS));		
		buttonPanel.add(selectLogButton);
		buttonPanel.add(Box.createRigidArea(new Dimension(15, 15)));
		buttonPanel.add(selectModelButton);
	
		buttonPanel.add(Box.createRigidArea(new Dimension(15, 30)));
		buttonPanel.add(checkConformanceButton);	
		
		buttonPanel.setAlignmentY(Component.TOP_ALIGNMENT);
		buttonPanel.setBorder(BorderFactory.createEmptyBorder(15, 5, 15, 5));
	
		this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));		
		this.add(buttonPanel);
		this.add(parameterPanel);
	
		selectLogButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				//change the cursor to waiting cursor
//				this.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
				
				IOFileDialogue fileImporter = new IOFileDialogue();
            	fileImporter.show("XOC Log Files (.xoc)","xoc", JFileChooser.FILES_ONLY, "Select", lastLogFile);
            	JFileChooser fileChooser = fileImporter.getFileChooser();           	
            	File file = fileImporter.getSelectedFile();
				if(file != null){

					lastLogFile = file.getParentFile();
					try {
//						XOCParser.parse(new File("D:\\log.xoc"));
						XOCParser.parse(file);
					} catch (Exception e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}			
					inputLog = XOCParser.getLog();
					selectLogButton.setText("log imported");
				}		
				
				//change the cursor to default cursor
//				fatherPanel.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
			}
		});
		
		selectModelButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				//change the cursor to waiting cursor
//				this.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
				
				IOFileDialogue fileImporter = new IOFileDialogue();
            	fileImporter.show("OCBC Model Files (.ocbcmml)","ocbcmml", JFileChooser.FILES_ONLY, "Select", lastModelFile);
            	JFileChooser fileChooser = fileImporter.getFileChooser();           	
            	File file = fileImporter.getSelectedFile();
				if(file != null){

					lastModelFile = file.getParentFile();
					FileInputStream input = null;
					try {
						input = new FileInputStream(file);
					} catch (FileNotFoundException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
					ImportOCBCModel importer = new ImportOCBCModel();
					OCBCModel model = null;
					try {
						model = importer.generateOCBCModel(input);
					} catch (IOException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
					inputModel = (OCBCModel) model.clone();
					inputModel.setModelEnvironment(mxConstants.conformance_checking_plugin);
					selectModelButton.setText("model imported");
				}		
				
				//change the cursor to default cursor
//				fatherPanel.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
			}
		});

		
		checkConformanceButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {	 
				if(inputLog != null && inputModel != null){	
					softwarePanel.updateConformanceCheckingPanel(inputLog, inputModel);
				}
				else if(inputLog == null){
					JOptionPane.showMessageDialog(null, "Wrong! Please import a log!");
				}
				else if(inputModel == null){
					JOptionPane.showMessageDialog(null, "Wrong! Please import a model!");
				}			
			}			
		});
		
		 

    }
   
    public static void main(String[] args) {
        JFrame frame = new JFrame("Hello world");    
        SoftwareParameterInterface parameterInterface = new SoftwareParameterInterface(null);      
        frame.add(parameterInterface);
        frame.setSize(600, 400);
        frame.setVisible(true);
       
     }
}