package org.processmining.ocbc.conformance.visualization.interaction;

import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.Set;
import java.util.UUID;

import javax.swing.JButton;

import org.jgraph.JGraph;
import org.processmining.ocbc.conformance.visualization.views.ConformanceCheckingResultView;
import org.processmining.ocbc.log.model.XOCEvent;
import org.processmining.ocbc.log.model.XOCLog;
import org.processmining.ocbc.models.ActivityEdge;
import org.processmining.ocbc.models.ClassEdge;
import org.processmining.ocbc.models.ClassNode;
import org.processmining.ocbc.models.OCBCModel;
import org.processmining.ocbc.models.editor.panels.EditorPanel;
import org.processmining.ocbc.models.editor.panels.InformationPanel;
import org.processmining.ocbc.models.visualization.ModelLayoutBuilder;
import org.processmining.ocbc.models.visualization.cell.ActivityCell;
import org.processmining.ocbc.models.visualization.cell.ClassCell;
import org.processmining.ocbc.models.visualization.cell.MyOriginalCell;
import org.processmining.ocbc.models.visualization.cell.TransparentCell;


public class LogGraphMouseListener extends MouseAdapter {
	OCBCModel model;
	ModelLayoutBuilder builder;
	
	UUID selectedID;
	UUID previouslySelectedID;
	
	private JGraph graph;
	private XOCLog log;
	private XOCEvent lastEvent;
	private XOCEvent selectedEvent;

	
	InformationPanel informationPanel;
	EditorPanel editorPanel;
	Set<UUID> foldedNodes;
	String elementType;
	String previouslyElementType;
	
	private JButton focusedButton;
	private JButton previouslyFocusedButton;
	
	private MyOriginalCell firstlyFocusedCell;//used for keeping the first point when drawing a edge
	int firstX;
	int firstY;
	
	private MyOriginalCell secondlyFocusedCell;//used for keeping the second point when drawing a edge
	int secondX;
	int secondY;
	
	private MyOriginalCell focusedCell;
	
	public LogGraphMouseListener( XOCLog log, ConformanceCheckingResultView conformanceCheckingResult){
//		this.graphPanel = graphPanel;
//		this.attributesPanel = attributesPanel;
		this.log = log;
		this.lastEvent = log.getAllEvents().get(log.getAllEvents().size()-1);
	}
	

	
	public void addInformationPanel(InformationPanel informationPanel){
		this.informationPanel = informationPanel;
	}
	
	public void addEditorPanel(EditorPanel editorPanel){
		this.editorPanel = editorPanel;
	}
	
	public void updateGraph(JGraph graph){
		this.graph = graph;
	}
	
	public UUID getSelectedID(){
		return selectedID;
	}
	
	public UUID getPreviouslySelectedID(){
		return previouslySelectedID;
	}
	
	/**
	 * Sets the selected ID, can be used for variables and originators (it will raise a mousePressed event)
	 * @param newID
	 */
	public void setSelectedButton(JButton focusedButton){
		this.previouslyFocusedButton = this.focusedButton;
		this.focusedButton = focusedButton;
		System.out.println("set focusedButton");
//		mousePressed(null);
	}
	
	public void setSelectedEvent(XOCEvent selectedEvent){
		this.selectedEvent = selectedEvent;
	}
	
	public JButton getSelectedButton(){
//		this.previouslyFocusedButton = this.focusedButton;
		return focusedButton;
	}
	
	
	public void setSelectedCell(MyOriginalCell cell){
//		this.previouslyFocusedButton = this.focusedButton;
		this.focusedCell = cell;
//		System.out.println("set focusedButton");
//		mousePressed(null);
	}
	
	public MyOriginalCell getSelectedCell(){
//		this.previouslyFocusedButton = this.focusedButton;
		return focusedCell;
	}
	
	public JButton getPreviouslySelectedButton(){
//		this.previouslyFocusedButton = this.focusedButton;
		System.out.println("getPreviouslySelectedButton()");
		return this.previouslyFocusedButton;
	}
	
	public void setSelectedID(UUID newID){
		this.previouslySelectedID = this.selectedID;
		this.selectedID = newID;
//		mousePressed(null);
	}
	
	public String getElementType(){
		return elementType;
	}
	
	public String getPreviouslyElementType(){
		return previouslyElementType;
	}
	
	/**
	 * Sets the selected elementType, can be used for variables and originators (it will raise a mousePressed event)
	 * @param newID
	 */
	public void setElementType(String elementType){
		this.previouslyElementType = this.elementType;
		this.elementType = elementType;
		System.out.println("set elementType");
//		mousePressed(null);
	}


	@Override
	public void mouseReleased(MouseEvent e){				
		int x = e.getX();
		int y = e.getY();
//		DefaultGraphCell cell = (DefaultGraphCell) graphPanel.getGraph().getFirstCellForLocation(x, y);
//		if(cell != null){
//			if(cell instanceof ObjectCell){
//				String cellId = cell.getUserObject().toString();
//				int positionX = (int) GraphConstants.getBounds(cell.getAttributes()).getX();
//				int positionY = (int) GraphConstants.getBounds(cell.getAttributes()).getY();
//				Pair<Integer, Integer> coordinate = new Pair<Integer, Integer>(positionX, positionY);
//				graphPanel.getOrderCoordinateMap().put(cellId, coordinate);
//				//update the object attributes and conformance checking result information for selected object
//				for(XOCObject object : lastEvent.getObjectModel().getObjectSet()){
//					if(object.getAttributes().get("id").toString().equals(cellId)){
//						attributesPanel.updateObjectAttributes(object, log, conformanceCheckingResult, selectedEvent);
//						break;
//					}
//				}					
//			}			
//			else if(cell instanceof ObjectEdgeCell){
//				String cellId = ((ObjectEdgeCell) cell).getCorrespondingXOCORelation().getAttributes().get("id").toString();	
//				//update the object attributes and conformance checking result information for selected object
//				for(XOCORelation objectRelation : lastEvent.getObjectModel().getObjectRelationSet()){
//					if(objectRelation.getAttributes().get("id").toString().equals(cellId)){
//						attributesPanel.updateObjectRelationAttributes(objectRelation, log, conformanceCheckingResult, selectedEvent);
//						break;
//					}
//				}					
//			}			
//			else if(cell instanceof EventCell || cell instanceof ViolatedEventCell){
//				attributesPanel.updateEventNodeAttributes(selectedEvent, log, conformanceCheckingResult, selectedEvent);
//			}

									
//		}					
	}
	
	public boolean drawNode(int x, int y){		
		
		if(focusedButton == null){
			return false;
		}
		else {
			
			
			
			if(focusedButton.getText().equals("ClassNode"))
				builder.drawClassNode(x,y);
			if(focusedButton.getText().equals("ActivityNode"))
				builder.drawActivityNode(x,y);
		
			
			focusedButton = null;
			return true;
		}
	}
	
	public boolean drawEdge(MyOriginalCell firstlyFocusedCell, int firstX, int firstY, MyOriginalCell secondlyFocusedCell, int  secondX, int secondY){
		if(focusedButton.getText().equals("ClassEdge"))
		{
			if((firstlyFocusedCell instanceof ClassCell) && (secondlyFocusedCell instanceof ClassCell))
			{
				builder.drawClassEdge(firstlyFocusedCell, firstX, firstY, secondlyFocusedCell, secondX, secondY);
				System.out.println("drawClassEdge");
			}					
			else System.out.println("Wrong selection! Please select two class nodes");
		}
		
		else if(focusedButton.getText().equals("AOCEdge"))
		{
//			if(((firstlyFocusedCell instanceof ClassCell) && (secondlyFocusedCell instanceof ActivityCell)) ||((firstlyFocusedCell instanceof ActivityCell) && (secondlyFocusedCell instanceof ClassCell)))
			if((firstlyFocusedCell instanceof ActivityCell) && (secondlyFocusedCell instanceof ClassCell))				
			{
				builder.drawAOCEdge(firstlyFocusedCell, firstX, firstY, secondlyFocusedCell, secondX, secondY);
				System.out.println("drawAOCEdge");
			}					
			else System.out.println("Wrong selection! Please select firstly one activity node and secondly one class node");
		}
		
		else if(focusedButton.getText().equals("CrelEdge"))
		{
			if((firstlyFocusedCell instanceof TransparentCell) && (firstlyFocusedCell.getCorrespondingElement() instanceof ActivityEdge))
			{
				if(((secondlyFocusedCell instanceof TransparentCell) && (secondlyFocusedCell.getCorrespondingElement() instanceof ClassEdge))
					||((secondlyFocusedCell instanceof ClassCell) && (secondlyFocusedCell.getCorrespondingElement() instanceof ClassNode)))
				{
					builder.drawCrelEdge(firstlyFocusedCell, firstX, firstY, secondlyFocusedCell, secondX, secondY);
					System.out.println("drawCrelEdge");	
				}
				else System.out.println("Wrong selection! Please select firstly  one activity edge and secondly one activity edge or class node");
			}									
			else System.out.println("Wrong selection! Please select firstly  one activity edge");
		}
		
		else if(focusedButton.getText().contains("ActivityEdge"))
		{
			if((firstlyFocusedCell instanceof ActivityCell) && (secondlyFocusedCell instanceof ActivityCell))
			{
				builder.drawActivityEdge(focusedButton.getText(), firstlyFocusedCell, firstX, firstY, secondlyFocusedCell, secondX, secondY);
				System.out.println("drawActivityEdge");
			}		
			else System.out.println("Wrong selection! Please select two activity nodes");
		}
		
		

		
		return true;
	}
	
}
