package org.processmining.ocbc.conformance.visualization.panels;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.JPanel;
import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;

import org.processmining.common.frame.CreateTreePanel;
import org.processmining.mxgraph.util.mxConstants;
import org.processmining.ocbc.conformance.checkers.ActivityExistenceChecking;
import org.processmining.ocbc.conformance.visualization.views.ResultTextView;
import org.processmining.ocbc.log.model.XAttributeMap;
import org.processmining.ocbc.log.model.XOCEvent;
import org.processmining.ocbc.log.model.XOCLog;
import org.processmining.ocbc.log.model.impl.xConstants;
import org.processmining.ocbc.models.OCBCModel;

import com.fluxicon.slickerbox.components.RoundedPanel;

public class ActivityExistencePanel extends RoundedPanel {

	
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	
	private JTree objectModelListTree;
	private JPanel eventListPane;
	private JPanel behavioralModelPane;
	private JPanel deviationPane;
	private ResultTextView resultTextView;

	
	private XOCLog log;
	private OCBCModel model;
	private List<XOCEvent> deviatingEventList;

	
	public ActivityExistencePanel(ResultTextView resultTextView, XOCLog log, OCBCModel model){
		this.resultTextView = resultTextView;
		this.log = log;
		this.model = model;
		this.setMaximumSize(new Dimension(230, 1000));
		this.setPreferredSize(new Dimension(230, 1000));
		this.setLayout(new GridLayout(1,2));
		this.setBorder(BorderFactory.createLineBorder(new Color(240, 240, 240)));
		eventListPane = createEventListHierarchy(log);
		behavioralModelPane = createActivityListHierarchy(model);
		deviationPane = createDeviationHierarchy(resultTextView);	
		
		this.add(eventListPane);
		this.add(deviationPane);
		this.add(behavioralModelPane);			
	}
	
	/**
	 * get the list of deviating events which have inexistent activities;
	 * @return
	 */
	public List<XOCEvent> getDeviatingEventList(){
		return deviatingEventList;
	}
	
	public CreateTreePanel createDeviationHierarchy(ResultTextView resultTextView){
		ActivityExistenceChecking activityExistenceChecker= resultTextView.getConformanceCheckingResultView().getActivityExistenceChecker();
		deviatingEventList = activityExistenceChecker.getDeviatingEventList();
		DefaultMutableTreeNode root = generateDeviatingEventSetNode(deviatingEventList, mxConstants.event_list);
		CreateTreePanel deviationTreePanel = new CreateTreePanel(root, mxConstants.conformance_checking_output);	
//		if(root.getChildCount() > 0){
//			deviatingFlag = true;
//		}
		return deviationTreePanel;
	}
	
	public CreateTreePanel createActivityListHierarchy(final OCBCModel model){
		DefaultMutableTreeNode root = new DefaultMutableTreeNode(mxConstants.behavioral_model);
		root.add(generateActivityListNode(model.getAllActivityNames(), mxConstants.activity_list));	
		CreateTreePanel classModelTreePanel = new CreateTreePanel(root, mxConstants.conformance_checking_input_model);	
		return classModelTreePanel;
	}
	
	/**
	 * create the object model list hierarchy
	 * @param log
	 */
	public CreateTreePanel createEventListHierarchy(final XOCLog log){
		DefaultMutableTreeNode root = new DefaultMutableTreeNode(
//				"object model list(" + log.getAllEvents().size() + ")");
				mxConstants.event_list);
		for(XOCEvent event : log.getAllEvents()){
			String eventId = event.getAttributes().get(xConstants.ID).toString();
			DefaultMutableTreeNode eventNode = generateEventNode(event, true, false, false, false, eventId);
			root.add(eventNode);
		}
		
		CreateTreePanel eventListTreePanel = new CreateTreePanel(root, mxConstants.conformance_checking_input_log);	
//		objectModelListTree = objectModelListTreePanel.getJTree();
//		
//		objectModelListTree.addTreeWillExpandListener(new TreeWillExpandListener() {		
//			public void treeWillExpand(TreeExpansionEvent event) throws ExpandVetoException {
//				// TODO Auto-generated method stub
//				DefaultMutableTreeNode expandedNode = (DefaultMutableTreeNode) event.getPath().getLastPathComponent();
//				String nodeName = expandedNode.getUserObject().toString();
//				String firstPart = nodeName.split("-")[0];			
//				if(firstPart.equals(mxConstants.object_model)){
//					String eventId = nodeName.split("-")[1];
//					XOCEvent expandedEvent = log.getIdVSEventMap().get(eventId);
//					expandedNode.removeAllChildren();
//					XOCObjectModel completeObjectModel = log.createCompleteObjectModel(expandedEvent);
////					expandedNode.add(generateObjectSetNode(completeObjectModel.getObjectSet(),
////					TREE_NODE_OBJECTS + "(" +completeObjectModel.getObjectSet().size() + ")"));
////			expandedNode.add(generateObjectRelationSetNode(completeObjectModel.getObjectRelationSet(),
////					TREE_NODE_RELATIONS + "(" +completeObjectModel.getObjectRelationSet().size() + ")"));
//			expandedNode.add(generateObjectSetNode(completeObjectModel.getObjectSet(),
//					mxConstants.object_list));
//			expandedNode.add(generateObjectRelationSetNode(completeObjectModel.getObjectRelationSet(),
//					mxConstants.object_relation_list));					
//					
//				}
//
//				System.out.println("tree open: " + event.getPath());
//				System.out.println("lastPath: " + nodeName);
//			}
//			
//			public void treeWillCollapse(TreeExpansionEvent event) throws ExpandVetoException {
//				// TODO Auto-generated method stub
//			}
//		});

		return eventListTreePanel;

	}

	private DefaultMutableTreeNode generateAttributeListNode(XAttributeMap attributeList, String name){
		DefaultMutableTreeNode listNode = new DefaultMutableTreeNode(new TreeEntry(name));
		for(String key : attributeList.keySet()){
			listNode.add(new DefaultMutableTreeNode(new TreeEntry(key + " : " + attributeList.get(key).toString())));
		}		
		return  listNode;
	}
	

	
	private DefaultMutableTreeNode generateEventNode(XOCEvent event, boolean displayAttributeListFlag, boolean displayObjectReferenceList,
			boolean displayNormalObjectModelFlag, boolean displayUpdatedObjectModelFlag, String name){
		DefaultMutableTreeNode eventNode = new DefaultMutableTreeNode(new TreeEntry(name));
		if(displayAttributeListFlag){
			eventNode.add(generateAttributeListNode(event.getAttributes(), mxConstants.attribute_list));
		}
		return eventNode;
	}
	
	public DefaultMutableTreeNode generateActivityListNode(List<String> activityList, String name){
		DefaultMutableTreeNode eventNode = new DefaultMutableTreeNode(new TreeEntry(name));
		for(String activity : activityList){
			eventNode.add(new DefaultMutableTreeNode(new TreeEntry(activity)));
		}
		return eventNode;
	}
	
	private DefaultMutableTreeNode generateDeviatingEventNode(XOCEvent element, String name){
		DefaultMutableTreeNode elementSetNode = new DefaultMutableTreeNode(new TreeEntry(name));
		String activity = element.getAttributes().get(xConstants.ACTIVITY).toString();
		elementSetNode.add(new DefaultMutableTreeNode(new TreeEntry(xConstants.ACTIVITY + ": " + activity)));
		return elementSetNode;
	}
	
	private DefaultMutableTreeNode generateDeviatingEventSetNode(List<XOCEvent> deviatingEventList, String name){
		DefaultMutableTreeNode elementSetNode = new DefaultMutableTreeNode(new TreeEntry(name));
		for (XOCEvent element : deviatingEventList) {
			String eventId = element.getAttributes().get(xConstants.ID).toString();
			elementSetNode.add(generateDeviatingEventNode(element, eventId));
		}
		return elementSetNode;
	}

	private class TreeEntry {
		private String key;
		private String displaytext;
		
		public TreeEntry(String key) {
			this(key, key);
		}
		
		public TreeEntry(String key, String displaytext) {
			this.key = key;
			this.displaytext = displaytext;
		}
		
		public String getKey() {
			return this.key;
		}
		
		public String getDisplayText() {
			return this.displaytext;
		}
		
		public String toString() {
			return getDisplayText();
		}
	}
	
}