package org.processmining.ocbc.conformance.visualization.panels;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.util.List;
import java.util.Map;

import javax.swing.BorderFactory;
import javax.swing.JPanel;
import javax.swing.JTree;
import javax.swing.event.TreeExpansionEvent;
import javax.swing.event.TreeWillExpandListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.ExpandVetoException;

import org.processmining.common.frame.CreateTreePanel;
import org.processmining.mxgraph.util.mxConstants;
import org.processmining.ocbc.conformance.checkers.FulfilmentChecking;
import org.processmining.ocbc.conformance.visualization.views.ResultTextView;
import org.processmining.ocbc.log.model.XOCEvent;
import org.processmining.ocbc.log.model.XOCLog;
import org.processmining.ocbc.log.model.XOCORelation;
import org.processmining.ocbc.log.model.XOCObject;
import org.processmining.ocbc.log.model.XOCObjectModel;
import org.processmining.ocbc.log.model.impl.xConstants;
import org.processmining.ocbc.models.ClassEdge;
import org.processmining.ocbc.models.ClassNode;
import org.processmining.ocbc.models.OCBCModel;

import com.fluxicon.slickerbox.components.RoundedPanel;

public class FulfilmentPanel extends RoundedPanel {

	
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	
	private JTree objectModelListTree;
	private JPanel objectModelListPane;
	private JPanel classModelPane;
	private JPanel deviationPane;
	private ResultTextView resultTextView;

	
	private XOCLog log;
	private OCBCModel model;
	private Map<ClassEdge, List<XOCObject>> violatedSourceClassRelationVSObjectList;
	private Map<ClassEdge, List<XOCObject>> violatedTargetClassRelationVSObjectList;
	private List<ClassEdge> violatedClassRelationList;
	
	public FulfilmentPanel(ResultTextView resultTextView, XOCLog log, OCBCModel model){
		this.resultTextView = resultTextView;
		this.log = log;
		this.model = model;
		this.setMaximumSize(new Dimension(230, 1000));
		this.setPreferredSize(new Dimension(230, 1000));
		this.setLayout(new GridLayout(1,2));
		this.setBorder(BorderFactory.createLineBorder(new Color(240, 240, 240)));
		objectModelListPane = createObjectModelListHierarchy(log);
		classModelPane = createClassModelHierarchy(model);
		deviationPane = createDeviationHierarchy(resultTextView);	
		
		this.add(objectModelListPane);
		this.add(deviationPane);
		this.add(classModelPane);			
	}
	
	/**
	 * get the list of violated class relations which are not fulfilled;
	 * @return
	 */
	public List<ClassEdge> getViolatedClassRelationList(){
		return violatedClassRelationList;
	}
	
	public CreateTreePanel createDeviationHierarchy(ResultTextView resultTextView){
		FulfilmentChecking fulfilmentChecker = resultTextView.getConformanceCheckingResultView().getFulfilmentChecker();

		violatedSourceClassRelationVSObjectList = fulfilmentChecker.getViolatedSourceClassRelationVSObjectListMap();
		violatedTargetClassRelationVSObjectList = fulfilmentChecker.getViolatedTargetClassRelationVSObjectListMap();		
		violatedClassRelationList = fulfilmentChecker.getViolatedClassRelationList();
//		DefaultMutableTreeNode root = new DefaultMutableTreeNode(
////				"deviating object model list(" + deviatingEventList.size() + ")");
//				mxConstants.last_object_model);
//		if(violatedClassRelationList.size() > 0){
//			DefaultMutableTreeNode violatedClassRelationListNode = generateDeviatingClassRelationSetNode(violatedClassRelationList, 
//					violatedSourceClassRelationVSObjectList, 
//					violatedTargetClassRelationVSObjectList, mxConstants.class_relation_list);
//			root.add(violatedClassRelationListNode);
//		}	
		
		DefaultMutableTreeNode root = new DefaultMutableTreeNode(
//				"deviating object model list(" + deviatingEventList.size() + ")");
				mxConstants.class_relation_list);
		
		if(violatedClassRelationList.size() > 0){
			root = generateDeviatingClassRelationSetNode(violatedClassRelationList, 
					violatedSourceClassRelationVSObjectList, 
					violatedTargetClassRelationVSObjectList, mxConstants.class_relation_list);
		}	
		
//		if(violatedClassRelationList.size() > 0){
//			DefaultMutableTreeNode violatedClassRelationListNode = generateDeviatingClassRelationSetNode(violatedClassRelationList, 
//					violatedSourceClassRelationVSObjectList, 
//					violatedTargetClassRelationVSObjectList, mxConstants.class_relation_list);
//			root.add(violatedClassRelationListNode);
//		}
		
		CreateTreePanel deviationTreePanel = new CreateTreePanel(root, mxConstants.conformance_checking_output);	
		return deviationTreePanel;
	}
	
	public CreateTreePanel createClassModelHierarchy(final OCBCModel model){
		DefaultMutableTreeNode root = new DefaultMutableTreeNode(mxConstants.class_model);
		root.add(generateClassSetNode(model.getAllClasses(), mxConstants.class_list));
		root.add(generateClassRelationSetNode(model.getAllClassRelations(), mxConstants.class_relation_list));			
		
		CreateTreePanel classModelTreePanel = new CreateTreePanel(root, mxConstants.conformance_checking_input_model);	
		return classModelTreePanel;
	}
	
	/**
	 * create the object model list hierarchy
	 * @param log
	 */
	public CreateTreePanel createObjectModelListHierarchy(final XOCLog log){
		DefaultMutableTreeNode root = new DefaultMutableTreeNode(
//				"object model list(" + log.getAllEvents().size() + ")");
				mxConstants.object_model_list);
		for(XOCEvent event : log.getAllEvents()){
			String eventId = event.getAttributes().get(xConstants.ID).toString();
			XOCObjectModel completeObjectModel = event.getObjectModel();
			DefaultMutableTreeNode objectModle = new DefaultMutableTreeNode(new TreeEntry(mxConstants.object_model + "-" + eventId));
			objectModle.add(new DefaultMutableTreeNode(new TreeEntry("fakingNode")));
//			objectModle.add(generateObjectSetNode(completeObjectModel.getObjectSet(), mxConstants.object_list));
//			objectModle.add(generateObjectRelationSetNode(completeObjectModel.getObjectRelationSet(), mxConstants.object_relation_list));					
			root.add(objectModle);
		}
		
		CreateTreePanel objectModelListTreePanel = new CreateTreePanel(root, mxConstants.conformance_checking_input_log);	
		objectModelListTree = objectModelListTreePanel.getJTree();
		
		objectModelListTree.addTreeWillExpandListener(new TreeWillExpandListener() {		
			public void treeWillExpand(TreeExpansionEvent event) throws ExpandVetoException {
				// TODO Auto-generated method stub
				DefaultMutableTreeNode expandedNode = (DefaultMutableTreeNode) event.getPath().getLastPathComponent();
				String nodeName = expandedNode.getUserObject().toString();
				String firstPart = nodeName.split("-")[0];			
				if(firstPart.equals(mxConstants.object_model)){
					String eventId = nodeName.split("-")[1];
					XOCEvent expandedEvent = log.getIdVSEventMap().get(eventId);
					expandedNode.removeAllChildren();
					XOCObjectModel completeObjectModel = log.createCompleteObjectModel(expandedEvent);
//					expandedNode.add(generateObjectSetNode(completeObjectModel.getObjectSet(),
//					TREE_NODE_OBJECTS + "(" +completeObjectModel.getObjectSet().size() + ")"));
//			expandedNode.add(generateObjectRelationSetNode(completeObjectModel.getObjectRelationSet(),
//					TREE_NODE_RELATIONS + "(" +completeObjectModel.getObjectRelationSet().size() + ")"));
			expandedNode.add(generateObjectSetNode(completeObjectModel.getObjectSet(),
					mxConstants.object_list));
			expandedNode.add(generateObjectRelationSetNode(completeObjectModel.getObjectRelationSet(),
					mxConstants.object_relation_list));					
					
				}

				System.out.println("tree open: " + event.getPath());
				System.out.println("lastPath: " + nodeName);
			}
			
			public void treeWillCollapse(TreeExpansionEvent event) throws ExpandVetoException {
				// TODO Auto-generated method stub
			}
		});

		return objectModelListTreePanel;

	}

	private DefaultMutableTreeNode generateObjectNode(XOCObject object){
		DefaultMutableTreeNode objectNode = new DefaultMutableTreeNode(new TreeEntry(object.getAttributes().get("id").toString()));
		for(String key : object.getAttributes().keySet()){
			objectNode.add(new DefaultMutableTreeNode(new TreeEntry(key + " : " + object.getAttributes().get(key).toString())));
		}		
		return  objectNode;
	}
	
	private DefaultMutableTreeNode generateObjectSetNode(List<XOCObject> objects, String name){
		DefaultMutableTreeNode objectSetNode = new DefaultMutableTreeNode(new TreeEntry(name));
		if(objects != null){
			for (XOCObject object : objects) {
				objectSetNode.add(generateObjectNode(object));
			}
		}	
		return objectSetNode;
	}

	private DefaultMutableTreeNode generateObjectRelationNode(XOCORelation relation){
		DefaultMutableTreeNode relationNode = new DefaultMutableTreeNode(new TreeEntry(relation.getAttributes().get("id").toString()));
		for(String key : relation.getAttributes().keySet()){
			relationNode.add(new DefaultMutableTreeNode(new TreeEntry(key + " : " + relation.getAttributes().get(key).toString())));
		}		
		return  relationNode;
	}
	
	private DefaultMutableTreeNode generateObjectRelationSetNode(List<XOCORelation> relations, String name){
		DefaultMutableTreeNode relationSetNode = new DefaultMutableTreeNode(new TreeEntry(name));
		for (XOCORelation relation : relations) {
			relationSetNode.add(generateObjectRelationNode(relation));
		}
		return relationSetNode;
	}
	
	private DefaultMutableTreeNode generateObjectModelNode(XOCObjectModel objectModel){
		DefaultMutableTreeNode objectModelNode = new DefaultMutableTreeNode(new TreeEntry("OM"));
		List<XOCObject> objectList = objectModel.getObjectSet();
		List<XOCORelation> objectRelationList = objectModel.getObjectRelationSet();
		objectModelNode.add(generateObjectSetNode(objectList, "Objects"));
		objectModelNode.add(generateObjectRelationSetNode(objectRelationList, "Relations"));
		return objectModelNode;
	}
	
	private DefaultMutableTreeNode generateClassNode(ClassNode classNode){
		DefaultMutableTreeNode classTreeNode = new DefaultMutableTreeNode(new TreeEntry(classNode.getNodeName()));
		//in future, add the attribute node if the class node has attributes
//		classTreeNode.add(new DefaultMutableTreeNode(new TreeEntry("node_name : " + classNode.getNodeName())));
		return  classTreeNode;
	}
	
	private DefaultMutableTreeNode generateClassSetNode(List<ClassNode> classList, String name){
		DefaultMutableTreeNode classSetNode = new DefaultMutableTreeNode(new TreeEntry(name));
		for (ClassNode classNode : classList) {
			classSetNode.add(generateClassNode(classNode));
		}
		return classSetNode;
	}
	
	private DefaultMutableTreeNode generateClassRelationNode(ClassEdge relation){
		DefaultMutableTreeNode relationNode = new DefaultMutableTreeNode(new TreeEntry(relation.getID()));
		relationNode.add(new DefaultMutableTreeNode(
				new TreeEntry(xConstants.relationship_type + ": " + relation.getEdgeName())));	
		relationNode.add(new DefaultMutableTreeNode(
				new TreeEntry(xConstants.source_class + ": " + relation.getSourceNode().getNodeName())));	
		relationNode.add(new DefaultMutableTreeNode(
				new TreeEntry(xConstants.target_class + ": " + relation.getTargetNode().getNodeName())));
		relationNode.add(new DefaultMutableTreeNode(
				new TreeEntry(xConstants.source_always_cardinality + ": " + relation.getSourceAlwaysCardinality().toString())));	
		relationNode.add(new DefaultMutableTreeNode(
				new TreeEntry(xConstants.source_eventually_cardinality + ": " + relation.getSourceEventuallyCardinality().toString())));	
		relationNode.add(new DefaultMutableTreeNode(
				new TreeEntry(xConstants.target_always_cardinality + ": " + relation.getTargetAlwaysCardinality().toString())));	
		relationNode.add(new DefaultMutableTreeNode(
				new TreeEntry(xConstants.target_eventually_cardinality + ": " + relation.getTargetEventuallyCardinality().toString())));				
		return  relationNode;
	}
	
	private DefaultMutableTreeNode generateClassRelationSetNode(List<ClassEdge> relations, String name){
		DefaultMutableTreeNode relationSetNode = new DefaultMutableTreeNode(new TreeEntry(name));
		for (ClassEdge relation : relations) {
			relationSetNode.add(generateClassRelationNode(relation));
		}
		return relationSetNode;
	}
	
	private DefaultMutableTreeNode generateDeviatingObjectRelationNode(XOCORelation relation, List<String> contentList){
		DefaultMutableTreeNode relationNode = new DefaultMutableTreeNode(new TreeEntry(relation.getAttributes().get("id").toString()));
		for(String content : contentList){
			relationNode.add(new DefaultMutableTreeNode(new TreeEntry(content)));
		}		
		return  relationNode;
	}
	
	public DefaultMutableTreeNode generateDeviatingObjectRelationSetNode(List<XOCORelation> relations, 
			Map<XOCORelation, List<String>> deviatingObjectRelationVSContentListMap, String name){
		DefaultMutableTreeNode relationSetNode = new DefaultMutableTreeNode(new TreeEntry(name));
		for (XOCORelation relation : relations) {
			List<String> contentList = deviatingObjectRelationVSContentListMap.get(relation);
			relationSetNode.add(generateDeviatingObjectRelationNode(relation, contentList));
		}
		return relationSetNode;
	}
	
	public DefaultMutableTreeNode generateDeviatingClassRelationNode(ClassEdge relation, List<XOCObject> objectListViolatedSource, 
			List<XOCObject> objectListViolatedTarget, String name){
		DefaultMutableTreeNode relationNode = new DefaultMutableTreeNode(new TreeEntry(name));
		String sourceEventuallyCardinality = relation.getSourceEventuallyCardinality().toString();
		String targetEventuallyCardinality = relation.getTargetEventuallyCardinality().toString();
		if(objectListViolatedSource != null && objectListViolatedSource.size()>0){
			relationNode.add(generateObjectSetNode(objectListViolatedSource, xConstants.source_eventually_cardinality + ": " + sourceEventuallyCardinality));
		}
		if(objectListViolatedTarget != null && objectListViolatedTarget.size()>0){
			relationNode.add(generateObjectSetNode(objectListViolatedTarget, xConstants.target_eventually_cardinality + ": " + targetEventuallyCardinality));
		}		
		return relationNode;
	}
	
	public DefaultMutableTreeNode generateDeviatingClassRelationSetNode(List<ClassEdge> relations, 
			Map<ClassEdge, List<XOCObject>> violatedSourceClassRelationVSObjectList, 
			Map<ClassEdge, List<XOCObject>> violatedTargetClassRelationVSObjectList, String name){
		DefaultMutableTreeNode relationSetNode = new DefaultMutableTreeNode(new TreeEntry(name));
		for (ClassEdge relation : relations) {
			String relationName = relation.getEdgeName();
			List<XOCObject> objectListViolatedSource = violatedSourceClassRelationVSObjectList.get(relation);
			List<XOCObject> objectListViolatedTarget = violatedTargetClassRelationVSObjectList.get(relation);
			relationSetNode.add(generateDeviatingClassRelationNode(relation, objectListViolatedSource, 
					objectListViolatedTarget, relationName));
		}
		return relationSetNode;
	}
	

	
	private class TreeEntry {
		private String key;
		private String displaytext;
		
		public TreeEntry(String key) {
			this(key, key);
		}
		
		public TreeEntry(String key, String displaytext) {
			this.key = key;
			this.displaytext = displaytext;
		}
		
		public String getKey() {
			return this.key;
		}
		
		public String getDisplayText() {
			return this.displaytext;
		}
		
		public String toString() {
			return getDisplayText();
		}
	}
	
}