package org.processmining.ocbc.conformance.visualization.panels;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.util.ArrayList;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.JPanel;
import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;

import org.processmining.common.frame.CreateTreePanel;
import org.processmining.common.structure.IntegerPair;
import org.processmining.framework.util.Pair;
import org.processmining.mxgraph.util.mxConstants;
import org.processmining.ocbc.conformance.checkers.MonotonicityChecking;
import org.processmining.ocbc.conformance.visualization.views.ResultTextView;
import org.processmining.ocbc.log.model.XOCAttribute;
import org.processmining.ocbc.log.model.XOCLog;
import org.processmining.ocbc.log.model.XOCObject;
import org.processmining.ocbc.log.model.impl.xConstants;
import org.processmining.ocbc.models.OCBCModel;

import com.fluxicon.slickerbox.components.RoundedPanel;

public class MonotonicityPanel extends RoundedPanel {

	
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	
	private JTree objectModelListTree;
	private JPanel objectListPane;
	private JPanel noInputPane;
	private JPanel deviationPane;
	private ResultTextView resultTextView;

	
	private XOCLog log;
	private OCBCModel model;
	private List<XOCObject> disappearedObjectList = new ArrayList<XOCObject>();
	private List<XOCObject> classChangedObjectList = new ArrayList<XOCObject>();
	
	public MonotonicityPanel(ResultTextView resultTextView, XOCLog log, OCBCModel model){
		this.resultTextView = resultTextView;
		this.log = log;
		this.model = model;
		this.setMaximumSize(new Dimension(230, 1000));
		this.setPreferredSize(new Dimension(230, 1000));
		this.setLayout(new GridLayout(1,2));
		this.setBorder(BorderFactory.createLineBorder(new Color(240, 240, 240)));
		objectListPane = createObjectListHierarchy(log);
		noInputPane = createNoInputHierarchy(model);
		deviationPane = createDeviationHierarchy(resultTextView);	
		
		this.add(objectListPane);
		this.add(deviationPane);
		this.add(noInputPane);			
	}
	
	/**
	 * get the list of deviating objects which disappear;
	 * @return
	 */
	public List<XOCObject> getDisappearedObjectList(){
		return disappearedObjectList;
	}
	
	/**
	 * get the list of deviating objects which change classes;
	 * @return
	 */
	public List<XOCObject> getClassChangedObjectList(){
		return classChangedObjectList;
	}
	
	public CreateTreePanel createDeviationHierarchy(ResultTextView resultTextView){
		MonotonicityChecking monotonicityChecker= resultTextView.getConformanceCheckingResultView().getMonotonicityChecker();
		disappearedObjectList = monotonicityChecker.getDisappearedObjectList();
		classChangedObjectList = monotonicityChecker.getClassChangedObjectList();
		
		DefaultMutableTreeNode root = generateDeviatingObjectListNode(disappearedObjectList,
				classChangedObjectList, mxConstants.object_list);
		CreateTreePanel deviationTreePanel = new CreateTreePanel(root, mxConstants.conformance_checking_output);	
		return deviationTreePanel;
	}
	
	public CreateTreePanel createNoInputHierarchy(final OCBCModel model){
		DefaultMutableTreeNode root = new DefaultMutableTreeNode(new TreeEntry(mxConstants.no_input));					
		CreateTreePanel classModelTreePanel = new CreateTreePanel(root, mxConstants.conformance_checking_input_model);	
		return classModelTreePanel;
	}
	
	/**
	 * create the object list hierarchy (include all objects in the log: normal objects, inexistent objects, disappearing objects, changing-class objects)
	 * @param log
	 */
	public CreateTreePanel createObjectListHierarchy(final XOCLog log){
		DefaultMutableTreeNode root = new DefaultMutableTreeNode(
//				"object model list(" + log.getAllEvents().size() + ")");
				mxConstants.object_list);
		for(XOCObject object : log.getObjectList()){
			DefaultMutableTreeNode objectNode = generateObjectNode(object);
			root.add(objectNode);
		}
		
		CreateTreePanel eventListTreePanel = new CreateTreePanel(root, mxConstants.conformance_checking_input_log);	
//		JTree eventListTree = eventListTreePanel.getJTree();
//		eventListTree.addTreeWillExpandListener(new TreeWillExpandListener() {		
//			public void treeWillExpand(TreeExpansionEvent operation) throws ExpandVetoException {
//				// TODO Auto-generated method stub
//				System.out.println("tree open in event list tree in object existence: " + operation.getPath());				
//				DefaultMutableTreeNode expandedNode = (DefaultMutableTreeNode) operation.getPath().getLastPathComponent();
//				String nodeName = expandedNode.getUserObject().toString();
//				DefaultMutableTreeNode parentNode = (DefaultMutableTreeNode) expandedNode.getParent();
//				String parentName = parentNode.getUserObject().toString();
//				System.out.println("nodeName: " + nodeName);
//				System.out.println("parentName: " + parentName);		
//				if(parentName.equals(mxConstants.event_list)){
//					String eventId = nodeName;			
//					XOCEvent expandedEvent = log.getIdVSEventMap().get(eventId);
//					expandedNode.removeAllChildren();
//					updateEventNode(expandedNode, expandedEvent, false, true, true, false, eventId);						
//				}					
//			}
//
//			public void treeWillCollapse(TreeExpansionEvent arg0) throws ExpandVetoException {
//				// TODO Auto-generated method stub				
//			}
//		});
		return eventListTreePanel;

	}

	/**
	 * generate object node including information of birth, death and class value versions
	 * @param object
	 * @return
	 */
	private DefaultMutableTreeNode generateObjectNode(XOCObject object){
		DefaultMutableTreeNode objectNode = new DefaultMutableTreeNode(new TreeEntry(object.getAttributes().get(xConstants.ID).toString()));
		DefaultMutableTreeNode lifecycleNode = new DefaultMutableTreeNode(new TreeEntry(mxConstants.lifecycle));
		DefaultMutableTreeNode classValueVersionNode = new DefaultMutableTreeNode(new TreeEntry(mxConstants.classVersionList));
		for(IntegerPair period : object.getLifecyclePeriodList()){
			int startEventOrderNum = period.getFirstValue();
			int endEventOrderNum = period.getSecondValue() == null ? log.size() : period.getSecondValue();
			lifecycleNode.add(new DefaultMutableTreeNode(new TreeEntry(mxConstants.startEventOrderNum + " : " + startEventOrderNum + " , "
					+ mxConstants.endEventOrderNum + " : " + endEventOrderNum)));
		}			
		objectNode.add(lifecycleNode);
		
		XOCAttribute classXOCAttribute = object.getXOCAttributes().get(xConstants.CLASS);
		if(classXOCAttribute != null){
			for(int i=0; i<classXOCAttribute.getValueList().size(); i++){
				String value = classXOCAttribute.getValueList().get(i);
				Pair<Integer, Integer> period = classXOCAttribute.getPeriodList().get(i);
				int startEventOrderNum = period.getFirst();
				int endEventOrderNum = period.getSecond() == null ? log.size() : period.getSecond();
				classValueVersionNode.add(new DefaultMutableTreeNode(value + " (" + new TreeEntry(mxConstants.startEventOrderNum + " : " + startEventOrderNum + " , "
						+ mxConstants.endEventOrderNum + " : " + endEventOrderNum + "）" )));
			}	
		}
		objectNode.add(classValueVersionNode);
		
		return objectNode;
	}
	
	
	private DefaultMutableTreeNode generateDeviatingObjectListNode(List<XOCObject> disappearedObjectList, 
			List<XOCObject> classChangedObjectList, String name){
		DefaultMutableTreeNode elementSetNode = new DefaultMutableTreeNode(new TreeEntry(name));
		DefaultMutableTreeNode disappearingSetNode = new DefaultMutableTreeNode(new TreeEntry(mxConstants.disappearingObjectList));
		DefaultMutableTreeNode classChangingSetNode = new DefaultMutableTreeNode(new TreeEntry(mxConstants.classChangedObjectList));		
		for (XOCObject element : disappearedObjectList) {
			disappearingSetNode.add(generateObjectNode(element));
		}
		for (XOCObject element : classChangedObjectList) {
			classChangingSetNode.add(generateObjectNode(element));
		}
		if(disappearedObjectList.size() > 0){
			elementSetNode.add(disappearingSetNode);
		}
		if(classChangedObjectList.size() > 0){
			elementSetNode.add(classChangingSetNode);
		}
		return elementSetNode;
	}
	
	private class TreeEntry {
		private String key;
		private String displaytext;
		
		public TreeEntry(String key) {
			this(key, key);
		}
		
		public TreeEntry(String key, String displaytext) {
			this.key = key;
			this.displaytext = displaytext;
		}
		
		public String getKey() {
			return this.key;
		}
		
		public String getDisplayText() {
			return this.displaytext;
		}
		
		public String toString() {
			return getDisplayText();
		}
	}
	
}