package org.processmining.ocbc.conformance.visualization.panels;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.util.List;
import java.util.Map;

import javax.swing.BorderFactory;
import javax.swing.JPanel;
import javax.swing.JTree;
import javax.swing.event.TreeExpansionEvent;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.event.TreeWillExpandListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.ExpandVetoException;

import org.processmining.common.frame.CreateTreePanel;
import org.processmining.framework.util.Pair;
import org.processmining.mxgraph.util.mxConstants;
import org.processmining.ocbc.conformance.checkers.ObjectModelValidityChecking;
import org.processmining.ocbc.conformance.visualization.views.ResultTextView;
import org.processmining.ocbc.log.model.XOCEvent;
import org.processmining.ocbc.log.model.XOCLog;
import org.processmining.ocbc.log.model.XOCORelation;
import org.processmining.ocbc.log.model.XOCObject;
import org.processmining.ocbc.log.model.XOCObjectModel;
import org.processmining.ocbc.log.model.impl.xConstants;
import org.processmining.ocbc.models.ClassEdge;
import org.processmining.ocbc.models.ClassNode;
import org.processmining.ocbc.models.OCBCModel;

import com.fluxicon.slickerbox.components.RoundedPanel;

public class OMValidityPanel extends RoundedPanel {

	
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	
	private JTree objectModelListTree;
	private JPanel objectModelListPane;
	private JPanel classModelPane;
	private JPanel deviationPane;
	private ResultTextView resultTextView;
	
	private List<XOCEvent> deviatingEventList;
	
	private XOCLog log;
	private OCBCModel model;
	private Map<XOCEvent, List<XOCORelation>> eventVSDeviatingObjectRelationListMap;
	private Map<XOCORelation, List<String>> deviatingObjectRelationVSContentListMap;
	private Map<ClassEdge, List<XOCObject>> violatedSourceClassRelationVSObjectList;
	private Map<ClassEdge, List<XOCObject>> violatedTargetClassRelationVSObjectList;
	private Map<ClassEdge, List<Pair<Integer, Integer>>> violatedClassRelationVSStartAndEndPairList;
	private Map<XOCObject, List<Pair<Integer, Integer>>> deviatingObjectVSStartAndEndPairList;
	
	private Map<XOCEvent, Map<XOCORelation, List<String>>>  eventVSDeviatingRelationVSContentList;
	
	private Map<XOCEvent, Map<ClassEdge, List<XOCObject>>>  eventVSDeviatingClassRelationVSSourceObjectList;
	private Map<XOCEvent, Map<ClassEdge, List<XOCObject>>>  eventVSDeviatingClassRelationVSTargetObjectList;
	
	public OMValidityPanel(ResultTextView resultTextView, XOCLog log, OCBCModel model){
		this.resultTextView = resultTextView;
		this.log = log;
		this.model = model;
		this.setMaximumSize(new Dimension(230, 1000));
		this.setPreferredSize(new Dimension(230, 1000));
		this.setLayout(new GridLayout(1,2));
		this.setBorder(BorderFactory.createLineBorder(new Color(240, 240, 240)));
		objectModelListPane = createObjectModelListHierarchy(log);
		classModelPane = createClassModelHierarchy(model);
		deviationPane = createDeviationHierarchy(resultTextView);

		this.add(objectModelListPane);
		this.add(deviationPane);
		this.add(classModelPane);	
		
	}
	
	/**
	 * get the list of deviating events which have invalid object models
	 * @return
	 */
	public List<XOCEvent> getDeviatingEventList(){
		return deviatingEventList;
	}
	
	public JPanel createDeviationHierarchy(ResultTextView resultTextView){
		ObjectModelValidityChecking objectModelValidityChecker = resultTextView.getConformanceCheckingResultView().getObjectModelValidityChecker();
		deviatingEventList = objectModelValidityChecker.getDeviatingEventList();
		eventVSDeviatingObjectRelationListMap = objectModelValidityChecker.getEventVSDeviatingObjectRelationListMap();
		deviatingObjectRelationVSContentListMap = objectModelValidityChecker.getDeviatingObjectRelationVSContentListMap();
		
		eventVSDeviatingRelationVSContentList = objectModelValidityChecker.getEventVSDeviatingObjectRelationVSContentListMap();
		
		eventVSDeviatingClassRelationVSSourceObjectList = objectModelValidityChecker.getEventVSDeviatingClassRelationVSSourceObjectListMap();
		eventVSDeviatingClassRelationVSTargetObjectList = objectModelValidityChecker.getEventVSDeviatingClassRelationVSTargetObjectListMap();
		
		violatedSourceClassRelationVSObjectList = objectModelValidityChecker.getViolatedSourceClassRelationVSObjectListMap();
		violatedTargetClassRelationVSObjectList = objectModelValidityChecker.getViolatedTargetClassRelationVSObjectListMap();		
		violatedClassRelationVSStartAndEndPairList = objectModelValidityChecker.getViolatedClassRelationVSStartAndEndPairListMap();
		deviatingObjectVSStartAndEndPairList = objectModelValidityChecker.getDeviatingObjectVSStartAndEndPairListMap();
		
		DefaultMutableTreeNode root = new DefaultMutableTreeNode(
//				"deviating object model list(" + deviatingEventList.size() + ")");
				mxConstants.object_model_list);
		
		for(XOCEvent event : deviatingEventList){
	
			String eventId = event.getAttributes().get(xConstants.ID).toString();
//			System.out.println("createDeviationHierarchy in validity" + eventId);
//			List<XOCORelation> relationList = eventVSDeviatingObjectRelationListMap.get(event);
			
			DefaultMutableTreeNode objectModle = new DefaultMutableTreeNode(new TreeEntry(mxConstants.object_model + "-" + eventId));
			objectModle.add(new DefaultMutableTreeNode(new TreeEntry(mxConstants.object_relation_list)));			
			root.add(objectModle);
		}
		
//		if(root.getChildCount() > 0){
//			deviatingFlag = true;
//		}
		
		CreateTreePanel deviationTreePanel = new CreateTreePanel(root, mxConstants.conformance_checking_output);	
		//add listeners
		final JTree mdtree = deviationTreePanel.getJTree();		
		mdtree.addTreeSelectionListener(new TreeSelectionListener() {		
		    public void valueChanged(TreeSelectionEvent e) {
		    	System.out.println("tree node selection change in deviation");
		        DefaultMutableTreeNode node = (DefaultMutableTreeNode)
		        		mdtree.getLastSelectedPathComponent();
		        String nodeName = node.getUserObject().toString();
		        System.out.println("lastPath: " + nodeName);	        
		        int level = node.getPath().length;
		        System.out.println("level: " + level);		        
		        System.out.println("e.getPath(): " + e.getPath().toString());		        
		        objectModelListTree.setSelectionPath(e.getPath());
		        objectModelListTree.scrollPathToVisible(e.getPath());
		    }
		});
		
		mdtree.addTreeWillExpandListener(new TreeWillExpandListener() {		
			public void treeWillExpand(TreeExpansionEvent operation) throws ExpandVetoException {
				// TODO Auto-generated method stub
				System.out.println("tree open in deviating tree: " + operation.getPath());				
				DefaultMutableTreeNode expandedNode = (DefaultMutableTreeNode) operation.getPath().getLastPathComponent();
				String nodeName = expandedNode.getUserObject().toString();
				System.out.println("nodeName: " + nodeName);
				String firstPart = nodeName.split("-")[0];			
				if(firstPart.equals(mxConstants.object_model)){
					String eventId = nodeName.split("-")[1];
					System.out.println("eventId: " + eventId);					
					XOCEvent expandedEvent = log.getIdVSEventMap().get(eventId);
					expandedNode.removeAllChildren();

					//if there exist deviating object relations
					if(eventVSDeviatingRelationVSContentList.get(expandedEvent) != null && eventVSDeviatingRelationVSContentList.get(expandedEvent).keySet().size()>0){
						expandedNode.add(generateDeviatingObjectRelationSetNode(eventVSDeviatingRelationVSContentList, expandedEvent, mxConstants.object_relation_list));	
					}

					//if there exist violated class relations
					if((eventVSDeviatingClassRelationVSSourceObjectList.get(expandedEvent) != null && eventVSDeviatingClassRelationVSSourceObjectList.get(expandedEvent).keySet().size()>0)
							|| (eventVSDeviatingClassRelationVSTargetObjectList.get(expandedEvent) != null && eventVSDeviatingClassRelationVSTargetObjectList.get(expandedEvent).keySet().size()>0)){
						expandedNode.add(generateDeviatingClassRelationSetNode(eventVSDeviatingClassRelationVSSourceObjectList,
								eventVSDeviatingClassRelationVSTargetObjectList, expandedEvent, mxConstants.class_relation_list));
					}				
				}					
			}

			public void treeWillCollapse(TreeExpansionEvent arg0) throws ExpandVetoException {
				// TODO Auto-generated method stub				
			}
		});
		return deviationTreePanel;
	}
	
	public CreateTreePanel createClassModelHierarchy(final OCBCModel model){
		DefaultMutableTreeNode root = new DefaultMutableTreeNode(mxConstants.class_model);
		root.add(generateClassSetNode(model.getAllClasses(), mxConstants.class_list));
		root.add(generateClassRelationSetNode(model.getAllClassRelations(), mxConstants.class_relation_list));					
		CreateTreePanel classModelPanel = new CreateTreePanel(root, mxConstants.conformance_checking_input_model);	
		return classModelPanel;
	}
	
	/**
	 * create the object model list hierarchy
	 * @param log
	 */
	public CreateTreePanel createObjectModelListHierarchy(final XOCLog log){
		DefaultMutableTreeNode root = new DefaultMutableTreeNode(
//				"object model list(" + log.getAllEvents().size() + ")");
				mxConstants.object_model_list);
		for(XOCEvent event : log.getAllEvents()){
			String eventId = event.getAttributes().get(xConstants.ID).toString();
			XOCObjectModel completeObjectModel = event.getObjectModel();
			DefaultMutableTreeNode objectModle = new DefaultMutableTreeNode(new TreeEntry(mxConstants.object_model + "-" + eventId));	
			objectModle.add(new DefaultMutableTreeNode(new TreeEntry("fakingNode")));
//			objectModle.add(generateObjectSetNode(completeObjectModel.getObjectSet(), mxConstants.object_list));
//			objectModle.add(generateObjectRelationSetNode(completeObjectModel.getObjectRelationSet(), mxConstants.object_relation_list));			
			root.add(objectModle);
		}
		
		CreateTreePanel objectModelListPanel = new CreateTreePanel(root, mxConstants.conformance_checking_input_log);	
		//add listeners
		objectModelListTree = objectModelListPanel.getJTree();			
		objectModelListTree.addTreeWillExpandListener(new TreeWillExpandListener() {		
			public void treeWillExpand(TreeExpansionEvent event) throws ExpandVetoException {
				// TODO Auto-generated method stub
				DefaultMutableTreeNode expandedNode = (DefaultMutableTreeNode) event.getPath().getLastPathComponent();
				String nodeName = expandedNode.getUserObject().toString();
				String firstPart = nodeName.split("-")[0];			
				if(firstPart.equals(mxConstants.object_model)){
					String eventId = nodeName.split("-")[1];
					XOCEvent expandedEvent = log.getIdVSEventMap().get(eventId);
					expandedNode.removeAllChildren();
					XOCObjectModel completeObjectModel = log.createCompleteObjectModel(expandedEvent);
//					expandedNode.add(generateObjectSetNode(completeObjectModel.getObjectSet(),
//					TREE_NODE_OBJECTS + "(" +completeObjectModel.getObjectSet().size() + ")"));
//			expandedNode.add(generateObjectRelationSetNode(completeObjectModel.getObjectRelationSet(),
//					TREE_NODE_RELATIONS + "(" +completeObjectModel.getObjectRelationSet().size() + ")"));
			expandedNode.add(generateObjectSetNode(completeObjectModel.getObjectSet(),
					mxConstants.object_list));
			expandedNode.add(generateObjectRelationSetNode(completeObjectModel.getObjectRelationSet(),
					mxConstants.object_relation_list));
					
				}

				System.out.println("tree open: " + event.getPath());
				System.out.println("lastPath: " + nodeName);
			}
			
			public void treeWillCollapse(TreeExpansionEvent event) throws ExpandVetoException {
				// TODO Auto-generated method stub
			}
		});
		return objectModelListPanel;
	}

	private DefaultMutableTreeNode generateObjectNode(XOCObject object){
		DefaultMutableTreeNode objectNode = new DefaultMutableTreeNode(new TreeEntry(object.getAttributes().get("id").toString()));
		for(String key : object.getAttributes().keySet()){
			objectNode.add(new DefaultMutableTreeNode(new TreeEntry(key + " : " + object.getAttributes().get(key).toString())));
		}		
		return  objectNode;
	}
	
	private DefaultMutableTreeNode generateObjectSetNode(List<XOCObject> objects, String name){
		DefaultMutableTreeNode objectSetNode = new DefaultMutableTreeNode(new TreeEntry(name));
		for (XOCObject object : objects) {
			objectSetNode.add(generateObjectNode(object));
		}
		return objectSetNode;
	}

	private DefaultMutableTreeNode generateObjectRelationNode(XOCORelation relation){
		DefaultMutableTreeNode relationNode = new DefaultMutableTreeNode(new TreeEntry(relation.getAttributes().get("id").toString()));
		for(String key : relation.getAttributes().keySet()){
			relationNode.add(new DefaultMutableTreeNode(new TreeEntry(key + " : " + relation.getAttributes().get(key).toString())));
		}		
		return  relationNode;
	}
	
	private DefaultMutableTreeNode generateObjectRelationSetNode(List<XOCORelation> relations, String name){
		DefaultMutableTreeNode relationSetNode = new DefaultMutableTreeNode(new TreeEntry(name));
		for (XOCORelation relation : relations) {
			relationSetNode.add(generateObjectRelationNode(relation));
		}
		return relationSetNode;
	}
	
	private DefaultMutableTreeNode generateObjectModelNode(XOCObjectModel objectModel){
		DefaultMutableTreeNode objectModelNode = new DefaultMutableTreeNode(new TreeEntry("OM"));
		List<XOCObject> objectList = objectModel.getObjectSet();
		List<XOCORelation> objectRelationList = objectModel.getObjectRelationSet();
		objectModelNode.add(generateObjectSetNode(objectList, "Objects"));
		objectModelNode.add(generateObjectRelationSetNode(objectRelationList, "Relations"));
		return objectModelNode;
	}
	
	private DefaultMutableTreeNode generateClassNode(ClassNode classNode){
		DefaultMutableTreeNode classTreeNode = new DefaultMutableTreeNode(new TreeEntry(classNode.getNodeName()));
		//in future, add the attribute node if the class node has attributes
//		classTreeNode.add(new DefaultMutableTreeNode(new TreeEntry("node_name : " + classNode.getNodeName())));
		return  classTreeNode;
	}
	
	private DefaultMutableTreeNode generateClassSetNode(List<ClassNode> classList, String name){
		DefaultMutableTreeNode classSetNode = new DefaultMutableTreeNode(new TreeEntry(name));
		for (ClassNode classNode : classList) {
			classSetNode.add(generateClassNode(classNode));
		}
		return classSetNode;
	}
	
	private DefaultMutableTreeNode generateClassRelationNode(ClassEdge relation){
		DefaultMutableTreeNode relationNode = new DefaultMutableTreeNode(new TreeEntry(relation.getEdgeName()));
		relationNode.add(new DefaultMutableTreeNode(
				new TreeEntry(xConstants.relationship_type + ": " + relation.getEdgeName())));	
		relationNode.add(new DefaultMutableTreeNode(
				new TreeEntry(xConstants.source_class + ": " + relation.getSourceNode().getNodeName())));	
		relationNode.add(new DefaultMutableTreeNode(
				new TreeEntry(xConstants.target_class + ": " + relation.getTargetNode().getNodeName())));
		relationNode.add(new DefaultMutableTreeNode(
				new TreeEntry(xConstants.source_always_cardinality + ": " + relation.getSourceAlwaysCardinality().toString())));	
		relationNode.add(new DefaultMutableTreeNode(
				new TreeEntry(xConstants.source_eventually_cardinality + ": " + relation.getSourceEventuallyCardinality().toString())));	
		relationNode.add(new DefaultMutableTreeNode(
				new TreeEntry(xConstants.target_always_cardinality + ": " + relation.getTargetAlwaysCardinality().toString())));	
		relationNode.add(new DefaultMutableTreeNode(
				new TreeEntry(xConstants.target_eventually_cardinality + ": " + relation.getTargetEventuallyCardinality().toString())));				
		return  relationNode;
	}
	
	private DefaultMutableTreeNode generateClassRelationSetNode(List<ClassEdge> relations, String name){
		DefaultMutableTreeNode relationSetNode = new DefaultMutableTreeNode(new TreeEntry(name));
		for (ClassEdge relation : relations) {
			relationSetNode.add(generateClassRelationNode(relation));
		}
		return relationSetNode;
	}
	
	private DefaultMutableTreeNode generateDeviatingObjectRelationNode(XOCORelation relation, List<String> contentList){
		DefaultMutableTreeNode relationNode = new DefaultMutableTreeNode(new TreeEntry(relation.getAttributes().get("id").toString()));
		for(String content : contentList){
			relationNode.add(new DefaultMutableTreeNode(new TreeEntry(content)));
		}		
		return  relationNode;
	}
	
//	public DefaultMutableTreeNode generateDeviatingObjectRelationSetNode(List<XOCORelation> relations, 
//			Map<XOCORelation, List<String>> deviatingObjectRelationVSContentListMap, String name){
	public DefaultMutableTreeNode generateDeviatingObjectRelationSetNode(Map<XOCEvent, Map<XOCORelation, List<String>>> eventVSDeviatingRelationVSContentList, 
			XOCEvent event, String name){
		DefaultMutableTreeNode relationSetNode = new DefaultMutableTreeNode(new TreeEntry(name));
		for (XOCORelation relation : eventVSDeviatingRelationVSContentList.get(event).keySet()) {
			List<String> contentList = eventVSDeviatingRelationVSContentList.get(event).get(relation);
			relationSetNode.add(generateDeviatingObjectRelationNode(relation, contentList));
		}	
		return relationSetNode;
	}
	
	/**
	 * generate the node for a deviating class relation;
	 * if there are no deviating objects for one cardinality, do not generate the corresponding node;
	 * @param relation
	 * @param objectListViolatedSource
	 * @param objectListViolatedTarget
	 * @param name
	 * @return
	 */
	public DefaultMutableTreeNode generateDeviatingClassRelationNode(ClassEdge relation, List<XOCObject> objectListViolatedSource, 
			List<XOCObject> objectListViolatedTarget, String name){
		DefaultMutableTreeNode relationNode = new DefaultMutableTreeNode(new TreeEntry(name));
		String sourceAlwaysCardinality = relation.getSourceAlwaysCardinality().toString();
		String targetAlwaysCardinality = relation.getTargetAlwaysCardinality().toString();
		if(objectListViolatedSource != null && objectListViolatedSource.size()>0){
			relationNode.add(generateObjectSetNode(objectListViolatedSource, xConstants.source_always_cardinality + ": " + sourceAlwaysCardinality));
		}
		if(objectListViolatedTarget != null && objectListViolatedTarget.size()>0){
			relationNode.add(generateObjectSetNode(objectListViolatedTarget, xConstants.target_always_cardinality + ": " + targetAlwaysCardinality));
		}		
		return relationNode;
	}
	
	public DefaultMutableTreeNode generateDeviatingClassRelationSetNode(List<ClassEdge> relations, 
			Map<ClassEdge, List<XOCObject>> violatedSourceClassRelationVSObjectList, 
			Map<ClassEdge, List<XOCObject>> violatedTargetClassRelationVSObjectList, String name){
		DefaultMutableTreeNode relationSetNode = new DefaultMutableTreeNode(new TreeEntry(name));
		for (ClassEdge relation : relations) {
			String relationName = relation.getEdgeName();
			List<XOCObject> objectListViolatedSource = violatedSourceClassRelationVSObjectList.get(relation);
			List<XOCObject> objectListViolatedTarget = violatedTargetClassRelationVSObjectList.get(relation);
			relationSetNode.add(generateDeviatingClassRelationNode(relation, objectListViolatedSource, 
					objectListViolatedTarget, relationName));
		}
		return relationSetNode;
	}

	/**
	 * generate the deviating class relation set node
	 * @param eventVSDeviatingClassRelationVSSourceObjectList
	 * @param eventVSDeviatingClassRelationVSTargetObjectList
	 * @param event
	 * @param name
	 * @return
	 */
	public DefaultMutableTreeNode generateDeviatingClassRelationSetNode(Map<XOCEvent, Map<ClassEdge, List<XOCObject>>>  eventVSDeviatingClassRelationVSSourceObjectList,
			Map<XOCEvent, Map<ClassEdge, List<XOCObject>>>  eventVSDeviatingClassRelationVSTargetObjectList, XOCEvent event, String name){		
		DefaultMutableTreeNode relationSetNode = new DefaultMutableTreeNode(new TreeEntry(name));
		for (ClassEdge relation : model.getAllClassRelations()) {
			String relationName = relation.getEdgeName();
			List<XOCObject> targetObjectList = null;
			List<XOCObject> sourceObjectList = null;
			
			if(eventVSDeviatingClassRelationVSTargetObjectList.containsKey(event) && eventVSDeviatingClassRelationVSTargetObjectList.get(event).containsKey(relation)){
				targetObjectList = eventVSDeviatingClassRelationVSTargetObjectList.get(event).get(relation);
			}
			if(eventVSDeviatingClassRelationVSSourceObjectList.containsKey(event) && eventVSDeviatingClassRelationVSSourceObjectList.get(event).containsKey(relation)){
				sourceObjectList = eventVSDeviatingClassRelationVSSourceObjectList.get(event).get(relation);
			}
			
			//if there existing deviating objects
			if((targetObjectList != null && targetObjectList.size()>0)
					|| (sourceObjectList != null && sourceObjectList.size()>0)){
				relationSetNode.add(generateDeviatingClassRelationNode(relation, targetObjectList, 
						sourceObjectList, relationName));
			}
			
		}
		return relationSetNode;
	}
	
	//filter violated class relation and deviating objects based on one input event
//	public DefaultMutableTreeNode generateDeviatingClassRelationSetNodeForOneEvent(XOCEvent event, String name){
//		List<ClassEdge> violatedClassRelationListForEvent = new ArrayList<ClassEdge>();
//		Map<ClassEdge, List<XOCObject>> violatedSourceClassRelationVSObjectListForEvent = new HashMap<ClassEdge, List<XOCObject>>(); 
//		Map<ClassEdge, List<XOCObject>> violatedTargetClassRelationVSObjectListForEvent = new HashMap<ClassEdge, List<XOCObject>>(); 
//		
//		for(ClassEdge violatedClassRelation : model.getAllClassRelations()){
//			int eventIndex = event.getOrderNum();
//			//if the class relation is violated
//			if(violatedClassRelationVSStartAndEndPairList.containsKey(violatedClassRelation)){
//				for(Pair<Integer, Integer> startAndEndPair : violatedClassRelationVSStartAndEndPairList.get(violatedClassRelation)){
//					int startIndex = startAndEndPair.getFirst();
//					int endIndex;
//					if(startAndEndPair.getSecond() == null){
//						endIndex = log.getAllEvents().size() + 1;
//					}
//					else{
//						endIndex = startAndEndPair.getSecond();
//					}
//					System.out.println("startIndex: " + startIndex);
//					System.out.println("endIndex: " + endIndex);
//					//if the event is between the start and end
//					if(eventIndex >= startIndex && eventIndex < endIndex){
//						violatedClassRelationListForEvent.add(violatedClassRelation);
//						violatedSourceClassRelationVSObjectListForEvent.put(violatedClassRelation, new ArrayList<XOCObject>());
//						violatedTargetClassRelationVSObjectListForEvent.put(violatedClassRelation, new ArrayList<XOCObject>());
//						
//						if(violatedSourceClassRelationVSObjectList.containsKey(violatedClassRelation)){
//							for(XOCObject object : violatedSourceClassRelationVSObjectList.get(violatedClassRelation)){
//								for(Pair<Integer, Integer> startAndEndPairObject : deviatingObjectVSStartAndEndPairList.get(object)){
//									int startIndexObject = startAndEndPairObject.getFirst();
//									int endIndexObject;
//									if(startAndEndPairObject.getSecond() == null){
//										endIndexObject = log.getAllEvents().size() + 1;
//									}
//									else{
//										endIndexObject = startAndEndPairObject.getSecond();
//									}
//									
//									if(eventIndex >= startIndexObject && eventIndex < endIndexObject){
//										violatedSourceClassRelationVSObjectListForEvent.get(violatedClassRelation).add(object);
//										break;
//									}
//								}
//							}
//						}
//						if(violatedTargetClassRelationVSObjectList.containsKey(violatedClassRelation)){
//							for(XOCObject object : violatedTargetClassRelationVSObjectList.get(violatedClassRelation)){
//								for(Pair<Integer, Integer> startAndEndPairObject : deviatingObjectVSStartAndEndPairList.get(object)){
//									int startIndexObject = startAndEndPairObject.getFirst();
//									int endIndexObject;
//									if(startAndEndPairObject.getSecond() == null){
//										endIndexObject = log.getAllEvents().size() + 1;
//									}
//									else{
//										endIndexObject = startAndEndPairObject.getSecond();
//									}
//									if(eventIndex >= startIndexObject && eventIndex < endIndexObject){
//										violatedTargetClassRelationVSObjectListForEvent.get(violatedClassRelation).add(object);
//										break;
//									}
//								}
//							}
//						}
//						
//						break;
//					}
//				}
//			}
//		}
//
//		DefaultMutableTreeNode relationSetNode = generateDeviatingClassRelationSetNode(violatedClassRelationListForEvent, 
//				violatedSourceClassRelationVSObjectListForEvent, 
//				violatedTargetClassRelationVSObjectListForEvent, name);
//		
//		return relationSetNode;
//	}
	
	private class TreeEntry {
		private String key;
		private String displaytext;
		
		public TreeEntry(String key) {
			this(key, key);
		}
		
		public TreeEntry(String key, String displaytext) {
			this.key = key;
			this.displaytext = displaytext;
		}
		
		public String getKey() {
			return this.key;
		}
		
		public String getDisplayText() {
			return this.displaytext;
		}
		
		public String toString() {
			return getDisplayText();
		}
	}
	
}