package org.processmining.ocbc.conformance.visualization.panels;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.util.List;
import java.util.Map;

import javax.swing.BorderFactory;
import javax.swing.JPanel;
import javax.swing.JTree;
import javax.swing.event.TreeExpansionEvent;
import javax.swing.event.TreeWillExpandListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.ExpandVetoException;

import org.processmining.common.frame.CreateTreePanel;
import org.processmining.mxgraph.util.mxConstants;
import org.processmining.ocbc.conformance.checkers.ObjectExistenceChecking;
import org.processmining.ocbc.conformance.visualization.views.ResultTextView;
import org.processmining.ocbc.log.model.XOCEvent;
import org.processmining.ocbc.log.model.XOCLog;
import org.processmining.ocbc.log.model.XOCORelation;
import org.processmining.ocbc.log.model.XOCObject;
import org.processmining.ocbc.log.model.XOCObjectModel;
import org.processmining.ocbc.log.model.impl.xConstants;
import org.processmining.ocbc.models.OCBCModel;

import com.fluxicon.slickerbox.components.RoundedPanel;

public class ObjectExistencePanel extends RoundedPanel {

	
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	
	private JTree objectModelListTree;
	private JPanel eventListPane;
	private JPanel noInputPane;
	private JPanel deviationPane;
	private ResultTextView resultTextView;

	
	private XOCLog log;
	private OCBCModel model;
	private Map<XOCEvent, List<XOCObject>> eventVSInexistentObjectList;
	private List<XOCEvent> deviatingEventList;
	
	public ObjectExistencePanel(ResultTextView resultTextView, XOCLog log, OCBCModel model){
		this.resultTextView = resultTextView;
		this.log = log;
		this.model = model;
		this.setMaximumSize(new Dimension(230, 1000));
		this.setPreferredSize(new Dimension(230, 1000));
		this.setLayout(new GridLayout(1,2));
		this.setBorder(BorderFactory.createLineBorder(new Color(240, 240, 240)));
		eventListPane = createEventListHierarchy(log);
		noInputPane = createNoInputHierarchy(model);
		deviationPane = createDeviationHierarchy(resultTextView);	
		
		this.add(eventListPane);
		this.add(deviationPane);
		this.add(noInputPane);			
	}
	
	/**
	 * get the list of deviating events which refer to inexistent objects;
	 * @return
	 */
	public List<XOCEvent> getDeviatingEventList(){
		return deviatingEventList;
	}
	
	public CreateTreePanel createDeviationHierarchy(ResultTextView resultTextView){
		ObjectExistenceChecking objectExistenceChecker= resultTextView.getConformanceCheckingResultView().getObjectExistenceChecker();
		eventVSInexistentObjectList = objectExistenceChecker.getEventVSInexistentObjectListMap();
		deviatingEventList = objectExistenceChecker.getDeviatingEventList();
		DefaultMutableTreeNode root = generateDeviatingEventSetNode(deviatingEventList,
				eventVSInexistentObjectList, mxConstants.event_list);
		CreateTreePanel deviationTreePanel = new CreateTreePanel(root, mxConstants.conformance_checking_output);	
		return deviationTreePanel;
	}
	
	public CreateTreePanel createNoInputHierarchy(final OCBCModel model){
		DefaultMutableTreeNode root = new DefaultMutableTreeNode(new TreeEntry(mxConstants.no_input));					
		CreateTreePanel classModelTreePanel = new CreateTreePanel(root, mxConstants.conformance_checking_input_model);	
		return classModelTreePanel;
	}
	
	/**
	 * create the object model list hierarchy
	 * @param log
	 */
	public CreateTreePanel createEventListHierarchy(final XOCLog log){
		DefaultMutableTreeNode root = new DefaultMutableTreeNode(
//				"object model list(" + log.getAllEvents().size() + ")");
				mxConstants.event_list);
		for(XOCEvent event : log.getAllEvents()){
			String eventId = event.getAttributes().get(xConstants.ID).toString();
//			DefaultMutableTreeNode eventNode = generateEventNode(event, false, true, true, false, eventId);
			DefaultMutableTreeNode eventNode = new DefaultMutableTreeNode(new TreeEntry(eventId));
			eventNode.add(new DefaultMutableTreeNode(new TreeEntry("fakingNode")));
			root.add(eventNode);
		}
		
		CreateTreePanel eventListTreePanel = new CreateTreePanel(root, mxConstants.conformance_checking_input_log);	
		JTree eventListTree = eventListTreePanel.getJTree();
		eventListTree.addTreeWillExpandListener(new TreeWillExpandListener() {		
			public void treeWillExpand(TreeExpansionEvent operation) throws ExpandVetoException {
				// TODO Auto-generated method stub
				System.out.println("tree open in event list tree in object existence: " + operation.getPath());				
				DefaultMutableTreeNode expandedNode = (DefaultMutableTreeNode) operation.getPath().getLastPathComponent();
				String nodeName = expandedNode.getUserObject().toString();
				DefaultMutableTreeNode parentNode = (DefaultMutableTreeNode) expandedNode.getParent();
				String parentName = parentNode.getUserObject().toString();
				System.out.println("nodeName: " + nodeName);
				System.out.println("parentName: " + parentName);		
				if(parentName.equals(mxConstants.event_list)){
					String eventId = nodeName;			
					XOCEvent expandedEvent = log.getIdVSEventMap().get(eventId);
					expandedNode.removeAllChildren();
					updateEventNode(expandedNode, expandedEvent, false, true, true, false, eventId);						
				}					
			}

			public void treeWillCollapse(TreeExpansionEvent arg0) throws ExpandVetoException {
				// TODO Auto-generated method stub				
			}
		});
		return eventListTreePanel;

	}

	private DefaultMutableTreeNode generateObjectNode(XOCObject object){
		DefaultMutableTreeNode objectNode = new DefaultMutableTreeNode(new TreeEntry(object.getAttributes().get("id").toString()));
		for(String key : object.getAttributes().keySet()){
			objectNode.add(new DefaultMutableTreeNode(new TreeEntry(key + " : " + object.getAttributes().get(key).toString())));
		}		
		return  objectNode;
	}
	
	private DefaultMutableTreeNode generateObjectSetNode(List<XOCObject> objects, String name){
		DefaultMutableTreeNode objectSetNode = new DefaultMutableTreeNode(new TreeEntry(name));
		if(objects != null){
			for (XOCObject object : objects) {
				objectSetNode.add(generateObjectNode(object));
			}
		}	
		return objectSetNode;
	}

	private DefaultMutableTreeNode generateObjectRelationNode(XOCORelation relation){
		DefaultMutableTreeNode relationNode = new DefaultMutableTreeNode(new TreeEntry(relation.getAttributes().get("id").toString()));
		for(String key : relation.getAttributes().keySet()){
			relationNode.add(new DefaultMutableTreeNode(new TreeEntry(key + " : " + relation.getAttributes().get(key).toString())));
		}		
		return  relationNode;
	}
	
	private DefaultMutableTreeNode generateObjectRelationSetNode(List<XOCORelation> relations, String name){
		DefaultMutableTreeNode relationSetNode = new DefaultMutableTreeNode(new TreeEntry(name));
		for (XOCORelation relation : relations) {
			relationSetNode.add(generateObjectRelationNode(relation));
		}
		return relationSetNode;
	}
	
	private DefaultMutableTreeNode generateObjectModelNode(XOCObjectModel objectModel, String name){
		DefaultMutableTreeNode objectModelNode = new DefaultMutableTreeNode(new TreeEntry(name));
		List<XOCObject> objectList = objectModel.getObjectSet();
		List<XOCORelation> objectRelationList = objectModel.getObjectRelationSet();
		objectModelNode.add(generateObjectSetNode(objectList, mxConstants.object_list));
		objectModelNode.add(generateObjectRelationSetNode(objectRelationList, mxConstants.object_relation_list));
		return objectModelNode;
	}
	
	private DefaultMutableTreeNode generateDeviatingEventNode(List<XOCObject> objectList, String name){
		DefaultMutableTreeNode elementSetNode = new DefaultMutableTreeNode(new TreeEntry(name));
		for (XOCObject element : objectList) {
			elementSetNode.add(generateObjectNode(element));
		}
		return elementSetNode;
	}
	
	private DefaultMutableTreeNode generateDeviatingEventSetNode(List<XOCEvent> deviatingEventList, 
			Map<XOCEvent, List<XOCObject>> eventVSObjectListOfUnrelatedClasses, String name){
		DefaultMutableTreeNode elementSetNode = new DefaultMutableTreeNode(new TreeEntry(name));
		for (XOCEvent element : deviatingEventList) {
			String eventId = element.getAttributes().get(xConstants.ID).toString();
			List<XOCObject> objectList = eventVSObjectListOfUnrelatedClasses.get(element);
			elementSetNode.add(generateDeviatingEventNode(objectList, eventId));
		}
		return elementSetNode;
	}
	

	
	private DefaultMutableTreeNode generateObjectReferenceListNode(List<XOCObject> objectReferenceList, String name){
		DefaultMutableTreeNode listNode = new DefaultMutableTreeNode(new TreeEntry(name));
		for (XOCObject element : objectReferenceList) {
			listNode.add(generateObjectNode(element));
		}
		return listNode;
	}
	
	private DefaultMutableTreeNode generateEventNode(XOCEvent event, boolean displayAttributeListFlag, boolean displayObjectReferenceList,
			boolean displayNormalObjectModelFlag, boolean displayUpdatedObjectModelFlag, String name){
		DefaultMutableTreeNode eventNode = new DefaultMutableTreeNode(new TreeEntry(name));
		if(displayObjectReferenceList){
			eventNode.add(generateObjectReferenceListNode(event.getRelatedObjects(), mxConstants.object_reference_list));
		}
//		if(displayNormalObjectModelFlag){
//			XOCObjectModel completeObjectModel = log.createCompleteObjectModel(event);
//			eventNode.add(generateObjectModelNode(completeObjectModel, mxConstants.object_model));
//		}
		return eventNode;
	}
	
	private void updateEventNode(DefaultMutableTreeNode eventNode, XOCEvent event, boolean displayAttributeListFlag, boolean displayObjectReferenceList,
			boolean displayNormalObjectModelFlag, boolean displayUpdatedObjectModelFlag, String name){
		eventNode.removeAllChildren();
		if(displayObjectReferenceList){
			eventNode.add(generateObjectReferenceListNode(event.getRelatedObjects(), mxConstants.object_reference_list));
		}
		if(displayNormalObjectModelFlag){
			XOCObjectModel completeObjectModel = log.createCompleteObjectModel(event);
			eventNode.add(generateObjectModelNode(completeObjectModel, mxConstants.object_model));
		}
	}
	

	
	private class TreeEntry {
		private String key;
		private String displaytext;
		
		public TreeEntry(String key) {
			this(key, key);
		}
		
		public TreeEntry(String key, String displaytext) {
			this.key = key;
			this.displaytext = displaytext;
		}
		
		public String getKey() {
			return this.key;
		}
		
		public String getDisplayText() {
			return this.displaytext;
		}
		
		public String toString() {
			return getDisplayText();
		}
	}
	
}