package org.processmining.ocbc.conformance.visualization.panels;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.swing.BorderFactory;
import javax.swing.JPanel;
import javax.swing.JTree;
import javax.swing.event.TreeExpansionEvent;
import javax.swing.event.TreeWillExpandListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.ExpandVetoException;

import org.processmining.common.frame.CreateTreePanel;
import org.processmining.mxgraph.util.mxConstants;
import org.processmining.ocbc.conformance.checkers.ObjectNumberPerEventChecking;
import org.processmining.ocbc.conformance.visualization.views.ResultTextView;
import org.processmining.ocbc.log.model.XOCEvent;
import org.processmining.ocbc.log.model.XOCLog;
import org.processmining.ocbc.log.model.XOCORelation;
import org.processmining.ocbc.log.model.XOCObject;
import org.processmining.ocbc.log.model.XOCObjectModel;
import org.processmining.ocbc.log.model.impl.xConstants;
import org.processmining.ocbc.models.AOCEdge;
import org.processmining.ocbc.models.ClassEdge;
import org.processmining.ocbc.models.ClassNode;
import org.processmining.ocbc.models.OCBCModel;

import com.fluxicon.slickerbox.components.RoundedPanel;

public class ObjectNumberPerEventPanel extends RoundedPanel {

	
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	
	private JTree objectModelListTree;
	private JPanel eventListPane;
	private JPanel aOCRelationListPane;
	private JPanel deviationPane;
	private ResultTextView resultTextView;
	private List<AOCEdge> violatedAOCRelationList = new ArrayList<AOCEdge>();
	
	private XOCLog log;
	private OCBCModel model;
	private Map<AOCEdge, List<XOCEvent>> violatedAOCRelationVSEventList;

	
	public ObjectNumberPerEventPanel(ResultTextView resultTextView, XOCLog log, OCBCModel model){
		this.resultTextView = resultTextView;
		this.log = log;
		this.model = model;
		this.setMaximumSize(new Dimension(230, 1000));
		this.setPreferredSize(new Dimension(230, 1000));
		this.setLayout(new GridLayout(1,2));
		this.setBorder(BorderFactory.createLineBorder(new Color(240, 240, 240)));
		eventListPane = createEventListHierarchy(log);
		aOCRelationListPane = createAOCRelationListHierarchy(model);
		deviationPane = createDeviationHierarchy(resultTextView);	
		
		this.add(eventListPane);
		this.add(deviationPane);
		this.add(aOCRelationListPane);			
	}
	
	/**
	 * get the list of violated AOC relations having violated target cardinalities;
	 * @return
	 */
	public List<AOCEdge> getViolatedAOCRelationList(){
		return violatedAOCRelationList;
	}
	
	public CreateTreePanel createDeviationHierarchy(ResultTextView resultTextView){
		ObjectNumberPerEventChecking objectNumberPerEventChecker = resultTextView.getConformanceCheckingResultView().getObjectNumberPerEventChecker();
		violatedAOCRelationVSEventList = objectNumberPerEventChecker.getViolatedAOCRelationVSEventListMap();	
		DefaultMutableTreeNode root = generateViolatedAOCRelationSetNode(
				violatedAOCRelationVSEventList, mxConstants.AOC_relation_list);
		CreateTreePanel deviationTreePanel = new CreateTreePanel(root, mxConstants.conformance_checking_output);	
		return deviationTreePanel;
	}
	
	public CreateTreePanel createAOCRelationListHierarchy(final OCBCModel model){
		DefaultMutableTreeNode root = generateAOCRelationSetNode(model.getAllAOCRelations(), mxConstants.AOC_relation_list);					
		CreateTreePanel classModelTreePanel = new CreateTreePanel(root, mxConstants.conformance_checking_input_model);	
		return classModelTreePanel;
	}
	
	/**
	 * create the object model list hierarchy
	 * @param log
	 */
	public CreateTreePanel createEventListHierarchy(final XOCLog log){
		DefaultMutableTreeNode root = new DefaultMutableTreeNode(
//				"object model list(" + log.getAllEvents().size() + ")");
				mxConstants.event_list);
		for(XOCEvent event : log.getAllEvents()){
			String eventId = event.getAttributes().get(xConstants.ID).toString();
//			DefaultMutableTreeNode eventNode = generateEventNode(event, false, true, false, false, eventId);
			DefaultMutableTreeNode eventNode = new DefaultMutableTreeNode(new TreeEntry(eventId));
			eventNode.add(new DefaultMutableTreeNode(new TreeEntry("fakingNode")));
			root.add(eventNode);
		}
		
		CreateTreePanel objectModelListTreePanel = new CreateTreePanel(root, mxConstants.conformance_checking_input_log);	
		objectModelListTree = objectModelListTreePanel.getJTree();
		
		objectModelListTree.addTreeWillExpandListener(new TreeWillExpandListener() {		
			public void treeWillExpand(TreeExpansionEvent operation) throws ExpandVetoException {
				// TODO Auto-generated method stub	
				System.out.println("tree open in event list tree in ObjectNumberPerEventPanel: " + operation.getPath());				
				DefaultMutableTreeNode expandedNode = (DefaultMutableTreeNode) operation.getPath().getLastPathComponent();
				String nodeName = expandedNode.getUserObject().toString();
				DefaultMutableTreeNode parentNode = (DefaultMutableTreeNode) expandedNode.getParent();
				String parentName = parentNode.getUserObject().toString();
				System.out.println("nodeName: " + nodeName);
				System.out.println("parentName: " + parentName);		
				if(parentName.equals(mxConstants.event_list)){
					String eventId = nodeName;			
					XOCEvent expandedEvent = log.getIdVSEventMap().get(eventId);
					updateEventNode(expandedNode, expandedEvent, false, true, false, false, eventId);
				}

				System.out.println("tree open: " + operation.getPath());
				System.out.println("lastPath: " + nodeName);
			}
			
			public void treeWillCollapse(TreeExpansionEvent event) throws ExpandVetoException {
				// TODO Auto-generated method stub
			}
		});

		return objectModelListTreePanel;

	}

	private DefaultMutableTreeNode generateObjectNode(XOCObject object){
		DefaultMutableTreeNode objectNode = new DefaultMutableTreeNode(new TreeEntry(object.getAttributes().get("id").toString()));
		for(String key : object.getAttributes().keySet()){
			objectNode.add(new DefaultMutableTreeNode(new TreeEntry(key + " : " + object.getAttributes().get(key).toString())));
		}		
		return  objectNode;
	}
	
	private DefaultMutableTreeNode generateObjectSetNode(List<XOCObject> objects, String name){
		DefaultMutableTreeNode objectSetNode = new DefaultMutableTreeNode(new TreeEntry(name));
		if(objects != null){
			for (XOCObject object : objects) {
				objectSetNode.add(generateObjectNode(object));
			}
		}	
		return objectSetNode;
	}

	private DefaultMutableTreeNode generateObjectRelationNode(XOCORelation relation){
		DefaultMutableTreeNode relationNode = new DefaultMutableTreeNode(new TreeEntry(relation.getAttributes().get("id").toString()));
		for(String key : relation.getAttributes().keySet()){
			relationNode.add(new DefaultMutableTreeNode(new TreeEntry(key + " : " + relation.getAttributes().get(key).toString())));
		}		
		return  relationNode;
	}
	
	private DefaultMutableTreeNode generateObjectRelationSetNode(List<XOCORelation> relations, String name){
		DefaultMutableTreeNode relationSetNode = new DefaultMutableTreeNode(new TreeEntry(name));
		for (XOCORelation relation : relations) {
			relationSetNode.add(generateObjectRelationNode(relation));
		}
		return relationSetNode;
	}
	
	private DefaultMutableTreeNode generateObjectModelNode(XOCObjectModel objectModel){
		DefaultMutableTreeNode objectModelNode = new DefaultMutableTreeNode(new TreeEntry("OM"));
		List<XOCObject> objectList = objectModel.getObjectSet();
		List<XOCORelation> objectRelationList = objectModel.getObjectRelationSet();
		objectModelNode.add(generateObjectSetNode(objectList, "Objects"));
		objectModelNode.add(generateObjectRelationSetNode(objectRelationList, "Relations"));
		return objectModelNode;
	}
	
	private DefaultMutableTreeNode generateClassNode(ClassNode classNode){
		DefaultMutableTreeNode classTreeNode = new DefaultMutableTreeNode(new TreeEntry(classNode.getNodeName()));
		//in future, add the attribute node if the class node has attributes
//		classTreeNode.add(new DefaultMutableTreeNode(new TreeEntry("node_name : " + classNode.getNodeName())));
		return  classTreeNode;
	}
	
	private DefaultMutableTreeNode generateClassSetNode(List<ClassNode> classList, String name){
		DefaultMutableTreeNode classSetNode = new DefaultMutableTreeNode(new TreeEntry(name));
		for (ClassNode classNode : classList) {
			classSetNode.add(generateClassNode(classNode));
		}
		return classSetNode;
	}
	
	private DefaultMutableTreeNode generateClassRelationNode(ClassEdge relation){
		DefaultMutableTreeNode relationNode = new DefaultMutableTreeNode(new TreeEntry(relation.getID()));
		relationNode.add(new DefaultMutableTreeNode(
				new TreeEntry(xConstants.relationship_type + ": " + relation.getEdgeName())));	
		relationNode.add(new DefaultMutableTreeNode(
				new TreeEntry(xConstants.source_class + ": " + relation.getSourceNode().getNodeName())));	
		relationNode.add(new DefaultMutableTreeNode(
				new TreeEntry(xConstants.target_class + ": " + relation.getTargetNode().getNodeName())));
		relationNode.add(new DefaultMutableTreeNode(
				new TreeEntry(xConstants.source_always_cardinality + ": " + relation.getSourceAlwaysCardinality().toString())));	
		relationNode.add(new DefaultMutableTreeNode(
				new TreeEntry(xConstants.source_eventually_cardinality + ": " + relation.getSourceEventuallyCardinality().toString())));	
		relationNode.add(new DefaultMutableTreeNode(
				new TreeEntry(xConstants.target_always_cardinality + ": " + relation.getTargetAlwaysCardinality().toString())));	
		relationNode.add(new DefaultMutableTreeNode(
				new TreeEntry(xConstants.target_eventually_cardinality + ": " + relation.getTargetEventuallyCardinality().toString())));				
		return  relationNode;
	}
	
	private DefaultMutableTreeNode generateClassRelationSetNode(List<ClassEdge> relations, String name){
		DefaultMutableTreeNode relationSetNode = new DefaultMutableTreeNode(new TreeEntry(name));
		for (ClassEdge relation : relations) {
			relationSetNode.add(generateClassRelationNode(relation));
		}
		return relationSetNode;
	}
	
	private DefaultMutableTreeNode generateAOCRelationNode(AOCEdge relation){
		DefaultMutableTreeNode relationNode = new DefaultMutableTreeNode(new TreeEntry(relation.getID()));
//		relationNode.add(new DefaultMutableTreeNode(
//				new TreeEntry(xConstants.relationship_type + ": " + relation.getEdgeName())));	
		relationNode.add(new DefaultMutableTreeNode(
				new TreeEntry(mxConstants.activity + ": " + relation.getActivityNode().getNodeName())));	
		relationNode.add(new DefaultMutableTreeNode(
				new TreeEntry(mxConstants.class_node + ": " + relation.getClassNode().getNodeName())));
//		relationNode.add(new DefaultMutableTreeNode(
//				new TreeEntry(xConstants.source_always_cardinality + ": " + relation.getActivityAlwaysCardinality())));	
//		relationNode.add(new DefaultMutableTreeNode(
//				new TreeEntry(xConstants.source_eventually_cardinality + ": " + relation.getActivityEventuallyCardinality())));	
		relationNode.add(new DefaultMutableTreeNode(
				new TreeEntry(mxConstants.target_cardinality + ": " + relation.getClassCardinality())));					
		return  relationNode;
	}
	
	private DefaultMutableTreeNode generateAOCRelationSetNode(List<AOCEdge> relations, String name){
		DefaultMutableTreeNode relationSetNode = new DefaultMutableTreeNode(new TreeEntry(name));
		for (AOCEdge relation : relations) {
			relationSetNode.add(generateAOCRelationNode(relation));
		}
		return relationSetNode;
	}
	
	private DefaultMutableTreeNode generateViolatedAOCRelationNode(List<XOCEvent> eventList, AOCEdge relation){
		String relationId = relation.getID();
		String cardinality = relation.getActivityAlwaysCardinality().toString();
		DefaultMutableTreeNode listNode = new DefaultMutableTreeNode(new TreeEntry(relationId + "-cardinality: " + cardinality));
		for (XOCEvent element : eventList) {
			String eventId = element.getAttributes().get(xConstants.ID).toString();
			DefaultMutableTreeNode eventNode = generateEventNode(element, false, true, false, false, eventId);
			listNode.add(eventNode);
		}
		return listNode;
	}
	
	private DefaultMutableTreeNode generateViolatedAOCRelationSetNode(Map<AOCEdge, List<XOCEvent>> violatedAOCRelationVSEventList, String name){
		DefaultMutableTreeNode relationSetNode = new DefaultMutableTreeNode(new TreeEntry(name));
		for (AOCEdge relation : violatedAOCRelationVSEventList.keySet()) {		
			List<XOCEvent> eventList = violatedAOCRelationVSEventList.get(relation);
			if(eventList.size() > 0){
				violatedAOCRelationList.add(relation);
				relationSetNode.add(generateViolatedAOCRelationNode(eventList, relation));
			}			
		}
		return relationSetNode;
	}
	
	private DefaultMutableTreeNode generateObjectReferenceListNode(List<XOCObject> objectReferenceList, String name){
		DefaultMutableTreeNode listNode = new DefaultMutableTreeNode(new TreeEntry(name));
		for (XOCObject element : objectReferenceList) {
			listNode.add(generateObjectNode(element));
		}
		return listNode;
	}
	
	private DefaultMutableTreeNode generateEventNode(XOCEvent event, boolean displayAttributeListFlag, boolean displayObjectReferenceList,
			boolean displayNormalObjectModelFlag, boolean displayUpdatedObjectModelFlag, String name){
		DefaultMutableTreeNode eventNode = new DefaultMutableTreeNode(new TreeEntry(name));
		if(displayObjectReferenceList){
			eventNode.add(generateObjectReferenceListNode(event.getRelatedObjects(), mxConstants.object_reference_list));
		}
		return eventNode;
	}
	
	private DefaultMutableTreeNode generateDeviatingObjectRelationNode(XOCORelation relation, List<String> contentList){
		DefaultMutableTreeNode relationNode = new DefaultMutableTreeNode(new TreeEntry(relation.getAttributes().get("id").toString()));
		for(String content : contentList){
			relationNode.add(new DefaultMutableTreeNode(new TreeEntry(content)));
		}		
		return  relationNode;
	}
	
	public DefaultMutableTreeNode generateDeviatingObjectRelationSetNode(List<XOCORelation> relations, 
			Map<XOCORelation, List<String>> deviatingObjectRelationVSContentListMap, String name){
		DefaultMutableTreeNode relationSetNode = new DefaultMutableTreeNode(new TreeEntry(name));
		for (XOCORelation relation : relations) {
			List<String> contentList = deviatingObjectRelationVSContentListMap.get(relation);
			relationSetNode.add(generateDeviatingObjectRelationNode(relation, contentList));
		}
		return relationSetNode;
	}

	private void updateEventNode(DefaultMutableTreeNode eventNode, XOCEvent event, boolean displayAttributeListFlag, boolean displayObjectReferenceList,
			boolean displayNormalObjectModelFlag, boolean displayUpdatedObjectModelFlag, String name){
		eventNode.removeAllChildren();
		if(displayObjectReferenceList){
			eventNode.add(generateObjectReferenceListNode(event.getRelatedObjects(), mxConstants.object_reference_list));
		}
	}

	
	private class TreeEntry {
		private String key;
		private String displaytext;
		
		public TreeEntry(String key) {
			this(key, key);
		}
		
		public TreeEntry(String key, String displaytext) {
			this.key = key;
			this.displaytext = displaytext;
		}
		
		public String getKey() {
			return this.key;
		}
		
		public String getDisplayText() {
			return this.displaytext;
		}
		
		public String toString() {
			return getDisplayText();
		}
	}
	
}