/*
 * Copyright (c) 2007 Christian W. Guenther (christian@deckfour.org)
 * 
 * LICENSE:
 * 
 * This code is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 * 
 * EXEMPTION:
 * 
 * License to link and use is also granted to open source programs which are not
 * licensed under the terms of the GPL, given that they satisfy one or more of
 * the following conditions: 1) Explicit license is granted to the ProM and
 * ProMimport programs for usage, linking, and derivative work. 2) Carte blance
 * license is granted to all programs developed at Eindhoven Technical
 * University, The Netherlands, or under the umbrella of STW Technology
 * Foundation, The Netherlands. For further exemptions not covered by the above
 * conditions, please contact the author of this code.
 */
package org.processmining.ocbc.conformance.visualization.views;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Image;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;

import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JPanel;

import org.deckfour.uitopia.api.event.TaskListener.InteractionResult;
import org.deckfour.xes.info.XLogInfo;
import org.jgraph.JGraph;
import org.jgraph.graph.DefaultGraphModel;
import org.jgraph.graph.GraphLayoutCache;
import org.jgraph.graph.GraphModel;
import org.processmining.common.runningtime.RunningTime;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.ocbc.conformance.checkers.ActivityExistenceChecking;
import org.processmining.ocbc.conformance.checkers.BehavioralConstraintsChecking;
import org.processmining.ocbc.conformance.checkers.EventNumberPerObjectChecking;
import org.processmining.ocbc.conformance.checkers.FulfilmentChecking;
import org.processmining.ocbc.conformance.checkers.MonotonicityChecking;
import org.processmining.ocbc.conformance.checkers.ObjectExistenceChecking;
import org.processmining.ocbc.conformance.checkers.ObjectModelValidityChecking;
import org.processmining.ocbc.conformance.checkers.ObjectNumberPerEventChecking;
import org.processmining.ocbc.conformance.checkers.ProperClassesChecking;
import org.processmining.ocbc.conformance.plugin.ParameterPanel;
import org.processmining.ocbc.log.inspector.panels.XOCLogViewer;
import org.processmining.ocbc.log.model.XOCLog;
import org.processmining.ocbc.loggeneration.generator.plugin.XOCLogGeneratorPlugin;
import org.processmining.ocbc.models.OCBCModel;
import org.processmining.ocbc.models.visualization.ModelCellViewFactory;
import org.processmining.ocbc.models.visualization.cell.ActivityCell;
import org.processmining.ocbc.models.visualization.cell.MyOriginalCell;

import com.fluxicon.slickerbox.components.IconVerticalTabbedPane;
import com.fluxicon.slickerbox.components.SlickerTabbedPane;

/**
 * @author Christian W. Guenther (christian@deckfour.org)
 * 
 */
//public class ConformanceCheckingResultView extends JPanel implements ScalableComponent {
public class ConformanceCheckingResultView extends JPanel {
	
	private static final long serialVersionUID = 4258730373866637365L;
	
	private final static String IMPORTICON = "import.png";
	protected Image importIcon = new ImageIcon(XOCLogGeneratorPlugin.class.getResource(IMPORTICON),IMPORTICON).getImage();
	
	protected XOCLogViewer logView;
	protected ResultModelView modelView;
	protected ResultTextView textView;
	protected ResultMenuBar menuBar;
	protected XLogInfo summary;	
	protected XOCLog inputLog;
	protected OCBCModel model;	
	protected SlickerTabbedPane tabbedPane;
		
	protected ParameterPanel parameterPanel;
	//control to check which perspectives
	boolean objectModelValidityCheckerSwitch = false;
	boolean fulfilmentCheckerSwitch = false;
	boolean activityExistenceCheckerSwitch = false;
	boolean monotonicityCheckerSwitch = false;
	boolean objectExistenceCheckerSwitch = false;
	boolean properClassesCheckerSwitch = false;
	boolean objectNumberPerEventCheckerSwitch = false;
	boolean eventNumberPerObjectCheckerSwitch = false;
	boolean behavioralConstraintsCheckerSwitch = false;
	
	
	
	protected ObjectModelValidityChecking objectModelValidityChecker = new ObjectModelValidityChecking();		
	protected FulfilmentChecking fulfilmentChecker = new FulfilmentChecking();
	protected MonotonicityChecking monotonicityChecker = new MonotonicityChecking();
	protected ActivityExistenceChecking activityExistenceChecker = new ActivityExistenceChecking();
	protected ObjectExistenceChecking objectExistenceChecker = new ObjectExistenceChecking();
	protected ProperClassesChecking properClassesChecker = new ProperClassesChecking();
	protected ObjectNumberPerEventChecking objectNumberPerEventChecker = new ObjectNumberPerEventChecking();
	protected EventNumberPerObjectChecking eventNumberPerObjectChecker = new EventNumberPerObjectChecking();
	protected BehavioralConstraintsChecking behavioralConstraintsChecker = new BehavioralConstraintsChecking();

	
	public ParameterPanel getParameterPanel(){
		return parameterPanel;
	}
	
	public boolean getObjectModelValidityCheckerSwitch(){
		return objectModelValidityCheckerSwitch;
	}
	
	public boolean getFulfilmentCheckerSwitch(){
		return fulfilmentCheckerSwitch;
	}
	
	public boolean getMonotonicityCheckerSwitch(){
		return monotonicityCheckerSwitch;
	}
	
	public boolean getActivityExistenceCheckerSwitch(){
		return activityExistenceCheckerSwitch;
	}
	
	public boolean getObjectExistenceCheckerSwitch(){
		return objectExistenceCheckerSwitch;
	}
	
	public boolean getProperClassesCheckerSwitch(){
		return properClassesCheckerSwitch;
	}
	
	public boolean getObjectNumberPerEventCheckerSwitch(){
		return objectNumberPerEventCheckerSwitch;
	}
	
	public boolean getEventNumberPerObjectCheckerSwitch(){
		return eventNumberPerObjectCheckerSwitch;
	}

	public boolean getBehavioralConstraintsCheckerSwitch(){
		return behavioralConstraintsCheckerSwitch;
	}	
	
	public void setParameterPanel(ParameterPanel parameterPanel){
		this.parameterPanel = parameterPanel;
	}
	
	public void setObjectModelValidityCheckerSwitch(boolean objectModelValidityCheckerSwitch){
		this.objectModelValidityCheckerSwitch = objectModelValidityCheckerSwitch;
	}
	
	public void setFulfilmentCheckerSwitch(boolean fulfilmentCheckerSwitch){
		this.fulfilmentCheckerSwitch = fulfilmentCheckerSwitch;
	}
	
	public void setActivityExistenceCheckerSwitch(boolean activityExistenceCheckerSwitch){
		this.activityExistenceCheckerSwitch = activityExistenceCheckerSwitch;
	}
	
	public void setObjectExistenceCheckerSwitch(boolean objectExistenceCheckerSwitch){
		this.objectExistenceCheckerSwitch = objectExistenceCheckerSwitch;
	}
	
	public void setProperClassesCheckerSwitch(boolean properClassesCheckerSwitch){
		this.properClassesCheckerSwitch = properClassesCheckerSwitch;
	}
	
	public void setObjectNumberPerEventCheckerSwitch(boolean objectNumberPerEventCheckerSwitch){
		this.objectNumberPerEventCheckerSwitch = objectNumberPerEventCheckerSwitch;
	}
	
	public void setEventNumberPerObjectCheckerSwitch(boolean eventNumberPerObjectCheckerSwitch){
		this.eventNumberPerObjectCheckerSwitch = eventNumberPerObjectCheckerSwitch;
	}

	public void setBehavioralConstraintsCheckerSwitch(boolean behavioralConstraintsCheckerSwitch){
		this.behavioralConstraintsCheckerSwitch = behavioralConstraintsCheckerSwitch;
	}	
	
	public JComponent conformanceChecking(UIPluginContext context, XOCLog log, OCBCModel model) {				
		initialize(log, model);		
		setParameter(context);		             
		long totalTime = estimateRunningTime(log, model);	
		
		
		long uint = 1;	
		long startTime = System.currentTimeMillis(); 
		long durationTime = System.currentTimeMillis();
		long allTime = System.currentTimeMillis();
		
		int logSize = log.getAllEvents().size();
		RunningTime.writeTimeInformationIntoFile("*********************logSize: " + logSize + "*****************", true, "D:\\timeConformanceChecking.txt");
		RunningTime.writeTimeInformationIntoFile("conformance checking start time: " + startTime/uint, true, "D:\\timeConformanceChecking.txt");
		
//		if(context != null){
//			context.getProgress().setMaximum(100); 		
//			context.getProgress().setValue(1); 
//		}	

		String[] stringResult = new String[9];
		stringResult[0] = "validity of object models:";
		stringResult[1] = "fulfilment:";
		stringResult[2] = "monotonicity:";
		stringResult[3] = "activity existence:";
		stringResult[4] = "object existence:";
		stringResult[5] = "proper classes:";
		stringResult[6] = "right number of events per object:";
		stringResult[7] = "right number of objects per event:";
		stringResult[8] = "behavioral constraints are respected:";
		
		//check the validity of object models
		if(objectModelValidityCheckerSwitch){		
			System.out.println("check the validity of object models: " + true);			
			RunningTime.writeTimeInformationIntoFile("check the validity of object models start time: " + System.currentTimeMillis()/uint, false, "D:\\timeConformanceChecking.txt");
			
			objectModelValidityChecker.checking(log, model);
					
			durationTime = (System.currentTimeMillis() - startTime)/uint;
			startTime = System.currentTimeMillis();
			RunningTime.writeTimeInformationIntoFile("check the validity of object models end time: " + System.currentTimeMillis()/uint, false, "D:\\timeConformanceChecking.txt");
			RunningTime.writeTimeInformationIntoFile("~~~~~~~~~~~~~~check the validity of object models duration time: " + durationTime, false, "D:\\timeConformanceChecking.txt");
		
		}	
		
		if(fulfilmentCheckerSwitch){		
			System.out.println("check the fulfilment of object models: " + true);			
			RunningTime.writeTimeInformationIntoFile("check the fulfilment of object models start time: " + System.currentTimeMillis()/uint, false, "D:\\timeConformanceChecking.txt");

			fulfilmentChecker.checking(log, model);
					
			durationTime = (System.currentTimeMillis() - startTime)/uint;
			startTime = System.currentTimeMillis();
			RunningTime.writeTimeInformationIntoFile("check the fulfilment of object models end time: " + System.currentTimeMillis()/uint, false, "D:\\timeConformanceChecking.txt");
			RunningTime.writeTimeInformationIntoFile("~~~~~~~~~~~~~~check the fulfilment of object models duration time: " + durationTime, false, "D:\\timeConformanceChecking.txt");
		
		}	
		
		if(monotonicityCheckerSwitch){		
			System.out.println("check the monotonicity of object models: " + true);			
			RunningTime.writeTimeInformationIntoFile("check the monotonicity of object models start time: " + System.currentTimeMillis()/uint, false, "D:\\timeConformanceChecking.txt");

			monotonicityChecker.checking(log, model);
					
			durationTime = (System.currentTimeMillis() - startTime)/uint;
			startTime = System.currentTimeMillis();
			RunningTime.writeTimeInformationIntoFile("check the monotonicity of object models end time: " + System.currentTimeMillis()/uint, false, "D:\\timeConformanceChecking.txt");
			RunningTime.writeTimeInformationIntoFile("~~~~~~~~~~~~~~check the monotonicity of object models duration time: " + durationTime, false, "D:\\timeConformanceChecking.txt");
		
		}
		
		if(activityExistenceCheckerSwitch){		
			System.out.println("activity existence Checker Switch: " + true);			
			RunningTime.writeTimeInformationIntoFile("check activity existence start time: " + System.currentTimeMillis()/uint, false, "D:\\timeConformanceChecking.txt");

			activityExistenceChecker.checking(log, model);
					
			durationTime = (System.currentTimeMillis() - startTime)/uint;
			startTime = System.currentTimeMillis();
			RunningTime.writeTimeInformationIntoFile("check activity existence end time: " + System.currentTimeMillis()/uint, false, "D:\\timeConformanceChecking.txt");
			RunningTime.writeTimeInformationIntoFile("~~~~~~~~~~~~~~check activity existence duration time: " + durationTime, false, "D:\\timeConformanceChecking.txt");
		
		}
		
		if(objectExistenceCheckerSwitch){		
			System.out.println("object existence Checker Switch: " + true);			
			RunningTime.writeTimeInformationIntoFile("check object existence start time: " + System.currentTimeMillis()/uint, false, "D:\\timeConformanceChecking.txt");

			objectExistenceChecker.checking(log, model);
					
			durationTime = (System.currentTimeMillis() - startTime)/uint;
			startTime = System.currentTimeMillis();
			RunningTime.writeTimeInformationIntoFile("check object existence end time: " + System.currentTimeMillis()/uint, false, "D:\\timeConformanceChecking.txt");
			RunningTime.writeTimeInformationIntoFile("~~~~~~~~~~~~~~check object existence duration time: " + durationTime, false, "D:\\timeConformanceChecking.txt");
		
		}
		
		if(properClassesCheckerSwitch){		
			System.out.println("proper Classes Checker Switch: " + true);			
			RunningTime.writeTimeInformationIntoFile("check proper classes start time: " + System.currentTimeMillis()/uint, false, "D:\\timeConformanceChecking.txt");
			
//			stringResult[1] = checkClassRelation(context, log, model, totalTime, startTime);
			properClassesChecker.checking(log, model);
			
			
			durationTime = (System.currentTimeMillis() - startTime)/uint;
			startTime = System.currentTimeMillis();
			RunningTime.writeTimeInformationIntoFile("check proper classes end time: " + System.currentTimeMillis()/uint, false, "D:\\timeConformanceChecking.txt");
			RunningTime.writeTimeInformationIntoFile("~~~~~~~~~~~~~~check proper classes duration time: " + durationTime, false, "D:\\timeConformanceChecking.txt");
		
		}
		
		if(objectNumberPerEventCheckerSwitch){		
			System.out.println("check object Number Per Event: " + true);			
			RunningTime.writeTimeInformationIntoFile("check object Number Per Event start time: " + System.currentTimeMillis()/uint, false, "D:\\timeConformanceChecking.txt");
			
//			stringResult[1] = checkClassRelation(context, log, model, totalTime, startTime);
			objectNumberPerEventChecker.checking(log, model);
			
			
			durationTime = (System.currentTimeMillis() - startTime)/uint;
			startTime = System.currentTimeMillis();
			RunningTime.writeTimeInformationIntoFile("check object Number Per Event end time: " + System.currentTimeMillis()/uint, false, "D:\\timeConformanceChecking.txt");
			RunningTime.writeTimeInformationIntoFile("~~~~~~~~~~~~~~check object Number Per Event duration time: " + durationTime, false, "D:\\timeConformanceChecking.txt");
		
		}	
		
		if(eventNumberPerObjectCheckerSwitch){		
			System.out.println("check event Number Per object: " + true);			
			RunningTime.writeTimeInformationIntoFile("check event Number Per object start time: " + System.currentTimeMillis()/uint, false, "D:\\timeConformanceChecking.txt");

			eventNumberPerObjectChecker.checking(log, model);
				
			durationTime = (System.currentTimeMillis() - startTime)/uint;
			startTime = System.currentTimeMillis();
			RunningTime.writeTimeInformationIntoFile("check event Number Per object end time: " + System.currentTimeMillis()/uint, false, "D:\\timeConformanceChecking.txt");
			RunningTime.writeTimeInformationIntoFile("~~~~~~~~~~~~~~check event Number Per object duration time: " + durationTime, false, "D:\\timeConformanceChecking.txt");
		
		}	
		
		
					
		updateProcessBarMaximum(context, 96, startTime, totalTime);		
	
//		updateProcessBarMaximum(context, 98, startTime, totalTime);
//		
		//check the behavior constraints
		if(behavioralConstraintsCheckerSwitch){		
			RunningTime.writeTimeInformationIntoFile("check behavioral constraint start time: " + System.currentTimeMillis()/uint, false, "D:\\timeConformanceChecking.txt");
			
			behavioralConstraintsChecker.checking(log, model);
		    durationTime = (System.currentTimeMillis() - startTime)/uint;
			startTime = System.currentTimeMillis();
			RunningTime.writeTimeInformationIntoFile("check behavioral constraint end time: " + System.currentTimeMillis()/uint, false, "D:\\timeConformanceChecking.txt");
			RunningTime.writeTimeInformationIntoFile("~~~~~~~~~~~~~~check behavioral constraint duration time: " + durationTime, false, "D:\\timeConformanceChecking.txt");
		}
	    
		durationTime = (System.currentTimeMillis() - allTime)/uint;
		RunningTime.writeTimeInformationIntoFile("conformance checking end time: " + System.currentTimeMillis()/uint, false, "D:\\timeConformanceChecking.txt");
		RunningTime.writeTimeInformationIntoFile("~~~~~~~~~~~~~~conformance checking duration time: " + durationTime, false, "D:\\timeConformanceChecking.txt");

		RunningTime.writeTimeInformationIntoFile("textView start time: " + System.currentTimeMillis()/uint, false, "D:\\timeConformanceChecking.txt");		
		textView = new ResultTextView(log, model, stringResult, this);
		durationTime = (System.currentTimeMillis() - startTime)/uint;
		startTime = System.currentTimeMillis();
		RunningTime.writeTimeInformationIntoFile("textView end time: " + System.currentTimeMillis()/uint, false, "D:\\timeConformanceChecking.txt");
		RunningTime.writeTimeInformationIntoFile("~~~~~~~~~~~~~~textView duration time: " + durationTime, false, "D:\\timeConformanceChecking.txt");	

		RunningTime.writeTimeInformationIntoFile("logView start time: " + System.currentTimeMillis()/uint, false, "D:\\timeConformanceChecking.txt");		
		//use the method of log inspector to show the log view of conformance checking result plus highlighting deviating cells when clicking event
		logView = new XOCLogViewer(log, this);
//		logView.getLogBrowser().getGraphPanel().highlightDeviatingCells(this);
		durationTime = (System.currentTimeMillis() - startTime)/uint;
		startTime = System.currentTimeMillis();
		RunningTime.writeTimeInformationIntoFile("logView end time: " + System.currentTimeMillis()/uint, false, "D:\\timeConformanceChecking.txt");
		RunningTime.writeTimeInformationIntoFile("~~~~~~~~~~~~~~logView duration time: " + durationTime, false, "D:\\timeConformanceChecking.txt");
		
		RunningTime.writeTimeInformationIntoFile("modelView start time: " + System.currentTimeMillis()/uint, false, "D:\\timeConformanceChecking.txt");		
		modelView = new ResultModelView(null, model, this);
		durationTime = (System.currentTimeMillis() - startTime)/uint;
		startTime = System.currentTimeMillis();
		RunningTime.writeTimeInformationIntoFile("modelView end time: " + System.currentTimeMillis()/uint, false, "D:\\timeConformanceChecking.txt");
		RunningTime.writeTimeInformationIntoFile("~~~~~~~~~~~~~~modelView duration time: " + durationTime, false, "D:\\timeConformanceChecking.txt");
			
		IconVerticalTabbedPane iconTabs = new IconVerticalTabbedPane(new Color(230, 230, 230, 210), new Color(20, 20,20, 160));

		GraphModel iconModel = new DefaultGraphModel();  
	    GraphLayoutCache view = new GraphLayoutCache(iconModel, new ModelCellViewFactory());  
	    JGraph graph = new JGraph(iconModel, view); 
		Color bg = null; // Use this to make the background transparent
		int inset = 0;
		MyOriginalCell[] cells = new MyOriginalCell[1];  
		cells[0] = new ActivityCell(new String("Hello")); 
		graph.getGraphLayoutCache().insert(cells);  		
		BufferedImage img = graph.getImage(bg, inset);		
		Image summaryIcon = new ImageIcon(img).getImage();
		
		iconTabs.addTab("Type View", summaryIcon, textView);
		iconTabs.addTab("Log View", summaryIcon, logView);	
		iconTabs.addTab("Model View", summaryIcon, modelView);

		menuBar = new ResultMenuBar(this);
//		add(menuBar.getMenuBar(), BorderLayout.NORTH);
		add(iconTabs, BorderLayout.CENTER);
		
		revalidate();
		repaint();		
		return this;
	}

	protected void initialize(XOCLog log, OCBCModel model) {
		this.inputLog = log;
		this.model = model;	
		setBackground(new Color(40, 40, 40));
		setBorder(BorderFactory.createEmptyBorder());
		setLayout(new BorderLayout());
	}
	
	public void setParameter(UIPluginContext context){
		if(context != null){
			parameterPanel = new ParameterPanel();
			InteractionResult interActionResult = context.showConfiguration("Conformance Checking Perspective Selection", parameterPanel);
			if (interActionResult.equals(InteractionResult.CANCEL)) {
				context.getFutureResult(0).cancel(true);			
			}				
			else if (interActionResult.equals(InteractionResult.CONTINUE)) {			
				objectModelValidityCheckerSwitch = parameterPanel.getValidityOfObjectModelSelectionFlag();
				fulfilmentCheckerSwitch = parameterPanel.getFulfilmentSelectionFlag();
				monotonicityCheckerSwitch = parameterPanel.getMonotonicitySelectionFlag();
				activityExistenceCheckerSwitch = parameterPanel.getActivityExistenceSelectionFlag();
				objectExistenceCheckerSwitch = parameterPanel.getObjectExistenceSelectionFlag();
				properClassesCheckerSwitch = parameterPanel.getProperClassesSelectionFlag();
				objectNumberPerEventCheckerSwitch = parameterPanel.getObjectsPerEventSelectionFlag();
				eventNumberPerObjectCheckerSwitch = parameterPanel.getEventsPerObjectSelectionFlag();
				behavioralConstraintsCheckerSwitch = parameterPanel.getBehavioralConstraintsSelectionFlag();	
			}
		}
		else if(parameterPanel != null){
			objectModelValidityCheckerSwitch = parameterPanel.getValidityOfObjectModelSelectionFlag();
			fulfilmentCheckerSwitch = parameterPanel.getFulfilmentSelectionFlag();
			monotonicityCheckerSwitch = parameterPanel.getMonotonicitySelectionFlag();
			activityExistenceCheckerSwitch = parameterPanel.getActivityExistenceSelectionFlag();
			objectExistenceCheckerSwitch = parameterPanel.getObjectExistenceSelectionFlag();
			properClassesCheckerSwitch = parameterPanel.getProperClassesSelectionFlag();
			objectNumberPerEventCheckerSwitch = parameterPanel.getObjectsPerEventSelectionFlag();
			eventNumberPerObjectCheckerSwitch = parameterPanel.getEventsPerObjectSelectionFlag();
			behavioralConstraintsCheckerSwitch = parameterPanel.getBehavioralConstraintsSelectionFlag();	
		}
		else{
			objectModelValidityCheckerSwitch = true;
			fulfilmentCheckerSwitch = true;
			monotonicityCheckerSwitch = true;
			activityExistenceCheckerSwitch = true;
			objectExistenceCheckerSwitch = true;
			properClassesCheckerSwitch = true;
			objectNumberPerEventCheckerSwitch = true;
			eventNumberPerObjectCheckerSwitch = true;
			behavioralConstraintsCheckerSwitch = true;			
		}	
	}
	
	//estimate the total running time (unit: ms)
	public long estimateRunningTime(XOCLog log, OCBCModel model){
		int eventNum = log.getAllEvents().size(); 
        int firstObjectModelSize = log.getAllEvents().get(0).getObjectModel().getObjectSet().size();
        int lastObjectModelSize = log.getAllEvents().get(eventNum-1).getObjectModel().getObjectSet().size();
        int averageObjectModelSize = (firstObjectModelSize + lastObjectModelSize)/2;
        long totalTime = eventNum*averageObjectModelSize/2; 
//        if(dataConstraintCheckSwitch){
//        	totalTime = totalTime + eventNum*averageObjectModelSize*5; 
//        }
        return totalTime;
	}
	
	public void updateProcessBarMaximum(UIPluginContext context, int processPercentage, long startTime, long totalTime){
		long endTime = System.currentTimeMillis();
		long percentage = (endTime - startTime)*100/totalTime;
		if(percentage < processPercentage){
			if(context != null){
				context.getProgress().setValue((int) (percentage) + 1);
			}				
		}
		else{
			if(context != null){
				context.getProgress().setValue(processPercentage);
			}	
		}			
	}	
	
	public XOCLog getXOCLog() {
		return inputLog;
	}
	
	public XLogInfo getLogInfo() {
		return summary;
	}
	
	public void writeTimeInformationIntoFile(String wirteLine, boolean clearFile){
		File log = new File("D:\\time.txt");
	    try{
		    if(log.exists()==false || clearFile){
	            System.out.println("We had to make a new file.");
	            log.createNewFile();
		    }
		    PrintWriter out = new PrintWriter(new FileWriter(log, true));
		    out.append(wirteLine + "\n");
		    out.close();
	    }catch(IOException e){
	        System.out.println("COULD NOT LOG!!");
	    }
	}
	
	public void outputContextInformation(UIPluginContext context, String information){
		if(context != null){
			context.log("\n " + information);
		}
	}
	
	public ObjectModelValidityChecking getObjectModelValidityChecker(){
		return objectModelValidityChecker;
	}
	
	public FulfilmentChecking getFulfilmentChecker(){
		return fulfilmentChecker;
	}
	
	public MonotonicityChecking getMonotonicityChecker(){
		return monotonicityChecker;
	}
	
	public ActivityExistenceChecking getActivityExistenceChecker(){
		return activityExistenceChecker;
	}
	
	public ObjectExistenceChecking getObjectExistenceChecker(){
		return objectExistenceChecker;
	}
	
	public ProperClassesChecking getProperClassesChecker(){
		return properClassesChecker;
	}
	
	public ObjectNumberPerEventChecking getObjectNumberPerEventChecker(){
		return objectNumberPerEventChecker;
	}
	
	public EventNumberPerObjectChecking getEventNumberPerObjectChecker(){
		return eventNumberPerObjectChecker;
	}
	
	public BehavioralConstraintsChecking getBehavioralConstraintsChecker(){
		return behavioralConstraintsChecker;
	}
	
	/**
	 * get the type view of the conformance checking result;
	 * @return
	 */
	public ResultTextView getTypeView(){
		return textView;
	}
	
	/**
	 * get the log view of the conformance checking result;
	 * @return
	 */
	public XOCLogViewer getLogView(){
		return logView;
	}
	
	/**
	 * get the model view of the conformance checking result;
	 * @return
	 */
	public ResultModelView getModelView(){
		return modelView;
	}
	

	
	
	public static void main(String[] args) throws Exception  
    {  
     			 
		File log = new File("D:\\time.txt");
			    try{
			    if(log.exists()==false){
			            System.out.println("We had to make a new file.");
			            log.createNewFile();
			    }
			    PrintWriter out = new PrintWriter(new FileWriter(log, true));
			    out.append("*******timestamp2******* " + "\n");
			    out.close();
			    }catch(IOException e){
			        System.out.println("COULD NOT LOG!!");
			    } 
    } 
}
