package org.processmining.ocbc.conformance.visualization.views;

import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

import org.processmining.common.file.SVG2PDF;
import org.processmining.mxgraph.canvas.mxICanvas;
import org.processmining.mxgraph.canvas.mxSvgCanvas;
import org.processmining.mxgraph.examples.swing.editor.DefaultFileFilter;
import org.processmining.mxgraph.util.mxCellRenderer;
import org.processmining.mxgraph.util.mxCellRenderer.CanvasFactory;
import org.processmining.mxgraph.util.mxDomUtils;
import org.processmining.mxgraph.util.mxResources;
import org.processmining.mxgraph.util.mxUtils;
import org.processmining.mxgraph.util.mxXmlUtils;
import org.processmining.mxgraph.view.mxGraph;

/**
 * Written/modified by alvin alexander, devdaily.com.
 * 
 * This class was nspired by a thread in the Mac Java-dev mailing list
 * (http://lists.apple.com/archives/java-dev/).
 */
public class ResultMenuBar implements Runnable, ActionListener
{
  private JFrame frame;
  private JMenuBar menuBar;
  private JMenu fileMenu;
  private JMenu editMenu;

  private JMenuItem openMenuItem;
  private JMenuItem exportModelMenuItem;
  private JMenuItem cutMenuItem;
  private JMenuItem copyMenuItem;
  private JMenuItem pasteMenuItem;

  private JMenu helpMenu;
  private JMenuItem aboutMenuItem;
  private JMenuItem manualMenuItem;
  
  private String lastDir = null;

  private JPanel conformanceCheckingResultView;
  private mxGraph graph;
  public static void main(String[] args)
  {
//    // needed on mac os x
//    System.setProperty("apple.laf.useScreenMenuBar", "true");
//
//    // the proper way to show a jframe (invokeLater)
//    SwingUtilities.invokeLater(new ResultMenuBar());
  }
  
  public ResultMenuBar(ConformanceCheckingResultView conformanceCheckingResultView){
	this.conformanceCheckingResultView = conformanceCheckingResultView;
	this.graph = conformanceCheckingResultView.getModelView().getGraphComponent().getGraph();
	
    menuBar = new JMenuBar();
    
    // build the File menu
    fileMenu = new JMenu("File");
    openMenuItem = new JMenuItem("Open");
    openMenuItem.addActionListener(this);
    fileMenu.add(openMenuItem);

    exportModelMenuItem = new JMenuItem("Export model view");
    exportModelMenuItem.addActionListener(new ActionListener() {
		
		public void actionPerformed(ActionEvent e) {
			// Adds the default file format					
			try
			{
				DefaultFileFilter svgFigureFilter = new DefaultFileFilter(".svg",
						"SVG Figure File " + " (.svg)");
				String wd = lastDir;
	
				JFileChooser fc = new JFileChooser(wd);
				fc.addChoosableFileFilter(svgFigureFilter);					
	
				// Adds special vector graphics formats
				fc.addChoosableFileFilter(new DefaultFileFilter(".pdf",
						"PDF " + mxResources.get("file") + " (.pdf)"));
	
				// Adds filter that accepts all supported image formats
				fc.setFileFilter(svgFigureFilter);
				int rc = fc.showDialog(null, mxResources.get("save"));
	
				if (rc != JFileChooser.APPROVE_OPTION)
				{
					return;
				}
				else
				{
					lastDir = fc.getSelectedFile().getParent();
				}
				
				System.out.println("lastDir: " + lastDir);
				
				String filename = fc.getSelectedFile().getAbsolutePath();
				
				
				String extension = ((DefaultFileFilter) fc.getFileFilter()).getExtension();

				if (!filename.toLowerCase().endsWith(extension))
				{
					filename += extension;
				}
				
				System.out.println("filename: " + filename);
				System.out.println("ext: " + extension);
				
				//check if the file name exists
				if (new File(filename).exists()
						&& JOptionPane.showConfirmDialog(null,
								mxResources.get("overwriteExistingFile")) != JOptionPane.YES_OPTION)
				{
					return;
				}	
				
				String ext = filename.substring(filename.lastIndexOf('.') + 1);
				
				if(ext.equalsIgnoreCase("svg")){
					System.out.println("enter equalsIgnoreCase svg in conformance checking");
					System.out.println("graph cell size: " + graph.getAllCells().size());
					
					mxSvgCanvas canvas = (mxSvgCanvas) mxCellRenderer
							.drawCells(graph, null, 1, null,
									new CanvasFactory()
									{
										public mxICanvas createCanvas(
												int width, int height)
										{
											mxSvgCanvas canvas = new mxSvgCanvas(
													mxDomUtils.createSvgDocument(
															width, height));
											canvas.setEmbedded(true);
		
											return canvas;
										}
									});	
					mxUtils.writeFile(mxXmlUtils.getXml(canvas.getDocument()),
							filename);
				}
				else if (ext.equalsIgnoreCase("pdf")){
					mxSvgCanvas canvas = (mxSvgCanvas) mxCellRenderer
							.drawCells(graph, null, 1, null,
									new CanvasFactory()
									{
										public mxICanvas createCanvas(
												int width, int height)
										{
											mxSvgCanvas canvas = new mxSvgCanvas(
													mxDomUtils.createSvgDocument(
															width, height));
											canvas.setEmbedded(true);

											return canvas;
										}

									});
					SVG2PDF converter= new SVG2PDF();
					File pdfFile=new File(filename);  
					converter.convertSvgString2PdfFile(mxXmlUtils.getXml(canvas.getDocument()),
							pdfFile);
				}						
			}
			catch (Throwable ex)
			{
				ex.printStackTrace();
//				JOptionPane.showMessageDialog(graphComponent,
//						ex.toString(), mxResources.get("error"),
//						JOptionPane.ERROR_MESSAGE);
			}

		}
	});
    fileMenu.add(exportModelMenuItem);
    
    
    // build the Edit menu
    editMenu = new JMenu("Edit");
    cutMenuItem = new JMenuItem("Cut");
    copyMenuItem = new JMenuItem("Copy");
    pasteMenuItem = new JMenuItem("Paste");
    editMenu.add(cutMenuItem);
    editMenu.add(copyMenuItem);
    editMenu.add(pasteMenuItem);

    // build the Edit menu
    helpMenu = new JMenu("Help");
    aboutMenuItem = new JMenuItem("About");
    manualMenuItem = new JMenuItem("Manual");
    helpMenu.add(aboutMenuItem);
    helpMenu.add(manualMenuItem);
    
    // add menus to menubar
    menuBar.add(fileMenu);
//    menuBar.add(editMenu);
    menuBar.add(helpMenu);
  }
  
  public JMenuBar getMenuBar(){
	  return menuBar;
  }
  
  public void run()
  {
    frame = new JFrame("Java Menubar Example");
    menuBar = new JMenuBar();
   
    // build the File menu
    fileMenu = new JMenu("File");
    openMenuItem = new JMenuItem("Open");
    openMenuItem.addActionListener(this);
    fileMenu.add(openMenuItem);

    // build the Edit menu
    editMenu = new JMenu("Edit");
    cutMenuItem = new JMenuItem("Cut");
    copyMenuItem = new JMenuItem("Copy");
    pasteMenuItem = new JMenuItem("Paste");
    editMenu.add(cutMenuItem);
    editMenu.add(copyMenuItem);
    editMenu.add(pasteMenuItem);
    

    
    // add menus to menubar
    menuBar.add(fileMenu);
    menuBar.add(editMenu);

    // put the menubar on the frame
    frame.setJMenuBar(menuBar);

    frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    frame.setPreferredSize(new Dimension(400, 300));
    frame.pack();
    frame.setLocationRelativeTo(null);
    frame.setVisible(true);
  }

  /**
   * This handles the action for the File/Open event, and demonstrates
   * the implementation of an ActionListener.
   * In a real-world program you'd handle this differently.
   */
  public void actionPerformed(ActionEvent ev)
  {
    SampleDialog dialog = new SampleDialog("general");
    dialog.setModal(true);
    dialog.setVisible(true);
  }

  /**
   * This dialog is displayed when the user selects the File/Open menu item.
   */
  private class SampleDialog extends JDialog implements ActionListener
  {
    private JButton okButton = new JButton("OK");

    private SampleDialog(String name)
    {
      super(frame, name, true);
      JPanel panel = new JPanel(new FlowLayout());
      panel.add(okButton);
      getContentPane().add(panel);
      okButton.addActionListener(this);
      setPreferredSize(new Dimension(300, 200));
      pack();
      setLocationRelativeTo(frame);
    }

    public void actionPerformed(ActionEvent ev)
    {
      setVisible(false);
    }
  }
}