package org.processmining.ocbc.conformance.visualization.views;

import java.awt.BorderLayout;
import java.awt.Color;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.swing.BorderFactory;
import javax.swing.JPanel;
import javax.swing.JSplitPane;
import javax.swing.tree.DefaultMutableTreeNode;

import org.processmining.common.frame.CreateTreePanel;
import org.processmining.common.structure.IntegerPair;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.framework.util.Pair;
import org.processmining.mxgraph.examples.swing.GraphEditor.CustomGraph;
import org.processmining.mxgraph.examples.swing.GraphEditor.CustomGraphComponent;
import org.processmining.mxgraph.examples.swing.editor.BasicGraphPanel;
import org.processmining.mxgraph.model.mxCell;
import org.processmining.mxgraph.swing.mxGraphComponent;
import org.processmining.mxgraph.util.mxConstants;
import org.processmining.mxgraph.view.mxGraph;
import org.processmining.ocbc.conformance.visualization.interaction.CCModelGraphMouseListener;
import org.processmining.ocbc.log.model.XOCEvent;
import org.processmining.ocbc.log.model.XOCObject;
import org.processmining.ocbc.log.model.impl.xConstants;
import org.processmining.ocbc.models.AOCEdge;
import org.processmining.ocbc.models.ActivityEdge;
import org.processmining.ocbc.models.ClassEdge;
import org.processmining.ocbc.models.OCBCModel;

import com.fluxicon.slickerbox.components.SlickerTabbedPane;


public class ResultModelView extends JPanel {
	protected SlickerTabbedPane tabPane;
	private ConformanceCheckingResultView conformanceCheckingResult;
	private mxGraphComponent graphComponent;
	private OCBCModel model;
	private Map<ClassEdge, List<XOCObject>> violatedSourceAlwaysClassRelationVSObjectList;
	private Map<ClassEdge, List<XOCObject>> violatedTargetAlwaysClassRelationVSObjectList;
	private Map<ClassEdge, List<XOCObject>> violatedSourceEventuallyClassRelationVSObjectList;
	private Map<ClassEdge, List<XOCObject>> violatedTargetEventuallyClassRelationVSObjectList;
	private Map<String, ClassEdge> violatedClassRelationIdVSRelation = new HashMap<String, ClassEdge>();
	
//	private Map<AOCEdge, Map<XOCObject, List<XOCEvent>>> violatedAOCRelationVSObjectVSMomentList;
	private Map<AOCEdge, Map<XOCObject, List<IntegerPair>>> violatedAOCRelationAlwaysVSObjectVSPeriodList;
	private Map<AOCEdge, Map<XOCObject, List<IntegerPair>>> violatedAOCRelationEventuallyVSObjectVSPeriodList;
	private Map<AOCEdge, List<XOCObject>> violatedAlwaysAOCRelationVSObjectList = new HashMap<AOCEdge, List<XOCObject>>();
	private Map<AOCEdge, List<XOCObject>> violatedEventuallyAOCRelationVSObjectList = new HashMap<AOCEdge, List<XOCObject>>();
	private Map<AOCEdge, List<XOCEvent>> violatedAOCRelationVSEventList;
	private Map<String, AOCEdge> violatedAOCRelationIdVSRelation = new HashMap<String, AOCEdge>();
	
	private Map<ActivityEdge, Map<XOCEvent, List<XOCEvent>>> behavioralConstraintVSEventVSPrecedenceEventList;
	private Map<ActivityEdge, Map<XOCEvent, List<XOCEvent>>> behavioralConstraintVSEventVSResponseEventList;
	private Map<String, ActivityEdge> violatedActivityRelationIdVSRelation = new HashMap<String, ActivityEdge>();
	
	private CreateTreePanel deviationTreePanel;
//	protected DeviatingInstancePanel deviatingInstancePanel = new DeviatingInstancePanel(10, 5, 0);
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	public ResultModelView(UIPluginContext context, OCBCModel model, ConformanceCheckingResultView conformanceCheckingResult){				
		this.conformanceCheckingResult = conformanceCheckingResult;
		this.model = model;
		createLayout(context);
	}

	public mxGraphComponent getGraphComponent(){
		return graphComponent;
	}
	
	public OCBCModel getModel(){
		return model;
	}
	
	public Map<ClassEdge, List<XOCObject>> getViolatedAlwaysSourceClassRelationVSObjectListMap(){
		return violatedSourceAlwaysClassRelationVSObjectList;
	}
	
	public Map<ClassEdge, List<XOCObject>> getViolatedAlwaysTargetClassRelationVSObjectListMap(){
		return violatedTargetAlwaysClassRelationVSObjectList;
	}
	
	public Map<ClassEdge, List<XOCObject>> getViolatedEventuallySourceClassRelationVSObjectListMap(){
		return violatedSourceEventuallyClassRelationVSObjectList;
	}
	
	public Map<ClassEdge, List<XOCObject>> getViolatedEventuallyTargetClassRelationVSObjectListMap(){
		return violatedTargetEventuallyClassRelationVSObjectList;
	}
	
	public Map<String, ClassEdge> getViolatedClassRelationIdVSRelationMap(){
		return violatedClassRelationIdVSRelation;
	}
	
	private void createLayout(UIPluginContext context){
		setOpaque(false);
		setBorder(BorderFactory.createEmptyBorder());
		setLayout(new BorderLayout());
		tabPane = new SlickerTabbedPane("Model inspector", new Color(240, 240, 240, 230), new Color(0, 0, 0, 230),
				new Color(220, 220, 220, 150));

		//create the graph panel (current we reuse the graph panel of the model editor)
		BasicGraphPanel basicGraphPanel = new BasicGraphPanel("mxGraph Editor", this, new CustomGraphComponent(new CustomGraph()));
		graphComponent = basicGraphPanel.getGraphComponent();
		graphComponent.setPageBreaksVisible(false);
		graphComponent.setGridVisible(false);
		mxGraph graph = graphComponent.getGraph();
		mxGraph graphInput = model.getGraphX();

		
		//get the deviation information of class edges
		Map<ClassEdge, List<Pair<Integer, Integer>>> violatedClassRelationVSStartAndEndPairList = conformanceCheckingResult.getObjectModelValidityChecker().getViolatedClassRelationVSStartAndEndPairListMap();
		violatedSourceAlwaysClassRelationVSObjectList = conformanceCheckingResult.getObjectModelValidityChecker().getViolatedSourceClassRelationVSObjectListMap();
		violatedTargetAlwaysClassRelationVSObjectList = conformanceCheckingResult.getObjectModelValidityChecker().getViolatedTargetClassRelationVSObjectListMap();
		List<ClassEdge> violatedClassRelationList = conformanceCheckingResult.getFulfilmentChecker().getViolatedClassRelationList();
		violatedSourceEventuallyClassRelationVSObjectList = conformanceCheckingResult.getFulfilmentChecker().getViolatedSourceClassRelationVSObjectListMap();
		violatedTargetEventuallyClassRelationVSObjectList = conformanceCheckingResult.getFulfilmentChecker().getViolatedTargetClassRelationVSObjectListMap();

		//get the deviation information of activity edges
		behavioralConstraintVSEventVSPrecedenceEventList = conformanceCheckingResult.getBehavioralConstraintsChecker().getBehavioralConstraintVSEventVSPrecedenceEventListMap();
		behavioralConstraintVSEventVSResponseEventList = conformanceCheckingResult.getBehavioralConstraintsChecker().getBehavioralConstraintVSEventVSResponseEventListMap();
		
		//add the graph mouse click listener
		CCModelGraphMouseListener graphComponentMouseListener = new CCModelGraphMouseListener(this);
		graphComponent.getGraphControl().addMouseListener(graphComponentMouseListener);
		
		//process the deviation information to get achieved violated class relations
		List<String> violatedClassRelationIdList = new ArrayList<String>();
		for(ClassEdge classRelation : violatedClassRelationVSStartAndEndPairList.keySet()){
			String classRelationId = classRelation.getID();
			System.out.println("classRelationId: " + classRelationId);
			if(!violatedClassRelationIdList.contains(classRelationId)){
				violatedClassRelationIdList.add(classRelationId);
			}
			violatedClassRelationIdVSRelation.put(classRelationId, classRelation);
		}
		for(ClassEdge classRelation : violatedClassRelationList){
			String classRelationId = classRelation.getID();
			System.out.println("classRelationId: " + classRelationId);
			if(!violatedClassRelationIdList.contains(classRelationId)){
				violatedClassRelationIdList.add(classRelationId);
			}
			violatedClassRelationIdVSRelation.put(classRelationId, classRelation);
		}		
		
		//get the deviation information of activity edges	
		behavioralConstraintVSEventVSPrecedenceEventList = conformanceCheckingResult.getBehavioralConstraintsChecker().getBehavioralConstraintVSEventVSPrecedenceEventListMap();
		List<String> violatedActivityRelationIdList = new ArrayList<String>();
		for(ActivityEdge relation : behavioralConstraintVSEventVSPrecedenceEventList.keySet()){
			String relationId = relation.getID();
			int deviatingEventNum = behavioralConstraintVSEventVSPrecedenceEventList.get(relation).size();
			if(deviatingEventNum>0 && !violatedActivityRelationIdList.contains(relationId)){
				violatedActivityRelationIdList.add(relationId);
				violatedActivityRelationIdVSRelation.put(relationId, relation);
			}
		}			
		
		//get the deviation information of AOC edges
		List<AOCEdge> violatedAOCRelationList = conformanceCheckingResult.getEventNumberPerObjectChecker().getViolatedAOCRelationList();
//		violatedAOCRelationVSObjectVSMomentList = conformanceCheckingResult.
//				getEventNumberPerObjectChecker().getViolatedAOCRelationVSObjectVSMomentListMap();
//		for(AOCEdge relation : violatedAOCRelationVSObjectVSMomentList.keySet()){
//			List<XOCObject> objectList = new ArrayList<>(violatedAOCRelationVSObjectVSMomentList.get(relation).keySet());
//			violatedAlwaysAOCRelationVSObjectList.put(relation, objectList);
//		}
		violatedAOCRelationAlwaysVSObjectVSPeriodList = conformanceCheckingResult.
				getEventNumberPerObjectChecker().getViolatedAOCRelationAlwaysVSObjectVSPeriodListMap();
		for(AOCEdge relation : violatedAOCRelationAlwaysVSObjectVSPeriodList.keySet()){
			List<XOCObject> objectList = new ArrayList<>(violatedAOCRelationAlwaysVSObjectVSPeriodList.get(relation).keySet());
			violatedAlwaysAOCRelationVSObjectList.put(relation, objectList);
		}
		
		
		violatedAOCRelationEventuallyVSObjectVSPeriodList = conformanceCheckingResult.
				getEventNumberPerObjectChecker().getViolatedAOCRelationEventuallyVSObjectVSPeriodListMap();
		for(AOCEdge relation : violatedAOCRelationEventuallyVSObjectVSPeriodList.keySet()){
			List<XOCObject> objectList = new ArrayList<>(violatedAOCRelationEventuallyVSObjectVSPeriodList.get(relation).keySet());
			violatedEventuallyAOCRelationVSObjectList.put(relation, objectList);
		}
		
		violatedAOCRelationVSEventList = conformanceCheckingResult.
				getObjectNumberPerEventChecker().getViolatedAOCRelationVSEventListMap();
		List<String> violatedAOCRelationIdList = new ArrayList<String>();
		for(AOCEdge relation : violatedAOCRelationList){
			String relationId = relation.getID();		
			if(!violatedAOCRelationIdList.contains(relationId)){
				violatedAOCRelationIdList.add(relationId);
				violatedAOCRelationIdVSRelation.put(relationId, relation);
			}
		}
		for(AOCEdge relation : violatedAOCRelationVSEventList.keySet()){
			String relationId = relation.getID();
			int deviatingEventNum = violatedAOCRelationVSEventList.get(relation).size();
			if(deviatingEventNum>0 && !violatedAOCRelationIdList.contains(relationId)){
				violatedAOCRelationIdList.add(relationId);
				violatedAOCRelationIdVSRelation.put(relationId, relation);
			}
		}
		
		//highlight the violated relations (cells in the graph)
		List<mxCell> deviatingCellList = new ArrayList<mxCell>();
		List<mxCell> normalCellList = new ArrayList<mxCell>();
		for(Object objectCell : graphInput.getOffspringCells()){
			System.out.println("add cell to graph");
			mxCell cell = (mxCell) objectCell;
			String cellType = cell.getAttributes().get(mxConstants.TYPE);
			System.out.println("cellType: " + cellType);
			String cellId = cell.getId();
			System.out.println("cellId: " + cellId);
			
			if(cellType.equals(mxConstants.CLASSRELATION)){
				System.out.println(mxConstants.CLASSRELATION);
				if(violatedClassRelationIdList.contains(cellId)){
					deviatingCellList.add(cell);
					//child list- 0:source always; 1:source eventually; 2:target always; 3:target eventually;
					int childNum = cell.getChildCount();
					System.out.println("childNum: " + childNum);
					//add the violated cardinality
					ClassEdge violatedClassRelation =  violatedClassRelationIdVSRelation.get(cellId);
					//if the source always cardinality is violated
					if(violatedSourceAlwaysClassRelationVSObjectList.containsKey(violatedClassRelation) &&
							violatedSourceAlwaysClassRelationVSObjectList.get(violatedClassRelation).size()>0){
						deviatingCellList.add((mxCell) cell.getChildAt(0));
					}
					if(violatedSourceEventuallyClassRelationVSObjectList.containsKey(violatedClassRelation) &&
							violatedSourceEventuallyClassRelationVSObjectList.get(violatedClassRelation).size()>0){
						deviatingCellList.add((mxCell) cell.getChildAt(1));
					}
					if(violatedTargetAlwaysClassRelationVSObjectList.containsKey(violatedClassRelation) &&
							violatedTargetAlwaysClassRelationVSObjectList.get(violatedClassRelation).size()>0){
						deviatingCellList.add((mxCell) cell.getChildAt(2));
					}
					if(violatedTargetEventuallyClassRelationVSObjectList.containsKey(violatedClassRelation) &&
							violatedTargetEventuallyClassRelationVSObjectList.get(violatedClassRelation).size()>0){
						deviatingCellList.add((mxCell) cell.getChildAt(3));
					}					
				}
				else{
					normalCellList.add(cell);
				}
			}			
			else if(cellType.equals(mxConstants.ACTIVITYRELATION)){
				System.out.println(mxConstants.ACTIVITYRELATION);
				String cardinalityType = cell.getAttributes().get(mxConstants.CARDINALITYTYPE);
				if(cardinalityType.split("-").length == 1 && violatedActivityRelationIdList.contains(cellId)){
					deviatingCellList.add(cell);
				}
				else if(cardinalityType.split("-").length == 2 && 
						(violatedActivityRelationIdList.contains(cellId + "_0") || violatedActivityRelationIdList.contains(cellId + "_1"))){
					deviatingCellList.add(cell);
				}
				else{
					normalCellList.add(cell);
				}
			}
			else if(cellType.equals(mxConstants.AOCRELATION)){
				System.out.println(mxConstants.AOCRELATION);
				if(violatedAOCRelationIdList.contains(cellId)){
					deviatingCellList.add(cell);
					//add the violated cardinality
					AOCEdge violatedAOCRelation =  violatedAOCRelationIdVSRelation.get(cellId);
					//if the source always cardinality is violated
					if(violatedAlwaysAOCRelationVSObjectList.containsKey(violatedAOCRelation) &&
							violatedAlwaysAOCRelationVSObjectList.get(violatedAOCRelation).size()>0){
						deviatingCellList.add((mxCell) cell.getChildAt(0));
					}
					if(violatedEventuallyAOCRelationVSObjectList.containsKey(violatedAOCRelation) &&
							violatedEventuallyAOCRelationVSObjectList.get(violatedAOCRelation).size()>0){
						deviatingCellList.add((mxCell) cell.getChildAt(1));
					}
					if(violatedAOCRelationVSEventList.containsKey(violatedAOCRelation) &&
							violatedAOCRelationVSEventList.get(violatedAOCRelation).size()>0){
						deviatingCellList.add((mxCell) cell.getChildAt(2));
					}		
				}
				else{
					normalCellList.add(cell);
				}
			}
			
			graph.addCell(cell);
		}	
		graph.setCellStyles(mxConstants.STYLE_STROKECOLOR, "red",  deviatingCellList.toArray());	
		graph.setCellStyles(mxConstants.STYLE_FONTCOLOR, "red",  deviatingCellList.toArray());	
		graph.refresh();
		graph.repaint();

		//create the deviation hierarchy panel
		deviationTreePanel = new CreateTreePanel(new DefaultMutableTreeNode("no relation is selected"), mxConstants.conformance_checking_output);
		
		basicGraphPanel.refreshGraph();//update the label (in case square symbol is ?)
		//create the layout for the top panel
		JSplitPane leftRight = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, basicGraphPanel.getGraphComponent(), 
//				new JScrollPane(deviatingInstancePanel));
				deviationTreePanel);

		
		leftRight.setResizeWeight(0.9);
		leftRight.validate();	
		tabPane.addTab("Browser", leftRight);
		this.add(tabPane);	
		
    }	
	
	public CreateTreePanel createDeviationHierarchyPanel(){

		CreateTreePanel deviationTreePanel = new CreateTreePanel(new DefaultMutableTreeNode("Event complex"), mxConstants.conformance_checking_output);	
		return deviationTreePanel;
	}
	
	public void updateDeviationHierarchyPanelForClassRelation(String relationId){
		ClassEdge relation = violatedClassRelationIdVSRelation.get(relationId);
		if(relation != null){
			String relationName = relation.getEdgeName();
			deviationTreePanel.updateTreePanel(generateDeviatingClassRelationNode(relation, relationName), mxConstants.conformance_checking_output);			
		}
		else{
			deviationTreePanel.updateTreePanel(new DefaultMutableTreeNode(relationId), mxConstants.conformance_checking_output);			
			
		}
	}
	
	/**
	 * update the deviation hierarchy panel for an input aoc relation id
	 * @param relationId
	 */
	public void updateDeviationHierarchyPanelForAOCRelation(String relationId){
		AOCEdge relation = violatedAOCRelationIdVSRelation.get(relationId);	
		if(relation != null){
//			String relationName = relation.getEdgeName();
			String relationName = relation.getID();
			deviationTreePanel.updateTreePanel(generateDeviatingAOCRelationNode(relation, relationName), mxConstants.conformance_checking_output);			
		}
		else{
			deviationTreePanel.updateTreePanel(new DefaultMutableTreeNode(relationId), mxConstants.conformance_checking_output);			
			
		}
	}
	
	/**
	 * update the deviation hierarchy panel for an input activity relation id
	 * @param relationId
	 * @param cardinalityType
	 */
	public void updateDeviationHierarchyPanelForActivityRelation(String relationId, String cardinalityType){
		if(cardinalityType.split("-").length == 1){		
			ActivityEdge relation = violatedActivityRelationIdVSRelation.get(relationId);	
			if(relation != null){
				deviationTreePanel.updateTreePanel(generateDeviatingActivityRelationNode(relation, relationId), mxConstants.conformance_checking_output);				
			}
			else{
				deviationTreePanel.updateTreePanel(new DefaultMutableTreeNode(relationId), mxConstants.conformance_checking_output);							
			}	
		}
		else if(cardinalityType.split("-").length == 2){
			String relationIdFirst = relationId + "_0";
			String relationIdSecond = relationId + "_1";
			ActivityEdge relationFirst = violatedActivityRelationIdVSRelation.get(relationIdFirst);	
			ActivityEdge relationSecond = violatedActivityRelationIdVSRelation.get(relationIdSecond);
			
			DefaultMutableTreeNode nodeCombined = new DefaultMutableTreeNode(relationId);
			DefaultMutableTreeNode nodeFirst = new DefaultMutableTreeNode(relationIdFirst);
			DefaultMutableTreeNode nodeSecond = new DefaultMutableTreeNode(relationIdSecond);
			if(relationFirst != null){			
				nodeFirst = generateDeviatingActivityRelationNode(relationFirst, relationIdFirst);		
			}
			if(relationSecond != null){			
				nodeSecond = generateDeviatingActivityRelationNode(relationSecond, relationIdSecond);		
			}
			nodeCombined.add(nodeFirst);
			nodeCombined.add(nodeSecond);
			deviationTreePanel.updateTreePanel(nodeCombined, mxConstants.conformance_checking_output);		
		}
	}
	
	private DefaultMutableTreeNode generateObjectNode(XOCObject object){
		DefaultMutableTreeNode objectNode = new DefaultMutableTreeNode(new TreeEntry(object.getAttributes().get("id").toString()));
		for(String key : object.getAttributes().keySet()){
			objectNode.add(new DefaultMutableTreeNode(new TreeEntry(key + " : " + object.getAttributes().get(key).toString())));
		}		
		return  objectNode;
	}
	
	private DefaultMutableTreeNode generateObjectSetNode(List<XOCObject> objects, String name){	
		DefaultMutableTreeNode objectSetNode = new DefaultMutableTreeNode(new TreeEntry(name));
		if(objects != null){
			for (XOCObject object : objects) {
				objectSetNode.add(generateObjectNode(object));
			}
		}
		return objectSetNode;
	}
	
	public DefaultMutableTreeNode generateDeviatingClassRelationNode(ClassEdge relation, String name){
		DefaultMutableTreeNode relationNode = new DefaultMutableTreeNode(new TreeEntry(name));
		String sourceAlwaysCardinality = relation.getSourceAlwaysCardinality().toString();
		String targetAlwaysCardinality = relation.getTargetAlwaysCardinality().toString();
		String sourceEventuallyCardinality = relation.getSourceEventuallyCardinality().toString();
		String targetEventuallyCardinality = relation.getTargetEventuallyCardinality().toString();
		
		List<XOCObject> objectListViolatedSourceAlways = violatedSourceAlwaysClassRelationVSObjectList.get(relation);
		List<XOCObject> objectListViolatedTargetAlways = violatedTargetAlwaysClassRelationVSObjectList.get(relation);
		List<XOCObject> objectListViolatedSourceEventually = violatedSourceEventuallyClassRelationVSObjectList.get(relation);
		List<XOCObject> objectListViolatedTargetEventually = violatedTargetEventuallyClassRelationVSObjectList.get(relation);
						
		relationNode.add(generateObjectSetNode(objectListViolatedSourceAlways, xConstants.source_always_cardinality + ": " + sourceAlwaysCardinality));
		relationNode.add(generateObjectSetNode(objectListViolatedTargetAlways, xConstants.target_always_cardinality + ": " + targetAlwaysCardinality));
		relationNode.add(generateObjectSetNode(objectListViolatedSourceEventually, xConstants.source_eventually_cardinality + ": " + sourceEventuallyCardinality));
		relationNode.add(generateObjectSetNode(objectListViolatedTargetEventually, xConstants.target_eventually_cardinality + ": " + targetEventuallyCardinality));
		
		return relationNode;
	}

	private DefaultMutableTreeNode generateObjectReferenceListNode(List<XOCObject> objectReferenceList, String name){
		DefaultMutableTreeNode listNode = new DefaultMutableTreeNode(new TreeEntry(name));
		for (XOCObject element : objectReferenceList) {
			listNode.add(generateObjectNode(element));
		}
		return listNode;
	}
	
	private DefaultMutableTreeNode generateEventNode(XOCEvent event, boolean displayAttributeListFlag, boolean displayObjectReferenceList,
			boolean displayNormalObjectModelFlag, boolean displayUpdatedObjectModelFlag, String name){
		DefaultMutableTreeNode eventNode = new DefaultMutableTreeNode(new TreeEntry(name));
		if(displayObjectReferenceList){
			eventNode.add(generateObjectReferenceListNode(event.getRelatedObjects(), mxConstants.object_reference_list));
		}
		return eventNode;
	}
	
	private DefaultMutableTreeNode generateEventSetNode(List<XOCEvent> eventList, String name){	
		DefaultMutableTreeNode eventSetNode = new DefaultMutableTreeNode(new TreeEntry(name));
		if(eventList != null){
			for (XOCEvent event : eventList) {
				String eventId = event.getAttributes().get(xConstants.ID).toString();
				eventSetNode.add(generateEventNode(event, false, true, false, false, eventId));
			}
		}
		return eventSetNode;
	}
	
	public DefaultMutableTreeNode generateDeviatingAOCRelationNode(AOCEdge relation, String name){
		DefaultMutableTreeNode relationNode = new DefaultMutableTreeNode(new TreeEntry(name));
		String sourceAlwaysCardinality = relation.getActivityAlwaysCardinality().toString();
		String sourceEventuallyCardinality = relation.getActivityEventuallyCardinality().toString();
		String targetCardinality = relation.getClassCardinality().toString();

		List<XOCObject> objectListViolatedSourceAlways = violatedAlwaysAOCRelationVSObjectList.get(relation);
		List<XOCObject> objectListViolatedSourceEventually = violatedEventuallyAOCRelationVSObjectList.get(relation);
		List<XOCEvent> eventListViolatedTarget =  violatedAOCRelationVSEventList.get(relation);
						
		if(objectListViolatedSourceAlways.size() > 0){
			relationNode.add(generateObjectSetNode(objectListViolatedSourceAlways, xConstants.source_always_cardinality + ": " + sourceAlwaysCardinality));		
		}
		if(objectListViolatedSourceEventually.size() > 0){
			relationNode.add(generateObjectSetNode(objectListViolatedSourceEventually, xConstants.source_eventually_cardinality + ": " + sourceEventuallyCardinality));		
		}
		if(eventListViolatedTarget.size() > 0){
			relationNode.add(generateEventSetNode(eventListViolatedTarget, mxConstants.target_cardinality + ": " + targetCardinality));			
		}
	
		return relationNode;
	}
	
	public DefaultMutableTreeNode generateDeviatingActivityRelationNode(ActivityEdge relation, String name){
		DefaultMutableTreeNode relationNode = new DefaultMutableTreeNode(new TreeEntry(name));
		Map<XOCEvent, List<XOCEvent>> eventVSPrecedenceEventList = behavioralConstraintVSEventVSPrecedenceEventList.get(relation);
		Map<XOCEvent, List<XOCEvent>> eventVSResponseEventList = behavioralConstraintVSEventVSResponseEventList.get(relation);
								
		for(XOCEvent event : eventVSPrecedenceEventList.keySet()){
			String eventId = event.getAttributes().get(xConstants.ID).toString();
			DefaultMutableTreeNode eventNode = new DefaultMutableTreeNode(new TreeEntry(eventId));
			eventNode.add(generateEventSetNode(eventVSPrecedenceEventList.get(event), mxConstants.PRECEDENCE + ": " + eventVSPrecedenceEventList.get(event).size()));
			eventNode.add(generateEventSetNode(eventVSResponseEventList.get(event), mxConstants.RESPONSE + ": " + eventVSResponseEventList.get(event).size()));
			relationNode.add(eventNode);
		}
		
		return relationNode;
	}
	

	
	private class TreeEntry {
		private String key;
		private String displaytext;
		
		public TreeEntry(String key) {
			this(key, key);
		}
		
		public TreeEntry(String key, String displaytext) {
			this.key = key;
			this.displaytext = displaytext;
		}
		
		public String getKey() {
			return this.key;
		}
		
		public String getDisplayText() {
			return this.displaytext;
		}
		
		public String toString() {
			return getDisplayText();
		}
	}
	
	public static void main(String[] args){	  
		
    }

}
