package org.processmining.ocbc.discovery.filter;

import java.util.ArrayList;
import java.util.List;

import org.processmining.common.structure.IntegerPair;
import org.processmining.ocbc.log.model.XOCAttribute;
import org.processmining.ocbc.log.model.XOCEvent;
import org.processmining.ocbc.log.model.XOCObject;
import org.processmining.ocbc.log.model.impl.xConstants;
import org.processmining.ocbc.models.ActivityEdge;


public class FilterFunction {
	
	/*
	 * 
	 */
	public List<XOCEvent> filterEvents(List<XOCEvent> inputEvents, XOCEvent eventReference, String activityName, int filterType){
		List<XOCEvent> outputEvents = new ArrayList<XOCEvent>();
		switch(filterType)
		{
			case 1:
				for(XOCEvent eventCandidate : inputEvents){
					if(inputEvents.indexOf(eventCandidate) <= inputEvents.indexOf(eventReference))
						outputEvents.add(eventCandidate);
				}			
				break;
			case 2:
				for(XOCEvent eventCandidate : inputEvents){
					if(inputEvents.indexOf(eventCandidate) >= inputEvents.indexOf(eventReference))
						outputEvents.add(eventCandidate);
				}				
				break;
			case 3:
				for(XOCEvent eventCandidate : inputEvents){
					if(inputEvents.indexOf(eventCandidate) < inputEvents.indexOf(eventReference))						
						outputEvents.add(eventCandidate);
				}			
				break;
			case 4:
				for(XOCEvent eventCandidate : inputEvents){
					if(inputEvents.indexOf(eventCandidate) > inputEvents.indexOf(eventReference))
						outputEvents.add(eventCandidate);
				}
				break;
			case 5:
				for(XOCEvent eventCandidate : inputEvents){
					if(eventCandidate.getAttributes().get("activity").toString().equals(activityName))
						outputEvents.add(eventCandidate);
				}
				break;
		}
		return outputEvents;
	}
	
	/**
	 * get objects which once have the input className
	 * @param inputObjects
	 * @param className
	 * @return
	 */
	public List<XOCObject> filterObjects(List<XOCObject> inputObjects, String className){
		List<XOCObject> outputObjects = new ArrayList<XOCObject>();
		for(XOCObject objectCandidate: inputObjects){
			XOCAttribute classXOCAttribute = objectCandidate.getXOCAttributes().get(xConstants.CLASS);
			if(classXOCAttribute != null && classXOCAttribute.getValueList().contains(className)){
				outputObjects.add(objectCandidate);
			}				
		}
		return outputObjects;
	}
	
	/**
	 * get objects which once have the input className
	 * @param inputObjects
	 * @param className
	 * @return
	 */
	public List<XOCObject> filterObjectsByClassAtEvent(List<XOCObject> inputObjects, String className, XOCEvent event){
		List<XOCObject> outputObjects = new ArrayList<XOCObject>();
		for(XOCObject objectCandidate: inputObjects){
			XOCAttribute classXOCAttribute = objectCandidate.getXOCAttributes().get(xConstants.CLASS);
			if(classXOCAttribute != null){
				String valueVersion = classXOCAttribute.getValueVersion(event.getOrderNum());
				if(valueVersion != null && valueVersion.equals(className)){
					outputObjects.add(objectCandidate);
				}		
			}				
		}
		return outputObjects;
	}
	
	//filter activity relations by the source node, target node, and relation type (at least one parameter is not empty)
	public List<ActivityEdge> filterActivityRelations(List<ActivityEdge> inputActivityEdges, String sourceNodeId,
			String targetNodeId, String relationType){
		List<ActivityEdge> outputActicityEdges = new ArrayList<ActivityEdge>();
//		System.out.printf("\n sourceNodeId" + sourceNodeId);
//		System.out.printf("\n targetNodeId" + targetNodeId);
//		System.out.printf("\n relationType" + relationType);
		for(ActivityEdge activityEdge : inputActivityEdges){			
			if((sourceNodeId == null ? true : activityEdge.getReferenceNode().getID().toString().equals(sourceNodeId)) && 
					(targetNodeId == null ? true : activityEdge.getTargetNode().getID().toString().equals(targetNodeId)) &&
					(relationType == null ? true : activityEdge.getCardinalityType().equals(relationType))){
				outputActicityEdges.add(activityEdge);
			}
		}
		return outputActicityEdges;
	}
	
	/**
	 * filter nonoverlapping periods parts, i.e., get the overlapping parts of two period lists;
	 * @param periodListA
	 * @param periodListB
	 * @param maximalValue
	 * @return
	 */
	public List<IntegerPair> filterNonOverlappingPeriodParts(List<IntegerPair> periodListA, List<IntegerPair> periodListB, int maximalValue){
		List<IntegerPair> overlappingPeriodList = new ArrayList<IntegerPair>();	
		for(IntegerPair periodA : periodListA){
			//identify the overlapping part
			int startA = periodA.getFirstValue();
			int endA = periodA.getSecondValue() == null ? maximalValue : periodA.getSecondValue();
			for(IntegerPair periodB : periodListB){
				int startB = periodB.getFirstValue();
				int endB = periodB.getSecondValue() == null ? maximalValue : periodB.getSecondValue();
				
				int startOutput = startA > startB ? startA : startB;
				int endOutput = endA < endB ? endA : endB;
				if(startOutput <= endOutput){
					overlappingPeriodList.add(new IntegerPair(startOutput, endOutput));
				}
			}
		}
		return overlappingPeriodList;
	}
	
    public static void main(String[] args) throws Exception {
  
    }
}