package org.processmining.ocbc.discovery.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.processmining.common.structure.IntegerPair;
import org.processmining.ocbc.discovery.filter.FilterFunction;
import org.processmining.ocbc.log.model.XOCEvent;
import org.processmining.ocbc.log.model.XOCLog;
import org.processmining.ocbc.log.model.XOCObject;
import org.processmining.ocbc.log.model.XOCObjectModel;
import org.processmining.ocbc.log.model.impl.xConstants;
import org.processmining.ocbc.models.visualization.ModelLayoutBuilder;


public class AOCEdgeDiscovery {
	private XOCLog log;
	private ModelLayoutBuilder builder;
	private Map<String, String> nodeNameVSNodeId;
	private FilterFunction filter;
	private Integer idCounter;
	private Map<String, Map<XOCEvent, List<XOCObject>>> relationIdVSEventVSObjectList = new HashMap<String, Map<XOCEvent, List<XOCObject>>>();;
	//record the related object id list of each object in object model of each event
	private Map<String, Map<String, Map<String, List<String>>>> eventIdVSRelationIdVSObjectVSEventList = new HashMap<String, Map<String, Map<String, List<String>>>>();

	//record the eventually referred event list for each object		
	private Map<String, List<String>> objectIdVSEventuallyReferredEventIdList = new HashMap<String, List<String>>();
	
	/**
	 * record the cardinality and life cycle period of objects of each AOC relation;
	 * in other words, give the details of the target always cardinality;
	 */
	private Map<String, Map<XOCObject, Map<IntegerPair, Integer>>> aOCRelationIdVSObjectVSPeriodVSCardinality = new HashMap<String, Map<XOCObject, Map<IntegerPair, Integer>>>();	
	/**
	 * record the life cycle period of objects of each AOC relation;
	 * indicate the order of periods;
	 */
	private Map<String, Map<XOCObject, List<IntegerPair>>> aOCRelationIdVSObjectVSPeriodList = new HashMap<String, Map<XOCObject, List<IntegerPair>>> ();
	
	
	public AOCEdgeDiscovery(XOCLog log, ModelLayoutBuilder builder, Map<String, String> nodeNameVSNodeId){
		this.log = log;
		this.builder = builder;
		this.nodeNameVSNodeId = nodeNameVSNodeId;
		filter = new FilterFunction();
	}
	
	public void setIdCounter(Integer idCounter){
		this.idCounter = idCounter;
	}
	
	public Integer getIdCounter(){
		return idCounter;
	}
	
	public void discoverAOCEdges(){
		 List<String> aOCRelationcombinationIds = new ArrayList<String>();
		//discover AOC edges
		for(XOCEvent event : log.getAllEvents()){
			String activity = event.getAttributes().get(xConstants.ACTIVITY).toString();
			for(XOCObject object : event.getRelatedObjects()){
				String className = object.getAttributes().get(xConstants.CLASS).toString();
				
				String activityNodeId = nodeNameVSNodeId.get(activity);		
				String classNodeId = nodeNameVSNodeId.get(className);
				
				String aOCRelationcombinationId = activity + "_" + className;				
				if(!aOCRelationcombinationIds.contains(aOCRelationcombinationId)){
					aOCRelationcombinationIds.add(aOCRelationcombinationId);
					
//					long uint = 1;	
//					long startTime = System.currentTimeMillis(); 
//					long durationTime = System.currentTimeMillis();
//
//					RunningTime.writeTimeInformationIntoFile("discoverOneAOCEdge start time: " + startTime/uint, false, "D:\\timeDiscovery.txt");
					
					//discover the always cardinality
					List<Integer> aOCSourceAlwaysCardinalityList = new ArrayList<Integer>();
					List<Integer> aOCSourceEventuallyCardinalityList = new ArrayList<Integer>();
					List<Integer> aOCTargetCardinalityList = new ArrayList<Integer>();
					String aocEdgeId = activity + "-" + className + (idCounter ++).toString();

					
					discoverAOCCardinality(log, activity, className,
							aOCSourceAlwaysCardinalityList, aOCSourceEventuallyCardinalityList, aOCTargetCardinalityList, aocEdgeId);
					
					
					String aOCSourceEventuallyCardinality =  standardizeEventuallyCardinality(aOCSourceEventuallyCardinalityList);
					String aOCSourceAlwaysCardinality = standardizeAlwaysCardinality(aOCSourceAlwaysCardinalityList, aOCSourceEventuallyCardinality);
					String aOCTargetCardinality =  standardizeEventuallyCardinality(aOCTargetCardinalityList);
					
//					Cardinality aOCSourceAlwaysCardinality = new Cardinality("*");
//					Cardinality aOCSourceEventuallyCardinality = new Cardinality("*");
//					Cardinality aOCTargetCardinality = new Cardinality("*");
														

					builder.addAOCEdge(aocEdgeId, activityNodeId, aOCSourceAlwaysCardinality.toString(), aOCSourceEventuallyCardinality.toString(),
							classNodeId, aOCTargetCardinality.toString(),  0, 0);		
					
//					durationTime = (System.currentTimeMillis() - startTime)/uint;
//					startTime = System.currentTimeMillis();
//					RunningTime.writeTimeInformationIntoFile("discoverOneAOCEdge end time: " + System.currentTimeMillis()/uint, false, "D:\\timeDiscovery.txt");
//					RunningTime.writeTimeInformationIntoFile("~~~~~~~~~~~~~~discoverOneAOCEdge duration time: " + durationTime, false, "D:\\timeDiscovery.txt");					
					System.out.println("aoc relation id: " + aocEdgeId);
				}
			}
		}
	}
	
	/**
	 * build the map which record the mother event
	 * (the object first appear in the object model of this event) for each object.
	 * note that the map should be clear for each aoc relation
	 */
	public Map<String, String> buildObjectIdVSMotherEventIdMap(String className){
		//record the mother event (the object first appear in the object model of this event) for each object		
		Map<String, String> objectIdVSMotherEventId = new HashMap<String, String>();
		List<String> bornObjectList = new ArrayList<String>();
		for(XOCEvent event : log.getAllEvents()){   
			String eventId = event.getAttributes().get(xConstants.ID).toString();
			XOCObjectModel objectModel = event.getObjectModel();
			for(XOCObject object : filter.filterObjects(objectModel.getObjectSet(), className)){
				String objectId = object.getAttributes().get(xConstants.ID).toString();
				if(!bornObjectList.contains(objectId)){
					bornObjectList.add(objectId);
					System.out.println("objectId in buildObjectIdVSMotherEventIdMap: " + objectId);
					System.out.println("eventId in buildObjectIdVSMotherEventIdMap: " + eventId);
					objectIdVSMotherEventId.put(objectId, eventId);
				}
			}
		}	
		return objectIdVSMotherEventId;
	}
	
	/**
	 * discover target cardinality
	 */
	public void discoverTargetCardinality(String activityName, String className,
			List<Integer> aOCTargetCardinality, String aocEdgeId){	
		for(XOCEvent event : filter.filterEvents(log.getAllEvents(), null, activityName, 5)){	
//			String eventId = event.getAttributes().get(xConstants.ID).toString();
//			List<XOCObject> relatedTargetObjects = filter.filterObjects(event.getRelatedObjects(), className);
			//get each object whose class value version at the event equals to className
			List<XOCObject> relatedTargetObjects = filter.filterObjectsByClassAtEvent(event.getRelatedObjects(), className, event);
			if(!relationIdVSEventVSObjectList.containsKey(aocEdgeId)){
				relationIdVSEventVSObjectList.put(aocEdgeId, new HashMap<XOCEvent, List<XOCObject>>());
			}
			if(!relationIdVSEventVSObjectList.get(aocEdgeId).containsKey(event)){
				relationIdVSEventVSObjectList.get(aocEdgeId).put(event, new ArrayList<XOCObject>());
			}
			
			for(XOCObject object : relatedTargetObjects){
//				String objectId = object.getAttributes().get(xConstants.ID).toString();
				relationIdVSEventVSObjectList.get(aocEdgeId).get(event).add(object);
//				System.out.println("objectId: " + objectId);
			}			
			int relatedObjectNumber = relatedTargetObjects.size();
//			System.out.println("activityName: " + activityName);
//			System.out.println("className: " + className);
//			System.out.println("eventId: " + eventId);
//			System.out.println("relatedObjectNumber in discoverTargetCardinality: " + relatedObjectNumber);
			if(!aOCTargetCardinality.contains(relatedObjectNumber)){
				aOCTargetCardinality.add(relatedObjectNumber);
			}
		}		
	}
	
	/**
	 * discover source Eventually cardinality
	 */
	public void discoverSourceEventuallyCardinality(List<Integer> aOCSourceEventuallyCardinality, String activityName, String className){	
		//discover source eventually cardinality
//		XOCEvent lastEvent = log.getLastEvent();
//		for(XOCObject object : filter.filterObjects(lastEvent.getObjectModel().getObjectSet(), className)){		
		for(XOCObject object : filter.filterObjects(log.getObjectList(), className)){	
			String objectId = object.getAttributes().get(xConstants.ID).toString();
//			System.out.println("object id eventually: " + objectId);
			objectIdVSEventuallyReferredEventIdList.put(objectId, new ArrayList<String>());
//			System.out.println("objectId in discoverSourceEventuallyCardinality: " + objectId);

			int activityCardinalityValue = 0;
			for(XOCEvent relatedEvent : filter.filterEvents(object.getRelatedEventList(), null, activityName, 5)){    									
				String eventId = relatedEvent.getAttributes().get(xConstants.ID).toString();
				objectIdVSEventuallyReferredEventIdList.get(objectId).add(eventId);
				activityCardinalityValue++;	        															
			}
			
			if(!aOCSourceEventuallyCardinality.contains(activityCardinalityValue)){
//				System.out.println("add into aOCSourceEventuallyCardinality: " + activityCardinalityValue);
				aOCSourceEventuallyCardinality.add(activityCardinalityValue);
			}
													
		}
	}
	
	/**
	 * discover source always cardinality
	 * idea: based on the eventually cardinality and the birth event
	 */
	public void discoverSourceAlwaysCardinality(List<Integer> aOCSourceAlwaysCardinality, String className, String aocEdgeId){	
		XOCEvent event = log.getAllEvents().get(log.getAllEvents().size()-1);	

		//to do ...
		aOCRelationIdVSObjectVSPeriodVSCardinality.put(aocEdgeId, new HashMap<XOCObject, Map<IntegerPair,Integer>>());
		aOCRelationIdVSObjectVSPeriodList.put(aocEdgeId, new HashMap<XOCObject, List<IntegerPair>>());
		
		//get the number of most events referring to one object
		int maximalNumber = 0;
		for(String objectId : objectIdVSEventuallyReferredEventIdList.keySet()){
			int eventNumber = objectIdVSEventuallyReferredEventIdList.get(objectId).size();
			if(maximalNumber < eventNumber){
				maximalNumber = eventNumber;
			}
		}
		//check if object does not have a referred event when it is born (i.e.,
		//check if 0 is in the source always cardinality)
		
//		List<XOCObject> targetObjects = filter.filterObjects(event.getObjectModel().getObjectSet(), className);	
		List<XOCObject> targetObjects = filter.filterObjects(log.getObjectList(), className);	
		for(XOCObject targetObject : targetObjects){
//			String birthEventId = targetObject.getBirthEventId();//changed at August 9, 2017
			int eventOrderNumOfObjectBirth = targetObject.getLifecyclePeriodList().get(0).getFirstValue();
			String birthEventId = log.get(eventOrderNumOfObjectBirth-1).getAttributes().get(xConstants.ID).toString();
			String targetObjectId = targetObject.getAttributes().get(xConstants.ID).toString();
			if(!objectIdVSEventuallyReferredEventIdList.get(targetObjectId).contains(birthEventId)){
//				System.out.println("not contain eventId in discoverSourceAlwaysCardinality: " + birthEventId);
				aOCSourceAlwaysCardinality.add(0);
				break;
			}
		}
		
		
		
		//the always must contain values from 1 to the maximal number
		for(int i=1; i<=maximalNumber; i++){
			aOCSourceAlwaysCardinality.add(i);
		}	
	}
	
	//discover cardinality for AOC edges
	public void discoverAOCCardinality(XOCLog log, String activityName, String className,
			List<Integer> aOCSourceAlwaysCardinality, List<Integer> aOCSourceEventuallyCardinality, List<Integer> aOCTargetCardinality, String aocEdgeId){
//		System.out.println("activityName discover aoc: " + activityName);
//		System.out.println("className discover aoc: " + className);
		objectIdVSEventuallyReferredEventIdList.clear();	
		

		
//		long uint = 1;	
//		long startTime = System.currentTimeMillis(); 
//		long durationTime = System.currentTimeMillis();
//		
//		RunningTime.writeTimeInformationIntoFile("discoverTargetCardinality start time: " + startTime/uint, false, "D:\\timeDiscovery.txt");
		
		
		//discover target cardinality
		discoverTargetCardinality(activityName, className, aOCTargetCardinality, aocEdgeId);		
		
//		durationTime = (System.currentTimeMillis() - startTime)/uint;
//		startTime = System.currentTimeMillis();
//		RunningTime.writeTimeInformationIntoFile("discoverTargetCardinality  end time: " + System.currentTimeMillis()/uint, false, "D:\\timeDiscovery.txt");
//		RunningTime.writeTimeInformationIntoFile("~~~~~~~~~~~~~~discoverTargetCardinality duration time: " + durationTime, false, "D:\\timeDiscovery.txt");
		
		//discover source eventually cardinality
		discoverSourceEventuallyCardinality(aOCSourceEventuallyCardinality, activityName, className);
		
//		durationTime = (System.currentTimeMillis() - startTime)/uint;
//		startTime = System.currentTimeMillis();
//		RunningTime.writeTimeInformationIntoFile("discoverSourceEventuallyCardinality  end time: " + System.currentTimeMillis()/uint, false, "D:\\timeDiscovery.txt");
//		RunningTime.writeTimeInformationIntoFile("~~~~~~~~~~~~~~discoverSourceEventuallyCardinality duration time: " + durationTime, false, "D:\\timeDiscovery.txt");
		
		
		//discover source always cardinality
		discoverSourceAlwaysCardinality(aOCSourceAlwaysCardinality, className, aocEdgeId);		
		
//		durationTime = (System.currentTimeMillis() - startTime)/uint;
//		startTime = System.currentTimeMillis();
//		RunningTime.writeTimeInformationIntoFile("discoverSourceAlwaysCardinality  end time: " + System.currentTimeMillis()/uint, false, "D:\\timeDiscovery.txt");
//		RunningTime.writeTimeInformationIntoFile("~~~~~~~~~~~~~~discoverSourceAlwaysCardinality duration time: " + durationTime, false, "D:\\timeDiscovery.txt");
		
	}
	
	public Map<String, Map<String, Map<String, List<String>>>> getEventIdVSRelationIdVSObjectVSEventList(){
		return eventIdVSRelationIdVSObjectVSEventList;
	}
	
	public Map<String, Map<XOCEvent, List<XOCObject>>> getRelationIdVSEventVSObjectList(){
		return relationIdVSEventVSObjectList;
	}
	
	//standardize always cardinalities as (1) "0..*" or "1..*" if objects have different cardinality sets and its
	//eventually cardinality is "0..*" or "1..*"
	//(2) as it is if all objects have same cardinality sets
	//(3)"0..k" or "1..k" if objects have different cardinality sets, its
	//eventually cardinality is k and any cardinality in the sets is not larger than k
	public String standardizeAlwaysCardinality(List<Integer> cardinalityList, String basedEventuallyCardinality){	
		String standardizeCardinality = null;		
		boolean differentFlag = true;
//		System.out.println("basedEventuallyCardinality: " + basedEventuallyCardinality);
		for(int card : cardinalityList){
//			System.out.println("card in standardizeAlwaysCardinality: " + card);
		}
		if(cardinalityList.size() == 1){
			differentFlag = false;
//			standardizeCardinality = new Cardinality(cardinalityList).toString();
			if(cardinalityList.get(0) > 1){
				standardizeCardinality = "1..*";
			}
			else if(cardinalityList.get(0) == 1){
				standardizeCardinality = "1";
			}
			else if(cardinalityList.get(0) == 0){
				standardizeCardinality = "0";
			}
		}

		//if objects have different cardinality sets
		if(differentFlag){
			//is not a single value
			if(basedEventuallyCardinality.length()>2){
				standardizeCardinality = "1..*";
				if(cardinalityList.contains(0)){
					standardizeCardinality = "0..*";
				}
			}
			//is a single value
			else{
				if(cardinalityList.contains(0)){
					standardizeCardinality = "0.." + basedEventuallyCardinality;
				}
			}
			
		}

//		System.out.println("standardizeCardinality: " + standardizeCardinality);
		return standardizeCardinality;
	}
	
	//standardize eventually cardinalities as 
	//(1)"0..*" or "1..*" (the cardinality list does not contain 0) if objects have different eventually cardinalities
	//(2)k if all objects have cardinality "k"
	public String standardizeEventuallyCardinality(List<Integer> cardinalityList){
		String standardizeCardinality = null;
		if(cardinalityList.size() == 1) {
//			eventuallyCardinalityString = eventuallyCardinality.get(0).toString();
			if(cardinalityList.get(0) > 1){
				standardizeCardinality = "1..*";
			}
			else if(cardinalityList.get(0) == 1){
				standardizeCardinality = "1";
			}
			else if(cardinalityList.get(0) == 0){
				standardizeCardinality = "0";
			}
		}		
		else{
			if(cardinalityList.contains(0)) standardizeCardinality = "0..*";
			else standardizeCardinality = "1..*";
		}
//		System.out.println("eventuallyCardinalityString: " + eventuallyCardinalityString);
		return standardizeCardinality;
	}
	
    public static void main(String[] args) throws Exception {
  
    }
}