package org.processmining.ocbc.discovery.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.processmining.framework.util.Pair;
import org.processmining.mxgraph.util.mxConstants;
import org.processmining.ocbc.discovery.filter.FilterFunction;
import org.processmining.ocbc.discovery.panels.DiscoveryParameterPanel;
import org.processmining.ocbc.log.model.XOCEvent;
import org.processmining.ocbc.log.model.XOCLog;
import org.processmining.ocbc.log.model.XOCObject;
import org.processmining.ocbc.log.model.XOCObjectModel;
import org.processmining.ocbc.log.model.impl.xConstants;
import org.processmining.ocbc.models.AOCEdge;
import org.processmining.ocbc.models.ClassEdge;
import org.processmining.ocbc.models.OCBCModel;
import org.processmining.ocbc.models.visualization.ModelLayoutBuilder;


public class BehaviorModelDiscovery {
	private XOCLog log;
	private ModelLayoutBuilder builder;
	private Map<String, String> nodeNameVSNodeId;
	
	private FilterFunction filter = new  FilterFunction();
	private Map<String, Map<XOCEvent, List<XOCEvent>>> activityEdgeVSEventVSPrecedenceEventList = new HashMap<String, Map<XOCEvent, List<XOCEvent>>>();
	private Map<String, Map<XOCEvent, List<XOCEvent>>> activityEdgeVSEventVSResponseEventList = new HashMap<String, Map<XOCEvent, List<XOCEvent>>>();
	private Map<String, Map<String, Map<String, List<String>>>> refActivityVSTarActivityVSTypeVSConstraintList = new HashMap<String, Map<String, Map<String, List<String>>>>();
	private Map<String, Map<String, Integer>> activityEdgeVSConstraintVSInstanceNum = new HashMap<String, Map<String, Integer>> ();
	private List<String> brotherRelationIdList = new ArrayList<String>();
	private Integer idCounter;
	private Map<String, Double> activityConstraintVSConfidence = new HashMap<String, Double>();//record the confidences (fitness) of constraints
	private Map<String, Map<Long, Integer>> constraintSourceIdVSVariantVSNum = new HashMap<String, Map<Long, Integer>>();//record;

    private String approachName;
    
    /**
     * records the employed algorithm to compute metrics
     */
    private String employedFitnessAlgorithm;
    private String employedPrecisionAlgorithm;
    private String employedGeneralizationAlgorithm;
 
    /**
     * records the employed threshold to compute metrics
     */
    private double employedFitnessThreshold;
    private double employedPrecisionThreshold;
    private double employedGeneralizationThreshold;
    
    /**
     * records the employed weights to compute combined metrics
     */
    private double employedFitnessWeight;
    private double employedPrecisionWeight;
    private double employedGeneralizationWeight;
    
    private List<BehavioralConstraintPattern> constraintPatternList;	
    private OCBCModel model;
    private DiscoveryParameterPanel parameterPanel;
	public BehaviorModelDiscovery(XOCLog log, ModelLayoutBuilder builder, Map<String, String> nodeNameVSNodeId){
		this.log = log;
		this.builder = builder;
		this.model =  builder.getModel();
		this.nodeNameVSNodeId = nodeNameVSNodeId;		
	}
	
	public BehaviorModelDiscovery(XOCLog log, ModelLayoutBuilder builder, Map<String, String> nodeNameVSNodeId, String approachName){
		this.log = log;
		this.builder = builder;
		this.nodeNameVSNodeId = nodeNameVSNodeId;
		this.approachName = approachName;
		this.model = builder.getModel();
//		constraintDiscovery = new BehavioralConstraintDiscovery(log, builder, nodeNameVSNodeId, approachName, null);
	}
	
	
	public BehaviorModelDiscovery(XOCLog log, ModelLayoutBuilder builder, Map<String, String> nodeNameVSNodeId, 
			String approachName, DiscoveryParameterPanel parameterPanel){
		this.log = log;
		this.builder = builder;
		this.nodeNameVSNodeId = nodeNameVSNodeId;
		this.approachName = approachName;
		this.model = builder.getModel();
		this.parameterPanel = parameterPanel;
//		constraintDiscovery = new BehavioralConstraintDiscovery(log, builder, nodeNameVSNodeId, approachName, null);
	}
	
	public void setIdCounter(Integer idCounter){
		this.idCounter = idCounter;
	}
	
	public Integer getIdCounter(){
		return idCounter;
	}
	
	/**
	 * discover activity nodes
	 */
	public void discoverActivityNodes(){
	    //discover activity nodes
		for(String activity : log.generateActivityNameList()){
//			String activityNodeId = UUID.randomUUID().toString();
			String activityNodeId = activity + (idCounter ++).toString();			
			nodeNameVSNodeId.put(activity, activityNodeId);
			
			//draw the class node
			builder.addNode("ActivityCell", activity, activityNodeId, 
	        		Math.random()*1000, Math.random()*1000,
	        		55.0, 65.0);
		}
	}
	
	/**
	 * discover behavioral constraints
	 */
//	public void discoverActivityEdges(){
	public void discoverActivityEdges(String configuredFitnessAlgorithm, String configuredPrecisionAlgorithm, String configuredGeneralizationAlgorithm,
			 double configuredFitnessThreshold, double configuredPrecisionThreshold, double configuredGeneralizationThreshold,
			 double configuredFitnessWeight, double configuredPrecisionWeight, double configuredGeneralizationWeight){
		//discover all possible patterns
		constraintPatternList = discoverConstraintPatternList();
		
		//identify target events for each reference event corresponding to each pattern
		for(BehavioralConstraintPattern constraintPattern : constraintPatternList){
			relateEventsForConstraintPattern(constraintPattern);
		}	
		
		updateActivityEdges(configuredFitnessAlgorithm, configuredPrecisionAlgorithm, configuredGeneralizationAlgorithm, 
				configuredFitnessThreshold, configuredPrecisionThreshold, configuredGeneralizationThreshold, 
				configuredFitnessWeight, configuredPrecisionWeight, configuredGeneralizationWeight);
	}
	
    /**
     * re-discover behavioral constraints based on configured algorithms, thresholds and weights;
     * @param employedFitnessAlgorithm
     * @param employedPrecisionAlgorithm
     * @param employedGeneralizationAlgorithm
     * @param employedFitnessThreshold
     * @param employedPrecisionThreshold
     * @param employedGeneralizationThreshold
     * @param employedFitnessWeight
     * @param employedPrecisionWeight
     * @param employedGeneralizationWeight
     */
	public OCBCModel updateActivityEdges(String configuredFitnessAlgorithm, String configuredPrecisionAlgorithm, String configuredGeneralizationAlgorithm,
			 double configuredFitnessThreshold, double configuredPrecisionThreshold, double configuredGeneralizationThreshold,
			 double configuredFitnessWeight, double configuredPrecisionWeight, double configuredGeneralizationWeight){
		model.getAllActivityRelations().clear();
		model.getAllCrelOCRelations().clear();
		model.getAllCrelRTRelations().clear();
		
//		System.out.println();
//		System.out.println("log size in discover activity edges: " + log.getAllEvents().size());
		boolean fitnessUpdateFlag = false;
		boolean precisionUpdateFlag = false;
		boolean generalizationUpdateFlag = false;
		
		if(employedFitnessAlgorithm != configuredFitnessAlgorithm || employedFitnessThreshold != configuredFitnessThreshold){
			employedFitnessAlgorithm = configuredFitnessAlgorithm;
			employedFitnessThreshold = configuredFitnessThreshold;
			fitnessUpdateFlag = true;
		}
		
		if(employedPrecisionAlgorithm != configuredPrecisionAlgorithm || employedPrecisionThreshold != configuredPrecisionThreshold){
			employedPrecisionAlgorithm = configuredPrecisionAlgorithm;
			employedPrecisionThreshold = configuredPrecisionThreshold;
			precisionUpdateFlag = true;
		}
		
		if(employedGeneralizationAlgorithm != configuredGeneralizationAlgorithm || employedGeneralizationThreshold != configuredGeneralizationThreshold){
			employedGeneralizationAlgorithm = configuredGeneralizationAlgorithm;
			employedGeneralizationThreshold = configuredGeneralizationThreshold;
			generalizationUpdateFlag = true;
		}
		
		for(BehavioralConstraintPattern constraintPattern : constraintPatternList){	
			System.out.println("*********pattern start**************");
			constraintPattern.computeMetricsOfAllSolutionsForPattern(configuredFitnessAlgorithm, configuredPrecisionAlgorithm, 
					configuredGeneralizationAlgorithm, configuredFitnessThreshold, configuredPrecisionThreshold, configuredGeneralizationThreshold, 
					fitnessUpdateFlag, precisionUpdateFlag, generalizationUpdateFlag);
			
			Long bestSolution = constraintPattern.searchBestSolutionBasedOnWeights(configuredFitnessWeight, configuredPrecisionWeight, configuredGeneralizationWeight);
			List<String> constraintTypeList = constraintPattern.transformSolutionIntoConstraintTypeList(bestSolution);
			for(String constraintType : constraintTypeList){
				String sourceActivityNodeId = constraintPattern.getReferenceActivityNodeId();
				String intermediaryId = constraintPattern.getIntermediaryId();
				String targetActivityNodeId = constraintPattern.getTargetActivityNodeId();
				String intermediaryType = constraintPattern.getPatternType();
				
				
//				System.out.println("sourceActivityNodeId: " + sourceActivityNodeId);
//				System.out.println("targetActivityNodeId: " + targetActivityNodeId);
//				System.out.println("intermediaryId: " + intermediaryId);
//				System.out.println("intermediaryType: " + intermediaryType);				
				
				String activityEdgeId = sourceActivityNodeId + "-" + constraintType + "-" + targetActivityNodeId + (idCounter ++).toString();	
			
						
				if(intermediaryType.equals(mxConstants.CRELOCRELATION)){
					String crelOCEdgeId = activityEdgeId + "--" + intermediaryId; 					
					builder.addActivityEdge(activityEdgeId, sourceActivityNodeId,
							targetActivityNodeId, constraintType, 0, 0);					
					builder.addCrelOCEdge(crelOCEdgeId, activityEdgeId, intermediaryId, 0, 0);
				}
				else if(intermediaryType.equals(mxConstants.CRELRTRELATION)){
					String crelRTEdgeId = activityEdgeId + "--" + intermediaryId; 				
					builder.addActivityEdge(activityEdgeId, sourceActivityNodeId,
							targetActivityNodeId, constraintType, 0, 0);					
					builder.addCrelRTEdge(crelRTEdgeId, activityEdgeId, intermediaryId, 0, 0);					
				}	
				
				System.out.println("*********pattern end**************");
			}
		}			
		return model;
	}
	
	/**
	 * discover and created all constraint patterns;
	 * in other words, after this function, each pattern has only basic elements, i.e., type, reference/target activity name/node id and
	 * source/target class name/node id as well as class relation;
	 * @return
	 */
	public List<BehavioralConstraintPattern> discoverConstraintPatternList(){
		List<BehavioralConstraintPattern> constraintPatternList = new ArrayList<BehavioralConstraintPattern>();
		//discover activity edges-get the source (reference) activity
		for(AOCEdge sourceAOCEdge  : model.getAllAOCRelations()){
			String sourceClassNodeId = sourceAOCEdge.getClassNode().getID().toString();	
			String sourceClassName = sourceAOCEdge.getClassNode().getNodeName();
			String referenceActivityName = sourceAOCEdge.getActivityNode().getNodeName();
			String referenceActivityNodeId = sourceAOCEdge.getActivityNode().getID().toString();			
//			System.out.printf("AOC relation className" + sourceClassNodeId);			
			
			//get the target activity
			for(AOCEdge targetAOCEdge  : model.getAllAOCRelations()){
				String targetClassNodeId = targetAOCEdge.getClassNode().getID().toString();
				String targetClassName = targetAOCEdge.getClassNode().getNodeName();
				String targetActivityName = targetAOCEdge.getActivityNode().getNodeName();
				String targetActivityNodeId = targetAOCEdge.getActivityNode().getID().toString();
				
				//source AOC relation and AOC relation are not the same AOC relation
				if(!referenceActivityNodeId.equals(targetActivityNodeId)){				
					ClassEdge classRelation = model.getClassRelationByTwoClassNodes(sourceClassNodeId, targetClassNodeId);				
					//and they contains the same class or two related classes
					if(sourceClassNodeId.equals(targetClassNodeId) || classRelation != null){		
						String constraintPatternType = null;		
						if(sourceClassNodeId.equals(targetClassNodeId)){
	
							constraintPatternType = mxConstants.CRELOCRELATION;
						}
						else if(classRelation != null){
					
							constraintPatternType = mxConstants.CRELRTRELATION;
						}
						
						BehavioralConstraintPattern constraintPattern = new BehavioralConstraintPattern(constraintPatternType, 
								referenceActivityName, referenceActivityNodeId, targetActivityName, targetActivityNodeId, 
								sourceClassName, sourceClassNodeId, targetClassName, targetClassNodeId, classRelation);
						constraintPatternList.add(constraintPattern);
										
					}
				}
			}			
		}	
		
		//filter out the constraint redundant RT patterns (there exist a OC pattern with the same 
		//reference activity and target activity)
		List<BehavioralConstraintPattern> constraintRTPatternList = new ArrayList<BehavioralConstraintPattern>();
		List<BehavioralConstraintPattern> redundantConstraintRTPatternList = new ArrayList<BehavioralConstraintPattern>();
		for(BehavioralConstraintPattern pattern : constraintPatternList){
			String constraintPatternType = pattern.getPatternType();
			if(constraintPatternType.equals(mxConstants.CRELRTRELATION)){
				constraintRTPatternList.add(pattern);
			}
		}
		for(BehavioralConstraintPattern patternRT : constraintRTPatternList){
			String referenceActivityNameRT = patternRT.getReferenceActivityName();
			String targetActivityNameRT = patternRT.getTargetActivityName();
			for(BehavioralConstraintPattern pattern : constraintPatternList){
				String constraintPatternType = pattern.getPatternType();
				String referenceActivityName = pattern.getReferenceActivityName();
				String targetActivityName = pattern.getTargetActivityName();
				if(referenceActivityNameRT.equals(referenceActivityName) &&
						targetActivityNameRT.equals(targetActivityName) &&
						constraintPatternType.equals(mxConstants.CRELOCRELATION)){
					redundantConstraintRTPatternList.add(patternRT);
				}
				
			}
		}
		constraintPatternList.removeAll(redundantConstraintRTPatternList);
		
		return constraintPatternList;
	}

	/**
	 * get all target events of each reference event for an input constraint pattern;
	 * in other words, after this function, each pattern has precedenceAndResponsePairList,
	 * eventVSPrecedenceEventList and eventVSResponseEventList;
	 * @param constraintPattern
	 */
	public void relateEventsForConstraintPattern(BehavioralConstraintPattern constraintPattern){
		List<Pair<Integer, Integer>> precedenceAndResponsePairList = constraintPattern.getPrecedenceAndResponsePairList();
		Map<XOCEvent, List<XOCEvent>> eventVSPrecedenceEventList = constraintPattern.getReferenceEventVSPrecedenceTargetEventListMap();
		Map<XOCEvent, List<XOCEvent>> eventVSResponseEventList = constraintPattern.getReferenceEventVSResposeTargetEventListMap(); 
		String sourceActivityName = constraintPattern.getReferenceActivityName();
		String sourceActivityNodeId = constraintPattern.getReferenceActivityNodeId();
		String targetActivityName = constraintPattern.getTargetActivityName();
		String targetActivityNodeId = constraintPattern.getTargetActivityNodeId();
		String patternType = constraintPattern.getPatternType();
		
		String sourceClassName = constraintPattern.getSourceClassName();
		String sourceClassNodeId = constraintPattern.getSourceClassNodeId();
		String targetClassName = constraintPattern.getTargetClassName();
		String targetClassNodeId = constraintPattern.getTargetClassNodeId();
		
		if(patternType.equals(mxConstants.CRELOCRELATION)){
			identifyTargetEventsByClass(sourceActivityName, sourceActivityNodeId, targetActivityName, 
					targetActivityNodeId, sourceClassName, sourceClassNodeId, precedenceAndResponsePairList,
					eventVSPrecedenceEventList,eventVSResponseEventList);
		}
		else if(patternType.equals(mxConstants.CRELRTRELATION)){
			identifyTargetEventsByClassRelation(sourceActivityName, sourceActivityNodeId, targetActivityName, 
					targetActivityNodeId, sourceClassName, sourceClassNodeId, targetClassName, targetClassNodeId, precedenceAndResponsePairList,
					eventVSPrecedenceEventList,eventVSResponseEventList);
		}
	}
	
	/**
	 * Through the a common class, relate target events to a reference event;
	 * @param sourceActivityName
	 * @param sourceActivityNodeId
	 * @param targetActivityName
	 * @param targetActivityNodeId
	 * @param className
	 * @param classNodeId
	 */
	public void identifyTargetEventsByClass(String sourceActivityName, String sourceActivityNodeId, String targetActivityName, 
			String targetActivityNodeId, String className, String classNodeId, List<Pair<Integer, Integer>> precedenceAndResponsePairList,
			Map<XOCEvent, List<XOCEvent>> eventVSPrecedenceEventList, Map<XOCEvent, List<XOCEvent>> eventVSResponseEventList){
//		long uint = 1;	
//		long startTime = System.currentTimeMillis(); 
//		long durationTime = System.currentTimeMillis();
		
//		RunningTime.writeTimeInformationIntoFile("specify the number for variants start time: " + startTime/uint, false, "D:\\timeDiscovery.txt");
		
		for(XOCEvent sourceEvent : filter.filterEvents(log.getAllEvents(), null, sourceActivityName, 5)){ 
			String sourceEventId = sourceEvent.getAttributes().get(xConstants.ID).toString();
//			System.out.println("*sourceEvent: " + sourceEvent.getAttributes().get(xConstants.ID).toString());
			List<XOCEvent> precedenceEventList = new ArrayList<XOCEvent>();
			List<XOCEvent> responseEventList = new ArrayList<XOCEvent>();
			for(XOCObject relatedObject : filter.filterObjectsByClassAtEvent(sourceEvent.getRelatedObjects(), className, log.getLastEvent())){
//				System.out.println("**sourceObject: " + sourceObject.getAttributes().get(xConstants.ID).toString());
//				String objectId = sourceObject.getAttributes().get("id").toString();
				for(XOCEvent targetEvent : filter.filterEvents(log.getAllEvents(), null, targetActivityName, 5)){ 
					String targetEventId = targetEvent.getAttributes().get(xConstants.ID).toString();
//					System.out.println("***targetEvent: " + targetEvent.getAttributes().get(xConstants.ID).toString());
					if(targetEvent.getRelatedObjects().contains(relatedObject)){
//							System.out.println("*****equal");
						if(sourceEvent.getOrderNum() > targetEvent.getOrderNum()){
							if(!precedenceEventList.contains(targetEvent)){
								precedenceEventList.add(targetEvent);
							}											
						}											
						else{
							if(!responseEventList.contains(targetEvent)){
								responseEventList.add(targetEvent);		
							}	
						}																			
					}							
				}
			}	
			
//			System.out.println("acticvityCardinalityBefore: " + precedenceEventIdList.size());
//			System.out.println("acticvityCardinalityAfter: " + responseEventIdList.size());
			precedenceAndResponsePairList.add(new Pair<Integer, Integer>(precedenceEventList.size(), responseEventList.size()));
			eventVSPrecedenceEventList.put(sourceEvent, precedenceEventList);
			eventVSResponseEventList.put(sourceEvent, responseEventList);
		}					
		
//		durationTime = (System.currentTimeMillis() - startTime)/uint;
//		startTime = System.currentTimeMillis();
//		RunningTime.writeTimeInformationIntoFile("specify the number for variants  end time: " + System.currentTimeMillis()/uint, false, "D:\\timeDiscovery.txt");
//		RunningTime.writeTimeInformationIntoFile("~~~~~~~~~~~~~~specify the number for variants duration time: " + durationTime, false, "D:\\timeDiscovery.txt");
//		RunningTime.writeTimeInformationIntoFile("search best solution start time: " + System.currentTimeMillis()/uint, false, "D:\\timeDiscovery.txt");
		
//		discoverConstraintTypesFromRelatedEvents(sourceActivityName, sourceActivityNodeId, targetActivityName, targetActivityNodeId,   
//				precedenceAndResponsePairList, mxConstants.CRELRTRELATION, classNodeId, eventVSPrecedenceEventList, eventVSResponseEventList);

//		durationTime = (System.currentTimeMillis() - startTime)/uint;
//		startTime = System.currentTimeMillis();
//		RunningTime.writeTimeInformationIntoFile("search best solution  end time: " + System.currentTimeMillis()/uint, false, "D:\\timeDiscovery.txt");
//		RunningTime.writeTimeInformationIntoFile("~~~~~~~~~~~~~~search best solution duration time: " + durationTime, false, "D:\\timeDiscovery.txt");
	}
	
	/**
	 * Through the two related classes, relate target events to a reference event;
	 * @param sourceActivityName
	 * @param sourceActivityNodeId
	 * @param targetActivityName
	 * @param targetActivityNodeId
	 * @param sourceClassName
	 * @param sourceClassNodeId
	 * @param targetClassName
	 * @param targetClassNodeId
	 * @param precedenceAndResponsePairList
	 * @param eventVSPrecedenceEventList
	 * @param eventVSResponseEventList
	 */
	public void identifyTargetEventsByClassRelation(String sourceActivityName, String sourceActivityNodeId, String targetActivityName, 
			String targetActivityNodeId, String sourceClassName, String sourceClassNodeId, String targetClassName, String targetClassNodeId,
			List<Pair<Integer, Integer>> precedenceAndResponsePairList,
			Map<XOCEvent, List<XOCEvent>> eventVSPrecedenceEventList, Map<XOCEvent, List<XOCEvent>> eventVSResponseEventList){
		OCBCModel model = builder.getModel();
		String classRelationId = model.checkTwoClassesIfRelated(sourceClassNodeId, targetClassNodeId);
		
//		System.out.println("sourceActivityName: " + sourceActivityName);
//		System.out.println("targetActivityName: " + targetActivityName);
		
//		long uint = 1;	
//		long startTime = System.currentTimeMillis(); 
//		long durationTime = System.currentTimeMillis();
//		
//		RunningTime.writeTimeInformationIntoFile("specify the number for variants start time: " + startTime/uint, false, "D:\\timeDiscovery.txt");		
		
			
		XOCObjectModel lastObjectModel = log.getLastEvent().getObjectModel();
		Map<String, XOCObject> idVSObjectMap = lastObjectModel.getIdVSObjectMap();
		
		//for every source (reference) event
		for(XOCEvent sourceEvent : filter.filterEvents(log.getAllEvents(), null, sourceActivityName, 5)){   
			
			String sourceEventId = sourceEvent.getAttributes().get(xConstants.ID).toString();
			List<XOCEvent> precedenceEventList = new ArrayList<XOCEvent>();
			List<XOCEvent> responseEventList = new ArrayList<XOCEvent>();
//			System.out.println("sourceEventId: " + sourceEventId);		
			
			for(XOCObject sourceObject : filter.filterObjects(sourceEvent.getRelatedObjects(), sourceClassName)){
//				String sourceObjectId = sourceObject.getAttributes().get(xConstants.ID).toString();
//				XOCObject sourceObjectOfLastObjectModel = idVSObjectMap.get(sourceObjectId);			
//				System.out.println("sourceObjectId: " + sourceObjectId);
				for(String relatedObjectIdOfSourceObject : sourceObject.getRelatedObjectIdList()){ 
					XOCObject relatedObjectOfSourceObject = idVSObjectMap.get(relatedObjectIdOfSourceObject);
//					String objectId = relatedObjectOfSourceObject.getAttributes().get(xConstants.ID).toString();
//					System.out.println("target candidate objectId: " + objectId);
					String objectClass = relatedObjectOfSourceObject.getAttributes().get(xConstants.CLASS).toString();
//					System.out.println("target candidate objectClass: " + objectClass);
					if(objectClass.equals(targetClassName)){
//						System.out.println("is target");
//						System.out.println("getRelatedEventList().size: " + relatedObjectOfSourceObject.getRelatedEventList().size());
						for(XOCEvent targetEvent : filter.filterEvents(relatedObjectOfSourceObject.getRelatedEventList(), null, targetActivityName, 5)){
//							System.out.println("getRelatedEventList");
							String targetEventId = targetEvent.getAttributes().get(xConstants.ID).toString();
							if(sourceEvent.getOrderNum() > targetEvent.getOrderNum()){
								if(!precedenceEventList.contains(targetEvent)){
									precedenceEventList.add(targetEvent);
								}											
							}																							
							else{
								if(!responseEventList.contains(targetEvent)){
									responseEventList.add(targetEvent);		
								}	
							}	
						}
					}					
				}
			}	
			
			precedenceAndResponsePairList.add(new Pair<Integer, Integer>(precedenceEventList.size(), responseEventList.size()));
			eventVSPrecedenceEventList.put(sourceEvent, precedenceEventList);
			eventVSResponseEventList.put(sourceEvent, responseEventList);
		}
		
//		durationTime = (System.currentTimeMillis() - startTime)/uint;
//		startTime = System.currentTimeMillis();
//		RunningTime.writeTimeInformationIntoFile("specify the number for variants  end time: " + System.currentTimeMillis()/uint, false, "D:\\timeDiscovery.txt");
//		RunningTime.writeTimeInformationIntoFile("~~~~~~~~~~~~~~specify the number for variants duration time: " + durationTime, false, "D:\\timeDiscovery.txt");
//		RunningTime.writeTimeInformationIntoFile("search best solution start time: " + System.currentTimeMillis()/uint, false, "D:\\timeDiscovery.txt");	
	}	
	
	public boolean addActivityConstraint(String referenceActivity, String targetActivity, String activityType, String intermediateId){
		boolean flag = false;
		if(!refActivityVSTarActivityVSTypeVSConstraintList.containsKey(referenceActivity)){
			refActivityVSTarActivityVSTypeVSConstraintList.put(referenceActivity, new HashMap<String, Map<String, List<String>>>());
		}
		if(!refActivityVSTarActivityVSTypeVSConstraintList.get(referenceActivity).containsKey(targetActivity)){
			refActivityVSTarActivityVSTypeVSConstraintList.get(referenceActivity).put(targetActivity, new HashMap<String, List<String>>());
		}
		if(!refActivityVSTarActivityVSTypeVSConstraintList.get(referenceActivity).get(targetActivity).containsKey(activityType)){
			refActivityVSTarActivityVSTypeVSConstraintList.get(referenceActivity).get(targetActivity).put(activityType,  new ArrayList<String>());
		}
		if(!refActivityVSTarActivityVSTypeVSConstraintList.get(referenceActivity).get(targetActivity).get(activityType).contains(intermediateId)){
			refActivityVSTarActivityVSTypeVSConstraintList.get(referenceActivity).get(targetActivity).get(activityType).add(intermediateId);
			flag = true;
		}
		return flag;
	}
	
	public Map<String, Map<XOCEvent, List<XOCEvent>>> getActivityEdgeVSEventVSPrecedenceEventList(){
		return activityEdgeVSEventVSPrecedenceEventList;
	}
	
	public Map<String, Map<XOCEvent, List<XOCEvent>>> getActivityEdgeVSEventVSResponseEventList(){
		return activityEdgeVSEventVSResponseEventList;
	}
	
	public Map<String, Map<String, Map<String, List<String>>>> getRefActivityVSTarActivityVSTypeVSConstraintList(){
		return refActivityVSTarActivityVSTypeVSConstraintList;
	}
	 
	public Map<String, Map<String, Integer>> getActivityEdgeVSConstraintVSInstanceNumMap(){
		return activityEdgeVSConstraintVSInstanceNum;
	}

	public List<String> getBrotherRelationIdList(){
		return brotherRelationIdList;
	}
	
	public Map<String, Double> getBehavioralConstraintVSConfidence(){
		return activityConstraintVSConfidence;
	}

	
    public static void main(String[] args) throws Exception {

    }
}