package org.processmining.ocbc.discovery.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.processmining.framework.util.Pair;
import org.processmining.ocbc.discovery.filter.FilterFunction;
import org.processmining.ocbc.log.model.XOCEvent;
import org.processmining.ocbc.log.model.XOCLog;
import org.processmining.ocbc.log.model.XOCObject;
import org.processmining.ocbc.log.model.XOCObjectModel;
import org.processmining.ocbc.log.model.impl.xConstants;
import org.processmining.ocbc.models.AOCEdge;
import org.processmining.ocbc.models.OCBCModel;
import org.processmining.ocbc.models.visualization.ModelLayoutBuilder;


public class BehaviorModelDiscoveryFitness {
	private XOCLog log;
	private ModelLayoutBuilder builder;
	private Map<String, String> nodeNameVSNodeId;
	
	private FilterFunction filter = new  FilterFunction();
	private Map<String, Map<String, List<String>>> activityEdgeVSEventVSPrecedenceEventList = new HashMap<String, Map<String, List<String>>>();
	private Map<String, Map<String, List<String>>> activityEdgeVSEventVSResponseEventList = new HashMap<String, Map<String, List<String>>>();
	private Map<String, Map<String, Map<String, List<String>>>> refActivityVSTarActivityVSTypeVSConstraintList = new HashMap<String, Map<String, Map<String, List<String>>>>();
	private Map<String, Map<String, Integer>> activityEdgeVSConstraintVSInstanceNum = new HashMap<String, Map<String, Integer>> ();
	private List<String> brotherRelationIdList = new ArrayList<String>();
	private Integer idCounter;
	private Map<String, Double> activityConstraintVSConfidence = new HashMap<String, Double>();//record the confidences (fitness) of constraints
	
	public BehaviorModelDiscoveryFitness(XOCLog log, ModelLayoutBuilder builder, Map<String, String> nodeNameVSNodeId){
		this.log = log;
		this.builder = builder;
		this.nodeNameVSNodeId = nodeNameVSNodeId;
	}
	
	public void setIdCounter(Integer idCounter){
		this.idCounter = idCounter;
	}
	
	public Integer getIdCounter(){
		return idCounter;
	}
	
	public void discoverActivityNodes(){
	    //discover activity nodes
		for(String activity : log.createActivityNames()){
//			String activityNodeId = UUID.randomUUID().toString();
			String activityNodeId = activity + (idCounter ++).toString();			
			nodeNameVSNodeId.put(activity, activityNodeId);
			
			//draw the class node
			builder.addNode("ActivityCell", activity, activityNodeId, 
	        		Math.random()*1000, Math.random()*1000,
	        		55.0, 65.0);
		}
	}
	
	public void discoverActivityEdges(){
		OCBCModel model = builder.getModel();
		System.out.println();
		System.out.println("log size in discover activity edges: " + log.getAllEvents().size());

		//discover activity edges-get the source (reference) activity
		for(AOCEdge sourceAOCEdge  : model.getAllAOCRelations()){
			String sourceClassNodeId = sourceAOCEdge.getClassNode().getID().toString();	
			String sourceClassName = sourceAOCEdge.getClassNode().getNodeName();
			String sourceActivityName = sourceAOCEdge.getActivityNode().getNodeName();
			String sourceActivityNodeId = sourceAOCEdge.getActivityNode().getID().toString();			
			System.out.printf("AOC relation className" + sourceClassNodeId);			
			
			//get the target activity
			for(AOCEdge targetAOCEdge  : model.getAllAOCRelations()){
				String targetClassNodeId = targetAOCEdge.getClassNode().getID().toString();
				String targetClassName = targetAOCEdge.getClassNode().getNodeName();
				String targetActivityName = targetAOCEdge.getActivityNode().getNodeName();
				String targetActivityNodeId = targetAOCEdge.getActivityNode().getID().toString();
				
				System.out.println("");
				System.out.println("sourceActivityName: " + sourceActivityName);
				System.out.println("targetActivityName: " + targetActivityName);
				
				//source AOC relation and AOC relation are not the same AOC relation
				if(!sourceActivityNodeId.equals(targetActivityNodeId)){
					Map<String, List<String>> eventVSPrecedenceEventList = new HashMap<String, List<String>>();
					Map<String, List<String>> eventVSResponseEventList = new HashMap<String, List<String>>(); 
					//and they contains the same class
					if(sourceClassNodeId.equals(targetClassNodeId)){
						System.out.println("AOC relation sourceClassName: " + sourceClassName);
						System.out.println("AOC relation targetClassName: " + targetClassName);
						
						List<Pair<Integer, Integer>> acticvityCardinalityPairList = new ArrayList<Pair<Integer, Integer>>();
															
						for(XOCEvent sourceEvent : filter.filterEvents(log.getAllEvents(), null, sourceActivityName, 5)){ 
							String sourceEventId = sourceEvent.getAttributes().get(xConstants.ID).toString();
							System.out.println("*sourceEvent: " + sourceEvent.getAttributes().get(xConstants.ID).toString());
							List<String> precedenceEventIdList = new ArrayList<String>();
							List<String> responseEventIdList = new ArrayList<String>();
							for(XOCObject sourceObject : filter.filterObjects(sourceEvent.getRelatedObjects(), sourceClassName)){
								System.out.println("**sourceObject: " + sourceObject.getAttributes().get(xConstants.ID).toString());
								String objectId = sourceObject.getAttributes().get("id").toString();
								for(XOCEvent targetEvent : filter.filterEvents(log.getAllEvents(), null, targetActivityName, 5)){ 
									String targetEventId = targetEvent.getAttributes().get(xConstants.ID).toString();
									System.out.println("***targetEvent: " + targetEvent.getAttributes().get(xConstants.ID).toString());
									for(XOCObject targetObject : filter.filterObjects(targetEvent.getRelatedObjects(), sourceClassName)){		
										System.out.println("****targetObject: " + targetObject.getAttributes().get(xConstants.ID).toString());
										if(objectId.equals(targetObject.getAttributes().get("id").toString())){
											System.out.println("*****equal");
											if(log.getAllEvents().indexOf(sourceEvent) > log.getAllEvents().indexOf(targetEvent)){
												if(!precedenceEventIdList.contains(targetEventId)){
													precedenceEventIdList.add(targetEventId);
												}											
											}											
											else{
												if(!responseEventIdList.contains(targetEventId)){
													responseEventIdList.add(targetEventId);		
												}	
											}																			
										}
									}								
								}
							}	
							
							System.out.println("acticvityCardinalityBefore: " + precedenceEventIdList.size());
							System.out.println("acticvityCardinalityAfter: " + responseEventIdList.size());
							acticvityCardinalityPairList.add(new Pair<Integer, Integer>(precedenceEventIdList.size(), responseEventIdList.size()));
							eventVSPrecedenceEventList.put(sourceEventId, precedenceEventIdList);
							eventVSResponseEventList.put(sourceEventId, responseEventIdList);
						}					
							
//						for(String activityType : model.transformCardinalityPairListToActivityTypeList(acticvityCardinalityPairList)){
						for(String activityType : model.transformCardinalityPairListToActivityTypeVSConfidence(acticvityCardinalityPairList).keySet()){
//							String activityType = model.transformCardinalityPairListToActicityType(acticvityCardinalityPairList);
							boolean flag = addActivityConstraint(sourceActivityName, targetActivityName, activityType, sourceClassNodeId);
							if(flag){
								Map<String, Integer> constraintVSInstanceNum = model.transformCardinalityPairListToConstraintVSNumMap(acticvityCardinalityPairList);
								String activityEdgeId = sourceActivityName + "-" + activityType + "-" + targetActivityName + (idCounter ++).toString();	
								if(!activityEdgeVSEventVSPrecedenceEventList.containsKey(activityEdgeId)){
									activityEdgeVSEventVSPrecedenceEventList.put(activityEdgeId, eventVSPrecedenceEventList);
									activityEdgeVSEventVSResponseEventList.put(activityEdgeId, eventVSResponseEventList);												
									//add the map between constraint type and its number of instances
									activityEdgeVSConstraintVSInstanceNum.put(activityEdgeId, constraintVSInstanceNum);
									
									System.out.println("activityType: " + activityType);
									if(activityType != null){
										model.addSequentialActivityPair(sourceActivityNodeId, targetActivityNodeId, activityType);					
										String crelOCEdgeId = activityEdgeId + "--" + sourceClassNodeId; 
										
										builder.addActivityEdge(activityEdgeId, sourceActivityNodeId,
												targetActivityNodeId, activityType, 0, 0);					
										builder.addCrelOCEdge(crelOCEdgeId, activityEdgeId, sourceClassNodeId, 0, 0);	
										double confidence = model.transformCardinalityPairListToActivityTypeVSConfidence(acticvityCardinalityPairList).get(activityType);
										activityConstraintVSConfidence.put(activityEdgeId, confidence);
									}	
									if(refActivityVSTarActivityVSTypeVSConstraintList.get(sourceActivityName).get(targetActivityName).get(activityType).size()>1){
										brotherRelationIdList.add(activityEdgeId);
									}
								}
							}

						}						
					}
					
					//and they contains two related classes
					else if(model.checkTwoClassesIfRelated(sourceClassNodeId, targetClassNodeId) != null){	
//					if(false){	
						System.out.printf("\n enter checkTwoClassesIfRelated");
						String classRelationId = model.checkTwoClassesIfRelated(sourceClassNodeId, targetClassNodeId);
						
						List<Pair<Integer, Integer>> acticvityCardinalityPairList = new ArrayList<Pair<Integer, Integer>>();
						//for every source (reference) event
						for(XOCEvent sourceEvent : filter.filterEvents(log.getAllEvents(), null, sourceActivityName, 5)){   
							String sourceEventId = sourceEvent.getAttributes().get(xConstants.ID).toString();
							List<String> precedenceEventIdList = new ArrayList<String>();
							List<String> responseEventIdList = new ArrayList<String>();
							for(XOCObject sourceObject : filter.filterObjects(sourceEvent.getRelatedObjects(), sourceClassName)){
								String sourceObjectId = sourceObject.getAttributes().get("id").toString();
								for(XOCEvent targetEvent : filter.filterEvents(log.getAllEvents(), null, targetActivityName, 5)){ 
									String targetEventId = targetEvent.getAttributes().get(xConstants.ID).toString();
									System.out.printf("\n enter  targetEvent : filterEvents");
									for(XOCObject targetObject : filter.filterObjects(targetEvent.getRelatedObjects(), targetClassName)){	
										String targetObjectId = targetObject.getAttributes().get("id").toString();								
										//get the object model of the last event
										XOCObjectModel objectModel = log.getAllEvents().get(log.getAllEvents().size()-1).getObjectModel();
										if(objectModel.checkTwoObjectsIfRelated(sourceObjectId, targetObjectId)){
											if(log.getAllEvents().indexOf(sourceEvent) > log.getAllEvents().indexOf(targetEvent)){
												if(!precedenceEventIdList.contains(targetEventId)){
													precedenceEventIdList.add(targetEventId);
												}											
											}																							
											else{
												if(!responseEventIdList.contains(targetEventId)){
													responseEventIdList.add(targetEventId);		
												}	
											}									
										}
									}
								}
							}	
							acticvityCardinalityPairList.add(new Pair<Integer, Integer>(precedenceEventIdList.size(), responseEventIdList.size()));
							eventVSPrecedenceEventList.put(sourceEventId, precedenceEventIdList);
							eventVSResponseEventList.put(sourceEventId, responseEventIdList);
						}
						
						
//						for(String activityType : model.transformCardinalityPairListToActivityTypeList(acticvityCardinalityPairList)){
						for(String activityType : model.transformCardinalityPairListToActivityTypeVSConfidence(acticvityCardinalityPairList).keySet()){
//						for(String activityType : model.transformCardinalityPairListToConstraintVSNumMap(acticvityCardinalityPairList).keySet()){
//							String activityType = model.transformCardinalityPairListToActicityType(acticvityCardinalityPairList);							
							boolean flag = addActivityConstraint(sourceActivityName, targetActivityName, activityType, classRelationId);
							if(flag){
								Map<String, Integer> constraintVSInstanceNum = model.transformCardinalityPairListToConstraintVSNumMap(acticvityCardinalityPairList);
								String activityEdgeId = sourceActivityName + "-" + activityType + "-" + targetActivityName + (idCounter ++).toString();	
								if(!activityEdgeVSEventVSPrecedenceEventList.containsKey(activityEdgeId)){
									activityEdgeVSEventVSPrecedenceEventList.put(activityEdgeId, eventVSPrecedenceEventList);
									activityEdgeVSEventVSResponseEventList.put(activityEdgeId, eventVSResponseEventList);	
									//add the map between constraint type and its number of instances
									activityEdgeVSConstraintVSInstanceNum.put(activityEdgeId, constraintVSInstanceNum);
									System.out.println("\n activityType in checkTwoClassesIfRelated_yes: " + activityType);
									System.out.println("\n id in checkTwoClassesIfRelated_yes: " + activityEdgeId);
									if(activityType != null){
										model.addSequentialActivityPair(sourceActivityNodeId, targetActivityNodeId, activityType);					
										model.addSequentialActivityPair(sourceActivityNodeId, targetActivityNodeId, activityType);					
										String crelOCEdgeId = activityEdgeId + "--" + classRelationId; 
										
										builder.addActivityEdge(activityEdgeId, sourceActivityNodeId,
												targetActivityNodeId, activityType, 0, 0);					
										builder.addCrelRTEdge(crelOCEdgeId, activityEdgeId, classRelationId, 0, 0);
										
										double confidence = model.transformCardinalityPairListToActivityTypeVSConfidence(acticvityCardinalityPairList).get(activityType);
										activityConstraintVSConfidence.put(activityEdgeId, confidence);
									}	
									if(refActivityVSTarActivityVSTypeVSConstraintList.get(sourceActivityName).get(targetActivityName).get(activityType).size()>1){
										brotherRelationIdList.add(activityEdgeId);
									}
								}			
							}							
						}																				
					}	
				}			
			}			
		}			
	}

	public boolean addActivityConstraint(String referenceActivity, String targetActivity, String activityType, String intermediateId){
		boolean flag = false;
		if(!refActivityVSTarActivityVSTypeVSConstraintList.containsKey(referenceActivity)){
			refActivityVSTarActivityVSTypeVSConstraintList.put(referenceActivity, new HashMap<String, Map<String, List<String>>>());
		}
		if(!refActivityVSTarActivityVSTypeVSConstraintList.get(referenceActivity).containsKey(targetActivity)){
			refActivityVSTarActivityVSTypeVSConstraintList.get(referenceActivity).put(targetActivity, new HashMap<String, List<String>>());
		}
		if(!refActivityVSTarActivityVSTypeVSConstraintList.get(referenceActivity).get(targetActivity).containsKey(activityType)){
			refActivityVSTarActivityVSTypeVSConstraintList.get(referenceActivity).get(targetActivity).put(activityType,  new ArrayList<String>());
		}
		if(!refActivityVSTarActivityVSTypeVSConstraintList.get(referenceActivity).get(targetActivity).get(activityType).contains(intermediateId)){
			refActivityVSTarActivityVSTypeVSConstraintList.get(referenceActivity).get(targetActivity).get(activityType).add(intermediateId);
			flag = true;
		}
		return flag;
	}
	
	public Map<String, Map<String, List<String>>> getActivityEdgeVSEventVSPrecedenceEventList(){
		return activityEdgeVSEventVSPrecedenceEventList;
	}
	
	public Map<String, Map<String, List<String>>> getActivityEdgeVSEventVSResponseEventList(){
		return activityEdgeVSEventVSResponseEventList;
	}
	
	public Map<String, Map<String, Map<String, List<String>>>> getRefActivityVSTarActivityVSTypeVSConstraintList(){
		return refActivityVSTarActivityVSTypeVSConstraintList;
	}
	 
	public Map<String, Map<String, Integer>> getActivityEdgeVSConstraintVSInstanceNumMap(){
		return activityEdgeVSConstraintVSInstanceNum;
	}

	public List<String> getBrotherRelationIdList(){
		return brotherRelationIdList;
	}
	
	public Map<String, Double> getBehavioralConstraintVSConfidence(){
		return activityConstraintVSConfidence;
	}
	
    public static void main(String[] args) throws Exception {
  
    }
}