package org.processmining.ocbc.discovery.impl;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.processmining.common.structure.IntegerPair;
import org.processmining.ocbc.discovery.filter.FilterFunction;
import org.processmining.ocbc.log.model.XOCEvent;
import org.processmining.ocbc.log.model.XOCLog;
import org.processmining.ocbc.log.model.XOCORelation;
import org.processmining.ocbc.log.model.XOCObject;
import org.processmining.ocbc.log.model.impl.xConstants;



public class ClassEdgeCardinalityDiscovery {

	
	private FilterFunction filter = new  FilterFunction();
	
	private String standardizedSourceAlwaysCardinality;	
	private String standardizedTargetAlwaysCardinality;	
	private String standardizedSourceEventuallyCardinality;	
	private String standardizedTargetEventuallyCardinality;
	private Map<String, Integer> objectVSSourceEventuallyCardinality = new HashMap<String, Integer>();
	private Map<String, Integer> objectVSTargetEventuallyCardinality = new HashMap<String, Integer>();
	
	private XOCLog log;
	/**
	 * record the cardinality and life cycle period of source objects of each class relation;
	 * in other words, give the details of the target always cardinality;
	 */
	private Map<String, Map<XOCObject, Map<IntegerPair, Integer>>> classRelationIdVSSourceObjectVSPeriodVSCardinality = new HashMap<String, Map<XOCObject, Map<IntegerPair, Integer>>>();	
	/**
	 * record the cardinality and life cycle period of target objects of each class relation;
	 * in other words, give the details of the source always cardinality;
	 */
	private Map<String, Map<XOCObject, Map<IntegerPair, Integer>>> classRelationIdVSTargetObjectVSPeriodVSCardinality = new HashMap<String, Map<XOCObject, Map<IntegerPair, Integer>>>();	
	
	/**
	 * record the life cycle period of source objects of each class relation;
	 * indicate the order of periods;
	 */
	private Map<String, Map<XOCObject, List<IntegerPair>>> classRelationIdVSSourceObjectVSPeriodList = new HashMap<String, Map<XOCObject, List<IntegerPair>>> ();
	/**
	 * record the life cycle period of target objects of each class relation;
	 * indicate the order of periods;
	 */
	private Map<String, Map<XOCObject, List<IntegerPair>>> classRelationIdVSTargetObjectVSPeriodList = new HashMap<String, Map<XOCObject, List<IntegerPair>>> ();	
	
	
	public ClassEdgeCardinalityDiscovery(){
		
	}
	
	
	//output cardinality string
	public String OutputSourceAlwaysCardinality(){
		return standardizedSourceAlwaysCardinality;
	}
	
	public String OutputTargeteAlwaysCardinality(){
		return standardizedTargetAlwaysCardinality;	
	}
	
	public String OutputSourceEventuallyCardinality(){
		return standardizedSourceEventuallyCardinality;	
	}
	
	public String OutputTargetEventuallyCardinality(){
		return standardizedTargetEventuallyCardinality;
	}
	
	/**
	 * get classRelationVSSourceObjectVSPeriodVSCardinality which records the cardinality and life cycle period of source objects of each class relation;
	 * in other words, get the details of the target always cardinality;
	 * @return
	 */
	public Map<String, Map<XOCObject, Map<IntegerPair, Integer>>> getClassRelationIdVSSourceObjectVSPeriodVSCardinalityMap(){
		return classRelationIdVSSourceObjectVSPeriodVSCardinality;
	}
	
	/**
	 * get classRelationIdVSSourceObjectVSPeriodList which records life cycle period of each cardinality for source objects of each class relation;
	 * @return
	 */
	public Map<String, Map<XOCObject, List<IntegerPair>>> getClassRelationIdVSSourceObjectVSPeriodList(){
		return classRelationIdVSSourceObjectVSPeriodList;
	}
	
	/**
	 * get classRelationVSTargetObjectVSPeriodVSCardinality which records the cardinality and life cycle period of target objects of each class relation;
	 * in other words, get the details of the source always cardinality;
	 * @return
	 */
	public Map<String, Map<XOCObject, Map<IntegerPair, Integer>>> getClassRelationIdVSTargetObjectVSPeriodVSCardinalityMap(){
		return classRelationIdVSTargetObjectVSPeriodVSCardinality;
	}
	
	/**
	 * get classRelationIdVSTargetObjectVSPeriodList which records life cycle period of each cardinality for target objects of each class relation;
	 * @return
	 */
	public Map<String, Map<XOCObject, List<IntegerPair>>> getClassRelationIdVSTargetObjectVSPeriodList(){
		return classRelationIdVSTargetObjectVSPeriodList;
	}
	
	public void discoverClassEdgeCardinality(XOCLog log, String classRelationName, String sourceClassName, String targetClassName){	
		this.log = log;		
//		long uint = 1;	
//		long startTime = System.currentTimeMillis(); 
//		long durationTime = System.currentTimeMillis();
		
		
//		RunningTime.writeTimeInformationIntoFile("discoverEventuallyCardinality start time: " + startTime/uint, false, "D:\\timeDiscovery.txt");
		
		//first discover eventually cardinality
		discoverEventuallyCardinality(log, classRelationName, sourceClassName, targetClassName);
		
//		standardizedSourceEventuallyCardinality = "*";	
//		standardizedTargetEventuallyCardinality = "*";	
		
//		durationTime = (System.currentTimeMillis() - startTime)/uint;
//		startTime = System.currentTimeMillis();
//		RunningTime.writeTimeInformationIntoFile("discoverEventuallyCardinality  end time: " + System.currentTimeMillis()/uint, false, "D:\\timeDiscovery.txt");
//		RunningTime.writeTimeInformationIntoFile("~~~~~~~~~~~~~~discoverEventuallyCardinality duration time: " + durationTime, false, "D:\\timeDiscovery.txt");
//		RunningTime.writeTimeInformationIntoFile("discoverAlwaysCardinality start time: " + System.currentTimeMillis()/uint, false, "D:\\timeDiscovery.txt");
			
//		standardizedSourceAlwaysCardinality = "*";
//		standardizedTargetAlwaysCardinality = "*";
				
		//second discover always cardinality (the standardization is based on eventually cardinality)
		discoverAlwaysCardinalityInterval(log, classRelationName, sourceClassName, targetClassName);	
		
//		durationTime = (System.currentTimeMillis() - startTime)/uint;
//		startTime = System.currentTimeMillis();
//		RunningTime.writeTimeInformationIntoFile("discoverAlwaysCardinality  end time: " + System.currentTimeMillis()/uint, false, "D:\\timeDiscovery.txt");
//		RunningTime.writeTimeInformationIntoFile("~~~~~~~~~~~~~~discoverAlwaysCardinality duration time: " + durationTime, false, "D:\\timeDiscovery.txt");
	
	}
	
	/**
	 * (1)discover always cardinality interval for both source and target sides of class edges;
	 * (2)the end of the interval is the largest value of the eventually cardinality;
	 * the start of the interval is identified by checking the related objects in the birth event
	 * (3) this function is fast with losing precision of cardinality
	 * @param log
	 * @param classRelationName
	 * @param sourceClassName
	 * @param targetClassName
	 */
	public void discoverAlwaysCardinalityInterval(XOCLog log, String classRelationName, String sourceClassName, String targetClassName){
		
		List<Integer> sourceAlwaysCardinalityList = new ArrayList<Integer>();
		List<Integer> targetAlwaysCardinalityList = new ArrayList<Integer>();
		
		long uint = 1;	
		long startTime = System.currentTimeMillis(); 
		long durationTime = System.currentTimeMillis();
		
//		RunningTime.writeTimeInformationIntoFile("prepare discoverAlwaysCardinalityInterval start time: " + startTime/uint, true, "D:\\timeDiscovery.txt");
		
		
		Map<String, XOCEvent> idVSEventMap = log.getIdVSEventMap();
		XOCEvent lastEvent = log.getLastEvent();		
		Map<String, XOCObject> idVSObjectMapLast = log.getIdVSObjectMap();
		List<XOCObject> sourceObjects = filter.filterObjects(log.getObjectList(), sourceClassName);
		List<XOCObject> targetObjects = filter.filterObjects(log.getObjectList(), targetClassName);
		String eventId = lastEvent.getAttributes().get(xConstants.ID).toString();
//		System.out.println("eventId in discoverAlwaysCardinality: " + eventId);
		


		durationTime = (System.currentTimeMillis() - startTime)/uint;
		startTime = System.currentTimeMillis();
//		RunningTime.writeTimeInformationIntoFile("prepare objectVSSourceAlwaysCardinalityList  end time: " + System.currentTimeMillis()/uint, false, "D:\\timeDiscovery.txt");
//		RunningTime.writeTimeInformationIntoFile("~~~~~~~~~~~~~~prepare objectVSSourceAlwaysCardinalityList duration time: " + durationTime, false, "D:\\timeDiscovery.txt");
		
		classRelationIdVSTargetObjectVSPeriodVSCardinality.put(classRelationName, new HashMap<XOCObject, Map<IntegerPair,Integer>>());
		classRelationIdVSTargetObjectVSPeriodList.put(classRelationName, new HashMap<XOCObject, List<IntegerPair>>());
		
		discoverAlwaysCardinalityListForObjectList(targetObjects, classRelationIdVSTargetObjectVSPeriodVSCardinality.get(classRelationName), 
				classRelationIdVSTargetObjectVSPeriodList.get(classRelationName), sourceAlwaysCardinalityList, classRelationName);
		
		durationTime = (System.currentTimeMillis() - startTime)/uint;
		startTime = System.currentTimeMillis();
//		RunningTime.writeTimeInformationIntoFile("create objectVSSourceAlwaysCardinalityList  end time: " + System.currentTimeMillis()/uint, false, "D:\\timeDiscovery.txt");
//		RunningTime.writeTimeInformationIntoFile("~~~~~~~~~~~~~~create objectVSSourceAlwaysCardinalityList duration time: " + durationTime, false, "D:\\timeDiscovery.txt");

		classRelationIdVSSourceObjectVSPeriodVSCardinality.put(classRelationName, new HashMap<XOCObject, Map<IntegerPair,Integer>>());
		classRelationIdVSSourceObjectVSPeriodList.put(classRelationName, new HashMap<XOCObject, List<IntegerPair>>());
		
		discoverAlwaysCardinalityListForObjectList(sourceObjects, classRelationIdVSSourceObjectVSPeriodVSCardinality.get(classRelationName), 
				classRelationIdVSSourceObjectVSPeriodList.get(classRelationName), targetAlwaysCardinalityList, classRelationName);
		
		
		
		durationTime = (System.currentTimeMillis() - startTime)/uint;
		startTime = System.currentTimeMillis();
//		RunningTime.writeTimeInformationIntoFile("create objectVSTargeteAlwaysCardinalityList  end time: " + System.currentTimeMillis()/uint, false, "D:\\timeDiscovery.txt");
//		RunningTime.writeTimeInformationIntoFile("~~~~~~~~~~~~~~create objectVSTargeteAlwaysCardinalityList duration time: " + durationTime, false, "D:\\timeDiscovery.txt");

		standardizedSourceAlwaysCardinality = standardizeAlwaysCardinalityInfinit(sourceAlwaysCardinalityList);
		standardizedTargetAlwaysCardinality = standardizeAlwaysCardinalityInfinit(targetAlwaysCardinalityList);
		
		durationTime = (System.currentTimeMillis() - startTime)/uint;
		startTime = System.currentTimeMillis();
//		RunningTime.writeTimeInformationIntoFile("standardized end time: " + System.currentTimeMillis()/uint, false, "D:\\timeDiscovery.txt");
//		RunningTime.writeTimeInformationIntoFile("~~~~~~~~~~~~~~standardized duration time: " + durationTime, false, "D:\\timeDiscovery.txt");

	}
	

	/**
	 * discover eventually cardinality for both source and target sides of class edges;
	 * we only incorporate the cardinality of each reference object in the last object model;
	 * @param log
	 * @param classRelationName
	 * @param sourceClassName
	 * @param targetClassName
	 */
	public void discoverEventuallyCardinality(XOCLog log, String classRelationName, String sourceClassName, String targetClassName){
//		XOCEvent event = log.getLastEvent();		
//		Map<String, XOCObject> idVSObjectMapLast = event.getObjectModel().generateIdVSObjectMap();
		List<XOCObject> sourceObjects = filter.filterObjectsByClassAtEvent(log.getObjectList(), sourceClassName, log.getLastEvent());
		List<XOCObject> targetObjects = filter.filterObjectsByClassAtEvent(log.getObjectList(), targetClassName, log.getLastEvent());
//		String eventId = event.getAttributes().get(xConstants.ID).toString();
//		System.out.println("eventId in discoverAlwaysCardinality: " + eventId);
		
		objectVSSourceEventuallyCardinality.clear();
		objectVSTargetEventuallyCardinality.clear();
			
		for(XOCObject targetObject : targetObjects){
			String targetObjectId = targetObject.getAttributes().get("id").toString();
			int cardinality = 0;		
			for(XOCORelation relatedObjectRelation: targetObject.getRelatedObjectRelationList()){
				//check the number of alive related object relations of corresponding class relation 
				//at the last moment
				String relationType = relatedObjectRelation.getAttributes().get(xConstants.RELATION).toString();
				if(relationType.equals(classRelationName) && relatedObjectRelation.isAliveAtEvent(log.getLastEvent())){
					cardinality ++;	
				}
			}
			
			if(!objectVSSourceEventuallyCardinality.containsKey(targetObjectId)){
				objectVSSourceEventuallyCardinality.put(targetObjectId, cardinality);
			}			
		}
		
		for(XOCObject sourceObject : sourceObjects){
			String sourceObjectId = sourceObject.getAttributes().get("id").toString();
			int cardinality = 0;
			for(XOCORelation relatedObjectRelation: sourceObject.getRelatedObjectRelationList()){
				//check the number of alive related object relations of corresponding class relation 
				//at the last moment
				String relationType = relatedObjectRelation.getAttributes().get(xConstants.RELATION).toString();
				if(relationType.equals(classRelationName) && relatedObjectRelation.isAliveAtEvent(log.getLastEvent())){
					cardinality ++;	
				}
			}
			
			if(!objectVSTargetEventuallyCardinality.containsKey(sourceObjectId)){
				objectVSTargetEventuallyCardinality.put(sourceObjectId, cardinality);
			}
		}
		System.out.println("classRelationName: " + classRelationName);
		standardizedSourceEventuallyCardinality = standardizeEventuallyCardinality(objectVSSourceEventuallyCardinality);
		standardizedTargetEventuallyCardinality = standardizeEventuallyCardinality(objectVSTargetEventuallyCardinality);		
	}
	
	/**
	 * discover the always cardinality list based on an object list and the class relation name;
	 * fill in the alwaysCardinalityList, objectVSPeriodVSCardinality and objectVSPeriodList;
	 * @param objectList
	 * @param objectVSPeriodVSCardinality
	 * @param objectVSPeriodList
	 * @param alwaysCardinalityList
	 * @param classRelationName
	 */
	public void discoverAlwaysCardinalityListForObjectList(List<XOCObject> objectList, Map<XOCObject, Map<IntegerPair, Integer>> objectVSPeriodVSCardinality,
			Map<XOCObject, List<IntegerPair>> objectVSPeriodList, List<Integer> alwaysCardinalityList, String classRelationName){
		for(XOCObject object : objectList){
			objectVSPeriodVSCardinality.put(object, new HashMap<IntegerPair,Integer>());
			objectVSPeriodList.put(object, new ArrayList<IntegerPair>());
			int cardinality = 0;
			//check all life cycle periods of objects
			for(IntegerPair periodObject : object.getLifecyclePeriodList()){
				Integer startValueObject = periodObject.getFirstValue();
				Integer endValueObject = periodObject.getSecondValue() != null ? periodObject.getSecondValue() : log.size();
				//add all start moment of the object in the checking set
				Set<Integer> checkingMomentSet = new HashSet<Integer>();
				checkingMomentSet.add(startValueObject);
				//at every moment of birth and death of object relation in the life cycle of object
				for(XOCORelation relatedObjectRelation: object.getRelatedObjectRelationList()){
					String relationType = relatedObjectRelation.getAttributes().get(xConstants.RELATION).toString();
					if(relationType.equals(classRelationName)){
						for(IntegerPair period : relatedObjectRelation.getLifecyclePeriodList()){
							Integer startValue = period.getFirstValue();
							Integer endValue = period.getSecondValue() != null ? period.getSecondValue() : log.size();
							if(startValueObject <= startValue  && startValue <= endValueObject){
								checkingMomentSet.add(startValue);
							}
							// the died moment is the end moment (i.e., last appearing moment) plus 1
							if(startValueObject <= endValue+1  && endValue+1 <= endValueObject){
								checkingMomentSet.add(endValue+1);
							}
						}
					}
				}
				
				List<Integer> checkingMomentList = new ArrayList<Integer>(checkingMomentSet);
				Collections.sort(checkingMomentList);
				Integer lastCardinality = null;//records the last previous cardinality value;
				IntegerPair lastPeriod = null;//records the period for the last previous cardinality value;
				for(Integer checkingMoment : checkingMomentList){
					cardinality = 0;
					for(XOCORelation relatedObjectRelation: object.getRelatedObjectRelationList()){
						//check the number of alive related object relations of corresponding class relation 
						//at every moment of birth and death of object relation
						String relationType = relatedObjectRelation.getAttributes().get(xConstants.RELATION).toString();
						if(relationType.equals(classRelationName) && relatedObjectRelation.isAliveAtEvent(log.get(checkingMoment-1))){
							cardinality ++;	
						}
					}
					if(lastCardinality != null){
						if(lastCardinality != cardinality){
							lastPeriod.setSecondValue(checkingMoment-1);
							objectVSPeriodVSCardinality.get(object).put(lastPeriod, lastCardinality);
							objectVSPeriodList.get(object).add(lastPeriod);
							lastCardinality = cardinality;
							lastPeriod = new IntegerPair(checkingMoment, null);
						}
						//if the cardinality is the same, do nothing
						else{
							
						}
					}
					//the first checking moment
					else{
						lastPeriod = new IntegerPair(checkingMoment, null);
						lastCardinality = cardinality;
					}	
					//add all cardinality at the (object) born moment
					if(!alwaysCardinalityList.contains(cardinality)){
						alwaysCardinalityList.add(cardinality);
					}
				}
				lastPeriod.setSecondValue(endValueObject);
				objectVSPeriodVSCardinality.get(object).put(lastPeriod, lastCardinality);	
				objectVSPeriodList.get(object).add(lastPeriod);
			}
				
		}
	}
	
	/**
	 * standardize always cardinalities as 
	 * (1) "0..*" or "1..*" if objects have cardinality more than 1
	 * (2) 1 if all objects have same cardinality (i.e., 1)
	 * (3)"0..1" if objects have two cardinalities (i.e., 0 and 1) 
	 * note that the cardinalityList should have unique element
	 * @param objectVSCardinalityList
	 * @return
	 */
	public String standardizeAlwaysCardinalityInfinit(List<Integer> cardinalityList){	
		String standardizeCardinality = null;
		int size = cardinalityList.size();
		if(size == 1){
			int singleValue = cardinalityList.get(0);
			if(singleValue == 1){
				standardizeCardinality = "1";
			}
			//impossible, used to find the wrong situation
			else if(singleValue == 0){
				standardizeCardinality = "0";
			}
			else if(singleValue > 1){
				standardizeCardinality = "1..*";
			}
		}
		else if(size == 2){
			int firstValue = cardinalityList.get(0);
			int secondValue = cardinalityList.get(1);
			int sum = firstValue + secondValue;
			if(sum == 1){
				standardizeCardinality = "0..1";
			}
			else if(firstValue == 0 || secondValue == 0){
				standardizeCardinality = "*";
			}
			else{
				standardizeCardinality = "1..*";
			}		
		}
		else if(size > 2){
			if(cardinalityList.contains(0)){
				standardizeCardinality = "*";
			}
			else{
				standardizeCardinality = "1..*";
			}
		}
		return standardizeCardinality;
	}
		
		
	//standardize always cardinalities as (1) "0..*" or "1..*" if objects have different cardinality sets and its
	//eventually cardinality is "0..*" or "1..*"
	//(2) as it is if all objects have same cardinality sets
	//(3)"0..k" or "1..k" if objects have different cardinality sets, its
	//eventually cardinality is k and any cardinality in the sets is not larger than k
	public String standardizeAlwaysCardinality(Map<String, List<Integer>> objectVSCardinalityList, String basedEventuallyCardinality){	
		String standardizeCardinality = null;		
		int objectIndex = 1;
		boolean differentFlag = false;
		List<Integer> firstCardinalityList = null;
//		System.out.println("basedEventuallyCardinality: " + basedEventuallyCardinality);
		for(String objectId : objectVSCardinalityList.keySet()){
			System.out.println("objectId in standardizeAlwaysCardinality: " + objectId);
			for(int card : objectVSCardinalityList.get(objectId)){
				System.out.println("card: " + card);
			}
			if(objectIndex == 1){
				firstCardinalityList = objectVSCardinalityList.get(objectId);
			}
			//all the other cardinality sets equal to the first cardinality set
			else{				
				if(objectVSCardinalityList.get(objectId).size() == firstCardinalityList.size()){
					for(int cardinality : objectVSCardinalityList.get(objectId)){
						if(!firstCardinalityList.contains(cardinality)){
							differentFlag = true;						
							break;
						}
					}
				}
				else differentFlag = true;				
			}
			objectIndex ++;
			if(differentFlag) break;			
		}
		//if objects have different cardinality sets
		if(differentFlag){
			//is not a single value
			if(basedEventuallyCardinality.length()>2){
				standardizeCardinality = "1..*";
				for(String objectId : objectVSCardinalityList.keySet()){
					if(objectVSCardinalityList.get(objectId).contains(0)){
						standardizeCardinality = "*";
						break;
					}
				}
			}
			//is a single value
			else{
				standardizeCardinality = "1.." + basedEventuallyCardinality;
				for(String objectId : objectVSCardinalityList.keySet()){
					if(objectVSCardinalityList.get(objectId).contains(0)){
						standardizeCardinality = "0.." + basedEventuallyCardinality;
						break;
					}
				}
			}
			
		}	
		else if(firstCardinalityList.size() == 1){
			differentFlag = false;
//			standardizeCardinality = new Cardinality(cardinalityList).toString();
			if(firstCardinalityList.get(0) > 1){
				standardizeCardinality = "1..*";
			}
			else if(firstCardinalityList.get(0) == 1){
				standardizeCardinality = "1";
			}
			else if(firstCardinalityList.get(0) == 0){
				standardizeCardinality = "0";
			}
		}
			
//		System.out.println("standardizeCardinality: " + standardizeCardinality);
		return standardizeCardinality;
	}
	
	//standardize eventually cardinalities as 
	//(1)"0..*" or "1..*" (the cardinality list does not contain 0) if objects have different eventually cardinalities
	//(2)k if all objects have cardinality "k"
	public String standardizeEventuallyCardinality(Map<String, Integer> objectVSEventuallyCardinality){
		String eventuallyCardinalityString = null;
		List<Integer> cardinalityList = new ArrayList<Integer>();
		for(String objectId : objectVSEventuallyCardinality.keySet()){
			int cardinalityValue = objectVSEventuallyCardinality.get(objectId);
			if(!cardinalityList.contains(cardinalityValue)){
				cardinalityList.add(cardinalityValue);
			}
//			System.out.println("objectId in standardizeEventuallyCardinality: " + objectId);
//			System.out.println("cardinalityValue in standardizeEventuallyCardinality: " + cardinalityValue);
		}
		if(cardinalityList.size() == 1) {		 
			if(cardinalityList.get(0) > 1){
				eventuallyCardinalityString = "1..*";
			}
			else if(cardinalityList.get(0) == 1){
				eventuallyCardinalityString = "1";
			}
			else if(cardinalityList.get(0) == 0){
				eventuallyCardinalityString = "0";
			}
		}	
		else if(cardinalityList.size() == 2) {		 
			if(cardinalityList.get(0) + cardinalityList.get(1) == 1){
				eventuallyCardinalityString = "0..1";
			}
			else if(cardinalityList.contains(0)) eventuallyCardinalityString = "*";
			else eventuallyCardinalityString = "1..*";
		}	
		else{
			if(cardinalityList.contains(0)) eventuallyCardinalityString = "*";
			else eventuallyCardinalityString = "1..*";
		}
		return eventuallyCardinalityString;
	}			
	
    public static void main(String[] args) throws Exception {
  
    }
	



}