package org.processmining.ocbc.discovery.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.processmining.mxgraph.util.mxConstants;
import org.processmining.ocbc.log.model.XOCLog;
import org.processmining.ocbc.log.model.XOCORelation;
import org.processmining.ocbc.log.model.XOCObject;
import org.processmining.ocbc.log.model.impl.xConstants;
import org.processmining.ocbc.models.visualization.ModelLayoutBuilder;


public class ClassModelDiscovery {
	private XOCLog log;
	private ModelLayoutBuilder builder;
	private Map<String, String> nodeNameVSNodeId;
	private ClassEdgeCardinalityDiscovery classEdgeCardinalityDiscovery = new ClassEdgeCardinalityDiscovery();
	//record the related object id list of each object in object model of each event
	private Map<String, Map<String, Map<String, List<String>>>> eventIdVSRelationIdVSObjectIdVSRelatedObjectIdList = new HashMap<String, Map<String, Map<String, List<String>>>>();
	private Map<String, Map<String, Map<String, List<String>>>> relationIdVSEventIdVSObjectIdVSRelatedObjectIdList = new HashMap<String, Map<String, Map<String, List<String>>>>();
	
	private Integer idCounter;
	public ClassModelDiscovery(XOCLog log, ModelLayoutBuilder builder, Map<String, String> nodeNameVSNodeId){
		this.log = log;
		this.builder = builder;
		this.nodeNameVSNodeId = nodeNameVSNodeId;
	}
	
	public void setIdCounter(Integer idCounter){
		this.idCounter = idCounter;
	}
	
	public Integer getIdCounter(){
		return idCounter;
	}
	
	public void discoverClassModel(){
		//discover class nodes
		for(String className : log.generateClassNameList()){
//			String classNodeId = UUID.randomUUID().toString();
			String classNodeId = className + (idCounter ++).toString();
			nodeNameVSNodeId.put(className, classNodeId);
			
			//draw the class node
			builder.addNode(mxConstants.CLASSCELL, className, classNodeId, 
	        		Math.random()*1000, Math.random()*1000,
	        		55.0, 65.0);
		}
		
		List<String> classRelationcombinationIds = new ArrayList<String>();
		//discover class edges
//		for(XOCEvent event : log.getAllEvents()){
		System.out.println("log.getLastEvent().getObjectModel().getObjectRelationSet().size(): " + log.getLastEvent().getObjectModel().getObjectRelationSet().size());
//			for(XOCORelation objectRelation : log.getLastEvent().getObjectModel().getObjectRelationSet()){			
			for(XOCORelation objectRelation : log.getObjectRelationList()){			
				String classRelationName = objectRelation.getAttributes().get("relation").toString();
			
//				XOCObject sourceObject = log.getSourceObjectOfObjectRelation(event.getObjectModel(), objectRelation);
				XOCObject sourceObject = objectRelation.getSourceObject();
				String sourceClassName = sourceObject.getAttributes().get(xConstants.CLASS).toString();
				String sourceClassNodeId = nodeNameVSNodeId.get(sourceClassName);
				
//				XOCObject targetObject = log.getTargetObjectOfObjectRelation(event.getObjectModel(), objectRelation);
				XOCObject targetObject = objectRelation.getTargetObject();
				String targetClassName = targetObject.getAttributes().get(xConstants.CLASS).toString();
				String targetClassNodeId = nodeNameVSNodeId.get(targetClassName);
				
				
				String classRelationcombinationId = classRelationName + "_" + sourceClassName + "_" + targetClassName;
				if(!classRelationcombinationIds.contains(classRelationcombinationId)){
					classRelationcombinationIds.add(classRelationcombinationId);
					//discover the class edge cardinality
//					String classRelationId = classRelationName + (idCounter ++).toString();
					String classRelationId = classRelationName;
					
//					System.out.println("class edge id when created:" + classRelationId);
//					System.out.println("class edge name when created:" + classRelationName);
					
					classEdgeCardinalityDiscovery.discoverClassEdgeCardinality(log, classRelationName, sourceClassName, targetClassName);
							
//					relationIdVSEventIdVSObjectIdVSRelatedObjectIdList.put(classRelationId, 
//							classEdgeCardinalityDiscovery.getEventIdVSObjectIdVSRelatedObjectIdListMap());
					//draw the class edge
					builder.addClassEdge(classRelationId, classRelationName, sourceClassNodeId, 
							classEdgeCardinalityDiscovery.OutputSourceAlwaysCardinality(),classEdgeCardinalityDiscovery.OutputSourceEventuallyCardinality(), 
							targetClassNodeId, classEdgeCardinalityDiscovery.OutputTargeteAlwaysCardinality(), 
							classEdgeCardinalityDiscovery.OutputTargetEventuallyCardinality(), 0, 0, 55.0, 65.0);
				}			
			}
//		}
	}
	
	public ClassEdgeCardinalityDiscovery getClassEdgeCardinalityDiscovery(){
		return classEdgeCardinalityDiscovery;
	}
	
	
//	public Map<String, Map<String, Map<String, List<String>>>> getRelationIdVSEventIdVSObjectIdVSRelatedObjectIdList(){
//		return relationIdVSEventIdVSObjectIdVSRelatedObjectIdList;
//	}
	
	public Map<String, Map<String, Map<String, List<String>>>> getEventIdVSRelationIdVSObjectIdVSRelatedObjectIdList(){
		return eventIdVSRelationIdVSObjectIdVSRelatedObjectIdList;
	}
	
    public static void main(String[] args) throws Exception {
  
    }
}