package org.processmining.ocbc.discovery.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.swing.JLabel;

import org.deckfour.uitopia.api.event.TaskListener.InteractionResult;
import org.processmining.common.number.ParseNumberFromString;
import org.processmining.common.runningtime.RunningTime;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.mxgraph.examples.swing.io.TransformBetweenJGraphXAndModel;
import org.processmining.mxgraph.io.mxCodec;
import org.processmining.mxgraph.layout.hierarchical.mxHierarchicalLayout;
import org.processmining.mxgraph.model.mxCell;
import org.processmining.mxgraph.model.mxGeometry;
import org.processmining.mxgraph.util.mxConstants;
import org.processmining.mxgraph.util.mxXmlUtils;
import org.processmining.mxgraph.view.mxGraph;
import org.processmining.ocbc.discovery.panels.DiscoveryParameterPanel;
import org.processmining.ocbc.log.model.XOCLog;
import org.processmining.ocbc.models.OCBCModel;
import org.processmining.ocbc.models.impl.OCBCModelImpl;
import org.processmining.ocbc.models.visualization.ModelLayoutBuilder;
import org.w3c.dom.Document;



public class OCBCModelDiscovery {	
	private Map<String, String> nodeNameVSNodeId = new HashMap<String, String>();
	private XOCLog log;
	private OCBCModel model = new OCBCModelImpl("OCBC model");
    private ModelLayoutBuilder builder = new ModelLayoutBuilder(model);
//    private ModelDiscoveryResult modelDiscoveryResult = new ModelDiscoveryResult();
    private ClassModelDiscovery classModelDiscovery;
    private BehaviorModelDiscovery behaviorModelDiscovery;
    private AOCEdgeDiscovery aOCEdgeDiscovery;
    private String approachName;
    private DiscoveryParameterPanel parameterPanel;
    
	String configuredFitnessAlgorithm;
	String configuredPrecisionAlgorithm;
	String configuredGeneralizationAlgorithm;
	
	Double configuredFitnessThreshold = null;	
	Double configuredPrecisionThreshold = null;	
	Double configuredGeneralizationThreshold = null;	
	
	double configuredFitnessWeight;
	double configuredPrecisionWeight;
	double configuredGeneralizationWeight;
	
	public OCBCModelDiscovery(XOCLog log){
		this.log = log;
	}

	public OCBCModelDiscovery(XOCLog log, String approachName){
		this.log = log;
		this.approachName = approachName;
	}
	
	public OCBCModel discover(UIPluginContext context) {	
		setParameter(context);
		
		long uint = 1;
//		long totalTime = estimateRunningTime(log, model);	
		long startTimeTotal = System.currentTimeMillis(); 
		long startTime = System.currentTimeMillis(); 
		long durationTime = System.currentTimeMillis();
		RunningTime.writeTimeInformationIntoFile("discover class model start time: " + startTime/uint, false,"D:\\timeDiscovery.txt");
		
		Integer idCounter = 2;
		classModelDiscovery = new ClassModelDiscovery(log, builder, nodeNameVSNodeId);
	    classModelDiscovery.setIdCounter(idCounter);
	    classModelDiscovery.discoverClassModel();
	    
	    durationTime = (System.currentTimeMillis() - startTime)/uint;
		startTime = System.currentTimeMillis();
		RunningTime.writeTimeInformationIntoFile("discover class model end time: " + System.currentTimeMillis()/uint, false,"D:\\timeDiscovery.txt");
		RunningTime.writeTimeInformationIntoFile("***discover class model duration time: " + durationTime, false,"D:\\timeDiscovery.txt");
		
		RunningTime.writeTimeInformationIntoFile("discover behavior node start time: " + System.currentTimeMillis()/uint, false,"D:\\timeDiscovery.txt");
		
	    behaviorModelDiscovery = new BehaviorModelDiscovery(log, builder, nodeNameVSNodeId, approachName);
	    System.out.println("idCounter after finishing class model: " + classModelDiscovery.getIdCounter());
	    behaviorModelDiscovery.setIdCounter(classModelDiscovery.getIdCounter());
	    behaviorModelDiscovery.discoverActivityNodes();
		
	    	
	    durationTime = (System.currentTimeMillis() - startTime)/uint;
		startTime = System.currentTimeMillis();
		RunningTime.writeTimeInformationIntoFile("discover behavior node end time: " + System.currentTimeMillis()/uint, false,"D:\\timeDiscovery.txt");
		RunningTime.writeTimeInformationIntoFile("***discover behavior node duration time: " + durationTime, false,"D:\\timeDiscovery.txt");
		
	    System.out.println("idCounter after behavior nodes: " + behaviorModelDiscovery.getIdCounter());
	    
	    
	    RunningTime.writeTimeInformationIntoFile("discover aoc relation start time: " + System.currentTimeMillis()/uint, false,"D:\\timeDiscovery.txt");
	    //it is possible to discover "aoc" edges after the activity and class nodes are discovered
	    aOCEdgeDiscovery = new AOCEdgeDiscovery(log, builder, nodeNameVSNodeId);
	    aOCEdgeDiscovery.setIdCounter(behaviorModelDiscovery.getIdCounter());
	    aOCEdgeDiscovery.discoverAOCEdges();
	    durationTime = (System.currentTimeMillis() - startTime)/uint;
		startTime = System.currentTimeMillis();
		RunningTime.writeTimeInformationIntoFile("discover aoc relation end time: " + System.currentTimeMillis()/uint, false,"D:\\timeDiscovery.txt");
		RunningTime.writeTimeInformationIntoFile("***discover aoc relation duration time: " + durationTime, false,"D:\\timeDiscovery.txt");
		
		
	    System.out.println("idCounter after finishing aoc edges: " + aOCEdgeDiscovery.getIdCounter());
	    RunningTime.writeTimeInformationIntoFile("discover behavioral relation start time: " + System.currentTimeMillis()/uint, false,"D:\\timeDiscovery.txt");
	    //it is possible to discover activity edges after the "aoc" edges are discovered
	    behaviorModelDiscovery.setIdCounter(aOCEdgeDiscovery.getIdCounter());
	    behaviorModelDiscovery.discoverActivityEdges(configuredFitnessAlgorithm, 
	    		configuredPrecisionAlgorithm, 
	    		configuredGeneralizationAlgorithm, 
	    		configuredFitnessThreshold, 
	    		configuredPrecisionThreshold, 
	    		configuredGeneralizationThreshold, 
	    		configuredFitnessWeight, configuredPrecisionWeight, configuredGeneralizationWeight);
		
	    durationTime = (System.currentTimeMillis() - startTime)/uint;
		startTime = System.currentTimeMillis();
		RunningTime.writeTimeInformationIntoFile("discover behavioral relation end time: " + System.currentTimeMillis()/uint, false,"D:\\timeDiscovery.txt");
		RunningTime.writeTimeInformationIntoFile("***discover behavioral relation duration time: " + durationTime, false,"D:\\timeDiscovery.txt");
		
		long endTimeTotal = System.currentTimeMillis(); 
		durationTime = (System.currentTimeMillis() - startTimeTotal)/uint;
		RunningTime.writeTimeInformationIntoFile("model discover  end time: " + System.currentTimeMillis()/uint, false,"D:\\timeDiscovery.txt");
		RunningTime.writeTimeInformationIntoFile("--------------model discover  duration time: " + durationTime, false,"D:\\timeDiscovery.txt");
		
//	    modelDiscoveryResult.setActivityEdgeVSEventVSPrecedenceEventList(behaviorModelDiscovery.getActivityEdgeVSEventVSPrecedenceEventList());
//	    modelDiscoveryResult.setActivityEdgeVSEventVSResponseEventList(behaviorModelDiscovery.getActivityEdgeVSEventVSResponseEventList());
	    
		visualizeModel(model);
		return model;					    
	}				
	
	public void parseInputParameters(){
		configuredFitnessAlgorithm = (String) parameterPanel.getMetricsAlgorithmPanel().getFitnessAlgorithmBox().getSelectedItem();
		configuredPrecisionAlgorithm = (String) parameterPanel.getMetricsAlgorithmPanel().getPrecisionAlgorithmBox().getSelectedItem();
		configuredGeneralizationAlgorithm = (String) parameterPanel.getMetricsAlgorithmPanel().getGeneralizationAlgorithmBox().getSelectedItem();
		configuredFitnessWeight = ((double)parameterPanel.getSliderPanel().getFitnessSlider().getValue())/100;	 
		configuredPrecisionWeight = ((double)parameterPanel.getSliderPanel().getPrecisionSlider().getValue())/100;	
		configuredGeneralizationWeight = ((double)parameterPanel.getSliderPanel().getGeneralizationSlider().getValue())/100;	
		
		String fitnessThresholdText =  parameterPanel.getMetricsAlgorithmPanel().getFitnessThresholdField().getText();
		String precisionThresholdText =  parameterPanel.getMetricsAlgorithmPanel().getPrecisionThresholdField().getText();
		String generalizationThresholdText =  parameterPanel.getMetricsAlgorithmPanel().getGeneralizationThresholdField().getText();
		
		ParseNumberFromString parseNumberFromString = new ParseNumberFromString();
		
		if(configuredFitnessAlgorithm.equals(mxConstants.fitnessAlgorithmWithNumThreshold)){	
			Integer parseInteger = parseNumberFromString.parseIntegerFromStringWithBoundary(fitnessThresholdText, 0, Integer.MAX_VALUE,
					"Wrong! The input for fitness algorithm should be a positive integer");
			configuredFitnessThreshold = parseInteger != null ? parseInteger.doubleValue() : null;
		}
		else if(configuredFitnessAlgorithm.equals(mxConstants.fitnessAlgorithmWithRatioThreshold)){		
			configuredFitnessThreshold = parseNumberFromString.parseDoubleFromStringWithBoundary(fitnessThresholdText, 0, 1,
					"Wrong! The input for fitness algorithm should be an decimal in [0,1]");
		}
		else if(configuredFitnessAlgorithm.equals(mxConstants.fitnessAlgorithmWithoutThreshold)){		
			configuredFitnessThreshold = 0.0;
		}
		
		
		if(configuredPrecisionAlgorithm.equals(mxConstants.precisionAlgorithmWithNumThreshold)){	
			Integer parseInteger = parseNumberFromString.parseIntegerFromStringWithBoundary(precisionThresholdText, 0, Integer.MAX_VALUE,
					"Wrong! The input for precision algorithm should be a positive integer");
			configuredPrecisionThreshold = parseInteger != null ? parseInteger.doubleValue() : null;
		}
		else if(configuredPrecisionAlgorithm.equals(mxConstants.precisionAlgorithmWithRatioThreshold)){		
			configuredPrecisionThreshold = parseNumberFromString.parseDoubleFromStringWithBoundary(precisionThresholdText, 0, 1,
					"Wrong! The input for precision algorithm should be an decimal in [0,1]");
		}
		else if(configuredPrecisionAlgorithm.equals(mxConstants.precisionAlgorithmByEntropy)){		
			configuredPrecisionThreshold = 0.0;
		}
		
		configuredGeneralizationThreshold = parseNumberFromString.parseDoubleFromStringWithBoundary(generalizationThresholdText, 0, 1,
				"Wrong! The input for generalization algorithm should be an decimal in [0,1]");
	}
	
	/**
	 * create a interface to visualize models and support functionalities to adjust models;
	 * the interface reuse the model editor and add the adjust panel (algorithm panel, weight panel, filter panel);
	 * @param model
	 */
	public void visualizeModel(OCBCModel model) {	
		TransformBetweenJGraphXAndModel transformer = new TransformBetweenJGraphXAndModel();
		mxGraph graph = transformer.transformModelToJGraphX(model, null);
		
		mxHierarchicalLayout layout = new mxHierarchicalLayout(graph, JLabel.WEST);
		Object cell = graph.getDefaultParent();
		graph.getModel().beginUpdate();
		Object[] childVertices = graph.getChildVertices(cell);
		Object[] childEdges = graph.getChildEdges(cell);		
		List<Object> childCellList = new ArrayList<Object>();
		for(int i=0; i<childVertices.length; i++){
			childCellList.add(childVertices[i]);
		}
		for(int i=0; i<childEdges.length; i++){
			childCellList.add(childEdges[i]);
		}		
		layout.execute(cell, childCellList);
		
		Object[] childEdgesTwo = graph.getChildEdges(graph.getDefaultParent());
		for(int i=0; i<childEdges.length; i++){
			mxCell edgeCell = (mxCell) childEdgesTwo[i];
			String type = edgeCell.getAttributes().get(mxConstants.CARDINALITYTYPE);		
			if(type != null && type.contains("non")){
				mxGeometry geometry = new mxGeometry();
				geometry.setRelative(true);
				geometry.setPoints(edgeCell.getGeometry().getPoints());
				edgeCell.setGeometry(geometry);				
			}
		}
		graph.refresh();
		
		mxCodec codec = new mxCodec();
		String modelXML = mxXmlUtils.getXml(codec.encode(graph.getModel()));
		System.out.println("xml: " + modelXML);
		Document document = mxXmlUtils.parseXml(modelXML);

		model.setModelString(modelXML);
		
		model.setModelFile(document);
	}
	
	public AOCEdgeDiscovery getAOCEdgeDiscovery(){
		return aOCEdgeDiscovery;
	}
	
	public BehaviorModelDiscovery getBehaviorModelDiscovery(){
		return behaviorModelDiscovery;
	}
	
	public ClassModelDiscovery getClassModelDiscovery(){
		return classModelDiscovery;
	}
	
//	public void writeTimeInformationIntoFile(String wirteLine, boolean clearFile){
//		File log = new File("D:\\timeDiscovery.txt");
//	    try{
//		    if(log.exists()==false || clearFile){
//	            System.out.println("We had to make a new file.");
//	            log.createNewFile();
//		    }
//		    PrintWriter out = new PrintWriter(new FileWriter(log, true));
//		    out.append(wirteLine + "\n");
//		    out.close();
//	    }catch(IOException e){
//	        System.out.println("COULD NOT LOG!!");
//	    }
//	}
	
	
	public void setParameter(UIPluginContext context){
		if(context != null){
			parameterPanel = new DiscoveryParameterPanel();

			InteractionResult interActionResult = context.showConfiguration("Discovery Configuration", parameterPanel);
			if (interActionResult.equals(InteractionResult.CANCEL)) {
				context.getFutureResult(0).cancel(true);			
			}				
			else if (interActionResult.equals(InteractionResult.CONTINUE)) {			
				parseInputParameters();
			}
		}
		else{
			configuredFitnessAlgorithm = mxConstants.fitnessAlgorithmWithoutThreshold;
			configuredPrecisionAlgorithm = mxConstants.precisionAlgorithmWithNumThreshold;
			configuredGeneralizationAlgorithm = mxConstants.generalizationAlgorithmBinomialDistribution;
			configuredFitnessWeight =  1;
			configuredPrecisionWeight =  1;
			configuredGeneralizationWeight = 1;
			configuredFitnessThreshold = 0.0;	
			configuredPrecisionThreshold = 1.0;	
			configuredGeneralizationThreshold = 0.95;	
		}
	}
	
    public static void main(String[] args) throws Exception {
    	
    }
	
}