package org.processmining.ocbc.discovery.metrics;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Scanner;

import org.processmining.mxgraph.examples.swing.io.TransformToOCBCSemanticModel;
import org.processmining.mxgraph.io.mxCodec;
import org.processmining.mxgraph.util.mxXmlUtils;
import org.processmining.mxgraph.view.mxGraph;
import org.processmining.ocbc.log.io.XocXmlParser;
import org.processmining.ocbc.log.model.XOCEvent;
import org.processmining.ocbc.log.model.XOCLog;
import org.processmining.ocbc.log.model.impl.xConstants;
import org.processmining.ocbc.models.ActivityEdge;
import org.processmining.ocbc.models.ClassNode;
import org.processmining.ocbc.models.OCBCModel;
import org.processmining.ocbc.models.editor.plugins.OCBCEditorConstants;
import org.processmining.ocbc.models.io.ImportFromFile;
import org.processmining.ocbc.models.visualization.cell.MNodeCell;
import org.processmining.ocbc.models.visualization.cell.MyOriginalCell;
import org.w3c.dom.Document;


public class ComputeBehavioralPrecision {
//	protected XOCLog log;
//	protected OCBCModel model;
	protected Map<ActivityEdge, Double> sortedRelationVSPrecision = new HashMap<ActivityEdge, Double>();
	protected List<ActivityEdge> sortedActivityList = new ArrayList<ActivityEdge>();
	protected List<ActivityEdge> decreaselySortedActivityList = new ArrayList<ActivityEdge>();
//	public ComputeBehavioralPrecision(XOCLog log, OCBCModel model){
//		this.log = log;
//		this.model = model;
//	}
	
	public Map<ActivityEdge, Double> getSortedBehaviorConstraintVSPrecisionMap(){
		return sortedRelationVSPrecision;
	}
	
	public List<ActivityEdge> getIncreaselySortedActivityList(){
		return sortedActivityList;
	}
	
	public List<ActivityEdge> getDecreaselySortedActivityList(){
		return decreaselySortedActivityList;
	}
	
	public double computePrecision(XOCLog log, OCBCModel model){
		//build model unit family
		SearchOCBCUnitFamily searchOCBCUnitFamily = new SearchOCBCUnitFamily(model);
		searchOCBCUnitFamily.search();
        Map<String, OCBCUnitFamily> idVSUnitFamily = searchOCBCUnitFamily.getIdVSUnitFamily();

        ExtractXOCSegment extractXOCSegment = new ExtractXOCSegment(log, idVSUnitFamily);
		extractXOCSegment.extractAllSegment();
		Map<String, XOCSegment> idVSSegment = extractXOCSegment.getIdVSSegment();	
		Map<String, Double> segmentIdVSPrecison = new HashMap<String, Double>();
		
		for(String id : idVSUnitFamily.keySet()){
			OCBCUnitFamily family = idVSUnitFamily.get(id);
        	GenerateUnitFamilyAutomaton generateFamilyAutomaton = new GenerateUnitFamilyAutomaton(family);
        	generateFamilyAutomaton.generateAutomation();
        	Map<String, Integer> stateIdVSEnabledEdgeModel = generateFamilyAutomaton.getStateIdVSEnabledEdge(); 
        	
			XOCSegment segment = idVSSegment.get(id);			
        	GenerateSegmentAutomaton generateSegmentAutomaton = new GenerateSegmentAutomaton(segment);
        	generateSegmentAutomaton.generateAutomation();
        	
//        	Map<String, AutomatonState> idVSState = generateSegmentAutomaton.getIdVSState();
        	Map<String, Integer> stateIdVSEnabledEdgeLog = generateSegmentAutomaton.getStateIdVSEnabledEdge(); 
        	System.out.println("********************************************************");
        	System.out.println("getReferenceActivity: " + family.getReferenceActivity().getNodeName());
        	System.out.println("getTargetActivity: " + family.getTargetActivity().getNodeName());
        	System.out.println("type: " + family.getType());
			for(ClassNode classNode : family.getClassNodeList()){
				System.out.println("classNode.getNodeName(): " + classNode.getNodeName());				
			}
        	//compute the precision between an unit family and a segment based on Wil's rule
        	double segmentPrecison = 0;
        	int eventNumInSegment = 0;
        	for(String caseId : segment.getIdVSCase().keySet()){      		
        		XOCCase xOCcase = segment.getIdVSCase().get(caseId);
        		System.out.println("\n caseId: " + xOCcase.getId());
        		
        		
        		String stateId = 0 + "-" + 0 + "-" + 0;
    			double enabledEdgeNumModel = stateIdVSEnabledEdgeModel.get(stateId);
    			double enabledEdgeNumLog = stateIdVSEnabledEdgeLog.get(stateId);    			
    			segmentPrecison = segmentPrecison + enabledEdgeNumLog/enabledEdgeNumModel;
    			
    			int precedenceNum = 0;
    			for(XOCEvent event : xOCcase.getPrecedenceTargetEventList()){
        			System.out.println("\n precedence event Id: " + event.getAttributes().get(xConstants.ID).toString());
    				precedenceNum ++;
            		stateId = precedenceNum + "-" + 0 + "-" + 0;
        			enabledEdgeNumModel = stateIdVSEnabledEdgeModel.get(stateId);
        			enabledEdgeNumLog = stateIdVSEnabledEdgeLog.get(stateId);    			
        			segmentPrecison = segmentPrecison + enabledEdgeNumLog/enabledEdgeNumModel;
        		}   			
    			
//    			// the reference event is not the last event, without adding the enable edges of the last state 
//    			int caseLength = xOCcase.getPrecedenceTargetEventList().size() + xOCcase.getResponseTargetEventList().size() + 1;
//    			if(xOCcase.getResponseTargetEventList().size() > 0){
//    				stateId = precedenceNum + "-" + 1 + "-" + 0;
//        			enabledEdgeNumModel = stateIdVSEnabledEdgeModel.get(stateId);
//        			enabledEdgeNumLog = stateIdVSEnabledEdgeLog.get(stateId);    			
//        			segmentPrecison = segmentPrecison + enabledEdgeNumLog/enabledEdgeNumModel;
//    			}        		    			
//    			
//    			for(int responseNum = 1; responseNum < xOCcase.getResponseTargetEventList().size(); responseNum ++){
////    				System.out.println("\n response event Id: " + event.getAttributes().get(xConstants.ID).toString());
//            		stateId = precedenceNum + "-" + 1 + "-" + responseNum;
//        			enabledEdgeNumModel = stateIdVSEnabledEdgeModel.get(stateId);
//        			enabledEdgeNumLog = stateIdVSEnabledEdgeLog.get(stateId);    			
//        			segmentPrecison = segmentPrecison + enabledEdgeNumLog/enabledEdgeNumModel;
//        		}     
    			
    			//add the enable edges of the last state
    			int caseLength = xOCcase.getPrecedenceTargetEventList().size() + xOCcase.getResponseTargetEventList().size() + 2;
				stateId = precedenceNum + "-" + 1 + "-" + 0;
				System.out.println("stateId for (*,1,*): " + stateId);
    			enabledEdgeNumModel = stateIdVSEnabledEdgeModel.get(stateId);
    			enabledEdgeNumLog = stateIdVSEnabledEdgeLog.get(stateId);    			
    			segmentPrecison = segmentPrecison + enabledEdgeNumLog/enabledEdgeNumModel;
    		      		    			
    			int responseNum = 0;
//    			for(int responseNum = 1; responseNum <= xOCcase.getResponseTargetEventList().size(); responseNum ++){
    			for(XOCEvent event : xOCcase.getResponseTargetEventList()){
    				System.out.println("\n response event Id: " + event.getAttributes().get(xConstants.ID).toString());
    				responseNum ++;
            		stateId = precedenceNum + "-" + 1 + "-" + responseNum;
            		System.out.println("stateId response event: " + stateId);
        			enabledEdgeNumModel = stateIdVSEnabledEdgeModel.get(stateId);
        			enabledEdgeNumLog = stateIdVSEnabledEdgeLog.get(stateId);    			
        			segmentPrecison = segmentPrecison + enabledEdgeNumLog/enabledEdgeNumModel;
        		}
        		eventNumInSegment = eventNumInSegment + caseLength;
        	}
        	segmentPrecison = segmentPrecison/eventNumInSegment;
        	segmentIdVSPrecison.put(id, segmentPrecison);

		}
		//average the segment precision based on the case number in the segment
		double logPrecison = 0;
		double caseNumInLog = 0;
		for(String id : idVSSegment.keySet()){
			caseNumInLog = caseNumInLog + idVSSegment.get(id).getIdVSCase().size();
		}
		System.out.println("\n ************************");
		System.out.println("\n setment size: " + segmentIdVSPrecison.size());
		System.out.println("\n ************************");
		for(String id : segmentIdVSPrecison.keySet()){
			System.out.println("\n setment id: " + id);

			double segmentPrecision = segmentIdVSPrecison.get(id);
			System.out.println("\n setment precision: " + segmentPrecision);
			double caseNumInSegment = idVSSegment.get(id).getIdVSCase().size();
			logPrecison = logPrecison + segmentPrecision*(caseNumInSegment/caseNumInLog);
		}    
		return logPrecison;

	}  
	
	
	public void createConstraintVSPrecisionMap(XOCLog log, OCBCModel model){
		System.out.println("\n model size before: " + model.getAllActivityRelations().size());
		Map<ActivityEdge, OCBCModel> removedActivityRelationVSRemainModel = new HashMap<ActivityEdge, OCBCModel>();
		Map<ActivityEdge, Double> removedActivityRelationVSPrecision = new HashMap<ActivityEdge, Double>();
		for(ActivityEdge activityRelatin : model.getAllActivityRelations()){
			int index = model.getAllActivityRelations().indexOf(activityRelatin);
			OCBCModel modelClone = (OCBCModel) model.clone();
			modelClone.getAllActivityRelations().remove(index);
			removedActivityRelationVSRemainModel.put(activityRelatin, modelClone);
		}
		System.out.println("\n model size: " + model.getAllActivityRelations().size());
		for(ActivityEdge activityEdge : removedActivityRelationVSRemainModel.keySet()){
			String id = activityEdge.getID();
			double precision = computePrecision(log, removedActivityRelationVSRemainModel.get(activityEdge));
			removedActivityRelationVSPrecision.put(activityEdge, precision);
		}

		List<Double> precisionList = new ArrayList<Double>();
		
		for(ActivityEdge activityEdge : removedActivityRelationVSPrecision.keySet()){
			String id = activityEdge.getID();
			
			double aPrecision = removedActivityRelationVSPrecision.get(activityEdge);
			System.out.println("**********************************");
			System.out.println("id: " + id);
			System.out.println("aPrecision: " + aPrecision);
			precisionList.add(aPrecision);

		}
		Double[] precisionArray = new Double[precisionList.size()];
		int j=0;
		for(ActivityEdge activityEdge : removedActivityRelationVSPrecision.keySet()){
			String id = activityEdge.getID();
			
			double aPrecision = removedActivityRelationVSPrecision.get(activityEdge);
			System.out.println("**********************************");
			System.out.println("id: " + id);
			System.out.println("aPrecision: " + aPrecision);
			precisionList.add(aPrecision);
			precisionArray[j] = aPrecision;
			j++;

		}
		
		System.out.println("*********ranked*************************");
		Arrays.sort(precisionArray);
		for(int k=0; k<precisionArray.length; k++){
			System.out.println("aPrecision: " + precisionArray[k]);
		}
		
		//create a sorted map between activity edge and precision
		for(int i=0; i<removedActivityRelationVSPrecision.size(); i++){
			
			for(ActivityEdge activityEdge : removedActivityRelationVSPrecision.keySet()){
				double precisionCheck = removedActivityRelationVSPrecision.get(activityEdge);
				if(precisionCheck == precisionArray[i] && 
						!sortedRelationVSPrecision.containsKey(activityEdge)){
					System.out.println("add to map");
					sortedRelationVSPrecision.put(activityEdge, precisionArray[i]);
					sortedActivityList.add(activityEdge);
					break;
				}
			}
		}
		
		for(int i = sortedActivityList.size()-1; i>=0; i--){
			ActivityEdge activityEdge = sortedActivityList.get(i);
			decreaselySortedActivityList.add(activityEdge);
		}
		
		
		System.out.println("*********ranked map*************************");
		for(ActivityEdge activityEdge : decreaselySortedActivityList){
			String id = activityEdge.getID();	
			String referenceActivityName = activityEdge.getReferenceNode().getNodeName();	
			String targetActivityName = activityEdge.getTargetNode().getNodeName();	
			String cardinalityType = activityEdge.getCardinalityType();	
			double aPrecision = sortedRelationVSPrecision.get(activityEdge);
			System.out.println("**********************************");
			System.out.println("id: " + id);
			System.out.println("referenceActivityName: " + referenceActivityName);
			System.out.println("targetActivityName: " + targetActivityName);
			System.out.println("cardinalityType: " + cardinalityType);
			System.out.println("aPrecision: " + aPrecision);
		}
	}
	
	 public static void main(String[] args) throws Exception {
//		   FileInputStream input = new FileInputStream("D:\\uresponse.ocbcmml");
//		   FileInputStream input = new FileInputStream("D:\\nprecedence.ocbcmml");
//		   FileInputStream input = new FileInputStream("D:\\response.ocbcmml");
		   FileInputStream input = new FileInputStream("D:\\uprecedence.ocbcmml");
		   
//		   FileInputStream input = new FileInputStream("D:\\AUnit.ocbcmml");
//		   FileInputStream input = new FileInputStream("D:\\otc.ocbcmml");
//		   FileInputStream input = new FileInputStream("D:\\completeModel.ocbcmml");
			ByteArrayOutputStream baos = new ByteArrayOutputStream();
			byte[] buffer = new byte[1024];
			int len;
			while ((len = input.read(buffer)) > -1 ) {
			    baos.write(buffer, 0, len);
			}
			baos.flush();

			InputStream inputA = new ByteArrayInputStream(baos.toByteArray()); 
			InputStream inputB = new ByteArrayInputStream(baos.toByteArray()); 
			
	        Scanner scanner = new Scanner(inputA, "UTF-8");
	        String fileString = scanner.useDelimiter("\\A").next();
	        scanner.close();
	        
	        String versionString = fileString.split("\n")[0];
			String versionName = versionString.split("\"")[1];
			String removeVersion = fileString.substring(fileString.indexOf('\n')+1);
			System.out.println("model file: " + removeVersion);
			
			OCBCModel model = null;
			if(versionName.equals(OCBCEditorConstants.VERSION2_0)){
				mxGraph graphX = new mxGraph();
				Document document = mxXmlUtils.parseXml(removeVersion);
				
				mxCodec codec = new mxCodec(document);
				codec.decode(document.getDocumentElement(), graphX.getModel());
				
				System.out.println("version 2");
				TransformToOCBCSemanticModel transformer = new TransformToOCBCSemanticModel();
				transformer.transform(graphX);
				model = transformer.getModel();
				model.setGraphX(graphX);
				model.setModelFile(mxXmlUtils.parseXml(removeVersion));
				model.setModelVersion(OCBCEditorConstants.VERSION2_0);
			}
			else{
				
				ImportFromFile fileParse = new ImportFromFile();       
		        fileParse.parse(inputB);
		        model = fileParse.getModel();
		        model.setModelVersion(OCBCEditorConstants.VERSION1_0);
		        int i=0;
		        for(MyOriginalCell cell : model.getGraphCells()){
		        	i++;
		        	System.out.println("cell number" + i);
		        	if(cell instanceof MNodeCell){
		        	}
		        }
			}	
			
			XocXmlParser parser = new XocXmlParser();
			parser.parse(new File("d:\\log.xoc"));		
			XOCLog log = parser.getLog();
			
			ComputeBehavioralPrecision computeBehavioralPrecision = new ComputeBehavioralPrecision();
			double precision = computeBehavioralPrecision.computePrecision(log, model);
			
			System.out.println("\n precision: " + precision);
//			computeBehavioralPrecision.createConstraintVSPrecisionMap(log, model);
			
//			
//			System.out.println("\n model size before: " + model.getAllActivityRelations().size());
//			Map<ActivityEdge, OCBCModel> removedActivityRelationVSRemainModel = new HashMap<ActivityEdge, OCBCModel>();
//			Map<ActivityEdge, Double> removedActivityRelationVSPrecision = new HashMap<ActivityEdge, Double>();
//			for(ActivityEdge activityRelatin : model.getAllActivityRelations()){
//				int index = model.getAllActivityRelations().indexOf(activityRelatin);
//				OCBCModel modelClone = (OCBCModel) model.clone();
//				modelClone.getAllActivityRelations().remove(index);
//				removedActivityRelationVSRemainModel.put(activityRelatin, modelClone);
//			}
//			System.out.println("\n model size: " + model.getAllActivityRelations().size());
//			for(ActivityEdge activityEdge : removedActivityRelationVSRemainModel.keySet()){
//				String id = activityEdge.getID();
//				computeBehavioralPrecision = new ComputeBehavioralPrecision(log, removedActivityRelationVSRemainModel.get(activityEdge));
//				precision = computeBehavioralPrecision.computePrecision();
//				removedActivityRelationVSPrecision.put(activityEdge, precision);
//			}
//
//			List<Double> precisionList = new ArrayList<Double>();
//			
//			for(ActivityEdge activityEdge : removedActivityRelationVSPrecision.keySet()){
//				String id = activityEdge.getID();
//				
//				double aPrecision = removedActivityRelationVSPrecision.get(activityEdge);
//				System.out.println("**********************************");
//				System.out.println("id: " + id);
//				System.out.println("aPrecision: " + aPrecision);
//				precisionList.add(aPrecision);
//
//			}
//			Double[] precisionArray = new Double[precisionList.size()];
//			int j=0;
//			for(ActivityEdge activityEdge : removedActivityRelationVSPrecision.keySet()){
//				String id = activityEdge.getID();
//				
//				double aPrecision = removedActivityRelationVSPrecision.get(activityEdge);
//				System.out.println("**********************************");
//				System.out.println("id: " + id);
//				System.out.println("aPrecision: " + aPrecision);
//				precisionList.add(aPrecision);
//				precisionArray[j] = aPrecision;
//				j++;
//
//			}
//			
//			System.out.println("*********ranked*************************");
//			Arrays.sort(precisionArray);
//			for(int k=0; k<precisionArray.length; k++){
//				System.out.println("aPrecision: " + precisionArray[k]);
//			}
//			
//
//			Map<ActivityEdge, Double> sortedRelationVSPrecision = new HashMap<ActivityEdge, Double>();
//			List<ActivityEdge> sortedActivityList = new ArrayList<ActivityEdge>();
//			//create a sorted map between activity edge and precision
//			for(int i=0; i<removedActivityRelationVSPrecision.size(); i++){
//				
//				for(ActivityEdge activityEdge : removedActivityRelationVSPrecision.keySet()){
//					double precisionCheck = removedActivityRelationVSPrecision.get(activityEdge);
////					System.out.println("rank precision: " + precisionArray[i]);
////					System.out.println("get precision: " + precisionCheck);
//					if(precisionCheck == precisionArray[i] && 
//							!sortedRelationVSPrecision.containsKey(activityEdge)){
//						System.out.println("add to map");
//						sortedRelationVSPrecision.put(activityEdge, precisionArray[i]);
//						sortedActivityList.add(activityEdge);
//						break;
//					}
//				}
//			}
//			
//			System.out.println("*********ranked map*************************");
//			for(ActivityEdge activityEdge : sortedActivityList){
//				String id = activityEdge.getID();				
//				double aPrecision = sortedRelationVSPrecision.get(activityEdge);
//				System.out.println("**********************************");
//				System.out.println("id: " + id);
//				System.out.println("aPrecision: " + aPrecision);
//			}
			
	 }
}