package org.processmining.ocbc.discovery.metrics;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.processmining.mxgraph.model.mxCell;
import org.processmining.mxgraph.swing.mxGraphComponent;
import org.processmining.mxgraph.util.mxConstants;
import org.processmining.mxgraph.view.mxGraph;
import org.processmining.ocbc.log.model.XOCEvent;
import org.processmining.ocbc.log.model.XOCLog;
import org.processmining.ocbc.log.model.impl.xConstants;
import org.processmining.ocbc.models.ActivityEdge;
import org.processmining.ocbc.models.ClassNode;
import org.processmining.ocbc.models.OCBCModel;

public class ComputePrecisionThread extends Thread {
	
	
	
    private Thread t;
    private String threadName;
    private XOCLog log;
    private OCBCModel model;
	private mxGraphComponent graphComponent;
    private boolean finishFlag = false;
	protected Map<ActivityEdge, Double> sortedRelationVSPrecision = new HashMap<ActivityEdge, Double>();
	protected List<ActivityEdge> sortedActivityList = new ArrayList<ActivityEdge>();
	protected List<ActivityEdge> decreaselySortedActivityList = new ArrayList<ActivityEdge>();
//	public ComputeBehavioralPrecision(XOCLog log, OCBCModel model){
//		this.log = log;
//		this.model = model;
//	}
	
	public Map<ActivityEdge, Double> getSortedBehaviorConstraintVSPrecisionMap(){
		return sortedRelationVSPrecision;
	}
	
	public List<ActivityEdge> getIncreaselySortedActivityList(){
		return sortedActivityList;
	}
	
	public List<ActivityEdge> getDecreaselySortedActivityList(){
		return decreaselySortedActivityList;
	}
	
	public boolean getFinishFlag(){
		return finishFlag;
	}
	
    public ComputePrecisionThread( String name, XOCLog log, OCBCModel model, mxGraphComponent graphComponent) {
    	this.threadName = name;
    	this.log = log;
    	this.model = model;
    	this.graphComponent = graphComponent;
    	System.out.println("Creating " +  threadName );
    }
   
    public void run() {
    	System.out.println("Running " +  threadName );
    	createConstraintVSPrecisionMap(log, model);
//    	try {
//    		for(int i = 4; i > 0; i--) {
//    			System.out.println("Thread: " + threadName + ", " + i);
////            // 让线程睡醒一会
////            Thread.sleep(50);
//    			createConstraintVSPrecisionMap(log, model);
//    		}
//    	}catch (InterruptedException e) {
//    		System.out.println("Thread " +  threadName + " interrupted.");
//    	}
    	finishFlag = true;
    	mxGraph graph = graphComponent.getGraph();
    	for(mxCell cell : graph.getActivityRelationList()){
    		String cellId = cell.getId();
    		for(ActivityEdge activityRelation : sortedRelationVSPrecision.keySet()){
    			String activityRelationId = activityRelation.getID();
    			if(cellId.equals(activityRelationId)){
    				double precision = 1 - sortedRelationVSPrecision.get(activityRelation);
    				cell.getAttributes().put(mxConstants.precision, precision + "");
    				break;
    			}  			
    		}			
		}
    	System.out.println("Thread " +  threadName + " exiting.");
    }
   
    public void start () {
    	System.out.println("Starting " +  threadName );
    	if (t == null) {
    		t = new Thread (this, threadName);
    		t.start ();
      	}
    }
   
	public double computePrecision(XOCLog log, OCBCModel model){
		//build model unit family
		SearchOCBCUnitFamily searchOCBCUnitFamily = new SearchOCBCUnitFamily(model);
		searchOCBCUnitFamily.search();
        Map<String, OCBCUnitFamily> idVSUnitFamily = searchOCBCUnitFamily.getIdVSUnitFamily();

        ExtractXOCSegment extractXOCSegment = new ExtractXOCSegment(log, idVSUnitFamily);
		extractXOCSegment.extractAllSegment();
		Map<String, XOCSegment> idVSSegment = extractXOCSegment.getIdVSSegment();	
		Map<String, Double> segmentIdVSPrecison = new HashMap<String, Double>();
		
		for(String id : idVSUnitFamily.keySet()){
			OCBCUnitFamily family = idVSUnitFamily.get(id);
        	GenerateUnitFamilyAutomaton generateFamilyAutomaton = new GenerateUnitFamilyAutomaton(family);
        	generateFamilyAutomaton.generateAutomation();
        	Map<String, Integer> stateIdVSEnabledEdgeModel = generateFamilyAutomaton.getStateIdVSEnabledEdge(); 
        	
			XOCSegment segment = idVSSegment.get(id);			
        	GenerateSegmentAutomaton generateSegmentAutomaton = new GenerateSegmentAutomaton(segment);
        	generateSegmentAutomaton.generateAutomation();
        	
//        	Map<String, AutomatonState> idVSState = generateSegmentAutomaton.getIdVSState();
        	Map<String, Integer> stateIdVSEnabledEdgeLog = generateSegmentAutomaton.getStateIdVSEnabledEdge(); 
        	System.out.println("********************************************************");
        	System.out.println("getReferenceActivity: " + family.getReferenceActivity().getNodeName());
        	System.out.println("getTargetActivity: " + family.getTargetActivity().getNodeName());
        	System.out.println("type: " + family.getType());
			for(ClassNode classNode : family.getClassNodeList()){
				System.out.println("classNode.getNodeName(): " + classNode.getNodeName());				
			}
        	//compute the precision between an unit family and a segment based on Wil's rule
        	double segmentPrecison = 0;
        	int eventNumInSegment = 0;
        	for(String caseId : segment.getIdVSCase().keySet()){      		
        		XOCCase xOCcase = segment.getIdVSCase().get(caseId);
        		System.out.println("\n caseId: " + xOCcase.getId());
        		
        		
        		String stateId = 0 + "-" + 0 + "-" + 0;
    			double enabledEdgeNumModel = stateIdVSEnabledEdgeModel.get(stateId);
    			double enabledEdgeNumLog = stateIdVSEnabledEdgeLog.get(stateId);    			
    			segmentPrecison = segmentPrecison + enabledEdgeNumLog/enabledEdgeNumModel;
    			
    			int precedenceNum = 0;
    			for(XOCEvent event : xOCcase.getPrecedenceTargetEventList()){
        			System.out.println("\n precedence event Id: " + event.getAttributes().get(xConstants.ID).toString());
    				precedenceNum ++;
            		stateId = precedenceNum + "-" + 0 + "-" + 0;
        			enabledEdgeNumModel = stateIdVSEnabledEdgeModel.get(stateId);
        			enabledEdgeNumLog = stateIdVSEnabledEdgeLog.get(stateId);    			
        			segmentPrecison = segmentPrecison + enabledEdgeNumLog/enabledEdgeNumModel;
        		}   			
    			
//    			// the reference event is not the last event, without adding the enable edges of the last state 
//    			int caseLength = xOCcase.getPrecedenceTargetEventList().size() + xOCcase.getResponseTargetEventList().size() + 1;
//    			if(xOCcase.getResponseTargetEventList().size() > 0){
//    				stateId = precedenceNum + "-" + 1 + "-" + 0;
//        			enabledEdgeNumModel = stateIdVSEnabledEdgeModel.get(stateId);
//        			enabledEdgeNumLog = stateIdVSEnabledEdgeLog.get(stateId);    			
//        			segmentPrecison = segmentPrecison + enabledEdgeNumLog/enabledEdgeNumModel;
//    			}        		    			
//    			
//    			for(int responseNum = 1; responseNum < xOCcase.getResponseTargetEventList().size(); responseNum ++){
////    				System.out.println("\n response event Id: " + event.getAttributes().get(xConstants.ID).toString());
//            		stateId = precedenceNum + "-" + 1 + "-" + responseNum;
//        			enabledEdgeNumModel = stateIdVSEnabledEdgeModel.get(stateId);
//        			enabledEdgeNumLog = stateIdVSEnabledEdgeLog.get(stateId);    			
//        			segmentPrecison = segmentPrecison + enabledEdgeNumLog/enabledEdgeNumModel;
//        		}     
    			
    			//add the enable edges of the last state
    			int caseLength = xOCcase.getPrecedenceTargetEventList().size() + xOCcase.getResponseTargetEventList().size() + 2;
				stateId = precedenceNum + "-" + 1 + "-" + 0;
				System.out.println("stateId for (*,1,*): " + stateId);
    			enabledEdgeNumModel = stateIdVSEnabledEdgeModel.get(stateId);
    			enabledEdgeNumLog = stateIdVSEnabledEdgeLog.get(stateId);    			
    			segmentPrecison = segmentPrecison + enabledEdgeNumLog/enabledEdgeNumModel;
    		      		    			
    			int responseNum = 0;
//    			for(int responseNum = 1; responseNum <= xOCcase.getResponseTargetEventList().size(); responseNum ++){
    			for(XOCEvent event : xOCcase.getResponseTargetEventList()){
    				System.out.println("\n response event Id: " + event.getAttributes().get(xConstants.ID).toString());
    				responseNum ++;
            		stateId = precedenceNum + "-" + 1 + "-" + responseNum;
            		System.out.println("stateId response event: " + stateId);
        			enabledEdgeNumModel = stateIdVSEnabledEdgeModel.get(stateId);
        			enabledEdgeNumLog = stateIdVSEnabledEdgeLog.get(stateId);    			
        			segmentPrecison = segmentPrecison + enabledEdgeNumLog/enabledEdgeNumModel;
        		}
        		eventNumInSegment = eventNumInSegment + caseLength;
        	}
        	segmentPrecison = segmentPrecison/eventNumInSegment;
        	segmentIdVSPrecison.put(id, segmentPrecison);

		}
		//average the segment precision based on the case number in the segment
		double logPrecison = 0;
		double caseNumInLog = 0;
		for(String id : idVSSegment.keySet()){
			caseNumInLog = caseNumInLog + idVSSegment.get(id).getIdVSCase().size();
		}
		System.out.println("\n ************************");
		System.out.println("\n setment size: " + segmentIdVSPrecison.size());
		System.out.println("\n ************************");
		for(String id : segmentIdVSPrecison.keySet()){
			System.out.println("\n setment id: " + id);

			double segmentPrecision = segmentIdVSPrecison.get(id);
			System.out.println("\n setment precision: " + segmentPrecision);
			double caseNumInSegment = idVSSegment.get(id).getIdVSCase().size();
			logPrecison = logPrecison + segmentPrecision*(caseNumInSegment/caseNumInLog);
		}    
		return logPrecison;

	}  
   
	public void createConstraintVSPrecisionMap(XOCLog log, OCBCModel model){
		System.out.println("\n model size before: " + model.getAllActivityRelations().size());
		Map<ActivityEdge, OCBCModel> removedActivityRelationVSRemainModel = new HashMap<ActivityEdge, OCBCModel>();
		Map<ActivityEdge, Double> removedActivityRelationVSPrecision = new HashMap<ActivityEdge, Double>();
		for(ActivityEdge activityRelatin : model.getAllActivityRelations()){
			int index = model.getAllActivityRelations().indexOf(activityRelatin);
			OCBCModel modelClone = (OCBCModel) model.clone();
			modelClone.getAllActivityRelations().remove(index);
			removedActivityRelationVSRemainModel.put(activityRelatin, modelClone);
		}
		System.out.println("\n model size: " + model.getAllActivityRelations().size());
		for(ActivityEdge activityEdge : removedActivityRelationVSRemainModel.keySet()){
			String id = activityEdge.getID();
			double precision = computePrecision(log, removedActivityRelationVSRemainModel.get(activityEdge));
			removedActivityRelationVSPrecision.put(activityEdge, precision);
		}

		List<Double> precisionList = new ArrayList<Double>();
		
		for(ActivityEdge activityEdge : removedActivityRelationVSPrecision.keySet()){
			String id = activityEdge.getID();
			
			double aPrecision = removedActivityRelationVSPrecision.get(activityEdge);
			System.out.println("**********************************");
			System.out.println("id: " + id);
			System.out.println("aPrecision: " + aPrecision);
			precisionList.add(aPrecision);

		}
		Double[] precisionArray = new Double[precisionList.size()];
		int j=0;
		for(ActivityEdge activityEdge : removedActivityRelationVSPrecision.keySet()){
			String id = activityEdge.getID();
			
			double aPrecision = removedActivityRelationVSPrecision.get(activityEdge);
			System.out.println("**********************************");
			System.out.println("id: " + id);
			System.out.println("aPrecision: " + aPrecision);
			precisionList.add(aPrecision);
			precisionArray[j] = aPrecision;
			j++;

		}
		
		System.out.println("*********ranked*************************");
		Arrays.sort(precisionArray);
		for(int k=0; k<precisionArray.length; k++){
			System.out.println("aPrecision: " + precisionArray[k]);
		}
		
		//create a sorted map between activity edge and precision
		for(int i=0; i<removedActivityRelationVSPrecision.size(); i++){
			
			for(ActivityEdge activityEdge : removedActivityRelationVSPrecision.keySet()){
				double precisionCheck = removedActivityRelationVSPrecision.get(activityEdge);
				if(precisionCheck == precisionArray[i] && 
						!sortedRelationVSPrecision.containsKey(activityEdge)){
					System.out.println("add to map");
					sortedRelationVSPrecision.put(activityEdge, precisionArray[i]);
					sortedActivityList.add(activityEdge);
					break;
				}
			}
		}
		
		for(int i = sortedActivityList.size()-1; i>=0; i--){
			ActivityEdge activityEdge = sortedActivityList.get(i);
			decreaselySortedActivityList.add(activityEdge);
		}
		
		
		System.out.println("*********ranked map*************************");
		for(ActivityEdge activityEdge : decreaselySortedActivityList){
			String id = activityEdge.getID();	
			String referenceActivityName = activityEdge.getReferenceNode().getNodeName();	
			String targetActivityName = activityEdge.getTargetNode().getNodeName();	
			String cardinalityType = activityEdge.getCardinalityType();	
			double aPrecision = sortedRelationVSPrecision.get(activityEdge);
			System.out.println("**********************************");
			System.out.println("id: " + id);
			System.out.println("referenceActivityName: " + referenceActivityName);
			System.out.println("targetActivityName: " + targetActivityName);
			System.out.println("cardinalityType: " + cardinalityType);
			System.out.println("aPrecision: " + aPrecision);
		}
	}
	
	public static void main(String args[]) {
		
//	    ComputePrecisionThread T1 = new ComputePrecisionThread( "Thread-1");
//	    T1.start();
//	    
//	    ComputePrecisionThread T2 = new ComputePrecisionThread( "Thread-2");
//	    T2.start();
	}   
}
