package org.processmining.ocbc.discovery.metrics;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.FileInputStream;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Scanner;

import org.processmining.mxgraph.examples.swing.io.TransformToOCBCSemanticModel;
import org.processmining.mxgraph.io.mxCodec;
import org.processmining.mxgraph.util.mxXmlUtils;
import org.processmining.mxgraph.view.mxGraph;
import org.processmining.ocbc.log.model.XOCEvent;
import org.processmining.ocbc.log.model.XOCLog;
import org.processmining.ocbc.log.model.impl.xConstants;
import org.processmining.ocbc.models.OCBCModel;
import org.processmining.ocbc.models.editor.plugins.OCBCEditorConstants;
import org.processmining.ocbc.models.io.ImportFromFile;
import org.processmining.ocbc.models.visualization.cell.MNodeCell;
import org.processmining.ocbc.models.visualization.cell.MyOriginalCell;
import org.w3c.dom.Document;


public class ExtractXOCSegment {
	
	
	private XOCLog log;
	private Map<String, OCBCUnitFamily> idVSUnitFamily;
	private Map<String, XOCSegment> idVSSegment = new HashMap<String, XOCSegment>();
	
	public ExtractXOCSegment(XOCLog log, Map<String, OCBCUnitFamily> idVSUnitFamily){
		this.log = log;
		this.idVSUnitFamily = idVSUnitFamily;
	}
	
	public Map<String, XOCSegment> getIdVSSegment(){
		return idVSSegment;		
	}
	
	public void extractAllSegment(){
		for(String familyId : idVSUnitFamily.keySet()){
			OCBCUnitFamily family = idVSUnitFamily.get(familyId);
			XOCSegment segment = extractOneSegment(family);
			idVSSegment.put(segment.getId(), segment);
		}
		simplifyAllSegment();
	}
	
	public void simplifyAllSegment(){
		for(String segmentId : idVSSegment.keySet()){
			XOCSegment segment = idVSSegment.get(segmentId);
			for(String caseId : segment.getIdVSCase().keySet()){
				XOCCase xOCCase = segment.getIdVSCase().get(caseId);
				//limit the number of target events in the list up to two
				List<XOCEvent> eventList = new ArrayList<XOCEvent>();
				if(xOCCase.getPrecedenceTargetEventList().size() > 2){
					eventList.add(xOCCase.getPrecedenceTargetEventList().get(0));
					eventList.add(xOCCase.getPrecedenceTargetEventList().get(1));
					xOCCase.getPrecedenceTargetEventList().clear();
					xOCCase.getPrecedenceTargetEventList().addAll(eventList);
				}				
				eventList = new ArrayList<XOCEvent>();
				if(xOCCase.getResponseTargetEventList().size() > 2){
					eventList.add(xOCCase.getResponseTargetEventList().get(0));
					eventList.add(xOCCase.getResponseTargetEventList().get(1));
					xOCCase.getResponseTargetEventList().clear();
					xOCCase.getResponseTargetEventList().addAll(eventList);
				}				
			}
		}
	}
	
	public XOCSegment extractOneSegment(OCBCUnitFamily family){
		String referenceActivity = family.getReferenceActivity().getNodeName();
		String targetActivity = family.getTargetActivity().getNodeName();
		int type = family.getType();
		
		XOCSegment segment = new XOCSegment();
		segment.setId(family.getId());
		segment.setReferenceActivityName(referenceActivity);
		segment.setTargetActivityName(targetActivity);
		segment.setType(type);
		
		for(XOCEvent referenceEvent : log.getEventsOfActivity(referenceActivity)){
			String referenceEventId = referenceEvent.getAttributes().get(xConstants.ID).toString();
			XOCCase xOCcase = new XOCCase();
			xOCcase.setId(referenceEventId);
			xOCcase.setReferenceActivityName(referenceActivity);
			xOCcase.setTargetActivityName(targetActivity);
			xOCcase.setType(type);
			//get all related event of reference event
			if(type == 0){
				String classNodeName = family.getClassNodeList().get(0).getNodeName();
				segment.getClassNodeList().add(family.getClassNodeList().get(0));
				for(XOCEvent targetEvent : log.getRelatedEventsForOneEventA(referenceEvent, classNodeName)){
					String targetEventActivity = targetEvent.getAttributes().get(xConstants.ACTIVITY).toString();
					//if the related event has the target activity
					if(targetEventActivity.equals(targetActivity)){
						//if the related event before the reference
						if(log.getAllEvents().indexOf(targetEvent) < log.getAllEvents().indexOf(referenceEvent)){
							xOCcase.getPrecedenceTargetEventList().add(targetEvent);
						}
						else{
							xOCcase.getResponseTargetEventList().add(targetEvent);
						}
					}				
				}	
			}
			
			else if(type == 1){
				String classNodeNameX = family.getClassNodeList().get(0).getNodeName();
				String classNodeNameY = family.getClassNodeList().get(1).getNodeName();
				segment.getClassNodeList().add(family.getClassNodeList().get(0));
				segment.getClassNodeList().add(family.getClassNodeList().get(1));
				System.out.println("classNodeNameX: " + classNodeNameX);
				System.out.println("classNodeNameY: " + classNodeNameY);
				System.out.println("referenceEvent type =1: " + referenceEvent.getAttributes().get(xConstants.ACTIVITY).toString());
//				System.out.println("");
				for(XOCEvent targetEvent : log.getRelatedEventsForOneEventB(referenceEvent, classNodeNameX, classNodeNameY)){
					
					String targetEventActivity = targetEvent.getAttributes().get(xConstants.ACTIVITY).toString();
					System.out.println("targetEventActivity type =1: " + targetEventActivity);
					//if the related event has the target activity
					if(targetEventActivity.equals(targetActivity)){
						//if the related event before the reference
						if(log.getAllEvents().indexOf(targetEvent) < log.getAllEvents().indexOf(referenceEvent)){
							xOCcase.getPrecedenceTargetEventList().add(targetEvent);
						}
						else{
							xOCcase.getResponseTargetEventList().add(targetEvent);
						}
					}				
				}
//				System.out.println("xOCcase id");
			}
			System.out.println("xOCcase id: " + xOCcase.getId());
			System.out.println("getTargetActivityName: " + xOCcase.getTargetActivityName());
			System.out.println("getPrecedenceTargetEventList: " + xOCcase.getPrecedenceTargetEventList().size());
			System.out.println("getResponseTargetEventList: " + xOCcase.getResponseTargetEventList().size());
			
			segment.getIdVSCase().put(referenceEventId, xOCcase);
		}
		return segment;
	}
	
    public static void main(String[] args) throws Exception {
		   FileInputStream input = new FileInputStream("D:\\AUnit.ocbcmml");
//			InputStream inputStream = new FileInputStream("d:/123.ocbcmml");
			
			ByteArrayOutputStream baos = new ByteArrayOutputStream();
			byte[] buffer = new byte[1024];
			int len;
			while ((len = input.read(buffer)) > -1 ) {
			    baos.write(buffer, 0, len);
			}
			baos.flush();

			// Open new InputStreams using the recorded bytes
			// Can be repeated as many times as you wish
			InputStream inputA = new ByteArrayInputStream(baos.toByteArray()); 
			InputStream inputB = new ByteArrayInputStream(baos.toByteArray()); 
			
	        Scanner scanner = new Scanner(inputA, "UTF-8");
	        String fileString = scanner.useDelimiter("\\A").next();
	        scanner.close();
	        
	        String versionString = fileString.split("\n")[0];
			String versionName = versionString.split("\"")[1];
			String removeVersion = fileString.substring(fileString.indexOf('\n')+1);
			System.out.println("model file: " + removeVersion);
			
			OCBCModel model = null;
			if(versionName.equals(OCBCEditorConstants.VERSION2_0)){
				mxGraph graphX = new mxGraph();
				Document document = mxXmlUtils.parseXml(removeVersion);
				
				mxCodec codec = new mxCodec(document);
				codec.decode(document.getDocumentElement(), graphX.getModel());
				
				System.out.println("version 2");
				TransformToOCBCSemanticModel transformer = new TransformToOCBCSemanticModel();
				transformer.transform(graphX);
				model = transformer.getModel();
				model.setGraphX(graphX);
				model.setModelFile(mxXmlUtils.parseXml(removeVersion));
				model.setModelVersion(OCBCEditorConstants.VERSION2_0);
			}
			else{
				
				ImportFromFile fileParse = new ImportFromFile();       
		        fileParse.parse(inputB);
		        model = fileParse.getModel();
		        model.setModelVersion(OCBCEditorConstants.VERSION1_0);
		        int i=0;
		        for(MyOriginalCell cell : model.getGraphCells()){
		        	i++;
		        	System.out.println("cell number" + i);
		        	if(cell instanceof MNodeCell){
		 //       		System.out.println("cell name" + cell.getUserObject().toString());
		 //       		System.out.println("number of related edges" + model.getRelatedEdgeCellsForOneCell(cell).size());
		        	}
		        }
			}			
	      
//	        for(ActivityEdge activityEdge : model.getAllActivityRelations()){
//	        	System.out.println("\n activityEdge id: " + activityEdge.getID());
//	        	System.out.println("\n activityEdge name: " + activityEdge.getCardinalityType());
//	        }   
//	        
//	        for(CrelRTEdge edge : model.getAllCrelRTRelations()){
//	        	System.out.println("\n CrelRTEdge id: " + edge.getID());
//	        	System.out.println("\n CrelRTEdge name: " + edge.getRelatedActivityRelation().getID());
//
//	        }
//	        
//	        for(CrelOCEdge edge : model.getAllCrelOCRelations()){
//	        	System.out.println("\n CrelOCEdge id: " + edge.getID());
//	        	System.out.println("\n CrelOCEdge name: " + edge.getRelatedActivityRelation().getID());
//
//	        }

	        
//	        for(OCBCUnitFamily family : familyList){
//	        	
//	        }
    }
}