package org.processmining.ocbc.discovery.metrics;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Scanner;

import org.processmining.mxgraph.examples.swing.io.TransformToOCBCSemanticModel;
import org.processmining.mxgraph.io.mxCodec;
import org.processmining.mxgraph.util.mxXmlUtils;
import org.processmining.mxgraph.view.mxGraph;
import org.processmining.ocbc.log.io.XocXmlParser;
import org.processmining.ocbc.log.model.XOCEvent;
import org.processmining.ocbc.log.model.XOCLog;
import org.processmining.ocbc.log.model.impl.xConstants;
import org.processmining.ocbc.models.OCBCModel;
import org.processmining.ocbc.models.editor.plugins.OCBCEditorConstants;
import org.processmining.ocbc.models.io.ImportFromFile;
import org.processmining.ocbc.models.visualization.cell.MNodeCell;
import org.processmining.ocbc.models.visualization.cell.MyOriginalCell;
import org.w3c.dom.Document;


public class GenerateSegmentAutomaton {
	protected String id;
	protected XOCSegment segment;
	
	protected List<String> possibleStateIdList = new ArrayList<String>();
	protected List<String> endStateIdList = new ArrayList<String>();
	
	protected Map<String, AutomatonState> idVSState = new HashMap<String, AutomatonState>();
	protected Map<String, Integer> stateIdVSEnabledEdge = new HashMap<String, Integer>();
	
	public GenerateSegmentAutomaton(XOCSegment segment){
		this.segment = segment;
	}
	
	public void setId(String id){
		this.id = id;
	}
	
	public String getId(){
		return id;
	}
	
	public List<String> getPossibleStateIdList(){
		return possibleStateIdList;
	}
	
	public List<String> getEndStateIdList(){
		return endStateIdList;
	}
	
	public Map<String, Integer> getStateIdVSEnabledEdge(){
		return stateIdVSEnabledEdge;
	}
	
	public Map<String, AutomatonState> getIdVSState(){
		return idVSState;
	}
	
	public void generateAutomation(){
		generateAllStates();
		//generate possible states
//		possibleStateIdList = new ArrayList<String>(idVSState.keySet());
//			
//		filterStatesAboveTwo(possibleStateIdList, cardinalityType);


		//generate possible edges for each state
		for(String stateId : idVSState.keySet()){
			int possibleEdgeNum = 0;
			AutomatonState state = idVSState.get(stateId);
			for(String stateNextId : idVSState.keySet()){
				AutomatonState stateNext = idVSState.get(stateNextId);
				if(isNextState(state,stateNext)){
					possibleEdgeNum ++;
				}
			}
			//if the state is an end state, plus one (ending can be considered as an edge)
			if(endStateIdList.contains(stateId)){
				possibleEdgeNum ++;
			}
			stateIdVSEnabledEdge.put(stateId, possibleEdgeNum);		
		}
	}
	
//	public void filterStatesAboveTwo(List<String> possibleStateIdList){
//		for(String stateId : possibleStateIdList){
//			AutomatonState state = idVSState.get(stateId);
//			if(){
//				
//			}
//		}
//	}
	
	public boolean isNextState(AutomatonState state, AutomatonState nextState){
		boolean flag = false;
		int beforeEventNum = state.getPrecedenceTargetEventNum();
		int referenceEventNum = state.getReferenceEventNum();
		int afterEventNum = state.getResponseTargetEventNum();		
		
		int beforeEventNumNext = nextState.getPrecedenceTargetEventNum();
		int referenceEventNumNext = nextState.getReferenceEventNum();
		int afterEventNumNext = nextState.getResponseTargetEventNum();		
		
		if(referenceEventNum == 0 && referenceEventNumNext == 0){
			if(beforeEventNumNext == beforeEventNum + 1){
				flag = true;
			}
		}
		else if(referenceEventNum == 1 && referenceEventNumNext == 1 && beforeEventNum == beforeEventNumNext){
			if(afterEventNumNext == afterEventNum + 1){
				flag = true;
			}
		}
		else if(referenceEventNum == 0 && referenceEventNumNext == 1 && beforeEventNum == beforeEventNumNext &&
				afterEventNum == 0 && afterEventNumNext == 0){
			flag = true;
		}
		return flag;
	}
	
	public void generateAllStates(){
		for(String caseId : segment.getIdVSCase().keySet()){
			XOCCase xOCCase = segment.getIdVSCase().get(caseId);
			
			AutomatonState state = new AutomatonState();
			state.setPrecedenceTargetEventNum(0);
			state.setReferenceEventNum(0);
			state.setResponseTargetEventNum(0);
			String stateId = 0 + "-" + 0 + "-" + 0;
			idVSState.put(stateId, state);
			
			int num = 0;
			for(XOCEvent precedenceEvent : xOCCase.getPrecedenceTargetEventList()){
				num ++;
				state = new AutomatonState();
				state.setPrecedenceTargetEventNum(num);
				state.setReferenceEventNum(0);
				state.setResponseTargetEventNum(0);
				stateId = num + "-" + 0 + "-" + 0;
				idVSState.put(stateId, state);
			}				
			
			state = new AutomatonState();
			state.setPrecedenceTargetEventNum(num);
			state.setReferenceEventNum(1);
			state.setResponseTargetEventNum(0);
			stateId = num + "-" + 1 + "-" + 0;
			idVSState.put(stateId, state);
			
			int numResponse = 0;
			for(XOCEvent responseEvent : xOCCase.getResponseTargetEventList()){
				numResponse ++;
				state = new AutomatonState();
				state.setPrecedenceTargetEventNum(num);
				state.setReferenceEventNum(1);
				state.setResponseTargetEventNum(numResponse);
				stateId = num + "-" + 1 + "-" + numResponse;
				idVSState.put(stateId, state);
			}	
			//add the end state when the case finishes
			if(!endStateIdList.contains(stateId)){
				endStateIdList.add(stateId);		
			}			
		}				
	}


    public static void main(String[] args) throws Exception {
		   FileInputStream input = new FileInputStream("D:\\BUnit.ocbcmml");
//			InputStream inputStream = new FileInputStream("d:/123.ocbcmml");
			
			ByteArrayOutputStream baos = new ByteArrayOutputStream();
			byte[] buffer = new byte[1024];
			int len;
			while ((len = input.read(buffer)) > -1 ) {
			    baos.write(buffer, 0, len);
			}
			baos.flush();

			// Open new InputStreams using the recorded bytes
			// Can be repeated as many times as you wish
			InputStream inputA = new ByteArrayInputStream(baos.toByteArray()); 
			InputStream inputB = new ByteArrayInputStream(baos.toByteArray()); 
			
	        Scanner scanner = new Scanner(inputA, "UTF-8");
	        String fileString = scanner.useDelimiter("\\A").next();
	        scanner.close();
	        
	        String versionString = fileString.split("\n")[0];
			String versionName = versionString.split("\"")[1];
			String removeVersion = fileString.substring(fileString.indexOf('\n')+1);
			System.out.println("model file: " + removeVersion);
			
			OCBCModel model = null;
			if(versionName.equals(OCBCEditorConstants.VERSION2_0)){
				mxGraph graphX = new mxGraph();
				Document document = mxXmlUtils.parseXml(removeVersion);
				
				mxCodec codec = new mxCodec(document);
				codec.decode(document.getDocumentElement(), graphX.getModel());
				
				System.out.println("version 2");
				TransformToOCBCSemanticModel transformer = new TransformToOCBCSemanticModel();
				transformer.transform(graphX);
				model = transformer.getModel();
				model.setGraphX(graphX);
				model.setModelFile(mxXmlUtils.parseXml(removeVersion));
				model.setModelVersion(OCBCEditorConstants.VERSION2_0);
			}
			else{
				
				ImportFromFile fileParse = new ImportFromFile();       
		        fileParse.parse(inputB);
		        model = fileParse.getModel();
		        model.setModelVersion(OCBCEditorConstants.VERSION1_0);
		        int i=0;
		        for(MyOriginalCell cell : model.getGraphCells()){
		        	i++;
		        	System.out.println("cell number" + i);
		        	if(cell instanceof MNodeCell){
		        	}
		        }
			}			

	        SearchOCBCUnitFamily searchOCBCUnitFamily = new SearchOCBCUnitFamily(model);
	        Map<String, OCBCUnitFamily> idVSUnitFamily = searchOCBCUnitFamily.getIdVSUnitFamily();
	        List<OCBCUnitFamily> familyList = searchOCBCUnitFamily.search();
	        
	        System.out.println("\n ***********************");
	        System.out.println("\n family size: " + idVSUnitFamily.size());
	        
	        for(String familyId : idVSUnitFamily.keySet()){
	        	System.out.println("\n ***********************");
	        	OCBCUnitFamily family = idVSUnitFamily.get(familyId);
	        	System.out.println("\n familyId: " + familyId);
	        	System.out.println("\n family reference id: " + family.getReferenceActivity().getID());
	        	System.out.println("\n family reference name: " + family.getReferenceActivity().getNodeName());
	        	System.out.println("\n family target id: " + family.getTargetActivity().getID());
	        	System.out.println("\n family target name: " + family.getTargetActivity().getNodeName());
	        	System.out.println("\n family type: " + family.getType());
	        	for(String unitId : family.getIdVSUnit().keySet()){
	        		OCBCUnit unit = family.getIdVSUnit().get(unitId);
	        		System.out.println("\n unitId: " + unit.getId());
	        		System.out.println("\n getCardinalityType: " + unit.getActivityRelation().getCardinalityType());
	        	}

//	        	GenerateSegmentAutomaton generateUnitFamilyAutomaton = new GenerateSegmentAutomaton(null);
//	        	generateUnitFamilyAutomaton.generateAutomation();
//	        	Map<String, AutomatonState> idVSState = generateUnitFamilyAutomaton.getIdVSState();
//	        	List<String> possibleStateIdList = generateUnitFamilyAutomaton.getPossibleStateIdList();
//	        	List<String> endStateIdList = generateUnitFamilyAutomaton.getEndStateIdList();
//	        	Map<String, Integer> stateIdVSEnabledEdge = generateUnitFamilyAutomaton.getStateIdVSEnabledEdge();
//	        	
//	        	System.out.println("*****************all StateIdList id size: " + idVSState.size() + "************");
//	        	for(String id : idVSState.keySet()){
//	        		System.out.println("id: " + id);
//	        	}  	
//	        	System.out.println("*****************possible StateIdList id size: " + possibleStateIdList.size() + "************");
//	        	for(String id : possibleStateIdList){
//	        		System.out.println("possibleStateIdList id: " + id);
//	        	}
//	        	System.out.println("*****************end StateIdList id size: " + endStateIdList.size() + "************");
//	        	for(String id : endStateIdList){
//	        		System.out.println("endStateIdList id: " + id);
//	        	}
//	        	System.out.println("*****************stateIdVSEnabledEdge size: " + stateIdVSEnabledEdge.size() + "************");
//	        	for(String id : stateIdVSEnabledEdge.keySet()){
//	        		System.out.println("state id: " + id);
//	        		System.out.println("possible edge num: " + stateIdVSEnabledEdge.get(id));
//	        	}
	        }
	        
	        XocXmlParser parser = new XocXmlParser();
			parser.parse(new File("d:\\log.xoc"));		
			XOCLog log = parser.getLog();
			
			ExtractXOCSegment extractXOCSegment = new ExtractXOCSegment(log, idVSUnitFamily);
			extractXOCSegment.extractAllSegment();
			Map<String, XOCSegment> idVSSegment = extractXOCSegment.getIdVSSegment();
			System.out.println("\n ***********************");
		    System.out.println("\n segment size: " + idVSSegment.size());
	        for(String segmentId : idVSSegment.keySet()){
	        	System.out.println("\n ***********************");
	        	XOCSegment segment = idVSSegment.get(segmentId);
	        	System.out.println("\n segmentId: " + segmentId);
	        	System.out.println("\n segment reference name: " + segment.getReferenceActivityName());
	        	System.out.println("\n segment target name: " + segment.getTargetActivityName());
	        	System.out.println("\n segment type: " + segment.getType());
	        	for(String caseId : segment.getIdVSCase().keySet()){
	        		XOCCase xOCcase = segment.getIdVSCase().get(caseId);
	        		System.out.println("\n caseId: " + xOCcase.getId());
	        		for(XOCEvent event : xOCcase.getPrecedenceTargetEventList()){
	        			System.out.println("\n precedence event Id: " + event.getAttributes().get(xConstants.ID).toString());
	        		}
	        		for(XOCEvent event : xOCcase.getResponseTargetEventList()){
	        			System.out.println("\n response event Id: " + event.getAttributes().get(xConstants.ID).toString());
	        		}
	        	}
	        	
	        	System.out.println("\n ********segment automaton***************");
	        	GenerateSegmentAutomaton generateSegmentAutomaton = new GenerateSegmentAutomaton(segment);
	        	generateSegmentAutomaton.generateAutomation();
	        	Map<String, AutomatonState> idVSState = generateSegmentAutomaton.getIdVSState();
//	        	List<String> possibleStateIdList = generateSegmentAutomaton.getPossibleStateIdList();
	        	List<String> endStateIdList = generateSegmentAutomaton.getEndStateIdList();
	        	Map<String, Integer> stateIdVSEnabledEdge = generateSegmentAutomaton.getStateIdVSEnabledEdge();
	        	
	        	System.out.println("*****************all StateIdList id size: " + idVSState.size() + "************");
	        	for(String id : idVSState.keySet()){
	        		System.out.println("id: " + id);
	        	}  	
	        	System.out.println("*****************end StateIdList id size: " + endStateIdList.size() + "************");
	        	for(String id : endStateIdList){
	        		System.out.println("endStateIdList id: " + id);
	        	}
	        	System.out.println("*****************stateIdVSEnabledEdge size: " + stateIdVSEnabledEdge.size() + "************");
	        	for(String id : stateIdVSEnabledEdge.keySet()){
	        		System.out.println("state id: " + id);
	        		System.out.println("possible edge num: " + stateIdVSEnabledEdge.get(id));
	        	}
	        }
    	
    }
}