package org.processmining.ocbc.discovery.metrics;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.FileInputStream;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Scanner;

import org.processmining.mxgraph.examples.swing.io.TransformToOCBCSemanticModel;
import org.processmining.mxgraph.io.mxCodec;
import org.processmining.mxgraph.util.mxConstants;
import org.processmining.mxgraph.util.mxXmlUtils;
import org.processmining.mxgraph.view.mxGraph;
import org.processmining.ocbc.models.AOCEdge;
import org.processmining.ocbc.models.ActivityEdge;
import org.processmining.ocbc.models.OCBCModel;
import org.processmining.ocbc.models.editor.plugins.OCBCEditorConstants;
import org.processmining.ocbc.models.impl.ActivityEdgeImpl;
import org.processmining.ocbc.models.io.ImportFromFile;
import org.processmining.ocbc.models.visualization.cell.MNodeCell;
import org.processmining.ocbc.models.visualization.cell.MyOriginalCell;
import org.w3c.dom.Document;


public class SearchOCBCUnitFamily {
	private OCBCModel model;
	private Map<String, OCBCUnitFamily> idVSUnitFamily = new HashMap<String, OCBCUnitFamily>();
	
	public SearchOCBCUnitFamily(OCBCModel model){
		this.model = model;
	}
	
	public Map<String, OCBCUnitFamily> getIdVSUnitFamily(){
		return idVSUnitFamily;		
	}
	
	public List<OCBCUnitFamily> search(){
		List<OCBCUnitFamily> unitFamilyList = new ArrayList<OCBCUnitFamily>();
		//discover activity edges-get the source (reference) activity
		for(AOCEdge sourceAOCEdge  : model.getAllAOCRelations()){
			String sourceClassNodeId = sourceAOCEdge.getClassNode().getID().toString();	
			String sourceClassName = sourceAOCEdge.getClassNode().getNodeName();
			String sourceActivityName = sourceAOCEdge.getActivityNode().getNodeName();
			String sourceActivityNodeId = sourceAOCEdge.getActivityNode().getID().toString();			
			System.out.printf("AOC relation className" + sourceClassNodeId);			
			
			//get the target activity
			for(AOCEdge targetAOCEdge  : model.getAllAOCRelations()){
				String targetClassNodeId = targetAOCEdge.getClassNode().getID().toString();
				String targetClassName = targetAOCEdge.getClassNode().getNodeName();
				String targetActivityName = targetAOCEdge.getActivityNode().getNodeName();
				String targetActivityNodeId = targetAOCEdge.getActivityNode().getID().toString();
				
				System.out.println("");
				System.out.println("sourceActivityName: " + sourceActivityName);
				System.out.println("targetActivityName: " + targetActivityName);
				
				//source AOC relation and AOC relation are not the same AOC relation
				if(!sourceActivityNodeId.equals(targetActivityNodeId)){
					//and they contains the same class
					if(sourceClassNodeId.equals(targetClassNodeId)){
						System.out.println("AOC relation sourceClassName: " + sourceClassName);
						System.out.println("AOC relation targetClassName: " + targetClassName);
						//if there are no corresponding activity relations
						if(model.getActivityRelationFamilyA(sourceActivityNodeId, targetActivityNodeId, sourceClassNodeId).size() == 0){
							String familyId = sourceActivityNodeId +  "-" + targetActivityNodeId + "-" + sourceClassNodeId;
							idVSUnitFamily.put(familyId, new OCBCUnitFamily());
							idVSUnitFamily.get(familyId).setId(familyId);
							idVSUnitFamily.get(familyId).setReferenceActivity(sourceAOCEdge.getActivityNode());
							idVSUnitFamily.get(familyId).setTargetActivity(targetAOCEdge.getActivityNode());
							idVSUnitFamily.get(familyId).setType(0);
							idVSUnitFamily.get(familyId).getClassNodeList().add(sourceAOCEdge.getClassNode());
							
							ActivityEdge dummyActivityEdge = new ActivityEdgeImpl(sourceAOCEdge.getActivityNode(), targetAOCEdge.getActivityNode(),
									mxConstants.NoConstraints);
							String unitId = sourceActivityNodeId + "-" + mxConstants.NoConstraints + "-" + targetActivityNodeId + "-" + sourceClassNodeId;	
							OCBCUnit unit = new OCBCUnit();		
							unit.setId(unitId);
							unit.setReferenceActivity(sourceAOCEdge.getActivityNode());
							unit.setTargetActivity(targetAOCEdge.getActivityNode());
							unit.setType(0);
							unit.setActivityRelation(dummyActivityEdge);
							unit.getClassNodeList().add(sourceAOCEdge.getClassNode());
							unit.getAOCRelationList().add(sourceAOCEdge);
							unit.getAOCRelationList().add(targetAOCEdge);
							
							idVSUnitFamily.get(familyId).getIdVSUnit().put(unitId, unit);
						}
						else{
							for(ActivityEdge activityRelation : model.getActivityRelationFamilyA(sourceActivityNodeId, targetActivityNodeId, sourceClassNodeId)){
								String activityRelationId = activityRelation.getID();
								String familyId = sourceActivityNodeId +  "-" + targetActivityNodeId + "-" + sourceClassNodeId;
								String unitId = sourceActivityNodeId + "-" + activityRelationId + "-" + targetActivityNodeId + "-" + sourceClassNodeId;							
								System.out.println("familyId: " + familyId);
								if(!idVSUnitFamily.containsKey(familyId)){
									System.out.println("add family");
									idVSUnitFamily.put(familyId, new OCBCUnitFamily());
									idVSUnitFamily.get(familyId).setId(familyId);
									idVSUnitFamily.get(familyId).setReferenceActivity(sourceAOCEdge.getActivityNode());
									idVSUnitFamily.get(familyId).setTargetActivity(targetAOCEdge.getActivityNode());
									idVSUnitFamily.get(familyId).setType(0);
									idVSUnitFamily.get(familyId).getClassNodeList().add(sourceAOCEdge.getClassNode());
//									idVSUnitFamily.setClassRelationId(classRelationId);
								}

								OCBCUnitFamily unitFamily = idVSUnitFamily.get(familyId);
								if(!unitFamily.getIdVSUnit().containsKey(unitId)){
									//build the a unit
									OCBCUnit unit = new OCBCUnit();		
									unit.setId(unitId);
									unit.setReferenceActivity(sourceAOCEdge.getActivityNode());
									unit.setTargetActivity(targetAOCEdge.getActivityNode());
									unit.setType(0);
									unit.setActivityRelation(activityRelation);
									unit.getClassNodeList().add(sourceAOCEdge.getClassNode());
									unit.getAOCRelationList().add(sourceAOCEdge);
									unit.getAOCRelationList().add(targetAOCEdge);
									
									unitFamily.getIdVSUnit().put(unitId, unit);
								}
							}	
						}					
					}
					
					//and they contains two related classes
					else if(model.checkTwoClassesIfRelated(sourceClassNodeId, targetClassNodeId) != null){	
						System.out.printf("\n enter checkTwoClassesIfRelated");
						String classRelationId = model.checkTwoClassesIfRelated(sourceClassNodeId, targetClassNodeId);
						//if there are no corresponding activity relations
						if(model.getActivityRelationFamilyB(sourceActivityNodeId, targetActivityNodeId, classRelationId).size() == 0){
							String familyId = sourceActivityNodeId +  "-" + targetActivityNodeId + "-" + classRelationId;
							idVSUnitFamily.put(familyId, new OCBCUnitFamily());
							idVSUnitFamily.get(familyId).setId(familyId);
							idVSUnitFamily.get(familyId).setReferenceActivity(sourceAOCEdge.getActivityNode());
							idVSUnitFamily.get(familyId).setTargetActivity(targetAOCEdge.getActivityNode());
							idVSUnitFamily.get(familyId).setType(1);
							
							idVSUnitFamily.get(familyId).getClassNodeList().add(sourceAOCEdge.getClassNode());
							idVSUnitFamily.get(familyId).getClassNodeList().add(targetAOCEdge.getClassNode());
							
							idVSUnitFamily.get(familyId).setClassRelationId(classRelationId);
							
							
							ActivityEdge dummyActivityEdge = new ActivityEdgeImpl(sourceAOCEdge.getActivityNode(), targetAOCEdge.getActivityNode(),
									mxConstants.NoConstraints);
							String unitId = sourceActivityNodeId + "-" + mxConstants.NoConstraints + "-" + targetActivityNodeId + "-" + sourceClassNodeId;	
							OCBCUnit unit = new OCBCUnit();		
							unit.setId(unitId);
							unit.setReferenceActivity(sourceAOCEdge.getActivityNode());
							unit.setTargetActivity(targetAOCEdge.getActivityNode());
							unit.setType(1);
							unit.setActivityRelation(dummyActivityEdge);
							unit.setClassRelationId(classRelationId);
							unit.getClassNodeList().add(sourceAOCEdge.getClassNode());
							unit.getClassNodeList().add(targetAOCEdge.getClassNode());
							unit.getAOCRelationList().add(sourceAOCEdge);
							unit.getAOCRelationList().add(targetAOCEdge);
							
							idVSUnitFamily.get(familyId).getIdVSUnit().put(unitId, unit);
						}
						else{
							for(ActivityEdge activityRelation : model.getActivityRelationFamilyB(sourceActivityNodeId, targetActivityNodeId, classRelationId)){
								String activityRelationId = activityRelation.getID();
								String familyId = sourceActivityNodeId +  "-" + targetActivityNodeId + "-" + classRelationId;
								String unitId = sourceActivityNodeId + "-" + activityRelationId + "-" + targetActivityNodeId + "-" + classRelationId;							
								
								if(!idVSUnitFamily.containsKey(familyId)){
									idVSUnitFamily.put(familyId, new OCBCUnitFamily());
									idVSUnitFamily.get(familyId).setId(familyId);
									idVSUnitFamily.get(familyId).setReferenceActivity(sourceAOCEdge.getActivityNode());
									idVSUnitFamily.get(familyId).setTargetActivity(targetAOCEdge.getActivityNode());
									idVSUnitFamily.get(familyId).setType(1);
									
									idVSUnitFamily.get(familyId).getClassNodeList().add(sourceAOCEdge.getClassNode());
									idVSUnitFamily.get(familyId).getClassNodeList().add(targetAOCEdge.getClassNode());
									
									idVSUnitFamily.get(familyId).setClassRelationId(classRelationId);
								}

								OCBCUnitFamily unitFamily = idVSUnitFamily.get(familyId);
								if(!unitFamily.getIdVSUnit().containsKey(unitId)){
									//build the a unit
									OCBCUnit unit = new OCBCUnit();		
									unit.setId(unitId);
									unit.setReferenceActivity(sourceAOCEdge.getActivityNode());
									unit.setTargetActivity(targetAOCEdge.getActivityNode());
									unit.setType(1);
									unit.setActivityRelation(activityRelation);
									unit.setClassRelationId(classRelationId);
									unit.getClassNodeList().add(sourceAOCEdge.getClassNode());
									unit.getClassNodeList().add(targetAOCEdge.getClassNode());
									unit.getAOCRelationList().add(sourceAOCEdge);
									unit.getAOCRelationList().add(targetAOCEdge);
									
									unitFamily.getIdVSUnit().put(unitId, unit);
								}
								else{
									OCBCUnit unit = unitFamily.getIdVSUnit().get(unitId);
									if(!unit.getAOCRelationList().contains(sourceAOCEdge)){
										unit.getAOCRelationList().add(sourceAOCEdge);
									}
									if(!unit.getAOCRelationList().contains(targetAOCEdge)){
										unit.getAOCRelationList().add(targetAOCEdge);
									}
								}
							}
						}																														
					}	
				}			
			}			
		}
		return unitFamilyList;
	}
	


	
    public static void main(String[] args) throws Exception {
		   FileInputStream input = new FileInputStream("D:\\AUnit.ocbcmml");
//			InputStream inputStream = new FileInputStream("d:/123.ocbcmml");
			
			ByteArrayOutputStream baos = new ByteArrayOutputStream();
			byte[] buffer = new byte[1024];
			int len;
			while ((len = input.read(buffer)) > -1 ) {
			    baos.write(buffer, 0, len);
			}
			baos.flush();

			// Open new InputStreams using the recorded bytes
			// Can be repeated as many times as you wish
			InputStream inputA = new ByteArrayInputStream(baos.toByteArray()); 
			InputStream inputB = new ByteArrayInputStream(baos.toByteArray()); 
			
	        Scanner scanner = new Scanner(inputA, "UTF-8");
	        String fileString = scanner.useDelimiter("\\A").next();
	        scanner.close();
	        
	        String versionString = fileString.split("\n")[0];
			String versionName = versionString.split("\"")[1];
			String removeVersion = fileString.substring(fileString.indexOf('\n')+1);
			System.out.println("model file: " + removeVersion);
			
			OCBCModel model = null;
			if(versionName.equals(OCBCEditorConstants.VERSION2_0)){
				mxGraph graphX = new mxGraph();
				Document document = mxXmlUtils.parseXml(removeVersion);
				
				mxCodec codec = new mxCodec(document);
				codec.decode(document.getDocumentElement(), graphX.getModel());
				
				System.out.println("version 2");
				TransformToOCBCSemanticModel transformer = new TransformToOCBCSemanticModel();
				transformer.transform(graphX);
				model = transformer.getModel();
				model.setGraphX(graphX);
				model.setModelFile(mxXmlUtils.parseXml(removeVersion));
				model.setModelVersion(OCBCEditorConstants.VERSION2_0);
			}
			else{
				
				ImportFromFile fileParse = new ImportFromFile();       
		        fileParse.parse(inputB);
		        model = fileParse.getModel();
		        model.setModelVersion(OCBCEditorConstants.VERSION1_0);
		        int i=0;
		        for(MyOriginalCell cell : model.getGraphCells()){
		        	i++;
		        	System.out.println("cell number" + i);
		        	if(cell instanceof MNodeCell){
		 //       		System.out.println("cell name" + cell.getUserObject().toString());
		 //       		System.out.println("number of related edges" + model.getRelatedEdgeCellsForOneCell(cell).size());
		        	}
		        }
			}			
	      
//	        for(ActivityEdge activityEdge : model.getAllActivityRelations()){
//	        	System.out.println("\n activityEdge id: " + activityEdge.getID());
//	        	System.out.println("\n activityEdge name: " + activityEdge.getCardinalityType());
//	        }   
//	        
//	        for(CrelRTEdge edge : model.getAllCrelRTRelations()){
//	        	System.out.println("\n CrelRTEdge id: " + edge.getID());
//	        	System.out.println("\n CrelRTEdge name: " + edge.getRelatedActivityRelation().getID());
//
//	        }
//	        
//	        for(CrelOCEdge edge : model.getAllCrelOCRelations()){
//	        	System.out.println("\n CrelOCEdge id: " + edge.getID());
//	        	System.out.println("\n CrelOCEdge name: " + edge.getRelatedActivityRelation().getID());
//
//	        }
	        SearchOCBCUnitFamily searchOCBCUnitFamily = new SearchOCBCUnitFamily(model);
	        Map<String, OCBCUnitFamily> idVSUnitFamily = searchOCBCUnitFamily.getIdVSUnitFamily();
	        List<OCBCUnitFamily> familyList = searchOCBCUnitFamily.search();
	        
	        System.out.println("\n ***********************");
	        System.out.println("\n family size: " + idVSUnitFamily.size());
	        
	        for(String familyId : idVSUnitFamily.keySet()){
	        	System.out.println("\n ***********************");
	        	OCBCUnitFamily family = idVSUnitFamily.get(familyId);
	        	System.out.println("\n familyId: " + familyId);
	        	System.out.println("\n family reference id: " + family.getReferenceActivity().getID());
	        	System.out.println("\n family reference name: " + family.getReferenceActivity().getNodeName());
	        	System.out.println("\n family target id: " + family.getTargetActivity().getID());
	        	System.out.println("\n family target name: " + family.getTargetActivity().getNodeName());
	        	System.out.println("\n family type: " + family.getType());
	        	for(String unitId : family.getIdVSUnit().keySet()){
	        		OCBCUnit unit = family.getIdVSUnit().get(unitId);
	        		System.out.println("\n unitId: " + unit.getId());
	        		System.out.println("\n getCardinalityType: " + unit.getActivityRelation().getCardinalityType());
	        	}

	        }

//			XocXmlParser parser = new XocXmlParser();
//			parser.parse(new File("d:\\log.xoc"));		
//			XOCLog log = parser.getLog();
//			
//			ExtractXOCSegment extractXOCSegment = new ExtractXOCSegment(log, idVSUnitFamily);
//			extractXOCSegment.extractAllSegment();
//			Map<String, XOCSegment> idVSSegment = extractXOCSegment.getIdVSSegment();
//			System.out.println("\n ***********************");
//		    System.out.println("\n segment size: " + idVSSegment.size());
//	        for(String segmentId : idVSSegment.keySet()){
//	        	System.out.println("\n ***********************");
//	        	XOCSegment segment = idVSSegment.get(segmentId);
//	        	System.out.println("\n segmentId: " + segmentId);
//	        	System.out.println("\n segment reference name: " + segment.getReferenceActivityName());
//	        	System.out.println("\n segment target name: " + segment.getTargetActivityName());
//	        	System.out.println("\n segment type: " + segment.getType());
//	        	for(String caseId : segment.getIdVSCase().keySet()){
//	        		XOCCase xOCcase = segment.getIdVSCase().get(caseId);
//	        		System.out.println("\n unitId: " + xOCcase.getId());
//	        		for(XOCEvent event : xOCcase.getPrecedenceTargetEventList()){
//	        			System.out.println("\n precedence event Id: " + event.getAttributes().get(xConstants.ID).toString());
//	        		}
//	        		for(XOCEvent event : xOCcase.getResponseTargetEventList()){
//	        			System.out.println("\n response event Id: " + event.getAttributes().get(xConstants.ID).toString());
//	        		}
////	        		System.out.println("\n getCardinalityType: " + xOCcase.getActivityRelation().getCardinalityType());
//	        	}
//
//	        }
    }
}