package org.processmining.ocbc.discovery.metrics;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.FileInputStream;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Scanner;

import org.processmining.mxgraph.examples.swing.io.TransformToOCBCSemanticModel;
import org.processmining.mxgraph.io.mxCodec;
import org.processmining.mxgraph.util.mxConstants;
import org.processmining.mxgraph.util.mxXmlUtils;
import org.processmining.mxgraph.view.mxGraph;
import org.processmining.ocbc.logconversion.classmodel.SearchAllPathsBetweenTwoNodes;
import org.processmining.ocbc.models.ClassNode;
import org.processmining.ocbc.models.OCBCModel;
import org.processmining.ocbc.models.editor.plugins.OCBCEditorConstants;
import org.processmining.ocbc.models.impl.ClassNodeImpl;
import org.processmining.ocbc.models.io.ImportFromFile;
import org.processmining.ocbc.models.visualization.cell.MNodeCell;
import org.processmining.ocbc.models.visualization.cell.MyOriginalCell;
import org.w3c.dom.Document;


public class SearchPathListForFamily {
	private OCBCUnitFamily family;
	private Map<String, OCBCUnitFamily> idVSUnitFamily = new HashMap<String, OCBCUnitFamily>();
	private List<String> pathList = new ArrayList<String>();
	private Map<String, List<OCBCUnitFamily>> pathVSFamilyList = new HashMap<String, List<OCBCUnitFamily>>();
	public SearchPathListForFamily(OCBCUnitFamily family, Map<String, OCBCUnitFamily> idVSUnitFamily){
		this.family = family;
		this.idVSUnitFamily = idVSUnitFamily;
	}
	
	public List<String> getPathList(){
		return pathList;		
	}
	
	public Map<String, List<OCBCUnitFamily>> getPathVSFamilyList(){
		return pathVSFamilyList;		
	}

	
	public List<String> searchForFamilyA(){
		//build the nodes (with its related nodes)
		List<ClassNode> classNodeList = new ArrayList<ClassNode>();
		Map<String, ClassNode> classNameVSNodeMap = new HashMap<String, ClassNode>();
		for(String familyId  : idVSUnitFamily.keySet()){
			OCBCUnitFamily unitFamily = idVSUnitFamily.get(familyId);
			String referenceActivityName = unitFamily.getReferenceActivity().getNodeName();
			String targetActivityName = unitFamily.getTargetActivity().getNodeName();
			//check if the reference class is the same
			if(family.getType() == 0 && unitFamily.getType() == 0){
				String className = family.getClassNodeList().get(0).getNodeName();
				String classNamePath = unitFamily.getClassNodeList().get(0).getNodeName();
				//the class is the same
				if(classNamePath.equals(className)){
					boolean noConstraintsFlag = false;
					for(String unitId : unitFamily.getIdVSUnit().keySet()){
		        		OCBCUnit unit = unitFamily.getIdVSUnit().get(unitId);
		        		if(unit.getActivityRelation().getCardinalityType() == mxConstants.NoConstraints){
		        			noConstraintsFlag = true;
		        			break;
		        		}
		        		System.out.println("\n unitId: " + unit.getId());       		
		        	}

					if(!noConstraintsFlag){
						ClassNode referenceNode;
						if(!classNameVSNodeMap.keySet().contains(referenceActivityName)){
							referenceNode = new ClassNodeImpl();    					
							referenceNode.setNodeName(referenceActivityName);
							classNameVSNodeMap.put(referenceActivityName, referenceNode);  
							classNodeList.add(referenceNode);
						}
						else{
							referenceNode = classNameVSNodeMap.get(referenceActivityName);
						}
						
						ClassNode targetNode;
						//if the father class does not exist before
						if(!classNameVSNodeMap.keySet().contains(targetActivityName)){
							targetNode = new ClassNodeImpl();    					
							targetNode.setNodeName(targetActivityName);
							classNameVSNodeMap.put(targetActivityName, targetNode);
							classNodeList.add(targetNode);
						}
						else{
							targetNode = classNameVSNodeMap.get(targetActivityName);
						}
						
						if(!referenceNode.getRelatedClassNodes().contains(targetNode)){
							referenceNode.getRelatedClassNodes().add(targetNode);
						}
						
						if(!targetNode.getRelatedClassNodes().contains(referenceNode)){
							targetNode.getRelatedClassNodes().add(referenceNode);
						}	
					}
				}
			}	
		}
		
		String startActivityName = family.getReferenceActivity().getNodeName();
		String endActivityName = family.getTargetActivity().getNodeName();
		ClassNode startNode = classNameVSNodeMap.get(startActivityName);
		ClassNode endNode = classNameVSNodeMap.get(endActivityName);
			
		SearchAllPathsBetweenTwoNodes searchAllPathsBetweenTwoNodes = new SearchAllPathsBetweenTwoNodes();
		searchAllPathsBetweenTwoNodes.clearPathList();
		searchAllPathsBetweenTwoNodes.getPaths(startNode, null, startNode, endNode);
		pathList.addAll(searchAllPathsBetweenTwoNodes.getPathList());
		
		//transform path string into family path
		for(String path : pathList){
			List<OCBCUnitFamily> familyPath = new ArrayList<OCBCUnitFamily>();
			String[] activityNameSet = path.split("->");
			if(activityNameSet.length > 2){
				for(int i=0; i<activityNameSet.length-1; i++){
					String referenceActivity = activityNameSet[i];
					String targetActivity = activityNameSet[i+1];
					String className = family.getClassNodeList().get(0).getNodeName();
					for(String familyId  : idVSUnitFamily.keySet()){
						OCBCUnitFamily unitFamily = idVSUnitFamily.get(familyId);
						String referenceActivityName = unitFamily.getReferenceActivity().getNodeName();
						String targetActivityName = unitFamily.getTargetActivity().getNodeName();
						if(unitFamily.getType() == 0){
							String classNamePath = unitFamily.getClassNodeList().get(0).getNodeName();
							if(referenceActivity.equals(referenceActivityName) && targetActivity.equals(targetActivityName) &&
									className.equals(classNamePath)){
								familyPath.add(unitFamily);
								break;
							}
						}						
					}
				}				
			}
			pathVSFamilyList.put(path, familyPath);
		}
		List<String> unqualifiedPathList = new ArrayList<String>();
		for(String path : pathVSFamilyList.keySet()){
    		for(int i=0; i<pathVSFamilyList.get(path).size(); i++){
    			OCBCUnitFamily familyPath = pathVSFamilyList.get(path).get(i);
    			if(familyPath.getActivityRelationList().get(0).getCardinalityType().equals(mxConstants.NoConstraints)){
    				unqualifiedPathList.add(path);
    				break;
    			}
    			//the reference aoc relation and target aoc relation
    			if(i>0){
//    				if(familyPath.get){
//        				unqualifiedPathList.add(path);
//        				break;
//        			}
    			}
    			
    		}
    	}
		//filter the wrong path
		return pathList;
	}
	


	
    public static void main(String[] args) throws Exception {
		   FileInputStream input = new FileInputStream("D:\\path.ocbcmml");
//			InputStream inputStream = new FileInputStream("d:/123.ocbcmml");
			
			ByteArrayOutputStream baos = new ByteArrayOutputStream();
			byte[] buffer = new byte[1024];
			int len;
			while ((len = input.read(buffer)) > -1 ) {
			    baos.write(buffer, 0, len);
			}
			baos.flush();

			// Open new InputStreams using the recorded bytes
			// Can be repeated as many times as you wish
			InputStream inputA = new ByteArrayInputStream(baos.toByteArray()); 
			InputStream inputB = new ByteArrayInputStream(baos.toByteArray()); 
			
	        Scanner scanner = new Scanner(inputA, "UTF-8");
	        String fileString = scanner.useDelimiter("\\A").next();
	        scanner.close();
	        
	        String versionString = fileString.split("\n")[0];
			String versionName = versionString.split("\"")[1];
			String removeVersion = fileString.substring(fileString.indexOf('\n')+1);
			System.out.println("model file: " + removeVersion);
			
			OCBCModel model = null;
			if(versionName.equals(OCBCEditorConstants.VERSION2_0)){
				mxGraph graphX = new mxGraph();
				Document document = mxXmlUtils.parseXml(removeVersion);
				
				mxCodec codec = new mxCodec(document);
				codec.decode(document.getDocumentElement(), graphX.getModel());
				
				System.out.println("version 2");
				TransformToOCBCSemanticModel transformer = new TransformToOCBCSemanticModel();
				transformer.transform(graphX);
				model = transformer.getModel();
				model.setGraphX(graphX);
				model.setModelFile(mxXmlUtils.parseXml(removeVersion));
				model.setModelVersion(OCBCEditorConstants.VERSION2_0);
			}
			else{
				
				ImportFromFile fileParse = new ImportFromFile();       
		        fileParse.parse(inputB);
		        model = fileParse.getModel();
		        model.setModelVersion(OCBCEditorConstants.VERSION1_0);
		        int i=0;
		        for(MyOriginalCell cell : model.getGraphCells()){
		        	i++;
		        	System.out.println("cell number" + i);
		        	if(cell instanceof MNodeCell){
		 //       		System.out.println("cell name" + cell.getUserObject().toString());
		 //       		System.out.println("number of related edges" + model.getRelatedEdgeCellsForOneCell(cell).size());
		        	}
		        }
			}			
	      
//	        for(ActivityEdge activityEdge : model.getAllActivityRelations()){
//	        	System.out.println("\n activityEdge id: " + activityEdge.getID());
//	        	System.out.println("\n activityEdge name: " + activityEdge.getCardinalityType());
//	        }   
//	        
//	        for(CrelRTEdge edge : model.getAllCrelRTRelations()){
//	        	System.out.println("\n CrelRTEdge id: " + edge.getID());
//	        	System.out.println("\n CrelRTEdge name: " + edge.getRelatedActivityRelation().getID());
//
//	        }
//	        
//	        for(CrelOCEdge edge : model.getAllCrelOCRelations()){
//	        	System.out.println("\n CrelOCEdge id: " + edge.getID());
//	        	System.out.println("\n CrelOCEdge name: " + edge.getRelatedActivityRelation().getID());
//
//	        }
	        SearchOCBCUnitFamily searchOCBCUnitFamily = new SearchOCBCUnitFamily(model);
	        Map<String, OCBCUnitFamily> idVSUnitFamily = searchOCBCUnitFamily.getIdVSUnitFamily();
	        List<OCBCUnitFamily> familyList = searchOCBCUnitFamily.search();
	        
	        System.out.println("\n ***********************");
	        System.out.println("\n family size: " + idVSUnitFamily.size());
	        
	        for(String familyId : idVSUnitFamily.keySet()){
	        	System.out.println("\n ***********************");
	        	OCBCUnitFamily family = idVSUnitFamily.get(familyId);
	        	System.out.println("\n familyId: " + familyId);
	        	System.out.println("\n family reference id: " + family.getReferenceActivity().getID());
	        	System.out.println("\n family reference name: " + family.getReferenceActivity().getNodeName());
	        	System.out.println("\n family target id: " + family.getTargetActivity().getID());
	        	System.out.println("\n family target name: " + family.getTargetActivity().getNodeName());
	        	System.out.println("\n family type: " + family.getType());
	        	for(String unitId : family.getIdVSUnit().keySet()){
	        		OCBCUnit unit = family.getIdVSUnit().get(unitId);
	        		System.out.println("\n unitId: " + unit.getId());
	        		System.out.println("\n getCardinalityType: " + unit.getActivityRelation().getCardinalityType());
	        	}
	        	System.out.println("\n **********path*************");
	        	SearchPathListForFamily searchPath = new SearchPathListForFamily(family, idVSUnitFamily);
	        	List<String> pathList = searchPath.searchForFamilyA();
	        	Map<String, List<OCBCUnitFamily>> pathVSFamilyList = searchPath.getPathVSFamilyList();
	        	for(String path : pathVSFamilyList.keySet()){
	        		System.out.println("\n path chain: " + path);
	        		for(OCBCUnitFamily familyPath : pathVSFamilyList.get(path)){
	        			System.out.println("\n path familyId: " + familyId);
	    	        	System.out.println("\n path family reference name: " + familyPath.getReferenceActivity().getNodeName());
	    	        	System.out.println("\n path family target name: " + familyPath.getTargetActivity().getNodeName());
	        		}
	        	}
//	        	for(String path : pathList){
//	        		System.out.println("\n path: " + path);
//	        	}
	        }

//	        for(String familyId : idVSUnitFamily.keySet()){
//	        	System.out.println("\n ***********************");
//	        	OCBCUnitFamily family = idVSUnitFamily.get(familyId);
////	        	System.out.println("\n familyId: " + familyId);
////	        	System.out.println("\n family reference id: " + family.getReferenceActivity().getID());
////	        	System.out.println("\n family reference name: " + family.getReferenceActivity().getNodeName());
////	        	System.out.println("\n family target id: " + family.getTargetActivity().getID());
////	        	System.out.println("\n family target name: " + family.getTargetActivity().getNodeName());
////	        	System.out.println("\n family type: " + family.getType());
////	        	for(String unitId : family.getIdVSUnit().keySet()){
////	        		OCBCUnit unit = family.getIdVSUnit().get(unitId);
////	        		System.out.println("\n unitId: " + unit.getId());
////	        		System.out.println("\n getCardinalityType: " + unit.getActivityRelation().getCardinalityType());
////	        	}
//	        	
//	        	SearchPathListForFamily searchPath = new SearchPathListForFamily(family, idVSUnitFamily);
//	        	List<String> pathList = searchPath.search();
//	        	for(String path : pathList){
//	        		System.out.println("\n path: " + path);
//	        	}
//	        }
    }
}