package org.processmining.ocbc.discovery.panels;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;

import org.processmining.common.number.ParseNumberFromString;
import org.processmining.mxgraph.examples.swing.editor.BasicGraphEditor;
import org.processmining.mxgraph.examples.swing.io.TransformBetweenJGraphXAndModel;
import org.processmining.mxgraph.examples.swing.io.TransformToOCBCSemanticModel;
import org.processmining.mxgraph.layout.hierarchical.mxHierarchicalLayout;
import org.processmining.mxgraph.model.mxCell;
import org.processmining.mxgraph.model.mxGeometry;
import org.processmining.mxgraph.swing.mxGraphComponent;
import org.processmining.mxgraph.util.mxConstants;
import org.processmining.mxgraph.view.mxGraph;
import org.processmining.ocbc.discovery.metrics.ComputeBehavioralPrecision;
import org.processmining.ocbc.discovery.metrics.ComputePrecisionThread;
import org.processmining.ocbc.models.ActivityEdge;
import org.processmining.ocbc.models.OCBCModel;

public class ButtonPanel extends JPanel {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	private mxGraphComponent graphComponent;
	private BasicGraphEditor graphEditor;
	private mxGraph graph;
	private JButton updateMetricsButton = new JButton("update metrics");
	private JButton updateModelButton = new JButton("update model");
    private List<ActivityEdge> sortedActivityList;
    private List<ActivityEdge> sortedShownActivityList = new ArrayList<ActivityEdge>();
    private Object[] allCells;
    private ComputePrecisionThread computePrecision;
	public ButtonPanel(final BasicGraphEditor graphEditor) {
	    super();
	    this.graphEditor = graphEditor;	  

//	    this.allCells = graph.getChildCells(graph.getDefaultParent());
//	    this.computePrecision = new ComputePrecisionThread("precision",graphEditor.getOCBCModel().getModelSourceLog(), graphEditor.getOCBCModel(),graphComponent);
//	    computePrecision.start();
	    setLayout(new GridLayout(1, 2));

	    updateMetricsButton.addActionListener(new ActionListener() {			
			public void actionPerformed(ActionEvent e) {
				TransformToOCBCSemanticModel transformer = new TransformToOCBCSemanticModel();
				transformer.transform(graphComponent.getGraph());
				OCBCModel model = transformer.getModel();
				ComputeBehavioralPrecision computeBehavioralPrecision = new ComputeBehavioralPrecision();
				double precision = computeBehavioralPrecision.computePrecision(graphEditor.getOCBCModel().getModelSourceLog(), model);  				
				graphEditor.getSliderPanel().getPrecisionSlider().setValue((int) (precision*100));
				double shortPrecison = ((double)((int) (precision*100)))/100;
				graphEditor.getSliderPanel().getPrecisionSlider().setBorder(BorderFactory.createTitledBorder("precision: " + shortPrecison));
			}
		});
	    

	    
	   
	    updateModelButton.addActionListener(new ActionListener() {			
			public void actionPerformed(ActionEvent e) {
				System.out.println("\n update model");

				String configuredFitnessAlgorithm = (String) graphEditor.getAlgorithmPanel().getFitnessAlgorithmBox().getSelectedItem();  
				String configuredPrecisionAlgorithm = (String) graphEditor.getAlgorithmPanel().getPrecisionAlgorithmBox().getSelectedItem();  
				String configuredGeneralizationAlgorithm = (String) graphEditor.getAlgorithmPanel().getGeneralizationAlgorithmBox().getSelectedItem();  
				
				String fitnessThresholdText = graphEditor.getAlgorithmPanel().getFitnessThresholdField().getText();
				String precisionThresholdText = graphEditor.getAlgorithmPanel().getPrecisionThresholdField().getText();
				String generalizationThresholdText = graphEditor.getAlgorithmPanel().getGeneralizationThresholdField().getText();

				ParseNumberFromString parseNumberFromString = new ParseNumberFromString();
				Double configuredFitnessThreshold = null;	
				if(configuredFitnessAlgorithm.equals(mxConstants.fitnessAlgorithmWithNumThreshold)){	
					Integer parseInteger = parseNumberFromString.parseIntegerFromStringWithBoundary(fitnessThresholdText, 0, Integer.MAX_VALUE,
							"Wrong! The input for fitness algorithm should be a positive integer");
					configuredFitnessThreshold = parseInteger != null ? parseInteger.doubleValue() : null;
				}
				else if(configuredFitnessAlgorithm.equals(mxConstants.fitnessAlgorithmWithRatioThreshold)){		
					configuredFitnessThreshold = parseNumberFromString.parseDoubleFromStringWithBoundary(fitnessThresholdText, 0, 1,
							"Wrong! The input for fitness algorithm should be an decimal in [0,1]");
				}
				else if(configuredFitnessAlgorithm.equals(mxConstants.fitnessAlgorithmWithoutThreshold)){		
					configuredFitnessThreshold = 0.0;
				}
				
				Double configuredPrecisionThreshold = null;	
				if(configuredPrecisionAlgorithm.equals(mxConstants.precisionAlgorithmWithNumThreshold)){	
					Integer parseInteger = parseNumberFromString.parseIntegerFromStringWithBoundary(precisionThresholdText, 0, Integer.MAX_VALUE,
							"Wrong! The input for precision algorithm should be a positive integer");
					configuredPrecisionThreshold = parseInteger != null ? parseInteger.doubleValue() : null;
				}
				else if(configuredPrecisionAlgorithm.equals(mxConstants.precisionAlgorithmWithRatioThreshold)){		
					configuredPrecisionThreshold = parseNumberFromString.parseDoubleFromStringWithBoundary(precisionThresholdText, 0, 1,
							"Wrong! The input for precision algorithm should be an decimal in [0,1]");
				}
				else if(configuredPrecisionAlgorithm.equals(mxConstants.precisionAlgorithmByEntropy)){		
					configuredPrecisionThreshold = 0.0;
				}
				
				Double configuredGeneralizationThreshold = parseNumberFromString.parseDoubleFromStringWithBoundary(generalizationThresholdText, 0, 1,
						"Wrong! The input for generalization algorithm should be an decimal in [0,1]");
				
				if(configuredFitnessThreshold != null && configuredPrecisionThreshold != null && configuredGeneralizationThreshold != null){				
					double configuredPrecisionWeight = ((double)graphEditor.getSliderPanel().getPrecisionSlider().getValue())/100;  
					double configuredFitnessWeight = ((double)graphEditor.getSliderPanel().getFitnessSlider().getValue())/100;  
					double configuredGeneralizationWeight = ((double)graphEditor.getSliderPanel().getGeneralizationSlider().getValue())/100;  
					
					OCBCModel model = graphEditor.getOCBCModel().getModelDiscovery().getBehaviorModelDiscovery().
							updateActivityEdges(configuredFitnessAlgorithm, configuredPrecisionAlgorithm, configuredGeneralizationAlgorithm, 
									configuredFitnessThreshold, configuredPrecisionThreshold, configuredGeneralizationThreshold, 
									configuredFitnessWeight, configuredPrecisionWeight, configuredGeneralizationWeight);	
					updateModel(model);

				}
			}
		});
	    
//	    this.add(updateMetricsButton);
	    this.add(updateModelButton);

	}

	/**
	 * update the model graph based on an input model;
	 * @param model
	 */
	public void updateModel(OCBCModel model){
	    graphComponent = graphEditor.getGraphComponent();
		graph = graphComponent.getGraph();
		TransformBetweenJGraphXAndModel transformer = new TransformBetweenJGraphXAndModel();
		mxGraph graphUpdated = transformer.transformModelToJGraphX(model, null);
		
		graph.removeCells(graph.getChildCells(graph.getDefaultParent()));
		graph.addCells(graphUpdated.getChildCells(graphUpdated.getDefaultParent()));
		
		mxHierarchicalLayout layout = new mxHierarchicalLayout(graph, JLabel.WEST);
		Object cell = graph.getDefaultParent();
		graph.getModel().beginUpdate();
		Object[] childVertices = graph.getChildVertices(cell);
		Object[] childEdges = graph.getChildEdges(cell);		
		List<Object> childCellList = new ArrayList<Object>();
		for(int i=0; i<childVertices.length; i++){
			childCellList.add(childVertices[i]);
		}
		for(int i=0; i<childEdges.length; i++){
			childCellList.add(childEdges[i]);
		}		
		layout.execute(cell, childCellList);
		
		Object[] childEdgesTwo = graph.getChildEdges(graph.getDefaultParent());
		for(int i=0; i<childEdges.length; i++){
			mxCell edgeCell = (mxCell) childEdgesTwo[i];
			String type = edgeCell.getAttributes().get(mxConstants.CARDINALITYTYPE);		
			if(type != null && type.contains("non")){
				mxGeometry geometry = new mxGeometry();
				geometry.setRelative(true);
				geometry.setPoints(edgeCell.getGeometry().getPoints());
				edgeCell.setGeometry(geometry);				
			}
		}
		graph.refresh();

		graphComponent.updateUI();
		graphComponent.refresh();	
		//update the number of constraints in the filter panel
//		behavioralConstraintTypeVSCellSet
		graphEditor.getFilterPanel().clearConstraintTypeVSCellsMap();
		graphEditor.getFilterPanel().updateConstraintNum();
		graphEditor.getFilterPanel().initializeAllRadioButtonsAsSelected();
		
		System.out.println("behavioral constraints number in model: " + model.getAllActivityRelations().size());
		System.out.println("behavioral constraints number in graph: " + graph.getActivityRelationList().size());
	}
	
	public void updateShownActivityList(){
		graphComponent = graphEditor.getGraphComponent();
		graph = graphComponent.getGraph();
		List<String> shownActivityRelationIdList = new ArrayList<String>();
		System.out.println("behavioral constraints number in graph in : updateShownActivityList" + graph.getActivityRelationList().size());
		for(mxCell cell : graph.getActivityRelationList()){
			String cellId =  cell.getId();
			shownActivityRelationIdList.add(cellId);
		}
		
		if(sortedActivityList != null){
			sortedShownActivityList.clear();
			for(ActivityEdge activityEdge : sortedActivityList){  
				String id = activityEdge.getID();
				if(shownActivityRelationIdList.contains(id)){
					sortedShownActivityList.add(activityEdge);
				}
			}
		}

		//update all cells after filtering by constraint type, new type and activity
		allCells = graph.getChildCells(graph.getDefaultParent());
	}
	
	public JButton getUpdateMetricsButton(){
		return updateMetricsButton;
	}
	
	public JButton getUpdateModelButton(){
		return updateModelButton;
	}
	
    public static void main(String[] args) {
    	
        JFrame frame = new JFrame("Hello world");
       
        ButtonPanel panel = new ButtonPanel(null);
       
        frame.add(panel);

        frame.setSize(600, 400);
        frame.setVisible(true);
       
     }

}