package org.processmining.ocbc.discovery.panels;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.util.List;
import java.util.Map;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JScrollBar;
import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.ScrollPaneConstants;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeCellRenderer;

import org.processmining.common.structure.IntegerPair;
import org.processmining.mxgraph.model.mxCell;
import org.processmining.mxgraph.util.mxConstants;
import org.processmining.ocbc.discovery.impl.ClassEdgeCardinalityDiscovery;
import org.processmining.ocbc.log.model.XAttributeMap;
import org.processmining.ocbc.log.model.XOCEvent;
import org.processmining.ocbc.log.model.XOCLog;
import org.processmining.ocbc.log.model.XOCORelation;
import org.processmining.ocbc.log.model.XOCObject;
import org.processmining.ocbc.log.model.impl.xConstants;
import org.processmining.ocbc.models.OCBCModel;

import com.fluxicon.slickerbox.components.RoundedPanel;
import com.fluxicon.slickerbox.ui.SlickerScrollBarUI;

/**
 * Presents the instance information (in hierarchy) of a selected constraint in the discovered model.
 * For example, when a class edge is selected, show all the source objects and target objects.
 * Each object with all observed cardinalities and corresponding life cycle period.
 * @author gli3
 *
 */
public class ConstraintInstancePanel extends RoundedPanel {
//public class ConstraintInstancePanel extends Panel {

	
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	
	private JTree mdtree;
//	private JPanel eventPane;
//	private JPanel objectPane;
	
	public static final Color COLOR_ENCLOSURE_BG = new Color(40, 40, 40);
	public static final Color COLOR_NON_FOCUS = new Color(70, 70, 70);
	public static final Color COLOR_LIST_BG = new Color(60, 60, 60);
	public static final Color COLOR_LIST_BG_LOWER = new Color(45, 45, 45);
	public static final Color COLOR_LIST_FG = new Color(180, 180, 180);
	public static final Color COLOR_LIST_SELECTION_BG = new Color(80, 0, 0);
	public static final Color COLOR_LIST_SELECTION_BG_LOWER = new Color(30, 10, 10);
	public static final Color COLOR_LIST_SELECTION_FG = new Color(240, 240, 240);

	private static final String TREE_ROOT_NODE_EVENT = "Event";
	private static final String TREE_ROOT_NODE_OBJECT = "Object";
	private static final String TREE_ROOT_NODE_RELATION = "Relation";
	private static final String TREE_NODE_REFERENCES = "References";
	private static final String TREE_NODE_OBJECTMODEL = "ObjectModel";
	private static final String TREE_NODE_OBJECTS = "Objects";
	private static final String TREE_NODE_RELATIONS = "Relations";
	private static final String TREE_NODE_ATTRIBUTES = "Attributes";
	private static final String TREE_NODE_REFERREDEVENTS = "ReferredEvents";
	private static final String TREE_NODE_DEVIATIONS = "Deviations";
	
	public ConstraintInstancePanel(int aRadius, int anOuterBorder, int anInnerBorder){
		super(aRadius, anOuterBorder, anInnerBorder);
//		this.setBackground(COLOR_ENCLOSURE_BG);
		this.setMaximumSize(new Dimension(230, 1000));
		this.setPreferredSize(new Dimension(230, 1000));
		this.setLayout(new GridLayout(1,2));
		
		DefaultMutableTreeNode root = new DefaultMutableTreeNode("selected constraint id: ");
		mdtree = new JTree(root);
		mdtree.setBackground(COLOR_ENCLOSURE_BG);
		mdtree.setForeground(Color.WHITE);
		DefaultTreeCellRenderer renderer = (DefaultTreeCellRenderer) mdtree
				.getCellRenderer();
		renderer.setBackground(COLOR_ENCLOSURE_BG);
		renderer
				.setBackgroundNonSelectionColor(COLOR_ENCLOSURE_BG);
		renderer
				.setBackgroundSelectionColor(COLOR_LIST_SELECTION_BG);
		renderer.setForeground(Color.WHITE);
		renderer.setTextNonSelectionColor(Color.WHITE);
		renderer.setTextSelectionColor(COLOR_LIST_SELECTION_FG);
		JScrollPane treePane= new JScrollPane(mdtree);
		this.add(treePane);		
	}

	/**
	 * elementId: the id (in the model) of selected cell
	 */
	public void updateConstraintInstances(mxCell selectedCell, OCBCModel model, String reference, String additionalType) {
		System.out.println("updateConstraintInstances selectedCell Id: " + selectedCell.getId());
		String type = selectedCell.getAttributes().get(mxConstants.TYPE);
		String constraintId = selectedCell.getId();
		System.out.println("updateConstraintInstances type: " + type);
		setOpaque(false);
		setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
		setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
		this.removeAll();

		// Create the root node of the tree.
		DefaultMutableTreeNode root = new DefaultMutableTreeNode("selected constraint id: " + constraintId);
		// Create metadata tree.
		mdtree = new JTree(root);
		mdtree.setBackground(COLOR_ENCLOSURE_BG);
		mdtree.setForeground(Color.WHITE);
		DefaultTreeCellRenderer renderer = (DefaultTreeCellRenderer) mdtree
				.getCellRenderer();
		renderer.setBackground(COLOR_ENCLOSURE_BG);
		renderer
				.setBackgroundNonSelectionColor(COLOR_ENCLOSURE_BG);
		renderer
				.setBackgroundSelectionColor(COLOR_LIST_SELECTION_BG);
		renderer.setForeground(Color.WHITE);
		renderer.setTextNonSelectionColor(Color.WHITE);
		renderer.setTextSelectionColor(COLOR_LIST_SELECTION_FG);


		JScrollPane treePane= new JScrollPane(mdtree);
		treePane.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
		treePane.setOpaque(false);
		treePane
				.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);
		treePane
				.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		JScrollBar vBar = treePane.getVerticalScrollBar();
		vBar.setUI(new SlickerScrollBarUI(vBar,
				COLOR_ENCLOSURE_BG, new Color(160, 160, 160),
				COLOR_NON_FOCUS, 4, 12));
		JScrollBar hBar = treePane.getHorizontalScrollBar();
		hBar.setUI(new SlickerScrollBarUI(hBar,
				COLOR_ENCLOSURE_BG, new Color(160, 160, 160),
				COLOR_NON_FOCUS, 4, 12));

		
		if(type.equals(mxConstants.ACTIVITYRELATION)){
//			if(additionalType.equals(mxConstants.PRECEDENCE)){
//				//add the PrecedenceEventList
//				Map<String, List<String>> eventVSPrecedenceEventList=  model.getModelDiscoveryResult().getActivityEdgeVSEventVSPrecedenceEventList().get(constraintId);		
//				for(String eventId : eventVSPrecedenceEventList.keySet()){
//					DefaultMutableTreeNode eventAndRelatedEventListNode = generateEventInstanceNode(eventId, eventVSPrecedenceEventList.get(eventId));
//					root.add(eventAndRelatedEventListNode);
//				}	
//			}
//			else{
//				//add the ResponseEventList
//				Map<String, List<String>> eventVSResponseEventList=  model.getModelDiscoveryResult().getActivityEdgeVSEventVSResponseEventList().get(constraintId);
//				for(String eventId : eventVSResponseEventList.keySet()){
//					DefaultMutableTreeNode eventAndRelatedEventListNode = generateEventInstanceNode(eventId, eventVSResponseEventList.get(eventId));
//					root.add(eventAndRelatedEventListNode);
//				}	
//			}
				
			//if secondId exists, add corresponding event instances
//			String secondId = selectedCell.getAttributes().get(mxConstants.SECONDID);
//			if(secondId != null){
//				//add the PrecedenceEventList
//				Map<String, List<String>> eventVSPrecedenceEventListSecond=  model.getModelDiscoveryResult().getActivityEdgeVSEventVSPrecedenceEventList().get(secondId);		
//				for(String eventId : eventVSPrecedenceEventListSecond.keySet()){
//					DefaultMutableTreeNode eventAndRelatedEventListNodeSecond = generateEventInstanceNode(eventId + "(precedence)", eventVSPrecedenceEventListSecond.get(eventId));
//					root.add(eventAndRelatedEventListNodeSecond);
//				}	
//				//add the ResponseEventList
//				Map<String, List<String>> eventVSResponseEventListSecond =  model.getModelDiscoveryResult().getActivityEdgeVSEventVSResponseEventList().get(secondId);
//				for(String eventId : eventVSResponseEventListSecond.keySet()){
//					DefaultMutableTreeNode eventAndRelatedEventListNodeSecond = generateEventInstanceNode(eventId + "(response)", eventVSResponseEventListSecond.get(eventId));
//					root.add(eventAndRelatedEventListNodeSecond);
//				}	
//			}
		}
		
		else if(type.equals(mxConstants.CLASSRELATION)){
			//add the PrecedenceEventList
//			Map<String, Map<String, Map<String, List<String>>>> eventIdVSRelationIdVSObjectIdVSRelatedObjectIdList =  model.getModelDiscovery()
//					.getClassModelDiscovery()
//					.getClassEdgeCardinalityDiscovery()
//					.getEventIdVSRelationIdVSObjectIdVSRelatedObjectIdList();	
			
//			Map<String, Map<String, Map<String, List<String>>>> relationIdVSEventIdVSObjectIdVSRelatedObjectIdList =  model.getModelDiscovery()
//					.getClassModelDiscovery()
//					.getRelationIdVSEventIdVSObjectIdVSRelatedObjectIdList();
			
			XOCLog log = model.getModelSourceLog();
			String selectedEventId = additionalType.split("\\)")[1];
//			if(relationIdVSEventIdVSObjectIdVSRelatedObjectIdList.containsKey(constraintId) 
//					&& relationIdVSEventIdVSObjectIdVSRelatedObjectIdList.get(constraintId).containsKey(selectedEventId)){
//				Map<String, List<String>> objectIdVSRelatedObjectIdList = 
//						relationIdVSEventIdVSObjectIdVSRelatedObjectIdList.get(constraintId).get(selectedEventId);
//				Map<String, List<String>> objectIdVSReferenceRelatedObjectIdList = new HashMap<String, List<String>>();
//				XOCEvent event = log.getIdVSEventMap().get(selectedEventId);//modified
//				for(String objectId : objectIdVSRelatedObjectIdList.keySet()){
//					XOCObject object = event.getObjectModel().getObjectById(objectId);
//					String className = object.getAttributes().get(xConstants.CLASS).toString();
//					if(className.equals(reference)){
//						objectIdVSReferenceRelatedObjectIdList.put(objectId, objectIdVSRelatedObjectIdList.get(objectId));
//					}
//				}				
//				addNodeAndRelatedNodeHierarchy(root, objectIdVSReferenceRelatedObjectIdList);
//			}
			
//			for(XOCEvent event : log.getAllEvents()){
//				String eventId = event.getAttributes().get(xConstants.ID).toString();
//				if(eventId.equals(selectedEventId) && eventIdVSRelationIdVSObjectIdVSRelatedObjectIdList.get(eventId).containsKey(constraintId)){
//
//					Map<String, List<String>> ObjectIdVSRelatedObjectIdList = new HashMap<String, List<String>>();
//					
//				}
//				if(eventId.equals(selectedEventId) && eventIdVSRelationIdVSObjectIdVSRelatedObjectIdList.get(eventId).containsKey(constraintId)){
////					DefaultMutableTreeNode eventAndObjectHierarchy = generateEventAndObjectHierarchy("(" + (i++) + ")" + eventId, 
////							eventIdVSRelationIdVSObjectIdVSRelatedObjectIdList.get(eventId).get(constraintId));
////					root.add(eventAndObjectHierarchy);
//					Map<String, List<String>> ObjectIdVSRelatedObjectIdList = new HashMap<String, List<String>>();
//					for(String objectId : eventIdVSRelationIdVSObjectIdVSRelatedObjectIdList.get(eventId).get(constraintId).keySet()){
//						XOCObject object = event.getObjectModel().getObjectById(objectId);
//						String className = object.getAttributes().get(xConstants.CLASS).toString();
//						if(className.equals(reference)){
//							ObjectIdVSRelatedObjectIdList.put(objectId, eventIdVSRelationIdVSObjectIdVSRelatedObjectIdList
//									.get(eventId).get(constraintId).get(objectId));
//						}
//					}				
//					addNodeAndRelatedNodeHierarchy(root, ObjectIdVSRelatedObjectIdList);
//				}
//			}	
		}								
		
		else if(type.equals(mxConstants.AOCRELATION)){
			//add the PrecedenceEventList
//			Map<String, Map<String, Map<String, List<String>>>> eventIdVSRelationIdVSObjectVSEventList =  model.getModelDiscovery()
//					.getAOCEdgeDiscovery().getEventIdVSRelationIdVSObjectVSEventList();
//			Map<String, Map<String, List<String>>> relationIdVSEventVSRelatedObjectList = model.getModelDiscovery()
//					.getAOCEdgeDiscovery().getRelationIdVSEventVSObjectList();
//			mxCell sourceCell = (mxCell) selectedCell.getSource();
//			String sourceName = (String) sourceCell.getValue();
//			if(sourceName.equals(reference) && relationIdVSEventVSRelatedObjectList.containsKey(constraintId)){
//				Map<String, List<String>> eventVSObjectList = relationIdVSEventVSRelatedObjectList.get(constraintId);
//				addNodeAndRelatedNodeHierarchy(root, eventVSObjectList);
//			}
//			else if(additionalType != null){
//				XOCLog log = model.getModelSourceLog();				
//				String selectedEventId = additionalType.split("\\)")[1];
//				for(XOCEvent event : log.getAllEvents()){
//					String eventId = event.getAttributes().get(xConstants.ID).toString();
//					if(eventId.equals(selectedEventId) && eventIdVSRelationIdVSObjectVSEventList.get(eventId).containsKey(constraintId)){
//						Map<String, List<String>> ObjectIdVSRelatedEventIdList = eventIdVSRelationIdVSObjectVSEventList.get(eventId).get(constraintId);								
//						addNodeAndRelatedNodeHierarchy(root, ObjectIdVSRelatedEventIdList);
//					}
//				}	
//			}

			
//			XOCLog log = model.getModelSourceLog();
//			for(XOCEvent event : log.getAllEvents()){
//				String eventId = event.getAttributes().get(xConstants.ID).toString();
//				System.out.println("eventId in updateConstraintInstances-aoc: " + eventId);
//				System.out.println("constraintId in updateConstraintInstances-aoc: " + constraintId);
//				for(String relationId : eventIdVSRelationIdVSObjectVSEventList.get(eventId).keySet()){
//					System.out.println("relationId in updateConstraintInstances-aoc: " + relationId);
//				}
//				if(eventIdVSRelationIdVSObjectVSEventList.get(eventId).containsKey(constraintId)){
//					DefaultMutableTreeNode eventAndObjectHierarchy = generateEventAndObjectHierarchy("(" + (i++) + ")" + eventId, 
//							eventIdVSRelationIdVSObjectVSEventList.get(eventId).get(constraintId));
//					root.add(eventAndObjectHierarchy);
//				}
//				else{
//					DefaultMutableTreeNode eventAndObjectHierarchy = generateEventAndObjectHierarchy("(" + (i++) + ")" + eventId, 
//							new HashMap<String, List<String>>());
//					root.add(eventAndObjectHierarchy);
//				}
//			}	
		}	
		
		mdtree.expandRow(0);
		// Expand all top-level rows of the Metadata Tree.
//		for (int i = 0; i < mdtree.getRowCount(); i++) {
//			mdtree.expandRow(i);
//		}
		
		this.add(treePane);
	}
	
	/**
	 * Generates a node for a selected class edge to show source and target objects (with cardinalities and corresponding life cycle periods).
	 * @param classEdgeId
	 * @param nodeName
	 * @return
	 */
	public DefaultMutableTreeNode generateInstanceHierarchyForClassRelation(String classEdgeId, OCBCModel model){
		System.out.println("generateInstanceHierarchyForClassRelation in ConstraintInstancePanel of org.processmining.ocbc.discovery.panels: " + classEdgeId);
		DefaultMutableTreeNode root = new DefaultMutableTreeNode("selected constraint id: " + classEdgeId);
		
		ClassEdgeCardinalityDiscovery classEdgeCardinalityDiscovery = model.getModelDiscovery().getClassModelDiscovery()
				.getClassEdgeCardinalityDiscovery();
		Map<XOCObject, Map<IntegerPair, Integer>> sourceObjectVSPeriodVSCardinality =  classEdgeCardinalityDiscovery
				.getClassRelationIdVSSourceObjectVSPeriodVSCardinalityMap().get(classEdgeId);
		Map<XOCObject, List<IntegerPair>> sourceObjectVSPeriodList =  classEdgeCardinalityDiscovery
				.getClassRelationIdVSSourceObjectVSPeriodList().get(classEdgeId);
		
		Map<XOCObject, Map<IntegerPair, Integer>> targetObjectVSPeriodVSCardinality =  classEdgeCardinalityDiscovery
				.getClassRelationIdVSTargetObjectVSPeriodVSCardinalityMap().get(classEdgeId);
		Map<XOCObject, List<IntegerPair>> targetObjectVSPeriodList =  classEdgeCardinalityDiscovery
				.getClassRelationIdVSTargetObjectVSPeriodList().get(classEdgeId);
		
		XOCLog log = model.getModelSourceLog();
		
		//add source object 
		DefaultMutableTreeNode sourceObjectListNode = new DefaultMutableTreeNode(
				new TreeEntry(mxConstants.source_object_list));
		for(XOCObject object : sourceObjectVSPeriodList.keySet()){
			String objectId = object.getAttributes().get(xConstants.ID).toString();
			DefaultMutableTreeNode objectNode = new DefaultMutableTreeNode(
					new TreeEntry(objectId));
			for(IntegerPair period : sourceObjectVSPeriodList.get(object)){
				int startEventOrderNum = period.getFirstValue();
				int endEventOrderNum = period.getSecondValue() == null ? log.size() : period.getSecondValue();
				int value = sourceObjectVSPeriodVSCardinality.get(object).get(period);
				objectNode.add(new DefaultMutableTreeNode(new TreeEntry("(" + mxConstants.startEventOrderNum + " : " + startEventOrderNum + " , "
						+ mxConstants.endEventOrderNum + " : " + endEventOrderNum + "): " + value)));

			}
			sourceObjectListNode.add(objectNode);
		}
		root.add(sourceObjectListNode);
		
		//add target object 
		DefaultMutableTreeNode targetObjectListNode = new DefaultMutableTreeNode(
				new TreeEntry(mxConstants.target_object_list));
		for(XOCObject object : targetObjectVSPeriodList.keySet()){
			String objectId = object.getAttributes().get(xConstants.ID).toString();
			DefaultMutableTreeNode objectNode = new DefaultMutableTreeNode(
					new TreeEntry(objectId));
			for(IntegerPair period : targetObjectVSPeriodList.get(object)){
				int startEventOrderNum = period.getFirstValue();
				int endEventOrderNum = period.getSecondValue() == null ? log.size() : period.getSecondValue();
				int value = targetObjectVSPeriodVSCardinality.get(object).get(period);
				objectNode.add(new DefaultMutableTreeNode(new TreeEntry("(" + mxConstants.startEventOrderNum + " : " + startEventOrderNum + " , "
						+ mxConstants.endEventOrderNum + " : " + endEventOrderNum + "):" + value)));

			}
			targetObjectListNode.add(objectNode);
		}
		root.add(targetObjectListNode);
		
		return root;
	}
	
	/**
	 * Generates a node for a selected AOC edge to show events and objects (with cardinalities and corresponding life cycle periods).
	 * @param aOCEdgeId
	 * @param nodeName
	 * @return
	 */
	public DefaultMutableTreeNode generateInstanceHierarchyForAOCRelation(String edgeId, OCBCModel model){
		System.out.println("generateInstanceHierarchyForAOCRelation in ConstraintInstancePanel of org.processmining.ocbc.discovery.panels: " + edgeId);
		DefaultMutableTreeNode root = new DefaultMutableTreeNode("selected constraint id: " + edgeId);
		
		Map<XOCEvent, List<XOCObject>> eventVSObjectList = model.getModelDiscovery().getAOCEdgeDiscovery()
				.getRelationIdVSEventVSObjectList().get(edgeId);
//
//		
//		Map<XOCObject, Map<IntegerPair, Integer>> targetObjectVSPeriodVSCardinality =  classEdgeCardinalityDiscovery
//				.getClassRelationIdVSTargetObjectVSPeriodVSCardinalityMap().get(edgeId);
//		Map<XOCObject, List<IntegerPair>> targetObjectVSPeriodList =  classEdgeCardinalityDiscovery
//				.getClassRelationIdVSTargetObjectVSPeriodList().get(edgeId);
		
		XOCLog log = model.getModelSourceLog();
		
		//add events
		DefaultMutableTreeNode eventListNode = new DefaultMutableTreeNode(
				new TreeEntry(mxConstants.event_list));
		for(XOCEvent event : eventVSObjectList.keySet()){
			String eventId = event.getAttributes().get(xConstants.ID).toString();
			DefaultMutableTreeNode eventNode = new DefaultMutableTreeNode(
					new TreeEntry(xConstants.EVENT + ": " + eventId) + " ("  + eventVSObjectList.get(event).size() + ")" );
			for(XOCObject object : eventVSObjectList.get(event)){
				String objectId = object.getAttributes().get(xConstants.ID).toString();
				eventNode.add(new DefaultMutableTreeNode(new TreeEntry(xConstants.OBJECT + ": "  + objectId)));
			}
			eventListNode.add(eventNode);
		}
		root.add(eventListNode);
		
//		//add target object 
//		DefaultMutableTreeNode targetObjectListNode = new DefaultMutableTreeNode(
//				new TreeEntry(mxConstants.target_object_list));
//		for(XOCObject object : targetObjectVSPeriodList.keySet()){
//			String objectId = object.getAttributes().get(xConstants.ID).toString();
//			DefaultMutableTreeNode objectNode = new DefaultMutableTreeNode(
//					new TreeEntry(objectId));
//			for(IntegerPair period : targetObjectVSPeriodList.get(object)){
//				int startEventOrderNum = period.getFirstValue();
//				int endEventOrderNum = period.getSecondValue() == null ? log.size() : period.getSecondValue();
//				int value = targetObjectVSPeriodVSCardinality.get(object).get(period);
//				objectNode.add(new DefaultMutableTreeNode(new TreeEntry("(" + mxConstants.startEventOrderNum + " : " + startEventOrderNum + " , "
//						+ mxConstants.endEventOrderNum + " : " + endEventOrderNum + "):" + value)));
//
//			}
//			targetObjectListNode.add(objectNode);
//		}
//		root.add(targetObjectListNode);
		
		return root;
	}
	
	/**
	 * Generates a node for a selected activity edge to show reference events and related target events.
	 * @param aOCEdgeId
	 * @param nodeName
	 * @return
	 */
	public DefaultMutableTreeNode generateInstanceHierarchyForActivityRelation(String edgeId, OCBCModel model){
		System.out.println("generateInstanceHierarchyForActivityRelation in ConstraintInstancePanel of org.processmining.ocbc.discovery.panels: " + edgeId);
		DefaultMutableTreeNode root = new DefaultMutableTreeNode("selected constraint id: " + edgeId);
		
		Map<XOCEvent, List<XOCEvent>> eventVSPrecedenceEventList=  model.getModelDiscovery().getBehaviorModelDiscovery().getActivityEdgeVSEventVSPrecedenceEventList().get(edgeId);		
		Map<XOCEvent, List<XOCEvent>> eventVSResponseEventList=  model.getModelDiscovery().getBehaviorModelDiscovery().getActivityEdgeVSEventVSResponseEventList().get(edgeId);
		
		for(XOCEvent event : eventVSPrecedenceEventList.keySet()){
			String eventId = event.getAttributes().get(xConstants.ID).toString();
			DefaultMutableTreeNode eventAndRelatedEventListNode = generateEventInstanceNode(eventId,
					eventVSPrecedenceEventList.get(event), eventVSResponseEventList.get(event));
			root.add(eventAndRelatedEventListNode);
		}
	
		
//		BehaviorModelDiscovery behavioralModelDiscovery = model.getModelDiscovery().getBehaviorModelDiscovery();
//		Map<XOCObject, Map<IntegerPair, Integer>> sourceObjectVSPeriodVSCardinality =  classEdgeCardinalityDiscovery
//				.getClassRelationIdVSSourceObjectVSPeriodVSCardinalityMap().get(classEdgeId);
//		Map<XOCObject, List<IntegerPair>> sourceObjectVSPeriodList =  classEdgeCardinalityDiscovery
//				.getClassRelationIdVSSourceObjectVSPeriodList().get(classEdgeId);
//		
//		Map<XOCObject, Map<IntegerPair, Integer>> targetObjectVSPeriodVSCardinality =  classEdgeCardinalityDiscovery
//				.getClassRelationIdVSTargetObjectVSPeriodVSCardinalityMap().get(classEdgeId);
//		Map<XOCObject, List<IntegerPair>> targetObjectVSPeriodList =  classEdgeCardinalityDiscovery
//				.getClassRelationIdVSTargetObjectVSPeriodList().get(classEdgeId);
//		
//		XOCLog log = model.getModelSourceLog();
//		
//		//add source object 
//		DefaultMutableTreeNode sourceObjectListNode = new DefaultMutableTreeNode(
//				new TreeEntry(mxConstants.source_object_list));
//		for(XOCObject object : sourceObjectVSPeriodList.keySet()){
//			String objectId = object.getAttributes().get(xConstants.ID).toString();
//			DefaultMutableTreeNode objectNode = new DefaultMutableTreeNode(
//					new TreeEntry(objectId));
//			for(IntegerPair period : sourceObjectVSPeriodList.get(object)){
//				int startEventOrderNum = period.getFirstValue();
//				int endEventOrderNum = period.getSecondValue() == null ? log.size() : period.getSecondValue();
//				int value = sourceObjectVSPeriodVSCardinality.get(object).get(period);
//				objectNode.add(new DefaultMutableTreeNode(new TreeEntry("(" + mxConstants.startEventOrderNum + " : " + startEventOrderNum + " , "
//						+ mxConstants.endEventOrderNum + " : " + endEventOrderNum + "): " + value)));
//
//			}
//			sourceObjectListNode.add(objectNode);
//		}
//		root.add(sourceObjectListNode);
//		
//		//add target object 
//		DefaultMutableTreeNode targetObjectListNode = new DefaultMutableTreeNode(
//				new TreeEntry(mxConstants.target_object_list));
//		for(XOCObject object : targetObjectVSPeriodList.keySet()){
//			String objectId = object.getAttributes().get(xConstants.ID).toString();
//			DefaultMutableTreeNode objectNode = new DefaultMutableTreeNode(
//					new TreeEntry(objectId));
//			for(IntegerPair period : targetObjectVSPeriodList.get(object)){
//				int startEventOrderNum = period.getFirstValue();
//				int endEventOrderNum = period.getSecondValue() == null ? log.size() : period.getSecondValue();
//				int value = targetObjectVSPeriodVSCardinality.get(object).get(period);
//				objectNode.add(new DefaultMutableTreeNode(new TreeEntry("(" + mxConstants.startEventOrderNum + " : " + startEventOrderNum + " , "
//						+ mxConstants.endEventOrderNum + " : " + endEventOrderNum + "):" + value)));
//
//			}
//			targetObjectListNode.add(objectNode);
//		}
//		root.add(targetObjectListNode);
		
		return root;
	}
	
	/**
	 * update the instance sub-panel to show all the source objects and target objects.
	 * Each object with all observed cardinalities and corresponding life cycle period.
	 * @param selectedCell
	 * @param model
	 */
	public void updateConstraintInstances(mxCell selectedCell, OCBCModel model) {
		System.out.println("updateConstraintInstances selectedCell Id: " + selectedCell.getId());
		String type = selectedCell.getAttributes().get(mxConstants.TYPE);
		String constraintId = selectedCell.getId();
		System.out.println("updateConstraintInstances type: " + type);
		setOpaque(false);
		setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
		setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
		this.removeAll();

		// Create the root node of the tree.
		DefaultMutableTreeNode root = null;


		
		if(type.equals(mxConstants.ACTIVITYRELATION)){
			
			root = generateInstanceHierarchyForActivityRelation(constraintId, model);	
//			if(additionalType.equals(mxConstants.PRECEDENCE)){
//				//add the PrecedenceEventList
//				Map<String, List<String>> eventVSPrecedenceEventList=  model.getModelDiscoveryResult().getActivityEdgeVSEventVSPrecedenceEventList().get(constraintId);		
//				for(String eventId : eventVSPrecedenceEventList.keySet()){
//					DefaultMutableTreeNode eventAndRelatedEventListNode = generateEventInstanceNode(eventId, eventVSPrecedenceEventList.get(eventId));
//					root.add(eventAndRelatedEventListNode);
//				}	
//			}
//			else{
//				//add the ResponseEventList
//				Map<String, List<String>> eventVSResponseEventList=  model.getModelDiscoveryResult().getActivityEdgeVSEventVSResponseEventList().get(constraintId);
//				for(String eventId : eventVSResponseEventList.keySet()){
//					DefaultMutableTreeNode eventAndRelatedEventListNode = generateEventInstanceNode(eventId, eventVSResponseEventList.get(eventId));
//					root.add(eventAndRelatedEventListNode);
//				}	
//			}
//				
//			//if secondId exists, add corresponding event instances
//			String secondId = selectedCell.getAttributes().get(mxConstants.SECONDID);
//			if(secondId != null){
//				//add the PrecedenceEventList
//				Map<String, List<String>> eventVSPrecedenceEventListSecond=  model.getModelDiscoveryResult().getActivityEdgeVSEventVSPrecedenceEventList().get(secondId);		
//				for(String eventId : eventVSPrecedenceEventListSecond.keySet()){
//					DefaultMutableTreeNode eventAndRelatedEventListNodeSecond = generateEventInstanceNode(eventId + "(precedence)", eventVSPrecedenceEventListSecond.get(eventId));
//					root.add(eventAndRelatedEventListNodeSecond);
//				}	
//				//add the ResponseEventList
//				Map<String, List<String>> eventVSResponseEventListSecond =  model.getModelDiscoveryResult().getActivityEdgeVSEventVSResponseEventList().get(secondId);
//				for(String eventId : eventVSResponseEventListSecond.keySet()){
//					DefaultMutableTreeNode eventAndRelatedEventListNodeSecond = generateEventInstanceNode(eventId + "(response)", eventVSResponseEventListSecond.get(eventId));
//					root.add(eventAndRelatedEventListNodeSecond);
//				}	
//			}
		}
		
		else if(type.equals(mxConstants.CLASSRELATION)){
			root = generateInstanceHierarchyForClassRelation(constraintId, model);		
		}								
		
		else if(type.equals(mxConstants.AOCRELATION)){
			root = generateInstanceHierarchyForAOCRelation(constraintId, model);	
			//add the PrecedenceEventList
//			Map<String, Map<String, Map<String, List<String>>>> eventIdVSRelationIdVSObjectVSEventList =  model.getModelDiscovery()
//					.getAOCEdgeDiscovery().getEventIdVSRelationIdVSObjectVSEventList();
//			Map<String, Map<String, List<String>>> relationIdVSEventVSRelatedObjectList = model.getModelDiscovery()
//					.getAOCEdgeDiscovery().getRelationIdVSEventVSObjectList();
//			mxCell sourceCell = (mxCell) selectedCell.getSource();
//			String sourceName = (String) sourceCell.getValue();
//			if(sourceName.equals(reference) && relationIdVSEventVSRelatedObjectList.containsKey(constraintId)){
//				Map<String, List<String>> eventVSObjectList = relationIdVSEventVSRelatedObjectList.get(constraintId);
//				addNodeAndRelatedNodeHierarchy(root, eventVSObjectList);
//			}
//			else if(additionalType != null){
//				XOCLog log = model.getModelSourceLog();				
//				String selectedEventId = additionalType.split("\\)")[1];
//				for(XOCEvent event : log.getAllEvents()){
//					String eventId = event.getAttributes().get(xConstants.ID).toString();
//					if(eventId.equals(selectedEventId) && eventIdVSRelationIdVSObjectVSEventList.get(eventId).containsKey(constraintId)){
//						Map<String, List<String>> ObjectIdVSRelatedEventIdList = eventIdVSRelationIdVSObjectVSEventList.get(eventId).get(constraintId);								
//						addNodeAndRelatedNodeHierarchy(root, ObjectIdVSRelatedEventIdList);
//					}
//				}	
//			}

			
//			XOCLog log = model.getModelSourceLog();
//			for(XOCEvent event : log.getAllEvents()){
//				String eventId = event.getAttributes().get(xConstants.ID).toString();
//				System.out.println("eventId in updateConstraintInstances-aoc: " + eventId);
//				System.out.println("constraintId in updateConstraintInstances-aoc: " + constraintId);
//				for(String relationId : eventIdVSRelationIdVSObjectVSEventList.get(eventId).keySet()){
//					System.out.println("relationId in updateConstraintInstances-aoc: " + relationId);
//				}
//				if(eventIdVSRelationIdVSObjectVSEventList.get(eventId).containsKey(constraintId)){
//					DefaultMutableTreeNode eventAndObjectHierarchy = generateEventAndObjectHierarchy("(" + (i++) + ")" + eventId, 
//							eventIdVSRelationIdVSObjectVSEventList.get(eventId).get(constraintId));
//					root.add(eventAndObjectHierarchy);
//				}
//				else{
//					DefaultMutableTreeNode eventAndObjectHierarchy = generateEventAndObjectHierarchy("(" + (i++) + ")" + eventId, 
//							new HashMap<String, List<String>>());
//					root.add(eventAndObjectHierarchy);
//				}
//			}	
		}	
		
		// Create metadata tree.
		mdtree = new JTree(root);
		mdtree.setBackground(COLOR_ENCLOSURE_BG);
		mdtree.setForeground(Color.WHITE);
		DefaultTreeCellRenderer renderer = (DefaultTreeCellRenderer) mdtree
				.getCellRenderer();
		renderer.setBackground(COLOR_ENCLOSURE_BG);
		renderer
				.setBackgroundNonSelectionColor(COLOR_ENCLOSURE_BG);
		renderer
				.setBackgroundSelectionColor(COLOR_LIST_SELECTION_BG);
		renderer.setForeground(Color.WHITE);
		renderer.setTextNonSelectionColor(Color.WHITE);
		renderer.setTextSelectionColor(COLOR_LIST_SELECTION_FG);


		JScrollPane treePane= new JScrollPane(mdtree);
		treePane.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
		treePane.setOpaque(false);
		treePane
				.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);
		treePane
				.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		JScrollBar vBar = treePane.getVerticalScrollBar();
		vBar.setUI(new SlickerScrollBarUI(vBar,
				COLOR_ENCLOSURE_BG, new Color(160, 160, 160),
				COLOR_NON_FOCUS, 4, 12));
		JScrollBar hBar = treePane.getHorizontalScrollBar();
		hBar.setUI(new SlickerScrollBarUI(hBar,
				COLOR_ENCLOSURE_BG, new Color(160, 160, 160),
				COLOR_NON_FOCUS, 4, 12));
		mdtree.expandRow(0);
		// Expand all top-level rows of the Metadata Tree.
//		for (int i = 0; i < mdtree.getRowCount(); i++) {
//			mdtree.expandRow(i);
//		}
		
		this.add(treePane);
	}
	
	private DefaultMutableTreeNode generateEventInstanceNode(String eventId, List<XOCEvent> relatedPrecedenceEventList, List<XOCEvent> relatedResponseEventList) {
		DefaultMutableTreeNode eventInstanceNode = new DefaultMutableTreeNode(new TreeEntry(eventId));
		
		DefaultMutableTreeNode precedenceEventListNode = new DefaultMutableTreeNode(new TreeEntry(mxConstants.PRECEDENCE));
		for(XOCEvent relatedEvent : relatedPrecedenceEventList){
			String relatedEventId = relatedEvent.getAttributes().get(xConstants.ID).toString();
			precedenceEventListNode.add(new DefaultMutableTreeNode(new TreeEntry("event id: " + relatedEventId)));
		}		
		eventInstanceNode.add(precedenceEventListNode);
		
		DefaultMutableTreeNode responseEventListNode = new DefaultMutableTreeNode(new TreeEntry(mxConstants.RESPONSE));
		for(XOCEvent relatedEvent : relatedResponseEventList){
			String relatedEventId = relatedEvent.getAttributes().get(xConstants.ID).toString();
			responseEventListNode.add(new DefaultMutableTreeNode(new TreeEntry("event id: " + relatedEventId)));
		}		
		eventInstanceNode.add(responseEventListNode);
		
		return eventInstanceNode;
	}
	
	private DefaultMutableTreeNode generateEventAndObjectHierarchy(String eventId,  Map<String, List<String>>  objectIdVSRelatedObjectIdList) {
		DefaultMutableTreeNode eventInstanceNode = new DefaultMutableTreeNode(new TreeEntry(eventId));
		for(String objectId : objectIdVSRelatedObjectIdList.keySet()){			
			DefaultMutableTreeNode objectNode = new DefaultMutableTreeNode(objectId);
			eventInstanceNode.add(objectNode);
			for(String relatedObjectId : objectIdVSRelatedObjectIdList.get(objectId)){
				objectNode.add(new DefaultMutableTreeNode(new TreeEntry(relatedObjectId)));
			}
		}		
		return eventInstanceNode;
	}
	
	private void addNodeAndRelatedNodeHierarchy(DefaultMutableTreeNode root,  Map<String, List<String>>  nodeVSRelatedNodeList) {
		for(String nodeId : nodeVSRelatedNodeList.keySet()){			
			DefaultMutableTreeNode node = new DefaultMutableTreeNode(nodeId);
			root.add(node);
			for(String relatedNodeId : nodeVSRelatedNodeList.get(nodeId)){
				node.add(new DefaultMutableTreeNode(new TreeEntry(relatedNodeId)));
			}
		}		
	}
	
	private DefaultMutableTreeNode generateAttributesNode(XAttributeMap attributes) {
		DefaultMutableTreeNode attributesNode = new DefaultMutableTreeNode(new TreeEntry(TREE_NODE_ATTRIBUTES));
		for(String key : attributes.keySet()){
			attributesNode.add(new DefaultMutableTreeNode(new TreeEntry(key + " : " + attributes.get(key).toString())));
		}		
		return attributesNode;
	}
	
	private DefaultMutableTreeNode generateReferredEventsNode(XOCObject object, XOCLog log, XOCEvent selectedEvent) {
		DefaultMutableTreeNode referredEventsNode = new DefaultMutableTreeNode(new TreeEntry(TREE_NODE_REFERREDEVENTS));
		for(XOCEvent event : log.getAllEvents()){
			if(event.getOrderNum() <= selectedEvent.getOrderNum()){
				for(XOCObject reference : event.getRelatedObjects()){
					if(reference.getAttributes().get("id").toString().equals(object.getAttributes().get("id").toString())){
						referredEventsNode.add(new DefaultMutableTreeNode(new TreeEntry(event.getAttributes().get("id").toString())));
					}
				}			
			}
		}		
		return referredEventsNode;
	}
	

	
	
	
	private DefaultMutableTreeNode generateObjectNode(XOCObject object){
		DefaultMutableTreeNode objectNode = new DefaultMutableTreeNode(new TreeEntry(object.getAttributes().get("id").toString()));
		for(String key : object.getAttributes().keySet()){
			objectNode.add(new DefaultMutableTreeNode(new TreeEntry(key + " : " + object.getAttributes().get(key).toString())));
		}		
		return  objectNode;
	}
	
	private DefaultMutableTreeNode generateObjectSetNode(List<XOCObject> objects, String name){
		DefaultMutableTreeNode objectSetNode = new DefaultMutableTreeNode(new TreeEntry(name));
		for (XOCObject object : objects) {
			objectSetNode.add(generateObjectNode(object));
		}
		return objectSetNode;
	}

	private DefaultMutableTreeNode generateRelationNode(XOCORelation relation){
		DefaultMutableTreeNode relationNode = new DefaultMutableTreeNode(new TreeEntry(relation.getAttributes().get("id").toString()));
		for(String key : relation.getAttributes().keySet()){
			relationNode.add(new DefaultMutableTreeNode(new TreeEntry(key + " : " + relation.getAttributes().get(key).toString())));
		}		
		return  relationNode;
	}
	
	private DefaultMutableTreeNode generateRelationSetNode(List<XOCORelation> relations, String name){
		DefaultMutableTreeNode relationSetNode = new DefaultMutableTreeNode(new TreeEntry(name));
		for (XOCORelation relation : relations) {
			relationSetNode.add(generateRelationNode(relation));
		}
		return relationSetNode;
	}
	
	private class TreeEntry {
		private String key;
		private String displaytext;
		
		public TreeEntry(String key) {
			this(key, key);
		}
		
		public TreeEntry(String key, String displaytext) {
			this.key = key;
			this.displaytext = displaytext;
		}
		
		public String getKey() {
			return this.key;
		}
		
		public String getDisplayText() {
			return this.displaytext;
		}
		
		public String toString() {
			return getDisplayText();
		}
	}
	
}