package org.processmining.ocbc.discovery.panels;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagLayout;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JFrame;
import javax.swing.JPanel;

import com.fluxicon.slickerbox.factory.SlickerFactory;

/**
 * The first panel of the ActiTraC-preferences-wizard.
 * 
 * @author Niels Lambrigts (niels.lambrigts@gmail.com)
 * @author Jochen De Weerdt (Jochen.DeWeerdt@econ.kuleuven.be)
 * @date 2011-08-01
 */
public class DiscoveryParameterPanel extends JPanel {

	private static final long serialVersionUID = -5858631772351410585L;
	
    private SliderPanel sliderPanel;
    private MetricsAlgorithmPanel metricsAlgorithmPanel;

	/**
	 * Creates the entire first panel of the settings.
	 */
	public DiscoveryParameterPanel() {
		this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		this.setOpaque(false);
		this.add(Box.createVerticalStrut(10));

		SlickerFactory slicker = SlickerFactory.instance();
		JPanel configurationPanel = slicker.createRoundedPanel(15, Color.GRAY);
		configurationPanel.setLayout(new GridBagLayout());
		metricsAlgorithmPanel = new MetricsAlgorithmPanel(null);
		sliderPanel = new SliderPanel( null);
//		buttonPanel = new ButtonPanel( null);
		metricsAlgorithmPanel.setBackground(Color.GRAY);

		metricsAlgorithmPanel.getFitnessAlgorithmBox().setBackground(Color.LIGHT_GRAY);
		metricsAlgorithmPanel.getPrecisionAlgorithmBox().setBackground(Color.LIGHT_GRAY);
		metricsAlgorithmPanel.getGeneralizationAlgorithmBox().setBackground(Color.LIGHT_GRAY);
		
//		metricsAlgorithmPanel.getFitnessAlgorithmBox().setMaximumSize(new Dimension(100,5));
//		metricsAlgorithmPanel.getPrecisionAlgorithmBox().setMaximumSize(new Dimension(100,5));
//		metricsAlgorithmPanel.getGeneralizationAlgorithmBox().setMaximumSize(new Dimension(100,5));
		
		metricsAlgorithmPanel.setPreferredSize(new Dimension(800, 100));
		sliderPanel.setPreferredSize(new Dimension(800, 400));
		
		sliderPanel.getFitnessSlider().setBackground(Color.LIGHT_GRAY);
		sliderPanel.getPrecisionSlider().setBackground(Color.LIGHT_GRAY);
		sliderPanel.getGeneralizationSlider().setBackground(Color.LIGHT_GRAY);
		
		this.add(metricsAlgorithmPanel);
		this.add(sliderPanel);
	}
	
	/**
	 * get the slider panel which control the weights of fitness, precision and generalization
	 * @return
	 */
	public SliderPanel getSliderPanel(){
		return sliderPanel;
	}
	
	/**
	 * get the algorithm  panel which control the algorithm (with corresponding threshold) to compute fitness, precision and generalization
	 * @return
	 */
	public MetricsAlgorithmPanel getMetricsAlgorithmPanel(){
		return metricsAlgorithmPanel;
	}	 
	

    public static void main(String[] args) {
        JFrame frame = new JFrame("Hello world");
       
        DiscoveryParameterPanel panel = new DiscoveryParameterPanel();
       
        frame.add(new JPanel().add(panel));

        frame.setSize(1600, 1400);
        frame.setVisible(true);
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
     }

}