package org.processmining.ocbc.discovery.panels;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JSplitPane;
import javax.swing.plaf.basic.BasicComboBoxRenderer;

import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartFrame;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.CategoryAxis;
import org.jfree.chart.axis.ValueAxis;
import org.jfree.chart.plot.CategoryPlot;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.title.LegendTitle;
import org.jfree.chart.title.TextTitle;
import org.jfree.data.category.CategoryDataset;
import org.jfree.data.category.DefaultCategoryDataset;
import org.processmining.mxgraph.examples.swing.GraphEditor;
import org.processmining.mxgraph.model.mxCell;
import org.processmining.mxgraph.model.mxGraphModel;
import org.processmining.mxgraph.util.mxConstants;
import org.processmining.ocbc.log.model.XOCEvent;
import org.processmining.ocbc.log.model.XOCLog;
import org.processmining.ocbc.log.model.impl.xConstants;
import org.processmining.ocbc.models.AOCEdge;
import org.processmining.ocbc.models.ActivityEdge;
import org.processmining.ocbc.models.ClassEdge;
import org.processmining.ocbc.models.OCBCModel;

import com.fluxicon.slickerbox.components.RoundedPanel;

public class DistributionPanel extends RoundedPanel {
//public class DistributionPanel extends Panel {
	//public class DistributionPanel extends JScrollPane {
//public class ConstraintInstancePanel extends Panel {

	
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	
	public static final Color COLOR_ENCLOSURE_BG = new Color(40, 40, 40);
	public static final Color COLOR_NON_FOCUS = new Color(70, 70, 70);
	public static final Color COLOR_LIST_BG = new Color(60, 60, 60);
	public static final Color COLOR_LIST_BG_LOWER = new Color(45, 45, 45);
	public static final Color COLOR_LIST_FG = new Color(180, 180, 180);
	public static final Color COLOR_LIST_SELECTION_BG = new Color(80, 0, 0);
	public static final Color COLOR_LIST_SELECTION_BG_LOWER = new Color(30, 10, 10);
	public static final Color COLOR_LIST_SELECTION_FG = new Color(240, 240, 240);

	
	private JComboBox<String> firstBox = new JComboBox<String>();
	private JComboBox<String> secondBox = new JComboBox<String>();
	private JComboBox<String> thirdBox = new JComboBox<String>();
	private JComboBox<String> fourthBox = new JComboBox<String>();
	private String thirdBoxType;
	private JLabel firstLabel = new JLabel("source");
	private JLabel secondLabel = new JLabel("target");
	private JLabel thirdLabel = new JLabel("constraint");
	private JLabel fourthLabel = new JLabel("type");
	private OCBCModel model;
	private JSplitPane completePanel;
	private mxCell selectedCell;
	private GraphEditor editor;
	public DistributionPanel(int aRadius, int anOuterBorder, int anInnerBorder, final OCBCModel model, final GraphEditor editor){
		super(aRadius, anOuterBorder, anInnerBorder);
		this.setMaximumSize(new Dimension(230, 1000));
		this.setPreferredSize(new Dimension(230, 1000));
		this.setLayout(new GridLayout(1,2));
		this.model = model;
		this.editor = editor;
		JFreeChart mBarChart = createBarChart(GetDataset(), "edge", "cardinality", "count");         
        ChartFrame mChartFrame = new ChartFrame("distribution", mBarChart);  

        JPanel controlPanel = new JPanel();
        controlPanel.setLayout(new GridLayout(2, 4));
        controlPanel.add(firstLabel);
        controlPanel.add(secondLabel);
        controlPanel.add(thirdLabel);
        controlPanel.add(fourthLabel);
        controlPanel.add(firstBox);
   
        firstBox.addActionListener(new ActionListener() {		
   			public void actionPerformed(ActionEvent e) {  	
   				System.out.println("first box item changes");
   				boxChangeListenerAction(1);
   			}
   		});
        
        controlPanel.add(secondBox);       
        secondBox.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent arg0) {
            	System.out.println("secondBox box item changes");
            	boxChangeListenerAction(2);
            }
        });

        thirdBox.addActionListener(new ActionListener() {		
   			public void actionPerformed(ActionEvent e) {  	
   				System.out.println("third box item changes");
   				boxChangeListenerAction(3);  				    				 
   			}
   		});

        fourthBox.addActionListener(new ActionListener() {		
   			public void actionPerformed(ActionEvent e) {  	
   				System.out.println("fourth box item changes");
   				boxChangeListenerAction(4);
   			}
   		});
        
        firstBox.setMaximumSize(firstBox.getPreferredSize());
//        thirdBox.setMaximumSize(thirdBox.getPreferredSize());
        
        firstBox.setToolTipText("select the source of a constraint");
        secondBox.setToolTipText("select the target of a constraint");
        thirdBox.setToolTipText("select a constraint");
        fourthBox.setToolTipText("select the type for a constraint");
        firstBox.setPrototypeDisplayValue("text here");
        secondBox.setPrototypeDisplayValue("text here");
        thirdBox.setPrototypeDisplayValue("text here");
        fourthBox.setPrototypeDisplayValue("text here");
        firstBox.setRenderer(new ComboBoxRenderer(firstBox));
        secondBox.setRenderer(new ComboBoxRenderer(secondBox));
        thirdBox.setRenderer(new ComboBoxRenderer(thirdBox));
        fourthBox.setRenderer(new ComboBoxRenderer(fourthBox));
//        thirdBox.setEditable(true);
        controlPanel.add(thirdBox);
        
        
        controlPanel.add(fourthBox);
        System.out.println("\n hello kfgz model.getAllActivityNames().size() kdf: " + model.getAllActivityNames().size());
        initializeDistributionPanelBoxes(model.getAllActivityNames(), model.getAllClassNames());
        
        completePanel = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
//        JScrollPane scrollPane = ;
//        completePanel.setTopComponent(new JScrollPane(controlPanel));
        completePanel.setTopComponent(controlPanel);
        completePanel.setBottomComponent(mChartFrame.getContentPane());
        completePanel.setResizeWeight(0.01);
        
//        this.add(controlPanel);
//		this.add(mChartFrame.getContentPane());		
		
		this.add(completePanel);	
		
	}

	public JFreeChart createBarChart(CategoryDataset mDataset, String title,
			String horizontalLabel, String verticalLabel){       
        /* 
        //创建主题样式 
        StandardChartTheme mChartTheme = new StandardChartTheme("CN"); 
        //设置标题字体 
        mChartTheme.setExtraLargeFont(new Font("黑体", Font.BOLD, 20)); 
        //设置轴向字体 
        mChartTheme.setLargeFont(new Font("宋体", Font.CENTER_BASELINE, 15)); 
        //设置图例字体 
        mChartTheme.setRegularFont(new Font("宋体", Font.CENTER_BASELINE, 15)); 
        //应用主题样式 
        ChartFactory.setChartTheme(mChartTheme);         
        ///////////////以上主题设置必须位于创建图表函数之前才能生效////////////         
         */  
        JFreeChart mBarChart = ChartFactory.createBarChart(  
        		title,  //图表标题  
        		horizontalLabel,       //横轴（目录轴）标签  
                verticalLabel,           //纵轴（数值轴）标签  
                mDataset,       //数据集  
                PlotOrientation.VERTICAL,   //图表方向  
                true,           //是否显示图例  
                true,           //是否生成提示工具  
                false);         //是否生成url连接  
        //图表标题设置  
        TextTitle mTextTitle = mBarChart.getTitle();  
        mTextTitle.setFont(new Font("黑体", Font.BOLD, 20));  
        //mBarChart.setTitle(new TextTitle("学校人员分布图",new Font("黑体", Font.BOLD, 20)));  
        //图表图例设置  
        LegendTitle mLegend = mBarChart.getLegend();  
        if(mLegend != null)  
            mLegend.setItemFont(new Font("宋体", Font.CENTER_BASELINE, 15));  
        //mBarChart.getLegend().setItemFont(new Font("宋体", Font.CENTER_BASELINE, 15));  
        //设置柱状图轴  
        CategoryPlot mPlot = (CategoryPlot)mBarChart.getPlot();  
          
        //x轴  
        CategoryAxis mDomainAxis = mPlot.getDomainAxis();  
        //设置x轴标题的字体  
        mDomainAxis.setLabelFont(new Font("宋体", Font.PLAIN, 15));  
        //设置x轴坐标字体  
        mDomainAxis.setTickLabelFont(new Font("宋体", Font.PLAIN, 15));  
        //y轴  
        ValueAxis mValueAxis = mPlot.getRangeAxis();  
        //设置y轴标题字体  
        mValueAxis.setLabelFont(new Font("宋体", Font.PLAIN, 15));  
        //设置y轴坐标字体  
        mValueAxis.setTickLabelFont(new Font("宋体", Font.PLAIN, 15));  
        //柱体显示数值  
//        BarRenderer mRenderer= new BarRenderer();  
//        mRenderer.setItemLabelGenerator(new StandardCategoryItemLabelGenerator());  
//        mRenderer.setItemLabelFont(new Font("宋体", Font.PLAIN, 10));  
//        mRenderer.setItemLabelsVisible(true);  
//        mPlot.setRenderer(mRenderer);  
        
        return mBarChart;
	}
	
    public static CategoryDataset GetDataset()  
    {  
        DefaultCategoryDataset mDataset = new DefaultCategoryDataset();  
//        mDataset.addValue(2000, "清华大学", "本科生");  
//        mDataset.addValue(2000, "北京大学", "本科生");  
//        mDataset.addValue(2000, "复旦大学", "本科生");  
//        mDataset.addValue(1500, "清华大学", "研究生");  
//        mDataset.addValue(1500, "北京大学", "研究生");  
//        mDataset.addValue(1500, "复旦大学", "研究生");  
//        mDataset.addValue(1000, "清华大学", "博士生");  
//        mDataset.addValue(1000, "北京大学", "博士生");  
//        mDataset.addValue(1000, "复旦大学", "博士生");  
//        mDataset.addValue(900, "清华大学", "讲师");  
//        mDataset.addValue(900, "北京大学", "讲师");  
//        mDataset.addValue(900, "复旦大学", "讲师");  
//        mDataset.addValue(800, "清华大学", "副教授");  
//        mDataset.addValue(800, "北京大学", "副教授");  
//        mDataset.addValue(800, "复旦大学", "副教授");  
//        mDataset.addValue(300, "清华大学", "教授");  
//        mDataset.addValue(300, "北京大学", "教授");  
//        mDataset.addValue(300, "复旦大学", "教授");  
//        mDataset.addValue(600, "清华大学", "行政人员");  
//        mDataset.addValue(600, "北京大学", "行政人员");  
//        mDataset.addValue(600, "复旦大学", "行政人员");  
//        mDataset.addValue(400, "清华大学", "管理人员");  
//        mDataset.addValue(400, "北京大学", "管理人员");  
//        mDataset.addValue(400, "复旦大学", "管理人员");  
        /* 
        mDataset.addValue(2000, "清华大学", "本科生"); 
        mDataset.addValue(1500, "清华大学", "研究生"); 
        mDataset.addValue(1000, "清华大学", "博士生"); 
        mDataset.addValue(900, "清华大学", "讲师"); 
        mDataset.addValue(800, "清华大学", "副教授"); 
        mDataset.addValue(300, "清华大学", "教授"); 
        mDataset.addValue(600, "清华大学", "行政人员"); 
        mDataset.addValue(400, "清华大学", "管理人员"); 
        */  
           
        return mDataset;  
    } 
    
    /*
     * initialize the three boxes, the first two contain all activities + classes
     */
    public void initializeDistributionPanelBoxes(List<String> activityList, List<String> classList){
    	firstBox.removeAllItems();
		secondBox.removeAllItems();
		for(String activity : activityList){
			firstBox.addItem(activity);
			secondBox.addItem(activity);
		}		
		for(String className : classList){
			firstBox.addItem(className);
			secondBox.addItem(className);
		}		
    }
    
    public void boxChangeListenerAction(int number){
    	//if the first or second box changes item
    	if(number == 1 || number == 2){
    		String selectedItemFirst = (String) firstBox.getSelectedItem();
    		String selectedItemSecond = (String) secondBox.getSelectedItem();
    		thirdBox.removeAllItems();
    		fourthBox.removeAllItems();
    		//items of two boxes are both activities
    		if(model.getAllActivityNames().contains(selectedItemFirst) 
    				&& model.getAllActivityNames().contains(selectedItemSecond)){
    			System.out.println("two box item are activities");
    			for(ActivityEdge activityRelation: model.getAllActivityRelations()){  						
    				String sourceNodeName = activityRelation.getReferenceNode().getNodeName();
    				String targetNodeName = activityRelation.getTargetNode().getNodeName();
    				thirdBoxType = mxConstants.ACTIVITYRELATION;
    				if(sourceNodeName.equals(selectedItemFirst) && targetNodeName.equals(selectedItemSecond)){
    					System.out.println("satisfied activity relations");
    					String activityRelationName = activityRelation.getID();
    					thirdBox.addItem(activityRelationName);
    				}

    			}
    			fourthBox.setEnabled(true);
        		fourthBox.addItem("response");
        		fourthBox.addItem("precedence");
            	  
    		}
    		//items of two boxes are both classes
    		else if(model.getAllClassNames().contains(selectedItemFirst) 
    				&& model.getAllClassNames().contains(selectedItemSecond)){
    			System.out.println("two box item are classes");  
    			thirdBoxType = mxConstants.CLASSRELATION;
    			for(ClassEdge classRelation: model.getAllClassRelations()){  						
    				String sourceNodeName = classRelation.getSourceNode().getNodeName();
    				String targetNodeName = classRelation.getTargetNode().getNodeName();
    				thirdBoxType = mxConstants.ACTIVITYRELATION;
    				if((sourceNodeName.equals(selectedItemFirst) && targetNodeName.equals(selectedItemSecond))
    						|| (sourceNodeName.equals(selectedItemSecond) && targetNodeName.equals(selectedItemFirst))){
    					System.out.println("satisfied activity relations");
    					String activityRelationName = classRelation.getID();
    					thirdBox.addItem(activityRelationName);
    				}

    			}
    			fourthBox.setEnabled(true);
    			int i = 1;
            	for(XOCEvent event : model.getModelSourceLog().getAllEvents()){
            		String eventId = event.getAttributes().get(xConstants.ID).toString();
            		fourthBox.addItem("(" + (i++) + ")" + eventId);
            	}  
    		}
    		//items of two boxes are one activity and one class
    		else if((model.getAllActivityNames().contains(selectedItemFirst) 
    				&& model.getAllClassNames().contains(selectedItemSecond))
    				|| (model.getAllClassNames().contains(selectedItemFirst) 
    				&& model.getAllActivityNames().contains(selectedItemSecond))){
    			System.out.println("two box item are activity and class");
    			thirdBoxType = mxConstants.AOCRELATION;
    			for(AOCEdge aOCRelation: model.getAllAOCRelations()){
    				String sourceNodeName = aOCRelation.getActivityNode().getNodeName();
    				String targetNodeName = aOCRelation.getClassNode().getNodeName();
    				if(sourceNodeName.equals(selectedItemFirst) && targetNodeName.equals(selectedItemSecond)){
    					System.out.println("satisfied activity relations");
    					String activityRelationName = aOCRelation.getID();
    					thirdBox.addItem(activityRelationName);
    					fourthBox.setEnabled(false);
    				}
    				else if(sourceNodeName.equals(selectedItemSecond) && targetNodeName.equals(selectedItemFirst)){
    					System.out.println("satisfied activity relations");
    					String activityRelationName = aOCRelation.getID();
    					thirdBox.addItem(activityRelationName);
    					fourthBox.setEnabled(true);
    					int i = 1;
    	            	for(XOCEvent event : model.getModelSourceLog().getAllEvents()){
    	            		String eventId = event.getAttributes().get(xConstants.ID).toString();
    	            		fourthBox.addItem("(" + (i++) + ")" + eventId);
    	            	} 
    				}
    			}
    		}
    	}
    	//if the third box changes item
    	else if(number == 3){
    		String selectedItemThird = (String) thirdBox.getSelectedItem();
			if(selectedItemThird != null){
				System.out.println("selectedItemThird: " + selectedItemThird);
   				mxGraphModel graphModel = (mxGraphModel) editor.getGraphComponent().getGraph().getModel();   			
   				selectedCell =   (mxCell) graphModel.getCell(selectedItemThird);
   				System.out.println("searchedCell: " + selectedCell.getId());
   				//highlight the selected cell and update the attribute panel
   				editor.restoreAllCells();
   				editor.highlightCell(selectedCell);
   				editor.getAttributePanel().updateAttributePanel(selectedCell);  				
  				String type = selectedCell.getAttributes().get(mxConstants.TYPE);
//   				if(type.equals(mxConstants.AOCRELATION)){
//   					updateDistributionPanelGraph(selectedCell, model);	 
//   				} 				
   				if(fourthBox.getSelectedItem() != null || type.equals(mxConstants.AOCRELATION)){
   					updateDistributionPanelGraph(selectedCell, model);
   					//update the instance panel
   					ConstraintInstancePanel constraintInstancePanel = ((ConstraintInstancePanel)editor.getLibraryPane().getComponentAt(1));
   					constraintInstancePanel.updateConstraintInstances(selectedCell, model, (String)firstBox.getSelectedItem(), (String)fourthBox.getSelectedItem());	
   				}
			}
    	}
    	//if the fourth box changes item
    	else if(number == 4){
			String selectedItemThird = (String) thirdBox.getSelectedItem();
			if(selectedItemThird != null){
				updateDistributionPanelGraph(selectedCell, model);		
				//update the instance panel
				ConstraintInstancePanel constraintInstancePanel = ((ConstraintInstancePanel)editor.getLibraryPane().getComponentAt(1));
				constraintInstancePanel.updateConstraintInstances(selectedCell, model, (String)firstBox.getSelectedItem(), (String)fourthBox.getSelectedItem());	
			}			
    	}  				  			
    }
    
    /*
     * update the boxes in the distribution panel, used in the click response action in the editor
     */
    public void updateDistributionPanelBoxes(mxCell selectedCell, OCBCModel model){
    	this.selectedCell = selectedCell;
		System.out.println("updateDistributionPanel selectedCell Id: " + selectedCell.getId());
		String type = selectedCell.getAttributes().get(mxConstants.TYPE);
		String cellId = selectedCell.getId();
		

		if(type.equals(mxConstants.ACTIVITYRELATION) || type.equals(mxConstants.CLASSRELATION)
				|| type.equals(mxConstants.AOCRELATION)){
			mxCell sourceCell = (mxCell) selectedCell.getSource();
			mxCell targetCell = (mxCell) selectedCell.getTarget();
			String sourceName = (String) sourceCell.getValue();
			String targetName = (String) targetCell.getValue();
			//update the boxes
			firstBox.setSelectedItem(sourceName);
			secondBox.setSelectedItem(targetName);
			thirdBox.setSelectedItem(cellId);
			//update the distribution panel
			updateDistributionPanelGraph(selectedCell, model);
			//update the instance panel
			ConstraintInstancePanel constraintInstancePanel = ((ConstraintInstancePanel)editor.getLibraryPane().getComponentAt(1));
			constraintInstancePanel.updateConstraintInstances(selectedCell, model, (String)firstBox.getSelectedItem(), (String)fourthBox.getSelectedItem());	
		}
		
		else if(type.equals(mxConstants.LABLE)){
			mxCell parentCell = (mxCell) selectedCell.getParent();
			mxCell sourceCell = (mxCell) parentCell.getSource();
			mxCell targetCell = (mxCell) parentCell.getTarget();
			String sourceName = (String) sourceCell.getValue();
			String targetName = (String) targetCell.getValue();
			String parentCellId = parentCell.getId();
			String position = selectedCell.getAttributes().get(mxConstants.CARDINALITYPOSITION);
			System.out.println("position: " + position);
			System.out.println("parentCellId: " + parentCellId);
			if(position.equals(mxConstants.TARGETCARDINALITY)){
				firstBox.setSelectedItem(sourceName);
				secondBox.setSelectedItem(targetName);
			}
			else{
				firstBox.setSelectedItem(targetName);
				secondBox.setSelectedItem(sourceName);
			}
			thirdBox.setSelectedItem(parentCellId);
			//update the distribution panel
			updateDistributionPanelGraph(parentCell, model);
			//update the instance panel
			ConstraintInstancePanel constraintInstancePanel = ((ConstraintInstancePanel)editor.getLibraryPane().getComponentAt(1));
			constraintInstancePanel.updateConstraintInstances(parentCell, model, (String)firstBox.getSelectedItem(), (String)fourthBox.getSelectedItem());	
		}	
    }
    
	/*
	 * elementId: the id (in the model) of selected cell
	 */
	public void updateDistributionPanelGraph(mxCell selectedCell, OCBCModel model) {
		this.selectedCell = selectedCell;
		System.out.println("updateDistributionPanel selectedCell Id: " + selectedCell.getId());
		DefaultCategoryDataset mDataset = new DefaultCategoryDataset();  
		Map<Integer, Integer> cardinalityVSCount = new HashMap<Integer, Integer>();
		String chartTitle;
		
		String type = selectedCell.getAttributes().get(mxConstants.TYPE);
		String constraintId = selectedCell.getId();
		mxCell sourceCell = (mxCell) selectedCell.getSource();
		mxCell targetCell = (mxCell) selectedCell.getTarget();
		String sourceName = (String) sourceCell.getValue();
		String targetName = (String) targetCell.getValue();
		
		String selectedReferenceClass = (String) firstBox.getSelectedItem();
		String selectedCardinalityClass = (String) secondBox.getSelectedItem();
		String selectedConstraint = (String) thirdBox.getSelectedItem();

		chartTitle = selectedCardinalityClass + " distribution for " + selectedReferenceClass;

		if(type.equals(mxConstants.ACTIVITYRELATION)){
			System.out.println("ACTIVITYRELATION  selected");


			String selectedType = (String) fourthBox.getSelectedItem();	
			//create the cardinality vs count map

//			Map<String, List<String>> eventVSEventList = null;
//			if(selectedType.equals(mxConstants.PRECEDENCE)){
//				eventVSEventList =  model.getModelDiscoveryResult().getActivityEdgeVSEventVSPrecedenceEventList().get(constraintId);		
//			}
//			else{
//				eventVSEventList =  model.getModelDiscoveryResult().getActivityEdgeVSEventVSResponseEventList().get(constraintId);		
//			}
//			for(String eventId : eventVSEventList.keySet()){
//				int cardinality = eventVSEventList.get(eventId).size();				
//				if(cardinalityVSCount.containsKey(cardinality)){
//					int count = cardinalityVSCount.get(cardinality);
//					cardinalityVSCount.put(cardinality, count + 1);
//				}
//				else{
//					cardinalityVSCount.put(cardinality, 1);
//				}
//			}


			//if secondId exists, add corresponding event instances
//			String secondId = selectedCell.getAttributes().get(mxConstants.SECONDID);
//			if(secondId != null){
//				//add the PrecedenceEventList
//				Map<String, List<String>> eventVSPrecedenceEventListSecond =  model.getModelDiscoveryResult().getActivityEdgeVSEventVSPrecedenceEventList().get(secondId);		
//				for(String eventId : eventVSPrecedenceEventListSecond.keySet()){
//
//				}	
//				//add the ResponseEventList
//				Map<String, List<String>> eventVSResponseEventListSecond =  model.getModelDiscoveryResult().getActivityEdgeVSEventVSResponseEventList().get(secondId);
//				for(String eventId : eventVSResponseEventListSecond.keySet()){
//
//				}	
//			}

		}
		else if(type.equals(mxConstants.CLASSRELATION) && fourthBox.getSelectedItem() != null){
			//add the PrecedenceEventList
//			Map<String, Map<String, Map<String, List<String>>>> eventIdVSRelationIdVSObjectIdVSRelatedObjectIdList =  model.getModelDiscovery()
//					.getClassModelDiscovery().getClassEdgeCardinalityDiscovery().getEventIdVSRelationIdVSObjectIdVSRelatedObjectIdList();	
		
//			Map<String, Map<String, Map<String, List<String>>>> relationIdVSEventIdVSObjectIdVSRelatedObjectIdList =  model.getModelDiscovery()
//					.getClassModelDiscovery().getRelationIdVSEventIdVSObjectIdVSRelatedObjectIdList();
			XOCLog log = model.getModelSourceLog();
			String selectedEventItem = (String) fourthBox.getSelectedItem();
			String eventId = selectedEventItem.split("\\)")[1];
			XOCEvent selectedEvent = log.getIdVSEventMap().get(eventId);
		
//			if(relationIdVSEventIdVSObjectIdVSRelatedObjectIdList.containsKey(constraintId) 
//					&& relationIdVSEventIdVSObjectIdVSRelatedObjectIdList.get(constraintId).containsKey(eventId)){
//				Map<String, List<String>> objectIdVSRelatedObjectIdList = relationIdVSEventIdVSObjectIdVSRelatedObjectIdList.get(constraintId).get(eventId);
////				Map<Integer, Integer> cardinalityVSCount = new HashMap<Integer, Integer>();
//				if(selectedReferenceClass != null && selectedCardinalityClass != null && selectedConstraint != null){
//					if(sourceName.equals(selectedReferenceClass)){
//						System.out.println("source end");
//						for(String objectId : objectIdVSRelatedObjectIdList.keySet()){
//							XOCObject object = selectedEvent.getObjectModel().getObjectById(objectId);
//							String classType = object.getAttributes().get(xConstants.CLASS).toString();					
//							if(classType.equals(sourceName)){
//								int cardinality = objectIdVSRelatedObjectIdList.get(objectId).size();
//								if(cardinalityVSCount.containsKey(cardinality)){
//									int count = cardinalityVSCount.get(cardinality);
//									cardinalityVSCount.put(cardinality, count + 1);
//								}
//								else{
//									cardinalityVSCount.put(cardinality, 1);
//								}								
//							}
//						}
//					}
//					else if(targetName.equals(selectedReferenceClass)){
//						System.out.println("target end");
//						for(String objectId : objectIdVSRelatedObjectIdList.keySet()){
//							XOCObject object = selectedEvent.getObjectModel().getObjectById(objectId);
//							String classType = object.getAttributes().get(xConstants.CLASS).toString();					
//							if(classType.equals(targetName)){
//								int cardinality = objectIdVSRelatedObjectIdList.get(objectId).size();								
//								if(cardinalityVSCount.containsKey(cardinality)){
//									int count = cardinalityVSCount.get(cardinality);
//									cardinalityVSCount.put(cardinality, count + 1);
//								}
//								else{
//									cardinalityVSCount.put(cardinality, 1);
//								}								
//							}
//						}
//					}				
//				}			
//			}	
		}	
		else if(type.equals(mxConstants.AOCRELATION) && selectedConstraint != null){
//			Map<String, Map<String, Map<String, List<String>>>> eventIdVSRelationIdVSObjectVSEventList =  model.getModelDiscovery()
//					.getAOCEdgeDiscovery().getEventIdVSRelationIdVSObjectVSEventList();	
//			Map<String, Map<XOCEvent, List<XOCObject>>> relationIdVSEventVSObjectList =  model.getModelDiscovery()
//					.getAOCEdgeDiscovery().getRelationIdVSEventVSObjectList();	
//			String selectedEventItem = (String) fourthBox.getSelectedItem();
//			if(selectedReferenceClass != null && selectedCardinalityClass != null && selectedConstraint != null){
//				if(targetName.equals(selectedReferenceClass) && selectedEventItem != null){				
//					String eventId = selectedEventItem.split("\\)")[1];
//					for(String constraintId2 : eventIdVSRelationIdVSObjectVSEventList.get(eventId).keySet()){
//						System.out.println("constraintId2: " + constraintId2);
//					}
//					if(eventIdVSRelationIdVSObjectVSEventList.get(eventId).containsKey(constraintId)){
//						Map<String, List<String>> objectIdVSEventList = eventIdVSRelationIdVSObjectVSEventList.get(eventId).get(constraintId);
//						System.out.println("source end");
//						System.out.println("constraintId: " + constraintId);
//						for(String objectId : objectIdVSEventList.keySet()){
//							int cardinality = objectIdVSEventList.get(objectId).size();
//							if(cardinalityVSCount.containsKey(cardinality)){
//								int count = cardinalityVSCount.get(cardinality);
//								cardinalityVSCount.put(cardinality, count + 1);
//							}
//							else{
//								cardinalityVSCount.put(cardinality, 1);
//							}		
//						}
//					}					
//				}
//				else if(sourceName.equals(selectedReferenceClass)){
//					Map<XOCEvent, List<XOCObject>> eventVSObjectList = relationIdVSEventVSObjectList.get(constraintId);
//					System.out.println("target end");
//					for(XOCEvent event : eventVSObjectList.keySet()){
//						int cardinality = eventVSObjectList.get(event).size();								
//						if(cardinalityVSCount.containsKey(cardinality)){
//							int count = cardinalityVSCount.get(cardinality);
//							cardinalityVSCount.put(cardinality, count + 1);
//						}
//						else{
//							cardinalityVSCount.put(cardinality, 1);
//						}		
//					}
//				}				
//			}			
		}	

		
		//create the data set for the graph
		if(cardinalityVSCount.keySet().size() > 0){
			int maximalCardinality = Collections.max(cardinalityVSCount.keySet());
			System.out.println("maximalCardinality in updateDistribution: " + maximalCardinality);
			for(int j=0; j<= maximalCardinality; j++){
				if(cardinalityVSCount.containsKey(j)){
					mDataset.addValue(cardinalityVSCount.get(j), "", j + "");  
				}
				else{
					mDataset.addValue(0, "", j + "");  
				}
				
			}
			if(maximalCardinality == 0){
				mDataset.addValue(0, "", 1 + ""); 
			}
		}
		
		//create the chart		
		JFreeChart mBarChart = createBarChart(mDataset, "", "cardinality", "count"); 
		mBarChart.removeLegend();
        ChartFrame mChartFrame = new ChartFrame("distribution", mBarChart);  
		completePanel.setBottomComponent(mChartFrame.getContentPane());
	}	
}

class ComboBoxRenderer extends BasicComboBoxRenderer
{
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	JComboBox<String> combo = new JComboBox<String>();
	
	public ComboBoxRenderer(JComboBox<String> combo){
		this.combo = combo;
	}
	
	public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus)
	{
		if (isSelected)
		{
			setBackground(list.getSelectionBackground());
			setForeground(list.getSelectionForeground());
			if (-1 < index)
			{
				if(combo.getItemAt(index) != null){
					list.setToolTipText(combo.getItemAt(index).toString());
				}			
			}
		}
		else
		{
			setBackground(list.getBackground());
			setForeground(list.getForeground());
		}
		setFont(list.getFont());
		setText((value == null) ? "" : value.toString());
		return this;
	}
}