package org.processmining.ocbc.discovery.panels;
import java.awt.Dimension;
import java.awt.GridLayout;

import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JTextField;

import org.processmining.common.frame.MyComboBox;
import org.processmining.mxgraph.examples.swing.editor.BasicGraphEditor;
import org.processmining.mxgraph.util.mxConstants;

public class MetricsAlgorithmPanel extends JPanel {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * pull down boxes to choose algorithm to compute metrics
	 */
	private MyComboBox fitnessAlgorithmBox = new MyComboBox();
	private MyComboBox precisionAlgorithmBox = new MyComboBox();
	private MyComboBox generalizationAlgorithmBox = new MyComboBox();
	
	private JTextField fitnessThresholdField = new JTextField();
	private JTextField precisionThresholdField = new JTextField();
	private JTextField generalizationThresholdField = new JTextField();
	
	public MetricsAlgorithmPanel(final BasicGraphEditor graphEditor) {
	    super();		
	    setLayout(new GridLayout(2, 3));		
		
		//add the boxes to choose algorithm to compute metrics
		fitnessAlgorithmBox.addItem(mxConstants.fitnessAlgorithmWithoutThreshold);
		fitnessAlgorithmBox.addItem(mxConstants.fitnessAlgorithmWithNumThreshold);
		fitnessAlgorithmBox.addItem(mxConstants.fitnessAlgorithmWithRatioThreshold);
		precisionAlgorithmBox.addItem(mxConstants.precisionAlgorithmWithNumThreshold);
		precisionAlgorithmBox.addItem(mxConstants.precisionAlgorithmWithRatioThreshold);
		precisionAlgorithmBox.addItem(mxConstants.precisionAlgorithmByEntropy);
		generalizationAlgorithmBox.addItem(mxConstants.generalizationAlgorithmBinomialDistribution);
		generalizationAlgorithmBox.addItem(mxConstants.generalizationAlgorithmBetaDistribution);
		
		fitnessThresholdField.setMaximumSize(new Dimension(50,10));
		precisionThresholdField.setMaximumSize(new Dimension(50,10));
		generalizationThresholdField.setMaximumSize(new Dimension(50,10));
		
		fitnessThresholdField.setToolTipText("Please input the threshold for the fitness algorithm.");
		precisionThresholdField.setToolTipText("Please input the threshold for the precision algorithm.");
		generalizationThresholdField.setToolTipText("Please input the threshold for the generalization algorithm.");
		
		precisionAlgorithmBox.setSelectedItem(mxConstants.precisionAlgorithmByEntropy);
		generalizationThresholdField.setText("0.95");
		
		this.add(fitnessAlgorithmBox);
		this.add(precisionAlgorithmBox);
		this.add(generalizationAlgorithmBox);
		
		this.add(fitnessThresholdField);
		this.add(precisionThresholdField);
		this.add(generalizationThresholdField);
	}

	/**
	 * get the threshold field for algorithms to compute fitness
	 * @return
	 */
	public JTextField getFitnessThresholdField(){
		return fitnessThresholdField;
	}
	
	/**
	 * get the threshold field for algorithms to compute precision
	 * @return
	 */
	public JTextField getPrecisionThresholdField(){
		return precisionThresholdField;
	}
	
	/**
	 * get the threshold field for algorithms to compute generalization
	 * @return
	 */
	public JTextField getGeneralizationThresholdField(){
		return generalizationThresholdField;
	}
	
	/**
	 * get the box of algorithms to compute fitness
	 * @return
	 */
	public JComboBox<String> getFitnessAlgorithmBox(){
		return fitnessAlgorithmBox;
	}
	
	/**
	 * get the box of algorithms to compute precision
	 * @return
	 */
	public JComboBox<String> getPrecisionAlgorithmBox(){
		return precisionAlgorithmBox;
	}
	
	/**
	 * get the box of algorithms to compute generalization
	 * @return
	 */
	public JComboBox<String> getGeneralizationAlgorithmBox(){
		return generalizationAlgorithmBox;
	}
	
    public static void main(String[] args) {
    	
        JFrame frame = new JFrame("Hello world");
       
        MetricsAlgorithmPanel panel = new MetricsAlgorithmPanel(null);
       
        frame.add(panel);

        frame.setSize(600, 400);
        frame.setVisible(true);
       
     }

}