package org.processmining.ocbc.discovery.plugin;

import java.awt.Toolkit;
import java.io.File;
import java.text.SimpleDateFormat;
import java.util.Calendar;

import javax.swing.JFrame;

import org.processmining.common.runningtime.RunningTime;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.mxgraph.util.mxConstants;
import org.processmining.ocbc.discovery.impl.OCBCModelDiscovery;
import org.processmining.ocbc.log.io.XocXmlParser;
import org.processmining.ocbc.log.model.XOCLog;
import org.processmining.ocbc.models.OCBCModel;
import org.processmining.ocbc.models.editor.plugins.OCBCEditorConstants;
import org.processmining.ocbc.models.editor.plugins.OCBCModelEditor;

public class OCBCModelDiscoveryPlugin {
//	@Plugin(
//		name = "OCBC Model Discovery (Perfect Fitness)", 
//		parameterLabels = {"XOCLog"}, 
//		returnTypes = OCBCModel.class, 
//		returnLabels = "OCBC model", 
//		mostSignificantResult = 1, 
//		help = "OCBC model discovery"
//	)
//	
//	@UITopiaVariant(
//		affiliation = UITopiaVariant.EHV,
//		author = "Guangming Li",
//		email = "g.li.3@tue.nl"
//	)	
//	public OCBCModel discoverOCBCModelWithPerfectFitness(UIPluginContext context, XOCLog log){
//		return discoverOCBCModel(context, log, mxConstants.perfectFitnessApproach);
//	}
	
	@Plugin(
			name = "OCBC Model Discovery (Customized Metric Weights)", 
			parameterLabels = {"XOCLog"}, 
			returnTypes = OCBCModel.class, 
			returnLabels = "OCBC model", 
			mostSignificantResult = 1, 
			help = "Discover an OCBC model whose behavioral constriants "
					+ "satisfying the combined metrics based on the configured weights (for fitness, precision and generalization)."
	)
	
	@UITopiaVariant(
		affiliation = UITopiaVariant.EHV,
		author = "Guangming Li",
		email = "g.li.3@tue.nl"
	)	
	public OCBCModel discoverOCBCModelWithBalancedMetrics(UIPluginContext context, XOCLog log){
		return discoverOCBCModel(context, log, mxConstants.balancedMetricsApproach);
	}
	
	
	public OCBCModel discoverOCBCModel(UIPluginContext context, XOCLog log, String approachName){
		System.out.println("Enter OCBCModelDiscoveryPlugin");			
		context.log("start show");		
	

		long uint = 1;	
		long startTime = System.currentTimeMillis(); 
		long durationTime = System.currentTimeMillis();
		
		RunningTime.writeTimeInformationIntoFile(" \n " , false, "D:\\timeDiscovery.txt");
		String timeStamp = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(Calendar.getInstance().getTime());
		RunningTime.writeTimeInformationIntoFile("current time: " + timeStamp, false, "D:\\timeDiscovery.txt");
		RunningTime.writeTimeInformationIntoFile("************log size: " + log.getAllEvents().size() + "*****approach: " + approachName + "*********", false, "D:\\timeDiscovery.txt");
		RunningTime.writeTimeInformationIntoFile("model discovery start time: " + startTime/uint, false, "D:\\timeDiscovery.txt");
		

		OCBCModelDiscovery discovery = new OCBCModelDiscovery(log, approachName);		
		OCBCModel model = discovery.discover(context);	
		
//		SimplifyModel reducer = new SimplifyModel(model);
//		reducer.filterTransitiveActicityEdges();
		
//		ModelLayout modelLayout = new ModelLayout(model);
//		model = modelLayout.optimizeLayout();
//		Map<String, Pair<Double, Double>> nodeNameVSCoordinateMap = modelLayout.getNodeNameVSCoordinateMap();
			
	    //add the "discovered" flag and result to model
	    model.setModelSource(mxConstants.discovered);
//	    model.setModelDiscoveryResult(modelDiscoveryResult);
	    model.setModelDiscovery(discovery);
	    model.setModelSourceLog(log);
		model.setModelVersion(OCBCEditorConstants.VERSION2_0);
		System.out.println("\n class node list size: " + model.getAllClasses().size());
		
		discovery.visualizeModel(model);
		
		durationTime = (System.currentTimeMillis() - startTime)/uint;
		RunningTime.writeTimeInformationIntoFile("~~~~~~~~~~~~~~discovery plugin duration time (include visualization): " + durationTime, false, "D:\\timeDiscovery.txt");

		return model;
	}	
	
//	public static void writeTimeInformationIntoFile(String wirteLine, boolean clearFile){
//		File log = new File("D:\\timeDiscovery.txt");
//	    try{
//		    if(log.exists()==false || clearFile){
//	            System.out.println("We had to make a new file.");
//	            log.createNewFile();
//		    }
//		    PrintWriter out = new PrintWriter(new FileWriter(log, true));
//		    out.append(wirteLine + "\n");
//		    out.close();
//	    }catch(IOException e){
//	        System.out.println("COULD NOT LOG!!");
//	    }
//	}
	
	public static void main(String[] args) throws Exception{  
		long uint = 1;	
		long startTime = System.currentTimeMillis(); 
		long durationTime = System.currentTimeMillis();
		
		RunningTime.writeTimeInformationIntoFile(" \n " , false, "D:\\timeDiscovery.txt");
		String timeStamp = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(Calendar.getInstance().getTime());
		RunningTime.writeTimeInformationIntoFile("current time: " + timeStamp, false, "D:\\timeDiscovery.txt");
		RunningTime.writeTimeInformationIntoFile("parse log start time: " + startTime/uint, false, "D:\\timeDiscovery.txt");
		
		XocXmlParser parser = new XocXmlParser();
//		parser.parse(new File("e:\\DolibarrXOCLog.xoc"));	outputDolibarrReimbursementLarge	
		parser.parse(new File("d:\\log47500.xoc"));
//		parser.parse(new File("d:\\logDuplicated.xoc"));
		
//		parser.parse(new File("d:\\simplifiedLog.xoc"));
		
		XOCLog log = parser.getLog();
		
	    durationTime = (System.currentTimeMillis() - startTime)/uint;
		startTime = System.currentTimeMillis();
		RunningTime.writeTimeInformationIntoFile("parse log end time: " + System.currentTimeMillis()/uint, false, "D:\\timeDiscovery.txt");
		RunningTime.writeTimeInformationIntoFile("***parse log duration time: " + durationTime, false, "D:\\timeDiscovery.txt");
		RunningTime.writeTimeInformationIntoFile("***log size: " + log.getAllEvents().size(), false, "D:\\timeDiscovery.txt");
		System.out.println("log size: " + log.getAllEvents().size());
		JFrame frame = new JFrame();

		
		OCBCModelDiscovery discovery = new OCBCModelDiscovery(log, mxConstants.balancedMetricsApproach);		
		OCBCModel model = discovery.discover(null);	
	
		
//		SimplifyModel reducer = new SimplifyModel(model);
//		reducer.filterTransitiveActicityEdges();
		
//		ModelLayout modelLayout = new ModelLayout(model);
//		model = modelLayout.optimizeLayout();
//		Map<String, Pair<Double, Double>> nodeNameVSCoordinateMap = modelLayout.getNodeNameVSCoordinateMap();
			
	    model.setModelSource(mxConstants.discovered);
//	    model.setModelDiscoveryResult(modelDiscoveryResult);
	    model.setModelDiscovery(discovery);
	    model.setModelSourceLog(log);
		model.setModelVersion(OCBCEditorConstants.VERSION2_0);
		System.out.println("\n class node list size: " + model.getAllClasses().size());
		
		discovery.visualizeModel(model);
		
				
		OCBCModelEditor editor = new OCBCModelEditor();			
		frame.add(editor.edit(null, model));
		
//		editor.getGraphEditor().setOCBCModel(model);
		
		frame.setSize(Toolkit.getDefaultToolkit().getScreenSize()); 
		frame.setVisible(true);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		
		System.out.println("\n class node list size: " + model.getAllClasses().size());
    }
	
	
}
