package org.processmining.ocbc.discovery.software;
 
import java.awt.Component;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.text.SimpleDateFormat;
import java.util.Calendar;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

import org.processmining.common.dialogue.IOFileDialogue;
import org.processmining.common.runningtime.RunningTime;
import org.processmining.mxgraph.util.mxConstants;
import org.processmining.ocbc.discovery.impl.OCBCModelDiscovery;
import org.processmining.ocbc.discovery.panels.DiscoveryParameterPanel;
import org.processmining.ocbc.log.io.XocXmlParser;
import org.processmining.ocbc.log.model.XOCLog;
import org.processmining.ocbc.models.OCBCModel;
import org.processmining.ocbc.models.editor.plugins.OCBCEditorConstants;
import org.processmining.ocbc.models.editor.plugins.OCBCModelEditor;
 
/**
 * @version 1.0
 * @author liangdong
 *
 */
public class DiscoveryParameterInterface extends JPanel{
 
    /**
     * 
     */
    private static final long serialVersionUID = 1L;

    private File file;//record the input log file
    private File lastLogFile;
    private File lastModelFile;
//    private ParameterPanel parameterPanel = new ParameterPanel();
    private XOCLog inputLog;
    private XocXmlParser XOCParser;
    private OCBCModelDiscoverySoftware softwarePanel;
    private JButton selectLogButton;	
    private JButton discoverModelButton;
    private JButton removeLogButton;
    private JButton removeModelButton;
    
    private DiscoveryParameterPanel parameterPanel;
    private JFrame fatherFrame;
    private OCBCModelDiscovery discovery;
    private OCBCModelEditor editor;
    private OCBCModel model;
    public DiscoveryParameterInterface(JFrame fatherFrame, OCBCModelDiscoverySoftware softwarePanel) { 	
    	this.fatherFrame = fatherFrame;
    	this.softwarePanel = softwarePanel;
    	generateParameterPanel();
    }

    public DiscoveryParameterPanel getParameterPanel(){
    	return parameterPanel;
    }

    
    public void generateParameterPanel(){	

		selectLogButton = new JButton("import log");		
		removeLogButton = new JButton("remove imported log");	
		removeModelButton = new JButton("remove discovered model");	
		discoverModelButton = new JButton("discover model");
		parameterPanel = new DiscoveryParameterPanel();
		
		selectLogButton.setToolTipText("import XOC log");
		removeLogButton.setToolTipText("remove XOC log");
		removeModelButton.setToolTipText("remove discovered model");
		discoverModelButton.setToolTipText("discover model");

		JPanel buttonPanel = new JPanel();
		buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.X_AXIS));		
		buttonPanel.add(selectLogButton);
	
		buttonPanel.add(Box.createRigidArea(new Dimension(15, 30)));
		buttonPanel.add(discoverModelButton);	
		
		buttonPanel.add(Box.createRigidArea(new Dimension(15, 30)));
		buttonPanel.add(removeLogButton);	
		
		buttonPanel.add(Box.createRigidArea(new Dimension(15, 30)));
		buttonPanel.add(removeModelButton);	
		
		buttonPanel.setAlignmentY(Component.TOP_ALIGNMENT);
		buttonPanel.setBorder(BorderFactory.createEmptyBorder(15, 5, 15, 5));
	
		this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));		
		this.add(buttonPanel);
		JPanel panel = new JPanel();
		panel.add(parameterPanel);
		this.add(panel);
	
		selectLogButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {	
				IOFileDialogue fileImporter = new IOFileDialogue();
            	fileImporter.show("XOC Log Files (.xoc)","xoc", JFileChooser.FILES_ONLY, "Select", lastLogFile);      	
            	file = fileImporter.getSelectedFile();
            	
				if(file != null){
			    	XOCParser = new XocXmlParser();
					lastLogFile = file.getParentFile();
					//change the cursor to waiting cursor
					fatherFrame.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));	
					
					try {
//						XOCParser.parse(new File("D:\\log.xoc"));
						
						XOCParser.parse(file);
					} catch (Exception e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}			
					inputLog = XOCParser.getLog();
					selectLogButton.setText("log imported: " + file.getName());
					//change the cursor to default cursor
					fatherFrame.setCursor(Cursor.getDefaultCursor());	
				}		
				
//				Runtime r = Runtime.getRuntime(); 
//				System.out.println("Total memory is: " + 
//						r.totalMemory()); 
//				long mem1 = r.freeMemory(); 
//				System.out.println("Initial free memory: " + mem1); 
			}
		});
		
		discoverModelButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {	 
				if(inputLog != null){	
					long uint = 1;	
					long startTime = System.currentTimeMillis(); 
					long durationTime = System.currentTimeMillis();
					
					//change the cursor to waiting cursor
					fatherFrame.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));	
					
					RunningTime.writeTimeInformationIntoFile(" \n " , false, "D:\\timeDiscovery.txt");
					String timeStamp = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(Calendar.getInstance().getTime());
					RunningTime.writeTimeInformationIntoFile("current time: " + timeStamp, false, "D:\\timeDiscovery.txt");
					RunningTime.writeTimeInformationIntoFile("***log size: " + inputLog.getAllEvents().size(), false, "D:\\timeDiscovery.txt");
//					RunningTime.writeTimeInformationIntoFile("model discover start time: " + System.currentTimeMillis()/uint, false,"D:\\timeDiscovery.txt");
					
					System.out.println("log size: " + inputLog.getAllEvents().size());

					discovery = new OCBCModelDiscovery(inputLog, mxConstants.balancedMetricsApproach);		
					model = discovery.discover(null);						
				    model.setModelSource(mxConstants.discovered);
				    model.setModelDiscovery(discovery);
//				    model.setModelSourceLog(inputLog);
					model.setModelVersion(OCBCEditorConstants.VERSION2_0);
					discovery.visualizeModel(model);
//					System.out.println("\n class node list size: " + model.getAllClasses().size());
					
					editor = new OCBCModelEditor();		
					softwarePanel.getMultiplePane().addTab("model-" + file.getName(), editor.edit(null, model));  
					softwarePanel.getMultiplePane().setSelectedIndex(softwarePanel.getMultiplePane().getComponentCount()-1);
					
//					durationTime = (System.currentTimeMillis() - startTime)/uint;
//					RunningTime.writeTimeInformationIntoFile("model discover plugin end time: " + System.currentTimeMillis()/uint, false,"D:\\timeDiscovery.txt");
//					RunningTime.writeTimeInformationIntoFile("--------------model discover plugin duration time: " + durationTime, false,"D:\\timeDiscovery.txt");
					
					//change the cursor to default cursor
//					discoverModelButton.setText("model discovered: " + "model-" + file.getName());
					fatherFrame.setCursor(Cursor.getDefaultCursor());	
				}
				else if(inputLog == null){
					JOptionPane.showMessageDialog(null, "Wrong! Please import a log!");
				}		
			}			
		});
		
		removeLogButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				inputLog = null;
				//change the cursor to waiting cursor
				fatherFrame.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));	
				
				selectLogButton.setText("import log");
				Runtime r = Runtime.getRuntime(); 
				r.gc(); 
				long mem1 = r.freeMemory(); 
				System.out.println("Free memory after garbage collection: " 
				+ mem1); 
				
				//change the cursor to default cursor
				fatherFrame.setCursor(Cursor.getDefaultCursor());	
			}
		});
		
		removeModelButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				discovery = null;
				editor = null;
				model = null;
				int panelAmount = softwarePanel.getMultiplePane().getComponentCount();
				if(panelAmount > 1){
					softwarePanel.getMultiplePane().remove(softwarePanel.getMultiplePane().getComponentCount()-1);
				}		
				//change the cursor to waiting cursor
				fatherFrame.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));	
				
				Runtime r = Runtime.getRuntime(); 
				r.gc(); 
				long mem1 = r.freeMemory(); 
				System.out.println("Free memory after garbage collection: " 
				+ mem1); 
				
				//change the cursor to default cursor
				fatherFrame.setCursor(Cursor.getDefaultCursor());	
			}
		});
		
    }
   
    public static void main(String[] args) {
        JFrame frame = new JFrame("OCBC model discovery software");    
        DiscoveryParameterInterface parameterInterface = new DiscoveryParameterInterface(frame, null);      
        frame.add(parameterInterface);
        frame.setSize(1600, 1400);
        frame.setVisible(true);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
     }
}