package org.processmining.ocbc.log.duplicator.impl;

import java.io.File;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.deckfour.spex.SXTag;
import org.joda.time.DateTime;
import org.processmining.common.runningtime.RunningTime;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.ocbc.log.factory.XFactoryRegistry;
import org.processmining.ocbc.log.io.OutputXMLFile;
import org.processmining.ocbc.log.io.XocXmlParser;
import org.processmining.ocbc.log.model.XAttribute;
import org.processmining.ocbc.log.model.XOCEvent;
import org.processmining.ocbc.log.model.XOCLog;
import org.processmining.ocbc.log.model.XOCORelation;
import org.processmining.ocbc.log.model.XOCObject;
import org.processmining.ocbc.log.model.impl.XOCEventImpl;
import org.processmining.ocbc.log.model.impl.XOCLogImpl;
import org.processmining.ocbc.log.model.impl.XOCORelationImpl;
import org.processmining.ocbc.log.model.impl.XOCObjectImpl;
import org.processmining.ocbc.log.model.impl.xConstants;
import org.processmining.ocbc.models.OCBCModel;

public class LogDuplicator {
	
//	@Plugin(
//			name = "XOC Log Duplicator", 
//			parameterLabels = {"XOCLog"}, 
//			returnTypes = XOCLog.class, 
//			returnLabels = "XOC Log", 
//			mostSignificantResult = 1, 
//			help = "XOC Log Duplicator"
//	)
//	
//	@UITopiaVariant(
//		affiliation = UITopiaVariant.EHV,
//		author = "Guangming Li",
//		email = "g.li.3@tue.nl"
//	)	
	public OCBCModel discoverOCBCModelWithBalancedMetrics(PluginContext context, XOCLog log){
		return null;
	}
	
	/**
	 * duplicate an input object with all attributes (id value + idSuffix)
	 * @param inputObject the input object template
	 * @param idSuffix the string added after the id of input object to distinguish
	 * @return
	 */
	public XOCObject duplicateObject(XOCObject inputObject, String idSuffix){
		XOCObject duplicatedObject = new XOCObjectImpl("");		
		for(String attributeKey : inputObject.getAttributes().keySet()){
			if(attributeKey.equals(xConstants.ID)){
				String duplicatedObjectId = inputObject.getAttributes().get(xConstants.ID).toString() + idSuffix;
				
				XAttribute attributeId = XFactoryRegistry.instance().currentDefault().createAttributeLiteral(attributeKey, duplicatedObjectId, null);
				duplicatedObject.getAttributes().put(attributeKey, attributeId);
			}
			else if(attributeKey.equals(xConstants.CLASS)){
				duplicatedObject.getAttributes().put(attributeKey, inputObject.getAttributes().get(xConstants.CLASS));
			}
			else if(attributeKey.equals(xConstants.lifecycle)){
				duplicatedObject.getAttributes().put(attributeKey, inputObject.getAttributes().get(xConstants.lifecycle));
			}
//			else{
//				duplicatedObject.getAttributes().put(attributeKey, inputObject.getAttributes().get(attributeKey));
//			}
		}
		return duplicatedObject;
	}
	
	/**
	 * duplicate an input object relation with all attributes (id, sourceobjectid and targetobjectid value + idSuffix)
	 * @param inputObjectRelation the input object relation template
	 * @param idSuffix the string added after the id of input object to distinguish
	 * @return
	 */
	public XOCORelation duplicateObjectRelation(XOCORelation inputObjectRelation, String idSuffix){
		XOCORelation duplicatedObjectRelation = new XOCORelationImpl("");		
		for(String attributeKey : inputObjectRelation.getAttributes().keySet()){
			if(attributeKey.equals(xConstants.ID) || attributeKey.equals(xConstants.TARGETOBJECTID) || attributeKey.equals(xConstants.SOURCEOBJECTID)){
				String duplicatedObjectAttributeValue = inputObjectRelation.getAttributes().get(attributeKey).toString() + idSuffix;			
				XAttribute duplicatedAttribute = XFactoryRegistry.instance().currentDefault().createAttributeLiteral(attributeKey, duplicatedObjectAttributeValue, null);
				duplicatedObjectRelation.getAttributes().put(attributeKey, duplicatedAttribute);
			}
			else{
				duplicatedObjectRelation.getAttributes().put(attributeKey, inputObjectRelation.getAttributes().get(attributeKey));
			}
		}
		return duplicatedObjectRelation;
	}
	
	/**
	 * duplicate an event 
	 * @param inputEvent the input event template
	 * @param idSuffix the string added to distinguish
	 * @return
	 */
	public XOCEvent duplicateEvent(XOCEvent inputEvent, int number){
		XOCEvent duplicatedEvent = new XOCEventImpl();	
		String idSuffix = "_copy" + number/5;
		//duplicate attributes
		for(String attributeKey : inputEvent.getAttributes().keySet()){
			if(attributeKey.equals(xConstants.ID)){
				String duplicatedObjectAttributeValue = inputEvent.getAttributes().get(attributeKey).toString() + idSuffix;			
				XAttribute duplicatedAttribute = XFactoryRegistry.instance().currentDefault().createAttributeLiteral(attributeKey, duplicatedObjectAttributeValue, null);
				duplicatedEvent.getAttributes().put(attributeKey, duplicatedAttribute);
			}
			else if(attributeKey.equals(xConstants.TIMESTAMP)){ //to do...
				String timeString = inputEvent.getAttributes().get(attributeKey).toString();
				String newTimeString = "9999-01-01 01:00:00";
				SimpleDateFormat dateTime = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");  	
				try {
					Date timestamp  = dateTime.parse(timeString);
					//add one month to the time
					DateTime time = new DateTime(timestamp);
					DateTime newTime = time.plusMonths(number);
					newTimeString = newTime.toString("yyyy-MM-dd HH:mm:ss");				
				} catch (ParseException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
								
				String duplicatedObjectAttributeValue = newTimeString;			
				XAttribute duplicatedAttribute = XFactoryRegistry.instance().currentDefault().createAttributeLiteral(attributeKey, duplicatedObjectAttributeValue, null);
				duplicatedEvent.getAttributes().put(attributeKey, duplicatedAttribute);
			}
			
			else{
				duplicatedEvent.getAttributes().put(attributeKey, inputEvent.getAttributes().get(attributeKey));
			}
		}
		//duplicate object model
		for(XOCObject inputObject : inputEvent.getObjectModelUpdate().getObjectSet()){
			
			XOCObject duplicatedObject = duplicateObject(inputObject, idSuffix);
//			System.out.println("duplicatedObject id: " + duplicatedObject.getAttributes().get(xConstants.ID).toString());
			duplicatedEvent.getObjectModelUpdate().getObjectSet().add(duplicatedObject);
		}
		for(XOCORelation inputObjectRelation : inputEvent.getObjectModelUpdate().getObjectRelationSet()){
			XOCORelation duplicatedObjectRelation = duplicateObjectRelation(inputObjectRelation, idSuffix);
//			System.out.println("duplicatedObjectRelation id: " + duplicatedObjectRelation.getAttributes().get(xConstants.ID).toString());
			duplicatedEvent.getObjectModelUpdate().getObjectRelationSet().add(duplicatedObjectRelation);
		}
		//duplicate related object list
		for(XOCObject inputObject : inputEvent.getSimplifiedRelatedObjects()){
			
			XOCObject duplicatedObject = duplicateObject(inputObject, idSuffix);
//			System.out.println("duplicatedObject id: " + duplicatedObject.getAttributes().get(xConstants.ID).toString());
			duplicatedEvent.getSimplifiedRelatedObjects().add(duplicatedObject);
		}
		return duplicatedEvent;
	}
	
	public static void main(String[] args) throws Exception{  
		long uint = 1;	
		long startTime = System.currentTimeMillis(); 
		long durationTime = System.currentTimeMillis();
		
		
		RunningTime.writeTimeInformationIntoFile("parse log start time: " + startTime/uint, true, "D:\\timeDuplication.txt");
		
		XocXmlParser parser = new XocXmlParser();
		parser.parse(new File("d:\\log.xoc"));		
		XOCLog log = parser.getLog();
		
		List<XOCEvent> originalEventList = new ArrayList<XOCEvent>();
		for(XOCEvent event : log.getAllEvents()){	
			originalEventList.add(event);
		}
		
		LogDuplicator duplicator = new LogDuplicator();
		
	    durationTime = (System.currentTimeMillis() - startTime)/uint;
		startTime = System.currentTimeMillis();
		RunningTime.writeTimeInformationIntoFile("parse log end time: " + System.currentTimeMillis()/uint, false, "D:\\timeDuplication.txt");
		RunningTime.writeTimeInformationIntoFile("***parse log duration time: " + durationTime, false, "D:\\timeDuplication.txt");
		RunningTime.writeTimeInformationIntoFile("***log size: " + log.getAllEvents().size(), false, "D:\\timeDuplication.txt");
		System.out.println("log size: " + log.getAllEvents().size());
		
		OutputXMLFile generateLog = new OutputXMLFile("d:\\logCopy.xoc");
//        generateLog.serialize(log, xConstants.UPDATE);
		SXTag logTag = generateLog.serializeLogTag(log, xConstants.UPDATE);
		generateLog.serializeEventListTag(log.getAllEvents(), xConstants.UPDATE, logTag);
		/*
		 * consume too much memory 350 times using 4G
		 * in future, we can write events to file for multiple times
		 * after each time, we clear the memory1
		 */
		XOCLog logCopy = new XOCLogImpl();
		int duplicatedTimes = 30;
		for(int i=1; i<duplicatedTimes; i++){
			System.out.println("copy times: " + i);
			
//			logCopy = new XOCLogImpl();
			logCopy.getAllEvents().clear();
			for(XOCEvent event : originalEventList){			
				XOCEvent duplicatedEvent = duplicator.duplicateEvent(event, i*5);
				logCopy.getAllEvents().add(duplicatedEvent);
//				logCopy.getAllEvents().add(event);
			}
			//write log into the file
			generateLog.serializeEventListTag(logCopy.getAllEvents(), xConstants.UPDATE, logTag);
		}
		generateLog.closeSerializingLog();;
//		for(int i=1; i<duplicatedTimes; i++){
//			System.out.println("copy times: " + i);
//			String suffix = "_copy" + i;
//			for(XOCEvent event : originalEventList){			
//				XOCEvent duplicatedEvent = duplicator.duplicateEvent(event, suffix);
//				log.getAllEvents().add(duplicatedEvent);
//				log.getAllEvents().add(event);
//			}
//		}
		
	    durationTime = (System.currentTimeMillis() - startTime)/uint;
		startTime = System.currentTimeMillis();
		RunningTime.writeTimeInformationIntoFile("duplicate log end time: " + System.currentTimeMillis()/uint, false, "D:\\timeDuplication.txt");
		RunningTime.writeTimeInformationIntoFile("***duplicate log duration time: " + durationTime, false, "D:\\timeDuplication.txt");
		RunningTime.writeTimeInformationIntoFile("***log size: " + log.getAllEvents().size(), false, "D:\\timeDuplication.txt");
	
		/*
		 * writing  event from memory into file does not consume memory
		 */
//		OutputXMLFile generateLog = new OutputXMLFile("d:\\logDuplicated.xoc");
//        generateLog.serialize(log, xConstants.UPDATE);
//		
//	    durationTime = (System.currentTimeMillis() - startTime)/uint;
//		startTime = System.currentTimeMillis();
//		RunningTime.writeTimeInformationIntoFile("output log end time: " + System.currentTimeMillis()/uint, false, "D:\\timeDuplication.txt");
//		RunningTime.writeTimeInformationIntoFile("***output log duration time: " + durationTime, false, "D:\\timeDuplication.txt");
//		RunningTime.writeTimeInformationIntoFile("***log size: " + log.getAllEvents().size(), false, "D:\\timeDuplication.txt");
    }
	
	
}
