package org.processmining.ocbc.log.inspector.interaction;

import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.UUID;

import javax.swing.JButton;

import org.processmining.mxgraph.model.mxCell;
import org.processmining.mxgraph.swing.mxGraphOutline;
import org.processmining.mxgraph.util.mxConstants;
import org.processmining.mxgraph.view.mxGraph;
import org.processmining.ocbc.conformance.visualization.views.ConformanceCheckingResultView;
import org.processmining.ocbc.log.inspector.graph.EventCell;
import org.processmining.ocbc.log.inspector.graph.InexistentObjectCell;
import org.processmining.ocbc.log.inspector.graph.ObjectCell;
import org.processmining.ocbc.log.inspector.graph.ObjectEdgeCell;
import org.processmining.ocbc.log.inspector.graph.ReferenceEdgeCell;
import org.processmining.ocbc.log.inspector.panels.AttributesHierarchyPanel;
import org.processmining.ocbc.log.inspector.panels.ObjectModelJGraphXPanel;
import org.processmining.ocbc.log.inspector.panels.XOCLogEvolution;
import org.processmining.ocbc.log.model.XOCEvent;
import org.processmining.ocbc.log.model.XOCLog;
import org.processmining.ocbc.log.model.XOCORelation;
import org.processmining.ocbc.log.model.XOCObject;
import org.processmining.ocbc.log.model.XOCRRelation;
import org.processmining.ocbc.models.OCBCModel;
import org.processmining.ocbc.models.editor.panels.EditorPanel;
import org.processmining.ocbc.models.editor.panels.InformationPanel;
import org.processmining.ocbc.models.visualization.ModelLayoutBuilder;
import org.processmining.ocbc.models.visualization.cell.MyOriginalCell;


public class GraphMouseListener extends MouseAdapter {
	OCBCModel model;
	ModelLayoutBuilder builder;
	
	UUID selectedID;
	UUID previouslySelectedID;
	

	private XOCLog log;
	private XOCEvent lastEvent;
	private XOCEvent selectEvent;
	private ObjectModelJGraphXPanel graphPanel;
	private mxGraphOutline graphOutlinePanel;
	private AttributesHierarchyPanel attributesPanel;	
	private XOCLogEvolution logEvolutionPanel;
	
	InformationPanel informationPanel;
	EditorPanel editorPanel;
	Set<UUID> foldedNodes;
	String elementType;
	String previouslyElementType;
	
	private JButton focusedButton;
	private JButton previouslyFocusedButton;
	
	private MyOriginalCell firstlyFocusedCell;//used for keeping the first point when drawing a edge
	int firstX;
	int firstY;
	
	private MyOriginalCell secondlyFocusedCell;//used for keeping the second point when drawing a edge
	int secondX;
	int secondY;
	
	private MyOriginalCell focusedCell;
	private String highlightedColor = "yellow";
	public GraphMouseListener(ObjectModelJGraphXPanel graphPanel, AttributesHierarchyPanel attributesPanel, XOCLog log){
		this.graphPanel = graphPanel;
		this.attributesPanel = attributesPanel;
		this.log = log;
		this.lastEvent = log.getLastEvent();
	}
	
	public void addGraphPanel(ObjectModelJGraphXPanel graphPanel){
		this.graphPanel = graphPanel;
	}
	
	public void addGraphOutlinePanel(mxGraphOutline graphOutlinePanel){
		this.graphOutlinePanel = graphOutlinePanel;
	}
	
	public void addLogEvolutionPanel(XOCLogEvolution logEvolutionPanel){
		this.logEvolutionPanel = logEvolutionPanel;
	}
	
	public void addInformationPanel(InformationPanel informationPanel){
		this.informationPanel = informationPanel;
	}
	
	public void addEditorPanel(EditorPanel editorPanel){
		this.editorPanel = editorPanel;
	}
	
//	public void setGraph(JGraph graph){
//		this.graph = graph;
//	}
	
	public UUID getSelectedID(){
		return selectedID;
	}
	
	public UUID getPreviouslySelectedID(){
		return previouslySelectedID;
	}
	
	/**
	 * Sets the selected ID, can be used for variables and originators (it will raise a mousePressed event)
	 * @param newID
	 */
	public void setSelectedButton(JButton focusedButton){
		this.previouslyFocusedButton = this.focusedButton;
		this.focusedButton = focusedButton;
		System.out.println("set focusedButton");
//		mousePressed(null);
	}
	
	public JButton getSelectedButton(){
//		this.previouslyFocusedButton = this.focusedButton;
		return focusedButton;
	}
	
	
	public void setSelectedCell(MyOriginalCell cell){
//		this.previouslyFocusedButton = this.focusedButton;
		this.focusedCell = cell;
//		System.out.println("set focusedButton");
//		mousePressed(null);
	}
	
	public void setSelectedEvent(XOCEvent selectEvent){
		this.selectEvent = selectEvent;
	}
	
	public MyOriginalCell getSelectedCell(){
//		this.previouslyFocusedButton = this.focusedButton;
		return focusedCell;
	}
	
	public JButton getPreviouslySelectedButton(){
//		this.previouslyFocusedButton = this.focusedButton;
		System.out.println("getPreviouslySelectedButton()");
		return this.previouslyFocusedButton;
	}
	
	public void setSelectedID(UUID newID){
		this.previouslySelectedID = this.selectedID;
		this.selectedID = newID;
//		mousePressed(null);
	}
	
	public String getElementType(){
		return elementType;
	}
	
	public String getPreviouslyElementType(){
		return previouslyElementType;
	}
	
	/**
	 * Sets the selected elementType, can be used for variables and originators (it will raise a mousePressed event)
	 * @param newID
	 */
	public void setElementType(String elementType){
		this.previouslyElementType = this.elementType;
		this.elementType = elementType;
		System.out.println("set elementType");
//		mousePressed(null);
	}

	@Override
	public void mouseDragged(MouseEvent e){
		System.out.println("\n mouseDragged");
	}
	
	@Override
	/**
	 * Highlight the selected cell (event, object, object relation) and its related cells when the mouse is released, 
	 * as well as updating the attribute panel
	 */
	public void mouseReleased(MouseEvent e){
		System.out.println("\n mouse released new detected in GraphMouseListener");
		clearHightlightColor();
		mxCell mxcell = (mxCell) graphPanel.getGraph().getSelectionCell();
//		graphPanel.getMxcellVSCellMap()
		if(mxcell != null){
			System.out.println("\n mxcell != null");
			MyOriginalCell cell = graphPanel.getMxcellVSCellMap().get(mxcell);
			List<mxCell> relevantMxcellList = new ArrayList<mxCell>();
			if(cell instanceof ObjectCell){		
				System.out.println("\n cell instanceof ObjectCell");
				XOCObject object = (XOCObject) ((ObjectCell) cell).getCorrespondingElement();
				//get the related cells and edges and update the attributes (with related object, object relations and events) of the selected object
				List<XOCORelation>  relevantObjectRelationList = object.getRelatedObjectRelationList();		
				List<String> relevantObjectIdList = object.getRelatedObjectIdList();
				List<XOCObject> relevantObjectList = new ArrayList<XOCObject>();
				Map<String, XOCObject> idVSObjectMap = log.getIdVSObjectMap();
				for(String relevantObjectId : relevantObjectIdList){
					XOCObject relevantObject = idVSObjectMap.get(relevantObjectId);
					relevantObjectList.add(relevantObject);
				}

				attributesPanel.updateAttributePanelForObject(object, selectEvent, log, relevantObjectList, relevantObjectRelationList);
								
				//get and highlight cells related to the selected object (including itself)
				relevantObjectList.add(object);
				for(XOCObject relevantObject : relevantObjectList){
					ObjectCell relevantCell = (ObjectCell) relevantObject.getCorrespondingCell();
					if(relevantCell != null){
						relevantMxcellList.add(relevantCell.getMxcell());
					}							
				}					
				for(XOCORelation relation : relevantObjectRelationList){
					ObjectEdgeCell relevantRelationCell = (ObjectEdgeCell) relation.getCorrespondingCell();
					if(relevantRelationCell != null){
						relevantMxcellList.add(relevantRelationCell.getMxcell());
					}						
				}				
				if(object.getRelatedEventList().contains(selectEvent)){
					EventCell eventCell = (EventCell) selectEvent.getCorrespondingCell();
					relevantMxcellList.add(eventCell.getMxcell());
					XOCRRelation referenceRelation = selectEvent.getReferenceRelation(object);
					ReferenceEdgeCell referenceRelationCell = (ReferenceEdgeCell) referenceRelation.getCorrespondingCell();
					relevantMxcellList.add(referenceRelationCell.getMxcell());
				}					
			}			
			else if(cell instanceof ObjectEdgeCell){					
				XOCORelation selectedRelation = (XOCORelation) ((ObjectEdgeCell) cell).getCorrespondingElement();	
				attributesPanel.updateAttributePanelForObjectRelation(selectedRelation, log, selectEvent);	
				ObjectCell sourceCell = (ObjectCell) selectedRelation.getSourceObject().getCorrespondingCell();
				ObjectCell targetCell = (ObjectCell) selectedRelation.getTargetObject().getCorrespondingCell();
				mxCell relationCell = ((ObjectEdgeCell) cell).getMxcell();
				relevantMxcellList.add(sourceCell.getMxcell());
				relevantMxcellList.add(targetCell.getMxcell());
				relevantMxcellList.add(relationCell);
			}						
			else if(cell instanceof EventCell){
				XOCEvent event = (XOCEvent) ((EventCell) cell).getCorrespondingElement();
				attributesPanel.updateAttributePanelForEvent(event);
				//highlight all related events
				List<XOCEvent> relatedEventList = log.getRelatedEventsForOneEvent(event);
				relatedEventList.add(event);
				for(XOCEvent eventCheck : log.getAllEvents()){		
					if(relatedEventList.contains(eventCheck)){
						EventCell eventCell = (EventCell) eventCheck.getCorrespondingCell();
						if(eventCell != null){
							mxCell eventMxcell = graphPanel.getCellVSMxcellMap().get(eventCell);
							if(eventMxcell != null){
								relevantMxcellList.add(eventMxcell);	
							}	
						}							
					}
				}				
				//highlight all related objects and edges
				for(XOCRRelation referenceRelation : event.getReferenceRelationList()){					
					ReferenceEdgeCell referenceRelationCell = (ReferenceEdgeCell) referenceRelation.getCorrespondingCell();
					ObjectCell targetObjectCell = (ObjectCell) referenceRelation.getTargetObject().getCorrespondingCell();
					relevantMxcellList.add(referenceRelationCell.getMxcell());
					if(targetObjectCell != null){
						relevantMxcellList.add(targetObjectCell.getMxcell());
					}					
				}												
			}	
			else if(cell instanceof ReferenceEdgeCell){					
				XOCRRelation selectedRelation = (XOCRRelation) ((ReferenceEdgeCell) cell).getCorrespondingElement();	
				attributesPanel.updateAttributePanelForReferenceRelation(selectedRelation);
				EventCell sourceCell = (EventCell) selectedRelation.getSourceEvent().getCorrespondingCell();
				ObjectCell targetCell = (ObjectCell) selectedRelation.getTargetObject().getCorrespondingCell();
				mxCell relationCell = ((ReferenceEdgeCell) cell).getMxcell();
				relevantMxcellList.add(sourceCell.getMxcell());
				relevantMxcellList.add(targetCell.getMxcell());
				relevantMxcellList.add(relationCell);
			}	
			else if(cell instanceof InexistentObjectCell){					
				System.out.println("\n cell instanceof InexistentObjectCell");
				XOCObject object = (XOCObject) ((InexistentObjectCell) cell).getCorrespondingElement();
				attributesPanel.updateAttributePanelForObject(object, selectEvent, log, new ArrayList<XOCObject>(), new ArrayList<XOCORelation>());
			}	
			
			graphPanel.getGraph().orderCells(false, relevantMxcellList.toArray());				
			graphPanel.getGraph().setCellStyles(mxConstants.STYLE_STROKECOLOR, highlightedColor, relevantMxcellList.toArray());
			graphPanel.getGraph().setCellStyles(mxConstants.STYLE_STROKEWIDTH, "3", relevantMxcellList.toArray());	
		}	


//		graphOutlinePanel.updateUI();
		
		mxGraph graphx = new mxGraph(); 
		Object[] allVertexs = graphPanel.getGraph().getChildVertices(graphPanel.getGraph().getDefaultParent());
		Object[] allEdges = graphPanel.getGraph().getChildEdges(graphPanel.getGraph().getDefaultParent());
		graphPanel.getGraph().updateGroupBounds();
//		graphx.addCells(allVertexs);
//		graphx.addCells(allEdges);
//		mxGraphComponent graphComponent = new mxGraphComponent(graphx);
		
//		logEvolutionPanel.updatePanel(new mxGraphOutline(graphComponent));
//		graphOutlinePanel.setGraphComponent(graphComponent);
//		graphOutlinePanel = new mxGraphOutline(graphPanel.getGraphComponent());
		//update the panel; necessary
		
		//if the conformance checking result is not null, highlight the deviating cells
		ConformanceCheckingResultView conformanceCheckingResult = graphPanel.getParentLogBrowser().
				getParentLogViewer().getConformanceCheckingResult();
		if(conformanceCheckingResult != null){		
			System.out.println("highlight deviating cells in updateEventGraph in log.inspector.panels;");
			graphPanel.highlightDeviatingCells(conformanceCheckingResult);
		}
				
		graphPanel.update();
	}	
	
	public void clearHightlightColor(){
		Object[] allVertexs = graphPanel.getGraph().getChildVertices(graphPanel.getGraph().getDefaultParent());
		Object[] allEdges = graphPanel.getGraph().getChildEdges(graphPanel.getGraph().getDefaultParent());
		graphPanel.getGraph().setCellStyles(mxConstants.STYLE_STROKEWIDTH, "0", allVertexs);
		graphPanel.getGraph().setCellStyles(mxConstants.STYLE_STROKECOLOR, "green", allEdges);
		graphPanel.getGraph().setCellStyles(mxConstants.STYLE_STROKEWIDTH, "1", allEdges);
		graphPanel.getGraph().orderCells(false, allVertexs);	
	}
}
