package org.processmining.ocbc.log.inspector.interaction;

import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionListener;
import java.util.Set;
import java.util.UUID;

import javax.swing.JButton;

import org.processmining.framework.util.Pair;
import org.processmining.mxgraph.model.mxCell;
import org.processmining.ocbc.log.inspector.graph.InexistentObjectCell;
import org.processmining.ocbc.log.inspector.graph.ObjectCell;
import org.processmining.ocbc.log.inspector.panels.AttributesHierarchyPanel;
import org.processmining.ocbc.log.inspector.panels.ObjectModelJGraphXPanel;
import org.processmining.ocbc.log.model.XOCEvent;
import org.processmining.ocbc.log.model.XOCLog;
import org.processmining.ocbc.log.model.XOCObject;
import org.processmining.ocbc.models.OCBCModel;
import org.processmining.ocbc.models.editor.panels.EditorPanel;
import org.processmining.ocbc.models.editor.panels.InformationPanel;
import org.processmining.ocbc.models.visualization.ModelLayoutBuilder;
import org.processmining.ocbc.models.visualization.cell.MyOriginalCell;


public class GraphMoveMouseListener implements MouseMotionListener{
	OCBCModel model;
	ModelLayoutBuilder builder;
	
	UUID selectedID;
	UUID previouslySelectedID;
	

	private XOCLog log;
	private XOCEvent lastEvent;
	private XOCEvent selectEvent;
	private ObjectModelJGraphXPanel graphPanel;
	private AttributesHierarchyPanel attributesPanel;
	
	
	InformationPanel informationPanel;
	EditorPanel editorPanel;
	Set<UUID> foldedNodes;
	String elementType;
	String previouslyElementType;
	
	private JButton focusedButton;
	private JButton previouslyFocusedButton;
	
	private MyOriginalCell firstlyFocusedCell;//used for keeping the first point when drawing a edge
	int firstX;
	int firstY;
	
	private MyOriginalCell secondlyFocusedCell;//used for keeping the second point when drawing a edge
	int secondX;
	int secondY;
	
	private MyOriginalCell focusedCell;
	
	public GraphMoveMouseListener(ObjectModelJGraphXPanel graphPanel, AttributesHierarchyPanel attributesPanel, XOCLog log){
		this.graphPanel = graphPanel;
		this.attributesPanel = attributesPanel;
		this.log = log;
		this.lastEvent = log.getAllEvents().get(log.getAllEvents().size()-1);
	}
	
	public void addGraphPanel(ObjectModelJGraphXPanel graphPanel){
		this.graphPanel = graphPanel;
	}
	
	public void addInformationPanel(InformationPanel informationPanel){
		this.informationPanel = informationPanel;
	}
	
	public void addEditorPanel(EditorPanel editorPanel){
		this.editorPanel = editorPanel;
	}
	
//	public void setGraph(JGraph graph){
//		this.graph = graph;
//	}
	
	public UUID getSelectedID(){
		return selectedID;
	}
	
	public UUID getPreviouslySelectedID(){
		return previouslySelectedID;
	}
	
	/**
	 * Sets the selected ID, can be used for variables and originators (it will raise a mousePressed event)
	 * @param newID
	 */
	public void setSelectedButton(JButton focusedButton){
		this.previouslyFocusedButton = this.focusedButton;
		this.focusedButton = focusedButton;
		System.out.println("set focusedButton");
//		mousePressed(null);
	}
	
	public JButton getSelectedButton(){
//		this.previouslyFocusedButton = this.focusedButton;
		return focusedButton;
	}
	
	
	public void setSelectedCell(MyOriginalCell cell){
//		this.previouslyFocusedButton = this.focusedButton;
		this.focusedCell = cell;
//		System.out.println("set focusedButton");
//		mousePressed(null);
	}
	
	public void setSelectedEvent(XOCEvent selectEvent){
		this.selectEvent = selectEvent;
	}
	
	public MyOriginalCell getSelectedCell(){
//		this.previouslyFocusedButton = this.focusedButton;
		return focusedCell;
	}
	
	public JButton getPreviouslySelectedButton(){
//		this.previouslyFocusedButton = this.focusedButton;
		System.out.println("getPreviouslySelectedButton()");
		return this.previouslyFocusedButton;
	}
	
	public void setSelectedID(UUID newID){
		this.previouslySelectedID = this.selectedID;
		this.selectedID = newID;
//		mousePressed(null);
	}
	
	public String getElementType(){
		return elementType;
	}
	
	public String getPreviouslyElementType(){
		return previouslyElementType;
	}
	
	/**
	 * Sets the selected elementType, can be used for variables and originators (it will raise a mousePressed event)
	 * @param newID
	 */
	public void setElementType(String elementType){
		this.previouslyElementType = this.elementType;
		this.elementType = elementType;
		System.out.println("set elementType");
//		mousePressed(null);
	}

	@Override
	public void mouseDragged(MouseEvent e){
//		System.out.println("\n mouse move Dragged");
//		int x = e.getX();
//		int y = e.getY();
//		
////		Object cell = graphPanel.getGraphComponent().getCellAt(e.getX(), e.getY());
//		mxCell mxcell = (mxCell) graphPanel.getGraph().getSelectionCell();
//		
//		if(mxcell != null){
//
//			MyOriginalCell cell = graphPanel.getMxcellVSCellMap().get(mxcell);
//
//			if(cell instanceof ObjectCell){
//				System.out.println("\n x: " + x);
//				System.out.println("\n y: " + y);
//				XOCObject object = (XOCObject) ((ObjectCell) cell).getCorrespondingElement();				
//				Pair<Integer, Integer> coordinate = new Pair<Integer, Integer>(x, y);
//				graphPanel.getNodeVSCoordinateMap().replace(object.getAttributes().get("id").toString(), coordinate);
//
//					
//			}
//		}
				
	}
	

	@Override
	public void mouseMoved(MouseEvent e) {
//		System.out.println("\n mouseMoved");
		int x = e.getX();
		int y = e.getY();
		
		mxCell mxcell = (mxCell) graphPanel.getGraphComponent().getCellAt(e.getX(), e.getY());
//		mxCell mxcell = (mxCell) graphPanel.getGraph().getSelectionCell();
		
		if(mxcell != null){

			MyOriginalCell cell = graphPanel.getMxcellVSCellMap().get(mxcell);
			int cellX = (int) mxcell.getGeometry().getX();
			int cellY = (int) mxcell.getGeometry().getY();
			Pair<Integer, Integer> coordinate = new Pair<Integer, Integer>(cellX, cellY);
			
			if(cell instanceof ObjectCell){
//				System.out.println("\n x: " + x);
//				System.out.println("\n y: " + y);			
				XOCObject object = (XOCObject) ((ObjectCell) cell).getCorrespondingElement();							
				graphPanel.getNodeVSCoordinateMap().put(object.getAttributes().get("id").toString(), coordinate);		
			}
			else if(cell instanceof InexistentObjectCell){
				XOCObject object = (XOCObject) ((InexistentObjectCell) cell).getCorrespondingElement();	
				graphPanel.getInexistentObjectVSCoordinateMap().put(object.getAttributes().get("id").toString(), coordinate);	
			}
		}
				
	}
	
}
