package org.processmining.ocbc.log.inspector.panels;



import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreePath;

import org.processmining.common.frame.CreateTreePanel;
import org.processmining.common.structure.IntegerPair;
import org.processmining.mxgraph.util.mxConstants;
import org.processmining.ocbc.conformance.visualization.views.ConformanceCheckingResultView;
import org.processmining.ocbc.conformance.visualization.views.ResultEventView;
import org.processmining.ocbc.conformance.visualization.views.ResultObjectVersionView;
import org.processmining.ocbc.log.model.XAttributeMap;
import org.processmining.ocbc.log.model.XOCEvent;
import org.processmining.ocbc.log.model.XOCLog;
import org.processmining.ocbc.log.model.XOCORelation;
import org.processmining.ocbc.log.model.XOCObject;
import org.processmining.ocbc.log.model.XOCObjectModel;
import org.processmining.ocbc.log.model.XOCRRelation;
import org.processmining.ocbc.log.model.impl.XOCEventImpl;
import org.processmining.ocbc.log.model.impl.XOCLogImpl;
import org.processmining.ocbc.log.model.impl.XOCObjectImpl;
import org.processmining.ocbc.log.model.impl.XOCObjectModelImpl;
import org.processmining.ocbc.log.model.impl.xConstants;
import org.processmining.ocbc.models.AOCEdge;
import org.processmining.ocbc.models.ActivityEdge;
import org.processmining.ocbc.models.ClassEdge;

import com.fluxicon.slickerbox.components.RoundedPanel;

public class AttributesHierarchyPanel extends RoundedPanel {

	
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private CreateTreePanel eventTreePanel;
	private CreateTreePanel objectTreePanel;
	
	public static final Color COLOR_ENCLOSURE_BG = new Color(40, 40, 40);
	public static final Color COLOR_NON_FOCUS = new Color(70, 70, 70);
	public static final Color COLOR_LIST_BG = new Color(60, 60, 60);
	public static final Color COLOR_LIST_BG_LOWER = new Color(45, 45, 45);
	public static final Color COLOR_LIST_FG = new Color(180, 180, 180);
	public static final Color COLOR_LIST_SELECTION_BG = new Color(80, 0, 0);
	public static final Color COLOR_LIST_SELECTION_BG_LOWER = new Color(30, 10, 10);
	public static final Color COLOR_LIST_SELECTION_FG = new Color(240, 240, 240);

	private static final String TREE_ROOT_NODE_EVENT = "Event";
	private static final String TREE_ROOT_NODE_OBJECT = "Object";
	private static final String TREE_ROOT_NODE_RELATION = "Relation";
	private static final String TREE_NODE_REFERENCES = "References";
	private static final String TREE_NODE_OBJECTMODEL = "ObjectModel";
	private static final String TREE_NODE_OBJECTS = "Objects";
	private static final String TREE_NODE_RELATIONS = "Relations";
	private static final String TREE_NODE_ATTRIBUTES = "Attributes";
	private static final String TREE_NODE_LIFECYCLE = "Lifecycle";
	private static final String TREE_NODE_RELATEDOBJECTS = "Related objects";
	private static final String TREE_NODE_RELATEDOBJECTRELATION = "Related object relations";
	private static final String TREE_NODE_REFERREDEVENTS = "Referring events";
	
	private boolean simplifiedFlag = true;//false: the panel shows all object model; true: just shows object model difference (updated part)
	private XOCLogBrowser logBrowser;
	public AttributesHierarchyPanel(XOCLogBrowser logBrowser, int aRadius, int anOuterBorder, 
			int anInnerBorder, boolean eventPanelFlag, boolean objectPanelFlag){
		super(aRadius, anOuterBorder, anInnerBorder);
		this.logBrowser = logBrowser;
		this.setBackground(COLOR_ENCLOSURE_BG);
		this.setMaximumSize(new Dimension(230, 1000));
		this.setPreferredSize(new Dimension(230, 1000));
		this.setLayout(new GridLayout(1,2));
		eventTreePanel = new CreateTreePanel(new DefaultMutableTreeNode("no event is selected"), "selected_event");	
		objectTreePanel = new CreateTreePanel(new DefaultMutableTreeNode("no object is selected"), "selected_element");	
		updateEventAttributes(new XOCEventImpl(), true);
		updateAttributePanelForObject(null, null, new XOCLogImpl(), new ArrayList<XOCObject>(), new ArrayList<XOCORelation>());

		if(eventPanelFlag) this.add(eventTreePanel);
		if(objectPanelFlag) this.add(objectTreePanel);		
	}
	
	public void updateEventAttributes(XOCEvent event, boolean flag) {			
		setOpaque(false);
		setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
		setLayout(new BoxLayout(this, BoxLayout.X_AXIS));

		// Create the root node of the tree.
		String eventId;
		if(event.getAttributes().get("id") == null) eventId = "no event selected";
		else eventId = event.getAttributes().get("id").toString();
		DefaultMutableTreeNode root = new DefaultMutableTreeNode("Event complex");
		DefaultMutableTreeNode eventNode = new DefaultMutableTreeNode(TREE_ROOT_NODE_EVENT);
		root.add(eventNode);
		
		DefaultMutableTreeNode eventAttributes = generateAttributesNode(event.getAttributes());
		eventNode.add(eventAttributes);


		List<XOCObject> relatedObjectSet = event.getRelatedObjects();
		// Build up list of extensions.
		DefaultMutableTreeNode references = generateObjectSetNode(relatedObjectSet, TREE_NODE_REFERENCES+ "(" + relatedObjectSet.size() + ")");
		root.add(references);

		XOCLog log = logBrowser.getLog();
		List<XOCObject> objectSet;
		List<XOCORelation> objectRelationSet;	
		DefaultMutableTreeNode objectModleTreeNode;
		XOCObjectModel shownObjectModel = new XOCObjectModelImpl();
//		if(log == null){
//			
//		}
		if(flag){
			if(log.getAttributes().get(xConstants.FORMAT).equals(xConstants.UPDATE)){
				shownObjectModel = event.getObjectModelUpdate();
			}
			else{
				shownObjectModel = log.createObjectModelUpdate(event);
			}	
			
			objectSet = shownObjectModel.getObjectSet();
			objectRelationSet = shownObjectModel.getObjectRelationSet();
			objectModleTreeNode = new DefaultMutableTreeNode(new TreeEntry("ObjectModelUpdate"));
		}
		else{
			shownObjectModel = log.createCompleteObjectModel(event);
			objectSet = shownObjectModel.getObjectSet();
			objectRelationSet = shownObjectModel.getObjectRelationSet();
			objectModleTreeNode = new DefaultMutableTreeNode(new TreeEntry("ObjectModel"));
		}				

		objectModleTreeNode.add(generateObjectSetNode(objectSet,TREE_NODE_OBJECTS + "(" + objectSet.size() + ")"));
		objectModleTreeNode.add(generateRelationSetNode(objectRelationSet,TREE_NODE_RELATIONS + "(" + objectRelationSet.size() + ")"));
		root.add(objectModleTreeNode);
		
		eventTreePanel.updateTreePanel(root, "selected_event");
		JTree eventTree = eventTreePanel.getJTree();
		// Expand all top-level rows of the Metadata Tree if in the simplified mode
		if(flag){
			for (int i = 0; i < eventTree.getRowCount(); i++) {
				eventTree.expandRow(i);
			}
		}	
		//expand "Event" and "References" nodes
		else{
			eventTree.expandPath(new TreePath(eventNode.getPath()));
			eventTree.expandPath(new TreePath(references.getPath()));
		}
	
	}

	/**
	 * Update the attribute panel for a selected object in the object model panel
	 * @param object
	 * @param event
	 * @param log
	 * @param relevantObjectList
	 * @param relevantObjectRelationList
	 */	
	public void updateAttributePanelForObject(XOCObject object, XOCEvent event, XOCLog log, 
			List<XOCObject> relevantObjectList, List<XOCORelation>  relevantObjectRelationList) {	
		System.out.println("enter updateObjectAttributes");		
		
		setOpaque(false);
		setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
		setLayout(new BoxLayout(this, BoxLayout.X_AXIS));

		// Create the root node of the tree.
		String rootLabel = null;
		if(object == null && event == null) {
			rootLabel = "no node is selected";
			object = new XOCObjectImpl("");
		}
		else if(object != null)rootLabel = object.getAttributes().get("id").toString();
		else if(event != null) rootLabel = event.getAttributes().get("id").toString();
		DefaultMutableTreeNode root = new DefaultMutableTreeNode(rootLabel);

		// Build up list of Attributes
		if(object != null){		
			DefaultMutableTreeNode objectAttributes;
			if(logBrowser != null && logBrowser.getGraphPanel() != null && logBrowser.getGraphPanel().getSimplifiedFlag()){
				objectAttributes = generateAttributesNode(object.createUpdatedAttributeMap(event));
			}
			else{
				objectAttributes = generateAttributesNode(object.getAttributes());
			}
			root.add(objectAttributes);
			
			DefaultMutableTreeNode relevantObjects = generateRelevantObjectsNode(relevantObjectList);
			root.add(relevantObjects);
			
			DefaultMutableTreeNode relevantRelations = generateRelevantObjectRelationsNode(relevantObjectRelationList);
			root.add(relevantRelations);
	
			DefaultMutableTreeNode referredEvents = generateReferredEventsNode(object, log);
			root.add(referredEvents);
			
			DefaultMutableTreeNode lifecycle = generateLifecycleNode(object, log, event);
			root.add(lifecycle);
			
			ConformanceCheckingResultView conformanceCheckingResult = null;
			if(logBrowser.getParentLogViewer() != null){
				conformanceCheckingResult = logBrowser.
						getParentLogViewer().getConformanceCheckingResult();
			}

			if(conformanceCheckingResult != null && event != null){		
				System.out.println("add deviation information node in updateAttributePanelForObject in AttributesHierarchyPanel;");

				//add the deviating information node
				DefaultMutableTreeNode deviationInformation = new DefaultMutableTreeNode(new TreeEntry("Deviation"));
				root.add(deviationInformation);
				
				//collect the deviating information for the event
				ResultObjectVersionView objectVersionView = new ResultObjectVersionView(conformanceCheckingResult);
				objectVersionView.collectDeviatingInformationForObjectVersion(object, event, log);;
				
				//add inexistent object information				
				if(!object.isAliveAtEvent(event)){
					DefaultMutableTreeNode inexistentObjectNode = new DefaultMutableTreeNode(new TreeEntry("inexistent_object"));
					deviationInformation.add(inexistentObjectNode);
				}
				
				//add violated class relation cardinality			
				Map<ClassEdge, List<XOCObject>> alwaysViolatedClassRelationVSObjectList = objectVersionView.getViolatedClassRelationAlwaysVSObjectListMap();
				Map<ClassEdge, List<XOCObject>> eventuallyViolatedAOCRelationVSObjectList = objectVersionView.getViolatedClassRelationEventuallyVSObjectListMap();
				List<ClassEdge> violatedClassRelationList = new ArrayList<ClassEdge>(alwaysViolatedClassRelationVSObjectList.keySet());
				for(ClassEdge relation : eventuallyViolatedAOCRelationVSObjectList.keySet()){
					if(!violatedClassRelationList.contains(relation)){
						violatedClassRelationList.add(relation);
					}
				}
				if(violatedClassRelationList.size() > 0){
					DefaultMutableTreeNode violatedClassRelationListNode = new DefaultMutableTreeNode(new TreeEntry("violated_class_relation_list"));
					deviationInformation.add(violatedClassRelationListNode);
					for(ClassEdge relation : violatedClassRelationList){
						DefaultMutableTreeNode violatedClassRelationNode = new DefaultMutableTreeNode(new TreeEntry(relation.getID()));
						violatedClassRelationListNode.add(violatedClassRelationNode);
						if(alwaysViolatedClassRelationVSObjectList.containsKey(relation)){
							violatedClassRelationNode.add(new DefaultMutableTreeNode(
									new TreeEntry(xConstants.source_always_cardinality + ": " + relation.getSourceAlwaysCardinality().toString())));
						}
						if(eventuallyViolatedAOCRelationVSObjectList.containsKey(relation)){
							violatedClassRelationNode.add(new DefaultMutableTreeNode(
									new TreeEntry(xConstants.source_eventually_cardinality + ": " + relation.getSourceEventuallyCardinality().toString())));
						}
					}
				}
				
				//add violated AOC relation information
				List<AOCEdge> alwaysViolatedAOCRelationList = objectVersionView.getAlwaysViolatedAOCRelationList();
				List<AOCEdge> eventuallyViolatedAOCRelationList = objectVersionView.getEventuallyViolatedAOCRelationList();
				List<AOCEdge> violatedAOCRelationList = new ArrayList<AOCEdge>();
				violatedAOCRelationList.addAll(alwaysViolatedAOCRelationList);
				for(AOCEdge relation : eventuallyViolatedAOCRelationList){
					if(!violatedAOCRelationList.contains(relation)){
						violatedAOCRelationList.add(relation);
					}
				}
				if(violatedAOCRelationList.size() > 0){
					DefaultMutableTreeNode violatedAOCRelationListNode = new DefaultMutableTreeNode(new TreeEntry("violated_AOC_relation_list"));
					deviationInformation.add(violatedAOCRelationListNode);
					for(AOCEdge relation : violatedAOCRelationList){
						DefaultMutableTreeNode violatedAOCRelationNode = new DefaultMutableTreeNode(new TreeEntry(relation.getID()));
						violatedAOCRelationListNode.add(violatedAOCRelationNode);
						if(alwaysViolatedAOCRelationList.contains(relation)){
							violatedAOCRelationNode.add(new DefaultMutableTreeNode(
									new TreeEntry(xConstants.source_always_cardinality + ": " + relation.getActivityAlwaysCardinality().toString())));
						}
						if(eventuallyViolatedAOCRelationList.contains(relation)){
							violatedAOCRelationNode.add(new DefaultMutableTreeNode(
									new TreeEntry(xConstants.source_eventually_cardinality + ": " + relation.getActivityEventuallyCardinality().toString())));
						}
					}
				}
			
			}		
		}		

		objectTreePanel.updateTreePanel(root, "selected_element");
		JTree objectTree = objectTreePanel.getJTree();
		// Expand all top-level rows of the Metadata Tree.
		for (int i = 0; i < objectTree.getRowCount(); i++) {
			objectTree.expandRow(i);
		}
	}
	
	/**
	 * Update the attribute panel for a selected event in the object model panel
	 * @param event
	 */
	public void updateAttributePanelForEvent(XOCEvent event) {			
		setOpaque(false);
		setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
		setLayout(new BoxLayout(this, BoxLayout.X_AXIS));

		// Create the root node of the tree.
		String eventId = event.getAttributes().get("id").toString();
		DefaultMutableTreeNode root = new DefaultMutableTreeNode(eventId);
		
		//add the attributes node
		DefaultMutableTreeNode eventAttributes = generateAttributesNode(event.getAttributes());
		root.add(eventAttributes);
		
		//add the references node
		DefaultMutableTreeNode references = generateObjectSetNode(event.getRelatedObjects(), TREE_NODE_REFERENCES + "(" + event.getRelatedObjects().size() + ")");
		root.add(references);

		//add deviation information
		ConformanceCheckingResultView conformanceCheckingResult = logBrowser.
				getParentLogViewer().getConformanceCheckingResult();
		if(conformanceCheckingResult != null){		
			System.out.println("add deviation information node in updateAttributePanelForEvent in AttributesHierarchyPanel;");

			//add the deviating information node
			DefaultMutableTreeNode deviationInformation = new DefaultMutableTreeNode(new TreeEntry("Deviation"));
			root.add(deviationInformation);
			
			//collect the deviating information for the event
			ResultEventView eventView = new ResultEventView(conformanceCheckingResult);
			eventView.collectDeviatingInformationForEvent(event);
			
			//add inexistent activity information
			Map<XOCEvent, String> eventVSInexistentActivity = eventView.getDeviatingEventVSInexistentActivityMap();
			if(eventVSInexistentActivity.containsKey(event)){
				DefaultMutableTreeNode inexistentActivitytNode = new DefaultMutableTreeNode(new TreeEntry("inexistent_activity: " + eventVSInexistentActivity.get(event)));
				deviationInformation.add(inexistentActivitytNode);
			}
			
			//add inexistent object reference information
			Map<XOCEvent, List<XOCObject>> eventVSInexistentObjectReferenceList = eventView.getDeviatingEventVSInexistentObjectReferenceListMap();
			if(eventVSInexistentObjectReferenceList.containsKey(event) && eventVSInexistentObjectReferenceList.get(event).size() > 0){
				DefaultMutableTreeNode inexistentObjectReferenceListNode = new DefaultMutableTreeNode(new TreeEntry("inexistent_object_list"));
				deviationInformation.add(inexistentObjectReferenceListNode);
				for(XOCObject object : eventVSInexistentObjectReferenceList.get(event)){
					inexistentObjectReferenceListNode.add(new DefaultMutableTreeNode(new TreeEntry(object.getAttributes().get(xConstants.ID).toString())));
				}
			}
			
			//add incorrect referred class information with objects
			Map<XOCEvent, Map<String, List<XOCObject>>> eventVSReferredIncorrectClassVSObjectList = eventView.getDeviatingEventVSReferredIncorrectClassVSObjectListMap();
			if(eventVSReferredIncorrectClassVSObjectList.containsKey(event) && eventVSReferredIncorrectClassVSObjectList.get(event).size() > 0){
				DefaultMutableTreeNode referredIncorrectClassVSObjectListNode = new DefaultMutableTreeNode(new TreeEntry("referred_unrelated_class_list"));
				deviationInformation.add(referredIncorrectClassVSObjectListNode);
				for(String className : eventVSReferredIncorrectClassVSObjectList.get(event).keySet()){
					DefaultMutableTreeNode referredIncorrectClassNode = new DefaultMutableTreeNode(new TreeEntry(className));
					referredIncorrectClassVSObjectListNode.add(referredIncorrectClassNode);
					for(XOCObject object : eventVSReferredIncorrectClassVSObjectList.get(event).get(className)){
						referredIncorrectClassNode.add(new DefaultMutableTreeNode(new TreeEntry(object.getAttributes().get(xConstants.ID).toString())));
					}
				}
			}
			
			//add violated AOC relation information
			Map<XOCEvent, List<AOCEdge>> eventVSAOCRelationList = eventView.getDeviatingEventVSAOCRelationListMap();
			if(eventVSAOCRelationList.containsKey(event) && eventVSAOCRelationList.get(event).size() > 0){
				DefaultMutableTreeNode violatedAOCRelationListNode = new DefaultMutableTreeNode(new TreeEntry("violated_AOC_relation_list"));
				deviationInformation.add(violatedAOCRelationListNode);
				for(AOCEdge relation : eventVSAOCRelationList.get(event)){
					violatedAOCRelationListNode.add(new DefaultMutableTreeNode(new TreeEntry(relation.getID())));
				}
			}
		
			//add violated behavioral constraint information
			Map<XOCEvent, Map<ActivityEdge, List<XOCEvent>>> eventVSBehavioralConstraintVSPrecedenceEventList = eventView.getDeviatingEventVSBehavioralConstraintVSPrecedenceEventListMap();
			Map<XOCEvent, Map<ActivityEdge, List<XOCEvent>>> eventVSBehavioralConstraintVSResponseEventList = eventView.getDeviatingEventVSBehavioralConstraintVSResponseEventListMap();		
			if(eventVSBehavioralConstraintVSPrecedenceEventList.containsKey(event) && eventVSBehavioralConstraintVSPrecedenceEventList.get(event).size() > 0){
				DefaultMutableTreeNode violatedBehavioralConstraintListNode = new DefaultMutableTreeNode(new TreeEntry("violated_behavioral_constraint_list"));
				deviationInformation.add(violatedBehavioralConstraintListNode);
				for(ActivityEdge relation : eventVSBehavioralConstraintVSPrecedenceEventList.get(event).keySet()){
					DefaultMutableTreeNode violatedBehavioralConstraintNode = new DefaultMutableTreeNode(new TreeEntry(relation.getID()));
					violatedBehavioralConstraintListNode.add(violatedBehavioralConstraintNode);
					//current, just add the number of precedence events and response events
					List<XOCEvent> eventVSPrecedenceEventList = eventVSBehavioralConstraintVSPrecedenceEventList.get(event).get(relation);
					List<XOCEvent> eventVSResponseEventList = eventVSBehavioralConstraintVSResponseEventList.get(event).get(relation);
					violatedBehavioralConstraintNode.add(new DefaultMutableTreeNode(new TreeEntry(mxConstants.PRECEDENCE + ": " + eventVSPrecedenceEventList.size())));
					violatedBehavioralConstraintNode.add(new DefaultMutableTreeNode(new TreeEntry(mxConstants.RESPONSE + ": " + eventVSResponseEventList.size())));					
				}
			}
		}
		
		objectTreePanel.updateTreePanel(root, "selected_element");
		JTree objectTree = objectTreePanel.getJTree();
		// Expand all top-level rows of the Metadata Tree.
		for (int i = 0; i < objectTree.getRowCount(); i++) {
			objectTree.expandRow(i);
		}
	}
	
	/**
	 * Update the attribute panel for a selected object relation in the object model panel
	 * @param relation
	 */
	public void updateAttributePanelForObjectRelation(XOCORelation relation, XOCLog log, XOCEvent event) {	
		System.out.println("enter updateObjectAttributes");
		setOpaque(false);
		setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
		setLayout(new BoxLayout(this, BoxLayout.X_AXIS));

		// Create the root node of the tree.
		String relationId = relation.getAttributes().get("id").toString();
		DefaultMutableTreeNode root = new DefaultMutableTreeNode(TREE_ROOT_NODE_RELATION + ": " + relationId);

		// Build up list of Attributes
		DefaultMutableTreeNode objectAttributes = generateAttributesNode(relation.getAttributes());
		root.add(objectAttributes);		
						
		List<XOCObject> relevantObjectList = new ArrayList<XOCObject>();
		relevantObjectList.add(relation.getSourceObject());
		relevantObjectList.add(relation.getTargetObject());
		DefaultMutableTreeNode relevantObjects = generateRelevantObjectsNode(relevantObjectList);
		root.add(relevantObjects);
		
		//add life cycle information
		DefaultMutableTreeNode lifecycle = generateLifecycleNode(relation, log, event);
		root.add(lifecycle);
		
		objectTreePanel.updateTreePanel(root, "selected_object_relation");
		JTree objectTree = objectTreePanel.getJTree();
		// Expand all top-level rows of the Metadata Tree.
		for (int i = 0; i < objectTree.getRowCount(); i++) {
			objectTree.expandRow(i);
		}		
	}
	
	/**
	 * Update the attribute panel for a selected reference relation in the object model panel
	 * @param relation
	 */
	public void updateAttributePanelForReferenceRelation(XOCRRelation relation) {	
		System.out.println("enter updateAttributePanelForReferenceRelation");
		setOpaque(false);
		setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
		setLayout(new BoxLayout(this, BoxLayout.X_AXIS));

		// Create the root node of the tree.
		String relationId = relation.getAttributes().get("id").toString();
		DefaultMutableTreeNode root = new DefaultMutableTreeNode(TREE_ROOT_NODE_RELATION + ": " + relationId);

		// Build up list of Attributes
		DefaultMutableTreeNode objectAttributes = generateAttributesNode(relation.getAttributes());
		root.add(objectAttributes);		
						
		List<XOCObject> relevantObjectList = new ArrayList<XOCObject>();
		relevantObjectList.add(relation.getTargetObject());
		DefaultMutableTreeNode relevantObjects = generateRelevantObjectsNode(relevantObjectList);
		root.add(relevantObjects);
		
		objectTreePanel.updateTreePanel(root, "selected_object_relation");
		JTree objectTree = objectTreePanel.getJTree();
		// Expand all top-level rows of the Metadata Tree.
		for (int i = 0; i < objectTree.getRowCount(); i++) {
			objectTree.expandRow(i);
		}		
	}
	
	/**
	 * generate attributes node (excluding lifecycle attribute)
	 * @param attributes
	 * @return
	 */
	private DefaultMutableTreeNode generateAttributesNode(XAttributeMap attributes) {
		DefaultMutableTreeNode attributesNode = new DefaultMutableTreeNode(new TreeEntry(TREE_NODE_ATTRIBUTES) + "(" + attributes.keySet().size() + ")");
		for(String key : attributes.keySet()){
			if(!key.equals(xConstants.lifecycle)){
				attributesNode.add(new DefaultMutableTreeNode(new TreeEntry(key + " : " + attributes.get(key).toString())));
			}			
		}		
		return attributesNode;
	}
	
	/**
	 * generate node indicating lifecycle information (birth, death)
	 * @param attributes
	 * @return
	 */
	private DefaultMutableTreeNode generateLifecycleNode(XOCObject object, XOCLog log, XOCEvent event) {
		DefaultMutableTreeNode lifecycleNode = new DefaultMutableTreeNode(new TreeEntry(TREE_NODE_LIFECYCLE));
		
		//indicate the object life cycle attribute (add, update, delete, ) in the current event
		if(event != null){
			String currentState = object.getEventOrderNumVSObjectStateMap().get(event.getOrderNum());
			if(currentState == null){
				currentState = xConstants.remain;
			}
			DefaultMutableTreeNode lifecycleAttributeNode = new DefaultMutableTreeNode(
					new TreeEntry(mxConstants.lifecycle_attribute + " : " + currentState));
			lifecycleNode.add(lifecycleAttributeNode);
		}		
		
		//indicate the object life cycle period (start, end) list 
		for(IntegerPair period : object.getLifecyclePeriodList()){
			int startEventOrderNum = period.getFirstValue();
			int endEventOrderNum = period.getSecondValue() != null ? period.getSecondValue() : log.size();
			lifecycleNode.add(new DefaultMutableTreeNode(new TreeEntry(mxConstants.startEventOrderNum + " : " + startEventOrderNum + " , "
					+ mxConstants.endEventOrderNum + " : " + endEventOrderNum)));
		}			
		return lifecycleNode;
	}
	
	/**
	 * generate node indicating lifecycle information (birth, death)
	 * @param attributes
	 * @return
	 */
	private DefaultMutableTreeNode generateLifecycleNode(XOCORelation relation, XOCLog log, XOCEvent event) {
		DefaultMutableTreeNode lifecycleNode = new DefaultMutableTreeNode(new TreeEntry(TREE_NODE_LIFECYCLE));
		
		//indicate the object life cycle attribute (add, update, delete, ) in the current event
		if(event != null){
			String currentState = relation.getEventOrderNumVSObjectRelationStateMap().get(event.getOrderNum());
			if(currentState == null){
				currentState = xConstants.remain;
			}
			DefaultMutableTreeNode lifecycleAttributeNode = new DefaultMutableTreeNode(
					new TreeEntry(mxConstants.lifecycle_attribute + " : " + currentState));
			lifecycleNode.add(lifecycleAttributeNode);
		}		
		
		//indicate the object life cycle period (start, end) list 
		for(IntegerPair period : relation.getLifecyclePeriodList()){
			int startEventOrderNum = period.getFirstValue();
			int endEventOrderNum = period.getSecondValue() != null ? period.getSecondValue() : log.size();
			lifecycleNode.add(new DefaultMutableTreeNode(new TreeEntry(mxConstants.startEventOrderNum + " : " + startEventOrderNum + " , "
					+ mxConstants.endEventOrderNum + " : " + endEventOrderNum)));
		}			
		return lifecycleNode;
	}
	
	private DefaultMutableTreeNode generateRelevantObjectsNode(List<XOCObject> relevantObjectList) {
		DefaultMutableTreeNode relevantObjectsNode = new DefaultMutableTreeNode(new TreeEntry(TREE_NODE_RELATEDOBJECTS) + "(" + relevantObjectList.size() + ")");
		for(XOCObject object : relevantObjectList){
			relevantObjectsNode.add(generateObjectNode(object));		
		}		
		return relevantObjectsNode;
	}
	
	private DefaultMutableTreeNode generateRelevantObjectRelationsNode(List<XOCORelation>  relevantObjectRelationList) {
		DefaultMutableTreeNode relevantObjectRelationsNode = new DefaultMutableTreeNode(new TreeEntry(TREE_NODE_RELATEDOBJECTRELATION) + "(" + relevantObjectRelationList.size() + ")");
		for(XOCORelation objectRelation : relevantObjectRelationList){
			relevantObjectRelationsNode.add(generateRelationNode(objectRelation));		
		}		
		return relevantObjectRelationsNode;
	}
	
	private DefaultMutableTreeNode generateReferredEventsNode(XOCObject object, XOCLog log) {
		DefaultMutableTreeNode referredEventsNode = new DefaultMutableTreeNode(new TreeEntry(TREE_NODE_REFERREDEVENTS));
		for(XOCEvent event : log.getAllEvents()){
			for(XOCObject reference : event.getRelatedObjects()){
				if(reference.getAttributes().get("id").toString().equals(object.getAttributes().get("id").toString())){
					referredEventsNode.add(new DefaultMutableTreeNode(new TreeEntry(event.getAttributes().get("id").toString())));
				}
			}			
		}		
		return referredEventsNode;
	}
	
	private DefaultMutableTreeNode generateObjectNode(XOCObject object){
		DefaultMutableTreeNode objectNode = new DefaultMutableTreeNode(new TreeEntry(object.getAttributes().get("id").toString()));
		for(String key : object.getAttributes().keySet()){
			if(!key.equals(xConstants.lifecycle)){
				objectNode.add(new DefaultMutableTreeNode(new TreeEntry(key + " : " + object.getAttributes().get(key).toString())));
			}				
		}		
		return  objectNode;
	}
	
	private DefaultMutableTreeNode generateObjectSetNode(List<XOCObject> objects, String name){
		DefaultMutableTreeNode objectSetNode = new DefaultMutableTreeNode(new TreeEntry(name));
		for (XOCObject object : objects) {
			objectSetNode.add(generateObjectNode(object));
		}
		return objectSetNode;
	}

	private DefaultMutableTreeNode generateRelationNode(XOCORelation relation){
		DefaultMutableTreeNode relationNode = new DefaultMutableTreeNode(new TreeEntry(relation.getAttributes().get("id").toString()));
		for(String key : relation.getAttributes().keySet()){
			if(!key.equals(xConstants.lifecycle)){
				relationNode.add(new DefaultMutableTreeNode(new TreeEntry(key + " : " + relation.getAttributes().get(key).toString())));
			}				
		}		
		return  relationNode;
	}
	
	private DefaultMutableTreeNode generateRelationSetNode(List<XOCORelation> relations, String name){
		DefaultMutableTreeNode relationSetNode = new DefaultMutableTreeNode(new TreeEntry(name));
		for (XOCORelation relation : relations) {
			relationSetNode.add(generateRelationNode(relation));
		}
		return relationSetNode;
	}
	
	private class TreeEntry {
		private String key;
		private String displaytext;
		
		public TreeEntry(String key) {
			this(key, key);
		}
		
		public TreeEntry(String key, String displaytext) {
			this.key = key;
			this.displaytext = displaytext;
		}
		
		public String getKey() {
			return this.key;
		}
		
		public String getDisplayText() {
			return this.displaytext;
		}
		
		public String toString() {
			return getDisplayText();
		}
	}
	
}