package org.processmining.ocbc.log.inspector.panels;

import java.awt.Color;
import java.awt.Component;
import java.awt.Cursor;
import java.awt.Dimension;
import java.util.ArrayList;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JScrollBar;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.ListSelectionModel;
import javax.swing.ScrollPaneConstants;
import javax.swing.SwingConstants;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import org.processmining.common.runningtime.RunningTime;
import org.processmining.framework.util.Pair;
import org.processmining.ocbc.log.inspector.interaction.GraphMouseListener;
import org.processmining.ocbc.log.model.XOCEvent;
import org.processmining.ocbc.log.model.XOCLog;

import com.fluxicon.slickerbox.components.RoundedPanel;
import com.fluxicon.slickerbox.ui.SlickerScrollBarUI;

public class EventListPanel extends RoundedPanel {

	
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	protected JList<Object> instancesList;
	private XOCLogBrowser logUI;
	private XOCLog log;
	boolean generateCoordinatesFlag = false;
	
	public static final Color COLOR_ENCLOSURE_BG = new Color(40, 40, 40);
	public static final Color COLOR_NON_FOCUS = new Color(70, 70, 70);
	public static final Color COLOR_LIST_BG = new Color(60, 60, 60);
	public static final Color COLOR_LIST_BG_LOWER = new Color(45, 45, 45);
	public static final Color COLOR_LIST_FG = new Color(180, 180, 180);
	public static final Color COLOR_LIST_SELECTION_BG = new Color(80, 0, 0);
	public static final Color COLOR_LIST_SELECTION_BG_LOWER = new Color(30, 10, 10);
	public static final Color COLOR_LIST_SELECTION_FG = new Color(240, 240, 240);
	public static final Color COLOR_LIST_TAGGED_SELECTION_BG = Color.RED;
	public static final Color COLOR_LIST_TAGGED_BG = new Color(60, 60, 0);

	
	public EventListPanel(int aRadius, int anOuterBorder, int anInnerBorder){
		super(aRadius, anOuterBorder, anInnerBorder);	
	}
	
	public void initializePanel(final XOCLogBrowser logUI, XOCLog log){
		this.logUI = logUI;
		this.log = log;
		instancesList = new JList();
		instancesList.setBackground(COLOR_LIST_BG);
		instancesList.setForeground(COLOR_LIST_FG);
		instancesList.setSelectionBackground(COLOR_LIST_SELECTION_BG);
		instancesList.setSelectionForeground(COLOR_LIST_SELECTION_FG);
		instancesList.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
		instancesList.addListSelectionListener(new ListSelectionListener() {
			public void valueChanged(ListSelectionEvent e) {
				System.out.println("enter valueChanged in initializePanel");
				//change the shape of mouse to waiting state
				logUI.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
				
				instancesSelectionChanged();
				
				//change the shape of mouse to default state
				logUI.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
			}
		});

		updateInstancesList(log);
		// reset events list
		instancesList.clearSelection();
		

		JScrollPane instancesScrollPane = new JScrollPane(instancesList);
		instancesScrollPane.setOpaque(false);
		instancesScrollPane.setBorder(BorderFactory.createEmptyBorder());
		instancesScrollPane.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);
		instancesScrollPane.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		JScrollBar vBar = instancesScrollPane.getVerticalScrollBar();
		vBar.setUI(new SlickerScrollBarUI(vBar, COLOR_ENCLOSURE_BG, new Color(160, 160, 160), COLOR_NON_FOCUS, 4, 12));
		JScrollBar hBar = instancesScrollPane.getHorizontalScrollBar();
		hBar.setUI(new SlickerScrollBarUI(hBar, COLOR_ENCLOSURE_BG, new Color(160, 160, 160), COLOR_NON_FOCUS, 4, 12));

		
		JLabel instancesListLabel = new JLabel("Events (" + log.getAllEvents().size() + ")");
		instancesListLabel.setOpaque(false);
		instancesListLabel.setForeground(COLOR_LIST_SELECTION_FG);
		instancesListLabel.setFont(instancesListLabel.getFont().deriveFont(13f));
		instancesListLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
		instancesListLabel.setHorizontalAlignment(SwingConstants.CENTER);
		instancesListLabel.setHorizontalTextPosition(SwingConstants.CENTER);	
		
		this.setBackground(COLOR_ENCLOSURE_BG);
		this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		this.setMaximumSize(new Dimension(180, 1000));
		this.setPreferredSize(new Dimension(180, 1000));
		this.add(instancesListLabel);	
		this.add(Box.createVerticalStrut(8));		
		this.add(instancesScrollPane);
	}
	
	private void updateInstancesList(XOCLog inputLog) {

		if (inputLog.getAllEvents().size() == 0) {
			return;
		}

		List<Pair<Integer, String>> instances = new ArrayList<Pair<Integer, String>>(inputLog.getAllEvents().size());
		for (int i = 0; i < inputLog.getAllEvents().size(); i++) {
			String name = inputLog.getAllEvents().get(i).getAttributes().get("id").toString();
			instances.add(new Pair<Integer, String>(i+1, name));
		}
		instancesList.setListData(instances.toArray());
	}

	protected void instancesSelectionChanged() {	
		System.out.printf("index:  " + instancesList.getSelectedIndex());
		
		long uint = 1;	
		long startTime = System.currentTimeMillis(); 
		long durationTime = System.currentTimeMillis();
		long startTimeTotal = System.currentTimeMillis(); 
		RunningTime.writeTimeInformationIntoFile("******************instancesSelectionChanged*********************", false, "D:\\timeVisualizingXOCLog.txt");
		
		
		int[] selectedIndices = instancesList.getSelectedIndices();
		XOCEvent event = null;

		ObjectModelJGraphXPanel graphPanel = logUI.getGraphPanel();
		AttributesHierarchyPanel attributesPanel = logUI.getAttributesPanel();
		GraphMouseListener graphMouseListener = logUI.getGraphMouseListener();		
		graphMouseListener.setSelectedEvent(event);
		
		if(instancesList.getSelectedIndex() < 0)
			event = log.getAllEvents().get(0);
		else
			event = log.getAllEvents().get(selectedIndices[0]);
		
		System.out.print("\n event id: " + event.getAttributes().get("id").toString());
		
		if(!generateCoordinatesFlag){
			generateCoordinatesFlag = true;
			int logSize = log.getAllEvents().size();
//			Map<String, Pair<Integer, Integer>> nodeVSCoordinateMap = graphPanel.generateObjectCoordinates(
//					log, graphPanel.getHeight(), graphPanel.getWidth(),0);
//			graphPanel.setNodeVSCoordinateMap(nodeVSCoordinateMap);
			graphPanel.generateObjectCoordinates(log, graphPanel.getHeight(), graphPanel.getWidth(),0);

		}
					
		logUI.removeAll();						
		
		graphPanel.updateEventGraph(event, 1, true);

		JSplitPane split1 = logUI.borderlessSplitPane(JSplitPane.HORIZONTAL_SPLIT, this, graphPanel);
		split1.setResizeWeight(0.1);
		JSplitPane split2 = logUI.borderlessSplitPane(JSplitPane.HORIZONTAL_SPLIT, split1, attributesPanel);
		split2.setResizeWeight(0.7);
		logUI.add(split2);		
		logUI.revalidate();
		logUI.repaint();
		
        durationTime = (System.currentTimeMillis() - startTime)/uint;
		startTime = System.currentTimeMillis();
		RunningTime.writeTimeInformationIntoFile("complete the updateEventGraph High level end: " + System.currentTimeMillis()/uint, false, "D:\\timeVisualizingXOCLog.txt");
		RunningTime.writeTimeInformationIntoFile("~~~~~~~~~~~~~~complete the updateEventGraph High level duration time: " + durationTime, false, "D:\\timeVisualizingXOCLog.txt");
		
		attributesPanel.updateEventAttributes(event, graphPanel.getSimplifiedFlag());		// graphPanel.getCompleteModel()
		
        durationTime = (System.currentTimeMillis() - startTime)/uint;
		startTime = System.currentTimeMillis();
		RunningTime.writeTimeInformationIntoFile("complete the updateEventAttributes end: " + System.currentTimeMillis()/uint, false, "D:\\timeVisualizingXOCLog.txt");
		RunningTime.writeTimeInformationIntoFile("~~~~~~~~~~~~~~complete the updateEventAttributes duration time: " + durationTime, false, "D:\\timeVisualizingXOCLog.txt");
		
		long totalTime = (System.currentTimeMillis() - startTimeTotal)/uint;
		RunningTime.writeTimeInformationIntoFile("~~~~~~~~~~~~~~complete the total duration time: " + totalTime, false, "D:\\timeVisualizingXOCLog.txt");	
	}	
}




