package org.processmining.ocbc.log.inspector.panels;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import org.processmining.common.runningtime.RunningTime;
import org.processmining.common.structure.IntegerPair;
import org.processmining.framework.util.Pair;
import org.processmining.mxgraph.model.mxCell;
import org.processmining.mxgraph.model.mxGraphModel;
import org.processmining.mxgraph.swing.mxGraphComponent;
import org.processmining.mxgraph.swing.mxGraphOutline;
import org.processmining.mxgraph.util.mxConstants;
import org.processmining.mxgraph.view.mxGraph;
import org.processmining.ocbc.conformance.visualization.views.ConformanceCheckingResultView;
import org.processmining.ocbc.conformance.visualization.views.ResultEventView;
import org.processmining.ocbc.conformance.visualization.views.ResultObjectVersionView;
import org.processmining.ocbc.log.factory.XFactoryRegistry;
import org.processmining.ocbc.log.inspector.graph.EventCell;
import org.processmining.ocbc.log.inspector.graph.InexistentObjectCell;
import org.processmining.ocbc.log.inspector.graph.ObjectCell;
import org.processmining.ocbc.log.inspector.graph.ObjectEdgeCell;
import org.processmining.ocbc.log.inspector.graph.ReferenceEdgeCell;
import org.processmining.ocbc.log.inspector.interaction.GraphMouseListener;
import org.processmining.ocbc.log.inspector.interaction.GraphMoveMouseListener;
import org.processmining.ocbc.log.inspector.interaction.GraphZoomMouseListener;
import org.processmining.ocbc.log.model.XAttribute;
import org.processmining.ocbc.log.model.XOCAttribute;
import org.processmining.ocbc.log.model.XOCEvent;
import org.processmining.ocbc.log.model.XOCLog;
import org.processmining.ocbc.log.model.XOCORelation;
import org.processmining.ocbc.log.model.XOCObject;
import org.processmining.ocbc.log.model.XOCObjectModel;
import org.processmining.ocbc.log.model.XOCRRelation;
import org.processmining.ocbc.log.model.impl.XOCObjectModelImpl;
import org.processmining.ocbc.log.model.impl.XOCRRelationImpl;
import org.processmining.ocbc.log.model.impl.xConstants;
import org.processmining.ocbc.models.MyOriginalElement;
import org.processmining.ocbc.models.visualization.cell.MyOriginalCell;

import com.fluxicon.slickerbox.components.RoundedPanel;
import com.fluxicon.slickerbox.components.SlickerButton;

public class ObjectModelJGraphXPanel extends RoundedPanel {

	public static final Color COLOR_ENCLOSURE_BG = new Color(40, 40, 40);
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	protected XOCLog log;
	protected XOCLogBrowser browser;
	protected mxGraph graphx;
	protected mxGraphComponent graphComponent;
	protected Object parent;
	protected GraphMouseListener graphMouseListener;
	protected GraphZoomMouseListener graphZoomMouseListener;
	protected GraphMoveMouseListener graphMoveMouseListener;
	protected Map<mxCell, MyOriginalCell> mxcellVSCellMap;//build mapping between each cell (self-defined cell) and each mxcell (jgraphx)	
	protected Map<MyOriginalCell, mxCell> cellMapVSMxcell;//build mapping between each mxcell (jgraphx) and each cell  (self-defined cell)	
	protected Map<String, Pair<Integer, Integer>> nodeVSCoordinateMap;//record coordinates of each node(object from OM or event)
	protected Map<String, Pair<Integer, Integer>> inexistentObjectVSCoordinateMap;//record coordinates of each node(inexistent object)
	protected mxGraphOutline graphOutline;		
	protected XOCEvent selectedEvent;
	protected JTextField searchText;
	protected JButton simplifyButton = new JButton("simplify");
	
	protected JLabel simplifyLabel = new JLabel("simplified view");
	protected JLabel objectNumberLabel = new JLabel("object: 0");
	protected JLabel relationNumberLabel = new JLabel("relation: 0");
	
	protected long objectShownMaxNum = 200;
	protected List<String> shownObjectIdList = new ArrayList<String>();
	
	protected XOCObjectModel objectModelComplete = new XOCObjectModelImpl();
	
	protected JCheckBox simplifyBox = new JCheckBox();
	protected double zoomRate = 1;
//	//achieve the deviation information; in other plugins except cc plugin, it is empty
//	protected ConformanceCheckingResultView conformanceCheckingResult;
	 
	private boolean simplifiedFlag = true;//false: the panel shows all object model; true: just shows object model difference (updated part)
	public ObjectModelJGraphXPanel(final XOCLogBrowser browser, int aRadius, int anOuterBorder, int anInnerBorder, XOCLog log, GraphMouseListener graphMouseListener,
			GraphZoomMouseListener graphZoomMouseListener, GraphMoveMouseListener graphMoveMouseListener){
		super(aRadius, anOuterBorder, anInnerBorder);
		this.log = log;		
		this.browser = browser;	
	    this.graphMouseListener = graphMouseListener;
	    this.graphZoomMouseListener = graphZoomMouseListener;
	    this.graphMoveMouseListener = graphMoveMouseListener;
	    this.graphx = new mxGraph();
	    this.parent = graphx.getDefaultParent();  
	    this.graphComponent = new mxGraphComponent(graphx);
	    this.initialize();	
	}

	public mxGraph getGraph(){
		return graphx;
	}
	
	public XOCObjectModel getCompleteModel(){
		return objectModelComplete;
	}
	
	public XOCLogBrowser getParentLogBrowser(){
		return browser;
	} 
	
//	public void setConformanceCheckingResult(ConformanceCheckingResultView conformanceCheckingResult){
//		this.conformanceCheckingResult = conformanceCheckingResult;
//	}
	
	public void initialize(){
		this.setBackground(COLOR_ENCLOSURE_BG);
		this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		this.setMaximumSize(new Dimension(230, 1000));
		this.setPreferredSize(new Dimension(230, 1000));
		graphComponent.addMouseWheelListener(this.graphZoomMouseListener);
		graphComponent.addMouseMotionListener(this.graphMoveMouseListener);
		graphComponent.getGraphControl().addMouseMotionListener(this.graphMoveMouseListener);
		graphComponent.getGraphControl().addMouseListener(this.graphMouseListener);
		graphComponent.setConnectable(false);
		mxcellVSCellMap = new HashMap<mxCell, MyOriginalCell>();
		cellMapVSMxcell = new HashMap<MyOriginalCell, mxCell>();
		nodeVSCoordinateMap = new HashMap<String, Pair<Integer, Integer>>();
		
		//add the head panel
		JLabel header = new JLabel("output graph");
		header.setOpaque(false);
		header.setForeground(new Color(200, 200, 200, 180));
		header.setFont(header.getFont().deriveFont(15f));
		JButton saveButton = new SlickerButton("save graph");
		saveButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				JFileChooser saveDialog = new JFileChooser();
				saveDialog.setSelectedFile(new File("ProM_LogSummary.html"));
			}
		});
		
		
		
		simplifyBox.setSelected(true);	
		simplifyBox.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {				
				if(simplifiedFlag){
					simplifiedFlag = false;
					simplifyBox.setSelected(false);
//					XFactory factory = XFactoryRegistry.instance().currentDefault();
//		     		XAttribute attribute = factory.createAttributeLiteral(xConstants.FORMAT, xConstants.TOTAL, null);
//	        		log.getAttributes().put(attribute.getKey(), attribute);
				}
				else{
					simplifiedFlag = true;
					simplifyBox.setSelected(true);	
//					XFactory factory = XFactoryRegistry.instance().currentDefault();
//		     		XAttribute attribute = factory.createAttributeLiteral(xConstants.FORMAT, xConstants.UPDATE, null);
//	        		log.getAttributes().put(attribute.getKey(), attribute);
				}	
				if(selectedEvent != null){
					updateEventGraph(selectedEvent, 1, true);
					browser.getAttributesPanel().updateEventAttributes(selectedEvent, simplifiedFlag); //, objectModelComplete
					
				}			
			}
		});
		
		simplifyLabel.setForeground(Color.WHITE);
		simplifyBox.setBackground(Color.BLACK);

		objectNumberLabel.setForeground(Color.WHITE);
		relationNumberLabel.setForeground(Color.WHITE);
		
		JPanel simplifyPanel = new JPanel();
		simplifyPanel.add(simplifyLabel);
		simplifyPanel.add(simplifyBox);
		simplifyPanel.setBackground(Color.BLACK);
//		simplifyBox.get
		//add search functionality
		JLabel searchLabel = new JLabel("search id");
		searchLabel.setOpaque(false);
		searchLabel.setForeground(Color.WHITE);
		searchLabel.setFont(header.getFont().deriveFont(15f));
		JButton searchButton = new SlickerButton("search");
		//highlight the searched cell
		searchButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				String cellId = searchText.getText();				
				mxGraphModel graphModel = (mxGraphModel) graphx.getModel();
				mxCell searchedCell =   (mxCell) graphModel.getCell(cellId);
				Map<String, XOCObject> idVSObjectMapLast = log.getLastEvent().getObjectModel().getIdVSObjectMap();		
				Map<String, XOCEvent> idVSEventMap = log.getIdVSEventMap();	
				//if the searched cell is not shown, add and show it
				if(searchedCell == null){
					System.out.println("searchedCell is null");
					String objectId =  cellId;	 
					XOCObject object = idVSObjectMapLast.get(objectId);	
//					String birthEventId = object.getBirthEventId();				
//					int birthEventIndex = idVSEventMap.get(birthEventId).getOrderNum();
//					int currentEventIndex = selectedEvent.getOrderNum();
					if(object.isAliveAtEvent(selectedEvent)){
						addObject(object, log, graphx, mxcellVSCellMap, false, zoomRate);
						System.out.println("add searchedCell");
						ObjectCell objectCell = (ObjectCell) object.getCorrespondingCell();
						
						for(String relatedObjectId : object.getRelatedObjectIdList()){
							XOCObject relatedObject = idVSObjectMapLast.get(relatedObjectId);
							addObject(relatedObject, log, graphx, mxcellVSCellMap, false, zoomRate);
							System.out.println("add related object of searchedCell");
							ObjectCell relatedObjectCell = (ObjectCell) relatedObject.getCorrespondingCell();
							
							mxCell edgeMxcell =  (mxCell) graphx.insertEdge(parent, null, null, objectCell.getMxcell(), relatedObjectCell.getMxcell());								 
							graphx.setCellStyles(mxConstants.STYLE_STROKECOLOR, "green", new Object[] {edgeMxcell});
							graphx.setCellStyles(mxConstants.STYLE_STROKEWIDTH, "1", new Object[] {edgeMxcell});
						}
					}					
				}
				graphx.orderCells(false, new Object[]{searchedCell});	
				graphx.setCellStyles(mxConstants.STYLE_STROKECOLOR, "red", new Object[]{searchedCell});
				graphx.setCellStyles(mxConstants.STYLE_STROKEWIDTH, "6", new Object[]{searchedCell});	
			}
		});
		searchText = new JTextField();
		searchText.setMaximumSize(new Dimension(100,5));
		searchText.setToolTipText("Please input the id of the node to search");
		
		JPanel headerPanel = new JPanel();
		headerPanel.setMaximumSize(new Dimension(1000,30));	
		headerPanel.setOpaque(false);
		headerPanel.setBorder(BorderFactory.createEmptyBorder());
		headerPanel.setLayout(new GridLayout(1,6));
		
		headerPanel.add(simplifyPanel);		
		headerPanel.add(Box.createHorizontalGlue());
		headerPanel.add(objectNumberLabel);
		headerPanel.add(relationNumberLabel);		
		headerPanel.add(searchButton);
		headerPanel.add(searchText);
		
		this.add(headerPanel);
		this.add(Box.createVerticalStrut(7));
		this.add(graphComponent);	
	}
	
	public boolean getSimplifiedFlag(){
		return simplifiedFlag;
	}
	
	public Map<mxCell, MyOriginalCell> getMxcellVSCellMap(){
		return mxcellVSCellMap;
	}
	
	public Map<MyOriginalCell, mxCell> getCellVSMxcellMap(){
		return cellMapVSMxcell;
	}
	
	public XOCEvent getSelectedEvent(){
		return selectedEvent;
	}
	
	public mxGraphComponent getGraphComponent(){
		return graphComponent;
	}
	
	public void update(){
		graphComponent.refresh();
	}
	
	public void setNodeVSCoordinateMap(Map<String, Pair<Integer, Integer>> nodeVSCoordinateMap){
		this.nodeVSCoordinateMap = nodeVSCoordinateMap;;
	}
	
	public Map<String, Pair<Integer, Integer>> getNodeVSCoordinateMap(){
		return nodeVSCoordinateMap;
	}
	
	public Map<String, Pair<Integer, Integer>> getInexistentObjectVSCoordinateMap(){
		return inexistentObjectVSCoordinateMap;
	}
	
	/**
	 * highlight the deviating cells based on ids
	 */
	public void highlightDeviatingCells(ConformanceCheckingResultView conformanceCheckingResult){
		List<mxCell> deviatingCellList = new ArrayList<mxCell>();
		//add deviating object relations		
		Map<XOCORelation, List<String>> deviatingObjectRelationVSContentList = conformanceCheckingResult.
				getObjectModelValidityChecker().getEventVSDeviatingObjectRelationVSContentListMap().get(selectedEvent);
		if(deviatingObjectRelationVSContentList != null){
			for(XOCORelation relation : deviatingObjectRelationVSContentList.keySet()){			
				if(deviatingObjectRelationVSContentList.get(relation).size() > 0){
					mxCell deviatingCell = cellMapVSMxcell.get(relation.getCorrespondingCell());
					deviatingCellList.add(deviatingCell);
				}			
			}
		}
		
		//add deviating event cell into the deviating cell list for highlighting
		ResultEventView eventView = new ResultEventView(conformanceCheckingResult);
		eventView.collectDeviatingInformationForEvent(selectedEvent);
		if(eventView.getDeviatingEventList().contains(selectedEvent)){				
			mxCell deviatingCell = cellMapVSMxcell.get(selectedEvent.getCorrespondingCell());
			if(!deviatingCellList.contains(deviatingCell)){				
				deviatingCellList.add(deviatingCell);
			}
		}
		
		//add deviating object cells into the deviating cell list for highlighting 
		for(MyOriginalCell elementCell : cellMapVSMxcell.keySet()){
			MyOriginalElement element = elementCell.getCorrespondingElement();
			if(element instanceof XOCObject){
				System.out.println("element instanceof XOCObject");
				XOCObject object = (XOCObject) element;
				ResultObjectVersionView objectView = new ResultObjectVersionView(conformanceCheckingResult);
				objectView.collectDeviatingInformationForObjectVersion(object, selectedEvent, log);
				if(objectView.getDeviatingFlag()){
					System.out.println("objectView.getDeviatingFlag(): " + objectView.getDeviatingFlag());
					deviatingCellList.add(cellMapVSMxcell.get(elementCell));
				}
			}
		}
		
		//add deviating reference relation cells into the deviating cell list for highlighting 
		List<XOCObject> unrelatedObjectList = conformanceCheckingResult
				.getProperClassesChecker().getEventVSObjectListOfUnrelatedClassesMap().get(selectedEvent);
		for(XOCRRelation element : selectedEvent.getReferenceRelationList()){
			MyOriginalCell elementCell = element.getCorrespondingCell();		
			XOCObject targetObject = element.getTargetObject();
			if(unrelatedObjectList != null && unrelatedObjectList.contains(targetObject)){			
				deviatingCellList.add(cellMapVSMxcell.get(elementCell));
			}
			
//			if(element instanceof XOCObject){
//				System.out.println("element instanceof XOCObject");
//				XOCObject object = (XOCObject) element;
//				ResultObjectVersionView objectView = new ResultObjectVersionView(conformanceCheckingResult);
//				objectView.collectDeviatingInformationForObjectVersion(object, selectedEvent, log);
//				if(objectView.getDeviatingFlag()){
//					System.out.println("objectView.getDeviatingFlag(): " + objectView.getDeviatingFlag());
//					deviatingCellList.add(cellMapVSMxcell.get(elementCell));
//				}
//			}
		}
				
		//add deviating object cell into the deviating cell list for highlighting 
		//inexistent objects (referred by events but inexistent in the object model)
//		Map<XOCEvent, List<XOCObject>> eventVSInexistentObjectList = conformanceCheckingResult.
//				getObjectExistenceChecker().getEventVSInexistentObjectListMap();
//		if(eventVSInexistentObjectList.keySet().contains(selectedEvent)){
//			for(XOCObject object : eventVSInexistentObjectList.get(selectedEvent)){
//				mxCell deviatingCell = cellMapVSMxcell.get(object.getCorrespondingCell());
//				deviatingCellList.add(deviatingCell);
//			}
//		}
		
//		graphComponent.getGraph().orderCells(false, deviatingCellList.toArray());				
		graphComponent.getGraph().setCellStyles(mxConstants.STYLE_STROKECOLOR, "red", deviatingCellList.toArray());
		graphComponent.getGraph().setCellStyles(mxConstants.STYLE_STROKEWIDTH, "6", deviatingCellList.toArray());	
	
	}
	
	public void updateEventGraph(XOCEvent event, double zoomRate, boolean showEventFlag){				
		this.graphx = new mxGraph();
	    this.parent = graphx.getDefaultParent();  
	    this.graphComponent.setGraph(graphx);
	    this.zoomRate = zoomRate;
	    shownObjectIdList.clear();	    
	    mxcellVSCellMap.clear();
	    
		long uint = 1;	
		long startTime = System.currentTimeMillis(); 
		long durationTime = System.currentTimeMillis();
		RunningTime.writeTimeInformationIntoFile("******************updateEventGraph*********************", false, "D:\\timeVisualizingXOCLog.txt");
		
		selectedEvent = event;//record the selected event	
		graphMouseListener.setSelectedEvent(event);


		graphx.getModel().beginUpdate();
		 

		XOCObjectModel objectModelShown = new XOCObjectModelImpl();
		XOCObjectModel objectModelOld = new XOCObjectModelImpl();
//		Map<String, XOCObject> idVSObjectMapLast = log.getLastEvent().getObjectModel().getIdVSObjectMap();	
		Map<String, XOCObject> idVSObjectMapLog = log.getIdVSObjectMap();	
		Map<String, XOCEvent> idVSEventMap = log.getIdVSEventMap();	
		
		 
        durationTime = (System.currentTimeMillis() - startTime)/uint;
		startTime = System.currentTimeMillis();
		RunningTime.writeTimeInformationIntoFile("complete the preparation end: " + System.currentTimeMillis()/uint, false, "D:\\timeVisualizingXOCLog.txt");
		RunningTime.writeTimeInformationIntoFile("~~~~~~~~~~~~~~complete the preparation duration time: " + durationTime, false, "D:\\timeVisualizingXOCLog.txt");
			
			
		if(simplifiedFlag){
			System.out.println("\n choose simple mode");	
			//objectModelShown = event.getObjectModelUpdate();
			objectModelShown = log.createObjectModelUpdate(event);		
			Map<String, XOCObject> idVSObjectMap = objectModelShown.generateIdVSObjectMap();
			int objectNum = objectModelShown.getObjectSet().size();
			int objectRelationNum = objectModelShown.getObjectRelationSet().size();
			objectNumberLabel.setText("object: " + objectNum);
			relationNumberLabel.setText("relation: " + objectRelationNum);
			
			//add the existent objects involved in the object relation into the old object model
			for(XOCORelation objectRelation : objectModelShown.getObjectRelationSet()){
				System.out.println("get objectRelationId");
				String sourceObjectId = objectRelation.getSourceObject().getAttributes().get(xConstants.ID).toString();
				String targetObjectId = objectRelation.getTargetObject().getAttributes().get(xConstants.ID).toString();
				if(!idVSObjectMap.keySet().contains(sourceObjectId)){
					XOCObject object = idVSObjectMapLog.get(sourceObjectId);
					objectModelOld.getObjectSet().add(object);
				}
				if(!idVSObjectMap.keySet().contains(targetObjectId)){
					XOCObject object = idVSObjectMapLog.get(targetObjectId);
					objectModelOld.getObjectSet().add(object);
				}
			}	
			
			//add the existent objects involved in the object relation into the old object model
			for(XOCObject relatedObject : event.getRelatedObjects()){
				System.out.println("get relatedObject");
				String objectId = relatedObject.getAttributes().get(xConstants.ID).toString();	
				XOCObject object = idVSObjectMapLog.get(objectId);
				if(!idVSObjectMap.keySet().contains(objectId) &&  log.createCompleteObjectModel(event).getObjectSet().contains(object)){				
					objectModelOld.getObjectSet().add(object);
				}			
			}	
		}
		else{
			
			System.out.println("\n choose normal mode");
			objectModelShown = log.createCompleteObjectModel(event);
			objectModelComplete = objectModelShown;
			int objectNum = objectModelShown.getObjectSet().size();
			int objectRelationNum = objectModelShown.getObjectRelationSet().size();
			objectNumberLabel.setText("object: " + objectNum);
			relationNumberLabel.setText("relation: " + objectRelationNum);

		}
		 
        durationTime = (System.currentTimeMillis() - startTime)/uint;
		startTime = System.currentTimeMillis();
		RunningTime.writeTimeInformationIntoFile("complete the createCompleteObjectModel end: " + System.currentTimeMillis()/uint, false, "D:\\timeVisualizingXOCLog.txt");
		RunningTime.writeTimeInformationIntoFile("~~~~~~~~~~~~~~complete the createCompleteObjectModel duration time: " + durationTime, false, "D:\\timeVisualizingXOCLog.txt");
			
			
	    //create the object cells (whose number is lower that a threshold)
		log.generateClassNameList();//generate the class list which are used in addObject function
		Map<String, Long> classVSObjectNum = new HashMap<String, Long>();
		long currentNum = 0;
		for(XOCObject object : objectModelShown.getObjectSet()){
			 String objectId =  object.getAttributes().get(xConstants.ID).toString();	
			 System.out.println("objectId in objectModelShown: " + objectId);
			 
			 String objectClass = null;
			 XOCAttribute classXOCAttribute = object.getXOCAttributes().get(xConstants.CLASS);
			 if(classXOCAttribute != null){
				 objectClass =  classXOCAttribute.getValueVersion(event.getOrderNum());
			 }
			 if(objectClass == null){
				 objectClass = "null";
			 }			
		 		 					 
			 if(!classVSObjectNum.containsKey(objectClass)){
				 classVSObjectNum.put(objectClass, 1L);
				 currentNum = 1;
			 }
			 else{
				 currentNum = classVSObjectNum.get(objectClass) + 1;
				 classVSObjectNum.put(objectClass, currentNum);
			 }
			 
			 //limit the number of shown objects of each class and the reference objects must be shown
			 if(currentNum <= objectShownMaxNum || event.getRelatedObjects().contains(object)){
				 System.out.println("add objectId in objectModelShown: " + objectId);
				 addObject(object, log, graphx, mxcellVSCellMap, false, zoomRate);
				 shownObjectIdList.add(objectId);
			 }	
		}

        durationTime = (System.currentTimeMillis() - startTime)/uint;
		startTime = System.currentTimeMillis();
		RunningTime.writeTimeInformationIntoFile("complete the objectModelShown end: " + System.currentTimeMillis()/uint, false, "D:\\timeVisualizingXOCLog.txt");
		RunningTime.writeTimeInformationIntoFile("~~~~~~~~~~~~~~complete the objectModelShown duration time: " + durationTime, false, "D:\\timeVisualizingXOCLog.txt");		
			

		for(XOCObject object : objectModelOld.getObjectSet()){
			 String objectId =  object.getAttributes().get(xConstants.ID).toString();			
			 System.out.println("objectId in objectModelOld: " + objectId);
			 System.out.println("add objectId in objectModelOld: " + objectId);
			 addObject(object, log, graphx, mxcellVSCellMap, true, zoomRate);
			 shownObjectIdList.add(objectId);
		}
		 
        durationTime = (System.currentTimeMillis() - startTime)/uint;
		startTime = System.currentTimeMillis();
		RunningTime.writeTimeInformationIntoFile("complete the objectModelOld end: " + System.currentTimeMillis()/uint, false, "D:\\timeVisualizingXOCLog.txt");
		RunningTime.writeTimeInformationIntoFile("~~~~~~~~~~~~~~complete the objectModelOld duration time: " + durationTime, false, "D:\\timeVisualizingXOCLog.txt");
		
			
		addObjectRelations(objectModelShown, log, graphx, mxcellVSCellMap);
		 
        durationTime = (System.currentTimeMillis() - startTime)/uint;
		startTime = System.currentTimeMillis();
		RunningTime.writeTimeInformationIntoFile("complete the addObjectRelations end: " + System.currentTimeMillis()/uint, false, "D:\\timeVisualizingXOCLog.txt");
		RunningTime.writeTimeInformationIntoFile("++++++++++++++complete the addObjectRelations duration time: " + durationTime, false, "D:\\timeVisualizingXOCLog.txt");
		
			
		//create the event cell + reference relation (objects)
		if(showEventFlag){			
			 addEvent(event, log, graphx, mxcellVSCellMap, zoomRate);
			 event.getReferenceRelationList().clear();
			 addReferenceRelations(event, log, graphx, mxcellVSCellMap);
		}
		 
        durationTime = (System.currentTimeMillis() - startTime)/uint;
		startTime = System.currentTimeMillis();
		RunningTime.writeTimeInformationIntoFile("complete the addReferenceRelations end: " + System.currentTimeMillis()/uint, false, "D:\\timeVisualizingXOCLog.txt");
		RunningTime.writeTimeInformationIntoFile("~~~~~~~~~~~~~~complete the addReferenceRelations duration time: " + durationTime, false, "D:\\timeVisualizingXOCLog.txt");
		
			
		graphx.getModel().endUpdate();
		graphx.setAllowDanglingEdges(false);
		graphComponent.updateUI();//necessary; otherwise, the previous graph stays
		this.add(graphComponent);	
		 
        durationTime = (System.currentTimeMillis() - startTime)/uint;
		startTime = System.currentTimeMillis();
		RunningTime.writeTimeInformationIntoFile("complete the update end: " + System.currentTimeMillis()/uint, false, "D:\\timeVisualizingXOCLog.txt");
		RunningTime.writeTimeInformationIntoFile("~~~~~~~~~~~~~~complete the update duration time: " + durationTime, false, "D:\\timeVisualizingXOCLog.txt");		
	
		//if the conformance checking result is not null, highlight the deviating cells
		ConformanceCheckingResultView conformanceCheckingResult = browser.
				getParentLogViewer().getConformanceCheckingResult();
		if(conformanceCheckingResult != null){		
			System.out.println("highlight deviating cells in updateEventGraph in log.inspector.panels;");
			highlightDeviatingCells(conformanceCheckingResult);
		}
	}
	
	//add an object cell of object from OM into the graphx, and record (eventMxcell, eventCell) into the mxcellVSCellMap
	public void addObject(XOCObject object, XOCLog log, mxGraph graphx, Map<mxCell, MyOriginalCell> mxcellVSCellMap, boolean idShaded, double zoomRate){
		 String objectId =  object.getAttributes().get(xConstants.ID).toString();
		  
		 //specify the location for the object
		 double positionX;;
		 double positionY;
		 if(nodeVSCoordinateMap.containsKey(object.getAttributes().get(xConstants.ID).toString())){
			 positionX = nodeVSCoordinateMap.get(objectId).getFirst()*zoomRate;
			 positionY = nodeVSCoordinateMap.get(objectId).getSecond()*zoomRate;
		 }
		 else{
			 System.out.print("wrong! id" + object.getAttributes().get("id").toString());
			 positionX = 500*zoomRate;
			 positionY = 500*zoomRate;
		 }	
		 
		 //specify the color for the object
		 int colorValue;
		 if(object.getAttributes().get(xConstants.CLASS) != null){
			 String classType = object.getAttributes().get(xConstants.CLASS).toString();
			 double classCount = log.getClassNameList().size()+1;			
			 double classIndex = log.getClassNameList().indexOf(classType);

			 colorValue =  (int)(classIndex/classCount)*0xff + ((int)((classIndex/classCount)*0xff))*0x100 
					 + ((int)((1-classIndex/classCount)*0xff))*0x10000;
		 }
		 else{
			 colorValue = 0xff;
		 }
		 
		 mxCell objectMxcell = (mxCell) graphx.insertVertex(parent, objectId, objectId, positionX, positionY, 80,30, 
	    		 "fillColor=#" + colorValue + ";gradientColor=#ffffff");	
		 graphx.setCellStyles(mxConstants.STYLE_STROKECOLOR, "#" + colorValue, new Object[] {objectMxcell});
		 graphx.setCellStyles(mxConstants.STYLE_STROKEWIDTH, "1", new Object[] {objectMxcell});
		 if(idShaded){
			 graphx.setCellStyles(mxConstants.STYLE_STROKECOLOR, "null", new Object[] {objectMxcell});
			 graphx.setCellStyles(mxConstants.STYLE_STROKEWIDTH, "0", new Object[] {objectMxcell});
			 graphx.setCellStyles(mxConstants.STYLE_OPACITY, "20", new Object[] {objectMxcell});
		 }
	     ObjectCell objectCell = new ObjectCell(objectMxcell);
	     objectCell.setCorrespondingElement(object);
	     object.setCorrespondingCell(objectCell);
	     mxcellVSCellMap.put(objectMxcell, objectCell);	
	     cellMapVSMxcell.put(objectCell, objectMxcell);
	}
	
	public void addObjectRelationMxCell(mxGraph graphx, Object sourceMxCell, Object targetMxCell, Map<mxCell, MyOriginalCell> mxcellVSCellMap){  		
		 mxCell edgeMxcell =  (mxCell) graphx.insertEdge(parent, null, null, sourceMxCell, targetMxCell);								 
		 graphx.setCellStyles(mxConstants.STYLE_STROKECOLOR, "green", new Object[] {edgeMxcell});
		 graphx.setCellStyles(mxConstants.STYLE_STROKEWIDTH, "1", new Object[] {edgeMxcell});
		 ObjectEdgeCell objectEdgeCell = new ObjectEdgeCell(edgeMxcell);
	     mxcellVSCellMap.put(edgeMxcell, objectEdgeCell);	
	     cellMapVSMxcell.put(objectEdgeCell, edgeMxcell);
	}
	
	//should be used after add objects
	public void addObjectRelations(XOCObjectModel objectModel, XOCLog log, mxGraph graphx, Map<mxCell, MyOriginalCell> mxcellVSCellMap){    
		 //create the object relation cells
		 System.out.println("enter addObjectRelations");
		 for(XOCORelation relation : objectModel.getObjectRelationSet()){
			 XOCObject sourceObject = relation.getSourceObject();
			 XOCObject targetObject = relation.getTargetObject();
			 String sourceObjectId = sourceObject.getAttributes().get(xConstants.ID).toString();
			 String targetObjectId = targetObject.getAttributes().get(xConstants.ID).toString();
			 ObjectCell sourceCell = (ObjectCell) sourceObject.getCorrespondingCell();
			 ObjectCell targetCell = (ObjectCell) targetObject.getCorrespondingCell();	
			 if(sourceCell != null && targetCell != null && sourceCell.getMxcell() != null && targetCell.getMxcell() != null){	
//				 mxCell edgeMxcell = new mxCell("", new mxGeometry(0, 0, 10, 10), null);					   
//				 edgeMxcell.setSource(sourceCell.getMxcell());
//				 edgeMxcell.setTarget(targetCell.getMxcell());
//				 edgeMxcell.setEdge(true);
//				 graphx.getModel().add(parent, edgeMxcell, 0);
				 
				 if(shownObjectIdList.contains(sourceObjectId) && shownObjectIdList.contains(targetObjectId)){
					 mxCell edgeMxcell =  (mxCell) graphx.insertEdge(parent, null, null, sourceCell.getMxcell(), targetCell.getMxcell());								 
					 graphx.setCellStyles(mxConstants.STYLE_STROKECOLOR, "green", new Object[] {edgeMxcell});
					 graphx.setCellStyles(mxConstants.STYLE_STROKEWIDTH, "1", new Object[] {edgeMxcell});
					 ObjectEdgeCell objectEdgeCell = new ObjectEdgeCell(edgeMxcell);
					 objectEdgeCell.setCorrespondingElement(relation);
					 relation.setCorrespondingCell(objectEdgeCell);
				     mxcellVSCellMap.put(edgeMxcell, objectEdgeCell);	
				     cellMapVSMxcell.put(objectEdgeCell, edgeMxcell);
				 }
			 }	 
		 }
	}

	//add an event cell into the graphx, and record (eventMxcell, eventCell) into the mxcellVSCellMap
	public void addEvent(XOCEvent event, XOCLog log, mxGraph graphx, Map<mxCell, MyOriginalCell> mxcellVSCellMap, double zoomRate){
		 String eventName = event.getAttributes().get("id").toString();	
		 String type = event.getAttributes().get("activity").toString();	
		 
		 double positionX = nodeVSCoordinateMap.get("event").getFirst()*zoomRate;
		 double positionY = nodeVSCoordinateMap.get("event").getSecond()*zoomRate;
		 
		 
		 List<String> activityList = log.createActivityNames();
		 double typeNum = activityList.size();			 
		 double typeIndex = activityList.indexOf(type);	//there exist risk		 
		 int colorValue =  (int)(1-typeIndex/typeNum)*0xff + ((int)((1-typeIndex/typeNum)*0xff))*0x100 
				 + ((int)((typeIndex/typeNum)*0xff))*0x10000;
        	   	 
	     mxCell eventMxcell = (mxCell) graphx.insertVertex(parent, eventName, eventName, positionX, positionY, 80, 30, 
	    		 "shape=ellipse;fillColor=#" + colorValue + ";gradientColor=#ffffff;shadow=1");
	     EventCell eventCell = new EventCell(eventMxcell);
	     eventCell.setCorrespondingElement(event);
	     event.setCorrespondingCell(eventCell);
	     mxcellVSCellMap.put(eventMxcell, eventCell);
	     cellMapVSMxcell.put(eventCell, eventMxcell);
	}
	
	//should be used after add object model
	public void addReferenceRelations(XOCEvent event, XOCLog log, mxGraph graphx, Map<mxCell, MyOriginalCell> mxcellVSCellMap){	
		 String eventId = event.getAttributes().get(xConstants.ID).toString();
		 String activity = event.getAttributes().get(xConstants.ACTIVITY).toString();
		 System.out.println("addReferenceRelations in ObjectModelJGraphXPanel");
		 System.out.println("eventId: " + eventId);
		 for(XOCObject object : event.getRelatedObjects()){
			 
			 String objectId = object.getAttributes().get(xConstants.ID).toString();
			 System.out.println("objectId: " + objectId);
			 String objectClass;
			 if(object.getAttributes().get(xConstants.CLASS) != null){
				 objectClass = object.getAttributes().get(xConstants.CLASS).toString();
			 }
			 else{
				 objectClass = "null";
			 }
			 String referenceRelationId = eventId + "-" + objectId;
			 XOCRRelation referenceRelation = new XOCRRelationImpl(activity + "-" + objectClass, event, object);      		       		
			 XAttribute attribute = XFactoryRegistry.instance().currentDefault().
					 createAttributeLiteral(xConstants.ID, referenceRelationId, null);
			 referenceRelation.getAttributes().put(attribute.getKey(), attribute);
			 //add the reference relation into the corresponding list of the event
			 event.getReferenceRelationList().add(referenceRelation);
			 System.out.println("\n referenceRelation id in addReferenceRelations: " + referenceRelation.getAttributes().get("id").toString());
			 System.out.println("\n referenceRelation.getTargetObject() id in addReferenceRelations: " + referenceRelation.getTargetObject().getAttributes().get("id").toString());
			 
			 EventCell eventCell = (EventCell) event.getCorrespondingCell();
			 ObjectCell objectCell = (ObjectCell) object.getCorrespondingCell();
			 
			 mxCell edgeMxcell = null;
			 if(log.createCompleteObjectModel(event).getObjectSet().contains(object)){
//			 if(object.isAliveAtEvent(event) || object.isAliveAtEventOrderNum(event.getOrderNum()-1)){
				 System.out.println("log.createCompleteObjectModel(event).getObjectSet().contains(object)");
//				 if(eventCell.getMxcell() != null && objectCell.getMxcell() != null){
//					 System.out.println("eventCell.getMxcell() != null  && objectCell.getMxcell() != null");
//				 }
				
				 edgeMxcell = (mxCell) graphx.insertEdge(parent, referenceRelationId, null, 
						 eventCell.getMxcell(), objectCell.getMxcell(), "dashed=1");
//				 if(edgeMxcell != null){
//					 System.out.println("sucessfully eventCell.getMxcell(): " + eventCell.getMxcell().getId());
//					 System.out.println("sucessfully objectCell.getMxcell(): " + objectCell.getMxcell().getId());
//					 System.out.println("sucessfully insert edge: " + edgeMxcell.getId());
//				 }
			 }

			 //if the event refers to an existent object in the object model,
			 //create the object cell and put it on the top
			 else{
				 double positionX = inexistentObjectVSCoordinateMap.get(objectId).getFirst()*zoomRate;
				 double positionY = inexistentObjectVSCoordinateMap.get(objectId).getSecond()*zoomRate;
				 mxCell objectMxCellNew = (mxCell) graphx.insertVertex(parent, objectId, objectId, positionX, positionY, 80,30, 
			    		 "fillColor=white" + ";gradientColor=#ffffff");	
				 graphx.setCellStyles(mxConstants.STYLE_STROKECOLOR, "null", new Object[] {objectMxCellNew});
				 graphx.setCellStyles(mxConstants.STYLE_STROKEWIDTH, "0", new Object[] {objectMxCellNew});
//				 graphx.setCellStyles(mxConstants.STYLE_OPACITY, "15", new Object[] {objectMxCellNew});
				 
				 edgeMxcell = (mxCell) graphx.insertEdge(parent, referenceRelationId, null, 
						 eventCell.getMxcell(), objectMxCellNew, "dashed=1");	
				 
				 
				 InexistentObjectCell objectCellNew = new InexistentObjectCell(objectMxCellNew);
				 objectCellNew.setCorrespondingElement(object);
//				 if(object.getCorrespondingCell() == null){
//					 object.setCorrespondingCell(objectCellNew);
//				 }			    
			     mxcellVSCellMap.put(objectMxCellNew, objectCellNew);	
			     cellMapVSMxcell.put(objectCellNew, objectMxCellNew);
			     
				 System.out.println("objectCell.getMxcell() == null in addReferenceRelations in ObjectModelJGraphXPanel, i.e., event refers to an inexistent object");
			 }
			 
			 ReferenceEdgeCell referenceEdgeCell = new ReferenceEdgeCell(edgeMxcell);
			 referenceEdgeCell.setCorrespondingElement(referenceRelation);
			 referenceRelation.setCorrespondingCell(referenceEdgeCell);
			 mxcellVSCellMap.put(edgeMxcell, referenceEdgeCell);
		     cellMapVSMxcell.put(referenceEdgeCell, edgeMxcell);
		 }
	}
	
	//create coordinates for each object 
	public Map<String, Pair<Integer, Integer>> generateObjectCoordinates(XOCEvent event, int height, int width, int offsetY){	
		 int positionX = 0;
		 int positionY = 0; 
		 int classNum = 0;
		 int horizontalUintNum = 0;
		 int verticalUintNum = 0;
		 int unitHeight = 0;
		 int unitWidth = 0;
		 int eventNodeHeight = 50;
		 
		 
		 Map<String, Pair<Integer, Integer>> nodeVSCoordinateMap = new HashMap<String, Pair<Integer, Integer>>();
		 nodeVSCoordinateMap.put("event", new Pair<Integer, Integer>(width/2, eventNodeHeight));
		 classNum = log.generateClassNameList().size();
		 
		 System.out.println("classNum: " + classNum);
		 height = height - 2*eventNodeHeight;	 
		 horizontalUintNum = (int) Math.sqrt(classNum-1) + 1;
		 if(classNum <= horizontalUintNum*(horizontalUintNum-1)) verticalUintNum = horizontalUintNum-1;
		 else verticalUintNum = horizontalUintNum;
		 unitHeight = height / verticalUintNum;
		 unitWidth = width / horizontalUintNum;
		 	 				 			 		 
		 for(XOCObject object : event.getObjectModel().getObjectSet()){
			 String classType = object.getAttributes().get(xConstants.CLASS).toString();
			 int classIndex = log.getClassNameList().indexOf(classType);	
			 positionX = (int) (Math.random()*unitWidth*0.7 + (classIndex%horizontalUintNum)*unitWidth);
			 positionY = 2*eventNodeHeight + (int) (Math.random()*unitHeight*0.7 + (classIndex/horizontalUintNum)*unitHeight);
			 Pair<Integer, Integer> coordinate = new Pair<Integer, Integer>(positionX, positionY + offsetY);
			 nodeVSCoordinateMap.put(object.getAttributes().get("id").toString(), coordinate);	
//			 System.out.print("\n id" + object.getAttributes().get("id").toString());
//			 System.out.print("\n value" + coordinate.getFirst());
		 }	 
		 return nodeVSCoordinateMap;
	}		
	
	//create coordinates for each object in the map of log
	public void generateObjectCoordinates(XOCLog log, int height, int width, int offsetY){	
		 int positionX = 0;
		 int positionY = 0; 
		 int classNum = 0;
		 int horizontalUintNum = 0;
		 int verticalUintNum = 0;
		 int unitHeight = 0;
		 int unitWidth = 0;
		 int eventNodeHeight = 50;
		 
		 System.out.println("generateObjectCoordinates for objects in log");
		 
		 nodeVSCoordinateMap = new HashMap<String, Pair<Integer, Integer>>();
		 inexistentObjectVSCoordinateMap =  new HashMap<String, Pair<Integer, Integer>>();
		 
		 //generate event coordinate
		 nodeVSCoordinateMap.put("event", new Pair<Integer, Integer>(width/2, eventNodeHeight));
		 
		//generate object coordinates
		 classNum = log.generateClassNameList().size();	 
		 System.out.println("classNum: " + classNum);
		 height = height - 2*eventNodeHeight;	 
		 horizontalUintNum = (int) Math.sqrt(classNum-1) + 1;
		 if(classNum <= horizontalUintNum*(horizontalUintNum-1)) verticalUintNum = horizontalUintNum-1;
		 else verticalUintNum = horizontalUintNum;
		 unitHeight = height / verticalUintNum;
		 unitWidth = width / horizontalUintNum;		 	 				 		
		 Map<String, XOCObject> idVSObject = log.getIdVSObjectMap();
		 for(String id : idVSObject.keySet()){
			 XOCObject object = idVSObject.get(id);
			 List<IntegerPair> objectLifecyclePeriodList = object.getLifecyclePeriodList();
			 XOCEvent referenceEvent = object.getReferenceEvent();
			 //if the object is inexistent for the whole log, it locate on the top left
			 if(objectLifecyclePeriodList.size() == 0){
				 System.out.println("inexistent object: " + id);
				 Pair<Integer, Integer> coordinate = new Pair<Integer, Integer>((int)((width/2)*Math.random()), (int)(eventNodeHeight*Math.random()));
				 inexistentObjectVSCoordinateMap.put(id, coordinate);	
			 }
			 //if the object exists, but after the first reference event, it locate on the top right
			 else if(referenceEvent != null && !object.isAliveAtEvent(referenceEvent)){
				 System.out.println("temprary inexistent object: " + id);
				 Pair<Integer, Integer> coordinate = new Pair<Integer, Integer>((int)(width/2 + (width/2)*Math.random()), (int)(eventNodeHeight*Math.random()));
				 inexistentObjectVSCoordinateMap.put(id, coordinate);	
			 }
			 //if the object exists, but removed some time later
			 else{
				 IntegerPair lifePeriod = object.getLifecyclePeriodList().get(0);
				 int endEventOrderNum = lifePeriod.getSecondValue() != null ? lifePeriod.getSecondValue() : log.size();
					if(endEventOrderNum < log.size()){					
						System.out.println("removed inexistent object: " + id);
						Pair<Integer, Integer> coordinate = new Pair<Integer, Integer>((int)(width/2 + (width/2)*Math.random()), (int)(eventNodeHeight*Math.random()));
						inexistentObjectVSCoordinateMap.put(id, coordinate);	
					}
			 }

			 //if the object exists, it locate in the object model and categorized by classes
			 if(object.getAttributes().get(xConstants.CLASS) != null){
				 System.out.println("existent object: " + id);
				 String classType = object.getAttributes().get(xConstants.CLASS).toString();
				 int classIndex = log.getClassNameList().indexOf(classType);	
				 positionX = (int) (Math.random()*unitWidth*0.7 + (classIndex%horizontalUintNum)*unitWidth);
				 positionY = 2*eventNodeHeight + (int) (Math.random()*unitHeight*0.7 + (classIndex/horizontalUintNum)*unitHeight);
				 Pair<Integer, Integer> coordinate = new Pair<Integer, Integer>(positionX, positionY + offsetY);;
				 nodeVSCoordinateMap.put(object.getAttributes().get("id").toString(), coordinate);	
			 }
		 }
	}	
}