package org.processmining.ocbc.log.inspector.panels;

import java.awt.Color;
import java.awt.Dimension;

import javax.swing.BoxLayout;

import org.processmining.framework.util.Pair;
import org.processmining.mxgraph.model.mxCell;
import org.processmining.ocbc.log.inspector.graph.EventCell;
import org.processmining.ocbc.log.inspector.graph.ObjectCell;
import org.processmining.ocbc.log.inspector.graph.ReferenceEdgeCell;
import org.processmining.ocbc.log.inspector.interaction.GraphMouseListener;
import org.processmining.ocbc.log.inspector.interaction.GraphMoveMouseListener;
import org.processmining.ocbc.log.inspector.interaction.GraphZoomMouseListener;
import org.processmining.ocbc.log.model.XOCEvent;
import org.processmining.ocbc.log.model.XOCLog;
import org.processmining.ocbc.log.model.XOCObject;
import org.processmining.ocbc.log.model.XOCRRelation;

public class ReferenceJGraphXPanel extends ObjectModelJGraphXPanel {

	public static final Color COLOR_ENCLOSURE_BG = new Color(40, 40, 40);
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private XOCEvent lastEvent;//the latest event in the log; specified when the class is instanced
	
	public ReferenceJGraphXPanel(int aRadius, int anOuterBorder, int anInnerBorder, XOCLog log, GraphMouseListener graphMouseListener,
			GraphZoomMouseListener graphZoomMouseListener, GraphMoveMouseListener graphMoveMouseListener){
		super(null, aRadius, anOuterBorder, anInnerBorder, log, graphMouseListener, graphZoomMouseListener, graphMoveMouseListener);
		this.setBackground(COLOR_ENCLOSURE_BG);
		this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		this.setMaximumSize(new Dimension(230, 1000));
		this.setPreferredSize(new Dimension(230, 1000));
		this.lastEvent = log.getAllEvents().get(log.getAllEvents().size()-1);
	}	
	
	public void update(){
		super.update();
	}	
	
	public void generateReferenceGraph(XOCLog log, boolean referenceRelationSwitch, boolean objectRelationSwitch){
		 this.removeAll();		 
		 super.setNodeVSCoordinateMap(super.generateObjectCoordinates(lastEvent, 1000, 1000, 500));
		 super.updateEventGraph(lastEvent, 1, false);
		 //create  event coordinates
		 generateEventCoordinates(log, 500, 1000);
		 

		 double positionX = 0;
		 double positionY = 0;

	     for(XOCEvent event : log.getAllEvents()){
	    	 String eventId = event.getAttributes().get("id").toString();	
			 if(nodeVSCoordinateMap.containsKey(eventId)){
				 System.out.print("\n contain id");
				 positionX = nodeVSCoordinateMap.get(eventId).getFirst();
				 positionY = nodeVSCoordinateMap.get(eventId).getSecond();
			 }
			 else{
				 System.out.print("wrong event id!" + eventId);
				 positionX = 500;
				 positionY = 500;
			 }					 	
			 System.out.print("\n enter draw event");
			 super.addEvent(event, log, super.graphx, super.mxcellVSCellMap, 1);	     
	     }

		 //add reference relations(event-object)
		 if(referenceRelationSwitch){
			 //create the reference relation cells
			 for(XOCRRelation referenceRelation : log.getReferenceRelations()){
				 EventCell soruceEventCell = (EventCell) referenceRelation.getSourceEvent().getCorrespondingCell();
				 XOCObject targetObject = referenceRelation.getTargetObject();
				 String referenceRelationId = referenceRelation.getID().toString();
				 if(true){
					 XOCObject existentObject = lastEvent.getObjectModel().getObjectById(targetObject.getAttributes().get("id").toString());
					 mxCell edgeMxcell = null;
//					 String objectId =  object.getAttributes().get("id").toString();
//					 String classType = object.getClassType().getClassName();	
//					 double classCount = log.getClassNames().size()+1;			
//					 double classIndex = log.getClassNames().indexOf(classType);
//
//					 int colorValue =  (int)(classIndex/classCount)*0xff + ((int)((classIndex/classCount)*0xff))*0x100 
//							 + ((int)((1-classIndex/classCount)*0xff))*0x10000;
//					 
//					 mxCell objectMxcell = (mxCell) graphx.insertVertex(parent, objectId, objectId, positionX, positionY, 80,30, 
//				    		 "fillColor=#" + colorValue + ";gradientColor=#ffffff");
					 if(existentObject != null && soruceEventCell != null){
						 ObjectCell targetObjectCell = (ObjectCell) existentObject.getCorrespondingCell();
						 if(targetObjectCell != null)
							 edgeMxcell = (mxCell) graphx.insertEdge(parent, referenceRelationId, null, 
									 soruceEventCell.getMxcell(),
									 targetObjectCell.getMxcell(), "dashed=1;fillColor=#ffff00");
					 }
					 //if the event refers to an existent object in the object model
					 else{
						 
					 }
					 ReferenceEdgeCell referenceEdgeCell = new ReferenceEdgeCell(edgeMxcell);
					 referenceEdgeCell.setCorrespondingElement(referenceRelation);
					 referenceRelation.setCorrespondingCell(referenceEdgeCell);
					 mxcellVSCellMap.put(edgeMxcell, referenceEdgeCell);
					 cellMapVSMxcell.put(referenceEdgeCell, edgeMxcell);
				 }
			 }
		 }

		 graphx.setAllowDanglingEdges(false);
		 this.add(graphComponent);	 		
	}
	
	public void generateEventCoordinates(XOCLog log, int height, int width){
		 int positionX = 0;
		 int positionY = 0; 
		 int rowNum = 0;
		 int columnNum = 0;	 
		 int unitHeight = 0;
		 int unitWidth = 0;
		 int eventNum = log.getAllEvents().size();

		 columnNum = (int) Math.sqrt(eventNum-1) + 1;		 	 
		 if(eventNum <= columnNum*(columnNum - 1)) rowNum = columnNum - 1;
		 else rowNum = columnNum;		 
		 unitHeight = height / rowNum;
		 unitWidth = width / columnNum;
		 
		 for(int i=0; i<eventNum; i++){		 
			 String eventID = log.getAllEvents().get(i).getAttributes().get("id").toString();		 
			 positionX = (int) (unitWidth*0.5 + i%columnNum*unitWidth);
			 positionY = (int) (unitHeight*0.5 + i/columnNum*unitHeight);
			 Pair<Integer, Integer> coordinate = new Pair<Integer, Integer>(positionX, positionY);
			 nodeVSCoordinateMap.put(eventID, coordinate);	
		 }		 	 	
	}		
}