/*
 * Copyright (c) 2007 Christian W. Guenther (christian@deckfour.org)
 * 
 * LICENSE:
 * 
 * This code is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 * 
 * EXEMPTION:
 * 
 * License to link and use is also granted to open source programs which are not
 * licensed under the terms of the GPL, given that they satisfy one or more of
 * the following conditions: 1) Explicit license is granted to the ProM and
 * ProMimport programs for usage, linking, and derivative work. 2) Carte blance
 * license is granted to all programs developed at Eindhoven Technical
 * University, The Netherlands, or under the umbrella of STW Technology
 * Foundation, The Netherlands. For further exemptions not covered by the above
 * conditions, please contact the author of this code.
 */
package org.processmining.ocbc.log.inspector.panels;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.text.SimpleDateFormat;
import java.util.Arrays;
import java.util.Collection;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;

import org.deckfour.xes.classification.XEventClass;
import org.deckfour.xes.classification.XEventClasses;
import org.deckfour.xes.classification.XEventClassifier;
import org.deckfour.xes.model.XTrace;
import org.processmining.ocbc.log.inspector.plugin.XOCLogInspectorPlugin;
import org.processmining.ocbc.log.model.XOCEvent;
import org.processmining.ocbc.log.model.XOCLog;
import org.processmining.ocbc.log.model.impl.xConstants;
import org.processmining.plugins.interactivevisualization.InteractivityContext;

import com.fluxicon.slickerbox.components.DistributionUI;
import com.fluxicon.slickerbox.components.RoundedPanel;

/**
 * @author Christian W. Guenther (christian@deckfour.org)
 * 
 */
public class XOCLogDashboard extends JPanel {

	/**
	 * 
	 */
	private static final long serialVersionUID = 6586390798776862988L;
	protected Color colorEnclosureBg = new Color(250, 250, 250, 105);
	protected Color colorTitleFg = new Color(20, 20, 20, 230);
	protected Color colorInfoBg = new Color(60, 60, 60, 160);
	protected Color colorInfoBgMouseOver = new Color(60, 60, 60, 240);
	protected Color colorInfoLabel = new Color(210, 210, 210);
	protected Color colorInfoValue = new Color(255, 255, 255);

	protected SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

	protected XOCLogInspectorPlugin parent;
	protected InteractivityContext interactivityContext;

	protected JLabel labelNumberOfProcesses;
	protected JLabel labelNumberOfEvents;
	protected JLabel labelNumberOfEventClasses;
	protected JLabel labelNumberOfObjects;
	protected JLabel labelNumberOfObjectVersions;
	protected JLabel labelNumberOfClasses;
	protected JLabel labelNumberOfOriginators;
	
	protected JLabel labelMinObjectsPerEvent;
	protected JLabel labelMeanObjectsPerEvent;
	protected JLabel labelMaxObjectsPerEvent;
	protected JPanel panelObjectsPerEventDistribution;
	
	protected JLabel labelMinObjectRelationsPerEvent;
	protected JLabel labelMeanObjectRelationsPerEvent;
	protected JLabel labelMaxObjectRelationsPerEvent;
	protected JPanel panelObjectRelationsPerEventDistribution;	

	protected JLabel labelStartDate;
	protected JLabel labelEndDate;
	protected XOCLog log;
	public XOCLogDashboard(XOCLogInspectorPlugin parent, XOCLog log, InteractivityContext ic) {
		this.parent = parent;
		this.interactivityContext = ic;
		this.log = log;
		//One-time initialization of the user interface.
		setupGui();
		
		//Update the user interface's information.
		setupInfo(log);
	}
	
//	private XLog getLog() {
//		return parent.getLog();
//	}
//	
//	private XLogInfo getSummary() {
//		return parent.getLogInfo();
//	}
	
	
	private void setupInfo(XOCLog log) {
//		XLogInfo summary = getSummary();
//		XLog log = getLog();
		
		if (log.getAllEvents().size() == 0) {
			labelNumberOfProcesses.setText("0");
			labelNumberOfClasses.setText("0");
			labelNumberOfEvents.setText("0");
			
			labelNumberOfObjects.setText("0");
			labelNumberOfObjectVersions.setText("0");
			labelNumberOfEventClasses.setText("0");
			labelNumberOfOriginators.setText("0");

			labelMinObjectsPerEvent.setText("N/A");
			labelMaxObjectsPerEvent.setText("N/A");
			labelMeanObjectsPerEvent.setText("N/A");

			labelStartDate.setText("<html>N/A</html>");
			labelEndDate.setText("<html>N/A</html>");
			return;
		}
		
		labelNumberOfProcesses.setText("1");
		labelNumberOfClasses.setText("" + log.getClassNames().size());
		int eventNum = log.getAllEvents().size();
		XOCEvent firstEvent = log.getAllEvents().get(0);
		XOCEvent lastEvent = log.getAllEvents().get(eventNum-1);
		labelNumberOfEvents.setText("" + eventNum);
		
		labelNumberOfObjects.setText("" + lastEvent.getObjectModel().getObjectSet().size());
		labelNumberOfObjectVersions.setText("" + log.getAllObjectVersions().size());
		labelNumberOfEventClasses.setText("" + log.createActivityNames().size());
		labelNumberOfOriginators.setText("" + log.getResources().size());
		
		int meanObjectNumPerEvent = 0;
		int meanObjectRelationNumPerEvent = 0;
		
		int[] objectRelationsPerEvent = new int[log.getAllEvents().size()];
		int[] objectsPerEvent = new int[log.getAllEvents().size()];

		for (int i = 0; i < log.size(); i++) {
			int objectNum = 0;
			int objectRelationNum = 0;
			String logFormat = log.getAttributes().get(xConstants.FORMAT).toString();
			if(logFormat.equals(xConstants.TOTAL)){
				objectNum = log.getAllEvents().get(i).getObjectModel().getObjectSet().size();
				objectRelationNum = log.getAllEvents().get(i).getObjectModel().getObjectRelationSet().size();
			}
			else if(logFormat.equals(xConstants.UPDATE)){
				objectNum = log.getAllEvents().get(i).getObjectModelUpdate().getObjectSet().size();
				objectRelationNum = log.getAllEvents().get(i).getObjectModelUpdate().getObjectRelationSet().size();
			}
			objectsPerEvent[i] = objectNum;
			meanObjectNumPerEvent += objectNum;

			objectRelationsPerEvent[i] = objectRelationNum;
			meanObjectRelationNumPerEvent += objectRelationNum;
		}
		Arrays.sort(objectsPerEvent);
		Arrays.sort(objectRelationsPerEvent);
		meanObjectNumPerEvent = meanObjectNumPerEvent/eventNum;
		meanObjectRelationNumPerEvent = (meanObjectRelationNumPerEvent + log.getAllEvents().size()/2) / log.getAllEvents().size();
		
		if (eventNum == 0) {
			return;
		}
		
		// it is not allowed to delete objects, therefore the number of objects is non-decreasing
//		Arrays.sort(objectsPerEvent);
		labelMinObjectsPerEvent.setText("" + objectsPerEvent[0]);
		labelMaxObjectsPerEvent.setText("" + objectsPerEvent[eventNum-1]);
		labelMeanObjectsPerEvent.setText("" + meanObjectNumPerEvent);
		panelObjectsPerEventDistribution.removeAll();
		
		panelObjectsPerEventDistribution.add(new DistributionUI(sample(objectsPerEvent)), BorderLayout.CENTER);
//		panelObjectsPerEventDistribution.add(new DistributionUI(new int[] {1,5,9,10,9}), BorderLayout.CENTER);

		panelObjectRelationsPerEventDistribution.removeAll();
		panelObjectRelationsPerEventDistribution.add(new DistributionUI(sample(objectRelationsPerEvent)), BorderLayout.CENTER);
		
		labelMinObjectRelationsPerEvent.setText("" + objectRelationsPerEvent[0]);
		labelMaxObjectRelationsPerEvent.setText("" + objectRelationsPerEvent[eventNum-1]);
		labelMeanObjectRelationsPerEvent.setText("" + meanObjectRelationNumPerEvent);
		
		String starttime = firstEvent.getAttributes().get("timestamp").toString();
		labelStartDate.setText("<html>"+(starttime == null ? "" : starttime.toString())+"</html>");
		String endtime = lastEvent.getAttributes().get("timestamp").toString();
		labelEndDate.setText("<html>"+(endtime == null ? "" : endtime.toString())+"</html>");
	}
	 
	private Collection<XEventClass> getClassesFor(XTrace trace, XEventClassifier classifier) {
		XEventClasses classes = new XEventClasses(classifier);
		classes.register(trace);
		return classes.getClasses();
	}

	protected void setupGui() {
		setOpaque(false);
		setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
		setBorder(BorderFactory.createEmptyBorder(1, 2, 4, 4));
		// create key data panel
		RoundedPanel keyPanel = new RoundedPanel(15, 0, 3);
		keyPanel.setMinimumSize(new Dimension(180, 100));
		keyPanel.setMaximumSize(new Dimension(300, 1000));
		keyPanel.setPreferredSize(new Dimension(200, 500));
		keyPanel.setBackground(colorEnclosureBg);
		keyPanel.setLayout(new BoxLayout(keyPanel, BoxLayout.Y_AXIS));
		keyPanel.add(getLeftAlignedHeader("Key data"));
		keyPanel.add(Box.createVerticalStrut(15));
		labelNumberOfProcesses = packInfo(keyPanel, "Processes");		
		keyPanel.add(Box.createVerticalStrut(15));
		labelNumberOfEvents = packInfo(keyPanel, "Events");
		keyPanel.add(Box.createVerticalStrut(3));
		labelNumberOfEventClasses = packInfo(keyPanel, "Event classes");
		keyPanel.add(Box.createVerticalStrut(15));
		labelNumberOfObjects = packInfo(keyPanel, "Objects");
		keyPanel.add(Box.createVerticalStrut(3));
		labelNumberOfObjectVersions = packInfo(keyPanel, "Object versions");
		keyPanel.add(Box.createVerticalStrut(3));
		labelNumberOfClasses = packInfo(keyPanel, "Object classes");
		keyPanel.add(Box.createVerticalStrut(15));
		labelNumberOfOriginators = packInfo(keyPanel, "Originators");
		keyPanel.add(Box.createVerticalGlue());
		// create info panel
		RoundedPanel infoPanel = new RoundedPanel(15, 0, 5);
		infoPanel.setMinimumSize(new Dimension(170, 100));
		infoPanel.setMaximumSize(new Dimension(300, 1000));
		infoPanel.setPreferredSize(new Dimension(190, 500));
		infoPanel.setBackground(colorEnclosureBg);
		infoPanel.setLayout(new BoxLayout(infoPanel, BoxLayout.Y_AXIS));
		infoPanel.add(getLeftAlignedHeader("Log info"));
		infoPanel.add(Box.createVerticalStrut(12));
		labelStartDate = getInfoPanel(infoPanel, "Start date");
		infoPanel.add(Box.createVerticalStrut(6));
		labelEndDate = getInfoPanel(infoPanel, "End date");
		infoPanel.add(Box.createVerticalStrut(12));
		//add the log format information
		String logFormat = "None";
		if(log.getAttributes().get(xConstants.FORMAT) != null){
			logFormat = log.getAttributes().get(xConstants.FORMAT).toString();
		}
		infoPanel.add(getLeftAlignedHeader("Log Format: " + logFormat));
		infoPanel.add(Box.createVerticalGlue());

		
		// create action panel
		RoundedPanel actionPanel = new RoundedPanel(15, 0, 0);
		actionPanel.setBackground(new Color(250, 250, 250, 80));
		actionPanel.setLayout(new BoxLayout(actionPanel, BoxLayout.X_AXIS));

		actionPanel.add(Box.createHorizontalGlue());
		JPanel rightPanel = new JPanel();
		rightPanel.setBorder(BorderFactory.createEmptyBorder());
		rightPanel.setMinimumSize(new Dimension(180, 100));
		rightPanel.setMaximumSize(new Dimension(300, 1000));
		rightPanel.setPreferredSize(new Dimension(200, 500));
		rightPanel.setOpaque(false);
		rightPanel.setLayout(new BoxLayout(rightPanel, BoxLayout.Y_AXIS));
		rightPanel.add(infoPanel);
		rightPanel.add(Box.createVerticalStrut(10));
		rightPanel.add(actionPanel);
		// right side
		JPanel distributionPanel = new JPanel();
		distributionPanel.setOpaque(false);
		distributionPanel.setBorder(BorderFactory.createEmptyBorder());
		distributionPanel.setLayout(new BoxLayout(distributionPanel, BoxLayout.Y_AXIS));
		
		JComponent[] components = new JComponent[4];
		int[] dummy = {1,2,3,4,5,6 };
		//		int[6] dummy;
		distributionPanel.add(getDistributionPanel("Objects per event", dummy, 0, components));
		labelMinObjectsPerEvent = (JLabel)components[0];
		labelMeanObjectsPerEvent = (JLabel)components[1];
		labelMaxObjectsPerEvent = (JLabel)components[2];		
		panelObjectsPerEventDistribution = (JPanel)components[3];		
		
		distributionPanel.add(Box.createVerticalStrut(10));
				
		distributionPanel.add(getDistributionPanel("Object relations per event", dummy, 0, components));
		labelMinObjectRelationsPerEvent = (JLabel)components[0];
		labelMeanObjectRelationsPerEvent = (JLabel)components[1];
		labelMaxObjectRelationsPerEvent = (JLabel)components[2];
		panelObjectRelationsPerEventDistribution = (JPanel)components[3];

		// assemble
		this.add(keyPanel);
		this.add(Box.createHorizontalStrut(10));
		this.add(distributionPanel);
		this.add(Box.createHorizontalStrut(10));
		this.add(rightPanel);
		revalidate();
	}
	
	protected JLabel packInfo(JPanel panel, String name) {
		String value = "Initializing ...";
		
		RoundedPanel packed = new RoundedPanel(10, 0, 0);
		packed.setBackground(colorInfoBg);
		final RoundedPanel target = packed;
		packed.addMouseListener(new MouseListener() {
			public void mouseClicked(MouseEvent arg0) { /* ignore */
			}

			public void mouseEntered(MouseEvent arg0) {
				target.setBackground(colorInfoBgMouseOver);
				target.repaint();
			}

			public void mouseExited(MouseEvent arg0) {
				target.setBackground(colorInfoBg);
				target.repaint();
			}

			public void mousePressed(MouseEvent arg0) { /* ignore */
			}

			public void mouseReleased(MouseEvent arg0) { /* ignore */
			}
		});
		packed.setLayout(new BoxLayout(packed, BoxLayout.X_AXIS));
		JLabel nameLabel = new JLabel(name);
		nameLabel.setOpaque(false);
		nameLabel.setForeground(colorInfoLabel);
		nameLabel.setFont(nameLabel.getFont().deriveFont(12f));
		JLabel valueLabel = new JLabel(value);
		valueLabel.setOpaque(false);
		valueLabel.setForeground(colorInfoValue);
		valueLabel.setFont(valueLabel.getFont().deriveFont(14f));
		packed.add(Box.createHorizontalStrut(5));
		packed.add(nameLabel);
		packed.add(Box.createHorizontalGlue());
		packed.add(valueLabel);
		packed.add(Box.createHorizontalStrut(5));
		packed.revalidate();
		
		panel.add(packed);
		
		return valueLabel;
	}

	protected RoundedPanel getDistributionPanel(String title, int[] values, int meanValue, JComponent[] result) {
		
		values[0] = 1;
		values[1] = 4;
		values[2] = 9;
		values[3] = 1;
		values[4] = 8;
		values[5] = 2;
		
		// create distribution panel
		RoundedPanel instancePanel = new RoundedPanel(15, 0, 0);
		instancePanel.setBackground(colorEnclosureBg);
		instancePanel.setLayout(new BoxLayout(instancePanel, BoxLayout.Y_AXIS));
		instancePanel.add(getLeftAlignedHeader(title));
		instancePanel.add(Box.createVerticalStrut(6));
		if (values.length == 0) {
			return instancePanel;
		}
		RoundedPanel instanceDistPanel = new RoundedPanel(10, 0, 0);
		result[3] = instanceDistPanel;
		instanceDistPanel.setBackground(new Color(20, 20, 20));
		instanceDistPanel.setLayout(new BorderLayout());
		DistributionUI instanceDistUI = new DistributionUI(sample(values));
		instanceDistPanel.add(instanceDistUI, BorderLayout.CENTER);
		JPanel keyPanel = new JPanel();
		keyPanel.setOpaque(false);
		keyPanel.setBorder(BorderFactory.createEmptyBorder());
		keyPanel.setLayout(new BoxLayout(keyPanel, BoxLayout.X_AXIS));
		result[0] = packInfo(keyPanel, "Min");
		keyPanel.add(Box.createHorizontalGlue());
		keyPanel.add(Box.createHorizontalGlue());
		keyPanel.add(Box.createHorizontalGlue());
		result[1] = packInfo(keyPanel, "Mean");
		keyPanel.add(Box.createHorizontalGlue());
		keyPanel.add(Box.createHorizontalGlue());
		keyPanel.add(Box.createHorizontalGlue());
		result[2] = packInfo(keyPanel, "Max");
		instancePanel.add(instanceDistPanel);
		instancePanel.add(Box.createVerticalStrut(4));
		instancePanel.add(keyPanel);
		return instancePanel;
	}

	protected JPanel getLeftAlignedHeader(String title) {
		JLabel hLabel = new JLabel(title);
		hLabel.setOpaque(false);
		hLabel.setForeground(colorTitleFg);
		hLabel.setFont(hLabel.getFont().deriveFont(15f));
		return alignLeft(hLabel);
	}

	protected JPanel alignLeft(JComponent component) {
		JPanel hPanel = new JPanel();
		hPanel.setBorder(BorderFactory.createEmptyBorder());
		hPanel.setOpaque(false);
		hPanel.setLayout(new BoxLayout(hPanel, BoxLayout.X_AXIS));
		hPanel.add(component);
		hPanel.add(Box.createHorizontalGlue());
		return hPanel;
	}

	protected JLabel getInfoPanel(JPanel parentPanel, String name) {
		String value = "Initializing ...";
		
		JPanel infoPanel = new JPanel();
		infoPanel.setOpaque(false);
		infoPanel.setBorder(BorderFactory.createEmptyBorder());
		infoPanel.setLayout(new BoxLayout(infoPanel, BoxLayout.Y_AXIS));
		JLabel nameLabel = new JLabel(name);
		nameLabel.setOpaque(false);
		nameLabel.setFont(nameLabel.getFont().deriveFont(10f));
		nameLabel.setForeground(new Color(60, 60, 60));
		JLabel valueLabel = new JLabel("<html>" + value + "</html>");
		valueLabel.setOpaque(false);
		valueLabel.setFont(nameLabel.getFont().deriveFont(12f));
		valueLabel.setForeground(new Color(10, 10, 10));
		infoPanel.add(alignLeft(nameLabel));
		infoPanel.add(Box.createVerticalStrut(2));
		infoPanel.add(alignLeft(valueLabel));
		
		parentPanel.add(infoPanel);
		
		return valueLabel;
	}

	
	/*
	 * Returns a sample of the provided values.
	 * There is no use in more than 2000 values if we cannot display them anyway.
	 * On some computers (Macs), providing more than 13000 values actually brings down the visualization.
	 * Therefore, we limit ourselves to visualizing only a sample (of 2000 values).
	 */
	private int[] sample(int[] values) {
		final int sampleSize = 2000;
		final double factor = ((double) values.length) / sampleSize;
		if (values.length <= sampleSize) {
			return values;
		}
		int[] sampledValues = new int[sampleSize];
		for (int i = sampleSize - 2; i >= 0; i--) {
			sampledValues[i] = values[(int) (i * factor)];
		}
		sampledValues[sampleSize - 1] = values[values.length - 1];
		return sampledValues;
	}
}
