package org.processmining.ocbc.log.inspector.panels;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.text.NumberFormat;
import java.util.Collection;
import java.util.List;
import java.util.Map;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollBar;
import javax.swing.JScrollPane;
import javax.swing.JTextPane;
import javax.swing.ScrollPaneConstants;
import javax.swing.SwingWorker;
import javax.swing.event.AncestorEvent;
import javax.swing.event.AncestorListener;

import org.deckfour.uitopia.ui.UITopiaFrame;
import org.deckfour.xes.classification.XEventClasses;
import org.deckfour.xes.classification.XEventClassifier;
import org.deckfour.xes.info.XLogInfo;
import org.deckfour.xes.info.impl.XLogInfoImpl;
import org.deckfour.xes.model.XLog;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.ocbc.log.inspector.plugin.XOCLogInspectorPlugin;
import org.processmining.ocbc.log.model.XOCLog;

import com.fluxicon.slickerbox.components.RoundedPanel;
import com.fluxicon.slickerbox.components.SlickerButton;
import com.fluxicon.slickerbox.ui.SlickerScrollBarUI;

public class XOCLogSummary extends JPanel {

	/**
	 *
	 */
	private static final long serialVersionUID = 58907178846268393L;

	private static final boolean ADDSTANDARDCLASSIFIER = false;
	
	/**
	 * The parent of the LogInfoUI.
	 */
	protected XOCLogInspectorPlugin parent;

	/**
	 * The summary of the log.
	 */
	protected XLogInfo summary;
	
	/**
	 * The text pane to put the summary in.
	 */
	protected JTextPane summaryPane = null;

	protected XOCLog log;
	/**
	 * The map from every extension found in the log to its classifier. If an
	 * extension comes with multiple classifiers, these will be combined using
	 * an XEventAndClassifier.
	 */
	//	private Map<XExtension, XEventClassifier> map;
	private Collection<XEventClassifier> logClassifiers;

	/**
	 * The map from every classifier to the event classes holding the summary
	 * for the start (end) events.
	 */
	private Map<XEventClassifier, XEventClasses> startMap;
	private Map<XEventClassifier, XEventClasses> endMap;

	/**
	 * The number format used for percentages.
	 */
	protected static final NumberFormat numberFormat = NumberFormat.getInstance();
	{
		numberFormat.setMinimumFractionDigits(1);
		numberFormat.setMaximumFractionDigits(3);
	}

	/**
	 * Creates the LogInfoUI, given the parent.
	 * 
	 * @param settings
	 *            The parent.
	 */
	public XOCLogSummary(XOCLogInspectorPlugin settings, final XOCLog log, final PluginContext context) {
		this.log = log;
		setOpaque(false);
		setBorder(BorderFactory.createEmptyBorder(3, 10, 5, 10));
		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		parent = settings;
		summaryPane = new JTextPane();
		summaryPane.setBorder(BorderFactory.createEmptyBorder());
		summaryPane.setContentType("text/html");
		// pre-populate the text pane with some teaser message
		summaryPane.setText("<html><body bgcolor=\"#888888\" text=\"#333333\">"
				+ "<br><br><br><br><br><center><font face=\"helvetica,arial,sans-serif\" size=\"4\">"
				+ "Please wait while the summary is created...</font></center></body></html>");
		summaryPane.setEditable(false);
		summaryPane.setCaretPosition(0);
		JScrollPane scrollPane = new JScrollPane(summaryPane);
		scrollPane.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
		scrollPane.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);
		JScrollBar vBar = scrollPane.getVerticalScrollBar();
		vBar.setUI(new SlickerScrollBarUI(vBar, new Color(0, 0, 0, 0), new Color(20, 20, 20), new Color(60, 60, 60), 4,
				12));
		vBar.setOpaque(false);
		scrollPane.setOpaque(false);
		scrollPane.setBorder(BorderFactory.createEmptyBorder());
		RoundedPanel scrollEnclosure = new RoundedPanel(10, 0, 0);
		scrollEnclosure.setBackground(Color.decode("#888888"));
		scrollEnclosure.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
		scrollEnclosure.setLayout(new BorderLayout());
		scrollEnclosure.add(scrollPane, BorderLayout.CENTER);
		JLabel header = new JLabel("Log Summary");
		header.setOpaque(false);
		header.setForeground(new Color(200, 200, 200, 180));
		header.setFont(header.getFont().deriveFont(15f));
		JButton saveButton = new SlickerButton("save HTML...");
		saveButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				JFileChooser saveDialog = new JFileChooser();
				saveDialog.setSelectedFile(new File("ProM_LogSummary.html"));
				UITopiaFrame parent = null;
				if (context != null && context instanceof UIPluginContext) {
					parent = ((UIPluginContext) context).getGlobalContext().getUI();
				}
				if (saveDialog.showSaveDialog(parent) == JFileChooser.APPROVE_OPTION) {
					File outFile = saveDialog.getSelectedFile();
					try {
						BufferedWriter outWriter = new BufferedWriter(new FileWriter(outFile));
						outWriter.write(summaryPane.getText());
						outWriter.flush();
						outWriter.close();
						JOptionPane.showMessageDialog(parent,
								"Log summary has been saved\nto HTML file!", "Log summary saved.",
								JOptionPane.INFORMATION_MESSAGE);
					} catch (IOException e1) {
						e1.printStackTrace();
					}
				}
			}
		});
		JPanel headerPanel = new JPanel();
		headerPanel.setOpaque(false);
		headerPanel.setBorder(BorderFactory.createEmptyBorder());
		headerPanel.setLayout(new BoxLayout(headerPanel, BoxLayout.X_AXIS));
		headerPanel.add(header);
		headerPanel.add(Box.createHorizontalGlue());
		headerPanel.add(saveButton);
		this.add(headerPanel);
		this.add(Box.createVerticalStrut(7));
		this.add(scrollEnclosure);
		System.out.printf("\n enter XOCLogSummary");
		addAncestorListener(new AncestorListener() {
			public void ancestorAdded(AncestorEvent arg0) {
				checkRecompileSummary();
			}

			public void ancestorMoved(AncestorEvent arg0) { /* ignore */
			}

			public void ancestorRemoved(AncestorEvent arg0) { /* ignore */
			}
		});
	}

	/**
	 * Recompiles the entire summary.
	 */
	protected void checkRecompileSummary() {
		System.out.printf("output: ");
		SwingWorker<Object, Object> worker = new SwingWorker<Object, Object>() {
			
			protected String htmlSummary = null;

			public Object doInBackground() {
				System.out.printf("\n doInBackground()");
				Thread.yield();
				htmlSummary = summaryPane.getText();
				summaryPane.setText("<html><body bgcolor=\"#888888\" text=\"#333333\">"
						+ "<br><br><br><br><br><center><font face=\"helvetica,arial,sans-serif\" size=\"4\">"
						+ "Please wait while the summary is created...</font></center></body></html>");
				repaint();
				Thread.yield();
				
				System.out.printf("\n after Thread.yield();");
				// Get the log from the parent.
//				XLogInfo nSummary = parent.getLogInfo();
//				// Recompile only if necessary.
//				if ((summary == null) || !(summary.equals(nSummary))) {
				if (log != null) {
					System.out.printf("\n log != null");
//					// Recompilation necessary.
//					XLog log = parent.getLog();
//					// Get the extensions and their classifiers.
//					//					map = getMap(log);
//					logClassifiers = log.getClassifiers();
//					//					Collection<XEventClassifier> values = map.values();
//					// Get a summary based on the classifiers found in the previous step.
//					XLogInfo info = getLogInfo(null, logClassifiers);
//					// Create the summary for the start and end events.
					createLogStartEndInfo(logClassifiers);
					System.out.printf("\n createLogStartEndInfo");
//					// Put the entire summary into HTML format.
					htmlSummary = format(log);
					System.out.printf("\n htmlSummary  ");
//					// Record last used log info.
//					summary = nSummary;
					
				}
				System.out.printf("\n output: " + htmlSummary);
				return null;
			}

			public void done() {
				if (htmlSummary != null) {
					// Put the HTML formatted summary in the text pane.
					summaryPane.setText(htmlSummary);
					summaryPane.setCaretPosition(0);
				}
				invalidate();
				repaint();
			}
		};
		worker.execute();
	}

	/**
	 * Creates and returns a mapping from extensions supported to the log by
	 * their classifiers. If an extension has multiple classifiers, then these
	 * classifiers will be combined in one AND classifier. As a result, every
	 * extension will be mapped onto at most one classifier. Extensions that
	 * have no classifiers will not be mapped.
	 * 
	 * @param log
	 * @return A mapping from extension supported by the log to their
	 *         classifiers.
	 */
	//	private Map<XExtension, XEventClassifier> getMap(XLog log) {
	//		Set<XExtension> extensions = log.getExtensions();
	//		HashMap<XExtension, XEventClassifier> map = new HashMap<XExtension, XEventClassifier>();
	//		for (XExtension extension : extensions) {
	//			XEventClassifier[] classifiers = new XEventAttributeClassifier[extension.getEventAttributes().size()];
	//			int size = 0;
	//			for (XAttribute attribute : extension.getEventAttributes()) {
	//				classifiers[size++] = new XEventAttributeClassifier(attribute.getKey(), attribute.getKey());
	//			}
	//			if (size > 1) {
	//				// Extension contains multiple classifiers, combine them using an XEventAndClassifier.
	//				map.put(extension, new XEventAndClassifier(classifiers));
	//			} else if (size == 1) {
	//				// Extension contains a single classifier, use it.
	//				map.put(extension, classifiers[0]);
	//			}
	//			// Else extension contains no classifier, so why bother?
	//		}
	//		return map;
	//	}

	/**
	 * Gets the summary for the given log, given the classifiers to use and the
	 * standard classifier.
	 * 
	 * @param log
	 *            The given log.
	 * @param classifiers
	 *            The classifiers to use.
	 * @return The summary.
	 */
	private XLogInfo getLogInfo(XLog log, Collection<XEventClassifier> classifiers) {
		//TODO Incorrect log info gets generated when there exist traces with 0 events.
		return XLogInfoImpl.create(log, XLogInfoImpl.STANDARD_CLASSIFIER, classifiers);
	}

	/**
	 * Creates the summary info for the start and end events, for all provided
	 * classifiers and the standard classifier.
	 * 
	 * @param log
	 *            The log to create the summary info for.
	 * @param classifiers
	 *            The provided classifiers.
	 */
	private void createLogStartEndInfo(Collection<XEventClassifier> classifiers) {
		// Initialize both maps.
//		startMap = new HashMap<XEventClassifier, XEventClasses>();
//		endMap = new HashMap<XEventClassifier, XEventClasses>();
//		if (ADDSTANDARDCLASSIFIER) {
//			startMap.put(XLogInfoImpl.STANDARD_CLASSIFIER, new XEventClasses(XLogInfoImpl.STANDARD_CLASSIFIER));
//			endMap.put(XLogInfoImpl.STANDARD_CLASSIFIER, new XEventClasses(XLogInfoImpl.STANDARD_CLASSIFIER));
//		}
//		for (XEventClassifier classifier : classifiers) {
//			startMap.put(classifier, new XEventClasses(classifier));
//			endMap.put(classifier, new XEventClasses(classifier));
//		}
		// Fill both maps by traversing the log.
//		for (XTrace trace : log) {
//			if (trace.size() > 0) {
//				for (XEventClasses eventClasses : startMap.values()) {
//					eventClasses.register(trace.get(0));
//				}
//				for (XEventClasses eventClasses : endMap.values()) {
//					eventClasses.register(trace.get(trace.size() - 1));
//				}
//			}
//		}
	}

	// HTML encodings
	protected static final String FONT_TEXT = "<font face=\"helvetica,arial,sans-serif\" size=\"4\">";
	protected static final String FONT_CODE = "<font face=\"andale mono,courier\" color=\"#222222\" size=\"3.2\">";
	protected static final String FONT_HEADING = "<font face=\"arial,helvetica,sans-serif\" size=\"+2\" color=\"#111111\">";
	protected static final String FONT_SUBHEADING = "<font face=\"arial,helvetica,sans-serif\" size=\"+1\" color=\"#111111\">";
	protected static final String TABLE = "<table bgcolor=\"#CCCCDD\" border=\"0\" width=\"95%\" cellpadding=\"4\" cellspacing=\"2\">";
	protected static final String TD = "<td bgcolor=\"#CCCCCC\" valign=\"top\" align=\"left\">";
	protected static final String TR = "<tr>";
	protected static final String PART_START = "<tr><td width=\"100%\" align=\"left\" bgcolor=\"#AAAAAA\">";
	protected static final String PART_STOP = "</td></tr>";

	/**
	 * Creates an HTML-formatted string of the provided summary info.
	 * 
	 * @param info
	 *            The provided summary info.
	 * @return The HTML formatted string.
	 */
	public String format(XOCLog log) {
		StringBuffer sumString = new StringBuffer();

		// Add the document header.
		writeDocumentHeader(sumString);

		// Add a general summary, not based on any extension.
		sumString.append(PART_START);
		heading(sumString, "Log Summary");
		sumString.append(FONT_TEXT + "Total number of events: <b>" + log.getAllEvents().size() + "</b></font><br>");
		sumString.append(FONT_TEXT + "Total number of object states: <b>" + log.getRepeatableObjects().size() + "</b></font><br>");
		//encodeInfoItem(sumString, summary.getWorkflowLog());
		sumString.append(PART_STOP);

		// Add a summary for the standard classifier.
		if (ADDSTANDARDCLASSIFIER) {
			writeTable(sumString, log, "Standard");
		}

		// Add a summary for every classifier that corresponds to an extension found in the log.
		//		for (XExtension extension : map.keySet()) {
		//			writeTable(sumString, info, map.get(extension), extension.toString());
		//		}
//		for (XEventClassifier classifier : logClassifiers) {
//			writeTable(sumString, log, classifier, "Activity");
//		}
		writeTable(sumString, log, "Activity");
		// Add the document footer.
		writeDocumentFooter(sumString);

		return sumString.toString();
	}

	/**
	 * Writes the summary for the given classifier to the given buffer.
	 * 
	 * @param sumString
	 *            The given buffer.
	 * @param info
	 *            The summary info.
	 * @param classifier
	 *            The given classifier.
	 * @param heading
	 *            The header text for this classifier.
	 */
	private void writeTable(StringBuffer sumString, XOCLog log, String heading) {
		// Get the summary info for all three parts.
//		XEventClasses allEventClasses = info.getEventClasses(classifier);
//		XEventClasses startEventClasses = startMap.get(classifier);
//		XEventClasses endEventClasses = endMap.get(classifier);

		sumString.append(PART_START);
		heading(sumString, heading);

		List<String> sortedActivityList =  log.createActivityNames();
		Map<String, Integer> activityVSNumMap = log.createActivityVSNumMap();
		int activityInstanceSize = log.getAllEvents().size();
		
		for (int i = 0; i < 3; i++) {
			XEventClasses eventClasses = null;
			switch (i) {
				case 0 :
					// First, deal with the general summary. 
					sumString.append(FONT_TEXT + "Event classes defined by Activity" + "</font><br>");
					subHeading(sumString, "All events");
					break;
				case 1 :
					// Second, deal with the summary concerning start events.
					sortedActivityList = log.createStartActivityNames();
					activityVSNumMap = log.createStartActivityVSNumMap();
					activityInstanceSize = log.createStartEvents().size();
					subHeading(sumString, "Start events");
					break;
				default :
					// Last, deal with the summary concerning end events.
					sortedActivityList = log.createEndActivityNames();
					activityVSNumMap = log.createEndActivityVSNumMap();
					activityInstanceSize = log.createEndEvents().size();
					subHeading(sumString, "End events");
			}
			// For all three parts, the summary looks alike.
			sumString.append(FONT_TEXT + "Total number of classes: <b>" + sortedActivityList.size()
					+ "</b></font><br>");

			sumString.append("<center>");
			sumString.append(TABLE);
			sumString.append(TR);
			sumString.append(TD);
			sumString.append(FONT_TEXT);
			sumString.append("<b>Class</b></font></td>");
			sumString.append(TD);
			sumString.append(FONT_TEXT);
			sumString.append("<b>Occurrences (absolute)</b></font></td>");
			sumString.append(TD);
			sumString.append(FONT_TEXT);
			sumString.append("<b>Occurrences (relative)</b></font></td></tr>");
			
			
			for (String activity : sortedActivityList) {
				float activityNum = activityVSNumMap.get(activity);
				sumString.append(TR);
				sumString.append(TD);
				sumString.append(FONT_TEXT);
				sumString.append(activity);
				sumString.append("</font></td>");
				sumString.append(TD);
				sumString.append(FONT_TEXT);
				sumString.append((int)activityNum);
				sumString.append("</font></td>");
				sumString.append(TD);
				sumString.append(FONT_TEXT);
				sumString.append(activityNum*100/activityInstanceSize + "%");
				sumString.append("</font></td></tr>");
			}
			sumString.append("</table>");
			sumString.append("</center>");
		}
		sumString.append(PART_STOP);
	}



	/**
	 * Writes the document header to the given buffer.
	 * 
	 * @param buffer
	 *            The given buffer.
	 */
	private void writeDocumentHeader(StringBuffer buffer) {
		buffer.append("<html><body bgcolor=\"#888888\" text=\"#111111\">");
		buffer.append("<center><br><table width=\"95%\" border=\"0\" cellspacing=\"10\" cellpadding=\"10\">");
	}

	/**
	 * Writes the document footer to the given buffer.
	 * 
	 * @param buffer
	 *            The given buffer.
	 */
	private void writeDocumentFooter(StringBuffer buffer) {
		buffer.append("</table><br><br></body></html>");
	}

	/**
	 * Writes the given heading to the given buffer.
	 * 
	 * @param buffer
	 *            The given buffer.
	 * @param heading
	 *            The given heading.
	 */
	private void heading(StringBuffer buffer, String heading) {
		buffer.append(FONT_HEADING);
		buffer.append(heading);
		buffer.append("</font><hr noshade width=\"100%\" size=\"1\"><br>");
	}

	/**
	 * Writes the given subheading to the given buffer.
	 * 
	 * @param buffer
	 *            The given buffer.
	 * @param heading
	 *            The given subheading.
	 */
	private void subHeading(StringBuffer buffer, String heading) {
		buffer.append(FONT_SUBHEADING);
		buffer.append(heading);
		buffer.append("</font><hr noshade width=\"100%\" size=\"1\"><br>");
	}



	public ActionListener getActivationListener() {
		ActionListener activationListener = new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				checkRecompileSummary();
			}
		};
		return activationListener;
	}
	
	public void updateVisualization(boolean logChanged, boolean tagsChanged) {
		
		if(logChanged) {
			checkRecompileSummary();
			repaint();
		}
	}
}
