/*
 * Copyright (c) 2007 Christian W. Guenther (christian@deckfour.org)
 * 
 * LICENSE:
 * 
 * This code is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 * 
 * EXEMPTION:
 * 
 * License to link and use is also granted to open source programs which are not
 * licensed under the terms of the GPL, given that they satisfy one or more of
 * the following conditions: 1) Explicit license is granted to the ProM and
 * ProMimport programs for usage, linking, and derivative work. 2) Carte blance
 * license is granted to all programs developed at Eindhoven Technical
 * University, The Netherlands, or under the umbrella of STW Technology
 * Foundation, The Netherlands. For further exemptions not covered by the above
 * conditions, please contact the author of this code.
 */
package org.processmining.ocbc.log.inspector.panels;



import java.awt.BorderLayout;
import java.awt.Color;
import java.io.File;

import javax.swing.BorderFactory;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.border.Border;
import javax.swing.plaf.basic.BasicSplitPaneDivider;
import javax.swing.plaf.basic.BasicSplitPaneUI;

import org.processmining.common.runningtime.RunningTime;
import org.processmining.ocbc.conformance.visualization.views.ConformanceCheckingResultView;
import org.processmining.ocbc.log.inspector.interaction.GraphMouseListener;
import org.processmining.ocbc.log.io.XocXmlParser;
import org.processmining.ocbc.log.model.XOCLog;

import com.fluxicon.slickerbox.components.SlickerTabbedPane;

/**
 * @author Christian W. Guenther (christian@deckfour.org)
 * 
 */
public class XOCLogViewer extends JPanel {

	/**
	 * 
	 */
	private static final long serialVersionUID = 7772305278301666972L;
	public static final Color COLOR_ENCLOSURE_BG = new Color(40, 40, 40);
	public static final Color COLOR_NON_FOCUS = new Color(70, 70, 70);
	public static final Color COLOR_LIST_BG = new Color(60, 60, 60);
	public static final Color COLOR_LIST_BG_LOWER = new Color(45, 45, 45);
	public static final Color COLOR_LIST_FG = new Color(180, 180, 180);
	public static final Color COLOR_LIST_SELECTION_BG = new Color(80, 0, 0);
	public static final Color COLOR_LIST_SELECTION_BG_LOWER = new Color(30, 10, 10);
	public static final Color COLOR_LIST_SELECTION_FG = new Color(240, 240, 240);
	
	private GraphMouseListener graphMouseListener;	
	
	protected SlickerTabbedPane tabPane;
	
	//achieve the deviation information; in other plugins except cc plugin, it is empty
	protected ConformanceCheckingResultView conformanceCheckingResult;
	protected XOCLogBrowser browser;
//	public void setConformanceCheckingResult(ConformanceCheckingResultView conformanceCheckingResult){
//		this.conformanceCheckingResult = conformanceCheckingResult;
//	}
	
	public ConformanceCheckingResultView getConformanceCheckingResult(){
		return conformanceCheckingResult;
	}
	
	public XOCLogBrowser getLogBrowser(){
		return browser;
	}
	
	public XOCLogViewer(XOCLog log, ConformanceCheckingResultView conformanceCheckingResult) {
		this.conformanceCheckingResult = conformanceCheckingResult;
		setOpaque(false);
		setBorder(BorderFactory.createEmptyBorder());
		setLayout(new BorderLayout());
		tabPane = new SlickerTabbedPane("Log inspector", new Color(240, 240, 240, 230), new Color(0, 0, 0, 230),
				new Color(220, 220, 220, 150));

		long uint = 1;	
		long startTimeTotal = System.currentTimeMillis(); 
		long startTime = System.currentTimeMillis(); 
		long durationTime = System.currentTimeMillis();
		
		browser = new XOCLogBrowser(log, this);
		tabPane.addTab("Browser", browser);

		
    	durationTime = (System.currentTimeMillis() - startTime)/uint;
 		startTime = System.currentTimeMillis();
 		RunningTime.writeTimeInformationIntoFile("complete browser end: " + System.currentTimeMillis()/uint, false, "D:\\timeVisualizingXOCLog.txt");
 		RunningTime.writeTimeInformationIntoFile("~~~~~~~~~~~~~~complete browser duration time: " + durationTime, false, "D:\\timeVisualizingXOCLog.txt");

		this.add(tabPane);
		
    	durationTime = (System.currentTimeMillis() - startTime)/uint;
 		startTime = System.currentTimeMillis();
 		RunningTime.writeTimeInformationIntoFile("complete evolution end: " + System.currentTimeMillis()/uint, false, "D:\\timeVisualizingXOCLog.txt");
 		RunningTime.writeTimeInformationIntoFile("~~~~~~~~~~~~~~complete evolution duration time: " + durationTime, false, "D:\\timeVisualizingXOCLog.txt");
	}
	
	public XOCLogViewer(XOCLog log) {
		
		setOpaque(false);
		setBorder(BorderFactory.createEmptyBorder());
		setLayout(new BorderLayout());
		tabPane = new SlickerTabbedPane("Log inspector", new Color(240, 240, 240, 230), new Color(0, 0, 0, 230),
				new Color(220, 220, 220, 150));

		long uint = 1;	
		long startTimeTotal = System.currentTimeMillis(); 
		long startTime = System.currentTimeMillis(); 
		long durationTime = System.currentTimeMillis();
		
		XOCLogBrowser browser = new XOCLogBrowser(log, this);
		tabPane.addTab("Browser", browser);
//		JPanel validityPanel = new JPanel();
//		tabPane.addTab("Browser", validityPanel);
		
    	durationTime = (System.currentTimeMillis() - startTime)/uint;
 		startTime = System.currentTimeMillis();
 		RunningTime.writeTimeInformationIntoFile("complete browser end: " + System.currentTimeMillis()/uint, false, "D:\\timeVisualizingXOCLog.txt");
 		RunningTime.writeTimeInformationIntoFile("~~~~~~~~~~~~~~complete browser duration time: " + durationTime, false, "D:\\timeVisualizingXOCLog.txt");

//		XOCLogEvolution evolution = new XOCLogEvolution((XOCLog) log.clone());
////		XOCLogEvolution evolution = new XOCLogEvolution(log);
//		tabPane.addTab("Reference", evolution);
		this.add(tabPane);
		
    	durationTime = (System.currentTimeMillis() - startTime)/uint;
 		startTime = System.currentTimeMillis();
 		RunningTime.writeTimeInformationIntoFile("complete evolution end: " + System.currentTimeMillis()/uint, false, "D:\\timeVisualizingXOCLog.txt");
 		RunningTime.writeTimeInformationIntoFile("~~~~~~~~~~~~~~complete evolution duration time: " + durationTime, false, "D:\\timeVisualizingXOCLog.txt");
	}

	
	public GraphMouseListener getGraphMouseListener(){
		return graphMouseListener;
	}
	
	public JSplitPane borderlessSplitPane(int orientation, JComponent left, JComponent right) {
		JSplitPane pane = new JSplitPane(orientation, left, right);
		pane.setBorder(BorderFactory.createEmptyBorder(0, 0, 0, 0));
		BasicSplitPaneUI flatDividerSplitPaneUI = new BasicSplitPaneUI() {
			@Override
			public BasicSplitPaneDivider createDefaultDivider() {
				return new BasicSplitPaneDivider(this) {
					@Override
					public void setBorder(Border b) {
					}
				};
			}
		};
		pane.setUI(flatDividerSplitPaneUI);
		pane.setBorder(null);
		pane.setOpaque(false);
		return pane;
	}


	
	public static void main(String[] args) throws Exception  
    {  
//		int classNum = 3;
//		int horizontalUintNum = (int) Math.sqrt(classNum);
//		System.out.print("value" + horizontalUintNum);
//		System.exit(0);
		XocXmlParser XOCParser = new XocXmlParser();
		XOCParser.parse(new File("e:\\normal.xoc"));		
		
		XOCLog inputLog = XOCParser.getLog();
      
		XOCLogViewer eventUI = new XOCLogViewer(inputLog);
		JFrame frame = new JFrame();
		frame.getContentPane().add(new JScrollPane(eventUI));
		frame.setSize(600, 600);
		frame.setVisible(true);
    }
}
