package org.processmining.ocbc.log.inspector.plugin;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Image;
import java.awt.Toolkit;
import java.io.File;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.SortedSet;

import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import org.processmining.common.runningtime.RunningTime;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.contexts.uitopia.annotations.Visualizer;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.util.ui.scalableview.ScalableComponent;
import org.processmining.ocbc.log.inspector.panels.XOCLogDashboard;
import org.processmining.ocbc.log.inspector.panels.XOCLogSummary;
import org.processmining.ocbc.log.inspector.panels.XOCLogViewer;
import org.processmining.ocbc.log.io.XocXmlParser;
import org.processmining.ocbc.log.model.XOCLog;
import org.processmining.plugins.interactivevisualization.EID;
import org.processmining.plugins.interactivevisualization.InteractiveVisualization;
import org.processmining.plugins.interactivevisualization.InteractivityContext;
import org.processmining.plugins.interactivevisualization.InteractivityManager;

import com.fluxicon.slickerbox.components.IconVerticalTabbedPane;

public class XOCLogInspectorPlugin extends JPanel implements ScalableComponent, InteractiveVisualization{
	
	protected XOCLogViewer viewerUI;
	protected XOCLogDashboard dashboardUI;
	protected XOCLogSummary summaryUI;
	
//	protected LogInfoUI loginfoUI;
	
	// variables used in ScalableComponent interface.
	private final List<UpdateListener> observers = new LinkedList<UpdateListener>();
	
	// variables used in InteractiveVisualization interface.
	private InteractivityContext interactivityContext = new InteractivityContext();
	
	private final static String DASHBOARDICON = "dashboard48.png";
	private final static String INSPECTORICON = "inspector48.png";
	private final static String SUMMARYICON = "summary48-2.png";
	// icons
	protected Image dashboardIcon = new ImageIcon(XOCLogInspectorPlugin.class.getResource(DASHBOARDICON),DASHBOARDICON).getImage();
	protected Image inspectorIcon = new ImageIcon(XOCLogInspectorPlugin.class.getResource(INSPECTORICON),INSPECTORICON).getImage();
	protected Image summaryIcon = new ImageIcon(XOCLogInspectorPlugin.class.getResource(SUMMARYICON),SUMMARYICON).getImage();
	
	protected XOCLog log;
	
	@Plugin(
			name = "XOCLog Inspector", 
			parameterLabels = {"XOCLog"},
			returnTypes = JComponent.class, 
			returnLabels = "Log Visualization", 
			mostSignificantResult = 1, 
			help = "Provides an overview of the log"
	)
	
	@Visualizer
	public JComponent edit(UIPluginContext context, XOCLog log) {
		this.log = log;
//		this.summary = summary;
		initialize(null);
		completeGui(context, log);
		return this;
	}

	@Plugin(
			name = "XOCLog Inspector", 
			parameterLabels = {"XOCLog"},
			returnTypes = XOCLog.class, 
			returnLabels = "Log Visualization", 
			mostSignificantResult = 1, 
			help = "Provides an overview of a XOC log."
	)
	@UITopiaVariant(
			affiliation = UITopiaVariant.EHV,
			author = "Guangming Li",
			email = "g.li.3@tue.nl"
	)	
	public XOCLog inspectXOCLog(UIPluginContext context, XOCLog log) {
		return log;
	}
	
	protected void initialize(String path) {
		setBackground(new Color(40, 40, 40));
		setBorder(BorderFactory.createEmptyBorder());
		setLayout(new BorderLayout());
	}
	
	protected void completeGui(PluginContext context, XOCLog log) {
		long uint = 1;	
		long startTimeTotal = System.currentTimeMillis(); 
		long startTime = System.currentTimeMillis(); 
		long durationTime = System.currentTimeMillis();
		
		if(log.size() > 0){
			// compile inspector and dashboard panels
			viewerUI = new XOCLogViewer(log);
			
        	durationTime = (System.currentTimeMillis() - startTime)/uint;
     		startTime = System.currentTimeMillis();
     		RunningTime.writeTimeInformationIntoFile("complete viewerUI end: " + System.currentTimeMillis()/uint, false, "D:\\timeVisualizingXOCLog.txt");
     		RunningTime.writeTimeInformationIntoFile("~~~~~~~~~~~~~~complete viewerUI duration time: " + durationTime, false, "D:\\timeVisualizingXOCLog.txt");
  
     		
			dashboardUI = new XOCLogDashboard(this, log, interactivityContext);
			
	        durationTime = (System.currentTimeMillis() - startTime)/uint;
			startTime = System.currentTimeMillis();
			RunningTime.writeTimeInformationIntoFile("complete the dashboardUI end: " + System.currentTimeMillis()/uint, false, "D:\\timeVisualizingXOCLog.txt");
			RunningTime.writeTimeInformationIntoFile("~~~~~~~~~~~~~~complete the dashboardUI duration time: " + durationTime, false, "D:\\timeVisualizingXOCLog.txt");
			

			
			summaryUI = new XOCLogSummary(this, log, context);
			
	        durationTime = (System.currentTimeMillis() - startTime)/uint;
			startTime = System.currentTimeMillis();
			RunningTime.writeTimeInformationIntoFile("complete the summaryUI end: " + System.currentTimeMillis()/uint, false, "D:\\timeVisualizingXOCLog.txt");
			RunningTime.writeTimeInformationIntoFile("~~~~~~~~~~~~~~complete the summaryUI duration time: " + durationTime, false, "D:\\timeVisualizingXOCLog.txt");
			
			
			// compile root panel
			IconVerticalTabbedPane iconTabs = new IconVerticalTabbedPane(new Color(230, 230, 230, 210), new Color(20, 20,
					20, 160));
			iconTabs.addTab("Dashboard", dashboardIcon, dashboardUI);
			iconTabs.addTab("Viewer", inspectorIcon, viewerUI);
			iconTabs.addTab("Summary", summaryIcon, summaryUI);
			add(iconTabs, BorderLayout.CENTER);
			revalidate();
			repaint();
			
			
	        durationTime = (System.currentTimeMillis() - startTime)/uint;
			startTime = System.currentTimeMillis();
			RunningTime.writeTimeInformationIntoFile("complete all end: " + System.currentTimeMillis()/uint, false, "D:\\timeVisualizingXOCLog.txt");
			RunningTime.writeTimeInformationIntoFile("~~~~~~~~~~~~~~complete the all duration time: " + durationTime, false, "D:\\timeVisualizingXOCLog.txt");
			
		}
		else{
			JOptionPane.showMessageDialog(null, "Wrong! The log is empty!");
		}
	}
	

	
	public static void main(String[] args) throws Exception  
    {  
     			 
//		   FileInputStream input = new FileInputStream("E:\\new-last.ocbcmml");

		  
		XocXmlParser XOCParser = new XocXmlParser();
//		XOCParser.parse(new File("d:\\logVersion1.xoc"));	
//		XOCParser.parse(new File("d:\\log.xoc"));	
		XOCParser.parse(new File("d:\\logDuplicated.xoc"));	
//		XOCParser.parse(new File("e:\\large.xoc"));	
		
		XOCLog inputLog = XOCParser.getLog();
		
		if(inputLog == null) System.out.print("null" );
		System.out.println("log.size: " + inputLog.size());
		System.out.println("log.size: " + inputLog.getAllEvents().size());
//		SlickerOpenLogSettings dialogue = new SlickerOpenLogSettings();
			 
		XOCLogInspectorPlugin plugin = new XOCLogInspectorPlugin();
		JPanel ret = (JPanel) plugin.edit(null,inputLog);;
		JFrame frame = new JFrame();
		frame.setSize(Toolkit.getDefaultToolkit().getScreenSize());
		frame.getContentPane().add(new JScrollPane(ret));
		frame.setVisible(true);
		System.out.print("end");
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    }

	@Override
	public void setManager(InteractivityManager manager) {
		interactivityContext.setManager(manager);
	}
	
	@Override
	public Map<String, SortedSet<EID>> getSelectedElements() {
		throw new UnsupportedOperationException("This visualization does not support communicating selected elements.");
	}
	
	@Override
	public Map<String, SortedSet<EID>> getVisibleElements() {
		throw new UnsupportedOperationException("This visualization does not support communicating visible elements.");
	}
	
	@Override
	public InteractivityManager getManager() {
		return interactivityContext.getManager();
	}
	
	/*
	 * Implementation of Scalable Component interface (non-Javadoc)
	 * @see org.processmining.framework.util.ui.scalableview.ScalableComponent
	 */
	
	@Override
	public JComponent getComponent() {
		return this;
	}
	
	@Override
	public double getScale() {
		return 1;
	}
	
	@Override
	public void setScale(double newScale) {
		//TODO debug code
		System.err.println("Supposed to set the scale to "+newScale);
		
		//TODO set scale
	}
	
	@Override
	public void addUpdateListener(UpdateListener listener) {
		
		if(listener == null) {
			throw new IllegalArgumentException("The UpdateListener cannot be 'null'.");
		}
		
		observers.add(listener);
	}
	
	@Override
	public void removeUpdateListener(UpdateListener listener) {
		
		if(listener == null) {
			throw new IllegalArgumentException("The UpdateListener cannot be 'null'.");
		}
		
		observers.remove(listener);
	}

	public void updateVisualization(boolean logChanged, boolean tagsChanged) {
		// TODO Auto-generated method stub
		
	}	
}
