package org.processmining.ocbc.log.io;

import java.io.IOException;
import java.io.InputStream;

import org.processmining.common.runningtime.RunningTime;
import org.processmining.contexts.uitopia.annotations.UIImportPlugin;
import org.processmining.framework.abstractplugins.AbstractImportPlugin;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.ocbc.log.model.XOCLog;



/**
 * Class with the import plugin for a collection of processes
 * 
 * @author Andrea Burattin
 */
@Plugin(
	name = "Import XOC log (.xoc)",
	parameterLabels = { "File" },
	returnLabels = { "Imported XOC log" },
	returnTypes = { XOCLog.class },
	userAccessible = true
)
@UIImportPlugin(
		description = "Import XOC log (.xoc)",//show the type in window for recording files
		extensions = { "xoc" }// the suffix name
)
public class ImportXOCLog extends AbstractImportPlugin {

	/**
	 * This plugin is used to import an OCBC model
	 * 
	 * @param context
	 * @param input
	 * @param filename
	 * @param fileSizeInBytes
	 * @return
	 * @throws IOException 
	 */
	protected Object importFromStream(
			PluginContext context, 
			InputStream input, 
			String filename, 
			long fileSizeInBytes)
			throws Exception {
		System.out.println("xoc input  start");

		long uint = 1;	
		long startTimeTotal = System.currentTimeMillis(); 
		long startTime = System.currentTimeMillis(); 
		long durationTime = System.currentTimeMillis();
		
		RunningTime.writeTimeInformationIntoFile("************start parsing************", false, "D:\\timeImportXOCLog.txt");
//		RunningTime.writeTimeInformationIntoFile("model discovery start time: " + startTime/uint, false, "D:\\timeImportXOCLog.txt");
		
		XocXmlParser fileParse = new XocXmlParser();   
		fileParse.parse(input);
		
		durationTime = (System.currentTimeMillis() - startTime)/uint;
		startTime = System.currentTimeMillis();
		RunningTime.writeTimeInformationIntoFile("parse log  end time: " + System.currentTimeMillis()/uint, false, "D:\\timeImportXOCLog.txt");
		RunningTime.writeTimeInformationIntoFile("~~~~~~~~~~~~~~parse log duration time: " + durationTime, false, "D:\\timeImportXOCLog.txt");
		RunningTime.writeTimeInformationIntoFile("process log start time: " + System.currentTimeMillis()/uint, false, "D:\\timeImportXOCLog.txt");
		
        XOCLog log = fileParse.getLog();  
        
        durationTime = (System.currentTimeMillis() - startTime)/uint;
		startTime = System.currentTimeMillis();
		RunningTime.writeTimeInformationIntoFile("process log  end time: " + System.currentTimeMillis()/uint, false, "D:\\timeImportXOCLog.txt");
		RunningTime.writeTimeInformationIntoFile("~~~~~~~~~~~~~~process log duration time: " + durationTime, false, "D:\\timeImportXOCLog.txt");
		
		durationTime = (System.currentTimeMillis() - startTimeTotal)/uint;		
		RunningTime.writeTimeInformationIntoFile("--------------log size: " + log.getAllEvents().size(), false, "D:\\timeImportXOCLog.txt");
		RunningTime.writeTimeInformationIntoFile("--------------total time: " + durationTime, false, "D:\\timeImportXOCLog.txt");
		
		return log;
	}
	
   public static void main (String args[]) throws ClassNotFoundException, IOException {	   		   
	   System.out.print("start read");
 
         
   }
}
