package org.processmining.ocbc.log.io;


import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.List;

import org.deckfour.spex.SXDocument;
import org.deckfour.spex.SXTag;
import org.processmining.ocbc.log.model.XOCEvent;
import org.processmining.ocbc.log.model.XOCLog;
import org.processmining.ocbc.log.model.XOCORelation;
import org.processmining.ocbc.log.model.XOCObject;
import org.processmining.ocbc.log.model.impl.xConstants;



public class OutputXMLFile {
	private String savePath;
	private FileOutputStream out;
	private SXDocument doc;

	public OutputXMLFile(String savePath){
		this.savePath = savePath;
	}
	
	public void serializeEventListTag(List<XOCEvent> eventList, String format, SXTag logTag) throws IOException {	
		System.out.println("event number: " + eventList.size());
		for (XOCEvent event : eventList) {
//			System.out.print("write event:  \n" );
			SXTag eventTag = logTag.addChildNode("event");		
			for(String attributeKey : event.getAttributes().keySet()){
				SXTag eventAttributeTag = eventTag.addChildNode("string");
				eventAttributeTag.addAttribute("key", attributeKey);
//				//process some special cases
//				if(attributeKey.equals(xConstants.TIMESTAMP)){
//					SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
//					String time = sdf.format(event.timestamp);
//					attributeTag.addAttribute("value", time);
//				}
				eventAttributeTag.addAttribute("value", event.getAttributes().get(attributeKey).toString());
			}
	
			SXTag objectModelTag = eventTag.addChildNode("model");
			
			if(format.equals(xConstants.UPDATE)){
				SXTag objectModeObjectsTag = objectModelTag.addChildNode("objects");
				addObjects(objectModeObjectsTag, event.getObjectModelUpdate().getObjectSet());
				
				SXTag objectModeObjectRelationsTag = objectModelTag.addChildNode("relations");
				addObjectRelations(objectModeObjectRelationsTag, event.getObjectModelUpdate().getObjectRelationSet());
			}
			else if(format.equals(xConstants.TOTAL)){
				SXTag objectModeObjectsTag = objectModelTag.addChildNode("objects");
				addObjects(objectModeObjectsTag, event.getObjectModel().getObjectSet());
				
				SXTag objectModeObjectRelationsTag = objectModelTag.addChildNode("relations");
				addObjectRelations(objectModeObjectRelationsTag, event.getObjectModel().getObjectRelationSet());
			}	
			
			//the reference must be behind object model
			SXTag objectsTag = eventTag.addChildNode("references");
			addReferences(objectsTag, event.getSimplifiedRelatedObjects());
//			System.out.println("event index: " + eventList.indexOf(event));
		}
	}
	
	public SXTag serializeLogTag(XOCLog log, String format) throws IOException {	
		
		File file = new File(savePath);  
	    System.out.println("\n" +  savePath);
	    // 路径为文件且不为空则进行删除  
//	    if (file.isFile() && file.exists()) {  
//	    	System.out.println("\n delete");
//	        file.delete();  
//	    } 
	    
		out = new FileOutputStream(savePath);
		long start = System.currentTimeMillis();
		doc = new SXDocument(out);
		doc.addComment("This file is a XOC log. Its definition and semantic are");
		doc.addComment("available from http://www.win.tue.nl/~gmli/ or");
		doc.addComment("https://svn.win.tue.nl/repos/prom/Packages/OCBC/Trunk/tests/testfiles/");
		doc.addComment("It has been generated with the OpenXES library. It conforms");
		doc.addComment("to the XML serialization of the XES standard for log storage and management.");
		doc.addComment("OpenXES is available from http://www.openxes.org/");
		
		SXTag logTag = doc.addNode("log");
		logTag.addAttribute("xoc.version", "1.0");
		for(String attributeKey : log.getAttributes().keySet()){
			SXTag logAttributeTag = logTag.addChildNode("string");
			logAttributeTag.addAttribute("key", attributeKey);
			logAttributeTag.addAttribute("value", log.getAttributes().get(attributeKey).toString());
		}	
		return logTag;
	}
	
	public void closeSerializingLog() throws IOException {
		doc.close();
		out.close();
//		String duration = " (" + (System.currentTimeMillis() - start)
//				+ " msec.)";
	}
	
	public void serialize(XOCLog log, String format) throws IOException {	
		SXTag logTag = serializeLogTag(log, format);
		serializeEventListTag(log, format, logTag);
		closeSerializingLog();
	}
	
	public void addObjects(SXTag tag, List<XOCObject> objects)
			throws IOException {
						
		for (XOCObject object : objects) {
			SXTag objectTag = tag.addChildNode("object");			
			for(String attributeKey : object.getAttributes().keySet()){
				SXTag objectAttributeTag = objectTag.addChildNode("string");
				objectAttributeTag.addAttribute("key", attributeKey);
				objectAttributeTag.addAttribute("value", object.getAttributes().get(attributeKey).toString());	
			}
		}
	}
	
	public void addReferences(SXTag tag, List<XOCObject> objects)
			throws IOException {					
		for (XOCObject object : objects) {
			SXTag objectTag = tag.addChildNode("object");		
			SXTag objectAttributeTag = objectTag.addChildNode("string");
			objectAttributeTag.addAttribute("key", xConstants.ID);
			objectAttributeTag.addAttribute("value", object.getAttributes().get(xConstants.ID).toString());	
		}
	}
	
	public void addObjectRelations(SXTag tag, List<XOCORelation> objectRelations)
			throws IOException {		
		for (XOCORelation objectRelation : objectRelations) {
			SXTag objectRelationTag = tag.addChildNode("relation");
			//in real application			
			for(String attributeName : objectRelation.getAttributes().keySet()){
				SXTag objectRelationAttributeTag = objectRelationTag.addChildNode("string");
				objectRelationAttributeTag.addAttribute("key", attributeName);
				objectRelationAttributeTag.addAttribute("value", objectRelation.getAttributes().get(attributeName).toString());
			}
//			SXTag objectRelationTag = tag.addChildNode("relation");;
//			
//			SXTag objectRelationIDTag = objectRelationTag.addChildNode("string");
//			objectRelationIDTag.addAttribute("key", "id");
//			objectRelationIDTag.addAttribute("value", objectRelation.getAttributes().get(xConstants.ID).toString());
//			
//			SXTag objectRelationTypeTag = objectRelationTag.addChildNode("string");
//			objectRelationTypeTag.addAttribute("key", "relation");
//			objectRelationTypeTag.addAttribute("value", objectRelation.getAttributes().get(xConstants.RELATION).toString());
//			
//			SXTag sourceObjectTag = objectRelationTag.addChildNode("string");
//			sourceObjectTag.addAttribute("key", "sourceobjectid");
////			sourceObjectTag.addAttribute("value", objectRelation.getSourceObject().getAttributes().get(xConstants.ID).toString());
//			sourceObjectTag.addAttribute("value", objectRelation.getAttributes().get(xConstants.SOURCEOBJECTID).toString());
//			
//			SXTag targetObjectTag = objectRelationTag.addChildNode("string");
//			targetObjectTag.addAttribute("key", "targetobjectid");
////			targetObjectTag.addAttribute("value", objectRelation.getTargetObject().getAttributes().get(xConstants.ID).toString());	
//			targetObjectTag.addAttribute("value", objectRelation.getAttributes().get(xConstants.TARGETOBJECTID).toString());
//			
		}
	}
 
}