/*
 * OpenXES
 * 
 * The reference implementation of the XES meta-model for event 
 * log data management.
 * 
 * Copyright (c) 2008 Christian W. Guenther (christian@deckfour.org)
 * 
 * 
 * LICENSE:
 * 
 * This code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 * 
 * EXEMPTION:
 * 
 * The use of this software can also be conditionally licensed for
 * other programs, which do not satisfy the specified conditions. This
 * requires an exemption from the general license, which may be
 * granted on a per-case basis.
 * 
 * If you want to license the use of this software with a program
 * incompatible with the LGPL, please contact the author for an
 * exemption at the following email address: 
 * christian@deckfour.org
 * 
 */
package org.processmining.ocbc.log.model;

import java.util.List;

import org.processmining.ocbc.models.MyOriginalElement;



/**
 * An event is an element of an XES event log structure. Events are sequentially
 * contained in traces.
 * 
 * Events refer to something that has happened during the execution of a
 * process, e.g. the execution of an activity.
 * 
 * @author Christian W. Guenther (christian@deckfour.org)
 * 
 */
public interface XOCEvent extends MyOriginalElement, XOCElement {

	/**
	 * Returns the id of the event
	 * 
	 * @return the ID of this event
	 */
//	public XID getID();
	
	public String getActivity();
	
	public XAttributeMap getAttributes();
	
	public List<XOCObject> getRelatedObjects();
	
	/**
	 * Get the list of reference relations of the event (e.g., used in the object model graph to refer to the corresponding cell of one reference relation)
	 * @return
	 */
	public List<XOCRRelation> getReferenceRelationList();
	
	/**
	 * Get the reference relation connecting the event and the input object
	 * @param object
	 * @return
	 */
	public XOCRRelation getReferenceRelation(XOCObject object);
	
	//each related object just contain one id attribute
	public List<XOCObject> getSimplifiedRelatedObjects();
	
	//get all related objects for one object
	public List<XOCObject> getRelevantObjectsForOneObject(XOCObject object);
	
	//get all related object relations for one object
	public List<XOCORelation> getRelevantObjectRelationsForOneObject(XOCObject object);
	
	/**
	 * return the index of one event
	 * note that the orderNum of the first event is 1; N+
	 */
	public int getOrderNum();
	
	public XOCObjectModel getObjectModel();
	
	public XOCObjectModel getObjectModelUpdate();
	
	public XOCObjectModel getMinimalObjectModel();
	
//	public void setID(XID ID);
	
	public void setActivity(String activity);
	
	public void setAttributes(XAttributeMap attributes);
	
	public void setRelatedObjects(List<XOCObject> relatedObjects);
	
	public void setSimplifiedRelatedObjects(List<XOCObject> relatedSimplifiedObjects);
	
	public void setOrderNum(int orderNum);
	
	public void setObjectModel(XOCObjectModel objectModel);
	
	public void setObjectModelUpdate(XOCObjectModel objectModelUpdate);
	
	public void setMinimalObjectModel(XOCObjectModel minimalObjectModel);
	
	public boolean relatedObjectsContain(XOCObject object);

	public boolean objectModelContain(XOCObject relatedObject);
	
	public Object clone();
}