/*
 * OpenXES
 * 
 * The reference implementation of the XES meta-model for event 
 * log data management.
 * 
 * Copyright (c) 2008 Christian W. Guenther (christian@deckfour.org)
 * 
 * 
 * LICENSE:
 * 
 * This code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 * 
 * EXEMPTION:
 * 
 * The use of this software can also be conditionally licensed for
 * other programs, which do not satisfy the specified conditions. This
 * requires an exemption from the general license, which may be
 * granted on a per-case basis.
 * 
 * If you want to license the use of this software with a program
 * incompatible with the LGPL, please contact the author for an
 * exemption at the following email address: 
 * christian@deckfour.org
 * 
 */
package org.processmining.ocbc.log.model;

import java.util.List;
import java.util.Map;

import org.processmining.common.structure.IntegerPair;
import org.processmining.ocbc.models.MyOriginalElement;



/**
 * An event is an element of an XES event log structure. Events are sequentially
 * contained in traces.
 * 
 * Events refer to something that has happened during the execution of a
 * process, e.g. the execution of an activity.
 * 
 * @author Christian W. Guenther (christian@deckfour.org)
 * 
 */
public interface XOCORelation extends MyOriginalElement, XOCElement {

	/**
	 * Returns the id of the event
	 * 
	 * @return the ID of this event
	 */
//	XID getID();

	public String getRelationType();
	
	public XOCObject getSourceObject();
	
	public XOCObject getTargetObject();
	
//	public String getBirthEventId();
//	
//	public String getDeathEventId();
//	
//	public XOCEvent getBirthEvent();
//	
//	public XOCEvent getDeathEvent();
//	
//	public void setBirthEventId(String birthEventId);
//	
//	public void setDeathEventId(String deathEventId);
//	
//	public void setBirthEvent(XOCEvent birthEvent);
//	
//	public void setDeathEvent(XOCEvent deathEvent);
	
	public void setRelationType(String relationType);
	
	public void setSourceObject(XOCObject sourceObject);
	
	public void setTargetObject(XOCObject targetObject);
	
	public boolean compareObjectRelation(XOCORelation objectRelation);
	
	public boolean isAttributesEqual(XOCORelation objectRelation);
	
	/**
	 * get the lifecycle period list of one object;
	 * each period consists of two values: start event order number and end event order number;
	 * the object still (maybe last) observed in the object model of the end event;
	 * @return
	 */
	public List<IntegerPair> getLifecyclePeriodList();
	
	/**
	 * check if the object relation is alive at the moment of input event based on the life cycle period list
	 * @param event
	 * @return
	 */
	public boolean isAliveAtEvent(XOCEvent event);
	
	/**
	 * check if the object relation is alive at the moment of input event order number based on the life cycle period list
	 * @param event
	 * @return
	 */
	public boolean isAliveAtEventOrderNum(int eventOrderNum);
	
	/**
	 * check if the object relation is added at the moment of input event based on the life cycle period list
	 * if one object relation is alive at current event and dead at previous event, the object relation is added at current event
	 * @param event
	 * @return
	 */
	public boolean isAddedAtEvent(XOCEvent event);
	
	/**
	 * get the map between event order number and object relation state;
	 * if one event number is not in the map, it means (1) the object relation is not alive (after delete && before add if any), or 
	 * (2) remain the same (state value: remain) after add or update && before delete (if any)
	 * @return
	 */
	public Map<Integer, String> getEventOrderNumVSObjectRelationStateMap();
}