/*
 * OpenXES
 * 
 * The reference implementation of the XES meta-model for event 
 * log data management.
 * 
 * Copyright (c) 2008 Christian W. Guenther (christian@deckfour.org)
 * 
 * 
 * LICENSE:
 * 
 * This code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 * 
 * EXEMPTION:
 * 
 * The use of this software can also be conditionally licensed for
 * other programs, which do not satisfy the specified conditions. This
 * requires an exemption from the general license, which may be
 * granted on a per-case basis.
 * 
 * If you want to license the use of this software with a program
 * incompatible with the LGPL, please contact the author for an
 * exemption at the following email address: 
 * christian@deckfour.org
 * 
 */
package org.processmining.ocbc.log.model;

import java.util.List;
import java.util.Map;


/**
 * A trace is an element of an XES event log structure.
 * Traces are contained in logs.
 * Any trace is a list of events.
 * 
 * Traces describe sequences of events, as they have occurred
 * during one execution of a process, in their given order.
 * 
 * @author Christian W. Guenther (christian@deckfour.org)
 *
 */
public interface XOCObjectModel extends XOCElement{
	
	public List<XOCObject> getObjectSet();
	
	public  List<XOCORelation> getObjectRelationSet();
	
	public void setObjectSet(List<XOCObject> objectSet);
	
	public  void setObjectRelationSet(List<XOCORelation> objectRelationSet);
	
	//get all related (directly and indirectly) objects for one input object
	//i.e., it recall DirectlyRelatedObjectsForOneObject for the input objects and its child related objects (recursion)
	public List<XOCObject> getAllRelatedObjectsForOneObject(String objectId);
	
	//get all directly related objects for one input object
	//i.e., there exist object relations between the related objects and the input object
	public List<XOCObject> getDirectlyRelatedObjectsForOneObject(String objectId);
	
	public Object clone();
	
	//make sure the source/target objects of relations are the same objects (addresses) with those in the object set
	public void updateObjectRelationSet();
	
	//from this object model (the father instance)
	public List<XOCObject> getObjectsOfOneClass(String className);
	
	//from this object model (the father instance), do not make difference between source and target nodes
	public List<XOCORelation> getObjectRelationsOfTwoClasses(String firstClassName, String secondClassName);
	
	public boolean contain(XOCObjectModel model);
	
	public boolean equalTo(XOCObjectModel model);
	
	public XOCObject getObjectById(String id);
	
	public List<XOCObject> computeMore(XOCObjectModel model);
	
	public List<XOCObject> computeLess(XOCObjectModel model);

	public boolean checkTwoObjectsIfRelated(String firstObjectId, String secondObjectId);

	public XOCObjectModel minusObjectModelOverlap(XOCObjectModel model);
	
	//generate the map between (object) ids and objects
	public Map<String, XOCObject> generateIdVSObjectMap();
	
	public void setIdVSObjectMap(Map<String, XOCObject> idVSObjectMap);
	
	//get the map between (object) ids and objects
	public Map<String, XOCObject> getIdVSObjectMap();
	
	//get the map between (object) ids and objects
	public Map<String, XOCORelation> getIdVSObjectRelationMap();
	
	public XOCObjectModel plusObjectModel(XOCObjectModel inputModel);
}