/*
 * OpenXES
 * 
 * The reference implementation of the XES meta-model for event 
 * log data management.
 * 
 * Copyright (c) 2008 Christian W. Guenther (christian@deckfour.org)
 * 
 * 
 * LICENSE:
 * 
 * This code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 * 
 * EXEMPTION:
 * 
 * The use of this software can also be conditionally licensed for
 * other programs, which do not satisfy the specified conditions. This
 * requires an exemption from the general license, which may be
 * granted on a per-case basis.
 * 
 * If you want to license the use of this software with a program
 * incompatible with the LGPL, please contact the author for an
 * exemption at the following email address: 
 * christian@deckfour.org
 * 
 */
package org.processmining.ocbc.log.model.impl;

import java.util.Set;

import org.processmining.ocbc.log.extension.XExtension;
import org.processmining.ocbc.log.id.XID;
import org.processmining.ocbc.log.id.XIDFactory;
import org.processmining.ocbc.log.model.XAttributeMap;
import org.processmining.ocbc.log.model.XOCCRelation;
import org.processmining.ocbc.log.model.XOCClass;



//import org.deckfour.xes.extension.XExtension;
//import org.deckfour.xes.id.XID;
//import org.deckfour.xes.id.XIDFactory;
//import org.deckfour.xes.model.XAttribute;
//import org.deckfour.xes.model.XAttributeMap;
//import org.deckfour.xes.model.XEvent;
//import org.deckfour.xes.model.XTrace;
//import org.deckfour.xes.model.XVisitor;
//import org.deckfour.xes.util.XAttributeUtils;

/**
 * Implementation for the XEvent interface.
 * 
 * @author Christian W. Guenther (christian@deckfour.org)
 * 
 */
public class XOCCRelationImpl implements XOCCRelation {

	/**
	 * ID of thi event.
	 */
	private XID ID;
	private String relationType;
	private XOCClass sourceClass;
	private XOCClass targetClass;
	private XAttributeMap attributes;
	

	public XOCCRelationImpl(String relationType, XOCClass sourceClass, XOCClass targetClass) {
		this(new XID(), null, relationType, sourceClass, targetClass);
	}
	

	public XOCCRelationImpl(XID ID, XAttributeMap attributes, String relationType, XOCClass sourceClass, XOCClass targetClass) {
		this.ID = ID;
		this.attributes = attributes;
		this.relationType = relationType;
		this.sourceClass = sourceClass;
		this.targetClass = targetClass;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.deckfour.xes.model.XAttributable#getAttributes()
	 */
	public XAttributeMap getAttributes() {
		return attributes;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.deckfour.xes.model.XAttributable#setAttributes(java.util.Map)
	 */
	public void setAttributes(XAttributeMap attributes) {
		this.attributes = attributes;
	}

	/* (non-Javadoc)
	 * @see org.deckfour.xes.model.XAttributable#hasAttributes()
	 */
	@Override
	public boolean hasAttributes() {
		return !attributes.isEmpty();
	}
	
//	/*
//	 * (non-Javadoc)
//	 * 
//	 * @see org.deckfour.xes.model.XAttributable#getExtensions()
//	 */
//	public Set<XExtension> getExtensions() {
//		return XAttributeUtils.extractExtensions(attributes);
//	}

	/**
	 * Clones this event, i.e. creates a deep copy, but with a new ID, so equals
	 * does not hold between this and the clone
	 */
	public Object clone() {
		XOCCRelationImpl clone;
		try {
			clone = (XOCCRelationImpl) super.clone();
		} catch (CloneNotSupportedException e) {
			e.printStackTrace();
			return null;
		}
		clone.ID = XIDFactory.instance().createId();
		clone.attributes = (XAttributeMap) attributes.clone();
		return clone;
	}

	/**
	 * Tests for equality of IDs
	 */
	public boolean equals(Object o) {
		if (o instanceof XOCCRelationImpl) {
			return ((XOCCRelationImpl) o).ID.equals(ID);
		} else {
			return false;
		}
	}

	/**
	 * Returns the hashCode of the id
	 */
	public int hashCode() {
		return ID.hashCode();
	}

	/*
	 * (non-Javadoc)
	 * @see org.deckfour.xes.model.XEvent#getID()
	 */
	public XID getID() {
		return ID;
	}

	/**
	 * Sets the ID. Should only be used for deserialization purposes
	 * 
	 * @param id
	 *            the new id.
	 */
	public void setID(XID ID) {
		this.ID = ID;
	}

	public Set<XExtension> getExtensions() {
		// TODO Auto-generated method stub
		return null;
	}


	public String getRelationType() {
		return relationType;
	}


	public XOCClass getSourceClass() {
		return sourceClass;
	}


	public XOCClass getTargetClass() {
		return targetClass;
	}


	public void setRelationType(String relationType) {
		this.relationType = relationType;
		
	}


	public void setSourceClass(XOCClass sourceClass) {
		this.sourceClass = sourceClass;
		
	}


	public void setTargetClass(XOCClass targetClass) {
		this.targetClass = targetClass;
		
	}

}
