package org.processmining.ocbc.log.model.impl;

import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.List;

import org.deckfour.spex.SXDocument;
import org.deckfour.spex.SXTag;
import org.deckfour.xes.logging.XLogging;
import org.deckfour.xes.util.XRuntimeUtils;
import org.processmining.ocbc.log.model.XOCEvent;
import org.processmining.ocbc.log.model.XOCLog;
import org.processmining.ocbc.log.model.XOCORelation;
import org.processmining.ocbc.log.model.XOCObject;
import org.processmining.ocbc.log.model.XOCObjectModel;


/**
 * Implementation for the XLog interface.
 * 
 * @author Christian W. Guenther (christian@deckfour.org)
 *
 */
public class XOCCreateExampleLog{

//	public static void main(String[] str) throws IOException{
	public static XOCLog createLog() throws IOException {
		
		FileOutputStream fout = new FileOutputStream("E:\\outputLog2.xoc");
		
		XOCLog exampleLog = new XOCLogImpl();
		int[][] array = {{1,2},{3,4}};
		
	String[] eventNames = {"create order", "create order", "create order", "pick item", "wrap item",
						  "pick item", "pick item", "deliver items", "wrap item", "wrap item",
						  "deliver items", "pick item", "wrap item", "pick item", "wrap item",
						  "pick item", "wrap item", "deliver items"};	
	String[][] relatedObjects = {{"o1", "order"}, {"o2", "order"}, {"o3", "order"}, {"l6", "order line"}, {"l6", "order line"},
								 {"l1", "order line"}, {"l2", "order line"}, {"d3", "delivery"}, {"l1", "order line"}, {"l2", "order line"},
								 {"d1", "delivery"}, {"l3", "order line"}, {"l3", "order line"}, {"l5", "order line"}, {"l5", "order line"},
								 {"l4", "order line"}, {"l4", "order line"}, {"d2", "delivery"}};	
	
		XOCObjectModel objectModel = new XOCObjectModelImpl();
			
		
		XOCObject objectO1 = new XOCObjectImpl("o1", new XOCClassImpl("order"));//event 1
		XOCObject objectL1 = new XOCObjectImpl("l1", new XOCClassImpl("order line"));
		XOCObject objectL2 = new XOCObjectImpl("l2", new XOCClassImpl("order line"));
		XOCObject objectL3 = new XOCObjectImpl("l3", new XOCClassImpl("order line"));		
		XOCObject objectC1 = new XOCObjectImpl("c1", new XOCClassImpl("customer"));
		XOCORelation objectRelationL1 = new XOCORelationImpl("order-order line", objectO1, objectL1);
		XOCORelation objectRelationL2 = new XOCORelationImpl("order-order line", objectO1, objectL2);
		XOCORelation objectRelationL3 = new XOCORelationImpl("order-order line", objectO1, objectL3);
		XOCORelation objectRelationC1 = new XOCORelationImpl("order-customer", objectO1, objectC1);
		
		XOCObject objectO2 = new XOCObjectImpl("o2", new XOCClassImpl("order"));//event 2
		XOCObject objectL4 = new XOCObjectImpl("l4", new XOCClassImpl("order line"));
		XOCObject objectL5 = new XOCObjectImpl("l5", new XOCClassImpl("order line"));	
		XOCORelation objectRelationL4 = new XOCORelationImpl("order-order line", objectO2, objectL4);
		XOCORelation objectRelationL5 = new XOCORelationImpl("order-order line", objectO2, objectL5);
		XOCORelation objectRelationO2C1 = new XOCORelationImpl("order-customer", objectO2, objectC1);
		
		XOCObject objectO3 = new XOCObjectImpl("o3", new XOCClassImpl("order"));//event 3		
		XOCObject objectL6 = new XOCObjectImpl("l6", new XOCClassImpl("order line"));
		XOCObject objectC2 = new XOCObjectImpl("c2", new XOCClassImpl("customer"));	
		XOCORelation objectRelationL6 = new XOCORelationImpl("order-order line", objectO3, objectL6);
		XOCORelation objectRelationC2 = new XOCORelationImpl("order-customer", objectO3, objectC2);
		
		XOCObject objectD3 = new XOCObjectImpl("d3", new XOCClassImpl("delivery"));//event 8
		XOCORelation objectRelationD3L6 = new XOCORelationImpl("delivery-order line", objectD3, objectL6);
		XOCORelation objectRelationD3C2 = new XOCORelationImpl("delivery-customer", objectD3, objectC2);
		
		XOCObject objectD1 = new XOCObjectImpl("d1", new XOCClassImpl("delivery"));//event 11
		XOCORelation objectRelationD1L1 = new XOCORelationImpl("delivery-order line", objectD1, objectL1);
		XOCORelation objectRelationD1L2 = new XOCORelationImpl("delivery-order line", objectD1, objectL2);
		XOCORelation objectRelationD1C1 = new XOCORelationImpl("delivery-customer", objectD1, objectC1);
		
		XOCObject objectD2 = new XOCObjectImpl("d2", new XOCClassImpl("delivery"));//event 18
		XOCORelation objectRelationD2L3 = new XOCORelationImpl("delivery-order line", objectD2, objectL3);
		XOCORelation objectRelationD2L4 = new XOCORelationImpl("delivery-order line", objectD2, objectL4);
		XOCORelation objectRelationD2L5 = new XOCORelationImpl("delivery-order line", objectD2, objectL5);
		XOCORelation objectRelationD2C1 = new XOCORelationImpl("delivery-customer", objectD2, objectC1);
		
		for(int i=1; i<=18; i++)
		{
			XOCEvent event = new XOCEventImpl(eventNames[i-1], i);		
			switch (i)
			{
				case 1: 			
					event.getRelatedObjects().add(objectO1);
					
					objectModel.getObjectSet().add(objectO1);
					objectModel.getObjectSet().add(objectL1);
					objectModel.getObjectSet().add(objectL2);
					objectModel.getObjectSet().add(objectL3);
					objectModel.getObjectSet().add(objectC1);
										
					objectModel.getObjectRelationSet().add(objectRelationL1);
					objectModel.getObjectRelationSet().add(objectRelationL2);
					objectModel.getObjectRelationSet().add(objectRelationL3);
					objectModel.getObjectRelationSet().add(objectRelationC1);
				break;
				
				case 2:			
					event.getRelatedObjects().add(objectO2);
					
					objectModel.getObjectSet().add(objectO2);
					objectModel.getObjectSet().add(objectL4);
					objectModel.getObjectSet().add(objectL5);

					objectModel.getObjectRelationSet().add(objectRelationL4);
					objectModel.getObjectRelationSet().add(objectRelationL5);
					objectModel.getObjectRelationSet().add(objectRelationO2C1);
					break;
					
				case 3:				
					event.getRelatedObjects().add(objectO3);
					
					objectModel.getObjectSet().add(objectO3);
					objectModel.getObjectSet().add(objectL6);
					objectModel.getObjectSet().add(objectC2);

					objectModel.getObjectRelationSet().add(objectRelationL6);
					objectModel.getObjectRelationSet().add(objectRelationC2);
					break;
					
				case 4:
					event.getRelatedObjects().add(objectL6);
					break;
				case 5:
					event.getRelatedObjects().add(objectL6);
					break;
				case 6:
					event.getRelatedObjects().add(objectL1);
					break;
				case 7:
					event.getRelatedObjects().add(objectL2);
					break;
					
				case 8:	
					event.getRelatedObjects().add(objectD3);
					
					objectModel.getObjectSet().add(objectD3);
				
					objectModel.getObjectRelationSet().add(objectRelationD3L6);
					objectModel.getObjectRelationSet().add(objectRelationD3C2);
					break;
					
				case 9:
					event.getRelatedObjects().add(objectL1);
					break;
				case 10:
					event.getRelatedObjects().add(objectL2);
					break;
				case 11:	
					event.getRelatedObjects().add(objectD1);
					
					objectModel.getObjectSet().add(objectD1);
				
					objectModel.getObjectRelationSet().add(objectRelationD1L1);
					objectModel.getObjectRelationSet().add(objectRelationD1L2);
					objectModel.getObjectRelationSet().add(objectRelationD1C1 );
					break;
				case 12:
					event.getRelatedObjects().add(objectL3);
					break;
				case 13:
					event.getRelatedObjects().add(objectL3);
					break;
				case 14:
					event.getRelatedObjects().add(objectL5);
					break;
				case 15:
					event.getRelatedObjects().add(objectL5);
					break;
				case 16:
					event.getRelatedObjects().add(objectL4);
					break;
				case 17:
					event.getRelatedObjects().add(objectL4);
					break;
				case 18:				
					event.getRelatedObjects().add(objectD2);
					
					objectModel.getObjectSet().add(objectD2);
				
					objectModel.getObjectRelationSet().add(objectRelationD2L3);
					objectModel.getObjectRelationSet().add(objectRelationD2L4);
					objectModel.getObjectRelationSet().add(objectRelationD2L5);
					objectModel.getObjectRelationSet().add(objectRelationD2C1);
										
					break;

			}
			
			
			event.setObjectModel((XOCObjectModel) objectModel.clone());
			exampleLog.add(event);
//			exampleLog.getAllEvents().add(event);
			
			
		}
		System.out.print(exampleLog.size());
		
		System.out.print("exampleLog.getAllEvents().size()" + exampleLog.getAllEvents().size());
		System.out.print("\n");

		for(XOCEvent event : exampleLog.getAllEvents())
		{
			System.out.print(event.getActivity() + "order" + event.getOrderNum());
			System.out.print("\n");
			
			for(XOCObject relatedObject: event.getRelatedObjects())
			{
				System.out.print("references:");
				System.out.print(relatedObject.getName());
				System.out.print("\n");
			}
			for(XOCObject modelObject: event.getObjectModel().getObjectSet())
			{
				System.out.print("objects:");
				System.out.print(modelObject.getName());
				System.out.print("\n");
			}
			
			for(XOCORelation objectRelation: event.getObjectModel().getObjectRelationSet())
			{
				System.out.print("relations:");
				System.out.print(objectRelation.getRelationType() + objectRelation.getSourceObject().getName() + objectRelation.getTargetObject().getName());
				System.out.print("\n");
			}
			
			System.out.print("next event:  \n" );
		}
		System.out.print("exampleLog:  \n" );
		serialize(exampleLog, fout);
		return exampleLog;
	}
	
	public static void serialize(XOCLog log, OutputStream out) throws IOException {
		XLogging.log("start serializing log to XES.XML",
				XLogging.Importance.DEBUG);
		long start = System.currentTimeMillis();
		SXDocument doc = new SXDocument(out);
		doc.addComment("This file has been generated with the OpenXES library. It conforms");
		doc.addComment("to the XML serialization of the XES standard for log storage and");
		doc.addComment("management.");
		doc.addComment("XES standard version: " + XRuntimeUtils.XES_VERSION);
		doc.addComment("OpenXES library version: "
				+ XRuntimeUtils.OPENXES_VERSION);
		doc.addComment("OpenXES is available from http://www.openxes.org/");
		SXTag logTag = doc.addNode("log");
		logTag.addAttribute("xes.version", XRuntimeUtils.XES_VERSION);
		logTag.addAttribute("xes.features", "nested-attributes");
		logTag.addAttribute("openxes.version", XRuntimeUtils.OPENXES_VERSION);
		
		for (XOCEvent event : log.getAllEvents()) {
		System.out.print("write event:  \n" );
		SXTag eventTag = logTag.addChildNode("event");
		SXTag attributeTag;
		attributeTag = eventTag.addChildNode("string");
		attributeTag.addAttribute("key", "concept:name");
		attributeTag.addAttribute("value", event.getActivity());
		
		attributeTag = eventTag.addChildNode("string");
		attributeTag.addAttribute("key", "event order");
		attributeTag.addAttribute("value", event.getOrderNum());
		
		SXTag objectsTag = eventTag.addChildNode("references");
		addObjects(objectsTag, event.getRelatedObjects());
		
		SXTag objectModelTag = eventTag.addChildNode("model");
		
		SXTag objectModeObjectsTag = objectModelTag.addChildNode("objects");
		addObjects(objectModeObjectsTag, event.getObjectModel().getObjectSet());
		
		SXTag objectModeObjectRelationsTag = objectModelTag.addChildNode("relations");
		addObjectRelations(objectModeObjectRelationsTag, event.getObjectModel().getObjectRelationSet());
//		addAttributes(traceTag, event.getActivity());
//		for (XEvent event : trace) {
//			SXTag eventTag = eventTag.addChildNode("related objects");
//			addAttributes(eventTag, event.getAttributes().values());
//		}
	}
		
		//
		doc.close();
		String duration = " (" + (System.currentTimeMillis() - start)
				+ " msec.)";
		XLogging.log("finished serializing log" + duration,
				XLogging.Importance.DEBUG);
	}
	
	protected static void addObjects(SXTag tag, List<XOCObject> objects)
			throws IOException {
						
		for (XOCObject object : objects) {
			//in real application			
//			for(XAttribute attribute : object.getAttributes()){
//				SXTag objectAttributeTag = objectTag.addChildNode("value.type");
//				objectAttributeTag.addAttribute("key", "concept:name");
//				objectAttributeTag.addAttribute("value", object.getName());
//			}
			SXTag objectTag = tag.addChildNode("object");
			
			SXTag objectIDTag = objectTag.addChildNode("string");
			objectIDTag.addAttribute("key", "ID");
			objectIDTag.addAttribute("value", object.getAttributes().get("id").toString());
			
			SXTag objectNameTag = objectTag.addChildNode("string");
			objectNameTag.addAttribute("key", "concept:name");
			objectNameTag.addAttribute("value", object.getName());
			
			SXTag objectClassTag = objectTag.addChildNode("string");
			objectClassTag.addAttribute("key", "class");
			objectClassTag.addAttribute("value", object.getAttributes().get(xConstants.CLASS).toString());
				
		}
	}
	
	protected static void addObjectRelations(SXTag tag, List<XOCORelation> objectRelations)
			throws IOException {		
		for (XOCORelation objectRelation : objectRelations) {
			//in real application			
//			for(XAttribute attribute : objectRelation.getAttributes()){
//				SXTag objectAttributeTag = objectTag.addChildNode("value.type");
//				objectAttributeTag.addAttribute("key", "concept:name");
//				objectAttributeTag.addAttribute("value", object.getName());
//			}
			SXTag objectRelationTag = tag.addChildNode("relation");;
			
			SXTag objectRelationIDTag = objectRelationTag.addChildNode("string");
			objectRelationIDTag.addAttribute("key", "ID");
			objectRelationIDTag.addAttribute("value", objectRelation.getAttributes().get(xConstants.ID).toString());
			
			SXTag objectRelationTypeTag = objectRelationTag.addChildNode("string");
			objectRelationTypeTag.addAttribute("key", "relation:type");
			objectRelationTypeTag.addAttribute("value", objectRelation.getRelationType());
			
			SXTag sourceObjectTag = objectRelationTag.addChildNode("string");
			sourceObjectTag.addAttribute("key", "source object");
			sourceObjectTag.addAttribute("value", objectRelation.getSourceObject().getAttributes().get("id").toString());
			
			SXTag targetObjectTag = objectRelationTag.addChildNode("string");
			targetObjectTag.addAttribute("key", "target object");
			targetObjectTag.addAttribute("value", objectRelation.getTargetObject().getAttributes().get("id").toString());
				
		}
	}

//	public static XOCLog createLog() {
//		// TODO Auto-generated method stub
//		return null;
//	}
	
//	protected void addAttributes(SXTag tag, Collection<XAttribute> attributes)
//			throws IOException {
//		for (XAttribute attribute : attributes) {
//			SXTag attributeTag;
//			if (attribute instanceof XAttributeList) {
//				attributeTag = tag.addChildNode("list");
//				attributeTag.addAttribute("key", attribute.getKey());
//			} else if (attribute instanceof XAttributeContainer) {
//				attributeTag = tag.addChildNode("container");
//				attributeTag.addAttribute("key", attribute.getKey());
//			} else if (attribute instanceof XAttributeLiteral) {
//				attributeTag = tag.addChildNode("string");
//				attributeTag.addAttribute("key", attribute.getKey());
//				attributeTag.addAttribute("value", attribute.toString());
//			} else if (attribute instanceof XAttributeDiscrete) {
//				attributeTag = tag.addChildNode("int");
//				attributeTag.addAttribute("key", attribute.getKey());
//				attributeTag.addAttribute("value", attribute.toString());
//			} else if (attribute instanceof XAttributeContinuous) {
//				attributeTag = tag.addChildNode("float");
//				attributeTag.addAttribute("key", attribute.getKey());
//				attributeTag.addAttribute("value", attribute.toString());
//			} else if (attribute instanceof XAttributeBoolean) {
//				attributeTag = tag.addChildNode("boolean");
//				attributeTag.addAttribute("key", attribute.getKey());
//				attributeTag.addAttribute("value", attribute.toString());
//			} else if (attribute instanceof XAttributeID) {
//				attributeTag = tag.addChildNode("id");
//				attributeTag.addAttribute("key", attribute.getKey());
//				attributeTag.addAttribute("value", attribute.toString());
//			} else {
//				throw new IOException("Unknown attribute type!");
//			}
//			if (attribute instanceof XAttributeCollection) {
//				/*
//				 * Use order as specified by the collection.
//				 */
//				Collection<XAttribute> childAttributes = ((XAttributeCollection) attribute).getCollection();
//				addAttributes(attributeTag, childAttributes);
//			} else {
//				// FM: First check for existence of nested-attributes 
//				//     to avoid unnecessary creation of XAttributeMapLazyImpl instances
//				if (attribute.hasAttributes()) {
//					addAttributes(attributeTag, attribute.getAttributes().values());
//				}
//			}
//		}
//	}

}
