/*
 * OpenXES
 * 
 * The reference implementation of the XES meta-model for event 
 * log data management.
 * 
 * Copyright (c) 2008 Christian W. Guenther (christian@deckfour.org)
 * 
 * 
 * LICENSE:
 * 
 * This code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 * 
 * EXEMPTION:
 * 
 * The use of this software can also be conditionally licensed for
 * other programs, which do not satisfy the specified conditions. This
 * requires an exemption from the general license, which may be
 * granted on a per-case basis.
 * 
 * If you want to license the use of this software with a program
 * incompatible with the LGPL, please contact the author for an
 * exemption at the following email address: 
 * christian@deckfour.org
 * 
 */
package org.processmining.ocbc.log.model.impl;

import java.util.ArrayList;
import java.util.List;

import org.processmining.ocbc.log.model.XOCFilterFunction;
import org.processmining.ocbc.log.model.XOCORelation;
import org.processmining.ocbc.log.model.XOCObject;

/**
 * Implementation for the XLog interface.
 * 
 * @author Christian W. Guenther (christian@deckfour.org)
 *
 */
public class XOCFilterFunctionImpl implements XOCFilterFunction {

	public List<XOCObject> getObjectsOfOneClassFromInputObjectList(String className, List<XOCObject> inputObjectLsit){
		List<XOCObject> objectsOfOneClass = new ArrayList<XOCObject>();
		for(XOCObject object : inputObjectLsit){
			if(object.getAttributes().get("class").toString().equals(className)){
				objectsOfOneClass.add(object);
			}
		}
		return objectsOfOneClass;
	}
	
	public List<XOCORelation> getObjectRelationsOfTwoClassesFromInputRelationList(String firstClassName, 
			String secondClassName, List<XOCORelation> inputObjectRelationLsit){
		List<XOCORelation> objectRelationsOfTwoClasses = new ArrayList<XOCORelation>();
		for(XOCORelation objectRelation : inputObjectRelationLsit){
			XOCObject sourceObject = objectRelation.getSourceObject();
			XOCObject targetObject = objectRelation.getTargetObject();
			
			if(sourceObject.getAttributes().get("class").toString().equals(firstClassName) && 
					targetObject.getAttributes().get("class").toString().equals(secondClassName)){
				objectRelationsOfTwoClasses.add(objectRelation);
			}
			else if(targetObject.getAttributes().get("class").toString().equals(firstClassName) && 
					sourceObject.getAttributes().get("class").toString().equals(secondClassName)){
				objectRelationsOfTwoClasses.add(objectRelation);
			}
		}
		return objectRelationsOfTwoClasses;
	}


}
