/*
 * OpenXES
 * 
 * The reference implementation of the XES meta-model for event 
 * log data management.
 * 
 * Copyright (c) 2008 Christian W. Guenther (christian@deckfour.org)
 * 
 * 
 * LICENSE:
 * 
 * This code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 * 
 * EXEMPTION:
 * 
 * The use of this software can also be conditionally licensed for
 * other programs, which do not satisfy the specified conditions. This
 * requires an exemption from the general license, which may be
 * granted on a per-case basis.
 * 
 * If you want to license the use of this software with a program
 * incompatible with the LGPL, please contact the author for an
 * exemption at the following email address: 
 * christian@deckfour.org
 * 
 */
package org.processmining.ocbc.log.model.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.processmining.ocbc.log.extension.XExtension;
import org.processmining.ocbc.log.model.XAttributeMap;
import org.processmining.ocbc.log.model.XOCORelation;
import org.processmining.ocbc.log.model.XOCObject;
import org.processmining.ocbc.log.model.XOCObjectModel;

/**
 * Memory-based implementation for the XTrace interface.
 * 
 * @author Christian W. Guenther (christian@deckfour.org)
 * 
 */
public class XOCObjectModelImpl  implements XOCObjectModel {
	/**
	 * The class function is embedded in object as the class attribute.
	 * Otherwise, we can build a map, e.g., Map<XOCObject, XOCClass>
	 */
	private XAttributeMap attributes;//current the attributes is not used, i.e., null
	private List<XOCObject> objectSet;
	private List<XOCORelation> objectRelationSet;
	private Map<String, XOCObject> idVSObjectMap;
	public XOCObjectModelImpl() {	
		this(new ArrayList<XOCObject>(), new ArrayList<XOCORelation>(), null);
	}
	
	public XOCObjectModelImpl(List<XOCObject> objectSet,List<XOCORelation> objectRelationSet) {	
		this(objectSet, objectRelationSet, null);
	}

	public XOCObjectModelImpl(List<XOCObject> objectSet,List<XOCORelation> objectRelationSet, XAttributeMap attributes) {	
		this.objectSet = objectSet;
		this.objectRelationSet = objectRelationSet;
		this.attributes = attributes;
	}


	public List<XOCObject> getObjectSet() {
		return objectSet;
	}
	
	public XOCObject getObjectById(String id){
		XOCObject targetObject = null;
		for(XOCObject object : objectSet){
			if(object.getAttributes().get("id").toString().equals(id)){
				targetObject = object;
				break;
			}
		}
		return targetObject;
	}
	
	public  List<XOCORelation> getObjectRelationSet() {
		return objectRelationSet;
	}
	
	public void setObjectSet(List<XOCObject> objectSet) {
		this.objectSet = objectSet;
	}
	
	public  void setObjectRelationSet(List<XOCORelation> objectRelationSet) {
		this.objectRelationSet = objectRelationSet;
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * add object or objectRelation
	 */
	
	public  void addObject(XOCObject object) {
		objectSet.add(object);
	}
	
	public  void addObjectRelation(XOCORelation objectRelation) {
		objectRelationSet.add(objectRelation);
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * remove object or objectRelation by index
	 */
	
	public  void removeObjectByIndex(int i) {
		objectSet.remove(i);
	}
	
	public  void removeObjectRelationByIndex(int i) { 
		objectRelationSet.remove(i);
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * remove object or objectRelation by "object"
	 */
	
	public  void removeObject(XOCObject object) {
		objectSet.remove(object);
	}
	
	public  void removeObjectRelationB(XOCORelation objectRelation) { 
		objectRelationSet.remove(objectRelation);
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.deckfour.xes.model.XAttributable#getExtensions()
	 */
//	public Set<XExtension> getExtensions() {
//		return XAttributeUtils.extractExtensions(attributes);
//	}

	public XAttributeMap getAttributes() {
		// TODO Auto-generated method stub
		return attributes;
	}

	
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.deckfour.xes.model.XAttributable#setAttributes(java.util.Map)
	 */
	public void setAttributes(XAttributeMap attributes) {
		this.attributes = attributes;
	}
	
	/* (non-Javadoc)
	 * @see org.deckfour.xes.model.XAttributable#hasAttributes()
	 */
	public boolean hasAttributes() {
		return !attributes.isEmpty();
	}

	public Set<XExtension> getExtensions() {
		// TODO Auto-generated method stub
		return null;
	}



	/**
	 * Creates a clone, i.e. deep copy, of this trace.
	 * @throws CloneNotSupportedException 
	 */
//	public Object clone() {
//		XOCObjectModelImpl clone = (XOCObjectModelImpl) super.clone();
//		clone.attributes = (XAttributeMap) attributes.clone();
//		clone.clear();
//		for (XEvent event : this) {
//			clone.add((XEvent) event.clone());
//		}
//		return null;
//	}
	
	@SuppressWarnings("unchecked")
	public Object clone(){
		System.out.printf("XOCObjectModelImpl->clone()");
//		XOCObjectModelImpl clone = (XOCObjectModelImpl) super.clone();
		XOCObjectModelImpl clone = new XOCObjectModelImpl();

		clone.objectRelationSet = (List<XOCORelation>) ((ArrayList<XOCORelation>)this.objectRelationSet).clone();
		clone.objectSet = (List<XOCObject>) ((ArrayList<XOCObject>) this.objectSet).clone();
		
//		for(XOCObject object : this.objectSet){
//		XOCObject objectClone = (XOCObject) object.clone();
//		clone.objectSet.add(objectClone);
//		if(this.relatedObjects.contains(object)){
//			clone.relatedObjects.add(objectClone);
//		}
//	}
		
		return clone;
	}
	
	public void updateObjectRelationSet(){
		generateIdVSObjectMap();
		for(XOCORelation objectRelation : objectRelationSet){
			String sourceObjectId = objectRelation.getAttributes().get(xConstants.SOURCEOBJECTID).toString();
			String targetObjectId = objectRelation.getAttributes().get(xConstants.TARGETOBJECTID).toString();
			if(getIdVSObjectMap().get(sourceObjectId) != null){
				objectRelation.setSourceObject(getIdVSObjectMap().get(sourceObjectId));
			}
			if(getIdVSObjectMap().get(targetObjectId) != null){
				objectRelation.setTargetObject(getIdVSObjectMap().get(targetObjectId));
			}			
			objectRelation.setRelationType(objectRelation.getAttributes().get(xConstants.RELATION).toString());
		}
	}
	
	public boolean contain(XOCObjectModel model){
		boolean containFlag = true;
		List<String> IDSet = new ArrayList<String>();
		for(XOCObject object : objectSet){
			IDSet.add(object.getAttributes().get("id").toString());
		}
		for(XOCObject object : model.getObjectSet()){
			if(!IDSet.contains(object.getAttributes().get("id").toString())){
				containFlag = false;
				break;
			}
		}
		return containFlag;
	}
	
	public boolean equalTo(XOCObjectModel model){
		boolean equalFlag = false;
		if(this.contain(model) && model.contain(this))
			equalFlag = true;
		return equalFlag;
	}
	
	public Map<String, XOCObject> generateIdVSObjectMap(){
		idVSObjectMap = new HashMap<String, XOCObject>();
		for(XOCObject object : objectSet){
			idVSObjectMap.put(object.getAttributes().get(xConstants.ID).toString(), object);
		}
		return idVSObjectMap;		
	}
	
	public Map<String, XOCObject> getIdVSObjectMap(){
		return idVSObjectMap;
	}
	
	public void setIdVSObjectMap(Map<String, XOCObject> idVSObjectMap){
		this.idVSObjectMap = idVSObjectMap;
	}
	
	public Map<String, XOCORelation> getIdVSObjectRelationMap(){
		Map<String, XOCORelation> idVSObjectRelationMap = new HashMap<String, XOCORelation>();
		for(XOCORelation objectRelation : objectRelationSet){
			idVSObjectRelationMap.put(objectRelation.getAttributes().get("id").toString(), objectRelation);
		}
		return idVSObjectRelationMap;		
	}
	
	/**
	 * minus the overlap part between the current object model and the input object model from the current object model
	 */
	public XOCObjectModel minusObjectModelOverlap(XOCObjectModel inputModel){
		XOCObjectModel objectModelDifference = new XOCObjectModelImpl();
		Map<String, XOCObject> idVSObjectMapThis = generateIdVSObjectMap();
		Map<String, XOCORelation> idVSObjectRelationMapThis = getIdVSObjectRelationMap();	
		
		Map<String, XOCObject> idVSObjectMapInput = inputModel.generateIdVSObjectMap();
		Map<String, XOCORelation> idVSObjectRelationMapInput = inputModel.getIdVSObjectRelationMap();
		
		List<XOCObject> objectListUpdated = new ArrayList<XOCObject>();//keep the minus result	
		List<XOCORelation> objectRelationListUpdated = new ArrayList<XOCORelation>();//keep the minus result	
		
		for(String objectId : idVSObjectMapThis.keySet()){
			XOCObject objectThis = idVSObjectMapThis.get(objectId);
			if(idVSObjectMapInput.keySet().contains(objectId)){						
				if(!objectThis.isAttributesEqual(idVSObjectMapInput.get(objectId))){
					XOCObject objectDifference = (XOCObject) idVSObjectMapThis.get(objectId).clone();
//					XOCObject objectDifference = idVSObjectMapThis.get(objectId);
					objectDifference.minusAttributeOverlap(idVSObjectMapInput.get(objectId));
					objectListUpdated.add(objectDifference);
				}
			}
			else{
				objectListUpdated.add(objectThis);
			}
		}
		
		//maybe it is problematic
		for(String objectRelationId : idVSObjectRelationMapThis.keySet()){
			XOCORelation objectRelationThis = idVSObjectRelationMapThis.get(objectRelationId);
			if(idVSObjectRelationMapInput.keySet().contains(objectRelationId)){
				
//				if(objectRelationThis == null){
//					System.out.println("objectRelationThis is null");
//				}
//				else{
//					System.out.println("objectRelationThis is not null");
//					if(objectRelationThis.getSourceObject() == null){
//						System.out.println("objectRelationThis.getSourceObject() is null");
//					}
//					else{
//						System.out.println("objectRelationThis.getSourceObject() is not null");
//					}
//				}
				if(!objectRelationThis.isAttributesEqual(idVSObjectRelationMapInput.get(objectRelationId))){
					objectRelationListUpdated.add(objectRelationThis);
				}
			}
			else{
				objectRelationListUpdated.add(objectRelationThis);
			}
		}
		
		objectModelDifference.setObjectSet(objectListUpdated);
		objectModelDifference.setObjectRelationSet(objectRelationListUpdated);
		objectModelDifference.updateObjectRelationSet();
		return objectModelDifference;
	}
	
	/**
	 * plus the difference (inexistent objects (attributes) and object relations) of input model to the current object model
	 * currently, we do not consider the deleted object relations
	 */
	public XOCObjectModel plusObjectModel(XOCObjectModel inputModel){
		XOCObjectModel objectModelPlus = new XOCObjectModelImpl();
		
		Map<String, XOCObject> idVSObjectMapThis = generateIdVSObjectMap();
		Map<String, XOCORelation> idVSObjectRelationMapThis = getIdVSObjectRelationMap();	
		
		Map<String, XOCObject> idVSObjectMapInput = inputModel.generateIdVSObjectMap();
		Map<String, XOCORelation> idVSObjectRelationMapInput = inputModel.getIdVSObjectRelationMap();
		
		List<XOCObject> objectListPlus= new ArrayList<XOCObject>();//keep the minus result	
		List<XOCORelation> objectRelationListPlus = new ArrayList<XOCORelation>();//keep the minus result	
				
		//add the objects in the current object model but not in the input model
		for(String objectId : idVSObjectMapThis.keySet()){
			if(!idVSObjectMapInput.keySet().contains(objectId)){			
				objectListPlus.add((XOCObject) idVSObjectMapThis.get(objectId).clone());
			}
		}
		
		//add the objects (and object attributes) not in the current object model
		for(String objectId : idVSObjectMapInput.keySet()){			
			if(idVSObjectMapThis.keySet().contains(objectId)){			
				XOCObject objectPlus = (XOCObject) idVSObjectMapThis.get(objectId).clone();
				objectPlus.plusAttributes(idVSObjectMapInput.get(objectId));
				objectListPlus.add(objectPlus);
			}
			else{
				XOCObject objectInput = idVSObjectMapInput.get(objectId);
				objectListPlus.add(objectInput);
			}
		}
		
//		for(String objectRelationId : idVSObjectRelationMapInput.keySet()){
//			if(idVSObjectRelationMapThis.keySet().contains(objectRelationId)){
//				XOCORelation objectRelationThis = idVSObjectRelationMapThis.get(objectRelationId);
//				if(!objectRelationThis.isAttributesEqual(idVSObjectRelationMapInput.get(objectRelationId))){
//					objectRelationListPlus.add(idVSObjectRelationMapInput.get(objectRelationId));
//				}
//			}
//			else{
//				objectRelationListPlus.add(idVSObjectRelationMapInput.get(objectRelationId));
//			}
//		}
		
		objectRelationListPlus.addAll(idVSObjectRelationMapThis.values());
		objectRelationListPlus.addAll(idVSObjectRelationMapInput.values());
		
		objectModelPlus.setObjectSet(objectListPlus);
		Map<String, XOCObject> IdVSObjectMapOfObjectModelPlus = objectModelPlus.generateIdVSObjectMap();
		
		for(XOCORelation objectRelation : objectRelationListPlus){
			String sourceObjectId = objectRelation.getAttributes().get(xConstants.SOURCEOBJECTID).toString();
			String targetObjectId = objectRelation.getAttributes().get(xConstants.TARGETOBJECTID).toString();
			objectRelation.setSourceObject(IdVSObjectMapOfObjectModelPlus.get(sourceObjectId));
			objectRelation.setTargetObject(IdVSObjectMapOfObjectModelPlus.get(targetObjectId));
		}
		objectModelPlus.setObjectRelationSet(objectRelationListPlus);
		return objectModelPlus;
	}
	
	public List<XOCObject> computeMore(XOCObjectModel model){
		List<XOCObject> objectSetMore = new ArrayList<XOCObject>();
		
		List<String> IDClassSet = new ArrayList<String>();
		for(XOCObject object : model.getObjectSet()){
			IDClassSet.add(object.getAttributes().get("id").toString() + "_" + object.getAttributes().get("class").toString());
		}
		for(XOCObject object : objectSet){
			if(!IDClassSet.contains(object.getAttributes().get("id").toString() + "_" + object.getAttributes().get("class").toString())){
				objectSetMore.add(object);
			}
		}
		return objectSetMore;
	}
	
	//get the objects (by comparing the id and class of the objects) which are include in model but not in this.model
	public List<XOCObject> computeLess(XOCObjectModel model){
		List<XOCObject> objectSetLess = new ArrayList<XOCObject>();
		List<String> IDClassSet = new ArrayList<String>();
		for(XOCObject object : objectSet){
			//create a string which is the combination of id and class with a "_" in between
			IDClassSet.add(object.getAttributes().get("id").toString() + "_" + object.getAttributes().get("class").toString());
		}
		for(XOCObject object : model.getObjectSet()){
			if(!IDClassSet.contains(object.getAttributes().get("id").toString() + "_" + object.getAttributes().get("class").toString())){
				objectSetLess.add(object);
			}
		}
		return objectSetLess;
	}
	
	//check if two class are related, i.e., there exist a class relation between the two classes
	//if yes, return the class relation id; if no, return null
	public boolean checkTwoObjectsIfRelated(String firstObjectId, String secondObjectId){
		boolean flag = false;
		System.out.printf("\n enter checkTwoObjectsIfRelated");
		for(XOCORelation objectRelation : objectRelationSet){
			String sourceObjectId = objectRelation.getSourceObject().getAttributes().get("id").toString();
			String targetObjectId = objectRelation.getTargetObject().getAttributes().get("id").toString();
			if((sourceObjectId.equals(firstObjectId) && targetObjectId.equals(secondObjectId)) ||
					(sourceObjectId.equals(secondObjectId) && targetObjectId.equals(firstObjectId))){
				flag = true;
				System.out.printf("\n objects equals");
				break;
			}
		}				
		return flag;
	}

	public List<XOCObject> getAllRelatedObjectsForOneObject(String objectId) {
		List<XOCObject> allRelatedObjects = new ArrayList<XOCObject>();
		XOCObject object = getObjectById(objectId);		
		if(object != null){
			
		}

		List<XOCObject> directlyRelatedObjects = getDirectlyRelatedObjectsForOneObject(objectId);
		// TODO Auto-generated method stub
		return null;
	}

	public List<XOCObject> getDirectlyRelatedObjectsForOneObject(String objectId) {
		List<XOCObject> directlyRelatedObjects = new ArrayList<XOCObject>();
		XOCObject object = getObjectById(objectId);		
		if(object != null){
			for(XOCORelation relation : objectRelationSet){
				XOCObject sourceObject = relation.getSourceObject();
				XOCObject targetObject = relation.getTargetObject();
				if(object == sourceObject){
					directlyRelatedObjects.add(targetObject);
				}
				else if(object == targetObject){
					directlyRelatedObjects.add(sourceObject);
				}
			}			
		}
		// TODO Auto-generated method stub
		return directlyRelatedObjects;
	}
	
	public List<XOCObject> getObjectsOfOneClass(String className){
		List<XOCObject> objectsOfOneClass = new ArrayList<XOCObject>();
		for(XOCObject object : objectSet){
			if(object.getAttributes().get("class").toString().equals(className)){
				objectsOfOneClass.add(object);
			}
		}
		return objectsOfOneClass;
	}
	

	
	public List<XOCORelation> getObjectRelationsOfTwoClasses(String firstClassName, String secondClassName){
		List<XOCORelation> objectRelationsOfTwoClasses = new ArrayList<XOCORelation>();
//		System.out.print("\n firstClassName: " + firstClassName);
//		System.out.print("\n secondClassName: " + secondClassName);
		for(XOCORelation objectRelation : objectRelationSet){
			XOCObject sourceObject = objectRelation.getSourceObject();
			XOCObject targetObject = objectRelation.getTargetObject();
			String sourceObjectClass = sourceObject.getAttributes().get("class").toString();
//			System.out.print("\n sourceObjectClass: " + sourceObjectClass);
			String targetObjectClass = targetObject.getAttributes().get("class").toString();
//			System.out.print("\n targetObjectClass: " + targetObjectClass);
			
			if(sourceObject.getAttributes().get("class").toString().equals(firstClassName) && 
					targetObject.getAttributes().get("class").toString().equals(secondClassName)){
//				System.out.print("\n equals; soruce first ");
				objectRelationsOfTwoClasses.add(objectRelation);
			}
			else if(targetObject.getAttributes().get("class").toString().equals(firstClassName) && 
					sourceObject.getAttributes().get("class").toString().equals(secondClassName)){
//				System.out.print("\n equals; soruce first ");
				objectRelationsOfTwoClasses.add(objectRelation);
			}
		}
		return objectRelationsOfTwoClasses;
	}
	

}
