package org.processmining.ocbc.logconversion.classmodel;

import java.awt.Dimension;
import java.awt.GraphicsEnvironment;
import java.awt.Rectangle;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.jgraph.graph.GraphConstants;
import org.processmining.framework.util.Pair;
import org.processmining.ocbc.log.model.XOCLog;
import org.processmining.ocbc.models.ActivityNode;
import org.processmining.ocbc.models.ClassEdge;
import org.processmining.ocbc.models.ClassNode;
import org.processmining.ocbc.models.OCBCModel;
import org.processmining.ocbc.models.impl.OCBCModelImpl;
import org.processmining.ocbc.models.visualization.ModelLayoutBuilder;
import org.processmining.ocbc.models.visualization.cell.MyOriginalCell;


public class ClassModelLayout {
	private OCBCModel model;
	private NodeSize nodeSize = new NodeSize();
	public ClassModelLayout(OCBCModel model){
		this.model = model;
	}
	
	public Map<String, Pair<Integer, Integer>> generateEventCoordinates(XOCLog log, int height, int width){
		 int positionX = 0;
		 int positionY = 0; 
		 int rowNum = 0;
		 int columnNum = 0;	 
		 int unitHeight = 0;
		 int unitWidth = 0;
		 int classNum = log.getClassNames().size();

		 Map<String, Pair<Integer, Integer>> nodeVSCoordinateMap = new HashMap<String, Pair<Integer, Integer>>();
		 
		 columnNum = (int) Math.sqrt(classNum-1) + 1;		 	 
		 if(classNum <= columnNum*(columnNum - 1)) rowNum = columnNum - 1;
		 else rowNum = columnNum;		 
		 unitHeight = height / rowNum;
		 unitWidth = width / columnNum;
		 
		 for(int i=0; i<classNum; i++){		 
			 String className = log.getClassNames().get(i);		 
			 positionX = (int) (unitWidth*0.5 + i%columnNum*unitWidth);
			 positionY = (int) (unitHeight*0.5 + i/columnNum*unitHeight);
			 Pair<Integer, Integer> coordinate = new Pair<Integer, Integer>(positionX, positionY);
			 nodeVSCoordinateMap.put(className, coordinate);	
		 }		 	 	
		 
		 return nodeVSCoordinateMap;
	}
	
	
	//create coordinates for each object
//	public Map<String, Pair<Integer, Integer>> generateClassCoordinates(XOCLog log, int height, int width, int offsetY){	 
//		 int positionX = 0;
//		 int positionY = 0; 
//		 int classNum = 0;
//		 int horizontalUintNum = 0;
//		 int verticalUintNum = 0;
//		 int unitHeight = 0;
//		 int unitWidth = 0;
//		 int eventNodeHeight = 50;
//		 
//		 Map<String, Pair<Integer, Integer>> nodeVSCoordinateMap = new HashMap<String, Pair<Integer, Integer>>();
////		 nodeVSCoordinateMap.put("event", new Pair<Integer, Integer>(width/2, eventNodeHeight));
//		 
//		 classNum = log.getClassNames().size();
//		 height = height - 2*eventNodeHeight;	
//		 
//		 horizontalUintNum = (int) Math.sqrt(classNum-1) + 1;
//		 if(classNum <= horizontalUintNum*(horizontalUintNum-1)) verticalUintNum = horizontalUintNum-1;
//		 else verticalUintNum = horizontalUintNum;
//		 unitHeight = height / verticalUintNum;
//		 unitWidth = width / horizontalUintNum;
//		 	 				 			 		 
//		 for(XOCObject object : event.getObjectModel().getObjectSet()){
//			 String classType = object.getClassType().getClassName();
//			 int classIndex = log.getClassNames().indexOf(classType);	
//			 positionX = (int) (Math.random()*unitWidth*0.7 + (classIndex%horizontalUintNum)*unitWidth);
//			 positionY = 2*eventNodeHeight + (int) (Math.random()*unitHeight*0.7 + (classIndex/horizontalUintNum)*unitHeight);
//			 Pair<Integer, Integer> coordinate = new Pair<Integer, Integer>(positionX, positionY + offsetY);;
//			 nodeVSCoordinateMap.put(object.getAttributes().get("id").toString(), coordinate);	
//			 System.out.print("\n id" + object.getAttributes().get("id").toString());
//			 System.out.print("\n value" + coordinate.getFirst());
//		 }	 
//		 return nodeVSCoordinateMap;
//	}
	
	//assume there do not exist parallel relations among activities
	//class model: row 1 for all the nodes connected with activity nodes
	//row 2 for nodes connected with nodes in row 1
	//row 3 for nodes connected with nodes in row 2
	//refer to the layout of object model in log inspector
	public OCBCModel optimizeLayout(){
		Rectangle screenRect = GraphicsEnvironment.getLocalGraphicsEnvironment().getMaximumWindowBounds();
		Dimension screenSize = screenRect.getSize();
		double rightMargin = 300;
		double bottonMargin = 300;
		double maxWidth = screenSize.getWidth() - rightMargin;
		double maxHeight = screenSize.getHeight() - bottonMargin;
		
		int activityNum = model.getAllActivities().size();
		//int the future, compute the numbers of rows and columns for activities and classes
		double widthUnit = maxWidth/(activityNum + 1);
		double heightUnit = maxHeight/4;
		
		OCBCModel optimizedModel = new OCBCModelImpl("optimized OCBC model");
	    ModelLayoutBuilder optimizedBuilder = new ModelLayoutBuilder(optimizedModel);
	    
	    List<Pair<String, String>> sequentialActivityPairList = model.getSequentialActivityPairList();
	    int acticityIndex = 0;
	    ActivityNode removeActivityNode = null;
	    

		
	    
	    //redraw class nodes
	   
	    List<ClassNode> classNodesInAOC = new ArrayList<ClassNode>();
	    List<ClassNode> classNodesOutAOC = new ArrayList<ClassNode>();
	    
	    
//	    List<Double> classNodesX = new ArrayList<Double>();
	    Map<ClassNode, Double> classNodeVSXCoordinate = new HashMap<ClassNode, Double>();
		for(ClassNode classNode : model.getAllClasses()){
			double classNodeX = 0;
			if(classNode.getRelatedActivityNodes().size()>0){
				classNodesInAOC.add(classNode);
				for(ActivityNode activityNode : classNode.getRelatedActivityNodes()){
					String activityNodeId = activityNode.getID().toString();
					MyOriginalCell relatedActicvityCell = optimizedModel.getNodeCellById(activityNodeId);
					classNodeX = classNodeX + GraphConstants.getBounds(relatedActicvityCell.getAttributes()).getX();
				}
				classNodeX = classNodeX/classNode.getRelatedActivityNodes().size();
				
				classNodeVSXCoordinate.put(classNode, classNodeX);
				
//				optimizedBuilder.addNode("ClassCell", classNode.getNodeName(), classNode.getID().toString(), 
//						classNodeX + Math.random()*30, heightUnit*2, 55.0, 65.0);
				System.out.printf("\n optimizedBuilder.addNode");
			}
			else{
				classNodesOutAOC.add(classNode);
			}			
		}	
		
		
		List<ClassNode> classNodesRest = classNodesOutAOC;
	    Map<ClassNode, Integer> classNodeVSColumn = new HashMap<ClassNode, Integer>();
	    Map<ClassNode, Integer> classNodeVSRow = new HashMap<ClassNode, Integer>();
	    Map<Integer, Integer> rowVSColumnNum = new HashMap<Integer, Integer>();
	    Map<ClassNode, Double> classNodeVSAverageX = new HashMap<ClassNode, Double>();
	    
		widthUnit = maxWidth/(classNodesInAOC.size() + 1);
//		double heightUnit = maxHeight/4;
		int classIndex = 0;
		while(classNodeVSXCoordinate.keySet().size() > 0){
			Double minX = Double.MAX_VALUE;
			ClassNode classNodeLeft = null;
			for(ClassNode classNode : classNodeVSXCoordinate.keySet()){
				if(minX > classNodeVSXCoordinate.get(classNode)){
					minX = classNodeVSXCoordinate.get(classNode);
					classNodeLeft = classNode;
				}
			}
			classIndex ++;
			classNodeVSColumn.put(classNodeLeft, classIndex);
			
			classNodeVSXCoordinate.keySet().remove(classNodeLeft);
			optimizedBuilder.addNode("ClassCell", classNodeLeft.getNodeName(), classNodeLeft.getID().toString(), 
					widthUnit*classIndex + Math.random()*30, heightUnit*2, 55.0, nodeSize.computeWidthAccordingName(classNodeLeft.getNodeName()));
		}
		
		
		//redraw class nodes not in AOC
		//currently, all the nodes not in AOC are connected with 
		
		List<ClassNode> classNodesInUpperRow = classNodesInAOC;
		List<ClassNode> classNodesInCurrentRow = new ArrayList<ClassNode>();				
		

		 
		int currentRowIndex = 1;
		while(classNodesRest.size() != 0){			
			System.out.printf("\n classNodesRest.size() != 0");
		    //calculate the the x average of all related class node for each node
			//specify the row of each class node
			for(ClassNode classNode : classNodesRest){
				double classNodeAverageX = 0;
				for(ClassNode relatedClassNode : classNode.getRelatedClassNodes()){
					if(classNodesInUpperRow.contains(relatedClassNode)){
						classNodesInCurrentRow.add(classNode);
						classNodeVSRow.put(classNode, currentRowIndex);
						classNodeAverageX = classNodeAverageX + classNodeVSColumn.get(relatedClassNode);
														
					}			
				}

				classNodeAverageX = classNodeAverageX/classNode.getRelatedClassNodes().size();
				classNodeVSAverageX.put(classNode, classNodeAverageX);		
			}
			//specify the number of elements in each row
			rowVSColumnNum.put(currentRowIndex, classNodesInCurrentRow.size());
			System.out.printf("\n rowVSColumnNum.put");
			
			//rank the x average
			//specify the column of each class node
			int currentColumnIndex = 0;
			while(classNodeVSAverageX.keySet().size() > 0){
				Double minX = Double.MAX_VALUE;
				ClassNode classNodeLeft = null;
				for(ClassNode classNode : classNodeVSAverageX.keySet()){
					if(minX > classNodeVSAverageX.get(classNode)){
						minX = classNodeVSAverageX.get(classNode);
						classNodeLeft = classNode;
					}
				}
				currentColumnIndex ++;
				classNodeVSAverageX.remove(classNodeLeft);
				classNodeVSColumn.put(classNodeLeft, currentColumnIndex);
			}
			
			//update the rest nodes, upper row nodes and current row nodes
			classNodesRest.removeAll(classNodesInCurrentRow);
			classNodesInUpperRow.clear();
			for(ClassNode classNode : classNodesInCurrentRow){
				classNodesInUpperRow.add(classNode);
			}
			classNodesInCurrentRow.clear();
			currentRowIndex ++;			
		}
	   
		//draw the class model except the first row (class nodes in AOC relation)
		if(rowVSColumnNum.keySet().size() > 0){
			double rowUnit = 2/(rowVSColumnNum.keySet().size()+1);
			double columnWidth;
		    for(ClassNode classNode : classNodeVSRow.keySet()){
		    	int rowIndex = classNodeVSRow.get(classNode);
		    	int columnIndex = classNodeVSColumn.get(classNode);
		    	int columnNum = rowVSColumnNum.get(rowIndex);	    	
		    	columnWidth = maxWidth/columnNum;
		    	
		    	optimizedBuilder.addNode("ClassCell", classNode.getNodeName(), classNode.getID().toString(), 
		    			columnWidth*columnIndex, heightUnit*(2 + rowUnit*rowIndex), 55.0, nodeSize.computeWidthAccordingName(classNode.getNodeName()));
		    }
		}


		
		
//		for(ClassNode classNode : classNodesOutAOC){
//			double classNodeX = 0;
//			for(ClassNode relatedClassNode : classNode.getRelatedClassNodes()){
//				if(classNodeVSXCoordinate.keySet().contains(relatedClassNode)){
//					String relatedClassNodeId = relatedClassNode.getID().toString();
//					MyOriginalCell relatedClassCell = optimizedModel.getNodeCellById(relatedClassNodeId);
//					classNodeX = classNodeX + GraphConstants.getBounds(relatedClassCell.getAttributes()).getX();
//				}			
//			}
//			
//			classNodeX = classNodeX/classNode.getRelatedClassNodes().size();
//			optimizedBuilder.addNode("ClassCell", classNode.getNodeName(), classNode.getID().toString(), 
//					classNodeX + Math.random()*50, heightUnit*3 + Math.random()*50, 55.0, 65.0);			
//		}
		
	    //redraw class edges
	    for(ClassEdge classEdge : model.getAllClassRelations()){
	    	optimizedBuilder.addClassEdge(classEdge.getID().toString(), classEdge.getEdgeName(), classEdge.getSourceNode().getID().toString(),
	    			classEdge.getSourceAlwaysCardinality().toString(), classEdge.getSourceEventuallyCardinality().toString(),
	    			classEdge.getTargetNode().getID().toString(), classEdge.getTargetAlwaysCardinality().toString(), 
	    			classEdge.getTargetEventuallyCardinality().toString(), 0, 0, 55.0, nodeSize.computeWidthAccordingName(classEdge.getEdgeName()));	    			
	    }
	    
	   
	    
		return optimizedModel;
	}
	
    public static void main(String[] args) throws Exception {
    	List<String> a = new ArrayList<String>();
    	List<String> b = new ArrayList<String>();
    	String hello = "hello";
    	System.out.printf("\n a size" + hello.length());
    	
    	a.add("a");
    	a.add("b");
//    	b = a;
    	for(String element : a){
    		b.add(element);
    	}
    	a.clear();
    	System.out.printf("\n a size" + a.size());
    	System.out.printf("\n b size" + b.size());
    	
    	
    }
}