package org.processmining.ocbc.logconversion.classmodel;

import java.awt.Toolkit;
import java.io.File;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.swing.JFrame;
import javax.swing.JScrollPane;

import org.processmining.framework.util.Pair;
import org.processmining.mxgraph.util.mxConstants;
import org.processmining.ocbc.discovery.impl.AOCEdgeDiscovery;
import org.processmining.ocbc.discovery.impl.BehaviorModelDiscovery;
import org.processmining.ocbc.discovery.layout.ModelLayout;
import org.processmining.ocbc.log.io.XocXmlParser;
import org.processmining.ocbc.log.model.XOCLog;
import org.processmining.ocbc.log.model.XOCORelation;
import org.processmining.ocbc.log.model.XOCObject;
import org.processmining.ocbc.log.model.impl.xConstants;
import org.processmining.ocbc.models.OCBCModel;
import org.processmining.ocbc.models.impl.OCBCModelImpl;
import org.processmining.ocbc.models.visualization.ModelLayoutBuilder;


public class ClassModelSkeletonDiscovery {
	private XOCLog log;
	private ModelLayoutBuilder builder;
	private Map<String, String> nodeNameVSNodeId;
	private  Map<String, Pair<Integer, Integer>> nodeVSCoordinateMap;
	
	public ClassModelSkeletonDiscovery(XOCLog log, ModelLayoutBuilder builder, Map<String, String> nodeNameVSNodeId){
		this.log = log;
		this.builder = builder;
		this.nodeNameVSNodeId = nodeNameVSNodeId;
	}
	
	public Map<String, Pair<Integer, Integer>> generateEventCoordinates(int height, int width){
		 int positionX = 0;
		 int positionY = 0; 
		 int rowNum = 0;
		 int columnNum = 0;	 
		 int unitHeight = 0;
		 int unitWidth = 0;
		 int classNum = log.getClassNames().size();

		 nodeVSCoordinateMap = new HashMap<String, Pair<Integer, Integer>>();
		 
		 columnNum = (int) Math.sqrt(classNum-1) + 1;		 	 
		 if(classNum <= columnNum*(columnNum - 1)) rowNum = columnNum - 1;
		 else rowNum = columnNum;		 
		 unitHeight = height / rowNum;
		 unitWidth = width / columnNum;
		 List<String> classNameList = log.getClassNames();
		 List<String> randomClassNameList = new ArrayList<String>();
		 
//		 (int)(1+Math.random()*(10-1+1))
//		 while()
//		 for(){
//			 
//		 }
		 
		 Collections.shuffle(classNameList);
		 for(int i=0; i<classNum; i++){		 
			 String className = classNameList.get(i);		 
			 positionX = (int) (unitWidth*0.5 + i%columnNum*unitWidth);
			 positionY = (int) (unitHeight*0.5 + i/columnNum*unitHeight);
			 Pair<Integer, Integer> coordinate = new Pair<Integer, Integer>(positionX, positionY);
			 nodeVSCoordinateMap.put(className, coordinate);	
		 }		 	 	
		 
		 return nodeVSCoordinateMap;
	}
	
	public void discoverClassModel(){
		//discover class nodes
		Integer idCounter = 1;
		for(String className : log.generateClassNameList()){
//					String classNodeId = UUID.randomUUID().toString();
			String classNodeId = className + (idCounter ++).toString();
			nodeNameVSNodeId.put(className, classNodeId);
			System.out.println("className: " + className);
			//draw the class node
			builder.addNode(mxConstants.CLASSCELL, className, classNodeId, 
	        		Math.random()*1000, Math.random()*1000,
	        		55.0, 65.0);
		}
		
		List<String> classRelationcombinationIds = new ArrayList<String>();
		//discover class edges
//				for(XOCEvent event : log.getAllEvents()){
		System.out.println("log.getLastEvent().getObjectModel().getObjectRelationSet().size(): " + log.getLastEvent().getObjectModel().getObjectRelationSet().size());
//					for(XOCORelation objectRelation : log.getLastEvent().getObjectModel().getObjectRelationSet()){			
		for(XOCORelation objectRelation : log.getObjectRelationList()){			
			String classRelationName = objectRelation.getAttributes().get("relation").toString();
		
//						XOCObject sourceObject = log.getSourceObjectOfObjectRelation(event.getObjectModel(), objectRelation);
			XOCObject sourceObject = objectRelation.getSourceObject();
			String sourceClassName = sourceObject.getAttributes().get(xConstants.CLASS).toString();
			String sourceClassNodeId = nodeNameVSNodeId.get(sourceClassName);
			
//						XOCObject targetObject = log.getTargetObjectOfObjectRelation(event.getObjectModel(), objectRelation);
			XOCObject targetObject = objectRelation.getTargetObject();
			String targetClassName = targetObject.getAttributes().get(xConstants.CLASS).toString();
			String targetClassNodeId = nodeNameVSNodeId.get(targetClassName);
			
			
			String classRelationcombinationId = classRelationName + "_" + sourceClassName + "_" + targetClassName;
			if(!classRelationcombinationIds.contains(classRelationcombinationId)){
				classRelationcombinationIds.add(classRelationcombinationId);
				//discover the class edge cardinality
//							String classRelationId = classRelationName + (idCounter ++).toString();
				String classRelationId = classRelationName;

				//draw the class edge
				builder.addClassEdge(classRelationId, classRelationName, sourceClassNodeId, 
						"*","*", targetClassNodeId, "*", "*", 0, 0, 55.0, 65.0);
			}			
		}
	}
	
    public static void main(String[] args) throws Exception {
		JFrame frame = new JFrame();
		OCBCModel model = new OCBCModelImpl();
		XocXmlParser parser = new XocXmlParser();	
		parser.parse(new File("e:\\normal.xoc"));		
		XOCLog log = parser.getLog();

		Map<String, String> nodeNameVSNodeId = new HashMap<String, String>();
		ModelLayoutBuilder builder = new ModelLayoutBuilder(model);
		ClassModelSkeletonDiscovery classModelSkeletonDiscovery = new ClassModelSkeletonDiscovery(log, builder, nodeNameVSNodeId);
		classModelSkeletonDiscovery.discoverClassModel();
		
		BehaviorModelDiscovery behaviorModelDiscovery = new BehaviorModelDiscovery(log, builder, nodeNameVSNodeId);
	    behaviorModelDiscovery.discoverActivityNodes();
		
	    //it is possible to discover "aoc" edges after the activity and class nodes are discovered
	    AOCEdgeDiscovery AOCEdgeDiscovery = new AOCEdgeDiscovery(log, builder, nodeNameVSNodeId);
	    AOCEdgeDiscovery.discoverAOCEdges();
		    
		//perfect the layout of the model
		ModelLayout modelLayout = new ModelLayout(model);
		model = modelLayout.optimizeLayout();
				
		frame.getContentPane().add(new JScrollPane(model.getGraph()));
		
		
		
		frame.setSize(Toolkit.getDefaultToolkit().getScreenSize()); 
		frame.setVisible(true);
    }
}