package org.processmining.ocbc.logconversion.classmodel;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Stack;

import org.processmining.ocbc.models.ClassNode;


public class SearchAllPathsBetweenTwoNodes {
	private List<String> pathList = new ArrayList<String>();
	/* 临时保存路径节点的栈 */
	public Stack<ClassNode> stack = new Stack<ClassNode>();
	/* 存储路径的集合 */
	public ArrayList<Object[]> sers = new ArrayList<Object[]>();

	/* 判断节点是否在栈中 */
	public boolean isNodeInStack(ClassNode node)
	{
		Iterator<ClassNode> it = stack.iterator();
		while (it.hasNext()) {
			ClassNode node1 = (ClassNode) it.next();
			if (node == node1)
				return true;
		}
		return false;
	}

	public List<String> getPathList(){
		return pathList;
	}
	
	public void clearPathList(){
		this.pathList.clear();
	}
	/* 此时栈中的节点组成一条所求路径，转储并打印输出 */
	public void showAndSavePath(List<String> pathList)
	{
		Object[] o = stack.toArray();
		String path = "";
		for (int i = 0; i < o.length; i++) {
			ClassNode nNode = (ClassNode) o[i];
			
			if(i < (o.length - 1)){
//				System.out.print(nNode.getNodeName() + "->");
				path = path + nNode.getNodeName() + "--";
			}
				
			else{
//				System.out.print(nNode.getNodeName());
				path = path + nNode.getNodeName();
			}
				
		}
		sers.add(o); /* 转储 */
		
		pathList.add(path);
		System.out.println(path);
//		System.out.println("\n");
	}

	/*
	 * 寻找路径的方法 
	 * cNode: 当前的起始节点currentNode
	 * pNode: 当前起始节点的上一节点previousNode
	 * sNode: 最初的起始节点startNode
	 * eNode: 终点endNode
	 */
	public boolean getPaths(ClassNode cNode, ClassNode pNode, ClassNode sNode, ClassNode eNode) {
		//pathList hold paths (a path is like project->employee->expenses, which indicate a path between project and expenses nodes)
//		pathList.clear();
		ClassNode nNode = null;
		/* 如果符合条件判断说明出现环路，不能再顺着该路径继续寻路，返回false */
		if (cNode != null && pNode != null && cNode == pNode)
			return false;

		if (cNode != null) {
			int i = 0;
			/* 起始节点入栈 */
			stack.push(cNode);
			/* 如果该起始节点就是终点，说明找到一条路径 */
			if (cNode == eNode)
			{
				/* 转储并打印输出该路径，返回true */
				showAndSavePath(pathList);
				return true;
			}
			/* 如果不是,继续寻路 */
			else
			{
				/* 
				 * 从与当前起始节点cNode有连接关系的节点集中按顺序遍历得到一个节点
				 * 作为下一次递归寻路时的起始节点 
				 */
				if(cNode.getRelatedClassNodes().size()>0){
					nNode = cNode.getRelatedClassNodes().get(i);
					while (nNode != null) {
						/*
						 * 如果nNode是最初的起始节点或者nNode就是cNode的上一节点或者nNode已经在栈中 ， 
						 * 说明产生环路 ，应重新在与当前起始节点有连接关系的节点集中寻找nNode
						 */
						if (pNode != null
								&& (nNode == sNode || nNode == pNode || isNodeInStack(nNode))) {
							i++;
							if (i >= cNode.getRelatedClassNodes().size())
								nNode = null;
							else
								nNode = cNode.getRelatedClassNodes().get(i);
							continue;
						}
						/* 以nNode为新的起始节点，当前起始节点cNode为上一节点，递归调用寻路方法 */
						if (getPaths(nNode, cNode, sNode, eNode))/* 递归调用 */
						{
							/* 如果找到一条路径，则弹出栈顶节点 */
							stack.pop();
						}
						/* 继续在与cNode有连接关系的节点集中测试nNode */
						i++;
						if (i >= cNode.getRelatedClassNodes().size())
							nNode = null;
						else
							nNode = cNode.getRelatedClassNodes().get(i);
					}
				}
				/* 
				 * 当遍历完所有与cNode有连接关系的节点后，
				 * 说明在以cNode为起始节点到终点的路径已经全部找到 
				 */
				stack.pop();
				return false;
			}
		} else
			return false;
	}

	public static void main(String[] args) {
		/* 定义节点关系 */
		int nodeRalation[][] =
		{
			{1},      //0
			{0,5,2,3},//1
			{1,4},    //2
			{1,4},    //3
			{2,3,5},  //4
			{1,4}     //5
		};
		
		/* 定义节点数组 */
		Node[] node = new Node[nodeRalation.length];
		List<ClassNode> classNodeList = new ArrayList<ClassNode>();
		
		for(int i=0;i<nodeRalation.length;i++)
		{
            node[i] = new Node();
			node[i].setName("node" + i);
		}
		
		/* 定义与节点相关联的节点集合 */
		for(int i=0;i<nodeRalation.length;i++)
		{
			ArrayList<Node> List = new ArrayList<Node>();
			
			for(int j=0;j<nodeRalation[i].length;j++)
			{
				List.add(node[nodeRalation[i][j]]);
			}
			node[i].setRelationNodes(List);
			List = null;  //释放内存
		}

		/* 开始搜索所有路径 */
//		getPaths(node[0], null, node[0], node[5]);
	}
}