package org.processmining.ocbc.logconversion.panels;

import java.awt.Color;
import java.awt.Component;
import java.util.ArrayList;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTree;
import javax.swing.ScrollPaneConstants;
import javax.swing.border.Border;
import javax.swing.plaf.basic.BasicSplitPaneDivider;
import javax.swing.plaf.basic.BasicSplitPaneUI;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeCellRenderer;

import org.processmining.framework.util.Pair;
import org.processmining.ocbc.log.model.XOCEvent;
import org.processmining.ocbc.log.model.XOCLog;
import org.processmining.ocbc.log.model.XOCORelation;
import org.processmining.ocbc.log.model.XOCObject;
import org.processmining.ocbc.log.model.impl.xConstants;

import com.fluxicon.slickerbox.components.RoundedPanel;

public class LogPanel extends RoundedPanel {

	
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	protected JList<Object> eventList;
//	protected JList<Object> objectReferenceList;
	protected JTree objectReferenceTree;
	protected JTree objectModelTree;
	
	private XOCLog log;
	boolean generateCoordinatesFlag = false;
	
	public static final Color COLOR_ENCLOSURE_BG = new Color(40, 40, 40);
	public static final Color COLOR_NON_FOCUS = new Color(70, 70, 70);
	public static final Color COLOR_LIST_BG = new Color(60, 60, 60);
	public static final Color COLOR_LIST_BG_LOWER = new Color(45, 45, 45);
	public static final Color COLOR_LIST_FG = new Color(180, 180, 180);
	public static final Color COLOR_LIST_SELECTION_BG = new Color(80, 0, 0);
	public static final Color COLOR_LIST_SELECTION_BG_LOWER = new Color(30, 10, 10);
	public static final Color COLOR_LIST_SELECTION_FG = new Color(240, 240, 240);
	public static final Color COLOR_LIST_TAGGED_SELECTION_BG = Color.RED;
	public static final Color COLOR_LIST_TAGGED_BG = new Color(60, 60, 0);

	private XOCLogConversionPanel logConversionPanel;
	
	private JScrollPane objectModelScrollPane;
	private JLabel objectModelLabel;
	
	private JScrollPane objectReferenceScrollPane;
	private JLabel objectReferenceListLabel;
	
	public LogPanel(int aRadius, int anOuterBorder, int anInnerBorder){
		super(aRadius, anOuterBorder, anInnerBorder);	
	}
	
	public LogPanel(final XOCLogConversionPanel logConversionPanel, XOCLog log){
		this.logConversionPanel = logConversionPanel;
		this.log = log;
		
		//build the event list
		eventList = new JList();
		eventList.setBackground(COLOR_LIST_BG);
		eventList.setForeground(Color.WHITE); 
		updateEventList(log);
		
		JScrollPane eventScrollPane = new JScrollPane(eventList);
		eventScrollPane.setBorder(BorderFactory.createEmptyBorder());
		eventScrollPane.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);
		eventScrollPane.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		
		JLabel eventListLabel = new JLabel("Events (" + log.getAllEvents().size() + ")");
		eventListLabel.setForeground(COLOR_LIST_SELECTION_FG);
		eventListLabel.setAlignmentX(Component.CENTER_ALIGNMENT);

		this.setBackground(COLOR_ENCLOSURE_BG);
		this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
//		this.setMaximumSize(new Dimension(180, 1000));
//		this.setPreferredSize(new Dimension(180, 1000));
		
//		this.add(eventListLabel);	
//		this.add(Box.createVerticalStrut(8));		
//		this.add(eventScrollPane);
		
		JPanel eventPanel = new JPanel();
		eventPanel.setBackground(COLOR_ENCLOSURE_BG);
		eventPanel.setLayout(new BoxLayout(eventPanel, BoxLayout.Y_AXIS));
		eventPanel.add(Box.createVerticalStrut(8));	
		eventPanel.add(eventListLabel);
		eventPanel.add(Box.createVerticalStrut(8));	
		eventPanel.add(eventScrollPane);
		
		createObjectReferencePanel(log);
		createObjectModelPanel(log);
		
		JPanel objectPanel = new JPanel();
		objectPanel.setBackground(COLOR_ENCLOSURE_BG);
		objectPanel.setLayout(new BoxLayout(objectPanel, BoxLayout.Y_AXIS));
		objectPanel.add(objectReferenceListLabel);
		objectPanel.add(Box.createVerticalStrut(8));	
		objectPanel.add(objectReferenceScrollPane);
		objectPanel.add(Box.createVerticalStrut(8));
		objectPanel.add(objectModelLabel);
		objectPanel.add(Box.createVerticalStrut(8));	
		objectPanel.add(objectModelScrollPane);
		
		JSplitPane logPanel = borderlessSplitPane(JSplitPane.VERTICAL_SPLIT, eventPanel, objectPanel);
		logPanel.setResizeWeight(0.4);
		this.add(logPanel);
		
//		//build the object reference tree (event -> all its object references)
//		createObjectReferencePanel(log);
//		this.add(Box.createVerticalStrut(8));	
//		this.add(objectReferenceListLabel);	
//		this.add(Box.createVerticalStrut(8));		
//		this.add(objectReferenceScrollPane);
//		
//		//build the object model tree (objects + object relations)
//		createObjectModelPanel(log);
//		this.add(Box.createVerticalStrut(8));	
//		this.add(objectModelLabel);	
//		this.add(Box.createVerticalStrut(8));		
//		this.add(objectModelScrollPane);
	}
	
	public JSplitPane borderlessSplitPane(int orientation, JComponent left, JComponent right) {
		JSplitPane pane = new JSplitPane(orientation, left, right);
		pane.setBorder(BorderFactory.createEmptyBorder(0, 0, 0, 0));
		BasicSplitPaneUI flatDividerSplitPaneUI = new BasicSplitPaneUI() {
			@Override
			public BasicSplitPaneDivider createDefaultDivider() {
				return new BasicSplitPaneDivider(this) {
					@Override
					public void setBorder(Border b) {
					}
				};
			}
		};
		pane.setUI(flatDividerSplitPaneUI);
		pane.setBorder(null);
		pane.setOpaque(false);
		return pane;
	}
	
	/**
	 * create the object model panel, in which a tree (root node) is used to represent the object model
	 * @param inputLog
	 */
	public void createObjectReferencePanel(XOCLog inputLog) {
		DefaultMutableTreeNode objectReferencelNode = new DefaultMutableTreeNode("Object reference");
		
		for(XOCEvent event : inputLog.getAllEvents()){
			String eventId = event.getAttributes().get(xConstants.ID).toString();
			DefaultMutableTreeNode references = generateObjectSetNode(event.getRelatedObjects(), eventId + "(" + event.getRelatedObjects().size() + ")");
			objectReferencelNode.add(references);
		}
	
		objectReferenceTree = new JTree(objectReferencelNode);
		objectReferenceTree.setBackground(COLOR_LIST_BG);
		objectReferenceTree.setForeground(Color.WHITE); 
		
		DefaultTreeCellRenderer renderer = (DefaultTreeCellRenderer) objectReferenceTree.getCellRenderer();
		renderer.setBackground(COLOR_LIST_BG);
		renderer.setBackgroundNonSelectionColor(COLOR_LIST_BG);
		renderer.setBackgroundSelectionColor(COLOR_LIST_SELECTION_BG);
		renderer.setForeground(Color.WHITE);
		renderer.setTextNonSelectionColor(Color.WHITE);
		renderer.setTextSelectionColor(COLOR_LIST_SELECTION_FG);
		
		objectReferenceScrollPane = new JScrollPane(objectReferenceTree);
		objectReferenceScrollPane.setBorder(BorderFactory.createEmptyBorder());
		objectReferenceScrollPane.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);
		objectReferenceScrollPane.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		
		objectReferenceListLabel = new JLabel("Object reference (" + log.getAllEvents().size() + ")");
		objectReferenceListLabel.setForeground(COLOR_LIST_SELECTION_FG);
		objectReferenceListLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
	}
	
	/**
	 * create the object model panel, in which a tree (root node) is used to represent the object model
	 * @param inputLog
	 */
	public void createObjectModelPanel(XOCLog inputLog) {
		DefaultMutableTreeNode objectModelNode = new DefaultMutableTreeNode("Object model");
		
		List<XOCObject> objectSet = log.getLastEvent().getObjectModel().getObjectSet();
		List<XOCORelation> objectRelationSet = log.getLastEvent().getObjectModel().getObjectRelationSet();
		objectModelNode.add(generateObjectSetNode(objectSet, "objects" + "(" + objectSet.size() + ")"));
		objectModelNode.add(generateRelationSetNode(objectRelationSet, "object relations" + "(" + objectRelationSet.size() + ")"));
		
		objectModelTree = new JTree(objectModelNode);
		objectModelTree.setBackground(COLOR_LIST_BG);
		objectModelTree.setForeground(Color.WHITE); 
		
		DefaultTreeCellRenderer renderer = (DefaultTreeCellRenderer) objectModelTree
				.getCellRenderer();
		renderer.setBackground(COLOR_LIST_BG);
		renderer.setBackgroundNonSelectionColor(COLOR_LIST_BG);
		renderer.setBackgroundSelectionColor(COLOR_LIST_SELECTION_BG);
		renderer.setForeground(Color.WHITE);
		renderer.setTextNonSelectionColor(Color.WHITE);
		renderer.setTextSelectionColor(COLOR_LIST_SELECTION_FG);
		
		objectModelScrollPane = new JScrollPane(objectModelTree);
		objectModelScrollPane.setBorder(BorderFactory.createEmptyBorder());
		objectModelScrollPane.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);
		objectModelScrollPane.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		
		objectModelLabel = new JLabel("Object model");
		objectModelLabel.setOpaque(false);
		objectModelLabel.setForeground(COLOR_LIST_SELECTION_FG);
		objectModelLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
	}
	
	/**
	 * update the event list based on input XOC log
	 * @param inputLog
	 */
	public void updateEventList(XOCLog inputLog) {
		if (inputLog == null || inputLog.getAllEvents().size() == 0) {
			return;
		}

		List<Pair<Integer, String>> instances = new ArrayList<Pair<Integer, String>>(inputLog.getAllEvents().size());
		for (int i = 0; i < inputLog.getAllEvents().size(); i++) {
			String name = inputLog.getAllEvents().get(i).getAttributes().get("id").toString();
			instances.add(new Pair<Integer, String>(i+1, name));
		}
		eventList.setListData(instances.toArray());
	}

	/**
	 * 
	 */
	protected void instancesSelectionChanged() {	
		System.out.printf("event list index:  " + eventList.getSelectedIndex());
		
		int[] selectedIndices = eventList.getSelectedIndices();
		XOCEvent event = null;

		
		
		if(eventList.getSelectedIndex() < 0)
			event = log.getAllEvents().get(0);
		else
			event = log.getAllEvents().get(selectedIndices[0]);
		
	}	
	
	private DefaultMutableTreeNode generateObjectSetNode(List<XOCObject> objects, String name){
		DefaultMutableTreeNode objectSetNode = new DefaultMutableTreeNode(new TreeEntry(name));
		for (XOCObject object : objects) {
			objectSetNode.add(generateObjectNode(object));
		}
		return objectSetNode;
	}
	
	private DefaultMutableTreeNode generateObjectNode(XOCObject object){
		DefaultMutableTreeNode objectNode = new DefaultMutableTreeNode(new TreeEntry(object.getAttributes().get("id").toString()));
		for(String key : object.getAttributes().keySet()){
			if(!key.equals(xConstants.lifecycle)){
				objectNode.add(new DefaultMutableTreeNode(new TreeEntry(key + " : " + object.getAttributes().get(key).toString())));
			}				
		}		
		return  objectNode;
	}
	
	private DefaultMutableTreeNode generateRelationSetNode(List<XOCORelation> relations, String name){
		DefaultMutableTreeNode relationSetNode = new DefaultMutableTreeNode(new TreeEntry(name));
		for (XOCORelation relation : relations) {
			relationSetNode.add(generateRelationNode(relation));
		}
		return relationSetNode;
	}
	
	private DefaultMutableTreeNode generateRelationNode(XOCORelation relation){
		DefaultMutableTreeNode relationNode = new DefaultMutableTreeNode(new TreeEntry(relation.getAttributes().get("id").toString()));
		for(String key : relation.getAttributes().keySet()){
			if(!key.equals(xConstants.lifecycle)){
				relationNode.add(new DefaultMutableTreeNode(new TreeEntry(key + " : " + relation.getAttributes().get(key).toString())));
			}				
		}		
		return  relationNode;
	}
	
	private class TreeEntry {
		private String key;
		private String displaytext;
		
		public TreeEntry(String key) {
			this(key, key);
		}
		
		public TreeEntry(String key, String displaytext) {
			this.key = key;
			this.displaytext = displaytext;
		}
		
		public String getKey() {
			return this.key;
		}
		
		public String getDisplayText() {
			return this.displaytext;
		}
		
		public String toString() {
			return getDisplayText();
		}
	}
}




