package org.processmining.ocbc.logconversion.plugin;

import java.awt.Toolkit;
import java.io.File;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.SortedSet;

import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JPanel;

import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.contexts.uitopia.annotations.Visualizer;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.util.ui.scalableview.ScalableComponent;
import org.processmining.ocbc.log.inspector.panels.XOCLogDashboard;
import org.processmining.ocbc.log.inspector.panels.XOCLogSummary;
import org.processmining.ocbc.log.inspector.panels.XOCLogViewer;
import org.processmining.ocbc.log.io.XocXmlParser;
import org.processmining.ocbc.log.model.XOCLog;
import org.processmining.ocbc.logconversion.panels.XOCLogConversionPanel;
import org.processmining.ocbc.loggeneration.panel.datamodel.DataModelInspector;
import org.processmining.plugins.interactivevisualization.EID;
import org.processmining.plugins.interactivevisualization.InteractiveVisualization;
import org.processmining.plugins.interactivevisualization.InteractivityContext;
import org.processmining.plugins.interactivevisualization.InteractivityManager;



public class XOCLogConvertorPlugin extends JPanel implements ScalableComponent, InteractiveVisualization{
	
	protected XOCLogViewer viewerUI;
	protected XOCLogDashboard dashboardUI;
	protected XOCLogSummary summaryUI;
	protected DataModelInspector dataModelUI;
//	protected LogInfoUI loginfoUI;
	
	// variables used in ScalableComponent interface.
	private final List<UpdateListener> observers = new LinkedList<UpdateListener>();
	
	// variables used in InteractiveVisualization interface.
	private InteractivityContext interactivityContext = new InteractivityContext();
	
	
	protected XOCLog log;
	

	@Plugin(
		name = "XOC Event Correlator",
		parameterLabels = {"XOCLog"},
		returnLabels = { "Event Correlation" },
		returnTypes = { XOCLogConvertorObject.class },
		userAccessible = true,
		help = "Start up editor"
	)
	@UITopiaVariant(
		affiliation = UITopiaVariant.EHV,
		author = "Guangming Li",
		email = "g.li.3@tue.nl"
	)

	public XOCLogConvertorObject logConvertor(UIPluginContext context, XOCLog log) throws Exception{
		
		context.log("XOC log generator starts");
		System.out.println("\n logConvertor");
//		logConvertorPanel(context, log);		       
		return new XOCLogConvertorObject(logConvertorPanel(context, log));
	}
	
	@Plugin(
			name = "XOCLog Convertor Visualization", 
			parameterLabels = {"visualizer"},
			returnTypes = JComponent.class, 
			returnLabels = "Log Convertor Visualization", 
			help = "Converting XOC logs"
	)
	
	@Visualizer
	public JComponent visualizeConvertorPanel(UIPluginContext context, final XOCLogConvertorObject input){
		System.out.println("XOC log Convertor visualizer");
		context.log("welcome to XOC log Convertor visualizer");
		JPanel resultPanel = input.panel;
		return resultPanel;
	}
	
	public JPanel logConvertorPanel(UIPluginContext context, XOCLog log) throws Exception {
		System.out.println("\n logConvertorPanel");
		XOCLogConversionPanel logConversion = new XOCLogConversionPanel(context, log);
//		initialize(null);
//		completeGui(null);
		return logConversion;
	}
//	JTextField selectedPath;


	

	
	public static void main(String[] args) throws Exception  
    {  
		//pop up a dialogue to select the XOC log
/*		IOFileDialogue fileImporter = new IOFileDialogue();
    	fileImporter.show("XOC Log (.xoc)", "XOC", JFileChooser.FILES_ONLY, "Select a XOC Log", null);
    	File file = fileImporter.getSelectedFile();*/
    	
		File file = new File("D:\\log.xoc");
		if(file != null){
			System.out.println("selected file is not empty!");
//			lastFile = file.getParentFile();
	    	
			XocXmlParser XOCParser = new XocXmlParser();
			XOCParser.parse(file);		
			
			XOCLog inputLog = XOCParser.getLog();
			
			if(inputLog == null) System.out.print("null" );
			System.out.print("\n log.size" + inputLog.size());
			System.out.print("\n log.size" + inputLog.getAllEvents().size());
//			SlickerOpenLogSettings dialogue = new SlickerOpenLogSettings();
				 
			XOCLogConvertorPlugin plugin = new XOCLogConvertorPlugin();
			System.out.print("\n after plunin creation");
			JPanel ret = plugin.logConvertorPanel(null, inputLog);
			JFrame frame = new JFrame();
			frame.setSize(Toolkit.getDefaultToolkit().getScreenSize());
			frame.getContentPane().add(ret);
			frame.setVisible(true);
			frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
			System.out.print("end");
				    		
		}				
    }

	@Override
	public void setManager(InteractivityManager manager) {
		interactivityContext.setManager(manager);
	}
	
	@Override
	public Map<String, SortedSet<EID>> getSelectedElements() {
		throw new UnsupportedOperationException("This visualization does not support communicating selected elements.");
	}
	
	@Override
	public Map<String, SortedSet<EID>> getVisibleElements() {
		throw new UnsupportedOperationException("This visualization does not support communicating visible elements.");
	}
	
	@Override
	public InteractivityManager getManager() {
		return interactivityContext.getManager();
	}
	
	/*
	 * Implementation of Scalable Component interface (non-Javadoc)
	 * @see org.processmining.framework.util.ui.scalableview.ScalableComponent
	 */
	
	@Override
	public JComponent getComponent() {
		return this;
	}
	
	@Override
	public double getScale() {
		return 1;
	}
	
	@Override
	public void setScale(double newScale) {
		//TODO debug code
		System.err.println("Supposed to set the scale to "+newScale);
		
		//TODO set scale
	}
	
	@Override
	public void addUpdateListener(UpdateListener listener) {
		
		if(listener == null) {
			throw new IllegalArgumentException("The UpdateListener cannot be 'null'.");
		}
		
		observers.add(listener);
	}
	
	@Override
	public void removeUpdateListener(UpdateListener listener) {
		
		if(listener == null) {
			throw new IllegalArgumentException("The UpdateListener cannot be 'null'.");
		}
		
		observers.remove(listener);
	}

	public void updateVisualization(boolean logChanged, boolean tagsChanged) {
		// TODO Auto-generated method stub
		
	}	
}
