package org.processmining.ocbc.loggeneration.generator.plugin;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Image;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.contexts.uitopia.annotations.Visualizer;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.ocbc.log.inspector.panels.XOCLogDashboard;
import org.processmining.ocbc.log.inspector.panels.XOCLogSummary;
import org.processmining.ocbc.log.inspector.panels.XOCLogViewer;
import org.processmining.ocbc.log.model.XOCLog;
import org.processmining.ocbc.logconversion.configurations.ConfigurationConstants;
import org.processmining.ocbc.logconversion.configurations.ConfigurationParameters;
import org.processmining.ocbc.loggeneration.panel.datamodel.DataModelInspector;
import org.processmining.ocbc.loggeneration.panel.importer.DataImporterInterface;

import com.fluxicon.slickerbox.components.IconVerticalTabbedPane;



public class XOCLogGeneratorPlugin extends JPanel{		
	@Plugin(
		name = "XOC Log Generator",
		parameterLabels = {},
		returnLabels = { "XOC log generator" },
		returnTypes = { XOCLogGeneratorObject.class },
		userAccessible = true,
		help = "Start up editor"
	)
	@UITopiaVariant(
		affiliation = UITopiaVariant.EHV,
		author = "Guangming Li",
		email = "g.li.3@tue.nl"
	)
	
	public XOCLogGeneratorObject logGeneratorPlugin(UIPluginContext context) throws Exception{		
		context.log("XOC log generator starts");		
		logGeneratorPanel(context);		       
		return new XOCLogGeneratorObject(this);
	}
	
	@Plugin(
			name = "XOCLog Generator Visualization", 
			parameterLabels = {"visualizer"},
			returnTypes = JComponent.class, 
			returnLabels = "Log Visualization", 
			help = "Generator XOC logs"
	)	
	@Visualizer
	public JComponent visualizeLogGenerator(UIPluginContext context, final XOCLogGeneratorObject input){
		System.out.println("XOC log generator visualizer");
		context.log("welcome to XOC log generator visualizer");
		JPanel resultPanel = input.panel;
		return resultPanel;
//		return new JScrollPane(resultPanel);
	}
	
	public JComponent logGeneratorPanel(UIPluginContext context) throws Exception {
		initialize(null);
		completeGui();
		return this;
	}
	
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	protected XOCLogViewer viewerUI;
	protected XOCLogDashboard dashboardUI;
	protected XOCLogSummary summaryUI;
	protected DataModelInspector dataModelUI;
	protected DataImporterInterface dataImporterUI;	
	
	private final static String IMPORTICON = "import.png";
	private final static String CONFIGUREICON = "configure.png";
	
	// icons
	protected Image importIcon = new ImageIcon(XOCLogGeneratorPlugin.class.getResource(IMPORTICON),IMPORTICON).getImage();
	protected Image configureIcon = new ImageIcon(XOCLogGeneratorPlugin.class.getResource(CONFIGUREICON),CONFIGUREICON).getImage();
		
	protected XOCLog log;	
	protected ConfigurationParameters configuration = new ConfigurationParameters();

    
	protected void initialize(String path) {
		setBackground(new Color(40, 40, 40));
		setBorder(BorderFactory.createEmptyBorder());
		setLayout(new BorderLayout());
	}
	
	@SuppressWarnings("unchecked")	
	public void setParameterIntoPanels(ConfigurationParameters configuration) {
		Map<String, String> connectionParameterNameVSValueMap = configuration.getNameVSMapParameterMap().get(ConfigurationConstants.connectionParameterNameVSValueMap);
		    
		Map<String, List<String>> tableVSClassList = configuration.getNameVSMapListParameterMap().get(ConfigurationConstants.TABLEVSCLASSLISTMAP);				
		Map<String, List<String>> tableVSSelectedRowIds = configuration.getNameVSMapListParameterMap().get(ConfigurationConstants.TABLEVSELECTEDROWIDSMAP);			
		Map<String, List<String>> tableColumnVSImpactedTableColumns = configuration.getNameVSMapListParameterMap().get(ConfigurationConstants.TABLECOLUMNVSIMPACTEDTABLECOLUMNSMAP);						
		Map<String, List<String>> activityVSClasses = configuration.getNameVSMapListParameterMap().get(ConfigurationConstants.ACTIVITYVSCLASSESMAP);			
		Map<String, List<String>> classVSPKs = configuration.getNameVSMapListParameterMap().get(ConfigurationConstants.CLASSVSPKSMAP);			
		Map<String, List<String>> classVSFKs = configuration.getNameVSMapListParameterMap().get(ConfigurationConstants.CLASSVSFKSMAP);					
	
		Map<String, Map<String, String>> classVSColumnVSName = configuration.getNameVSMapMapParameterMap().get(ConfigurationConstants.CLASSVSCOLUMNVSNAMEMAP);	
		Map<String, Map<String, String>> tableVSEventChainedChildTableVSFK = configuration.getNameVSMapMapParameterMap().get(ConfigurationConstants.TABLEVSEVENTCHAINEDCHILDTABLEVSFKMAP);	
		Map<String, Map<String, String>> tableVSObjectChainedChildTableVSFK = configuration.getNameVSMapMapParameterMap().get(ConfigurationConstants.TABLEVSOBJECTCHAINEDCHILDTABLEVSFKMAP);			
		Map<String, Map<String, String>> tableClassVSColumnVSConstraint = configuration.getNameVSMapMapParameterMap().get(ConfigurationConstants.TABLECLASSVSCOLUMNVSCONSTRAINTMAP);	
		Map<String, Map<String, String>> activityVSReferredClassVSPath = configuration.getNameVSMapMapParameterMap().get(ConfigurationConstants.ACTIVITYVSREFERREDCLASSVSPATHMAP);	
		
		Map<String, Map<String, List<String>>> tableVSChildTableVSForeignKeyList = configuration.getNameVSMapMapListParameterMap().get(ConfigurationConstants.TABLEVSCHILDTABLEVSFKLISTMAP);			
		Map<String, Map<String, List<String>>> classVSActivityColumnVSAttributeColumns = configuration.getNameVSMapMapListParameterMap().get(ConfigurationConstants.CLASSVSACTIVITYCOLUMNVSATTRIBUTECOLUMNSMAP);
		Map<String, Map<String, List<String>>> tableVSObjectChainedFatherTableVSFKs = configuration.getNameVSMapMapListParameterMap().get(ConfigurationConstants.TABLEVSOBJECTCHAINEDFATHERTABLEVSFKSMAP);
		Map<String, Map<String, List<String>>> tableVSEventChainedFatherTableVSFKs = configuration.getNameVSMapMapListParameterMap().get(ConfigurationConstants.TABLEVSEVENTCHAINEDFATHERTABLEVSFKSMAP);
		
		for(String table : tableVSSelectedRowIds.keySet()){
			System.out.println("table: " + table);
			for(String id : tableVSSelectedRowIds.get(table)){
				System.out.println("id: " + id);
			}
		}
		
		Map<String, List<String>> tableVSFatherTables = new HashMap<String, List<String>>();
		for(String tableName : tableVSChildTableVSForeignKeyList.keySet()){
			for(String chilcTableName : tableVSChildTableVSForeignKeyList.get(tableName).keySet()){
				if(!tableVSFatherTables.containsKey(chilcTableName)){
					tableVSFatherTables.put(chilcTableName, new ArrayList<String>());
				}
				if(!tableVSFatherTables.get(chilcTableName).contains(tableName)){
					tableVSFatherTables.get(chilcTableName).add(tableName);
				}
			}
		}
	
		dataImporterUI.getDatabasePanel().getDatabaseConnectionPanel().setConnectionParameterNameVSValueMap(connectionParameterNameVSValueMap);
		dataImporterUI.getDatabasePanel().getDatabaseTablePanel().setSelectedtTableList(new ArrayList<String>(tableVSClassList.keySet()));
		dataImporterUI.getContentPanel().setTableVSSelectedRowIdsMap(tableVSSelectedRowIds);
		dataModelUI.getActivityImpactingColumnPanel().setTableColumnVSImpactedTableColumnsMap(tableColumnVSImpactedTableColumns);
		dataModelUI.getMappingFKToPKPanel().setTableVSFatherTablesMap(tableVSFatherTables);
		dataModelUI.getMappingFKToPKPanel().setTableVSChildTableVSForeignKeyListMap(tableVSChildTableVSForeignKeyList);
		dataModelUI.getTableStructurePanel().setTableVSClassListMap(tableVSClassList);
		dataModelUI.getTableStructurePanel().setClassVSPKColumnsMap(classVSPKs);
		dataModelUI.getTableStructurePanel().setClassVSFKColumnsMap(classVSFKs);
		dataModelUI.getTableStructurePanel().setClassVSColumnVSNameMap(classVSColumnVSName);
		dataModelUI.getRelateActivityToClassPanel().setActivityVSClassesMap(activityVSClasses);
		dataModelUI.getTableStructurePanel().setClassVSActivityColumnVSAttributeColumnsMap(classVSActivityColumnVSAttributeColumns);
		dataModelUI.getMappingFKToPKPanel().setTableVSEventChainedChildTableVSFKMap(tableVSEventChainedChildTableVSFK);
		dataModelUI.getMappingFKToPKPanel().setTableVSEventChainedFatherTableVSFKsMap(tableVSEventChainedFatherTableVSFKs);
		dataModelUI.getMappingFKToPKPanel().setTableVSObjectChainedChildTableVSFKMap(tableVSObjectChainedChildTableVSFK);
		dataModelUI.getTableStructurePanel().setTableClassVSColumnVSConstraintMap(tableClassVSColumnVSConstraint);
		dataModelUI.getRelateActivityToClassPanel().setActivityVSReferredClassVSPathMap(activityVSReferredClassVSPath);
		dataModelUI.getMappingFKToPKPanel().setTableVSObjectChainedFatherTableVSFKsMap(tableVSObjectChainedFatherTableVSFKs);
	}


	
	protected void completeGui() throws Exception {
		dataImporterUI = new DataImporterInterface(this);	
		dataModelUI = new DataModelInspector(this);
		// compile root panel
		IconVerticalTabbedPane iconTabs = new IconVerticalTabbedPane(new Color(230, 230, 230, 210), new Color(20, 20,
				20, 160));
		iconTabs.addTab("Import Data", importIcon, new JScrollPane(dataImporterUI));
		iconTabs.addTab("Configure", configureIcon, new JScrollPane(dataModelUI));				              
      
		add(iconTabs, BorderLayout.CENTER);
		revalidate();
		repaint();
	}
	
    public DataImporterInterface getDataImporterUI(){
		return dataImporterUI;
    }
    
    public DataModelInspector getDataModelUI(){
		return dataModelUI;
    }
    	
    public ConfigurationParameters getConfigurationParameters(){
    	return configuration;
    }
    
	public static void main(String[] args) throws Exception{  	
		XOCLogGeneratorPlugin plugin = new XOCLogGeneratorPlugin();
		JFrame frame = new JFrame();
		frame.getContentPane().add(plugin.logGeneratorPanel(null));
		frame.setExtendedState(javax.swing.JFrame.MAXIMIZED_BOTH);
		frame.setVisible(true);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    }
}