package org.processmining.ocbc.loggeneration.mysql.eventlist;

import java.sql.SQLException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.swing.JTable;

import org.processmining.common.dialogue.DoNotShowAgainMessageBox;
import org.processmining.common.table.MyTableModel;
import org.processmining.ocbc.logconversion.configurations.ConfigurationConstants;
import org.processmining.ocbc.logconversion.configurations.ConfigurationParameters;
 

public class CreateEventList{

	private  List<TableEvent> changeList = new ArrayList<TableEvent>();
	private  List<String> eventIdList = new ArrayList<String>();//record the events
	

    //record what the primary key of one table is called in its children table as foreign key
//	Map<String, Map<String, String>> tableVSTableForeignKey = new HashMap<String, Map<String, String>>();
	Map<String, Map<String, List<String>>> tableVSChildTableVSForeignKeyList = new HashMap<String, Map<String, List<String>>>();
    //for one table (name), record the tables (name) it relies on
	private Map<String, List<String>> tableVSFatherTables = new HashMap<String, List<String>>();
    
    //for one table (name), record other child tables (name)
	private Map<String, List<String>> tableVSChildTables = new HashMap<String, List<String>>();
	
    //for one table (name), record other chained child tables (name)
	private Map<String, List<String>> tableVSChainedChildTables = new HashMap<String, List<String>>();
    
    //for one table (table name), which columns (column names) will be considered as event source, e.g., question-[creationdata, closeddate] (events related to objects)
	private Map<String, List<String>> tableVSEventColumns = new HashMap<String, List<String>>();
    
    //In a business process, some happening objects (events), called source objects (events), trigger the occurrences of following objects (events), 
    //e.g., creating order will trigger pick item, deliver items.
    //for one source table (table name), which rows (id names) will be considered as source objects, e.g., question-[181,182]
	private Map<String, List<String>> sourceTableVSRowIds = new HashMap<String, List<String>>();
    
    //for one table event column (table name + column name), which table column will be impacted through which column, table-column-[table-column-intermediacolumn]
    //e.g., question-creationdate-[question-id-id,question-creationdate-id,question-body-id]
    //e.g., order-creationdate-[order-id-id,order-creationdate-id,orderline-id-command...]
	private Map<String, List<String>> tableColumnVSImpactedTableColumns = new HashMap<String, List<String>>();
    
    //assign a change type to each table event column (table name + column name), e.g., "I" for insertion, "U" for update
	private Map<String, String> tableColumnVSChangeType = new HashMap<String, String>();
	
//    //assign a activity to each table event column (table name + column name), e.g., "create project" for "projet-datec"
//	private Map<String, String> tableColumnVSActivity = new HashMap<String, String>();
       
    //assign a activity to each table event column of one class (class name + column name), e.g., "create project" for "project-datec"
	private Map<String,  Map<String, String>> classVSActivitycolumnVSName = new HashMap<String, Map<String, String>>();
    
	 //for one class from a table, assign constraint values to some columns, (Map<table-class, Map<column, value>>,
    //e.g., Map<llx_commande_fournisseur_log-order_created, Map<fk_status, "0">>
	private Map<String, Map<String, String>> tableClassVSColumnVSConstraint = new HashMap<String, Map<String, String>>();
	 
	Map<String, List<String>> tableVSClassList = new HashMap<String, List<String>>();
	
	private Map<String, List<String>> classVSPKs = new HashMap<String, List<String>>();
	
	private Map<String, Map<String, String>> tableVSEventChainedChildTableVSFK = new HashMap<String, Map<String, String>>();
	
//	private Map<String, Map<String, String>> tableVSEventChainedFatherTableVSFKs = new HashMap<String, Map<String, String>>();
	private Map<String, Map<String, List<String>>> tableVSEventChainedFatherTableVSFKs = new HashMap<String, Map<String, List<String>>>();
	
    private String schema;

    private Map<String, JTable> nameVSJtable = new HashMap<String, JTable>();
    
    protected ConfigurationParameters configuration = new ConfigurationParameters();
	
    private DoNotShowAgainMessageBox messageBox = new DoNotShowAgainMessageBox();
    
	@SuppressWarnings("unchecked")
	public CreateEventList(ConfigurationParameters configuration){
		this.configuration = configuration;
		
		this.schema = configuration.getNameVSMapParameterMap().get(ConfigurationConstants.connectionParameterNameVSValueMap).get(ConfigurationConstants.SCHEMA);
		
		if(configuration.getNameVSObjectParameterMap().get(ConfigurationConstants.nameVSJtableMap) != null)
			this.nameVSJtable = (Map<String, JTable>) configuration.getNameVSObjectParameterMap().get(ConfigurationConstants.nameVSJtableMap);		
		
		if(configuration.getNameVSMapMapListParameterMap().get(ConfigurationConstants.TABLEVSCHILDTABLEVSFKLISTMAP) != null)
			this.tableVSChildTableVSForeignKeyList = configuration.getNameVSMapMapListParameterMap().get(ConfigurationConstants.TABLEVSCHILDTABLEVSFKLISTMAP);		
		if(configuration.getNameVSMapListParameterMap().get(ConfigurationConstants.TABLEVSFATHERTABLESMAP) != null)
			this.tableVSFatherTables = configuration.getNameVSMapListParameterMap().get(ConfigurationConstants.TABLEVSFATHERTABLESMAP);		
		if(configuration.getNameVSMapListParameterMap().get(ConfigurationConstants.tableVSChildTables) != null)
			this.tableVSChildTables = configuration.getNameVSMapListParameterMap().get(ConfigurationConstants.tableVSChildTables);		
		if(configuration.getNameVSMapListParameterMap().get(ConfigurationConstants.tableVSChainedChildTables) != null)
			this.tableVSChainedChildTables = configuration.getNameVSMapListParameterMap().get(ConfigurationConstants.tableVSChainedChildTables);			
		if(configuration.getNameVSMapListParameterMap().get(ConfigurationConstants.tableVSEventColumns) != null)
			this.tableVSEventColumns = configuration.getNameVSMapListParameterMap().get(ConfigurationConstants.tableVSEventColumns);			
		if(configuration.getNameVSMapListParameterMap().get(ConfigurationConstants.TABLECOLUMNVSIMPACTEDTABLECOLUMNSMAP) != null)
			this.tableColumnVSImpactedTableColumns = configuration.getNameVSMapListParameterMap().get(ConfigurationConstants.TABLECOLUMNVSIMPACTEDTABLECOLUMNSMAP);			
		if(configuration.getNameVSMapListParameterMap().get(ConfigurationConstants.TABLEVSELECTEDROWIDSMAP) != null)
			this.sourceTableVSRowIds = configuration.getNameVSMapListParameterMap().get(ConfigurationConstants.TABLEVSELECTEDROWIDSMAP);
		if(configuration.getNameVSMapListParameterMap().get(ConfigurationConstants.TABLEVSCLASSLISTMAP) != null)
			this.tableVSClassList = configuration.getNameVSMapListParameterMap().get(ConfigurationConstants.TABLEVSCLASSLISTMAP);				
		if(configuration.getNameVSMapListParameterMap().get(ConfigurationConstants.CLASSVSPKSMAP) != null)
			this.classVSPKs = configuration.getNameVSMapListParameterMap().get(ConfigurationConstants.CLASSVSPKSMAP);	
		
		if(configuration.getNameVSMapMapParameterMap().get(ConfigurationConstants.classVSActivityColumnVSName) != null)
			this.classVSActivitycolumnVSName = configuration.getNameVSMapMapParameterMap().get(ConfigurationConstants.classVSActivityColumnVSName);	 			
		if(configuration.getNameVSMapMapParameterMap().get(ConfigurationConstants.TABLECLASSVSCOLUMNVSCONSTRAINTMAP) != null)
			this.tableClassVSColumnVSConstraint = configuration.getNameVSMapMapParameterMap().get(ConfigurationConstants.TABLECLASSVSCOLUMNVSCONSTRAINTMAP);	
		if(configuration.getNameVSMapMapParameterMap().get(ConfigurationConstants.TABLEVSEVENTCHAINEDCHILDTABLEVSFKMAP) != null)
			this.tableVSEventChainedChildTableVSFK = configuration.getNameVSMapMapParameterMap().get(ConfigurationConstants.TABLEVSEVENTCHAINEDCHILDTABLEVSFKMAP);	
		if(configuration.getNameVSMapMapListParameterMap().get(ConfigurationConstants.TABLEVSEVENTCHAINEDFATHERTABLEVSFKSMAP) != null){
			System.out.println("input tableVSEventChainedFatherTableVSFKs");
			this.tableVSEventChainedFatherTableVSFKs = configuration.getNameVSMapMapListParameterMap().get(ConfigurationConstants.TABLEVSEVENTCHAINEDFATHERTABLEVSFKSMAP);	
		}
	}	
	
	public List<TableEvent> rankChangeSequence() throws ParseException{
		List<TableEvent> rankedChangeList = new ArrayList<TableEvent>();
		//get source tables
		SimpleDateFormat dateTime = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");  	
		SimpleDateFormat date = new SimpleDateFormat("yyyy-MM-dd"); 
		while(changeList.size()>0){
			Date earlistTime= dateTime.parse("2020-01-01 01:00:00");
			int  earlistTimeIndex = 0;
			for(int i=0; i<changeList.size(); i++){
				String time = changeList.get(i).time;	
				if(time == null || time.equals("null") || time.equals("")){
					time = "2019-01-01 01:00:00";
				}
				Date timestamp;
				if(time.length() > 12){
					timestamp = dateTime.parse(time);
				}
				else{
					System.out.println("\n time in rankChangeSequence: " + time);	
					timestamp = date.parse(time);	
				}
							 			
				if(timestamp.before(earlistTime)){
					earlistTimeIndex = i;
					earlistTime = timestamp;
				}
			}			
			System.out.println("\n rank time: " + changeList.get(earlistTimeIndex).time);			
			rankedChangeList.add(changeList.get(earlistTimeIndex));
			changeList.remove(earlistTimeIndex);
			
		}		
		changeList = rankedChangeList;
		return rankedChangeList;
	}
	
	public void createChangeSequence() throws SQLException{
		//get source tables
		for(String sourceTable : sourceTableVSRowIds.keySet()){
			//get source objects from source tables
			for(String sourceObjectId : sourceTableVSRowIds.get(sourceTable)){
				addChangesRelatedToObject(sourceTable, sourceObjectId);								
			}				
		}			
	}
	
	public void addChangesRelatedToObject(String sourceTable,  String objectId) throws SQLException{
		String className = getObjectClassThroughCheckingConstraint(sourceTable, objectId, schema);
		boolean objectValidFlag;
		if(className.equals("")){
			objectValidFlag = false;
		}
		else
			objectValidFlag = true;

		
		if(objectValidFlag && !eventIdList.contains(sourceTable + "-" + objectId)){
//		if(!eventIdList.contains(sourceTable + "-" + objectId)){
			eventIdList.add(sourceTable + "-" + objectId);  		
    		//add changes from the source table
			if(tableVSEventColumns.containsKey(sourceTable)){
//				eventIdList.add(sourceTable + "-" + objectId);
				for(String eventColumn : tableVSEventColumns.get(sourceTable)){
					System.out.println("\n className: " + className);   				
    				String rowId = classVSPKs.get(className).get(0);     
    				for(String name : nameVSJtable.keySet()){
    					System.out.println("\n nameVSJtable name: " + name);  
    				}
    				if(nameVSJtable.containsKey(sourceTable)){
        				MyTableModel tableModel = (MyTableModel) nameVSJtable.get(sourceTable).getModel();
        				System.out.println("\n nameVSJtable contain: " + sourceTable);  
        				List<Object> columnValueList= tableModel.getColumnValuesWithColumnConstraint(eventColumn, rowId, objectId);
    	    			if(columnValueList.size() > 0){
    		    	        TableEvent change = new TableEvent();
    		    			change.table = sourceTable;
    		    			change.className = className;
    		    			change.column = eventColumn;
    		    			change.id = Integer.parseInt(objectId);
    		    			change.time = (String) columnValueList.get(0);
    		    			change.changeId = sourceTable + "-" + eventColumn + "-" + objectId;
    		    			change.activity = classVSActivitycolumnVSName.get(className).get(eventColumn);
    		    			
    		    			if(change.time != null) {
    		    				changeList.add(change);
    		    				System.out.println("\n add change activity: " + change.activity);  
    		    			}
    		    	    }
    				}
	    		}
			}
			
    		
			//add changes from child tables-tableVSChainedChildTables
			for(String table : tableVSChainedChildTables.keySet()){
				for(String childTable : tableVSChainedChildTables.get(table)){
					System.out.println("table-chainedchildtable: " + table + "-" + childTable);
				}
			}
			System.out.println("********up chained****************");
			for(String table : tableVSChildTables.keySet()){
				for(String childTable : tableVSChildTables.get(table)){
					System.out.println("table-childtable: " + table + "-" + childTable);
				}
			}
//			System.exit(1);
			
			//add changes from event-chained child tables		
    		if(tableVSEventChainedChildTableVSFK.containsKey(sourceTable)){
    			for(String childTable : tableVSEventChainedChildTableVSFK.get(sourceTable).keySet()){
    				System.out.println("\n tableVSChainedChildTableVSFK->child table: " + childTable);
    				//if the table have corresponding classes
    				if(tableVSClassList.containsKey(childTable) && tableVSClassList.get(childTable).size()>0){
    					List<String> childClassNameList = tableVSClassList.get(childTable);
        				//currently, we assume all the classes in one table have the same pks
        				for(int i=0; i<1; i++){
        					String childClassName = childClassNameList.get(0);
        					String rowId = classVSPKs.get(childClassName).get(0);   					
            				if(nameVSJtable.containsKey(childTable)){
                				MyTableModel tableModel = (MyTableModel) nameVSJtable.get(childTable).getModel();
                				List<Object> columnValueList= tableModel.getColumnValuesWithColumnConstraint(rowId, tableVSEventChainedChildTableVSFK.get(sourceTable).get(childTable), objectId);
            	    			
                				for(Object columnValue : columnValueList){
                					addChangesRelatedToObject(childTable, (String) columnValue); 
                				}
            				}
        				}			
    				}
    				else{
    					messageBox.showConfirmDialog(null, "Warn! Table " + childTable + " has no classes and no corresponding events are created"
    							+ " in the log!");
    				}
    			}
    		}					
    		
    		//add changes from event-chained parent tables, to do
    		if(tableVSEventChainedFatherTableVSFKs.containsKey(sourceTable)){
    			System.out.println("\n table: " + sourceTable);  
    			for(String fatherTable : tableVSEventChainedFatherTableVSFKs.get(sourceTable).keySet()){
    				System.out.println("\n father table: " + fatherTable);    				   				
    				String rowId = classVSPKs.get(className).get(0);    				    				
    				//get father objects related to the object through each fk-2016-Oct-28
//    				for(String fk : tableVSChildTableVSForeignKeyList.get(fatherTable).get(sourceTable)){
    				for(String fk : tableVSEventChainedFatherTableVSFKs.get(sourceTable).get(fatherTable)){
    					//get the id of the father object   
    					if(nameVSJtable.containsKey(sourceTable)){
            				MyTableModel tableModel = (MyTableModel) nameVSJtable.get(sourceTable).getModel();
            				List<Object> columnValueList= tableModel.getColumnValuesWithColumnConstraint(fk, rowId, objectId);       	    			
            				for(Object columnValue : columnValueList){
            					addChangesRelatedToObject(fatherTable, (String) columnValue); 
            				}
        				}
    				}                   
    			}
    		}
    		
//    		if(tableVSFatherTables.containsKey(sourceTable)){
//    			System.out.println("\n table: " + sourceTable);  
//    			for(String fatherTable : tableVSFatherTables.get(sourceTable)){
//    				System.out.println("\n father table: " + fatherTable);    				   				
//    				String rowId = classVSPKs.get(className).get(0);    				    				
//    				//get father objects related to the object through each fk-2016-Oct-28
//    				for(String fk : tableVSTableForeignKeyList.get(fatherTable).get(sourceTable)){
//    					//get the id of the father object   
//    					if(nameVSJtable.containsKey(sourceTable)){
//            				MyTableModel tableModel = (MyTableModel) nameVSJtable.get(sourceTable).getModel();
//            				List<Object> columnValueList= tableModel.getColumnValuesWithColumnConstraint(fk, rowId, objectId);       	    			
//            				for(Object columnValue : columnValueList){
//            					addChangesRelatedToObject(fatherTable, (String) columnValue); 
//            				}
//        				}
//    				}                   
//    			}
//    		}
    	}               	
    }
	
	/*
	 * check which class the input object (id, table) belongs to;
	 * if there exist class, return class name;
	 * if not, return ""
	 */
	public String getObjectClassThroughCheckingConstraint(String table, String id, String schema) throws SQLException{
		String className = "";

		boolean objectSatisfyingConstraintFlag = false;
		boolean existCorrespondingConstraint = false;
		for(String tableClassComplex : tableClassVSColumnVSConstraint.keySet()){
			String tableName = tableClassComplex.split("-")[0];
			className = tableClassComplex.split("-")[1];
			if(tableName.equals(table)){
				existCorrespondingConstraint = true;
				Map<String, String> columnVSConstraint = tableClassVSColumnVSConstraint.get(tableClassComplex);
				boolean satisfyingAllConstraintsOfOneClassFlag = true;
				for(String column : columnVSConstraint.keySet()){					
					String constraintValue = columnVSConstraint.get(column);
					//if the constraint value is not empty (if empty, it indicate there is no constraint)
					if(!constraintValue.equals("") && !constraintValue.equals("null")){
						
						String rowId = classVSPKs.get(className).get(0);
						
						if(nameVSJtable.containsKey(table)){
            				MyTableModel tableModel = (MyTableModel) nameVSJtable.get(table).getModel();
            				List<Object> columnValueList= tableModel.getColumnValuesWithColumnConstraint(column, rowId, id);       	    			
            				Object columnValue =  columnValueList.get(0);
            				if(columnValue == null ||  !((String)columnValue).contains(constraintValue)){
            					satisfyingAllConstraintsOfOneClassFlag = false;
        					}
        				}						
					}
					
				}
				if(satisfyingAllConstraintsOfOneClassFlag){
					className = tableClassComplex.split("-")[1];
					objectSatisfyingConstraintFlag = true;
					break;
				}
					
			}
		}
		//there not exist constraint for the object
		//which means (1)there is just one class for the table
		//(2)there are multiple classes without constraints to distinguish, we select the first class name
		if(!existCorrespondingConstraint){	
			if(tableVSClassList.containsKey(table) && tableVSClassList.get(table).size() > 0){
				className = tableVSClassList.get(table).get(0);
			}
			else{
				className = "";
				messageBox.showConfirmDialog(null, "Warn! Table " + table + " has no classes and no corresponding objects are created"
						+ " in the log!");
			}			
		}
		//there exist constraint for the object and the object satisfy all constraints
		else if(objectSatisfyingConstraintFlag){
//			objectValidFlag = true;
		}		
		else{
			className = "";
//			objectValidFlag = false;
		}
			
		
		return className;
	}
}
