package org.processmining.ocbc.loggeneration.panel.datamodel;
 
import java.awt.Cursor;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;

import org.processmining.common.dialogue.IOFileDialogue;
import org.processmining.ocbc.log.model.impl.xConstants;
import org.processmining.ocbc.logconversion.configurations.ConfigurationConstants;
import org.processmining.ocbc.logconversion.configurations.ConfigurationParameters;
import org.processmining.ocbc.logconversion.configurations.WriteConfigurationsIntoXML;
import org.processmining.ocbc.loggeneration.mysql.dolibarr.GenerateXOCLog;
import org.processmining.ocbc.loggeneration.panel.importer.DataImporterInterface;
 
/**
 * @version 1.0
 * @author liangdong
 *
 */
public class OutputPanel extends JPanel{
 
    /**
     * 
     */
    private static final long serialVersionUID = 1L;

 
    private File lastLogFile;
    private ConfigurationParameters configuration = new ConfigurationParameters();
    private DataImporterInterface dataImporterUI;	
    private DataModelInspector dataModelUI;
    private JRadioButton simpleVersion = new JRadioButton("simple");
    private JRadioButton totalVersion = new JRadioButton("total");
    private JLabel versionLabel = new JLabel("log format: ");
    private String logFormat = xConstants.UPDATE;
    
    public OutputPanel(DataModelInspector fatherPanel) {
    	this.dataImporterUI = fatherPanel.getLogGenerator().getDataImporterUI();
    	this.dataModelUI = fatherPanel;
    	
    	JButton generateLogButton = new JButton("output log to disk");
    	JButton outputConfigurationFileButton = new JButton("output configuration");
    	JButton saveLogButton = new JButton("save log in ProM");

//    	JPanel versionPanel = new JPanel();  	
//    	versionPanel.add(simpleVersion);
//    	versionPanel.add(totalVersion);
//    	versionPanel.setMaximumSize(new Dimension(300,30));
    	simpleVersion.setSelected(true);
    	totalVersion.setSelected(false);
      	
    	this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
    	this.add(outputConfigurationFileButton);
    	this.add(Box.createVerticalStrut(50));
    	this.add(versionLabel);
    	this.add(Box.createVerticalStrut(10));
    	this.add(simpleVersion);
    	this.add(Box.createVerticalStrut(10));
    	this.add(totalVersion);
    	this.add(generateLogButton);
    	this.add(Box.createVerticalStrut(10));
    	this.add(saveLogButton);
    	this.setBorder(BorderFactory.createTitledBorder("output"));
    	
    	simpleVersion.addItemListener(new ItemListener() {		
			public void itemStateChanged(ItemEvent e) {
				if (e.getStateChange() == ItemEvent.SELECTED) {
					logFormat = xConstants.UPDATE;
					totalVersion.setSelected(false);
			    }
			    else if (e.getStateChange() == ItemEvent.DESELECTED) {
			    	totalVersion.setSelected(true);
			    }
			}
		});
    	
    	totalVersion.addItemListener(new ItemListener() {		
			public void itemStateChanged(ItemEvent e) {
				if (e.getStateChange() == ItemEvent.SELECTED) {
					logFormat = xConstants.TOTAL;
					simpleVersion.setSelected(false);
					System.out.println("ItemEvent.SELECTED");
			    }
			    else if (e.getStateChange() == ItemEvent.DESELECTED) {
			    	simpleVersion.setSelected(true);
			    	System.out.println("ItemEvent.DESELECTED");
			    }
			}
		});
    	
    	generateLogButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				System.out.println("generate logs");
				IOFileDialogue fileImporter = new IOFileDialogue();				
            	fileImporter.show("XOC Log (.xoc)", "xoc", JFileChooser.FILES_ONLY, "Output", lastLogFile);
            	File file = fileImporter.getSelectedFile();
				if(file != null){
					dataModelUI.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
					lastLogFile = file.getParentFile();
					getParameterFormPanels(configuration);
					GenerateXOCLog generateXOCLog = null;								
					generateXOCLog = new GenerateXOCLog(configuration);    
					String fileName = file.getAbsolutePath();
 					if(!fileName.contains(".")){
 						fileName = fileName + ".xoc";
 					}
					try {
						generateXOCLog.generate(fileName, logFormat);
						//change the cursor to default cursor
						dataModelUI.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
						JOptionPane.showMessageDialog(null, "Succeed generating log: " + fileName + " !"); 		
					} catch (Exception e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
						//change the cursor to default cursor
						dataModelUI.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
						JOptionPane.showMessageDialog(null, "Wrong! Fail generating log!"); 	
					}	 							
				}
			}
		});        
        
        
        outputConfigurationFileButton.addActionListener(new ActionListener() {
 			@Override
 			public void actionPerformed(ActionEvent arg0) { 									
				System.out.println("generate configuration file");	
				IOFileDialogue fileImporter = new IOFileDialogue();
            	fileImporter.show("Configuration File (.xml)", "xml", JFileChooser.FILES_ONLY, "Output", lastLogFile);
            	File file = fileImporter.getSelectedFile();
				if(file != null){
					getParameterFormPanels(configuration);
					lastLogFile = file.getParentFile();
					getParameterFormPanels(configuration);						 
					String fileName = file.getAbsolutePath();
					WriteConfigurationsIntoXML writer = new WriteConfigurationsIntoXML(configuration);
					if(!fileName.contains(".")){
 						fileName = fileName + ".xml";
 					}										
					try {
						writer.createXml(fileName); 
						JOptionPane.showMessageDialog(null, "Succeed generating configuration file: " + fileName + " !"); 		
					} catch (Exception e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
						JOptionPane.showMessageDialog(null, "Wrong! Fail generating configuration file!"); 	
					}	
				}
				
 			}
 		});
               

    }
 
    public void removeInexistentTable(Map<String, List<String>> tableVSRelatedTable, List<String> existentTableList){
    	Map<String, List<String>> tableVSRelatedTableExistent = new HashMap<String, List<String>>();
    	for(String table: tableVSRelatedTable.keySet()){
			if(existentTableList.contains(table)){			
				tableVSRelatedTableExistent.put(table, new ArrayList<String>());
				for(String relatedTable : tableVSRelatedTable.get(table)){
					if(existentTableList.contains(relatedTable)){
						tableVSRelatedTableExistent.get(table).add(relatedTable);
					}
				}				
			}
		}
    	tableVSRelatedTable = tableVSRelatedTableExistent;
    }
    
	public void getParameterFormPanels(ConfigurationParameters configuration) {
		List<String> existentTableList = dataImporterUI.getDatabasePanel().getDatabaseTablePanel().getSelectedTables();
		removeInexistentTable(dataModelUI.getMappingFKToPKPanel().getTableVSFatherTablesMap(), existentTableList);
		//related to dataImporterUI		
		configuration.getNameVSMapParameterMap().put(ConfigurationConstants.connectionParameterNameVSValueMap, dataImporterUI.getDatabasePanel().getDatabaseConnectionPanel().getConnectionParameterNameVSValueMap());		
		configuration.getNameVSMapListParameterMap().put(ConfigurationConstants.TABLEVSELECTEDROWIDSMAP, dataImporterUI.getContentPanel().getTableVSSelectedRowIdsMap());		
		//related to dataModelUI->getTableStructurePanel
		configuration.getNameVSMapListParameterMap().put(ConfigurationConstants.TABLEVSCLASSLISTMAP, dataModelUI.getTableStructurePanel().getTableVSClassListMap());	
		configuration.getNameVSMapMapParameterMap().put(ConfigurationConstants.CLASSVSCOLUMNVSNAMEMAP, dataModelUI.getTableStructurePanel().getClassVSColumnVSNameMap());	
		configuration.getNameVSMapMapListParameterMap().put(ConfigurationConstants.CLASSVSACTIVITYCOLUMNVSATTRIBUTECOLUMNSMAP, dataModelUI.getTableStructurePanel().getClassVSActivityColumnVSAttributesColumnsMap());
		configuration.getNameVSMapMapParameterMap().put(ConfigurationConstants.TABLECLASSVSCOLUMNVSCONSTRAINTMAP, dataModelUI.getTableStructurePanel().getTableClassVSColumnVSConstraintMap());		
		//currently, there just is one pk, maybe in the future, there are multiple pks (such as unique keys)
		configuration.getNameVSMapListParameterMap().put(ConfigurationConstants.CLASSVSPKSMAP, dataModelUI.getTableStructurePanel().getClassVSPKColumnsMap());	
		configuration.getNameVSMapListParameterMap().put(ConfigurationConstants.CLASSVSFKSMAP, dataModelUI.getTableStructurePanel().getClassVSFKColumnsMap());//currently, it is not used
		//related to dataModelUI->getActivityImpactingColumnPanel
		configuration.getNameVSMapListParameterMap().put(ConfigurationConstants.TABLECOLUMNVSIMPACTEDTABLECOLUMNSMAP, dataModelUI.getActivityImpactingColumnPanel().getTableColumnVSImpactedTableColumnsMap());		
		//related to dataModelUI->getMappingFKToPKPanel
		configuration.getNameVSMapMapParameterMap().put(ConfigurationConstants.TABLEVSEVENTCHAINEDCHILDTABLEVSFKMAP, dataModelUI.getMappingFKToPKPanel().getTableVSEventChainedChildTableVSFKMap());
		configuration.getNameVSMapMapParameterMap().put(ConfigurationConstants.TABLEVSOBJECTCHAINEDCHILDTABLEVSFKMAP, dataModelUI.getMappingFKToPKPanel().getTableVSObjectChainedChildTableVSFKMap());
		configuration.getNameVSMapMapListParameterMap().put(ConfigurationConstants.TABLEVSEVENTCHAINEDFATHERTABLEVSFKSMAP, dataModelUI.getMappingFKToPKPanel().getTableVSEventChainedFatherTableVSFKsMap());
		configuration.getNameVSMapMapListParameterMap().put(ConfigurationConstants.TABLEVSOBJECTCHAINEDFATHERTABLEVSFKSMAP, dataModelUI.getMappingFKToPKPanel().getTableVSObjectChainedFatherTableVSFKsMap());		
		configuration.getNameVSMapListParameterMap().put(ConfigurationConstants.TABLEVSFATHERTABLESMAP, dataModelUI.getMappingFKToPKPanel().getTableVSFatherTablesMap());		
		configuration.getNameVSMapMapListParameterMap().put(ConfigurationConstants.TABLEVSCHILDTABLEVSFKLISTMAP, dataModelUI.getMappingFKToPKPanel().getTableVSChildTableVSForeignKeyListMap());
		//related to dataModelUI->getRelateActivityToClassPanel
		configuration.getNameVSMapMapParameterMap().put(ConfigurationConstants.ACTIVITYVSREFERREDCLASSVSPATHMAP, dataModelUI.getRelateActivityToClassPanel().getActivityVSReferredClassVSPathMap());
		configuration.getNameVSMapListParameterMap().put(ConfigurationConstants.ACTIVITYVSCLASSESMAP, dataModelUI.getRelateActivityToClassPanel().getActivityVSClassesMap());	
		
		//build the classVSActivityColumnVSName
		Map<String, Map<String, String>> classVSColumnVSName = dataModelUI.getTableStructurePanel().getClassVSColumnVSNameMap();
		Map<String, Map<String, List<String>>> classVSActivityColumnVSAttributeColumns = dataModelUI.getTableStructurePanel().getClassVSActivityColumnVSAttributesColumnsMap();

	    //assign a activity to each table event column of one class (class name + column name), e.g., "create project" for "project-datec"
	    Map<String,  Map<String, String>> classVSActivitycolumnVSName = new HashMap<String, Map<String, String>>();		    
		for(String className : classVSActivityColumnVSAttributeColumns.keySet()){
			for(String activityColumnName : classVSActivityColumnVSAttributeColumns.get(className).keySet()){
				String activityName = classVSColumnVSName.get(className).get(activityColumnName);
				if(classVSActivitycolumnVSName.get(className) == null){
					classVSActivitycolumnVSName.put(className, new HashMap<String, String>());
				}
				classVSActivitycolumnVSName.get(className).put(activityColumnName, activityName);
				
			}
		}
		configuration.getNameVSMapMapParameterMap().put(ConfigurationConstants.classVSActivityColumnVSName, classVSActivitycolumnVSName);	
		
		//build the tableVSChildTables
		Map<String, Map<String, List<String>>> tableVSChildTableVSForeignKeyList = dataModelUI.getMappingFKToPKPanel().getTableVSChildTableVSForeignKeyListMap();
		Map<String, List<String>> tableVSChildTables = new HashMap<String, List<String>>();
		for(String table : tableVSChildTableVSForeignKeyList.keySet()){		
			tableVSChildTables.put(table, new ArrayList<String>(tableVSChildTableVSForeignKeyList.get(table).keySet()));
		}
		configuration.getNameVSMapListParameterMap().put(ConfigurationConstants.tableVSChildTables, tableVSChildTables);	
		
		//build the tableVSChildTables
		Map<String, Map<String, String>> tableVSChainedChildTableVSFK = dataModelUI.getMappingFKToPKPanel().getTableVSEventChainedChildTableVSFKMap();
		Map<String, List<String>> tableVSChainedChildTables = new HashMap<String, List<String>>();
		for(String table : tableVSChainedChildTableVSFK.keySet()){
			tableVSChainedChildTables.put(table, new ArrayList<String>(tableVSChainedChildTableVSFK.get(table).keySet()));			
		}		
		configuration.getNameVSMapListParameterMap().put(ConfigurationConstants.tableVSChainedChildTables, tableVSChainedChildTables);	
		
		//build tableVSEventColumns map
		Map<String, List<String>> tableVSEventColumns = new HashMap<String, List<String>>();
		Map<String, List<String>> tableColumnVSImpactedTableColumns = dataModelUI.getActivityImpactingColumnPanel().getTableColumnVSImpactedTableColumnsMap();
		for(String tableActivityColumnComplex : tableColumnVSImpactedTableColumns.keySet()){
			if(tableColumnVSImpactedTableColumns.get(tableActivityColumnComplex).size() > 0){
				String[] tableActivityColumnSet = tableActivityColumnComplex.split("-");
				String tableName = tableActivityColumnSet[0];
				String activityColumnName = tableActivityColumnSet[1];
				if(!tableVSEventColumns.containsKey(tableName)){
					tableVSEventColumns.put(tableName, new ArrayList<String>());
				}			
				
				if(!tableVSEventColumns.get(tableName).contains(activityColumnName)){
					tableVSEventColumns.get(tableName).add(activityColumnName);
				}
			}				
		}		
		configuration.getNameVSMapListParameterMap().put(ConfigurationConstants.tableVSEventColumns, tableVSEventColumns);			
		configuration.getNameVSObjectParameterMap().put(ConfigurationConstants.nameVSJtableMap, dataImporterUI.getNameVSJtableMap());	
	}	
 
}