package org.processmining.ocbc.loggeneration.panel.importer;
 
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import org.processmining.common.dialogue.IOFileDialogue;
import org.processmining.common.frame.ComboBoxRenderer;
import org.processmining.ocbc.logconversion.configurations.ConfigurationConstants;
import org.processmining.ocbc.logconversion.configurations.ConfigurationParameters;
import org.processmining.ocbc.logconversion.configurations.ReadConfigurationsFromXML;
 
/**
 * @version 1.0
 * @author liangdong
 *
 */
public class SelectingConfigurationPanel extends JPanel implements ListSelectionListener{
 
    /**
     * 
     */
    private static final long serialVersionUID = 1L;
    private JComboBox<String> configurationFileBox = new JComboBox<String>();
    
    //record the file name (with suffix) and the table list in the file
//    private Map<String, List<String>> fileNameVSTableList = new HashMap<String, List<String>>();
    
    //record the file name (with suffix) and absolute path
    private Map<String, String> fileNameVSAbsolutePath = new HashMap<String, String>();
    
    //record the file name (with suffix) and the selected/non-selected table (the configuration related to the
    //selected tables will be load in the data model) list in the file
    private Map<String, List<String>> fileNameVSSelectedTableList = new HashMap<String, List<String>>();
    private Map<String, List<String>> fileNameVSNonselectedTableList = new HashMap<String, List<String>>();
    
    private String tablename = "";   
    private File lastFile;
    
//    private List<String> tableList;
    private JList<String> sourceList;
    private DefaultListModel<String> sourceListModel;
    private DefaultListModel<String> targetListModel;
    private JList<String> targetList;
    private JLabel sourceLabel;
    private JLabel targetLabel;
 
    private DataImporterInterface fatherPanel;
    
    private List<String> selectedTableList = new ArrayList<String>();
	int screenHeight = (int) Toolkit.getDefaultToolkit().getScreenSize().getHeight();
	int screenWidth = (int) Toolkit.getDefaultToolkit().getScreenSize().getWidth();
    private String selectedListPrefixName = "child configuration for tables: ";
    private String nonselectedListPrefixName = "selected child configuration: ";
    
    public SelectingConfigurationPanel(DataImporterInterface fatherPanel) {
    	this.fatherPanel = fatherPanel;
    	JPanel generatedPanel = generateTableSelectionPanel();
    	add(generatedPanel);
    	this.setBorder(BorderFactory.createTitledBorder("import configuration file(s)"));  
    }
 
    public SelectingConfigurationPanel(DataImporterInterface fatherPanel, int width, int height) {
    	this.screenWidth = width;
    	this.screenHeight = height;
    	this.fatherPanel = fatherPanel;
    	JPanel generatedPanel = generateTableSelectionPanel();
    	add(generatedPanel);
    	this.setBorder(BorderFactory.createTitledBorder("import configuration file(s)"));  
    }
 
    public String getTablename() {
        return tablename;
    }
 
    public void setTablename(String tablename) {
        this.tablename = tablename;
    }
 
    public void setSize(int width, int height){
    	this.screenWidth = width;
    	this.screenHeight = height;
    }
 
  //used to configure tables by file
    public void setSelectedtTableList(List<String> selectedTableList) {
        this.selectedTableList = selectedTableList;
    }
    
 
    public void polulateNonSelectedList(List<String> list){
		if(list != null){
			sourceListModel.removeAllElements();
			for(String tableName : list){
				sourceListModel.addElement(tableName);
			}
		}
		sourceList.setModel(sourceListModel);
    }
    
    public JPanel generateTableSelectionPanel(){	
    	JPanel middlePanel = new JPanel();
	    sourceLabel = new JLabel(selectedListPrefixName);
		sourceLabel.setAlignmentY(Component.TOP_ALIGNMENT);
		sourceLabel.setBorder(BorderFactory.createEmptyBorder(4, 5, 0, 5));
		//build source list
		sourceListModel = new DefaultListModel<String>();

		sourceList = new JList<String>(sourceListModel);	
		sourceList.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
		sourceList.setVisibleRowCount(5);
		//build source list panel
		JScrollPane sourceListScroller = new JScrollPane(sourceList);
		sourceListScroller.setPreferredSize(new Dimension(screenWidth/6, screenHeight/3));
		sourceListScroller.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		sourceListScroller.setAlignmentY(Component.TOP_ALIGNMENT);
		JPanel sourceListPanel = new JPanel();
		sourceListPanel.setLayout(new BoxLayout(sourceListPanel,BoxLayout.Y_AXIS));
		sourceListPanel.add(sourceLabel);
		sourceListPanel.add(sourceListScroller);
		sourceListPanel.setAlignmentY(Component.TOP_ALIGNMENT);
		sourceListPanel.setBorder(BorderFactory.createEmptyBorder(0, 0, 0, 30));
		middlePanel.add(sourceListPanel);
				
		//build operating button panel
		JPanel buttonPanel = new JPanel();
		JButton toTargetButton = new JButton("    >    ");	
		JButton toSourceButton = new JButton("    <    ");		
		JButton allToTargetButton = new JButton("   >>   ");		
		JButton allToSourceButton = new JButton("   <<   ");
		JButton removeFileButton = new JButton("remove");
		JButton selectFileButton = new JButton("import");	
		JButton loadFileButton = new JButton("load");
		
		selectFileButton.setToolTipText("import a configuration file");
		configurationFileBox.setPrototypeDisplayValue("text");
		configurationFileBox.setRenderer(new ComboBoxRenderer(configurationFileBox));
		configurationFileBox.setToolTipText("use a configuration file");
		removeFileButton.setToolTipText("remove imported configuration files");
		loadFileButton.setToolTipText("load the selected child configuration into `Configure` interface for log generation");
		
		buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.Y_AXIS));
		buttonPanel.add(selectFileButton);
		buttonPanel.add(Box.createRigidArea(new Dimension(15, 15)));
		buttonPanel.add(configurationFileBox);
		buttonPanel.add(Box.createRigidArea(new Dimension(15, 15)));
		buttonPanel.add(removeFileButton);		
		buttonPanel.add(Box.createRigidArea(new Dimension(15, 30)));
		buttonPanel.add(toTargetButton);
		buttonPanel.add(Box.createRigidArea(new Dimension(15, 15)));
		buttonPanel.add(toSourceButton);
		buttonPanel.add(Box.createRigidArea(new Dimension(15, 15)));
		buttonPanel.add(allToTargetButton);
		buttonPanel.add(Box.createRigidArea(new Dimension(15, 15)));
		buttonPanel.add(allToSourceButton);
		buttonPanel.add(Box.createRigidArea(new Dimension(15, 30)));
		buttonPanel.add(loadFileButton);	
		
		buttonPanel.setAlignmentY(Component.TOP_ALIGNMENT);
		buttonPanel.setBorder(BorderFactory.createEmptyBorder(15, 5, 15, 5));
		middlePanel.add(buttonPanel);
	
		targetLabel = new JLabel(nonselectedListPrefixName);
		targetLabel.setAlignmentY(Component.TOP_ALIGNMENT);
		targetLabel.setBorder(BorderFactory.createEmptyBorder(4, 5, 0, 5));
		//build target list
		targetListModel = new DefaultListModel<String>();
		targetList = new JList<String>(targetListModel);
		targetList.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
		targetList.setVisibleRowCount(5);
		//build target list panel
		JScrollPane targetListScroller = new JScrollPane(targetList);
		targetListScroller.setPreferredSize(new Dimension(screenWidth/6, screenHeight/3));
		targetListScroller.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		targetListScroller.setAlignmentY(Component.TOP_ALIGNMENT);
		JPanel targetListPanel = new JPanel();
		targetListPanel.setLayout(new BoxLayout(targetListPanel,BoxLayout.Y_AXIS));
		targetListPanel.add(targetLabel);
		targetListPanel.add(targetListScroller);
		targetListPanel.setAlignmentY(Component.TOP_ALIGNMENT);
		targetListPanel.setBorder(BorderFactory.createEmptyBorder(0, 30, 0, 0));
		middlePanel.add(targetListPanel);
		
		
		selectFileButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				IOFileDialogue fileImporter = new IOFileDialogue();
            	fileImporter.show("Configuration Files (.xml)","xml", JFileChooser.FILES_ONLY, "Select", lastFile);
            	JFileChooser fileChooser = fileImporter.getFileChooser();           	
            	File file = fileImporter.getSelectedFile();
				if(file != null){
					File[] files = fileChooser.getSelectedFiles();
					lastFile = file.getParentFile();
					transformConfigurationFilesToMaps(files);
				}									
			}
		});
				
		configurationFileBox.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
	            String selectedFileName = (String) configurationFileBox.getSelectedItem();
	            if(selectedFileName != null){
	            	setSourceList(fileNameVSNonselectedTableList.get(selectedFileName));
	            }
	            
			}
		});
		
		removeFileButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				String fileName = (String) configurationFileBox.getSelectedItem();
				configurationFileBox.removeItem(fileName);
				fileNameVSNonselectedTableList.remove(fileName);
				fileNameVSSelectedTableList.remove(fileName);
				sourceListModel.clear();
//				int targetListSize = targetListModel.getSize();
				for(int i=0; i<targetListModel.getSize(); i++){
					String element = targetListModel.get(i);
					String configurationFileName = element.split("->")[0];
					System.out.println("element: " + element);
					System.out.println("configurationFileName: " + configurationFileName);
					System.out.println("fileName: " + fileName);
					if(configurationFileName.equals(fileName)){
						targetListModel.removeElement(element);
						i--;
					}
				}				
			}			
		});
				
		loadFileButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {		
				ConfigurationParameters configuration = new ConfigurationParameters();
				
				addParametersToConfiguration(configuration); 
				
//	         	Map<String, Object> parameterMap = parameters.getParameterMap();
				for(String fileName : fileNameVSSelectedTableList.keySet()){
					String filePath = fileNameVSAbsolutePath.get(fileName);
					List<String> selectedTableList = fileNameVSSelectedTableList.get(fileName);		
		          	ReadConfigurationsFromXML reader = new ReadConfigurationsFromXML(configuration);
//		    		reader.parserXml(filePath, selectedTableList);	
		          	for(String table : selectedTableList){
		          		System.out.println("load file in the select configuration, table: " + table);
		          	}
		    		reader.parserXml(filePath, null);	
		    		fatherPanel.getLogGenerator().setParameterIntoPanels(configuration);
				}
			}			
		});
		
		//add listeners for the operating buttons and list changes
		toTargetButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				int[] indexSet = sourceList.getSelectedIndices();
				List<String> selectedItemList = new ArrayList<String>();
				for(int i=0; i<indexSet.length; i++){
					int index = indexSet[i];
					if(index >= 0){
						selectedItemList.add(sourceList.getModel().getElementAt(index));
					}
				}		
				String selectedFileName = (String) configurationFileBox.getSelectedItem();
				if(!fileNameVSSelectedTableList.containsKey(selectedFileName)){
					fileNameVSSelectedTableList.put(selectedFileName, new ArrayList<String>());
				}
				//change the list by adding/removing object values instead of object indices;
				//otherwise, the index will shift when adding/removing items
				for(String selectedItem : selectedItemList){
					//check if there already exist a table with the same name in the selected table list 
					boolean existentFlag = false;
					for(String fileName : fileNameVSSelectedTableList.keySet()){
						if(fileNameVSSelectedTableList.get(fileName).contains(selectedItem)){
							existentFlag = true;
							break;
						}
					}
					if(!existentFlag){
						targetListModel.addElement(selectedFileName + "->" + selectedItem);
						targetList.setModel(targetListModel);
						sourceListModel.removeElement(selectedItem);
						sourceList.setModel(sourceListModel);
						//add into the selected list
						fileNameVSSelectedTableList.get(selectedFileName).add(selectedItem);
						//remove from the non-selected list
						if(fileNameVSNonselectedTableList.containsKey(selectedFileName)){
							if(fileNameVSNonselectedTableList.get(selectedFileName).contains(selectedItem)){
								fileNameVSNonselectedTableList.get(selectedFileName).remove(selectedItem);
							}						
						}
					}
					else{//there exist table with the same name, output the warning dialog
						JOptionPane.showMessageDialog(null, "the table " + selectedItem + " already exists");						
					}									
				}				
				updateTableNumberLabel();
			}
		});

		
		allToTargetButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				int sourceListSize = sourceList.getModel().getSize();
				List<String> selectedItemList = new ArrayList<String>();
				for(int i=0; i<sourceListSize; i++){
					selectedItemList.add(sourceList.getModel().getElementAt(i));				
				}		
				String selectedFileName = (String) configurationFileBox.getSelectedItem();
				if(!fileNameVSSelectedTableList.containsKey(selectedFileName)){
					fileNameVSSelectedTableList.put(selectedFileName, new ArrayList<String>());
				}
				//change the list by adding/removing object values instead of object indices;
				//otherwise, the index will shift when adding/removing items
				for(String selectedItem : selectedItemList){
					//check if there already exist a table with the same name in the selected table list 
					boolean existentFlag = false;
					for(String fileName : fileNameVSSelectedTableList.keySet()){
						if(fileNameVSSelectedTableList.get(fileName).contains(selectedItem)){
							existentFlag = true;
							break;
						}
					}
					if(!existentFlag){
						targetListModel.addElement(selectedFileName + "->" + selectedItem);
						targetList.setModel(targetListModel);
						sourceListModel.removeElement(selectedItem);
						sourceList.setModel(sourceListModel);
						//add into the selected list
						fileNameVSSelectedTableList.get(selectedFileName).add(selectedItem);
						//remove from the non-selected list
						if(fileNameVSNonselectedTableList.containsKey(selectedFileName)){
							if(fileNameVSNonselectedTableList.get(selectedFileName).contains(selectedItem)){
								fileNameVSNonselectedTableList.get(selectedFileName).remove(selectedItem);
							}						
						}
					}
					else{//there exist table with the same name, output the warning dialog
						JOptionPane.showMessageDialog(null, "the table " + selectedItem + " already exists");						
					}									
				}				
				updateTableNumberLabel();
			}
		});
		
		toSourceButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {				
				int[] indexSet = targetList.getSelectedIndices();
				List<String> selectedItemList = new ArrayList<String>();
				for(int i=0; i<indexSet.length; i++){
					int index = indexSet[i];
					if(index >= 0){
						selectedItemList.add(targetList.getModel().getElementAt(index));
					}
				}		
				String selectedFileName = (String) configurationFileBox.getSelectedItem();	
				if(!fileNameVSNonselectedTableList.containsKey(selectedFileName)){
					fileNameVSNonselectedTableList.put(selectedFileName, new ArrayList<String>());
				}
				//change the list by adding/removing object values instead of object indices;
				//otherwise, the index will shift when adding/removing items
				for(String selectedItem : selectedItemList){
					String fileName = selectedItem.split("->")[0];
					String tableName = selectedItem.split("->")[1];
					if(fileName.equals(selectedFileName)){
						sourceListModel.addElement(tableName);
						sourceList.setModel(sourceListModel);
					}					
					targetListModel.removeElement(selectedItem);
					targetList.setModel(targetListModel);
					//add into the non-selected list
					fileNameVSNonselectedTableList.get(fileName).add(tableName);
					//remove from the selected list
					if(fileNameVSSelectedTableList.containsKey(fileName)){
						if(fileNameVSSelectedTableList.get(fileName).contains(tableName)){
							fileNameVSSelectedTableList.get(fileName).remove(tableName);
						}						
					}
				}
				updateTableNumberLabel();
			}
		});
		
		allToSourceButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				int targetListSize = targetList.getModel().getSize();
				List<String> selectedItemList = new ArrayList<String>();
				for(int i=0; i<targetListSize; i++){
					selectedItemList.add(targetList.getModel().getElementAt(i));				
				}			
				String selectedFileName = (String) configurationFileBox.getSelectedItem();	
				if(!fileNameVSNonselectedTableList.containsKey(selectedFileName)){
					fileNameVSNonselectedTableList.put(selectedFileName, new ArrayList<String>());
				}
				//change the list by adding/removing object values instead of object indices;
				//otherwise, the index will shift when adding/removing items
				for(String selectedItem : selectedItemList){
					String fileName = selectedItem.split("->")[0];
					String tableName = selectedItem.split("->")[1];
					if(fileName.equals(selectedFileName)){
						sourceListModel.addElement(tableName);
						sourceList.setModel(sourceListModel);
					}					
					targetListModel.removeElement(selectedItem);
					targetList.setModel(targetListModel);
					//add into the non-selected list
					fileNameVSNonselectedTableList.get(fileName).add(tableName);
					//remove from the selected list
					if(fileNameVSSelectedTableList.containsKey(fileName)){
						if(fileNameVSSelectedTableList.get(fileName).contains(tableName)){
							fileNameVSSelectedTableList.get(fileName).remove(tableName);
						}						
					}
				}
				updateTableNumberLabel();
			}
		});
		
		sourceList.addListSelectionListener(this);
		targetList.addListSelectionListener(this);
		
		return middlePanel;
    }
    
    public void updateTableNumberLabel(){
		sourceLabel.setText(selectedListPrefixName + sourceListModel.size());
        targetLabel.setText(nonselectedListPrefixName + targetListModel.size());
    }

    //get the selected item list (from the target list) of the List<String> format
    public List<String> getSelectedTables(){
    	List<String> selectedTableList = new ArrayList<String>();
    	int listSize = targetList.getModel().getSize();
    	for(int i =0; i<listSize; i++){
    		String selectedContent = targetList.getModel().getElementAt(i);
    		selectedTableList.add(selectedContent);
    	}   	
		return selectedTableList;
    }
    

    //get the selected item list (from the target list) of the JList<String> format
    public JList<String> getSelectedTableList(){   	
    	DefaultListModel<String> selectedTableListModel = new DefaultListModel<String>();
    	for(String tableName : getSelectedTables()){
    		selectedTableListModel.addElement(tableName);
    	}
    	JList<String> selectedTableList = new JList<String>(selectedTableListModel);
    	selectedTableList.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
		return selectedTableList;
    }

    //set a item list of the List<String> format to the source list
    public void setSourceList(List<String> itemList){       
        //use the configuration file to configure tables (record the selected tables)
    	if(itemList != null){
    		sourceListModel.clear();
    		for(String tableName : itemList){
            	if(!sourceListModel.contains(tableName)){
            		sourceListModel.addElement(tableName);
            	}     	        	
    		}
        	sourceList.setModel(sourceListModel);   		
            sourceLabel.setText(selectedListPrefixName + sourceListModel.size());
    	}        
    }
    
    //put a item list of the List<String> format to the source list
    public void addingIntoSourceList(List<String> itemList){       
        //use the configuration file to configure tables (record the selected tables)
    	if(itemList != null){

        	sourceList.setModel(sourceListModel);
    		targetList.setModel(targetListModel);
            sourceLabel.setText(selectedListPrefixName+ selectedTableList.size());
            targetLabel.setText("selected tables: " + selectedTableList.size());
    	}        
    }
    
    //put a item list of the List<String> format to the target (right) list
    public void addingIntoTargetList(List<String> itemList){       
        //use the configuration file to configure tables (record the selected tables)
    	if(itemList != null){
    		for(String item : itemList){
            	if(!targetListModel.contains(item)){
            		sourceListModel.addElement(item);
            	}     	     	
    		}
        	sourceList.setModel(sourceListModel);
    		targetList.setModel(targetListModel);
            sourceLabel.setText(selectedListPrefixName + selectedTableList.size());
            targetLabel.setText("selected tables: " + selectedTableList.size());
    	}        
    }
    
    
    public void transformConfigurationFilesToMaps(File[] files){
    	String nameWithoutSuffix = null;
        for(int i=0; i<files.length; i++){
        	File file = files[i];
        	System.out.println("file path: " + file);    		
            String fileName = file.getName();
            nameWithoutSuffix = fileName.substring(0, fileName.lastIndexOf('.'));
            //check if the file already exists
			if(!fileNameVSNonselectedTableList.containsKey(nameWithoutSuffix) &&
					!fileNameVSSelectedTableList.containsKey(nameWithoutSuffix)){          
	        	ConfigurationParameters configuration = new ConfigurationParameters();
	        	addParametersToConfiguration(configuration);        		        	
	          	ReadConfigurationsFromXML reader = new ReadConfigurationsFromXML(configuration);
	    		reader.parserXml(file.getAbsolutePath(), null);

				Map<String, List<String>> tableVSClassList = configuration.getNameVSMapListParameterMap().get(ConfigurationConstants.TABLEVSCLASSLISTMAP);
				if(tableVSClassList != null){
					List<String> tableList = new ArrayList<String>(tableVSClassList.keySet());  			         	
					configurationFileBox.addItem(nameWithoutSuffix);
					fileNameVSNonselectedTableList.put(nameWithoutSuffix, tableList);
					fileNameVSSelectedTableList.put(nameWithoutSuffix, new ArrayList<String>());
		            fileNameVSAbsolutePath.put(nameWithoutSuffix, file.getAbsolutePath());
				}
				else{
					JOptionPane.showMessageDialog(null, "Wrong! Please choose a valid configuration file.");
				}         	
			}
			else{//there exist file with the same name, output the warning dialog
				JOptionPane.showMessageDialog(null, "the file " + nameWithoutSuffix + " already exists");
			}
        }
        if(nameWithoutSuffix != null){
        	configurationFileBox.setSelectedItem(nameWithoutSuffix);
        	setSourceList(fileNameVSNonselectedTableList.get(nameWithoutSuffix));
        }		
    }  
    
    public void addParametersToConfiguration(ConfigurationParameters configuration){
    	configuration.getNameVSMapParameterMap().put(ConfigurationConstants.connectionParameterNameVSValueMap, null);
    	
    	configuration.getNameVSMapListParameterMap().put(ConfigurationConstants.TABLEVSCLASSLISTMAP, null);
    	configuration.getNameVSMapListParameterMap().put(ConfigurationConstants.TABLEVSELECTEDROWIDSMAP, null);
    	configuration.getNameVSMapListParameterMap().put(ConfigurationConstants.TABLEVSFATHERTABLESMAP, null);
    	configuration.getNameVSMapListParameterMap().put(ConfigurationConstants.TABLECOLUMNVSIMPACTEDTABLECOLUMNSMAP, null);
    	configuration.getNameVSMapListParameterMap().put(ConfigurationConstants.ACTIVITYVSCLASSESMAP, null);
    	configuration.getNameVSMapListParameterMap().put(ConfigurationConstants.CLASSVSPKSMAP, null);
    	configuration.getNameVSMapListParameterMap().put(ConfigurationConstants.CLASSVSFKSMAP, null);
    	
    	configuration.getNameVSMapMapParameterMap().put(ConfigurationConstants.CLASSVSCOLUMNVSNAMEMAP, null);
    	configuration.getNameVSMapMapParameterMap().put(ConfigurationConstants.TABLEVSEVENTCHAINEDCHILDTABLEVSFKMAP, null);
    	configuration.getNameVSMapMapParameterMap().put(ConfigurationConstants.TABLECLASSVSCOLUMNVSCONSTRAINTMAP, null);
    	configuration.getNameVSMapMapParameterMap().put(ConfigurationConstants.TABLEVSOBJECTCHAINEDCHILDTABLEVSFKMAP, null);
    	configuration.getNameVSMapMapParameterMap().put(ConfigurationConstants.ACTIVITYVSREFERREDCLASSVSPATHMAP, null);
    	
    	configuration.getNameVSMapMapListParameterMap().put(ConfigurationConstants.TABLEVSCHILDTABLEVSFKLISTMAP, null);
    	configuration.getNameVSMapMapListParameterMap().put(ConfigurationConstants.CLASSVSACTIVITYCOLUMNVSATTRIBUTECOLUMNSMAP, null);
    	configuration.getNameVSMapMapListParameterMap().put(ConfigurationConstants.TABLEVSOBJECTCHAINEDFATHERTABLEVSFKSMAP, null);
    	configuration.getNameVSMapMapListParameterMap().put(ConfigurationConstants.TABLEVSEVENTCHAINEDFATHERTABLEVSFKSMAP, null);
    }


	public void valueChanged(ListSelectionEvent e) {
		// TODO Auto-generated method stub
		
	}
}