package org.processmining.ocbc.loggeneration.postgresql.changetable;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map; 
 

public class CreateChangeTables{

	private  Connection conn = null;
	private  List<Change> changeList = new ArrayList<Change>();
	private  List<String> changeIdList = new ArrayList<String>();
	private  List<String> eventIdList = new ArrayList<String>();//record the events

    //record what the primary key of one table is called in its children table as foreign key
	Map<String, Map<String, String>> tableVSTableForeignKey = new HashMap<String, Map<String, String>>();
    
    //for one table (name), record the tables (name) it relies on
	private Map<String, List<String>> tableVSFatherTables = new HashMap<String, List<String>>();
    
    //for one table (name), record other child tables (name)
	private Map<String, List<String>> tableVSChildTables = new HashMap<String, List<String>>();
    
    //for one table (table name), which columns (column names) will be considered as event source, e.g., question-[creationdata, closeddate] (events related to objects)
	private Map<String, List<String>> tableVSEventColumns = new HashMap<String, List<String>>();
    
    //In a business process, some happening objects (events), called source objects (events), trigger the occurrences of following objects (events), 
    //e.g., creating order will trigger pick item, deliver items.
    //for one source table (table name), which rows (id names) will be considered as source objects, e.g., question-[181,182]
	private Map<String, List<Integer>> sourceTableVSRowIds = new HashMap<String, List<Integer>>();
    
    //for one table event column (table name + column name), which table column will be impacted through which column, table-column-[table-column-intermediacolumn]
    //e.g., question-creationdate-[question-id-id,question-creationdate-id,question-body-id]
    //e.g., order-creationdate-[order-id-id,order-creationdate-id,orderline-id-command...]
	private Map<String, List<String>> tableColumnVSImpactedTableColumns = new HashMap<String, List<String>>();
    
    //assign a change type to each table event column (table name + column name), e.g., "I" for insertion, "U" for update
	private Map<String, String> tableColumnVSChangeType = new HashMap<String, String>();
       
    private String schema;
    private String cdhdrTable;
    private String cdposTable;
    
	public CreateChangeTables(Map<String, Map<String, String>> tableVSTableForeignKey, Map<String, List<String>> tableVSFatherTables,
			Map<String, List<String>> tableVSChildTables, Map<String, List<String>> tableVSEventColumns,
			Map<String, List<String>> tableColumnVSImpactedTableColumns, Map<String, String> tableColumnVSChangeType,
			Map<String, List<Integer>> sourceTableVSRowIds, String schema, String cdhdrTable, String cdposTable){
		this.schema = schema;
		this.cdhdrTable = cdhdrTable;
		this.cdposTable = cdposTable;
		if(tableVSTableForeignKey != null)
			this.tableVSTableForeignKey = tableVSTableForeignKey;
		if(tableVSFatherTables != null)
			this.tableVSFatherTables = tableVSFatherTables;
		if(tableVSChildTables != null)
			this.tableVSChildTables = tableVSChildTables;
		if(tableVSEventColumns != null)
			this.tableVSEventColumns = tableVSEventColumns;
		if(tableColumnVSImpactedTableColumns != null)
			this.tableColumnVSImpactedTableColumns = tableColumnVSImpactedTableColumns;
		if(tableColumnVSChangeType != null)
			this.tableColumnVSChangeType = tableColumnVSChangeType;
		if(sourceTableVSRowIds != null)
			this.sourceTableVSRowIds = sourceTableVSRowIds;	    		
	}
	
	public List<Change> rankChangeSequence() throws ParseException{
		List<Change> rankedChangeList = new ArrayList<Change>();
		//get source tables
		SimpleDateFormat sdf=new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");  		
		while(changeList.size()>0){
			Date earlistTime= sdf.parse("2020-01-01 01:00:00");
			int  earlistTimeIndex = 0;
			for(int i=0; i<changeList.size(); i++){
				String time = changeList.get(i).time;
				System.out.println("\n parse time: " + time);	
				Date timestamp = sdf.parse(time);			
				if(timestamp.before(earlistTime)){
					earlistTimeIndex = i;
					earlistTime = timestamp;
				}
			}			
			System.out.println("\n rank time: " + changeList.get(earlistTimeIndex).time);			
			rankedChangeList.add(changeList.get(earlistTimeIndex));
			changeList.remove(earlistTimeIndex);
			
		}		
		changeList = rankedChangeList;
		return rankedChangeList;
	}
	
	public void createChangeSequence() throws SQLException{
		//get source tables
		for(String sourceTable : sourceTableVSRowIds.keySet()){
			//get source objects from source tables
			for(int sourceObjectId : sourceTableVSRowIds.get(sourceTable)){
				addChangesRelatedToObject(sourceTable, sourceObjectId);								
			}				
		}			
	}
	
	public void addChangesRelatedToObject(String sourceTable, int objectId) throws SQLException{
		if(!eventIdList.contains(sourceTable + "-" + objectId) && tableVSEventColumns.containsKey(sourceTable)){
			eventIdList.add(sourceTable + "-" + objectId);
    		
    		//add changes from the source table
    		for(String eventColumn : tableVSEventColumns.get(sourceTable)){
    			String sql = "SELECT " + eventColumn +" FROM " + schema + "." + sourceTable + " where id = " + objectId;			       
    	        Statement stmt = conn.createStatement();
    	        System.out.println("\n sql: " + sql);
    	        ResultSet rs = stmt.executeQuery(sql);
    	        rs.next();
    	        	
    			Change change = new Change();
    			change.table = sourceTable;
    			change.column = eventColumn;
    			change.id = objectId;
    			change.time = rs.getString(1);
    			change.changeId = sourceTable + "-" + eventColumn + "-" + objectId;
    			if(change.time != null) changeList.add(change);
    		}
    		
    		//add changes from child tables
    		if(tableVSChildTables.containsKey(sourceTable)){
    			for(String childTable : tableVSChildTables.get(sourceTable)){
    				Statement referedStmt = conn.createStatement();
    				//get the ids of child objects  
    				String childSql = "SELECT id FROM " + schema + "."  + childTable + " where " + tableVSTableForeignKey.get(sourceTable).get(childTable) + " = " + objectId;
    				
    				if(childTable.equals("votes")){
    					childSql = "SELECT id FROM " + schema + "." + childTable + " where " + tableVSTableForeignKey.get(sourceTable).get(childTable) + " = " + objectId +
    							" and votetypeid = 6";
    				}
            		System.out.println("\n child sql: " + childSql);
                    ResultSet childRS = referedStmt.executeQuery(childSql);
                    while (childRS.next()) {
                    	int childObjectId = childRS.getInt(1);
                    	addChangesRelatedToObject(childTable, childObjectId);              	
                    }
    			}
    		}	
    		
    		//add changes from parent tables
    		if(tableVSFatherTables.containsKey(sourceTable)){
    			for(String fatherTable : tableVSFatherTables.get(sourceTable)){
    				Statement fatherStmt = conn.createStatement();
                    //get the id of the father object   
            		String fatherSql = "SELECT " + tableVSTableForeignKey.get(fatherTable).get(sourceTable) + " FROM " + schema + "." + sourceTable + " where id  = " + objectId;
            		System.out.println("\n father sql: " + fatherSql);
                    ResultSet fatherRS = fatherStmt.executeQuery(fatherSql);
                    while (fatherRS.next()) {
                    	int fatherObjectId = fatherRS.getInt(1);
                    	addChangesRelatedToObject(fatherTable, fatherObjectId);
               	
                    }
    			}
    		}
    	}               	
    }
	
	public void connect() throws Exception{
        Class.forName("org.postgresql.Driver");       
        conn = DriverManager 
                .getConnection("jdbc:postgresql://localhost:5432/simplestackoverflow", 
                "postgres", "kfgz0874981211"); 
        System.out.println("Opened database successfully");           
	}
	
	public void close() throws Exception{
		conn.close();
	}
	
	//create the CDPOS and CDHDR tables
	public void createChangeTables() throws SQLException{
		int cdhdrChangeId = 0;
		for(Change change : changeList){		
			String[] dateTime = change.time.split(" ");
			cdhdrChangeId ++;
			String changeType = tableColumnVSChangeType.get(change.table + "-" + change.column);			
			Integer objectId = change.id;
			
			Statement insertCDHDRStmt = conn.createStatement();
            //get the id of the father object   
    		String insertCDHDRSql = "INSERT INTO " + schema + "." + cdhdrTable + "(changenr, udate, utime, change_ind) VALUES (" + cdhdrChangeId + ", '" + dateTime[0]
    				+ "' , '" + dateTime[1] + "' , '" + changeType + "')";
    		System.out.println("\n insertCDHDR sql: " + insertCDHDRSql);
    		insertCDHDRStmt.execute(insertCDHDRSql);
    		
    		for(String tableColumn : tableColumnVSImpactedTableColumns.get(change.table + "-" + change.column)){
    			String[] tableColumnArray = tableColumn.split("-");
    			String table = tableColumnArray[0];
    			String column = tableColumnArray[1];
    			String key = tableColumnArray[2];
    			
    			//get the attribute value
    			Statement queryAttributeStmt = conn.createStatement();
    			String queryAttributeSql = "SELECT " + column + " FROM " + schema + "." + table + " where " + key + " = " + objectId;
    			System.out.println("\n queryAttribute Sql: " + queryAttributeSql);
    			ResultSet queryAttributeRS = queryAttributeStmt.executeQuery(queryAttributeSql);
                queryAttributeRS.next();
                String attributeValue = queryAttributeRS.getString(1);

    			//insert a record into the cdpos table
    			Statement insertCDPOSStmt = conn.createStatement();              
        		String insertCDPOSSql = "INSERT INTO " + schema + "." + cdposTable + "(objectid, changenr, tabname, tabkey, fname, chngind, value_new) VALUES "
        				+ "(" + objectId + ", '" + cdhdrChangeId
        				+ "' , '" + table + "' , '" + objectId + "' , '" + column + "' , '" + changeType + "' , '" + attributeValue +"')";
        		System.out.println("\n insertCDPOS sql: " + insertCDPOSSql);
        		insertCDPOSStmt.execute(insertCDPOSSql);

    		}
		}
	}	
}
