package org.processmining.ocbc.loggeneration.postgresql.outputlog;


import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.swing.JOptionPane;

import org.deckfour.spex.SXDocument;
import org.deckfour.spex.SXTag;
import org.processmining.ocbc.log.model.impl.xConstants;
import org.processmining.ocbc.loggeneration.Event;
import org.processmining.ocbc.loggeneration.Log;
import org.processmining.ocbc.loggeneration.ObjectRelation;
import org.processmining.ocbc.loggeneration.TableObject;



public class OutputXMLLog {
	private String savePath;
	
	public OutputXMLLog(String savePath){
		this.savePath = savePath;
	}
    
	public void createLog(List<Event> eventList, String logFormat) throws IOException{
		Log log = new Log();		
	
	    System.out.println("\n path: " +  savePath);	    
		FileOutputStream fout = new FileOutputStream(savePath);
		
		if(logFormat.equals(xConstants.TOTAL)){
			System.out.println("enter logFormat: " + xConstants.TOTAL);
			for(Event event : eventList){										
				log.eventList.add(event);
			}	
		}
		else if(logFormat.equals(xConstants.UPDATE)){
			System.out.println("logFormat: " + xConstants.UPDATE);
//			List<String> accumulatedObjectIdList = new ArrayList<String>();
			Map<String, TableObject> accumulatedObjectIdVSObject = new HashMap<String, TableObject>();
			List<String> accumulatedObjectRelationIdList = new ArrayList<String>();
			for(Event event : eventList){					
				List<TableObject> bornObjectList = new ArrayList<TableObject>();
				List<ObjectRelation> bornObjectRelationList = new ArrayList<ObjectRelation>();
				
				for(TableObject object : event.currentObjectList){
					String objectId = object.name;//currently, consider the name as id
					//add the born objects
					if(!accumulatedObjectIdVSObject.containsKey(objectId)){
						accumulatedObjectIdVSObject.put(objectId, object);
						//add the life cycle attribute
						object.getAttributeMap().put(xConstants.lifecycle, xConstants.add);
						bornObjectList.add(object);
					}
					//add the updated objects
					else{
						TableObject previousObject = accumulatedObjectIdVSObject.get(objectId);
						Map<String, String> differentAttributeMap = previousObject.compareObjectAttributeMap(object);
						if(differentAttributeMap.size() > 0){
							accumulatedObjectIdVSObject.put(objectId, object);
							TableObject updatedObject = new TableObject();
							updatedObject.name = objectId;
							updatedObject.className = object.className;
							//add the life cycle attribute
							differentAttributeMap.put(xConstants.lifecycle, xConstants.update);
							updatedObject.setAttributeMap(differentAttributeMap);
							bornObjectList.add(updatedObject);
						}
					}
				}
				
				for(ObjectRelation objectRelation : event.currentObjectRelationList){
					String relationId = objectRelation.id;
					if(!accumulatedObjectRelationIdList.contains(relationId)){
						accumulatedObjectRelationIdList.add(relationId);
						objectRelation.getAttributeMap().put(xConstants.lifecycle, xConstants.add);
						bornObjectRelationList.add(objectRelation);
					}
				}
				
				event.currentObjectList.clear();
				event.currentObjectList.addAll(bornObjectList);
				event.currentObjectRelationList.clear();
				event.currentObjectRelationList.addAll(bornObjectRelationList);
				log.eventList.add(event);
			}
		}
		
		if(log.eventList.size() == 0){
			JOptionPane.showMessageDialog(null, "Warn! The log contains no events!"); 		
		}
		serialize(log, fout, logFormat);	
		fout.close();	
		System.out.println("logFormat: " + logFormat);

	    
	}
	
	public void serialize(Log log, OutputStream out, String logFormat) throws IOException {		
		long start = System.currentTimeMillis();
		SXDocument doc = new SXDocument(out);
		doc.addComment("This file is a XOC log. Its definition and semantic are");
		doc.addComment("available from http://www.win.tue.nl/ocbc/.");
		doc.addComment("It conforms to the XML serialization.");
		
		SXTag logTag = doc.addNode("log");
		logTag.addAttribute("xoc.version", "1.0");
		SXTag attributeTag;
		attributeTag = logTag.addChildNode("string");
		attributeTag.addAttribute("key", xConstants.FORMAT);
		attributeTag.addAttribute("value", logFormat);
		for (Event event : log.eventList) {
			System.out.print("write event:  \n" );
			SXTag eventTag = logTag.addChildNode("event");
			attributeTag = eventTag.addChildNode("string");
			attributeTag.addAttribute("key", "id");
			attributeTag.addAttribute("value", event.id);
			
			attributeTag = eventTag.addChildNode("string");
			attributeTag.addAttribute("key", "activity");
			attributeTag.addAttribute("value", event.activity);
			
//			attributeTag = eventTag.addChildNode("string");
//			attributeTag.addAttribute("key", "resource");
//			attributeTag.addAttribute("value", event.resource);
			
			attributeTag = eventTag.addChildNode("string");
			attributeTag.addAttribute("key", "timestamp");
			
			SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
			String time = sdf.format(event.timestamp);
			attributeTag.addAttribute("value", time);
//			attributeTag.addAttribute("value", event.timestamp.toString());
			
			for(String attributeKey : event.attributes.keySet()){
				SXTag eventAttributeTag = eventTag.addChildNode("string");
				eventAttributeTag.addAttribute("key", attributeKey);
				eventAttributeTag.addAttribute("value", event.attributes.get(attributeKey));	
			}
	
			SXTag objectModelTag = eventTag.addChildNode("model");			
			SXTag objectModeObjectsTag = objectModelTag.addChildNode("objects");
			addObjects(objectModeObjectsTag, event.currentObjectList);			
			SXTag objectModeObjectRelationsTag = objectModelTag.addChildNode("relations");
			addObjectRelations(objectModeObjectRelationsTag, event.currentObjectRelationList);
			
			//the reference must be written into XOC logs after object model to make it easier for future importing
			SXTag objectsTag = eventTag.addChildNode("references");
			addReferenceObjects(objectsTag, event.referredObjectList);
		}
		
		doc.close();
		String duration = " (" + (System.currentTimeMillis() - start)
				+ " msec.)";

	}
	
	public void addObjects(SXTag tag, List<TableObject> objects)
			throws IOException {
						
		for (TableObject object : objects) {
			SXTag objectTag = tag.addChildNode("object");
			
			SXTag objectIDTag = objectTag.addChildNode("string");
			objectIDTag.addAttribute("key", "id");
//			objectIDTag.addAttribute("value", object.id);
			objectIDTag.addAttribute("value", object.name);
			
			SXTag objectClassTag = objectTag.addChildNode("string");
			objectClassTag.addAttribute("key", "class");
			objectClassTag.addAttribute("value", object.className);	
			
			for(String attributeKey : object.attributes.keySet()){
				SXTag objectAttributeTag = objectTag.addChildNode("string");
				objectAttributeTag.addAttribute("key", attributeKey);
				objectAttributeTag.addAttribute("value", object.attributes.get(attributeKey));	
			}
		}
	}
	
	/**
	 * the function write reference objects (simple version, i.e., only with id attribute) into XOC logs
	 * @param tag
	 * @param objects
	 * @throws IOException
	 */
	public void addReferenceObjects(SXTag tag, List<TableObject> objects)
			throws IOException {
						
		for (TableObject object : objects) {
			SXTag objectTag = tag.addChildNode("object");			
			SXTag objectIDTag = objectTag.addChildNode("string");
			objectIDTag.addAttribute("key", "id");
//			objectIDTag.addAttribute("value", object.id);
			objectIDTag.addAttribute("value", object.name);			
		}
	}
	
	public void addObjectRelations(SXTag tag, List<ObjectRelation> objectRelations)
			throws IOException {		
		for (ObjectRelation objectRelation : objectRelations) {
			//in real application			
//			for(XAttribute attribute : objectRelation.getAttributes()){
//				SXTag objectAttributeTag = objectTag.addChildNode("value.type");
//				objectAttributeTag.addAttribute("key", "concept:name");
//				objectAttributeTag.addAttribute("value", object.getName());
//			}
			SXTag objectRelationTag = tag.addChildNode("relation");;
			
			SXTag objectRelationIDTag = objectRelationTag.addChildNode("string");
			objectRelationIDTag.addAttribute("key", "id");
			objectRelationIDTag.addAttribute("value", objectRelation.id);
			
			SXTag objectRelationTypeTag = objectRelationTag.addChildNode("string");
			objectRelationTypeTag.addAttribute("key", "relation");
			objectRelationTypeTag.addAttribute("value", objectRelation.relationType);
			
			SXTag sourceObjectTag = objectRelationTag.addChildNode("string");
			sourceObjectTag.addAttribute("key", "sourceobjectid");
			sourceObjectTag.addAttribute("value", objectRelation.sourceObjectId);
			
			SXTag targetObjectTag = objectRelationTag.addChildNode("string");
			targetObjectTag.addAttribute("key", "targetobjectid");
			targetObjectTag.addAttribute("value", objectRelation.targetObjectId);	
			
			for(String attributeKey : objectRelation.attributes.keySet()){
				SXTag objectAttributeTag = objectRelationTag.addChildNode("string");
				objectAttributeTag.addAttribute("key", attributeKey);
				objectAttributeTag.addAttribute("value", objectRelation.attributes.get(attributeKey));	
			}
		}
	}
 
}