package org.processmining.ocbc.loggeneration.postgresql.stackoverflow;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.processmining.ocbc.loggeneration.postgresql.changetable.Change;
import org.processmining.ocbc.loggeneration.postgresql.changetable.CreateChangeTables; 
 

public class CloseQuestionChangeTables{

    
	public static void main(String[] str) throws Exception{
		
	    //record what the primary key of one table is called in its children table as foreign key
		Map<String, Map<String, String>> tableVSTableForeignKey = new HashMap<String, Map<String, String>>();
	    
	    //for one table (name), record the tables (name) it relies on
	    Map<String, List<String>> tableVSFatherTables = new HashMap<String, List<String>>();
	    
	   //for one table (name), record other child tables (name)
	    Map<String, List<String>> tableVSChildTables = new HashMap<String, List<String>>();
	    
	    //for one table (table name), which columns (column names) will be considered as event source, e.g., question-[creationdata, closeddate] (events related to objects)
	    Map<String, List<String>> tableVSEventColumns = new HashMap<String, List<String>>();
	    
	    //In a business process, some happening objects (events), called source objects (events), trigger the occurrences of following objects (events), 
	    //e.g., creating order will trigger pick item, deliver items.
	    //for one source table (table name), which rows (id names) will be considered as source objects, e.g., question-[181,182]
	    Map<String, List<Integer>> sourceTableVSRowIds = new HashMap<String, List<Integer>>();
	    
	    //for one table event column (table name + column name), which table column will be impacted through which column, table-column-[table-column-intermediacolumn]
	    //e.g., question-creationdate-[question-id-id,question-creationdate-id,question-body-id]
	    //e.g., order-creationdate-[order-id-id,order-creationdate-id,orderline-id-command...]
	    Map<String, List<String>> tableColumnVSImpactedTableColumns = new HashMap<String, List<String>>();
	    
	    //assign a change type to each table event column (table name + column name), e.g., "I" for insertion, "U" for update
	    Map<String, String> tableColumnVSChangeType = new HashMap<String, String>();
	    
		//build the database schema
		tableVSFatherTables.put("votes", new ArrayList<String>());
		tableVSFatherTables.get("votes").add("questions");
		
		tableVSChildTables.put("questions", new ArrayList<String>());
		tableVSChildTables.get("questions").add("votes"); 
		
		tableVSTableForeignKey.put("questions", new HashMap<String, String>());
		tableVSTableForeignKey.get("questions").put("votes", "postid");

		
		//build the reference relations (class-activity)
		tableVSEventColumns.put("questions", new ArrayList<String>());
		tableVSEventColumns.get("questions").add("creationdate"); 
		tableVSEventColumns.get("questions").add("closeddate"); 
		
		tableVSEventColumns.put("votes", new ArrayList<String>());
		tableVSEventColumns.get("votes").add("creationdate"); 
		
		//build the cdpos table format
		tableColumnVSImpactedTableColumns.put("questions-creationdate", new ArrayList<String>());
		tableColumnVSImpactedTableColumns.get("questions-creationdate").add("questions-id-id");
		tableColumnVSImpactedTableColumns.get("questions-creationdate").add("questions-creationdate-id");
		tableColumnVSImpactedTableColumns.get("questions-creationdate").add("questions-owneruserid-id");
		tableColumnVSChangeType.put("questions-creationdate", "I");
		
		tableColumnVSImpactedTableColumns.put("questions-closeddate", new ArrayList<String>());
		tableColumnVSImpactedTableColumns.get("questions-closeddate").add("questions-closeddate-id");
		tableColumnVSImpactedTableColumns.get("questions-closeddate").add("questions-isclosed-id");
		tableColumnVSChangeType.put("questions-closeddate", "U");
		
		tableColumnVSImpactedTableColumns.put("votes-creationdate", new ArrayList<String>());
		tableColumnVSImpactedTableColumns.get("votes-creationdate").add("votes-id-id");
		tableColumnVSImpactedTableColumns.get("votes-creationdate").add("votes-creationdate-id");
		tableColumnVSImpactedTableColumns.get("votes-creationdate").add("votes-postid-id");
		tableColumnVSChangeType.put("votes-creationdate", "I");

		//build the source objects
		sourceTableVSRowIds.put("questions", new ArrayList<Integer>());
		sourceTableVSRowIds.get("questions").add(51782); 
		sourceTableVSRowIds.get("questions").add(52781); 
				
		List<Change> rankedChangeList = new ArrayList<Change>();
		CreateChangeTables changeTable = new CreateChangeTables(tableVSTableForeignKey, tableVSFatherTables,
				tableVSChildTables, tableVSEventColumns,
				tableColumnVSImpactedTableColumns, tableColumnVSChangeType,
				sourceTableVSRowIds, "ubuntu", "cdhdr2", "cdpos2");
        changeTable.connect();
        changeTable.createChangeSequence();
        rankedChangeList = changeTable.rankChangeSequence();
        System.out.println("size of change list: " + rankedChangeList.size());
        for(Change change :  rankedChangeList){
        	 System.out.println("\n change table: " + change.table);
        	 System.out.println("\n change column: " + change.column);
        	 System.out.println("\n change id: " + change.id);
        	 System.out.println("\n change time: " + change.time);
        }       
        changeTable.createChangeTables();       			    		
	}
}
