package org.processmining.ocbc.models;

import java.util.List;




public interface ActivityEdge extends Edge{

	/**
	 * returns the reference activity of a activity relation
	 * 
	 * @return
	 */
	public ActivityNode getReferenceNode();
	
	/**
	 * returns the target activity of a activity relation
	 * 
	 * @return
	 */
	public ActivityNode getTargetNode();
	
	/**
	 * returns the type of a activity relation
	 * 
	 * @return
	 */
	public String getCardinalityType();
	
	/**
	 * returns the before cardinality of a activity relation
	 * 	 
	 * * @return
	 */
	public Cardinality getBeforeCardinality();
	
	/**
	 * returns the after cardinality of a activity relation
	 * 
	 * @return
	 */
	public Cardinality getAfterCardinality();
	
	/**
	 * returns the set of the crelrt relations of a activity relation
	 * 
	 * 
	 * @return
	 */
	public  List<CrelRTEdge> getRelatedCrelRTRelations();
	
	/**
	 * returns the set of the creloc relations of a activity relation
	 * 
	 * 
	 * @return
	 */
	public  List<CrelOCEdge> getRelatedCrelOCRelations();
	
	/**
	 * sets the type of a activity relation
	 * @return 
	 * 
	 * @return
	 */
	public void setCardinalityType(String cardinalityType);
	
	/**
	 * sets the before cardinality of a activity relation
	 * 	 
	 * * @return
	 */
	public void setBeforeCardinality(Cardinality beforeCardinality);
	
	/**
	 * sets the after cardinality of a activity relation
	 * 
	 * @return
	 */
	public void setAfterCardinality(Cardinality afterCardinality);
	
	
	/**
	 * set the set of the crelrt relations of a activity relation
	 * 
	 * 
	 * @return
	 */
	public void setRelatedCrelRTRelations(List<CrelRTEdge> relatedCrelRTRelations);
	
	/**
	 * set the set of the creloc relations of a activity relation
	 * 
	 * 
	 * @return
	 */
	public void setRelatedCrelOCRelations(List<CrelOCEdge> relatedCrelOCRelations);
	
	public boolean checkActivityEdgeCardinality(int beforeEventNum, int afterEventNum);
	
	public Object clone();
	
	public boolean isFatherActivityRelationOf(ActivityEdge childEdge);	
	
	public boolean isTwinActivityRelationOf(ActivityEdge childEdge);
	
	public boolean isFatherConstraintOf(String fatherConstraintType, String childConstraintType);
	
	public void setCorrelationType(String correlationType);
	
	public String getCorrelationType();
	
	public void setCorrelationOCMedia(ClassNode correlationOCMedia);
	
	public ClassNode getCorrelationOCMedia();
	
	public void setCorrelationRTMedia(ClassEdge correlationRTMedia);
	
	public ClassEdge getCorrelationRTMedia();
}