package org.processmining.ocbc.models;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;



public class Cardinality implements Serializable{
	/**
	 * 
	 */
	private static final long serialVersionUID = 7922424704483472228L;

	/**
	 * indicate the start of a cardinality
	 * 
	 */
	private int startValue;
	
	/**
	 * indicate the end of a cardinality
	 * 
	 */
	private int endValue;
	
	/**
	 * indicate the list of values if a cardinality is not continuous
	 * 
	 */
	private List<Integer> valueList;
	
	/**
	 * indicate the type of values in a cardinality. valueType = 0:half open section, e.g., 0..*; 1:close section, e.g., 0..4; 2: discrete points
	 * 
	 */
	private int valueType;
	
	/**
	 * indicate the type of cardinality time scope. valueType = 0:null;1:always; 2:eventually
	 * 
	 */
	private int cardinalityTimeScope;//1:Eventually; 2:Always; 0:just for the object-class side of an AOE relation
	
	

	public Cardinality()
	{
		valueList = new ArrayList<Integer>();
	}
	
	//single value
	public Cardinality(int startValue)
	{
		this.startValue = startValue;				
		this.endValue =	startValue;
		valueType = 0;
		valueList = new ArrayList<Integer>();
	}
	
	//a period of values
	public Cardinality(int startValue, int endValue)
	{
		this.startValue = startValue;				
		this.endValue =	endValue;
		valueType = 1;
		valueList = new ArrayList<Integer>();
	}
	
	//several single values
	public Cardinality(List<Integer> valueList)
	{
		if(valueList.size() == 1){
			startValue = valueList.get(0);		
			endValue =	valueList.get(0);
			valueType = 0;
			this.valueList = new ArrayList<Integer>();
		}		
		else if(checkNumberListContinuity(valueList)){
			valueType = 1;
			this.valueList = new ArrayList<Integer>();
		}
		else{			
			valueType = 2;
			this.valueList = valueList;
		}
	}

	// transform a letter to Cardinality
	public Cardinality(String cardinality)
	{
		valueList = new ArrayList<Integer>();
    	if(cardinality.contains(",")) 
    	{
    		valueType = 2;
    		String[] splitResult = cardinality.split(",");   		
    		for(int i=0; i<splitResult.length; i++)
    		{
    			valueList.add(Integer.parseInt(splitResult[i]));
    		}   		
    					    		
    	}
    	
    	//add the function to parse string with ".." such as "1..*"
    	else if(cardinality.contains(".."))
    	{       				      	       	
    		String newCardinality = cardinality.replace("..", "-");
    		String[] arrayCardinality = newCardinality.split("-");
    		
    		String firstPart = arrayCardinality[0];
        	String lastPart = arrayCardinality[1];
    		
        	startValue = Integer.parseInt(firstPart);
			valueType = 1;
			valueList = new ArrayList<Integer>();
			
    		if(lastPart.equals("*"))
    			endValue = Integer.MAX_VALUE;
    		else
    			endValue = Integer.parseInt(lastPart);    		
    	}
    	
    	else 
    	{
    		if(cardinality.equals("*")){
    			startValue = 0;
    			endValue = Integer.MAX_VALUE;
    			valueType = 1;
    			valueList = new ArrayList<Integer>();
    		}
    		else{
    			startValue = Integer.parseInt(cardinality);
    			endValue = Integer.parseInt(cardinality);
    			valueType = 0;
    			valueList = new ArrayList<Integer>();
    		}
    	}
	}
		
	@SuppressWarnings("unchecked")
	public Object clone() {		
		Cardinality clone = new Cardinality();

		clone.startValue = startValue;
		clone.endValue = endValue;
		clone.valueType = valueType;
		clone.cardinalityTimeScope = cardinalityTimeScope;
		clone.valueList = (ArrayList<Integer>) ((ArrayList<Integer>)valueList).clone();
		
		return clone;
	}
	
	public int getStartValue()
	{
		return startValue;
	}
	
	public int getEndValue()
	{
		return endValue;		
	}
	
	public List<Integer> getValueList()
	{
		return valueList;
	}
	
	public int getValueType()
	{
		return valueType;
	}
	
	public int getCardinalityTimeScope()
	{
		return cardinalityTimeScope;		
	}
	
	/**
	 * transform the cardinality value list into cardinality string;
	 * current, we only have such string: "1", "1..*","0..1", "*"
	 */
	public String toString()
	{
		String cardinalityDisplay = null;

		if(valueType == 0)
		{
			cardinalityDisplay = String.valueOf(startValue);
//			if(startValue>1){
////				cardinalityDisplay = 
//			}

		}
		else if(valueType == 1)
		{
			
			if(endValue == Integer.MAX_VALUE)
			{
				if(startValue == 0) 
					cardinalityDisplay ="*";
				else
					cardinalityDisplay = startValue +".." + "*";
			}
			else 
				cardinalityDisplay = startValue +".." + endValue;
		}
		else if(valueType == 2)
		{
			cardinalityDisplay = valueList.get(0).toString();
			for(int i=1;i<valueList.size();i++)
			{
				cardinalityDisplay = cardinalityDisplay + "," +valueList.get(i);
			}
		}		
		return cardinalityDisplay;
	}
	
//	public String toString()
//	{
//		String cardinalityDisplay = null;
//
//		if(valueType == 0)
//		{
//			cardinalityDisplay = String.valueOf(startValue);
//
//		}
//		else if(valueType == 1)
//		{
//			
//			if(endValue == Integer.MAX_VALUE)
//			{
//				if(startValue == 0) 
//					cardinalityDisplay ="*";
//				else
//					cardinalityDisplay = startValue +".." + "*";
//			}
//			else 
//				cardinalityDisplay = startValue +".." + endValue;
//		}
//		else if(valueType == 2)
//		{
//			cardinalityDisplay = valueList.get(0).toString();
//			for(int i=1;i<valueList.size();i++)
//			{
//				cardinalityDisplay = cardinalityDisplay + "," +valueList.get(i);
//			}
//		}		
//		return cardinalityDisplay;
//	}
	
	public String toLetters()
	{
		String cardinalityDisplay = null;

		if(valueType == 0)
		{
			cardinalityDisplay = String.valueOf(startValue);

		}
		else if(valueType == 1)
		{
			
			if(endValue == Integer.MAX_VALUE)
			{
				if(startValue == 0) 
					cardinalityDisplay ="*";
				else
					cardinalityDisplay = startValue +".." + "*";
			}
			else 
				cardinalityDisplay = startValue +".." + endValue;
		}
		else if(valueType == 2)
		{
			cardinalityDisplay = valueList.get(0).toString();
			for(int i=1;i<valueList.size();i++)
			{
				cardinalityDisplay = cardinalityDisplay + "," +valueList.get(i);
			}
		}
		
//		if(cardinalityTimeScope == 1)
//		{
//			cardinalityDisplay = "E" + cardinalityDisplay;
//		}
//		else if(cardinalityTimeScope == 2)
//		{
//			cardinalityDisplay = "A" + cardinalityDisplay;
//		}
		
			
		return cardinalityDisplay;
	}
	
	
	public void setStartValue(int startValue)
	{
		this.startValue = startValue;
	}
	
	public void setEndValue(int endValue)
	{
		this.endValue = endValue;		
	}
	
	public void addValueToList(int value)
	{
		this.valueList.add(value);		
	}
	
	public void setCardinalityTimeScope(int cardinalityTimeScope)
	{
		this.cardinalityTimeScope = cardinalityTimeScope;		
	}
	
	/**
	 * check if the cardinality is followed
	 * @param inputValue
	 * @return
	 */
	public boolean checkCardinality(int inputValue)
	{
		if(inputValue <= endValue && inputValue >= startValue)
			return true;
		else 
			return false;
	}
	
	/*check if the input cardinality string follow the rule;if yes, return "true"
	 * there are two types.type=1/2:the cardinality contains E/A. type=0: the cardinality does not contain E/A
	 */
	public String checkCardinalityString(String cardinality)
	{
		String checkResult = "Wrong! The cardinality violates the rule.";	

		if(cardinality.length() == 4){
			if(Character.isDigit(cardinality.charAt(0)) && //the first value is a digit (0~9)
			   cardinality.substring(1, 3).equals("..") && //the second and third values are ".."
			   (cardinality.substring(3, 4).equals("*")|| Character.isDigit(cardinality.charAt(3)))){//the fourth value is "*" or a digit (0~9)			
				if(Character.isDigit(cardinality.charAt(3))){
					int startValue = Integer.parseInt(cardinality.substring(0, 1));
					int endValue = Integer.parseInt(cardinality.substring(3, 4));
					if(endValue > startValue){
						checkResult = "true";						
					}
					else
						checkResult = "Wrong! The end value should be larger than the start value.";
				}
				else
					checkResult = "true";
			}
			else
				checkResult = "Wrong! The cardinality should be like 1..*.";
		}

		//parse cardinality string such as E1 or A*
		else if(cardinality.length() == 1){
			if((cardinality.substring(0, 1).equals("*") || Character.isDigit(cardinality.charAt(0)))){//the value is "*" or a digit (0~9)					
				checkResult = "true";
			}
			else
				checkResult = "Wrong! The cardinality should be like 1.";
		}

		else 
			checkResult = "Wrong! The cardinality should be like 1..* or 1.";
		
		return checkResult;
	}
	
	public boolean checkNumberListContinuity(List<Integer> valueList){
		boolean flag = false;
		int maxElement = 0;
		int minElement = Integer.MAX_VALUE;
		List<Integer> valueSet = new ArrayList<Integer>();
		for(int element : valueList){
			if(maxElement < element) maxElement = element;
			if(minElement > element) minElement = element;
			if(!valueSet.contains(element)) valueSet.add(element);
		}
		if((maxElement - minElement) == (valueSet.size() - 1)) {
			flag = true;
			startValue = minElement;
			endValue = maxElement;
		}
		return flag;
	}
	public static void main(String[] str){
		String cardinality = "1..7";
		String car = cardinality.replace("..", "-");
		String[] array = car.split("-");
		System.out.print("\n true" + array[0]);
		System.out.print("\n true" + array[1]);
		Cardinality cardinality1 = new Cardinality("A*");
		Cardinality cardinality2 = new Cardinality("E*");
		System.out.print("string" + cardinality1.toString());
		String checkResult = new Cardinality().checkCardinalityString(cardinality);
		if(checkResult.equals("true")){
			System.out.print("\n true" + checkResult);
		}
		else
			System.out.print("\n false" + checkResult);
	}
}