package org.processmining.ocbc.models;

import java.util.List;



public interface ClassEdge extends Edge{
	/**
	 * returns the source of a class relation
	 * 
	 * @return
	 */
	public ClassNode getSourceNode();
	
	/**
	 * returns the target of a class relation
	 * 
	 * @return
	 */
	public ClassNode getTargetNode();
	
	/**
	 * returns the cardinality of the source of a class relation
	 * 
	 * @return
	 */
	public Cardinality getSourceAlwaysCardinality();
	
	/**
	 * returns the cardinality of the source of a class relation
	 * 
	 * @return
	 */
	public Cardinality getSourceEventuallyCardinality();
	
	/**
	 * returns the cardinality of the target of a class relation
	 * 
	 * @return
	 */
	public Cardinality getTargetAlwaysCardinality();
	
	/**
	 * returns the cardinality of the target of a class relation
	 * 
	 * @return
	 */
	public Cardinality getTargetEventuallyCardinality();
	
	/**
	 * returns the set of the crel relations of a class relation
	 * 
	 * 
	 * @return
	 */
	public  List<CrelRTEdge> getRelatedCrelRTRelations();
	

	/**
	 * set the cardinality of the source of a class relation
	 * 
	 * @return
	 */
	public void setSourceAlwaysCardinality(Cardinality sourceCardinality);
	
	/**
	 * set the cardinality of the source of a class relation
	 * 
	 * @return
	 */
	public void setSourceEventuallyCardinality(Cardinality sourceCardinality);
	
	/**
	 * set the cardinality of the target of a class relation
	 * 
	 * @return
	 */
	public void setTargetAlwaysCardinality(Cardinality targetCardinality);
	
	/**
	 * set the cardinality of the target of a class relation
	 * 
	 * @return
	 */
	public void setTargetEventuallyCardinality(Cardinality targetCardinality);
	
	/**
	 * set the set of the crel relations of a class relation
	 * 
	 * 
	 * @return
	 */
	public void setRelatedCrelRTRelations(List<CrelRTEdge> relatedCrelRTRelations);
	
	public Object clone();
}