package org.processmining.ocbc.models;

import java.util.List;
import java.util.Map;

import org.jgraph.JGraph;
import org.processmining.framework.util.Pair;
import org.processmining.mxgraph.view.mxGraph;
import org.processmining.ocbc.discovery.impl.OCBCModelDiscovery;
import org.processmining.ocbc.discovery.result.ModelDiscoveryResult;
import org.processmining.ocbc.log.model.XOCLog;
import org.processmining.ocbc.models.visualization.cell.MEdgeCell;
import org.processmining.ocbc.models.visualization.cell.MyOriginalCell;
import org.w3c.dom.Document;



public interface OCBCModel extends Cloneable {
	

	public String getModelSource();              

	public void setModelSource(String source);
	
	public String getModelEnvironment();

	public void setModelEnvironment(String environment);
	
	public XOCLog getModelSourceLog();

	public void setModelSourceLog(XOCLog sourceLog);
	
	public ModelDiscoveryResult getModelDiscoveryResult();

	public void setModelDiscoveryResult(ModelDiscoveryResult modelDiscoveryResult);
	
	public OCBCModelDiscovery getModelDiscovery();
	
	public void setModelDiscovery(OCBCModelDiscovery modelDiscovery);
	
	public List<String> getTransitiveRelationIdList();
	
	public void setTransitiveRelationIdList(List<String> transitiveRelationIdList);
	/**
	 * returns the version of a model
	 * 
	 * @return
	 */
	public String getModelVersion();
	
	/**
	 * sets the version of a model
	 * @param version
	 * 
	 */
	public void setModelVersion(String version);
	
	
	/**
	 * returns the name of a model
	 * 
	 * @return
	 */
	public String getModelName();
	
	/**
	 * sets the name of a model
	 * @param name
	 * 
	 */
	public void setModelName(String name);
	
	/**
	 * returns the graph of a model
	 * 
	 * @return
	 */
	public JGraph getGraph();
	
	/**
	 * sets the graph of a model
	 * @param name
	 * 
	 */
	public void setGraph(JGraph graph);
	
	/**
	 * returns the graphX of a model
	 * 
	 * @return
	 */
	public mxGraph getGraphX();
	
	/**
	 * sets the graphX of a model
	 * @param name
	 * 
	 */
	public void setGraphX(mxGraph graphX);
		
	/**
	 * get the file of a model
	 * @param name
	 * 
	 */
	public Document getModelFile();
	
	/**
	 * get the string of a model
	 * @param name
	 * 
	 */
	public String getModelString();

	/**
	 * sets the file of a model
	 * @param name
	 * 
	 */
	public void setModelFile(Document modelFile);
	
	/**
	 * sets the string of a model
	 * @param name
	 * 
	 */
	public void setModelString(String modelString);
	
	/**
	 * returns the cells of a graph of a model
	 * 
	 * @return
	 */
	public List<MyOriginalCell> getGraphCells();
	
	/**
	 * sets the cells of a graph of a model
	 * @param name
	 * 
	 */
	public void setGraphCells(List<MyOriginalCell> graphCells);
		
	/**
	 * returns the set of all classes in the model
	 * 
	 */
	public List<ClassNode> getAllClasses();
	
	/**
	 * returns the set of all class names in the model
	 * 
	 */
	public List<String> getAllClassNames();
	
	/**
	 * returns one class of the input name in the model
	 * 
	 */
	public ClassNode getClassByName(String uniqueClassName);
	
	/**
	 * returns one class of the id in the model
	 * 
	 */
	public ClassNode getClassById(String id);
	
	/**
	 * returns one class relation of the id in the model
	 * 
	 */
	public ClassEdge getClassRelationById(String id);
	
	/**
	 * returns one aoc relation of the activity id and class id in the model
	 * 
	 */
	public AOCEdge getAOCRelationByActivityIdAndClassId(String activityId, String classId);
	
	/**
	 * returns the set of all activities in the model
	 * 
	 */
	public List<ActivityNode> getAllActivities();
	
	/**
	 * returns one activity of the id in the model
	 * 
	 */
	public ActivityNode getActivityById(String id);
	
	/**
	 * returns the set of all activity names in the model
	 * 
	 */
	public List<String> getAllActivityNames();
	/**
	 * returns the set of all class relations in the model
	 * 
	 */
	public List<ClassEdge> getAllClassRelations();
	
	/**
	 * returns the set of all activity relations in the model
	 * 
	 */
	public List<ActivityEdge> getAllActivityRelations();
	
	/**
	 * returns one activity relation of the id in the model
	 * 
	 */
	public ActivityEdge getActivityRelationById(String id);
	
	/**
	 * returns the set of all activity relations with the same reference activity,
	 * target activity and class
	 * 
	 */
	public List<ActivityEdge> getActivityRelationFamilyA(String refActicityId, String tarActivityId, String classId);
	
	/**
	 * returns the set of all activity relations with the same reference activity,
	 * target activity and class relation
	 * 
	 */
	public List<ActivityEdge> getActivityRelationFamilyB(String refActicityId, String tarActivityId, String classRelationId);
	
	/**
	 * returns the set of all AOC relations in the model
	 * 
	 */
	public List<AOCEdge> getAllAOCRelations();
	
	/**
	 * returns the set of all crelOC relations in the model
	 * 
	 */
	public List<CrelOCEdge> getAllCrelOCRelations();
	
	/**
	 * returns the crel oc relaton with the activity edge and class
	 * 
	 */
	public CrelOCEdge getCrelOCRelationByActivityEdgeAndClass(String activityEdgeId, String ClassId);
	
	/**
	 * returns the set of all crelRT relations in the model
	 * 
	 */
	public List<CrelRTEdge> getAllCrelRTRelations();
	
	/**
	 * returns the crel rt relaton with the activity edge and class edge
	 * 
	 */
	public CrelRTEdge getCrelRTRelationByActivityEdgeAndClassEdge(String activityEdgeId, String ClassEdgeId);
	
	//get the sequential activity pair list
	public List<Pair<String, String>> getSequentialActivityPairList();
	
	/**
	 * sets the set of all classes in the model
	 * 
	 */
	public void setAllClasses(List<ClassNode> allClasses);
	
	/**
	 * sets the set of all activities in the model
	 * 
	 */
	public void setAllActivities(List<ActivityNode> allActivities);
	
	/**
	 * sets the set of all class relations in the model
	 * 
	 */
	public void setAllClassRelations(List<ClassEdge> allClassRelations);
	
	/**
	 * sets the set of all activity relations in the model
	 * 
	 */
	public void setAllActivityRelations(List<ActivityEdge> allActivityRelations);
	
	/**
	 * sets the set of all AOC relations in the model
	 * 
	 */
	public void setAllAOCRelations(List<AOCEdge> allAOCRelations);
	
	/**
	 * sets the set of all crelOC relations in the model
	 * 
	 */
	public void setAllCrelOCRelations(List<CrelOCEdge> allCrelOCRelations);
	
	/**
	 * sets the set of all crelRT relations in the model
	 * 
	 */
	public void setAllCrelRTRelations(List<CrelRTEdge> allCrelRTRelations);
	
	/**
	 * add one class into the set of all classes in the model
	 * 
	 */
	public void addClass(ClassNode classNode);
	
	/**
	 * add one activity into the set of all activities in the model
	 * 
	 */
	public void addActivity(ActivityNode activityNode);
	
	/**
	 * delete one class from the set of all classes in the model
	 * 
	 */
	public void deleteClass(ClassNode classNode);
	
	/**
	 * delete one activity from the set of all activities in the model
	 * 
	 */
	public void deleteActivity(ActivityNode activityNode);
	
	/**
	 * get related edge cells for one class cell
	 * 
	 */
	public List<MEdgeCell> getRelatedEdgeCellsForOneCell(MyOriginalCell cell);
	
//	/**
//	 * get related edge cells for one activity cell
//	 */
//	public List<MEdgeCell> getRelatedEdgeCellsForOneActivityCell(ActivityCell cell);
	
	public Object clone();
	
	//select an activity type based on a list of (before, after)
	public String transformCardinalityPairListToActicityType(List<Pair<Integer, Integer>> acticvityCardinalityPairList);
	
	//generate an activity type list based on a list of (before, after)
	public List<String> transformCardinalityPairListToActivityTypeList(List<Pair<Integer, Integer>> activityCardinalityPairList);

	//generate an activity type vs confidence map based on a list of (before, after)
	public Map<String, Double> transformCardinalityPairListToActivityTypeVSConfidence(List<Pair<Integer, Integer>> activityCardinalityPairList);

	/**
	 * @param acticvityCardinalityPairList
	 * @return a map between constraint type and its number of instances based on a list of cardinality pair
	 */
	public Map<String, Integer> transformCardinalityPairListToConstraintVSNumMap(List<Pair<Integer, Integer>> acticvityCardinalityPairList);
		
		
	//check if two class are related, i.e., there exist a class relation between the two classes
	//if yes, return the class relation id; if no, return null
	public String checkTwoClassesIfRelated(String firstClassNodeId, String secondClassNodeId);

	//check if one class and one activity are related (directly connected )
//	public boolean isClassAndActivityRelated(String classNodeId, String activityNodeId);
	
	//remove an activity edge from the model (edge and cell)
	public void removeModelActivityEdge(String activityEdgeId);

	//add the sequential activity pair
	public void addSequentialActivityPair(String sourceActivityNodeId, String targetActivityNodeId,
			String activityType);

	//add the cell (for acticity or class) by id
	public MyOriginalCell getNodeCellById(String Id);
	
	public void generateTypeVSClassRelationMap();
		
	public Map<String, ClassEdge> getTypeVSClassRelationMap();

	/**
	 * get the class relation which has the input two class nodes
	 * if yes, return the class relation id; if no, return null
	 * @param firstClassNodeId
	 * @param secondClassNodeId
	 * @return
	 */
	public ClassEdge getClassRelationByTwoClassNodes(String firstClassNodeId, String secondClassNodeId);
	
	/**
	 * create a map to record all related activity for each class in the OCBC model
	 * @return
	 */
	public Map<String, List<String>> generateClassVSRelatedActivityListMap();
	
	/**
	 * create a map to record all related class for each activity in the OCBC model
	 * @return
	 */
	public Map<String, List<String>> generateActivityVSRelatedClassListMap();
}