package org.processmining.ocbc.models.editor.interaction;

import java.awt.event.ActionEvent;
import java.util.UUID;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JSlider;
import javax.swing.JTextField;

import org.processmining.ocbc.models.editor.panels.GraphPanel;
import org.processmining.ocbc.models.editor.panels.InformationPanel;

public class EditorActionListenerAddNode implements EditorActionListener {
	GraphMouseListener graphMouseListener;
	GraphPanel graphPanel;
	InformationPanel informationPanel;
	String elementType;
	JTextField nameField;
	JSlider childIndex;
	String flag;
	JButton focusedButton;
	
	public EditorActionListenerAddNode(JButton focusedButton, GraphMouseListener graphMouseListener, InformationPanel informationPanel,
			GraphPanel graphPanel, String elementType, JTextField nameField, JSlider childIndex){
		this.graphMouseListener = graphMouseListener;
		this.informationPanel = informationPanel;
		this.graphPanel = graphPanel;
		this.elementType = elementType;
		this.nameField = nameField;
		this.childIndex = childIndex;
		this.focusedButton = focusedButton;
	}
	

	@Override
	public void actionPerformed(ActionEvent e) {
		System.out.println("button click relation to redraw the graph panel");
		graphPanel.redraw();
		if(focusedButton.equals(graphMouseListener.getSelectedButton()))
		{
			informationPanel.updateInformation("release drawing button");
			graphMouseListener.getSelectedButton().setBorder(BorderFactory.createRaisedBevelBorder());
			graphMouseListener.setSelectedButton(null);
		}
		else
		{
			focusedButton.setBorder(BorderFactory.createLoweredBevelBorder());			
			graphMouseListener.setSelectedButton(focusedButton);	
			graphMouseListener.getSelectedButton().setText(elementType.toString());			
			
			switch(elementType){
			case "ClassNode":
				informationPanel.updateInformation(elementType.toString() + " button is selected. It is used to draw a class node."
						+ "\n Please click where you want to draw it on the graph panel.");
				break;
			case "ActivityNode":
				informationPanel.updateInformation(elementType.toString() + " button is selected. It is used to draw an activity node."
						+ "\n Please click where you want to draw it on the graph panel.");
				break;
			case "ClassEdge":
				informationPanel.updateInformation(elementType.toString() + " button is selected. It is used to draw a class edge between "
						+ "two class nodes. \n Please select two class nodes sequentially. "
						+ "\n The first selected node is considered as the source node while the second selected node is considered "
						+ "as the target node.");
				break;
			case "AOCEdge":
				informationPanel.updateInformation(elementType.toString() + " button is selected. It is used to draw an AOC edge between "
						+ "an activity node and a class node. \n Please select an activity node and then a class node.");
				break;
			case "CrelEdge":
				informationPanel.updateInformation(elementType.toString() + " button is selected. It is used to draw an crel edge between "
						+ "an activity edge and a class node/edge. \n Please select an activity edge (by clicking on the middle transparent node"
						+ ") and then a class node/edge (by clicking on the middle transparent node).");
				break;
			case "ActivityEdgeResponse":
				informationPanel.updateInformation(elementType.toString() + " button is selected. It is used to draw an acticity edge between "
						+ "two activity nodes. \n Please click two activity nodes sequentially. The first selected activity is considered as "
						+ "the reference activity and the second one is considered as target activity. \n This relation indicates for each reference event "
						+ ", the number of the corresponding target events occuring after the reference event is equal to or more than 1.");
				break;
			case "ActivityEdgeNonResponse":
				informationPanel.updateInformation(elementType.toString() + " button is selected. It is used to draw an acticity edge between "
						+ "two activity nodes. \n Please click two activity nodes sequentially. The first selected activity is considered as "
						+ "the reference activity and the second one is considered as target activity. \n This relation indicates for each reference event "
						+ ", the number of the corresponding target events occuring after the reference "
						+ "event is equal to 0.");
				break;
			case "ActivityEdgeUnaryResponse":
				informationPanel.updateInformation(elementType.toString() + " button is selected. It is used to draw an acticity edge between "
						+ "two activity nodes. \n Please click two activity nodes sequentially. The first selected activity is considered as "
						+ "the reference activity and the second one is considered as target activity. \n This relation indicates for each reference event "
						+ ", the number of the corresponding target events occuring after the reference event is equal to 1.");
				break;
			case "ActivityEdgePrecedence":
				informationPanel.updateInformation(elementType.toString() + " button is selected. It is used to draw an acticity edge between "
						+ "two activity nodes. \n Please click two activity nodes sequentially. The first selected activity is considered as "
						+ "the reference activity and the second one is considered as target activity. \n This relation indicates for each reference event "
						+ ", the number of the corresponding target events occuring before the reference "
						+ "event is equal to or more than 1.");
				break;
			case "ActivityEdgeUnaryPrecedence":
				informationPanel.updateInformation(elementType.toString() + " button is selected. It is used to draw an acticity edge between "
						+ "two activity nodes. \n Please click two activity nodes sequentially. The first selected activity is considered as "
						+ "the reference activity and the second one is considered as target activity. \n This relation indicates for each reference event "
						+ ", the number of the corresponding target events occuring before the reference "
						+ "event is equal to 1.");
				break;
			case "ActivityEdgeNonPrecedence":
				informationPanel.updateInformation(elementType.toString() + " button is selected. It is used to draw an acticity edge between "
						+ "two activity nodes. \n Please click two activity nodes sequentially. The first selected activity is considered as "
						+ "the reference activity and the second one is considered as target activity. \n This relation indicates for each reference event "
						+ ", the number of the corresponding target events occuring before the reference "
						+ "event is equal to 0.");
				break;
			case "ActivityEdgeCoExistence":
				informationPanel.updateInformation(elementType.toString() + " button is selected. It is used to draw an acticity edge between "
						+ "two activity nodes. \n Please click two activity nodes sequentially. The first selected activity is considered as "
						+ "the reference activity and the second one is considered as target activity. \n This relation indicates for each reference event "
						+ ", the number of the corresponding target events "
						+ "is equal to or more than 1.");
				break;
			case "ActivityEdgeNonCoExistence":
				informationPanel.updateInformation(elementType.toString() + " button is selected. It is used to draw an acticity edge between"
						+ "two activity nodes. \n Please click two activity nodes sequentially. The first selected activity is considered as "
						+ "the reference activity and the second one is considered as target activity. \n This relation indicates for each reference event "
						+ ", the number of the corresponding target events "
						+ "is equal to 0.");
				break;	
			case "ActivityEdgeResponse_ActivityEdgePrecedence":
				informationPanel.updateInformation(elementType.toString() + " button is selected. It is used to draw a combined activity edge "
						+ "(with two reference dots) between two activity nodes. \n It equals to two seprate activity edges (" + elementType.toString().split("_")[0]
						+" and " + elementType.toString().split("_")[1]+ " ). \n Please click two activity nodes sequentially. The direction of the arrow is from the "
						+ "first selected activity to the second selected activity");
				break;
			case "ActivityEdgeUnaryResponse_ActivityEdgeUnaryPrecedence":
				informationPanel.updateInformation(elementType.toString() + " button is selected. It is used to draw a combined activity edge "
						+ "(with two reference dots) between two activity nodes. \n It equals to two seprate activity edges (" + elementType.toString().split("_")[0]
						+" and " + elementType.toString().split("_")[1]+ " ). \n Please click two activity nodes sequentially. The direction of the arrow is from the "
						+ "first selected activity to the second selected activity");
				break;
			case "ActivityEdgeCoExistence_ActivityEdgeCoExistence":
				informationPanel.updateInformation(elementType.toString() + " button is selected. It is used to draw a combined activity edge "
						+ "(with two reference dots) between two activity nodes. \n It equals to two seprate activity edges (" + elementType.toString().split("_")[0]
						+" and " + elementType.toString().split("_")[1]+ " ). \n Please click two activity nodes sequentially. The direction of the arrow is from the "
						+ "first selected activity to the second selected activity");
				break;
			case "ActivityEdgeNonResponse_ActivityEdgeNonPrecedence":
				informationPanel.updateInformation(elementType.toString() + " button is selected. It is used to draw a combined activity edge "
						+ "(with two reference dots) between two activity nodes. \n It equals to two seprate activity edges (" + elementType.toString().split("_")[0]
						+" and " + elementType.toString().split("_")[1]+ " ). \n Please click two activity nodes sequentially. The direction of the arrow is from the "
						+ "first selected activity to the second selected activity");
				break;
			case "ActivityEdgeNonCoExistence_ActivityEdgeNonCoExistence":
				informationPanel.updateInformation(elementType.toString() + " button is selected. It is used to draw a combined activity edge "
						+ "(with two reference dots) between two activity nodes. \n It equals to two seprate activity edges (" + elementType.toString().split("_")[0]
						+" and " + elementType.toString().split("_")[1]+ " ). \n Please click two activity nodes sequentially. The direction of the arrow is from the "
						+ "first selected activity to the second selected activity");
				break;
			}
			
			if(graphMouseListener.getPreviouslySelectedButton() != null)
			{
				graphMouseListener.getPreviouslySelectedButton().setBorder(BorderFactory.createRaisedBevelBorder());
			}
			
		}

		graphPanel.redraw();		
	}
	
	public boolean isEnabled(){
		UUID elementId = graphMouseListener.getSelectedID();
//		if(!(tree.getElement(elementId) instanceof Block) && (tree.getRoot() != null)){
//			return false;
//		}
		return true;
	}


}
