package org.processmining.ocbc.models.editor.interaction;

import java.awt.Color;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.util.UUID;

import javax.swing.BorderFactory;
import javax.swing.JButton;

import org.jgraph.JGraph;
import org.jgraph.graph.GraphConstants;
import org.processmining.ocbc.models.ActivityEdge;
import org.processmining.ocbc.models.ClassEdge;
import org.processmining.ocbc.models.ClassNode;
import org.processmining.ocbc.models.OCBCModel;
import org.processmining.ocbc.models.editor.panels.EditorPanel;
import org.processmining.ocbc.models.editor.panels.GraphPanel;
import org.processmining.ocbc.models.editor.panels.InformationPanel;
import org.processmining.ocbc.models.visualization.ModelLayoutBuilder;
import org.processmining.ocbc.models.visualization.cell.AOCEdgeCell;
import org.processmining.ocbc.models.visualization.cell.ActivityCell;
import org.processmining.ocbc.models.visualization.cell.ActivityEdgeCell;
import org.processmining.ocbc.models.visualization.cell.ClassCell;
import org.processmining.ocbc.models.visualization.cell.ClassEdgeCell;
import org.processmining.ocbc.models.visualization.cell.CrelEdgeCell;
import org.processmining.ocbc.models.visualization.cell.LabelCell;
import org.processmining.ocbc.models.visualization.cell.MNodeCell;
import org.processmining.ocbc.models.visualization.cell.MyOriginalCell;
import org.processmining.ocbc.models.visualization.cell.TransparentCell;




//public class GraphMouseListener extends MouseAdapter {
public class GraphMouseListener  implements MouseMotionListener,MouseListener{  

	OCBCModel model;
	ModelLayoutBuilder builder;
	
	UUID selectedID;
	UUID previouslySelectedID;
	JGraph graph;
	GraphPanel graphPanel;
	InformationPanel informationPanel;
	EditorPanel editorPanel;
	String elementType;
	String previouslyElementType;
	
	MyOriginalCell highlightedCell;//when the mouse arrow is on a cell, the cell is highlighted
	
	private JButton focusedButton;
	private JButton previouslyFocusedButton;
	
	private MyOriginalCell firstlyFocusedCell;//used for keeping the first point when drawing a edge
	int firstX;
	int firstY;
	
	private MyOriginalCell secondlyFocusedCell;//used for keeping the second point when drawing a edge
	int secondX;
	int secondY;
	
	private MyOriginalCell focusedCell;
	
	public GraphMouseListener(OCBCModel model, ModelLayoutBuilder builder, JGraph graph){
		this.builder = builder;
		this.model = model;
		this.graph = graph;

//		this.foldedNodes = foldedNodes;
	}
	
	public void addGraphPanel(GraphPanel graphPanel){
		this.graphPanel = graphPanel;
	}
	
	public void addInformationPanel(InformationPanel informationPanel){
		this.informationPanel = informationPanel;
	}
	
	public void addEditorPanel(EditorPanel editorPanel){
		this.editorPanel = editorPanel;
	}
	
	public void updateGraph(JGraph graph){
		this.graph = graph;
	}
	
	public EditorPanel getEditorPanel(){
		return editorPanel;
	}
	
	public UUID getSelectedID(){
		return selectedID;
	}
	
	public UUID getPreviouslySelectedID(){
		return previouslySelectedID;
	}
	
	/**
	 * Sets the selected ID, can be used for variables and originators (it will raise a mousePressed event)
	 * @param newID
	 */
	public void setSelectedButton(JButton focusedButton){
		this.previouslyFocusedButton = this.focusedButton;
		this.focusedButton = focusedButton;
		System.out.println("set focusedButton");
	}
	
	public JButton getSelectedButton(){
		return focusedButton;
	}
	
	
	public void setSelectedCell(MyOriginalCell cell){
		this.focusedCell = cell;
	}
	
	public MyOriginalCell getSelectedCell(){
		return focusedCell;
	}
	
	public JButton getPreviouslySelectedButton(){
		System.out.println("getPreviouslySelectedButton()");
		return this.previouslyFocusedButton;
	}
	
	public void setSelectedID(UUID newID){
		this.previouslySelectedID = this.selectedID;
		this.selectedID = newID;
	}
	
	public String getElementType(){
		return elementType;
	}
	
	public String getPreviouslyElementType(){
		return previouslyElementType;
	}
	
	/**
	 * Sets the selected elementType, can be used for variables and originators (it will raise a mousePressed event)
	 * @param newID
	 */
	public void setElementType(String elementType){
		this.previouslyElementType = this.elementType;
		this.elementType = elementType;
		System.out.println("set elementType");
	}

	
	@Override
	public void mousePressed(MouseEvent e){		
		System.out.println("\n monitor mouse pressed");
		int cellFlag;// 0:node; 1:edge
		
		// if some drawing button is focused, the click is for drawing a cell
		if(focusedButton != null) {
			if(focusedButton.getText().equals("ClassNode") || focusedButton.getText().equals("ActivityNode"))
				cellFlag=0;
			else 
				cellFlag=1;
			//node button is selected
			if(cellFlag == 0) {
				System.out.println("focusedButton is not null");		
				focusedButton.setBorder(BorderFactory.createRaisedBevelBorder());						
				drawNode(e.getX(),e.getY());	
			}
			//edge button is selected
			else if(cellFlag == 1) {
				System.out.println(" null");
				int x = e.getX();
				int y = e.getY();
				MyOriginalCell cell = (MyOriginalCell) graph.getFirstCellForLocation(x, y);
				if(cell !=null){
					selectedID = (UUID) cell.getAttributes().get(ModelLayoutBuilder.ID);
					
					focusedCell = cell;
					System.out.println("selectedID"+selectedID);
					System.out.println("selectedID"+cell.getUserObject());
					editorPanel.mousePressed();
					
					if(firstlyFocusedCell == null) 
					{
						firstlyFocusedCell = cell;
						firstX = e.getX();
						firstY = e.getY();
						informationPanel.updateInformation("The first element is selected. Please select the second element");
						System.out.println("get the first poing");
					}
					else 
					{
						secondlyFocusedCell = cell;
						secondX = e.getX();
						secondY = e.getY();
						focusedButton.setBorder(BorderFactory.createRaisedBevelBorder());
						System.out.println("draw a edge between two cells");
						if(firstlyFocusedCell != null){
							System.out.println(firstlyFocusedCell.getUserObject());
						}
						drawEdge(firstlyFocusedCell, firstX, firstY, secondlyFocusedCell, secondX, secondY);
						focusedButton = null;
						firstlyFocusedCell = null;
						secondlyFocusedCell = null;												
					}
				
				}			
				else {
					firstlyFocusedCell = null;
					secondlyFocusedCell = null;
					focusedCell = null;
					focusedButton.setBorder(BorderFactory.createRaisedBevelBorder());
					focusedButton = null;
					editorPanel.mousePressed();
					informationPanel.updateInformation("Wrong selection. Release drawing button");
					
				}
			}
		}			
		// if no drawing button is focused, the click is for selecting the cell
		else if (focusedButton == null)
		{
			System.out.println(" focusedButton ==  null");
			int x = e.getX();
			int y = e.getY();
			MyOriginalCell cell = (MyOriginalCell) graph.getFirstCellForLocation(x, y);
			if(cell != null)
			{
				selectedID = (UUID) cell.getAttributes().get(ModelLayoutBuilder.ID);
				
				focusedCell = cell;
				System.out.println("selectedID"+selectedID);
				System.out.println("selectedID"+cell.getUserObject());
				editorPanel.mousePressed();
				if(cell instanceof ClassCell){
					informationPanel.updateInformation("A class node is selected. You can modify its name or delete it (provided that it has no related edges)"
							+ " by the `Modify model` panel. The node can be moved by dragging it.");
				}
				else if(cell instanceof ActivityCell){
					informationPanel.updateInformation("An activity node is selected. You can modify its name or delete it (provided that it has no related edges)"
							+ " by the `Modify model` panel. The node can be moved by dragging it.");
				}
				else if(cell instanceof LabelCell){
					informationPanel.updateInformation("An label node is selected. You can modify its name by the `Modify model` panel."
							+ " The node can be moved by dragging it.");
				}
				else if(cell instanceof ClassEdgeCell){
					informationPanel.updateInformation("A class edge is selected. You can modify its cardinality or delete it (provided that it has no related edges) "
							+ "by the `Modify model` panel."
							+ " The edge can be moved by dragging its corresponding transparent node.");
				}
				else if(cell instanceof AOCEdgeCell){
					informationPanel.updateInformation("An AOC edge is selected. You can modify its cardinality or delete it (provided that it has no related edges) "
							+ "by the `Modify model` panel."
							+ " The edge can be moved by dragging its corresponding transparent node.");
				}
				else if(cell instanceof ActivityEdgeCell){
					informationPanel.updateInformation("An actiity edge is selected. You can delete it (provided that it has no related edges)."
							+ " The edge can be moved by dragging its corresponding transparent node.");
				}
				else if(cell instanceof CrelEdgeCell){
					informationPanel.updateInformation("An crel edge is selected. You can delete it."
							+ " The edge can be moved by dragging its corresponding transparent node.");
				}
				else if(cell instanceof TransparentCell){
					informationPanel.updateInformation("An transparent node is selected."
							+ " It can be moved by dragging it.");
				}
				System.out.println("something is really selected");
			}
			
			else
			{
				System.out.println("Nothing is selected");
				informationPanel.updateInformation("Nothing is selected");
				focusedCell = null;
				editorPanel.mousePressed();
			}
		}
		

//		graphPanel.redraw();			
	}
	
	@Override
	public void mouseReleased(MouseEvent e){			
		graphPanel.redraw();
	}	
	
	public boolean drawNode(int x, int y){				
		if(focusedButton == null){
			return false;
		}
		else {
								
			if(focusedButton.getText().equals("ClassNode")){
				builder.drawClassNode(x,y);
				informationPanel.updateInformation("Succeed in drawing a class node");
			}
				
			if(focusedButton.getText().equals("ActivityNode")){
				builder.drawActivityNode(x,y);
				informationPanel.updateInformation("Succeed in drawing an activity node");
			}
					
			focusedButton = null;
			return true;
		}
	}
	
	public boolean drawEdge(MyOriginalCell firstlyFocusedCell, int firstX, int firstY, MyOriginalCell secondlyFocusedCell, int  secondX, int secondY){
		if(focusedButton.getText().equals("ClassEdge"))
		{
			if((firstlyFocusedCell instanceof ClassCell) && (secondlyFocusedCell instanceof ClassCell))
			{
				builder.drawClassEdge(firstlyFocusedCell, firstX, firstY, secondlyFocusedCell, secondX, secondY);
				informationPanel.updateInformation("Succeed in drawing a class edge");
			}					
			else 				
				informationPanel.updateInformation("Wrong selection! Please select two class nodes");			
		}
		
		else if(focusedButton.getText().equals("AOCEdge"))
		{
			if((firstlyFocusedCell instanceof ActivityCell) && (secondlyFocusedCell instanceof ClassCell))				
			{
				builder.drawAOCEdge(firstlyFocusedCell, firstX, firstY, secondlyFocusedCell, secondX, secondY);				
				informationPanel.updateInformation("Succeed in drawing an AOC edge");
			}					
			else 
				informationPanel.updateInformation("Wrong selection! Please select firstly one activity node and secondly one class node");
		}
		
		else if(focusedButton.getText().equals("CrelEdge"))
		{
			if((firstlyFocusedCell instanceof TransparentCell) && (firstlyFocusedCell.getCorrespondingElement() instanceof ActivityEdge))
			{
				if(((secondlyFocusedCell instanceof LabelCell) && (secondlyFocusedCell.getCorrespondingElement() instanceof ClassEdge))
					||((secondlyFocusedCell instanceof ClassCell) && (secondlyFocusedCell.getCorrespondingElement() instanceof ClassNode)))
				{
					builder.drawCrelEdge(firstlyFocusedCell, firstX, firstY, secondlyFocusedCell, secondX, secondY);
					informationPanel.updateInformation("Succeed in drawing a crel edge");
				}
				else 
					informationPanel.updateInformation("Wrong selection! Please select firstly  one activity edge and secondly one class edge or class node");
			}									
			else 
				informationPanel.updateInformation("Wrong selection! Please select firstly  one activity edge");
		}
		
		else if(focusedButton.getText().contains("ActivityEdge"))
		{
			if((firstlyFocusedCell instanceof ActivityCell) && (secondlyFocusedCell instanceof ActivityCell))
			{
				builder.drawActivityEdge(focusedButton.getText(), firstlyFocusedCell, firstX, firstY, secondlyFocusedCell, secondX, secondY);				
				informationPanel.updateInformation("Succeed in drawing an activity edge");
			}		
			else 
				informationPanel.updateInformation("Wrong selection! Please select two activity nodes");
		}
		
		graphPanel.redraw();		
		return true;
	}

	public void mouseClicked(MouseEvent arg0) {
		// TODO Auto-generated method stub
		
	}

	public void mouseEntered(MouseEvent e) {
		// TODO Auto-generated method stub
//		 System.out.println("mouse enter");  
	}

	public void mouseExited(MouseEvent e) {
		// TODO Auto-generated method stub
//		 System.out.println("mouse exist");  
	}

	public void mouseDragged(MouseEvent e) {
		// TODO Auto-generated method stub
//		 System.out.println("mouse drag"); 
	}

	public void mouseMoved(MouseEvent e) {
		// TODO Auto-generated method stub		
		int x = e.getX();
		int y = e.getY();
		
		MyOriginalCell mouseArrowPointedCell = (MyOriginalCell) graph.getFirstCellForLocation(x, y);
		if(mouseArrowPointedCell != null && mouseArrowPointedCell != highlightedCell)
		{
			System.out.println("mouse move on a cell"); 
			//at the same moment, there exists only one cell which is highlighted
			if(highlightedCell != null){
		    	if(highlightedCell instanceof MNodeCell)
		    		GraphConstants.setBorderColor(highlightedCell.getAttributes(),Color.white);		    	
		    	else
		    		GraphConstants.setLineColor(highlightedCell.getAttributes(),Color.black);	    				
			}
			
			highlightedCell =  mouseArrowPointedCell;
	    	if(highlightedCell instanceof MNodeCell)
	    		GraphConstants.setBorderColor(highlightedCell.getAttributes(),Color.gray);	    	
	    	else
	    		GraphConstants.setLineColor(highlightedCell.getAttributes(),Color.gray);
	    			
			//add a random cell into the graph and then remove it. In this way, we can change the color of the cell immediately
			LabelCell transparentCell = new LabelCell("name");  
		    MyOriginalCell[] cells = {transparentCell};			
			graphPanel.getGraph().getGraphLayoutCache().insert(cells);
			graphPanel.getGraph().getGraphLayoutCache().remove(cells);			
			graphPanel.getGraph().refresh();
		}
	    if(mouseArrowPointedCell == null && highlightedCell != null){			
	    	if(highlightedCell instanceof MNodeCell)
	    		GraphConstants.setBorderColor(highlightedCell.getAttributes(),Color.white);	    	
	    	else
	    		GraphConstants.setLineColor(highlightedCell.getAttributes(),Color.black);
	    				
			LabelCell transparentCell = new LabelCell("name");  
		    MyOriginalCell[] cells = {transparentCell};			
			graphPanel.getGraph().getGraphLayoutCache().insert(cells);
			graphPanel.getGraph().getGraphLayoutCache().remove(cells);			
			graphPanel.getGraph().refresh();	
			
			highlightedCell = null;
		}	
	}
	
}
