package org.processmining.ocbc.models.editor.panels;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.Collection;
import java.util.HashSet;

import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSlider;
import javax.swing.JTextField;
import javax.swing.border.BevelBorder;

import org.jgraph.JGraph;
import org.jgraph.graph.DefaultGraphModel;
import org.jgraph.graph.DefaultPort;
import org.jgraph.graph.GraphConstants;
import org.jgraph.graph.GraphLayoutCache;
import org.jgraph.graph.GraphModel;
import org.processmining.mxgraph.util.mxConstants;
import org.processmining.ocbc.models.Cardinality;
import org.processmining.ocbc.models.ClassEdge;
import org.processmining.ocbc.models.OCBCModel;
import org.processmining.ocbc.models.editor.interaction.EditorActionListenerAddNode;
import org.processmining.ocbc.models.editor.interaction.GraphMouseListener;
import org.processmining.ocbc.models.impl.ClassEdgeImpl;
import org.processmining.ocbc.models.io.ImportFromFile;
import org.processmining.ocbc.models.visualization.ModelCellViewFactory;
import org.processmining.ocbc.models.visualization.cell.AOCEdgeCell;
import org.processmining.ocbc.models.visualization.cell.ActivityEdgeCell;
import org.processmining.ocbc.models.visualization.cell.ClassEdgeCell;
import org.processmining.ocbc.models.visualization.cell.MyOriginalCell;
import org.processmining.ocbc.models.visualization.cell.TransparentCell;

import info.clearthought.layout.TableLayout;




public class EditorCreateModelPanel {
	OCBCModel model;
	int maxWidth;
	int maxHeight;
	GraphMouseListener graphMouseListener;
	GraphPanel graphPanel;
	InformationPanel informationPanel;
	JSlider childIndex;
	Collection<JComponent> components;
	JTextField nameField;
	JButton editNameButton;
	JButton deleteButton;
	
	JTextField edgeTextField;
//	JComboBox<Type> typeCombo;
	private int rows = 10;
	private int heightOfRows = 25;
	private int heightOfOperators = 50;
	private Collection<JButton> localComponents;
	
	
	public EditorCreateModelPanel(OCBCModel model, int maxWidth, int maxHeight, GraphMouseListener graphMouseListener, InformationPanel informationPanel,
			GraphPanel graphPanel, Collection<JComponent> components){
		this.model = model;
		this.maxWidth = maxWidth;
		this.maxHeight = maxHeight;
		this.graphMouseListener = graphMouseListener;
		this.graphPanel = graphPanel;
		this.informationPanel = informationPanel;
		this.components = components;
		this.localComponents = new HashSet<JButton>();
		this.childIndex = new JSlider();
	}
	
	public int getHeight(){
		// return the height in terms of elements
		// added some for the border
		return ((rows - 3) * heightOfRows) + (3 * heightOfOperators) + 24;
	}
	
	public JComponent createModel(){
		int columns = 8;
		// scaling for the border
//		int width = (maxWidth - 8) / columns;
		int width = (maxWidth ) / columns;
		width = 135;
		double[][] layout = new double[2][Math.max(columns, rows)];
		for(int i = 0; i < columns; i++){
			layout[0][i] = width;
		}
		for(int i = 0; i < rows; i++){
			layout[1][i] = heightOfOperators;
			layout[1][4] = heightOfOperators;
			layout[1][5] = heightOfOperators;
			layout[1][6] = heightOfOperators;
		}		
		JPanel panel = new JPanel();
		panel.setLayout(new TableLayout(layout));
		panel.setBorder(BorderFactory.createCompoundBorder(BorderFactory.createTitledBorder("Create model"), BorderFactory.createBevelBorder(BevelBorder.LOWERED)));
		
		int offset = 0;

		createAddButton(panel, mxConstants.CLASSCELL, offset, 0, width, heightOfOperators);
		createAddButton(panel, mxConstants.ACTIVITYCELL, offset, 1, width, heightOfOperators);
		createAddButton(panel, mxConstants.NONCOEXISTENCE_NONCOEXISTENCE, offset, 2, width, heightOfOperators);
		offset += 1;
		createAddButton(panel, mxConstants.CLASSRELATION, offset, 0, width, heightOfOperators);
		createAddButton(panel, mxConstants.AOCRELATION, offset, 1, width, heightOfOperators);	
		createAddButton(panel, mxConstants.CRELRELATION, offset, 2, width, heightOfOperators);	
		offset += 1;
		createAddButton(panel, mxConstants.RESPONSE, offset, 0, width, heightOfOperators);		
		createAddButton(panel, mxConstants.UNARYRESPONSE, offset, 1, width, heightOfOperators);	
		createAddButton(panel, mxConstants.NONRESPONSE, offset, 2, width, heightOfOperators);	
		offset += 1;
		createAddButton(panel, mxConstants.PRECEDENCE, offset, 0, width, heightOfOperators);			
		createAddButton(panel, mxConstants.UNARYPRECEDENCE, offset, 1, width, heightOfOperators);
		createAddButton(panel, mxConstants.NONPRECEDENCE, offset, 2, width, heightOfOperators);
		offset += 1;
		createAddButton(panel, mxConstants.COEXISTENCE, offset, 0, width, heightOfOperators);		
		createAddButton(panel, mxConstants.NONCOEXISTENCE, offset, 1, width, heightOfOperators);
		createAddButton(panel, mxConstants.NONRESPONSE_NONPRECEDENCE, offset, 2, width, heightOfOperators);	
		offset += 1;
		createAddButton(panel, mxConstants.RESPONSE_PRECEDENCE, offset, 0, width, heightOfOperators);	
		createAddButton(panel, mxConstants.UNARYRESPONSE_UNARYPRECEDENCE, offset, 1, width, heightOfOperators);	
		createAddButton(panel, mxConstants.COEXISTENCE_COEXISTENCE, offset, 2, width, heightOfOperators);	
		
		return panel;
	}
	

	
	private void createAddButton(JPanel panel, String type, int row, int column, int width, int height){
		JButton addNode = new JButton(renderOperator(type, width, height));
		EditorActionListenerAddNode listener = new EditorActionListenerAddNode(addNode, graphMouseListener, informationPanel, graphPanel, type, nameField, childIndex);
		addNode.addActionListener(listener);
		addNode.setPreferredSize(new Dimension(width, height));
		addNode.setText(type.toString());
		addNode.setBorder(BorderFactory.createRaisedBevelBorder());
		System.out.println("type.toString()"+type.toString());
		panel.add(addNode, "" + column + ", " + row + ", " + column + ", " + row + ", l, c");
		components.add(addNode);
		localComponents.add(addNode);
//		addNode.setBorder(BorderFactory.createLineBorder(Color.RED));
	}
		
	private ImageIcon renderOperator(String t, int width, int height){
        GraphModel model = new DefaultGraphModel();  
        GraphLayoutCache view = new GraphLayoutCache(model, new ModelCellViewFactory());  
        JGraph graph = new JGraph(model, view);  
          
        MyOriginalCell[] cells = generateCells(t, width, height);
       
        graph.getGraphLayoutCache().insert(cells);  
        
		//Required to make the graph do the layout thingy otherwise image is empty
		JFrame frame = new JFrame("");
		JScrollPane scroll = new JScrollPane(graph);
		frame.add(scroll);
		frame.setSize(graph.getSize());
		frame.setPreferredSize(graph.getSize());
		frame.pack();
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		Color bg = null; // Use this to make the background transparent
		int inset = 0;
		graph.setOpaque(false); // Use this to make the background transparent
		BufferedImage img = graph.getImage(bg, inset);
		if(img != null){
			return new ImageIcon(img);
		}
		return null;
	}
	
	
	private MyOriginalCell[] generateCells(String t, int width, int height){
		if(t.toString().equals("ClassNode"))
		{
			MyOriginalCell[] cellsClassNode = new MyOriginalCell[1];
			MyOriginalCell cell = new MyOriginalCell();
			
	        GraphConstants.setBounds(cell.getAttributes(), new Rectangle2D.Double(0,0,1,1));  
	        GraphConstants.setOpaque(cell.getAttributes(), false); 
	        
			cellsClassNode[0] = cell;
			return cellsClassNode;
		}
		else if(t.toString().equals("ActivityNode"))
		{
			MyOriginalCell[] cellsActivityNode = new MyOriginalCell[1];
			MyOriginalCell cell = new MyOriginalCell();
			
	        GraphConstants.setBounds(cell.getAttributes(), new Rectangle2D.Double(0,0,1,1));  
	        GraphConstants.setOpaque(cell.getAttributes(), false); 
	        
			cellsActivityNode[0] = cell;
			return cellsActivityNode;
		}
		
		else if(t.toString().equals("ClassEdge"))
		{
			
			MyOriginalCell[] cellsClassEdge = new MyOriginalCell[5];   			
		    TransparentCell firstTransparentCell = new TransparentCell(null);     
		    DefaultPort firstPort = new DefaultPort();  
		    firstTransparentCell.add(firstPort);
		    cellsClassEdge[0] = firstTransparentCell;
			GraphConstants.setBounds(firstTransparentCell.getAttributes(), new Rectangle2D.Double(20,height/2,width/4,height/4));  
	        GraphConstants.setGradientColor(firstTransparentCell.getAttributes(),Color.black);  
	        GraphConstants.setOpaque(firstTransparentCell.getAttributes(), true);
		    
		    TransparentCell secondTransparentCell = new TransparentCell(null);     
		    DefaultPort secondPort = new DefaultPort();  
		    secondTransparentCell.add(secondPort);
		    cellsClassEdge[1] = secondTransparentCell;
			GraphConstants.setBounds(secondTransparentCell.getAttributes(), new Rectangle2D.Double(width/2,height/2,width/4,height/4));  
	        GraphConstants.setGradientColor(secondTransparentCell.getAttributes(),Color.black);  
	        GraphConstants.setOpaque(secondTransparentCell.getAttributes(), true);
		    
		    TransparentCell thirdTransparentCell = new TransparentCell(null);     
		    DefaultPort thirdPort = new DefaultPort();  
		    thirdTransparentCell.add(thirdPort);
		    cellsClassEdge[2] = thirdTransparentCell;
			GraphConstants.setBounds(thirdTransparentCell.getAttributes(), new Rectangle2D.Double(width,height/2,width/4,height/4));  
	        GraphConstants.setGradientColor(thirdTransparentCell.getAttributes(),Color.black);  
	        GraphConstants.setOpaque(thirdTransparentCell.getAttributes(), true);		   	
		    		    		    		    
		   
	        Cardinality cardinality = new Cardinality(0,Integer.MAX_VALUE);
	        ClassEdge classEdge = new ClassEdgeImpl(null, null, cardinality, cardinality, cardinality, cardinality);

	        
	        ClassEdgeCell firstHalfEdge = new ClassEdgeCell(cardinality.toString());  
//	        firstHalfEdge.displayCardinality();	        
	        firstHalfEdge.setTarget(firstTransparentCell.getChildAt(0));  	        
	        firstHalfEdge.setSource(secondTransparentCell.getChildAt(0));  
	        cellsClassEdge[3] = firstHalfEdge;
	        
	        ClassEdgeCell secondHalfEdge = new ClassEdgeCell(cardinality.toString());  
//	        secondHalfEdge.displayCardinality();	         
	        secondHalfEdge.setTarget(secondTransparentCell.getChildAt(0));
	        secondHalfEdge.setSource(thirdTransparentCell.getChildAt(0)); 
	        cellsClassEdge[4] = secondHalfEdge; 
	        return cellsClassEdge;
			
		}
		
		else if(t.toString().equals("AOCEdge"))
		{
			float[] myDash = { 5f, 5f };
			MyOriginalCell[] cellsClassEdge = new MyOriginalCell[5];
			
		    TransparentCell firstTransparentCell = new TransparentCell(null);     
		    DefaultPort firstPort = new DefaultPort();  
		    firstTransparentCell.add(firstPort);
		    cellsClassEdge[0] = firstTransparentCell;
			GraphConstants.setBounds(firstTransparentCell.getAttributes(), new Rectangle2D.Double(20,height/2,width/4,height/4));  
	        GraphConstants.setGradientColor(firstTransparentCell.getAttributes(),Color.black);  
	        GraphConstants.setOpaque(firstTransparentCell.getAttributes(), true);
		    
		    TransparentCell secondTransparentCell = new TransparentCell(null);     
		    DefaultPort secondPort = new DefaultPort();  
		    secondTransparentCell.add(secondPort);
		    cellsClassEdge[1] = secondTransparentCell;
			GraphConstants.setBounds(secondTransparentCell.getAttributes(), new Rectangle2D.Double(width/2,height/2,width/4,height/4));  
	        GraphConstants.setGradientColor(secondTransparentCell.getAttributes(),Color.black);  
	        GraphConstants.setOpaque(secondTransparentCell.getAttributes(), true);
		    
		    TransparentCell thirdTransparentCell = new TransparentCell(null);     
		    DefaultPort thirdPort = new DefaultPort();  
		    thirdTransparentCell.add(thirdPort);
		    cellsClassEdge[2] = thirdTransparentCell;
			GraphConstants.setBounds(thirdTransparentCell.getAttributes(), new Rectangle2D.Double(width,height/2,width/4,height/4));  
	        GraphConstants.setGradientColor(thirdTransparentCell.getAttributes(),Color.black);  
	        GraphConstants.setOpaque(thirdTransparentCell.getAttributes(), true);
 			    		    		    		            
	        AOCEdgeCell firstHalfEdge = new AOCEdgeCell();  
//	        firstHalfEdge.setCardinalityCharacters("A");
//	        firstHalfEdge.displayCardinality();	        
	        firstHalfEdge.setTarget(firstTransparentCell.getChildAt(0));  	        
	        firstHalfEdge.setSource(secondTransparentCell.getChildAt(0));  
	        GraphConstants.setDashPattern(firstHalfEdge.getAttributes(), myDash);
	        cellsClassEdge[3] = firstHalfEdge;
	        
	        AOCEdgeCell secondHalfEdge = new AOCEdgeCell();  
//	        secondHalfEdge.setCardinalityCharacters("A");
//	        secondHalfEdge.displayCardinality();	         
	        secondHalfEdge.setTarget(secondTransparentCell.getChildAt(0));
	        secondHalfEdge.setSource(thirdTransparentCell.getChildAt(0)); 
	        cellsClassEdge[4] = secondHalfEdge; 
	        GraphConstants.setDashPattern(secondHalfEdge.getAttributes(), myDash);
	        
	        return cellsClassEdge;			
		}
		
		else if(t.toString().equals("CrelEdge"))
		{
			float[] myDash = {10f, 10f };
			MyOriginalCell[] cellsClassEdge = new MyOriginalCell[5];
			
		    TransparentCell firstTransparentCell = new TransparentCell(null);     
		    DefaultPort firstPort = new DefaultPort();  
		    firstTransparentCell.add(firstPort);
		    cellsClassEdge[0] = firstTransparentCell;
			GraphConstants.setBounds(firstTransparentCell.getAttributes(), new Rectangle2D.Double(20,height/2,width/4,height/4));  
	        GraphConstants.setGradientColor(firstTransparentCell.getAttributes(),Color.black);  
	        GraphConstants.setOpaque(firstTransparentCell.getAttributes(), true);
		    
		    TransparentCell secondTransparentCell = new TransparentCell(null);     
		    DefaultPort secondPort = new DefaultPort();  
		    secondTransparentCell.add(secondPort);
		    cellsClassEdge[1] = secondTransparentCell;
			GraphConstants.setBounds(secondTransparentCell.getAttributes(), new Rectangle2D.Double(width/2,height/2,width/4,height/4));  
	        GraphConstants.setGradientColor(secondTransparentCell.getAttributes(),Color.black);  
	        GraphConstants.setOpaque(secondTransparentCell.getAttributes(), true);
		    
		    TransparentCell thirdTransparentCell = new TransparentCell(null);     
		    DefaultPort thirdPort = new DefaultPort();  
		    thirdTransparentCell.add(thirdPort);
		    cellsClassEdge[2] = thirdTransparentCell;
			GraphConstants.setBounds(thirdTransparentCell.getAttributes(), new Rectangle2D.Double(width,height/2,width/4,height/4));  
	        GraphConstants.setGradientColor(thirdTransparentCell.getAttributes(),Color.black);  
	        GraphConstants.setOpaque(thirdTransparentCell.getAttributes(), true);
 			    		    		    		            
	        AOCEdgeCell firstHalfEdge = new AOCEdgeCell();  
//	        firstHalfEdge.setCardinalityCharacters("");
//	        firstHalfEdge.displayCardinality();	        
	        firstHalfEdge.setTarget(firstTransparentCell.getChildAt(0));  	        
	        firstHalfEdge.setSource(secondTransparentCell.getChildAt(0));  
	        GraphConstants.setDashPattern(firstHalfEdge.getAttributes(), myDash);
	        cellsClassEdge[3] = firstHalfEdge;
	        
	        AOCEdgeCell secondHalfEdge = new AOCEdgeCell();  
//	        secondHalfEdge.setCardinalityCharacters("");
//	        secondHalfEdge.displayCardinality();	         
	        secondHalfEdge.setTarget(secondTransparentCell.getChildAt(0));
	        secondHalfEdge.setSource(thirdTransparentCell.getChildAt(0)); 
	        cellsClassEdge[4] = secondHalfEdge; 
	        GraphConstants.setDashPattern(secondHalfEdge.getAttributes(), myDash);
	        
	        return cellsClassEdge;			
		}
		
		else if(t.toString().contains("ActivityEdge"))
		{
			
			MyOriginalCell[] cellsClassEdge = new MyOriginalCell[5];
			
		    TransparentCell firstTransparentCell = new TransparentCell(null);     
		    DefaultPort firstPort = new DefaultPort();  
		    firstTransparentCell.add(firstPort);
		    cellsClassEdge[0] = firstTransparentCell;
			GraphConstants.setBounds(firstTransparentCell.getAttributes(), new Rectangle2D.Double(20,height/2,width/4,height/4));  
	        GraphConstants.setGradientColor(firstTransparentCell.getAttributes(),Color.black);  
	        GraphConstants.setOpaque(firstTransparentCell.getAttributes(), true);
		    
		    TransparentCell secondTransparentCell = new TransparentCell(null);     
		    DefaultPort secondPort = new DefaultPort();  
		    secondTransparentCell.add(secondPort);
		    cellsClassEdge[1] = secondTransparentCell;
			GraphConstants.setBounds(secondTransparentCell.getAttributes(), new Rectangle2D.Double(width/2+20,height/2,width/4,height/4));  
	        GraphConstants.setGradientColor(secondTransparentCell.getAttributes(),Color.black);  
	        GraphConstants.setOpaque(secondTransparentCell.getAttributes(), true);
		    
		    TransparentCell thirdTransparentCell = new TransparentCell(null);     
		    DefaultPort thirdPort = new DefaultPort();  
		    thirdTransparentCell.add(thirdPort);
		    cellsClassEdge[2] = thirdTransparentCell;
			GraphConstants.setBounds(thirdTransparentCell.getAttributes(), new Rectangle2D.Double(width,height/2,width/4,height/4));  
	        GraphConstants.setGradientColor(thirdTransparentCell.getAttributes(),Color.black);  
	        GraphConstants.setOpaque(thirdTransparentCell.getAttributes(), true);
 			    		    		    		            
	        ActivityEdgeCell firstHalfEdge = new ActivityEdgeCell();  
//	        firstHalfEdge.setCardinalityCharacters("*");
//	        firstHalfEdge.displayCardinality();	      
	        firstHalfEdge.setSource(secondTransparentCell.getChildAt(0)); 
	        firstHalfEdge.setTarget(firstTransparentCell.getChildAt(0));  	        
	        cellsClassEdge[3] = firstHalfEdge;
	        
	        ActivityEdgeCell secondHalfEdge = new ActivityEdgeCell();  
//	        secondHalfEdge.setCardinalityCharacters("*");
//	        secondHalfEdge.displayCardinality();	         
	        secondHalfEdge.setSource(secondTransparentCell.getChildAt(0));
	        secondHalfEdge.setTarget(thirdTransparentCell.getChildAt(0)); 	        
	        cellsClassEdge[4] = secondHalfEdge; 
	        
	        int startShape;
	        int endShape;
	        
//	        switch(t)
//	        {
//	        	case ActivityEdgeResponse: startShape = 5; endShape = 15;
//					break;
//				case ActivityEdgeUnaryResponse: startShape = 5; endShape = 2;
//					break;
//				case ActivityEdgeNonResponse: startShape =5; endShape = 15;
//					break;
//				case ActivityEdgePrecedence: startShape =17; endShape = 6;	        	
//					break;
//				case ActivityEdgeUnaryPrecedence: startShape =16; endShape = 6;	        	
//					break;
//				case ActivityEdgeNonPrecedence: startShape =17; endShape = 6;	        	
//					break;
//				case ActivityEdgeCoExistence: startShape =5; endShape = 6;	        	
//					break;
//				case ActivityEdgeNonCoExistence: startShape =5; endShape = 6;	        	
//					break;
//				case ActivityEdgeResponse_ActivityEdgePrecedence: startShape =5; endShape = 17;	        	
//					break;
//				case ActivityEdgeUnaryResponse_ActivityEdgeUnaryPrecedence: startShape =5; endShape = 16;	        	
//					break;
//				case ActivityEdgeCoExistence_ActivityEdgeCoExistence: startShape =5; endShape = 5;	        	
//					break;
//				case ActivityEdgeNonResponse_ActivityEdgeNonPrecedence: startShape =5; endShape = 17;	        	
//					break;
//				case ActivityEdgeNonCoExistence_ActivityEdgeNonCoExistence: startShape =5; endShape = 5;	        	
//					break;
//				default: startShape =6; endShape = 6;
//	        }
	        
	        startShape =6; endShape = 6;
	        GraphConstants.setLineEnd(firstHalfEdge.getAttributes(), startShape);
	        GraphConstants.setLineEnd(secondHalfEdge.getAttributes(), endShape);
	        
	        GraphConstants.setEndSize(firstHalfEdge.getAttributes(), 6);
	        GraphConstants.setEndSize(secondHalfEdge.getAttributes(), 6);
	        GraphConstants.setEndFill(firstHalfEdge.getAttributes(), true);
	        GraphConstants.setEndFill(secondHalfEdge.getAttributes(), true);
	        
//	        if(t.equals(OCBCMElementType.ActivityEdgeNonCoExistence) || t.equals(OCBCMElementType.ActivityEdgeNonPrecedence) || 
//	        		t.equals(OCBCMElementType.ActivityEdgeNonResponse) || t.equals(OCBCMElementType.ActivityEdgeNonResponse_ActivityEdgeNonPrecedence)||
//	        		t.equals(OCBCMElementType.ActivityEdgeNonCoExistence_ActivityEdgeNonCoExistence))
//	        {
//		        System.out.print("enter crossing:" + t.toString());
//		        GraphConstants.setLineBegin(firstHalfEdge.getAttributes(), 4);
//		        GraphConstants.setLineBegin(secondHalfEdge.getAttributes(), 4);
//		        GraphConstants.setBeginSize(firstHalfEdge.getAttributes(), 4);
//		        GraphConstants.setBeginSize(secondHalfEdge.getAttributes(), 4);
//	        }
	        return cellsClassEdge;
			
		}
		return null;
		
		
	}
	
	public static void main(String[] args) throws IOException, ClassNotFoundException  
    {  

      

		JPanel ret = new JPanel();
		
		 
//		   FileInputStream input = new FileInputStream("E:\\new-last.ocbcmml");
		 FileInputStream input = new FileInputStream("E:\\project\\CPNTools\\OCBCModel\\OCBCLogs\\weakModel.ocbcmml");
			ImportFromFile fileParse = new ImportFromFile();       
	        fileParse.parse(input);
	        OCBCModel model = fileParse.getModel();
		   

	        OCBCModel modelClone = (OCBCModel) model.clone();	   
	       
		JFrame frame = new JFrame();
		frame.setSize(500, 500);
		frame.setVisible(true);
		frame.getContentPane().add(new JScrollPane(ret));
    }
}
