package org.processmining.ocbc.models.editor.panels;

import java.io.FileInputStream;
import java.io.IOException;
import java.util.Collection;
import java.util.HashSet;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSlider;
import javax.swing.JTextField;
import javax.swing.border.BevelBorder;

import org.processmining.ocbc.models.OCBCModel;
import org.processmining.ocbc.models.editor.interaction.EditorActionListenerDeleteElement;
import org.processmining.ocbc.models.editor.interaction.EditorActionListenerEditConstraint;
import org.processmining.ocbc.models.editor.interaction.EditorActionListenerEditName;
import org.processmining.ocbc.models.editor.interaction.EditorMouseListenerAOCClassSideCardinality;
import org.processmining.ocbc.models.editor.interaction.EditorMouseListenerAlwaysCardinality;
import org.processmining.ocbc.models.editor.interaction.EditorMouseListenerEventuallyCardinality;
import org.processmining.ocbc.models.editor.interaction.EditorMouseListenerInputName;
import org.processmining.ocbc.models.editor.interaction.GraphMouseListener;
import org.processmining.ocbc.models.io.ImportFromFile;
import org.processmining.ocbc.models.visualization.cell.AOCEdgeCell;
import org.processmining.ocbc.models.visualization.cell.ActivityCell;
import org.processmining.ocbc.models.visualization.cell.ActivityEdgeCell;
import org.processmining.ocbc.models.visualization.cell.ClassCell;
import org.processmining.ocbc.models.visualization.cell.ClassEdgeCell;
import org.processmining.ocbc.models.visualization.cell.CrelEdgeCell;
import org.processmining.ocbc.models.visualization.cell.LabelCell;
import org.processmining.ocbc.models.visualization.cell.MyOriginalCell;
import org.processmining.ocbc.models.visualization.cell.TransparentCell;

import info.clearthought.layout.TableLayout;




public class EditorModifyModelPanel {
	OCBCModel model;
	int maxWidth;
	int maxHeight;
	GraphMouseListener graphMouseListener;
	GraphPanel graphPanel;
	InformationPanel informationPanel;
	JSlider childIndex;
	Collection<JComponent> components;
	
	JTextField nameField;
	JTextField alwaysField;
	JTextField eventuallyField;
	JTextField classSideField;
	
	JButton editNameButton;
	JButton editConstraintButton;
	JButton deleteButton;
	
	JTextField edgeTextField;
//	JComboBox<Type> typeCombo;
	private int rows = 5;
	private int heightOfRows = 25;
	private int heightOfOperators = 50;
	private Collection<JButton> localComponents;
	
	
	public EditorModifyModelPanel(OCBCModel model, int maxWidth, int maxHeight, GraphMouseListener graphMouseListener, 
			InformationPanel informationPanel, GraphPanel graphPanel, Collection<JComponent> components){
		this.model = model;
		this.maxWidth = maxWidth;
		this.maxHeight = maxHeight;
		this.graphMouseListener = graphMouseListener;
		this.informationPanel = informationPanel;
		this.graphPanel = graphPanel;
		this.components = components;
		this.localComponents = new HashSet<JButton>();
		this.childIndex = new JSlider();
	}
	
	public int getHeight(){
		// return the height in terms of elements
		// added some for the border
		return ((rows - 3) * heightOfRows) + (3 * heightOfOperators) + 24;
	}
	
	
	public JComponent modifyModel(){
		int columns = 6;
		int width = (maxWidth) / columns - 2;
		double[][] layout = new double[2][columns];
		for(int i = 0; i < columns; i++){
			layout[0][i] = width;
		}
		for(int i = 0; i < rows; i++){
			layout[1][i] = heightOfRows;			
		}		
		JPanel panel = new JPanel();
		panel.setLayout(new TableLayout(layout));

		panel.setBorder(BorderFactory.createCompoundBorder(BorderFactory.createTitledBorder("Modify model"), BorderFactory.createBevelBorder(BevelBorder.LOWERED)));
		
		//create the "modify name" button and its corresponding field
		editNameButton = new JButton("Modify node/edge name");
		nameField = new JTextField();
		nameField.setEnabled(false);
		editNameButton.setEnabled(false);
		panel.add(editNameButton,"0,0,2,0");
		panel.add(nameField,"3,0,5,0");
		nameField.addMouseListener(new EditorMouseListenerInputName(model, graphMouseListener, nameField, informationPanel, graphPanel, childIndex));
		editNameButton.addActionListener(new EditorActionListenerEditName(model, graphMouseListener, nameField, informationPanel, graphPanel, childIndex));

		//create the "modify constraint" button and its corresponding fields
		editConstraintButton = new JButton("Modify constraint");
		editConstraintButton.setEnabled(false);
		panel.add(editConstraintButton,"0,2,1,3");
		
		JLabel alwaysLabel = new JLabel("□(Always):");
		panel.add(alwaysLabel,"2,2");
		alwaysField = new JTextField();
		panel.add(alwaysField,"3,2");
		alwaysField.setEnabled(false);
		alwaysField.addMouseListener(new EditorMouseListenerAlwaysCardinality(model, graphMouseListener, alwaysField, informationPanel, graphPanel, childIndex));
			
		JLabel eventuallyLabel = new JLabel("◇(Finally):");
		panel.add(eventuallyLabel,"4,2");
		eventuallyField = new JTextField();
		panel.add(eventuallyField,"5,2");
		eventuallyField.setEnabled(false);
		eventuallyField.addMouseListener(new EditorMouseListenerEventuallyCardinality(model, graphMouseListener, eventuallyField, informationPanel, graphPanel, childIndex));
		
		JLabel classSideLabel = new JLabel("class side of AOC relation:");
		panel.add(classSideLabel,"2,3,4,3");
		classSideField = new JTextField();
		panel.add(classSideField,"5,3");
		classSideField.setEnabled(false);
		classSideField.addMouseListener(new EditorMouseListenerAOCClassSideCardinality(model, graphMouseListener, classSideField, informationPanel, graphPanel, childIndex));
		
		editConstraintButton.addActionListener(new EditorActionListenerEditConstraint(model, graphMouseListener, alwaysField, eventuallyField, 
				classSideField, informationPanel, graphPanel, childIndex));
		
		//create the "delete" button
		deleteButton = new JButton("Delete node/edge");	
		deleteButton.setEnabled(false);
		deleteButton.addActionListener(new EditorActionListenerDeleteElement(model, graphMouseListener, informationPanel,graphPanel));
		panel.add(deleteButton, "0,1,5,1");//decide the width (how many columns) column + 3
				
		return panel;
	}
	
	//update the state of buttons and fields in the edit panel,i.e., enable or disable them
	public void updateEditPanel(){
		MyOriginalCell cell = graphMouseListener.getSelectedCell();
        for(MyOriginalCell cell2 : model.getGraphCells()){

        	System.out.println("cell number" );
        	if(cell2 instanceof ClassCell){
        		System.out.println("cell name" + cell2.getUserObject().toString());
        		System.out.println("number of related edges" + model.getRelatedEdgeCellsForOneCell(cell2).size());
        		System.out.println("id name" + cell2.getID().toString());     		
        		System.out.println("address" + cell2.hashCode());
        		
        	}
        	else if(cell2 instanceof ClassEdgeCell){
        		System.out.println("related class node name" + ((ClassEdgeCell)cell2).getSourceNode().getUserObject().toString());
        		System.out.println("related class node id" + ((ClassEdgeCell)cell2).getSourceNode().getID().toString());
        		System.out.println("address" + ((ClassEdgeCell)cell2).getSourceNode().hashCode());
        		
        	}
        }
		//if there is no creating button is selected
		if(graphMouseListener.getSelectedButton() == null){		
			//if one node has related edges, it cannot be removed
			if(cell instanceof ClassCell || cell instanceof ActivityCell) 
			{
				System.out.println("related edge number" + model.getRelatedEdgeCellsForOneCell(cell).size());
				if(model.getRelatedEdgeCellsForOneCell(cell).size() == 0){
					deleteButton.setEnabled(true);
				}
				else{
					deleteButton.setEnabled(false);
				}
				editNameButton.setEnabled(true);
				editConstraintButton.setEnabled(false);
				nameField.setEnabled(true);
				alwaysField.setEnabled(false);
				eventuallyField.setEnabled(false);
				classSideField.setEnabled(false);
			}		
			else if(cell instanceof LabelCell){
				editNameButton.setEnabled(true);
				deleteButton.setEnabled(false);
				editConstraintButton.setEnabled(false);
				nameField.setEnabled(true);
				alwaysField.setEnabled(false);
				eventuallyField.setEnabled(false);
				classSideField.setEnabled(false);
			}
			else if(cell instanceof ClassEdgeCell){
				ClassEdgeCell classEdgeCell = (ClassEdgeCell) cell;
				if(model.getRelatedEdgeCellsForOneCell(classEdgeCell.getTargetNode()).size() == 2){
					deleteButton.setEnabled(true);
				}
				else{
					deleteButton.setEnabled(false);
				}
				editNameButton.setEnabled(false);
				editConstraintButton.setEnabled(true);	
				nameField.setEnabled(false);
				alwaysField.setEnabled(true);
				eventuallyField.setEnabled(true);
				classSideField.setEnabled(false);		
			}
			else if(cell instanceof AOCEdgeCell){		
				AOCEdgeCell aOCEdgeCell = (AOCEdgeCell) cell;
				if(aOCEdgeCell.getWhichEnd().equals("Source")){
					editNameButton.setEnabled(false);
					deleteButton.setEnabled(true);
					editConstraintButton.setEnabled(true);	
					nameField.setEnabled(false);
					alwaysField.setEnabled(true);
					eventuallyField.setEnabled(true);
					classSideField.setEnabled(false);
				}
				else if(aOCEdgeCell.getWhichEnd().equals("Target")){
					editNameButton.setEnabled(false);
					deleteButton.setEnabled(true);
					editConstraintButton.setEnabled(true);	
					nameField.setEnabled(false);
					alwaysField.setEnabled(false);
					eventuallyField.setEnabled(false);
					classSideField.setEnabled(true);
				}
			}
			else if(cell instanceof ActivityEdgeCell){	
				ActivityEdgeCell activityEdgeCell = (ActivityEdgeCell) cell;
				if(model.getRelatedEdgeCellsForOneCell(activityEdgeCell.getSourceNode()).size() == 2){
					deleteButton.setEnabled(true);
				}
				else{
					deleteButton.setEnabled(false);
				}
				editNameButton.setEnabled(false);
				editConstraintButton.setEnabled(false);				
				nameField.setEnabled(false);
				alwaysField.setEnabled(false);
				eventuallyField.setEnabled(false);
				classSideField.setEnabled(false);
			}
			else if(cell instanceof CrelEdgeCell){		
				editNameButton.setEnabled(false);
				editConstraintButton.setEnabled(false);
				deleteButton.setEnabled(true);
				nameField.setEnabled(false);
				alwaysField.setEnabled(false);
				eventuallyField.setEnabled(false);
				classSideField.setEnabled(false);
			}
			else if(cell == null || cell instanceof TransparentCell){
				editNameButton.setEnabled(false);
				deleteButton.setEnabled(false);
				editConstraintButton.setEnabled(false);
				nameField.setEnabled(false);
				alwaysField.setEnabled(false);
				eventuallyField.setEnabled(false);
				classSideField.setEnabled(false);
			}
		}
		else{
			editNameButton.setEnabled(false);
			deleteButton.setEnabled(false);
			editConstraintButton.setEnabled(false);
			nameField.setEnabled(false);
			alwaysField.setEnabled(false);
			eventuallyField.setEnabled(false);
			classSideField.setEnabled(false);
		}
	}
	
	public static void main(String[] args) throws IOException, ClassNotFoundException  
    {  

      

		JPanel ret = new JPanel();
		
		 
//		   FileInputStream input = new FileInputStream("E:\\new-last.ocbcmml");
		 FileInputStream input = new FileInputStream("E:\\project\\CPNTools\\OCBCModel\\OCBCLogs\\weakModel.ocbcmml");
			ImportFromFile fileParse = new ImportFromFile();       
	        fileParse.parse(input);
	        OCBCModel model = fileParse.getModel();
		   

	        OCBCModel modelClone = (OCBCModel) model.clone();	   
	       
		JFrame frame = new JFrame();
		frame.setSize(500, 500);
		frame.setVisible(true);
		frame.getContentPane().add(new JScrollPane(ret));
    }
}
