package org.processmining.ocbc.models.editor.panels;

import info.clearthought.layout.TableLayout;

import java.awt.Dimension;
import java.util.Collection;
import java.util.HashSet;

import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;

import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.ocbc.models.OCBCModel;
import org.processmining.ocbc.models.editor.interaction.GraphMouseListener;


public class EditorPanel {
	UIPluginContext context;
	JComponent editorPanel;
	OCBCModel model;
	GraphPanel graphPanel;
	InformationPanel informationPanel;
	GraphMouseListener graphMouseListener;
	Dimension dimEditorPanel;
	Collection<JComponent> frozenComponents;
	Collection<JComponent> components;
	
	EditorCreateModelPanel editorCreateModelPanel;

	EditorSaveModelPanel editorSaveModelPanel;
	EditorModifyModelPanel editorModifyModelPanel;
	

//	Boolean beginner;
	private int spacing = 15;
	int width = 420;
	
	public EditorPanel(UIPluginContext context, OCBCModel model, InformationPanel informationPanel, 
			GraphPanel graphPanel, GraphMouseListener graphMouseListener, Dimension dimEditorPanel){
		this.context = context;
		this.model = model;
		this.graphPanel = graphPanel;
		this.informationPanel = informationPanel;
		this.graphMouseListener = graphMouseListener;
		this.dimEditorPanel = dimEditorPanel;
		this.components = new HashSet<JComponent>();
		this.width = Math.min(width, (int) dimEditorPanel.getWidth() - 4);
		this.editorPanel = editorPanel();
	}
	
	public JComponent getEditorPanel(){
		return editorPanel;
	}
	
	public int getWidth(){
		return width;
	}
	
	public void mousePressed(){
		System.out.println("EditorPanel");
//		editorCreateModelPanel.updateEditPanel();
		editorModifyModelPanel.updateEditPanel();
	}
	
	public void freezePanel(){
		frozenComponents = new HashSet<JComponent>();
		for(JComponent component: components){
			component.setEnabled(false);
			frozenComponents.add(component);
		}
	}
	
	public void defrostPanel(){
		for(JComponent component: frozenComponents){
			component.setEnabled(true);
		}
	}
	
	private JComponent editorPanel(){
		// we have the following rows
		
		editorCreateModelPanel = new EditorCreateModelPanel(model, width - 8, (int) dimEditorPanel.getHeight(), graphMouseListener, informationPanel, graphPanel, components);
		JComponent cf = editorCreateModelPanel.createModel();

		editorModifyModelPanel = new EditorModifyModelPanel(model, width - 8, (int) dimEditorPanel.getHeight(), graphMouseListener, informationPanel, graphPanel, components);
		JComponent modifyPanel = editorModifyModelPanel.modifyModel();
		
		editorSaveModelPanel = new EditorSaveModelPanel(context, model, width - 8, (int) dimEditorPanel.getHeight(), graphMouseListener, informationPanel, graphPanel, components);
		JComponent cCF = editorSaveModelPanel.createCommon();
		

		
		JTabbedPane tabPane = new JTabbedPane();
		
		int columns = 1;
		int rows = 5;
		double[][] layoutCF = new double[2][Math.max(columns, rows)];
		double[][] layoutD = new double[2][Math.max(columns, rows)];
		double[][] layoutO = new double[2][Math.max(columns, rows)];
		for(int i = 0; i < columns; i++){
			layoutCF[0][i] = width - 4;
			layoutD[0][i] = width - 4;
			layoutO[0][i] = width - 4;
		}
		layoutCF[1][0] = editorCreateModelPanel.getHeight() + 1;
		layoutCF[1][1] = spacing;
		layoutCF[1][2] = editorModifyModelPanel.getHeight() + 1;
		layoutCF[1][3] = spacing;
		layoutCF[1][4] = editorSaveModelPanel.getHeight() + 1;
		

		

		
		JPanel cfPanel = new JPanel();
		cfPanel.setLayout(new TableLayout(layoutCF));
		cfPanel.add(cf, "0, 0, 0, 0, l, c");
		cfPanel.add(modifyPanel, "0, 2, 0, 2, l, c");
		cfPanel.add(cCF, "0, 4, 0, 4, l, c");
		
		
//		tabPane.addTab("Control-Flow", cfPanel);
//		return new JScrollPane(tabPane);
				
		return new JScrollPane(cfPanel);
		
	}
	
}
