package org.processmining.ocbc.models.editor.panels;

import java.awt.Dimension;
import java.util.Collection;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.border.BevelBorder;

import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.ocbc.models.OCBCModel;
import org.processmining.ocbc.models.editor.interaction.EditorActionListenerSave;
import org.processmining.ocbc.models.editor.interaction.GraphMouseListener;

import info.clearthought.layout.TableLayout;


public class EditorSaveModelPanel {
	UIPluginContext context;

	OCBCModel model;
	int maxWidth;
	int maxHeight;
	GraphMouseListener graphMouseListener;
	GraphPanel graphPanel;
	InformationPanel informationPanel;
	Collection<JComponent> components;
	private int rows = 3;
	private int heightOfRows = 25;
	
	
	public EditorSaveModelPanel(UIPluginContext context, OCBCModel model, int maxWidth, int maxHeight, 
			GraphMouseListener graphMouseListener, InformationPanel informationPanel, GraphPanel graphPanel, Collection<JComponent> components){
		this.context = context;
		this.model = model;
		this.maxWidth = maxWidth;
		this.maxHeight = maxHeight;
		this.graphMouseListener = graphMouseListener;
		this.graphPanel = graphPanel;
		this.informationPanel = informationPanel;
		this.components = components;
	}
	
	public int getHeight(){
		// return the height in terms of elements
		return rows * heightOfRows + 24;
	}
	
	public JComponent createCommon(){
		int columns = 4;
		// scaling for the border
		int width = (maxWidth - 8) / columns;
		double[][] layout = new double[2][Math.max(columns, rows)];
		for(int i = 0; i < columns; i++){
			layout[0][i] = width;
		}
		for(int i = 0; i < rows; i++){
			layout[1][i] = heightOfRows;
		}		
		JPanel panel = new JPanel();
		panel.setLayout(new TableLayout(layout));
		panel.setBorder(BorderFactory.createCompoundBorder(BorderFactory.createTitledBorder("Save model"), BorderFactory.createBevelBorder(BevelBorder.LOWERED)));
		JLabel label = new JLabel("Model Name:");
		JTextField field = new JTextField();
		field.setPreferredSize(new Dimension(3 * width, heightOfRows));
		panel.add(label, "" + 0 + ", " + 0 + ", " + 0 + ", " + 0 + ", l, c");
		panel.add(field, "" + 1 + ", " + 0 + ", " + 3 + ", " + 0 + ", l, c");
//		createCheckConsistency(panel, 2, 0, width, heightOfRows);
		createSaveButton(panel, field, 1, 0, width, heightOfRows);
		
		return panel;
	}
		
	
	private void createSaveButton(JPanel panel, JTextField field, int row, int column, int width, int height){
		JButton saveButton = new JButton("Save to workspace");
		saveButton.setPreferredSize(new Dimension(4*width, height));
		saveButton.addActionListener(new EditorActionListenerSave(context, model, field, informationPanel));
		panel.add(saveButton, "" + column + ", " + row + ", " + (column + 3) + ", " + row + ", l, c");
		components.add(saveButton);
	}
	
}
