package org.processmining.ocbc.models.editor.panels;

import java.awt.Dimension;
import java.awt.Rectangle;
import java.awt.event.MouseListener;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JScrollPane;

import org.jgraph.JGraph;
import org.jgraph.graph.DefaultGraphModel;
import org.jgraph.graph.GraphConstants;
import org.jgraph.graph.GraphLayoutCache;
import org.processmining.ocbc.models.ActivityNode;
import org.processmining.ocbc.models.OCBCModel;
import org.processmining.ocbc.models.editor.interaction.GraphMouseListener;
import org.processmining.ocbc.models.visualization.ModelLayoutBuilder;
import org.processmining.ocbc.models.visualization.cell.AOCEdgeCell;
import org.processmining.ocbc.models.visualization.cell.ClassEdgeCell;
import org.processmining.ocbc.models.visualization.cell.MyOriginalCell;



public class GraphPanel {
	
	private OCBCModel model;
	JFrame frame;
	Dimension dimGraphPanel;
	ModelLayoutBuilder builder;
	JScrollPane scrollPane;
	List<MouseListener> mouseListeners;
	EditorPanel editorPanel;
	
	public GraphPanel(OCBCModel model, Dimension dimGraphPanel){
		this.model = model;	
		this.mouseListeners = new ArrayList<MouseListener>();
		this.dimGraphPanel = dimGraphPanel;
		this.frame = new JFrame();
		redraw();

	}
	
	public JComponent getComponent(){
		redraw();
		return (JComponent) frame.getContentPane();
	}
	
	public JGraph getGraph(){
		return model.getGraph();
	}
	
	public void addGraphMouseListener(GraphMouseListener graphMouseListener){
		this.mouseListeners.add(graphMouseListener);
		model.getGraph().addMouseListener(graphMouseListener);
		model.getGraph().addMouseMotionListener(graphMouseListener);
		
//		model.getGraph().addMouseMotionListener(l);
//		scrollPane.addMouseListener(mouseListener);
	}
	
	public void addMouseListener(MouseListener mouseListener){
		this.mouseListeners.add(mouseListener);
		model.getGraph().addMouseListener(mouseListener);
	}
	
	public void setEditorPanel(EditorPanel editorPanel){
		this.editorPanel = editorPanel;
	}
	
	public void setModel(OCBCModel model){
		this.model = model;
	}
		
	public void redraw(){
		
		System.out.println("detect redraw");
		
		if(editorPanel != null){
			editorPanel.freezePanel();
		}

		if(scrollPane != null){
			frame.getContentPane().remove(scrollPane);
		}

		model.getGraph().setEditable(true);
		model.getGraph().setMoveable(true);
		
		//update labels of edges
		for(MyOriginalCell myOriginalCell : model.getGraphCells()){
			if(myOriginalCell instanceof ClassEdgeCell || myOriginalCell instanceof AOCEdgeCell){ 	
//				System.out.println("update label");
		        Point2D[] labelPositions = GraphConstants.getExtraLabelPositions(myOriginalCell.getAttributes());	    				    
			    Object[] labels = GraphConstants.getExtraLabels(myOriginalCell.getAttributes());				    
			    GraphConstants.setExtraLabelPositions(myOriginalCell.getAttributes(), labelPositions);
			    GraphConstants.setExtraLabels(myOriginalCell.getAttributes(), labels);
			    model.getGraph().repaint();
			    model.getGraph().refresh();				    
			}
		}	
		
        //there must exist the following sentence to keep the label of the edge visible all the time
		for(MyOriginalCell myOriginalCell : model.getGraphCells()){
			new JGraph(new DefaultGraphModel(), new GraphLayoutCache()).getGraphLayoutCache().insert(myOriginalCell); 			
		}
		
		model.getGraph().refresh();
		model.getGraph().repaint();
		
		this.scrollPane = new JScrollPane(model.getGraph());
			
		this.scrollPane.repaint();
		
		frame.getContentPane().add(scrollPane);

		frame.getContentPane().validate();
		
		frame.repaint();
		System.out.println("delete repaint");
				
	}
	
	public void adjustGraph(){
		Rectangle bound = frame.getBounds();
		double panelWidth = bound.getWidth();
		double panelHeight = bound.getWidth();
		for(ActivityNode activity : model.getAllActivities()){
			if(activity.getNodeName().equals("create order")){												
				GraphConstants.setBounds(activity.getCorrespondingCell().getAttributes(), new Rectangle2D.Double(panelWidth/2, panelHeight/2, 100, 100));
			}
		}
		redraw();
	}
}
