package org.processmining.ocbc.models.editor.plugins;

import java.awt.Dimension;
import java.awt.GraphicsEnvironment;
import java.awt.Rectangle;
import java.awt.Toolkit;

import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;

import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.contexts.uitopia.annotations.Visualizer;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginCategory;
import org.processmining.mxgraph.examples.swing.GraphEditor;
import org.processmining.mxgraph.examples.swing.GraphEditor.CustomGraph;
import org.processmining.mxgraph.examples.swing.GraphEditor.CustomGraphComponent;
import org.processmining.mxgraph.examples.swing.editor.EditorMenuBar;
import org.processmining.mxgraph.io.mxCodec;
import org.processmining.mxgraph.util.mxConstants;
import org.processmining.mxgraph.util.mxXmlUtils;
import org.processmining.mxgraph.view.mxGraph;
import org.processmining.ocbc.models.OCBCModel;
import org.processmining.ocbc.models.editor.interaction.GraphMouseListener;
import org.processmining.ocbc.models.editor.panels.EditorPanel;
import org.processmining.ocbc.models.editor.panels.GraphPanel;
import org.processmining.ocbc.models.editor.panels.InformationPanel;
import org.processmining.ocbc.models.impl.OCBCModelImpl;
import org.processmining.ocbc.models.visualization.ModelLayoutBuilder;
import org.w3c.dom.Document;


public class OCBCModelEditor {
	@Plugin(
			name = "OCBC Model Editor",
			parameterLabels = { },
			returnLabels = { "OCBC Model" },
			returnTypes = { EditorObject.class },
			userAccessible = true,
			categories = {PluginCategory.Discovery},
			help = "Edit and output an OCBC model, which consists of a class model, "
					+ "a behavioral model and relationships between these two models. "
					+ "Please access http://www.win.tue.nl/ocbc/ for more information."
	)
	@UITopiaVariant(
		affiliation = UITopiaVariant.EHV,
		pack = "OCBC",
		author = "Guangming Li",
		email = "g.li.3@tue.nl"
	)
	public EditorObject generateEmptyModel2_0(PluginContext context){
		System.out.println("enter plugin");
		OCBCModel model = new OCBCModelImpl("version 2.0");
		model.setModelVersion(OCBCEditorConstants.VERSION2_0);
		return new EditorObject(model);

	}
	
	@Plugin(
			name = "OCBC Model Inspector", 
			parameterLabels = {"OCBC Model Editor"},
			returnTypes = OCBCModel.class, 
			returnLabels = "Editor", 
			mostSignificantResult = 1, 
			help = "Provides an overview of an OCBC model."
	)
	@UITopiaVariant(
			affiliation = UITopiaVariant.EHV,
			author = "Guangming Li",
			email = "g.li.3@tue.nl"
	)	
	public OCBCModel inspectOCBCModel(UIPluginContext context, OCBCModel modelOriginal) {
		OCBCModel model = (OCBCModel) modelOriginal.clone();
		return model;
	}
	
	@Plugin(name = "OCBC Editor Object Visualizer", 
			returnLabels = { "Editor" }, 
			returnTypes = { JComponent.class }, 
			parameterLabels = {	"OCBC Editor Object Visualizer"})
	@Visualizer
	public JComponent edit(UIPluginContext context, final EditorObject input){
		System.out.println("\n edit(UIPluginContext context, final EditorObject input)");
		context.getGlobalContext().getResourceManager().getResourceForInstance(input).destroy();
		OCBCModel model = input.model;
		return edit(context, model);
	}
	
	
	@Plugin(name = "OCBC Model Visualizer", 
			returnLabels = { "Editor" }, 
			returnTypes = { JComponent.class }, 
			parameterLabels = {	"OCBC Model Visualizer" })
	@Visualizer
	public JComponent edit(UIPluginContext context, OCBCModel modelOriginal){
		System.out.println("\n edit(UIPluginContext context, OCBCModel model)");	
		OCBCModel model = (OCBCModel) modelOriginal.clone();
		model.setModelEnvironment(mxConstants.model_editor);
		if(model.getModelVersion() == null || model.getModelVersion().equals(OCBCEditorConstants.VERSION2_0)){			
			
//			editor.setOCBCModel(model);
			
//			Document document = model.getModelFile();
			
			mxGraph graphX = new CustomGraph();
			//if true, the imported model is not an empty model
			if(model.getModelString() != null){
				Document document = mxXmlUtils.parseXml(model.getModelString());
				System.out.println("\n input model");
				mxCodec codec = new mxCodec(document);
				codec.decode(document.getDocumentElement(),graphX.getModel());		
			}

			GraphEditor editor = new GraphEditor("mxGraph Editor", new CustomGraphComponent(graphX), model);
			
			System.out.print("\n new mxGraphComponent() kfgz");
			model.setGraphX(graphX);
			editor.setContext(context);
			this.editor = editor;
			
			editor.refreshGraph();
			
			JSplitPane completePanel = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
			
			completePanel.setTopComponent(new EditorMenuBar(editor));
			completePanel.setBottomComponent(editor);
			completePanel.setResizeWeight(0.02);
			completePanel.setEnabled(false);

			
			JFrame frame = new JFrame();
			frame.getContentPane().add(completePanel);
			frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
			frame.setJMenuBar(new EditorMenuBar(editor));
			
			System.out.print("\n refresh graph 2018");
			editor.refreshGraph();
			
			return (JComponent) frame.getContentPane();
		}
		else{
			JPanel ret = new JPanel();
			OCBCModel modelClone =  (OCBCModel) model.clone();
//			OCBCModel modelClone = model;
			createLayout(context, modelClone, ret);
			JFrame frame = new JFrame();
			frame.getContentPane().add(new JScrollPane(ret));
			System.out.print("\n changshacall the visualizer");
			return (JComponent) frame.getContentPane();	
		}
		
		
	}
	
	GraphEditor editor;
	public GraphEditor getGraphEditor(){
		return editor;
	}
	
	public void createLayout(UIPluginContext context, OCBCModel model, JPanel parent){
		Rectangle screenRect = GraphicsEnvironment.getLocalGraphicsEnvironment().getMaximumWindowBounds();
		Dimension screenSize = screenRect.getSize();
		int maxWidth = (int) screenSize.getWidth() - 30;

		// scaling due to ProM overhead
		int maxHeight = (int) screenSize.getHeight() - 130;
		Dimension dimGraphPanel = new Dimension((int)(0.7 * maxWidth), (int)(0.7 * maxHeight));
		Dimension dimPropertyPanel = new Dimension((int)(0.7 * maxWidth), (int)(0.3 * maxHeight));
		Dimension dimEditorPanel = new Dimension((int)(0.3 * maxWidth), maxHeight);

//		model.getGraph().setBendable(true);
//		model.getGraph().setMoveable(false);
		//builder decide how to draw model
		ModelLayoutBuilder builder = new ModelLayoutBuilder(model);

		// create the graph panel
		GraphPanel graphPanel = new GraphPanel(model, dimGraphPanel);
		
		graphPanel.redraw();
		
		//create the information panel
		InformationPanel informationPanel = new InformationPanel(dimPropertyPanel);
		informationPanel.setGraphPanel(graphPanel);
		
		GraphMouseListener graphMouseListener = new GraphMouseListener(model, builder, graphPanel.getGraph());
		graphMouseListener.addGraphPanel(graphPanel);
		graphMouseListener.addInformationPanel(informationPanel);
		graphPanel.addGraphMouseListener(graphMouseListener);

		// add the editor panel
		EditorPanel editorPanel = new EditorPanel(context, model, informationPanel, graphPanel, graphMouseListener, dimEditorPanel);		
		graphMouseListener.addEditorPanel(editorPanel);
		
		//graph pane at the top and properties panel at the bottom
		JSplitPane topBottom = new JSplitPane(JSplitPane.VERTICAL_SPLIT, graphPanel.getComponent(),informationPanel.getComponent());
		topBottom.setDividerLocation((int)(0.7 * maxHeight));
		
	
		//graph pane and properties panel at the left while  editor panel at the right
		JSplitPane leftRight = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, topBottom, editorPanel.getEditorPanel());
		topBottom.validate();
		leftRight.validate();
		parent.add(leftRight);

		int widthEditorPanel = editorPanel.getWidth();
		int widthOfTopBottom = (maxWidth - widthEditorPanel);
		topBottom.setPreferredSize(new Dimension(widthOfTopBottom, maxHeight - 2));
		
	}

	public void discoverLayout(UIPluginContext context, OCBCModel model, JPanel parent){
		Rectangle screenRect = GraphicsEnvironment.getLocalGraphicsEnvironment().getMaximumWindowBounds();
		Dimension screenSize = screenRect.getSize();
		int maxWidth = (int) screenSize.getWidth() - 30;

		// scaling due to ProM overhead
		int maxHeight = (int) screenSize.getHeight() - 130;
		Dimension dimGraphPanel = new Dimension((int)(0.7 * maxWidth), (int)(0.7 * maxHeight));
		Dimension dimPropertyPanel = new Dimension((int)(0.7 * maxWidth), (int)(0.3 * maxHeight));
		Dimension dimEditorPanel = new Dimension((int)(0.3 * maxWidth), maxHeight);

//		model.getGraph().setBendable(true);
//		model.getGraph().setMoveable(false);
		//builder decide how to draw model
		ModelLayoutBuilder builder = new ModelLayoutBuilder(model);

		// create the graph panel
		GraphPanel graphPanel = new GraphPanel(model, dimGraphPanel);
		
		//create the information panel
		InformationPanel informationPanel = new InformationPanel(dimPropertyPanel);
		informationPanel.setGraphPanel(graphPanel);
		
		GraphMouseListener graphMouseListener = new GraphMouseListener(model, builder, graphPanel.getGraph());
		graphMouseListener.addGraphPanel(graphPanel);
		graphMouseListener.addInformationPanel(informationPanel);
		graphPanel.addGraphMouseListener(graphMouseListener);

		// add the editor panel
		EditorPanel editorPanel = new EditorPanel(context, model, informationPanel, graphPanel, graphMouseListener, dimEditorPanel);		
		graphMouseListener.addEditorPanel(editorPanel);
		
		//graph pane at the top and properties panel at the bottom
		JSplitPane topBottom = new JSplitPane(JSplitPane.VERTICAL_SPLIT, graphPanel.getComponent(),informationPanel.getComponent());
		topBottom.setDividerLocation((int)(0.7 * maxHeight));
		
	
		//graph pane and properties panel at the left while  editor panel at the right
		JSplitPane leftRight = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, topBottom, editorPanel.getEditorPanel());
		topBottom.validate();
		leftRight.validate();
		parent.add(leftRight);

		int widthEditorPanel = editorPanel.getWidth();
		int widthOfTopBottom = (maxWidth - widthEditorPanel);
		topBottom.setPreferredSize(new Dimension(widthOfTopBottom, maxHeight - 2));
		
	}
	
	public static void main(String str[]){
		JFrame frame = new JFrame(); 
		System.out.print("\n changshacall the visualizer");
		OCBCModel model = new OCBCModelImpl();
		model.setModelEnvironment(mxConstants.model_editor);
		OCBCModelEditor editor = new OCBCModelEditor();
		frame.add(editor.edit(null, model));
		frame.setSize(Toolkit.getDefaultToolkit().getScreenSize()); 
		frame.setExtendedState(javax.swing.JFrame.MAXIMIZED_BOTH);
		frame.setVisible(true);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		
//		mxGraphComponent graphComponent = editor.getGraphEditor().getGraphComponent();
//		double scale = 1.0;
//		graphComponent.setCenterZoom(true);
//		graphComponent.zoomTo(scale, graphComponent
//				.isCenterZoom());
	}

}
