package org.processmining.ocbc.models.impl;

import java.util.ArrayList;
import java.util.List;

import org.processmining.mxgraph.util.mxConstants;
import org.processmining.ocbc.models.ActivityEdge;
import org.processmining.ocbc.models.ActivityNode;
import org.processmining.ocbc.models.Cardinality;
import org.processmining.ocbc.models.ClassEdge;
import org.processmining.ocbc.models.ClassNode;
import org.processmining.ocbc.models.CrelOCEdge;
import org.processmining.ocbc.models.CrelRTEdge;

public class ActivityEdgeImpl extends EdgeImpl implements ActivityEdge{

	/**
	 * 
	 */
	private static final long serialVersionUID = 7054236069708888129L;
	/**
	 * 
	 */

	private ActivityNode referenceNode;
	private ActivityNode targetNode;
	private String cardinalityType;
	private List<CrelRTEdge> relatedCrelRTRelations;
	private List<CrelOCEdge> relatedCrelOCRelations;
	private Cardinality beforeCardinality;
	private Cardinality afterCardinality;
	private String correlationType; //indicate the correlation type: "ClassCell" or "ClassRelation"
	private ClassNode correlationOCMedia; //indicate the correlation media: a class 
	private ClassEdge correlationRTMedia; //indicate the correlation media: a class relation
	
	public ActivityEdgeImpl(ActivityNode referenceNode, ActivityNode targetNode,  String cardinalityType){
		this(null,  referenceNode, targetNode, cardinalityType, new ArrayList<CrelRTEdge>(), 
				new ArrayList<CrelOCEdge>(), new Cardinality("*"), new Cardinality("*"));
	}
	
	public ActivityEdgeImpl(String name, ActivityNode referenceNode, ActivityNode targetNode,  String cardinalityType, 
			List<CrelRTEdge> relatedCrelRTRelations, List<CrelOCEdge> relatedCrelOCRelations, Cardinality beforeCardinality, 
			Cardinality afterCardinality){
		this.name = name;
		this.referenceNode = referenceNode;
		this.targetNode = targetNode;
		this.cardinalityType = cardinalityType;
		this.relatedCrelRTRelations = relatedCrelRTRelations;
		this.relatedCrelOCRelations = relatedCrelOCRelations;
		this.beforeCardinality = beforeCardinality;
		this.afterCardinality = afterCardinality;
	}

	public void setCorrelationOCMedia(ClassNode correlationOCMedia){
		this.correlationOCMedia = correlationOCMedia;
	}
	
	public ClassNode getCorrelationOCMedia(){
		return correlationOCMedia;
	}
	
	public void setCorrelationRTMedia(ClassEdge correlationRTMedia){
		this.correlationRTMedia = correlationRTMedia;
	}
	
	public ClassEdge getCorrelationRTMedia(){
		return correlationRTMedia;
	}
	
	public void setCorrelationType(String correlationType){
		this.correlationType = correlationType;
	}
	
	public String getCorrelationType(){
		return correlationType;
	}
	
	public Object clone() {
		ActivityEdgeImpl clone;
		try {
			clone = (ActivityEdgeImpl) super.clone();
		} catch (CloneNotSupportedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return null;
		}
		clone.cardinalityType = cardinalityType;
		clone.referenceNode =  (ActivityNode) referenceNode.clone();
		clone.targetNode = (ActivityNode) targetNode.clone();
		clone.beforeCardinality = (Cardinality) beforeCardinality.clone();
		clone.afterCardinality = (Cardinality) afterCardinality.clone();
		
//		List<CrelRTEdge> relatedCrelRTEdgesClone = new ArrayList<CrelRTEdge>();
//		List<CrelOCEdge> relatedCrelOCEdgesClone = new ArrayList<CrelOCEdge>();
//		for(CrelRTEdge crelRTEdge : relatedCrelRTRelations){
//			relatedCrelRTEdgesClone.add((CrelRTEdge) crelRTEdge.clone());			
//		}
//		for(CrelOCEdge crelOCEdge : relatedCrelOCRelations){
//			relatedCrelOCEdgesClone.add((CrelOCEdge) crelOCEdge.clone());			
//		}	
//		clone.setRelatedCrelRTRelations(relatedCrelRTEdgesClone);
//		clone.setRelatedCrelOCRelations(relatedCrelOCEdgesClone);
		return clone;
	}
	
	public ActivityNode getReferenceNode() {
		return referenceNode;
	}

	public ActivityNode getTargetNode() {
		return targetNode;
	}

	public String getCardinalityType() {
		return cardinalityType;
	}

	public Cardinality getBeforeCardinality() {
		return beforeCardinality;
	}

	public Cardinality getAfterCardinality() {
		return afterCardinality;
	}

	public List<CrelRTEdge> getRelatedCrelRTRelations() {
		return relatedCrelRTRelations;
	}

	public List<CrelOCEdge> getRelatedCrelOCRelations() {
		return relatedCrelOCRelations;
	}
	
	public void setCardinalityType(String cardinalityType) {
		this.cardinalityType = cardinalityType;
	}

	public void setBeforeCardinality(Cardinality beforeCardinality) {
		this.beforeCardinality = beforeCardinality;
	}

	public void setAfterCardinality(Cardinality afterCardinality) {
		this.afterCardinality = afterCardinality;
	}

	public void setRelatedCrelRTRelations(List<CrelRTEdge> relatedCrelRTRelations) {
		this.relatedCrelRTRelations = relatedCrelRTRelations;
	}

	public void setRelatedCrelOCRelations(List<CrelOCEdge> relatedCrelOCRelations) {
		this.relatedCrelOCRelations = relatedCrelOCRelations;
	}
	
	/**
	 * return true: satisfied; false: violated
	 * 
	 */
	public boolean checkActivityEdgeCardinality(int beforeEventNum, int afterEventNum) {
		boolean checkFlag = true;
		
//        OCBCMElementType edgeType = null;
//        for(OCBCMElementType type : OCBCMElementType.values()){
//        	if(type.toString().equals(cardinalityType))  edgeType = type;
//   	 }
		      
		switch(cardinalityType)
		{
			case mxConstants.RESPONSE: 
				if(afterEventNum < 1) checkFlag = false;
				System.out.print("\n mxConstants.RESPONSE" + checkFlag);
				break;
			case mxConstants.UNARYRESPONSE: 
				if(afterEventNum != 1) checkFlag = false;
				break;
			case mxConstants.NONRESPONSE: 
				if(afterEventNum != 0) checkFlag = false;
				break;
			case mxConstants.PRECEDENCE: 
				if(beforeEventNum < 1) checkFlag = false;
				break;
			case mxConstants.UNARYPRECEDENCE: 
				if(beforeEventNum != 1) checkFlag = false;
				break;
			case mxConstants.NONPRECEDENCE: 
				if(beforeEventNum != 0) checkFlag = false;
				break;
			case mxConstants.COEXISTENCE: 
				if(beforeEventNum + afterEventNum < 1) checkFlag = false;
				break;
			case mxConstants.NONCOEXISTENCE: 
				if(beforeEventNum + afterEventNum != 0) checkFlag = false;
				break;

			default:checkFlag = false;
			System.out.print("\n default" + checkFlag);
				break;
		}
		return checkFlag;
	}

	public boolean isFatherConstraintOf(String fatherConstraintType, String childConstraintType){
		boolean checkFlag = false;
		
		switch(fatherConstraintType)
		{
			case mxConstants.RESPONSE: 
				if(childConstraintType.equals(mxConstants.UNARYRESPONSE)) checkFlag = true;
				System.out.print("\n mxConstants.RESPONSE" + checkFlag);
				break;
			case mxConstants.NONRESPONSE: 
				if(childConstraintType.equals(mxConstants.NONCOEXISTENCE)) checkFlag = true;
				break;
			case mxConstants.PRECEDENCE: 
				if(childConstraintType.equals(mxConstants.UNARYPRECEDENCE)) checkFlag = true;
				break;
			case mxConstants.NONPRECEDENCE: 
				if(childConstraintType.equals(mxConstants.NONCOEXISTENCE)) checkFlag = true;
				break;
			case mxConstants.COEXISTENCE: 
				if(childConstraintType.equals(mxConstants.UNARYRESPONSE) ||
						childConstraintType.equals(mxConstants.RESPONSE) ||
						childConstraintType.equals(mxConstants.UNARYPRECEDENCE)||
						childConstraintType.equals(mxConstants.PRECEDENCE)) checkFlag = true;
				break;

			default:checkFlag = false;
			System.out.print("\n default" + checkFlag);
				break;
		}
		
		return checkFlag;
	}
	
	public boolean isFatherActivityRelationOf(ActivityEdge childEdge){
		boolean flag = false;
		if(relatedCrelRTRelations.size()>0 && childEdge.getRelatedCrelRTRelations().size()>0){
			String childEdgeReferenceId = childEdge.getRelatedCrelRTRelations().get(0).getRelatedClassRelation().getID();
			String edgeReferenceId = relatedCrelRTRelations.get(0).getRelatedClassRelation().getID();
			if(referenceNode.getID().equals(childEdge.getReferenceNode().getID()) &&
					targetNode.getID().equals(childEdge.getTargetNode().getID()) &&
					edgeReferenceId.equals(childEdgeReferenceId)){
				if(isFatherConstraintOf(cardinalityType, childEdge.getCardinalityType())){
					flag = true;
				}
			}
		}	
		else if(relatedCrelOCRelations.size()>0 && childEdge.getRelatedCrelOCRelations().size()>0){
			String childEdgeReferenceId = childEdge.getRelatedCrelOCRelations().get(0).getRelatedClass().getID();
			String edgeReferenceId = relatedCrelOCRelations.get(0).getRelatedClass().getID();
			if(referenceNode.getID().equals(childEdge.getReferenceNode().getID()) &&
					targetNode.getID().equals(childEdge.getTargetNode().getID()) &&
					edgeReferenceId.equals(childEdgeReferenceId)){
				if(isFatherConstraintOf(cardinalityType, childEdge.getCardinalityType())){
					flag = true;
				}
			}
		}
		return flag;
	}
	
	public boolean isTwinActivityRelationOf(ActivityEdge childEdge){
		boolean flag = false;
		if(referenceNode.getID().equals(childEdge.getReferenceNode().getID()) &&
				targetNode.getID().equals(childEdge.getTargetNode().getID()) &&
				cardinalityType.equals(childEdge.getCardinalityType())){
			if(!this.getID().equals(childEdge.getID())){
				flag = true;
			}
		}
		return flag;
	}
}