package org.processmining.ocbc.models.impl;

import java.util.ArrayList;
import java.util.List;

import org.processmining.ocbc.models.Cardinality;
import org.processmining.ocbc.models.ClassEdge;
import org.processmining.ocbc.models.ClassNode;
import org.processmining.ocbc.models.CrelRTEdge;

public class ClassEdgeImpl extends EdgeImpl implements ClassEdge{

	/**
	 * 
	 */
	private static final long serialVersionUID = 7118382843113182404L;
	private ClassNode relatedSourceNode;
	private ClassNode relatedTargetNode;
	private Cardinality sourceAlwaysCardinality;// the always cardinality on the source side	
	private Cardinality sourceEventuallyCardinality;// the eventually cardinality on the source side
	private Cardinality targetAlwaysCardinality;// the always cardinality on the target side
	private Cardinality targetEventuallyCardinality;// the eventually cardinality on the target side
	private List<CrelRTEdge> relatedCrelRTRelations;	
	
	public ClassEdgeImpl(String name, ClassNode relatedSourceNode, ClassNode relatedTargetNode, Cardinality sourceAlwaysCardinality, 
			Cardinality sourceEventuallyCardinality, Cardinality targetAlwaysCardinality, Cardinality targetEventuallyCardinality, List<CrelRTEdge> relatedCrelRTRelations){
		this.name = name;
		this.relatedSourceNode = relatedSourceNode;
		this.relatedTargetNode = relatedTargetNode;
		this.sourceAlwaysCardinality = sourceAlwaysCardinality;
		this.sourceEventuallyCardinality = sourceEventuallyCardinality;
		this.targetAlwaysCardinality = targetAlwaysCardinality;
		this.targetEventuallyCardinality = targetEventuallyCardinality;
		this.relatedCrelRTRelations = relatedCrelRTRelations;
	}

	public ClassEdgeImpl(ClassNode relatedSourceNode, ClassNode relatedTargetNode,  Cardinality sourceAlwaysCardinality, 
			Cardinality sourceEventuallyCardinality, Cardinality targetAlwaysCardinality, Cardinality targetEventuallyCardinality){
		this(null,  relatedSourceNode, relatedTargetNode, sourceAlwaysCardinality, sourceEventuallyCardinality, targetAlwaysCardinality, 
				targetEventuallyCardinality, new ArrayList<CrelRTEdge>());
	}
	
	public Object clone() {
		ClassEdgeImpl clone;
		try {
			clone = (ClassEdgeImpl) super.clone();
		} catch (CloneNotSupportedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return null;
		}

		clone.relatedSourceNode = (ClassNode) relatedSourceNode.clone();
		clone.relatedTargetNode = (ClassNode) relatedTargetNode.clone();
		clone.sourceAlwaysCardinality = (Cardinality) sourceAlwaysCardinality.clone();
		clone.sourceEventuallyCardinality = (Cardinality) sourceEventuallyCardinality.clone();
		clone.targetAlwaysCardinality = (Cardinality) targetAlwaysCardinality.clone();
		clone.targetEventuallyCardinality = (Cardinality) targetEventuallyCardinality.clone();
		
//		List<CrelRTEdge> relatedCrelRTEdgesClone = new ArrayList<CrelRTEdge>();
//		for(CrelRTEdge crelRTEdge : relatedCrelRTRelations){
//			relatedCrelRTEdgesClone.add((CrelRTEdge) crelRTEdge.clone());			
//		}
//		clone.setRelatedCrelRTRelations(relatedCrelRTEdgesClone);
		return clone;
	}
	
	public ClassNode getSourceNode() {
		return relatedSourceNode;
	}

	public ClassNode getTargetNode() {
		return relatedTargetNode;
	}

	public Cardinality getSourceAlwaysCardinality() {
		return sourceAlwaysCardinality;
	}
	
	public Cardinality getSourceEventuallyCardinality() {
		return sourceEventuallyCardinality;
	}

	public Cardinality getTargetAlwaysCardinality() {
		return targetAlwaysCardinality;
	}
	
	public Cardinality getTargetEventuallyCardinality() {
		return targetEventuallyCardinality;
	}

	public List<CrelRTEdge> getRelatedCrelRTRelations() {
		return relatedCrelRTRelations;
	}

	public void setSourceAlwaysCardinality(Cardinality sourceAlwaysCardinality) {
		this.sourceAlwaysCardinality = sourceAlwaysCardinality;		
	}

	public void setSourceEventuallyCardinality(Cardinality sourceEventuallyCardinality) {
		this.sourceEventuallyCardinality = sourceEventuallyCardinality;		
	}
	
	public void setTargetAlwaysCardinality(Cardinality targetAlwaysCardinality) {
		this.targetAlwaysCardinality = targetAlwaysCardinality;			
	}
	
	public void setTargetEventuallyCardinality(Cardinality targetEventuallyCardinality) {
		this.targetEventuallyCardinality = targetEventuallyCardinality;			
	}

	public void setRelatedCrelRTRelations(List<CrelRTEdge> relatedCrelRTRelations) {
		this.relatedCrelRTRelations = relatedCrelRTRelations;		
	}	
}