package org.processmining.ocbc.models.impl;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.jgraph.JGraph;
import org.jgraph.graph.DefaultGraphModel;
import org.jgraph.graph.DefaultPort;
import org.jgraph.graph.GraphLayoutCache;
import org.jgraph.graph.GraphModel;
import org.processmining.framework.util.Pair;
import org.processmining.mxgraph.util.mxConstants;
import org.processmining.mxgraph.view.mxGraph;
import org.processmining.ocbc.discovery.impl.OCBCModelDiscovery;
import org.processmining.ocbc.discovery.result.ModelDiscoveryResult;
import org.processmining.ocbc.log.model.XOCLog;
import org.processmining.ocbc.models.AOCEdge;
import org.processmining.ocbc.models.ActivityEdge;
import org.processmining.ocbc.models.ActivityNode;
import org.processmining.ocbc.models.ClassEdge;
import org.processmining.ocbc.models.ClassNode;
import org.processmining.ocbc.models.CrelOCEdge;
import org.processmining.ocbc.models.CrelRTEdge;
import org.processmining.ocbc.models.OCBCModel;
import org.processmining.ocbc.models.io.ImportFromFile;
import org.processmining.ocbc.models.visualization.ModelCellViewFactory;
import org.processmining.ocbc.models.visualization.cell.LabelCell;
import org.processmining.ocbc.models.visualization.cell.MEdgeCell;
import org.processmining.ocbc.models.visualization.cell.MNodeCell;
import org.processmining.ocbc.models.visualization.cell.MyOriginalCell;
import org.processmining.ocbc.models.visualization.cell.TransparentCell;
import org.w3c.dom.Document;


public class OCBCModelImpl implements OCBCModel, Serializable{


//	/**
//	 * Possible types of node. For each final subclass of node there is a
//	 * corresponding type defined here.
//	 * 
//	 * @author bfvdonge
//	 * 
//	 */
//	public static enum OCBCMElementType {
//		ClassNode,
//		ActivityNode,
//		ClassEdge,
//		AOCEdge,
//		CrelEdge,
//		ActivityEdgeResponse,
//		ActivityEdgeNonResponse,
//		ActivityEdgeUnaryResponse,
//		ActivityEdgePrecedence,
//		ActivityEdgeUnaryPrecedence,
//		ActivityEdgeNonPrecedence,
//		ActivityEdgeCoExistence,
//		ActivityEdgeNonCoExistence,
//		ActivityEdgeResponse_ActivityEdgePrecedence,
//		ActivityEdgeNonResponse_ActivityEdgeNonPrecedence,
//		ActivityEdgeUnaryResponse_ActivityEdgeUnaryPrecedence,
//		ActivityEdgeCoExistence_ActivityEdgeCoExistence,
//		ActivityEdgeNonCoExistence_ActivityEdgeNonCoExistence
////		SEQ("Sequence", Block.Seq.class), 
//	}
	
	/**
	 * 
	 */
	private static final long serialVersionUID = -6670328084777702641L;
	/**
	 * 
	 */
	
	private String name;
	private String version;// such as 2.0
	private String source;// indicate the source of the model, such as "discovered" or "designed" or "imported"
	private String environment;// indicate the environment where the model is in, such as "conformance_checking_plugin" or "model_editor"
	private XOCLog sourceLog;//if the model is discovered, record the log where the model is from
	private JGraph graph;
	private mxGraph graphX;
	private Document modelFile;//record the model file
	private String modelString;//record the model string
	private List<MyOriginalCell> graphCells;
	private List<ClassNode> allClasses;
	private List<ActivityNode> allActivities;
	private List<ClassEdge> allClassRelations;
	private List<ActivityEdge> allActivityRelations;
	private List<AOCEdge> allAOCRelations;
	private List<CrelOCEdge> allCrelOCRelations;
	private List<CrelRTEdge> allCrelRTRelations;
	private List<Pair<String, String>> sequentialActivityPairList = new ArrayList<Pair<String, String>>();
	private ModelDiscoveryResult modelDiscoveryResult = new ModelDiscoveryResult();//record the discovery information
	private OCBCModelDiscovery modelDiscovery;
	private List<String> transitiveRelationIdList = new ArrayList<String>();
	private Map<String, ClassEdge> typeVSClassRelationMap = new HashMap<String, ClassEdge>();
	
	public OCBCModelImpl(String name, JGraph graph, List<MyOriginalCell> graphCells, List<ClassNode> allClasses, List<ActivityNode> allActivities, List<ClassEdge> allClassRelations, List<ActivityEdge> allActivityRelations, List<AOCEdge> allAOCRelations, List<CrelOCEdge> allCrelOCRelations, List<CrelRTEdge> allCrelRTRelations){
		this.name = name;
		this.graph = graph;
		this.graphCells = graphCells;
		this.allClasses = allClasses;
		this.allActivities= allActivities;
		this.allClassRelations = allClassRelations;
		this.allActivityRelations = allActivityRelations;
		this.allAOCRelations = allAOCRelations;
		this.allCrelOCRelations = allCrelOCRelations;
		this.allCrelRTRelations = allCrelRTRelations;		
	}

	public OCBCModelImpl(String name, JGraph graph){		
		this(name, graph, new ArrayList<MyOriginalCell>(), new ArrayList<ClassNode>(), new ArrayList<ActivityNode>(), new ArrayList<ClassEdge>(), new ArrayList<ActivityEdge>(), new ArrayList<AOCEdge>(), new ArrayList<CrelOCEdge>(), new ArrayList<CrelRTEdge>());
	}
	
	public OCBCModelImpl(String name){	
		this(name, initializeGraph(), new ArrayList<MyOriginalCell>(), new ArrayList<ClassNode>(), new ArrayList<ActivityNode>(), new ArrayList<ClassEdge>(), new ArrayList<ActivityEdge>(), new ArrayList<AOCEdge>(), new ArrayList<CrelOCEdge>(), new ArrayList<CrelRTEdge>());
	}
	
//	public OCBCModelImpl(String name, GraphModel graphModel){			
//		this(name, new JGraph(), new ArrayList<ClassNode>(), new ArrayList<ActivityNode>(), new ArrayList<ClassEdge>(), new ArrayList<ActivityEdge>(), new ArrayList<AOCEdge>(), new ArrayList<CrelOCEdge>(), new ArrayList<CrelRTEdge>());
//		this.graphModel = graphModel;
//	}
	
	public OCBCModelImpl(){	
		this("new OCBC model", initializeGraph(), new ArrayList<MyOriginalCell>(), new ArrayList<ClassNode>(), new ArrayList<ActivityNode>(), new ArrayList<ClassEdge>(), new ArrayList<ActivityEdge>(), new ArrayList<AOCEdge>(), new ArrayList<CrelOCEdge>(), new ArrayList<CrelRTEdge>());
	}
	
//	public GraphModel  getGraphModel() {
//		return graphModel;
//	}
	
	public static JGraph initializeGraph()
	{
		GraphModel graphModel = new DefaultGraphModel();  
	    GraphLayoutCache view = new GraphLayoutCache(graphModel, new ModelCellViewFactory());    
	    JGraph graph = new JGraph(graphModel, view); // must be initialized
		return graph;
	}
	
	public static mxGraph initializeGraphX()
	{
		mxGraph graphX = new mxGraph();
		return graphX;
	}
	
	public String getModelEnvironment() {
		return environment;
	}

	public void setModelEnvironment(String environment) {
		this.environment = environment;		
	}
	
	public String getModelSource() {
		return source;
	}

	public void setModelSource(String source) {
		this.source = source;		
	}
	
	public XOCLog getModelSourceLog() {
		return sourceLog;
	}

	public void setModelSourceLog(XOCLog sourceLog) {
		this.sourceLog = sourceLog;		
	}
	
	public ModelDiscoveryResult getModelDiscoveryResult() {
		return modelDiscoveryResult;
	}

	public void setModelDiscoveryResult(ModelDiscoveryResult modelDiscoveryResult) {
		this.modelDiscoveryResult = modelDiscoveryResult;		
	}
	
	public OCBCModelDiscovery getModelDiscovery() {
		return modelDiscovery;
	}
	
	public void setModelDiscovery(OCBCModelDiscovery modelDiscovery){
		this.modelDiscovery = modelDiscovery;	
	}
	
	public List<String> getTransitiveRelationIdList(){
		return transitiveRelationIdList;
	}
	
	public void setTransitiveRelationIdList(List<String> transitiveRelationIdList){
		this.transitiveRelationIdList = transitiveRelationIdList;
	}
	
	public String getModelVersion() {
		return version;
	}

	public void setModelVersion(String version) {
		this.version = version;		
	}
	
	public Document getModelFile() {
		return modelFile;
	}

	public void setModelFile(Document modelFile) {
		this.modelFile = modelFile;		
	}
	
	public String getModelString() {
		return modelString;
	}

	public void setModelString(String modelString) {
		this.modelString = modelString;		
	}
	
	public String getModelName() {
		return name;
	}

	public void setModelName(String name) {
		this.name = name;		
	}

	/**
	 * returns the graph of a model
	 * 
	 * @return
	 */
	public JGraph getGraph(){
		return graph;
	}
	
	/**
	 * sets the graph of a model
	 * @param name
	 * 
	 */
	public void setGraph(JGraph graph){
		this.graph = graph;
	}
	
	/**
	 * returns the graphX of a model
	 * 
	 * @return
	 */
	public mxGraph getGraphX(){
		return graphX;
	}
	
	/**
	 * sets the graphX of a model
	 * @param name
	 * 
	 */
	public void setGraphX(mxGraph graphX){
		this.graphX = graphX;
	}
	
	/**
	 * returns the cells of a graph of a model
	 * 
	 * @return
	 */
	public List<MyOriginalCell> getGraphCells(){
		return graphCells;
	}
	
	/**
	 * sets the cells of a graph of a model
	 * @param name
	 * 
	 */
	public void setGraphCells(List<MyOriginalCell> graphCells){
		this.graphCells = graphCells;
	}
	
	public List<ClassNode> getAllClasses() {
		return allClasses;
	}

	/**
	 * returns the set of all class names in the model
	 * 
	 */
	public List<String> getAllClassNames(){
		List<String> nameList = new ArrayList<String>();
		for(ClassNode classNode : allClasses){
			String className = classNode.getNodeName();
			nameList.add(className);
		}
		return nameList;
	}
	
	public ClassNode getClassByName(String uniqueClassName){
		ClassNode classNode = null;
		for(ClassNode existentNode : allClasses){
			if(existentNode.getNodeName().equals(uniqueClassName)){
				classNode = existentNode;
				break;
			}
		}
		return classNode;
	}
	
	public ClassNode getClassById(String id){
		ClassNode classNode = null;
		for(ClassNode existentNode : allClasses){
			if(existentNode.getID().equals(id)){
				classNode = existentNode;
				break;
			}
		}
		return classNode;
	}
	
	public AOCEdge getAOCRelationByActivityIdAndClassId(String activityId, String classId){
		AOCEdge aOCRelation = null;
		for(AOCEdge existAOCRelation : allAOCRelations){
			String activityNodeId = existAOCRelation.getActivityNode().getID();
			String classNodeId= existAOCRelation.getClassNode().getID();
			if(activityId.equals(activityNodeId) && classId.equals(classNodeId)){
				aOCRelation = existAOCRelation;
				break;
			}
		}
		return aOCRelation;
	}
	
	public List<ActivityNode> getAllActivities() {
		return allActivities;
	}

	public ActivityNode getActivityById(String id){
		ActivityNode node = null;
		for(ActivityNode existentNode : allActivities){
			if(existentNode.getID().equals(id)){
				node = existentNode;
				break;
			}
		}
		return node;
	}
	
	/**
	 * returns the set of all activity names in the model
	 * 
	 */
	public List<String> getAllActivityNames(){
		List<String> nameList = new ArrayList<String>();
		for(ActivityNode activityNode : allActivities){
			String activityName = activityNode.getNodeName();
			nameList.add(activityName);
		}
		return nameList;
	}
	
	public List<ClassEdge> getAllClassRelations() {
		return allClassRelations;
	}

	/**
	 * returns one class relation of the id in the model
	 * 
	 */
	public ClassEdge getClassRelationById(String id){
		ClassEdge edge = null;
		for(ClassEdge existentEdge : allClassRelations){
			if(existentEdge.getID().equals(id)){
				edge = existentEdge;
				break;
			}
		}
		return edge;
	}
	
	public List<ActivityEdge> getAllActivityRelations() {
		return allActivityRelations;
	}

	public ActivityEdge getActivityRelationById(String id){
		ActivityEdge activityEdge = null;
		for(ActivityEdge existentActivityEdge : allActivityRelations){
			if(existentActivityEdge.getID().equals(id)){
				activityEdge = existentActivityEdge;
				break;
			}
		}
		return activityEdge;
	}
	
	/**
	 * create a map to record all related activity for each class in the OCBC model
	 * @return
	 */
	public Map<String, List<String>> generateClassVSRelatedActivityListMap(){
		Map<String, List<String>> classVSRelatedActivityList = new HashMap<String, List<String>>();
		for(AOCEdge aOCRelation : allAOCRelations){
			String className = aOCRelation.getClassNode().getNodeName();
			String activityName = aOCRelation.getActivityNode().getNodeName();
			if(!classVSRelatedActivityList.containsKey(className)){
				classVSRelatedActivityList.put(className, new ArrayList<String>());
			}
			if(!classVSRelatedActivityList.get(className).contains(activityName)){
				classVSRelatedActivityList.get(className).add(activityName);
			}		
		}
		return classVSRelatedActivityList;
	}
	
	/**
	 * create a map to record all related class for each activity in the OCBC model
	 * @return
	 */
	public Map<String, List<String>> generateActivityVSRelatedClassListMap(){
		Map<String, List<String>> activityVSRelatedClassList = new HashMap<String, List<String>>();
		for(AOCEdge aOCRelation : allAOCRelations){
			String className = aOCRelation.getClassNode().getNodeName();
			String activityName = aOCRelation.getActivityNode().getNodeName();
			if(!activityVSRelatedClassList.containsKey(activityName)){
				activityVSRelatedClassList.put(activityName, new ArrayList<String>());
			}
			if(!activityVSRelatedClassList.get(activityName).contains(className)){
				activityVSRelatedClassList.get(activityName).add(className);
			}		
		}
		return activityVSRelatedClassList;
	}
	public List<AOCEdge> getAllAOCRelations() {
		return allAOCRelations;
	}

	public List<CrelOCEdge> getAllCrelOCRelations() {
		return allCrelOCRelations;
	}

	public List<CrelRTEdge> getAllCrelRTRelations() {
		return allCrelRTRelations;
	}

	public List<Pair<String, String>> getSequentialActivityPairList() {
		return sequentialActivityPairList;
	}
	
	public void setAllClasses(List<ClassNode> allClasses) {
		this.allClasses = allClasses;
	}

	public void setAllActivities(List<ActivityNode> allActivities) {
		this.allActivities= allActivities;
	}

	public void setAllClassRelations(List<ClassEdge> allClassRelations) {
		this.allClassRelations = allClassRelations;
	}

	public void setAllActivityRelations(List<ActivityEdge> allActivityRelations) {
		this.allActivityRelations = allActivityRelations;
	}

	public void setAllAOCRelations(List<AOCEdge> allAOCRelations) {
		this.allAOCRelations = allAOCRelations;
	}

	public void setAllCrelOCRelations(List<CrelOCEdge> allCrelOCRelations) {
		this.allCrelOCRelations = allCrelOCRelations;
	}

	public void setAllCrelRTRelations(List<CrelRTEdge> allCrelRTRelations) {
		this.allCrelRTRelations = allCrelRTRelations;
	}
	
	/**
	 * add one class into the set of all classes in the model
	 * 
	 */
	public void addClass(ClassNode classNode){
		allClasses.add(classNode);
	}
	
	/**
	 * add one activity into the set of all activities in the model
	 * 
	 */
	public void addActivity(ActivityNode activityNode){
		allActivities.add(activityNode);
	}
	
	/**
	 * delete one class from the set of all classes in the model
	 * 
	 */
	public void deleteClass(ClassNode classNode){
		allClasses.remove(classNode);		
	}
	
	/**
	 * delete one activity from the set of all activities in the model
	 * 
	 */
	public void deleteActivity(ActivityNode activityNode){
		allActivities.remove(activityNode);
	}
	
	/**
	 * get related edge cells for one class cell
	 * 
	 */
	public List<MEdgeCell> getRelatedEdgeCellsForOneCell(MyOriginalCell cell){
		List<MEdgeCell> relatedEdgeCells = new ArrayList<MEdgeCell>();
		for(MyOriginalCell graphCell : graphCells){		
			if(graphCell instanceof MEdgeCell){
				MEdgeCell edgeCell = (MEdgeCell) graphCell;
//				if(edgeCell.getSourceNode() == cell || edgeCell.getTargetNode() == cell){
				if(edgeCell.getSourceNode().getID() == cell.getID() || edgeCell.getTargetNode().getID() == cell.getID()){
					relatedEdgeCells.add(edgeCell);
				}
			}
		}
		return relatedEdgeCells;	
	}

	
	/**
	 * Clones this event, i.e. creates a deep copy, but with a new ID, so equals
	 * does not hold between this and the clone
	 */

	@Override
	public Object clone() {
		OCBCModelImpl clone = null;
		
		try {
			clone = (OCBCModelImpl) super.clone();
		} catch (CloneNotSupportedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
			
		List<ClassNode> relatedClassNodesClone = new ArrayList<ClassNode>();
		List<ActivityNode> relatedActivityNodesClone = new ArrayList<ActivityNode>();
		List<ClassEdge> relatedClassEdgesClone = new ArrayList<ClassEdge>();
		List<ActivityEdge> relatedActivityEdgeClone = new ArrayList<ActivityEdge>();
		List<AOCEdge> relatedAOCEdgesClone = new ArrayList<AOCEdge>();
		List<CrelOCEdge> relatedCrelOCEdgesClone = new ArrayList<CrelOCEdge>();
		List<CrelRTEdge> relatedCrelRTEdgesClone = new ArrayList<CrelRTEdge>();	
		List<MyOriginalCell> graphCellsClone = new ArrayList<MyOriginalCell>();
		
		//clone the graphX
		
		mxGraph graphXClone = new mxGraph();
		if(graphX != null && graphX.getChildCells(graphX.getDefaultParent()) != null){
			graphXClone.addCells(graphX.cloneCells(graphX.getChildCells(graphX.getDefaultParent())));
		}		
		this.graphX = graphXClone;
		
		GraphModel graphModel = new DefaultGraphModel();  
	    GraphLayoutCache view = new GraphLayoutCache(graphModel, new ModelCellViewFactory());    
	    JGraph graphClone = new JGraph(graphModel, view); // must be initialized
		
	    //clone all the nodes
		for(MyOriginalCell cell : graphCells){					
			if(cell instanceof MNodeCell){
				MNodeCell cellClone = (MNodeCell) cell.clone();
 				DefaultPort port = new DefaultPort();  
 				cellClone.add(port);
 				graphClone.getGraphLayoutCache().insert(cellClone);
 				graphCellsClone.add(cellClone);				
 			}
		}
		
	    //clone all the edges
		for(MyOriginalCell cell : graphCells){
 			if(cell instanceof MEdgeCell){

 				MEdgeCell cellClone = (MEdgeCell) cell.clone();
 				String sourceNodeID = ((MEdgeCell) cell).getSourceNode().getID().toString();
 				for(MyOriginalCell myOriginalCell : graphCellsClone){
 					if(myOriginalCell instanceof MNodeCell && myOriginalCell.getID().toString().equals(sourceNodeID)){
						cellClone.setSource(myOriginalCell.getChildAt(0));
 					}					
 				}				
				String targetNodeID = ((MEdgeCell) cell).getTargetNode().getID().toString();
 				for(MyOriginalCell myOriginalCell : graphCellsClone){
 					if(myOriginalCell instanceof MNodeCell && myOriginalCell.getID().toString().equals(targetNodeID)){
						cellClone.setTarget(myOriginalCell.getChildAt(0));
 					}						
 				}
 				graphClone.getGraphLayoutCache().insert(cellClone);
 				graphCellsClone.add(cellClone);
 			}
		}
		
		
		for(ClassNode classNode : allClasses){
			ClassNode classNodeClone = (ClassNode) classNode.clone();			
			String classNodeId = classNode.getID().toString();			
			for(MyOriginalCell cell : graphCellsClone){
				if(cell.getID().toString().equals(classNodeId)){
					classNodeClone.setCorrespondingCell(cell);
					cell.setCorrespondingElement(classNodeClone);
					break;
				}
			}
			relatedClassNodesClone.add(classNodeClone);
		}
		for(ActivityNode activityNode : allActivities){
			ActivityNode activityNodeClone = (ActivityNode) activityNode.clone();			
			String activityNodeId = activityNode.getID().toString();			
			for(MyOriginalCell cell : graphCellsClone){
				if(cell.getID().toString().equals(activityNodeId)){
					activityNodeClone.setCorrespondingCell(cell);
					cell.setCorrespondingElement(activityNodeClone);
					break;
				}
			}
			relatedActivityNodesClone.add(activityNodeClone);			
		}
		for(ClassEdge classEdge : allClassRelations){
			ClassEdge classEdgeClone = (ClassEdge) classEdge.clone();			
			String classEdgeId = classEdge.getID().toString();			
			for(MyOriginalCell cell : graphCellsClone){
				if(cell.getID().toString().equals(classEdgeId)){					
					cell.setCorrespondingElement(classEdgeClone);
					if(cell instanceof LabelCell)
						classEdgeClone.setCorrespondingCell(cell);					
				}
			}
			relatedClassEdgesClone.add(classEdgeClone);			
		}
		for(ActivityEdge activityEdge : allActivityRelations){
			ActivityEdge activityEdgeClone = (ActivityEdge) activityEdge.clone();			
			String activityEdgeId = activityEdge.getID().toString();			
			for(MyOriginalCell cell : graphCellsClone){
				if(cell.getID().toString().equals(activityEdgeId)){					
					cell.setCorrespondingElement(activityEdgeClone);
					if(cell instanceof TransparentCell)
						activityEdgeClone.setCorrespondingCell(cell);					
				}
			}
			relatedActivityEdgeClone.add(activityEdgeClone);			
		}		
		for(AOCEdge aOCEdge : allAOCRelations){
			AOCEdge aOCEdgeClone = (AOCEdge) aOCEdge.clone();			
			String aOCEdgeId = aOCEdge.getID().toString();			
			for(MyOriginalCell cell : graphCellsClone){
				if(cell.getID().toString().equals(aOCEdgeId)){					
					cell.setCorrespondingElement(aOCEdgeClone);
					if(cell instanceof TransparentCell)
						aOCEdgeClone.setCorrespondingCell(cell);					
				}
			}
			relatedAOCEdgesClone.add(aOCEdgeClone);			
		}
		for(CrelOCEdge crelOCEdge : allCrelOCRelations){
			CrelOCEdge crelOCEdgeClone = (CrelOCEdge) crelOCEdge.clone();			
			String crelOCEdgeId = crelOCEdge.getID().toString();			
			for(MyOriginalCell cell : graphCellsClone){
				if(cell.getID().toString().equals(crelOCEdgeId)){					
					cell.setCorrespondingElement(crelOCEdgeClone);
					if(cell instanceof TransparentCell)
						crelOCEdgeClone.setCorrespondingCell(cell);					
				}
			}
			relatedCrelOCEdgesClone.add(crelOCEdgeClone);			
		}
		for(CrelRTEdge crelRTEdge : allCrelRTRelations){
			CrelRTEdge crelRTEdgeClone = (CrelRTEdge) crelRTEdge.clone();			
			String crelRTEdgeId = crelRTEdge.getID().toString();			
			for(MyOriginalCell cell : graphCellsClone){
				if(cell.getID().toString().equals(crelRTEdgeId)){					
					cell.setCorrespondingElement(crelRTEdgeClone);
					if(cell instanceof TransparentCell)
						crelRTEdgeClone.setCorrespondingCell(cell);					
				}
			}
			relatedCrelRTEdgesClone.add(crelRTEdgeClone);			
		}
	
		
		clone.name = name;					   	    
		clone.graph = graphClone;
		
		clone.setAllClasses(relatedClassNodesClone);
		clone.setAllActivities(relatedActivityNodesClone);
		clone.setAllClassRelations(relatedClassEdgesClone);
		clone.setAllActivityRelations(relatedActivityEdgeClone);
		clone.setAllAOCRelations(relatedAOCEdgesClone);
		clone.setAllCrelOCRelations(relatedCrelOCEdgesClone);
		clone.setAllCrelRTRelations(relatedCrelRTEdgesClone);
		clone.setGraphCells(graphCellsClone);
		
		return clone;
	}
	
	//select an activity type based on a list of (before, after)
	public String transformCardinalityPairListToActicityType(List<Pair<Integer, Integer>> acticvityCardinalityPairList){
		String activityType = null;
		
		int acticvityCardinalityBefore = 0;
		int acticvityCardinalityAfter = 0;
		int moreThanOneNumBefore = 0;
		int equalToOneNumBefore = 0;
		int equalToZeroNumBefore = 0;
		int moreThanOneNumAfter = 0;
		int equalToOneNumAfter = 0;
		int equalToZeroNumAfter = 0;
		int NoLessThanOneNumBeforePlusAfter = 0;
		int equalToZeroNumBeforePlusAfter = 0;
		
		for(Pair<Integer, Integer> acticvityCardinalityPair : acticvityCardinalityPairList){
			acticvityCardinalityBefore = acticvityCardinalityPair.getFirst();
			acticvityCardinalityAfter = acticvityCardinalityPair.getSecond();
			
			if(acticvityCardinalityBefore == 0)
				equalToZeroNumBefore ++;
			else if(acticvityCardinalityBefore == 1)
				equalToOneNumBefore ++;
			else if (acticvityCardinalityBefore > 1)
				moreThanOneNumBefore ++;
			
			if(acticvityCardinalityAfter == 0)
				equalToZeroNumAfter ++;
			else if(acticvityCardinalityAfter == 1)
				equalToOneNumAfter ++;
			else if (acticvityCardinalityAfter > 1)
				moreThanOneNumAfter ++;

			if(acticvityCardinalityBefore == 0 && acticvityCardinalityAfter == 0)
				equalToZeroNumBeforePlusAfter ++;
			else if(acticvityCardinalityBefore + acticvityCardinalityAfter >= 1)
				NoLessThanOneNumBeforePlusAfter ++;			
		}

		
		//in the future, the conflict should be solved. assign priority
		
		//the order below impact the discovered activity edges
		if(NoLessThanOneNumBeforePlusAfter == acticvityCardinalityPairList.size()){
			activityType = mxConstants.COEXISTENCE;
		}
		
		if(equalToZeroNumBefore == acticvityCardinalityPairList.size()){
			activityType = mxConstants.NONPRECEDENCE;
		}
		
		if(equalToZeroNumAfter == acticvityCardinalityPairList.size()){
			activityType = mxConstants.NONRESPONSE;
		}
		
		if(moreThanOneNumBefore + equalToOneNumBefore == acticvityCardinalityPairList.size()){
			activityType = mxConstants.PRECEDENCE;
		}
		
		if(equalToOneNumBefore == acticvityCardinalityPairList.size()){
			activityType = mxConstants.UNARYPRECEDENCE;
		}
		
		if(moreThanOneNumAfter + equalToOneNumAfter == acticvityCardinalityPairList.size()){
			activityType = mxConstants.RESPONSE;
		}
				
	    if(equalToOneNumAfter == acticvityCardinalityPairList.size()){
	    	activityType = mxConstants.UNARYRESPONSE;
	    }
	    
		if(equalToZeroNumBeforePlusAfter == acticvityCardinalityPairList.size()){
			activityType = mxConstants.NONCOEXISTENCE;
		}
		
		System.out.println("activityType in transformCardinalityPairListToActicityType: " + activityType);
		return activityType;
	}

	//select an activity type list based on a list of (before, after)
	public List<String> transformCardinalityPairListToActivityTypeList(List<Pair<Integer, Integer>> acticvityCardinalityPairList){
		String activityType = null;
		List<String> activityTypeList = new ArrayList<String>();
		
		int acticvityCardinalityBefore = 0;
		int acticvityCardinalityAfter = 0;
		int moreThanOneNumBefore = 0;
		int equalToOneNumBefore = 0;
		int equalToZeroNumBefore = 0;
		int moreThanOneNumAfter = 0;
		int equalToOneNumAfter = 0;
		int equalToZeroNumAfter = 0;
		int NoLessThanOneNumBeforePlusAfter = 0;
		int equalToZeroNumBeforePlusAfter = 0;
		
		for(Pair<Integer, Integer> acticvityCardinalityPair : acticvityCardinalityPairList){
			acticvityCardinalityBefore = acticvityCardinalityPair.getFirst();
			acticvityCardinalityAfter = acticvityCardinalityPair.getSecond();
			
			if(acticvityCardinalityBefore == 0)
				equalToZeroNumBefore ++;
			else if(acticvityCardinalityBefore == 1)
				equalToOneNumBefore ++;
			else if (acticvityCardinalityBefore > 1)
				moreThanOneNumBefore ++;
			
			if(acticvityCardinalityAfter == 0)
				equalToZeroNumAfter ++;
			else if(acticvityCardinalityAfter == 1)
				equalToOneNumAfter ++;
			else if (acticvityCardinalityAfter > 1)
				moreThanOneNumAfter ++;

			if(acticvityCardinalityBefore == 0 && acticvityCardinalityAfter == 0)
				equalToZeroNumBeforePlusAfter ++;
			else if(acticvityCardinalityBefore + acticvityCardinalityAfter >= 1)
				NoLessThanOneNumBeforePlusAfter ++;			
		}

		
		//add all satisfied constraint type into the list
		
		//the order below impact the discovered activity edges
		if(NoLessThanOneNumBeforePlusAfter == acticvityCardinalityPairList.size()){
			activityType = mxConstants.COEXISTENCE;
			activityTypeList.add(activityType);
		}
		
		if(equalToZeroNumBefore == acticvityCardinalityPairList.size()){
			activityType = mxConstants.NONPRECEDENCE;
			activityTypeList.add(activityType);
		}
		
		if(equalToZeroNumAfter == acticvityCardinalityPairList.size()){
			activityType = mxConstants.NONRESPONSE;
			activityTypeList.add(activityType);
		}
		
		if(moreThanOneNumBefore + equalToOneNumBefore == acticvityCardinalityPairList.size()){
			activityType = mxConstants.PRECEDENCE;
			activityTypeList.add(activityType);
		}
		
		if(equalToOneNumBefore == acticvityCardinalityPairList.size()){
			activityType = mxConstants.UNARYPRECEDENCE;
			activityTypeList.add(activityType);
		}
		
		if(moreThanOneNumAfter + equalToOneNumAfter == acticvityCardinalityPairList.size()){
			activityType = mxConstants.RESPONSE;
			activityTypeList.add(activityType);
		}
				
	    if(equalToOneNumAfter == acticvityCardinalityPairList.size()){
	    	activityType = mxConstants.UNARYRESPONSE;
	    	activityTypeList.add(activityType);
	    }
	    
		if(equalToZeroNumBeforePlusAfter == acticvityCardinalityPairList.size()){
			activityType = mxConstants.NONCOEXISTENCE;
			activityTypeList.add(activityType);
		}
		
		System.out.println("activityType in transformCardinalityPairListToActicityType: " + activityType);
		return activityTypeList;
	}
	
	/*
	 * compute the confidence (fitness) of each constraint type 
	 * based on the input pair list
	 */
	public Map<String, Double> transformCardinalityPairListToActivityTypeVSConfidence(
			List<Pair<Integer, Integer>> activityCardinalityPairList){
		Map<String, Double> activityConstraintTypeVSConfidence = new HashMap<String, Double>();
		double cardinalityPairNum = activityCardinalityPairList.size();
		String activityType = null;
		double confidence = 0;
		
		int acticvityCardinalityBefore = 0;
		int acticvityCardinalityAfter = 0;
		int moreThanOneNumBefore = 0;
		int equalToOneNumBefore = 0;
		int equalToZeroNumBefore = 0;
		int moreThanOneNumAfter = 0;
		int equalToOneNumAfter = 0;
		int equalToZeroNumAfter = 0;
		int NoLessThanOneNumBeforePlusAfter = 0;
		int equalToZeroNumBeforePlusAfter = 0;
		
		for(Pair<Integer, Integer> acticvityCardinalityPair : activityCardinalityPairList){
			acticvityCardinalityBefore = acticvityCardinalityPair.getFirst();
			acticvityCardinalityAfter = acticvityCardinalityPair.getSecond();
			
			if(acticvityCardinalityBefore == 0)
				equalToZeroNumBefore ++;
			else if(acticvityCardinalityBefore == 1)
				equalToOneNumBefore ++;
			else if (acticvityCardinalityBefore > 1)
				moreThanOneNumBefore ++;
			
			if(acticvityCardinalityAfter == 0)
				equalToZeroNumAfter ++;
			else if(acticvityCardinalityAfter == 1)
				equalToOneNumAfter ++;
			else if (acticvityCardinalityAfter > 1)
				moreThanOneNumAfter ++;

			if(acticvityCardinalityBefore == 0 && acticvityCardinalityAfter == 0)
				equalToZeroNumBeforePlusAfter ++;
			else if(acticvityCardinalityBefore + acticvityCardinalityAfter >= 1)
				NoLessThanOneNumBeforePlusAfter ++;			
		}
		
		//the order below impact the discovered activity edges
		activityType = mxConstants.COEXISTENCE;
		confidence = NoLessThanOneNumBeforePlusAfter/cardinalityPairNum;
		activityConstraintTypeVSConfidence.put(activityType, confidence);
		
		activityType = mxConstants.NONPRECEDENCE;
		confidence = equalToZeroNumBefore/cardinalityPairNum;
		activityConstraintTypeVSConfidence.put(activityType, confidence);
		
		activityType = mxConstants.NONRESPONSE;
		confidence = equalToZeroNumAfter/cardinalityPairNum;
		activityConstraintTypeVSConfidence.put(activityType, confidence);
		
		activityType = mxConstants.PRECEDENCE;
		confidence = (moreThanOneNumBefore + equalToOneNumBefore)/cardinalityPairNum;
		activityConstraintTypeVSConfidence.put(activityType, confidence);
		
		activityType = mxConstants.UNARYPRECEDENCE;
		confidence = equalToOneNumBefore/cardinalityPairNum;
		activityConstraintTypeVSConfidence.put(activityType, confidence);
		
		activityType = mxConstants.RESPONSE;
		confidence = (moreThanOneNumAfter + equalToOneNumAfter)/cardinalityPairNum;
		activityConstraintTypeVSConfidence.put(activityType, confidence);
		
		activityType = mxConstants.UNARYRESPONSE;
		confidence = equalToOneNumAfter/cardinalityPairNum;
		activityConstraintTypeVSConfidence.put(activityType, confidence);
		
		activityType = mxConstants.NONCOEXISTENCE;
		confidence = equalToZeroNumBeforePlusAfter/cardinalityPairNum;
		activityConstraintTypeVSConfidence.put(activityType, confidence);
		
		for(String type : activityConstraintTypeVSConfidence.keySet()){
			System.out.println("type in transformCardinalityPairListToActicityType: " + type);
			System.out.println("confidence in transformCardinalityPairListToActicityType: " + activityConstraintTypeVSConfidence.get(type));
		}		
		return activityConstraintTypeVSConfidence;
	}

	/**
	 * @param acticvityCardinalityPairList
	 * @return a map between constraint type and its number of instances based on a list of cardinality pair
	 */
	public Map<String, Integer> transformCardinalityPairListToConstraintVSNumMap(List<Pair<Integer, Integer>> acticvityCardinalityPairList){
		Map<String, Integer> constraintVSInstanceNum = new HashMap<String, Integer>();
		
		int acticvityCardinalityBefore = 0;
		int acticvityCardinalityAfter = 0;
		int moreThanOneNumBefore = 0;
		int equalToOneNumBefore = 0;
		int equalToZeroNumBefore = 0;
		int moreThanOneNumAfter = 0;
		int equalToOneNumAfter = 0;
		int equalToZeroNumAfter = 0;
		int NoLessThanOneNumBeforePlusAfter = 0;
		int equalToZeroNumBeforePlusAfter = 0;
		
		for(Pair<Integer, Integer> acticvityCardinalityPair : acticvityCardinalityPairList){
			acticvityCardinalityBefore = acticvityCardinalityPair.getFirst();
			acticvityCardinalityAfter = acticvityCardinalityPair.getSecond();
			
			if(acticvityCardinalityBefore == 0)
				equalToZeroNumBefore ++;
			else if(acticvityCardinalityBefore == 1)
				equalToOneNumBefore ++;
			else if (acticvityCardinalityBefore > 1)
				moreThanOneNumBefore ++;
			
			if(acticvityCardinalityAfter == 0)
				equalToZeroNumAfter ++;
			else if(acticvityCardinalityAfter == 1)
				equalToOneNumAfter ++;
			else if (acticvityCardinalityAfter > 1)
				moreThanOneNumAfter ++;

			if(acticvityCardinalityBefore == 0 && acticvityCardinalityAfter == 0)
				equalToZeroNumBeforePlusAfter ++;
			else if(acticvityCardinalityBefore + acticvityCardinalityAfter >= 1)
				NoLessThanOneNumBeforePlusAfter ++;			
		}

		
		//add all constraint type and its number of instances
		constraintVSInstanceNum.put(mxConstants.COEXISTENCE, NoLessThanOneNumBeforePlusAfter);
		constraintVSInstanceNum.put(mxConstants.NONPRECEDENCE, equalToZeroNumBefore);
		constraintVSInstanceNum.put(mxConstants.NONRESPONSE, equalToZeroNumAfter);
		constraintVSInstanceNum.put(mxConstants.PRECEDENCE, moreThanOneNumBefore + equalToOneNumBefore);
		constraintVSInstanceNum.put(mxConstants.UNARYPRECEDENCE, equalToOneNumBefore);
		constraintVSInstanceNum.put(mxConstants.RESPONSE, moreThanOneNumAfter + equalToOneNumAfter);
		constraintVSInstanceNum.put(mxConstants.UNARYRESPONSE, equalToOneNumAfter);
		constraintVSInstanceNum.put(mxConstants.NONCOEXISTENCE, equalToZeroNumBeforePlusAfter);
	
		System.out.println("constraintVSInstanceNum size: " + constraintVSInstanceNum.size());
		return constraintVSInstanceNum;
	}
	
	/**
	 * check if two class are related, i.e., there exist a class relation between the two classes
	 * if yes, return the class relation id; if no, return null
	 */
	public String checkTwoClassesIfRelated(String firstClassNodeId, String secondClassNodeId){
		String classEdgeId = null;	
		for(ClassEdge classEdge : allClassRelations){
			String sourceNodeId = classEdge.getSourceNode().getID().toString();
			String targetNodeId = classEdge.getTargetNode().getID().toString();
			if((sourceNodeId.equals(firstClassNodeId) && targetNodeId.equals(secondClassNodeId)) ||
					(sourceNodeId.equals(secondClassNodeId) && targetNodeId.equals(firstClassNodeId))){;
				classEdgeId = classEdge.getID().toString();
			}
		}				
		return classEdgeId;
	}
	
	/**
	 * get the class relation which has the input two class nodes
	 * if yes, return the class relation id; if no, return null
	 * @param firstClassNodeId
	 * @param secondClassNodeId
	 * @return
	 */
	public ClassEdge getClassRelationByTwoClassNodes(String firstClassNodeId, String secondClassNodeId){
		ClassEdge retrievedClassEdge = null;	
		for(ClassEdge classEdge : allClassRelations){
			String sourceNodeId = classEdge.getSourceNode().getID().toString();
			String targetNodeId = classEdge.getTargetNode().getID().toString();
			if((sourceNodeId.equals(firstClassNodeId) && targetNodeId.equals(secondClassNodeId)) ||
					(sourceNodeId.equals(secondClassNodeId) && targetNodeId.equals(firstClassNodeId))){;
				retrievedClassEdge = classEdge;
				break;
			}
		}				
		return retrievedClassEdge;
	}
	
	//remove a activity edge (and the crel edge connected to it if there exist)
	public void removeModelActivityEdge(String activityEdgeId){
		List<MyOriginalCell> removeCells = new ArrayList<MyOriginalCell>();
		ActivityEdge removeEdge = null;
		
		//remove the activity edge and its corresponding cells
		for(ActivityEdge activityEdge : allActivityRelations){
			if(activityEdge.getID().toString().equals(activityEdgeId)){			
				removeEdge = activityEdge;
				for(MyOriginalCell removeCell : graphCells){
					if(removeCell.getID().toString().equals(activityEdgeId)){
						removeCells.add(removeCell);					
					}
				}						
			}
		}
		if(removeEdge != null){				     
			//remove the crel_oc edge and its corresponding cells
			for(CrelOCEdge crelOCEdge : removeEdge.getRelatedCrelOCRelations()){
				String crelOCEdgeId = crelOCEdge.getID().toString();
				for(MyOriginalCell removeCell : graphCells){
					if(removeCell.getID().toString().equals(crelOCEdgeId)){
						removeCells.add(removeCell);					
					}
				}
			}
			
			//remove the crel_rt edge and its corresponding cells
			for(CrelRTEdge crelRTEdge : removeEdge.getRelatedCrelRTRelations()){
				String crelRTEdgeId = crelRTEdge.getID().toString();
				for(MyOriginalCell removeCell : graphCells){
					if(removeCell.getID().toString().equals(crelRTEdgeId)){
						removeCells.add(removeCell);					
					}
				}
			}
			
			allActivityRelations.remove(removeEdge);
			allCrelOCRelations.removeAll(removeEdge.getRelatedCrelOCRelations());
			allCrelRTRelations.removeAll(removeEdge.getRelatedCrelRTRelations());
			graph.getGraphLayoutCache().removeCells(removeCells.toArray());; 
	        getGraphCells().removeAll(removeCells);		        			
		}						
	}
	
	public void addSequentialActivityPair(String sourceActivityId, String targetActivityId, String relationType){
//		if(!sequentialActivityPairList.contains(new Pair<String, String>(sourceActivityId, targetActivityId)) &&
//				!sequentialActivityPairList.contains(new Pair<String, String>(targetActivityId, sourceActivityId))){
//			if(relationType.equals("ActivityEdge_UnaryResponse") || relationType.equals("ActivityEdge_Response"))
//				sequentialActivityPairList.add(new Pair<String, String>(sourceActivityId, targetActivityId));		
//			else if(relationType.equals("ActivityEdge_Precedence") || relationType.equals("ActivityEdge_Precedence"))//problematic
//				sequentialActivityPairList.add(new Pair<String, String>(targetActivityId, sourceActivityId));	
//		}
		if(!sequentialActivityPairList.contains(new Pair<String, String>(sourceActivityId, targetActivityId)) &&
				!sequentialActivityPairList.contains(new Pair<String, String>(targetActivityId, sourceActivityId))){
			if(relationType.equals(mxConstants.UNARYRESPONSE) || relationType.equals(mxConstants.RESPONSE))
				sequentialActivityPairList.add(new Pair<String, String>(sourceActivityId, targetActivityId));		
			else if(relationType.equals(mxConstants.PRECEDENCE) || relationType.equals(mxConstants.UNARYPRECEDENCE))//problematic
				sequentialActivityPairList.add(new Pair<String, String>(targetActivityId, sourceActivityId));	
		}
		
	}
	
	//add the cell (for acticity or class) by id
	public MyOriginalCell getNodeCellById(String Id){
		for(MyOriginalCell cell : graphCells){
			if(cell.getID().toString().equals(Id)){
				return cell;
			}
		}
		return null;
	}
	
	/**
	 * returns the set of all activity relations with the same reference activity,
	 * target activity and class
	 * 
	 */
	public List<ActivityEdge> getActivityRelationFamilyA(String refActicityId, String tarActivityId, String classId){
		List<ActivityEdge> activityEdgeList = new ArrayList<ActivityEdge>();
		for(ActivityEdge activityEdge : allActivityRelations){
			String referenceActivityId = activityEdge.getReferenceNode().getID();	
			String targetActivityId = activityEdge.getTargetNode().getID();
			//if the two activities are satisfied
			if(referenceActivityId.equals(refActicityId) && targetActivityId.equals(tarActivityId)){
				String activityEdgeId = activityEdge.getID();
				CrelOCEdge crelOCEdge = getCrelOCRelationByActivityEdgeAndClass(activityEdgeId, classId);
				if(crelOCEdge != null){
					activityEdgeList.add(activityEdge);
				}
			}			
		}	
		return activityEdgeList;
	}
	
	/**
	 * returns the set of all activity relations with the same reference activity,
	 * target activity and class relation
	 * 
	 */
	public List<ActivityEdge> getActivityRelationFamilyB(String refActicityId, String tarActivityId, String classRelationId){
		List<ActivityEdge> activityEdgeList = new ArrayList<ActivityEdge>();
		for(ActivityEdge activityEdge : allActivityRelations){
			String referenceActivityId = activityEdge.getReferenceNode().getID();	
			String targetActivityId = activityEdge.getTargetNode().getID();
			//if the two activities are satisfied
			if(referenceActivityId.equals(refActicityId) && targetActivityId.equals(tarActivityId)){
				String activityEdgeId = activityEdge.getID();
				CrelRTEdge crelRTEdge = getCrelRTRelationByActivityEdgeAndClassEdge(activityEdgeId, classRelationId);
				if(crelRTEdge != null){
					activityEdgeList.add(activityEdge);
				}
			}			
		}	
		return activityEdgeList;
	}
	
	/**
	 * returns the crel oc relaton with the activity edge and class
	 * 
	 */
	public CrelOCEdge getCrelOCRelationByActivityEdgeAndClass(String activityEdgeIdInput, String ClassIdInput){
		CrelOCEdge crelOCEdgeOutput = null;
		for(CrelOCEdge crelOCEdge : allCrelOCRelations){
			String activityEdgeId = crelOCEdge.getRelatedActivityRelation().getID();
			String classId = crelOCEdge.getRelatedClass().getID();
			if(activityEdgeId.equals(activityEdgeIdInput) && classId.equals(ClassIdInput)){
				crelOCEdgeOutput = crelOCEdge;
				break;
			}
		}
		return crelOCEdgeOutput;
	}
	
	
	/**
	 * returns the crel rt relaton with the activity edge and class edge
	 * 
	 */
	public CrelRTEdge getCrelRTRelationByActivityEdgeAndClassEdge(String activityEdgeIdInput, String ClassEdgeIdInput){
		CrelRTEdge crelRTEdgeOutput = null;
		for(CrelRTEdge crelRTEdge : allCrelRTRelations){
			String activityEdgeId = crelRTEdge.getRelatedActivityRelation().getID();
			String classEdgeId = crelRTEdge.getRelatedClassRelation().getID();
			if(activityEdgeId.equals(activityEdgeIdInput) && classEdgeId.equals(ClassEdgeIdInput)){
				crelRTEdgeOutput = crelRTEdge;
				break;
			}
		}
		return crelRTEdgeOutput;
	}
	
	public void generateTypeVSClassRelationMap(){
		typeVSClassRelationMap.clear();
		for(ClassEdge classRelation : allClassRelations){
			String type = classRelation.getEdgeName();
			typeVSClassRelationMap.put(type, classRelation);
		}
	}
	
	public Map<String, ClassEdge> getTypeVSClassRelationMap(){
		return typeVSClassRelationMap;
	}
	
	public static void main(String str[]) throws FileNotFoundException{
		
//		List<Pair<String, String>> sequentialActivityPairList = new ArrayList<Pair<String, String>>();
//		sequentialActivityPairList.add(new Pair<String, String>("aa", "bb"));
//		if(sequentialActivityPairList.contains(new Pair<String, String>("aa", "bbb"))) System.out.printf("contain");
//		else System.out.printf("not contain");
//		
//		System.exit(0);
		FileInputStream input = new FileInputStream("E:\\reimbursement.ocbcmml");
		ImportFromFile fileParse = new ImportFromFile();       
        fileParse.parse(input);
        OCBCModel inputModel = fileParse.getModel();
        
//		OCBCModel modela = new OCBCModelImpl();
		OCBCModel modelb = (OCBCModel) inputModel.clone();
		
		if(modelb == inputModel){
			System.out.print("\n model equal");
		}
//		else System.out.print("model not equal");
//		
//		if(modelb.getGraph() == modela.getGraph()){
//			System.out.print("\n graph equal");
//		}
//		else System.out.print("\n graph not equal");
//			
//		if(modelb.getAllActivities() == modela.getAllActivities()){
//			System.out.print("\n Activities equal");
//		}
//		else System.out.print("\n Activities not equal");
	}
}