package org.processmining.ocbc.models.io;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.OutputStream;

import org.deckfour.spex.SXDocument;
import org.deckfour.spex.SXTag;
import org.deckfour.xes.logging.XLogging;
import org.jgraph.JGraph;
import org.jgraph.graph.GraphConstants;
import org.processmining.ocbc.models.AOCEdge;
import org.processmining.ocbc.models.ActivityEdge;
import org.processmining.ocbc.models.ClassEdge;
import org.processmining.ocbc.models.CrelOCEdge;
import org.processmining.ocbc.models.CrelRTEdge;
import org.processmining.ocbc.models.OCBCModel;
import org.processmining.ocbc.models.impl.OCBCModelImpl;
import org.processmining.ocbc.models.visualization.cell.AOCEdgeCell;
import org.processmining.ocbc.models.visualization.cell.ActivityCell;
import org.processmining.ocbc.models.visualization.cell.ActivityEdgeCell;
import org.processmining.ocbc.models.visualization.cell.ClassCell;
import org.processmining.ocbc.models.visualization.cell.ClassEdgeCell;
import org.processmining.ocbc.models.visualization.cell.CrelEdgeCell;
import org.processmining.ocbc.models.visualization.cell.MEdgeCell;
import org.processmining.ocbc.models.visualization.cell.MNodeCell;
import org.processmining.ocbc.models.visualization.cell.MyOriginalCell;
import org.processmining.ocbc.models.visualization.cell.TransparentCell;

	public class ExportToFileOld{
		
		public static void exportToFile(OCBCModel model, OutputStream out) throws IOException {
			
		XLogging.log("start serializing log to XES.XML",
				XLogging.Importance.DEBUG);
		long start = System.currentTimeMillis();
		SXDocument doc = new SXDocument(out);
		doc.addComment("This file has been generated with the OpenXES library. It conforms");

		
		SXTag modelTag = doc.addNode("OCBC");
		modelTag.addAttribute("name", model.getModelName());
		JGraph graph = model.getGraph();
		
	       
		for(MyOriginalCell cell : model.getGraphCells()){
			
			if(cell instanceof ClassCell || cell instanceof ActivityCell){
				addNode(modelTag, (MNodeCell) cell);
			}

		}
		SXTag edgeTag;
		/*
		for(ActivityEdge activityEdge : model.getAllActivityRelations()){											
			edgeTag = modelTag.addChildNode("Edge");
			edgeTag.addAttribute("ID", activityEdge.getID().toString());
			edgeTag.addAttribute("referenceNodeID", activityEdge.getReferenceNode().getID().toString());			
			edgeTag.addAttribute("targetNodeID", activityEdge.getTargetNode().getID().toString());
			edgeTag.addAttribute("cardinalityType", activityEdge.getCardinalityType());
			edgeTag.addAttribute("transparentCellX", GraphConstants.getBounds(activityEdge.getCorrespondingCell().getAttributes()).getX());
			edgeTag.addAttribute("transparentCellY", GraphConstants.getBounds(activityEdge.getCorrespondingCell().getAttributes()).getY());
			edgeTag.addAttribute("type", "ActivityEdge");			
		}*/
		
		for(MyOriginalCell cell : model.getGraphCells()){		
			if(cell instanceof TransparentCell && cell.getCorrespondingElement() instanceof ActivityEdge){
				ActivityEdge activityEdge = (ActivityEdge) cell.getCorrespondingElement();
				String activityTypeCombination = activityEdge.getCardinalityType();
				if(cell.getCorrespondingSecondElement() != null && cell.getCorrespondingSecondElement() instanceof ActivityEdge){
					ActivityEdge activityEdgeSecond = (ActivityEdge) cell.getCorrespondingSecondElement();
					activityTypeCombination = activityTypeCombination + "_" + activityEdgeSecond.getCardinalityType();
				}				
				edgeTag = modelTag.addChildNode("Edge");
				edgeTag.addAttribute("ID", activityEdge.getID().toString());
				edgeTag.addAttribute("referenceNodeID", activityEdge.getReferenceNode().getID().toString());			
				edgeTag.addAttribute("targetNodeID", activityEdge.getTargetNode().getID().toString());
				edgeTag.addAttribute("cardinalityType", activityTypeCombination);
				edgeTag.addAttribute("transparentCellX", GraphConstants.getBounds(activityEdge.getCorrespondingCell().getAttributes()).getX());
				edgeTag.addAttribute("transparentCellY", GraphConstants.getBounds(activityEdge.getCorrespondingCell().getAttributes()).getY());
				edgeTag.addAttribute("type", "ActivityEdge");
			}			
		}

		for(AOCEdge aOCEdge : model.getAllAOCRelations()){			
			edgeTag = modelTag.addChildNode("Edge");
			edgeTag.addAttribute("ID", aOCEdge.getID().toString());
			edgeTag.addAttribute("sourceNodeID", aOCEdge.getActivityNode().getID().toString());	
			edgeTag.addAttribute("sourceAlwaysCardinality", aOCEdge.getActivityAlwaysCardinality().toLetters());
			edgeTag.addAttribute("sourceEventuallyCardinality", aOCEdge.getActivityEventuallyCardinality().toLetters());
			edgeTag.addAttribute("targetNodeID", aOCEdge.getClassNode().getID().toString());
			edgeTag.addAttribute("targetCardinality", aOCEdge.getClassCardinality().toLetters());
			edgeTag.addAttribute("transparentCellX", GraphConstants.getBounds(aOCEdge.getCorrespondingCell().getAttributes()).getX());
			edgeTag.addAttribute("transparentCellY", GraphConstants.getBounds(aOCEdge.getCorrespondingCell().getAttributes()).getY());
			edgeTag.addAttribute("type", "AOCEdge");
		}
		
		for(ClassEdge classEdge : model.getAllClassRelations()){			
			edgeTag = modelTag.addChildNode("Edge");
			edgeTag.addAttribute("ID", classEdge.getID().toString());
			edgeTag.addAttribute("name", classEdge.getCorrespondingCell().getUserObject().toString());
			edgeTag.addAttribute("sourceNodeID", classEdge.getSourceNode().getID().toString());		
			edgeTag.addAttribute("sourceAlwaysCardinality", classEdge.getSourceAlwaysCardinality().toLetters());	
			edgeTag.addAttribute("sourceEventuallyCardinality", classEdge.getSourceEventuallyCardinality().toLetters());
			edgeTag.addAttribute("targetNodeID", classEdge.getTargetNode().getID().toString());
			edgeTag.addAttribute("targetAlwaysCardinality", classEdge.getTargetAlwaysCardinality().toLetters());
			edgeTag.addAttribute("targetEventuallyCardinality", classEdge.getTargetEventuallyCardinality().toLetters());
			edgeTag.addAttribute("transparentCellX", GraphConstants.getBounds(classEdge.getCorrespondingCell().getAttributes()).getX());
			edgeTag.addAttribute("transparentCellY", GraphConstants.getBounds(classEdge.getCorrespondingCell().getAttributes()).getY());
			edgeTag.addAttribute("height", GraphConstants.getBounds(classEdge.getCorrespondingCell().getAttributes()).getHeight());
			edgeTag.addAttribute("width", GraphConstants.getBounds(classEdge.getCorrespondingCell().getAttributes()).getWidth());
			edgeTag.addAttribute("type", "ClassEdge");
		}
		
		for(MyOriginalCell cell : model.getGraphCells()){		
			if(cell instanceof TransparentCell && cell.getCorrespondingElement() instanceof CrelOCEdge){
				CrelOCEdge crelOCEdge = (CrelOCEdge) cell.getCorrespondingElement();						
				edgeTag = modelTag.addChildNode("Edge");
				edgeTag.addAttribute("ID", crelOCEdge.getID().toString());
				edgeTag.addAttribute("activityRelationID", crelOCEdge.getRelatedActivityRelation().getID().toString());			
				edgeTag.addAttribute("classNodeID", crelOCEdge.getRelatedClass().getID().toString());
				edgeTag.addAttribute("transparentCellX", GraphConstants.getBounds(crelOCEdge.getCorrespondingCell().getAttributes()).getX());
				edgeTag.addAttribute("transparentCellY", GraphConstants.getBounds(crelOCEdge.getCorrespondingCell().getAttributes()).getY());
				edgeTag.addAttribute("type", "CrelOCEdge");	
			}		
			else if(cell instanceof TransparentCell && cell.getCorrespondingElement() instanceof CrelRTEdge){
				CrelRTEdge crelRTEdge = (CrelRTEdge) cell.getCorrespondingElement();						
				edgeTag = modelTag.addChildNode("Edge");
				edgeTag.addAttribute("ID", crelRTEdge.getID().toString());
				edgeTag.addAttribute("activityRelationID", crelRTEdge.getRelatedActivityRelation().getID().toString());			
				edgeTag.addAttribute("classRelationID", crelRTEdge.getRelatedClassRelation().getID().toString());
				edgeTag.addAttribute("transparentCellX", GraphConstants.getBounds(crelRTEdge.getCorrespondingCell().getAttributes()).getX());
				edgeTag.addAttribute("transparentCellY", GraphConstants.getBounds(crelRTEdge.getCorrespondingCell().getAttributes()).getY());
				edgeTag.addAttribute("type", "CrelRTEdge");
			}	
		}
		/*
		for(CrelOCEdge crelOCEdge : model.getAllCrelOCRelations()){			
			edgeTag = modelTag.addChildNode("Edge");
			edgeTag.addAttribute("ID", crelOCEdge.getID().toString());
			edgeTag.addAttribute("activityRelationID", crelOCEdge.getRelatedActivityRelation().getID().toString());			
			edgeTag.addAttribute("classNodeID", crelOCEdge.getRelatedClass().getID().toString());
			edgeTag.addAttribute("transparentCellX", GraphConstants.getBounds(crelOCEdge.getCorrespondingCell().getAttributes()).getX());
			edgeTag.addAttribute("transparentCellY", GraphConstants.getBounds(crelOCEdge.getCorrespondingCell().getAttributes()).getY());
			edgeTag.addAttribute("type", "CrelOCEdge");			
		}
		
		for(CrelRTEdge crelRTEdge : model.getAllCrelRTRelations()){			
			edgeTag = modelTag.addChildNode("Edge");
			edgeTag.addAttribute("ID", crelRTEdge.getID().toString());
			edgeTag.addAttribute("activityRelationID", crelRTEdge.getRelatedActivityRelation().getID().toString());			
			edgeTag.addAttribute("classRelationID", crelRTEdge.getRelatedClassRelation().getID().toString());
			edgeTag.addAttribute("transparentCellX", GraphConstants.getBounds(crelRTEdge.getCorrespondingCell().getAttributes()).getX());
			edgeTag.addAttribute("transparentCellY", GraphConstants.getBounds(crelRTEdge.getCorrespondingCell().getAttributes()).getY());
			edgeTag.addAttribute("type", "CrelRTEdge");
		}*/
		
		doc.close();
		String duration = " (" + (System.currentTimeMillis() - start)
				+ " msec.)";
		XLogging.log("finished serializing log" + duration,
				XLogging.Importance.DEBUG);
	}
		
	protected static void addNode(SXTag tag, MNodeCell cell)
			throws IOException {
		SXTag nodeTag;			
		nodeTag = tag.addChildNode("Node");
		nodeTag.addAttribute("ID", cell.getID().toString());
		nodeTag.addAttribute("name", cell.getUserObject().toString());
		if(cell.getAttributes() == null){
			System.out.println("\n null");
		}
		System.out.println("\n x position:" + GraphConstants.getBounds(cell.getAttributes()).getX());
		System.out.println("\n y position:" + GraphConstants.getBounds(cell.getAttributes()).getY());
		nodeTag.addAttribute("positionX", GraphConstants.getBounds(cell.getAttributes()).getX());
		nodeTag.addAttribute("positionY", GraphConstants.getBounds(cell.getAttributes()).getY());
		nodeTag.addAttribute("height", GraphConstants.getBounds(cell.getAttributes()).getHeight());
		nodeTag.addAttribute("width", GraphConstants.getBounds(cell.getAttributes()).getWidth());
		
		if(cell instanceof ActivityCell){
			nodeTag.addAttribute("type", "ActivityCell");
		}

		else if(cell instanceof ClassCell){
			nodeTag.addAttribute("type", "ClassCell");
		}
					
	}
	
	protected static void addEdge(SXTag tag, MEdgeCell cell)
			throws IOException {
		SXTag edgeTag;		
		edgeTag = tag.addChildNode("Edge");
		edgeTag.addAttribute("ID", cell.getID().toString());
		edgeTag.addAttribute("soruceNode", cell.getSourceNode().getID().toString());
		edgeTag.addAttribute("targetNode", cell.getTargetNode().getID().toString());
		if(cell instanceof ActivityEdgeCell){
			edgeTag.addAttribute("type", "ActivityEdgeCell");
		}
		else if(cell instanceof AOCEdgeCell){
			edgeTag.addAttribute("type", "AOCEdgeCell");
		}
		else if(cell instanceof ClassEdgeCell){
			edgeTag.addAttribute("type", "ClassEdgeCell");
		}
		else if(cell instanceof CrelEdgeCell){
			edgeTag.addAttribute("type", "CrelEdgeCell");
		}								
	}
	
	public static void main(String[] str) throws ClassNotFoundException, IOException{
		OCBCModel model = new OCBCModelImpl("hello world");
		  //
		   FileInputStream fout = new FileInputStream("E:\\555.ocbcmml");
		   @SuppressWarnings("resource")
		   ObjectInputStream oos = new ObjectInputStream(fout);   
		   model = (OCBCModel) oos.readObject();
		   
		   FileOutputStream out = new FileOutputStream("E:\\understandableModel.ocbcmml");
		   exportToFile(model, out) ;
	}
}