package org.processmining.ocbc.models.io;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.util.HashMap;
import java.util.Map;

import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.processmining.ocbc.models.OCBCModel;
import org.processmining.ocbc.models.impl.OCBCModelImpl;
import org.processmining.ocbc.models.visualization.ModelLayoutBuilder;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;
public class ImportFromFile {
	private OCBCModel model = new OCBCModelImpl("OCBC model");
    private ModelLayoutBuilder builder = new ModelLayoutBuilder(model);
    
    public static void main(String[] args) throws FileNotFoundException {
    	 
        ImportFromFile ajavademo = new ImportFromFile();
        System.out.println("\n" + ajavademo.model.getModelName());
        System.out.println("\n" + ajavademo.model.getAllActivities().size());
        FileInputStream is = new FileInputStream("D:\\otc.ocbcmml");
        ajavademo.parse(is);
        
        System.out.println("\n" + ajavademo.model.getAllActivities().size());
        
        JFrame frame = new JFrame();  
        frame.setSize(1000, 500);  
        frame.setTitle("Activity Diagram");  
        frame.getContentPane().add(new JScrollPane(ajavademo.model.getGraph()));  
          
         frame.setBounds(50, 50, 700, 800);  
          
         frame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);  
         frame.setVisible(true);  
    }
    public void parse(InputStream input) {
        try {
            SAXParserFactory factory = SAXParserFactory.newInstance();
            SAXParser parser = factory.newSAXParser();
            DefaultHandler handler = new SAXHandler();
            parser.parse(input, handler);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
    
    public OCBCModel getModel(){
    	return model;
    }
    
    class SAXHandler extends DefaultHandler {
        @Override
        public void startElement(String uri, String localName,
                                 String qName, Attributes attributes)
                throws SAXException {
            int attributeLength = attributes.getLength();
            
            if ("Node".equals(qName)) {
            	Map<String, String> nameValue = new HashMap<String, String>();
                for (int i = 0; i < attributeLength; i++) {                  
                    String name = attributes.getQName(i);
                    String value = attributes.getValue(i);
                    nameValue.put(name, value);
                }              
                builder.addNode(nameValue.get("type"), nameValue.get("name"), nameValue.get("ID"), 
                		Double.parseDouble(nameValue.get("positionX")), Double.parseDouble(nameValue.get("positionY")),
                		Double.parseDouble(nameValue.get("height")), Double.parseDouble(nameValue.get("width")));
            }
            
            if ("Edge".equals(qName)) {
            	Map<String, String> nameValue = new HashMap<String, String>();

                for (int i = 0; i < attributeLength; i++) {
                    String name = attributes.getQName(i);
                    String value = attributes.getValue(i);
                    nameValue.put(name, value);
                }

                String edgeType = nameValue.get("type");
  			               	
        		if(edgeType.equals("ClassEdge")) {
//        			Cardinality sourceAlwaysCardinality = new Cardinality(nameValue.get("sourceAlwaysCardinality").toString());
//        			Cardinality sourceEventuallyCardinality = new Cardinality(nameValue.get("sourceEventuallyCardinality").toString());
//        			Cardinality targetAlwaysCardinality = new Cardinality(nameValue.get("targetAlwaysCardinality").toString());
//        			Cardinality targetEventuallyCardinality = new Cardinality(nameValue.get("targetEventuallyCardinality").toString());
        			builder.addClassEdge(nameValue.get("ID"), nameValue.get("name"), nameValue.get("sourceNodeID"), 
        					nameValue.get("sourceAlwaysCardinality"), nameValue.get("sourceEventuallyCardinality"), 
        				nameValue.get("targetNodeID"), nameValue.get("targetAlwaysCardinality"),  
        				nameValue.get("targetEventuallyCardinality"),
        				Double.parseDouble(nameValue.get("transparentCellX")), Double.parseDouble(nameValue.get("transparentCellY")),
        				Double.parseDouble(nameValue.get("height")), Double.parseDouble(nameValue.get("width")));
        			}
        			else if(edgeType.equals("ActivityEdge")) 
        			builder.addActivityEdge(nameValue.get("ID"), nameValue.get("referenceNodeID"), 
        				 nameValue.get("targetNodeID"), nameValue.get("cardinalityType"),  
        				Double.parseDouble(nameValue.get("transparentCellX")), Double.parseDouble(nameValue.get("transparentCellY")));
        		else if(edgeType.equals("AOCEdge")) 
        			builder.addAOCEdge(nameValue.get("ID"), nameValue.get("sourceNodeID"), 
        				nameValue.get("sourceAlwaysCardinality"), nameValue.get("sourceEventuallyCardinality"), nameValue.get("targetNodeID"), nameValue.get("targetCardinality"),  
        				Double.parseDouble(nameValue.get("transparentCellX")), Double.parseDouble(nameValue.get("transparentCellY")));
        		else if(edgeType.equals("CrelOCEdge")) 
        			builder.addCrelOCEdge(nameValue.get("ID"), nameValue.get("activityRelationID"), nameValue.get("classNodeID"),  
        				Double.parseDouble(nameValue.get("transparentCellX")), Double.parseDouble(nameValue.get("transparentCellY")));
				else if(edgeType.equals("CrelRTEdge")) 
        			builder.addCrelRTEdge(nameValue.get("ID"), nameValue.get("activityRelationID"), nameValue.get("classRelationID"),  
        				Double.parseDouble(nameValue.get("transparentCellX")), Double.parseDouble(nameValue.get("transparentCellY")));
            }
        }
    }
}