package org.processmining.ocbc.models.io;

import java.awt.Toolkit;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Scanner;

import javax.swing.JFrame;

import org.processmining.contexts.uitopia.annotations.UIImportPlugin;
import org.processmining.framework.abstractplugins.AbstractImportPlugin;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.mxgraph.examples.swing.GraphEditor;
import org.processmining.mxgraph.examples.swing.GraphEditor.CustomGraphComponent;
import org.processmining.mxgraph.examples.swing.io.TransformToOCBCSemanticModel;
import org.processmining.mxgraph.io.mxCodec;
import org.processmining.mxgraph.util.mxConstants;
import org.processmining.mxgraph.util.mxXmlUtils;
import org.processmining.mxgraph.view.mxGraph;
import org.processmining.ocbc.models.ActivityEdge;
import org.processmining.ocbc.models.CrelOCEdge;
import org.processmining.ocbc.models.CrelRTEdge;
import org.processmining.ocbc.models.OCBCModel;
import org.processmining.ocbc.models.editor.plugins.OCBCEditorConstants;
import org.processmining.ocbc.models.visualization.cell.MNodeCell;
import org.processmining.ocbc.models.visualization.cell.MyOriginalCell;
import org.w3c.dom.Document;



/**
 * Class with the import plugin for a collection of processes
 * 
 * @author Andrea Burattin
 */
@Plugin(
	name = "Import OCBC Model (.ocbcmml)",
	parameterLabels = { "File" },
	returnLabels = { "Imported OCBC model" },
	returnTypes = { OCBCModel.class },
	userAccessible = true
)
@UIImportPlugin(
		description = "Import OCBC Model (.ocbcmml)",//show the type in window for recording files
		extensions = { "ocbcmml" }// the suffix name
)
public class ImportOCBCModel extends AbstractImportPlugin {
	/**
	 * This plugin is used to import an OCBC model
	 * 
	 * @param context
	 * @param input
	 * @param filename
	 * @param fileSizeInBytes
	 * @return
	 * @throws IOException 
	 */
	protected Object importFromStream(PluginContext context, InputStream input, String filename, 
			long fileSizeInBytes) throws Exception {
		
		context.log("Import started..."); 
		System.out.println("old input OCBC model start");
	
		return generateOCBCModel(input);
	}
	
	public OCBCModel generateOCBCModel(InputStream input) throws IOException{
		OCBCModel model = null;
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		byte[] buffer = new byte[1024];
		int len;
		while ((len = input.read(buffer)) > -1 ) {
		    baos.write(buffer, 0, len);
		}
		baos.flush();

		// Open new InputStreams using the recorded bytes
		// Can be repeated as many times as you wish
		InputStream inputA = new ByteArrayInputStream(baos.toByteArray()); 
		InputStream inputB = new ByteArrayInputStream(baos.toByteArray()); 
		
        Scanner scanner = new Scanner(inputA, "UTF-8");
        String fileString = scanner.useDelimiter("\\A").next();
        scanner.close();
        
        String versionString = fileString.split("\n")[0];
		String versionName = versionString.split("\"")[1];
		String removeVersion = fileString.substring(fileString.indexOf('\n')+1);
		System.out.println("model file: " + removeVersion);
		
		if(versionName.equals(OCBCEditorConstants.VERSION2_0)){
			mxGraph graphX = new mxGraph();
			Document document = mxXmlUtils.parseXml(removeVersion);
			
			mxCodec codec = new mxCodec(document);
			codec.decode(document.getDocumentElement(), graphX.getModel());
			
			System.out.println("version 2");
			TransformToOCBCSemanticModel transformer = new TransformToOCBCSemanticModel();
			transformer.transform(graphX);
			model = transformer.getModel();
			model.setGraphX(graphX);
			model.setModelFile(mxXmlUtils.parseXml(removeVersion));
			model.setModelString(removeVersion);
			model.setModelVersion(OCBCEditorConstants.VERSION2_0);
		}
		else{
			
			ImportFromFile fileParse = new ImportFromFile();       
	        fileParse.parse(inputB);
	        model = fileParse.getModel();
	        model.setModelVersion(OCBCEditorConstants.VERSION1_0);
	        int i=0;
	        for(MyOriginalCell cell : model.getGraphCells()){
	        	i++;
	        	System.out.println("cell number" + i);
	        	if(cell instanceof MNodeCell){
	 //       		System.out.println("cell name" + cell.getUserObject().toString());
	 //       		System.out.println("number of related edges" + model.getRelatedEdgeCellsForOneCell(cell).size());
	        	}
	        }
		}
		return model;
	}
	
	
	   public static void main (String args[]) throws IOException {
		   FileInputStream input = new FileInputStream("D:\\otc.ocbcmml");
//			InputStream inputStream = new FileInputStream("d:/123.ocbcmml");
		   ImportOCBCModel importer = new ImportOCBCModel();
		   OCBCModel model = importer.generateOCBCModel(input);
			
			
	        for(ActivityEdge activityEdge : model.getAllActivityRelations()){
	        	System.out.println("\n activityEdge id: " + activityEdge.getID());
	        	System.out.println("\n activityEdge name: " + activityEdge.getCardinalityType());
	        }   
	        
	        for(CrelRTEdge edge : model.getAllCrelRTRelations()){
	        	System.out.println("\n activityEdge id: " + edge.getID());
	        	System.out.println("\n activityEdge name: " + edge.getRelatedActivityRelation().getID());

	        }
	        
	        for(CrelOCEdge edge : model.getAllCrelOCRelations()){
	        	System.out.println("\n activityEdge id: " + edge.getID());
	        	System.out.println("\n activityEdge name: " + edge.getRelatedActivityRelation().getID());

	        }     

	        GraphEditor editor = new GraphEditor("mxGraph Editor", new CustomGraphComponent(model.getGraphX()), model);
	        
	 		JFrame frame = new JFrame();
			model.setModelEnvironment(mxConstants.model_editor);


	         
//			JSplitPane completePanel = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
//			
//			completePanel.setTopComponent(new EditorMenuBar(editor));
////			completePanel.setBottomComponent(editor);
//			completePanel.setResizeWeight(0.02);
//			completePanel.setEnabled(false);
			frame.getContentPane().add(editor);

			frame.setSize(Toolkit.getDefaultToolkit().getScreenSize()); 
			frame.setVisible(true);
	   }
}
