package org.processmining.ocbc.models.visualization;

import java.awt.Color;
import java.awt.geom.Rectangle2D;
import java.io.Serializable;
import java.util.Set;
import java.util.UUID;

import org.deckfour.xes.classification.XEventClasses;
import org.jgraph.JGraph;
import org.jgraph.graph.DefaultGraphModel;
import org.jgraph.graph.DefaultPort;
import org.jgraph.graph.GraphConstants;
import org.jgraph.graph.GraphLayoutCache;
import org.jgraph.graph.GraphModel;
import org.processmining.ocbc.models.AOCEdge;
import org.processmining.ocbc.models.ActivityEdge;
import org.processmining.ocbc.models.ActivityNode;
import org.processmining.ocbc.models.Cardinality;
import org.processmining.ocbc.models.ClassEdge;
import org.processmining.ocbc.models.ClassNode;
import org.processmining.ocbc.models.CrelOCEdge;
import org.processmining.ocbc.models.CrelRTEdge;
import org.processmining.ocbc.models.OCBCModel;
import org.processmining.ocbc.models.impl.AOCEdgeImpl;
import org.processmining.ocbc.models.impl.ActivityEdgeImpl;
import org.processmining.ocbc.models.impl.ActivityNodeImpl;
import org.processmining.ocbc.models.impl.ClassEdgeImpl;
import org.processmining.ocbc.models.impl.ClassNodeImpl;
import org.processmining.ocbc.models.impl.CrelOCEdgeImpl;
import org.processmining.ocbc.models.impl.CrelRTEdgeImpl;
import org.processmining.ocbc.models.visualization.cell.AOCEdgeCell;
import org.processmining.ocbc.models.visualization.cell.ActivityCell;
import org.processmining.ocbc.models.visualization.cell.ActivityEdgeCell;
import org.processmining.ocbc.models.visualization.cell.ClassCell;
import org.processmining.ocbc.models.visualization.cell.ClassEdgeCell;
import org.processmining.ocbc.models.visualization.cell.CrelEdgeCell;
import org.processmining.ocbc.models.visualization.cell.LabelCell;
import org.processmining.ocbc.models.visualization.cell.MyOriginalCell;
import org.processmining.ocbc.models.visualization.cell.TransparentCell;



public class ModelLayoutBuilderOld implements Serializable{

	/**
	 * 
	 */
	private static final long serialVersionUID = 6199234143944904086L;

	protected OCBCModel model;
	
	private double transparentCellSize = 50;
	
	private int classNodeNum = 0;
	private int activityNodeNum = 0;
	private int activityEdgeNum = 0;
	private float[] AOCEdgeDash = { 5f, 5f };
	private float[] CrelEdgeDash = { 15f, 15f };
	
	protected XEventClasses classes;
	protected Set<UUID> foldedElements;
	
	public static final int PERFORMANCEWIDTH = 100;
	public static final int PERFORMANCEHEIGHT = 100;

	public static final int BORDER = 24;
	public static final int LEAFHEIGHT = 50 + 2 * BORDER;
	public int LEAFWIDTH = 80 + 2 * BORDER;
	public int OPERATORHEIGHT = 30;
	public int OPERATORWIDTH = 30;
	public static final int ARROWSIZE = 7;
	public static final int CONFIGSIZE = 3 * ARROWSIZE;
	private static final int POINTERWIDTH = 10;
	public static final String BLOCKABLE = "Blockable";
	public static final String HIDEABLE = "Hideable";
	public static final String ID = "id";
	public static final String Name = "name";

	public ModelLayoutBuilderOld(OCBCModel model) {
		this.model = model;
	}

	public OCBCModel getModel(){
		return model;
	}
	public JGraph getJGraph() {
		GraphModel graphModel = new DefaultGraphModel();
		GraphLayoutCache view = new GraphLayoutCache(graphModel, new ModelCellViewFactory());
		final JGraph graph = new JGraph(graphModel, view);
		graph.setAntiAliased(true);
		graph.setScale(1.0);
		return graph;
	}
	
	public void addNode(String type, String displayName, String ID, Double x, Double y, Double height, Double width){	    
		String cellID = ID;
		
		if(type.equals("ClassCell")){		    
		    classNodeNum ++;
		    //create a class cell with configuring its name and id
		    ClassCell classCell = new ClassCell(displayName);    
		    classCell.setID(cellID);
		    DefaultPort port = new DefaultPort();  
		    classCell.add(port);  
		    GraphConstants.setBounds(classCell.getAttributes(), new Rectangle2D.Double(x,y,width,height));  
		    GraphConstants.setGradientColor(classCell.getAttributes(),Color.black);  
		    GraphConstants.setOpaque(classCell.getAttributes(), true);
		    
		    //create a class node with configuring its name and id
		    ClassNode classNode = new ClassNodeImpl(displayName);
		    classNode.setID(cellID);
		      		 	    
		    classCell.setCorrespondingElement(classNode);
		    classNode.setCorrespondingCell(classCell);
		    
		    model.getGraph().getGraphLayoutCache().insert(classCell); 
		    model.getGraphCells().add(classCell);
		    model.addClass(classNode);
		}
		else if(type.equals("ActivityCell")){
		    activityNodeNum ++;
		    //create a class cell with configuring its name and id
		    ActivityCell activityCell = new ActivityCell(displayName);     
		    activityCell.setID(cellID);	    
		    DefaultPort port = new DefaultPort();  
		    activityCell.add(port);  
		    GraphConstants.setBounds(activityCell.getAttributes(), new Rectangle2D.Double(x,y,width,height));  
		    GraphConstants.setGradientColor(activityCell.getAttributes(),Color.black);  
		    GraphConstants.setOpaque(activityCell.getAttributes(), true);  	
		    
		    //create a class node with configuring its name and id
		    ActivityNode activityNode = new ActivityNodeImpl(displayName);
		    activityNode.setID(cellID);
		    
		    activityCell.setCorrespondingElement(activityNode);
		    activityNode.setCorrespondingCell(activityCell);
		    
		    model.getGraph().getGraphLayoutCache().insert(activityCell); 
		    model.getGraphCells().add(activityCell);
		    model.addActivity(activityNode);
		}
	}
	
	//if transparentCellX = transparentCellY = 0, the transparent cell in at the mide of the sounrce node and the target node
	public void addClassEdge(String ID, String name, String soruceNodeID, String sourceAlwaysCardinality,
			String sourceEventuallyCardinality, String targetNodeID, String targetAlwaysCardinality, 
			String targetEventuallyCardinality, double transparentCellX, double transparentCellY, double height, double width){	     
	    String classEdgeID = ID;
	    MyOriginalCell firstlyFocusedCell = new MyOriginalCell();
	    MyOriginalCell secondlyFocusedCell = new MyOriginalCell();
	    for(MyOriginalCell cell : model.getGraphCells()){
	    	if(cell.getID().toString().equals(soruceNodeID)){
	    		firstlyFocusedCell = cell;
	    	}
	    	else if(cell.getID().toString().equals(targetNodeID)){
	    		secondlyFocusedCell = cell;
	    	}
	    }
	    
	    //create a edge with its source node and cardinality, and target node and cardinality
	    ClassNode sourceNode = (ClassNode) firstlyFocusedCell.getCorrespondingElement();
	    ClassNode targetNode = (ClassNode) secondlyFocusedCell.getCorrespondingElement();
        ClassEdge classEdge = new ClassEdgeImpl(sourceNode, targetNode, new Cardinality(sourceAlwaysCardinality), 
        		new Cardinality(sourceEventuallyCardinality), new Cardinality(targetAlwaysCardinality), new Cardinality(targetEventuallyCardinality));
                  
        classEdge.setID(classEdgeID);
        classEdge.setEdgeName(name);
        model.getAllClassRelations().add(classEdge);                             
        
        if(transparentCellX == 0 && transparentCellY == 0){
        	double firstX = GraphConstants.getBounds(firstlyFocusedCell.getAttributes()).getX();
            double firstY = GraphConstants.getBounds(firstlyFocusedCell.getAttributes()).getY();      	
        	double secondX = GraphConstants.getBounds(secondlyFocusedCell.getAttributes()).getX();
            double secondY = GraphConstants.getBounds(secondlyFocusedCell.getAttributes()).getY();
            transparentCellX = (firstX + secondX)/2;
            transparentCellY = (firstY + secondY)/2;
        }
        	   	         
        //create cells(a class edge corresponds to two edge cells and a transparent node cell) for a edge
        LabelCell transparentCell = new LabelCell(name);     
	    transparentCell.setID(classEdgeID);	    	    
	    GraphConstants.setBounds(transparentCell.getAttributes(), new Rectangle2D.Double(transparentCellX, transparentCellY, width,height));  
	    GraphConstants.setGradientColor(transparentCell.getAttributes(),Color.black);  
	    GraphConstants.setOpaque(transparentCell.getAttributes(), true);  		    
	    DefaultPort port = new DefaultPort();  
	    transparentCell.add(port);  	
	   
	    model.getGraph().getGraphLayoutCache().insert(transparentCell); 
	    model.getGraphCells().add(transparentCell);
	    	    	           
        ClassEdgeCell firstHalfEdge = new ClassEdgeCell(sourceAlwaysCardinality, sourceEventuallyCardinality);  
        firstHalfEdge.displayCardinality();        
        firstHalfEdge.setTarget(transparentCell.getChildAt(0));         
        firstHalfEdge.setSource(firstlyFocusedCell.getChildAt(0));  
        firstHalfEdge.setTargetNode(transparentCell); // for each edge, restore its two end nodes   
        firstHalfEdge.setSourceNode(firstlyFocusedCell);
        firstHalfEdge.setID(classEdgeID);
        firstHalfEdge.setWhichEnd("Source");
        model.getGraph().getGraphLayoutCache().insert(firstHalfEdge); 
        model.getGraphCells().add(firstHalfEdge);       
        
        ClassEdgeCell secondHalfEdge = new ClassEdgeCell(targetAlwaysCardinality, targetEventuallyCardinality);  
        secondHalfEdge.displayCardinality();         
        secondHalfEdge.setTarget(transparentCell.getChildAt(0));
        secondHalfEdge.setSource(secondlyFocusedCell.getChildAt(0)); 
        secondHalfEdge.setTargetNode(transparentCell);// for each edge, restore its two end nodes
        secondHalfEdge.setSourceNode(secondlyFocusedCell); 
        secondHalfEdge.setID(classEdgeID);
        secondHalfEdge.setWhichEnd("Target");
        GraphConstants.setLineBegin(secondHalfEdge.getAttributes(), 4);
        GraphConstants.setBeginSize(secondHalfEdge.getAttributes(), 5);
        GraphConstants.setBeginFill(secondHalfEdge.getAttributes(), true);
        
        model.getGraph().getGraphLayoutCache().insert(secondHalfEdge); 	
        model.getGraphCells().add(secondHalfEdge);
        
        transparentCell.setCorrespondingElement(classEdge);
        firstHalfEdge.setCorrespondingElement(classEdge);
        secondHalfEdge.setCorrespondingElement(classEdge);
        classEdge.setCorrespondingCell(transparentCell);

        targetNode.getRelatedClassNodes().add(sourceNode);
        sourceNode.getRelatedClassNodes().add(targetNode);
	}
	
	//if transparentCellX = transparentCellY = 0, the transparent cell in at the mide of the sounrce node and the target node
	public void addAOCEdge(String ID, String soruceNodeID, String sourceAlwaysCardinality, String sourceEventuallyCardinality,
			String targetNodeID, String targetCardinality,  double transparentCellX, double transparentCellY){		    
	    String AOCEdgeID = ID;
	    MyOriginalCell firstlyFocusedCell = new MyOriginalCell();
	    MyOriginalCell secondlyFocusedCell = new MyOriginalCell();
	    for(MyOriginalCell cell : model.getGraphCells()){
	    	if(cell.getID().toString().equals(soruceNodeID)){
	    		firstlyFocusedCell = cell;
	    	}
	    	else if(cell.getID().toString().equals(targetNodeID)){
	    		secondlyFocusedCell = cell;
	    	}
	    }
	    
	    //create a edge with its source node and cardinality, and target node and cardinality
	    ActivityNode sourceNode;
	    ClassNode targetNode;
	    if(firstlyFocusedCell instanceof ActivityCell)
	    {
	    	sourceNode = (ActivityNode) firstlyFocusedCell.getCorrespondingElement();
	    	targetNode = (ClassNode) secondlyFocusedCell.getCorrespondingElement();
	    }
	    else
	    {
	    	sourceNode = (ActivityNode) secondlyFocusedCell.getCorrespondingElement();
	    	targetNode = (ClassNode) firstlyFocusedCell.getCorrespondingElement();
	    }	

        AOCEdge aOCEdge = new AOCEdgeImpl(null, sourceNode, targetNode, new Cardinality(sourceAlwaysCardinality), 
        		new Cardinality(sourceEventuallyCardinality), new Cardinality(targetCardinality));              
        aOCEdge.setID(AOCEdgeID);
        model.getAllAOCRelations().add(aOCEdge);
        
        if(transparentCellX == 0 && transparentCellY == 0){
        	double firstX = GraphConstants.getBounds(firstlyFocusedCell.getAttributes()).getX();
            double firstY = GraphConstants.getBounds(firstlyFocusedCell.getAttributes()).getY();      	
        	double secondX = GraphConstants.getBounds(secondlyFocusedCell.getAttributes()).getX();
            double secondY = GraphConstants.getBounds(secondlyFocusedCell.getAttributes()).getY();
            transparentCellX = (firstX + secondX)/2;
            transparentCellY = (firstY + secondY)/2;
        }
        
        //create cells(a class edge corresponds to two edge cells and a transparent node cell) for a edge
	    TransparentCell transparentCell = new TransparentCell(null);     
	    transparentCell.setID(AOCEdgeID);	    	    
	    GraphConstants.setBounds(transparentCell.getAttributes(), new Rectangle2D.Double(transparentCellX, transparentCellY, transparentCellSize, transparentCellSize));  
	    GraphConstants.setGradientColor(transparentCell.getAttributes(),Color.black);  
	    GraphConstants.setOpaque(transparentCell.getAttributes(), true);  		    
	    DefaultPort port = new DefaultPort();  
	    transparentCell.add(port);  	
	   
	    model.getGraph().getGraphLayoutCache().insert(transparentCell); 
	    model.getGraphCells().add(transparentCell);
	    	    	           
        AOCEdgeCell firstHalfEdge = new AOCEdgeCell();  
        firstHalfEdge.setSourceCardinalityCharacters(sourceAlwaysCardinality, sourceEventuallyCardinality);
        firstHalfEdge.setWhichEnd("Source");
        firstHalfEdge.displayCardinality();        
        firstHalfEdge.setTarget(transparentCell.getChildAt(0));         
        firstHalfEdge.setSource(firstlyFocusedCell.getChildAt(0));  
        firstHalfEdge.setTargetNode(transparentCell);// for each edge, restore its two end nodes
        firstHalfEdge.setSourceNode(firstlyFocusedCell); 
        firstHalfEdge.setID(AOCEdgeID);
        
       
        GraphConstants.setDashPattern(firstHalfEdge.getAttributes(), AOCEdgeDash);
        model.getGraph().getGraphLayoutCache().insert(firstHalfEdge); 
        model.getGraphCells().add(firstHalfEdge);
        
        AOCEdgeCell secondHalfEdge = new AOCEdgeCell();  
        secondHalfEdge.setTargetCardinalityCharacters(targetCardinality);
        secondHalfEdge.setWhichEnd("Target");
        secondHalfEdge.displayCardinality();         
        secondHalfEdge.setTarget(transparentCell.getChildAt(0));
        secondHalfEdge.setSource(secondlyFocusedCell.getChildAt(0)); 
        secondHalfEdge.setTargetNode(transparentCell);// for each edge, restore its two end nodes
        secondHalfEdge.setSourceNode(secondlyFocusedCell); 
        secondHalfEdge.setID(AOCEdgeID);
       
        GraphConstants.setDashPattern(secondHalfEdge.getAttributes(), AOCEdgeDash);
        model.getGraph().getGraphLayoutCache().insert(secondHalfEdge); 	
        model.getGraphCells().add(secondHalfEdge);
        
        transparentCell.setCorrespondingElement(aOCEdge);
        firstHalfEdge.setCorrespondingElement(aOCEdge);
        secondHalfEdge.setCorrespondingElement(aOCEdge);
        aOCEdge.setCorrespondingCell(transparentCell);
        
        targetNode.getRelatedActivityNodes().add(sourceNode);
        sourceNode.getRelatedClassNodes().add(targetNode);
	}
	
	//if transparentCellX = transparentCellY = 0, the transparent cell in at the mide of the sounrce node and the target node
	public void addCrelRTEdge(String ID, String activityRelationID, String classRelationID,  double transparentCellX, double transparentCellY){	    
	    String crelRTEdgeID = ID;
	    MyOriginalCell firstlyFocusedCell = new MyOriginalCell();
	    MyOriginalCell secondlyFocusedCell = new MyOriginalCell();
	    
	    for(ActivityEdge activityEdge : model.getAllActivityRelations()){
	    	if(activityEdge.getID().toString().equals(activityRelationID)){
	    		firstlyFocusedCell = activityEdge.getCorrespondingCell();
	    	}
	    }
	    
	    for(ClassEdge classEdge : model.getAllClassRelations()){
	    	if(classEdge.getID().toString().equals(classRelationID)){
	    		secondlyFocusedCell = classEdge.getCorrespondingCell();
	    	}
	    }
	    
	    // if transparentCellX = transparentCellY = 0
        if(transparentCellX == 0 && transparentCellY == 0){
        	double firstX = GraphConstants.getBounds(firstlyFocusedCell.getAttributes()).getX();
            double firstY = GraphConstants.getBounds(firstlyFocusedCell.getAttributes()).getY();      	
        	double secondX = GraphConstants.getBounds(secondlyFocusedCell.getAttributes()).getX();
            double secondY = GraphConstants.getBounds(secondlyFocusedCell.getAttributes()).getY();
            transparentCellX = (firstX + secondX)/2;
            transparentCellY = (firstY + secondY)/2;
        }
        
        //create cells(a class edge corresponds to two edge cells and a transparent node cell) for a edge
	    TransparentCell transparentCell = new TransparentCell(null);     
	    transparentCell.setID(crelRTEdgeID);	    	    
	    GraphConstants.setBounds(transparentCell.getAttributes(), new Rectangle2D.Double(transparentCellX, transparentCellY, transparentCellSize, transparentCellSize));  
	    GraphConstants.setGradientColor(transparentCell.getAttributes(),Color.black);  
	    GraphConstants.setOpaque(transparentCell.getAttributes(), true);  		    
	    DefaultPort port = new DefaultPort();  
	    transparentCell.add(port);  		   
	    model.getGraph().getGraphLayoutCache().insert(transparentCell); 
	    model.getGraphCells().add(transparentCell);
	    	    	           
        CrelEdgeCell firstHalfEdge = new CrelEdgeCell();        
        firstHalfEdge.setTarget(transparentCell.getChildAt(0));         
        firstHalfEdge.setSource(firstlyFocusedCell.getChildAt(0));  
        firstHalfEdge.setTargetNode(transparentCell);// for each edge, restore its two end nodes
        firstHalfEdge.setSourceNode(firstlyFocusedCell); 
        firstHalfEdge.setID(crelRTEdgeID);
        firstHalfEdge.setWhichEnd("Source");       
        GraphConstants.setDashPattern(firstHalfEdge.getAttributes(), CrelEdgeDash);

        
        CrelEdgeCell secondHalfEdge = new CrelEdgeCell();          
        secondHalfEdge.setTarget(transparentCell.getChildAt(0));
        secondHalfEdge.setSource(secondlyFocusedCell.getChildAt(0)); 
        secondHalfEdge.setTargetNode(transparentCell);// for each edge, restore its two end nodes
        secondHalfEdge.setSourceNode(secondlyFocusedCell); 
        secondHalfEdge.setID(crelRTEdgeID);
        secondHalfEdge.setWhichEnd("Target");
        GraphConstants.setDashPattern(secondHalfEdge.getAttributes(), CrelEdgeDash);
          
    	ActivityEdge activityEdge = (ActivityEdge) firstlyFocusedCell.getCorrespondingElement();
    	ClassEdge classEdge = (ClassEdge) secondlyFocusedCell.getCorrespondingElement();	    	
    	CrelRTEdge crelRTEdge = new CrelRTEdgeImpl(null, activityEdge, classEdge);
    	
    	//add the crelRTEdge into the corresponding set of activity edge and class edge
    	activityEdge.getRelatedCrelRTRelations().add(crelRTEdge);
    	classEdge.getRelatedCrelRTRelations().add(crelRTEdge);
    	
    	crelRTEdge.setID(crelRTEdgeID);
        transparentCell.setCorrespondingElement(crelRTEdge);
        firstHalfEdge.setCorrespondingElement(crelRTEdge);
        secondHalfEdge.setCorrespondingElement(crelRTEdge);
        crelRTEdge.setCorrespondingCell(transparentCell);
    	model.getAllCrelRTRelations().add(crelRTEdge);
    	firstHalfEdge.setOCRT("RT");
    	secondHalfEdge.setOCRT("RT");
	   	    	    
        //if the selected transparent corresponds two activity edges (i.e., a combined edge)
        if(firstlyFocusedCell.getCorrespondingSecondElement() != null && firstlyFocusedCell.getCorrespondingElement() instanceof ActivityEdge){
    	    String crelEdgeIDSecond = UUID.randomUUID().toString();
        	ActivityEdge activityEdgeSecond = (ActivityEdge) firstlyFocusedCell.getCorrespondingSecondElement();	    	
        	CrelRTEdge crelRTEdgeSecond = new CrelRTEdgeImpl(null, activityEdgeSecond, classEdge);
	    	activityEdgeSecond.getRelatedCrelRTRelations().add(crelRTEdgeSecond);
	    	crelRTEdgeSecond.setID(crelEdgeIDSecond);
	    	
            transparentCell.setCorrespondingSecondElement(crelRTEdgeSecond);
            firstHalfEdge.setCorrespondingSecondElement(crelRTEdgeSecond);
            secondHalfEdge.setCorrespondingSecondElement(crelRTEdgeSecond);
            transparentCell.setSecondID(crelEdgeIDSecond);
            firstHalfEdge.setSecondID(crelEdgeIDSecond);
            secondHalfEdge.setSecondID(crelEdgeIDSecond);
            crelRTEdgeSecond.setCorrespondingCell(transparentCell);
            model.getAllCrelRTRelations().add(crelRTEdgeSecond);		    	
        }
    	
        model.getGraph().getGraphLayoutCache().insert(firstHalfEdge); 
        model.getGraphCells().add(firstHalfEdge);
        model.getGraph().getGraphLayoutCache().insert(secondHalfEdge); 	
        model.getGraphCells().add(secondHalfEdge);
	}
	
	//if transparentCellX = transparentCellY = 0, the transparent cell in at the mide of the sounrce node and the target node
	public void addCrelOCEdge(String ID, String activityRelationID, String classNodeID,  double transparentCellX, double transparentCellY){	    
	    String crelOCEdgeID = ID;
	    MyOriginalCell firstlyFocusedCell = new MyOriginalCell();
	    MyOriginalCell secondlyFocusedCell = new MyOriginalCell();	    
	    System.out.println("activityRelationID: " + activityRelationID);
	    for(ActivityEdge activityEdge : model.getAllActivityRelations()){
	    	System.out.println("activityEdge.getID().toString(): " + activityEdge.getID().toString());
	    	if(activityEdge.getID().toString().equals(activityRelationID)){
	    		if(activityEdge.getCorrespondingCell() != null){
	    			firstlyFocusedCell = activityEdge.getCorrespondingCell();
	    		}	    		
	    	}
	    }
	    
	    for(ClassNode classNode : model.getAllClasses()){
	    	if(classNode.getID().toString().equals(classNodeID)){
	    		if(classNode.getCorrespondingCell() != null){
	    			secondlyFocusedCell = classNode.getCorrespondingCell();
	    		}	    		
	    	}
	    }
	    
	    // if transparentCellX = transparentCellY = 0
        if(transparentCellX == 0 && transparentCellY == 0){
        	double firstX = GraphConstants.getBounds(firstlyFocusedCell.getAttributes()).getX();
            double firstY = GraphConstants.getBounds(firstlyFocusedCell.getAttributes()).getY();      	
        	double secondX = GraphConstants.getBounds(secondlyFocusedCell.getAttributes()).getX();
            double secondY = GraphConstants.getBounds(secondlyFocusedCell.getAttributes()).getY();
            transparentCellX = (firstX + secondX)/2;
            transparentCellY = (firstY + secondY)/2;
        }
        
        //create cells(a class edge corresponds to two edge cells and a transparent node cell) for a edge
	    TransparentCell transparentCell = new TransparentCell(null);     
	    transparentCell.setID(crelOCEdgeID);	    	    
	    GraphConstants.setBounds(transparentCell.getAttributes(), new Rectangle2D.Double(transparentCellX, transparentCellY, transparentCellSize, transparentCellSize));  
	    GraphConstants.setGradientColor(transparentCell.getAttributes(),Color.black);  
	    GraphConstants.setOpaque(transparentCell.getAttributes(), true);  		    
	    DefaultPort port = new DefaultPort();  
	    transparentCell.add(port);  		   
	    model.getGraph().getGraphLayoutCache().insert(transparentCell); 
	    model.getGraphCells().add(transparentCell);
	    	    	           
        CrelEdgeCell firstHalfEdge = new CrelEdgeCell();        
        firstHalfEdge.setTarget(transparentCell.getChildAt(0));         
        firstHalfEdge.setSource(firstlyFocusedCell.getChildAt(0));  
        firstHalfEdge.setTargetNode(transparentCell);// for each edge, restore its two end nodes
        firstHalfEdge.setSourceNode(firstlyFocusedCell); 
        firstHalfEdge.setID(crelOCEdgeID);
        firstHalfEdge.setWhichEnd("Source");       
        GraphConstants.setDashPattern(firstHalfEdge.getAttributes(), CrelEdgeDash);

        
        CrelEdgeCell secondHalfEdge = new CrelEdgeCell();          
        secondHalfEdge.setTarget(transparentCell.getChildAt(0));
        secondHalfEdge.setSource(secondlyFocusedCell.getChildAt(0)); 
        secondHalfEdge.setTargetNode(transparentCell);// for each edge, restore its two end nodes
        secondHalfEdge.setSourceNode(secondlyFocusedCell); 
        secondHalfEdge.setID(crelOCEdgeID);
        secondHalfEdge.setWhichEnd("Target");
        GraphConstants.setDashPattern(secondHalfEdge.getAttributes(), CrelEdgeDash);
        
    	ActivityEdge activityEdge = (ActivityEdge) firstlyFocusedCell.getCorrespondingElement();
    	ClassNode classNode = (ClassNode) secondlyFocusedCell.getCorrespondingElement();	    	
    	CrelOCEdge crelOCEdge = new CrelOCEdgeImpl(null, activityEdge, classNode);
    	
    	//add the crelOCEdge into the corresponding set of activity edge and class node
    	activityEdge.getRelatedCrelOCRelations().add(crelOCEdge);
//    	classNode.getRelatedCrelOCEdges().add(crelOCEdge);
    	
    	crelOCEdge.setID(crelOCEdgeID);
        transparentCell.setCorrespondingElement(crelOCEdge);
        firstHalfEdge.setCorrespondingElement(crelOCEdge);
        secondHalfEdge.setCorrespondingElement(crelOCEdge);
        crelOCEdge.setCorrespondingCell(transparentCell);
    	model.getAllCrelOCRelations().add(crelOCEdge);
    	firstHalfEdge.setOCRT("OC");
    	secondHalfEdge.setOCRT("OC");
	   	    	    
        //if the selected transparent corresponds two activity edges (i.e., a combined edge)
        if(firstlyFocusedCell.getCorrespondingSecondElement() != null && firstlyFocusedCell.getCorrespondingElement() instanceof ActivityEdge){
    	    String crelEdgeIDSecond = UUID.randomUUID().toString();
        	ActivityEdge activityEdgeSecond = (ActivityEdge) firstlyFocusedCell.getCorrespondingSecondElement();	    	
	    	CrelOCEdge crelOCEdgeSecond = new CrelOCEdgeImpl(null, activityEdgeSecond, classNode);
	    	activityEdgeSecond.getRelatedCrelOCRelations().add(crelOCEdgeSecond);
	    	crelOCEdgeSecond.setID(crelEdgeIDSecond);
	    	
            transparentCell.setCorrespondingSecondElement(crelOCEdgeSecond);
            firstHalfEdge.setCorrespondingSecondElement(crelOCEdgeSecond);
            secondHalfEdge.setCorrespondingSecondElement(crelOCEdgeSecond);
            transparentCell.setSecondID(crelEdgeIDSecond);
            firstHalfEdge.setSecondID(crelEdgeIDSecond);
            secondHalfEdge.setSecondID(crelEdgeIDSecond);
            crelOCEdgeSecond.setCorrespondingCell(transparentCell);
            model.getAllCrelOCRelations().add(crelOCEdgeSecond);		    	
        }
        
        model.getGraph().getGraphLayoutCache().insert(firstHalfEdge); 
        model.getGraphCells().add(firstHalfEdge);
        model.getGraph().getGraphLayoutCache().insert(secondHalfEdge); 	
        model.getGraphCells().add(secondHalfEdge);
	}
	
	//if transparentCellX = transparentCellY = 0, the transparent cell in around the mide of the sounrce node and the target node
	public void addActivityEdge(String ID, String sourceNodeID,
			String targetNodeID, String cardinalityType,  double transparentCellX, double transparentCellY){	
	    String activityEdgeID  = ID;
//	    String[] typeComplex = null;
//	    //make it compatible with the old version: old version ActivityEdge_Response vs new version ActivityEdgeResponse
//	    //if true, it is new version
//	    if(cardinalityType.split("_Activity").length == 2 || cardinalityType.split("_").length == 1){
//	    	typeComplex = cardinalityType.split("_");
//	    }
//	    else  {
//	    	typeComplex = (cardinalityType.split("_")[0] + cardinalityType.split("_")[1]).split("_");//remove the "-" in old version
//	    	cardinalityType = typeComplex[0];
//	    }
	    	
	    String[] typeComplex = cardinalityType.split("-");
	    cardinalityType = typeComplex[0];
	    
	    System.out.println("ID: " + ID);
	    System.out.println("transparentCellX: " + transparentCellX);
	    System.out.println("transparentCellY: " + transparentCellY);
	    System.out.println("cardinalityType: " + cardinalityType);
	    System.out.println("sourceNodeID: " + sourceNodeID);
	    System.out.println("targetNodeID: " + targetNodeID);
	    
	    MyOriginalCell firstlyFocusedCell = new MyOriginalCell();
	    MyOriginalCell secondlyFocusedCell = new MyOriginalCell();
	    for(MyOriginalCell cell : model.getGraphCells()){
	    	if(cell.getID().toString().equals(sourceNodeID)){
	    		firstlyFocusedCell = cell;
	    		System.out.println("firstlyFocusedCell = cell");
	    	}
	    	else if(cell.getID().toString().equals(targetNodeID)){
	    		secondlyFocusedCell = cell;
	    		System.out.println("secondlyFocusedCell = cell");
	    	}
	    }
	    
	    //create a edge with its source node and cardinality, and target node and cardinality
	    ActivityNode referenceNode = (ActivityNode) firstlyFocusedCell.getCorrespondingElement();
	    ActivityNode targetNode = (ActivityNode) secondlyFocusedCell.getCorrespondingElement();

//        ActivityEdge activityEdge = new ActivityEdgeImpl(referenceNode, targetNode, typeComplex[0]);       
        ActivityEdge activityEdge = new ActivityEdgeImpl(referenceNode, targetNode, cardinalityType);    
        
        activityEdge.setID(activityEdgeID);
        model.getAllActivityRelations().add(activityEdge);
        
	    // if transparentCellX = transparentCellY = 0
        if(transparentCellX == 0 && transparentCellY == 0){
        	double firstX = GraphConstants.getBounds(firstlyFocusedCell.getAttributes()).getX();
            double firstY = GraphConstants.getBounds(firstlyFocusedCell.getAttributes()).getY();      	
        	double secondX = GraphConstants.getBounds(secondlyFocusedCell.getAttributes()).getX();
            double secondY = GraphConstants.getBounds(secondlyFocusedCell.getAttributes()).getY();

            transparentCellX = (firstX + secondX)/2;
            transparentCellY = (firstY + secondY)/2;
            
            //judge if there exist an edge between the two nodes
            for(ActivityEdge existActivityEdge : model.getAllActivityRelations()){
            	String existSourceNodeId = existActivityEdge.getReferenceNode().getID().toString();
            	String existTargetNodeId = existActivityEdge.getTargetNode().getID().toString();
                if(existSourceNodeId.equals(targetNodeID) && existTargetNodeId.equals(sourceNodeID)){
                    transparentCellX = (firstX + secondX)/2;
                    transparentCellY = (firstY + secondY)/2;
                    break;
                }
            }           
        }
                
        //create cells(a activity edge corresponds to two edge cells and a transparent node cell) for a edge
	    TransparentCell transparentCell = new TransparentCell(null);     
	    transparentCell.setID(activityEdgeID);	    	    
	    GraphConstants.setBounds(transparentCell.getAttributes(), new Rectangle2D.Double(transparentCellX, transparentCellY, transparentCellSize, transparentCellSize));
	    GraphConstants.setGradientColor(transparentCell.getAttributes(),Color.black);  
	    GraphConstants.setOpaque(transparentCell.getAttributes(), true);  		    
	    DefaultPort port = new DefaultPort();  
	    transparentCell.add(port);  	
	   
	    model.getGraph().getGraphLayoutCache().insert(transparentCell); 
	    model.getGraphCells().add(transparentCell);
	    	    	           
        ActivityEdgeCell firstHalfEdge = new ActivityEdgeCell();  
      
        firstHalfEdge.setSource(transparentCell.getChildAt(0));         
        firstHalfEdge.setTarget(firstlyFocusedCell.getChildAt(0));  
        firstHalfEdge.setSourceNode(transparentCell);// for each edge, restore its two end nodes
        firstHalfEdge.setTargetNode(firstlyFocusedCell); 
        firstHalfEdge.setID(activityEdgeID);
//        firstHalfEdge.setWhichEnd("Source");
       
        model.getGraph().getGraphLayoutCache().insert(firstHalfEdge); 
        model.getGraphCells().add(firstHalfEdge);
        
        ActivityEdgeCell secondHalfEdge = new ActivityEdgeCell();  
//        secondHalfEdge.displayCardinality();         
        secondHalfEdge.setSource(transparentCell.getChildAt(0));
        secondHalfEdge.setTarget(secondlyFocusedCell.getChildAt(0)); 
        secondHalfEdge.setSourceNode(transparentCell);// for each edge, restore its two end nodes
        secondHalfEdge.setTargetNode(secondlyFocusedCell); 
        secondHalfEdge.setID(activityEdgeID);
//        secondHalfEdge.setWhichEnd("Target");
        
        model.getGraph().getGraphLayoutCache().insert(secondHalfEdge); 	
        model.getGraphCells().add(secondHalfEdge);
        
        int startShape;
        int endShape;
               	
        System.out.printf("\n after switch");
        startShape =6; endShape = 6;
        
        GraphConstants.setLineEnd(firstHalfEdge.getAttributes(), startShape);
        GraphConstants.setLineEnd(secondHalfEdge.getAttributes(), endShape);
        
        GraphConstants.setEndSize(firstHalfEdge.getAttributes(), 6);
        GraphConstants.setEndSize(secondHalfEdge.getAttributes(), 6);
        GraphConstants.setEndFill(firstHalfEdge.getAttributes(), true);
        GraphConstants.setEndFill(secondHalfEdge.getAttributes(), true);
        
        transparentCell.setCorrespondingElement(activityEdge);
        firstHalfEdge.setCorrespondingElement(activityEdge);
        secondHalfEdge.setCorrespondingElement(activityEdge);
        activityEdge.setCorrespondingCell(transparentCell); 
        
        targetNode.getRelatedActivityNodes().add(referenceNode);
        referenceNode.getRelatedActivityNodes().add(targetNode);
        
        //if the edge is a combination edge, add the second edge
        if(typeComplex.length == 2){
//        	String activityEdgeIDSecond  = UUID.randomUUID().toString();
        	String activityEdgeIDSecond  = activityEdgeID + "-2";       	
        	ActivityEdge activityEdgeSecond = new ActivityEdgeImpl(targetNode, referenceNode, typeComplex[1]);              
        	activityEdgeSecond.setID(activityEdgeIDSecond);         
            transparentCell.setCorrespondingSecondElement(activityEdgeSecond);
            transparentCell.setSecondID(activityEdgeIDSecond);  
            //if there is a second edge, the secondHalfEdge corresponds to the second edge
            secondHalfEdge.setCorrespondingElement(activityEdgeSecond);  
            secondHalfEdge.setID(activityEdgeIDSecond);
            activityEdgeSecond.setCorrespondingCell(transparentCell);
            model.getAllActivityRelations().add(activityEdgeSecond);
        }
	}
	
	/**
	 * draw a class node, add the corresponding cell into the graph and add the corresponding node into ClassSet
	 */
	public void drawClassNode(int x, int y){			    
	    String classID  = UUID.randomUUID().toString();
	    classNodeNum ++;
	    //create a class cell with configuring its name and id
	    ClassCell classCell = new ClassCell(new String("Class"+classNodeNum));     
	    classCell.setID(classID);
	    DefaultPort port = new DefaultPort();  
	    classCell.add(port);  
	    GraphConstants.setBounds(classCell.getAttributes(), new Rectangle2D.Double(x,y,65,55));  
	    GraphConstants.setGradientColor(classCell.getAttributes(),Color.black);  
	    GraphConstants.setOpaque(classCell.getAttributes(), true);
	    
	    //create a class node with configuring its name and id
	    ClassNode classNode = new ClassNodeImpl(new String("Class"+classNodeNum));
	    classNode.setID(classID);
	      		 	    
	    classCell.setCorrespondingElement(classNode);
	    classNode.setCorrespondingCell(classCell);
	    
	    model.getGraph().getGraphLayoutCache().insert(classCell); 
	    model.getGraphCells().add(classCell);
	    model.addClass(classNode);
	}
	
	public void drawActivityNode(int x, int y){	    
	    String activityID  = UUID.randomUUID().toString();
	    activityNodeNum ++;
	    //create a class cell with configuring its name and id
	    ActivityCell activityCell = new ActivityCell(new String("Activity" + activityNodeNum));     
	    activityCell.setID(activityID);	    
	    DefaultPort port = new DefaultPort();  
	    activityCell.add(port);  
	    GraphConstants.setBounds(activityCell.getAttributes(), new Rectangle2D.Double(x,y,65,55));  
	    GraphConstants.setGradientColor(activityCell.getAttributes(),Color.black);  
	    GraphConstants.setOpaque(activityCell.getAttributes(), true);  	
	    
	    //create a class node with configuring its name and id
	    ActivityNode activityNode = new ActivityNodeImpl(new String("Activity" + classNodeNum));
	    activityNode.setID(activityID);
	    
	    activityCell.setCorrespondingElement(activityNode);
	    activityNode.setCorrespondingCell(activityCell);
	    
	    model.getGraph().getGraphLayoutCache().insert(activityCell); 
	    model.getGraphCells().add(activityCell);
	    model.addActivity(activityNode);
	}
	
	/**
	 * draw a class edge, add the corresponding cells(a class edge corresponds to two edge cells and a transparent node cell) into the graph and add the corresponding node into ClassSet
	 */
	public void drawClassEdge(MyOriginalCell firstlyFocusedCell, int firstX, int firstY, MyOriginalCell secondlyFocusedCell, int  secondX, int secondY){
	    String classEdgeID  = UUID.randomUUID().toString();
	    //create a edge with its source node and cardinality, and target node and cardinality
	    ClassNode sourceNode = (ClassNode) firstlyFocusedCell.getCorrespondingElement();
	    ClassNode targetNode = (ClassNode) secondlyFocusedCell.getCorrespondingElement();     
        Cardinality alwaysCardinality = new Cardinality("*");
        Cardinality eventuallyCardinality = new Cardinality("*");
        ClassEdge classEdge = new ClassEdgeImpl(sourceNode, targetNode, alwaysCardinality, eventuallyCardinality, 
        		alwaysCardinality, eventuallyCardinality);
        classEdge.setID(classEdgeID);
        model.getAllClassRelations().add(classEdge);
        
        //create cells(a class edge corresponds to two edge cells and a transparent node cell) for a edge
//	    TransparentCell transparentCell = new TransparentCell(null); 
        LabelCell transparentCell = new LabelCell("edge name"); 
	    transparentCell.setID(classEdgeID);	    	    
	    GraphConstants.setBounds(transparentCell.getAttributes(), new Rectangle2D.Double((firstX + secondX)/2,(firstY + secondY)/2,70,50));  
	    GraphConstants.setGradientColor(transparentCell.getAttributes(),Color.black);  
	    GraphConstants.setOpaque(transparentCell.getAttributes(), true);  
	    DefaultPort port = new DefaultPort();  
	    transparentCell.add(port);  	
	   	    	    	           
        ClassEdgeCell firstHalfEdge = new ClassEdgeCell("*", "*");  
        firstHalfEdge.setWhichEnd("Source");
        firstHalfEdge.displayCardinality();        
        firstHalfEdge.setTarget(transparentCell.getChildAt(0));         
        firstHalfEdge.setSource(firstlyFocusedCell.getChildAt(0));  
        firstHalfEdge.setTargetNode(transparentCell); // for each edge, restore its two end nodes   
        firstHalfEdge.setSourceNode(firstlyFocusedCell);
        firstHalfEdge.setID(classEdgeID);
       
       
        ClassEdgeCell secondHalfEdge = new ClassEdgeCell("*", "*");  
        secondHalfEdge.setWhichEnd("Target");
        secondHalfEdge.displayCardinality();         
        secondHalfEdge.setTarget(transparentCell.getChildAt(0));
        secondHalfEdge.setSource(secondlyFocusedCell.getChildAt(0)); 
        secondHalfEdge.setTargetNode(transparentCell);// for each edge, restore its two end nodes
        secondHalfEdge.setSourceNode(secondlyFocusedCell); 
        secondHalfEdge.setID(classEdgeID);
        GraphConstants.setLineBegin(secondHalfEdge.getAttributes(), 4);
        GraphConstants.setBeginSize(secondHalfEdge.getAttributes(), 5);
        GraphConstants.setBeginFill(secondHalfEdge.getAttributes(), true);

        transparentCell.setCorrespondingElement(classEdge);
        firstHalfEdge.setCorrespondingElement(classEdge);
        secondHalfEdge.setCorrespondingElement(classEdge);
        classEdge.setCorrespondingCell(transparentCell);
        
	    model.getGraph().getGraphLayoutCache().insert(transparentCell); 
	    model.getGraphCells().add(transparentCell);
        model.getGraph().getGraphLayoutCache().insert(firstHalfEdge); 
        model.getGraphCells().add(firstHalfEdge);
        model.getGraph().getGraphLayoutCache().insert(secondHalfEdge); 	
        model.getGraphCells().add(secondHalfEdge);
        
        targetNode.getRelatedClassNodes().add(sourceNode);
        sourceNode.getRelatedClassNodes().add(targetNode);
	}
	
	/**
	 * draw a AOC edge, add the corresponding cells(a AOC edge corresponds to two edge cells and a transparent node cell) into the graph and add the corresponding node into ClassSet
	 */
	public void drawAOCEdge(MyOriginalCell firstlyFocusedCell, int firstX, int firstY, MyOriginalCell secondlyFocusedCell, int  secondX, int secondY){
	    String AOCEdgeID  = UUID.randomUUID().toString();
	    //create a edge with its source node and cardinality, and target node and cardinality
	    ActivityNode sourceNode;
	    ClassNode targetNode;
	    if(firstlyFocusedCell instanceof ActivityCell)
	    {
	    	sourceNode = (ActivityNode) firstlyFocusedCell.getCorrespondingElement();
	    	targetNode = (ClassNode) secondlyFocusedCell.getCorrespondingElement();
	    }
	    else
	    {
	    	sourceNode = (ActivityNode) secondlyFocusedCell.getCorrespondingElement();
	    	targetNode = (ClassNode) firstlyFocusedCell.getCorrespondingElement();
	    }	
	    Cardinality sourceAlwaysCardinality = new Cardinality("*");
	    Cardinality sourceEventuallyCardinality = new Cardinality("*");
	    Cardinality targetCardinality = new Cardinality("*");
        AOCEdge aOCEdge = new AOCEdgeImpl(null, sourceNode, targetNode, sourceAlwaysCardinality, sourceEventuallyCardinality, 
        		targetCardinality);              
        aOCEdge.setID(AOCEdgeID);
        model.getAllAOCRelations().add(aOCEdge);
        
       
        
        //create cells(a class edge corresponds to two edge cells and a transparent node cell) for a edge
	    TransparentCell transparentCell = new TransparentCell(null); 	
	    transparentCell.setID(AOCEdgeID);	    	    
	    GraphConstants.setBounds(transparentCell.getAttributes(), new Rectangle2D.Double((firstX + secondX)/2,(firstY + secondY)/2,transparentCellSize,transparentCellSize));  
	    GraphConstants.setGradientColor(transparentCell.getAttributes(),Color.black);  
	    GraphConstants.setOpaque(transparentCell.getAttributes(), true);  		    
	    DefaultPort port = new DefaultPort();  
	    transparentCell.add(port);  	
	   
	    model.getGraph().getGraphLayoutCache().insert(transparentCell); 
	    model.getGraphCells().add(transparentCell);
	    	    	           
        AOCEdgeCell firstHalfEdge = new AOCEdgeCell();  
        firstHalfEdge.setSourceCardinalityCharacters("*", "*");
        firstHalfEdge.setWhichEnd("Source");
        firstHalfEdge.displayCardinality();        
        firstHalfEdge.setTarget(transparentCell.getChildAt(0));         
        firstHalfEdge.setSource(firstlyFocusedCell.getChildAt(0));  
        firstHalfEdge.setTargetNode(transparentCell);// for each edge, restore its two end nodes
        firstHalfEdge.setSourceNode(firstlyFocusedCell); 
        firstHalfEdge.setID(AOCEdgeID);
       
       
        GraphConstants.setDashPattern(firstHalfEdge.getAttributes(), AOCEdgeDash);
        model.getGraph().getGraphLayoutCache().insert(firstHalfEdge); 
        model.getGraphCells().add(firstHalfEdge);
        
        AOCEdgeCell secondHalfEdge = new AOCEdgeCell(); 
        secondHalfEdge.setTargetCardinalityCharacters("*");
        secondHalfEdge.setWhichEnd("Target");
        secondHalfEdge.displayCardinality();         
        secondHalfEdge.setTarget(transparentCell.getChildAt(0));
        secondHalfEdge.setSource(secondlyFocusedCell.getChildAt(0)); 
        secondHalfEdge.setTargetNode(transparentCell);// for each edge, restore its two end nodes
        secondHalfEdge.setSourceNode(secondlyFocusedCell); 
        secondHalfEdge.setID(AOCEdgeID);
        
        GraphConstants.setDashPattern(secondHalfEdge.getAttributes(), AOCEdgeDash);
        model.getGraph().getGraphLayoutCache().insert(secondHalfEdge); 	
        model.getGraphCells().add(secondHalfEdge);
        
        transparentCell.setCorrespondingElement(aOCEdge);
        firstHalfEdge.setCorrespondingElement(aOCEdge);
        secondHalfEdge.setCorrespondingElement(aOCEdge);
        aOCEdge.setCorrespondingCell(transparentCell);
        
        targetNode.getRelatedActivityNodes().add(sourceNode);
        sourceNode.getRelatedClassNodes().add(targetNode);
	}
	
	public void drawCrelEdge(MyOriginalCell firstlyFocusedCell, int firstX, int firstY, MyOriginalCell secondlyFocusedCell, int  secondX, int secondY){
	    String crelEdgeID  = UUID.randomUUID().toString();
        //create cells(a class edge corresponds to two edge cells and a transparent node cell) for a edge
	    TransparentCell transparentCell = new TransparentCell(null);     
	    transparentCell.setID(crelEdgeID);	    	    
	    GraphConstants.setBounds(transparentCell.getAttributes(), new Rectangle2D.Double((firstX + secondX)/2,(firstY + secondY)/2,transparentCellSize,transparentCellSize));  
	    GraphConstants.setGradientColor(transparentCell.getAttributes(),Color.black);  
	    GraphConstants.setOpaque(transparentCell.getAttributes(), true);  		    
	    DefaultPort port = new DefaultPort();  
	    transparentCell.add(port);  		   
	    model.getGraph().getGraphLayoutCache().insert(transparentCell); 
	    model.getGraphCells().add(transparentCell);
	    	    	           
        CrelEdgeCell firstHalfEdge = new CrelEdgeCell();        
        firstHalfEdge.setTarget(transparentCell.getChildAt(0));         
        firstHalfEdge.setSource(firstlyFocusedCell.getChildAt(0));  
        firstHalfEdge.setTargetNode(transparentCell);// for each edge, restore its two end nodes
        firstHalfEdge.setSourceNode(firstlyFocusedCell); 
        firstHalfEdge.setID(crelEdgeID);
        firstHalfEdge.setWhichEnd("Source");       
        GraphConstants.setDashPattern(firstHalfEdge.getAttributes(), CrelEdgeDash);

        
        CrelEdgeCell secondHalfEdge = new CrelEdgeCell();          
        secondHalfEdge.setTarget(transparentCell.getChildAt(0));
        secondHalfEdge.setSource(secondlyFocusedCell.getChildAt(0)); 
        secondHalfEdge.setTargetNode(transparentCell);// for each edge, restore its two end nodes
        secondHalfEdge.setSourceNode(secondlyFocusedCell); 
        secondHalfEdge.setID(crelEdgeID);
        secondHalfEdge.setWhichEnd("Target");
        GraphConstants.setDashPattern(secondHalfEdge.getAttributes(), CrelEdgeDash);
              
	    //create a edge with its source node and cardinality, and target node and cardinality
	    if(secondlyFocusedCell.getCorrespondingElement() instanceof ClassNode)
	    {
	    	
	    	ActivityEdge activityEdge = (ActivityEdge) firstlyFocusedCell.getCorrespondingElement();
	    	ClassNode classNode = (ClassNode) secondlyFocusedCell.getCorrespondingElement();	    	
	    	CrelOCEdge crelOCEdge = new CrelOCEdgeImpl(null, activityEdge, classNode);
	    	
	    	//add the crelOCEdge into the corresponding set of activity edge and class node
	    	activityEdge.getRelatedCrelOCRelations().add(crelOCEdge);
//	    	classNode.getRelatedCrelOCEdges().add(crelOCEdge);
	    	
	    	crelOCEdge.setID(crelEdgeID);
	        transparentCell.setCorrespondingElement(crelOCEdge);
	        firstHalfEdge.setCorrespondingElement(crelOCEdge);
	        secondHalfEdge.setCorrespondingElement(crelOCEdge);
	        crelOCEdge.setCorrespondingCell(transparentCell);
	    	model.getAllCrelOCRelations().add(crelOCEdge);
	    	firstHalfEdge.setOCRT("OC");
	    	secondHalfEdge.setOCRT("OC");
	    	
	        //if the selected transparent corresponds two activity edges (i.e., a combined edge)
	        if(firstlyFocusedCell.getCorrespondingSecondElement() != null && firstlyFocusedCell.getCorrespondingElement() instanceof ActivityEdge){
	    	    String crelEdgeIDSecond = UUID.randomUUID().toString(); 
	        	ActivityEdge activityEdgeSecond = (ActivityEdge) firstlyFocusedCell.getCorrespondingSecondElement();	    	
		    	CrelOCEdge crelOCEdgeSecond = new CrelOCEdgeImpl(null, activityEdgeSecond, classNode);
		    	activityEdgeSecond.getRelatedCrelOCRelations().add(crelOCEdgeSecond);
		    	crelOCEdgeSecond.setID(crelEdgeIDSecond);
		    	
	            transparentCell.setCorrespondingSecondElement(crelOCEdgeSecond);
	            firstHalfEdge.setCorrespondingSecondElement(crelOCEdgeSecond);
	            secondHalfEdge.setCorrespondingSecondElement(crelOCEdgeSecond);
	            transparentCell.setSecondID(crelEdgeIDSecond);
	            firstHalfEdge.setSecondID(crelEdgeIDSecond);
	            secondHalfEdge.setSecondID(crelEdgeIDSecond);
	            crelOCEdgeSecond.setCorrespondingCell(transparentCell);
	            model.getAllCrelOCRelations().add(crelOCEdgeSecond);		    	
	        }
	    }
	    else
	    {
	    	ActivityEdge activityEdge = (ActivityEdge) firstlyFocusedCell.getCorrespondingElement();
	    	ClassEdge classEdge = (ClassEdge) secondlyFocusedCell.getCorrespondingElement();	    	
	    	CrelRTEdge crelRTEdge = new CrelRTEdgeImpl(null, activityEdge, classEdge);
	    	
	    	//add the crelRTEdge into the corresponding set of activity edge and class edge
	    	activityEdge.getRelatedCrelRTRelations().add(crelRTEdge);
	    	classEdge.getRelatedCrelRTRelations().add(crelRTEdge);
	    	
	    	
	    	crelRTEdge.setID(crelEdgeID);
	        transparentCell.setCorrespondingElement(crelRTEdge);
	        firstHalfEdge.setCorrespondingElement(crelRTEdge);
	        secondHalfEdge.setCorrespondingElement(crelRTEdge);
	        crelRTEdge.setCorrespondingCell(transparentCell);
	    	model.getAllCrelRTRelations().add(crelRTEdge);
	    	firstHalfEdge.setOCRT("RT");
	    	secondHalfEdge.setOCRT("RT");
	    	
	        //if the selected transparent corresponds two activity edges (i.e., a combined edge)
	        if(firstlyFocusedCell.getCorrespondingSecondElement() != null && firstlyFocusedCell.getCorrespondingElement() instanceof ActivityEdge){
	    	    String crelEdgeIDSecond = UUID.randomUUID().toString(); 
	        	ActivityEdge activityEdgeSecond = (ActivityEdge) firstlyFocusedCell.getCorrespondingSecondElement();	    	
	        	CrelRTEdge crelRTEdgeSecond = new CrelRTEdgeImpl(null, activityEdgeSecond, classEdge);
		    	activityEdgeSecond.getRelatedCrelRTRelations().add(crelRTEdgeSecond);
		    	crelRTEdgeSecond.setID(crelEdgeIDSecond);
		    	
	            transparentCell.setCorrespondingSecondElement(crelRTEdgeSecond);
	            firstHalfEdge.setCorrespondingSecondElement(crelRTEdgeSecond);
	            secondHalfEdge.setCorrespondingSecondElement(crelRTEdgeSecond);
	            transparentCell.setSecondID(crelEdgeIDSecond);
	            firstHalfEdge.setSecondID(crelEdgeIDSecond);
	            secondHalfEdge.setSecondID(crelEdgeIDSecond);
	            crelRTEdgeSecond.setCorrespondingCell(transparentCell);
	            model.getAllCrelRTRelations().add(crelRTEdgeSecond);		    	
	        }
	    }		
	    	    
        model.getGraph().getGraphLayoutCache().insert(firstHalfEdge); 
        model.getGraphCells().add(firstHalfEdge);
        model.getGraph().getGraphLayoutCache().insert(secondHalfEdge); 	
        model.getGraphCells().add(secondHalfEdge);       
	}
	/**
	 * draw a activity edge, add the corresponding cells(a AOC edge corresponds to two edge cells and a transparent node cell) into the graph and add the corresponding node into ClassSet
	 */
	public void drawActivityEdge(String activityEdgeType, MyOriginalCell firstlyFocusedCell, int firstX, int firstY, MyOriginalCell secondlyFocusedCell, int  secondX, int secondY){	    
	    //create a edge with its source node and cardinality, and target node and cardinality
	    ActivityNode referenceNode = (ActivityNode) firstlyFocusedCell.getCorrespondingElement();
	    ActivityNode targetNode = (ActivityNode) secondlyFocusedCell.getCorrespondingElement();

	    String[] typeComplex = activityEdgeType.split("_");
	    String activityEdgeID  = UUID.randomUUID().toString();
        ActivityEdge activityEdge = new ActivityEdgeImpl(referenceNode, targetNode, typeComplex[0]);              
        activityEdge.setID(activityEdgeID);
        model.getAllActivityRelations().add(activityEdge);
                       
        //create cells(a activity edge corresponds to two edge cells and a transparent node cell) for a edge
	    TransparentCell transparentCell = new TransparentCell(null);     
	    transparentCell.setID(activityEdgeID);	    	    
	    GraphConstants.setBounds(transparentCell.getAttributes(), new Rectangle2D.Double((firstX + secondX)/2,(firstY + secondY)/2,transparentCellSize,transparentCellSize));  
	    GraphConstants.setGradientColor(transparentCell.getAttributes(),Color.black);  
	    GraphConstants.setOpaque(transparentCell.getAttributes(), true);  		    
	    DefaultPort port = new DefaultPort();  
	    transparentCell.add(port);  	
	      	    	           
        ActivityEdgeCell firstHalfEdge = new ActivityEdgeCell();       
        firstHalfEdge.setSource(transparentCell.getChildAt(0));         
        firstHalfEdge.setTarget(firstlyFocusedCell.getChildAt(0));  
        firstHalfEdge.setSourceNode(transparentCell);// for each edge, restore its two end nodes
        firstHalfEdge.setTargetNode(firstlyFocusedCell); 
        firstHalfEdge.setID(activityEdgeID);
//        firstHalfEdge.setWhichEnd("Source");
             
        ActivityEdgeCell secondHalfEdge = new ActivityEdgeCell();  
//        secondHalfEdge.displayCardinality();         
        secondHalfEdge.setSource(transparentCell.getChildAt(0));
        secondHalfEdge.setTarget(secondlyFocusedCell.getChildAt(0)); 
        secondHalfEdge.setSourceNode(transparentCell);// for each edge, restore its two end nodes
        secondHalfEdge.setTargetNode(secondlyFocusedCell); 
        secondHalfEdge.setID(activityEdgeID);
//        secondHalfEdge.setWhichEnd("Target");
        
        //design the appearance of the edge based on its type      
        int startShape;
        int endShape;
//        OCBCMElementType edgeType = null;
//        for(OCBCMElementType t : OCBCMElementType.values()){
//        	if(t.toString().equals(activityEdgeType))  edgeType = t;
//   	 	}
//        
//        //number vs shape
//        //5:dot; 16:single arrow + dot; 17: double arrows + dot; 
//        switch(edgeType)
//        {
//        	case ActivityEdgeResponse: startShape = 5; endShape = 15;
//				break;
//			case ActivityEdgeUnaryResponse: startShape = 5; endShape = 2;
//				break;
//			case ActivityEdgeNonResponse: startShape =5; endShape = 15;
//				break;
//			case ActivityEdgePrecedence: startShape =17; endShape = 6;	        	
//				break;
//			case ActivityEdgeUnaryPrecedence: startShape =16; endShape = 6;	        	
//				break;
//			case ActivityEdgeNonPrecedence: startShape =17; endShape = 6;	        	
//				break;
//			case ActivityEdgeCoExistence: startShape =5; endShape = 6;	        	
//				break;
//			case ActivityEdgeNonCoExistence: startShape =5; endShape = 6;	        	
//				break;
//			case ActivityEdgeResponse_ActivityEdgePrecedence: startShape =5; endShape = 17;	        	
//				break;
//			case ActivityEdgeUnaryResponse_ActivityEdgeUnaryPrecedence: startShape =5; endShape = 16;	        	
//				break;
//			case ActivityEdgeCoExistence_ActivityEdgeCoExistence: startShape =5; endShape = 5;	        	
//				break;
//			case ActivityEdgeNonResponse_ActivityEdgeNonPrecedence: startShape =5; endShape = 17;	        	
//				break;
//			case ActivityEdgeNonCoExistence_ActivityEdgeNonCoExistence: startShape =5; endShape = 5;	        	
//				break;				
//			default: startShape =6; endShape = 6;
//        }      
        startShape =6; endShape = 6;
        GraphConstants.setLineEnd(firstHalfEdge.getAttributes(), startShape);
        GraphConstants.setLineEnd(secondHalfEdge.getAttributes(), endShape);
       
        GraphConstants.setEndSize(firstHalfEdge.getAttributes(), 6);
        GraphConstants.setEndSize(secondHalfEdge.getAttributes(), 6);
        GraphConstants.setEndFill(firstHalfEdge.getAttributes(), true);
        GraphConstants.setEndFill(secondHalfEdge.getAttributes(), true);
        
//        if(edgeType.equals(OCBCMElementType.ActivityEdgeNonCoExistence) || edgeType.equals(OCBCMElementType.ActivityEdgeNonPrecedence) || 
//        		edgeType.equals(OCBCMElementType.ActivityEdgeNonResponse) || edgeType.equals(OCBCMElementType.ActivityEdgeNonResponse_ActivityEdgeNonPrecedence)||
//        		edgeType.equals(OCBCMElementType.ActivityEdgeNonCoExistence_ActivityEdgeNonCoExistence))
//        {
//	        System.out.print("enter NEW crossing:" + edgeType.toString());
//	        GraphConstants.setLineBegin(firstHalfEdge.getAttributes(), 4);
//	        GraphConstants.setLineBegin(secondHalfEdge.getAttributes(), 4);
//	        GraphConstants.setBeginSize(firstHalfEdge.getAttributes(), 5);
//	        GraphConstants.setBeginSize(secondHalfEdge.getAttributes(), 5);
//        }
        
        transparentCell.setCorrespondingElement(activityEdge);
        firstHalfEdge.setCorrespondingElement(activityEdge);
        secondHalfEdge.setCorrespondingElement(activityEdge);
        activityEdge.setCorrespondingCell(transparentCell);  
        
        
	    model.getGraph().getGraphLayoutCache().insert(transparentCell); 
	    model.getGraphCells().add(transparentCell);
        model.getGraph().getGraphLayoutCache().insert(firstHalfEdge); 
        model.getGraphCells().add(firstHalfEdge);
        model.getGraph().getGraphLayoutCache().insert(secondHalfEdge); 	
        model.getGraphCells().add(secondHalfEdge);
        
        targetNode.getRelatedActivityNodes().add(referenceNode);
        referenceNode.getRelatedActivityNodes().add(targetNode);
        
        //if the edge is a combination edge, add the second edge
        if(typeComplex.length == 2){
        	String activityEdgeIDSecond = UUID.randomUUID().toString();
        	ActivityEdge activityEdgeSecond = new ActivityEdgeImpl(targetNode, referenceNode, typeComplex[1]);              
        	activityEdgeSecond.setID(activityEdgeIDSecond);         
            transparentCell.setCorrespondingSecondElement(activityEdgeSecond);
            transparentCell.setSecondID(activityEdgeIDSecond);           
            //if there is a second edge, the secondHalfEdge corresponds to the second edge
            secondHalfEdge.setCorrespondingElement(activityEdgeSecond);  
            secondHalfEdge.setID(activityEdgeIDSecond);                     
            activityEdgeSecond.setCorrespondingCell(transparentCell);
            model.getAllActivityRelations().add(activityEdgeSecond);
        }
	}
	


	protected DefaultPort drawGraph(JGraph graph, MyOriginalCell parent, OCBCModel model) {
		return null;
		
	}


}
