package org.processmining.ocbc.models.visualization.declare;

/**
 * <p>
 * Title: DECLARE
 * </p>
 * 
 * <p>
 * Description:
 * </p>
 * 
 * <p>
 * Copyright: Copyright (c) 2006
 * </p>
 * 
 * <p>
 * Company: TU/e
 * </p>
 * 
 * @author Maja Pesic
 * @version 1.0
 */

import java.awt.Color;
import java.awt.Point;
import java.util.ArrayList;
import java.util.UUID;

import org.jgraph.graph.AttributeMap;
import org.jgraph.graph.Edge;
import org.jgraph.graph.GraphConstants;
import org.processmining.ocbc.models.visualization.cell.MyOriginalCell;

//public class DEdge extends DefaultEdge implements Cloneable , Edge{
public class DEdge extends MyOriginalCell implements Cloneable , Edge{
	private static final long serialVersionUID = -693583188250090210L;
	private static final float LINE_WIDTH = 2;
	private static final int SYMBOL_SIZE = 10;
	private LineStyle lineStyle;
	private Color color = Color.BLACK;
	private UUID ID;
	
	/** Source and target of the edge. */
	protected Object source, target;
	
	public DEdge() {
		super();
		setLineStyle(new LineStyle());
		prepare(false);
	}

	public DEdge(Object object) {
		super(object);
		setLineStyle(new LineStyle());
		prepare(false);
	}

	public DEdge(Object object, AttributeMap attributeMap) {
		super(object, attributeMap);
		setLineStyle(new LineStyle());
		prepare(false);
	}

	public DEdge(Object object, LineStyle anLineStyle) {
		super(object);
		setLineStyle(anLineStyle);
		prepare(false);
	}

	public DEdge(LineStyle anLineStyle) {
		super();
		setLineStyle(anLineStyle);
		prepare(false);
	}

	public DEdge(LineStyle anLineStyle, ArrayList<Point> points) {
		super();
		setLineStyle(anLineStyle);
		GraphConstants.setPoints(getAttributes(), points);
		prepare(false);
	}

	protected float getLineWidth() {
		return LINE_WIDTH;
	}

	public LineStyle getLineStyle() {
		return lineStyle;
	}

	private void setLineStyle(LineStyle anStyle) {
		lineStyle = anStyle;

		resetLineDash();
		resetLineNumber();
		resetLineBegin();
		resetLineMiddle();
		resetLineEnd();
	}

	protected void resetLineStyle() {
		setLineStyle(lineStyle);
	}

	private int getSymbolSize(int symbol) {
		int size = SYMBOL_SIZE;
		if (symbol == GraphConstants.ARROW_DOUBLELINE) {
			size = size * (new Float(LINE_WIDTH)).intValue();
		}
		return size;
	}

	protected void resetLineBegin() {
		// Set Arrow and Fill Style for edge begin
		AttributeMap map = getAttributes();
		GraphConstants.setLineBegin(map, lineStyle.getBegin());
		GraphConstants.setBeginFill(map, lineStyle.isBeginFill());
		GraphConstants.setBeginSize(map, getSymbolSize(lineStyle.getBegin()));
	}

	protected void resetLineEnd() {
		// Set Arrow and Fill Style for edge end
		AttributeMap map = getAttributes();
		GraphConstants.setLineEnd(map, lineStyle.getEnd());
		GraphConstants.setEndFill(map, lineStyle.isEndFill());
		GraphConstants.setEndSize(map, getSymbolSize(lineStyle.getEnd()));
	}
	
	public void setSelectable(boolean flag){
		AttributeMap map = getAttributes();
		GraphConstants.setSelectable(map, flag);
	}
	
	public void setOpaque(boolean flag){
		AttributeMap map = getAttributes();
		GraphConstants.setOpaque(map, flag);
	}

	@SuppressWarnings("unchecked")
	protected void resetLineMiddle() {
		// Set Arrow and Fill Style for edge end

		AttributeMap map = getAttributes();
		DGraphConstants.setLineMiddle(map, lineStyle.getMiddle());
		DGraphConstants.setMiddleFill(map, lineStyle.isMiddleFill());
		DGraphConstants.setMiddleSize(map, getSymbolSize(lineStyle.getMiddle()));
	}

	protected void resetLineDash() {
		// Set Dash Style for edge
		setLineDash(lineStyle.getLine());
	}

	protected void setLineDash(float[] dash) {
		if (dash != null) {
			GraphConstants.setDashPattern(getAttributes(), dash);
		} else {
			getAttributes().remove(GraphConstants.DASHPATTERN);
		}

	}

	@SuppressWarnings("unchecked")
	protected void resetLineNumber() {
		// Set Dash Style for edge
		DGraphConstants.setLineNumber(getAttributes(), lineStyle.getNumber());
	}

	/**
	 * setUp
	 * 
	 * @param editable
	 *            boolean
	 */
	private void prepare(boolean editable) {
		GraphConstants.setLabelAlongEdge(getAttributes(), true);
		GraphConstants.setEditable(getAttributes(), editable);
		GraphConstants.setDisconnectable(getAttributes(), false);
		GraphConstants.setLineWidth(getAttributes(), getLineWidth());
	}

	/**
	 * setEditable
	 * 
	 * @param editable
	 *            boolean
	 */
	protected void setEditable(boolean editable) {
		GraphConstants.setEditable(getAttributes(), editable);
	}

	/**
	 * setColor
	 * 
	 * @param color
	 *            Color
	 */
	public void setLineColor(Color color) {
		GraphConstants.setLineColor(getAttributes(), color);
		this.color = color;
	}

	/**
	 * 
	 * @return Object
	 */
//	public Object clone() {
//		DEdge myClone = null;
//		Object lineClone = lineStyle.clone();
//		if (lineClone != null) {
//			if (lineClone instanceof LineStyle) {
//				myClone = new DEdge((LineStyle) lineClone);
//			}
//		}
//		return myClone;
//	}

	/**
	 * clearLineBegin
	 */
	protected void clearLineBegin() {
		// Set Arrow Style for edge
		GraphConstants.setLineBegin(getAttributes(), GraphConstants.ARROW_NONE);

		// Set Arrow Style for edge
		GraphConstants.setBeginFill(getAttributes(), false);
	}

	/**
	 * clearLineEnd
	 */
	protected void clearLineEnd() {
		// Set Arrow Style for edge
		GraphConstants.setLineEnd(getAttributes(), GraphConstants.ARROW_NONE);
		// Set Arrow Style for edge
		GraphConstants.setEndFill(getAttributes(), false);
	}

	/**
	 * clearLineMiddle
	 */

	@SuppressWarnings("unchecked")
	protected void clearLineMiddle() {
		// Set Arrow Style for edge
		DGraphConstants.setLineMiddle(getAttributes(), GraphConstants.ARROW_NONE);

		// Set Arrow Style for edge
		DGraphConstants.setMiddleFill(getAttributes(), false);
	}

	public void refresh() {

	}
	
	public Color getLineColor(){
		return color;
	}

	/**
	 * Returns the source of the edge.
	 */
	public Object getSource() {
		return source;
	}

	/**
	 * Returns the target of the edge.
	 */
	public Object getTarget() {
		return target;
	}

	/**
	 * Sets the source of the edge.
	 */
	public void setSource(Object port) {
		source = port;
	}

	/**
	 * Returns the target of <code>edge</code>.
	 */
	public void setTarget(Object port) {
		target = port;
	}


//	//
//	// Default Routing
//	// 
//
//	public static class LoopRouting implements Edge.Routing {
//
//		public List route(GraphLayoutCache cache, EdgeView edge) {
//			if (edge.isLoop()) {
//				return routeLoop(cache, edge);
//			}
//			return routeEdge(cache, edge);
//		}
//
//		protected List routeLoop(GraphLayoutCache cache, EdgeView edge) {
//			List newPoints = new ArrayList();
//			newPoints.add(edge.getSource());
//			CellView sourceParent = (edge.getSource() != null) ? edge
//					.getSource().getParentView() : edge.getSourceParentView();
//			if (sourceParent != null) {
//				Point2D from = AbstractCellView.getCenterPoint(sourceParent);
//				Rectangle2D rect = sourceParent.getBounds();
//				double width = rect.getWidth();
//				double height2 = rect.getHeight() / 2;
//				double loopWidth = Math.min(20, Math.max(10, width / 8));
//				double loopHeight = Math.min(30, Math.max(12, Math.max(
//						loopWidth + 4, height2 / 2)));
//				newPoints.add(edge.getAttributes().createPoint(
//						from.getX() - loopWidth,
//						from.getY() - height2 - loopHeight * 1.2));
//				newPoints.add(edge.getAttributes().createPoint(from.getX(),
//						from.getY() - height2 - 1.5 * loopHeight));
//				newPoints.add(edge.getAttributes().createPoint(
//						from.getX() + loopWidth,
//						from.getY() - height2 - loopHeight * 1.2));
//				newPoints.add(edge.getTarget());
//				return newPoints;
//			}
//			return null;
//		}
//
//		protected List routeEdge(GraphLayoutCache cache, EdgeView edge) {
//			return null;
//		}
//
//		public int getPreferredLineStyle(EdgeView edge) {
//			if (edge.isLoop()) {
//				return getLoopStyle();
//			}
//			return getEdgeStyle();
//		}
//
//		protected int getLoopStyle() {
//			return GraphConstants.STYLE_BEZIER;
//		}
//
//		protected int getEdgeStyle() {
//			return NO_PREFERENCE;
//		}
//
//	}
//
//	public static class DefaultRouting extends LoopRouting {
}
