package org.processmining.ocbc.performance.logprojection;

import javax.swing.JComponent;

import org.deckfour.xes.model.XLog;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.contexts.uitopia.annotations.Visualizer;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginLevel;
import org.processmining.ocbc.performance.logprojection.plugins.dottedchart.DottedChart.DottedChartException;
import org.processmining.ocbc.performance.logprojection.plugins.dottedchart.ui.DottedChartInspector;

/**/
public class LogProjectionPlugin {

	private static final String UIHELP = "The LogProjection package provides a brand new "
			+ "implementation for a dottedchart visualization of event logs, capable of "
			+ "handling large event logs. The visualisation allows the user to project "
			+ "arbitrary attributes on the x and y-axis, as well as for the color (max 1024) "
			+ "and shape (max 8) of a dot. The visualisation detects the speed of the computer "
			+ "it is used on and uses that to decide what fraction of the dots to show in order "
			+ "to guarantee a responsive user experience. Besides the visualisation, the "
			+ "dottedchart offers sorting of traces and filtering of events. Sorting is "
			+ "available from the left panel and filtering by right-clicking. All sorting and "
			+ "filtering is done in place without changing the underlying log. The right-click "
			+ "menu also provides access to export methods to export the current view as a log "
			+ "or as an image. For the dottedchart to function properly the log should declare "
			+ "attributes as global and should use standard XES extensions as much as possible.";

	@Plugin(name = "kfgz Project Log on DottedChart", level = PluginLevel.PeerReviewed, parameterLabels = { "Event log" }, //
	returnLabels = { "Dotted Chart" }, returnTypes = { LogView.class }, //
	userAccessible = true, help = "Projects the log onto a dotted chart", handlesCancel = false)
	@UITopiaVariant(uiLabel = "kfgz Project Log on DottedChart", affiliation = "Eindhoven University of Technology", author = "B.F. van Dongen", email = "b.f.v.dongen@tue.nl", pack = "LogProjection", uiHelp = UIHELP)
	public static LogView logToDottedChart(final PluginContext context, XLog eventlog) throws DottedChartException {
		return new LogView(eventlog);
	}

	@Plugin(name = "Project Log as DottedChart", level = PluginLevel.PeerReviewed, parameterLabels = "Event Log", //
	returnTypes = JComponent.class, returnLabels = "Dotted Chart", userAccessible = true, //
	mostSignificantResult = 1, help = "Projects the log onto a dotted chart")
	@Visualizer(name = "Dotted Chart", pack = "LogProjection")
	public static JComponent visualize(final PluginContext context, XLog eventlog) throws DottedChartException {
		//		return new DottedChartPanel(context, eventlog);
		return new DottedChartInspector(new LogView(eventlog, context.getProgress()), context);
	}

	@Plugin(name = "Visualize DottedChart", level = PluginLevel.PeerReviewed, parameterLabels = "Log View", //
	returnTypes = JComponent.class, returnLabels = "Dotted Chart", userAccessible = true, //
	mostSignificantResult = 1, help = "Visualize a dotted chart")
	@Visualizer(name = "Dotted Chart", pack = "LogProjection")
	public static DottedChartInspector visualize(final PluginContext context, LogView logView)
			throws DottedChartException {
		return new DottedChartInspector(logView, context);
	}
	


}
