package org.processmining.ocbc.performance.logprojection;

public interface Statistics {

	public static class Double implements Statistics {
		// average keeps track of the sum of the part of all observations higher
		// than the minimum
		private double sum;
		// average keeps track of the sum of the part of all squared
		// observations
		// higher than the squared minimum
		private double sumSq;
		// minimum
		private double minimum;
		// first
		private double first;
		// maximum
		private double maximum;
		// number of observations
		protected int numObservations;

		public Double(double minimum, double maximum, double first, double sum,
				double sumSq, int numObservations) {
			this.minimum = minimum;
			this.maximum = maximum;
			this.sum = sum;
			this.sumSq = sumSq;
			this.numObservations = numObservations;

		}

		public Double() {

		}

		public String toHTMLString() {
			StringBuilder builder = new StringBuilder();
			builder.append("<HTML>");
			fillHTMLString(builder);
			return builder.toString();
		}

		protected void fillHTMLString(StringBuilder builder) {
			builder.append("average:   "
					+ java.lang.Double.toString(getAverage()));
			builder.append("<BR>");
			builder.append("std. dev.: "
					+ java.lang.Double.toString(Math.sqrt(getVariance())));
			builder.append("<BR>");
			builder.append("minimum:   "
					+ java.lang.Double.toString(getMinimum()));
			builder.append("<BR>");
			builder.append("maximum:   "
					+ java.lang.Double.toString(getMaximum()));
			builder.append("<BR>");
			builder.append("observations: " + getNumObservations());
			builder.append("<BR>");

		}

		public void addObservation(double value) {
			if (numObservations == 0) {
				sum = 0;
				sumSq = 0;
				numObservations = 1;
				minimum = value;
				maximum = value;
				first = value;
				return;
			}
			if (value > maximum) {
				maximum = value;
			}
			if (value < minimum) {
				minimum = value;
				// the new minimum was found
			}
			double dif = value - first;
			sum += dif;
			sumSq += dif * dif;
			numObservations++;

		}

		public int getNumObservations() {
			return numObservations;
		}

		public double getAverage() {
			return first + sum / numObservations;
		}

		public double getVariance() {
			return (sumSq - (sum * sum) / numObservations)
					/ (numObservations - 1);
		}

		public double getMinimum() {
			return minimum;
		}

		public double getMaximum() {
			return maximum;
		}

		public void addObservation(long value) {
			addObservation((double) value);
		}

		public void addObservation(int value) {
			addObservation((double) value);
		}

		public void addObservation(short value) {
			addObservation((double) value);
		}

		public void addObservation(float value) {
			addObservation((double) value);
		}

		public double getMinimumDouble() {
			return getMinimum();
		}

		public double getMaximumDouble() {
			return getMaximum();
		}

	}

	public static class Long implements Statistics {
		// average keeps track of the sum of the part of all observations higher
		// than the minimum
		private long sum;
		// average keeps track of the sum of the part of all squared
		// observations
		// higher than the squared minimum
		private long sumSq;
		// minimum
		private long minimum;
		// first
		private long first;
		// maximum
		private long maximum;
		// number of observations
		protected int numObservations;

		public Long(long minimum, long maximum, long first, long sum,
				long sumSq, int numObservations) {
			this.minimum = minimum;
			this.maximum = maximum;
			this.sum = sum;
			this.sumSq = sumSq;
			this.numObservations = numObservations;

		}

		public Long() {

		}

		public String toHTMLString() {
			StringBuilder builder = new StringBuilder();
			builder.append("<HTML>");
			fillHTMLString(builder);
			return builder.toString();
		}

		protected void fillHTMLString(StringBuilder builder) {

			builder.append("average:   "
					+ java.lang.Double.toString(getAverage()));
			builder.append("<BR>");
			builder.append("std. dev.: "
					+ java.lang.Double.toString(Math.sqrt(getVariance())));
			builder.append("<BR>");
			builder.append("minimum:   " + getMinimum());
			builder.append("<BR>");
			builder.append("maximum:   " + getMaximum());
			builder.append("<BR>");
			builder.append("observations: " + getNumObservations());
			builder.append("<BR>");

		}

		public void addObservation(long value) {
			if (numObservations == 0) {
				sum = 0;
				sumSq = 0;
				numObservations = 1;
				minimum = value;
				maximum = value;
				first = value;
				return;
			}
			if (value > maximum) {
				maximum = value;
			}
			if (value < minimum) {
				minimum = value;
				// the new minimum was found
			}
			long dif = value - first;
			sum += dif;
			sumSq += dif * dif;
			numObservations++;

		}

		public int getNumObservations() {
			return numObservations;
		}

		public double getAverage() {
			return first + ((double) sum) / numObservations;
		}

		public double getVariance() {
			return (sumSq - ((double) (sum * sum)) / numObservations)
					/ (numObservations - 1);
		}

		public long getMinimum() {
			return minimum;
		}

		public long getMaximum() {
			return maximum;
		}

		public void addObservation(double value) {
			addObservation((long) value);
		}

		public void addObservation(int value) {
			addObservation((long) value);
		}

		public void addObservation(short value) {
			addObservation((long) value);
		}

		public void addObservation(float value) {
			addObservation((long) value);
		}

		public double getMinimumDouble() {
			return (double) getMinimum();
		}

		public double getMaximumDouble() {
			return (double) getMaximum();
		}

	}

	public double getAverage();

	public double getVariance();

	public void addObservation(int value);

	public void addObservation(short value);

	public void addObservation(float value);

	public void addObservation(long value);

	public void addObservation(double value);

	public abstract String toHTMLString();

	public int getNumObservations();

	public double getMinimumDouble();

	public double getMaximumDouble();
}
