package org.processmining.ocbc.performance.logprojection.abstraction.ui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.geom.Rectangle2D;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSlider;
import javax.swing.SwingConstants;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.plaf.basic.BasicSliderUI;

import org.jgraph.JGraph;
import org.jgraph.graph.ConnectionSet;
import org.jgraph.graph.DefaultCellViewFactory;
import org.jgraph.graph.DefaultEdge;
import org.jgraph.graph.DefaultGraphCell;
import org.jgraph.graph.DefaultGraphModel;
import org.jgraph.graph.DefaultPort;
import org.jgraph.graph.EdgeView;
import org.jgraph.graph.GraphConstants;
import org.jgraph.graph.GraphLayoutCache;
import org.jgraph.graph.ParentMap;
import org.processmining.ocbc.performance.logprojection.abstraction.CausalDependencyMatrix;
import org.processmining.ocbc.performance.logprojection.abstraction.DirectSuccessionCounter;
import org.processmining.ocbc.performance.logprojection.abstraction.HeurisicMinerCausalDepencency;
import org.processmining.ocbc.performance.logprojection.abstraction.MultiPhaseMinerMatrix;

import com.jgraph.layout.JGraphFacade;
import com.jgraph.layout.JGraphLayout;
import com.jgraph.layout.graph.JGraphSimpleLayout;
import com.jgraph.layout.hierarchical.JGraphHierarchicalLayout;
import com.jgraph.layout.organic.JGraphFastOrganicLayout;

public class LogAbstractionGraphPanel implements ItemListener, ChangeListener,
		ActionListener {

	private static final long serialVersionUID = 5340443412303523829L;

	private static class ProMLoopRouting extends DefaultEdge.DefaultRouting {

		public static ProMLoopRouting ROUTER = new ProMLoopRouting();

		private static final long serialVersionUID = -1502015269578934172L;

		public List<?> route(GraphLayoutCache cache, EdgeView edge) {
			// No routing is performed "on the fly", i.e. all routing
			// information
			// is introduced during the layout phase where all internal control
			// points
			// of edges are set. Hence, the route should just return null.
			return null;
		}
	}

	public static final DecimalFormat FORMAT = new DecimalFormat("#.##");

	private final DirectSuccessionCounter counter;
	private Map<String, Color> colors;
	private JGraphLayout layout;
	private JGraph graph;
	private DefaultGraphModel model;
	private final JComboBox<JGraphLayout> layoutCombo;
	// private final JRadioButton direct;
	// private final JRadioButton causal;
	// private boolean useCausalRelation = true;
	private final JScrollPane scrollPane;
	private final JSlider thresholdSlider;
	private DefaultGraphCell[] cells;
	private DefaultEdge[] edges;
	private double threshold;
	private JSlider zoomSlider;
	private JComboBox<CausalDependencyMatrix> graphTypeCombo;
	private HeurisicMinerCausalDepencency causalMatrix;
	private MultiPhaseMinerMatrix partialOrderBuilder;

	private int currentNumberOfNodes;

	public LogAbstractionGraphPanel(DirectSuccessionCounter counter,
			Map<String, Color> colors) {
		this.counter = counter;
		this.causalMatrix = new HeurisicMinerCausalDepencency(counter);
		this.partialOrderBuilder = new MultiPhaseMinerMatrix(counter.getLog(),
				causalMatrix);
		this.colors = colors;

		graphTypeCombo = new JComboBox<CausalDependencyMatrix>(
				new CausalDependencyMatrix[] { counter, causalMatrix,
						partialOrderBuilder });
		graphTypeCombo.setSelectedItem(partialOrderBuilder);
		graphTypeCombo.addItemListener(this);

		layoutCombo = new JComboBox<JGraphLayout>(getLayouts());
		layoutCombo.setSelectedIndex(-1);
		layoutCombo.addItemListener(this);

		updateGraph(partialOrderBuilder);
		updateEdges(partialOrderBuilder);
		updateVisibility(partialOrderBuilder);
		scrollPane = new JScrollPane(graph);

		// direct = new JRadioButton("Direct succession");
		// causal = new JRadioButton("Causal relation");
		// ButtonGroup group = new ButtonGroup();
		// group.add(direct);
		// group.add(causal);
		// causal.setSelected(true);

		// direct.addItemListener(this);
		// causal.addItemListener(this);

		thresholdSlider = makeSlider(1);
		zoomSlider = makeSlider(2);
		zoomSlider.setValue(1000);
	}

	private JSlider makeSlider(int max) {
		JSlider slider = new JSlider(0, max * 1000, 0);
		slider.setMinorTickSpacing(max * 50);
		slider.setMajorTickSpacing(max * 250);
		slider.setPaintLabels(true);
		slider.setPaintTicks(true);
		// slider.setPaintTrack(false);
		slider.addChangeListener(this);
		Hashtable<Integer, JLabel> labelTable = new Hashtable<Integer, JLabel>();
		for (int i = 0; i <= max * 1000; i += max * 250) {
			labelTable.put(i, new JLabel(Double.toString(i / 1000.0)));
		}
		slider.setLabelTable(labelTable);
		slider.setUI(new BasicSliderUI(slider));
		return slider;
	}

	public JPanel getPanel() {

		JPanel panel = new JPanel(new BorderLayout());

		panel.add(scrollPane, BorderLayout.CENTER);

		JPanel configPanel = new JPanel(new GridLayout(8, 1));

		layoutCombo.setSelectedIndex(0);

		configPanel.add(new JLabel(graph.getName()));
		configPanel.add(layoutCombo);
		configPanel.add(graphTypeCombo);
		configPanel.add(new JLabel("Threshold:"));
		configPanel.add(thresholdSlider);
		configPanel.add(new JLabel("Zoom:"));
		configPanel.add(zoomSlider);
		JButton fitButton = new JButton("fit");
		fitButton.addActionListener(this);

		configPanel.add(fitButton);

		JPanel westPanel = new JPanel(new BorderLayout());
		westPanel.add(configPanel, BorderLayout.NORTH);

		panel.add(westPanel, BorderLayout.WEST);
		return panel;
	}

	private void updateGraph(CausalDependencyMatrix matrix) {
		model = new DefaultGraphModel();
		graph = new JGraph(model);
		GraphLayoutCache cache = new GraphLayoutCache(model,
				new DefaultCellViewFactory(), true);
		// cache.setShowsChangedConnections(false);
		graph.setGraphLayoutCache(cache);
		graph.setName(counter.getLog().getName());

		ConnectionSet cs = new ConnectionSet();

		model.beginUpdate();

		ParentMap pm = new ParentMap();

		cells = new DefaultGraphCell[matrix.getNumberOfNodes()];
		for (int i = 0; i < matrix.getNumberOfNodes(); i++) {
			if (matrix.getOccurenceCount(i) == 0) {
				continue;
			}
			cells[i] = new DefaultGraphCell("<HTML>" + matrix.getEventClass(i)
					+ "<BR>" + matrix.getOccurenceCount(i) + "</HTML>");
			GraphConstants.setGradientColor(cells[i].getAttributes(),
					getColor(matrix.getEventClass(i)));
			GraphConstants.setBounds(cells[i].getAttributes(),
					new Rectangle2D.Double(140, 140, 100, 50));
			GraphConstants.setOpaque(cells[i].getAttributes(), true);
			DefaultPort port = new DefaultPort();
			// cells[i].add(port);

			pm.addEntry(port, cells[i]);

			// graph.getGraphLayoutCache().insert(cells[i]);
		}
		// graph.setGraphLayoutCache(new GraphLayoutCache(model,
		// new DefaultCellViewFactory(), true));

		model.insert(cells, null, cs, pm, null);
		model.endUpdate();

		graph.getGraphLayoutCache().setVisible(cells, true);

		graph.setAntiAliased(true);
		currentNumberOfNodes = matrix.getNumberOfNodes();

	}

	private Color getColor(String label) {
		Color c = colors.get(label);
		if (c == null) {
			return Color.GRAY;
		} else {
			return c;
		}
	}

	private void updateEdges(CausalDependencyMatrix matrix) {

		ParentMap pm = new ParentMap();
		ConnectionSet cs = new ConnectionSet();
		model.beginUpdate();
		if (edges != null) {
			for (DefaultEdge edge : edges) {
				cs.disconnect(edge);
			}
			model.remove(edges);
			// graph.getGraphLayoutCache().remove(edges);
		}
		List<DefaultEdge> edgeList = new ArrayList<DefaultEdge>();
		for (int i = 0; i < matrix.getNumberOfNodes(); i++) {
			if (matrix.getOccurenceCount(i) == 0) {
				continue;
			}
			for (int j = 0; j < matrix.getNumberOfNodes(); j++) {
				if (matrix.getOccurenceCount(j) == 0) {
					continue;
				}
				if (acceptEdge(i, j, 0.0)) {
					DefaultEdge edge = new DefaultEdge(getCount(i, j));
					edge.setSource(cells[i].getChildAt(0));
					edge.setTarget(cells[j].getChildAt(0));

					// GraphConstants.setLineColor(edge.getAttributes(),
					// new Color(0, 0, 0,
					// 128 + (int) (127 * getValue(i, j))));

					GraphConstants.setRouting(edge.getAttributes(),
							ProMLoopRouting.ROUTER);
					// GraphConstants.setLabelPosition(edge.getAttributes(),
					// new Point2D.Double(GraphConstants.PERMILLE / 2, 0));
					GraphConstants.setLineStyle(edge.getAttributes(),
							GraphConstants.STYLE_SPLINE);
					GraphConstants.setLineWidth(edge.getAttributes(), 2.0f);
					GraphConstants.setLineEnd(edge.getAttributes(),
							GraphConstants.ARROW_CLASSIC);
					GraphConstants.setEndFill(edge.getAttributes(), true);
					GraphConstants.setLabelEnabled(edge.getAttributes(), true);
					// GraphConstants
					// .setLabelAlongEdge(edge.getAttributes(), true);
					graph.getGraphLayoutCache().insert(edge);

					cs.connect(edge, cells[i].getChildAt(0),
							cells[j].getChildAt(0));

					edgeList.add(edge);
				}
			}
		}
		this.edges = edgeList.toArray(new DefaultEdge[edgeList.size()]);

		model.insert(edges, null, cs, pm, null);
		model.endUpdate();
	}

	private void updateVisibility(CausalDependencyMatrix matrix) {

		List<DefaultEdge> edgeList = new ArrayList<DefaultEdge>();
		int k = 0;
		for (int i = 0; i < matrix.getNumberOfNodes(); i++) {
			if (matrix.getOccurenceCount(i) == 0) {
				continue;
			}
			for (int j = 0; j < matrix.getNumberOfNodes(); j++) {
				if (matrix.getOccurenceCount(j) == 0) {
					continue;
				}
				if (acceptEdge(i, j, 0.0)) {
					if (acceptEdge(i, j, threshold)) {

						edgeList.add(edges[k]);
					}
					k++;
				}
			}
		}

		graph.getGraphLayoutCache().setVisible(edges, false);
		graph.getGraphLayoutCache().setVisible(
				edgeList.toArray(new DefaultEdge[edgeList.size()]), true);
		graph.getGraphLayoutCache().update();
	}

	public void setLayout(JGraphLayout layout) {
		this.layout = layout;
	}

	public void layout() {
		if (layout != null) {

			JGraphFacade facade = new JGraphFacade(graph.getGraphLayoutCache());
			facade.setOrdered(true);
			facade.setEdgePromotion(true);
			facade.setIgnoresCellsInGroups(false);
			facade.setIgnoresHiddenCells(true);
			facade.setIgnoresUnconnectedCells(false);
			facade.setDirected(true);
			facade.resetControlPoints();
			if (layout instanceof JGraphHierarchicalLayout) {
				facade.run((JGraphHierarchicalLayout) layout, true);
			} else {
				facade.run(layout, true);
			}
			Map<?, ?> nested = facade.createNestedMap(true, true);
			graph.getGraphLayoutCache().edit(nested);
			graph.getGraphLayoutCache().reload();

		}
	}

	private JGraphLayout[] getLayouts() {
		JGraphHierarchicalLayout l0 = new JGraphHierarchicalLayout();
		l0.setOrientation(SwingConstants.WEST);
		l0.setDeterministic(true);
		l0.setCompactLayout(true);
		l0.setInterRankCellSpacing(70.0);
		l0.setIntraCellSpacing(50.0);
		l0.setInterHierarchySpacing(50.0);
		l0.setFineTuning(true);
		l0.setParallelEdgeSpacing(15);
		l0.setFixRoots(false);
		// l0.setLayoutFromSinks(true);

		JGraphSimpleLayout l1 = new JGraphSimpleLayout(
				JGraphSimpleLayout.TYPE_CIRCLE);

		JGraphFastOrganicLayout l2 = new JGraphFastOrganicLayout();
		// l2.setForceConstant(150);
		// l2.setInitialTemp(500);

		return new JGraphLayout[] { l0, l1, l2 };
	}

	@Override
	public void itemStateChanged(ItemEvent e) {
		if (e.getStateChange() != ItemEvent.SELECTED) {
			return;
		}
		if (e.getSource() == graphTypeCombo) {
			CausalDependencyMatrix matrix = (CausalDependencyMatrix) graphTypeCombo
					.getSelectedItem();
			// if (matrix instanceof MultiPhaseMinerMatrix) {
			// graph = new JGraph(
			// ((MultiPhaseMinerMatrix) matrix).toPetrinet());
			// JGraphFacade facade = new JGraphFacade(
			// graph.getGraphLayoutCache());
			// facade.setOrdered(true);
			// facade.setEdgePromotion(true);
			// facade.setIgnoresCellsInGroups(false);
			// facade.setIgnoresHiddenCells(true);
			// facade.setIgnoresUnconnectedCells(false);
			// facade.setDirected(true);
			// facade.resetControlPoints();
			// if (layout instanceof JGraphHierarchicalLayout) {
			// facade.run((JGraphHierarchicalLayout) layout, true);
			// } else {
			// facade.run(layout, true);
			// }
			// Map<?, ?> nested = facade.createNestedMap(true, true);
			// graph.getGraphLayoutCache().edit(nested);
			// graph.getGraphLayoutCache().reload();
			//
			// scrollPane.setViewportView(graph);
			// return;
			// }

			if (matrix.getNumberOfNodes() != currentNumberOfNodes) {
				updateGraph(matrix);
				scrollPane.setViewportView(graph);
				graph.setScale(zoomSlider.getValue() / 1000.0);
			}
			updateEdges(matrix);
		}
		JGraphLayout layout = (JGraphLayout) layoutCombo.getSelectedItem();
		setLayout(layout);
		layout();
	}

	@Override
	public void stateChanged(ChangeEvent e) {
		if (e.getSource() == thresholdSlider) {
			threshold = thresholdSlider.getValue() / 1000.0;
			updateVisibility((CausalDependencyMatrix) graphTypeCombo
					.getSelectedItem());
		} else if (e.getSource() == zoomSlider) {
			graph.setScale(zoomSlider.getValue() / 1000.0);
		}
	}

	private boolean acceptEdge(int i, int j, double threshold) {
		return ((CausalDependencyMatrix) graphTypeCombo.getSelectedItem())
				.getCausalDependency(i, j) > threshold;
	}

	private double getValue(int i, int j) {
		return ((CausalDependencyMatrix) graphTypeCombo.getSelectedItem())
				.getCausalDependency(i, j);
	}

	private int getCount(int i, int j) {
		return ((CausalDependencyMatrix) graphTypeCombo.getSelectedItem())
				.getCount(i, j);
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		double scale = Math.min(graph.getParent().getWidth()
				/ graph.getPreferredSize().getWidth(), graph.getParent()
				.getHeight() / graph.getPreferredSize().getHeight())
				* graph.getScale();
		zoomSlider.setValue((int) (1000 * scale));
	}
}
