package org.processmining.ocbc.performance.logprojection.calculators;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.deckfour.xes.model.XAttribute;
import org.deckfour.xes.model.XTrace;
import org.deckfour.xes.model.impl.XAttributeBooleanImpl;
import org.deckfour.xes.model.impl.XAttributeContinuousImpl;
import org.deckfour.xes.model.impl.XAttributeDiscreteImpl;
import org.processmining.ocbc.performance.logprojection.Attribute;
import org.processmining.ocbc.performance.logprojection.LogView;
import org.processmining.ocbc.performance.logprojection.XEventDataManager;
import org.processmining.ocbc.performance.logprojection.XTraceDataManager;
import org.processmining.ocbc.performance.logprojection.DataManager.AttributeValueCalculator;

public class IdenticalEventAttributeTraceCalculator implements AttributeValueCalculator<XTrace> {

	private final LogView view;
	private final List<Attribute> traceAttributes;
	private final List<Attribute> eventAttributes;

	public IdenticalEventAttributeTraceCalculator(LogView view) {
		this.view = view;
		this.eventAttributes = new ArrayList<Attribute>(10);
		this.traceAttributes = new ArrayList<Attribute>(10);

		XEventDataManager eventManager = view.getEventDataManager();
		XTraceDataManager traceManager = view.getTraceDataManager();
		Attribute[] keys = eventManager.getKeys();
		for (Attribute a : keys) {
			int index = eventManager.getKeyIndex(a);

			// Do not overwrite existing trace attributes
			boolean ok = traceManager.getAttributeForKey(a.getKey()) == null;

			for (int t = 0; ok && t < view.getTraceDataManager().getObjectCount(); t++) {
				boolean same = true;

				if (view.getTraceLength(t) == 0) {
					continue;
				}

				byte[] val2;
				byte[] val = eventManager.getBytes(index, Attribute.sizeForType(a.getType()),
						view.getFirstEventNumber(t));

				int max = view.getFirstEventNumber(t) + view.getTraceLength(t);
				for (int e = view.getFirstEventNumber(t) + 1; same && e < max; e++) {
					val2 = eventManager.getBytes(index, Attribute.sizeForType(a.getType()), e);
					same &= Arrays.equals(val, val2);
				}

				ok &= same;
			}

			if (ok) {
				// Attribute a can be made into a trace attribute.
				Attribute ta = new Attribute(a.getKey(), a.getType(), a.getDefaultValue(), false);
				eventAttributes.add(a);
				traceAttributes.add(ta);
			}
		}

	}

	public Attribute[] getNewTraceAttributes() {
		return traceAttributes.toArray(new Attribute[0]);
	}

	@Override
	public XAttribute[] computeAttributes(XTrace object, int objectIndex, int... header) {

		XAttribute[] result = new XAttribute[traceAttributes.size()];
		for (int i = 0; i < result.length; i++) {

			Attribute ta = traceAttributes.get(i);
			Attribute a = eventAttributes.get(i);
			switch (a.getType()) {
				case Attribute.LITR : {
					int val = view.getEventDataManager().getAttributeValueLiteralIndex(
							view.getFirstEventNumber(objectIndex), a);
					result[i] = new XLightLiteralAttribute.WithIndex(ta.getKey(), val);
					break;
				}
				case Attribute.CONT : {
					double val = view.getEventDataManager().getAttributeValueContinuous(
							view.getFirstEventNumber(objectIndex), a);
					result[i] = new XAttributeContinuousImpl(ta.getKey(), val);
					break;
				}
				case Attribute.DISC : {
					long val = view.getEventDataManager().getAttributeValueDiscrete(
							view.getFirstEventNumber(objectIndex), a);
					result[i] = new XAttributeDiscreteImpl(ta.getKey(), val);
					break;
				}
				case Attribute.BOOL : {
					boolean val = view.getEventDataManager().getAttributeValueBoolean(
							view.getFirstEventNumber(objectIndex), a);
					result[i] = new XAttributeBooleanImpl(ta.getKey(), val);
					break;
				}
				case Attribute.TIME_REL :
				case Attribute.TIME_ABS : {
					long val = view.getEventDataManager().getAttributeValueTimestamp(
							view.getFirstEventNumber(objectIndex), a);
					result[i] = new XLightTimestampAttribute(ta.getKey(), val);
					break;
				}
			}
		}
		return result;
	}
}
