package org.processmining.ocbc.performance.logprojection.calculators;

import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.Locale;

import org.deckfour.xes.model.XAttribute;
import org.deckfour.xes.model.XAttributeTimestamp;
import org.deckfour.xes.model.XEvent;
import org.processmining.ocbc.performance.logprojection.Attribute;
import org.processmining.ocbc.performance.logprojection.LogView;
import org.processmining.ocbc.performance.logprojection.DataManager.AttributeValueCalculator;

/**
 * This implementation requires events per trace to contain a "time:startime"
 * attribute
 * 
 * @author bfvdonge
 * 
 */
public class RelativeTimeAttributeCalculator implements AttributeValueCalculator<XEvent> {

	private static final SimpleDateFormat WEEKDAYFORMAT = new SimpleDateFormat("EEEE", Locale.ENGLISH);

	private final String weekDay;
	private final String dayTime;
	private final String weekTime;
	private final String caseTime;
	private final LogView view;
	private final Attribute eventTimestamp;
	private final int traceStartTimeIndex;

	public RelativeTimeAttributeCalculator(LogView view, String weekDay, String dayTime, String weekTime,
			String caseTime, Attribute eventTimestamp, Attribute traceStartTime) {
		this.view = view;
		this.weekDay = weekDay;
		this.dayTime = dayTime;
		this.weekTime = weekTime;
		this.caseTime = caseTime;
		this.traceStartTimeIndex = view.getTraceDataManager().getKeyIndex(traceStartTime);
		this.eventTimestamp = eventTimestamp;

	}

	private int currentTrace = -1;
	private long startCurrentTrace = -1;

	@Override
	public XAttribute[] computeAttributes(XEvent event, int eventIndex, int... header) {

		Date timestamp = ((XAttributeTimestamp) event.getAttributes().get(eventTimestamp.getKey())).getValue();

		// header[0] for an event stores the trace index this event belongs to
		long startCurrentTrace;
		int t = header[0];
		synchronized (this) {
			if (t != this.currentTrace) {
				this.startCurrentTrace = view.getTraceDataManager().getAttributeValueTimestamp(t, traceStartTimeIndex);
				this.currentTrace = t;
			}
			startCurrentTrace = this.startCurrentTrace;
		}

		String day = WEEKDAYFORMAT.format(timestamp);

		@SuppressWarnings("deprecation")
		XAttribute a1 = new XLightLiteralAttribute(weekDay, timestamp.getDay() + ": " + day);

		// move to first week since epoch
		long newTime = timestamp.getTime();
		Calendar cal = Calendar.getInstance();
		cal.setTime(timestamp);
		newTime += (cal.get(Calendar.ZONE_OFFSET) + cal.get(Calendar.DST_OFFSET));

		newTime %= (1000 * 3600 * 24 * 7);
		// add 4 days so middle of weekend is first date
		newTime += 4 * 24 * 3600 * 1000;
		// move to epoch again
		newTime %= 1000 * 3600 * 24 * 7;

		XAttribute a2 = new XLightTimestampAttribute(weekTime, newTime);
		Date start = new Date(timestamp.getYear(), timestamp.getMonth(), timestamp.getDate());
		newTime = timestamp.getTime() - start.getTime();
		//		
		//		newTime += cal.get(Calendar.ZONE_OFFSET) + cal.get(Calendar.DST_OFFSET);
		//		newTime %= (1000 * 3600 * 24);

		//		if (newTime > 12 * 3600 * 1000 && newTime < 13 * 3600 * 1000) {
		//			System.out.println(newTime);
		//		}

		XAttribute a3 = new XLightTimestampAttribute(dayTime, newTime);

		assert timestamp.getTime() - startCurrentTrace >= 0;
		XAttribute a4 = new XLightTimestampAttribute(caseTime, timestamp.getTime() - startCurrentTrace);

		assert timestamp.getTime() >= startCurrentTrace;

		return new XAttribute[] { a1, a2, a3, a4 };

	}
}