package org.processmining.ocbc.performance.logprojection.calculators;

import org.deckfour.xes.model.XAttribute;
import org.deckfour.xes.model.XTrace;
import org.processmining.ocbc.performance.logprojection.Attribute;
import org.processmining.ocbc.performance.logprojection.LogView;
import org.processmining.ocbc.performance.logprojection.DataManager.AttributeValueCalculator;

public class StartEndTimeTraceCalculator implements AttributeValueCalculator<XTrace> {

	private final Attribute eventTimestamp;
	private final LogView view;
	private final String startTime;
	private final String endTime;
	private String duration;

	public StartEndTimeTraceCalculator(LogView view, Attribute eventTimestamp, String startTime, String endTime,
			String duration) {
		this.view = view;
		this.eventTimestamp = eventTimestamp;
		this.startTime = startTime;
		this.endTime = endTime;
		this.duration = duration;
	}

	@Override
	public XAttribute[] computeAttributes(XTrace trace, int traceIndex, int... header) {

		int size = header[0];

		long min = Long.MAX_VALUE;
		long max = Long.MIN_VALUE;

		int start = view.getFirstEventNumber(traceIndex);
		int attrIndex = view.getEventDataManager().getKeyIndex(eventTimestamp);

		for (int i = 0; i < size; i++) {
			long val = view.getEventDataManager().getAttributeValueTimestamp(start + i, attrIndex);
			if (val < min) {
				min = val;
			}
			if (val > max) {
				max = val;
			}
		}
		if (size ==0) {
			// handle empty traces
			min = 0;
			max = 0;
		}

		XAttribute a1 = new XLightTimestampAttribute(startTime, min);
		XAttribute a2 = new XLightTimestampAttribute(endTime, max);
		XAttribute a3 = new XLightTimestampAttribute(duration, max - min);

		assert max >= min;

		return new XAttribute[] { a1, a2, a3 };
	}

}
