package org.processmining.ocbc.performance.logprojection.filters;

import org.processmining.ocbc.performance.logprojection.Attribute;
import org.processmining.ocbc.performance.logprojection.LogView;

public final class ContinuousRangeAttributeFilter extends AbstractEventFilter {

	private double min = Double.NEGATIVE_INFINITY;
	private double max = Double.POSITIVE_INFINITY;
	private final boolean inverted;

	/**
	 * Keeps events as long as the value of the given attribute is the range
	 * [min, max] (inclusive) If inverted is set to true, then events are kept
	 * as long as the value of the given attribute is outside the range [min,
	 * max] (exclusive)
	 * 
	 * @param view
	 * @param attribute
	 *            The attribute to be checked for each event. Note that this has
	 *            to be a continuous attribute, i.e. the type should be
	 *            Attribute.CONT. Otherwise no events are filtered (or all in
	 *            case inverted is true)
	 * @param isEventAttribute
	 * @param min
	 * @param max
	 * @param inverted
	 */
	public ContinuousRangeAttributeFilter(LogView view, Attribute attribute,
			double min, double max, boolean inverted) {
		super(view, attribute);
		this.inverted = inverted;
		if ((attribute.getType() & Attribute.CONTINUOUS_FLAG) == Attribute.CONTINUOUS_FLAG) {
			this.min = min;
			this.max = max;
		}
	}

	@Override
	public boolean keepEvent(LogView view, int traceIndex, int eventIndex) {
		double val;
		if (attribute.isEventAttribute()) {
			val = getValue(eventIndex);
		} else {
			val = getValue(traceIndex);
		}
		if (inverted) {
			return val < min || val > max;
		}
		return val >= min && val <= max;
	}

	private double getValue(int objectIndex) {
		switch (attribute.getType()) {
		case Attribute.CONT:
			return manager.getAttributeValueContinuous(objectIndex,
					attributeIndex);
		default:
			return min;
		}
	}

}
