package org.processmining.ocbc.performance.logprojection.lightxlog;

import gnu.trove.map.TIntIntMap;
import gnu.trove.map.TObjectIntMap;
import gnu.trove.map.hash.TIntIntHashMap;
import gnu.trove.map.hash.TObjectIntHashMap;

import java.util.Arrays;
import java.util.Iterator;

import org.processmining.ocbc.performance.logprojection.LogView;

public class LightLog extends AbstractAttributable implements
		Iterable<LightTrace> {

	public static final int NONAME = -3;
	private LightTrace[] traces;
	private int numTraces;
	private final String name;
	private String[] int2EventClass = null;
	private TObjectIntMap<String> eventClass2int = null;
	private int numEventClasses = -1;

	public LightLog(LogView view, String name, LightTrace... traces) {
		super(view, view.getLogDataManager(), 0);
		this.name = name;
		this.numTraces = traces.length;
		this.traces = traces;
		computeEventClasses();
	}

	private void computeEventClasses() {
		numEventClasses = 0;
		int2EventClass = new String[10];
		eventClass2int = new TObjectIntHashMap<String>(10, 0.7f, NONAME);

		TIntIntMap old2new = new TIntIntHashMap(10, 0.7f, -1, -1);

		int next = 0;
		for (int t = 0; t < numTraces; t++) {
			for (int e = 0; e < traces[t].size(); e++) {
				int oldN = traces[t].getEvent(e).getNameIndex();
				int n = old2new.get(oldN);
				if (n < 0) {
					old2new.put(oldN, next);
					n = next;
					next++;
					numEventClasses++;
				}
				traces[t].getEvent(e).setNameIndex(n);
				String name = traces[t].getEvent(e).getName();
				if (n >= int2EventClass.length) {
					int2EventClass = Arrays.copyOfRange(int2EventClass, 0,
							n + 16);
				}
				int2EventClass[n] = name;
				eventClass2int.put(name, n);
			}
		}
		int2EventClass = Arrays.copyOfRange(int2EventClass, 0, numEventClasses);
	}

	public LightLog(LogView view, String name, int numTraces) {
		super(view, view.getLogDataManager(), 0);
		this.name = name;
		this.numTraces = 0;
		this.traces = new LightTrace[numTraces];
	}

	public String getName() {
		return name;
	}

	public String toString() {
		return getName();
	}

	public boolean equals(Object o) {
		return o == this;
	}

	public int hashCode() {
		return super.hashCode();
	}

	@Override
	public Iterator<LightTrace> iterator() {
		return new Iterator<LightTrace>() {

			private int next = 0;

			@Override
			public boolean hasNext() {
				return next < numTraces;
			}

			@Override
			public LightTrace next() {
				return traces[next++];
			}

			@Override
			public void remove() {
				throw new UnsupportedOperationException(
						"Cannot remove events from traces");
			}

		};
	}

	public void addTrace(LightTrace trace) {
		if (traces.length == numTraces) {
			// grow the array by 16 traces
			traces = Arrays.copyOf(traces, numTraces + 16);
		}
		traces[numTraces++] = trace;
	}

	public LightTrace getTrace(int i) {
		return traces[i];
	}

	public int size() {
		return numTraces;
	}

	public String[] getInt2EventClass() {
		if (int2EventClass == null) {
			computeEventClasses();
		}
		return int2EventClass;
	}

	public int getNumEventClasses() {
		if (numEventClasses < 0) {
			computeEventClasses();
		}
		return numEventClasses;
	}

	public void invalidate() {
		this.int2EventClass = null;
		this.numEventClasses = -1;
	}

	public TObjectIntMap<String> getName2EventIndex() {
		return eventClass2int;
	}

}
