package org.processmining.ocbc.performance.logprojection.plugins.dottedchart;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.Stroke;
import java.awt.geom.GeneralPath;

import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.plaf.PanelUI;

import org.jfree.chart.renderer.xy.XYItemRenderer;
import org.jfree.data.xy.XYDataset;

/**
 * A general class for plotting data in the form of (x, y) pairs. This plot can
 * use data from any class that implements the {@link XYDataset} interface.
 * <P>
 * <code>XYPlot</code> makes use of an {@link XYItemRenderer} to draw each point
 * on the plot. By using different renderers, various chart types can be
 * produced.
 * <p>
 * The {@link org.jfree.chart.ChartFactory} class contains static methods for
 * creating pre-configured charts.
 */
public class IntensityChart extends PanelUI {

	private JLabel label = new JLabel();

	/** The default grid line stroke. */
	public static final Stroke DEFAULT_STROKE = new BasicStroke(1.0f, BasicStroke.CAP_BUTT, BasicStroke.JOIN_BEVEL,
			0.0f);

	private int[][] data;

	private int height;

	private DottedChart chart;

	private int width;

	private int max;

	private Color[] colorMap;

	private double dataAreaXOffset;

	private int[] lines;

	public IntensityChart(DottedChart chart, int height, int width) {
		super();
		this.chart = chart;
		this.height = height;
		this.width = width;
	}

	public void setData(int[][] data, int max, Color[] colorMap, double dataAreaXOffset) {
		this.data = data;
		this.max = max;
		this.colorMap = colorMap;
		this.dataAreaXOffset = dataAreaXOffset;
		this.label.setVerticalAlignment(JLabel.TOP);
	}

	public void setLines(int... lines) {
		this.lines = lines;

	}

	/**
	 * Returns the specified component's preferred size appropriate for the look
	 * and feel. If <code>null</code> is returned, the preferred size will be
	 * calculated by the component's layout manager instead (this is the
	 * preferred approach for any component with a specific layout manager
	 * installed). The default implementation of this method returns
	 * <code>null</code>.
	 * 
	 * @param c
	 *            the component whose preferred size is being queried; this
	 *            argument is often ignored, but might be used if the UI object
	 *            is stateless and shared by multiple components
	 * 
	 * @see javax.swing.JComponent#getPreferredSize
	 * @see java.awt.LayoutManager#preferredLayoutSize
	 */
	public Dimension getPreferredSize(JComponent c) {
		return new Dimension(width, height);
	}

	/**
	 * Paints the specified component appropriately for the look and feel. This
	 * method is invoked from the <code>ComponentUI.update</code> method when
	 * the specified component is being painted. Subclasses should override this
	 * method and use the specified <code>Graphics</code> object to render the
	 * content of the component.
	 * 
	 * @param g
	 *            the <code>Graphics</code> context in which to paint
	 * @param c
	 *            the component being painted; this argument is often ignored,
	 *            but might be used if the UI object is stateless and shared by
	 *            multiple components
	 * 
	 * @see #update
	 */
	public void paint(Graphics g, JComponent c) {

		if (data == null) {
			super.paint(g, c);
			return;
		}
		label.setText("<html>" + chart.getPercentageRendered() + " % rendered" + "<br>in " + chart.getRenderTime()
				+ " sec.</html>");
		label.setSize(c.getSize());

		label.getUI().paint(g, label);

		Graphics2D g2 = (Graphics2D) g;
		// g2.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER,
		// 0));
		g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);

		double w = (c.getWidth() - dataAreaXOffset);
		double wf = w / (2 * data.length);

		double h = c.getHeight();

		double hf = max > 0 ? h / Math.log1p(max) : h;

		for (int line = 0; line < colorMap.length; line++) {
			GeneralPath p = new GeneralPath();

			double y = h - Math.log1p(data[0][line]) * hf;
			double x = dataAreaXOffset;
			p.moveTo(x, h);

			double v = h;
			for (int i = 0; i < data.length; i++) {
				v = data[i][line];
				if (v == 0) {
					v = h;
				} else {
					assert v <= max;
					v = h - Math.log1p(v) * hf;
				}
				// draw the data point
				p.curveTo(x, y, x, v, x + wf, v);

				// p.lineTo(x+wf, v);
				y = v;
				x += 2 * wf;
			}
			x -= wf;
			p.curveTo(x + wf, y, x + wf, v, x + wf, h);
			p.moveTo(dataAreaXOffset, h);

			g2.setStroke(DEFAULT_STROKE);
			Color color = new Color(colorMap[line].getRed(), colorMap[line].getGreen(), colorMap[line].getBlue());
			g2.setColor(color);
			g2.draw(p);
			g2.setColor(new Color(colorMap[line].getRed(), colorMap[line].getGreen(), colorMap[line].getBlue(), 16));
			g2.fill(p);

		}
		// int transX = (int) (chart.getDomainAxis().valueToJava2D(x, dataArea,
		// RectangleEdge.BOTTOM));

	}

}
