package org.processmining.ocbc.performance.logprojection.plugins.dottedchart.filtering;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import javax.swing.JComboBox;
import javax.swing.JPanel;

import org.processmining.ocbc.performance.logprojection.Attribute;
import org.processmining.ocbc.performance.logprojection.LogView;
import org.processmining.ocbc.performance.logprojection.filters.EventFilter;
import org.processmining.ocbc.performance.logprojection.filters.TraceFilter;

public abstract class AbstractAttributeFilterPanel<V> extends JPanel implements
		AttributeFilterPanel, ItemListener {

	private static final long serialVersionUID = 358268605631483900L;
	protected final LogView view;
	protected final AttributeComboBoxModel model;
	private final Map<Attribute, V> attribute2selected;

	private List<EventFilter> toAdd;
	private List<EventFilter> toRemove;
	protected JComboBox<Attribute> attributeComboBox;

	protected Attribute attribute;
	protected JPanel northPanel;
	private Type type;

	public AbstractAttributeFilterPanel(LogView view, byte attributeType,
			Map<Attribute, V> attribute2selected, Type type) {
		this.view = view;
		this.attribute2selected = attribute2selected;
		this.type = type;

		setLayout(new BorderLayout());
		northPanel = new JPanel(new FlowLayout(FlowLayout.LEFT));
		model = new AttributeComboBoxModel(view, attributeType);
		if (model.getSize()==0) {
			// empty model
			model.addNone();
		}
		attributeComboBox = new JComboBox<Attribute>(model);
		
		northPanel.add(attributeComboBox);
		attributeComboBox.addItemListener(this);
		this.add(northPanel, BorderLayout.NORTH);

	}

	@Override
	public void itemStateChanged(ItemEvent e) {
		Attribute a = (Attribute) e.getItem();
		if (e.getStateChange() == ItemEvent.DESELECTED) {
			attributeDeselected(a);
			return;
		}
		// new attribute selected
		attributeSelected(a);
	}

	public Map<Attribute, V> getAttribute2Selected() {
		return attribute2selected;
	}

	@Override
	public void closing() {
		Attribute a = model.getSelectedItem();
		if (a != null) {
			V chosen = getSelectedValue();
			attribute2selected.put(a, chosen);
		}
		buildEventFilters();
	}

	@Override
	public void opening() {
		setAttribute(model.getSelectedItem(),
				attribute2selected.get(model.getSelectedItem()));
	}

	private void buildEventFilters() {

		toRemove = new ArrayList<EventFilter>(attribute2selected.size());
		toAdd = new ArrayList<EventFilter>(attribute2selected.size());

		// convert the map
		Iterator<Entry<Attribute, V>> it = attribute2selected.entrySet()
				.iterator();
		while (it.hasNext()) {
			Entry<Attribute, V> e = it.next();
			if (e.getValue() == null) {
				toRemove.add(getTypedEventFilter(e.getKey(), null));
			} else {
				toAdd.add(getTypedEventFilter(e.getKey(), e.getValue()));
			}
		}

	}

	protected void attributeDeselected(Attribute a) {
		attribute2selected.put(a, getSelectedValue());
	}

	protected void attributeSelected(Attribute a) {
		setAttribute(a, attribute2selected.get(a));
	}

	@Override
	public List<EventFilter> getFiltersToAdd() {
		return toAdd;
	}

	@Override
	public List<EventFilter> getFiltersToRemove() {
		return toRemove;
	}

	@Override
	public Type getType() {
		return type;
	}

	public abstract V getSelectedValue();

	protected abstract void setAttribute(Attribute a, V selectedValue);

	private EventFilter getTypedEventFilter(Attribute a, V selectedValue) {
		EventFilter filter = getEventFilter(a, selectedValue);
		switch (type) {
		case EVENT:
			return filter;
		case FIRSTEVENTINTRACE:
			return new TraceFilter.First(filter);
		case LASTEVENTINTRACE:
			return new TraceFilter.Last(filter);
		case ANYEVENTINTRACE:
			return new TraceFilter.Any(filter);
		default:
			return null;
		}
	}

	protected abstract EventFilter getEventFilter(Attribute a, V selectedValue);

	public int getAttributeCount() {
		return attributeComboBox.getItemCount();
	}

}
