package org.processmining.ocbc.performance.logprojection.plugins.dottedchart.filtering;

import java.awt.Dimension;
import java.awt.GridLayout;
import java.util.Hashtable;
import java.util.Map;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.plaf.basic.BasicSliderUI;

import org.processmining.ocbc.performance.logprojection.Attribute;
import org.processmining.ocbc.performance.logprojection.Histogram;
import org.processmining.ocbc.performance.logprojection.LogView;
import org.processmining.ocbc.performance.logprojection.Statistics;

public abstract class AbstractRangeAttributeFilterPanel<V> extends
		AbstractAttributeFilterPanel<V> implements ChangeListener {

	private static final long serialVersionUID = -4475556594641291329L;
	private static final String[] AND = new String[] {
			"Keep if value is more than:", "and less than:" };
	private static final String[] OR = new String[] {
			"Keep if value is more than:", "or less than:" };

	protected static final int MAJORTICKSIZE = 10000;

	protected JLabel[] labels = new JLabel[11];

	public static class MySliderUI extends BasicSliderUI {

		private static final long serialVersionUID = 4618626691985972748L;

		public MySliderUI(JSlider slider) {
			super(slider);
		}

		public double getTrackWidth() {
			return trackRect.getWidth();
		}

	}

	protected JSlider minSlider;
	protected JSlider maxSlider;
	private JLabel middleLabel;
	private JPanel sliderPanel;
	protected double conversion;
	protected double minimum;
	private JLabel topLabel;
	private JLabel minSliderValue;
	private JLabel maxSliderValue;
	private HistogramPanel histogramPanel;

	public AbstractRangeAttributeFilterPanel(LogView view, byte attributeType,
			Map<Attribute, V> map, Type type) {
		super(view, attributeType, map, type);

		sliderPanel = new JPanel(new GridLayout(7, 1));

		topLabel = new JLabel(AND[0]);
		topLabel.setHorizontalAlignment(JLabel.CENTER);
		sliderPanel.add(topLabel);

		histogramPanel = new HistogramPanel();
		sliderPanel.add(histogramPanel);

		Dimension d = new Dimension(200, 40);
		Hashtable<Integer, JLabel> labelTable = new Hashtable<Integer, JLabel>();
		for (int i = 0; i < 11; i++) {
			labels[i] = new JLabel("" + i);
			if (i % 2 == 1) {
				labels[i].setVerticalAlignment(JLabel.BOTTOM);
			} else {
				labels[i].setVerticalAlignment(JLabel.TOP);
			}
			labels[i].setHorizontalAlignment(JLabel.CENTER);
			labels[i].setSize(d);
			labels[i].setMinimumSize(d);
			labels[i].setPreferredSize(d);
			labelTable.put(i * MAJORTICKSIZE, labels[i]);
		}

		minSlider = makeSlider(0, labelTable);

		minSliderValue = new JLabel("");

		middleLabel = new JLabel(AND[1]);

		maxSlider = makeSlider(MAJORTICKSIZE * 10, labelTable);

		maxSliderValue = new JLabel("");

		sliderPanel.add(minSlider);
		sliderPanel.add(minSliderValue);
		minSliderValue.setHorizontalAlignment(JLabel.CENTER);
		sliderPanel.add(middleLabel);
		middleLabel.setHorizontalAlignment(JLabel.CENTER);
		sliderPanel.add(maxSlider);
		maxSliderValue.setHorizontalAlignment(JLabel.CENTER);
		sliderPanel.add(maxSliderValue);

		add(sliderPanel);

		if (attributeComboBox.getItemCount() > 0) {
			attributeComboBox.setSelectedIndex(0);
		}

	}

	private JSlider makeSlider(int value, Hashtable<Integer, JLabel> labelTable) {
		JSlider slider = new JSlider(0, MAJORTICKSIZE * 10, value);
		slider.setMinorTickSpacing(MAJORTICKSIZE / 10);
		slider.setMajorTickSpacing(MAJORTICKSIZE);
		slider.setPaintLabels(true);
		slider.setPaintTicks(true);
		// slider.setPaintTrack(false);
		slider.addChangeListener(this);
		slider.setLabelTable(labelTable);
		slider.setUI(new MySliderUI(slider));
		return slider;
	}

	@Override
	public void stateChanged(ChangeEvent e) {
		if (maxSlider.getValue() < minSlider.getValue()) {
			topLabel.setText(OR[0]);
			middleLabel.setText(OR[1]);
		} else {
			topLabel.setText(AND[0]);
			middleLabel.setText(AND[1]);
		}
		double min, max;
		min = minimum + (minSlider.getValue() * conversion);
		if (e.getSource() == minSlider) {
			minSliderValue.setText(format(min));
		}
		max = (minimum + (maxSlider.getValue() * conversion));
		if (e.getSource() == maxSlider) {
			maxSliderValue.setText(format(max));
		}
		histogramPanel.setRange(min, max,
				maxSlider.getValue() < minSlider.getValue());

	}

	@Override
	protected void setAttribute(Attribute a, V selectedValue) {
		if (a != null) {
			Statistics s = getStatistics(a);
			Histogram h = getHistogram(a, s, 50);
			histogramPanel.setDrawWidth((int) ((MySliderUI) minSlider.getUI())
					.getTrackWidth());

			double length = s.getMaximumDouble() - s.getMinimumDouble();
			// we divide length over the 10000 steps in the slider.
			minimum = s.getMinimumDouble();
			conversion = length / (MAJORTICKSIZE * 10.0);
			for (int i = 0; i < 11; i++) {
				labels[i]
						.setText(format((minimum + (i * MAJORTICKSIZE * conversion))));
				// labels[i].setSize(200, 50);
				// labels[i].repaint();
			}
			if (selectedValue == null) {
				minSlider.setValue(0);
				maxSlider.setValue(MAJORTICKSIZE * 10);
				histogramPanel.setHistogram(h, s.getMinimumDouble(),
						s.getMaximumDouble(), false);
			} else {
				setSliderValues(a, selectedValue);
				histogramPanel.setHistogram(h, minimum + minSlider.getValue()
						* conversion, minimum + maxSlider.getValue()
						* conversion,
						maxSlider.getValue() < minSlider.getValue());

			}
			minSlider.revalidate();
			maxSlider.revalidate();
			minSlider.repaint();
			maxSlider.repaint();
		}
	}

	protected abstract Histogram getHistogram(Attribute a, Statistics stats,
			int buckets);

	protected abstract void setSliderValues(Attribute a, V selectedValue);

	protected abstract Statistics getStatistics(Attribute a);

	protected abstract String format(double tick);

}
