package org.processmining.ocbc.performance.logprojection.plugins.dottedchart.filtering;

import java.awt.BorderLayout;
import java.util.HashMap;
import java.util.Map;

import javax.swing.DefaultListModel;
import javax.swing.JList;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;

import org.processmining.ocbc.performance.logprojection.Attribute;
import org.processmining.ocbc.performance.logprojection.DataManager;
import org.processmining.ocbc.performance.logprojection.LogView;
import org.processmining.ocbc.performance.logprojection.filters.BooleanAttributeFilter;
import org.processmining.ocbc.performance.logprojection.filters.EventFilter;
import org.processmining.ocbc.performance.logprojection.filters.LiteralAttributeFilter;

public class BooleanAttributeFilterPanel extends
		AbstractAttributeFilterPanel<int[]> {

	private static final long serialVersionUID = 358268605631483900L;

	private boolean[] values;
	private JList<Boolean> list;
	private DefaultListModel<Boolean> listModel;

	public BooleanAttributeFilterPanel(LogView view, Type type) {
		this(view, new HashMap<Attribute, int[]>(), type);
	}

	public BooleanAttributeFilterPanel(LogView view,
			Map<Attribute, int[]> attribute2selected, Type type) {
		super(view, Attribute.BOOL, attribute2selected, type);

		listModel = new DefaultListModel<Boolean>();
		list = new JList<Boolean>(listModel);
		this.add(new JScrollPane(list), BorderLayout.CENTER);

		if (attributeComboBox.getItemCount() > 0) {
			attributeComboBox.setSelectedIndex(0);
		}
	}

	public int[] getSelectedValue() {
		int[] sel = list.getSelectedIndices();
		if (sel.length == values.length) {
			return null;
		} else {
			return sel;
		}
	}

	protected void setAttribute(Attribute a, int[] selectedIndices) {
		this.attribute = a;
		DataManager<?> manager;
		if (a.isEventAttribute()) {
			manager = view.getEventDataManager();
		} else {
			manager = view.getTraceDataManager();
		}

		values = manager.getAttributeValuesBoolean(attribute, true);

		listModel.removeAllElements();
		for (Boolean s : values) {
			listModel.addElement(s);
		}
		list.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);

		if (selectedIndices == null) {
			selectedIndices = new int[values.length];
			for (int i = 0; i < selectedIndices.length; i++) {
				selectedIndices[i] = i;
			}
		}
		list.setSelectedIndices(selectedIndices);
		list.revalidate();
		this.revalidate();
	}

	@Override
	protected EventFilter getEventFilter(Attribute a, int[] selectedIndices) {
		if (selectedIndices == null) {
			return new LiteralAttributeFilter(view, a, new int[0]);
		}
		DataManager<?> manager;
		if (a.isEventAttribute()) {
			manager = view.getEventDataManager();
		} else {
			manager = view.getTraceDataManager();
		}
		boolean keepTrue = false;
		boolean keepFalse = false;
		boolean[] values = manager.getAttributeValuesBoolean(a, true);
		for (int i = 0; i < selectedIndices.length; i++) {
			keepTrue |= values[selectedIndices[i]] == true;
			keepFalse |= values[selectedIndices[i]] == false;
		}
		return new BooleanAttributeFilter(view, a, keepTrue, keepFalse);
	}

}
