package org.processmining.ocbc.performance.logprojection.plugins.dottedchart.filtering;

import java.text.DecimalFormat;
import java.util.HashMap;

import javax.swing.event.ChangeListener;

import org.processmining.ocbc.performance.logprojection.Attribute;
import org.processmining.ocbc.performance.logprojection.Histogram;
import org.processmining.ocbc.performance.logprojection.LogView;
import org.processmining.ocbc.performance.logprojection.Statistics;
import org.processmining.ocbc.performance.logprojection.filters.ContinuousRangeAttributeFilter;
import org.processmining.ocbc.performance.logprojection.filters.EventFilter;
import org.processmining.ocbc.performance.logprojection.plugins.dottedchart.filtering.AttributeFilterPanel.Type;

public class ContinuousAttributeFilterPanel extends
		AbstractRangeAttributeFilterPanel<double[]> implements ChangeListener {

	private static final long serialVersionUID = 5500143718929691982L;

	private static final DecimalFormat formatter = new DecimalFormat(
			"0.#####E0");

	protected double conversion;
	protected double minimum;

	public ContinuousAttributeFilterPanel(LogView view, Type type) {
		super(view, Attribute.CONT, new HashMap<Attribute, double[]>(5), type);

	}

	@Override
	public double[] getSelectedValue() {
		if (minSlider.getValue() == 0
				&& maxSlider.getValue() == MAJORTICKSIZE * 10) {
			return null;
		}
		return new double[] { minimum + minSlider.getValue() * conversion,
				minimum + maxSlider.getValue() * conversion };
	}

	@Override
	protected EventFilter getEventFilter(Attribute a, double[] selectedValue) {
		if (selectedValue == null) {
			return new ContinuousRangeAttributeFilter(view, a, 0, 0, false);
		} else {
			if (selectedValue[0] < selectedValue[1]) {
				return new ContinuousRangeAttributeFilter(view, a,
						selectedValue[0], selectedValue[1], false);
			} else {
				return new ContinuousRangeAttributeFilter(view, a,
						selectedValue[1], selectedValue[0], true);
			}
		}
	}

	@Override
	protected String format(double tick) {
		return formatter.format(tick);
	}

	@Override
	protected Statistics getStatistics(Attribute a) {
		return view.getStatisticsContinuous(a);
	}

	@Override
	protected void setSliderValues(Attribute a, double[] selectedValue) {
		minSlider.setValue((int) ((selectedValue[0] - minimum) / conversion));
		maxSlider.setValue((int) ((selectedValue[1] - minimum) / conversion));
	}

	@Override
	protected Histogram getHistogram(Attribute a, Statistics stats, int buckets) {
		return view.getHistogramContinuous(a, stats, buckets);
	}

}
