package org.processmining.ocbc.performance.logprojection.plugins.dottedchart.filtering;

import java.text.DecimalFormat;
import java.util.HashMap;

import javax.swing.event.ChangeListener;

import org.processmining.ocbc.performance.logprojection.Attribute;
import org.processmining.ocbc.performance.logprojection.Histogram;
import org.processmining.ocbc.performance.logprojection.LogView;
import org.processmining.ocbc.performance.logprojection.Statistics;
import org.processmining.ocbc.performance.logprojection.filters.DiscreteRangeAttributeFilter;
import org.processmining.ocbc.performance.logprojection.filters.EventFilter;

public class DiscreteAttributeFilterPanel extends
		AbstractRangeAttributeFilterPanel<long[]> implements ChangeListener {

	private static final long serialVersionUID = 5500143718929691982L;

	private static final DecimalFormat formatter = new DecimalFormat("00000E0");

	public DiscreteAttributeFilterPanel(LogView view, Type type) {
		super(view, Attribute.DISC, new HashMap<Attribute, long[]>(5), type);

	}

	@Override
	public long[] getSelectedValue() {
		if (minSlider.getValue() == 0
				&& maxSlider.getValue() == MAJORTICKSIZE * 10) {
			return null;
		}
		return new long[] {
				(long) (minimum + minSlider.getValue() * conversion),
				(long) (minimum + maxSlider.getValue() * conversion) };
	}

	@Override
	protected EventFilter getEventFilter(Attribute a, long[] selectedValue) {
		if (selectedValue == null) {
			return new DiscreteRangeAttributeFilter(view, a, 0, 0, false);
		} else {
			if (selectedValue[0] < selectedValue[1]) {
				return new DiscreteRangeAttributeFilter(view, a,
						selectedValue[0], selectedValue[1], false);
			} else {
				return new DiscreteRangeAttributeFilter(view, a,
						selectedValue[1], selectedValue[0], true);
			}
		}
	}

	@Override
	protected String format(double tick) {
		return formatter.format((long) tick);
	}

	@Override
	protected Statistics getStatistics(Attribute a) {
		return view.getStatisticsDiscrete(a);
	}

	@Override
	protected void setSliderValues(Attribute a, long[] selectedValue) {
		minSlider.setValue((int) ((selectedValue[0] - minimum) / conversion));
		maxSlider.setValue((int) ((selectedValue[1] - minimum) / conversion));
	}

	@Override
	protected Histogram getHistogram(Attribute a, Statistics stats, int buckets) {
		return view.getHistogramDiscrete(a, stats, buckets);
	}

}
