package org.processmining.ocbc.performance.logprojection.plugins.dottedchart.filtering;

import java.awt.BasicStroke;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Stroke;

import javax.swing.JPanel;

import org.processmining.ocbc.performance.logprojection.Histogram;

public class HistogramPanel extends JPanel {
	public static final Stroke DEFAULT_STROKE = new BasicStroke(1.0f,
			BasicStroke.CAP_BUTT, BasicStroke.JOIN_BEVEL, 0.0f);

	private static final long serialVersionUID = 6221012110480829177L;

	private static final Color DEFAULT_FILL = new Color(255, 0, 0, 100);
	private static final Color DEFAULT_COLOR = new Color(255, 0, 0);

	private Histogram histogram;
	private int buckets;
	private double bucketWidth;
	private double maxBucketSize;
	private double max;

	private double min;

	private boolean inverted;

	private int drawWidth;

	public HistogramPanel() {
		super();
		this.setPreferredSize(new Dimension(100, 25));
	}

	public void setHistogram(Histogram histogram, double min, double max,
			boolean inverted) {
		this.histogram = histogram;
		buckets = histogram.getHistogram().length;
		bucketWidth = histogram.getBucketWidth();
		int maxBucketSize = histogram.getHistogram()[0];
		for (int i = 0; i < buckets; i++) {
			if (histogram.getHistogram()[i] > maxBucketSize) {
				maxBucketSize = histogram.getHistogram()[i];
			}
		}
		this.maxBucketSize = maxBucketSize;
		this.setRange(min, max, inverted);
	}

	@Override
	protected void paintChildren(Graphics g) {
		Graphics2D g2 = (Graphics2D) g;
		g2.setStroke(DEFAULT_STROKE);

		int w = drawWidth;
		double x = (getWidth() - drawWidth) / 2.0;
		int y = getHeight();
		if (buckets == 0) {
			return;
		}
		double bw = w / (double) buckets;
		if (bw == 0) {
			return;
		}
		double hf = (double) getHeight() / (double) maxBucketSize;
		// paint the Histogram
		if (inverted) {
			int last = histogram.getBucketFor(min);
			int xc0 = (int) (x);
			int xc1;
			for (int i = 0; i < last; i++) {
				xc1 = (int) (x + (i + 1) * bw);

				int h = (int) (histogram.getHistogram()[i] * hf);
				if (h == 0 && histogram.getHistogram()[i] > 0) {
					h = 1;
				}

				g2.setColor(DEFAULT_FILL);
				g2.fillRect(xc0, y - h, xc1 - xc0, h);
				g2.setColor(DEFAULT_COLOR);
				g2.drawRect(xc0, y - h, xc1 - xc0, h);
				xc0 = xc1;

			}
			xc0 = (int) (x + histogram.getBucketFor(max) * bw);
			for (int i = histogram.getBucketFor(max); i < buckets; i++) {
				xc1 = (int) (x + (i + 1) * bw);

				int h = (int) (histogram.getHistogram()[i] * hf);
				if (h == 0 && histogram.getHistogram()[i] > 0) {
					h = 1;
				}

				g2.setColor(DEFAULT_FILL);
				g2.fillRect(xc0, y - h, xc1 - xc0, h);
				g2.setColor(DEFAULT_COLOR);
				g2.drawRect(xc0, y - h, xc1 - xc0, h);
				xc0 = xc1;
			}

		} else {
			int start = histogram.getBucketFor(min);
			int xc0 = (int) (x + start * bw);
			int xc1;
			int last = histogram.getBucketFor(max);
			// System.out.println();
			for (int i = start; i < last; i++) {
				xc1 = (int) (x + (i + 1) * bw);

				int h = (int) (histogram.getHistogram()[i] * hf);
				if (h == 0 && histogram.getHistogram()[i] > 0) {
					h = 1;
				}
				// System.out.print("(" + i + "," + h + "),");

				g2.setColor(DEFAULT_FILL);
				g2.fillRect(xc0, y - h, xc1 - xc0, h);
				g2.setColor(DEFAULT_COLOR);
				g2.drawRect(xc0, y - h, xc1 - xc0, h);
				xc0 = xc1;
			}
			// System.out.println();
		}

	}

	public void setRange(double min, double max, boolean inverted) {
		if (min <= max) {
			this.min = min;
			this.max = max;
		} else {
			this.min = max;
			this.max = min;
		}
		this.inverted = inverted;
		invalidate();
		repaint();
	}

	public void setDrawWidth(int width) {
		drawWidth = width;
	}
}
