package org.processmining.ocbc.performance.logprojection.plugins.dottedchart.ui;

import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.MouseEvent;
import java.util.concurrent.ExecutorService;

import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPopupMenu;
import javax.swing.SwingUtilities;

import org.deckfour.xes.model.XLog;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.entity.ChartEntity;
import org.jfree.chart.entity.EntityCollection;
import org.jfree.chart.event.PlotChangeEvent;
import org.jfree.chart.event.PlotChangeListener;
import org.processmining.ocbc.performance.logprojection.plugins.dottedchart.DottedChart;
import org.processmining.ocbc.performance.logprojection.plugins.dottedchart.DottedChart.DottedChartException;
import org.processmining.ocbc.performance.logprojection.plugins.dottedchart.filtering.AttributeFilterDialog;

public class ToolTipChartPanel extends ChartPanel implements PlotChangeListener {
	private static final long serialVersionUID = 8912656236354079153L;
	private DottedChart dottedChart;
	private JMenuItem all2xLogItem;
	private JMenuItem view2xLogItem;

	public static final String VIEW_TO_XLOG_COMMAND = "VIEW_TO_XLOG";
	public static final String ALL_TO_XLOG_COMMAND = "ALL_TO_XLOG";
	private static final String FILTER_COMMAND = "FILTER_LOG";
	//	private static final String VIEW_TO_MINE_COMMAND = "MINE_VIEW";
	//	private static final String ALL_TO_MINE_COMMAND = "MINE_ALL";
	private ExportListener exportListener;
	private JMenuItem filterItem;
	private final AttributeFilterDialog filterDialog;
	//	private JMenuItem view2mineItem;
	//	private JMenuItem all2mineItem;
	private final ExecutorService executor;

	public ToolTipChartPanel(JFreeChart chart, DottedChart dottedChart, ExecutorService executor) {
		super(chart, DEFAULT_WIDTH, DEFAULT_HEIGHT, DEFAULT_MINIMUM_DRAW_WIDTH, DEFAULT_MINIMUM_DRAW_HEIGHT,
				DEFAULT_MAXIMUM_DRAW_WIDTH, DEFAULT_MAXIMUM_DRAW_HEIGHT, true, // use double buffering
				false, // properties
				true, // save
				true, // print
				true, // zoom
				true // tooltips
		);
		this.dottedChart = dottedChart;
		this.executor = executor;
		this.dottedChart.addChangeListener(this);
		this.setMouseWheelEnabled(true);
		filterDialog = new AttributeFilterDialog((JFrame) SwingUtilities.getWindowAncestor(this), dottedChart);

	}

//	@Override
//	public String getToolTipText(MouseEvent event) {
//		String result = null;
//		if (this.getChartRenderingInfo() != null) {
//			EntityCollection entities = this.getChartRenderingInfo().getEntityCollection();
//			if (entities != null) {
//				Insets insets = getInsets();
//				ChartEntity entity = entities.getEntity((int) ((event.getX() - insets.left) / this.getScaleX()),
//						(int) ((event.getY() - insets.top) / this.getScaleY()));
//				if (entity != null) {
//					result = dottedChart.getToolTipForScreenLocation(this, event);
//				}
//			}
//		}
//		return result;
//	}
	
	@Override
	/**
	 * show the tip text when the cursor is on the dot;
	 * modified by Guangming on June 18th, 2018
	 */
	public String getToolTipText(MouseEvent event) {
		String result = null;
		if (this.getChartRenderingInfo() != null) {
			EntityCollection entities = this.getChartRenderingInfo().getEntityCollection();
			if (entities != null) {
				Insets insets = getInsets();
				ChartEntity entity = entities.getEntity((int) ((event.getX() - insets.left) / this.getScaleX()),
						(int) ((event.getY() - insets.top) / this.getScaleY()));
				if (entity != null) {
					result = dottedChart.getToolTipForScreenLocation(this, event);
				}
			}
		}
		return result;
//		return "hello";
	}

	public void setExportListener(ExportListener listener) {
		this.exportListener = listener;
		view2xLogItem.setEnabled(!dottedChart.isYAxisContinuous() && exportListener != null);
		all2xLogItem.setEnabled(!dottedChart.isYAxisContinuous() && exportListener != null);
	}

	@Override
	protected JPopupMenu createPopupMenu(boolean properties, boolean copy, boolean save, boolean print, boolean zoom) {
		// TODO: Add option to export log and edit colors. There is a nice
		// colorchooser implemented in JFreeChart
		JPopupMenu menu = super.createPopupMenu(properties, copy, save, print, zoom);
		menu.addSeparator();

		//		JMenu mineSubMenu = new JMenu("Mine Log");
		//
		//		view2mineItem = new JMenuItem("Visible events...");
		//		view2mineItem.setActionCommand(VIEW_TO_MINE_COMMAND);
		//		view2mineItem.addActionListener(this);
		//		mineSubMenu.add(view2mineItem);
		//		all2mineItem = new JMenuItem("All events...");
		//		all2mineItem.setActionCommand(ALL_TO_MINE_COMMAND);
		//		all2mineItem.addActionListener(this);
		//		mineSubMenu.add(all2mineItem);

		JMenu saveSubMenu = new JMenu("Export XLog");

		view2xLogItem = new JMenuItem("Visible events...");
		view2xLogItem.setActionCommand(VIEW_TO_XLOG_COMMAND);
		view2xLogItem.addActionListener(this);
		saveSubMenu.add(view2xLogItem);
		all2xLogItem = new JMenuItem("All events...");
		all2xLogItem.setActionCommand(ALL_TO_XLOG_COMMAND);
		all2xLogItem.addActionListener(this);
		saveSubMenu.add(all2xLogItem);

		//		menu.add(mineSubMenu);
		menu.add(saveSubMenu);

		filterItem = new JMenuItem("Filter");
		filterItem.setActionCommand(FILTER_COMMAND);
		filterItem.addActionListener(this);
		menu.add(filterItem);

		return menu;
	}

	@Override
	public void actionPerformed(ActionEvent event) {
		final String command = event.getActionCommand();

		if (command.equals(VIEW_TO_XLOG_COMMAND) || command.equals(ALL_TO_XLOG_COMMAND)
		//				|| command.equals(VIEW_TO_MINE_COMMAND)
		//				|| command.equals(ALL_TO_MINE_COMMAND)
		) {
			final String name = JOptionPane.showInputDialog("Provide a name for the new log:");
			if (name == null) {
				return;
			}
			executor.execute(new Runnable() {

				@Override
				public void run() {
					try {
						if (command.equals(VIEW_TO_XLOG_COMMAND)) {
							XLog log = dottedChart.exportViewToXLog(true, name);
							if (log != null) {
								exportListener.logExported(log);
							}
						} else if (command.equals(ALL_TO_XLOG_COMMAND)) {
							XLog log = dottedChart.exportViewToXLog(false, name);
							if (log != null) {
								exportListener.logExported(log);
							}
							//						} else if (command.equals(VIEW_TO_MINE_COMMAND)) {
							//							LightLog log = dottedChart.exportViewToLightLog(true, name);
							//							if (log != null) {
							//								exportListener.logExported(log);
							//							}
							//						} else if (command.equals(ALL_TO_MINE_COMMAND)) {
							//							LightLog log = dottedChart.exportViewToLightLog(false, name);
							//							if (log != null) {
							//								exportListener.logExported(log);
							//							}
						}
					} catch (DottedChartException e) {
						e.printStackTrace();
					}
				}

			});
		} else if (command.equals(FILTER_COMMAND)) {
			filterDialog.setVisible(true);
		} else {
			super.actionPerformed(event);
			return;
		}

	}

	@Override
	public void plotChanged(PlotChangeEvent event) {
		// enable exporting only if the y axis is discrete. This is required to
		// get case IDs
		view2xLogItem.setEnabled(!dottedChart.isYAxisContinuous() && exportListener != null);
		all2xLogItem.setEnabled(!dottedChart.isYAxisContinuous() && exportListener != null);
		//		view2mineItem.setEnabled(!dottedChart.isYAxisContinuous() && exportListener != null);
		//		all2mineItem.setEnabled(!dottedChart.isYAxisContinuous() && exportListener != null);
	}

}