package org.processmining.ocbc.performance.logprojection.sorting;

import org.processmining.ocbc.performance.logprojection.Attribute;
import org.processmining.ocbc.performance.logprojection.DataManager;
import org.processmining.ocbc.performance.logprojection.LogView;

public abstract class AbstractEventBasedTraceSorter implements TraceSorter {

	private AlphanumComparator comp = new AlphanumComparator();

	protected LogView view;
	protected Attribute attribute;

	public AbstractEventBasedTraceSorter(LogView view, Attribute attribute) {
		this.view = view;
		this.attribute = attribute;
	}

	protected int compare(DataManager<?> manager, int e1, int e2) {

		int res = 0;
		if (attribute.getType() == Attribute.CONT) {
			double d1 = manager.getAttributeValueContinuous(e1, attribute);
			double d2 = manager.getAttributeValueContinuous(e2, attribute);
			res = Double.compare(d1, d2);
		} else if (attribute.getType() == Attribute.LITR) {
			String s1 = manager.getAttributeValueLiteral(e1, attribute).trim();
			String s2 = manager.getAttributeValueLiteral(e2, attribute).trim();
			res = comp.compare(s1, s2);
		} else if (attribute.getType() == Attribute.BOOL) {
			boolean b1 = manager.getAttributeValueBoolean(e1, attribute);
			boolean b2 = manager.getAttributeValueBoolean(e2, attribute);
			res = Boolean.compare(b1, b2);
		} else if (attribute.getType() == Attribute.DISC) {
			long t1 = manager.getAttributeValueDiscrete(e1, attribute);
			long t2 = manager.getAttributeValueDiscrete(e2, attribute);
			res = Long.compare(t1, t2);
		} else if (attribute.getType() == Attribute.TIME_ABS
				|| (attribute.getType() == Attribute.TIME_REL)) {
			long t1 = manager.getAttributeValueTimestamp(e1, attribute);
			long t2 = manager.getAttributeValueTimestamp(e2, attribute);
			res = Long.compare(t1, t2);
		}
		return res;
	}

	public String toString() {
		return getDescription();
	}

	public boolean equals(Object o) {
		if (o instanceof AbstractEventBasedTraceSorter) {
			return ((AbstractEventBasedTraceSorter) o).attribute
					.equals(attribute);
		}
		return false;
	}
}
