package org.processmining.ocbc.performance.panels;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.util.ArrayList;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JScrollPane;
import javax.swing.ScrollPaneConstants;

import org.processmining.ocbc.log.model.XOCEvent;
import org.processmining.ocbc.log.model.XOCLog;
import org.processmining.ocbc.log.model.impl.xConstants;

import com.fluxicon.slickerbox.components.RoundedPanel;

public class InstancePanel extends RoundedPanel {

	
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	protected JList<Object> instancesList;
	private XOCLog log;
	boolean generateCoordinatesFlag = false;
	
	public static final Color COLOR_ENCLOSURE_BG = new Color(40, 40, 40);
	public static final Color COLOR_NON_FOCUS = new Color(70, 70, 70);
	public static final Color COLOR_LIST_BG = new Color(60, 60, 60);
	public static final Color COLOR_LIST_BG_LOWER = new Color(45, 45, 45);
	public static final Color COLOR_LIST_FG = new Color(180, 180, 180);
	public static final Color COLOR_LIST_SELECTION_BG = new Color(80, 0, 0);
	public static final Color COLOR_LIST_SELECTION_BG_LOWER = new Color(30, 10, 10);
	public static final Color COLOR_LIST_SELECTION_FG = new Color(240, 240, 240);
	public static final Color COLOR_LIST_TAGGED_SELECTION_BG = Color.RED;
	public static final Color COLOR_LIST_TAGGED_BG = new Color(60, 60, 0);

	private PatternInstanceGenerationPanel instanceGenerationPanel;
	protected JLabel instancesListLabel;
	
	public InstancePanel(int aRadius, int anOuterBorder, int anInnerBorder){
		super(aRadius, anOuterBorder, anInnerBorder);	
	}
	
	public InstancePanel(final PatternInstanceGenerationPanel instanceGenerationPanel, XOCLog log){
		this.instanceGenerationPanel = instanceGenerationPanel;
		this.log = log;
		instancesList = new JList();
		instancesList.setBackground(COLOR_LIST_BG);
		instancesList.setForeground(Color.WHITE); 
//		updateInstancesList(log);
		

		JScrollPane eventScrollPane = new JScrollPane(instancesList);
		eventScrollPane.setBorder(BorderFactory.createEmptyBorder());
		eventScrollPane.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);
		eventScrollPane.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		
		instancesListLabel = new JLabel("Instances");
		instancesListLabel.setForeground(COLOR_LIST_SELECTION_FG);
		instancesListLabel.setAlignmentX(Component.CENTER_ALIGNMENT);

		
		this.setBackground(COLOR_ENCLOSURE_BG);
		this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		this.setMaximumSize(new Dimension(180, 1000));
		this.setPreferredSize(new Dimension(180, 1000));
		this.add(Box.createVerticalStrut(8));	
		this.add(instancesListLabel);	
		this.add(Box.createVerticalStrut(8));		
		this.add(eventScrollPane);
	}
	
	/**
	 * update the instance list based on Map<String, List<String>> idVSInstance
	 * @param idVSInstance
	 */
	public void updateInstancesList(List<List<XOCEvent>> sortedInstance) {
		List<List<String>> instances = new ArrayList<List<String>>();
		for(List<XOCEvent> instance : sortedInstance){	
			List<String> instanceString = new ArrayList<String>();
			for(XOCEvent event: instance){
				String eventId = event.getAttributes().get(xConstants.ID).toString();
				instanceString.add(eventId);
			} 
			instances.add(instanceString);
		}
		instancesList.setListData(instances.toArray());
		instancesListLabel.setText("Instances (" + instances.size() + ")");
	}

	protected void instancesSelectionChanged() {	
		System.out.printf("index:  " + instancesList.getSelectedIndex());
		
		int[] selectedIndices = instancesList.getSelectedIndices();
		XOCEvent event = null;

		
		
		if(instancesList.getSelectedIndex() < 0)
			event = log.getAllEvents().get(0);
		else
			event = log.getAllEvents().get(selectedIndices[0]);
		
	}	
}




