/*
 * Copyright (c) 2007 Christian W. Guenther (christian@deckfour.org)
 * 
 * LICENSE:
 * 
 * This code is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 * 
 * EXEMPTION:
 * 
 * License to link and use is also granted to open source programs which are not
 * licensed under the terms of the GPL, given that they satisfy one or more of
 * the following conditions: 1) Explicit license is granted to the ProM and
 * ProMimport programs for usage, linking, and derivative work. 2) Carte blance
 * license is granted to all programs developed at Eindhoven Technical
 * University, The Netherlands, or under the umbrella of STW Technology
 * Foundation, The Netherlands. For further exemptions not covered by the above
 * conditions, please contact the author of this code.
 */
package org.processmining.ocbc.performance.panels;



import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

import org.deckfour.xes.model.XAttributeMap;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.deckfour.xes.model.impl.XAttributeMapImpl;
import org.deckfour.xes.model.impl.XEventImpl;
import org.deckfour.xes.model.impl.XLogImpl;
import org.deckfour.xes.model.impl.XTraceImpl;
import org.deckfour.xes.out.XesXmlSerializer;
import org.processmining.common.dialogue.IOFileDialogue;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.ocbc.log.io.XocXmlParser;
import org.processmining.ocbc.log.model.XOCEvent;
import org.processmining.ocbc.log.model.XOCLog;
import org.processmining.ocbc.logconversion.xlog.XLogFunctions;
import org.processmining.ocbc.performance.logprojection.plugins.dottedchart.DottedChart.DottedChartException;

/**
 * @author Christian W. Guenther (christian@deckfour.org)
 * 
 */
public class LogOutputPanel extends JPanel {

	/**
	 * 
	 */
	private static final long serialVersionUID = 7772305278301666972L;

	private JButton importButton;
	private JButton generateInstancesButton;
	private JButton generateLogButton;
	private XOCLog log;
	private UIPluginContext context;
    private File lastFile;
	private PathConfigurationPanel pathConfigurationPanel;
	private InstancePanel instancePanel;
	private List<List<XOCEvent>> sortedInstanceList = new ArrayList<List<XOCEvent>>();
	private List<XOCEvent> sortedReferenceEventList = new ArrayList<XOCEvent>();
	
	public LogOutputPanel(final PatternInstanceGenerationPanel fatherPanel, final XOCLog log) {

		this.log = log;
		setOpaque(false);
		setBorder(BorderFactory.createEmptyBorder());
		setLayout(new BorderLayout());
		pathConfigurationPanel = fatherPanel.getPathConfigurationPanel();
		instancePanel = fatherPanel.getInstancePanel();
		
		generateInstancesButton = new JButton("generate instances & analyze performance");
		generateInstancesButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				  String selectedPath = pathConfigurationPanel.getSelectPath();
				  String referenceActivity = selectedPath.split("-->")[0];
				  String pathClassNameSequence = selectedPath.split("-->")[1];
				  String targetActivity = selectedPath.split("-->")[2];		
				  
				  
				  String[] pathClassNameSet = pathClassNameSequence.split("--");	
				  
				  Map<XOCEvent, List<XOCEvent>> eventVSInstance = new HashMap<XOCEvent, List<XOCEvent>>();
				  
				  sortedInstanceList.clear();
				  sortedReferenceEventList.clear();
				  for(XOCEvent event : log.getEventsOfActivity(referenceActivity)){
//					  String eventId = event.getAttributes().get(xConstants.ID).toString();
					  eventVSInstance.put(event, new ArrayList<XOCEvent>());
					  eventVSInstance.get(event).add(event);
					  List<XOCEvent> relatedEventList = log.getRelatedEventsForOneEventByPath(event, pathClassNameSet, targetActivity);
					  for(XOCEvent targetEvent : relatedEventList){
						  eventVSInstance.get(event).add(targetEvent);
					  }
					  List<XOCEvent> instance = eventVSInstance.get(event);
					  sortedReferenceEventList.add(event);
					  sortedInstanceList.add(log.sortEventList(instance));
				  }
			
				  instancePanel.updateInstancesList(sortedInstanceList);
				  
				  XLog xesLog = generateLog(referenceActivity, targetActivity); 	
				  try {
						fatherPanel.getPerformanceAnalysisPanel().updateDotChartByXESLog(xesLog);
				  } catch (DottedChartException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
				  }
			  }					  			  		 
		});
		
		
		generateLogButton = new JButton("analyze performance");
		generateLogButton.addActionListener(new ActionListener() {
		    public void actionPerformed(ActionEvent arg0) {	    	
//		    	XLog xesLog = generateLog(); 	
//		    	try {
//					fatherPanel.getPerformanceAnalysisPanel().updateDotChartByXESLog(xesLog);
//				} catch (DottedChartException e) {
//					// TODO Auto-generated catch block
//					e.printStackTrace();
//				}
//	    		context = fatherPanel.getContext();
//	    		if(context != null){
//			    	context.getProvidedObjectManager().createProvidedObject("GeneratedXESLog", xesLog, XLog.class, context);
//			        if (context instanceof UIPluginContext) {
//			               UIPluginContext uiPluginContext = (UIPluginContext) context;
//			               uiPluginContext.getGlobalContext().getResourceManager().getResourceForInstance(xesLog).setFavorite(true);
//			        }	
//	    		}
//	    		else{
//	    			outputLogToDisk(xesLog);
//	    		}		
		    }
		});
 
//		importButton = new JButton("import");
//		importButton.addActionListener(new ActionListener() {
//		    public void actionPerformed(ActionEvent arg0) {	    	
//		    	XLog xesLog = generateLog(); 			    		
//	    		context = fatherPanel.getContext();
//	    		if(context != null){
//			    	context.getProvidedObjectManager().createProvidedObject("GeneratedXESLog", xesLog, XLog.class, context);
//			        if (context instanceof UIPluginContext) {
//			               UIPluginContext uiPluginContext = (UIPluginContext) context;
//			               uiPluginContext.getGlobalContext().getResourceManager().getResourceForInstance(xesLog).setFavorite(true);
//			        }	
//	    		}
//	    		else{
//	    			outputLogToDisk(xesLog);
//	    		}		
//		    }
//		});
		
		JPanel buttonPanel = new JPanel();
		
		buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.X_AXIS));
		buttonPanel.add(generateInstancesButton);	
//		buttonPanel.add(generateLogButton);	
		this.add(buttonPanel);

	}

	/**
	 * generate a XES log based on correlated instances
	 * @return XLog xesLog
	 */
	public XLog generateLog(String referenceActivity, String targetActivity){
    	XAttributeMap logAttributeMap = new XAttributeMapImpl();
    	XLogFunctions.putLiteral(logAttributeMap, "pattern:reference", referenceActivity);
    	XLogFunctions.putLiteral(logAttributeMap, "pattern:target", targetActivity);
    	XLog xesLog = new XLogImpl(logAttributeMap);
    	int i=1;
//		for(List<XOCEvent> instance : sortedInstanceList){
		for(int n=0; n<sortedInstanceList.size(); n++){
			List<XOCEvent> instance = sortedInstanceList.get(n);
			XAttributeMap traceAttributeMap = new XAttributeMapImpl();			
			XLogFunctions.putLiteral(traceAttributeMap, "concept:name", "instance " + i);
			i++;
			XTrace trace = new XTraceImpl(traceAttributeMap);
			Date dateTimeReference = null;
			XOCEvent referenceEvent = sortedReferenceEventList.get(n);
			String timestampReference = referenceEvent.getAttributes().get("timestamp").toString();		
			SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
			
			try {
				dateTimeReference = sdf.parse(timestampReference);
			} catch (ParseException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			
			for(XOCEvent event : instance){
    			String activity = event.getAttributes().get("activity").toString();
    			String timestamp = event.getAttributes().get("timestamp").toString();
    			String id = event.getAttributes().get("id").toString();

//    			SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
    			Date dateTime = null;
    			
				try {
					dateTime = sdf.parse(timestamp);
				} catch (ParseException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
    			
				String degree = "1";
				if(referenceActivity.equals(activity)){
					degree = Integer.toString(instance.size());
				}
				String referenceTarget = "reference";
				boolean isReference = true;
				if(activity.equals(targetActivity)){
					referenceTarget = "target";
					isReference = false;
				}
    			trace.add(makeEvent(activity, dateTime, dateTimeReference, "complete", id, degree, referenceTarget, isReference));
    			System.out.println("\n event id: " + event.getAttributes().get("id").toString());
    			System.out.println("\n etime: " + timestamp);
    		}
			xesLog.add(trace);
		}
		
		return xesLog;
	}
	
	/**
	 * output generated XES log to disk on a selected path by a dialogue
	 * @param xesLog
	 */
	public void outputLogToDisk(XLog xesLog){
			System.out.println("generate logs");
			IOFileDialogue fileImporter = new IOFileDialogue();
    	fileImporter.show("XES Log (.xes)", "xes", JFileChooser.FILES_ONLY, "Output", lastFile);
    	File file = fileImporter.getSelectedFile();
		if(file != null){			
			lastFile = file.getParentFile();
			String fileName = file.getAbsolutePath();				        
    		//save xes log
			if(fileName != null){
				XesXmlSerializer serializer = new XesXmlSerializer();
				OutputStream outputLog = null;
				try {
					outputLog = new FileOutputStream("\\" + fileName + ".xes");
				} catch (FileNotFoundException e1) {
					// TODO Auto-generated catch block
					e1.printStackTrace();
				}
				try {
					serializer.serialize(xesLog, outputLog);
					if(xesLog.size()>0){
						JOptionPane.showMessageDialog(null, "Generate a XES log successfully!");
					}
					else{
						JOptionPane.showMessageDialog(null, "Warn! Generate an empty XES log.");
					}
					
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
	    		
				try {
					outputLog.close();
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		}	
	}

	/**
	 * sort the path by the length from short to long
	 * @param pathList
	 * @return
	 */
	public List<String> sortPathByLength(List<String> pathList){
		List<String> originalList = pathList;
		List<String> rankedList = new ArrayList<String>();
	
		while(originalList.size()>0){
			int shortestIndex = 0;
			int shortestSize = Integer.MAX_VALUE;
			for(int i=0; i<originalList.size(); i++){
				//every event in the simulation log should have a "createTime" attribute, otherwise, it goes wrong
				String path = originalList.get(i);
				int length = path.split("->").length;		
				if(length < shortestSize){
					shortestIndex = i;
					shortestSize = length;
				}
			}			
//			System.out.println("\n rank time: " + changeList.get(earlistTimeIndex).time);			
			rankedList.add(originalList.get(shortestIndex));
			originalList.remove(shortestIndex);
			
		}
		return rankedList;
	}

	/**
	 * 
	 * @param name
	 * @param time
	 * @param lifecycle
	 * @return
	 */
	private XEvent makeEvent(String name, Date time, Date timeReference, String lifecycle, 
			String id, String degree, String referenceTarget, boolean isReference) {
		XAttributeMap attMap = new XAttributeMapImpl();
		XLogFunctions.putLiteral(attMap, "concept:name", name);
//    		XLogFunctions.putLiteral(attMap, "lifecycle:transition", "COMPLETE");
		XLogFunctions.putLiteral(attMap, "lifecycle:transition", lifecycle);
		XLogFunctions.putLiteral(attMap, "org:resource", "artificial");
		XLogFunctions.putTimestamp(attMap, "time:timestamp", time);
		
//		Date date1 = myFormat.parse(inputString1);
//	    Date date2 = myFormat.parse(inputString2);
//	    long diff = ;
//	    
//	    System.out.println ("Days: " + d.toString());
	    Date relativeDate = new Date(time.getTime() - timeReference.getTime());
		XLogFunctions.putTimestamp(attMap, "relativetime:timestamp", relativeDate);
		XLogFunctions.putLiteral(attMap, "id:name", id);
		XLogFunctions.putLiteral(attMap, "degree:int", degree);
//		XLogFunctions.putLiteral(attMap, "pattern:name", referenceTarget + " " + "(" + name + ")");		
		XLogFunctions.putLiteral(attMap, "pattern:name", referenceTarget);
		XLogFunctions.putBoolean(attMap, "isReference:boolean", isReference);
		XEvent newEvent = new XEventImpl(attMap);
		
		return newEvent;
	}
	
	public static void main(String[] args) throws Exception  
    {  
//		int classNum = 3;
//		int horizontalUintNum = (int) Math.sqrt(classNum);
//		System.out.print("value" + horizontalUintNum);
//		System.exit(0);
		System.out.print("\n start");
		XocXmlParser XOCParser = new XocXmlParser();
		XOCParser.parse(new File("e:\\NewXOCLog.xoc"));		
		
		
		XOCLog inputLog = XOCParser.getLog();
		System.out.print("\n contains(targetObject)");
		PatternInstanceGenerationPanel eventUI = new PatternInstanceGenerationPanel(null, null, inputLog, null);
		JFrame frame = new JFrame();
		JPanel panel = new JPanel();
		JButton button = new JButton("hello");
		panel.add(button);
//		frame.add(panel);	
		
//		JButton button2 = new JButton("world");
//		panel = new JPanel();
//		panel.add(button2);
//		frame.add(panel);
		
		frame.add(eventUI);
		frame.setSize(600, 600);
		frame.setVisible(true);
    }
}
