package org.processmining.ocbc.performance.panels;
 

import java.awt.Toolkit;
import java.io.FileInputStream;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JFrame;
import javax.swing.JPanel;

import org.processmining.mxgraph.examples.swing.GraphEditor.CustomGraph;
import org.processmining.mxgraph.examples.swing.GraphEditor.CustomGraphComponent;
import org.processmining.mxgraph.examples.swing.editor.BasicGraphPanel;
import org.processmining.mxgraph.examples.swing.visualization.Style;
import org.processmining.mxgraph.io.mxCodec;
import org.processmining.mxgraph.model.mxCell;
import org.processmining.mxgraph.model.mxICell;
import org.processmining.mxgraph.swing.mxGraphComponent;
import org.processmining.mxgraph.util.mxConstants;
import org.processmining.mxgraph.util.mxXmlUtils;
import org.processmining.mxgraph.view.mxGraph;
import org.processmining.ocbc.models.OCBCModel;
import org.processmining.ocbc.models.impl.OCBCModelImpl;
import org.processmining.ocbc.models.io.ImportOCBCModel;
import org.w3c.dom.Document;
 
/**
 * @version 1.0
 * @author guangming li
 * there are two lists (left list and right list) to show the not-selected values and selected values, respectivly
 */
 
public class ModelPanel extends JPanel{
    /**
     * 
     */
    private static final long serialVersionUID = 1L;
    
	private OCBCModel model = new OCBCModelImpl();//contain the class model and AOC relationships in the log
	private BasicGraphPanel basicGraphPanel = new BasicGraphPanel("mxGraph Editor", null, new CustomGraphComponent(new CustomGraph()));
	private PatternInstanceGenerationPanel instanceGenerationPanel;
	
    public static void main(String[] args) throws Exception {
		FileInputStream input = new FileInputStream("D:\\otc.ocbcmml");
		ImportOCBCModel importer = new ImportOCBCModel();
		OCBCModel model = importer.generateOCBCModel(input);
//		OCBCModel  inputModel = (OCBCModel) model.clone();
			 
		

		
		ModelPanel modelViewPanel = new ModelPanel(null, model);
		modelViewPanel.createGraphBasedOnModel();
		System.out.print("\n after plunin creation");
	
		JFrame frame = new JFrame();
		frame.setSize(Toolkit.getDefaultToolkit().getScreenSize());
		frame.getContentPane().add(modelViewPanel.getModelView());
//		frame.add(modelViewPanel);
		frame.setVisible(true);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		System.out.print("end");	
    }
 
    public ModelPanel(PatternInstanceGenerationPanel instanceGenerationPanel, OCBCModel model) {
    	this.model = model;
    	this.instanceGenerationPanel = instanceGenerationPanel;
		basicGraphPanel.getGraphComponent().setPageBreaksVisible(false);
		basicGraphPanel.getGraphComponent().setToolTips(false);;
		basicGraphPanel.getGraphComponent().setToolTipText("hello");
		basicGraphPanel.getGraphComponent().setPageVisible(false);
		basicGraphPanel.getGraphComponent().getHorizontalScrollBar().setToolTipText("press ctrl to zoom in/out the graph");
		basicGraphPanel.getGraphComponent().getVerticalScrollBar().setToolTipText("press ctrl to zoom in/out the graph");
//		graphComponent.setGridVisible(false);

    }
    
    /**
     * get the father panel of CorrelationModelPanel, i.e., XOCLogConversionPanel
     * @return
     */
    public PatternInstanceGenerationPanel getInstanceGenerationPanel(){
    	return instanceGenerationPanel;
    }
    
    /**
     * get the panel for the discovered model
     * @return
     */
    public mxGraphComponent getModelView(){
    	return basicGraphPanel.getGraphComponent(); 
    }
    
    /**
     * get the discovered model used to correlating events
     * @return
     */
    public OCBCModel getModel(){
    	return model; 
    }
    
    /**
     * transform the model (class model and AOC relationships) as graph in the basicGraphPanel
     * @return
     */
    public void createGraphBasedOnModel() {
	
		mxGraph graphX = new CustomGraph();
		Document document = mxXmlUtils.parseXml(model.getModelString());
		System.out.println("\n input model");
		mxCodec codec = new mxCodec(document);
		codec.decode(document.getDocumentElement(),graphX.getModel());					
		basicGraphPanel = new BasicGraphPanel("mxGraph Editor", null, new CustomGraphComponent(graphX));
		basicGraphPanel.getGraphComponent().setPageBreaksVisible(false);
		basicGraphPanel.getGraphComponent().setToolTips(false);;
		basicGraphPanel.getGraphComponent().setToolTipText("hello");
		basicGraphPanel.getGraphComponent().setPageVisible(false);
		basicGraphPanel.getGraphComponent().getHorizontalScrollBar().setToolTipText("press ctrl to zoom in/out the graph");
		basicGraphPanel.getGraphComponent().getVerticalScrollBar().setToolTipText("press ctrl to zoom in/out the graph");
//		basicGraphPanel.getGraphComponent().setGraph(graphX);
		mxGraph graph = basicGraphPanel.getGraphComponent().getGraph();			
		for(mxCell cell: model.getGraphX().getAllCells()){
			if(cell.getAttributes().get(mxConstants.TYPE).equals(mxConstants.CLASSCELL) || 
					cell.getAttributes().get(mxConstants.TYPE).equals(mxConstants.ACTIVITYCELL) ||
					cell.getAttributes().get(mxConstants.TYPE).equals(mxConstants.CLASSRELATION) ||
					cell.getAttributes().get(mxConstants.TYPE).equals(mxConstants.AOCRELATION)){
				System.out.println("cell type: " + cell.getAttributes().get(mxConstants.TYPE));
				System.out.println("cell.getChildCount(): " + cell.getChildCount());
				int childCount = cell.getChildCount();
				List<mxICell> childCellList = new ArrayList<>();
				for(int i=0; i<childCount; i++){
					System.out.println("i: " + i);
//					System.out.println("i child id: " + cell.getChildAt(i).getId());
					mxICell childCell = cell.getChildAt(i);
					childCellList.add(childCell);	
				}
				for(int i=0; i<childCount; i++){
					cell.remove(childCellList.get(i));				
				}
				
				graph.addCell(cell);
			}
		}
    }
    
    /**
     * transform the model (class model and AOC relationships) as graph (without behavioral constraints, cardinalities) in the basicGraphPanel
     * @return
     */
    public void createSimplifiedGraphBasedOnModel() {
		mxGraph graph = basicGraphPanel.getGraphComponent().getGraph();			
		for(mxCell cell: model.getGraphX().getAllCells()){
			if(cell.getAttributes().get(mxConstants.TYPE).equals(mxConstants.CLASSCELL) || 
					cell.getAttributes().get(mxConstants.TYPE).equals(mxConstants.ACTIVITYCELL) ||
					cell.getAttributes().get(mxConstants.TYPE).equals(mxConstants.CLASSRELATION) ||
					cell.getAttributes().get(mxConstants.TYPE).equals(mxConstants.AOCRELATION)){
				System.out.println("cell type: " + cell.getAttributes().get(mxConstants.TYPE));
				System.out.println("cell.getChildCount(): " + cell.getChildCount());
				int childCount = cell.getChildCount();
				List<mxICell> childCellList = new ArrayList<>();
				for(int i=0; i<childCount; i++){
					System.out.println("i: " + i);
//					System.out.println("i child id: " + cell.getChildAt(i).getId());
					mxICell childCell = cell.getChildAt(i);
					childCellList.add(childCell);	
				}
				for(int i=0; i<childCount; i++){
					cell.remove(childCellList.get(i));				
				}
				
				graph.addCell(cell);
			}
		}
    }
    
    /**
     * transform the model (class model and AOC relationships) as graph (without behavioral constraints, cardinalities) in the basicGraphPanel
     * @return
     */
    public void createBriefGraphBasedOnModel() {
		mxGraph graph = basicGraphPanel.getGraphComponent().getGraph();		
		basicGraphPanel.getGraphComponent().getHorizontalScrollBar().setAutoscrolls(true);
		basicGraphPanel.getGraphComponent().getHorizontalScrollBar().setValue(Integer.MAX_VALUE);
		basicGraphPanel.getGraphComponent().getHorizontalScrollBar().setUnitIncrement(Integer.MIN_VALUE);;

		//		basicGraphPanel.getGraphComponent().getHorizontalScrollBar().setEnabled(false);;
		for(mxCell cell: model.getGraphX().getAllCells()){
			if(cell.getAttributes().get(mxConstants.TYPE).equals(mxConstants.CLASSCELL) || 
					cell.getAttributes().get(mxConstants.TYPE).equals(mxConstants.ACTIVITYCELL) ||
					cell.getAttributes().get(mxConstants.TYPE).equals(mxConstants.CLASSRELATION) ||
					cell.getAttributes().get(mxConstants.TYPE).equals(mxConstants.AOCRELATION) ||
					cell.getAttributes().get(mxConstants.TYPE).equals(mxConstants.ACTIVITYRELATION) ||
					cell.getAttributes().get(mxConstants.TYPE).equals(mxConstants.CRELOCRELATION) ||
					cell.getAttributes().get(mxConstants.TYPE).equals(mxConstants.CRELRTRELATION)){
				System.out.println("cell type: " + cell.getAttributes().get(mxConstants.TYPE));
				System.out.println("cell.getChildCount(): " + cell.getChildCount());
				int childCount = cell.getChildCount();
				List<mxICell> childCellList = new ArrayList<>();
				for(int i=0; i<childCount; i++){
					System.out.println("i: " + i);
//					System.out.println("i child id: " + cell.getChildAt(i).getId());
					mxICell childCell = cell.getChildAt(i);
					if(!((mxCell) childCell).getAttributes().get(mxConstants.TYPE).equals(mxConstants.CONNECTOR)){
						childCellList.add(childCell);
					}
						
				}
				for(int i=0; i<childCellList.size(); i++){
					cell.remove(childCellList.get(i));				
				}
				
				graph.addCell(cell);
			}
		}
    }
    
    /**
     * highlight the selected correlation pattern in the model
     * @param referenceActivity
     * @param classList
     * @param targetActivity
     */
    public void highlightSelectedPattern(String referenceActivity, List<String> classList, String targetActivity){
    	restoreAllCells();
    	System.out.println("enter highlightSelectedPattern");
    	
    	List<mxCell> cellList = new ArrayList<mxCell>();
    	mxGraph graph = basicGraphPanel.getGraphComponent().getGraph();		
    	
    	//get the node cells
    	mxCell referenceCell = graph.getNodeByName(referenceActivity);
    	mxCell targetCell = graph.getNodeByName(targetActivity);
    	cellList.add(referenceCell);
    	for(String className : classList){
    		cellList.add(graph.getNodeByName(className));
    	}	
    	cellList.add(targetCell);
    	
    	//get the edge cells
    	List<mxCell> edgeCellList = new ArrayList<mxCell>(); 	
    	for(int i=0; i<cellList.size()-1; i++){
    		mxCell edgeCell = graph.getEdgeBetweenTwoNodes(cellList.get(i), cellList.get(i+1));
    		edgeCellList.add(edgeCell);
    	}   	
    	cellList.addAll(edgeCellList);	
    	
    	//highlight the cells
		graph.setCellStyles(mxConstants.STYLE_STROKECOLOR, "red",  cellList.toArray());	
		graph.setCellStyles(mxConstants.STYLE_FONTCOLOR, "red",  cellList.toArray());
		graph.setCellStyles(mxConstants.STYLE_STROKEWIDTH, "3",  cellList.toArray());	
		graph.refresh();
		graph.repaint();
    }
    
    /**
     * 
     */
	public void restoreAllCells(){
		mxGraphComponent graphComponent = basicGraphPanel.getGraphComponent();
		Object[] childCells = graphComponent.getGraph().getChildCells(graphComponent.getGraph().getDefaultParent());
		for(int i=0; i<childCells.length; i++){
			mxCell childCell = (mxCell)childCells[i];
			String childCellType = childCell.getAttributes().get(mxConstants.TYPE);
			if(childCellType.equals(mxConstants.AOCRELATION) || childCellType.equals(mxConstants.CRELOCRELATION) || childCellType.equals(mxConstants.CRELRTRELATION)){
				String edgeColor = Style.getStyleElementValueFromCell(graphComponent.getGraph(), childCell, mxConstants.STYLE_STROKECOLOR);
    			System.out.println("in black edgeColor: " + edgeColor);
				Style.addStyleElementToCell((mxCell)childCells[i], mxConstants.STYLE_STROKECOLOR, "blue");
//				Style.addStyleElementToCell((mxCell)childCells[i], mxConstants.STYLE_STROKECOLOR, "black");
			}
			else{
				Style.addStyleElementToCell((mxCell)childCells[i], mxConstants.STYLE_STROKECOLOR, "black");
			}

			Style.addStyleElementToCell((mxCell)childCells[i], mxConstants.STYLE_STROKEWIDTH, "1");
			Style.addStyleElementToCell((mxCell)childCells[i], mxConstants.STYLE_FONTCOLOR, "black");
			Object[] labelCells = graphComponent.getGraph().getChildCells(childCells[i]);
			for(int j=0; j<labelCells.length; j++){
				mxCell labelCell = (mxCell) labelCells[j];
				String cellType = labelCell.getAttributes().get(mxConstants.TYPE);
				if(cellType != null){
					System.out.println("label style : " + labelCell.getStyle());
					Style.addStyleElementToCell(labelCell, mxConstants.STYLE_FONTCOLOR, "black");
				}				
			}						
		}
		graphComponent.getGraph().refresh();
	}
}