package org.processmining.ocbc.performance.panels;
 
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionAdapter;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.ListModel;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import org.processmining.common.frame.MyComboBox;
import org.processmining.ocbc.log.model.XOCLog;
import org.processmining.ocbc.logconversion.classmodel.SearchAllPathsBetweenTwoNodes;
import org.processmining.ocbc.models.ClassNode;
import org.processmining.ocbc.models.OCBCModel;
 
/**
 * @version 1.0
 * @author guangming li
 * there are two lists (left list and right list) to show the not-selected values and selected values, respectivly
 */
 
public class PathConfigurationPanel extends JPanel implements ListSelectionListener{
    /**
     * 
     */
    private static final long serialVersionUID = 1L;
    private List<String> leftListValues;
    private List<String> rightListValues;    
    private JLabel leftListLabel;
    private JLabel rightListLabel;
    private DefaultListModel<String> leftListModel = new DefaultListModel<String>();
    private DefaultListModel<String> rightListModel = new DefaultListModel<String>();
    private JList<String> leftList = new JList<String>(leftListModel);
    private JList<String> rightList = new JList<String>(rightListModel);
    
	private PatternInstanceGenerationPanel instanceGenerationPanel;
	private ModelPanel modelViewPanel;
	private XOCLog log;
	private JButton showModelButton = new JButton("show model");
	private JComboBox<String> referenceBox = new JComboBox<String>();
	private JComboBox<String> targetBox = new JComboBox<String>();
	private MyComboBox pathBox = new MyComboBox();
	private JCheckBox filterBox = new JCheckBox();
	private JLabel referenceBoxLabel = new JLabel("reference activity: ");//reference activity selection
	private JLabel targetBoxLabel = new JLabel("target activity: ");//target activity selection
	private JLabel pathLabel = new JLabel("pattern: ");//target activity selection
	private JLabel filterLabel = new JLabel("only show V and U patterns: ");//target activity selection
 
    public PathConfigurationPanel(List<String> leftListValues, List<String> rightListValues) {
    	this.leftListValues = leftListValues;
    	this.rightListValues = rightListValues;
    	this.leftList.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
    	this.rightList.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
    }
 
    public PathConfigurationPanel(PatternInstanceGenerationPanel fatherPanel, XOCLog log) {
    	this.instanceGenerationPanel = fatherPanel;
    	this.modelViewPanel = fatherPanel.getModelPanel();
    	this.log = log;
//    	modelViewPanel.createSimplifiedGraphBasedOnModel();
    	modelViewPanel.createBriefGraphBasedOnModel();
    	
    	referenceBox.setMaximumSize(new Dimension(200,30));
//		for(String className : log.getClassNames()){
//			referenceBox.addItem(className);
//		}
		for(String activityName : log.generateActivityNameList()){
			referenceBox.addItem(activityName);
		}
		
		referenceBox.addActionListener(new ActionListener() {
		    public void actionPerformed(ActionEvent arg0) {
		    	OCBCModel model = modelViewPanel.getModel();
		    	updateCorrelationPathsBasedOnReferenceAndTargetActivities(model);
//		    	else if(model.getAllClassNames().contains(selectedName)){
//		    		updateCorrelationPathListForClass(model, selectedName);		
//		    	}	    	
		    }
		});
		referenceBox.setSelectedItem(referenceBox.getItemAt(0));
		
    	targetBox.setMaximumSize(new Dimension(200,30));
		for(String activityName : log.generateActivityNameList()){
			targetBox.addItem(activityName);
		}
		
		targetBox.addActionListener(new ActionListener() {
		    public void actionPerformed(ActionEvent arg0) {
		    	OCBCModel model = modelViewPanel.getModel();
		    	updateCorrelationPathsBasedOnReferenceAndTargetActivities(model);	
		    }
		});
		targetBox.setSelectedItem(targetBox.getItemAt(0));
		
		pathBox.setPreferredSize(new Dimension(500,30));
		pathBox.setMaximumSize(new Dimension(500,30));
		pathBox.addActionListener(new ActionListener() {
		    public void actionPerformed(ActionEvent arg0) {
		    	if(pathBox.getSelectedItem() != null){
		    		String path = pathBox.getSelectedItem().toString();
		    		pathBox.setToolTipText(path);
		    		String referenceActivity = path.split("-->")[0];
		    		String classSequence = path.split("-->")[1];
		    		String targetActivity = path.split("-->")[2];
		    		
		    		List<String> classList = new ArrayList<String>();
		    		for(int i=0; i<classSequence.split("--").length; i++){
		    			classList.add(classSequence.split("--")[i]);
		    		}		    		 	
		    		
		    		//highlight the selected pattern in the graph
		    		instanceGenerationPanel.getModelPanel().highlightSelectedPattern(referenceActivity, classList, targetActivity);
		    	}	 
		    	else{
		    		pathBox.setToolTipText(null);
		    		//restore the color of all elements as it is at first
		    		instanceGenerationPanel.getModelPanel().restoreAllCells();
		    	}	    	
		    }
		});
		filterBox.setSelected(true);
		filterBox.addActionListener(new ActionListener() {
		    public void actionPerformed(ActionEvent arg0) {
		    	OCBCModel model = modelViewPanel.getModel();
		    	updateCorrelationPathsBasedOnReferenceAndTargetActivities(model);
		    }
		});
		
		JPanel buttonPanel = new JPanel();		
		buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.X_AXIS));
		buttonPanel.add(referenceBoxLabel);
//		buttonPanel.add(Box.createVerticalStrut(100));
		buttonPanel.add(referenceBox);
		buttonPanel.add(Box.createVerticalStrut(100));
		buttonPanel.add(targetBoxLabel);
//		buttonPanel.add(Box.createVerticalStrut(100));
		buttonPanel.add(targetBox);
		buttonPanel.add(Box.createVerticalStrut(100));
		buttonPanel.add(pathLabel);
//		buttonPanel.add(Box.createVerticalStrut(100));
		buttonPanel.add(pathBox);
		buttonPanel.add(Box.createVerticalStrut(100));
		buttonPanel.add(filterLabel);
		buttonPanel.add(filterBox);
		
//		updateJListsForBoxChange(modelViewPanel.getModel(),log);	
		
		this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		this.add(buttonPanel);
//		this.add(generateTwoListSwapValuesPanel());
    }
    

    
    /**
     * update the correlation paths for a pair of selected reference and target activities
     * @param model
     * @param selectedClassName
     */
    public void updateCorrelationPathsBasedOnReferenceAndTargetActivities(OCBCModel model){
    	String referenceActivity = (String) referenceBox.getSelectedItem();
    	String targetActivity = (String) targetBox.getSelectedItem();
        SearchAllPathsBetweenTwoNodes searchAllPathsBetweenTwoNodes = new SearchAllPathsBetweenTwoNodes();
        
		List<String> newPathList = new ArrayList<String>();
		Map<String, List<String>> activityVSRelatedClassList = model.generateActivityVSRelatedClassListMap();
		if(!referenceActivity.equals(targetActivity)){
			//check all classes (as start node) related to the reference activity
			if(activityVSRelatedClassList.containsKey(referenceActivity)){
		        for(String startClassName : activityVSRelatedClassList.get(referenceActivity)){
		    	    ClassNode startNode = model.getClassByName(startClassName);
		    	    List<String> pathList = new ArrayList<String>();
		    	    //check all classes (as end node) related to the target activity
		    	    if(activityVSRelatedClassList.containsKey(targetActivity)){
		    	    	  for(String endClassName : activityVSRelatedClassList.get(targetActivity)){	    		  
		    		    	  ClassNode endNode = model.getClassByName(endClassName);
		    		    	  searchAllPathsBetweenTwoNodes.clearPathList();
		    		    	  searchAllPathsBetweenTwoNodes.getPaths(startNode, null, startNode, endNode);
		    		    	  pathList.addAll(searchAllPathsBetweenTwoNodes.getPathList());
		    	    	  }
		    	    }
		    	    
		    		System.out.println("pathList size" + pathList.size());
		    		pathList = sortPathByLength(pathList);
		    		//create path with activities
		    		for(String path : pathList){
		    			//currently, we only consider v and u patterns
		    			if(filterBox.isSelected()){
		    				if(path.split("--").length <= 2){
			    				newPathList.add(referenceActivity + "-->" + path + "-->" + targetActivity);
			    			}   
		    			}
		    			else{
		    				newPathList.add(referenceActivity + "-->" + path + "-->" + targetActivity);
		    			}	    			 	    			
		    		}	    		
		        }
			}
		}
		
		System.out.println("new pathList size" + newPathList.size());
		

		pathBox.removeAllItems();
		for(String newPath: newPathList){
			pathBox.addItem(newPath);
		}
	}
    

    
    /**
     * sort the path by the length from short to long
     * @param pathList
     * @return
     */
	public List<String> sortPathByLength(List<String> pathList){
		List<String> originalList = pathList;
		List<String> rankedList = new ArrayList<String>();
	
		while(originalList.size()>0){
			int shortestIndex = 0;
			int shortestSize = Integer.MAX_VALUE;
			for(int i=0; i<originalList.size(); i++){
				//every event in the simulation log should have a "createTime" attribute, otherwise, it goes wrong
				String path = originalList.get(i);
				int length = path.split("--").length;		
				if(length < shortestSize){
					shortestIndex = i;
					shortestSize = length;
				}
			}			
//			System.out.println("\n rank time: " + changeList.get(earlistTimeIndex).time);			
			rankedList.add(originalList.get(shortestIndex));
			originalList.remove(shortestIndex);
			
		}
		return rankedList;
	}
	
    /**
     * get the father panel of CorrelationModelPanel, i.e., XOCLogConversionPanel
     * @return
     */
    public PatternInstanceGenerationPanel getInstanceGenerationPanel(){
    	return instanceGenerationPanel;
    }
    
    public List<String> getLeftListValues() {
    	leftListValues.clear();
    	for(int i=0; i<leftListModel.getSize(); i++){
    		leftListValues.add(leftListModel.getElementAt(i));
    	}
        return leftListValues;
    }
 
//    public List<String> getRightListValues() {
//    	rightListValues.clear();
//    	for(int i=0; i<rightListModel.getSize(); i++){
//    		rightListValues.add(rightListModel.getElementAt(i));
//    	}
//        return rightListValues;
//    }
//    
    /**
     * get the selected path 
     * @return
     */
    public String getSelectPath(){
    	return pathBox.getSelectedItem().toString();
    }
    
    /**
     * get selected item (a class or an activity) as case id to create instances
     * @return
     */
    public String getSelectedCaseId(){
    	return (String) referenceBox.getSelectedItem();
    }
    
    public JList<String> getLeftList() {
        return leftList;
    }
 
    public JList<String> getRightList() {
        return rightList;
    }
    
    public void setLeftListValues(List<String> leftListValues) {
        this.leftListValues = leftListValues;
    }
    
    public void setRightListValues(List<String> rightListValues) {
        this.rightListValues = rightListValues;
    }
 

	public void updateLeftList(){
    	if(leftListValues != null){
    		leftListModel.removeAllElements();
			for(String value : leftListValues){
				leftListModel.addElement(value);
			}
			leftList.setModel(leftListModel);
		}	
    }
	
	public void updateRightList(){
    	if(rightListValues != null){
    		rightListModel.removeAllElements();
			for(String value : rightListValues){
				rightListModel.addElement(value);
			}
			rightList.setModel(rightListModel);
		}	
    }
       
    public JPanel generateTwoListSwapValuesPanel(){	
    	updateLeftList();
    	updateRightList();
    	
    	//lable
	    leftListLabel = new JLabel("non-selected paths");
		leftListLabel.setAlignmentY(Component.TOP_ALIGNMENT);
		leftListLabel.setBorder(BorderFactory.createEmptyBorder(4, 5, 0, 5));

		//scroll panel
		JScrollPane leftListScroller = new JScrollPane(leftList);
		leftListScroller.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		leftListScroller.setAlignmentY(Component.TOP_ALIGNMENT);
		//list panel
		JPanel sourceListPanel = new JPanel();
		sourceListPanel.setLayout(new BoxLayout(sourceListPanel,BoxLayout.Y_AXIS));
		sourceListPanel.add(leftListLabel);
		sourceListPanel.add(leftListScroller);
		sourceListPanel.setAlignmentY(Component.TOP_ALIGNMENT);
		sourceListPanel.setBorder(BorderFactory.createEmptyBorder(0, 0, 0, 30));

		
		JButton toRightButton = new JButton(">");		
		JButton toLeftButton = new JButton("<");		
		JButton allToRightButton = new JButton(">>");		
		JButton allToLeftButton = new JButton("<<");
		
		JPanel buttonPanel = new JPanel();
		buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.Y_AXIS));
		buttonPanel.setAlignmentY(Component.TOP_ALIGNMENT);
		buttonPanel.setBorder(BorderFactory.createEmptyBorder(15, 5, 15, 5));
		buttonPanel.add(Box.createVerticalStrut(10));
		buttonPanel.add(toRightButton);
		buttonPanel.add(Box.createVerticalStrut(5));
		buttonPanel.add(toLeftButton);
		buttonPanel.add(Box.createVerticalStrut(5));
		buttonPanel.add(allToRightButton);
		buttonPanel.add(Box.createVerticalStrut(5));
		buttonPanel.add(allToLeftButton);
		

	
		rightListLabel = new JLabel("selected paths");
		rightListLabel.setAlignmentY(Component.TOP_ALIGNMENT);
		rightListLabel.setBorder(BorderFactory.createEmptyBorder(4, 5, 0, 5));


		JScrollPane rightListScroller = new JScrollPane(rightList);
		rightListScroller.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		rightListScroller.setAlignmentY(Component.TOP_ALIGNMENT);
		JPanel rightListPanel = new JPanel();
		rightListPanel.setLayout(new BoxLayout(rightListPanel,BoxLayout.Y_AXIS));
		rightListPanel.add(rightListLabel);
		rightListPanel.add(rightListScroller);
		rightListPanel.setAlignmentY(Component.TOP_ALIGNMENT);
		rightListPanel.setBorder(BorderFactory.createEmptyBorder(0, 30, 0, 0));
	
    	JPanel topPanel = new JPanel(); 	
    	topPanel.setLayout(new BoxLayout(topPanel,BoxLayout.X_AXIS));
        
		JSplitPane leftPanel = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, sourceListPanel, buttonPanel);
		leftPanel.setResizeWeight(0.9);
		
		JSplitPane completePanel = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, leftPanel, rightListPanel);
		completePanel.setResizeWeight(0.5);
		
		topPanel.add(completePanel);
		
		toRightButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				for(String selectedContent : leftList.getSelectedValuesList()){
					rightListModel.addElement(selectedContent);				
					leftListModel.removeElement(selectedContent);
				}
				leftList.setModel(leftListModel);
				rightList.setModel(rightListModel);						
				updateTableNumberLabel();
			}
		});
		
		allToRightButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				for(int i=0; i<leftListModel.getSize(); i++){
					rightListModel.addElement(leftListModel.getElementAt(i));
				}
				rightList.setModel(rightListModel);
				leftListModel.removeAllElements();				
				leftList.setModel(leftListModel);
				
				updateTableNumberLabel();
			}
		});
		
		toLeftButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
//				int index = rightList.getSelectedIndex();
//				if(index >= 0){
//					String selectedContent =(String) rightList.getModel().getElementAt(index);
//					leftListModel.addElement(selectedContent);
//					leftList.setModel(leftListModel);
//					rightListModel.removeElementAt(index);
//					rightList.setModel(rightListModel);
//				}
//				updateTableNumberLabel();
				
				
				for(String selectedContent : rightList.getSelectedValuesList()){
					leftListModel.addElement(selectedContent);				
					rightListModel.removeElement(selectedContent);
				}
				leftList.setModel(leftListModel);
				rightList.setModel(rightListModel);						
				updateTableNumberLabel();
			}
		});
		
		allToLeftButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				for(int i=0; i<rightListModel.getSize(); i++){
					leftListModel.addElement(rightListModel.getElementAt(i));
				}
				leftList.setModel(leftListModel);
				rightListModel.removeAllElements();				
				rightList.setModel(rightListModel);
				
				updateTableNumberLabel();			
			}
		});
		allToLeftButton.setToolTipText("unselect all paths");
		
		leftList.addListSelectionListener(this);
		leftList.setToolTipText(leftList.getSelectedValue());
		rightList.addListSelectionListener(this);
		rightList.setToolTipText(rightList.getSelectedValue());
		
		leftList.addMouseMotionListener(new MouseMotionAdapter() {
	        @Override
	        public void mouseMoved(MouseEvent e) {
	            JList l = (JList)e.getSource();
	            ListModel m = l.getModel();
	            int index = l.locationToIndex(e.getPoint());
	            if( index>-1 ) {
	                l.setToolTipText(m.getElementAt(index).toString());
	            }
	        }
	    });
		rightList.addMouseMotionListener(new MouseMotionAdapter() {
	        @Override
	        public void mouseMoved(MouseEvent e) {
	            JList l = (JList)e.getSource();
	            ListModel m = l.getModel();
	            int index = l.locationToIndex(e.getPoint());
	            if( index>-1 ) {
	                l.setToolTipText(m.getElementAt(index).toString());
	            }
	        }
	    });
		return topPanel;	
    }
    
    public void updateTableNumberLabel(){
		leftListLabel.setText("non-selected paths: " + leftListModel.size());
        rightListLabel.setText("selected paths: " + rightListModel.size());
    }

	public void valueChanged(ListSelectionEvent arg0) {
		// TODO Auto-generated method stub
		
	}
	
    public static void main(String[] args) {
    	JFrame frame = new JFrame();
    	List<String> leftListValues = new ArrayList<String>();
    	leftListValues.add("hello");
    	leftListValues.add("world");
    	List<String> rightListValues = new ArrayList<String>();
    	rightListValues.add("nihao");
    	
    	PathConfigurationPanel twoListSelectionPanel = new PathConfigurationPanel(leftListValues, rightListValues);
		frame.add(twoListSelectionPanel.generateTwoListSwapValuesPanel());		
		rightListValues.add("home");
		twoListSelectionPanel.setRightListValues(rightListValues);
		twoListSelectionPanel.updateRightList();
		frame.setSize(Toolkit.getDefaultToolkit().getScreenSize()); 
		frame.setVisible(true);
    }

}