/*
 * Copyright (c) 2007 Christian W. Guenther (christian@deckfour.org)
 * 
 * LICENSE:
 * 
 * This code is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 * 
 * EXEMPTION:
 * 
 * License to link and use is also granted to open source programs which are not
 * licensed under the terms of the GPL, given that they satisfy one or more of
 * the following conditions: 1) Explicit license is granted to the ProM and
 * ProMimport programs for usage, linking, and derivative work. 2) Carte blance
 * license is granted to all programs developed at Eindhoven Technical
 * University, The Netherlands, or under the umbrella of STW Technology
 * Foundation, The Netherlands. For further exemptions not covered by the above
 * conditions, please contact the author of this code.
 */
package org.processmining.ocbc.performance.panels;



import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Toolkit;
import java.io.File;
import java.io.FileInputStream;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSplitPane;
import javax.swing.border.Border;
import javax.swing.plaf.basic.BasicSplitPaneDivider;
import javax.swing.plaf.basic.BasicSplitPaneUI;

import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.ocbc.log.io.XocXmlParser;
import org.processmining.ocbc.log.model.XOCFilterFunction;
import org.processmining.ocbc.log.model.XOCLog;
import org.processmining.ocbc.log.model.XOCORelation;
import org.processmining.ocbc.log.model.XOCObject;
import org.processmining.ocbc.log.model.XOCObjectModel;
import org.processmining.ocbc.log.model.impl.XOCFilterFunctionImpl;
import org.processmining.ocbc.models.OCBCModel;
import org.processmining.ocbc.models.io.ImportOCBCModel;

/**
 * @author Christian W. Guenther (christian@deckfour.org)
 * 
 */
public class PatternInstanceGenerationPanel extends JPanel {

	/**
	 * 
	 */
	private static final long serialVersionUID = 7772305278301666972L;
	public static final Color COLOR_ENCLOSURE_BG = new Color(40, 40, 40);
	public static final Color COLOR_NON_FOCUS = new Color(70, 70, 70);
	public static final Color COLOR_LIST_BG = new Color(60, 60, 60);
	public static final Color COLOR_LIST_BG_LOWER = new Color(45, 45, 45);
	public static final Color COLOR_LIST_FG = new Color(180, 180, 180);
	public static final Color COLOR_LIST_SELECTION_BG = new Color(80, 0, 0);
	public static final Color COLOR_LIST_SELECTION_BG_LOWER = new Color(30, 10, 10);
	public static final Color COLOR_LIST_SELECTION_FG = new Color(240, 240, 240);
	


	private XOCLog log;


	private UIPluginContext context;
	

	private ModelPanel modelViewPanel;
	private PathConfigurationPanel pathConfigurationPanel;
//	private LogPanel logPanel;//display the events of input XOC log
	private InstancePanel instancePanel;//display generated instances based on event correlation result
	private LogOutputPanel logOutputPanel;//display the buttons to output logs
	
	private PerformanceAnalysisPanel fatherPanel;
	/**
	 * get the CorrelationModelPanel;
	 * @return
	 */
	public ModelPanel getModelPanel(){
		return modelViewPanel;
	}
	
	/**
	 * get the PathConfigurationPanel;
	 * @return
	 */
	public PathConfigurationPanel getPathConfigurationPanel(){
		return pathConfigurationPanel;
	}
	
	/**
	 * get the PathConfigurationPanel;
	 * @return
	 */
	public InstancePanel getInstancePanel(){
		return instancePanel;
	}
	
	/**
	 * get the LogOutputPanel;
	 * @return
	 */
	public LogOutputPanel getLogOutputPanel(){
		return logOutputPanel;
		 
	}
	
	/**
	 * get the context;
	 * @return
	 */
	public UIPluginContext getContext(){
		return context;
	}
	
	public PatternInstanceGenerationPanel(PerformanceAnalysisPanel fatherPanel, final UIPluginContext context, final XOCLog log, OCBCModel model) {
		this.context = context;		
		this.log = log;
		this.fatherPanel = fatherPanel;
		
		setOpaque(false);
		setBorder(BorderFactory.createEmptyBorder());
		setLayout(new BorderLayout());

		JPanel headPanel = new JPanel();
		JLabel classLabel = new JLabel("select id class: ");
		headPanel.add(classLabel);
		
		modelViewPanel = new ModelPanel(this, model);
		pathConfigurationPanel = new PathConfigurationPanel(this, log);
//		logPanel = new LogPanel(this, log);
		instancePanel = new InstancePanel(this, log);
		logOutputPanel = new LogOutputPanel(this, log);
		
		
		
		JSplitPane middlePanel = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
		middlePanel.setTopComponent(modelViewPanel.getModelView());		
		middlePanel.setBottomComponent(pathConfigurationPanel);
		middlePanel.setResizeWeight(0.95);
	
		middlePanel.setToolTipText("press ctrl to zoom in/out");
		
//		JSplitPane leftPanel = borderlessSplitPane(JSplitPane.HORIZONTAL_SPLIT, logPanel, middlePanel);
//		leftPanel.setResizeWeight(0.3);
		
		JSplitPane rightPanel = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
		rightPanel.setTopComponent(instancePanel);		
		rightPanel.setBottomComponent(logOutputPanel);
		rightPanel.setResizeWeight(0.95);
		
		JSplitPane completePanel = borderlessSplitPane(JSplitPane.HORIZONTAL_SPLIT, middlePanel, rightPanel);
		completePanel.setResizeWeight(0.8);
		
		this.add(Box.createHorizontalStrut(100));
		this.add(completePanel);
	}
	
	/**
	 * get the PerformanceAnalysisPanel
	 * @return
	 */
	public PerformanceAnalysisPanel getPerformanceAnalysisPanel(){
		return fatherPanel;
	}
	
	/*
	 * get objects by the last edge list (e.g., {project->employee, project->expense line, expense line->expenses})
	 * initialObjectCaseList contains the case id object
	 */
	public void getObjectsByLastEdgeList(List<String> lastEdgeList, List<XOCObject> initialObjectCaseList, XOCObjectModel lastModel){
		for(String lastEdge : lastEdgeList){
			addObjectsByOneLastEdge(lastEdge, initialObjectCaseList, lastModel);
		}	
	}
	

	/*
	 * add all the objects (in the last model) to objectCaseList on condition that the objects are connected to
	 * at least one object in objectCaseList through lastEdge
	 */
	public void addObjectsByOneLastEdge(String lastEdge, List<XOCObject> objectCaseList, XOCObjectModel lastModel){
		System.out.print("\n enter addObjectsByOneLastEdge");
		String startClass = lastEdge.split("->")[0];
		String endClass = lastEdge.split("->")[1];
		XOCFilterFunction filterFunction = new XOCFilterFunctionImpl();
		System.out.print("\n lastModel.getObjectRelationSet().size(): " + lastModel.getObjectRelationSet().size());
		List<XOCORelation> targetObjectRelationList = lastModel.getObjectRelationsOfTwoClasses(startClass, endClass);

		System.out.print("\n targetObjectRelationList size: " + targetObjectRelationList.size());
		for(XOCORelation relation : targetObjectRelationList){
			System.out.print("\n enter XOCORelation relation : targetObjectRelationList ");
			XOCObject sourceObject = relation.getSourceObject();
			XOCObject targetObject = relation.getTargetObject();
			if(objectCaseList.contains(sourceObject) && targetObject.getAttributes().get("class").toString().equals(endClass)){
				if(!objectCaseList.contains(targetObject)){
					objectCaseList.add(targetObject);
				}
				System.out.print("\n enter contains(sourceObject)");
			}
			else if(objectCaseList.contains(targetObject) && sourceObject.getAttributes().get("class").toString().equals(endClass)){
				if(!objectCaseList.contains(sourceObject)){
					objectCaseList.add(sourceObject);
				}
				
				System.out.print("\n contains(targetObject)");
			}
		}
	}

	public JSplitPane borderlessSplitPane(int orientation, JComponent left, JComponent right) {
		JSplitPane pane = new JSplitPane(orientation, left, right);
		pane.setBorder(BorderFactory.createEmptyBorder(0, 0, 0, 0));
		BasicSplitPaneUI flatDividerSplitPaneUI = new BasicSplitPaneUI() {
			@Override
			public BasicSplitPaneDivider createDefaultDivider() {
				return new BasicSplitPaneDivider(this) {
					@Override
					public void setBorder(Border b) {
					}
				};
			}
		};
		pane.setUI(flatDividerSplitPaneUI);
		pane.setBorder(null);
		pane.setOpaque(false);
		return pane;
	}



	
	public static void main(String[] args) throws Exception  
    {  
//		int classNum = 3;
//		int horizontalUintNum = (int) Math.sqrt(classNum);
//		System.out.print("value" + horizontalUintNum);
//		System.exit(0);
		System.out.print("\n start");
		XocXmlParser XOCParser = new XocXmlParser();
		XOCParser.parse(new File("D:\\normal.xoc"));		
		
		FileInputStream input = new FileInputStream("D:\\model.ocbcmml");
		ImportOCBCModel importer = new ImportOCBCModel();
		OCBCModel model = importer.generateOCBCModel(input);
		
		XOCLog inputLog = XOCParser.getLog();
		System.out.print("\n contains(targetObject)");
		PatternInstanceGenerationPanel eventUI = new PatternInstanceGenerationPanel(null, null, inputLog, model);
		JFrame frame = new JFrame();
		JPanel panel = new JPanel();
		JButton button = new JButton("hello");
		panel.add(button);
//		frame.add(panel);	
		
//		JButton button2 = new JButton("world");
//		panel = new JPanel();
//		panel.add(button2);
//		frame.add(panel);
		
		frame.add(eventUI);
		frame.setVisible(true);	
		frame.setSize(Toolkit.getDefaultToolkit().getScreenSize());
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    }
}
