/*
 * Copyright (c) 2007 Christian W. Guenther (christian@deckfour.org)
 * 
 * LICENSE:
 * 
 * This code is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 * 
 * EXEMPTION:
 * 
 * License to link and use is also granted to open source programs which are not
 * licensed under the terms of the GPL, given that they satisfy one or more of
 * the following conditions: 1) Explicit license is granted to the ProM and
 * ProMimport programs for usage, linking, and derivative work. 2) Carte blance
 * license is granted to all programs developed at Eindhoven Technical
 * University, The Netherlands, or under the umbrella of STW Technology
 * Foundation, The Netherlands. For further exemptions not covered by the above
 * conditions, please contact the author of this code.
 */
package org.processmining.ocbc.performance.panels;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Image;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;

import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JPanel;

import org.deckfour.xes.info.XLogInfo;
import org.deckfour.xes.model.XAttributeMap;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.impl.XAttributeMapImpl;
import org.deckfour.xes.model.impl.XLogImpl;
import org.jgraph.JGraph;
import org.jgraph.graph.DefaultGraphModel;
import org.jgraph.graph.GraphLayoutCache;
import org.jgraph.graph.GraphModel;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.ocbc.log.model.XOCLog;
import org.processmining.ocbc.loggeneration.generator.plugin.XOCLogGeneratorPlugin;
import org.processmining.ocbc.models.OCBCModel;
import org.processmining.ocbc.models.visualization.ModelCellViewFactory;
import org.processmining.ocbc.models.visualization.cell.ActivityCell;
import org.processmining.ocbc.models.visualization.cell.MyOriginalCell;
import org.processmining.ocbc.performance.logprojection.LogProjectionPlugin;
import org.processmining.ocbc.performance.logprojection.LogView;
import org.processmining.ocbc.performance.logprojection.plugins.dottedchart.DottedChart.DottedChartException;
import org.processmining.ocbc.performance.logprojection.plugins.dottedchart.ui.DottedChartInspector;

import com.fluxicon.slickerbox.components.IconVerticalTabbedPane;
import com.fluxicon.slickerbox.components.SlickerTabbedPane;

/**
 * @author Christian W. Guenther (christian@deckfour.org)
 * 
 */
//public class ConformanceCheckingResultView extends JPanel implements ScalableComponent {
public class PerformanceAnalysisPanel extends JPanel {
	
	private static final long serialVersionUID = 4258730373866637365L;
	
	private final static String IMPORTICON = "import.png";
	protected Image importIcon = new ImageIcon(XOCLogGeneratorPlugin.class.getResource(IMPORTICON),IMPORTICON).getImage();
	
//	protected XOCLogViewer logView;
	protected XLogInfo summary;	
	protected XOCLog inputLog;
	protected OCBCModel model;	
	protected SlickerTabbedPane tabbedPane;
		
	protected IconVerticalTabbedPane iconTabs = new IconVerticalTabbedPane(new Color(230, 230, 230, 210), new Color(20, 20,20, 160));
	protected DottedChartInspector dotChartPanel; //the dot panel to show the performance
	protected PatternInstanceGenerationPanel instanceGenerationPanel;//the panel to select pattern and generate instances
	protected UIPluginContext context;
	public PerformanceAnalysisPanel(UIPluginContext context, XOCLog log, OCBCModel model) throws DottedChartException {				
		initialize(log, model);		
	
		this.context = context;
		instanceGenerationPanel = new PatternInstanceGenerationPanel(this, context, log, model);
		
    	XAttributeMap logAttributeMap = new XAttributeMapImpl();
    	XLog xesLog = new XLogImpl(logAttributeMap);

//    	XAttributeMap traceAttributeMap = new XAttributeMapImpl();			
//		XLogFunctions.putLiteral(traceAttributeMap, "concept:name", "case");
//		XTrace trace = new XTraceImpl(traceAttributeMap);	
//		xesLog.add(trace);
						
		LogView logView = LogProjectionPlugin.logToDottedChart(context, xesLog);	
		dotChartPanel = new DottedChartInspector(logView, context);

		GraphModel iconModel = new DefaultGraphModel();  
	    GraphLayoutCache view = new GraphLayoutCache(iconModel, new ModelCellViewFactory());  
	    JGraph graph = new JGraph(iconModel, view); 
		Color bg = null; // Use this to make the background transparent
		int inset = 0;
		MyOriginalCell[] cells = new MyOriginalCell[1];  
		cells[0] = new ActivityCell(new String("Hello")); 
		graph.getGraphLayoutCache().insert(cells);  		
		BufferedImage img = graph.getImage(bg, inset);		
		Image summaryIcon = new ImageIcon(img).getImage();
		iconTabs.addTab("Pattern", summaryIcon, instanceGenerationPanel);
		iconTabs.addTab("Chart", summaryIcon, dotChartPanel);
		add(iconTabs, BorderLayout.CENTER);
		
		revalidate();
		repaint();		
	}

	/**
	 * update the dot chart based on a XES log
	 * @param context
	 * @param xesLog
	 * @throws DottedChartException
	 */
	public void updateDotChartByXESLog(XLog xesLog) throws DottedChartException{
		LogView logView = LogProjectionPlugin.logToDottedChart(context, xesLog);
		dotChartPanel.updatePanelBasedOnLogView(logView, context);
	}
	
	/**
	 * get the PatternInstanceGenerationPanel
	 * @return
	 */
	public PatternInstanceGenerationPanel getInstanceGenerationPanel(){
		return instanceGenerationPanel;
	}
	
	/**
	 * get the DottedChartInspector
	 * @return
	 */
	public DottedChartInspector getDotChartPanel(){
		return dotChartPanel;
	}
	
	protected void initialize(XOCLog log, OCBCModel model) {
		this.inputLog = log;
		this.model = model;	
		setBackground(new Color(40, 40, 40));
		setBorder(BorderFactory.createEmptyBorder());
		setLayout(new BorderLayout());
	}
	
	
	
	//estimate the total running time (unit: ms)
	public long estimateRunningTime(XOCLog log, OCBCModel model){
		int eventNum = log.getAllEvents().size(); 
        int firstObjectModelSize = log.getAllEvents().get(0).getObjectModel().getObjectSet().size();
        int lastObjectModelSize = log.getAllEvents().get(eventNum-1).getObjectModel().getObjectSet().size();
        int averageObjectModelSize = (firstObjectModelSize + lastObjectModelSize)/2;
        long totalTime = eventNum*averageObjectModelSize/2; 
//        if(dataConstraintCheckSwitch){
//        	totalTime = totalTime + eventNum*averageObjectModelSize*5; 
//        }
        return totalTime;
	}
	
	public void updateProcessBarMaximum(UIPluginContext context, int processPercentage, long startTime, long totalTime){
		long endTime = System.currentTimeMillis();
		long percentage = (endTime - startTime)*100/totalTime;
		if(percentage < processPercentage){
			if(context != null){
				context.getProgress().setValue((int) (percentage) + 1);
			}				
		}
		else{
			if(context != null){
				context.getProgress().setValue(processPercentage);
			}	
		}			
	}	
	
	public XOCLog getXOCLog() {
		return inputLog;
	}
	
	public XLogInfo getLogInfo() {
		return summary;
	}
	
	public void writeTimeInformationIntoFile(String wirteLine, boolean clearFile){
		File log = new File("D:\\time.txt");
	    try{
		    if(log.exists()==false || clearFile){
	            System.out.println("We had to make a new file.");
	            log.createNewFile();
		    }
		    PrintWriter out = new PrintWriter(new FileWriter(log, true));
		    out.append(wirteLine + "\n");
		    out.close();
	    }catch(IOException e){
	        System.out.println("COULD NOT LOG!!");
	    }
	}
	
	public void outputContextInformation(UIPluginContext context, String information){
		if(context != null){
			context.log("\n " + information);
		}
	}
	
	
	
	
	public static void main(String[] args) throws Exception  
    {  
     			 
		File log = new File("D:\\time.txt");
			    try{
			    if(log.exists()==false){
			            System.out.println("We had to make a new file.");
			            log.createNewFile();
			    }
			    PrintWriter out = new PrintWriter(new FileWriter(log, true));
			    out.append("*******timestamp2******* " + "\n");
			    out.close();
			    }catch(IOException e){
			        System.out.println("COULD NOT LOG!!");
			    } 
    } 
}
