package org.processmining.ocbc.performance.plugin;

import java.awt.Toolkit;
import java.io.File;
import java.io.FileInputStream;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.SortedSet;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JPanel;

import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.contexts.uitopia.annotations.Visualizer;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.util.ui.scalableview.ScalableComponent;
import org.processmining.ocbc.log.inspector.panels.XOCLogDashboard;
import org.processmining.ocbc.log.inspector.panels.XOCLogSummary;
import org.processmining.ocbc.log.inspector.panels.XOCLogViewer;
import org.processmining.ocbc.log.io.XocXmlParser;
import org.processmining.ocbc.log.model.XOCLog;
import org.processmining.ocbc.loggeneration.panel.datamodel.DataModelInspector;
import org.processmining.ocbc.models.OCBCModel;
import org.processmining.ocbc.models.io.ImportOCBCModel;
import org.processmining.ocbc.performance.panels.PerformanceAnalysisPanel;
import org.processmining.plugins.interactivevisualization.EID;
import org.processmining.plugins.interactivevisualization.InteractiveVisualization;
import org.processmining.plugins.interactivevisualization.InteractivityContext;
import org.processmining.plugins.interactivevisualization.InteractivityManager;



public class OCBCPerformancePlugin extends JPanel implements ScalableComponent, InteractiveVisualization{
	
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	protected XOCLogViewer viewerUI;
	protected XOCLogDashboard dashboardUI;
	protected XOCLogSummary summaryUI;
	protected DataModelInspector dataModelUI;
//	protected LogInfoUI loginfoUI;
	
	// variables used in ScalableComponent interface.
	private final List<UpdateListener> observers = new LinkedList<UpdateListener>();
	
	// variables used in InteractiveVisualization interface.
	private InteractivityContext interactivityContext = new InteractivityContext();
	
	
	protected XOCLog log;
	

	@Plugin(
		name = "OCBC Performance Analyzer",
		parameterLabels = {"XOCLog", "OCBC model"}, 
		returnTypes = OCBCPerformanceObject.class, 
		returnLabels = "OCBC Performance Analysis", 
		userAccessible = true,
		help = "Start up editor"
	)
	@UITopiaVariant(
		affiliation = UITopiaVariant.EHV,
		author = "Guangming Li",
		email = "g.li.3@tue.nl"
	)

	public OCBCPerformanceObject performanceAnalyzer(UIPluginContext context, XOCLog log, OCBCModel model) throws Exception{
		
		context.log("OCBC performanceAnalyzer starts");
		System.out.println("\n performanceAnalyzer");
//		logConvertorPanel(context, log);		       
		return new OCBCPerformanceObject(new PerformanceAnalysisPanel(context, log, model));
	}
	
	@Plugin(
			name = "XOC Performance Analyzer", 
			parameterLabels = {"visualizer"},
			returnTypes = JComponent.class, 
			returnLabels = "Performance Visualization", 
			help = "OCBC Performance"
	)
	
	@Visualizer
	public JComponent visualizePerformancePanel(UIPluginContext context, final OCBCPerformanceObject input){
		System.out.println("OCBC Performance visualizer");
		context.log("welcome to OCBC Performance visualizer");
		JPanel resultPanel = input.panel;
		return resultPanel;
	}
	
//	public JPanel performancePanel(UIPluginContext context, XOCLog log, OCBCModel model) throws Exception {
//		System.out.println("\n visualizePerformancePanel");
//		XOCLogConversionPanel logConversion = new XOCLogConversionPanel(context, log);
////		initialize(null);
////		completeGui(null);
//		return logConversion;
//	}
//	JTextField selectedPath;


	

	
	public static void main(String[] args) throws Exception  
    {  
		//pop up a dialogue to select the XOC log
/*		IOFileDialogue fileImporter = new IOFileDialogue();
    	fileImporter.show("XOC Log (.xoc)", "XOC", JFileChooser.FILES_ONLY, "Select a XOC Log", null);
    	File file = fileImporter.getSelectedFile();*/
    	 /*
		System.out.print("\n start");
		SimpleDateFormat myFormat = new SimpleDateFormat("dd MM yyyy");
		String inputString1 = "23 01 1997";
		String inputString2 = "27 04 1997";

		try {
		    Date date1 = myFormat.parse(inputString1);
		    Date date2 = myFormat.parse(inputString2);
		    long diff = date2.getTime() - date1.getTime();
		    Date d = new Date(diff);
		    System.out.println ("Days: " + d.toString());
		} catch (ParseException e) {
		    e.printStackTrace();
		}
	    */	
	   
		XocXmlParser XOCParser = new XocXmlParser();
		XOCParser.parse(new File("D:\\normal.xoc"));		
		
		FileInputStream input = new FileInputStream("D:\\model.ocbcmml");
		ImportOCBCModel importer = new ImportOCBCModel();
		OCBCModel model = importer.generateOCBCModel(input);
		
		XOCLog inputLog = XOCParser.getLog();
		System.out.print("\n contains(targetObject)");
		PerformanceAnalysisPanel eventUI = new PerformanceAnalysisPanel(null, inputLog, model);
		JFrame frame = new JFrame();
		JPanel panel = new JPanel();
		JButton button = new JButton("hello");
		panel.add(button);
//		frame.add(panel);	
		
//		JButton button2 = new JButton("world");
//		panel = new JPanel();
//		panel.add(button2);
//		frame.add(panel);
		
		frame.add(eventUI);
		frame.setVisible(true);	
		frame.setSize(Toolkit.getDefaultToolkit().getScreenSize());
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);	
			
    }

	@Override
	public void setManager(InteractivityManager manager) {
		interactivityContext.setManager(manager);
	}
	
	@Override
	public Map<String, SortedSet<EID>> getSelectedElements() {
		throw new UnsupportedOperationException("This visualization does not support communicating selected elements.");
	}
	
	@Override
	public Map<String, SortedSet<EID>> getVisibleElements() {
		throw new UnsupportedOperationException("This visualization does not support communicating visible elements.");
	}
	
	@Override
	public InteractivityManager getManager() {
		return interactivityContext.getManager();
	}
	
	/*
	 * Implementation of Scalable Component interface (non-Javadoc)
	 * @see org.processmining.framework.util.ui.scalableview.ScalableComponent
	 */
	
	@Override
	public JComponent getComponent() {
		return this;
	}
	
	@Override
	public double getScale() {
		return 1;
	}
	
	@Override
	public void setScale(double newScale) {
		//TODO debug code
		System.err.println("Supposed to set the scale to "+newScale);
		
		//TODO set scale
	}
	
	@Override
	public void addUpdateListener(UpdateListener listener) {
		
		if(listener == null) {
			throw new IllegalArgumentException("The UpdateListener cannot be 'null'.");
		}
		
		observers.add(listener);
	}
	
	@Override
	public void removeUpdateListener(UpdateListener listener) {
		
		if(listener == null) {
			throw new IllegalArgumentException("The UpdateListener cannot be 'null'.");
		}
		
		observers.remove(listener);
	}

	public void updateVisualization(boolean logChanged, boolean tagsChanged) {
		// TODO Auto-generated method stub
		
	}	
}
